@@ -3,7 +3,7 @@
  * El'Beem - the visual lattice boltzmann freesurface simulator
  * All code distributed as part of El'Beem is covered by the version 2 of the 
  * GNU General Public License. See the file COPYING for details.
- * Copyright 2003-2005 Nils Thuerey
+ * Copyright 2003-2006 Nils Thuerey
  *
  * Combined 2D/3D Lattice Boltzmann standard solver classes
  *
@@ -106,6 +106,10 @@
 #endif
 #endif
 
+#if LBM_INCLUDE_TESTSOLVERS==1
+#include "solver_test.h"
+#endif // LBM_INCLUDE_TESTSOLVERS==1
+
 /*****************************************************************************/
 /*! cell access classes */
 class UniformFsgrCellIdentifier : 
@@ -216,10 +220,10 @@ class LbmFsgrSolver :
 		//! notify object that dump is in progress (e.g. for field dump) 
 		virtual void notifySolverOfDump(int dumptype, int frameNr,char *frameNrStr,string outfilename);
 
-#if LBM_USE_GUI==1
+#		if LBM_USE_GUI==1
 		//! show simulation info (implement LbmSolverInterface pure virtual func)
 		virtual void debugDisplay(int set);
-#endif
+#		endif
 		
 		// implement CellIterator<UniformFsgrCellIdentifier> interface
 		typedef UniformFsgrCellIdentifier stdCellId;
@@ -254,13 +258,18 @@ class LbmFsgrSolver :
 		LBM_INLINED void initEmptyCell(int level, int i,int j,int k, CellFlagType flag, LbmFloat rho, LbmFloat mass);
 		LBM_INLINED void initVelocityCell(int level, int i,int j,int k, CellFlagType flag, LbmFloat rho, LbmFloat mass, LbmVec vel);
 		LBM_INLINED void changeFlag(int level, int xx,int yy,int zz,int set,CellFlagType newflag);
+		LBM_INLINED void forceChangeFlag(int level, int xx,int yy,int zz,int set,CellFlagType newflag);
+		LBM_INLINED void initInterfaceVars(int level, int i,int j,int k,int workSet, bool initMass);
 		//! interpolate velocity and density at a given position
 		void interpolateCellValues(int level,int ei,int ej,int ek,int workSet, LbmFloat &retrho, LbmFloat &retux, LbmFloat &retuy, LbmFloat &retuz);
 
 		/*! perform a single LBM step */
 		void stepMain();
+		//! advance fine grid
 		void fineAdvance();
+		//! advance coarse grid
 		void coarseAdvance(int lev);
+		//! update flux area values on coarse grids
 		void coarseCalculateFluxareas(int lev);
 		// adaptively coarsen grid
 		bool adaptGrid(int lev);
@@ -273,33 +282,37 @@ class LbmFsgrSolver :
 		virtual int initParticles();
 		/*! move all particles */
 		virtual void advanceParticles();
+		/*! move a particle at a boundary */
+		void handleObstacleParticle(ParticleObject *p);
+		/*! check whether to add particle 
+		bool checkAddParticle();
+		void performAddParticle();*/
 
 
 		/*! debug object display (used e.g. for preview surface) */
 		virtual vector<ntlGeometryObject*> getDebugObjects();
 	
 		// gui/output debugging functions
-#if LBM_USE_GUI==1
+#		if LBM_USE_GUI==1
 		virtual void debugDisplayNode(int dispset, CellIdentifierInterface* cell );
 		virtual void lbmDebugDisplay(int dispset);
 		virtual void lbmMarkedCellDisplay();
-#endif // LBM_USE_GUI==1
+#		endif // LBM_USE_GUI==1
 		virtual void debugPrintNodeInfo(CellIdentifierInterface* cell, int forceSet=-1);
 
 		//! for raytracing, preprocess
 		void prepareVisualization( void );
 
-		/*! type for cells */
-		//typedef typename this->LbmCell LbmCell;
-		
 	protected:
 
 		//! internal quick print function (for debugging) 
 		void printLbmCell(int level, int i, int j, int k,int set);
 		// debugging use CellIterator interface to mark cell
 		void debugMarkCellCall(int level, int vi,int vj,int vk);
-
+		
+		// loop over grid, stream&collide update
 		void mainLoop(int lev);
+		// change time step size
 		void adaptTimestep();
 		//! init mObjectSpeeds for current parametrization
 		void recalculateObjectSpeeds();
@@ -309,6 +322,11 @@ class LbmFsgrSolver :
 		void reinitFlags( int workSet );
 		//! mass dist weights
 		LbmFloat getMassdWeight(bool dirForw, int i,int j,int k,int workSet, int l);
+		//! compute surface normals: fluid, fluid more accurate, and for obstacles
+		void computeFluidSurfaceNormal(LbmFloat *cell, CellFlagType *cellflag,    LbmFloat *snret);
+		void computeFluidSurfaceNormalAcc(LbmFloat *cell, CellFlagType *cellflag, LbmFloat *snret);
+		void computeObstacleSurfaceNormal(LbmFloat *cell, CellFlagType *cellflag, LbmFloat *snret);
+		void computeObstacleSurfaceNormalAcc(int i,int j,int k, LbmFloat *snret);
 		//! add point to mListNewInter list
 		LBM_INLINED void addToNewInterList( int ni, int nj, int nk );	
 		//! cell is interpolated from coarse level (inited into set, source sets are determined by t)
@@ -320,6 +338,13 @@ class LbmFsgrSolver :
 		LBM_INLINED int getForZMin1();
 		LBM_INLINED int getForZMaxBnd(int lev);
 		LBM_INLINED int getForZMax1(int lev);
+		LBM_INLINED bool checkDomainBounds(int lev,int i,int j,int k);
+		LBM_INLINED bool checkDomainBoundsPos(int lev,LbmVec pos);
+
+		// touch grid and flags once
+		void preinitGrids();
+		// one relaxation step for standing fluid
+		void standingFluidPreinit();
 
 
 		// member vars
@@ -349,12 +374,24 @@ class LbmFsgrSolver :
 		LbmFloat mFVArea;
 		bool mUpdateFVHeight;
 
-		//! require some geo setup from the viz?
-		//int mGfxGeoSetup;
 		//! force quit for gfx
 		LbmFloat mGfxEndTime;
 		//! smoother surface initialization?
 		int mInitSurfaceSmoothing;
+		//! surface generation settings, default is all off (=0)
+		//  each flag switches side on off,  fssgNoObs is for obstacle sides
+		//  -1 equals all on
+		typedef enum {
+			 fssgNormal   =  0,
+			 fssgNoNorth  =  1,
+			 fssgNoSouth  =  2,
+			 fssgNoEast   =  4,
+			 fssgNoWest   =  8,
+			 fssgNoTop    = 16,
+			 fssgNoBottom = 32,
+			 fssgNoObs    = 64
+		} fsSurfaceGen;
+		int mFsSurfGenSetting;
 
 		//! lock time step down switching
 		int mTimestepReduceLock;
@@ -392,10 +429,6 @@ class LbmFsgrSolver :
 		int mMaxNoCells, mMinNoCells;
 		LONGINT mAvgNumUsedCells;
 
-		//! for interactive - how to drop drops?
-		int mDropMode;
-		LbmFloat mDropSize;
-		LbmVec mDropSpeed;
 		//! precalculated objects speeds for current parametrization
 		vector<LbmVec> mObjectSpeeds;
 		//! partslip bc. values for obstacle boundary conditions
@@ -406,6 +439,7 @@ class LbmFsgrSolver :
 		//! permanent movobj vert storage
 	  vector<ntlVec3Gfx>  mMOIVertices;
   	vector<ntlVec3Gfx>  mMOIVerticesOld;
+	  vector<ntlVec3Gfx>  mMOINormals;
 
 		//! get isofield weights
 		int mIsoWeightMethod;
@@ -443,6 +477,8 @@ class LbmFsgrSolver :
 
 		//! debug function to disable standing f init
 		int mDisableStandingFluidInit;
+		//! init 2d with skipped Y/Z coords
+		bool mInit2dYZ;
 		//! debug function to force tadap syncing
 		int mForceTadapRefine;
 		//! border cutoff value
@@ -463,14 +499,42 @@ class LbmFsgrSolver :
 #		endif // FSGR_STRICT_DEBUG==1
 
 		bool mUseTestdata;
+#		if LBM_INCLUDE_TESTSOLVERS==1
+		// test functions
+		LbmTestdata *mpTest;
+		void initTestdata();
+		void destroyTestdata();
+		void handleTestdata();
+		void set3dHeight(int ,int );
 
-	public: // former LbmModelLBGK  functions
+		void initCpdata();
+		void handleCpdata();
+		void cpDebugDisplay(int dispset);
+
+		int mMpNum,mMpIndex;
+		int mOrgSizeX;
+		LbmFloat mOrgStartX;
+		LbmFloat mOrgEndX;
+		void mrSetup();
+		void mrExchange(); 
+		void mrIsoExchange(); 
+		LbmFloat mrInitTadap(LbmFloat max); 
+		void gcFillBuffer(  LbmGridConnector *gc, int *retSizeCnt, const int *bdfs);
+		void gcUnpackBuffer(LbmGridConnector *gc, int *retSizeCnt, const int *bdfs);
+	public:
+		// needed for testdata
+		void find3dHeight(int i,int j, LbmFloat prev, LbmFloat &ret, LbmFloat *retux, LbmFloat *retuy, LbmFloat *retuz);
+		// mptest
+		int getMpIndex() { return mMpIndex; };
+#		endif // LBM_INCLUDE_TESTSOLVERS==1
+
+		// former LbmModelLBGK  functions
 		// relaxation funtions - implemented together with relax macros
 		static inline LbmFloat getVelVecLen(int l, LbmFloat ux,LbmFloat uy,LbmFloat uz);
 		static inline LbmFloat getCollideEq(int l, LbmFloat rho,  LbmFloat ux, LbmFloat uy, LbmFloat uz);
 		inline LbmFloat getLesNoneqTensorCoeff( LbmFloat df[], 				LbmFloat feq[] );
 		inline LbmFloat getLesOmega(LbmFloat omega, LbmFloat csmago, LbmFloat Qo);
-		inline void collideArrays( int i, int j, int k, // position - more for debugging
+		inline void collideArrays( int lev, int i, int j, int k, // position - more for debugging
 				LbmFloat df[], LbmFloat &outrho, // out only!
 				// velocity modifiers (returns actual velocity!)
 				LbmFloat &mux, LbmFloat &muy, LbmFloat &muz, 
@@ -479,12 +543,10 @@ class LbmFsgrSolver :
 
 
 		// former LBM models
-	public:
+		//! shorten static const definitions
+#		define STCON static const
 
-//! shorten static const definitions
-#define STCON static const
-
-#if LBMDIM==3
+#		if LBMDIM==3
 		
 		//! id string of solver
 		virtual string getIdString() { return string("FreeSurfaceFsgrSolver[BGK_D3Q19]"); }
@@ -575,13 +637,13 @@ class LbmFsgrSolver :
 		STCON LbmFloat dfLength[ 19 ];
 
 		/*! equilibrium distribution functions, precalculated = getCollideEq(i, 0,0,0,0) */
-		static LbmFloat dfEquil[ 19 ];
+		static LbmFloat dfEquil[ dTotalNum ];
 
 		/*! arrays for les model coefficients */
 		static LbmFloat lesCoeffDiag[ (3-1)*(3-1) ][ 27 ];
 		static LbmFloat lesCoeffOffdiag[ 3 ][ 27 ];
 
-#else // end LBMDIM==3 , LBMDIM==2
+#		else // end LBMDIM==3 , LBMDIM==2
 		
 		//! id string of solver
 		virtual string getIdString() { return string("FreeSurfaceFsgrSolver[BGK_D2Q9]"); }
@@ -661,13 +723,13 @@ class LbmFsgrSolver :
 		STCON LbmFloat dfLength[ 9 ];
 
 		/* equilibrium distribution functions, precalculated = getCollideEq(i, 0,0,0,0) */
-		static LbmFloat dfEquil[ 9 ];
+		static LbmFloat dfEquil[ dTotalNum ];
 
 		/*! arrays for les model coefficients */
 		static LbmFloat lesCoeffDiag[ (2-1)*(2-1) ][ 9 ];
 		static LbmFloat lesCoeffOffdiag[ 2 ][ 9 ];
 
-#endif  // LBMDIM==2
+#		endif  // LBMDIM==2
 };
 
 #undef STCON
@@ -691,7 +753,8 @@ class LbmFsgrSolver :
 
 // flag array defines -----------------------------------------------------------------------------------------------
 
-// lbm testsolver get index define
+// lbm testsolver get index define, note - ignores is (set) as flag
+// array is only a single entry
 #define _LBMGI(level, ii,ij,ik, is) ( (mLevel[level].lOffsy*(ik)) + (mLevel[level].lOffsx*(ij)) + (ii) )
 
 //! flag array acces macro
@@ -701,7 +764,6 @@ class LbmFsgrSolver :
 
 // array handling  -----------------------------------------------------------------------------------------------
 
-//#define _LBMQI(level, ii,ij,ik, is, lunused) ( ((is)*mLevel[level].lOffsz) + (mLevel[level].lOffsy*(ik)) + (mLevel[level].lOffsx*(ij)) + (ii) )
 #define _LBMQI(level, ii,ij,ik, is, lunused) ( (mLevel[level].lOffsy*(ik)) + (mLevel[level].lOffsx*(ij)) + (ii) )
 #define _QCELL(level,xx,yy,zz,set,l) (mLevel[level].mprsCells[(set)][ LBMQI((level),(xx),(yy),(zz),(set), l)*dTotalNum +(l)])
 #define _QCELL_NB(level,xx,yy,zz,set, dir,l) (mLevel[level].mprsCells[(set)][ LBMQI((level),(xx)+this->dfVecX[dir],(yy)+this->dfVecY[dir],(zz)+this->dfVecZ[dir],set, l)*dTotalNum +(l)])
@@ -744,7 +806,8 @@ class LbmFsgrSolver :
 
 // general defines -----------------------------------------------------------------------------------------------
 
-#define TESTFLAG(flag, compflag) ((flag & compflag)==compflag)
+// replace TESTFLAG
+#define FLAGISEXACT(flag, compflag)  ((flag & compflag)==compflag)
 
 #if LBMDIM==2
 #define dC 0
@@ -835,12 +898,17 @@ inline LbmFloat LbmFsgrSolver::getCollideEq(int l, LbmFloat rho,  LbmFloat ux, L
 				+ rho - (3.0/2.0*(ux*ux + uy*uy + uz*uz)) 
 				+ 3.0 *tmp 
 				+ 9.0/2.0 *(tmp*tmp) )
-			);
+		 	); // */
 };
 
 /*****************************************************************************/
 /* init a given cell with flag, density, mass and equilibrium dist. funcs */
 
+void LbmFsgrSolver::forceChangeFlag(int level, int xx,int yy,int zz,int set,CellFlagType newflag) {
+	// also overwrite persisting flags
+	// function is useful for tracking accesses...
+	RFLAG(level,xx,yy,zz,set) = newflag;
+}
 void LbmFsgrSolver::changeFlag(int level, int xx,int yy,int zz,int set,CellFlagType newflag) {
 	CellFlagType pers = RFLAG(level,xx,yy,zz,set) & CFPersistMask;
 	RFLAG(level,xx,yy,zz,set) = newflag | pers;
@@ -857,7 +925,6 @@ LbmFsgrSolver::initEmptyCell(int level, int i,int j,int k, CellFlagType flag, Lb
 	RAC(ecel, dMass) = mass;
 	RAC(ecel, dFfrac) = mass/rho;
 	RAC(ecel, dFlux) = FLUX_INIT;
-	//RFLAG(level, i,j,k, workSet)= flag;
 	changeFlag(level, i,j,k, workSet, flag);
 
   workSet ^= 1;
@@ -875,7 +942,6 @@ LbmFsgrSolver::initVelocityCell(int level, int i,int j,int k, CellFlagType flag,
 	RAC(ecel, dMass) = mass;
 	RAC(ecel, dFfrac) = mass/rho;
 	RAC(ecel, dFlux) = FLUX_INIT;
-	//RFLAG(level, i,j,k, workSet) = flag;
 	changeFlag(level, i,j,k, workSet, flag);
 
   workSet ^= 1;
@@ -900,6 +966,41 @@ int LbmFsgrSolver::getForZMax1(int lev)   {
 	return mLevel[lev].lSizez -1;
 }
 
+bool LbmFsgrSolver::checkDomainBounds(int lev,int i,int j,int k) { 
+	if(i<0) return false;
+	if(j<0) return false;
+	if(k<0) return false;
+	if(i>mLevel[lev].lSizex-1) return false;
+	if(j>mLevel[lev].lSizey-1) return false;
+	if(k>mLevel[lev].lSizez-1) return false;
+	return true;
+}
+bool LbmFsgrSolver::checkDomainBoundsPos(int lev,LbmVec pos) { 
+	const int i= (int)pos[0]; 
+	if(i<0) return false;
+	if(i>mLevel[lev].lSizex-1) return false;
+	const int j= (int)pos[1]; 
+	if(j<0) return false;
+	if(j>mLevel[lev].lSizey-1) return false;
+	const int k= (int)pos[2];
+	if(k<0) return false;
+	if(k>mLevel[lev].lSizez-1) return false;
+	return true;
+}
+
+void LbmFsgrSolver::initInterfaceVars(int level, int i,int j,int k,int workSet, bool initMass) {
+	LbmFloat *ccel = &QCELL(level ,i,j,k, workSet,0);
+	LbmFloat nrho = 0.0;
+	FORDF0 { nrho += RAC(ccel,l); }
+	if(initMass) {
+		RAC(ccel,dMass) = nrho;
+  	RAC(ccel, dFfrac) = 1.;
+	} else {
+		// preinited, e.g. from reinitFlags
+		RAC(ccel, dFfrac) = RAC(ccel, dMass)/nrho;
+		RAC(ccel, dFlux) = FLUX_INIT;
+	}
+}
 
 
 #endif
