@@ -1,5 +1,5 @@
 /**
- * $Id: GHOST_C-api.cpp,v 1.3 2002/11/25 09:52:56 mein Exp $
+ * $Id: GHOST_C-api.cpp,v 1.6 2006/10/24 16:42:12 sirdude Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -35,7 +35,7 @@
  *
  * C Api for GHOST
  *
- * Version: $Id: GHOST_C-api.cpp,v 1.3 2002/11/25 09:52:56 mein Exp $
+ * Version: $Id: GHOST_C-api.cpp,v 1.6 2006/10/24 16:42:12 sirdude Exp $
  */
 
 #include <stdlib.h>
@@ -289,6 +289,24 @@ GHOST_TSuccess GHOST_SetCustomCursorShape(GHOST_WindowHandle windowhandle,
 	return window->setCustomCursorShape(bitmap, mask, hotX, hotY);
 }
 
+GHOST_TSuccess GHOST_SetCustomCursorShapeEx(GHOST_WindowHandle windowhandle,
+										  GHOST_TUns8 *bitmap, 
+										  GHOST_TUns8 *mask, 
+										  int sizex, 
+										  int sizey,
+										  int hotX, 
+										  int hotY, 
+										  int fg_color, 
+										  int bg_color)
+{
+	GHOST_IWindow* window = (GHOST_IWindow*) windowhandle;
+
+	return window->setCustomCursorShape(bitmap, mask, sizex, sizey, 
+												hotX, hotY, fg_color, bg_color);
+}
+
+
+
 int GHOST_GetCursorVisibility(GHOST_WindowHandle windowhandle)
 {
 	GHOST_IWindow* window = (GHOST_IWindow*) windowhandle;
@@ -479,6 +497,8 @@ char* GHOST_GetTitle(GHOST_WindowHandle windowhandle)
 	window->getTitle(title);
 
 	char *ctitle = (char*) malloc(title.Length() + 1);
+
+	if (ctitle == NULL) return NULL;
 	strcpy(ctitle, title.Ptr());
 		
 	return ctitle;
@@ -631,6 +651,11 @@ GHOST_TSuccess GHOST_InvalidateWindow(GHOST_WindowHandle windowhandle)
 }
 
 
+extern const GHOST_TabletData* GHOST_GetTabletData(GHOST_WindowHandle windowhandle)
+{
+	return ((GHOST_IWindow*)windowhandle)->GetTabletData();
+}
+
 
 GHOST_TInt32 GHOST_GetWidthRectangle(GHOST_RectangleHandle rectanglehandle)
 {
@@ -777,6 +802,3 @@ GHOST_TSuccess GHOST_ClipRectangle(GHOST_RectangleHandle rectanglehandle,
 
 	return result;
 }
-
-
-
