@@ -1,5 +1,5 @@
 /**
- * $Id: GHOST_SystemCarbon.cpp,v 1.23 2006/06/20 20:24:07 lukep Exp $
+ * $Id: GHOST_SystemCarbon.cpp,v 1.25 2006/12/24 21:21:49 erwin Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -31,7 +31,7 @@
 
 /**
 
- * $Id: GHOST_SystemCarbon.cpp,v 1.23 2006/06/20 20:24:07 lukep Exp $
+ * $Id: GHOST_SystemCarbon.cpp,v 1.25 2006/12/24 21:21:49 erwin Exp $
  * Copyright (C) 2001 NaN Technologies B.V.
  * @author	Maarten Gribnau
  * @date	May 7, 2001
@@ -493,7 +493,13 @@ GHOST_TSuccess GHOST_SystemCarbon::setCursorPosition(GHOST_TInt32 x, GHOST_TInt3
 {
 	float xf=(float)x, yf=(float)y;
 
+	CGAssociateMouseAndMouseCursorPosition(false);
+	CGSetLocalEventsSuppressionInterval(0);
 	CGWarpMouseCursorPosition(CGPointMake(xf, yf));
+	CGAssociateMouseAndMouseCursorPosition(true);
+
+//this doesn't work properly, see game engine mouse-look scripts
+//	CGWarpMouseCursorPosition(CGPointMake(xf, yf));
 	// this call below sends event, but empties other events (like shift)
 	// CGPostMouseEvent(CGPointMake(xf, yf), TRUE, 1, FALSE, 0);
 
@@ -688,6 +694,79 @@ OSStatus GHOST_SystemCarbon::handleWindowEvent(EventRef event)
 	return err;
 }
 
+OSStatus GHOST_SystemCarbon::handleTabletEvent(EventRef event)
+{
+	GHOST_IWindow* window = m_windowManager->getActiveWindow();
+	TabletPointRec tabletPointRecord;
+	TabletProximityRec	tabletProximityRecord;
+	UInt32 anInt32;
+	GHOST_TabletData& ct=((GHOST_WindowCarbon*)window)->GetCarbonTabletData();
+	OSStatus err = eventNotHandledErr;
+	
+	ct.Pressure = 0;
+	ct.Xtilt = 0;
+	ct.Ytilt = 0;
+	
+	// is there an embedded tablet event inside this mouse event? 
+	if(noErr == GetEventParameter(event, kEventParamTabletEventType, typeUInt32, NULL, sizeof(UInt32), NULL, (void *)&anInt32))
+	{
+		// yes there is one!
+		// Embedded tablet events can either be a proximity or pointer event.
+		if(anInt32 == kEventTabletPoint)
+		{
+			//GHOST_PRINT("Embedded pointer event!\n");
+			
+			// Extract the tablet Pointer Event. If there is no Tablet Pointer data
+			// in this event, then this call will return an error. Just ignore the
+			// error and go on. This can occur when a proximity event is embedded in
+			// a mouse event and you did not check the mouse event to see which type
+			// of tablet event was embedded.
+			if(noErr == GetEventParameter(event, kEventParamTabletPointRec,
+										  typeTabletPointRec, NULL,
+										  sizeof(TabletPointRec),
+										  NULL, (void *)&tabletPointRecord))
+			{
+				ct.Pressure = tabletPointRecord.pressure / 65535.0f;
+				ct.Xtilt = tabletPointRecord.tiltX / 32767.0f; /* can be positive or negative */
+				ct.Ytilt = tabletPointRecord.tiltY / 32767.0f; /* can be positive or negative */
+			}
+		} else {
+			//GHOST_PRINT("Embedded proximity event\n");
+			
+			// Extract the Tablet Proximity record from the event.
+			if(noErr == GetEventParameter(event, kEventParamTabletProximityRec,
+										  typeTabletProximityRec, NULL,
+										  sizeof(TabletProximityRec),
+										  NULL, (void *)&tabletProximityRecord))
+			{
+				if (tabletProximityRecord.enterProximity) {
+					//pointer is entering tablet area proximity
+					
+					switch(tabletProximityRecord.pointerType)
+					{
+						case 1: /* stylus */
+							ct.Active = 1;
+							break;
+						case 2: /* puck, not supported so far */
+							ct.Active = 0;
+							break;
+						case 3: /* eraser */
+							ct.Active = 2;
+							break;
+						default:
+							ct.Active = 0;
+							break;
+					}
+				} else {
+					// pointer is leaving - return to mouse
+					ct.Active = 0;
+				}
+			}
+		}
+	err = noErr;
+	}
+}
+
 OSStatus GHOST_SystemCarbon::handleMouseEvent(EventRef event)
 {
     OSStatus err = eventNotHandledErr;
@@ -708,6 +787,9 @@ OSStatus GHOST_SystemCarbon::handleMouseEvent(EventRef event)
 				
 				/* Window still gets mouse up after command-H */
 				if (m_windowManager->getActiveWindow()) {
+					// handle any tablet events that may have come with the mouse event (optional)
+					handleTabletEvent(event);
+					
 					::GetEventParameter(event, kEventParamMouseButton, typeMouseButton, NULL, sizeof(button), NULL, &button);
 					pushEvent(new GHOST_EventButton(getMilliSeconds(), type, window, convertButton(button)));
 					err = noErr;
@@ -716,15 +798,19 @@ OSStatus GHOST_SystemCarbon::handleMouseEvent(EventRef event)
             break;
 			
 		case kEventMouseMoved:
-        case kEventMouseDragged:
-			Point mousePos;
+		case kEventMouseDragged: {
+ 			Point mousePos;
+
 			if (window) {
+				//handle any tablet events that may have come with the mouse event (optional)
+				handleTabletEvent(event);
+
 				::GetEventParameter(event, kEventParamMouseLocation, typeQDPoint, NULL, sizeof(Point), NULL, &mousePos);
 				pushEvent(new GHOST_EventCursor(getMilliSeconds(), GHOST_kEventCursorMove, window, mousePos.h, mousePos.v));
 				err = noErr;
-			}
-            break;
-
+ 			}
+			break;
+		}
 		case kEventMouseWheelMoved:
 			{
 				OSStatus status;
