@@ -1,5 +1,5 @@
 /**
- * $Id: GHOST_SystemWin32.cpp,v 1.10 2004/11/07 16:30:19 bjornmose Exp $
+ * $Id: GHOST_SystemWin32.cpp,v 1.11 2006/11/05 21:55:27 elubie Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -31,7 +31,7 @@
 
 /**
 
- * $Id: GHOST_SystemWin32.cpp,v 1.10 2004/11/07 16:30:19 bjornmose Exp $
+ * $Id: GHOST_SystemWin32.cpp,v 1.11 2006/11/05 21:55:27 elubie Exp $
  * Copyright (C) 2001 NaN Technologies B.V.
  * @author	Maarten Gribnau
  * @date	May 7, 2001
@@ -627,7 +627,16 @@ LRESULT WINAPI GHOST_SystemWin32::s_wndProc(HWND hwnd, UINT msg, WPARAM wParam,
 					 * a dead key that is pressed while holding down the alt key. 
 					 */
 					break;
-
+				////////////////////////////////////////////////////////////////////////
+				// Tablet events, processed
+				////////////////////////////////////////////////////////////////////////
+				case WT_PACKET:
+					((GHOST_WindowWin32*)window)->processWin32TabletEvent(wParam, lParam);
+					break;
+				case WT_CSRCHANGE:
+				case WT_PROXIMITY:
+					((GHOST_WindowWin32*)window)->processWin32TabletInitEvent();
+					break;
 				////////////////////////////////////////////////////////////////////////
 				// Mouse events, processed
 				////////////////////////////////////////////////////////////////////////
