@@ -1,5 +1,5 @@
 /**
- * $Id: GHOST_WindowX11.cpp,v 1.8 2003/11/06 00:01:50 rwenzlaff Exp $
+ * $Id: GHOST_WindowX11.cpp,v 1.17 2006/11/11 22:47:41 blendix Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -29,10 +29,6 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "GHOST_WindowX11.h"
 #include "GHOST_SystemX11.h"
 #include "STR_String.h"
@@ -45,10 +41,10 @@
 // lifted verbatim from blut.
 
 typedef struct {
-  long flags;
-  long functions;
-  long decorations;
-  long input_mode;
+	long flags;
+	long functions;
+	long decorations;
+	long input_mode;
 } MotifWmHints;
 
 #define MWM_HINTS_DECORATIONS         (1L << 1)
@@ -68,7 +64,7 @@ GHOST_WindowX11(
 	GHOST_TDrawingContextType type,
 	const bool stereoVisual
 ) :
-	GHOST_Window(title,left,top,width,height,state,type),
+	GHOST_Window(title,left,top,width,height,state,type,stereoVisual),
 	m_display(display),
 	m_valid_setup (false),
 	m_system (system),
@@ -193,7 +189,9 @@ GHOST_WindowX11(
 	XFree(xclasshint);
 
 	setTitle(title);
-	
+
+	initXInputDevices();
+
 	// now set up the rendering context.
 	if (installDrawingContext(type) == GHOST_kSuccess) {
 		m_valid_setup = true;
@@ -206,6 +204,102 @@ GHOST_WindowX11(
 	XFlush(m_display);
 }
 
+/* 
+	Dummy function to get around IO Handler exiting if device invalid
+	Basically it will not crash blender now if you have a X device that 
+	is configured but not plugged in.
+
+*/
+static int ApplicationErrorHandler(Display *display, XErrorEvent *theEvent) {
+	fprintf(stderr, "Ignoring Xlib error: error code %d request code %d\n",
+		theEvent->error_code, theEvent->request_code) ;
+
+	/* No exit! - but keep lint happy */
+	return 0 ;
+}
+
+void GHOST_WindowX11::initXInputDevices()
+{
+	static XErrorHandler old_handler = (XErrorHandler) 0 ;
+	XExtensionVersion *version = XGetExtensionVersion(m_display, INAME);
+
+	if(version && (version != (XExtensionVersion*)NoSuchExtension)) {
+		if(version->present) {
+			int device_count;
+			XDeviceInfo* device_info = XListInputDevices(m_display, &device_count);
+			m_xtablet.StylusDevice = 0;
+			m_xtablet.EraserDevice = 0;
+			m_xtablet.CommonData.Active= 0;
+
+			/* Install our error handler to override Xlib's termination behavior */
+			old_handler = XSetErrorHandler(ApplicationErrorHandler) ;
+
+			for(int i=0; i<device_count; ++i) {
+				if(!strcasecmp(device_info[i].name, "stylus")) {
+					m_xtablet.StylusID= device_info[i].id;
+					m_xtablet.StylusDevice = XOpenDevice(m_display, m_xtablet.StylusID);
+
+					if (m_xtablet.StylusDevice != NULL) {
+						/* Find how many pressure levels tablet has */
+						XAnyClassPtr ici = device_info[i].inputclassinfo;
+						for(int j=0; j<m_xtablet.StylusDevice->num_classes; ++j) {
+							if(ici->c_class==ValuatorClass) {
+								XValuatorInfo* xvi = (XValuatorInfo*)ici;
+								m_xtablet.PressureLevels = xvi->axes[2].max_value;
+							
+								/* this is assuming that the tablet has the same tilt resolution in both
+								 * positive and negative directions. It would be rather weird if it didn't.. */
+								m_xtablet.XtiltLevels = xvi->axes[3].max_value;
+								m_xtablet.YtiltLevels = xvi->axes[4].max_value;
+								break;
+							}
+						
+							ici = (XAnyClassPtr)(((char *)ici) + ici->length);
+						}
+					} else {
+ 						m_xtablet.StylusID= 0;
+					}
+				}
+				if(!strcasecmp(device_info[i].name, "eraser")) {
+					m_xtablet.EraserID= device_info[i].id;
+					m_xtablet.EraserDevice = XOpenDevice(m_display, m_xtablet.EraserID);
+					if (m_xtablet.EraserDevice == NULL) m_xtablet.EraserID= 0;
+				}
+			}
+
+			/* Restore handler */
+			(void) XSetErrorHandler(old_handler) ;
+
+			XFreeDeviceList(device_info);
+
+
+			XEventClass xevents[10], ev;
+			int dcount = 0;
+
+			if(m_xtablet.StylusDevice) {
+				DeviceMotionNotify(m_xtablet.StylusDevice, m_xtablet.MotionEvent, ev);
+				if(ev) xevents[dcount++] = ev;
+				ProximityIn(m_xtablet.StylusDevice, m_xtablet.ProxInEvent, ev);
+				if(ev) xevents[dcount++] = ev;
+				ProximityOut(m_xtablet.StylusDevice, m_xtablet.ProxOutEvent, ev);
+				if(ev) xevents[dcount++] = ev;
+			}
+			if(m_xtablet.EraserDevice) {
+				DeviceMotionNotify(m_xtablet.EraserDevice, m_xtablet.MotionEvent, ev);
+				if(ev) xevents[dcount++] = ev;
+				ProximityIn(m_xtablet.EraserDevice, m_xtablet.ProxInEvent, ev);
+				if(ev) xevents[dcount++] = ev;
+				ProximityOut(m_xtablet.EraserDevice, m_xtablet.ProxOutEvent, ev);
+				if(ev) xevents[dcount++] = ev;
+			}
+
+			XSelectExtensionEvent(m_display, m_window, xevents, dcount);
+		}
+		XFree(version);
+	}
+}	
+
+
 	Window 
 GHOST_WindowX11::
 getXWindow(
@@ -390,6 +484,9 @@ setState(
 
 }
 
+#include <iostream>
+using namespace std;
+
 	GHOST_TSuccess 
 GHOST_WindowX11::
 setOrder(
@@ -397,14 +494,45 @@ setOrder(
 ){
 	if (order == GHOST_kWindowOrderTop) {
 		XWindowAttributes attr;	  
+		Atom atom;
 
-		XRaiseWindow(m_display,m_window);
+		/* We use both XRaiseWindow and _NET_ACTIVE_WINDOW, since some
+		   window managers ignore the former (e.g. kwin from kde) and others
+		   don't implement the latter (e.g. fluxbox pre 0.9.9) */
+
+		XRaiseWindow(m_display, m_window);
+
+		atom = XInternAtom(m_display, "_NET_ACTIVE_WINDOW", True);
+
+		if (atom != None) {
+			Window root;
+			XEvent xev;
+			long eventmask;
+
+			xev.xclient.type = ClientMessage;
+			xev.xclient.serial = 0;
+			xev.xclient.send_event = True;
+			xev.xclient.window = m_window;
+			xev.xclient.message_type = atom;
+
+			xev.xclient.format = 32;
+			xev.xclient.data.l[0] = 0;
+			xev.xclient.data.l[1] = 0;
+			xev.xclient.data.l[2] = 0;
+			xev.xclient.data.l[3] = 0;
+			xev.xclient.data.l[4] = 0;
+
+			root = RootWindow(m_display, m_visual->screen),
+			eventmask = SubstructureRedirectMask | SubstructureNotifyMask;
+
+			XSendEvent(m_display, root, False, eventmask, &xev);
+		}
 
 		XGetWindowAttributes(m_display, m_window, &attr);
 
 		/* iconized windows give bad match error */
 		if (attr.map_state == IsViewable)
-		  XSetInputFocus(m_display, m_window, RevertToPointerRoot,
+			XSetInputFocus(m_display, m_window, RevertToPointerRoot,
 						 CurrentTime);
 		XFlush(m_display);
 	} else if (order == GHOST_kWindowOrderBottom) {
@@ -640,7 +768,7 @@ getEmptyCursor(
 		m_empty_cursor = XCreatePixmapCursor(m_display, blank, blank, &dummy, &dummy, 0, 0);
 		XFreePixmap(m_display, blank);
 	}
-    
+
 	return m_empty_cursor;
 }
 
@@ -684,6 +812,24 @@ setWindowCustomCursorShape(
 	int hotX, 
 	int hotY
 ){
+
+setWindowCustomCursorShape((GHOST_TUns8*)bitmap, (GHOST_TUns8*)mask, 
+									16, 16, hotX, hotY, 0, 1);
+	return GHOST_kSuccess;
+}
+
+	GHOST_TSuccess
+GHOST_WindowX11::
+setWindowCustomCursorShape(	
+	GHOST_TUns8 *bitmap, 
+	GHOST_TUns8 *mask, 
+	int sizex, 
+	int sizey, 
+	int hotX, 
+	int hotY, 
+	int fg_color, 
+	int bg_color
+){
 	Pixmap bitmap_pix, mask_pix;
 	XColor fg, bg;
 	
@@ -696,8 +842,8 @@ setWindowCustomCursorShape(
 		XFreeCursor(m_display, m_custom_cursor);
 	}
 
-	bitmap_pix = XCreateBitmapFromData(m_display,  m_window, (char*) bitmap, 16, 16);
-	mask_pix = XCreateBitmapFromData(m_display, m_window, (char*) mask, 16, 16);
+	bitmap_pix = XCreateBitmapFromData(m_display, m_window, (char*) bitmap, sizex, sizey);
+	mask_pix = XCreateBitmapFromData(m_display, m_window, (char*) mask, sizex, sizey);
 		
 	m_custom_cursor = XCreatePixmapCursor(m_display, bitmap_pix, mask_pix, &fg, &bg, hotX, hotY);
 	XDefineCursor(m_display, m_window, m_custom_cursor);
@@ -723,7 +869,7 @@ void glutCustomCursor(char *data1, char *data2, int size)
 		"Red", &bg, &bg) == 0) return;
 
 
-	source= XCreateBitmapFromData(__glutDisplay,  xdraw, data2, size, size);
+	source= XCreateBitmapFromData(__glutDisplay, xdraw, data2, size, size);
 	mask= XCreateBitmapFromData(__glutDisplay, xdraw, data1, size, size);
 		
 	cursor= XCreatePixmapCursor(__glutDisplay, source, mask, &fg, &bg, 7, 7);
