@@ -4,7 +4,7 @@
  * Stuff that definitely does not belong in the kernel! These will all
  * have to be removed in order to restore sanity.
  *
- * $Id: BKE_bad_level_calls.h,v 1.24 2006/06/12 00:10:00 theeth Exp $ 
+ * $Id: BKE_bad_level_calls.h,v 1.35 2007/01/24 01:34:04 nicholasbishop Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -66,7 +66,6 @@ struct Object;
 void BPY_do_pyscript (struct ID *id, short int event);
 void BPY_clear_script (struct Script *script);
 void BPY_free_compiled_text (struct Text *text);
-void BPY_free_screen_spacehandlers (struct bScreen *sc);
 /* pydrivers */
 struct Object **BPY_pydriver_get_objects(struct IpoDriver *driver);
 float BPY_pydriver_eval(struct IpoDriver *driver);
@@ -146,9 +145,75 @@ void bglVertex3fv(float *vec);
 void bglVertex3f(float x, float y, float z);
 void bglEnd(void);
 
-struct DispListMesh;
 struct Object;
-struct DispListMesh *NewBooleanMeshDLM(struct Object *ob, struct Object *ob_select, int int_op_type);
+
+/* booleanops.c */
+struct DerivedMesh *NewBooleanDerivedMesh(struct Object *ob,
+                                struct Object *ob_select, int int_op_type);
+
+/* verse_*.c */
+struct VerseVert;
+struct VerseFace;
+struct VerseSession;
+struct VNode;
+struct VTag;
+struct VTagGroup;
+struct VBitmapLayer;
+struct VLink;
+struct VLayer;
+struct Mesh;
+
+void post_vertex_create(struct VerseVert *vvert);
+void post_vertex_set_xyz(struct VerseVert *vvert);
+void post_vertex_delete(struct VerseVert *vvert);
+void post_vertex_free_constraint(struct VerseVert *vvert);
+void post_polygon_create(struct VerseFace *vface);
+void post_polygon_set_corner(struct VerseFace *vface);
+void post_polygon_delete(struct VerseFace *vface);
+void post_polygon_free_constraint(struct VerseFace *vface);
+void post_polygon_set_uint8(struct VerseFace *vface);
+void post_node_create(struct VNode *vnode);
+void post_node_destroy(struct VNode *vnode);
+void post_node_name_set(struct VNode *vnode);
+void post_tag_change(struct VTag *vtag);
+void post_taggroup_create(struct VTagGroup *vtaggroup);
+char *verse_client_name(void);
+void post_transform(struct VNode *vnode);
+void post_object_free_constraint(struct VNode *vnode);
+void post_link_set(struct VLink *vlink);
+void post_link_destroy(struct VLink *vlink);
+void post_connect_accept(struct VerseSession *session);
+void post_connect_terminated(struct VerseSession *session);
+void post_connect_update(struct VerseSession *session);
+void add_screenhandler(struct bScreen *sc, short eventcode, short val);
+void post_bitmap_dimension_set(struct VNode *vnode);
+void post_bitmap_layer_create(struct VBitmapLayer *vblayer);
+void post_bitmap_layer_destroy(struct VBitmapLayer *vblayer);
+void post_bitmap_tile_set(struct VBitmapLayer *vblayer, unsigned int xs, unsigned int ys);
+void create_meshdata_from_geom_node(struct Mesh *me, struct VNode *vnode);
+void post_geometry_free_constraint(struct VNode *vnode);
+void post_layer_create(struct VLayer *vlayer);
+void post_layer_destroy(struct VLayer *vlayer);
+void post_server_add(void);
+
+/* multires.c */
+struct Multires;
+struct MultiresLevel;
+struct MultiresLevel *multires_level_n(struct Multires *mr, int n);
+void multires_free(struct Multires *mr);
+void multires_set_level(void *ob, void *me_v);
+void multires_update_levels(struct Mesh *me);
+void multires_calc_level_maps(struct MultiresLevel *lvl);
+struct Multires *multires_copy(struct Multires *orig);
+/* sculptmode.c */
+void sculptmode_free_all(struct Scene *sce);
+void sculptmode_init(struct Scene *sce);
+
+/* zbuf.c */
+void antialias_tagbuf(int xsize, int ysize, char *rectmove);
+
+/* imagetexture.c */
+void ibuf_sample(struct ImBuf *ibuf, float fx, float fy, float dx, float dy, float *result);
 
 #endif
 
