@@ -3,7 +3,7 @@
  *	
  * Library
  *
- * $Id: BKE_library.h,v 1.8 2006/01/28 18:33:06 hos Exp $ 
+ * $Id: BKE_library.h,v 1.10 2006/12/20 17:57:41 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -50,6 +50,8 @@ void id_us_plus(struct ID *id);
 int new_id(struct ListBase *lb, struct ID *id, const char *name);
 
 struct ListBase *wich_libbase(struct Main *mainlib, short type);
+
+#define MAX_LIBARRAY	40
 int set_listbasepointers(struct Main *main, struct ListBase **lb);
 
 void free_libblock(struct ListBase *lb, void *idv);
@@ -64,6 +66,7 @@ struct ID *find_id(char *type, char *name);
 void clear_id_newpoins(void);
 
 void IDnames_to_pupstring(char **str, char *title, char *extraops, struct ListBase *lb,struct ID* link, short *nr);
+void IMAnames_to_pupstring(char **str, char *title, char *extraops, struct ListBase *lb, struct ID *link, short *nr);
 void IPOnames_to_pupstring(char **str, char *title, char *extraops, struct ListBase *lb, struct ID* link, short *nr, int blocktype);
 
 #endif
