@@ -1,5 +1,5 @@
 /**
- * $Id: BKE_node.h,v 1.15 2006/07/03 09:49:12 ton Exp $
+ * $Id: BKE_node.h,v 1.38 2007/02/07 18:47:48 scourage Exp $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -43,6 +43,7 @@ struct uiBlock;
 struct rctf;
 struct ListBase;
 struct RenderData;
+struct Scene;
 
 #define SOCK_IN		1
 #define SOCK_OUT	2
@@ -88,6 +89,8 @@ typedef struct bNodeType {
 #define NODE_CLASS_GROUP		6
 #define NODE_CLASS_FILE			7
 #define NODE_CLASS_CONVERTOR	8
+#define NODE_CLASS_MATTE		9
+#define NODE_CLASS_DISTORT		10
 
 /* ************** GENERIC API, TREES *************** */
 
@@ -177,7 +180,10 @@ struct ShadeResult;
 #define SH_NODE_MAPPING		109
 #define SH_NODE_CURVE_VEC	110
 #define SH_NODE_CURVE_RGB	111
-
+#define SH_NODE_CAMERA		114
+#define SH_NODE_MATH		115
+#define SH_NODE_VECT_MATH	116
+#define SH_NODE_SQUEEZE	    117
 /* custom defines: options for Material node */
 #define SH_NODE_MAT_DIFF	1
 #define SH_NODE_MAT_SPEC	2
@@ -189,7 +195,7 @@ extern bNodeType *node_all_shaders[];
 /* API */
 
 void			ntreeShaderExecTree(struct bNodeTree *ntree, struct ShadeInput *shi, struct ShadeResult *shr);
-int				ntreeShaderGetTexco(struct bNodeTree *ntree, int osa);
+void			ntreeShaderGetTexcoMode(struct bNodeTree *ntree, int osa, short *texco, int *mode);
 void			nodeShaderSynchronizeID(struct bNode *node, int copyto);
 
 				/* switch material render loop */
@@ -224,9 +230,35 @@ void			set_node_shader_lamp_loop(void (*lamp_loop_func)(struct ShadeInput *, str
 #define CMP_NODE_TEXTURE		224
 #define CMP_NODE_TRANSLATE		225
 #define CMP_NODE_ZCOMBINE		226
+#define CMP_NODE_COMBRGBA		227
+#define CMP_NODE_DILATEERODE	228
+#define CMP_NODE_ROTATE			229
+#define CMP_NODE_SCALE			230
+#define CMP_NODE_SEPYCCA		231
+#define CMP_NODE_COMBYCCA		232
+#define CMP_NODE_SEPYUVA		233
+#define CMP_NODE_COMBYUVA		234
+#define CMP_NODE_DIFF_MATTE		235
+#define CMP_NODE_COLOR_SPILL	236
+#define CMP_NODE_CHROMA			237
+#define CMP_NODE_CHANNEL_MATTE	238
+#define CMP_NODE_FLIP			239
+#define CMP_NODE_SPLITVIEWER	240
+#define CMP_NODE_INDEX_MASK		241
+#define CMP_NODE_MAP_UV			242
+#define CMP_NODE_ID_MASK		243
+#define CMP_NODE_DEFOCUS		244
+#define CMP_NODE_DISPLACE		245
+#define CMP_NODE_COMBHSVA		246
+#define CMP_NODE_MATH			247
+#define CMP_NODE_LUMA_MATTE		248
 
 
 /* filter types */
+
+
+/* filter types, in custom1 */
+
 #define CMP_FILT_SOFT		0
 #define CMP_FILT_SHARP		1
 #define CMP_FILT_LAPLACE	2
@@ -235,17 +267,20 @@ void			set_node_shader_lamp_loop(void (*lamp_loop_func)(struct ShadeInput *, str
 #define CMP_FILT_KIRSCH		5
 #define CMP_FILT_SHADOW		6
 
+/* scale node type, in custom1 */
+#define CMP_SCALE_RELATIVE	0
+#define CMP_SCALE_ABSOLUTE	1
 
 /* the type definitions array */
 extern bNodeType *node_all_composit[];
 
 /* API */
 struct CompBuf;
-void ntreeCompositTagRender(struct bNodeTree *ntree);
+void ntreeCompositTagRender(struct Scene *sce);
 void ntreeCompositTagAnimated(struct bNodeTree *ntree);
 void ntreeCompositTagGenerators(struct bNodeTree *ntree);
+void ntreeCompositForceHidden(struct bNodeTree *ntree);
 
 void free_compbuf(struct CompBuf *cbuf); /* internal...*/
 
 #endif
-
