@@ -1,5 +1,5 @@
 /**
- * $Id: BKE_object.h,v 1.16 2006/06/14 08:50:41 ton Exp $
+ * $Id: BKE_object.h,v 1.21 2007/01/24 04:44:49 jesterking Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -44,6 +44,7 @@ struct Camera;
 struct BoundBox;
 struct View3D;
 struct SoftBody;
+struct Group;
 
 void clear_workob(void);
 void copy_baseflags(void);
@@ -55,6 +56,8 @@ void free_object(struct Object *ob);
 void object_free_display(struct Object *ob);
 void object_free_modifiers(struct Object *ob);
 
+void object_make_proxy(struct Object *ob, struct Object *target, struct Object *gob);
+
 void unlink_object(struct Object *ob);
 int exist_object(struct Object *obtest);
 void *add_camera(void);
@@ -67,6 +70,7 @@ void free_camera(struct Camera *ca);
 void free_lamp(struct Lamp *la);
 void *add_wave(void);
 
+struct Object *add_only_object(int type, char *name);
 struct Object *add_object(int type);
 void base_init_from_view3d(struct Base *base, struct View3D *v3d);
 
@@ -96,6 +100,7 @@ void what_does_parent(struct Object *ob);
 struct BoundBox *unit_boundbox(void);
 void boundbox_set_from_min_max(struct BoundBox *bb, float min[3], float max[3]);
 struct BoundBox *object_get_boundbox(struct Object *ob);
+void object_boundbox_flag(struct Object *ob, int flag, int set);
 void minmax_object(struct Object *ob, float *min, float *max);
 void solve_tracking (struct Object *ob, float targetmat[][4]);
 void solve_constraints (struct Object *ob, short obtype, void *obdata, float ctime);
