@@ -1,7 +1,7 @@
 /**
  * blenlib/BKE_text.h (mar-2001 nzc)
  *	
- * $Id: BKE_text.h,v 1.10 2005/04/10 14:01:41 stiv Exp $ 
+ * $Id: BKE_text.h,v 1.12 2006/03/05 19:50:13 themyers Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -94,6 +94,7 @@ void 	comment			(struct Text *text);
 void 	indent			(struct Text *text);
 void	uncomment		(struct Text *text);
 int	setcurr_tab		(struct Text *text);
+void	convert_tabs		(struct SpaceText *st, int tab);
 
 /* Undo opcodes */
 
@@ -131,6 +132,11 @@ int	setcurr_tab		(struct Text *text);
 /* Misc */
 #define UNDO_SWAP		031	/* Swap cursors */
 
+#define UNDO_INDENT		032
+#define UNDO_UNINDENT		033
+#define UNDO_COMMENT		034
+#define UNDO_UNCOMMENT		035
+
 #ifdef __cplusplus
 }
 #endif
