@@ -1,7 +1,5 @@
-/* util defines  -- might go away ?*/
-
 /* 
-	$Id: BKE_utildefines.h,v 1.13 2004/12/27 19:28:48 ton Exp $
+	$Id: BKE_utildefines.h,v 1.21 2007/01/14 09:30:04 aligorith Exp $
 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
@@ -44,10 +42,12 @@
 #define TRUE 1
 #endif
 
-/* also fill in structs itself, dna cannot handle defines, duplicate in blendef.h still */
+/* these values need to be hardcoded in structs, dna does not recognize defines */
+/* also defined in DNA_space_types.h */
 #ifndef FILE_MAXDIR
 #define FILE_MAXDIR			160
 #define FILE_MAXFILE		80
+#define FILE_MAX			240
 #endif
 
 #define ELEM(a, b, c)           ( (a)==(b) || (a)==(c) )
@@ -58,7 +58,6 @@
 #define ELEM7(a, b, c, d, e, f, g, h)   ( ELEM3(a, b, c, d) || ELEM4(a, e, f, g, h) )
 #define ELEM8(a, b, c, d, e, f, g, h, i)        ( ELEM4(a, b, c, d, e) || ELEM4(a, f, g, h, i) )
 
-	
 /* string compare */
 #define STREQ(str, a)           ( strcmp((str), (a))==0 )
 #define STREQ2(str, a, b)       ( STREQ(str, a) || STREQ(str, b) )
@@ -73,21 +72,21 @@
 #define MAX3(x,y,z)             MAX2( MAX2((x),(y)) , (z) )
 #define MAX4(x,y,z,a)           MAX2( MAX2((x),(y)) , MAX2((z),(a)) )
 
-#define INIT_MINMAX(min, max) (min)[0]= (min)[1]= (min)[2]= 1.0e30; (max)[0]= (max)[1]= (max)[2]= -1.0e30;
+#define INIT_MINMAX(min, max) { (min)[0]= (min)[1]= (min)[2]= 1.0e30f; (max)[0]= (max)[1]= (max)[2]= -1.0e30f; }
 
-#define INIT_MINMAX2(min, max) (min)[0]= (min)[1]= 1.0e30; (max)[0]= (max)[1]= -1.0e30;
+#define INIT_MINMAX2(min, max) { (min)[0]= (min)[1]= 1.0e30f; (max)[0]= (max)[1]= -1.0e30f; }
 
-#define DO_MINMAX(vec, min, max) if( (min)[0]>(vec)[0] ) (min)[0]= (vec)[0]; \
+#define DO_MINMAX(vec, min, max) { if( (min)[0]>(vec)[0] ) (min)[0]= (vec)[0]; \
 							  if( (min)[1]>(vec)[1] ) (min)[1]= (vec)[1]; \
 							  if( (min)[2]>(vec)[2] ) (min)[2]= (vec)[2]; \
 							  if( (max)[0]<(vec)[0] ) (max)[0]= (vec)[0]; \
 							  if( (max)[1]<(vec)[1] ) (max)[1]= (vec)[1]; \
-							  if( (max)[2]<(vec)[2] ) (max)[2]= (vec)[2]; \
+							  if( (max)[2]<(vec)[2] ) (max)[2]= (vec)[2]; } \
 
-#define DO_MINMAX2(vec, min, max) if( (min)[0]>(vec)[0] ) (min)[0]= (vec)[0]; \
+#define DO_MINMAX2(vec, min, max) { if( (min)[0]>(vec)[0] ) (min)[0]= (vec)[0]; \
 							  if( (min)[1]>(vec)[1] ) (min)[1]= (vec)[1]; \
 							  if( (max)[0]<(vec)[0] ) (max)[0]= (vec)[0]; \
-							  if( (max)[1]<(vec)[1] ) (max)[1]= (vec)[1];
+							  if( (max)[1]<(vec)[1] ) (max)[1]= (vec)[1]; }
 
 #define MINSIZE(val, size)	( ((val)>=0.0) ? (((val)<(size)) ? (size): (val)) : ( ((val)>(-size)) ? (-size) : (val)))
 
@@ -104,6 +103,7 @@
 
 #define VECADD(v1,v2,v3) 	{*(v1)= *(v2) + *(v3); *(v1+1)= *(v2+1) + *(v3+1); *(v1+2)= *(v2+2) + *(v3+2);}
 #define VECSUB(v1,v2,v3) 	{*(v1)= *(v2) - *(v3); *(v1+1)= *(v2+1) - *(v3+1); *(v1+2)= *(v2+2) - *(v3+2);}
+#define VECADDFAC(v1,v2,v3,fac) {*(v1)= *(v2) + *(v3)*(fac); *(v1+1)= *(v2+1) + *(v3+1)*(fac); *(v1+2)= *(v2+2) + *(v3+2)*(fac);}
 
 #define INPR(v1, v2)		( (v1)[0]*(v2)[0] + (v1)[1]*(v2)[1] + (v1)[2]*(v2)[2] )
 
@@ -115,11 +115,15 @@
 
 #define IS_EQ(a,b) ((fabs((double)(a)-(b)) >= (double) FLT_EPSILON) ? 0 : 1)
 
+#define IS_EQT(a, b, c) ((a > b)? (((a-b) <= c)? 1:0) : ((((b-a) <= c)? 1:0)))
+#define IN_RANGE(a, b, c) ((b < c)? ((b<a && a<c)? 1:0) : ((c<a && a<b)? 1:0))
 
 /* this weirdo pops up in two places ... */
 #if !defined(WIN32) && !defined(__BeOS)
+#ifndef O_BINARY
 #define O_BINARY 0
 #endif
+#endif
 
 /* INTEGER CODES */
 #if defined(__sgi) || defined (__sparc) || defined (__sparc__) || defined (__PPC__) || defined (__ppc__) || defined (__BIG_ENDIAN__)
@@ -163,6 +167,11 @@
     s_i=p_i[0]; p_i[0]=p_i[3]; p_i[3]=s_i; \
     s_i=p_i[1]; p_i[1]=p_i[2]; p_i[2]=s_i; }
 
+#define SWITCH_SHORT(a)	{ \
+    char s_i, *p_i; \
+		p_i= (char *)&(a); \
+			s_i=p_i[0]; p_i[0]=p_i[1]; p_i[1]=s_i; }
+
 
 /* Bit operations */
 #define BTST(a,b)     ( ( (a) & 1<<(b) )!=0 )   
