@@ -1,7 +1,7 @@
 /** anim.c
  *
  *
- * $Id: anim.c,v 1.37 2006/06/23 17:56:15 ton Exp $
+ * $Id: anim.c,v 1.45 2006/12/05 17:42:01 artificer Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -114,8 +114,8 @@ void calc_curvepath(Object *ob)
 	else tot= bl->nr-1;
 	
 	path->len= tot+1;
-	/* exception: vector handle paths and polygon paths should be subdivided at least a factor 6 (or more?) */
-	if(path->len<6*nu->pntsu) path->len= 6*nu->pntsu;
+	/* exception: vector handle paths and polygon paths should be subdivided at least a factor resolu */
+	if(path->len<nu->resolu*nu->pntsu) path->len= nu->resolu*nu->pntsu;
 	
 	dist= (float *)MEM_mallocN((tot+1)*4, "calcpathdist");
 
@@ -277,9 +277,9 @@ int where_on_path(Object *ob, float ctime, float *vec, float *dir)	/* returns OK
 
 /* ****************** DUPLICATOR ************** */
 
-static void new_dupli_object(ListBase *lb, Object *ob, float mat[][4], int lay, int index)
+static DupliObject *new_dupli_object(ListBase *lb, Object *ob, float mat[][4], int lay, int index)
 {
-	DupliObject *dob= MEM_mallocN(sizeof(DupliObject), "dupliobject");
+	DupliObject *dob= MEM_callocN(sizeof(DupliObject), "dupliobject");
 	
 	BLI_addtail(lb, dob);
 	dob->ob= ob;
@@ -297,30 +297,39 @@ static void new_dupli_object(ListBase *lb, Object *ob, float mat[][4], int lay,
 			Mat4CpyMat4(paf->imat, ob->imat);
 		}
 	}
+	return dob;
 }
 
-static void group_duplilist(ListBase *lb, Object *ob)
+static void group_duplilist(ListBase *lb, Object *ob, int level)
 {
 	DupliObject *dob;
+	Group *group;
 	GroupObject *go;
 	float mat[4][4];
 	
 	if(ob->dup_group==NULL) return;
+	group= ob->dup_group;
+	
+	/* simple preventing of too deep nested groups */
+	if(level>4) return;
 	
 	/* handles animated groups, and */
 	/* we need to check update for objects that are not in scene... */
-	group_handle_recalc_and_update(ob, ob->dup_group);
+	group_handle_recalc_and_update(ob, group);
 	
-	for(go= ob->dup_group->gobject.first; go; go= go->next) {
+	for(go= group->gobject.first; go; go= go->next) {
+		/* note, if you check on layer here, render goes wrong... it still deforms verts and uses parent imat */
 		if(go->ob!=ob) {
 			Mat4MulMat4(mat, go->ob->obmat, ob->obmat);
-			new_dupli_object(lb, go->ob, mat, ob->lay, 0);
+			dob= new_dupli_object(lb, go->ob, mat, ob->lay, 0);
+			dob->no_draw= (dob->origlay & group->layer)==0;
+			if(go->ob->dup_group) {
+				Mat4CpyMat4(dob->ob->obmat, dob->mat);
+				group_duplilist(lb, go->ob, level+1);
+				Mat4CpyMat4(dob->ob->obmat, dob->omat);
+			}
 		}
 	}
-	
-	/* make copy already, because in group dupli's deform displists can be makde, requiring parent matrices */
-	for(dob= lb->first; dob; dob= dob->next)
-		Mat4CpyMat4(dob->ob->obmat, dob->mat);
 }
 
 static void frames_duplilist(ListBase *lb, Object *ob)
@@ -394,7 +403,6 @@ static void vertex_duplilist(ListBase *lb, Scene *sce, Object *par)
 	Base *base;
 	float vec[3], no[3], pmat[4][4];
 	int lay, totvert, a;
-	int dmNeedsFree;
 	DerivedMesh *dm;
 	
 	Mat4CpyMat4(pmat, par->obmat);
@@ -402,9 +410,9 @@ static void vertex_duplilist(ListBase *lb, Scene *sce, Object *par)
 	lay= G.scene->lay;
 	
 	if(par==G.obedit)
-		dm= editmesh_get_derived_cage(&dmNeedsFree);
+		dm= editmesh_get_derived_cage(CD_MASK_BAREMESH);
 	else
-		dm = mesh_get_derived_deform(par, &dmNeedsFree);
+		dm = mesh_get_derived_deform(par, CD_MASK_BAREMESH);
 	
 	totvert = dm->getNumVerts(dm);
 
@@ -446,10 +454,114 @@ static void vertex_duplilist(ListBase *lb, Scene *sce, Object *par)
 		base= base->next;
 	}
 
-	if (dmNeedsFree)
-		dm->release(dm);
+	dm->release(dm);
 }
 
+static void face_duplilist(ListBase *lb, Scene *sce, Object *par)
+{
+	Object *ob;
+	Base *base;
+	DerivedMesh *dm;
+	MFace *mface;
+	MVert *mvert;
+	float pmat[4][4], imat[3][3];
+	int lay, totface, a;
+	
+	Mat4CpyMat4(pmat, par->obmat);
+	
+	lay= G.scene->lay;
+	
+	if(par==G.obedit) {
+		int totvert;
+		dm= editmesh_get_derived_cage(CD_MASK_BAREMESH);
+		
+		totface= dm->getNumFaces(dm);
+		mface= MEM_mallocN(sizeof(MFace)*totface, "mface temp");
+		dm->copyFaceArray(dm, mface);
+		totvert= dm->getNumVerts(dm);
+		mvert= MEM_mallocN(sizeof(MVert)*totvert, "mvert temp");
+		dm->copyVertArray(dm, mvert);
+	}
+	else {
+		dm = mesh_get_derived_deform(par, CD_MASK_BAREMESH);
+		
+		totface= dm->getNumFaces(dm);
+		mface= dm->getFaceArray(dm);
+		mvert= dm->getVertArray(dm);
+	}
+	
+	
+	for(base= sce->base.first; base; base= base->next) {
+		
+		if(base->object->type>0 && (lay & base->lay) && G.obedit!=base->object) {
+			ob= base->object->parent;
+			while(ob) {
+				if(ob==par) {
+					
+					ob= base->object;
+					Mat3CpyMat4(imat, ob->parentinv);
+					
+					/* mballs have a different dupli handling */
+					if(ob->type!=OB_MBALL) ob->flag |= OB_DONE;	/* doesnt render */
+
+					for(a=0; a<totface; a++) {
+						float *v1= mvert[ mface[a].v1 ].co;
+						float *v2= mvert[ mface[a].v2 ].co;
+						float *v3= mvert[ mface[a].v3 ].co;
+						float *v4= mface[a].v4?mvert[ mface[a].v4 ].co:NULL;
+						float cent[3], quat[4], mat[3][3], mat3[3][3], tmat[4][4], obmat[4][4];
+
+						/* translation */
+						if(v4)
+							CalcCent4f(cent, v1, v2, v3, v4);
+						else
+							CalcCent3f(cent, v1, v2, v3);
+						Mat4MulVecfl(pmat, cent);
+						
+						VecSubf(cent, cent, pmat[3]);
+						VecAddf(cent, cent, ob->obmat[3]);
+						
+						Mat4CpyMat4(obmat, ob->obmat);
+						VECCOPY(obmat[3], cent);
+						
+						/* rotation */
+						triatoquat(v1, v2, v3, quat);
+						QuatToMat3(quat, mat);
+						
+						/* scale */
+						if(par->transflag & OB_DUPLIFACES_SCALE) {
+							float size= v4?AreaQ3Dfl(v1, v2, v3, v4):AreaT3Dfl(v1, v2, v3);
+							size= sqrt(size);
+							Mat3MulFloat(mat[0], size);
+						}
+						
+						Mat3CpyMat3(mat3, mat);
+						Mat3MulMat3(mat, imat, mat3);
+						
+						Mat4CpyMat4(tmat, obmat);
+						Mat4MulMat43(obmat, tmat, mat);
+						
+						new_dupli_object(lb, ob, obmat, lay, a);
+
+					}
+					
+					break;
+				}
+				ob= ob->parent;
+			}
+		}
+	}
+	
+	if(par==G.obedit) {
+		MEM_freeN(mface);
+		MEM_freeN(mvert);
+	}
+	
+	dm->release(dm);
+}
+
+
+
 static void particle_duplilist(ListBase *lb, Scene *sce, Object *par, PartEff *paf)
 {
 	Object *ob, copyob;
@@ -663,20 +775,31 @@ ListBase *object_duplilist(Scene *sce, Object *ob)
 	if(ob->transflag & OB_DUPLI) {
 		if(ob->transflag & OB_DUPLIVERTS) {
 			if(ob->type==OB_MESH) {
-				if(ob->transflag & OB_DUPLIVERTS) {
-					PartEff *paf;
-					if( (paf=give_parteff(ob)) ) particle_duplilist(&duplilist, sce, ob, paf);
-					else vertex_duplilist(&duplilist, sce, ob);
-				}
+				PartEff *paf;
+				if( (paf=give_parteff(ob)) ) 
+					particle_duplilist(&duplilist, sce, ob, paf);
+				else 
+					vertex_duplilist(&duplilist, sce, ob);
 			}
 			else if(ob->type==OB_FONT) {
 				font_duplilist(&duplilist, ob);
 			}
 		}
+		else if(ob->transflag & OB_DUPLIFACES) {
+			if(ob->type==OB_MESH)
+				face_duplilist(&duplilist, sce, ob);
+		}
 		else if(ob->transflag & OB_DUPLIFRAMES) 
 			frames_duplilist(&duplilist, ob);
-		else if(ob->transflag & OB_DUPLIGROUP)
-			group_duplilist(&duplilist, ob);
+		else if(ob->transflag & OB_DUPLIGROUP) {
+			DupliObject *dob;
+			
+			group_duplilist(&duplilist, ob, 0); /* now recursive */
+			
+			/* make copy already, because in group dupli's deform displists can be makde, requiring parent matrices */
+			for(dob= duplilist.first; dob; dob= dob->next)
+				Mat4CpyMat4(dob->ob->obmat, dob->mat);
+		}
 		
 	}
 	
