@@ -1,5 +1,5 @@
 /**
- * $Id: armature.c,v 1.69 2006/06/03 17:21:45 ton Exp $
+ * $Id: armature.c,v 1.98 2007/02/05 04:52:39 aligorith Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -37,28 +37,32 @@
 #include "BLI_arithb.h"
 #include "BLI_blenlib.h"
 
-#include "DNA_mesh_types.h"
-#include "DNA_meshdata_types.h"
 #include "DNA_armature_types.h"
 #include "DNA_action_types.h"
+#include "DNA_constraint_types.h"
+#include "DNA_mesh_types.h"
+#include "DNA_lattice_types.h"
+#include "DNA_meshdata_types.h"
+#include "DNA_nla_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_view3d_types.h"
-#include "DNA_constraint_types.h"
 
-#include "BKE_curve.h"
-#include "BKE_depsgraph.h"
-#include "BKE_displist.h"
-#include "BKE_global.h"
-#include "BKE_main.h"
-#include "BKE_library.h"
-#include "BKE_blender.h"
 #include "BKE_armature.h"
 #include "BKE_action.h"
+#include "BKE_blender.h"
 #include "BKE_constraint.h"
-#include "BKE_object.h"
-#include "BKE_object.h"
+#include "BKE_curve.h"
 #include "BKE_deform.h"
+#include "BKE_depsgraph.h"
+#include "BKE_DerivedMesh.h"
+#include "BKE_displist.h"
+#include "BKE_global.h"
+#include "BKE_library.h"
+#include "BKE_lattice.h"
+#include "BKE_main.h"
+#include "BKE_object.h"
+#include "BKE_object.h"
 #include "BKE_utildefines.h"
 
 #include "BIF_editdeform.h"
@@ -239,25 +243,6 @@ Bone *get_named_bone (bArmature *arm, const char *name)
 	return bone;
 }
 
-static char *strcasestr_(register char *s, register char *find)
-{
-    register char c, sc;
-    register size_t len;
-	
-    if ((c = *find++) != 0) {
-		c= tolower(c);
-		len = strlen(find);
-		do {
-			do {
-				if ((sc = *s++) == 0)
-					return (NULL);
-				sc= tolower(sc);
-			} while (sc != c);
-		} while (BLI_strncasecmp(s, find, len) != 0);
-		s--;
-    }
-    return ((char *) s);
-}
 
 #define IS_SEPARATOR(a)	(a=='.' || a==' ' || a=='-' || a=='_')
 
@@ -336,7 +321,7 @@ void bone_flip_name (char *name, int strip_number)
 	}
 	else if(len > 5) {
 		/* hrms, why test for a separator? lets do the rule 'ultimate left or right' */
-		index = strcasestr_(prefix, "right");
+		index = BLI_strcasestr(prefix, "right");
 		if (index==prefix || index==prefix+len-5) {
 			if(index[0]=='r') 
 				strcpy (replace, "left");
@@ -350,7 +335,7 @@ void bone_flip_name (char *name, int strip_number)
 			strcpy (suffix, index+5);
 		}
 		else {
-			index = strcasestr_(prefix, "left");
+			index = BLI_strcasestr(prefix, "left");
 			if (index==prefix || index==prefix+len-4) {
 				if(index[0]=='l') 
 					strcpy (replace, "right");
@@ -661,43 +646,69 @@ static void pchan_bone_deform(bPoseChannel *pchan, float weight, float *vec, flo
 	(*contrib)+=weight;
 }
 
-void armature_deform_verts(Object *armOb, Object *target, float (*vertexCos)[3], int numVerts, int deformflag) 
+void armature_deform_verts(Object *armOb, Object *target, DerivedMesh *dm,
+                           float (*vertexCos)[3], int numVerts, int deformflag,
+                           const char *defgrp_name)
 {
 	bPoseChannel *pchan, **defnrToPC = NULL;
-	MDeformVert *dverts= NULL;
+	MDeformVert *dverts = NULL;
+	bDeformGroup *dg;
 	float obinv[4][4], premat[4][4], postmat[4][4];
-	int use_envelope= deformflag & ARM_DEF_ENVELOPE;
-	int numGroups= 0;   /* safety for vertexgroup index overflow too */
-	int i;
+	int use_envelope = deformflag & ARM_DEF_ENVELOPE;
+	int numGroups = 0;		/* safety for vertexgroup index overflow */
+	int i, target_totvert = 0;	/* safety for vertexgroup overflow */
+	int use_dverts = 0;
+	int armature_def_nr = -1;
 
-	if(armOb==G.obedit) return;
+	if(armOb == G.obedit) return;
 	
 	Mat4Invert(obinv, target->obmat);
 	Mat4CpyMat4(premat, target->obmat);
 	Mat4MulMat4(postmat, armOb->obmat, obinv);
-
 	Mat4Invert(premat, postmat);
 
 	/* bone defmats are already in the channels, chan_mat */
 	
 	/* initialize B_bone matrices */
-	for(pchan= armOb->pose->chanbase.first; pchan; pchan= pchan->next) {
+	for(pchan = armOb->pose->chanbase.first; pchan; pchan = pchan->next) {
 		if(!(pchan->bone->flag & BONE_NO_DEFORM))
-			if(pchan->bone->segments>1)
+			if(pchan->bone->segments > 1)
 				pchan_b_bone_defmats(pchan);
 	}
 
+	/* get the def_nr for the overall armature vertex group if present */
+	for(i = 0, dg = target->defbase.first; dg; i++, dg = dg->next)
+		if(defgrp_name && strcmp(defgrp_name, dg->name) == 0)
+			armature_def_nr = i;
+
 	/* get a vertex-deform-index to posechannel array */
 	if(deformflag & ARM_DEF_VGROUP) {
-		if (target->type==OB_MESH){
-			bDeformGroup *dg;
-		
+		if(ELEM(target->type, OB_MESH, OB_LATTICE)) {
 			numGroups = BLI_countlist(&target->defbase);
 			
-			dverts = ((Mesh*)target->data)->dvert;
-			if(dverts) {
-				defnrToPC = MEM_callocN(sizeof(*defnrToPC)*numGroups, "defnrToBone");
-				for (i=0,dg=target->defbase.first; dg; i++,dg=dg->next) {
+			if(target->type==OB_MESH) {
+				Mesh *me= target->data;
+				dverts = me->dvert;
+				target_totvert = me->totvert;
+			}
+			else {
+				Lattice *lt= target->data;
+				dverts = lt->dvert;
+				if(dverts)
+					target_totvert = lt->pntsu*lt->pntsv*lt->pntsw;
+			}
+			/* if we have a DerivedMesh, only use dverts if it has them */
+			if(dm)
+				if(dm->getVertData(dm, 0, CD_MDEFORMVERT))
+					use_dverts = 1;
+				else use_dverts = 0;
+			else if(dverts) use_dverts = 1;
+
+			if(use_dverts) {
+				defnrToPC = MEM_callocN(sizeof(*defnrToPC) * numGroups,
+				                        "defnrToBone");
+				for(i = 0, dg = target->defbase.first; dg;
+				    i++, dg = dg->next) {
 					defnrToPC[i] = get_pose_channel(armOb->pose, dg->name);
 					/* exclude non-deforming bones */
 					if(defnrToPC[i]) {
@@ -709,77 +720,98 @@ void armature_deform_verts(Object *armOb, Object *target, float (*vertexCos)[3],
 		}
 	}
 
-	for(i=0; i<numVerts; i++) {
+	for(i = 0; i < numVerts; i++) {
 		MDeformVert *dvert;
 		float *co = vertexCos[i];
-		float	vec[3];
-		float	contrib=0.0;
-		int		j;
+		float vec[3];
+		float contrib = 0.0f;
+		float armature_weight = 1.0f; /* default to 1 if no overall def group */
+		int	  j;
 
-		vec[0]=vec[1]=vec[2]=0;
+		vec[0] = vec[1] = vec[2] = 0.0f;
 
+		if(use_dverts || armature_def_nr >= 0) {
+			if(dm) dvert = dm->getVertData(dm, i, CD_MDEFORMVERT);
+			else if(i < target_totvert) dvert = dverts + i;
+			else dvert = NULL;
+		} else
+			dvert = NULL;
+
+		if(armature_def_nr >= 0 && dvert) {
+			armature_weight = 0.0f; /* a def group was given, so default to 0 */
+			for(j = 0; j < dvert->totweight; j++) {
+				if(dvert->dw[j].def_nr == armature_def_nr) {
+					armature_weight = dvert->dw[j].weight;
+					break;
+				}
+			}
+		}
+
+		/* check if there's any  point in calculating for this vert */
+		if(armature_weight == 0.0f) continue;
+		
 		/* Apply the object's matrix */
 		Mat4MulVecfl(premat, co);
 		
-		if(dverts && i<((Mesh*)target->data)->totvert)
-			dvert= dverts+i;
-		else
-			dvert= NULL;
-		
-		if(dvert && dvert->totweight) {	// use weight groups ?
-			int deformed= 0;
+		if(use_dverts && dvert && dvert->totweight) { // use weight groups ?
+			int deformed = 0;
 			
-			for (j=0; j<dvert->totweight; j++){
-				int index= dvert->dw[j].def_nr;
-				pchan = index<numGroups?defnrToPC[index]:NULL;
-				if (pchan) {
-					float weight= dvert->dw[j].weight;
-					Bone *bone= pchan->bone;
-					
-					deformed= 1;
+			for(j = 0; j < dvert->totweight; j++){
+				int index = dvert->dw[j].def_nr;
+				pchan = index < numGroups?defnrToPC[index]:NULL;
+				if(pchan) {
+					float weight = dvert->dw[j].weight;
+					Bone *bone = pchan->bone;
+
+					deformed = 1;
 					
 					if(bone && bone->flag & BONE_MULT_VG_ENV) {
-						
-						weight*= distfactor_to_bone(co, bone->arm_head, bone->arm_tail, bone->rad_head, bone->rad_tail, bone->dist);
+						weight *= distfactor_to_bone(co, bone->arm_head,
+						                             bone->arm_tail,
+						                             bone->rad_head,
+						                             bone->rad_tail,
+						                             bone->dist);
 					}
 					pchan_bone_deform(pchan, weight, vec, co, &contrib);
 				}
 			}
-			/* if there are vertexgroups but not groups with bones (like for softbody groups) */
-			if(deformed==0 && use_envelope) {
-				for(pchan= armOb->pose->chanbase.first; pchan; pchan= pchan->next) {
+			/* if there are vertexgroups but not groups with bones
+			 * (like for softbody groups)
+			 */
+			if(deformed == 0 && use_envelope) {
+				for(pchan = armOb->pose->chanbase.first; pchan;
+				    pchan = pchan->next) {
 					if(!(pchan->bone->flag & BONE_NO_DEFORM))
-						contrib+= dist_bone_deform(pchan, vec, co);
+						contrib += dist_bone_deform(pchan, vec, co);
 				}
 			}
 		}
 		else if(use_envelope) {
-			for(pchan= armOb->pose->chanbase.first; pchan; pchan= pchan->next) {
+			for(pchan = armOb->pose->chanbase.first; pchan;
+			    pchan = pchan->next) {
 				if(!(pchan->bone->flag & BONE_NO_DEFORM))
-					contrib+= dist_bone_deform(pchan, vec, co);
+					contrib += dist_bone_deform(pchan, vec, co);
 			}
 		}
 
-		if (contrib>0.0){
-			vec[0]/=contrib;
-			vec[1]/=contrib;
-			vec[2]/=contrib;
+		/* actually should be EPSILON? weight values and contrib can be like 10e-39 small */
+		if(contrib > 0.0001f) {
+			VecMulf(vec, armature_weight / contrib);
+			VecAddf(co, vec, co);
 		}
-
-		VecAddf(co, vec, co);
+		/* always, check above code */
 		Mat4MulVecfl(postmat, co);
 	}
 
-	if (defnrToPC) MEM_freeN(defnrToPC);
+	if(defnrToPC) MEM_freeN(defnrToPC);
 	
 	/* free B_bone matrices */
-	for(pchan= armOb->pose->chanbase.first; pchan; pchan= pchan->next) {
+	for(pchan = armOb->pose->chanbase.first; pchan; pchan = pchan->next) {
 		if(pchan->b_bone_mats) {
 			MEM_freeN(pchan->b_bone_mats);
-			pchan->b_bone_mats= NULL;
+			pchan->b_bone_mats = NULL;
 		}
 	}
-	
 }
 
 /* ************ END Armature Deform ******************* */
@@ -789,6 +821,102 @@ void get_objectspace_bone_matrix (struct Bone* bone, float M_accumulatedMatrix[]
 	Mat4CpyMat4(M_accumulatedMatrix, bone->arm_mat);
 }
 
+/* **************** Space to Space API ****************** */
+
+/* Convert World-Space Matrix to Pose-Space Matrix */
+void armature_mat_world_to_pose(Object *ob, float inmat[][4], float outmat[][4]) 
+{
+	float obmat[4][4];
+	
+	/* prevent crashes */
+	if (ob==NULL) return;
+	
+	/* get inverse of (armature) object's matrix  */
+	Mat4Invert(obmat, ob->obmat);
+	
+	/* multiply given matrix by object's-inverse to find pose-space matrix */
+	Mat4MulMat4(outmat, obmat, inmat);
+}
+
+/* Convert Wolrd-Space Location to Pose-Space Location
+ * NOTE: this cannot be used to convert to pose-space location of the supplied
+ * 		pose-channel into its local space (i.e. 'visual'-keyframing) 
+ */
+void armature_loc_world_to_pose(Object *ob, float *inloc, float *outloc) 
+{
+	float xLocMat[4][4];
+	float nLocMat[4][4];
+	
+	/* build matrix for location */
+	Mat4One(xLocMat);
+	VECCOPY(xLocMat[3], inloc);
+
+	/* get bone-space cursor matrix and extract location */
+	armature_mat_world_to_pose(ob, xLocMat, nLocMat);
+	VECCOPY(outloc, nLocMat[3]);
+}
+
+/* Convert Pose-Space Matrix to Bone-Space Matrix 
+ * NOTE: this cannot be used to convert to pose-space transforms of the supplied
+ * 		pose-channel into its local space (i.e. 'visual'-keyframing)
+ */
+void armature_mat_pose_to_bone(bPoseChannel *pchan, float inmat[][4], float outmat[][4])
+{
+	float pc_trans[4][4], inv_trans[4][4];
+	float pc_posemat[4][4], inv_posemat[4][4];
+	
+	/* paranoia: prevent crashes with no pose-channel supplied */
+	if (pchan==NULL) return;
+	
+	/* get the inverse matrix of the pchan's transforms */
+	LocQuatSizeToMat4(pc_trans, pchan->loc, pchan->quat, pchan->size);
+	Mat4Invert(inv_trans, pc_trans);
+	
+	/* Remove the pchan's transforms from it's pose_mat.
+	 * This should leave behind the effects of restpose + 
+	 * parenting + constraints
+	 */
+	Mat4MulMat4(pc_posemat, inv_trans, pchan->pose_mat);
+	
+	/* get the inverse of the leftovers so that we can remove 
+	 * that component from the supplied matrix
+	 */
+	Mat4Invert(inv_posemat, pc_posemat);
+	
+	/* get the new matrix */
+	Mat4MulMat4(outmat, inmat, inv_posemat);
+}
+
+/* Convert Pose-Space Location to Bone-Space Location
+ * NOTE: this cannot be used to convert to pose-space location of the supplied
+ * 		pose-channel into its local space (i.e. 'visual'-keyframing) 
+ */
+void armature_loc_pose_to_bone(bPoseChannel *pchan, float *inloc, float *outloc) 
+{
+	float xLocMat[4][4];
+	float nLocMat[4][4];
+	
+	/* build matrix for location */
+	Mat4One(xLocMat);
+	VECCOPY(xLocMat[3], inloc);
+
+	/* get bone-space cursor matrix and extract location */
+	armature_mat_pose_to_bone(pchan, xLocMat, nLocMat);
+	VECCOPY(outloc, nLocMat[3]);
+}
+
+/* Remove rest-position effects from pose-transform for obtaining
+ * 'visual' transformation of pose-channel. 
+ * (used by the Visual-Keyframing stuff)
+ */
+void armature_mat_pose_to_delta(float delta_mat[][4], float pose_mat[][4], float arm_mat[][4])
+{
+ 	float imat[4][4];
+ 
+ 	Mat4Invert(imat, arm_mat);
+ 	Mat4MulMat4(delta_mat, pose_mat, imat);
+}
+
 
 /* **************** The new & simple (but OK!) armature evaluation ********* */ 
 
@@ -811,7 +939,8 @@ void get_objectspace_bone_matrix (struct Bone* bone, float M_accumulatedMatrix[]
   *************************************************************************** */
 /*  Computes vector and roll based on a rotation. "mat" must
      contain only a rotation, and no scaling. */ 
-void mat3_to_vec_roll(float mat[][3], float *vec, float *roll) {
+void mat3_to_vec_roll(float mat[][3], float *vec, float *roll) 
+{
      if (vec)
          VecCopyf(vec, mat[1]);
 
@@ -839,11 +968,12 @@ void vec_roll_to_mat3(float *vec, float roll, float mat[][3])
 	
 	/*	Find Axis & Amount for bone matrix*/
 	Crossf (axis,target,nor);
-	
+
 	if (Inpf(axis,axis) > 0.0000000000001) {
 		/* if nor is *not* a multiple of target ... */
 		Normalise (axis);
-		theta=(float) acos (Inpf (target,nor));
+		
+		theta= NormalizedVecAngle2(target, nor);
 		
 		/*	Make Bone matrix*/
 		VecRotToMat3(axis, theta, bMatrix);
@@ -934,6 +1064,54 @@ void where_is_armature (bArmature *arm)
 	}
 }
 
+/* if bone layer is protected, copy the data from from->pose */
+static void pose_proxy_synchronize(Object *ob, Object *from, int layer_protected)
+{
+	bPose *pose= ob->pose, *frompose= from->pose;
+	bPoseChannel *pchan, *pchanp, pchanw;
+	bConstraint *con;
+	char *str;
+	
+	if(frompose==NULL) return;
+	
+	/* exception, armature local layer should be proxied too */
+	if(pose->proxy_layer)
+		((bArmature *)ob->data)->layer= pose->proxy_layer;
+	
+	/* clear all transformation values from library */
+	rest_pose(frompose);
+	
+	pchan= pose->chanbase.first;
+	for(; pchan; pchan= pchan->next) {
+		if(pchan->bone->layer & layer_protected) {
+			pchanp= get_pose_channel(frompose, pchan->name);
+			
+			/* copy posechannel to temp, but restore important pointers */
+			pchanw= *pchanp;
+			pchanw.prev= pchan->prev;
+			pchanw.next= pchan->next;
+			pchanw.parent= pchan->parent;
+			pchanw.child= pchan->child;
+			pchanw.path= NULL;
+			
+			/* constraints, set target ob pointer to own object */
+			copy_constraints(&pchanw.constraints, &pchanp->constraints);
+
+			for(con= pchanw.constraints.first; con; con= con->next) {
+				if(from==get_constraint_target(con, &str))
+					set_constraint_target(con, ob, NULL);
+			}
+			
+			/* free stuff from current channel */
+			if(pchan->path) MEM_freeN(pchan->path);
+			free_constraints(&pchan->constraints);
+			
+			/* the final copy */
+			*pchan= pchanw;
+		}
+	}
+}
+
 static int rebuild_pose_bone(bPose *pose, Bone *bone, bPoseChannel *parchan, int counter)
 {
 	bPoseChannel *pchan = verify_pose_channel (pose, bone->name);   // verify checks and/or adds
@@ -953,7 +1131,7 @@ static int rebuild_pose_bone(bPose *pose, Bone *bone, bPoseChannel *parchan, int
 	return counter;
 }
 
-/* only after leave editmode, duplicating, but also for validating older files */
+/* only after leave editmode, duplicating, validating older files, library syncing */
 /* NOTE: pose->flag is set for it */
 void armature_rebuild_pose(Object *ob, bArmature *arm)
 {
@@ -987,7 +1165,11 @@ void armature_rebuild_pose(Object *ob, bArmature *arm)
 			BLI_freelinkN(&pose->chanbase, pchan);
 		}
 	}
-//	printf("rebuild pose %s, %d bones\n", ob->id.name, counter);
+	// printf("rebuild pose %s, %d bones\n", ob->id.name, counter);
+	
+	/* synchronize protected layers with proxy */
+	if(ob->proxy)
+		pose_proxy_synchronize(ob, ob->proxy, arm->layer_protected);
 	
 	update_pose_constraint_flags(ob->pose); // for IK detection for example
 	
@@ -1026,6 +1208,7 @@ static void initialize_posetree(struct Object *ob, bPoseChannel *pchan_tip)
 	if(data->flag & CONSTRAINT_IK_AUTO);
 	else {
 		if(con->flag & CONSTRAINT_DISABLE) return;	/* checked in editconstraint.c */
+		if(con->enforce == 0.0f) return;
 		if(data->tar==NULL) return;
 		if(data->tar->type==OB_ARMATURE && data->subtarget[0]==0) return;
 	}
@@ -1069,9 +1252,9 @@ static void initialize_posetree(struct Object *ob, bPoseChannel *pchan_tip)
 		/* make new tree */
 		tree= MEM_callocN(sizeof(PoseTree), "posetree");
 
-		tree->tolerance= data->tolerance;
 		tree->iterations= data->iterations;
 		tree->totchannel= segcount;
+		tree->stretch = (data->flag & CONSTRAINT_IK_STRETCH);
 		
 		tree->pchan= MEM_callocN(segcount*sizeof(void*), "ik tree pchan");
 		tree->parent= MEM_callocN(segcount*sizeof(int), "ik tree parent");
@@ -1085,8 +1268,8 @@ static void initialize_posetree(struct Object *ob, bPoseChannel *pchan_tip)
 		BLI_addtail(&pchan_root->iktree, tree);
 	}
 	else {
-		tree->tolerance= MIN2(tree->tolerance, data->tolerance);
 		tree->iterations= MAX2(data->iterations, tree->iterations);
+		tree->stretch= tree->stretch && !(data->flag & CONSTRAINT_IK_STRETCH);
 
 		/* skip common pose channels and add remaining*/
 		size= MIN2(segcount, tree->totchannel);
@@ -1167,7 +1350,7 @@ static void execute_posetree(Object *ob, PoseTree *tree)
 		if((pchan->ikflag & BONE_IK_NO_ZDOF) == 0)
 			flag |= IK_ZDOF;
 
-		if(pchan->ikstretch > 0.0) {
+		if(tree->stretch && (pchan->ikstretch > 0.0)) {
 			flag |= IK_TRANS_YDOF;
 			hasstretch = 1;
 		}
@@ -1232,7 +1415,7 @@ static void execute_posetree(Object *ob, PoseTree *tree)
 		IK_SetStiffness(seg, IK_Y, pchan->stiffness[1]);
 		IK_SetStiffness(seg, IK_Z, pchan->stiffness[2]);
 
-		if(pchan->ikstretch > 0.0) {
+		if(tree->stretch && (pchan->ikstretch > 0.0)) {
 			float ikstretch = pchan->ikstretch*pchan->ikstretch;
 			IK_SetStiffness(seg, IK_TRANS_Y, MIN2(1.0-ikstretch, 0.99));
 			IK_SetLimit(seg, IK_TRANS_Y, 0.001, 1e10);
@@ -1303,7 +1486,7 @@ static void execute_posetree(Object *ob, PoseTree *tree)
 	}
 
 	/* solve */
-	IK_Solve(solver, tree->tolerance, tree->iterations);
+	IK_Solve(solver, 0.0f, tree->iterations);
 	IK_FreeSolver(solver);
 
 	/* gather basis changes */
@@ -1321,7 +1504,7 @@ static void execute_posetree(Object *ob, PoseTree *tree)
 			pchan= tree->pchan[a];
 			parentstretch= (tree->parent[a] >= 0)? ikstretch[tree->parent[a]]: 1.0;
 
-			if(pchan->ikstretch > 0.0) {
+			if(tree->stretch && (pchan->ikstretch > 0.0)) {
 				float trans[3], length;
 
 				IK_GetTranslationChange(iktree[a], trans);
@@ -1426,16 +1609,17 @@ static void do_local_constraint(bPoseChannel *pchan, bConstraint *con)
 		case CONSTRAINT_TYPE_LOCLIKE:
 		{
 			bLocateLikeConstraint *data= con->data;
+			float fac= con->enforce;
 			
 			if(data->tar && data->subtarget[0]) {
 				bPoseChannel *pchant= get_pose_channel(data->tar->pose, data->subtarget);
 				if(pchant) {
 					if (data->flag & LOCLIKE_X)
-						pchan->loc[0]= pchant->loc[0];
+						pchan->loc[0]= FloatLerpf(pchant->loc[0], pchan->loc[0], fac);
 					if (data->flag & LOCLIKE_Y)
-						pchan->loc[1]= pchant->loc[1];
+						pchan->loc[1]= FloatLerpf(pchant->loc[1], pchan->loc[1], fac);
 					if (data->flag & LOCLIKE_Z)
-						pchan->loc[2]= pchant->loc[2];
+						pchan->loc[2]= FloatLerpf(pchant->loc[2], pchan->loc[2], fac);
 				}
 			}
 		}
@@ -1452,9 +1636,9 @@ static void do_local_constraint(bPoseChannel *pchan, bConstraint *con)
 						QuatToEul(pchan->quat, eul);
 						QuatToEul(pchant->quat, eult);
 						VECCOPY(euln, eul);
-						if(data->flag & ROTLIKE_X) euln[0]= fac*eult[0] + (1.0f-fac)*eul[0];
-						if(data->flag & ROTLIKE_Y) euln[1]= fac*eult[1] + (1.0f-fac)*eul[1];
-						if(data->flag & ROTLIKE_Z) euln[2]= fac*eult[2] + (1.0f-fac)*eul[2];
+						if(data->flag & ROTLIKE_X) euln[0]= FloatLerpf(eult[0], eul[0], fac); 
+						if(data->flag & ROTLIKE_Y) euln[1]= FloatLerpf(eult[1], eul[1], fac);
+						if(data->flag & ROTLIKE_Z) euln[2]= FloatLerpf(eult[2], eul[2], fac);
 						compatible_eul(eul, euln);
 						EulToQuat(euln, pchan->quat);
 					}
@@ -1464,6 +1648,165 @@ static void do_local_constraint(bPoseChannel *pchan, bConstraint *con)
 				}
 			}
 		}
+			break;
+		case CONSTRAINT_TYPE_SIZELIKE:
+		{
+			bSizeLikeConstraint *data= con->data;
+			float fac= con->enforce;
+			
+			if(data->tar && data->subtarget[0]) {
+				bPoseChannel *pchant= get_pose_channel(data->tar->pose, data->subtarget);
+				if(pchant) {
+					if (data->flag & SIZELIKE_X)
+						pchan->size[0]= FloatLerpf(pchant->size[0], pchan->size[0], fac);
+					if (data->flag & SIZELIKE_Y)
+						pchan->size[1]= FloatLerpf(pchant->size[1], pchan->size[1], fac);
+					if (data->flag & SIZELIKE_Z)
+						pchan->size[2]= FloatLerpf(pchant->size[2], pchan->size[2], fac);
+				}
+			}
+		}
+			break;
+		case CONSTRAINT_TYPE_LOCLIMIT:
+		{
+			bLocLimitConstraint *data= con->data;
+			float fac= con->enforce;
+			
+			if (data->flag & LIMIT_XMIN) {
+				if(pchan->loc[0] < data->xmin)
+					pchan->loc[0] = FloatLerpf(data->xmin, pchan->loc[0], fac);
+			}
+			if (data->flag & LIMIT_XMAX) {
+				if (pchan->loc[0] > data->xmax)
+					pchan->loc[0] = FloatLerpf(data->xmax, pchan->loc[0], fac);
+			}
+			if (data->flag & LIMIT_YMIN) {
+				if(pchan->loc[1] < data->ymin)
+					pchan->loc[1] = FloatLerpf(data->ymin, pchan->loc[1], fac);
+			}
+			if (data->flag & LIMIT_YMAX) {
+				if (pchan->loc[1] > data->ymax)
+					pchan->loc[1] = FloatLerpf(data->ymax, pchan->loc[1], fac);
+			}
+			if (data->flag & LIMIT_ZMIN) {
+				if(pchan->loc[2] < data->zmin)
+					pchan->loc[2] = FloatLerpf(data->zmin, pchan->loc[2], fac);
+			}
+			if (data->flag & LIMIT_ZMAX) {
+				if (pchan->loc[2] > data->zmax)
+					pchan->loc[2] = FloatLerpf(data->zmax, pchan->loc[2], fac);
+			}
+		}	
+			break;
+		case CONSTRAINT_TYPE_ROTLIMIT:
+		{
+			bRotLimitConstraint *data = con->data;
+			float eul[3];
+			float fac= con->enforce;
+			
+			QuatToEul(pchan->quat, eul);
+			
+			/* eulers: radians to degrees! */
+			eul[0] = (eul[0] / (2*M_PI) * 360);
+			eul[1] = (eul[1] / (2*M_PI) * 360);
+			eul[2] = (eul[2] / (2*M_PI) * 360);
+			
+			/* limiting of euler values... */
+			if (data->flag & LIMIT_XROT) {
+				if (eul[0] < data->xmin) 
+					eul[0] = FloatLerpf(data->xmin, eul[0], fac);
+					
+				if (eul[0] > data->xmax)
+					eul[0] = FloatLerpf(data->xmax, eul[0], fac);
+			}
+			if (data->flag & LIMIT_YROT) {
+				if (eul[1] < data->ymin)
+					eul[1] = FloatLerpf(data->ymin, eul[1], fac);
+					
+				if (eul[1] > data->ymax)
+					eul[1] = FloatLerpf(data->ymax, eul[1], fac);
+			}
+			if (data->flag & LIMIT_ZROT) {
+				if (eul[2] < data->zmin)
+					eul[2] = FloatLerpf(data->zmin, eul[2], fac);
+					
+				if (eul[2] > data->zmax)
+					eul[2] = FloatLerpf(data->zmax, eul[2], fac);
+			}
+				
+			/* eulers: degrees to radians ! */
+			eul[0] = (eul[0] / 360 * (2*M_PI)); 
+			eul[1] = (eul[1] / 360 * (2*M_PI));
+			eul[2] = (eul[2] / 360 * (2*M_PI));
+			
+			/* convert back */
+			EulToQuat(eul, pchan->quat);
+		}
+			break;
+		case CONSTRAINT_TYPE_SIZELIMIT:
+		{
+			bSizeLimitConstraint *data= con->data;
+			float fac= con->enforce;
+			
+			if (data->flag & LIMIT_XMIN) {
+				if(pchan->size[0] < data->xmin)
+					pchan->size[0] = FloatLerpf(data->xmin, pchan->size[0], fac);
+			}
+			if (data->flag & LIMIT_XMAX) {
+				if (pchan->size[0] > data->xmax)
+					pchan->size[0] = FloatLerpf(data->xmax, pchan->size[0], fac);
+			}
+			if (data->flag & LIMIT_YMIN) {
+				if(pchan->size[1] < data->ymin)
+					pchan->size[1] = FloatLerpf(data->ymin, pchan->size[1], fac);
+			}
+			if (data->flag & LIMIT_YMAX) {
+				if (pchan->size[1] > data->ymax)
+					pchan->size[1] = FloatLerpf(data->ymax, pchan->size[1], fac);
+			}
+			if (data->flag & LIMIT_ZMIN) {
+				if(pchan->size[2] < data->zmin)
+					pchan->size[2] = FloatLerpf(data->zmin, pchan->size[2], fac);
+			}
+			if (data->flag & LIMIT_ZMAX) {
+				if (pchan->size[2] > data->zmax)
+					pchan->size[2] = FloatLerpf(data->zmax, pchan->size[2], fac);
+			}
+		}
+			break;
+	}
+}
+
+static void do_strip_modifiers(Object *armob, Bone *bone, bPoseChannel *pchan)
+{
+	bActionModifier *amod;
+	bActionStrip *strip;
+	float scene_cfra= G.scene->r.cfra;
+
+	for (strip=armob->nlastrips.first; strip; strip=strip->next) {
+		if(scene_cfra>=strip->start && scene_cfra<=strip->end) {
+			
+			/* temporal solution to prevent 2 strips accumulating */
+			if(scene_cfra==strip->end && strip->next && strip->next->start==scene_cfra)
+				continue;
+			
+			for(amod= strip->modifiers.first; amod; amod= amod->next) {
+				if(amod->type==ACTSTRIP_MOD_DEFORM) {
+					/* validate first */
+					if(amod->ob && amod->ob->type==OB_CURVE && amod->channel[0]) {
+						
+						if( strcmp(pchan->name, amod->channel)==0 ) {
+							float mat4[4][4], mat3[3][3];
+							
+							curve_deform_vector(amod->ob, armob, bone->arm_mat[3], pchan->pose_mat[3], mat3, amod->no_rot_axis);
+							Mat4CpyMat4(mat4, pchan->pose_mat);
+							Mat4MulMat34(pchan->pose_mat, mat3, mat4);
+							
+						}
+					}
+				}
+			}
+		}
 	}
 }
 
@@ -1532,9 +1875,13 @@ static void where_is_pose_bone(Object *ob, bPoseChannel *pchan, float ctime)
 		else 
 			Mat4MulSerie(pchan->pose_mat, parchan->pose_mat, offs_bone, pchan->chan_mat, NULL, NULL, NULL, NULL, NULL);
 	}
-	else 
+	else {
 		Mat4MulMat4(pchan->pose_mat, pchan->chan_mat, bone->arm_mat);
+		/* only rootbones get the cyclic offset */
+		VecAddf(pchan->pose_mat[3], pchan->pose_mat[3], ob->pose->cyclic_offset);
+	}
 	
+	do_strip_modifiers(ob, bone, pchan);
 	
 	/* Do constraints */
 	if(pchan->constraints.first) {
