@@ -1,5 +1,5 @@
 /* 
- * $Id: colortools.c,v 1.7 2006/06/09 13:04:21 ton Exp $
+ * $Id: colortools.c,v 1.11 2007/01/17 14:12:48 ton Exp $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -43,6 +43,7 @@
 #include "BKE_curve.h"
 #include "BKE_global.h"
 #include "BKE_ipo.h"
+#include "BKE_image.h"
 #include "BKE_main.h"
 #include "BKE_utildefines.h"
 
@@ -53,8 +54,6 @@
 #include "IMB_imbuf.h"
 #include "IMB_imbuf_types.h"
 
-/* NOTE: uses threadsafe malloc/calloc/free, for use in compositor */
-
 /* ********************************* color curve ********************* */
 
 /* ***************** operations on full struct ************* */
@@ -75,6 +74,7 @@ CurveMapping *curvemapping_add(int tot, float minx, float miny, float maxx, floa
 	cumap->bwmul[0]= cumap->bwmul[1]= cumap->bwmul[2]= 1.0f;
 	
 	for(a=0; a<tot; a++) {
+		cumap->cm[a].flag= CUMA_EXTEND_EXTRAPOLATE;
 		cumap->cm[a].totpoint= 2;
 		cumap->cm[a].curve= MEM_callocN(2*sizeof(CurveMapPoint), "curve points");
 		
@@ -280,6 +280,37 @@ static void calchandle_curvemap(BezTriple *bezt, BezTriple *prev, BezTriple *nex
 	}
 }
 
+/* in X, out Y. 
+   X is presumed to be outside first or last */
+static float curvemap_calc_extend(CurveMap *cuma, float x, float *first, float *last)
+{
+	if(x <= first[0]) {
+		if((cuma->flag & CUMA_EXTEND_EXTRAPOLATE)==0) {
+			/* no extrapolate */
+			return first[1];
+		}
+		else {
+			if(cuma->ext_in[0]==0.0f)
+				return first[1] + cuma->ext_in[1]*10000.0f;
+			else
+				return first[1] + cuma->ext_in[1]*(x - first[0])/cuma->ext_in[0];
+		}
+	}
+	else if(x >= last[0]) {
+		if((cuma->flag & CUMA_EXTEND_EXTRAPOLATE)==0) {
+			/* no extrapolate */
+			return last[1];
+		}
+		else {
+			if(cuma->ext_out[0]==0.0f)
+				return last[1] - cuma->ext_out[1]*10000.0f;
+			else
+				return last[1] + cuma->ext_out[1]*(x - last[0])/cuma->ext_out[0];
+		}
+	}
+	return 0.0f;
+}
+
 /* only creates a table for a single channel in CurveMapping */
 static void curvemap_make_table(CurveMap *cuma, rctf *clipr)
 {
@@ -295,7 +326,7 @@ static void curvemap_make_table(CurveMap *cuma, rctf *clipr)
 	cuma->maxtable= clipr->xmax;
 	
 	/* hrmf... we now rely on blender ipo beziers, these are more advanced */
-	bezt= MEM_callocT(cuma->totpoint*sizeof(BezTriple), "beztarr");
+	bezt= MEM_callocN(cuma->totpoint*sizeof(BezTriple), "beztarr");
 	
 	for(a=0; a<cuma->totpoint; a++) {
 		cuma->mintable= MIN2(cuma->mintable, cmp[a].x);
@@ -335,6 +366,7 @@ static void curvemap_make_table(CurveMap *cuma, rctf *clipr)
 			if(nlen>FLT_EPSILON) {
 				VecMulf(vec, hlen/nlen);
 				VecAddf(bezt[0].vec[2], vec, bezt[0].vec[1]);
+				VecSubf(bezt[0].vec[0], bezt[0].vec[1], vec);
 			}
 		}
 		a= cuma->totpoint-1;
@@ -351,14 +383,15 @@ static void curvemap_make_table(CurveMap *cuma, rctf *clipr)
 			if(nlen>FLT_EPSILON) {
 				VecMulf(vec, hlen/nlen);
 				VecAddf(bezt[a].vec[0], vec, bezt[a].vec[1]);
+				VecSubf(bezt[a].vec[2], bezt[a].vec[1], vec);
 			}
 		}
 	}	
 	/* make the bezier curve */
 	if(cuma->table)
-		MEM_freeT(cuma->table);
+		MEM_freeN(cuma->table);
 	totpoint= (cuma->totpoint-1)*CM_RESOL;
-	fp= allpoints= MEM_callocT(totpoint*2*sizeof(float), "table");
+	fp= allpoints= MEM_callocN(totpoint*2*sizeof(float), "table");
 	
 	for(a=0; a<cuma->totpoint-1; a++, fp += 2*CM_RESOL) {
 		correct_bezpart(bezt[a].vec[1], bezt[a].vec[2], bezt[a+1].vec[0], bezt[a+1].vec[1]);
@@ -366,19 +399,32 @@ static void curvemap_make_table(CurveMap *cuma, rctf *clipr)
 		forward_diff_bezier(bezt[a].vec[1][1], bezt[a].vec[2][1], bezt[a+1].vec[0][1], bezt[a+1].vec[1][1], fp+1, CM_RESOL-1, 2);
 	}
 	
-	MEM_freeT(bezt);
+	/* store first and last handle for extrapolation, unit length */
+	cuma->ext_in[0]= bezt[0].vec[0][0] - bezt[0].vec[1][0];
+	cuma->ext_in[1]= bezt[0].vec[0][1] - bezt[0].vec[1][1];
+	range= sqrt(cuma->ext_in[0]*cuma->ext_in[0] + cuma->ext_in[1]*cuma->ext_in[1]);
+	cuma->ext_in[0]/= range;
+	cuma->ext_in[1]/= range;
 	
+	a= cuma->totpoint-1;
+	cuma->ext_out[0]= bezt[a].vec[1][0] - bezt[a].vec[2][0];
+	cuma->ext_out[1]= bezt[a].vec[1][1] - bezt[a].vec[2][1];
+	range= sqrt(cuma->ext_out[0]*cuma->ext_out[0] + cuma->ext_out[1]*cuma->ext_out[1]);
+	cuma->ext_out[0]/= range;
+	cuma->ext_out[1]/= range;
+	
+	/* cleanup */
+	MEM_freeN(bezt);
+
 	range= CM_TABLEDIV*(cuma->maxtable - cuma->mintable);
 	cuma->range= 1.0f/range;
 	
 	/* now make a table with CM_TABLE equal x distances */
 	fp= allpoints;
 	lastpoint= allpoints + 2*(totpoint-1);
-	cmp= MEM_callocT((CM_TABLE+1)*sizeof(CurveMapPoint), "dist table");
-	cmp[0].x= cuma->mintable;
-	cmp[0].y= allpoints[1];
+	cmp= MEM_callocN((CM_TABLE+1)*sizeof(CurveMapPoint), "dist table");
 	
-	for(a=1; a<CM_TABLE; a++) {
+	for(a=0; a<=CM_TABLE; a++) {
 		curf= cuma->mintable + range*(float)a;
 		cmp[a].x= curf;
 		
@@ -386,8 +432,8 @@ static void curvemap_make_table(CurveMap *cuma, rctf *clipr)
 		while(curf >= fp[0] && fp!=lastpoint) {
 			fp+=2;
 		}
-		if(curf >= fp[0] && fp==lastpoint)
-			cmp[a].y= fp[1];
+		if(fp==allpoints || (curf >= fp[0] && fp==lastpoint))
+			cmp[a].y= curvemap_calc_extend(cuma, curf, allpoints, lastpoint);
 		else {
 			float fac1= fp[0] - fp[-2];
 			float fac2= fp[0] - curf;
@@ -398,10 +444,8 @@ static void curvemap_make_table(CurveMap *cuma, rctf *clipr)
 			cmp[a].y= fac1*fp[-1] + (1.0f-fac1)*fp[1];
 		}
 	}
-	cmp[CM_TABLE].x= cuma->maxtable;
-	cmp[CM_TABLE].y= allpoints[2*totpoint-1];
 	
-	MEM_freeT(allpoints);
+	MEM_freeN(allpoints);
 	cuma->table= cmp;
 }
 
@@ -414,7 +458,7 @@ void curvemapping_premultiply(CurveMapping *cumap, int restore)
 	if(restore) {
 		if(cumap->flag & CUMA_PREMULLED) {
 			for(a=0; a<3; a++) {
-				MEM_freeT(cumap->cm[a].table);
+				MEM_freeN(cumap->cm[a].table);
 				cumap->cm[a].table= cumap->cm[a].premultable;
 				cumap->cm[a].premultable= NULL;
 			}
@@ -429,7 +473,7 @@ void curvemapping_premultiply(CurveMapping *cumap, int restore)
 				if(cumap->cm[a].table==NULL)
 					curvemap_make_table(cumap->cm+a, &cumap->clipr);
 				cumap->cm[a].premultable= cumap->cm[a].table;
-				cumap->cm[a].table= MEM_mallocT((CM_TABLE+1)*sizeof(CurveMapPoint), "premul table");
+				cumap->cm[a].table= MEM_mallocN((CM_TABLE+1)*sizeof(CurveMapPoint), "premul table");
 				memcpy(cumap->cm[a].table, cumap->cm[a].premultable, (CM_TABLE+1)*sizeof(CurveMapPoint));
 			}
 			
@@ -467,23 +511,32 @@ void curvemapping_changed(CurveMapping *cumap, int rem_doubles)
 	CurveMapPoint *cmp= cuma->curve;
 	rctf *clipr= &cumap->clipr;
 	float thresh= 0.01f*(clipr->xmax - clipr->xmin);
-	float dx, dy;
+	float dx= 0.0f, dy= 0.0f;
 	int a;
 	
 	/* clamp with clip */
 	if(cumap->flag & CUMA_DO_CLIP) {
 		for(a=0; a<cuma->totpoint; a++) {
-			if(cmp[a].x < clipr->xmin)
-				cmp[a].x= clipr->xmin;
-			else if(cmp[a].x > clipr->xmax)
-				cmp[a].x= clipr->xmax;
-			if(cmp[a].y < clipr->ymin)
-				cmp[a].y= clipr->ymin;
-			else if(cmp[a].y > clipr->ymax)
-				cmp[a].y= clipr->ymax;
+			if(cmp[a].flag & CUMA_SELECT) {
+				if(cmp[a].x < clipr->xmin)
+					dx= MIN2(dx, cmp[a].x - clipr->xmin);
+				else if(cmp[a].x > clipr->xmax)
+					dx= MAX2(dx, cmp[a].x - clipr->xmax);
+				if(cmp[a].y < clipr->ymin)
+					dy= MIN2(dy, cmp[a].y - clipr->ymin);
+				else if(cmp[a].y > clipr->ymax)
+					dy= MAX2(dy, cmp[a].y - clipr->ymax);
+			}
+		}
+		for(a=0; a<cuma->totpoint; a++) {
+			if(cmp[a].flag & CUMA_SELECT) {
+				cmp[a].x -= dx;
+				cmp[a].y -= dy;
+			}
 		}
 	}
 	
+	
 	qsort(cmp, cuma->totpoint, sizeof(CurveMapPoint), sort_curvepoints);
 	
 	/* remove doubles, threshold set on 1% of default range */
@@ -520,11 +573,16 @@ float curvemap_evaluateF(CurveMap *cuma, float value)
 	/* index in table */
 	fi= (value-cuma->mintable)*cuma->range;
 	i= (int)fi;
-	if(i<0) return cuma->table[0].y;
-	if(i>=CM_TABLE) return cuma->table[CM_TABLE].y;
 	
-	fi= fi-(float)i;
-	return (1.0f-fi)*cuma->table[i].y + (fi)*cuma->table[i+1].y; 
+	if(fi<0.0f || fi>cuma->range)
+		return curvemap_calc_extend(cuma, value, &cuma->table[0].x, &cuma->table[CM_TABLE].x);
+	else {
+		if(i<0) return cuma->table[0].y;
+		if(i>=CM_TABLE) return cuma->table[CM_TABLE].y;
+		
+		fi= fi-(float)i;
+		return (1.0f-fi)*cuma->table[i].y + (fi)*cuma->table[i+1].y; 
+	}
 }
 
 /* works with curve 'cur' */
@@ -575,30 +633,43 @@ void curvemapping_evaluate_premulRGBF(CurveMapping *cumap, float *vecout, const
 
 #define FTOCHAR(val) val<=0.0f?0: (val>=1.0f?255: (char)(255.0f*val))
 
-void curvemapping_do_image(CurveMapping *cumap, Image *ima)
+void curvemapping_do_ibuf(CurveMapping *cumap, ImBuf *ibuf)
 {
 	int pixel;
 	
-	if(ima==NULL || ima->ibuf==NULL)
+	if(ibuf==NULL)
 		return;
-	if(ima->ibuf->rect_float==NULL)
-		IMB_float_from_rect(ima->ibuf);
-	else if(ima->ibuf->rect==NULL)
-		imb_addrectImBuf(ima->ibuf);
+	if(ibuf->rect_float==NULL)
+		IMB_float_from_rect(ibuf);
+	else if(ibuf->rect==NULL)
+		imb_addrectImBuf(ibuf);
 	
 	curvemapping_premultiply(cumap, 0);
 	
-	if(ima->ibuf->rect_float && ima->ibuf->rect) {
-		float *pixf= ima->ibuf->rect_float;
+	if(ibuf->rect_float && ibuf->rect) {
+		float *pixf= ibuf->rect_float;
 		float col[3];
-		char *pixc= (char *)ima->ibuf->rect;
+		int stride= 4;
+		char *pixc= (char *)ibuf->rect;
 		
-		for(pixel= ima->ibuf->x*ima->ibuf->y; pixel>0; pixel--, pixf+=4, pixc+=4) {
-			curvemapping_evaluate_premulRGBF(cumap, col, pixf);
-			pixc[0]= FTOCHAR(col[0]);
-			pixc[1]= FTOCHAR(col[1]);
-			pixc[2]= FTOCHAR(col[2]);
-			pixc[3]= FTOCHAR(pixf[3]);
+		if(ibuf->channels)
+			stride= ibuf->channels;
+		
+		for(pixel= ibuf->x*ibuf->y; pixel>0; pixel--, pixf+=stride, pixc+=4) {
+			if(stride<3) {
+				col[0]= curvemap_evaluateF(cumap->cm, *pixf);
+				pixc[1]= pixc[2]= pixc[3]= pixc[0]= FTOCHAR(col[0]);
+			}
+			else {
+				curvemapping_evaluate_premulRGBF(cumap, col, pixf);
+				pixc[0]= FTOCHAR(col[0]);
+				pixc[1]= FTOCHAR(col[1]);
+				pixc[2]= FTOCHAR(col[2]);
+				if(stride>3)
+					pixc[3]= FTOCHAR(pixf[3]);
+				else
+					pixc[3]= 255;
+			}
 		}
 	}
 	
