@@ -1,5 +1,5 @@
 /**
- * $Id: constraint.c,v 1.46 2006/06/03 17:21:45 ton Exp $
+ * $Id: constraint.c,v 1.61 2007/01/14 23:42:06 theeth Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -205,7 +205,32 @@ void relink_constraints (struct ListBase *list)
 				ID_NEW(data->tar);
 			}
 				break;
+			case CONSTRAINT_TYPE_RIGIDBODYJOINT:
+			{
+				bRigidBodyJointConstraint *data;
+				data = con->data;
 				
+				ID_NEW(data->tar);
+			}
+				break;
+			case CONSTRAINT_TYPE_LOCLIMIT:
+			{
+				bLocLimitConstraint *data;
+				data = con->data;
+			}
+				break;
+			case CONSTRAINT_TYPE_ROTLIMIT:
+			{
+				bRotLimitConstraint *data;
+				data = con->data;
+			}
+				break;
+			case CONSTRAINT_TYPE_SIZELIMIT:
+			{
+				bSizeLimitConstraint *data;
+				data = con->data;
+			}
+				break;
 		}
 	}
 }
@@ -323,6 +348,13 @@ char constraint_has_target (bConstraint *con)
 				return 1;
 		}
 		break;
+	case CONSTRAINT_TYPE_RIGIDBODYJOINT:
+		{
+			bRigidBodyJointConstraint *data = con->data;
+			if (data->tar)
+				return 1;
+		}
+		break;
 	}
 	// Unknown types or CONSTRAINT_TYPE_NULL or no target
 	return 0;
@@ -359,6 +391,7 @@ Object *get_constraint_target(bConstraint *con, char **subtarget)
 	case CONSTRAINT_TYPE_SIZELIKE:
 		{
 			bSizeLikeConstraint *data = con->data;
+			*subtarget= data->subtarget;
 			return data->tar;
 		}
 		break;
@@ -404,6 +437,13 @@ Object *get_constraint_target(bConstraint *con, char **subtarget)
 			return (data->tar);
 		}
 		break;
+	case CONSTRAINT_TYPE_RIGIDBODYJOINT: 
+		{
+			bRigidBodyJointConstraint *data = con->data;
+			*subtarget= NULL;
+			return data->tar;
+		}
+		break;
 	default:
 		*subtarget= NULL;
 		break;
@@ -480,6 +520,12 @@ void set_constraint_target(bConstraint *con, Object *ob, char *subtarget)
 			if(subtarget) BLI_strncpy(data->subtarget, subtarget, 32);
 		}
 			break;
+		case CONSTRAINT_TYPE_RIGIDBODYJOINT: 
+		{
+			bRigidBodyJointConstraint *data = con->data;
+			data->tar= ob;
+		}
+			break;
 		case CONSTRAINT_TYPE_MINMAX:
 		{
 			bMinMaxConstraint *data = (bMinMaxConstraint*)con->data;
@@ -545,10 +591,10 @@ void *new_constraint_data (short type)
 			bKinematicConstraint *data;
 			data = MEM_callocN(sizeof(bKinematicConstraint), "kinematicConstraint");
 
-			data->tolerance = (float)0.001;
 			data->weight= (float)1.0;
+			data->orientweight= (float)1.0;
 			data->iterations = 500;
-			data->flag= CONSTRAINT_IK_TIP;
+			data->flag= CONSTRAINT_IK_TIP|CONSTRAINT_IK_STRETCH|CONSTRAINT_IK_POS;
 			
 			result = data;
 		}
@@ -580,8 +626,7 @@ void *new_constraint_data (short type)
 			data->minmaxflag = TRACK_Z;
 			data->offset = 0.0f;
 			data->cache[0] = data->cache[1] = data->cache[2] = 0.0f;
-			data->sticky = 0;
-			data->stuck = 0;
+			data->flag = 0;
 
 			result = data;
 
@@ -657,7 +702,89 @@ void *new_constraint_data (short type)
 			result = data;
 		}
 		break; 
-	default:
+	case CONSTRAINT_TYPE_LOCLIMIT:
+		{
+			bLocLimitConstraint *data;
+			data = MEM_callocN(sizeof(bLocLimitConstraint), "LocLimitConstraint");
+			
+			data->flag = 0;
+			data->flag2 = 0;
+			data->xmin = 0.0f;
+			data->xmax = 0.0f;
+			data->ymin = 0.0f;
+			data->ymax = 0.0f;
+			data->zmin = 0.0f;
+			data->zmax = 0.0f;
+			
+			result = data;
+		}
+		break;
+	case CONSTRAINT_TYPE_ROTLIMIT:
+		{
+			bRotLimitConstraint *data;
+			data = MEM_callocN(sizeof(bRotLimitConstraint), "RotLimitConstraint");
+			
+			data->flag = 0;
+			data->xmin = 0.0f;
+			data->xmax = 0.0f;
+			data->ymin = 0.0f;
+			data->ymax = 0.0f;
+			data->zmin = 0.0f;
+			data->zmax = 0.0f;
+			
+			result = data;
+		}
+		break;
+	case CONSTRAINT_TYPE_SIZELIMIT:
+		{
+			bSizeLimitConstraint *data;
+			data = MEM_callocN(sizeof(bSizeLimitConstraint), "SizeLimitConstraint");
+			
+			data->flag = 0;
+			data->xmin = 0.0f;
+			data->xmax = 0.0f;
+			data->ymin = 0.0f;
+			data->ymax = 0.0f;
+			data->zmin = 0.0f;
+			data->zmax = 0.0f;
+			
+			result = data;
+		}
+		break;
+    case CONSTRAINT_TYPE_RIGIDBODYJOINT:
+		{
+			bRigidBodyJointConstraint *data;
+			int i;
+			Base *base_iter;
+
+			data = MEM_callocN(sizeof(bRigidBodyJointConstraint), "RigidBodyToConstraint");
+			base_iter = G.scene->base.first;
+            while( base_iter && !data->tar ) {
+                if( ( ( base_iter->flag & SELECT ) &&
+//                    ( base_iter->lay & G.vd->lay ) ) &&
+                    ( base_iter != G.scene->basact ) )
+                    )
+                        data->tar=base_iter->object;
+                base_iter = base_iter->next;
+            }
+            data->type=1;
+            data->pivX=0.0;
+            data->pivY=0.0;
+            data->pivZ=0.0;
+            data->axX=0.0;
+            data->axY=0.0;
+            data->axZ=0.0;
+			for (i=0;i<6;i++)
+			{
+				data->minLimit[i]=0.0;
+				data->maxLimit[i]=0.0;
+			}
+            data->extraFz=0.0;
+			result = data;
+		}
+		break;
+  
+   	default:
 		result = NULL;
 		break;
 	}
@@ -759,7 +886,7 @@ void Mat4BlendMat4(float out[][4], float dst[][4], float src[][4], float srcweig
 	VECCOPY (out[3], floc);
 }
 
-static void constraint_target_to_mat4 (Object *ob, const char *substring, float mat[][4], float size[3], float ctime)
+static void constraint_target_to_mat4 (Object *ob, const char *substring, float mat[][4], float size[3])
 {
 
 	/*	Case OBJECT */
@@ -787,9 +914,96 @@ static void constraint_target_to_mat4 (Object *ob, const char *substring, float
 	}
 }
 
+
+/* stupid little cross product function, 0:x, 1:y, 2:z axes */
+static int basis_cross(int n, int m)
+{
+	if(n-m == 1) return 1;
+	if(n-m == -1) return -1;
+	if(n-m == 2) return -1;
+	if(n-m == -2) return 1;
+	else return 0;
+}
+
+static void vectomat(float *vec, float *target_up, short axis, short upflag, short flags, float m[][3])
+{
+	float n[3];
+	float u[3]; /* vector specifying the up axis */
+	float proj[3];
+	float right[3];
+	float neg = -1;
+	int right_index;
+
+	VecCopyf(n, vec);
+	if(Normalise(n) == 0.0) { 
+		n[0] = 0.0;
+		n[1] = 0.0;
+		n[2] = 1.0;
+	}
+	if(axis > 2) axis -= 3;
+	else VecMulf(n,-1);
+
+	/* n specifies the transformation of the track axis */
+
+	if(flags & TARGET_Z_UP) { 
+		/* target Z axis is the global up axis */
+		u[0] = target_up[0];
+		u[1] = target_up[1];
+		u[2] = target_up[2];
+	}
+	else { 
+		/* world Z axis is the global up axis */
+		u[0] = 0;
+		u[1] = 0;
+		u[2] = 1;
+	}
+
+	/* project the up vector onto the plane specified by n */
+	Projf(proj, u, n); /* first u onto n... */
+	VecSubf(proj, u, proj); /* then onto the plane */
+	/* proj specifies the transformation of the up axis */
+
+	if(Normalise(proj) == 0.0) { /* degenerate projection */
+		proj[0] = 0.0;
+		proj[1] = 1.0;
+		proj[2] = 0.0;
+	}
+
+	/* normalised cross product of n and proj specifies transformation of the right axis */
+	Crossf(right, proj, n);
+	Normalise(right);
+
+	if(axis != upflag) {
+		right_index = 3 - axis - upflag;
+		neg = (float) basis_cross(axis, upflag);
+
+		/* account for up direction, track direction */
+		m[right_index][0] = neg * right[0];
+		m[right_index][1] = neg * right[1];
+		m[right_index][2] = neg * right[2];
+
+		m[upflag][0] = proj[0];
+		m[upflag][1] = proj[1];
+		m[upflag][2] = proj[2];
+
+		m[axis][0] = n[0];
+		m[axis][1] = n[1];
+		m[axis][2] = n[2];
+	}
+
+	else {
+		m[0][0]= m[1][1]= m[2][2]= 1.0;
+		m[0][1]= m[0][2]= m[0][3]= 0.0;
+		m[1][0]= m[1][2]= m[1][3]= 0.0;
+		m[2][0]= m[2][1]= m[2][3]= 0.0;
+	}
+}
+
+
 /* called during solve_constraints */
 /* also for make_parent, to find correct inverse of "follow path" */
 /* warning, ownerdata is void... is not Bone anymore, but PoseChannel or Object */
+/* ctime is global time, uncorrected for local bsystem_time */
 short get_constraint_target_matrix (bConstraint *con, short ownertype, void* ownerdata, float mat[][4], float size[3], float ctime)
 {
 	short valid=0;
@@ -853,7 +1067,7 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 				else {
 					float ans[4][4];
 					
-					constraint_target_to_mat4(data->tar, data->subtarget, ans, size, ctime);
+					constraint_target_to_mat4(data->tar, data->subtarget, ans, size);
 					/* extract rotation, is in global world coordinates */
 					Mat3CpyMat4(tempmat3, ans);
 				}
@@ -895,7 +1109,7 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 			bLocateLikeConstraint *data = (bLocateLikeConstraint*)con->data;
 
 			if (data->tar){
-				constraint_target_to_mat4(data->tar, data->subtarget, mat, size, ctime);
+				constraint_target_to_mat4(data->tar, data->subtarget, mat, size);
 				valid=1;
 			}
 			else
@@ -907,7 +1121,7 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 			bMinMaxConstraint *data = (bMinMaxConstraint*)con->data;
 
 			if (data->tar){
-				constraint_target_to_mat4(data->tar, data->subtarget, mat, size, ctime);
+				constraint_target_to_mat4(data->tar, data->subtarget, mat, size);
 				valid=1;
 			}
 			else
@@ -920,7 +1134,7 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 			data = (bRotateLikeConstraint*)con->data;
 
 			if (data->tar){
-				constraint_target_to_mat4(data->tar, data->subtarget, mat, size, ctime);
+				constraint_target_to_mat4(data->tar, data->subtarget, mat, size);
 				valid=1;
 			}
 			else
@@ -933,9 +1147,7 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 			data = (bSizeLikeConstraint*)con->data;
 
 			if (data->tar){
-				/*	Update the location of the target object */
-				where_is_object_time (data->tar, ctime);	
-				constraint_target_to_mat4(data->tar, data->subtarget, mat, size, ctime);
+				constraint_target_to_mat4(data->tar, data->subtarget, mat, size);
 				valid=1;
 			}
 			else
@@ -948,7 +1160,7 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 			data = (bTrackToConstraint*)con->data;
 
 			if (data->tar){
-				constraint_target_to_mat4(data->tar, data->subtarget, mat, size, ctime);
+				constraint_target_to_mat4(data->tar, data->subtarget, mat, size);
 				valid=1;
 			}
 			else
@@ -961,7 +1173,7 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 			data = (bKinematicConstraint*)con->data;
 
 			if (data->tar){
-				constraint_target_to_mat4(data->tar, data->subtarget, mat, size, ctime);
+				constraint_target_to_mat4(data->tar, data->subtarget, mat, size);
 				valid=1;
 			}
 			else if (data->flag & CONSTRAINT_IK_AUTO) {
@@ -988,7 +1200,7 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 			data = (bLockTrackConstraint*)con->data;
 
 			if (data->tar){
-				constraint_target_to_mat4(data->tar, data->subtarget, mat, size, ctime);
+				constraint_target_to_mat4(data->tar, data->subtarget, mat, size);
 				valid=1;
 			}
 			else
@@ -1016,6 +1228,7 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 				if(cu->path==NULL || cu->path->data==NULL) /* only happens on reload file, but violates depsgraph still... fix! */
 					makeDispListCurveTypes(data->tar, 0);
 				if(cu->path && cu->path->data) {
+					
 					curvetime= bsystem_time(data->tar, data->tar->parent, (float)ctime, 0.0) - data->offset;
 
 					if(calc_ipo_spec(cu->ipo, CU_SPEED, &curvetime)==0) {
@@ -1056,7 +1269,7 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 			data = (bStretchToConstraint*)con->data;
 
 			if (data->tar){
-				constraint_target_to_mat4(data->tar, data->subtarget, mat, size, ctime);
+				constraint_target_to_mat4(data->tar, data->subtarget, mat, size);
 				valid = 1;
 			}
 			else
@@ -1083,7 +1296,7 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 	
 	if (!constraint || !ob)
 		return;
-	
+
 	Mat4One (M_identity);
 	
 	switch (constraint->type){
@@ -1115,43 +1328,26 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 	case CONSTRAINT_TYPE_ROTLIKE:
 		{
 			bRotateLikeConstraint *data;
-			float	tmat[3][3];
+			float	loc[3];
+			float	eul[3], obeul[3];
 			float	size[3];
 			
 			data = constraint->data;
 			
-			/* old files stuff only... version patch is too much code! */
-			if(data->flag==0) data->flag = ROTLIKE_X|ROTLIKE_Y|ROTLIKE_Z;
-
+			VECCOPY(loc, ob->obmat[3]);
 			Mat4ToSize(ob->obmat, size);
 			
-			Mat3CpyMat4 (tmat, targetmat);
-			Mat3Ortho(tmat);
+			Mat4ToEul(targetmat, eul);
+			Mat4ToEul(ob->obmat, obeul);
 			
 			if(data->flag != (ROTLIKE_X|ROTLIKE_Y|ROTLIKE_Z)) {
-				float obeul[3], eul[3], obmat[3][3];
-				
-				Mat3ToEul(tmat, eul);
-				Mat3CpyMat4(obmat, ob->obmat);
-				Mat3ToEul(obmat, obeul);
 				if(!(data->flag & ROTLIKE_X)) eul[0]= obeul[0];
 				if(!(data->flag & ROTLIKE_Y)) eul[1]= obeul[1];
 				if(!(data->flag & ROTLIKE_Z)) eul[2]= obeul[2];
 				compatible_eul(eul, obeul);
-				EulToMat3(eul, tmat);
 			}
 
-			ob->obmat[0][0] = tmat[0][0]*size[0];
-			ob->obmat[0][1] = tmat[0][1]*size[1];
-			ob->obmat[0][2] = tmat[0][2]*size[2];
-
-			ob->obmat[1][0] = tmat[1][0]*size[0];
-			ob->obmat[1][1] = tmat[1][1]*size[1];
-			ob->obmat[1][2] = tmat[1][2]*size[2];
-
-			ob->obmat[2][0] = tmat[2][0]*size[0];
-			ob->obmat[2][1] = tmat[2][1]*size[1];
-			ob->obmat[2][2] = tmat[2][2]*size[2];
+			LocEulSizeToMat4(ob->obmat, loc, eul, size);
 		}
 		break;
  	case CONSTRAINT_TYPE_SIZELIKE:
@@ -1181,38 +1377,50 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 			float val1, val2;
 			int index;
 			bMinMaxConstraint *data;
+			float obmat[4][4],imat[4][4],tarmat[4][4],tmat[4][4];
 
 			data = constraint->data;
+			
+			Mat4CpyMat4(obmat,ob->obmat);
+			Mat4CpyMat4(tarmat,targetmat);
+			
+			if (data->flag&MINMAX_USEROT) {
+			/* take rotation of target into account by doing the transaction in target's localspace */
+				Mat4Invert(imat,tarmat);
+				Mat4MulMat4(tmat,obmat,imat);
+				Mat4CpyMat4(obmat,tmat);
+				Mat4One(tarmat);
+			}
 
 			switch (data->minmaxflag){
 			case TRACK_Z:
-				val1 = targetmat[3][2];
-				val2 = ob->obmat[3][2]-data->offset;
+				val1 = tarmat[3][2];
+				val2 = obmat[3][2]-data->offset;
 				index = 2;
 				break;
 			case TRACK_Y:
-				val1 = targetmat[3][1];
-				val2 = ob->obmat[3][1]-data->offset;
+				val1 = tarmat[3][1];
+				val2 = obmat[3][1]-data->offset;
 				index = 1;
 				break;
 			case TRACK_X:
-				val1 = targetmat[3][0];
-				val2 = ob->obmat[3][0]-data->offset;
+				val1 = tarmat[3][0];
+				val2 = obmat[3][0]-data->offset;
 				index = 0;
 				break;
 			case TRACK_nZ:
-				val2 = targetmat[3][2];
-				val1 = ob->obmat[3][2]-data->offset;
+				val2 = tarmat[3][2];
+				val1 = obmat[3][2]-data->offset;
 				index = 2;
 				break;
 			case TRACK_nY:
-				val2 = targetmat[3][1];
-				val1 = ob->obmat[3][1]-data->offset;
+				val2 = tarmat[3][1];
+				val1 = obmat[3][1]-data->offset;
 				index = 1;
 				break;
 			case TRACK_nX:
-				val2 = targetmat[3][0];
-				val1 = ob->obmat[3][0]-data->offset;
+				val2 = tarmat[3][0];
+				val1 = obmat[3][0]-data->offset;
 				index = 0;
 				break;
 			default:
@@ -1220,25 +1428,33 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 			}
 			
 			if (val1 > val2) {
-				ob->obmat[3][index] = targetmat[3][index] + data->offset;
-				if (data->sticky==1) {
-					if (data->stuck==1) {
-						VECCOPY(ob->obmat[3], data->cache);
+				obmat[3][index] = tarmat[3][index] + data->offset;
+				if (data->flag&MINMAX_STICKY) {
+					if (data->flag&MINMAX_STUCK) {
+						VECCOPY(obmat[3], data->cache);
 					} else {
-						VECCOPY(data->cache, ob->obmat[3]);
-						data->stuck = 1;
+						VECCOPY(data->cache, obmat[3]);
+						data->flag|=MINMAX_STUCK;
 					}
 				}
+				if (data->flag&MINMAX_USEROT) {
+					/* get out of localspace */
+					Mat4MulMat4(tmat,obmat,targetmat);
+					Mat4CpyMat4(ob->obmat,tmat);
+				} else {			
+					VECCOPY(ob->obmat[3],obmat[3]);
+				}
+
 			} else {
-				data->stuck=0;
+				data->flag&=~MINMAX_STUCK;
 			}
+			
 		}
 		break;
 	case CONSTRAINT_TYPE_TRACKTO:
 		{
 			bTrackToConstraint *data;
 			float size[3];
-			float *quat;
 			float vec[3];
 			float totmat[3][3];
 			float tmat[4][4];
@@ -1265,8 +1481,9 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 	
 			
 				VecSubf(vec, ob->obmat[3], targetmat[3]);
-				quat= vectoquat(vec, (short)data->reserved1, (short)data->reserved2);
-				QuatToMat3(quat, totmat);
+				vectomat(vec, targetmat[2], 
+						(short)data->reserved1, (short)data->reserved2, 
+						data->flags, totmat);
 
 				Mat4CpyMat4(tmat, ob->obmat);
 				
@@ -1291,7 +1508,6 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 
 
 			data=(bLockTrackConstraint*)constraint->data;			
-			
 
 			if (data->tar){
 	
@@ -1603,14 +1819,28 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 		{
 			bFollowPathConstraint *data;
 			float obmat[4][4];
+			float size[3], obsize[3];
 
 			data=(bFollowPathConstraint*)constraint->data;			
 
 			if (data->tar) {
-				// weird, this is needed? doesnt work for workob (ton)
-				object_to_mat4(ob, obmat);
-
-				Mat4MulSerie(ob->obmat, targetmat, obmat, NULL, NULL, NULL, NULL, NULL, NULL);
+				/* get Object local transform (loc/rot/size) to determine transformation from path */
+ 				object_to_mat4(ob, obmat);
+				
+				/* get scaling of object before applying constraint */
+				Mat4ToSize(ob->obmat, size);
+ 
+				/* apply targetmat - containing location on path, and rotation */
+ 				Mat4MulSerie(ob->obmat, targetmat, obmat, NULL, NULL, NULL, NULL, NULL, NULL);
+				
+				/* un-apply scaling caused by path */
+				Mat4ToSize(ob->obmat, obsize);
+				if (obsize[0] != 0)
+	 				VecMulf(ob->obmat[0], size[0] / obsize[0]);
+	 			if (obsize[1] != 0)
+	 				VecMulf(ob->obmat[1], size[1] / obsize[1]);
+	 			if (obsize[2] != 0)
+	 				VecMulf(ob->obmat[2], size[2] / obsize[2]);
 			}
 		}
 		break;
@@ -1738,7 +1968,216 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
             }
         }
         break;
-	
+	case CONSTRAINT_TYPE_LOCLIMIT:
+		{
+			bLocLimitConstraint *data;
+
+			data = constraint->data;
+			
+			/* limit location relative to origin or parent   */
+			if (data->flag2 & LIMIT_NOPARENT) {
+				/* limiting relative to parent */
+				float parmat[4][4]; /* matrix of parent */
+				float objLoc[3], parLoc[3]; /* location of object, and location of parent */
+				float relLoc[3]; /* objLoc  - parLoc*/
+				
+				/* get matrix of parent */
+				Mat4CpyMat4(parmat, ob->parent->obmat);
+				
+				/* get locations as vectors */
+				objLoc[0] = ob->obmat[3][0];
+				objLoc[1] = ob->obmat[3][1];
+				objLoc[2] = ob->obmat[3][2];
+				
+				parLoc[0] = parmat[3][0];
+				parLoc[1] = parmat[3][1];
+				parLoc[2] = parmat[3][2];
+				
+				/* get relative location of obj from parent */
+				VecSubf(relLoc, objLoc, parLoc);
+				
+				/* limiting location */
+				if (data->flag & LIMIT_XMIN) {
+					if(relLoc[0] < data->xmin) 
+						ob->obmat[3][0] = (parLoc[0]+data->xmin);
+				}
+				if (data->flag & LIMIT_XMAX) {
+					if (relLoc[0] > data->xmax) 
+						ob->obmat[3][0] = (parLoc[0]+data->xmax);
+				}
+				if (data->flag & LIMIT_YMIN) {
+					if(relLoc[1] < data->ymin) 
+						ob->obmat[3][1] = (parLoc[1]+data->ymin);
+				}
+				if (data->flag & LIMIT_YMAX) {
+					if (relLoc[1] > data->ymax) 
+						ob->obmat[3][1] = (parLoc[1]+data->ymax);
+				}
+				if (data->flag & LIMIT_ZMIN) {
+					if(relLoc[2] < data->zmin) 
+						ob->obmat[3][2] = (parLoc[2]+data->zmin);
+				}
+				if (data->flag & LIMIT_ZMAX) {
+					if (relLoc[2] > data->zmax) 
+						ob->obmat[3][2] = (parLoc[2]+data->zmax);
+				}
+			} else {
+				/* limiting relative to origin */
+				if (data->flag & LIMIT_XMIN) {
+					if(ob->obmat[3][0] < data->xmin)
+						ob->obmat[3][0] = data->xmin;
+				}
+				if (data->flag & LIMIT_XMAX) {
+					if (ob->obmat[3][0] > data->xmax)
+						ob->obmat[3][0] = data->xmax;
+				}
+				if (data->flag & LIMIT_YMIN) {
+					if(ob->obmat[3][1] < data->ymin)
+						ob->obmat[3][1] = data->ymin;
+				}
+				if (data->flag & LIMIT_YMAX) {
+					if (ob->obmat[3][1] > data->ymax)
+						ob->obmat[3][1] = data->ymax;
+				}
+				if (data->flag & LIMIT_ZMIN) {
+					if(ob->obmat[3][2] < data->zmin) 
+						ob->obmat[3][2] = data->zmin;
+				}
+				if (data->flag & LIMIT_ZMAX) {
+					if (ob->obmat[3][2] > data->zmax)
+						ob->obmat[3][2] = data->zmax;
+				}
+			}
+		}
+		break;
+	case CONSTRAINT_TYPE_ROTLIMIT:
+		{
+			bRotLimitConstraint *data;
+			float loc[3];
+			float eul[3];
+			float size[3];
+			
+			data = constraint->data;
+			
+			VECCOPY(loc, ob->obmat[3]);
+			Mat4ToSize(ob->obmat, size);
+			
+			Mat4ToEul(ob->obmat, eul);
+			
+			/* eulers: radians to degrees! */
+			eul[0] = (eul[0] / M_PI * 180);
+			eul[1] = (eul[1] / M_PI * 180);
+			eul[2] = (eul[2] / M_PI * 180);
+			
+			/* limiting of euler values... */
+			if (data->flag & LIMIT_XROT) {
+				if (eul[0] < data->xmin) 
+					eul[0] = data->xmin;
+					
+				if (eul[0] > data->xmax)
+					eul[0] = data->xmax;
+			}
+			if (data->flag & LIMIT_YROT) {
+				if (eul[1] < data->ymin)
+					eul[1] = data->ymin;
+					
+				if (eul[1] > data->ymax)
+					eul[1] = data->ymax;
+			}
+			if (data->flag & LIMIT_ZROT) {
+				if (eul[2] < data->zmin)
+					eul[2] = data->zmin;
+					
+				if (eul[2] > data->zmax)
+					eul[2] = data->zmax;
+			}
+				
+			/* eulers: degrees to radians ! */
+			eul[0] = (eul[0] / 180 * M_PI); 
+			eul[1] = (eul[1] / 180 * M_PI);
+			eul[2] = (eul[2] / 180 * M_PI);
+			
+			LocEulSizeToMat4(ob->obmat, loc, eul, size);
+		}
+		break;
+	case CONSTRAINT_TYPE_SIZELIMIT:
+		{
+			bSizeLimitConstraint *data;
+			float obsize[3], size[3];
+			int clearNegScale=0;
+			
+			data = constraint->data;
+			
+			Mat4ToSize(ob->obmat, size);
+			Mat4ToSize(ob->obmat, obsize);
+			
+			if (data->flag & LIMIT_XMIN) {
+				if (ob->transflag & OB_NEG_SCALE) {
+					size[0] *= -1;
+					
+					if (size[0] < data->xmin) { 
+						size[0] = data->xmin;
+						clearNegScale += 1;
+					}
+				} else {
+					if (size[0] < data->xmin) 
+						size[0] = data->xmin;	
+				}	
+			}
+			if (data->flag & LIMIT_XMAX) {
+				if (size[0] > data->xmax) 
+					size[0] = data->xmax;
+			}
+			if (data->flag & LIMIT_YMIN) {
+				if (ob->transflag & OB_NEG_SCALE) {
+					size[1] *= -1;
+					
+					if (size[1] < data->ymin) { 
+						size[1] = data->ymin;
+						clearNegScale += 1;
+					}
+				} else {
+					if (size[1] < data->ymin) 
+						size[1] = data->ymin;	
+				}	
+			}
+			if (data->flag & LIMIT_YMAX) {
+				if (size[1] > data->ymax) 
+					size[1] = data->ymax;
+			}
+			if (data->flag & LIMIT_ZMIN) {
+				if (ob->transflag & OB_NEG_SCALE) {
+					size[2] *= -1;
+					
+					if (size[2] < data->zmin) { 
+						size[2] = data->zmin;
+						clearNegScale += 1;
+					}
+				} else {
+					if (size[2] < data->zmin) 
+						size[2] = data->zmin;	
+				}	
+			}
+			if (data->flag & LIMIT_ZMAX) {
+				if (size[2] > data->zmax) 
+					size[2] = data->zmax;
+			}
+			
+			if (clearNegScale != 0) {
+				ob->transflag &= ~OB_NEG_SCALE;  /* is this how we remove that flag? */	
+			}
+			
+			VecMulf(ob->obmat[0], size[0]/obsize[0]);
+			VecMulf(ob->obmat[1], size[1]/obsize[1]);
+			VecMulf(ob->obmat[2], size[2]/obsize[2]);
+		}
+		break;
+	case CONSTRAINT_TYPE_RIGIDBODYJOINT:
+        {
+
+
+        }
+        break;		
 	default:
 		printf ("Error: Unknown constraint type\n");
 		break;
