@@ -2,7 +2,7 @@
 /*  curve.c 
  * 
  *  
- * $Id: curve.c,v 1.34 2006/07/06 14:20:44 ton Exp $
+ * $Id: curve.c,v 1.41 2007/01/22 23:11:18 broken Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -126,7 +126,7 @@ Curve *add_curve(int type)
 	cu->size[0]= cu->size[1]= cu->size[2]= 1.0;
 	cu->flag= CU_FRONT+CU_BACK;
 	cu->pathlen= 100;
-	cu->resolu= cu->resolv= 6;
+	cu->resolu= cu->resolv= 12;
 	cu->width= 1.0;
 	cu->wordspace = 1.0;
 	cu->spacing= cu->linedist= 1.0;
@@ -262,7 +262,7 @@ void tex_space_curve(Curve *cu)
 	float *data, min[3], max[3], loc[3], size[3];
 	int tot, doit= 0;
 	
-	if(cu->bb==0) cu->bb= MEM_callocN(sizeof(BoundBox), "boundbox");
+	if(cu->bb==NULL) cu->bb= MEM_callocN(sizeof(BoundBox), "boundbox");
 	bb= cu->bb;
 	
 	INIT_MINMAX(min, max);
@@ -1097,7 +1097,7 @@ void makebevelcurve(Object *ob, ListBase *disp)
 {
 	DispList *dl, *dlnew;
 	Curve *bevcu, *cu;
-	float *fp, facx, facy, hoek, dhoek;
+	float *fp, facx, facy, angle, dangle;
 	int nr, a;
 
 	cu= ob->data;
@@ -1174,14 +1174,14 @@ void makebevelcurve(Object *ob, ListBase *disp)
 
 		/* a circle */
 		fp= dl->verts;
-		dhoek= (2.0f*M_PI/(nr));
-		hoek= -(nr-1)*dhoek;
+		dangle= (2.0f*M_PI/(nr));
+		angle= -(nr-1)*dangle;
 		
 		for(a=0; a<nr; a++) {
 			fp[0]= 0.0;
-			fp[1]= (float)(cos(hoek)*(cu->ext2));
-			fp[2]= (float)(sin(hoek)*(cu->ext2)) - cu->ext1;
-			hoek+= dhoek;
+			fp[1]= (float)(cos(angle)*(cu->ext2));
+			fp[2]= (float)(sin(angle)*(cu->ext2)) - cu->ext1;
+			angle+= dangle;
 			fp+= 3;
 		}
 	}
@@ -1204,14 +1204,14 @@ void makebevelcurve(Object *ob, ListBase *disp)
 
 		/* half a circle */
 		fp= dl->verts;
-		dhoek= (0.5*M_PI/(dnr-1));
-		hoek= -(nr-1)*dhoek;
+		dangle= (0.5*M_PI/(dnr-1));
+		angle= -(nr-1)*dangle;
 		
 		for(a=0; a<nr; a++) {
 			fp[0]= 0.0;
-			fp[1]= (float)(cos(hoek)*(cu->ext2));
-			fp[2]= (float)(sin(hoek)*(cu->ext2)) - cu->ext1;
-			hoek+= dhoek;
+			fp[1]= (float)(cos(angle)*(cu->ext2));
+			fp[2]= (float)(sin(angle)*(cu->ext2)) - cu->ext1;
+			angle+= dangle;
 			fp+= 3;
 		}
 		
@@ -1260,14 +1260,14 @@ void makebevelcurve(Object *ob, ListBase *disp)
 		
 		/* half a circle */
 		fp= dl->verts;
-		hoek= 0.0;
-		dhoek= (0.5*M_PI/(dnr-1));
+		angle= 0.0;
+		dangle= (0.5*M_PI/(dnr-1));
 		
 		for(a=0; a<nr; a++) {
 			fp[0]= 0.0;
-			fp[1]= (float)(cos(hoek)*(cu->ext2));
-			fp[2]= (float)(sin(hoek)*(cu->ext2)) + cu->ext1;
-			hoek+= dhoek;
+			fp[1]= (float)(cos(angle)*(cu->ext2));
+			fp[2]= (float)(sin(angle)*(cu->ext2)) + cu->ext1;
+			angle+= dangle;
 			fp+= 3;
 		}
 	}
@@ -1441,7 +1441,7 @@ static void alfa_bezpart(BezTriple *prevbezt, BezTriple *bezt, Nurb *nu, float *
 	
 	for(a=0; a<resolu; a++, fac+= dfac) {
 		
-		set_four_ipo(fac, t, KEY_BSPLINE);
+		set_four_ipo(fac, t, KEY_LINEAR);
 		
 		data_a[a]= t[0]*pprev->alfa + t[1]*prevbezt->alfa + t[2]*bezt->alfa + t[3]*next->alfa;
 	}
@@ -1471,7 +1471,7 @@ void makeBevelList(Object *ob)
 	/* STEP 1: MAKE POLYS  */
 
 	BLI_freelistN(&(cu->bev));
-	if(ob==G.obedit) nu= editNurb.first;
+	if(ob==G.obedit && ob->type!=OB_FONT) nu= editNurb.first;
 	else nu= cu->nurb.first;
 	
 	while(nu) {
@@ -1593,6 +1593,7 @@ void makeBevelList(Object *ob)
 					bevp->x= prevbezt->vec[1][0];
 					bevp->y= prevbezt->vec[1][1];
 					bevp->z= prevbezt->vec[1][2];
+					bevp->alfa= prevbezt->alfa;
 					bl->nr++;
 				}
 	
@@ -1863,6 +1864,134 @@ void makeBevelList(Object *ob)
 	}
 }
 
+/* calculates a bevel width (radius) for a particular subdivided curve part,
+ * based on the radius value of the surrounding CVs */
+float calc_curve_subdiv_radius(Curve *cu, Nurb *nu, int cursubdiv)
+{
+	BezTriple *bezt, *beztfirst, *beztlast, *beztnext, *beztprev;
+	BPoint *bp, *bpfirst, *bplast;
+	int resolu;
+	float prevrad=0.0, nextrad=0.0, rad=0.0, ratio=0.0;
+	int vectseg=0, subdivs=0;
+	
+	if((nu==NULL) || (nu->pntsu<=1)) return 1.0; 
+	bezt= nu->bezt;
+	bp = nu->bp;
+	
+	if(G.rendering && cu->resolu_ren!=0) resolu= cu->resolu_ren;
+	else resolu= nu->resolu;
+	
+	if(((nu->type & 7)==CU_BEZIER) && (bezt != NULL)) {
+		beztfirst = nu->bezt;
+		beztlast = nu->bezt + (nu->pntsu - 1);
+		
+		/* loop through the CVs to end up with a pointer to the CV before the subdiv in question, and a ratio
+		 * of how far that subdiv is between this CV and the next */
+		while(bezt<=beztlast) {
+			beztnext = bezt+1;
+			beztprev = bezt-1;
+			vectseg=0;
+			
+			if (subdivs==cursubdiv) {
+				ratio= 0.0;
+				break;
+			}
+
+			/* check to see if we're looking at a vector segment (no subdivisions) */
+			if (nu->flagu & CU_CYCLIC) {
+				if (bezt == beztfirst) {
+					if ((beztlast->h2==HD_VECT) && (bezt->h1==HD_VECT)) vectseg = 1;
+				} else {
+					if ((beztprev->h2==HD_VECT) && (bezt->h1==HD_VECT)) vectseg = 1;
+				}
+			} else if ((bezt->h2==HD_VECT) && (beztnext->h1==HD_VECT)) vectseg = 1;
+			
+			
+			if (vectseg==0) {
+				/* if it's NOT a vector segment, check to see if the subdiv falls within the segment */
+				subdivs += resolu;
+				
+				if (cursubdiv < subdivs) {
+					ratio = 1.0 - ((subdivs - cursubdiv)/(float)resolu);
+					break;
+				}
+			} else {
+				/* must be a vector segment.. loop again! */
+				subdivs += 1;
+			} 
+			
+			bezt++;
+		}
+		
+		/* Now we have a nice bezt pointer to the CV that we want. But cyclic messes it up, so must correct for that..
+		 * (cyclic goes last-> first -> first+1 -> first+2 -> ...) */
+		if (nu->flagu & CU_CYCLIC) {
+			if (bezt == beztfirst) bezt = beztlast;
+			else bezt--;
+		}
+		 
+		/* find the radii at the bounding CVs and interpolate between them based on ratio */
+		rad = prevrad = bezt->radius;
+		
+		if ((bezt == beztlast) && (nu->flagu & CU_CYCLIC)) { /* loop around */
+			bezt= beztfirst;
+		} else if (bezt != beztlast) {
+			bezt++;
+		}
+		nextrad = bezt->radius;
+		
+	}
+	else if( ( ((nu->type & 7)==CU_NURBS) || ((nu->type & 7)==CU_POLY)) && (bp != NULL)) {
+		/* follows similar algo as for bezt above */
+		bpfirst = nu->bp;
+		bplast = nu->bp + (nu->pntsu - 1);
+		
+		if ((nu->type & 7)==CU_POLY) resolu=1;
+		
+		while(bp<=bplast) {
+			if (subdivs==cursubdiv) {
+				ratio= 0.0;
+				break;
+			}
+			
+			subdivs += resolu;
+
+			if (cursubdiv < subdivs) {
+				ratio = 1.0 - ((subdivs - cursubdiv)/(float)resolu);
+				break;
+			}
+
+			bp++;
+		}
+		
+		if ( ((nu->type & 7)==CU_NURBS) && (nu->flagu & CU_CYCLIC)) {
+			if (bp == bplast) bp = bpfirst;
+			else bp++;
+		}
+		
+		rad = prevrad = bp->radius;
+		
+		if ((bp == bplast) && (nu->flagu & CU_CYCLIC)) { /* loop around */
+			bp= bpfirst;
+		} else if (bp != bplast) {
+			bp++;
+		}
+		nextrad = bp->radius;
+		
+	}
+	
+	
+	if (nextrad != prevrad) {
+		/* smooth interpolation */
+		rad = prevrad + (nextrad-prevrad)*(3.0f*ratio*ratio - 2.0f*ratio*ratio*ratio);
+	}
+
+	if (rad > 0.0) 
+		return rad;
+	else
+		return 1.0;
+}
+
 /* ****************** HANDLES ************** */
 
 /*
@@ -2064,7 +2193,7 @@ void calchandlesNurb(Nurb *nu) /* first, if needed, set handle flags */
 	BezTriple *bezt, *prev, *next;
 	short a;
 
-	if((nu->type & 7)!=1) return;
+	if((nu->type & 7)!=CU_BEZIER) return;
 	if(nu->pntsu<2) return;
 	
 	a= nu->pntsu;
