@@ -1,5 +1,5 @@
 /**
- * $Id: depsgraph.c,v 1.53 2006/07/13 11:36:26 ton Exp $
+ * $Id: depsgraph.c,v 1.62 2007/01/08 13:10:17 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -71,6 +71,7 @@
 #include "BKE_mball.h"
 #include "BKE_modifier.h"
 #include "BKE_object.h"
+#include "BKE_scene.h"
 #include "BKE_utildefines.h"
 
 #include "MEM_guardedalloc.h"
@@ -389,7 +390,7 @@ static void build_dag_object(DagForest *dag, DagNode *scenenode, Object *ob, int
 		}
 	}
 	
-	/* driver dependencies */
+	/* driver dependencies, nla modifiers */
 	if(ob->ipo) 
 		dag_add_driver_relation(ob->ipo, dag, node, 0);
 	
@@ -399,9 +400,13 @@ static void build_dag_object(DagForest *dag, DagNode *scenenode, Object *ob, int
 	
 	if(ob->action) {
 		bActionChannel *chan;
+		bConstraintChannel *conchan;
 		for (chan = ob->action->chanbase.first; chan; chan=chan->next){
 			if(chan->ipo)
 				dag_add_driver_relation(chan->ipo, dag, node, 1);
+			for (conchan=chan->constraintChannels.first; conchan; conchan=conchan->next)
+				if(conchan->ipo)
+					dag_add_driver_relation(conchan->ipo, dag, node, 1);
 		}
 	}
 	if(ob->nlastrips.first) {
@@ -412,6 +417,15 @@ static void build_dag_object(DagForest *dag, DagNode *scenenode, Object *ob, int
 				for (chan = strip->act->chanbase.first; chan; chan=chan->next)
 					if(chan->ipo)
 						dag_add_driver_relation(chan->ipo, dag, node, 1);
+			if(strip->modifiers.first) {
+				bActionModifier *amod;
+				for(amod= strip->modifiers.first; amod; amod= amod->next) {
+					if(amod->ob) {
+						node2 = dag_get_node(dag, amod->ob);
+						dag_add_relation(dag, node2, node, DAG_RL_DATA_DATA|DAG_RL_OB_DATA);
+					}
+				}
+			}
 		}
 	}
 	if (ob->modifiers.first) {
@@ -446,6 +460,11 @@ static void build_dag_object(DagForest *dag, DagNode *scenenode, Object *ob, int
 					else
 						dag_add_relation(dag,node2,node,DAG_RL_OB_OB);
 		}
+		/* exception case: parent is duplivert */
+		if(ob->type==OB_MBALL && (ob->parent->transflag & OB_DUPLIVERTS)) {
+			dag_add_relation(dag, node2, node,DAG_RL_DATA_DATA|DAG_RL_OB_OB);
+		}
+		
 		addtoroot = 0;
 	}
 	if (ob->track) {
@@ -453,6 +472,11 @@ static void build_dag_object(DagForest *dag, DagNode *scenenode, Object *ob, int
 		dag_add_relation(dag,node2,node,DAG_RL_OB_OB);
 		addtoroot = 0;
 	}
+	if (ob->proxy) {
+		node2 = dag_get_node(dag, ob->proxy);
+		dag_add_relation(dag, node, node2, DAG_RL_DATA_DATA|DAG_RL_OB_OB);
+		/* inverted relation, so addtoroot shouldn't be set to zero */
+	}
 	
 	if (ob->transflag & OB_DUPLI) {
 		if((ob->transflag & OB_DUPLIGROUP) && ob->dup_group) {
@@ -466,7 +490,25 @@ static void build_dag_object(DagForest *dag, DagNode *scenenode, Object *ob, int
 			}
 		}
 	}
-	
+    
+	/* softbody collision  */
+	if((ob->type==OB_MESH) || (ob->type==OB_CURVE) || (ob->type==OB_LATTICE)) {
+		Base *base;
+		if(modifiers_isSoftbodyEnabled(ob)){
+			// would be nice to have a list of colliders here
+			// so for now walk all objects in scene check 'same layer rule'
+			for(base = G.scene->base.first; base; base= base->next) {
+				if( (base->lay & ob->lay) && base->object->pd) {
+					Object *ob1= base->object;
+					if((ob1->pd->deflect) && (ob1 != ob))  {
+						node2 = dag_get_node(dag, ob1);					
+						dag_add_relation(dag, node2, node, DAG_RL_DATA_DATA|DAG_RL_OB_DATA);						
+					}
+				}
+			}
+		}
+	}
+		
 	if (ob->type==OB_MBALL) {
 		Object *mom= find_basis_mball(ob);
 		if(mom!=ob) {
@@ -575,6 +617,8 @@ struct DagForest *build_dag(struct Scene *sce, short mask)
 		ob= base->object;
 		
 		build_dag_object(dag, scenenode, ob, mask);
+		if(ob->proxy)
+			build_dag_object(dag, scenenode, ob->proxy, mask);
 		
 		/* handled in next loop */
 		if(ob->dup_group) 
@@ -637,6 +681,14 @@ void free_forest(DagForest *Dag)
 			itA = itA->next;
 			MEM_freeN(tempA);			
 		}
+		
+		itA = itN->parent;	
+		while (itA) {
+			tempA = itA;
+			itA = itA->next;
+			MEM_freeN(tempA);			
+		}
+		
 		tempN = itN;
 		itN = itN->next;
 		MEM_freeN(tempN);
@@ -746,6 +798,27 @@ void dag_add_relation(DagForest *forest, DagNode *fob1, DagNode *fob2, short rel
 	fob1->child = itA;
 }
 
+static void dag_add_parent_relation(DagForest *forest, DagNode *fob1, DagNode *fob2, short rel) 
+{
+	DagAdjList *itA = fob2->parent;
+	
+	while (itA) { /* search if relation exist already */
+		if (itA->node == fob1) {
+			itA->type |= rel;
+			itA->count += 1;
+			return;
+		}
+		itA = itA->next;
+	}
+	/* create new relation and insert at head. MALLOC alert! */
+	itA = MEM_mallocN(sizeof(DagAdjList),"DAG adj list");
+	itA->node = fob1;
+	itA->type = rel;
+	itA->count = 1;
+	itA->next = fob2->parent;
+	fob2->parent = itA;
+}
+
 
 /*
  * MainDAG is the DAG of all objects in current scene
@@ -1473,7 +1546,7 @@ static void flush_update_node(DagNode *node, unsigned int layer, int curtime)
 	node->lasttime= curtime;
 	
 	ob= node->ob;
-	if(ob->recalc & OB_RECALC) {
+	if(ob && (ob->recalc & OB_RECALC)) {
 		all_layer= ob->lay;
 		/* got an object node that changes, now check relations */
 		for(itA = node->child; itA; itA= itA->next) {
@@ -1725,15 +1798,16 @@ static void dag_object_time_update_flags(Object *ob)
 }
 
 /* flag all objects that need recalc, for changes in time for example */
-void DAG_scene_update_flags(Scene *sce, unsigned int lay)
+void DAG_scene_update_flags(Scene *scene, unsigned int lay)
 {
 	Base *base;
 	Object *ob;
 	Group *group;
 	GroupObject *go;
+	Scene *sce;
 	
 	/* set ob flags where animated systems are */
-	for(base= sce->base.first; base; base= base->next) {
+	for(SETLOOPER(scene, base)) {
 		ob= base->object;
 		
 		/* now if DagNode were part of base, the node->lay could be checked... */
@@ -1743,8 +1817,7 @@ void DAG_scene_update_flags(Scene *sce, unsigned int lay)
 		/* handled in next loop */
 		if(ob->dup_group) 
 			ob->dup_group->id.flag |= LIB_DOIT;
-		
-	}
+	}	
 	
 	/* we do groups each once */
 	for(group= G.main->group.first; group; group= group->id.next) {
@@ -1755,25 +1828,26 @@ void DAG_scene_update_flags(Scene *sce, unsigned int lay)
 		}
 	}
 	
-	DAG_scene_flush_update(sce, lay);
+	for(sce= scene; sce; sce= sce->set)
+		DAG_scene_flush_update(sce, lay);
 	
 	/* test: set time flag, to disable baked systems to update */
-	for(base= sce->base.first; base; base= base->next) {
+	for(SETLOOPER(scene, base)) {
 		ob= base->object;
 		if(ob->recalc)
 			ob->recalc |= OB_RECALC_TIME;
 	}
 	
 	/* hrmf... an exception to look at once, for invisible camera object we do it over */
-	if(sce->camera)
-		dag_object_time_update_flags(sce->camera);
+	if(scene->camera)
+		dag_object_time_update_flags(scene->camera);
 	
 	/* and store the info in groupobject */
 	for(group= G.main->group.first; group; group= group->id.next) {
 		if(group->id.flag & LIB_DOIT) {
 			for(go= group->gobject.first; go; go= go->next) {
 				go->recalc= go->ob->recalc;
-//				printf("ob %s recalc %d\n", go->ob->id.name, go->recalc);
+				// printf("ob %s recalc %d\n", go->ob->id.name, go->recalc);
 			}
 			group->id.flag &= ~LIB_DOIT;
 		}
@@ -1912,6 +1986,51 @@ void DAG_object_update_flags(Scene *sce, Object *ob, unsigned int lay)
 
 /* ******************* DAG FOR ARMATURE POSE ***************** */
 
+static int node_recurs_level(DagNode *node, int level)
+{
+	DagAdjList *itA;
+	
+	node->color= DAG_BLACK;	/* done */
+	level++;
+	
+	for(itA= node->parent; itA; itA= itA->next) {
+		if(itA->node->color==DAG_WHITE)
+			itA->node->ancestor_count= node_recurs_level(itA->node, level);
+	}
+	
+	return level;
+}
+
+static void pose_check_cycle(DagForest *dag)
+{
+	DagNode *node;
+	DagAdjList *itA;
+
+	/* tag nodes unchecked */
+	for(node = dag->DagNode.first; node; node= node->next)
+		node->color= DAG_WHITE;
+	
+	for(node = dag->DagNode.first; node; node= node->next) {
+		if(node->color==DAG_WHITE) {
+			node->ancestor_count= node_recurs_level(node, 0);
+		}
+	}
+	
+	/* check relations, and print errors */
+	for(node = dag->DagNode.first; node; node= node->next) {
+		for(itA= node->parent; itA; itA= itA->next) {
+			if(itA->node->ancestor_count > node->ancestor_count) {
+				bPoseChannel *pchan= (bPoseChannel *)node->ob;
+				bPoseChannel *parchan= (bPoseChannel *)itA->node->ob;
+				
+				if(pchan && parchan) 
+					if(pchan->parent!=parchan)
+						printf("Cycle in %s to %s\n", pchan->name, parchan->name);
+			}
+		}
+	}
+}
+
 /* we assume its an armature with pose */
 void DAG_pose_sort(Object *ob)
 {
@@ -1941,6 +2060,7 @@ void DAG_pose_sort(Object *ob)
 		if(pchan->parent) {
 			node2 = dag_get_node(dag, pchan->parent);
 			dag_add_relation(dag, node2, node, 0);
+			dag_add_parent_relation(dag, node2, node, 0);
 			addtoroot = 0;
 		}
 		for (con = pchan->constraints.first; con; con=con->next){
@@ -1953,6 +2073,7 @@ void DAG_pose_sort(Object *ob)
 					if(target) {
 						node2= dag_get_node(dag, target);
 						dag_add_relation(dag, node2, node, 0);
+						dag_add_parent_relation(dag, node2, node, 0);
 						
 						if(con->type==CONSTRAINT_TYPE_KINEMATIC) {
 							bKinematicConstraint *data = (bKinematicConstraint*)con->data;
@@ -1969,6 +2090,7 @@ void DAG_pose_sort(Object *ob)
 							while (parchan){
 								node3= dag_get_node(dag, parchan);
 								dag_add_relation(dag, node2, node3, 0);
+								dag_add_parent_relation(dag, node2, node3, 0);
 								
 								segcount++;
 								if(segcount==data->rootbone || segcount>255) break; // 255 is weak
@@ -1979,10 +2101,14 @@ void DAG_pose_sort(Object *ob)
 				}
 			}
 		}
-		if (addtoroot == 1 )
+		if (addtoroot == 1 ) {
 			dag_add_relation(dag, rootnode, node, 0);
+			dag_add_parent_relation(dag, rootnode, node, 0);
+		}
 	}
 
+	pose_check_cycle(dag);
+	
 	/* now we try to sort... */
 	tempbase.first= tempbase.last= NULL;
 
