@@ -1,7 +1,7 @@
 /*  displist.c
  * 
  * 
- * $Id: displist.c,v 1.136 2006/06/21 15:14:43 blendix Exp $
+ * $Id: displist.c,v 1.153 2007/02/14 14:42:48 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -86,6 +86,7 @@
 #include "BKE_scene.h"
 #include "BKE_subsurf.h"
 #include "BKE_modifier.h"
+#include "BKE_customdata.h"
 
 #include "RE_pipeline.h"
 #include "RE_shader_ext.h"
@@ -93,90 +94,6 @@
 
 static void boundbox_displist(Object *ob);
 
-void displistmesh_free(DispListMesh *dlm) 
-{
-	// also check on mvert and mface, can be NULL after decimator (ton)
-	if (!dlm->dontFreeVerts && dlm->mvert) MEM_freeN(dlm->mvert);
-	if (!dlm->dontFreeNors && dlm->nors) MEM_freeN(dlm->nors);
-	if (!dlm->dontFreeOther) {
-		if (dlm->medge) MEM_freeN(dlm->medge);
-		if (dlm->mface) MEM_freeN(dlm->mface);
-		if (dlm->mcol) MEM_freeN(dlm->mcol);
-		if (dlm->tface) MEM_freeN(dlm->tface);
-	}
-	MEM_freeN(dlm);
-}
-
-DispListMesh *displistmesh_copy(DispListMesh *odlm) 
-{
-	DispListMesh *ndlm= MEM_dupallocN(odlm);
-	ndlm->mvert= MEM_dupallocN(odlm->mvert);
-	if (odlm->medge) ndlm->medge= MEM_dupallocN(odlm->medge);
-	ndlm->mface= MEM_dupallocN(odlm->mface);
-	if (odlm->nors) ndlm->nors = MEM_dupallocN(odlm->nors);
-	if (odlm->mcol) ndlm->mcol= MEM_dupallocN(odlm->mcol);
-	if (odlm->tface) ndlm->tface= MEM_dupallocN(odlm->tface);
-
-	return ndlm;
-}
-
-DispListMesh *displistmesh_copyShared(DispListMesh *odlm) 
-{
-	DispListMesh *ndlm= MEM_dupallocN(odlm);
-	ndlm->dontFreeNors = ndlm->dontFreeOther = ndlm->dontFreeVerts = 1;
-	
-	return ndlm;
-}
-
-void displistmesh_to_mesh(DispListMesh *dlm, Mesh *me) 
-{
-		/* We assume, rather courageously, that any
-		 * shared data came from the mesh itself and so
-		 * we can ignore the dlm->dontFreeOther flag.
-		 */
-
-	if (me->mvert && dlm->mvert!=me->mvert) MEM_freeN(me->mvert);
-	if (me->mface && dlm->mface!=me->mface) MEM_freeN(me->mface);
-	if (me->tface && dlm->tface!=me->tface) MEM_freeN(me->tface);
-	if (me->mcol && dlm->mcol!=me->mcol) MEM_freeN(me->mcol);
-	if (me->medge && dlm->medge!=me->medge) MEM_freeN(me->medge);
-
-	me->tface = NULL;
-	me->mcol = NULL;
-	me->medge = NULL;
-
-	if (dlm->totvert!=me->totvert) {
-		if (me->msticky) MEM_freeN(me->msticky);
-		me->msticky = NULL;
-
-		if (me->dvert) free_dverts(me->dvert, me->totvert);
-		me->dvert = NULL;
-
-		if(me->key) me->key->id.us--;
-		me->key = NULL;
-	}
-
-	me->totface= dlm->totface;
-	me->totvert= dlm->totvert;
-	me->totedge= 0;
-
-	me->mvert= dlm->mvert;
-	me->mface= dlm->mface;
-	if (dlm->tface)
-		me->tface= dlm->tface;
-	if (dlm->mcol)
-		me->mcol= dlm->mcol;
-
-	if(dlm->medge) {
-		me->totedge= dlm->totedge;
-		me->medge= dlm->medge;
-	}
-
-	if (dlm->nors && !dlm->dontFreeNors) MEM_freeN(dlm->nors);
-
-	MEM_freeN(dlm);
-}
-
 void free_disp_elem(DispList *dl)
 {
 	if(dl) {
@@ -279,14 +196,14 @@ void addnormalsDispList(Object *ob, ListBase *lb)
 	
 	while(dl) {
 		if(dl->type==DL_INDEX3) {
-			if(dl->nors==0) {
+			if(dl->nors==NULL) {
 				dl->nors= MEM_callocN(sizeof(float)*3, "dlnors");
 				if(dl->verts[2]<0.0) dl->nors[2]= -1.0;
 				else dl->nors[2]= 1.0;
 			}
 		}
 		else if(dl->type==DL_SURF) {
-			if(dl->nors==0) {
+			if(dl->nors==NULL) {
 				dl->nors= MEM_callocN(sizeof(float)*3*dl->nr*dl->parts, "dlnors");
 				
 				vdata= dl->verts;
@@ -359,16 +276,16 @@ void count_displist(ListBase *lb, int *totvert, int *totface)
 }
 
 
-/* ***************************** shade displist ******************** */
+/* ***************************** shade displist. note colors now are in rgb(a) order ******************** */
 
 /* create default shade input... save cpu cycles with ugly global */
 static ShadeInput shi;
 static void init_fastshade_shadeinput(void)
 {
-	
 	memset(&shi, 0, sizeof(ShadeInput));
 	shi.lay= G.scene->lay;
 	shi.view[2]= -1.0f;
+	shi.passflag= SCE_PASS_COMBINED;
 }
 
 static Render *fastshade_get_render(void)
@@ -377,7 +294,7 @@ static Render *fastshade_get_render(void)
 	if(re==NULL) {
 		re= RE_NewRender("_Shade View_");
 	
-		RE_Database_Shaded(re, G.scene);
+		RE_Database_Baking(re, G.scene, 0);	/* 0= no faces */
 	}
 	return re;
 }
@@ -393,22 +310,69 @@ void fastshade_free_render(void)
 	}
 }
 
-static void fastshade(float *co, float *nor, float *orco, Material *ma, char *col1, char *col2, char *vertcol)
+static int fastshade_customdata_layer_num(int n, int active)
+{   
+	/* make the active layer the first */
+	if (n == active) return 0;
+	else if (n < active) return n+1;
+	else return n;
+}
+
+static void fastshade_customdata(CustomData *fdata, int a, int j, Material *ma)
+{
+	CustomDataLayer *layer;
+	MTFace *mtface;
+	int index, n, needuv= ma->texco & TEXCO_UV;
+	char *vertcol;
+
+	shi.totuv= 0;
+	shi.totcol= 0;
+
+	for(index=0; index<fdata->totlayer; index++) {
+		layer= &fdata->layers[index];
+		
+		if(needuv && layer->type == CD_MTFACE && shi.totuv < MAX_MTFACE) {
+			n= fastshade_customdata_layer_num(shi.totuv, layer->active);
+			mtface= &((MTFace*)layer->data)[a];
+
+			shi.uv[shi.totuv].uv[0]= 2.0f*mtface->uv[j][0]-1.0f;
+			shi.uv[shi.totuv].uv[1]= 2.0f*mtface->uv[j][1]-1.0f;
+			shi.uv[shi.totuv].uv[2]= 1.0f;
+
+			shi.uv[shi.totuv].name= layer->name;
+			shi.totuv++;
+		}
+		else if(layer->type == CD_MCOL && shi.totcol < MAX_MCOL) {
+			n= fastshade_customdata_layer_num(shi.totcol, layer->active);
+			vertcol= (char*)&((MCol*)layer->data)[a*4 + j];
+
+			shi.col[shi.totcol].col[0]= ((float)vertcol[3])/255.0f;
+			shi.col[shi.totcol].col[1]= ((float)vertcol[2])/255.0f;
+			shi.col[shi.totcol].col[2]= ((float)vertcol[1])/255.0f;
+
+			shi.col[shi.totcol].name= layer->name;
+			shi.totcol++;
+		}
+	}
+
+	if(needuv && shi.totuv == 0)
+		VECCOPY(shi.uv[0].uv, shi.lo);
+
+	if(shi.totcol)
+		VECCOPY(shi.vcol, shi.col[0].col);
+}
+
+static void fastshade(float *co, float *nor, float *orco, Material *ma, char *col1, char *col2)
 {
 	ShadeResult shr;
 	int a;
 	
-	if(vertcol) {
-		shi.vcol[0]= ((float)vertcol[3])/255.0f;
-		shi.vcol[1]= ((float)vertcol[2])/255.0f;
-		shi.vcol[2]= ((float)vertcol[1])/255.0f;
-	}
-	
 	VECCOPY(shi.co, co);
 	shi.vn[0]= -nor[0];
 	shi.vn[1]= -nor[1];
 	shi.vn[2]= -nor[2];
 	VECCOPY(shi.vno, shi.vn);
+	VECCOPY(shi.facenor, shi.vn);
 	
 	if(ma->texco) {
 		VECCOPY(shi.lo, orco);
@@ -422,9 +386,6 @@ static void fastshade(float *co, float *nor, float *orco, Material *ma, char *co
 		if(ma->texco & TEXCO_STICKY) {
 			VECCOPY(shi.sticky, shi.lo);
 		}
-		if(ma->texco & TEXCO_UV) {
-			VECCOPY(shi.uv, shi.lo);
-		}
 		if(ma->texco & TEXCO_OBJECT) {
 			VECCOPY(shi.co, shi.lo);
 		}
@@ -442,14 +403,12 @@ static void fastshade(float *co, float *nor, float *orco, Material *ma, char *co
 	shi.mat= ma;	/* set each time... node shaders change it */
 	RE_shade_external(NULL, &shi, &shr);
 	
-	VECADD(shr.combined, shr.diff, shr.spec);
-	
 	a= 256.0f*(shr.combined[0]);
-	col1[3]= CLAMPIS(a, 0, 255);
+	col1[0]= CLAMPIS(a, 0, 255);
 	a= 256.0f*(shr.combined[1]);
-	col1[2]= CLAMPIS(a, 0, 255);
-	a= 256.0f*(shr.combined[2]);
 	col1[1]= CLAMPIS(a, 0, 255);
+	a= 256.0f*(shr.combined[2]);
+	col1[2]= CLAMPIS(a, 0, 255);
 	
 	if(col2) {
 		shi.vn[0]= -shi.vn[0];
@@ -459,14 +418,12 @@ static void fastshade(float *co, float *nor, float *orco, Material *ma, char *co
 		shi.mat= ma;	/* set each time... node shaders change it */
 		RE_shade_external(NULL, &shi, &shr);
 		
-		VECADD(shr.combined, shr.diff, shr.spec);
-		
 		a= 256.0f*(shr.combined[0]);
-		col2[3]= CLAMPIS(a, 0, 255);
+		col2[0]= CLAMPIS(a, 0, 255);
 		a= 256.0f*(shr.combined[1]);
-		col2[2]= CLAMPIS(a, 0, 255);
-		a= 256.0f*(shr.combined[2]);
 		col2[1]= CLAMPIS(a, 0, 255);
+		a= 256.0f*(shr.combined[2]);
+		col2[2]= CLAMPIS(a, 0, 255);
 	}
 }
 
@@ -517,42 +474,48 @@ static void end_fastshade_for_ob(Object *ob)
 static void mesh_create_shadedColors(Render *re, Object *ob, int onlyForMesh, unsigned int **col1_r, unsigned int **col2_r)
 {
 	Mesh *me= ob->data;
-	int dmNeedsFree;
 	DerivedMesh *dm;
-	DispListMesh *dlm;
+	MVert *mvert;
+	MFace *mface;
 	unsigned int *col1, *col2;
-	float *orco, *vnors, imat[3][3], mat[4][4], vec[3];
-	int a, i, need_orco;
+	float *orco, *vnors, *nors, imat[3][3], mat[4][4], vec[3];
+	int a, i, need_orco, totface, totvert;
+	CustomDataMask dataMask = CD_MASK_BAREMESH | CD_MASK_MCOL
+	                          | CD_MASK_MTFACE | CD_MASK_NORMAL;
 
 	init_fastshade_for_ob(re, ob, &need_orco, mat, imat);
 
-	if (need_orco) {
-		orco = mesh_create_orco(ob);
-	} else {
-		orco = NULL;
-	}
+	orco = (need_orco)? mesh_create_orco(ob): NULL;
+
+	if (onlyForMesh)
+		dm = mesh_get_derived_deform(ob, dataMask);
+	else
+		dm = mesh_get_derived_final(ob, dataMask);
+	
+	mvert = dm->getVertArray(dm);
+	mface = dm->getFaceArray(dm);
+	nors = dm->getFaceDataArray(dm, CD_NORMAL);
+	totvert = dm->getNumVerts(dm);
+	totface = dm->getNumFaces(dm);
 
 	if (onlyForMesh) {
-		dm = mesh_get_derived_deform(ob, &dmNeedsFree);
-	} else {
-		dm = mesh_get_derived_final(ob, &dmNeedsFree);
-	}
-	dlm= dm->convertToDispListMesh(dm, 1);
-
-	col1 = MEM_mallocN(sizeof(*col1)*dlm->totface*4, "col1");
-	if (col2_r && (me->flag & ME_TWOSIDED)) {
-		col2 = MEM_mallocN(sizeof(*col2)*dlm->totface*4, "col1");
-	} else {
+		col1 = *col1_r;
 		col2 = NULL;
+	} else {
+		*col1_r = col1 = MEM_mallocN(sizeof(*col1)*totface*4, "col1");
+
+		if (col2_r && (me->flag & ME_TWOSIDED))
+			col2 = MEM_mallocN(sizeof(*col2)*totface*4, "col2");
+		else
+			col2 = NULL;
+		
+		if (col2_r) *col2_r = col2;
 	}
-	
-	*col1_r = col1;
-	if (col2_r) *col2_r = col2;
 
 		/* vertexnormals */
-	vnors= MEM_mallocN(dlm->totvert*3*sizeof(float), "vnors disp");
-	for (a=0; a<dlm->totvert; a++) {
-		MVert *mv = &dlm->mvert[a];
+	vnors= MEM_mallocN(totvert*3*sizeof(float), "vnors disp");
+	for (a=0; a<totvert; a++) {
+		MVert *mv = &mvert[a];
 		float *vn= &vnors[a*3];
 		float xn= mv->no[0]; 
 		float yn= mv->no[1]; 
@@ -565,37 +528,28 @@ static void mesh_create_shadedColors(Render *re, Object *ob, int onlyForMesh, un
 		Normalise(vn);
 	}		
 
-	for (i=0; i<dlm->totface; i++) {
-		MFace *mf= &dlm->mface[i];
+	for (i=0; i<totface; i++) {
+		MFace *mf= &mface[i];
+		Material *ma= give_current_material(ob, mf->mat_nr+1);
 		int j, vidx[4], nverts= mf->v4?4:3;
 		unsigned char *col1base= (unsigned char*) &col1[i*4];
 		unsigned char *col2base= (unsigned char*) (col2?&col2[i*4]:NULL);
-		unsigned char *mcolbase;
-		Material *ma= give_current_material(ob, mf->mat_nr+1);
 		float nor[3], n1[3];
 		
 		if(ma==NULL) ma= &defmaterial;
 		
-		if (dlm->tface) {
-			mcolbase = (unsigned char*) dlm->tface[i].col;
-		} else if (dlm->mcol) {
-			mcolbase = (unsigned char*) &dlm->mcol[i*4];
-		} else {
-			mcolbase = NULL;
-		}
-
 		vidx[0]= mf->v1;
 		vidx[1]= mf->v2;
 		vidx[2]= mf->v3;
 		vidx[3]= mf->v4;
 
-		if (dlm->nors) {
-			VECCOPY(nor, &dlm->nors[i*3]);
+		if (nors) {
+			VECCOPY(nor, &nors[i*3]);
 		} else {
 			if (mf->v4)
-				CalcNormFloat4(dlm->mvert[mf->v1].co, dlm->mvert[mf->v2].co, dlm->mvert[mf->v3].co, dlm->mvert[mf->v4].co, nor);
+				CalcNormFloat4(mvert[mf->v1].co, mvert[mf->v2].co, mvert[mf->v3].co, mvert[mf->v4].co, nor);
 			else
-				CalcNormFloat(dlm->mvert[mf->v1].co, dlm->mvert[mf->v2].co, dlm->mvert[mf->v3].co, nor);
+				CalcNormFloat(mvert[mf->v1].co, mvert[mf->v2].co, mvert[mf->v3].co, nor);
 		}
 
 		n1[0]= imat[0][0]*nor[0]+imat[0][1]*nor[1]+imat[0][2]*nor[2];
@@ -604,33 +558,46 @@ static void mesh_create_shadedColors(Render *re, Object *ob, int onlyForMesh, un
 		Normalise(n1);
 
 		for (j=0; j<nverts; j++) {
-			MVert *mv= &dlm->mvert[vidx[j]];
-			unsigned char *col1= &col1base[j*4];
-			unsigned char *col2= col2base?&col2base[j*4]:NULL;
-			unsigned char *mcol= mcolbase?&mcolbase[j*4]:NULL;
+			MVert *mv= &mvert[vidx[j]];
+			char *col1= (char*)&col1base[j*4];
+			char *col2= (char*)(col2base?&col2base[j*4]:NULL);
 			float *vn = (mf->flag & ME_SMOOTH)?&vnors[3*vidx[j]]:n1;
-
+			
 			VECCOPY(vec, mv->co);
 			Mat4MulVecfl(mat, vec);
-			fastshade(vec, vn, orco?&orco[vidx[j]*3]:mv->co, ma, col1, col2, mcol);
+			vec[0]+= 0.001*vn[0];
+			vec[1]+= 0.001*vn[1];
+			vec[2]+= 0.001*vn[2];
+
+			fastshade_customdata(&dm->faceData, i, j, ma);
+			fastshade(vec, vn, orco?&orco[vidx[j]*3]:mv->co, ma, col1, col2);
 		}
 	} 
 	MEM_freeN(vnors);
-	displistmesh_free(dlm);
 
-	if (orco) {
+	if (orco)
 		MEM_freeN(orco);
-	}
 
-	if (dmNeedsFree) dm->release(dm);
+	dm->release(dm);
 
 	end_fastshade_for_ob(ob);
 }
 
 void shadeMeshMCol(Object *ob, Mesh *me)
 {
+	int a;
+	char *cp;
+	unsigned int *mcol= (unsigned int*)me->mcol;
+	
 	Render *re= fastshade_get_render();
-	mesh_create_shadedColors(re, ob, 1, (unsigned int**)&me->mcol, NULL);
+	mesh_create_shadedColors(re, ob, 1, &mcol, NULL);
+	me->mcol= (MCol*)mcol;
+
+	/* swap bytes */
+	for(cp= (char *)me->mcol, a= 4*me->totface; a>0; a--, cp+=4) {
+		SWAP(char, cp[0], cp[3]);
+		SWAP(char, cp[1], cp[2]);
+	}
 }
 
 /* has base pointer, to check for layer */
@@ -657,10 +624,13 @@ void shadeDispList(Base *base)
 
 	if(ob->type==OB_MESH) {
 		dl= MEM_callocN(sizeof(DispList), "displistshade");
-		BLI_addtail(&ob->disp, dl);
 		dl->type= DL_VERTCOL;
 
 		mesh_create_shadedColors(re, ob, 0, &dl->col1, &dl->col2);
+
+		/* add dl to ob->disp after mesh_create_shadedColors, because it
+		   might indirectly free ob->disp */
+		BLI_addtail(&ob->disp, dl);
 	}
 	else {
 
@@ -705,7 +675,7 @@ void shadeDispList(Base *base)
 							VECCOPY(vec, fp);
 							Mat4MulVecfl(mat, vec);
 							
-							fastshade(vec, n1, fp, ma, (char *)col1, 0, 0);
+							fastshade(vec, n1, fp, ma, (char *)col1, NULL);
 							
 							fp+= 3; col1++;
 						}
@@ -726,7 +696,7 @@ void shadeDispList(Base *base)
 							n1[2]= imat[2][0]*nor[0]+imat[2][1]*nor[1]+imat[2][2]*nor[2];
 							Normalise(n1);
 				
-							fastshade(vec, n1, fp, ma, (char *)col1, 0, 0);
+							fastshade(vec, n1, fp, ma, (char *)col1, NULL);
 							
 							fp+= 3; nor+= 3; col1++;
 						}
@@ -764,7 +734,7 @@ void shadeDispList(Base *base)
 							n1[2]= imat[2][0]*nor[0]+imat[2][1]*nor[1]+imat[2][2]*nor[2];
 							Normalise(n1);
 						
-							fastshade(vec, n1, fp, ma, (char *)col1, 0, 0);
+							fastshade(vec, n1, fp, ma, (char *)col1, NULL);
 							
 							fp+= 3; col1++; nor+= 3;
 						}
@@ -779,6 +749,7 @@ void shadeDispList(Base *base)
 }
 
 /* frees render and shade part of displists */
+/* note: dont do a shade again, until a redraw happens */
 void reshadeall_displist(void)
 {
 	Base *base;
@@ -816,22 +787,23 @@ static void curve_to_displist(Curve *cu, ListBase *nubase, ListBase *dispbase)
 			else
 				resolu= nu->resolu;
 			
-			if((nu->type & 7)==CU_BEZIER) {
+			if(nu->pntsu<2);
+			else if((nu->type & 7)==CU_BEZIER) {
 				
 				/* count */
 				len= 0;
 				a= nu->pntsu-1;
-				if(nu->flagu & 1) a++;
+				if(nu->flagu & CU_CYCLIC) a++;
 
 				prevbezt= nu->bezt;
 				bezt= prevbezt+1;
 				while(a--) {
-					if(a==0 && (nu->flagu & 1)) bezt= nu->bezt;
+					if(a==0 && (nu->flagu & CU_CYCLIC)) bezt= nu->bezt;
 					
 					if(prevbezt->h2==HD_VECT && bezt->h1==HD_VECT) len++;
 					else len+= resolu;
 					
-					if(a==0 && (nu->flagu & 1)==0) len++;
+					if(a==0 && (nu->flagu & CU_CYCLIC)==0) len++;
 					
 					prevbezt= bezt;
 					bezt++;
@@ -1119,7 +1091,6 @@ void curve_to_filledpoly(Curve *cu, ListBase *nurb, ListBase *dispbase)
 	}
 }
 
-
 /* taper rules:
   - only 1 curve
   - first point left, last point right
@@ -1190,13 +1161,18 @@ static ModifierData *curve_get_tesselate_point(Object *ob, int forRender, int ed
 {
 	ModifierData *md = modifiers_getVirtualModifierList(ob);
 	ModifierData *preTesselatePoint;
+	int required_mode;
+
+	if(forRender) required_mode = eModifierMode_Render;
+	else required_mode = eModifierMode_Realtime;
+
+	if(editmode) required_mode |= eModifierMode_Editmode;
 
 	preTesselatePoint = NULL;
 	for (; md; md=md->next) {
 		ModifierTypeInfo *mti = modifierType_getInfo(md->type);
 
-		if (!(md->mode&(1<<forRender))) continue;
-		if (editmode && !(md->mode&eModifierMode_Editmode)) continue;
+		if ((md->mode & required_mode) != required_mode) continue;
 		if (mti->isDisabled && mti->isDisabled(md)) continue;
 
 		if (md->type==eModifierType_Hook || md->type==eModifierType_Softbody) {
@@ -1215,6 +1191,12 @@ void curve_calc_modifiers_pre(Object *ob, ListBase *nurb, int forRender, float (
 	int numVerts = 0;
 	float (*originalVerts)[3] = NULL;
 	float (*deformedVerts)[3] = NULL;
+	int required_mode;
+
+	if(forRender) required_mode = eModifierMode_Render;
+	else required_mode = eModifierMode_Realtime;
+
+	if(editmode) required_mode |= eModifierMode_Editmode;
 
 	if(ob!=G.obedit && do_ob_key(ob)) {
 		deformedVerts = curve_getVertexCos(ob->data, nurb, &numVerts);
@@ -1225,8 +1207,7 @@ void curve_calc_modifiers_pre(Object *ob, ListBase *nurb, int forRender, float (
 		for (; md; md=md->next) {
 			ModifierTypeInfo *mti = modifierType_getInfo(md->type);
 
-			if (!(md->mode&(1<<forRender))) continue;
-			if (editmode && !(md->mode&eModifierMode_Editmode)) continue;
+			if ((md->mode & required_mode) != required_mode) continue;
 			if (mti->isDisabled && mti->isDisabled(md)) continue;
 			if (mti->type!=eModifierTypeType_OnlyDeform) continue;
 
@@ -1257,6 +1238,12 @@ void curve_calc_modifiers_post(Object *ob, ListBase *nurb, ListBase *dispbase, i
 	ModifierData *md = modifiers_getVirtualModifierList(ob);
 	ModifierData *preTesselatePoint = curve_get_tesselate_point(ob, forRender, editmode);
 	DispList *dl;
+	int required_mode;
+
+	if(forRender) required_mode = eModifierMode_Render;
+	else required_mode = eModifierMode_Realtime;
+
+	if(editmode) required_mode |= eModifierMode_Editmode;
 
 	if (preTesselatePoint) {
 		md = preTesselatePoint->next;
@@ -1265,8 +1252,7 @@ void curve_calc_modifiers_post(Object *ob, ListBase *nurb, ListBase *dispbase, i
 	for (; md; md=md->next) {
 		ModifierTypeInfo *mti = modifierType_getInfo(md->type);
 
-		if (!(md->mode&(1<<forRender))) continue;
-		if (editmode && !(md->mode&eModifierMode_Editmode)) continue;
+		if ((md->mode & required_mode) != required_mode) continue;
 		if (mti->isDisabled && mti->isDisabled(md)) continue;
 		if (mti->type!=eModifierTypeType_OnlyDeform) continue;
 
@@ -1282,6 +1268,35 @@ void curve_calc_modifiers_post(Object *ob, ListBase *nurb, ListBase *dispbase, i
 	}
 }
 
+static void displist_surf_indices(DispList *dl)
+{
+	int a, b, p1, p2, p3, p4;
+	int *index;
+	
+	dl->totindex= 0;
+	
+	index=dl->index= MEM_mallocN( 4*sizeof(int)*(dl->parts+1)*(dl->nr+1), "index array nurbs");
+	
+	for(a=0; a<dl->parts; a++) {
+		
+		DL_SURFINDEX(dl->flag & DL_CYCL_U, dl->flag & DL_CYCL_V, dl->nr, dl->parts);
+		
+		for(; b<dl->nr; b++, index+=4) {	
+			index[0]= p1;
+			index[1]= p2;
+			index[2]= p4;
+			index[3]= p3;
+			
+			dl->totindex++;
+			
+			p2= p1; p1++;
+			p4= p3; p3++;
+
+		}
+	}
+	
+}
+
 void makeDispListSurf(Object *ob, ListBase *dispbase, int forRender)
 {
 	ListBase *nubase;
@@ -1344,6 +1359,9 @@ void makeDispListSurf(Object *ob, ListBase *dispbase, int forRender)
 				if(nu->flagu & CU_CYCLIC) dl->flag|= DL_CYCL_V;
 
 				makeNurbfaces(nu, data, 0);
+				
+				/* gl array drawing: using indices */
+				displist_surf_indices(dl);
 			}
 		}
 	}
@@ -1389,7 +1407,7 @@ void makeDispListCurveTypes(Object *ob, int forOrco)
 
 		makeBevelList(ob);
 
-			/* If curve has no bevel will return nothing */
+		/* If curve has no bevel will return nothing */
 		makebevelcurve(ob, &dlbev);
 
 		/* no bevel or extrude, and no width correction? */
@@ -1461,7 +1479,13 @@ void makeDispListCurveTypes(Object *ob, int forOrco)
 							/* for each point of poly make a bevel piece */
 						bevp= (BevPoint *)(bl+1);
 						for(a=0; a<bl->nr; a++,bevp++) {
-							float fac = calc_taper(cu->taperobj, a, bl->nr);
+							float fac=1.0;
+							if (cu->taperobj==NULL) {
+								if ( (cu->bevobj!=NULL) || !((cu->flag & CU_FRONT) || (cu->flag & CU_BACK)) )
+								fac = calc_curve_subdiv_radius(cu, nu, a);
+							} else {
+								fac = calc_taper(cu->taperobj, a, bl->nr);
+							}
 							
 							if (bevp->f1) {
 								dl->bevelSplitFlag[a>>5] |= 1<<(a&0x1F);
@@ -1490,6 +1514,9 @@ void makeDispListCurveTypes(Object *ob, int forOrco)
 								}
 							}
 						}
+						
+						/* gl array drawing: using indices */
+						displist_surf_indices(dl);
 					}
 				}
 
@@ -1513,6 +1540,7 @@ void imagestodisplist(void)
 	/* removed */
 }
 
+/* this is confusing, there's also min_max_object, appplying the obmat... */
 static void boundbox_displist(Object *ob)
 {
 	BoundBox *bb=0;
@@ -1525,6 +1553,7 @@ static void boundbox_displist(Object *ob)
 
 	if(ELEM3(ob->type, OB_CURVE, OB_SURF, OB_FONT)) {
 		Curve *cu= ob->data;
+		int doit= 0;
 
 		if(cu->bb==0) cu->bb= MEM_callocN(sizeof(BoundBox), "boundbox");	
 		bb= cu->bb;
@@ -1537,11 +1566,18 @@ static void boundbox_displist(Object *ob)
 			
 			vert= dl->verts;
 			for(a=0; a<tot; a++, vert+=3) {
+				doit= 1;
 				DO_MINMAX(vert, min, max);
 			}
 
 			dl= dl->next;
 		}
+		
+		if(!doit) {
+			min[0] = min[1] = min[2] = -1.0f;
+			max[0] = max[1] = max[2] = 1.0f;
+		}
+		
 	}
 	
 	if(bb) {
@@ -1549,43 +1585,3 @@ static void boundbox_displist(Object *ob)
 	}
 }
 
-void displistmesh_add_edges(DispListMesh *dlm)
-{
-	EdgeHash *eh = BLI_edgehash_new();
-	EdgeHashIterator *ehi;
-	int i;
-
-	for (i=0; i<dlm->totface; i++) {
-		MFace *mf = &dlm->mface[i];
-
-		if (!BLI_edgehash_haskey(eh, mf->v1, mf->v2))
-			BLI_edgehash_insert(eh, mf->v1, mf->v2, NULL);
-		if (!BLI_edgehash_haskey(eh, mf->v2, mf->v3))
-			BLI_edgehash_insert(eh, mf->v2, mf->v3, NULL);
-		
-		if (mf->v4) {
-			if (!BLI_edgehash_haskey(eh, mf->v3, mf->v4))
-				BLI_edgehash_insert(eh, mf->v3, mf->v4, NULL);
-			if (!BLI_edgehash_haskey(eh, mf->v4, mf->v1))
-				BLI_edgehash_insert(eh, mf->v4, mf->v1, NULL);
-		} else {
-			if (!BLI_edgehash_haskey(eh, mf->v3, mf->v1))
-				BLI_edgehash_insert(eh, mf->v3, mf->v1, NULL);
-		}
-	}
-
-	dlm->totedge = BLI_edgehash_size(eh);
-	dlm->medge = MEM_callocN(dlm->totedge*sizeof(*dlm->medge), "medge");
-
-	ehi = BLI_edgehashIterator_new(eh);
-	for (i=0; !BLI_edgehashIterator_isDone(ehi); BLI_edgehashIterator_step(ehi)) {
-		MEdge *med = &dlm->medge[i++];
-
-		BLI_edgehashIterator_getKey(ehi, &med->v1, &med->v2);
-
-		med->flag = ME_EDGEDRAW|ME_EDGERENDER;
-	}
-	BLI_edgehashIterator_free(ehi);
-
-	BLI_edgehash_free(eh, NULL);
-}
