@@ -1,7 +1,7 @@
 /*  effect.c
  * 
  * 
- * $Id: effect.c,v 1.96 2006/07/02 12:46:41 ton Exp $
+ * $Id: effect.c,v 1.105 2006/12/23 11:56:22 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -51,11 +51,6 @@
 #include "DNA_object_force.h"
 #include "DNA_texture_types.h"
 #include "DNA_scene_types.h"
-// FSPARTICLE
-#include "DNA_object_fluidsim.h"
-#include "LBM_fluidsim.h"
-#include <zlib.h>
-#include <string.h>
 
 #include "BLI_arithb.h"
 #include "BLI_blenlib.h"
@@ -81,15 +76,24 @@
 #include "BKE_mesh.h"
 #include "BKE_material.h"
 #include "BKE_main.h"
+#include "BKE_modifier.h"
 #include "BKE_object.h"
 #include "BKE_scene.h"
 #include "BKE_screen.h"
 #include "BKE_utildefines.h"
 
 #include "PIL_time.h"
-#include "elbeem.h"
 #include "RE_render_ext.h"
 
+/* fluid sim particle import */
+#ifndef DISABLE_ELBEEM
+#include "DNA_object_fluidsim.h"
+#include "LBM_fluidsim.h"
+#include "elbeem.h"
+#include <zlib.h>
+#include <string.h>
+#endif // DISABLE_ELBEEM
+
 /* temporal struct, used for reading return of mesh_get_mapped_verts_nors() */
 typedef struct VeNoCo {
 	float co[3], no[3];
@@ -693,7 +697,7 @@ static int pdDoDeflection(RNG *rng, float opco[3], float npco[3], float opno[3],
 	MFace *mface, *deflection_face = NULL;
 	float *v1, *v2, *v3, *v4, *vcache=NULL;
 	float nv1[3], nv2[3], nv3[3], nv4[3], edge1[3], edge2[3];
-	float dv1[3], dv2[3], dv3[3];
+	float dv1[3] = {0}, dv2[3] = {0}, dv3[3] = {0};
 	float vect_to_int[3], refl_vel[3];
 	float d_intersect_co[3], d_intersect_vect[3], d_nvect[3], d_i_co_above[3];
 	float forcec[3];
@@ -706,7 +710,7 @@ static int pdDoDeflection(RNG *rng, float opco[3], float npco[3], float opno[3],
 	float perm_val, rdamp_val;
 	int a, deflected=0, deflected_now=0;
 	float t,t2, min_t;
-	float mat[3][3], obloc[3];
+	float mat[3][3], obloc[3] = {0};
 	int cur_frame;
 	float time_before, time_after;
 	float force_mag_norm;
@@ -1014,7 +1018,7 @@ static void make_particle_keys(RNG *rng, Object *ob, int depth, int nr, PartEff
 {
 	Particle *pa, *opa = NULL;
 	float damp, deltalife, life;
-	float cur_time;
+	float cur_time, maxspeed= paf->maxlen/(float)paf->totkey;
 	float opco[3], opno[3], npco[3], npno[3], new_force[3], new_speed[3];
 	int b, rt1, rt2, deflected, deflection, finish_defs, def_count;
 	int last_ob, last_fc, same_fc;
@@ -1097,6 +1101,13 @@ static void make_particle_keys(RNG *rng, Object *ob, int depth, int nr, PartEff
 		pa->no[1]= deltalife * (new_speed[1] + new_force[1]);
 		pa->no[2]= deltalife * (new_speed[2] + new_force[2]);
 		
+		/* speed limitor */
+		if((paf->flag & PAF_STATIC) && maxspeed!=0.0f) {
+			float len= VecLength(pa->no);
+			if(len > maxspeed)
+				VecMulf(pa->no, maxspeed/len);
+		}
+			
 		/* new location */
 		pa->co[0]= opa->co[0] + pa->no[0];
 		pa->co[1]= opa->co[1] + pa->no[1];
@@ -1536,7 +1547,7 @@ static pMatrixCache *cache_object_matrices(Object *ob, int start, int end)
 	Group *group= NULL;
 	Object *obcopy;
 	Base *base;
-	float framelenold, cfrao;
+	float framelenold, cfrao, sfo;
 	
 	/* object can be linked in group... stupid exception */
 	if(NULL==object_in_scene(ob, G.scene))
@@ -1547,16 +1558,19 @@ static pMatrixCache *cache_object_matrices(Object *ob, int start, int end)
 	framelenold= G.scene->r.framelen;
 	G.scene->r.framelen= 1.0f;
 	cfrao= G.scene->r.cfra;
+	sfo= ob->sf;
 	ob->sf= 0.0f;
-	
+
 	/* clear storage */
 	for(obcopy= G.main->object.first; obcopy; obcopy= obcopy->id.next) 
 		obcopy->id.newid= NULL;
 	
 	/* all objects get tagged recalc that influence this object (does group too) */
 	/* another hack; while transform you cannot call this, it sets own recalc flags */
-	if(G.moving==0)
-		DAG_object_update_flags(G.scene, ob, G.scene->lay);
+	if(G.moving==0) {
+		ob->recalc |= OB_RECALC_OB; /* make sure a recalc gets flushed */
+		DAG_object_update_flags(G.scene, ob, -1);
+	}
 	
 	for(G.scene->r.cfra= start; G.scene->r.cfra<=end; G.scene->r.cfra++, mc++) {
 		
@@ -1600,7 +1614,8 @@ static pMatrixCache *cache_object_matrices(Object *ob, int start, int end)
 	/* restore */
 	G.scene->r.cfra= cfrao;
 	G.scene->r.framelen= framelenold;
-
+	ob->sf= sfo;
+	
 	if(group) {
 		GroupObject *go;
 		
@@ -1678,10 +1693,11 @@ void build_particle_system(Object *ob)
 	if(paf->keys) MEM_freeN(paf->keys);	/* free as early as possible, for returns */
 	paf->keys= NULL;
 	
-	printf("build particles\n");
+	//printf("build particles\n");
 	
-	// FSPARTICLE all own created...
-	if( (1) && (ob->fluidsimFlag & OB_FLUIDSIM_ENABLE) && 
+	/* fluid sim particle import handling, actual loading of particles from file */
+	#ifndef DISABLE_ELBEEM
+	if( (1) && (ob->fluidsimFlag & OB_FLUIDSIM_ENABLE) &&  // broken, disabled for now!
 	    (ob->fluidsimSettings) && 
 		  (ob->fluidsimSettings->type == OB_FLUIDSIM_PARTICLE)) {
 		char *suffix  = "fluidsurface_particles_#";
@@ -1694,7 +1710,7 @@ void build_particle_system(Object *ob)
 		float vel[3];
 
 		if(ob==G.obedit) { // off...
-			paf->totpart = 1;
+			paf->totpart = 0; // 1 or 0?
 			return;
 		}
 
@@ -1708,7 +1724,7 @@ void build_particle_system(Object *ob)
 		if (!gzf) {
 			snprintf(debugStrBuffer,256,"readFsPartData::error - Unable to open file for reading '%s' \n", filename); 
 			//elbeemDebugOut(debugStrBuffer);
-			paf->totpart = 1;
+			paf->totpart = 0;
 			return;
 		}
 
@@ -1745,7 +1761,6 @@ void build_particle_system(Object *ob)
 				// convert range of  1.0-10.0 to shorts 1000-10000)
 				shsize = (short)(convertSize*1000.0);
 				pa->rt = shsize;
-				//if(a<200) fprintf(stderr,"SREAD %f %d %d \n",convertSize,shsize,pa->rt);
 
 				for(j=0; j<3; j++) {
 					float wrf;
@@ -1773,8 +1788,8 @@ void build_particle_system(Object *ob)
 		snprintf(debugStrBuffer,256,"readFsPartData::done - particles:%d, active:%d, file:%d, mask:%d  \n", paf->totpart,activeParts,fileParts,readMask);
 		elbeemDebugOut(debugStrBuffer);
 		return;
-	}
-	
+	} // fluid sim particles done
+	#endif // DISABLE_ELBEEM
 	
 	if(paf->end < paf->sta) return;
 	
@@ -1837,8 +1852,11 @@ void build_particle_system(Object *ob)
 	
 	if( paf->flag & PAF_STATIC ) deform= 0;
 	else {
-		deform= (ob->parent && ob->parent->type==OB_LATTICE && ob->partype==PARSKEL);
-		if(deform) init_latt_deform(ob->parent, 0);
+		Object *parlatt= modifiers_isDeformedByLattice(ob);
+		if(parlatt) {
+			deform= 1;
+			init_latt_deform(parlatt, 0);
+		}
 	}
 	
 	/* get the effectors */
@@ -1911,7 +1929,10 @@ void build_particle_system(Object *ob)
 						remainder += foweights[curface];
 						curface++;
 					}
-					foweights[curface] += remainder;
+					/* if this is the last face, the foweights[] can be zero, so we don't add a particle extra */
+					if(curface!=totface-1)
+						foweights[curface] += remainder;
+					
 					maxw= (paf->end-paf->sta)/foweights[curface];
 				}
 
