@@ -1,6 +1,6 @@
 /* ipo.c
  * 
- * $Id: ipo.c,v 1.40 2006/07/04 10:19:26 ton Exp $
+ * $Id: ipo.c,v 1.48 2007/01/15 12:44:45 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -164,8 +164,9 @@ int la_ar[LA_TOTIPO]= {
 };
 
 /* yafray: aperture & focal distance curves added */
+/* qdn: FDIST now available to Blender as well for defocus node */
 int cam_ar[CAM_TOTIPO]= {
-	CAM_LENS, CAM_STA, CAM_END, CAM_YF_APERT, CAM_YF_FDIST
+	CAM_LENS, CAM_STA, CAM_END, CAM_YF_APERT, CAM_YF_FDIST, CAM_SHIFT_X, CAM_SHIFT_Y
 };
 
 int snd_ar[SND_TOTIPO]= {
@@ -453,7 +454,9 @@ void calchandles_ipocurve(IpoCurve *icu)
 	int a;
 
 	a= icu->totvert;
-	if(a<2) return;
+	
+	/* IPO_CONST doesn't have handles */
+	if(a<2 || icu->ipo==IPO_CONST) return;
 	
 	bezt= icu->bezt;
 	prev= 0;
@@ -550,7 +553,7 @@ void sort_time_ipocurve(IpoCurve *icu)
 						ok= 1;
 					}
 				}
-				if(bezt->vec[0][0]>=bezt->vec[1][0] && bezt->vec[2][0]<=bezt->vec[1][0]) {
+				if(bezt->vec[0][0]>bezt->vec[1][0] && bezt->vec[2][0]<bezt->vec[1][0]) {
 					SWAP(float, bezt->vec[0][0], bezt->vec[2][0]);
 					SWAP(float, bezt->vec[0][1], bezt->vec[2][1]);
 				}
@@ -751,6 +754,8 @@ static float eval_driver(IpoDriver *driver, float ipotime)
 		Object *ob= driver->ob;
 
 		if(ob==NULL) return 0.0f;
+		if(ob->proxy_from)
+			ob= ob->proxy_from;
 		
 		if(driver->blocktype==ID_OB) {
 			/* depsgraph failure; ob ipos are calculated in where_is_object, this might get called too late */
@@ -1471,6 +1476,10 @@ void *get_ipo_poin(ID *id, IpoCurve *icu, int *type)
 			poin= &(ca->YF_aperture); break;
 		case CAM_YF_FDIST:
 			poin= &(ca->YF_dofdist); break;
+		case CAM_SHIFT_X:
+			poin= &(ca->shiftx); break;
+		case CAM_SHIFT_Y:
+			poin= &(ca->shifty); break;
 		}
 	}
 	else if(GS(id->name)==ID_SO) {
@@ -1752,6 +1761,13 @@ void set_icu_vars(IpoCurve *icu)
 		case CAM_YF_FDIST:
 			icu->ymin = 0.0;
 			icu->ymax = 5000.0;
+			break;
+			
+		case CAM_SHIFT_X:
+		case CAM_SHIFT_Y:
+			icu->ymin= -2.0f;
+			icu->ymax= 2.0f;
+			break;
 		}
 	}
 	else if(icu->blocktype==ID_SO) {
@@ -2206,7 +2222,6 @@ void add_to_cfra_elem(ListBase *lb, BezTriple *bezt)
 void make_cfra_list(Ipo *ipo, ListBase *elems)
 {
 	IpoCurve *icu;
-	CfraElem *ce;
 	BezTriple *bezt;
 	int a;
 	
@@ -2292,7 +2307,9 @@ void make_cfra_list(Ipo *ipo, ListBase *elems)
 			}
 		}
 	}
-	
+
+	/* what's the point of this little block of code?  */
+#if 0
 	if(ipo->showkey==0) {
 		/* deselect all keys */
 		ce= elems->first;
@@ -2301,6 +2318,7 @@ void make_cfra_list(Ipo *ipo, ListBase *elems)
 			ce= ce->next;
 		}
 	}
+#endif
 }
 
 /* *********************** INTERFACE FOR KETSJI ********** */
