@@ -1,5 +1,5 @@
 /** 
- * $Id: library.c,v 1.19 2006/07/07 17:08:03 ton Exp $
+ * $Id: library.c,v 1.28 2007/01/25 11:59:29 ton Exp $
  * 
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -77,6 +77,7 @@
 #include "DNA_node_types.h"
 #include "DNA_nla_types.h"
 #include "DNA_effect_types.h"
+#include "DNA_brush_types.h"
 
 #include "BLI_blenlib.h"
 #include "BLI_dynstr.h"
@@ -108,11 +109,12 @@
 #include "BKE_action.h"
 #include "BKE_node.h"
 #include "BKE_effect.h"
+#include "BKE_brush.h"
+#include "BKE_idprop.h"
 
 #include "BPI_script.h"
 
 #define MAX_IDPUP		60	/* was 24 */
-#define MAX_LIBARRAY	100 /* was 30, warning: used it readfile.c too */
 
 /* ************* general ************************ */
 
@@ -182,9 +184,6 @@ ListBase *wich_libbase(Main *mainlib, short type)
 			return &(mainlib->script);
 		case ID_SO:
 			return &(mainlib->sound);
-		case ID_SAMPLE:
-			/* declared as an external in sound.h !!! */
-			return (samples);
 		case ID_GR:
 			return &(mainlib->group);
 		case ID_AR:
@@ -193,10 +192,13 @@ ListBase *wich_libbase(Main *mainlib, short type)
 			return &(mainlib->action);
 		case ID_NT:
 			return &(mainlib->nodetree);
+		case ID_BR:
+			return &(mainlib->brush);
 	}
 	return 0;
 }
 
+/* note: MAX_LIBARRAY define should match this code */
 int set_listbasepointers(Main *main, ListBase **lb)
 {
 	/* BACKWARDS! also watch order of free-ing! (mesh<->mat) */
@@ -223,19 +225,20 @@ int set_listbasepointers(Main *main, ListBase **lb)
 	lb[12]= &(main->latt);
 	lb[13]= &(main->lamp);
 	lb[14]= &(main->camera);
-
-	lb[15]= &(main->world);
-	lb[16]= &(main->screen);
-	lb[17]= &(main->object);
-	lb[18]= &(main->scene);
-	lb[19]= &(main->library);
-	lb[20]= &(main->text);
-	lb[21]= &(main->sound);
-	lb[22]= &(main->group);
-	lb[23]= &(main->nodetree);
-
-	lb[24]= samples;
-	lb[25]= &(main->script);
+	
+	lb[15]= &(main->text);
+	lb[16]= &(main->sound);
+	lb[17]= &(main->group);
+	lb[18]= &(main->nodetree);
+	lb[19]= &(main->brush);
+	lb[20]= &(main->script);
+	
+	lb[21]= &(main->world);
+	lb[22]= &(main->screen);
+	lb[23]= &(main->object);
+	lb[24]= &(main->scene);
+	lb[25]= &(main->library);
+	
 	lb[26]= NULL;
 
 	return 26;
@@ -319,11 +322,8 @@ static ID *alloc_libblock_notest(short type)
 		case ID_SO:
 			id= MEM_callocN(sizeof(bSound), "sound");
 			break;
-		case ID_SAMPLE:
-			id = MEM_callocN(sizeof(bSample), "sound");
-			break;
 		case ID_GR:
-			id= MEM_callocN(sizeof(Group), "sound");
+			id= MEM_callocN(sizeof(Group), "group");
 			break;
 		case ID_AR:
 			id = MEM_callocN(sizeof(bArmature), "armature");
@@ -332,7 +332,10 @@ static ID *alloc_libblock_notest(short type)
 			id = MEM_callocN(sizeof(bAction), "action");
 			break;
 		case ID_NT:
-			id = MEM_callocN(sizeof(bNodeTree), "action");
+			id = MEM_callocN(sizeof(bNodeTree), "nodetree");
+			break;
+		case ID_BR:
+			id = MEM_callocN(sizeof(Brush), "brush");
 			break;
 	}
 	return id;
@@ -462,9 +465,6 @@ void free_libblock(ListBase *lb, void *idv)
 		case ID_SO:
 			sound_free_sound((bSound *)id);
 			break;
-		case ID_SAMPLE:
-			sound_free_sample((bSample *)id);
-			break;
 		case ID_GR:
 			free_group((Group *)id);
 			break;
@@ -477,8 +477,15 @@ void free_libblock(ListBase *lb, void *idv)
 		case ID_NT:
 			ntreeFreeTree((bNodeTree *)id);
 			break;
+		case ID_BR:
+			free_brush((Brush *)id);
+			break;
 	}
 
+	if (id->properties) {
+		IDP_FreeProperty(id->properties);
+		MEM_freeN(id->properties);
+	}
 	BLI_remlink(lb, id);
 	MEM_freeN(id);
 
@@ -533,7 +540,8 @@ ID *find_id(char *type, char *name)		/* type: "OB" or "MA" etc */
 	
 	id= lb->first;
 	while(id) {
-		if( strcmp(id->name+2, name)==0 ) return id;
+		if(id->name[2]==name[0] && strcmp(id->name+2, name)==0 ) 
+			return id;
 		id= id->next;
 	}
 	return 0;
@@ -563,7 +571,9 @@ static void get_flags_for_id(ID *id, char *buf)
 		sprintf(buf, "%c%c%c ", id->lib?'L':' ', isfake?'F':' ', (id->us==0)?'O':' ');
 }
 
-static void IDnames_to_dyn_pupstring(DynStr *pupds, ListBase *lb, ID *link, short *nr)
+#define IDPUP_NO_VIEWER 1
+
+static void IDnames_to_dyn_pupstring(DynStr *pupds, ListBase *lb, ID *link, short *nr, int hideflag)
 {
 	int i, nids= BLI_countlist(lb);
 		
@@ -582,6 +592,10 @@ static void IDnames_to_dyn_pupstring(DynStr *pupds, ListBase *lb, ID *link, shor
 
 			if (U.uiflag & USER_HIDE_DOT && id->name[2]=='.')
 				continue;
+			if (hideflag & IDPUP_NO_VIEWER)
+				if (GS(id->name)==ID_IM)
+					if ( ((Image *)id)->source==IMA_SRC_VIEWER )
+						continue;
 			
 			get_flags_for_id(id, buf);
 				
@@ -675,12 +689,35 @@ void IDnames_to_pupstring(char **str, char *title, char *extraops, ListBase *lb,
 			BLI_dynstr_append(pupds, "|");
 	}
 
-	IDnames_to_dyn_pupstring(pupds, lb, link, nr);
+	IDnames_to_dyn_pupstring(pupds, lb, link, nr, 0);
 	
 	*str= BLI_dynstr_get_cstring(pupds);
 	BLI_dynstr_free(pupds);
 }
 
+/* skips viewer images */
+void IMAnames_to_pupstring(char **str, char *title, char *extraops, ListBase *lb, ID *link, short *nr)
+{
+	DynStr *pupds= BLI_dynstr_new();
+	
+	if (title) {
+		BLI_dynstr_append(pupds, title);
+		BLI_dynstr_append(pupds, "%t|");
+	}
+	
+	if (extraops) {
+		BLI_dynstr_append(pupds, extraops);
+		if (BLI_dynstr_get_len(pupds))
+			BLI_dynstr_append(pupds, "|");
+	}
+	
+	IDnames_to_dyn_pupstring(pupds, lb, link, nr, IDPUP_NO_VIEWER);
+	
+	*str= BLI_dynstr_get_cstring(pupds);
+	BLI_dynstr_free(pupds);
+}
+
+
 // only used by headerbuttons.c
 void IPOnames_to_pupstring(char **str, char *title, char *extraops, ListBase *lb, ID *link, short *nr, int blocktype)
 {
@@ -757,7 +794,7 @@ int new_id(ListBase *lb, ID *id, const char *tname)
 {
 	ID *idtest;
 	int nr= 0, nrtest, maxtest=32, a;
-	char aname[32], *name, left[24], leftest[24], in_use[32];
+	char aname[32], *name, left[32], leftest[32], in_use[32];
 	
 	/* - split name
 	 * - search
@@ -794,7 +831,7 @@ int new_id(ListBase *lb, ID *id, const char *tname)
 
 	/* if there is no double return */
 	if(idtest==0) {
-		strcpy(id->name+2, name);
+		strncpy(id->name+2, name, 21);
 		return 0;
 	}
 	
@@ -829,7 +866,7 @@ int new_id(ListBase *lb, ID *id, const char *tname)
 		}
 	}
 	
-	if(nr==0) sprintf(id->name+2, "%s", left);
+	if(nr==0) strncpy(id->name+2, left, 21);
 	else {
 		if (nr >= 1000 && strlen(left) > 16) {
 			// this would overflow name buffer
@@ -903,6 +940,8 @@ static void lib_indirect_test_id(ID *id)
 			LIBTAG(paf->group);
 	
 		LIBTAG(ob->dup_group);
+		LIBTAG(ob->proxy);
+		
 		me= ob->data;
 		LIBTAG(me);
 	}
