@@ -2,7 +2,7 @@
 /*  material.c
  *
  * 
- * $Id: material.c,v 1.26 2005/04/17 17:43:06 ton Exp $
+ * $Id: material.c,v 1.55 2007/02/11 14:59:01 halley Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -34,35 +34,50 @@
  */
 
 #include <string.h>
+#include <math.h>
+
 #include "MEM_guardedalloc.h"
 
-#include "DNA_material_types.h"
-#include "DNA_texture_types.h"
-#include "DNA_mesh_types.h"
-#include "DNA_object_types.h"
 #include "DNA_curve_types.h"
+#include "DNA_material_types.h"
+#include "DNA_mesh_types.h"
 #include "DNA_meta_types.h"
+#include "DNA_node_types.h"
+#include "DNA_object_types.h"
 #include "DNA_scene_types.h"
+#include "DNA_texture_types.h"
 
 #include "BLI_blenlib.h"
+#include "BLI_arithb.h"		
 
 #include "BKE_bad_level_calls.h"
-#include "BKE_utildefines.h"
-
-#include "BKE_global.h"
-#include "BKE_main.h"
-
-#include "BKE_mesh.h"
-#include "BKE_library.h"
+#include "BKE_blender.h"
 #include "BKE_displist.h"
+#include "BKE_global.h"
+#include "BKE_icons.h"
+#include "BKE_library.h"
+#include "BKE_main.h"
 #include "BKE_material.h"
+#include "BKE_mesh.h"
+#include "BKE_node.h"
+#include "BKE_utildefines.h"
 
 #include "BPY_extern.h"
 
+/* used in UI and render */
+Material defmaterial;
+
+/* called on startup, creator.c */
+void init_def_material(void)
+{
+	init_material(&defmaterial);
+}
+
+/* not material itself */
 void free_material(Material *ma)
 {
-	int a;
 	MTex *mtex;
+	int a;
 
 	BPY_free_scriptlink(&ma->scriptlink);
 	
@@ -74,11 +89,19 @@ void free_material(Material *ma)
 	
 	if(ma->ramp_col) MEM_freeN(ma->ramp_col);
 	if(ma->ramp_spec) MEM_freeN(ma->ramp_spec);
+	
+	BKE_icon_delete((struct ID*)ma);
+	ma->id.icon_id = 0;
+	
+	/* is no lib link block, but material extension */
+	if(ma->nodetree) {
+		ntreeFreeTree(ma->nodetree);
+		MEM_freeN(ma->nodetree);
+	}
 }
 
 void init_material(Material *ma)
 {
-	ma->lay= 1;
 	ma->r= ma->g= ma->b= ma->ref= 0.8;
 	ma->specr= ma->specg= ma->specb= 1.0;
 	ma->mirr= ma->mirg= ma->mirb= 1.0;
@@ -100,8 +123,10 @@ void init_material(Material *ma)
 	ma->param[1]= 0.1;
 	ma->param[2]= 0.5;
 	ma->param[3]= 0.1;
-	ma->rms=0.1;
-	ma->darkness=1.0;	
+	ma->rms= 0.1;
+	ma->darkness= 1.0;	
+	
+	ma->strand_sta= ma->strand_end= 1.0f;
 	
 	ma->ang= 1.0;
 	ma->ray_depth= 2;
@@ -110,12 +135,16 @@ void init_material(Material *ma)
 	ma->fresnel_tra= 0.0;
 	ma->fresnel_tra_i= 1.25;
 	ma->fresnel_mir_i= 1.25;
+	ma->tx_limit= 0.0;
+	ma->tx_falloff= 1.0;
+	ma->shad_alpha= 1.0f;
 	
 	ma->rampfac_col= 1.0;
 	ma->rampfac_spec= 1.0;
-	ma->pr_lamp= 3; // two lamps, is bits
-	
-	ma->mode= MA_TRACEBLE+MA_SHADOW+MA_RADIO;	
+	ma->pr_lamp= 3;			/* two lamps, is bits */
+	ma->pr_type= MA_SPHERE;
+
+	ma->mode= MA_TRACEBLE|MA_SHADBUF|MA_SHADOW|MA_RADIO|MA_RAYBIAS|MA_TANGENT_STR;
 }
 
 Material *add_material(char *name)
@@ -147,9 +176,14 @@ Material *copy_material(Material *ma)
 	}
 	
 	BPY_copy_scriptlink(&ma->scriptlink);
+	
 	if(ma->ramp_col) man->ramp_col= MEM_dupallocN(ma->ramp_col);
 	if(ma->ramp_spec) man->ramp_spec= MEM_dupallocN(ma->ramp_spec);
 	
+	if(ma->nodetree) {
+		man->nodetree= ntreeCopyTree(ma->nodetree, 0);	/* 0 == full new tree */
+	}
+	
 	return man;
 }
 
@@ -243,6 +277,7 @@ void make_local_material(Material *ma)
 		new_id(0, (ID *)ma, 0);
 	}
 	else if(local && lib) {
+		
 		man= copy_material(ma);
 		man->id.us= 0;
 		
@@ -359,16 +394,23 @@ Material *give_current_material(Object *ob, int act)
 {
 	Material ***matarar, *ma;
 	
-	if(ob==0) return 0;
+	if(ob==NULL) return 0;
 	if(ob->totcol==0) return 0;
 	
 	if(act>ob->totcol) act= ob->totcol;
-	else if(act==0) act= 1;
+	else if(act<=0) act= 1;
 
 	if( BTST(ob->colbits, act-1) ) {	/* in object */
 		ma= ob->mat[act-1];
 	}
 	else {								/* in data */
+		short *totcolp= give_totcolp(ob);
+
+		/* check for inconsistancy */
+		if(*totcolp < ob->totcol)
+			ob->totcol= *totcolp;
+		if(act>ob->totcol) act= ob->totcol;
+
 		matarar= give_matarar(ob);
 		
 		if(matarar && *matarar) ma= (*matarar)[act-1];
@@ -532,8 +574,7 @@ void new_material_to_objectdata(Object *ob)
 	ob->actcol= ob->totcol;
 }
 
-
-void init_render_material(Material *ma)
+static void do_init_render_material(Material *ma, int r_mode, float *amb)
 {
 	MTex *mtex;
 	int a, needuv=0;
@@ -544,81 +585,115 @@ void init_render_material(Material *ma)
 	ma->texco= 0;
 	ma->mapto= 0;
 	for(a=0; a<MAX_MTEX; a++) {
+		
+		/* separate tex switching */
+		if(ma->septex & (1<<a)) continue;
+
 		mtex= ma->mtex[a];
-		if(mtex && mtex->tex) {
+		if(mtex && mtex->tex && mtex->tex->type) {
 			
 			ma->texco |= mtex->texco;
 			ma->mapto |= mtex->mapto;
-			if(R.osa) {
+			if(r_mode & R_OSA) {
 				if ELEM3(mtex->tex->type, TEX_IMAGE, TEX_PLUGIN, TEX_ENVMAP) ma->texco |= TEXCO_OSA;
 			}
 			
-			if(ma->texco & (511)) needuv= 1;
-			
-			if(mtex->object) mtex->object->flag |= OB_DO_IMAT;
-			
+			if(ma->texco & (TEXCO_ORCO|TEXCO_REFL|TEXCO_NORM|TEXCO_STRAND|TEXCO_STRESS)) needuv= 1;
+			else if(ma->texco & (TEXCO_GLOB|TEXCO_UV|TEXCO_OBJECT|TEXCO_SPEED)) needuv= 1;
+			else if(ma->texco & (TEXCO_LAVECTOR|TEXCO_VIEW|TEXCO_STICKY)) needuv= 1;
 		}
 	}
-	if(ma->mode & MA_ZTRA) {
-		/* if(ma->alpha==0.0 || ma->alpha==1.0) */
-		if(R.flag & R_RENDERING) R.flag |= R_ZTRA;
-	}
-	if(ma->mode & MA_VERTEXCOLP) ma->mode |= MA_VERTEXCOL; 
 	
-	if(ma->mode & MA_RADIO) needuv= 1;
+	if(r_mode & R_RADIO)
+		if(ma->mode & MA_RADIO) needuv= 1;
 	
-	if(ma->mode & (MA_VERTEXCOL|MA_FACETEXTURE)) {
+	if(ma->mode & (MA_VERTEXCOL|MA_VERTEXCOLP|MA_FACETEXTURE)) {
 		needuv= 1;
-		if(R.osa) ma->texco |= TEXCO_OSA;		/* for texfaces */
+		if(r_mode & R_OSA) ma->texco |= TEXCO_OSA;		/* for texfaces */
 	}
 	if(needuv) ma->texco |= NEED_UV;
 	
-	// since the raytracer doesnt recalc O structs for each ray, we have to preset them all
-	if(ma->mode & (MA_RAYMIRROR|MA_RAYTRANSP|MA_SHADOW_TRA)) { 
-		ma->texco |= NEED_UV|TEXCO_ORCO|TEXCO_REFL|TEXCO_NORM;
-		if(R.osa) ma->texco |= TEXCO_OSA;
-	}
-
-	ma->ambr= ma->amb*R.wrld.ambr;
-	ma->ambg= ma->amb*R.wrld.ambg;
-	ma->ambb= ma->amb*R.wrld.ambb;
-	
-}
-
-void init_render_materials()
-{
-	Material *ma;
-	
-	ma= G.main->mat.first;
-	while(ma) {
-		if(ma->id.us) init_render_material(ma);
-		ma= ma->id.next;
-	}
-	
-}
-
-void end_render_material(Material *ma)
-{
-	/* XXXX obsolete? check! */
-	if(ma->mode & (MA_VERTEXCOLP|MA_FACETEXTURE)) {
-		if( !(ma->mode & MA_HALO) ) {
-			ma->r= ma->g= ma->b= 1.0;
+	/* since the raytracer doesnt recalc O structs for each ray, we have to preset them all */
+	if(r_mode & R_RAYTRACE) {
+		if(ma->mode & (MA_RAYMIRROR|MA_RAYTRANSP|MA_SHADOW_TRA)) { 
+			ma->texco |= NEED_UV|TEXCO_ORCO|TEXCO_REFL|TEXCO_NORM;
+			if(r_mode & R_OSA) ma->texco |= TEXCO_OSA;
 		}
 	}
+	
+	if(amb) {
+		ma->ambr= ma->amb*amb[0];
+		ma->ambg= ma->amb*amb[1];
+		ma->ambb= ma->amb*amb[2];
+	}	
+	/* will become or-ed result of all node modes */
+	ma->mode_l= ma->mode;
+	ma->mode_l &= ~MA_SHLESS;
 }
 
-void end_render_materials()
+static void init_render_nodetree(bNodeTree *ntree, Material *basemat, int r_mode, float *amb)
+{
+	bNode *node;
+	
+	for(node=ntree->nodes.first; node; node= node->next) {
+		if(node->id) {
+			if(GS(node->id->name)==ID_MA) {
+				Material *ma= (Material *)node->id;
+				if(ma!=basemat) {
+					do_init_render_material(ma, r_mode, amb);
+					basemat->texco |= ma->texco;
+					basemat->mode_l |= ma->mode_l;
+				}
+			}
+			else if(node->type==NODE_GROUP)
+				init_render_nodetree((bNodeTree *)node->id, basemat, r_mode, amb);
+		}
+	}
+	/* parses the geom+tex nodes */
+	ntreeShaderGetTexcoMode(ntree, r_mode, &basemat->texco, &basemat->mode_l);
+}
+
+void init_render_material(Material *mat, int r_mode, float *amb)
+{
+	
+	do_init_render_material(mat, r_mode, amb);
+	
+	if(mat->nodetree && mat->use_nodes) {
+		init_render_nodetree(mat->nodetree, mat, r_mode, amb);
+		
+		ntreeBeginExecTree(mat->nodetree); /* has internal flag to detect it only does it once */
+	}
+}
+
+void init_render_materials(int r_mode, float *amb)
 {
 	Material *ma;
 	
-	ma= G.main->mat.first;
-	while(ma) {
-		if(ma->id.us) end_render_material(ma);
-		ma= ma->id.next;
+	/* two steps, first initialize, then or the flags for layers */
+	for(ma= G.main->mat.first; ma; ma= ma->id.next) {
+		/* is_used flag comes back in convertblender.c */
+		ma->flag &= ~MA_IS_USED;
+		if(ma->id.us) 
+			init_render_material(ma, r_mode, amb);
 	}
 	
+	do_init_render_material(&defmaterial, r_mode, amb);
 }
 
+/* only needed for nodes now */
+void end_render_material(Material *mat)
+{
+	if(mat && mat->nodetree && mat->use_nodes)
+		ntreeEndExecTree(mat->nodetree); /* has internal flag to detect it only does it once */
+}
+
+void end_render_materials(void)
+{
+	Material *ma;
+	for(ma= G.main->mat.first; ma; ma= ma->id.next)
+		if(ma->id.us) 
+			end_render_material(ma);
+}
 
 /* ****************** */
 
@@ -641,7 +716,7 @@ char colname_array[125][20]= {
 "LightBlue","Purple","MediumOrchid","Magenta","Magenta",
 "RoyalBlue","SlateBlue","MediumOrchid","Orchid","Magenta",
 "DeepSkyBlue","LightSteelBlue","LightSkyBlue","Violet","LightPink",
-"Cyaan","DarkTurquoise","SkyBlue","Grey","Snow",
+"Cyan","DarkTurquoise","SkyBlue","Grey","Snow",
 "Mint","Mint","Aquamarine","MintCream","Ivory",
 "Blue","Blue","DarkMagenta","DarkOrchid","Magenta",
 "SkyBlue","RoyalBlue","LightSlateBlue","MediumOrchid","Magenta",
@@ -737,16 +812,240 @@ void delete_material_index()
 	if(ob->type==OB_MESH) {
 		Mesh *me= get_mesh(ob);
 		mesh_delete_material_index(me, actcol-1);
-		makeDispList(ob);
+		freedisplist(&ob->disp);
 	}
 	else if ELEM(ob->type, OB_CURVE, OB_SURF) {
 		cu= ob->data;
 		nu= cu->nurb.first;
 		
 		while(nu) {
-			if(nu->mat_nr && nu->mat_nr>=actcol-1) nu->mat_nr--;
+			if(nu->mat_nr && nu->mat_nr>=actcol-1) {
+				nu->mat_nr--;
+				if (ob->type == OB_CURVE) nu->charidx--;
+			}
 			nu= nu->next;
 		}
-		makeDispList(ob);
+		freedisplist(&ob->disp);
 	}
 }
+
+
+/* r g b = current value, col = new value, fac==0 is no change */
+/* if g==NULL, it only does r channel */
+void ramp_blend(int type, float *r, float *g, float *b, float fac, float *col)
+{
+	float tmp, facm= 1.0-fac;
+	
+	switch (type) {
+		case MA_RAMP_BLEND:
+			*r = facm*(*r) + fac*col[0];
+			if(g) {
+				*g = facm*(*g) + fac*col[1];
+				*b = facm*(*b) + fac*col[2];
+			}
+				break;
+		case MA_RAMP_ADD:
+			*r += fac*col[0];
+			if(g) {
+				*g += fac*col[1];
+				*b += fac*col[2];
+			}
+				break;
+		case MA_RAMP_MULT:
+			*r *= (facm + fac*col[0]);
+			if(g) {
+				*g *= (facm + fac*col[1]);
+				*b *= (facm + fac*col[2]);
+			}
+				break;
+		case MA_RAMP_SCREEN:
+			*r = 1.0 - (facm + fac*(1.0 - col[0])) * (1.0 - *r);
+			if(g) {
+				*g = 1.0 - (facm + fac*(1.0 - col[1])) * (1.0 - *g);
+				*b = 1.0 - (facm + fac*(1.0 - col[2])) * (1.0 - *b);
+			}
+				break;
+		case MA_RAMP_OVERLAY:
+			if(*r < 0.5f)
+				*r *= (facm + 2.0f*fac*col[0]);
+			else
+				*r = 1.0 - (facm + 2.0f*fac*(1.0 - col[0])) * (1.0 - *r);
+			if(g) {
+				if(*g < 0.5f)
+					*g *= (facm + 2.0f*fac*col[1]);
+				else
+					*g = 1.0 - (facm + 2.0f*fac*(1.0 - col[1])) * (1.0 - *g);
+				if(*b < 0.5f)
+					*b *= (facm + 2.0f*fac*col[2]);
+				else
+					*b = 1.0 - (facm + 2.0f*fac*(1.0 - col[2])) * (1.0 - *b);
+			}
+				break;
+		case MA_RAMP_SUB:
+			*r -= fac*col[0];
+			if(g) {
+				*g -= fac*col[1];
+				*b -= fac*col[2];
+			}
+				break;
+		case MA_RAMP_DIV:
+			if(col[0]!=0.0)
+				*r = facm*(*r) + fac*(*r)/col[0];
+			if(g) {
+				if(col[1]!=0.0)
+					*g = facm*(*g) + fac*(*g)/col[1];
+				if(col[2]!=0.0)
+					*b = facm*(*b) + fac*(*b)/col[2];
+			}
+				break;
+		case MA_RAMP_DIFF:
+			*r = facm*(*r) + fac*fabs(*r-col[0]);
+			if(g) {
+				*g = facm*(*g) + fac*fabs(*g-col[1]);
+				*b = facm*(*b) + fac*fabs(*b-col[2]);
+			}
+				break;
+		case MA_RAMP_DARK:
+			tmp= fac*col[0];
+			if(tmp < *r) *r= tmp; 
+				if(g) {
+					tmp= fac*col[1];
+					if(tmp < *g) *g= tmp; 
+					tmp= fac*col[2];
+					if(tmp < *b) *b= tmp; 
+				}
+					break;
+		case MA_RAMP_LIGHT:
+			tmp= fac*col[0];
+			if(tmp > *r) *r= tmp; 
+				if(g) {
+					tmp= fac*col[1];
+					if(tmp > *g) *g= tmp; 
+					tmp= fac*col[2];
+					if(tmp > *b) *b= tmp; 
+				}
+					break;	
+		case MA_RAMP_DODGE:			
+			
+				
+			if(*r !=0.0){
+				tmp = 1.0 - fac*col[0];
+				if(tmp <= 0.0)
+					*r = 1.0;
+				else if ((tmp = (*r) / tmp)> 1.0)
+					*r = 1.0;
+				else 
+					*r = tmp;
+			}
+			if(g) {
+				if(*g !=0.0){
+					tmp = 1.0 - fac*col[1];
+					if(tmp <= 0.0 )
+						*g = 1.0;
+					else if ((tmp = (*g) / tmp) > 1.0 )
+						*g = 1.0;
+					else
+						*g = tmp;
+				}
+				if(*b !=0.0){
+					tmp = 1.0 - fac*col[2];
+					if(tmp <= 0.0)
+						*b = 1.0;
+					else if ((tmp = (*b) / tmp) > 1.0 )
+						*b = 1.0;
+					else
+						*b = tmp;
+				}
+
+			}
+				break;	
+		case MA_RAMP_BURN:
+			
+			tmp = facm + fac*col[0];
+			
+			if(tmp <= 0.0)
+				*r = 0.0;
+			else if (( tmp = (1.0 - (1.0 - (*r)) / tmp )) < 0.0)
+			        *r = 0.0;
+			else if (tmp > 1.0)
+				*r=1.0;
+			else 
+				*r = tmp; 
+
+			if(g) {
+				tmp = facm + fac*col[1];
+				if(tmp <= 0.0)
+					*g = 0.0;
+				else if (( tmp = (1.0 - (1.0 - (*g)) / tmp )) < 0.0 )
+			        	*g = 0.0;
+				else if(tmp >1.0)
+					*g=1.0;
+				else
+					*g = tmp;
+			        	
+			        tmp = facm + fac*col[2];
+			        if(tmp <= 0.0)
+					*b = 0.0;
+				else if (( tmp = (1.0 - (1.0 - (*b)) / tmp )) < 0.0  )
+			        	*b = 0.0;
+				else if(tmp >1.0)
+					*b= 1.0;
+				else
+					*b = tmp;
+			}
+				break;
+		case MA_RAMP_HUE:		
+			if(g){
+				float rH,rS,rV;
+				float colH,colS,colV; 
+				float tmpr,tmpg,tmpb;
+				rgb_to_hsv(col[0],col[1],col[2],&colH,&colS,&colV);
+				if(colS!=0 ){
+					rgb_to_hsv(*r,*g,*b,&rH,&rS,&rV);
+					hsv_to_rgb( colH , rS, rV, &tmpr, &tmpg, &tmpb);
+					*r = facm*(*r) + fac*tmpr;  
+					*g = facm*(*g) + fac*tmpg; 
+					*b = facm*(*b) + fac*tmpb;
+				}
+			}
+				break;
+		case MA_RAMP_SAT:		
+			if(g){
+				float rH,rS,rV;
+				float colH,colS,colV;
+				rgb_to_hsv(*r,*g,*b,&rH,&rS,&rV);
+				if(rS!=0){
+					rgb_to_hsv(col[0],col[1],col[2],&colH,&colS,&colV);
+					hsv_to_rgb( rH, (facm*rS +fac*colS), rV, r, g, b);
+				}
+			}
+				break;
+		case MA_RAMP_VAL:		
+			if(g){
+				float rH,rS,rV;
+				float colH,colS,colV;
+				rgb_to_hsv(*r,*g,*b,&rH,&rS,&rV);
+				rgb_to_hsv(col[0],col[1],col[2],&colH,&colS,&colV);
+				hsv_to_rgb( rH, rS, (facm*rV +fac*colV), r, g, b);
+			}
+				break;
+		case MA_RAMP_COLOR:		
+			if(g){
+				float rH,rS,rV;
+				float colH,colS,colV;
+				float tmpr,tmpg,tmpb;
+				rgb_to_hsv(col[0],col[1],col[2],&colH,&colS,&colV);
+				if(colS!=0){
+					rgb_to_hsv(*r,*g,*b,&rH,&rS,&rV);
+					hsv_to_rgb( colH, colS, rV, &tmpr, &tmpg, &tmpb);
+					*r = facm*(*r) + fac*tmpr;
+					*g = facm*(*g) + fac*tmpg;
+					*b = facm*(*b) + fac*tmpb;
+				}
+			}
+				break;
+	}
+	
+}
+
+
