@@ -6,7 +6,7 @@
  *  
  * texture coordinates are patched within the displist
  * 
- * $Id: mball.c,v 1.22 2005/06/04 16:22:49 lukep Exp $
+ * $Id: mball.c,v 1.30 2006/11/20 21:25:01 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -30,8 +30,6 @@
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
  *
- * The Original Code is: all of this file.
- *
  * Contributor(s): Jiri Hnidek <jiri.hnidek@vslib.cz>.
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
@@ -65,6 +63,7 @@
 #include "BKE_library.h"
 #include "BKE_displist.h"
 #include "BKE_mball.h"
+#include "BKE_object.h"
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -217,29 +216,20 @@ void tex_space_mball(Object *ob)
 		dl= dl->next;
 	}
 
-	if(doit) {
-		loc[0]= (min[0]+max[0])/2.0f;
-		loc[1]= (min[1]+max[1])/2.0f;
-		loc[2]= (min[2]+max[2])/2.0f;
-		
-		size[0]= (max[0]-min[0])/2.0f;
-		size[1]= (max[1]-min[1])/2.0f;
-		size[2]= (max[2]-min[2])/2.0f;
-	}
-	else {
-		loc[0]= loc[1]= loc[2]= 0.0f;
-		size[0]= size[1]= size[2]= 1.0f;
+	if(!doit) {
+		min[0] = min[1] = min[2] = -1.0f;
+		max[0] = max[1] = max[2] = 1.0f;
 	}
 	
-	bb->vec[0][0]=bb->vec[1][0]=bb->vec[2][0]=bb->vec[3][0]= loc[0]-size[0];
-	bb->vec[4][0]=bb->vec[5][0]=bb->vec[6][0]=bb->vec[7][0]= loc[0]+size[0];
+	loc[0]= (min[0]+max[0])/2.0f;
+	loc[1]= (min[1]+max[1])/2.0f;
+	loc[2]= (min[2]+max[2])/2.0f;
 	
-	bb->vec[0][1]=bb->vec[1][1]=bb->vec[4][1]=bb->vec[5][1]= loc[1]-size[1];
-	bb->vec[2][1]=bb->vec[3][1]=bb->vec[6][1]=bb->vec[7][1]= loc[1]+size[1];
+	size[0]= (max[0]-min[0])/2.0f;
+	size[1]= (max[1]-min[1])/2.0f;
+	size[2]= (max[2]-min[2])/2.0f;
 
-	bb->vec[0][2]=bb->vec[3][2]=bb->vec[4][2]=bb->vec[7][2]= loc[2]-size[2];
-	bb->vec[1][2]=bb->vec[2][2]=bb->vec[5][2]=bb->vec[6][2]= loc[2]+size[2];
-	
+	boundbox_set_from_min_max(bb, min, max);
 }
 
 void make_orco_mball(Object *ob)
@@ -311,6 +301,8 @@ Object *find_basis_mball(Object *basis)
 				/* if bob object is in edit mode, then dynamic list of all MetaElems
 				 * is stored in editelems */
 				if(ob==G.obedit) ml= editelems.first;
+				/* keep track of linked data too! */
+				else if(G.obedit && G.obedit->data==ob->data) ml= editelems.first;
 				/* if bob object is in object mode */
 				else ml= ((MetaBall*)ob->data)->elems.first;
 			}
@@ -579,7 +571,7 @@ float metaball(float x, float y, float z)
 
 /* ******************************************** */
 
-int *indices=0;
+int *indices=NULL;
 int totindex, curindex;
 
 
@@ -601,25 +593,15 @@ void accum_mballfaces(int i1, int i2, int i3, int i4)
 	
 	cur= indices+4*curindex;
 
-	/* prevent zero codes for faces indices */
-	if(i3==0) {
-		if(i4) {
-			i3= i4;
-			i4= i1;
-			i1= i2;
-			i2= 0;
-		}
-		else {
-			i3= i2;
-			i2= i1;
-			i1= 0;
-		}
-	}
+	/* diplists now support array drawing, we treat trias as fake quad */
 	
 	cur[0]= i1;
 	cur[1]= i2;
 	cur[2]= i3;
-	cur[3]= i4;
+	if(i4==0)
+		cur[3]= i3;
+	else 
+		cur[3]= i4;
 	
 	curindex++;
 
@@ -1114,7 +1096,6 @@ void vnormal (MB_POINT *point, PROCESS *p, MB_POINT *v)
 	}
 	
 	if(FALSE) {
-	/* if(R.flag & R_RENDERING) { */
 		MB_POINT temp;
 		
 		delta*= 2.0;
@@ -1445,11 +1426,12 @@ float init_meta(Object *ob)	/* return totsize */
 	Object *bob;
 	MetaBall *mb;
 	MetaElem *ml;
-	float size, totsize, (*mat)[4] = NULL, (*imat)[4] = NULL, obinv[4][4], vec[3];
+	float size, totsize, (*mat)[4] = NULL, (*imat)[4] = NULL, obinv[4][4], obmat[4][4], vec[3];
 	float temp1[4][4], temp2[4][4], temp3[4][4]; //max=0.0;
-	int a, obnr;
+	int a, obnr, zero_size=0;
 	char obname[32];
 	
+	Mat4CpyMat4(obmat, ob->obmat);	/* to cope with duplicators from next_object */
 	Mat4Invert(obinv, ob->obmat);
 	a= 0;
 	
@@ -1461,8 +1443,10 @@ float init_meta(Object *ob)	/* return totsize */
 	while(next_object(1, &base, &bob)) {
 
 		if(bob->type==OB_MBALL) {
-			ml= 0;
-			if(bob==ob) {
+			zero_size= 0;
+			ml= NULL;
+
+			if(bob==ob && (base->flag & OB_FROMDUPLI)==0) {
 				mat= imat= 0;
 				mb= ob->data;
 	
@@ -1477,12 +1461,37 @@ float init_meta(Object *ob)	/* return totsize */
 				splitIDname(bob->id.name+2, name, &nr);
 				if( strcmp(obname, name)==0 ) {
 					mb= bob->data;
-					
 					if(G.obedit && G.obedit->type==OB_MBALL && G.obedit->data==mb) 
 						ml= editelems.first;
 					else ml= mb->elems.first;
 				}
 			}
+
+			/* when metaball object hase zero scale, then MetaElem ot this MetaBall
+			 * will not be put to mainb array */
+			if(bob->size[0]==0.0 || bob->size[1]==0.0 || bob->size[2]==0.0) {
+				zero_size= 1;
+			}
+			else if(bob->parent) {
+				struct Object *pob=bob->parent;
+				while(pob) {
+					if(pob->size[0]==0.0 || pob->size[1]==0.0 || pob->size[2]==0.0) {
+						zero_size= 1;
+						break;
+					}
+					pob= pob->parent;
+				}
+			}
+
+			if (zero_size) {
+				unsigned int ml_count=0;
+				while(ml) {
+					ml_count++;
+					ml= ml->next;
+				}
+				totelem -= ml_count;
+			}
+			else {
 			while(ml) {
 				if(!(ml->flag & MB_HIDE)) {
 					int i;
@@ -1515,7 +1524,7 @@ float init_meta(Object *ob)	/* return totsize */
 					imat= new_pgn_element(4*4*sizeof(float));
 					
 					/* mat is the matrix to transform from mball into the basis-mball */
-					Mat4Invert(obinv, ob->obmat);
+					Mat4Invert(obinv, obmat);
 					Mat4MulMat4(temp2, bob->obmat, obinv);
 					/* MetaBall transformation */
 					Mat4MulMat4(mat, temp1, temp2);
@@ -1591,6 +1600,7 @@ float init_meta(Object *ob)	/* return totsize */
 				}
 				ml= ml->next;
 			}
+			}
 		}
 	}
 
@@ -1599,9 +1609,9 @@ float init_meta(Object *ob)	/* return totsize */
 	totsize= 0.0;
 	for(a=0; a<totelem; a++) {
 		
-		vec[0]= mainb[a]->x + mainb[a]->rad;
-		vec[1]= mainb[a]->y + mainb[a]->rad;
-		vec[2]= mainb[a]->z + mainb[a]->rad;
+		vec[0]= mainb[a]->x + mainb[a]->rad + mainb[a]->expx;
+		vec[1]= mainb[a]->y + mainb[a]->rad + mainb[a]->expy;
+		vec[2]= mainb[a]->z + mainb[a]->rad + mainb[a]->expz;
 
 		calc_mballco(mainb[a], vec);
 	
@@ -1981,7 +1991,7 @@ void metaball_polygonize(Object *ob)
 	mb= ob->data;
 
 	if(totelem==0) return;
-	if(!(R.flag & R_RENDERING) && (mb->flag==MB_UPDATE_NEVER)) return;
+	if(!(G.rendering) && (mb->flag==MB_UPDATE_NEVER)) return;
 	if(G.moving && mb->flag==MB_UPDATE_FAST) return;
 
 	freedisplist(&ob->disp);
@@ -2004,11 +2014,22 @@ void metaball_polygonize(Object *ob)
 	if((totelem > 1) && (totelem <= 64)) init_metaball_octal_tree(1);
 	if((totelem > 64) && (totelem <= 128)) init_metaball_octal_tree(2);
 	if((totelem > 128) && (totelem <= 512))	init_metaball_octal_tree(3);
-	if((totelem > 512) && (totelem <= 1024))init_metaball_octal_tree(4);
+	if((totelem > 512) && (totelem <= 1024)) init_metaball_octal_tree(4);
 	if(totelem > 1024) init_metaball_octal_tree(5);
 
+	/* don't polygonize metaballs with too high resolution (base mball to small) */
+	if(metaball_tree) {
+		if(ob->size[0]<=0.0001f*(metaball_tree->first->x_max - metaball_tree->first->x_min) ||
+		       ob->size[1]<=0.0001f*(metaball_tree->first->y_max - metaball_tree->first->y_min) ||
+		       ob->size[2]<=0.0001f*(metaball_tree->first->z_max - metaball_tree->first->z_min))
+		{
+			MEM_freeN(mainb);
+			return;
+		}
+	}
+
 	/* width is size per polygonize cube */
-	if(R.flag & R_RENDERING) width= mb->rendersize;
+	if(G.rendering) width= mb->rendersize;
 	else {
 		width= mb->wiresize;
 		if(G.moving && mb->flag==MB_UPDATE_HALFRES) width*= 2;
