@@ -3,7 +3,7 @@
  *
  *  
  * 
- * $Id: mesh.c,v 1.25 2005/04/04 12:22:33 zuster Exp $
+ * $Id: mesh.c,v 1.81 2007/02/09 15:09:55 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -53,7 +53,12 @@
 #include "DNA_key_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
+#include "DNA_ipo_types.h"
 
+#include "BDR_sculptmode.h"
+
+#include "BKE_customdata.h"
+#include "BKE_depsgraph.h"
 #include "BKE_main.h"
 #include "BKE_DerivedMesh.h"
 #include "BKE_global.h"
@@ -70,13 +75,17 @@
 #include "BKE_utildefines.h"
 #include "BKE_bad_level_calls.h"
 
+#ifdef WITH_VERSE
+#include "BKE_verse.h"
+#endif
+
 #include "BLI_blenlib.h"
 #include "BLI_editVert.h"
 #include "BLI_arithb.h"
 
+#include "multires.h"
 
-
-int update_realtime_texture(TFace *tface, double time)
+int update_realtime_texture(MTFace *tface, double time)
 {
 	Image *ima;
 	int	inc = 0;
@@ -115,6 +124,24 @@ int update_realtime_texture(TFace *tface, double time)
 	return inc;
 }
 
+void mesh_update_customdata_pointers(Mesh *me)
+{
+	me->mvert = CustomData_get_layer(&me->vdata, CD_MVERT);
+	me->dvert = CustomData_get_layer(&me->vdata, CD_MDEFORMVERT);
+	me->msticky = CustomData_get_layer(&me->vdata, CD_MSTICKY);
+
+	me->medge = CustomData_get_layer(&me->edata, CD_MEDGE);
+
+	me->mface = CustomData_get_layer(&me->fdata, CD_MFACE);
+	me->mcol = CustomData_get_layer(&me->fdata, CD_MCOL);
+	me->mtface = CustomData_get_layer(&me->fdata, CD_MTFACE);
+}
+
+/* Note: unlinking is called when me->id.us is 0, question remains how
+ * much unlinking of Library data in Mesh should be done... probably
+ * we need a more generic method, like the expand() functions in
+ * readfile.c */
+
 void unlink_mesh(Mesh *me)
 {
 	int a;
@@ -125,7 +152,12 @@ void unlink_mesh(Mesh *me)
 		if(me->mat[a]) me->mat[a]->id.us--;
 		me->mat[a]= 0;
 	}
-	if(me->key) me->key->id.us--;
+
+	if(me->key) {
+	   	me->key->id.us--;
+		if (me->key->id.us == 0 && me->key->ipo )
+			me->key->ipo->id.us--;
+	}
 	me->key= 0;
 	
 	if(me->texcomesh) me->texcomesh= 0;
@@ -135,24 +167,29 @@ void unlink_mesh(Mesh *me)
 /* do not free mesh itself */
 void free_mesh(Mesh *me)
 {
-
 	unlink_mesh(me);
 
-	if(me->mvert) MEM_freeN(me->mvert);
-	if(me->medge) MEM_freeN(me->medge);
-	if(me->mface) MEM_freeN(me->mface);
-	
-	if(me->tface) MEM_freeN(me->tface);
-	if(me->dvert) free_dverts(me->dvert, me->totvert);
-	if(me->mcol) MEM_freeN(me->mcol);
-	if(me->msticky) MEM_freeN(me->msticky);
+	if(me->pv) {
+		if(me->pv->vert_map) MEM_freeN(me->pv->vert_map);
+		if(me->pv->edge_map) MEM_freeN(me->pv->edge_map);
+		if(me->pv->old_faces) MEM_freeN(me->pv->old_faces);
+		if(me->pv->old_edges) MEM_freeN(me->pv->old_edges);
+		me->totvert= me->pv->totvert;
+		me->totedge= me->pv->totedge;
+		me->totface= me->pv->totface;
+		MEM_freeN(me->pv);
+	}
+
+	CustomData_free(&me->vdata, me->totvert);
+	CustomData_free(&me->edata, me->totedge);
+	CustomData_free(&me->fdata, me->totface);
 
 	if(me->mat) MEM_freeN(me->mat);
-	if(me->orco) MEM_freeN(me->orco);
-
+	
 	if(me->bb) MEM_freeN(me->bb);
-	if(me->disp.first) freedisplist(&me->disp);
-	if(me->derived) me->derived->release(me->derived);
+	if(me->mselect) MEM_freeN(me->mselect);
+
+	if(me->mr) multires_free(me->mr);
 }
 
 void copy_dverts(MDeformVert *dst, MDeformVert *src, int copycount)
@@ -173,6 +210,7 @@ void copy_dverts(MDeformVert *dst, MDeformVert *src, int copycount)
 	}
 
 }
+
 void free_dverts(MDeformVert *dvert, int totvert)
 {
 	/* Instead of freeing the verts directly,
@@ -200,17 +238,20 @@ Mesh *add_mesh()
 	me->smoothresh= 30;
 	me->texflag= AUTOSPACE;
 	me->flag= ME_TWOSIDED;
-	me->subdiv= 1;
-	me->subdivr = 1;
 	me->bb= unit_boundbox();
-	
+
+#ifdef WITH_VERSE
+	me->vnode = NULL;
+#endif
+
 	return me;
 }
 
 Mesh *copy_mesh(Mesh *me)
 {
 	Mesh *men;
-	int a;
+	MTFace *tface;
+	int a, i;
 	
 	men= copy_libblock(me);
 	
@@ -220,58 +261,62 @@ Mesh *copy_mesh(Mesh *me)
 	}
 	id_us_plus((ID *)men->texcomesh);
 
-	men->mvert= MEM_dupallocN(me->mvert);
-	men->medge= MEM_dupallocN(me->medge);
-	men->mface= MEM_dupallocN(me->mface);
-	men->tface= MEM_dupallocN(me->tface);
-	men->dface= NULL;
+	CustomData_copy(&me->vdata, &men->vdata, CD_MASK_MESH, CD_DUPLICATE, men->totvert);
+	CustomData_copy(&me->edata, &men->edata, CD_MASK_MESH, CD_DUPLICATE, men->totedge);
+	CustomData_copy(&me->fdata, &men->fdata, CD_MASK_MESH, CD_DUPLICATE, men->totface);
+	mesh_update_customdata_pointers(men);
 
-	if (me->dvert){
-		men->dvert = MEM_mallocN (sizeof (MDeformVert)*me->totvert, "MDeformVert");
-		copy_dverts(men->dvert, me->dvert, me->totvert);
+	/* ensure indirect linked data becomes lib-extern */
+	for(i=0; i<me->fdata.totlayer; i++) {
+		if(me->fdata.layers[i].type == CD_MTFACE) {
+			tface= (MTFace*)me->fdata.layers[i].data;
+
+			for(a=0; a<me->totface; a++, tface++)
+				if(tface->tpage)
+					id_lib_extern((ID*)tface->tpage);
+		}
 	}
-
-	men->mcol= MEM_dupallocN(me->mcol);
-	men->msticky= MEM_dupallocN(me->msticky);
-	men->texcomesh= NULL;
-	men->orco= NULL;
-	men->bb= MEM_dupallocN(men->bb);
 	
-	men->disp.first= men->disp.last= NULL;	// dont copy, editmode version has pointers in it
+	if(me->mr)
+		men->mr= multires_copy(me->mr);
+
+	men->mselect= NULL;
+
+	men->bb= MEM_dupallocN(men->bb);
 	
 	men->key= copy_key(me->key);
 	if(men->key) men->key->from= (ID *)men;
-	
-	men->derived = NULL;
-	men->decimated = NULL;
+
+#ifdef WITH_VERSE
+	men->vnode = NULL;
+#endif	
 
 	return men;
 }
 
 void make_local_tface(Mesh *me)
 {
-	TFace *tface;
+	MTFace *tface;
 	Image *ima;
-	int a;
+	int a, i;
 	
-	if(me->tface==0) return;
-	
-	a= me->totface;
-	tface= me->tface;
-	while(a--) {
-		
-		/* special case: ima always local immediately */
-		if(tface->tpage) {
-			ima= tface->tpage;
-			if(ima->id.lib) {
-				ima->id.lib= 0;
-				ima->id.flag= LIB_LOCAL;
-				new_id(0, (ID *)ima, 0);
+	for(i=0; i<me->fdata.totlayer; i++) {
+		if(me->fdata.layers[i].type == CD_MTFACE) {
+			tface= (MTFace*)me->fdata.layers[i].data;
+			
+			for(a=0; a<me->totface; a++, tface++) {
+				/* special case: ima always local immediately */
+				if(tface->tpage) {
+					ima= tface->tpage;
+					if(ima->id.lib) {
+						ima->id.lib= 0;
+						ima->id.flag= LIB_LOCAL;
+						new_id(0, (ID *)ima, 0);
+					}
+				}
 			}
 		}
-		tface++;
 	}
-	
 }
 
 void make_local_mesh(Mesh *me)
@@ -291,7 +336,7 @@ void make_local_mesh(Mesh *me)
 		me->id.flag= LIB_LOCAL;
 		new_id(0, (ID *)me, 0);
 		
-		if(me->tface) make_local_tface(me);
+		if(me->mtface) make_local_tface(me);
 		
 		return;
 	}
@@ -310,7 +355,7 @@ void make_local_mesh(Mesh *me)
 		me->id.flag= LIB_LOCAL;
 		new_id(0, (ID *)me, 0);
 		
-		if(me->tface) make_local_tface(me);
+		if(me->mtface) make_local_tface(me);
 		
 	}
 	else if(local && lib) {
@@ -350,28 +395,20 @@ void boundbox_mesh(Mesh *me, float *loc, float *size)
 		DO_MINMAX(mvert->co, min, max);
 	}
 
-	if(me->totvert) {
-		loc[0]= (min[0]+max[0])/2.0f;
-		loc[1]= (min[1]+max[1])/2.0f;
-		loc[2]= (min[2]+max[2])/2.0f;
-		
-		size[0]= (max[0]-min[0])/2.0f;
-		size[1]= (max[1]-min[1])/2.0f;
-		size[2]= (max[2]-min[2])/2.0f;
+	if(!me->totvert) {
+		min[0] = min[1] = min[2] = -1.0f;
+		max[0] = max[1] = max[2] = 1.0f;
 	}
-	else {
-		loc[0]= loc[1]= loc[2]= 0.0;
-		size[0]= size[1]= size[2]= 0.0;
-	}
-	
-	bb->vec[0][0]=bb->vec[1][0]=bb->vec[2][0]=bb->vec[3][0]= loc[0]-size[0];
-	bb->vec[4][0]=bb->vec[5][0]=bb->vec[6][0]=bb->vec[7][0]= loc[0]+size[0];
-	
-	bb->vec[0][1]=bb->vec[1][1]=bb->vec[4][1]=bb->vec[5][1]= loc[1]-size[1];
-	bb->vec[2][1]=bb->vec[3][1]=bb->vec[6][1]=bb->vec[7][1]= loc[1]+size[1];
 
-	bb->vec[0][2]=bb->vec[3][2]=bb->vec[4][2]=bb->vec[7][2]= loc[2]-size[2];
-	bb->vec[1][2]=bb->vec[2][2]=bb->vec[5][2]=bb->vec[6][2]= loc[2]+size[2];
+	loc[0]= (min[0]+max[0])/2.0f;
+	loc[1]= (min[1]+max[1])/2.0f;
+	loc[2]= (min[2]+max[2])/2.0f;
+		
+	size[0]= (max[0]-min[0])/2.0f;
+	size[1]= (max[1]-min[1])/2.0f;
+	size[2]= (max[2]-min[2])/2.0f;
+	
+	boundbox_set_from_min_max(bb, min, max);
 }
 
 void tex_space_mesh(Mesh *me)
@@ -405,132 +442,126 @@ void tex_space_mesh(Mesh *me)
 			}
 		}
 
+		for (a=0; a<3; a++) {
+			if(size[a]==0.0) size[a]= 1.0;
+			else if(size[a]>0.0 && size[a]<0.00001) size[a]= 0.00001;
+			else if(size[a]<0.0 && size[a]> -0.00001) size[a]= -0.00001;
+		}
+
 		VECCOPY(me->loc, loc);
 		VECCOPY(me->size, size);
 		me->rot[0]= me->rot[1]= me->rot[2]= 0.0;
-
-		if(me->size[0]==0.0) me->size[0]= 1.0;
-		else if(me->size[0]>0.0 && me->size[0]<0.00001) me->size[0]= 0.00001;
-		else if(me->size[0]<0.0 && me->size[0]> -0.00001) me->size[0]= -0.00001;
-	
-		if(me->size[1]==0.0) me->size[1]= 1.0;
-		else if(me->size[1]>0.0 && me->size[1]<0.00001) me->size[1]= 0.00001;
-		else if(me->size[1]<0.0 && me->size[1]> -0.00001) me->size[1]= -0.00001;
-	
-		if(me->size[2]==0.0) me->size[2]= 1.0;
-		else if(me->size[2]>0.0 && me->size[2]<0.00001) me->size[2]= 0.00001;
-		else if(me->size[2]<0.0 && me->size[2]> -0.00001) me->size[2]= -0.00001;
 	}
-	
 }
 
-void make_orco_displist_mesh(Object *ob, int subdivlvl)
+BoundBox *mesh_get_bb(Mesh *me)
 {
-	Mesh *me;
-	DerivedMesh *dm;
-	DispListMesh *dlm;
-	int i;
-	
-	me= ob->data;
-
-	if (G.obedit && G.obedit->data==me) {
-		dm= subsurf_make_derived_from_editmesh(G.editMesh, subdivlvl, me->subsurftype, NULL);
-		dlm= dm->convertToDispListMesh(dm);
-		dm->release(dm);
-	} else {
-			/* if there's a key, set the first one */
-		if(me->key && me->texcomesh==0) {
-			cp_key(0, me->totvert, me->totvert, (char*) me->mvert->co, me->key, me->key->refkey, 0);
-		}
-
-		dm= subsurf_make_derived_from_mesh(me, subdivlvl);
-		dlm= dm->convertToDispListMesh(dm);
-		dm->release(dm);
-		
-			/* Restore correct key */
-		do_ob_key(ob);
+	if (!me->bb) {
+		tex_space_mesh(me);
 	}
 
-	if (me->orco) MEM_freeN(me->orco);
-	me->orco= MEM_mallocN(dlm->totvert*3*sizeof(float), "mesh displist orco");
-	
-	for(i=0; i<dlm->totvert; i++) {
-		float *fp= &me->orco[i*3];
-
-		VECCOPY(fp, dlm->mvert[i].co);
-
-		fp[0]= (fp[0]-me->loc[0])/me->size[0];
-		fp[1]= (fp[1]-me->loc[1])/me->size[1];
-		fp[2]= (fp[2]-me->loc[2])/me->size[2];
-	}
-	
-	displistmesh_free(dlm);
+	return me->bb;
 }
 
-void make_orco_mesh(Mesh *me)
+void mesh_get_texspace(Mesh *me, float *loc_r, float *rot_r, float *size_r)
 {
-	MVert *mvert;
-	KeyBlock *kb;
-	float *orco, *fp;
+	if (!me->bb) {
+		tex_space_mesh(me);
+	}
+
+	if (loc_r) VECCOPY(loc_r, me->loc);
+	if (rot_r) VECCOPY(rot_r, me->rot);
+	if (size_r) VECCOPY(size_r, me->size);
+}
+
+static float *make_orco_mesh_internal(Object *ob, int render)
+{
+	Mesh *me = ob->data;
+	float (*orcoData)[3];
 	int a, totvert;
-	
-	totvert= me->totvert;
-	if(totvert==0) return;
-	orco= me->orco= MEM_mallocN(sizeof(float)*3*totvert, "orco mesh");
+	float loc[3], size[3];
+	DerivedMesh *dm;
+	float (*vcos)[3] = NULL;
 
-	if(me->key && me->texcomesh==0) {
-		kb= me->key->refkey;
-		if (kb) {		/***** BUG *****/
-			fp= kb->data;
-			
-			for(a=0; a<totvert; a++, orco+=3) {
-				orco[0]= (fp[0]-me->loc[0])/me->size[0];
-				orco[1]= (fp[1]-me->loc[1])/me->size[1];
-				orco[2]= (fp[2]-me->loc[2])/me->size[2];
-				
-				/* only increase mvert when totvert <= kb->totelem */
-				if(a<kb->totelem) fp+=3;
-			}
+		/* Get appropriate vertex coordinates */
+
+	if(me->key && me->texcomesh==0 && me->key->refkey) {
+		KeyBlock *kb= me->key->refkey;
+		float *fp= kb->data;
+		totvert= MIN2(kb->totelem, me->totvert);
+		vcos = MEM_callocN(sizeof(*vcos)*me->totvert, "orco mesh");
+
+		for(a=0; a<totvert; a++, fp+=3) {
+			vcos[a][0]= fp[0];
+			vcos[a][1]= fp[1];
+			vcos[a][2]= fp[2];
 		}
 	}
 	else {
-		if(me->texcomesh) {
-			me= me->texcomesh;
-		}	
-	
-		mvert= me->mvert;
-		for(a=0; a<totvert; a++, orco+=3) {
-			orco[0]= (mvert->co[0]-me->loc[0])/me->size[0];
-			orco[1]= (mvert->co[1]-me->loc[1])/me->size[1];
-			orco[2]= (mvert->co[2]-me->loc[2])/me->size[2];
-			
-			/* only increase mvert when totvert <= me->totvert */
-			if(a<me->totvert) mvert++;
+		MultiresLevel *lvl = NULL;
+		MVert *mvert = NULL;
+		
+		if(me->mr) {
+			lvl = multires_level_n(me->mr, me->mr->renderlvl);
+			vcos = MEM_callocN(sizeof(*vcos)*lvl->totvert, "orco mr mesh");
+			mvert = lvl->verts;
+			totvert = lvl->totvert;
+		}
+		else {
+			Mesh *tme = me->texcomesh?me->texcomesh:me;
+			vcos = MEM_callocN(sizeof(*vcos)*me->totvert, "orco mesh");
+			mvert = tme->mvert;
+			totvert = MIN2(tme->totvert, me->totvert);
+		}
+
+		for(a=0; a<totvert; a++, mvert++) {
+			vcos[a][0]= mvert->co[0];
+			vcos[a][1]= mvert->co[1];
+			vcos[a][2]= mvert->co[2];
 		}
 	}
+
+		/* Apply orco-changing modifiers */
+
+	if (render) {
+		dm = mesh_create_derived_no_deform_render(ob, vcos, CD_MASK_BAREMESH);
+	} else {
+		dm = mesh_create_derived_no_deform(ob, vcos, CD_MASK_BAREMESH);
+	}
+	totvert = dm->getNumVerts(dm);
+
+	orcoData = MEM_mallocN(sizeof(*orcoData)*totvert, "orcoData");
+	dm->getVertCos(dm, orcoData);
+	dm->release(dm);
+	MEM_freeN(vcos);
+
+	mesh_get_texspace(me->texcomesh?me->texcomesh:me, loc, NULL, size);
+
+	for(a=0; a<totvert; a++) {
+		float *co = orcoData[a];
+		co[0] = (co[0]-loc[0])/size[0];
+		co[1] = (co[1]-loc[1])/size[1];
+		co[2] = (co[2]-loc[2])/size[2];
+	}
+
+	return (float*) orcoData;
 }
 
-/** rotates the vertices of a face in case v[2] or v[3] (vertex index)
-  * is = 0.
-  * Helaas, the MFace structure has no pointer to its
-  * texture face, therefore, texture can not be fixed inside
-  * this function. 
-  * 
-  * see also blender/src/editmesh.c, fix_faceindices()
-
-  * THIS FUNCTION WILL BE DINOSOURCE. For the moment, another hack
-	is added to fix texture coordinates / vertex colors:
-
-	void test_index_face(MFace *mface, TFace *tface, int nr)
-  */
-
-void test_index_mface(MFace *mface, int nr)
+float *mesh_create_orco_render(Object *ob) 
 {
-	int a;
+	return make_orco_mesh_internal(ob, 1);
+}
 
-	
+float *mesh_create_orco(Object *ob)
+{
+	return make_orco_mesh_internal(ob, 0);
+}
+
+/* rotates the vertices of a face in case v[2] or v[3] (vertex index) is = 0.
+   this is necessary to make the if(mface->v4) check for quads work */
+void test_index_face(MFace *mface, CustomData *fdata, int mfindex, int nr)
+{
 	/* first test if the face is legal */
-
 	if(mface->v3 && mface->v3==mface->v4) {
 		mface->v4= 0;
 		nr--;
@@ -548,207 +579,26 @@ void test_index_mface(MFace *mface, int nr)
 	}
 
 	/* prevent a zero at wrong index location */
-	if(nr==2) {
-		if(mface->v2==0) SWAP(int, mface->v1, mface->v2);
-	}
-	else if(nr==3) {
+	if(nr==3) {
 		if(mface->v3==0) {
+			static int corner_indices[4] = {1, 2, 0, 3};
+
 			SWAP(int, mface->v1, mface->v2);
 			SWAP(int, mface->v2, mface->v3);
-			
-			a= mface->edcode;
-			mface->edcode= 0;
-			if(a & ME_V1V2) mface->edcode |= ME_V3V1;
-			if(a & ME_V2V3) mface->edcode |= ME_V1V2;
-			if(a & ME_V3V1) mface->edcode |= ME_V2V3;
-			
-			a= mface->puno;
-			mface->puno &= ~15;
-			if(a & ME_FLIPV1) mface->puno |= ME_FLIPV2;
-			if(a & ME_FLIPV2) mface->puno |= ME_FLIPV3;
-			if(a & ME_FLIPV3) mface->puno |= ME_FLIPV1;
+
+			if(fdata)
+				CustomData_swap(fdata, mfindex, corner_indices);
 		}
 	}
 	else if(nr==4) {
 		if(mface->v3==0 || mface->v4==0) {
+			static int corner_indices[4] = {2, 3, 0, 1};
+
 			SWAP(int, mface->v1, mface->v3);
 			SWAP(int, mface->v2, mface->v4);
-			a= mface->edcode;
-			mface->edcode= 0;
-			if(a & ME_V1V2) mface->edcode |= ME_V3V4;
-			if(a & ME_V2V3) mface->edcode |= ME_V2V3;
-			if(a & ME_V3V4) mface->edcode |= ME_V1V2;
-			if(a & ME_V4V1) mface->edcode |= ME_V4V1;
 
-			a= mface->puno;
-			mface->puno &= ~15;
-			if(a & ME_FLIPV1) mface->puno |= ME_FLIPV3;
-			if(a & ME_FLIPV2) mface->puno |= ME_FLIPV4;
-			if(a & ME_FLIPV3) mface->puno |= ME_FLIPV1;
-			if(a & ME_FLIPV4) mface->puno |= ME_FLIPV2;
-		}
-	}
-}
-
-/**	This function should die as soon as there is another mesh
-	structure. Functionality is the same as
-
-		void test_index_mface()
-
-	but it fixes texture coordinates as well. 
-*/
-
-#define UVCOPY(t, s) memcpy(t, s, 2 * sizeof(float));
-void test_index_face(MFace *mface, TFace *tface, int nr)
-{
-	int a;
-	float tmpuv[2];
-	unsigned int tmpcol;
-
-
-	/* first test if the face is legal */
-
-	if(mface->v3 && mface->v3==mface->v4) {
-		mface->v4= 0;
-		nr--;
-	}
-	if(mface->v2 && mface->v2==mface->v3) {
-		mface->v3= mface->v4;
-		mface->v4= 0;
-		nr--;
-	}
-	if(mface->v1==mface->v2) {
-		mface->v2= mface->v3;
-		mface->v3= mface->v4;
-		mface->v4= 0;
-		nr--;
-	}
-
-	/* prevent a zero at wrong index location */
-	if(nr==2) {
-		if(mface->v2==0) SWAP(int, mface->v1, mface->v2);
-	}
-	else if(nr==3) {
-		if(mface->v3==0) {
-			SWAP(int, mface->v1, mface->v2);
-			SWAP(int, mface->v2, mface->v3);
-			/* rotate face UV coordinates, too */
-			UVCOPY(tmpuv, tface->uv[0]);
-			UVCOPY(tface->uv[0], tface->uv[1]);
-			UVCOPY(tface->uv[1], tface->uv[2]);
-			UVCOPY(tface->uv[2], tmpuv);
-			/* same with vertex colours */
-			tmpcol = tface->col[0];
-			tface->col[0] = tface->col[1];
-			tface->col[1] = tface->col[2];
-			tface->col[2] = tmpcol;
-
-			
-			a= mface->edcode;
-			mface->edcode= 0;
-			if(a & ME_V1V2) mface->edcode |= ME_V3V1;
-			if(a & ME_V2V3) mface->edcode |= ME_V1V2;
-			if(a & ME_V3V1) mface->edcode |= ME_V2V3;
-			
-			a= mface->puno;
-			mface->puno &= ~15;
-			if(a & ME_FLIPV1) mface->puno |= ME_FLIPV2;
-			if(a & ME_FLIPV2) mface->puno |= ME_FLIPV3;
-			if(a & ME_FLIPV3) mface->puno |= ME_FLIPV1;
-		}
-	}
-	else if(nr==4) {
-		if(mface->v3==0 || mface->v4==0) {
-			SWAP(int, mface->v1, mface->v3);
-			SWAP(int, mface->v2, mface->v4);
-			/* swap UV coordinates */
-			UVCOPY(tmpuv, tface->uv[0]);
-			UVCOPY(tface->uv[0], tface->uv[2]);
-			UVCOPY(tface->uv[2], tmpuv);
-			UVCOPY(tmpuv, tface->uv[1]);
-			UVCOPY(tface->uv[1], tface->uv[3]);
-			UVCOPY(tface->uv[3], tmpuv);
-			/* swap vertex colours */
-			tmpcol = tface->col[0];
-			tface->col[0] = tface->col[2];
-			tface->col[2] = tmpcol;
-			tmpcol = tface->col[1];
-			tface->col[1] = tface->col[3];
-			tface->col[3] = tmpcol;
-
-			a= mface->edcode;
-			mface->edcode= 0;
-			if(a & ME_V1V2) mface->edcode |= ME_V3V4;
-			if(a & ME_V2V3) mface->edcode |= ME_V2V3;
-			if(a & ME_V3V4) mface->edcode |= ME_V1V2;
-			if(a & ME_V4V1) mface->edcode |= ME_V4V1;
-
-			a= mface->puno;
-			mface->puno &= ~15;
-			if(a & ME_FLIPV1) mface->puno |= ME_FLIPV3;
-			if(a & ME_FLIPV2) mface->puno |= ME_FLIPV4;
-			if(a & ME_FLIPV3) mface->puno |= ME_FLIPV1;
-			if(a & ME_FLIPV4) mface->puno |= ME_FLIPV2;
-		}
-	}
-}
-
-void flipnorm_mesh(Mesh *me)
-{
-	MFace *mface;
-	MVert *mvert;
-	DispList *dl;
-	float *fp;
-	int a, temp;
-	
-	mvert= me->mvert;
-	a= me->totvert;
-	while(a--) {
-		mvert->no[0]= -mvert->no[0];
-		mvert->no[1]= -mvert->no[1];
-		mvert->no[2]= -mvert->no[2];
-		mvert++;
-	}
-	
-	mface= me->mface;
-	a= me->totface;
-	while(a--) {
-		if(mface->v3) {
-			if(mface->v4) {
-				SWAP(int, mface->v4, mface->v1);
-				SWAP(int, mface->v3, mface->v2);
-				test_index_mface(mface, 4);
-				temp= mface->puno;
-				mface->puno &= ~15;
-				if(temp & ME_FLIPV1) mface->puno |= ME_FLIPV4;
-				if(temp & ME_FLIPV2) mface->puno |= ME_FLIPV3;
-				if(temp & ME_FLIPV3) mface->puno |= ME_FLIPV2;
-				if(temp & ME_FLIPV4) mface->puno |= ME_FLIPV1;
-			}
-			else {
-				SWAP(int, mface->v3, mface->v1);
-				test_index_mface(mface, 3);
-				temp= mface->puno;
-				mface->puno &= ~15;
-				if(temp & ME_FLIPV1) mface->puno |= ME_FLIPV3;
-				if(temp & ME_FLIPV2) mface->puno |= ME_FLIPV2;
-				if(temp & ME_FLIPV3) mface->puno |= ME_FLIPV1;
-			}
-		}
-		mface++;
-	}
-
-	if(me->disp.first) {
-		dl= me->disp.first;
-		fp= dl->nors;
-		if(fp) {
-			a= dl->nr;
-			while(a--) {
-				fp[0]= -fp[0];
-				fp[1]= -fp[1];
-				fp[2]= -fp[2];
-				fp+= 3;
-			}
+			if(fdata)
+				CustomData_swap(fdata, mfindex, corner_indices);
 		}
 	}
 }
@@ -781,11 +631,11 @@ void set_mesh(Object *ob, Mesh *me)
 
 struct edgesort {
 	int v1, v2;
-	int flag;
+	short is_loose, is_draw;
 };
 
 /* edges have to be added with lowest index first for sorting */
-static void to_edgesort(struct edgesort *ed, int v1, int v2, int flag)
+static void to_edgesort(struct edgesort *ed, int v1, int v2, short is_loose, short is_draw)
 {
 	if(v1<v2) {
 		ed->v1= v1; ed->v2= v2;
@@ -793,7 +643,8 @@ static void to_edgesort(struct edgesort *ed, int v1, int v2, int flag)
 	else {
 		ed->v1= v2; ed->v2= v1;
 	}
-	ed->flag= flag;
+	ed->is_loose= is_loose;
+	ed->is_draw= is_draw;
 }
 
 static int vergedgesort(const void *v1, const void *v2)
@@ -808,8 +659,7 @@ static int vergedgesort(const void *v1, const void *v2)
 	return 0;
 }
 
-
-void make_edges(Mesh *me)
+void make_edges(Mesh *me, int old)
 {
 	MFace *mface;
 	MEdge *medge;
@@ -824,27 +674,25 @@ void make_edges(Mesh *me)
 		else totedge+=1;
 	}
 	
-	if(totedge==0) return;
+	if(totedge==0) {
+		/* flag that mesh has edges */
+		me->medge = MEM_callocN(0, "make mesh edges");
+		me->totedge = 0;
+		return;
+	}
 	
 	ed= edsort= MEM_mallocN(totedge*sizeof(struct edgesort), "edgesort");
 	
 	for(a= me->totface, mface= me->mface; a>0; a--, mface++) {
-		
-		to_edgesort(ed, mface->v1, mface->v2, mface->edcode & ME_V1V2);
-		ed++;
+		to_edgesort(ed++, mface->v1, mface->v2, !mface->v3, mface->edcode & ME_V1V2);
 		if(mface->v4) {
-			to_edgesort(ed, mface->v2, mface->v3, mface->edcode & ME_V2V3);
-			ed++;
-			to_edgesort(ed, mface->v3, mface->v4, mface->edcode & ME_V3V4);
-			ed++;
-			to_edgesort(ed, mface->v4, mface->v1, mface->edcode & ME_V4V1);
-			ed++;
+			to_edgesort(ed++, mface->v2, mface->v3, 0, mface->edcode & ME_V2V3);
+			to_edgesort(ed++, mface->v3, mface->v4, 0, mface->edcode & ME_V3V4);
+			to_edgesort(ed++, mface->v4, mface->v1, 0, mface->edcode & ME_V4V1);
 		}
 		else if(mface->v3) {
-			to_edgesort(ed, mface->v2, mface->v3, mface->edcode & ME_V2V3);
-			ed++;
-			to_edgesort(ed, mface->v3, mface->v1, mface->edcode & ME_V3V1);
-			ed++;
+			to_edgesort(ed++, mface->v2, mface->v3, 0, mface->edcode & ME_V2V3);
+			to_edgesort(ed++, mface->v3, mface->v1, 0, mface->edcode & ME_V3V1);
 		}
 	}
 	
@@ -854,15 +702,12 @@ void make_edges(Mesh *me)
 	for(a=totedge, ed=edsort; a>1; a--, ed++) {
 		/* edge is unique when it differs from next edge, or is last */
 		if(ed->v1 != (ed+1)->v1 || ed->v2 != (ed+1)->v2) final++;
-		else {
-			/* this makes sure identical edges both get draw flag */
-			if(ed->flag) (ed+1)->flag= 1;
-			else if((ed+1)->flag) ed->flag= 1;
-		}
 	}
 	final++;
 	
-	medge= me->medge= MEM_callocN(final*sizeof(MEdge), "make mesh edges");
+
+	medge= CustomData_add_layer(&me->edata, CD_MEDGE, CD_CALLOC, NULL, final);
+	me->medge= medge;
 	me->totedge= final;
 	
 	for(a=totedge, ed=edsort; a>1; a--, ed++) {
@@ -870,18 +715,43 @@ void make_edges(Mesh *me)
 		if(ed->v1 != (ed+1)->v1 || ed->v2 != (ed+1)->v2) {
 			medge->v1= ed->v1;
 			medge->v2= ed->v2;
-			if(ed->flag) medge->flag= ME_EDGEDRAW;
+			if(old==0 || ed->is_draw) medge->flag= ME_EDGEDRAW|ME_EDGERENDER;
+			if(ed->is_loose) medge->flag|= ME_LOOSEEDGE;
 			medge++;
 		}
+		else {
+			/* equal edge, we merge the drawflag */
+			(ed+1)->is_draw |= ed->is_draw;
+		}
 	}
 	/* last edge */
 	medge->v1= ed->v1;
 	medge->v2= ed->v2;
-	if(ed->flag) medge->flag= ME_EDGEDRAW;
+	medge->flag= ME_EDGEDRAW;
+	if(ed->is_loose) medge->flag|= ME_LOOSEEDGE;
+	medge->flag |= ME_EDGERENDER;
 
 	MEM_freeN(edsort);
+
+	mesh_strip_loose_faces(me);
 }
 
+void mesh_strip_loose_faces(Mesh *me)
+{
+	int a,b;
+
+	for (a=b=0; a<me->totface; a++) {
+		if (me->mface[a].v3) {
+			if (a!=b) {
+				memcpy(&me->mface[b],&me->mface[a],sizeof(me->mface[b]));
+				CustomData_copy_data(&me->fdata, &me->fdata, a, b, 1);
+				CustomData_free_elem(&me->fdata, a, 1);
+			}
+			b++;
+		}
+	}
+	me->totface = b;
+}
 
 
 void mball_to_mesh(ListBase *lb, Mesh *me)
@@ -900,7 +770,11 @@ void mball_to_mesh(ListBase *lb, Mesh *me)
 		me->totvert= dl->nr;
 		me->totface= dl->parts;
 		
-		me->mvert=mvert= MEM_callocN(dl->nr*sizeof(MVert), "mverts");
+		mvert= CustomData_add_layer(&me->vdata, CD_MVERT, CD_CALLOC, NULL, dl->nr);
+		mface= CustomData_add_layer(&me->fdata, CD_MFACE, CD_CALLOC, NULL, dl->parts);
+		me->mvert= mvert;
+		me->mface= mface;
+
 		a= dl->nr;
 		nors= dl->nors;
 		verts= dl->verts;
@@ -914,7 +788,6 @@ void mball_to_mesh(ListBase *lb, Mesh *me)
 			verts+= 3;
 		}
 		
-		me->mface=mface= MEM_callocN(dl->parts*sizeof(MFace), "mface");
 		a= dl->parts;
 		index= dl->index;
 		while(a--) {
@@ -922,14 +795,16 @@ void mball_to_mesh(ListBase *lb, Mesh *me)
 			mface->v2= index[1];
 			mface->v3= index[2];
 			mface->v4= index[3];
+			mface->flag= ME_SMOOTH;
+
+			if(mface->v3==mface->v4)
+				mface->v4= 0;
 
-			mface->puno= 0;
-			mface->edcode= ME_V1V2+ME_V2V3;
-			mface->flag = ME_SMOOTH;
-			
 			mface++;
 			index+= 4;
 		}
+
+		make_edges(me, 0);	// all edges
 	}	
 }
 
@@ -947,14 +822,6 @@ void nurbs_to_mesh(Object *ob)
 
 	cu= ob->data;
 
-	if(ob->type==OB_CURVE) {
-		/* rule: dl->type INDEX3 always as first in list */
-		dl= cu->disp.first;
-		if(dl->type!=DL_INDEX3) {
-			curve_to_filledpoly(ob->data, &cu->nurb, &cu->disp);
-		}
-	}
-
 	/* count */
 	dl= cu->disp.first;
 	while(dl) {
@@ -971,7 +838,7 @@ void nurbs_to_mesh(Object *ob)
 		}
 		else if(dl->type==DL_SURF) {
 			totvert+= dl->parts*dl->nr;
-			totvlak+= (dl->parts-1+((dl->flag & 2)==2))*(dl->nr-1+(dl->flag & 1));
+			totvlak+= (dl->parts-1+((dl->flag & DL_CYCL_V)==2))*(dl->nr-1+(dl->flag & DL_CYCL_U));
 		}
 		else if(dl->type==DL_INDEX3) {
 			totvert+= dl->nr;
@@ -994,8 +861,10 @@ void nurbs_to_mesh(Object *ob)
 	cu->mat= 0;
 	cu->totcol= 0;
 
-	mvert=me->mvert= MEM_callocN(me->totvert*sizeof(MVert), "cumesh1");
-	mface=me->mface= MEM_callocN(me->totface*sizeof(MFace), "cumesh2");
+	mvert= CustomData_add_layer(&me->vdata, CD_MVERT, CD_CALLOC, NULL, me->totvert);
+	mface= CustomData_add_layer(&me->fdata, CD_MFACE, CD_CALLOC, NULL, me->totface);
+	me->mvert= mvert;
+	me->mface= mface;
 
 	/* verts and faces */
 	vertcount= 0;
@@ -1018,8 +887,6 @@ void nurbs_to_mesh(Object *ob)
 				for(b=1; b<dl->nr; b++) {
 					mface->v1= startvert+ofs+b-1;
 					mface->v2= startvert+ofs+b;
-					mface->edcode= ME_V1V2;
-					test_index_mface(mface, 2);
 					mface++;
 				}
 			}
@@ -1044,8 +911,6 @@ void nurbs_to_mesh(Object *ob)
 						mface->v1= startvert+ofs+b;
 						if(b==dl->nr-1) mface->v2= startvert+ofs;
 						else mface->v2= startvert+ofs+b+1;
-						mface->edcode= ME_V1V2;
-						test_index_mface(mface, 2);
 						mface++;
 					}
 				}
@@ -1069,10 +934,7 @@ void nurbs_to_mesh(Object *ob)
 				mface->v2= startvert+index[1];
 				mface->v3= startvert+index[2];
 				mface->v4= 0;
-	
-				mface->puno= 7;
-				mface->edcode= ME_V1V2+ME_V2V3;
-				test_index_mface(mface, 3);
+				test_index_face(mface, NULL, 0, 3);
 				
 				mface++;
 				index+= 3;
@@ -1093,9 +955,9 @@ void nurbs_to_mesh(Object *ob)
 
 			for(a=0; a<dl->parts; a++) {
 
-				if( (dl->flag & 2)==0 && a==dl->parts-1) break;
+				if( (dl->flag & DL_CYCL_V)==0 && a==dl->parts-1) break;
 
-				if(dl->flag & 1) {			/* p2 -> p1 -> */
+				if(dl->flag & DL_CYCL_U) {			/* p2 -> p1 -> */
 					p1= startvert+ dl->nr*a;	/* p4 -> p3 -> */
 					p2= p1+ dl->nr-1;		/* -----> next row */
 					p3= p1+ dl->nr;
@@ -1109,7 +971,7 @@ void nurbs_to_mesh(Object *ob)
 					p3= p1+ dl->nr;
 					b= 1;
 				}
-				if( (dl->flag & 2) && a==dl->parts-1) {
+				if( (dl->flag & DL_CYCL_V) && a==dl->parts-1) {
 					p3-= dl->parts*dl->nr;
 					p4-= dl->parts*dl->nr;
 				}
@@ -1120,8 +982,7 @@ void nurbs_to_mesh(Object *ob)
 					mface->v3= p4;
 					mface->v4= p2;
 					mface->mat_nr= (unsigned char)dl->col;
-					mface->edcode= ME_V1V2+ME_V2V3;
-					test_index_mface(mface, 4);
+					test_index_face(mface, NULL, 0, 4);
 					mface++;
 
 					p4= p3; 
@@ -1136,14 +997,15 @@ void nurbs_to_mesh(Object *ob)
 		dl= dl->next;
 	}
 
+	make_edges(me, 0);	// all edges
+	mesh_calc_normals(me->mvert, me->totvert, me->mface, me->totface, NULL);
+
 	if(ob->data) {
 		free_libblock(&G.main->curve, ob->data);
 	}
 	ob->data= me;
 	ob->type= OB_MESH;
 	
-	tex_space_mesh(me);
-	
 	/* other users */
 	ob1= G.main->object.first;
 	while(ob1) {
@@ -1158,97 +1020,6 @@ void nurbs_to_mesh(Object *ob)
 
 }
 
-void edge_drawflags_mesh(Mesh *me)
-{
-	MFace *mface;
-	int a;
-	
-	mface= me->mface;
-	for(a=0; a<me->totface; a++, mface++) {
-		mface->edcode= ME_V1V2|ME_V2V3|ME_V3V4|ME_V4V1;
-	}
-}
-
-void tface_to_mcol(Mesh *me)
-{
-	TFace *tface;
-	unsigned int *mcol;
-	int a;
-	
-	me->mcol= MEM_mallocN(4*sizeof(int)*me->totface, "nepmcol");
-	mcol= (unsigned int *)me->mcol;
-	
-	a= me->totface;
-	tface= me->tface;
-	while(a--) {
-		memcpy(mcol, tface->col, 16);
-		mcol+= 4;
-		tface++;
-	}
-}
-
-void mcol_to_tface(Mesh *me, int freedata)
-{
-	TFace *tface;
-	unsigned int *mcol;
-	int a;
-	
-	a= me->totface;
-	tface= me->tface;
-	mcol= (unsigned int *)me->mcol;
-	while(a--) {
-		memcpy(tface->col, mcol, 16);
-		mcol+= 4;
-		tface++;
-	}
-	
-	if(freedata) {
-		MEM_freeN(me->mcol);
-		me->mcol= 0;
-	}
-}
-
-int mesh_uses_displist(Mesh *me) {
-	return (me->flag&ME_SUBSURF) && me->subdiv;
-}
-
-void mesh_calculate_vertex_normals(Mesh *me)
-{
-	float (*tempNorms)[3]= MEM_callocN(me->totvert*sizeof(*tempNorms), "tempNorms");
-	int i;
-
-	for (i=0; i<me->totface; i++) {
-		MFace *mf= &((MFace*) me->mface)[i];
-		float f_no[3];
-
-		if (!mf->v3)
-			continue;
-			
-		if (mf->v4) {
-			CalcNormFloat4(me->mvert[mf->v1].co, me->mvert[mf->v2].co, me->mvert[mf->v3].co, me->mvert[mf->v4].co, f_no);
-		} else {
-			CalcNormFloat(me->mvert[mf->v1].co, me->mvert[mf->v2].co, me->mvert[mf->v3].co, f_no);
-		}
-		
-		VecAddf(tempNorms[mf->v1], tempNorms[mf->v1], f_no);
-		VecAddf(tempNorms[mf->v2], tempNorms[mf->v2], f_no);
-		VecAddf(tempNorms[mf->v3], tempNorms[mf->v3], f_no);
-		if (mf->v4)
-			VecAddf(tempNorms[mf->v4], tempNorms[mf->v4], f_no);
-	}
-	for (i=0; i<me->totvert; i++) {
-		MVert *mv= &me->mvert[i];
-		float *no= tempNorms[i];
-		
-		Normalise(no);
-		mv->no[0]= (short)(no[0]*32767.0);
-		mv->no[1]= (short)(no[1]*32767.0);
-		mv->no[2]= (short)(no[2]*32767.0);
-	}
-	
-	MEM_freeN(tempNorms);
-}
-
 void mesh_delete_material_index(Mesh *me, int index) {
 	int i;
 
@@ -1259,7 +1030,8 @@ void mesh_delete_material_index(Mesh *me, int index) {
 	}
 }
 
-void mesh_set_smooth_flag(Mesh *me, int enableSmooth) {
+void mesh_set_smooth_flag(Object *meshOb, int enableSmooth) {
+	Mesh *me = meshOb->data;
 	int i;
 
 	for (i=0; i<me->totface; i++) {
@@ -1271,4 +1043,204 @@ void mesh_set_smooth_flag(Mesh *me, int enableSmooth) {
 			mf->flag &= ~ME_SMOOTH;
 		}
 	}
+
+	DAG_object_flush_update(G.scene, meshOb, OB_RECALC_DATA);
 }
+
+void mesh_calc_normals(MVert *mverts, int numVerts, MFace *mfaces, int numFaces, float **faceNors_r) 
+{
+	float (*tnorms)[3]= MEM_callocN(numVerts*sizeof(*tnorms), "tnorms");
+	float *fnors= MEM_mallocN(sizeof(*fnors)*3*numFaces, "meshnormals");
+	int i;
+
+	for (i=0; i<numFaces; i++) {
+		MFace *mf= &mfaces[i];
+		float *f_no= &fnors[i*3];
+
+		if (mf->v4)
+			CalcNormFloat4(mverts[mf->v1].co, mverts[mf->v2].co, mverts[mf->v3].co, mverts[mf->v4].co, f_no);
+		else
+			CalcNormFloat(mverts[mf->v1].co, mverts[mf->v2].co, mverts[mf->v3].co, f_no);
+		
+		VecAddf(tnorms[mf->v1], tnorms[mf->v1], f_no);
+		VecAddf(tnorms[mf->v2], tnorms[mf->v2], f_no);
+		VecAddf(tnorms[mf->v3], tnorms[mf->v3], f_no);
+		if (mf->v4)
+			VecAddf(tnorms[mf->v4], tnorms[mf->v4], f_no);
+	}
+	for (i=0; i<numVerts; i++) {
+		MVert *mv= &mverts[i];
+		float *no= tnorms[i];
+		
+		if (Normalise(no)==0.0) {
+			VECCOPY(no, mv->co);
+			Normalise(no);
+		}
+
+		mv->no[0]= (short)(no[0]*32767.0);
+		mv->no[1]= (short)(no[1]*32767.0);
+		mv->no[2]= (short)(no[2]*32767.0);
+	}
+	
+	MEM_freeN(tnorms);
+
+	if (faceNors_r) {
+		*faceNors_r = fnors;
+	} else {
+		MEM_freeN(fnors);
+	}
+}
+
+float (*mesh_getVertexCos(Mesh *me, int *numVerts_r))[3]
+{
+#ifdef WITH_VERSE
+	if(me->vnode) {
+		struct VLayer *vlayer;
+		struct VerseVert *vvert;
+		unsigned int i, numVerts;
+		float (*cos)[3];
+
+		vlayer = find_verse_layer_type((VGeomData*)((VNode*)me->vnode)->data, VERTEX_LAYER);
+
+		vvert = vlayer->dl.lb.first;
+		numVerts = vlayer->dl.da.count;
+		cos = MEM_mallocN(sizeof(*cos)*numVerts, "verse_vertexcos");
+
+		for(i=0; i<numVerts && vvert; vvert = vvert->next, i++) {
+			VECCOPY(cos[i], vvert->co);
+		}
+
+		return cos;
+	}
+	else {
+#endif
+		int i, numVerts = me->totvert;
+		float (*cos)[3] = MEM_mallocN(sizeof(*cos)*numVerts, "vertexcos1");
+        
+		if (numVerts_r) *numVerts_r = numVerts;
+		for (i=0; i<numVerts; i++) {
+			VECCOPY(cos[i], me->mvert[i].co);
+		}
+        
+		return cos;
+#ifdef WITH_VERSE
+	}
+#endif
+}
+
+/* UvVertMap */
+
+struct UvVertMap {
+	struct UvMapVert **vert;
+	struct UvMapVert *buf;
+};
+
+UvVertMap *make_uv_vert_map(struct MFace *mface, struct MTFace *tface, unsigned int totface, unsigned int totvert, int selected, float *limit)
+{
+	UvVertMap *vmap;
+	UvMapVert *buf;
+	MFace *mf;
+	MTFace *tf;
+	unsigned int a;
+	int	i, totuv, nverts;
+
+	totuv = 0;
+
+	/* generate UvMapVert array */
+	mf= mface;
+	tf= tface;
+	for(a=0; a<totface; a++, mf++, tf++)
+		if(!selected || (!(tf->flag & TF_HIDE) && (tf->flag & TF_SELECT)))
+			totuv += (mf->v4)? 4: 3;
+		
+	if(totuv==0)
+		return NULL;
+	
+	vmap= (UvVertMap*)MEM_mallocN(sizeof(*vmap), "UvVertMap");
+	if (!vmap)
+		return NULL;
+
+	vmap->vert= (UvMapVert**)MEM_callocN(sizeof(*vmap->vert)*totvert, "UvMapVert*");
+	buf= vmap->buf= (UvMapVert*)MEM_mallocN(sizeof(*vmap->buf)*totuv, "UvMapVert");
+
+	if (!vmap->vert || !vmap->buf) {
+		free_uv_vert_map(vmap);
+		return NULL;
+	}
+
+	mf= mface;
+	tf= tface;
+	for(a=0; a<totface; a++, mf++, tf++) {
+		if(!selected || (!(tf->flag & TF_HIDE) && (tf->flag & TF_SELECT))) {
+			nverts= (mf->v4)? 4: 3;
+
+			for(i=0; i<nverts; i++) {
+				buf->tfindex= i;
+				buf->f= a;
+				buf->separate = 0;
+				buf->next= vmap->vert[*(&mf->v1 + i)];
+				vmap->vert[*(&mf->v1 + i)]= buf;
+				buf++;
+			}
+		}
+	}
+	
+	/* sort individual uvs for each vert */
+	tf= tface;
+	for(a=0; a<totvert; a++) {
+		UvMapVert *newvlist= NULL, *vlist=vmap->vert[a];
+		UvMapVert *iterv, *v, *lastv, *next;
+		float *uv, *uv2, uvdiff[2];
+
+		while(vlist) {
+			v= vlist;
+			vlist= vlist->next;
+			v->next= newvlist;
+			newvlist= v;
+
+			uv= (tf+v->f)->uv[v->tfindex];
+			lastv= NULL;
+			iterv= vlist;
+
+			while(iterv) {
+				next= iterv->next;
+
+				uv2= (tf+iterv->f)->uv[iterv->tfindex];
+				Vec2Subf(uvdiff, uv2, uv);
+
+
+				if(fabs(uv[0]-uv2[0]) < limit[0] && fabs(uv[1]-uv2[1]) < limit[1]) {
+					if(lastv) lastv->next= next;
+					else vlist= next;
+					iterv->next= newvlist;
+					newvlist= iterv;
+				}
+				else
+					lastv=iterv;
+
+				iterv= next;
+			}
+
+			newvlist->separate = 1;
+		}
+
+		vmap->vert[a]= newvlist;
+	}
+	
+	return vmap;
+}
+
+UvMapVert *get_uv_map_vert(UvVertMap *vmap, unsigned int v)
+{
+	return vmap->vert[v];
+}
+
+void free_uv_vert_map(UvVertMap *vmap)
+{
+	if (vmap) {
+		if (vmap->vert) MEM_freeN(vmap->vert);
+		if (vmap->buf) MEM_freeN(vmap->buf);
+		MEM_freeN(vmap);
+	}
+}
+
