@@ -1,5 +1,5 @@
 /**
- * $Id: node.c,v 1.40 2006/07/09 11:54:41 ton Exp $
+ * $Id: node.c,v 1.69 2007/01/26 19:17:20 ton Exp $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -31,6 +31,7 @@
 #include <string.h>
 
 #include "DNA_ID.h"
+#include "DNA_image_types.h"
 #include "DNA_node_types.h"
 #include "DNA_material_types.h"
 #include "DNA_scene_types.h"
@@ -47,6 +48,7 @@
 
 #include "BLI_arithb.h"
 #include "BLI_blenlib.h"
+#include "BLI_rand.h"
 #include "BLI_threads.h"
 
 #include "PIL_time.h"
@@ -57,7 +59,7 @@
 /* not very important, but the stack solver likes to know a maximum */
 #define MAX_SOCKET	64
 
-#pragma mark /* ************** Type stuff **********  */
+/* ************** Type stuff **********  */
 
 static bNodeType *node_get_type(bNodeTree *ntree, int type, bNodeTree *ngroup)
 {
@@ -214,7 +216,8 @@ void ntreeVerifyTypes(bNodeTree *ntree)
 {
 	bNode *node;
 	
-	if((ntree->init & NTREE_TYPE_INIT)==0)
+	/* commented out, in linux a 2nd initialize on a Scene ntree from a lib crashes... */
+/*	if((ntree->init & NTREE_TYPE_INIT)==0) */
 		ntreeInitTypes(ntree);
 	
 	/* check inputs and outputs, and remove or insert them */
@@ -223,7 +226,7 @@ void ntreeVerifyTypes(bNodeTree *ntree)
 	
 }
 
-#pragma mark /* ************** Group stuff ********** */
+/* ************** Group stuff ********** */
 
 bNodeType node_group_typeinfo= {
 	/* type code   */	NODE_GROUP,
@@ -509,6 +512,17 @@ bNode *nodeMakeGroupFromSelected(bNodeTree *ntree)
 		}
 	}
 	
+	/* initialize variables of unused input sockets */
+	for(node= ngroup->nodes.first; node; node= node->next) {
+		for(sock= node->inputs.first; sock; sock= sock->next) {
+			if(sock->intern==0) {
+				bNodeSocket *nsock= groupnode_find_tosock(gnode, sock->own_index);
+				if(nsock) {
+					QUATCOPY(nsock->ns.vec, sock->ns.vec);
+				}
+			}
+		}
+	}
 	return gnode;
 }
 
@@ -628,7 +642,7 @@ static void find_node_with_socket(bNodeTree *ntree, bNodeSocket *sock, bNode **n
 {
 	bNode *node;
 	bNodeSocket *tsock;
-	int index;
+	int index= 0;
 	
 	for(node= ntree->nodes.first; node; node= node->next) {
 		for(index=0, tsock= node->inputs.first; tsock; tsock= tsock->next, index++)
@@ -664,7 +678,7 @@ int nodeGroupUnGroup(bNodeTree *ntree, bNode *gnode)
 	
 	/* clear new pointers, set in copytree */
 	for(node= ntree->nodes.first; node; node= node->next)
-		node->new= NULL;
+		node->new_node= NULL;
 
 	wgroup= ntreeCopyTree(ngroup, 0);
 	
@@ -690,9 +704,9 @@ int nodeGroupUnGroup(bNodeTree *ntree, bNode *gnode)
 			/* link->tosock->tosock is on the node we look for */
 			find_node_with_socket(ngroup, link->tosock->tosock, &nextn, &index);
 			if(nextn==NULL) printf("wrong stuff!\n");
-			else if(nextn->new==NULL) printf("wrong stuff too!\n");
+			else if(nextn->new_node==NULL) printf("wrong stuff too!\n");
 			else {
-				link->tonode= nextn->new;
+				link->tonode= nextn->new_node;
 				link->tosock= BLI_findlink(&link->tonode->inputs, index);
 			}
 		}
@@ -700,9 +714,9 @@ int nodeGroupUnGroup(bNodeTree *ntree, bNode *gnode)
 			/* link->fromsock->tosock is on the node we look for */
 			find_node_with_socket(ngroup, link->fromsock->tosock, &nextn, &index);
 			if(nextn==NULL) printf("1 wrong stuff!\n");
-			else if(nextn->new==NULL) printf("1 wrong stuff too!\n");
+			else if(nextn->new_node==NULL) printf("1 wrong stuff too!\n");
 			else {
-				link->fromnode= nextn->new;
+				link->fromnode= nextn->new_node;
 				link->fromsock= BLI_findlink(&link->fromnode->outputs, index);
 			}
 		}
@@ -713,10 +727,14 @@ int nodeGroupUnGroup(bNodeTree *ntree, bNode *gnode)
 	
 	nodeFreeNode(ntree, gnode);
 	
+	/* solve order goes fine, but the level tags not... doing it twice works for now. solve this once */
+	ntreeSolveOrder(ntree);
+	ntreeSolveOrder(ntree);
+
 	return 1;
 }
 
-#pragma mark /* ************** Add stuff ********** */
+/* ************** Add stuff ********** */
 
 bNode *nodeAddNodeType(bNodeTree *ntree, int type, bNodeTree *ngroup)
 {
@@ -767,6 +785,8 @@ bNode *nodeAddNodeType(bNodeTree *ntree, int type, bNodeTree *ngroup)
 			node->storage= curvemapping_add(3, -1.0f, -1.0f, 1.0f, 1.0f);
 		else if(type==SH_NODE_CURVE_RGB)
 			node->storage= curvemapping_add(4, 0.0f, 0.0f, 1.0f, 1.0f);
+		else if(type==SH_NODE_GEOMETRY)
+			node->storage= MEM_callocN(sizeof(NodeGeometry), "NodeGeometry");
 	}
 	else if(ntree->type==NTREE_COMPOSIT) {
 		if(type==CMP_NODE_VALTORGB)
@@ -784,18 +804,101 @@ bNode *nodeAddNodeType(bNodeTree *ntree, int type, bNodeTree *ngroup)
 			node->storage= add_mapping();
 		else if(type==CMP_NODE_BLUR)
 			node->storage= MEM_callocN(sizeof(NodeBlurData), "node blur data");
+		else if(type==CMP_NODE_DEFOCUS) {
+			/* qdn: defocus node */
+			NodeDefocus *nbd = MEM_callocN(sizeof(NodeDefocus), "node defocus data");
+			nbd->bktype = 0;
+			nbd->rotation = 0.f;
+			nbd->preview = 1;
+			nbd->gamco = 0;
+			nbd->samples = 16;
+			nbd->fstop = 128.f;
+			nbd->maxblur = 0;
+			nbd->bthresh = 1.f;
+			nbd->scale = 1.f;
+			nbd->no_zbuf = 1;
+			node->storage = nbd;
+		}
 		else if(type==CMP_NODE_VECBLUR) {
 			NodeBlurData *nbd= MEM_callocN(sizeof(NodeBlurData), "node blur data");
 			node->storage= nbd;
 			nbd->samples= 32;
 			nbd->fac= 1.0f;
 		}
+		else if(ELEM3(type, CMP_NODE_IMAGE, CMP_NODE_VIEWER, CMP_NODE_SPLITVIEWER)) {
+			ImageUser *iuser= MEM_callocN(sizeof(ImageUser), "node image user");
+			node->storage= iuser;
+			iuser->sfra= 1;
+			iuser->fie_ima= 2;
+			iuser->ok= 1;
+			
+			if(type==CMP_NODE_SPLITVIEWER){
+				node->custom1= 50;	/* default 50% split */
+			}
+		}
 		else if(type==CMP_NODE_HUE_SAT) {
 			NodeHueSat *nhs= MEM_callocN(sizeof(NodeHueSat), "node hue sat");
 			node->storage= nhs;
 			nhs->hue= 0.5f;
 			nhs->sat= 1.0f;
+			nhs->val= 1.0f;
 		}
+		else if(type==CMP_NODE_OUTPUT_FILE) {
+			NodeImageFile *nif= MEM_callocN(sizeof(NodeImageFile), "node image file");
+			node->storage= nif;
+			BLI_strncpy(nif->name, G.scene->r.pic, sizeof(nif->name));
+			nif->imtype= G.scene->r.imtype;
+			nif->subimtype= G.scene->r.subimtype;
+			nif->quality= G.scene->r.quality;
+			nif->sfra= G.scene->r.sfra;
+			nif->efra= G.scene->r.efra;
+		}
+		else if(type==CMP_NODE_DIFF_MATTE){
+			NodeChroma *c= MEM_callocN(sizeof(NodeChroma), "node chroma");
+			node->storage= c;
+			c->t1= 0.01f;
+			c->t2= 0.01f;
+			c->t3= 0.01f;
+			c->fsize= 0.0f;
+			c->fstrength= 0.0f;
+			node->custom1= 1; /* RGB */
+		}
+		else if(type==CMP_NODE_COLOR_SPILL){
+			NodeChroma *c= MEM_callocN(sizeof(NodeChroma), "node chroma");
+			node->storage=c;
+			c->t1= 0.0f;
+			c->t2= 0.0f;
+			c->t3= 0.0f;
+			c->fsize= 0.0f;
+			c->fstrength= 0.0f;
+			node->custom1= 2; /* green channel */
+		}
+		else if(type==CMP_NODE_CHROMA){
+			NodeChroma *c= MEM_callocN(sizeof(NodeChroma), "node chroma");
+			node->storage= c;
+			c->t1= 30.0f;
+			c->t2= 10.0f;
+			c->t3= 0.0f;
+			c->fsize= 0.0f;
+			c->fstrength= 1.0f;
+		}
+      else if(type==CMP_NODE_CHANNEL_MATTE){
+         NodeChroma *c= MEM_callocN(sizeof(NodeChroma), "node chroma");
+         node->storage=c;
+         c->t1= 0.0f;
+         c->t2= 0.0f;
+         c->t3= 0.0f;
+         c->fsize= 0.0f;
+         c->fstrength= 0.0f;
+         node->custom1= 1; /* RGB channel */
+         node->custom2= 2; /* Green Channel */
+      }
+      else if(type==CMP_NODE_LUMA_MATTE){
+         NodeChroma *c= MEM_callocN(sizeof(NodeChroma), "node chroma");
+         node->storage=c;
+         c->t1= 0.0f;
+         c->t2= 0.0f;
+      }
 	}
 	
 	return node;
@@ -843,8 +946,8 @@ bNode *nodeCopyNode(struct bNodeTree *ntree, struct bNode *node)
 			nnode->storage= MEM_dupallocN(nnode->storage);
 	}
 	
-	node->new= nnode;
-	nnode->new= NULL;
+	node->new_node= nnode;
+	nnode->new_node= NULL;
 	nnode->preview= NULL;
 
 	return nnode;
@@ -908,7 +1011,7 @@ bNodeTree *ntreeCopyTree(bNodeTree *ntree, int internal_select)
 	last= ntree->nodes.last;
 	for(node= ntree->nodes.first; node; node= node->next) {
 		
-		node->new= NULL;
+		node->new_node= NULL;
 		if(internal_select==0 || (node->flag & NODE_SELECT)) {
 			nnode= nodeCopyNode(newtree, node);	/* sets node->new */
 			if(internal_select) {
@@ -922,20 +1025,20 @@ bNodeTree *ntreeCopyTree(bNodeTree *ntree, int internal_select)
 	
 	/* check for copying links */
 	for(link= ntree->links.first; link; link= link->next) {
-		if(link->fromnode->new && link->tonode->new) {
-			nlink= nodeAddLink(newtree, link->fromnode->new, NULL, link->tonode->new, NULL);
+		if(link->fromnode->new_node && link->tonode->new_node) {
+			nlink= nodeAddLink(newtree, link->fromnode->new_node, NULL, link->tonode->new_node, NULL);
 			/* sockets were copied in order */
 			for(a=0, sock= link->fromnode->outputs.first; sock; sock= sock->next, a++) {
 				if(sock==link->fromsock)
 					break;
 			}
-			nlink->fromsock= BLI_findlink(&link->fromnode->new->outputs, a);
+			nlink->fromsock= BLI_findlink(&link->fromnode->new_node->outputs, a);
 			
 			for(a=0, sock= link->tonode->inputs.first; sock; sock= sock->next, a++) {
 				if(sock==link->tosock)
 					break;
 			}
-			nlink->tosock= BLI_findlink(&link->tonode->new->inputs, a);
+			nlink->tosock= BLI_findlink(&link->tonode->new_node->inputs, a);
 		}
 	}
 	
@@ -956,7 +1059,7 @@ bNodeTree *ntreeCopyTree(bNodeTree *ntree, int internal_select)
 	return newtree;
 }
 
-#pragma mark /* ************** Free stuff ********** */
+/* ************** Free stuff ********** */
 
 /* goes over entire tree */
 static void node_unlink_node(bNodeTree *ntree, bNode *node)
@@ -1006,8 +1109,7 @@ void nodeFreeNode(bNodeTree *ntree, bNode *node)
 	node_unlink_node(ntree, node);
 	BLI_remlink(&ntree->nodes, node);
 
-	if(node->id)
-		node->id->us--;
+	/* since it is called while free database, node->id is undefined */
 	
 	if(ntree->type==NTREE_COMPOSIT)
 		composit_free_node_cache(node);
@@ -1181,7 +1283,7 @@ void ntreeMakeLocal(bNodeTree *ntree)
 }
 
 
-#pragma mark /* ************ find stuff *************** */
+/* ************ find stuff *************** */
 
 static int ntreeHasType(bNodeTree *ntree, int type)
 {
@@ -1300,7 +1402,7 @@ void ntreeSocketUseFlags(bNodeTree *ntree)
 	}
 }
 
-#pragma mark /* ************** dependency stuff *********** */
+/* ************** dependency stuff *********** */
 
 /* node is guaranteed to be not checked before */
 static int node_recurs_check(bNode *node, bNode ***nsort, int level)
@@ -1385,6 +1487,11 @@ void ntreeSolveOrder(bNodeTree *ntree)
 		if(node->typeinfo->nclass==NODE_CLASS_OUTPUT) {
 			bNode *tnode;
 			int output= 0;
+			
+			/* we need a check for which output node should be tagged like this, below an exception */
+			if(node->type==CMP_NODE_OUTPUT_FILE)
+			   continue;
+			   
 			/* there is more types having output class, each one is checked */
 			for(tnode= ntree->nodes.first; tnode; tnode= tnode->next) {
 				if(tnode->typeinfo->nclass==NODE_CLASS_OUTPUT) {
@@ -1417,10 +1524,10 @@ void NodeTagChanged(bNodeTree *ntree, bNode *node)
 				free_compbuf(sock->ns.data);
 				sock->ns.data= NULL;
 				
-				if(node->preview && node->preview->rect) {
-					MEM_freeN(node->preview->rect);
-					node->preview->rect= NULL;
-				}
+				//if(node->preview && node->preview->rect) {
+				//	MEM_freeN(node->preview->rect);
+				//	node->preview->rect= NULL;
+				//}
 					
 			}
 		}
@@ -1430,6 +1537,9 @@ void NodeTagChanged(bNodeTree *ntree, bNode *node)
 
 void NodeTagIDChanged(bNodeTree *ntree, ID *id)
 {
+	if(id==NULL)
+		return;
+	
 	if(ntree->type==NTREE_COMPOSIT) {
 		bNode *node;
 		
@@ -1440,7 +1550,7 @@ void NodeTagIDChanged(bNodeTree *ntree, ID *id)
 }
 
 
-#pragma mark /* *************** preview *********** */
+/* *************** preview *********** */
 
 /* if node->preview, then we assume the rect to exist */
 
@@ -1530,7 +1640,7 @@ void nodeAddToPreview(bNode *node, float *col, int x, int y)
 
 
 
-#pragma mark /* ******************* executing ************* */
+/* ******************* executing ************* */
 
 /* see notes at ntreeBeginExecTree */
 static void group_node_get_stack(bNode *node, bNodeStack *stack, bNodeStack **in, bNodeStack **out, bNodeStack **gin, bNodeStack **gout)
@@ -1650,7 +1760,7 @@ static void composit_begin_exec(bNodeTree *ntree, int is_group)
 	for(node= ntree->nodes.first; node; node= node->next) {
 		if(is_group==0) {
 			for(sock= node->outputs.first; sock; sock= sock->next) {
-				bNodeStack *ns= ntree->stack + sock->stack_index;
+				bNodeStack *ns= ntree->stack[0] + sock->stack_index;
 				
 				if(sock->ns.data) {
 					ns->data= sock->ns.data;
@@ -1683,7 +1793,7 @@ static void composit_end_exec(bNodeTree *ntree, int is_group)
 			bNodeSocket *sock;
 		
 			for(sock= node->outputs.first; sock; sock= sock->next) {
-				ns= ntree->stack + sock->stack_index;
+				ns= ntree->stack[0] + sock->stack_index;
 				if(ns->data) {
 					sock->ns.data= ns->data;
 					ns->data= NULL;
@@ -1701,7 +1811,7 @@ static void composit_end_exec(bNodeTree *ntree, int is_group)
 	
 	if(is_group==0) {
 		/* internally, group buffers are not stored */
-		for(ns= ntree->stack, a=0; a<ntree->stacksize; a++, ns++) {
+		for(ns= ntree->stack[0], a=0; a<ntree->stacksize; a++, ns++) {
 			if(ns->data) {
 				printf("freed leftover buffer from stack\n");
 				free_compbuf(ns->data);
@@ -1726,7 +1836,10 @@ static void group_tag_used_outputs(bNode *gnode, bNodeStack *stack)
 					if(sock->link) {
 						bNodeStack *ns= stack + sock->link->fromsock->stack_index;
 						ns->hasoutput= 1;
+						ns->sockettype= sock->link->fromsock->type;
 					}
+					else
+						sock->ns.sockettype= sock->type;
 				}
 			}
 		}
@@ -1745,6 +1858,9 @@ void ntreeBeginExecTree(bNodeTree *ntree)
 	if(ntree->init & NTREE_EXEC_INIT)
 		return;
 	
+	/* allocate the stack pointer array */
+	ntree->stack= MEM_callocN(BLENDER_MAX_THREADS*sizeof(void *), "stack array");
+	
 	/* goes recursive over all groups */
 	ntree->stacksize= ntree_begin_exec_tree(ntree);
 
@@ -1753,8 +1869,8 @@ void ntreeBeginExecTree(bNodeTree *ntree)
 		bNodeStack *ns;
 		int a;
 		
-		/* allocate stack */
-		ns=ntree->stack= MEM_callocN(ntree->stacksize*sizeof(bNodeStack), "node stack");
+		/* allocate the base stack */
+		ns=ntree->stack[0]= MEM_callocN(ntree->stacksize*sizeof(bNodeStack), "node stack");
 		
 		/* tag inputs, the get_stack() gives own socket stackdata if not in use */
 		for(a=0; a<ntree->stacksize; a++, ns++) ns->hasinput= 1;
@@ -1764,17 +1880,24 @@ void ntreeBeginExecTree(bNodeTree *ntree)
 			bNodeSocket *sock;
 			for(sock= node->inputs.first; sock; sock= sock->next) {
 				if(sock->link) {
-					ns= ntree->stack + sock->link->fromsock->stack_index;
+					ns= ntree->stack[0] + sock->link->fromsock->stack_index;
 					ns->hasoutput= 1;
+					ns->sockettype= sock->link->fromsock->type;
 				}
+				else
+					sock->ns.sockettype= sock->type;
 			}
 			if(node->type==NODE_GROUP && node->id)
-				group_tag_used_outputs(node, ntree->stack);
+				group_tag_used_outputs(node, ntree->stack[0]);
 		}
+		
+		/* composite does 1 node per thread, so no multiple stacks needed */
 		if(ntree->type==NTREE_COMPOSIT)
 			composit_begin_exec(ntree, 0);
-		else
-			ntree->stack1= MEM_dupallocN(ntree->stack);
+		else {
+			for(a=1; a<BLENDER_MAX_THREADS; a++)
+				ntree->stack[a]= MEM_dupallocN(ntree->stack[0]);
+		}
 	}
 	
 	ntree->init |= NTREE_EXEC_INIT;
@@ -1784,18 +1907,20 @@ void ntreeEndExecTree(bNodeTree *ntree)
 {
 	
 	if(ntree->init & NTREE_EXEC_INIT) {
+		int a;
 		
 		/* another callback candidate! */
 		if(ntree->type==NTREE_COMPOSIT)
 			composit_end_exec(ntree, 0);
 		
-		if(ntree->stack)
-			MEM_freeN(ntree->stack);
-		ntree->stack= NULL;
+		if(ntree->stack) {
+			for(a=0; a<BLENDER_MAX_THREADS; a++)
+				if(ntree->stack[a])
+					MEM_freeN(ntree->stack[a]);
 		
-		if(ntree->stack1)
-			MEM_freeN(ntree->stack1);
-		ntree->stack1= NULL;
+			MEM_freeN(ntree->stack);
+			ntree->stack= NULL;
+		}
 
 		ntree->init &= ~NTREE_EXEC_INIT;
 	}
@@ -1830,10 +1955,7 @@ void ntreeExecTree(bNodeTree *ntree, void *callerdata, int thread)
 	if((ntree->init & NTREE_EXEC_INIT)==0)
 		ntreeBeginExecTree(ntree);
 		
-	if(thread)
-		stack= ntree->stack1;
-	else
-		stack= ntree->stack;
+	stack= ntree->stack[thread];
 	
 	for(node= ntree->nodes.first; node; node= node->next) {
 		if(node->typeinfo->execfunc) {
@@ -1866,7 +1988,7 @@ static void *exec_composite_node(void *node_v)
 	bNodeStack *nsin[MAX_SOCKET];	/* arbitrary... watch this */
 	bNodeStack *nsout[MAX_SOCKET];	/* arbitrary... watch this */
 	bNode *node= node_v;
-	ThreadData *thd= (ThreadData *)node->new;
+	ThreadData *thd= (ThreadData *)node->new_node; /* abuse */
 	
 	node_get_stack(node, thd->stack, nsin, nsout);
 	
@@ -1881,6 +2003,27 @@ static void *exec_composite_node(void *node_v)
 	return 0;
 }
 
+/* these are nodes without input, only giving values */
+/* or nodes with only value inputs */
+static int node_only_value(bNode *node)
+{
+	bNodeSocket *sock;
+	
+	if(ELEM3(node->type, CMP_NODE_TIME, CMP_NODE_VALUE, CMP_NODE_RGB))
+		return 1;
+	
+	/* doing this for all node types goes wrong. memory free errors */
+	if(node->inputs.first && node->type==CMP_NODE_MAP_VALUE) {
+		int retval= 1;
+		for(sock= node->inputs.first; sock; sock= sock->next) {
+			if(sock->link)
+				retval &= node_only_value(sock->link->fromnode);
+		}
+		return retval;
+	}
+	return 0;
+}
+
 /* return total of executable nodes, for timecursor */
 /* only compositor uses it */
 static int setExecutableNodes(bNodeTree *ntree, ThreadData *thd)
@@ -1905,24 +2048,27 @@ static int setExecutableNodes(bNodeTree *ntree, ThreadData *thd)
 		node_get_stack(node, thd->stack, nsin, nsout);
 		
 		/* test the outputs */
-		for(a=0, sock= node->outputs.first; sock; sock= sock->next, a++) {
-			if(nsout[a]->data==NULL && nsout[a]->hasoutput) {
-				node->need_exec= 1;
-				break;
+		/* skip value-only nodes (should be in type!) */
+		if(!node_only_value(node)) {
+			for(a=0, sock= node->outputs.first; sock; sock= sock->next, a++) {
+				if(nsout[a]->data==NULL && nsout[a]->hasoutput) {
+					node->need_exec= 1;
+					break;
+				}
 			}
 		}
 		
 		/* test the inputs */
 		for(a=0, sock= node->inputs.first; sock; sock= sock->next, a++) {
 			/* skip viewer nodes in bg render or group edit */
-			if(node->type==CMP_NODE_VIEWER && (G.background || group_edit))
+			if( ELEM(node->type, CMP_NODE_VIEWER, CMP_NODE_SPLITVIEWER) && (G.background || group_edit))
 				node->need_exec= 0;
 			/* is sock in use? */
 			else if(sock->link) {
 				bNodeLink *link= sock->link;
 				/* this is the test for a cyclic case */
 				if(link->fromnode->level >= link->tonode->level && link->tonode->level!=0xFFF) {
-					if(nsin[a]->data==NULL || sock->link->fromnode->need_exec) {
+					if(link->fromnode->need_exec) {
 						node->need_exec= 1;
 						break;
 					}
@@ -1944,15 +2090,31 @@ static int setExecutableNodes(bNodeTree *ntree, ThreadData *thd)
 				}
 			}
 			totnode++;
-			// printf("node needs exec %s\n", node->name);
+			/* printf("node needs exec %s\n", node->name); */
 			
 			/* tag for getExecutableNode() */
 			node->exec= 0;
 		}
-		else
+		else {
 			/* tag for getExecutableNode() */
 			node->exec= NODE_READY|NODE_FINISHED;
+			
+		}
 	}
+	
+	/* last step: set the stack values for only-value nodes */
+	/* just does all now, compared to a full buffer exec this is nothing */
+	if(totnode) {
+		for(node= ntree->nodes.first; node; node= node->next) {
+			if(node->need_exec==0 && node_only_value(node)) {
+				if(node->typeinfo->execfunc) {
+					node_get_stack(node, thd->stack, nsin, nsout);
+					node->typeinfo->execfunc(thd->rd, node, nsin, nsout);
+				}
+			}
+		}
+	}
+	
 	return totnode;
 }
 
@@ -1984,7 +2146,7 @@ static void freeExecutableNode(bNodeTree *ntree)
 	for(node= ntree->nodes.first; node; node= node->next) {
 		if(node->exec & NODE_FREEBUFS) {
 			for(sock= node->outputs.first; sock; sock= sock->next) {
-				bNodeStack *ns= ntree->stack + sock->stack_index;
+				bNodeStack *ns= ntree->stack[0] + sock->stack_index;
 				if(ns->data) {
 					free_compbuf(ns->data);
 					ns->data= NULL;
@@ -2023,15 +2185,10 @@ void ntreeCompositExecTree(bNodeTree *ntree, RenderData *rd, int do_preview)
 	bNode *node;
 	ListBase threads;
 	ThreadData thdata;
-	int totnode, maxthreads, rendering= 1;
+	int totnode, rendering= 1;
 	
 	if(ntree==NULL) return;
 	
-	if(rd->mode & R_THREADS)
-		maxthreads= 2;
-	else
-		maxthreads= 1;
-	
 	if(do_preview)
 		ntreeInitPreview(ntree, 0, 0);
 	
@@ -2043,12 +2200,15 @@ void ntreeCompositExecTree(bNodeTree *ntree, RenderData *rd, int do_preview)
 	
 	/* setup callerdata for thread callback */
 	thdata.rd= rd;
-	thdata.stack= ntree->stack;
+	thdata.stack= ntree->stack[0];
 	
+	/* fixed seed, for example noise texture */
+	BLI_srandom(rd->cfra);
+
 	/* sets need_exec tags in nodes */
 	totnode= setExecutableNodes(ntree, &thdata);
 	
-	BLI_init_threads(&threads, exec_composite_node, maxthreads);
+	BLI_init_threads(&threads, exec_composite_node, rd->threads);
 	
 	while(rendering) {
 		
@@ -2065,7 +2225,7 @@ void ntreeCompositExecTree(bNodeTree *ntree, RenderData *rd, int do_preview)
 				}
 				totnode--;
 				
-				node->new = (bNode *)&thdata;
+				node->new_node = (bNode *)&thdata;
 				node->exec= NODE_PROCESSING;
 				BLI_insert_thread(&threads, node);
 			}
@@ -2103,8 +2263,6 @@ void ntreeCompositExecTree(bNodeTree *ntree, RenderData *rd, int do_preview)
 	
 	ntreeEndExecTree(ntree);
 	
-	free_unused_animimages();
-	
 }
 
 
