@@ -1,7 +1,7 @@
 /* object.c
  *
  * 
- * $Id: object.c,v 1.100 2006/07/20 15:27:55 n_t Exp $
+ * $Id: object.c,v 1.123 2006/12/27 10:21:33 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -111,7 +111,7 @@
 #include "BPY_extern.h"
 
 /* Local function protos */
-static void solve_parenting (Object *ob, Object *par, float slowmat[][4], int simul);
+static void solve_parenting (Object *ob, Object *par, float obmat[][4], float slowmat[][4], int simul);
 
 float originmat[3][3];	/* after where_is_object(), can be used in other functions (bad!) */
 Object workob;
@@ -170,10 +170,12 @@ void object_free_modifiers(Object *ob)
 void object_free_display(Object *ob)
 {
 	if(ob->derivedDeform) {
+		ob->derivedDeform->needsFree = 1;
 		ob->derivedDeform->release(ob->derivedDeform);
 		ob->derivedDeform= NULL;
 	}
 	if(ob->derivedFinal) {
+		ob->derivedFinal->needsFree = 1;
 		ob->derivedFinal->release(ob->derivedFinal);
 		ob->derivedFinal= NULL;
 	}
@@ -266,75 +268,90 @@ void unlink_object(Object *ob)
 	unlink_controllers(&ob->controllers);
 	unlink_actuators(&ob->actuators);
 	
-	/* check all objects: parents en bevels and fields */
+	/* check all objects: parents en bevels and fields, also from libraries */
 	obt= G.main->object.first;
 	while(obt) {
-		if(obt->id.lib==NULL) {
-			
-			if(obt->parent==ob) {
-				obt->parent= NULL;
+		if(obt->proxy==ob)
+			obt->proxy= NULL;
+		if(obt->proxy_from==ob) {
+			obt->proxy_from= NULL;
+			obt->recalc |= OB_RECALC_OB;
+		}
+		if(obt->proxy_group==ob)
+			obt->proxy_group= NULL;
+		
+		if(obt->parent==ob) {
+			obt->parent= NULL;
+			obt->recalc |= OB_RECALC;
+		}
+		
+		if(obt->track==ob) {
+			obt->track= NULL;
+			obt->recalc |= OB_RECALC_OB;
+		}
+		
+		modifiers_foreachObjectLink(obt, unlink_object__unlinkModifierLinks, ob);
+		
+		if ELEM(obt->type, OB_CURVE, OB_FONT) {
+			cu= obt->data;
+
+			if(cu->bevobj==ob) {
+				cu->bevobj= NULL;
 				obt->recalc |= OB_RECALC;
 			}
-			
-			if(obt->track==ob) {
-				obt->track= NULL;
-				obt->recalc |= OB_RECALC_OB;
+			if(cu->taperobj==ob) {
+				cu->taperobj= NULL;
+				obt->recalc |= OB_RECALC;
 			}
-			
-			modifiers_foreachObjectLink(obt, unlink_object__unlinkModifierLinks, ob);
-			
-			if ELEM(obt->type, OB_CURVE, OB_FONT) {
-				cu= obt->data;
-				if(cu->bevobj==ob) {
-					cu->bevobj= NULL;
-					obt->recalc |= OB_RECALC;
-				}
-				if(cu->taperobj==ob) {
-					cu->taperobj= NULL;
-					obt->recalc |= OB_RECALC;
-				}
-				if(cu->textoncurve==ob) {
-					cu->textoncurve= NULL;
-					obt->recalc |= OB_RECALC;
-				}
-			}
-			else if(obt->type==OB_ARMATURE && obt->pose) {
-				bPoseChannel *pchan;
-				for(pchan= obt->pose->chanbase.first; pchan; pchan= pchan->next) {
-					for (con = pchan->constraints.first; con; con=con->next) {
-						if(ob==get_constraint_target(con, &str)) {
-							set_constraint_target(con, NULL, NULL);
-							obt->recalc |= OB_RECALC_DATA;
-						}
-					}
-					if(pchan->custom==ob)
-						pchan->custom= NULL;
-				}
-			}
-			
-			sca_remove_ob_poin(obt, ob);
-			
-			for (con = obt->constraints.first; con; con=con->next) {
-				if(ob==get_constraint_target(con, &str)) {
-					set_constraint_target(con, NULL, NULL);
-					obt->recalc |= OB_RECALC_OB;
-				}
-			}
-			
-			/* object is deflector or field */
-			if(ob->pd) {
-				if(give_parteff(obt))
-					obt->recalc |= OB_RECALC_DATA;
-				else if(obt->soft)
-					obt->recalc |= OB_RECALC_DATA;
-			}
-			
-			/* strips */
-			for(strip= ob->nlastrips.first; strip; strip= strip->next) {
-				if(strip->object==ob)
-					strip->object= NULL;
+			if(cu->textoncurve==ob) {
+				cu->textoncurve= NULL;
+				obt->recalc |= OB_RECALC;
 			}
 		}
+		else if(obt->type==OB_ARMATURE && obt->pose) {
+			bPoseChannel *pchan;
+			for(pchan= obt->pose->chanbase.first; pchan; pchan= pchan->next) {
+				for (con = pchan->constraints.first; con; con=con->next) {
+					if(ob==get_constraint_target(con, &str)) {
+						set_constraint_target(con, NULL, NULL);
+						obt->recalc |= OB_RECALC_DATA;
+					}
+				}
+				if(pchan->custom==ob)
+					pchan->custom= NULL;
+			}
+		}
+		
+		sca_remove_ob_poin(obt, ob);
+		
+		for (con = obt->constraints.first; con; con=con->next) {
+			if(ob==get_constraint_target(con, &str)) {
+				set_constraint_target(con, NULL, NULL);
+				obt->recalc |= OB_RECALC_OB;
+			}
+		}
+		
+		/* object is deflector or field */
+		if(ob->pd) {
+			if(give_parteff(obt))
+				obt->recalc |= OB_RECALC_DATA;
+			else if(obt->soft)
+				obt->recalc |= OB_RECALC_DATA;
+		}
+		
+		/* strips */
+		for(strip= obt->nlastrips.first; strip; strip= strip->next) {
+			if(strip->object==ob)
+				strip->object= NULL;
+			
+			if(strip->modifiers.first) {
+				bActionModifier *amod;
+				for(amod= strip->modifiers.first; amod; amod= amod->next)
+					if(amod->ob==ob)
+						amod->ob= NULL;
+			}
+		}
+
 		obt= obt->id.next;
 	}
 	
@@ -569,7 +586,7 @@ void *add_lamp(void)
 	la->spotsize= 45.0;
 	la->spotblend= 0.15;
 	la->att2= 1.0;
-	la->mode= LA_SHAD;
+	la->mode= LA_SHAD_BUF;
 	la->bufsize= 512;
 	la->clipsta= 0.5;
 	la->clipend= 40.0;
@@ -580,6 +597,7 @@ void *add_lamp(void)
 	la->ray_samp= la->ray_sampy= la->ray_sampz= 1; 
 	la->area_size=la->area_sizey=la->area_sizez= 1.0; 
 	la->buffers= 1;
+	la->buftype= LA_SHADBUF_HALFWAY;
 	
 	return la;
 }
@@ -731,29 +749,24 @@ static char *get_obdata_defname(int type)
 	}
 }
 
-/* general add: to G.scene, with layer from area and default name */
-/* creates minimum required data, but without vertices etc. */
-Object *add_object(int type)
+/* more general add: creates minimum required data, but without vertices etc. */
+Object *add_only_object(int type, char *name)
 {
 	Object *ob;
-	Base *base;
-	char name[32];
 
-	strcpy(name, get_obdata_defname(type));
-	
 	ob= alloc_libblock(&G.main->object, ID_OB, name);
 	G.totobj++;
-	
+
 	/* default object vars */
 	ob->type= type;
 	/* ob->transflag= OB_QUAT; */
-	
+
 	QuatOne(ob->quat);
 	QuatOne(ob->dquat);
 
 	ob->col[0]= ob->col[1]= ob->col[2]= 0.0;
 	ob->col[3]= 1.0;
-	
+
 	ob->loc[0]= ob->loc[1]= ob->loc[2]= 0.0;
 	ob->rot[0]= ob->rot[1]= ob->rot[2]= 0.0;
 	ob->size[0]= ob->size[1]= ob->size[2]= 1.0;
@@ -764,7 +777,7 @@ Object *add_object(int type)
 	if(U.flag & USER_MAT_ON_OB) ob->colbits= -1;
 	ob->empty_drawtype= OB_ARROWS;
 	ob->empty_drawsize= 1.0;
-	
+
 	if(type==OB_CAMERA || type==OB_LAMP) {
 		ob->trackflag= OB_NEGZ;
 		ob->upflag= OB_POSY;
@@ -791,15 +804,29 @@ Object *add_object(int type)
 	/* NT fluid sim defaults */
 	ob->fluidsimFlag = 0;
 	ob->fluidsimSettings = NULL;
-	
+
+	return ob;
+}
+
+/* general add: to G.scene, with layer from area and default name */
+/* creates minimum required data, but without vertices etc. */
+Object *add_object(int type)
+{
+	Object *ob;
+	Base *base;
+	char name[32];
+
+	strcpy(name, get_obdata_defname(type));
+	ob = add_only_object(type, name);
+
 	ob->data= add_obdata_from_type(type);
-	
+
 	ob->lay= G.scene->lay;
 
 	base= scene_add_base(G.scene, ob);
 	scene_select_base(G.scene, base);
 	ob->recalc |= OB_RECALC;
-	
+
 	return ob;
 }
 
@@ -839,9 +866,27 @@ SoftBody *copy_softbody(SoftBody *sb)
 	sbn->keys= NULL;
 	sbn->totkey= sbn->totpointkey= 0;
 	
+	sbn->scratch= NULL;
 	return sbn;
 }
 
+static void copy_object_pose(Object *obn, Object *ob)
+{
+	bPoseChannel *chan;
+	
+	copy_pose(&obn->pose, ob->pose, 1);
+
+	for (chan = obn->pose->chanbase.first; chan; chan=chan->next){
+		bConstraint *con;
+		char *str;
+		chan->flag &= ~(POSE_LOC|POSE_ROT|POSE_SIZE);
+		for(con= chan->constraints.first; con; con= con->next) {
+			if(ob==get_constraint_target(con, &str))
+				set_constraint_target(con, obn, NULL);
+		}
+	}
+}
+
 Object *copy_object(Object *ob)
 {
 	Object *obn;
@@ -855,7 +900,7 @@ Object *copy_object(Object *ob)
 	}
 	
 	if(ob->bb) obn->bb= MEM_dupallocN(ob->bb);
-	obn->path= 0;
+	obn->path= NULL;
 	obn->flag &= ~OB_FROMGROUP;
 	
 	copy_effects(&obn->effect, &ob->effect);
@@ -877,7 +922,7 @@ Object *copy_object(Object *ob)
 	copy_actuators(&obn->actuators, &ob->actuators);
 	
 	if(ob->pose) {
-		copy_pose(&obn->pose, ob->pose, 1);
+		copy_object_pose(obn, ob);
 		/* backwards compat... non-armatures can get poses in older files? */
 		if(ob->type==OB_ARMATURE)
 			armature_rebuild_pose(obn, obn->data);
@@ -904,12 +949,20 @@ Object *copy_object(Object *ob)
 	/* NT copy fluid sim setting memory */
 	if(obn->fluidsimSettings) {
 		obn->fluidsimSettings = fluidsimSettingsCopy(ob->fluidsimSettings);
-		obn->fluidsimSettings->orgMesh = (Mesh *)obn->data;
+		/* copying might fail... */
+		if(obn->fluidsimSettings) {
+			obn->fluidsimSettings->orgMesh = (Mesh *)obn->data;
+		}
 	}
 	
 	obn->derivedDeform = NULL;
 	obn->derivedFinal = NULL;
 
+#ifdef WITH_VERSE
+	obn->vnode = NULL;
+#endif
+
+
 	return obn;
 }
 
@@ -943,9 +996,12 @@ void make_local_object(Object *ob)
 	    * - mixed: make copy
 	    */
 	
-	if(ob->id.lib==0) return;
+	if(ob->id.lib==NULL) return;
+	
+	ob->proxy= ob->proxy_from= NULL;
+	
 	if(ob->id.us==1) {
-		ob->id.lib= 0;
+		ob->id.lib= NULL;
 		ob->id.flag= LIB_LOCAL;
 		new_id(0, (ID *)ob, 0);
 
@@ -995,6 +1051,57 @@ void make_local_object(Object *ob)
 	expand_local_object(ob);
 }
 
+/* *************** PROXY **************** */
+
+/* proxy rule: lib_object->proxy_from == the one we borrow from, set temporally while object_update */
+/*             local_object->proxy == pointer to library object, saved in files and read */
+/*             local_object->proxy_group == pointer to group dupli-object, saved in files and read */
+
+void object_make_proxy(Object *ob, Object *target, Object *gob)
+{
+	/* paranoia checks */
+	if(ob->id.lib || target->id.lib==NULL) {
+		printf("cannot make proxy\n");
+		return;
+	}
+	
+	ob->proxy= target;
+	ob->proxy_group= gob;
+	id_lib_extern(&target->id);
+	
+	ob->recalc= target->recalc= OB_RECALC;
+	
+	/* copy transform */
+	if(gob) {
+		VECCOPY(ob->loc, gob->loc);
+		VECCOPY(ob->rot, gob->rot);
+		VECCOPY(ob->size, gob->size);
+	}
+	else {
+		VECCOPY(ob->loc, target->loc);
+		VECCOPY(ob->rot, target->rot);
+		VECCOPY(ob->size, target->size);
+	}
+	
+	ob->parent= target->parent;	/* libdata */
+	Mat4CpyMat4(ob->parentinv, target->parentinv);
+	ob->ipo= target->ipo;		/* libdata */
+	
+	/* skip constraints, constraintchannels, nla? */
+	
+	ob->type= target->type;
+	ob->data= target->data;
+	id_us_plus((ID *)ob->data);		/* ensures lib data becomes LIB_EXTERN */
+	
+	/* type conversions */
+	if(target->type == OB_ARMATURE) {
+		copy_object_pose(ob, target);	/* data copy, object pointers in constraints */
+		rest_pose(ob->pose);			/* clear all transforms in channels */
+		armature_rebuild_pose(ob, ob->data);	/* set all internal links */
+	}
+}
+
+
 /* *************** CALC ****************** */
 
 /* there is also a timing calculation in drawobject() */
@@ -1184,7 +1291,7 @@ static void ob_parbone(Object *ob, Object *par, float mat[][4])
 	/* Make sure the bone is still valid */
 	pchan= get_pose_channel(par->pose, ob->parsubstr);
 	if (!pchan){
-		printf ("Bone parent: no bone %s\n", ob->parsubstr);
+		printf ("Object %s with Bone parent: bone %s doesn't exist\n", ob->id.name+2, ob->parsubstr);
 		Mat4One(mat);
 		return;
 	}
@@ -1200,30 +1307,20 @@ static void ob_parbone(Object *ob, Object *par, float mat[][4])
 
 static void give_parvert(Object *par, int nr, float *vec)
 {
-	EditMesh *em = G.editMesh;
-	EditVert *eve;
-/*  	extern ListBase editNurb; already in bad lev calls */
-	Nurb *nu;
-	Curve *cu;
-	BPoint *bp;
-	BezTriple *bezt;
 	int a, count;
 	
-	vec[0]=vec[1]=vec[2]= 0.0;
+	vec[0]=vec[1]=vec[2]= 0.0f;
 	
 	if(par->type==OB_MESH) {
 		if(G.obedit && (par->data==G.obedit->data)) {
-			if(nr >= G.totvert) nr= 0;
-
-			count= 0;
-			eve= em->verts.first;
-			while(eve) {
-				if(count==nr) {
+			EditMesh *em = G.editMesh;
+			EditVert *eve;
+			
+			for(eve= em->verts.first; eve; eve= eve->next) {
+				if(eve->keyindex==nr) {
 					memcpy(vec, eve->co, 12);
 					break;
 				}
-				eve= eve->next;
-				count++;
 			}
 		}
 		else {
@@ -1241,7 +1338,11 @@ static void give_parvert(Object *par, int nr, float *vec)
 		}
 	}
 	else if ELEM(par->type, OB_CURVE, OB_SURF) {
-
+		Nurb *nu;
+		Curve *cu;
+		BPoint *bp;
+		BezTriple *bezt;
+		
 		cu= par->data;
 		nu= cu->nurb.first;
 		if(par==G.obedit) nu= editNurb.first;
@@ -1276,7 +1377,30 @@ static void give_parvert(Object *par, int nr, float *vec)
 		}
 
 	}
-	else return;
+	else if(par->type==OB_LATTICE) {
+		Lattice *latt= par->data;
+		BPoint *bp;
+		DispList *dl = find_displist(&par->disp, DL_VERTS);
+		float *co = dl?dl->verts:NULL;
+		
+		if(par==G.obedit) latt= editLatt;
+		
+		a= latt->pntsu*latt->pntsv*latt->pntsw;
+		count= 0;
+		bp= latt->def;
+		while(a--) {
+			if(count==nr) {
+				if(co)
+					memcpy(vec, co, 3*sizeof(float));
+				else
+					memcpy(vec, bp->vec, 3*sizeof(float));
+				break;
+			}
+			count++;
+			if(co) co+= 3;
+			else bp++;
+		}
+	}
 }
 
 static void ob_parvert3(Object *ob, Object *par, float mat[][4])
@@ -1286,7 +1410,7 @@ static void ob_parvert3(Object *ob, Object *par, float mat[][4])
 	/* in local ob space */
 	Mat4One(mat);
 	
-	if ELEM3(par->type, OB_MESH, OB_SURF, OB_CURVE) {
+	if ELEM4(par->type, OB_MESH, OB_SURF, OB_CURVE, OB_LATTICE) {
 		
 		give_parvert(par, ob->par1, v1);
 		give_parvert(par, ob->par2, v2);
@@ -1335,7 +1459,6 @@ int during_scriptlink(void) {
 
 void where_is_object_time(Object *ob, float ctime)
 {
-	Object *par;
 	float *fp1, *fp2, slowmat[4][4] = MAT4_UNITY;
 	float stime, fac1, fac2, vec[3];
 	int a;
@@ -1350,6 +1473,7 @@ void where_is_object_time(Object *ob, float ctime)
 	/* this is needed to be able to grab objects with ipos, otherwise it always freezes them */
 	stime= bsystem_time(ob, 0, ctime, 0.0);
 	if(stime != ob->ctime) {
+		
 		ob->ctime= stime;
 		
 		if(ob->ipo) {
@@ -1359,8 +1483,8 @@ void where_is_object_time(Object *ob, float ctime)
 		else 
 			do_all_object_actions(ob);
 		
-		/* do constraint ipos ... */
-		do_constraint_channels(&ob->constraints, &ob->constraintChannels, ctime);
+		/* do constraint ipos ..., note it needs stime */
+		do_constraint_channels(&ob->constraints, &ob->constraintChannels, stime);
 	}
 	else {
 		/* but, the drivers have to be done */
@@ -1368,22 +1492,24 @@ void where_is_object_time(Object *ob, float ctime)
 	}
 	
 	if(ob->parent) {
-		par= ob->parent;
-
+		Object *par= ob->parent;
+		
 		if(ob->ipoflag & OB_OFFS_PARENT) ctime-= ob->sf;
 		
 		/* hurms, code below conflicts with depgraph... (ton) */
+		/* and even worse, it gives bad effects for NLA stride too (try ctime != par->ctime, with MBlur) */
 		pop= 0;
-		if(no_parent_ipo==0 && ctime != par->ctime) {
+		if(no_parent_ipo==0 && stime != par->ctime) {
 		
 			// only for ipo systems? 
 			pushdata(par, sizeof(Object));
 			pop= 1;
 			
-			where_is_object_time(par, ctime);
+			if(par->proxy_from);	// was a copied matrix, no where_is! bad...
+			else where_is_object_time(par, ctime);
 		}
 		
-		solve_parenting(ob, par, slowmat, 0);
+		solve_parenting(ob, par, ob->obmat, slowmat, 0);
 
 		if(pop) {
 			poplast(par);
@@ -1415,6 +1541,7 @@ void where_is_object_time(Object *ob, float ctime)
 		
 	}
 
+	/* constraints need ctime, not stime. it calls where_is_object_time and bsystem_time */
 	solve_constraints (ob, TARGET_OBJECT, NULL, ctime);
 
 	if(ob->scriptlink.totscript && !during_script()) {
@@ -1427,17 +1554,17 @@ void where_is_object_time(Object *ob, float ctime)
 	else ob->transflag &= ~OB_NEG_SCALE;
 }
 
-static void solve_parenting (Object *ob, Object *par, float slowmat[][4], int simul)
+static void solve_parenting (Object *ob, Object *par, float obmat[][4], float slowmat[][4], int simul)
 {
 	float totmat[4][4];
 	float tmat[4][4];
-	float obmat[4][4];
+	float locmat[4][4];
 	float vec[3];
 	int ok;
 
-	object_to_mat4(ob, obmat);
+	object_to_mat4(ob, locmat);
 	
-	if(ob->partype & PARSLOW) Mat4CpyMat4(slowmat, ob->obmat);
+	if(ob->partype & PARSLOW) Mat4CpyMat4(slowmat, obmat);
 	
 
 	switch(ob->partype & PARTYPE) {
@@ -1486,10 +1613,10 @@ static void solve_parenting (Object *ob, Object *par, float slowmat[][4], int si
 	// total 
 	Mat4MulSerie(tmat, totmat, ob->parentinv,         
 		NULL, NULL, NULL, NULL, NULL, NULL);
-	Mat4MulSerie(ob->obmat, tmat, obmat,         
+	Mat4MulSerie(obmat, tmat, locmat,         
 		NULL, NULL, NULL, NULL, NULL, NULL);
 	
-	if (simul){
+	if (simul) {
 
 	}
 	else{
@@ -1536,7 +1663,6 @@ void solve_tracking (Object *ob, float targetmat[][4])
 
 void where_is_object(Object *ob)
 {
-	
 	where_is_object_time(ob, (float)G.scene->r.cfra);
 }
 
@@ -1562,7 +1688,7 @@ for a lamp that is the child of another object */
 	if(ob->parent) {
 		par= ob->parent;
 		
-		solve_parenting(ob, par, slowmat, 1);
+		solve_parenting(ob, par, ob->obmat, slowmat, 1);
 
 		if(ob->partype & PARSLOW) {
 
@@ -1770,6 +1896,16 @@ BoundBox *object_get_boundbox(Object *ob)
 	return bb;
 }
 
+/* used to temporally disable/enable boundbox */
+void object_boundbox_flag(Object *ob, int flag, int set)
+{
+	BoundBox *bb= object_get_boundbox(ob);
+	if(bb) {
+		if(set) bb->flag |= flag;
+		else bb->flag &= ~flag;
+	}
+}
+
 void minmax_object(Object *ob, float *min, float *max)
 {
 	BoundBox bb;
@@ -1836,21 +1972,43 @@ void minmax_object(Object *ob, float *min, float *max)
 	}
 }
 
+/* proxy rule: lib_object->proxy_from == the one we borrow from, only set temporal and cleared here */
+/*           local_object->proxy      == pointer to library object, saved in files and read */
+
+/* function below is polluted with proxy exceptions, cleanup will follow! */
+
 /* the main object update call, for object matrix, constraints, keys and displist (modifiers) */
 /* requires flags to be set! */
 void object_handle_update(Object *ob)
 {
 	if(ob->recalc & OB_RECALC) {
 		
-		if(ob->recalc & OB_RECALC_OB) where_is_object(ob);
+		if(ob->recalc & OB_RECALC_OB) {
+			
+			// printf("recalcob %s\n", ob->id.name+2);
+			
+			/* handle proxy copy for target */
+			if(ob->id.lib && ob->proxy_from) {
+				// printf("ob proxy copy, lib ob %s proxy %s\n", ob->id.name, ob->proxy_from->id.name);
+				if(ob->proxy_from->proxy_group) {/* transform proxy into group space */
+					Object *obg= ob->proxy_from->proxy_group;
+					Mat4Invert(obg->imat, obg->obmat);
+					Mat4MulMat4(ob->obmat, ob->proxy_from->obmat, obg->imat);
+				}
+				else
+					Mat4CpyMat4(ob->obmat, ob->proxy_from->obmat);
+			}
+			else
+				where_is_object(ob);
+		}
 		
 		if(ob->recalc & OB_RECALC_DATA) {
 			
-//			printf("recalcdata %s\n", ob->id.name+2);
+			// printf("recalcdata %s\n", ob->id.name+2);
 			
 			/* includes all keys and modifiers */
 			if(ob->type==OB_MESH) {
-				makeDispListMesh(ob);
+				makeDerivedMesh(ob, get_viewedit_datamask());
 			}
 			else if(ob->type==OB_MBALL) {
 				makeDispListMBall(ob);
@@ -1862,16 +2020,37 @@ void object_handle_update(Object *ob)
 				lattice_calc_modifiers(ob);
 			}
 			else if(ob->type==OB_ARMATURE) {
-				/* this actually only happens for reading old files... */
+				/* this happens for reading old files and to match library armatures with poses */
 				if(ob->pose==NULL || (ob->pose->flag & POSE_RECALC))
 					armature_rebuild_pose(ob, ob->data);
-				do_all_pose_actions(ob);
-				where_is_pose(ob);
+				
+				if(ob->id.lib && ob->proxy_from) {
+					copy_pose_result(ob->pose, ob->proxy_from->pose);
+					// printf("pose proxy copy, lib ob %s proxy %s\n", ob->id.name, ob->proxy_from->id.name);
+				}
+				else {
+					do_all_pose_actions(ob);
+					where_is_pose(ob);
+				}
 			}
 		}
 	
+		/* the no-group proxy case, we call update */
+		if(ob->proxy && ob->proxy_group==NULL) {
+			/* set pointer in library proxy target, for copying, but restore it */
+			ob->proxy->proxy_from= ob;
+			// printf("call update, lib ob %s proxy %s\n", ob->proxy->id.name, ob->id.name);
+			object_handle_update(ob->proxy);
+		}
+	
 		ob->recalc &= ~OB_RECALC;
 	}
+
+	/* the case when this is a group proxy, object_update is called in group.c */
+	if(ob->proxy) {
+		ob->proxy->proxy_from= ob;
+		// printf("set proxy pointer for later group stuff %s\n", ob->id.name);
+	}
 }
 
 
