@@ -1,7 +1,7 @@
 /* texture.c
  *
  *
- * $Id: texture.c,v 1.25 2005/05/27 19:02:39 ton Exp $
+ * $Id: texture.c,v 1.34 2007/01/29 20:43:27 sirdude Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -37,10 +37,6 @@
 #include <string.h>
 #include <math.h>
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "MEM_guardedalloc.h"
 
 #include "PIL_dynlib.h"
@@ -57,6 +53,8 @@
 #include "DNA_lamp_types.h"
 #include "DNA_material_types.h"
 #include "DNA_image_types.h"
+#include "DNA_world_types.h"
+#include "DNA_brush_types.h"
 
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"
@@ -74,13 +72,15 @@
 #include "BKE_material.h"
 #include "BKE_texture.h"
 #include "BKE_key.h"
+#include "BKE_icons.h"
 #include "BKE_ipo.h"
+#include "BKE_brush.h"
 
 
 /* ------------------------------------------------------------------------- */
 
 /* All support for plugin textures: */
-int test_dlerr(const char *name,  const char *symbol)
+int test_dlerr(const char *name, const char *symbol)
 {
 	char *err;
 	
@@ -126,12 +126,13 @@ void open_plugin_tex(PluginTex *pit)
 		
 		if (version != 0) {
 			pit->version= version();
-			if (pit->version==2 || pit->version==3) {
+			if (pit->version>=2 && pit->version<=5) {
 				int (*info_func)(PluginInfo *);
 				PluginInfo *info= (PluginInfo*) MEM_mallocN(sizeof(PluginInfo), "plugin_info"); 
 
 				info_func= (int (*)(PluginInfo *))PIL_dynlib_find_symbol(pit->handle, "plugin_getinfo");
 				if (!test_dlerr(pit->name, "plugin_getinfo")) {
+					info->instance_init = NULL;
 
 					info_func(info);
 
@@ -144,6 +145,7 @@ void open_plugin_tex(PluginTex *pit)
 					pit->varstr= info->varstr;
 					pit->result= info->result;
 					pit->cfra= info->cfra;
+					pit->instance_init = info->instance_init;
 					if (info->init) info->init();
 				}
 				MEM_freeN(info);
@@ -157,6 +159,10 @@ void open_plugin_tex(PluginTex *pit)
 
 /* ------------------------------------------------------------------------- */
 
+/* very badlevel define to bypass linking with BIF_interface.h */
+#define INT	96
+#define FLO	128
+
 PluginTex *add_plugin_tex(char *str)
 {
 	PluginTex *pit;
@@ -183,6 +189,9 @@ PluginTex *add_plugin_tex(char *str)
 			*((int *)(pit->data+a))= (int) varstr->def;
 	}
 
+	if (pit->instance_init)
+		pit->instance_init((void *) pit->data);
+
 	return pit;
 }
 
@@ -196,27 +205,72 @@ void free_plugin_tex(PluginTex *pit)
 	MEM_freeN(pit);	
 }
 
+/* ****************** Mapping ******************* */
+
+TexMapping *add_mapping(void)
+{
+	TexMapping *texmap= MEM_callocN(sizeof(TexMapping), "Tex map");
+	
+	texmap->size[0]= texmap->size[1]= texmap->size[2]= 1.0f;
+	texmap->max[0]= texmap->max[1]= texmap->max[2]= 1.0f;
+	Mat4One(texmap->mat);
+	
+	return texmap;
+}
+
+void init_mapping(TexMapping *texmap)
+{
+	float eul[3], smat[3][3], rmat[3][3], mat[3][3];
+	
+	SizeToMat3(texmap->size, smat);
+	
+	eul[0]= (M_PI/180.0f)*texmap->rot[0];
+	eul[1]= (M_PI/180.0f)*texmap->rot[1];
+	eul[2]= (M_PI/180.0f)*texmap->rot[2];
+	EulToMat3(eul, rmat);
+	
+	Mat3MulMat3(mat, rmat, smat);
+	
+	Mat4CpyMat3(texmap->mat, mat);
+	VECCOPY(texmap->mat[3], texmap->loc);
+
+}
+
 /* ****************** COLORBAND ******************* */
 
-ColorBand *add_colorband()
+ColorBand *add_colorband(int rangetype)
 {
 	ColorBand *coba;
 	int a;
 	
 	coba= MEM_callocN( sizeof(ColorBand), "colorband");
 	
-	coba->data[0].r= 0.0;
-	coba->data[0].g= 0.0;
-	coba->data[0].b= 0.0;
-	coba->data[0].a= 0.0;
 	coba->data[0].pos= 0.0;
-
-	coba->data[1].r= 0.0;
-	coba->data[1].g= 1.0;
-	coba->data[1].b= 1.0;
-	coba->data[1].a= 1.0;
 	coba->data[1].pos= 1.0;
 	
+	if(rangetype==0) {
+		coba->data[0].r= 0.0;
+		coba->data[0].g= 0.0;
+		coba->data[0].b= 0.0;
+		coba->data[0].a= 0.0;
+		
+		coba->data[1].r= 0.0;
+		coba->data[1].g= 1.0;
+		coba->data[1].b= 1.0;
+		coba->data[1].a= 1.0;
+	}
+	else {
+		coba->data[0].r= 0.0;
+		coba->data[0].g= 0.0;
+		coba->data[0].b= 0.0;
+		coba->data[0].a= 1.0;
+		
+		coba->data[1].r= 1.0;
+		coba->data[1].g= 1.0;
+		coba->data[1].b= 1.0;
+		coba->data[1].a= 1.0;
+	}
+	
 	for(a=2; a<MAXCOLORBAND; a++) {
 		coba->data[a].r= 0.5;
 		coba->data[a].g= 0.5;
@@ -248,20 +302,32 @@ int do_colorband(ColorBand *coba, float in, float out[4])
 		out[3]= cbd1->a;
 	}
 	else {
-		if(in < cbd1->pos) {
+		if(in <= cbd1->pos && coba->ipotype<2) {
 			out[0]= cbd1->r;
 			out[1]= cbd1->g;
 			out[2]= cbd1->b;
 			out[3]= cbd1->a;
 		}
 		else {
-
+			CBData left, right;
+			
 			/* we're looking for first pos > in */
 			for(a=0; a<coba->tot; a++, cbd1++) if(cbd1->pos > in) break;
 				
-			if(a==coba->tot) cbd1--;
+			if(a==coba->tot) {
+				cbd2= cbd1-1;
+				right= *cbd2;
+				right.pos= 1.0f;
+				cbd1= &right;
+			}
+			else if(a==0) {
+				left= *cbd1;
+				left.pos= 0.0f;
+				cbd2= &left;
+			}
+			else cbd2= cbd1-1;
 			
-			if(in > cbd1->pos) {
+			if(in >= cbd1->pos && coba->ipotype<2) {
 				out[0]= cbd1->r;
 				out[1]= cbd1->g;
 				out[2]= cbd1->b;
@@ -269,10 +335,12 @@ int do_colorband(ColorBand *coba, float in, float out[4])
 			}
 			else {
 		
-				cbd2= cbd1-1;
-				fac= (in-cbd1->pos)/(cbd2->pos-cbd1->pos);
+				if(cbd2->pos!=cbd1->pos)
+					fac= (in-cbd1->pos)/(cbd2->pos-cbd1->pos);
+				else
+					fac= 0.0f;
 				
-				if(coba->ipotype==2) {
+				if(coba->ipotype>=2) {
 					/* ipo from right to left: 3 2 1 0 */
 					
 					if(a>=coba->tot-1) cbd0= cbd1;
@@ -280,7 +348,12 @@ int do_colorband(ColorBand *coba, float in, float out[4])
 					if(a<2) cbd3= cbd2;
 					else cbd3= cbd2-1;
 					
-					set_four_ipo(fac, t, KEY_BSPLINE);
+					CLAMP(fac, 0.0f, 1.0f);
+					
+					if(coba->ipotype==3)
+						set_four_ipo(fac, t, KEY_CARDINAL);
+					else
+						set_four_ipo(fac, t, KEY_BSPLINE);
 
 					out[0]= t[3]*cbd3->r +t[2]*cbd2->r +t[1]*cbd1->r +t[0]*cbd0->r;
 					out[1]= t[3]*cbd3->g +t[2]*cbd2->g +t[1]*cbd1->g +t[0]*cbd0->g;
@@ -316,7 +389,9 @@ void free_texture(Tex *tex)
 {
 	free_plugin_tex(tex->plugin);
 	if(tex->coba) MEM_freeN(tex->coba);
-	if(tex->env) RE_free_envmap(tex->env);
+	if(tex->env) BKE_free_envmap(tex->env);
+	BKE_icon_delete((struct ID*)tex);
+	tex->id.icon_id = 0;
 }
 
 /* ------------------------------------------------------------------------- */
@@ -376,13 +451,17 @@ void default_tex(Tex *tex)
 
 	pit = tex->plugin;
 	if (pit) {
-			varstr= pit->varstr;
-			if(varstr) {
-					for(a=0; a<pit->vars; a++, varstr++) {
-						pit->data[a] = varstr->def;
-					}
+		varstr= pit->varstr;
+		if(varstr) {
+			for(a=0; a<pit->vars; a++, varstr++) {
+				pit->data[a] = varstr->def;
 			}
+		}
 	}
+	
+	tex->iuser.fie_ima= 2;
+	tex->iuser.ok= 1;
+	tex->iuser.frames= 100;
 }
 
 /* ------------------------------------------------------------------------- */
@@ -460,7 +539,7 @@ Tex *copy_texture(Tex *tex)
 	}
 	
 	if(texn->coba) texn->coba= MEM_dupallocN(texn->coba);
-	if(texn->env) texn->env= RE_copy_envmap(texn->env);
+	if(texn->env) texn->env= BKE_copy_envmap(texn->env);
 	
 	return texn;
 }
@@ -473,6 +552,7 @@ void make_local_texture(Tex *tex)
 	Material *ma;
 	World *wrld;
 	Lamp *la;
+	Brush *br;
 	int a, local=0, lib=0;
 
 	/* - only lib users: do nothing
@@ -527,6 +607,16 @@ void make_local_texture(Tex *tex)
 		}
 		wrld= wrld->id.next;
 	}
+	br= G.main->brush.first;
+	while(br) {
+		for(a=0; a<MAX_MTEX; a++) {
+			if(br->mtex[a] && br->mtex[a]->tex==tex) {
+				if(br->id.lib) lib= 1;
+				else local= 1;
+			}
+		}
+		br= br->id.next;
+	}
 	
 	if(local && lib==0) {
 		tex->id.lib= 0;
@@ -576,7 +666,19 @@ void make_local_texture(Tex *tex)
 			}
 			wrld= wrld->id.next;
 		}
-
+		br= G.main->brush.first;
+		while(br) {
+			for(a=0; a<MAX_MTEX; a++) {
+				if(br->mtex[a] && br->mtex[a]->tex==tex) {
+					if(br->id.lib==0) {
+						br->mtex[a]->tex= texn;
+						texn->id.us++;
+						tex->id.us--;
+					}
+				}
+			}
+			br= br->id.next;
+		}
 	}
 }
 
@@ -584,7 +686,7 @@ void make_local_texture(Tex *tex)
 
 void autotexname(Tex *tex)
 {
-/*  	extern char texstr[20][12];	 *//* buttons.c, already in bad lev calls*/
+/*	extern char texstr[20][12];	 *//* buttons.c, already in bad lev calls*/
 	Image *ima;
 	char di[FILE_MAXDIR], fi[FILE_MAXFILE];
 	
@@ -647,3 +749,61 @@ Tex *give_current_texture(Object *ob, int act)
 	
 	return tex;
 }
+
+
+/* ------------------------------------------------------------------------- */
+
+EnvMap *BKE_add_envmap(void)
+{
+	EnvMap *env;
+	
+	env= MEM_callocN(sizeof(EnvMap), "envmap");
+	env->type= ENV_CUBE;
+	env->stype= ENV_STATIC;
+	env->clipsta= 0.1;
+	env->clipend= 100.0;
+	env->cuberes= 100;
+	
+	return env;
+} 
+
+/* ------------------------------------------------------------------------- */
+
+EnvMap *BKE_copy_envmap(EnvMap *env)
+{
+	EnvMap *envn;
+	int a;
+	
+	envn= MEM_dupallocN(env);
+	envn->ok= 0;
+	for(a=0; a<6; a++) envn->cube[a]= NULL;
+	if(envn->ima) id_us_plus((ID *)envn->ima);
+	
+	return envn;
+}
+
+/* ------------------------------------------------------------------------- */
+
+void BKE_free_envmapdata(EnvMap *env)
+{
+	unsigned int part;
+	
+	for(part=0; part<6; part++) {
+		if(env->cube[part])
+			IMB_freeImBuf(env->cube[part]);
+		env->cube[part]= NULL;
+	}
+	env->ok= 0;
+}
+
+/* ------------------------------------------------------------------------- */
+
+void BKE_free_envmap(EnvMap *env)
+{
+	
+	BKE_free_envmapdata(env);
+	MEM_freeN(env);
+	
+}
+
+/* ------------------------------------------------------------------------- */
