@@ -2,7 +2,7 @@
  * Functions for writing avi-format files.
  * Added interface for generic movie support (ton)
  *
- * $Id: writeavi.c,v 1.10 2006/02/05 19:12:25 schlaile Exp $
+ * $Id: writeavi.c,v 1.11 2006/10/26 23:34:44 blendix Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -191,7 +191,7 @@ void append_avi(int frame, int *pixels, int rectx, int recty)
 	/* note that libavi free's the buffer... stupid interface - zr */
 	rectot= MEM_mallocN(rectx*recty*sizeof(int), "rectot");
 	rt1= rectot;
-	rt2= pixels + (recty-1)*rectx;
+	rt2= (unsigned int*)pixels + (recty-1)*rectx;
 	/* flip y and convert to abgr */
 	for (y=0; y < recty; y++, rt1+= rectx, rt2-= rectx) {
 		memcpy (rt1, rt2, rectx*sizeof(int));
