@@ -3,7 +3,7 @@
 /**
  * blenlib/BLI_arithb.h    mar 2001 Nzc
  *
- * $Id: BLI_arithb.h,v 1.25 2006/06/03 17:21:45 ton Exp $ 
+ * $Id: BLI_arithb.h,v 1.35 2006/12/22 09:05:37 aligorith Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -155,8 +155,16 @@ Mat3ToEul(
 	float *eul
 );
 
+	void 
+Mat4ToEul(
+	float tmat[][4],
+	float *eul
+);
+
 void compatible_eul(float *eul, float *oldrot);
 
+void Mat3ToCompatibleEul(float mat[][3], float *eul, float *oldrot);
+
 /**
  * @section Quaternion arithmetic routines
  */
@@ -392,6 +400,11 @@ Mat3Clr(
 	void 
 Mat3One(
 	float m[][3]
+);
+	void
+Mat4MulVec3Project(
+	float mat[][4],
+	float *vec
 );
 	void 
 Mat4MulVec(
@@ -459,6 +472,12 @@ FloatCompare(
 	float limit
 );
 	float 
+FloatLerpf(
+	float target,
+	float origin,
+	float fac
+);
+	float 
 Normalise(
 	float *n
 );
@@ -503,6 +522,11 @@ VecCompare(
 	float *v1, 
 	float *v2, 
 	float limit
+);
+	int
+VecEqual(
+	float *v1,
+	float *v2
 );
 	float 
 Sqrt3f(
@@ -524,6 +548,10 @@ saacos(
 	float fac
 );
 	float 
+saasin(
+	float fac
+);
+	float 
 sasqrt(
 	float fac
 );
@@ -621,6 +649,13 @@ VecRotToMat3(
 extern short IsectLL2Df(float *v1, float *v2, float *v3, float *v4);
 extern short IsectLL2Ds(short *v1, short *v2, short *v3, short *v4);
 
+/* interpolation weights of point in a triangle or quad, v4 may be NULL */
+	void
+InterpWeightsQ3Dfl(
+	float *v1, float *v2, float *v3, float *v4,
+	float *co,
+	float *w
+);
 
 	float *
 vectoquat(
@@ -641,6 +676,11 @@ VecAngle2(
 	float *v1, 
 	float *v2
 );
+	float
+NormalizedVecAngle2(
+	float *v1,
+	float *v2
+);
 
 	void 
 i_lookat(
@@ -664,8 +704,37 @@ hsv_to_rgb(
 	float *g, float *b
 );
 
-void hex_to_rgb(char *hexcol, float *r, float *g, float *b);
+	void 
+hex_to_rgb(
+	char *hexcol, 
+	float *r, 
+	float *g, 
+	float *b
+);
 
+	void 
+rgb_to_yuv(
+	float r, float g, float b, 
+	float *ly, float *lu, float *lv
+);
+
+	void 
+yuv_to_rgb(
+	float y, float u, float v, 
+	float *lr, float *lg, float *lb
+);
+
+	void 
+ycc_to_rgb(
+	float y, float cb, float cr, 
+	float *lr, float *lg, float *lb
+);
+
+	void 
+rgb_to_ycc(
+	float r, float g, float b, 
+	float *ly, float *lcb, float *lcr
+);
 	void 
 rgb_to_hsv(
 	float r, float g, float b, 
@@ -785,11 +854,7 @@ MinMax3(
 	float *max, 
 	float *vec
 );
-	void 
-Mat3ToEuln(
-	float tmat[][3],
-	float *eul
-);
+
 	void 
 SizeToMat3(
 	float *size, 
@@ -883,10 +948,14 @@ Normalise2(
 	float *n
 );
 
+void LocEulSizeToMat4(float mat[][4], float loc[3], float eul[3], float size[3]);
+void LocQuatSizeToMat4(float mat[][4], float loc[3], float quat[4], float size[3]);
+
 void tubemap(float x, float y, float z, float *u, float *v);
 void spheremap(float x, float y, float z, float *u, float *v);
 			  
 int LineIntersectsTriangle(float p1[3], float p2[3], float v0[3], float v1[3], float v2[3], float *lambda);
+int point_in_tri_prism(float p[3], float v1[3], float v2[3], float v3[3]);
 			  
 #ifdef __cplusplus
 }
