@@ -1,5 +1,5 @@
 /**
- * $Id: BLI_blenlib.h,v 1.19 2006/06/19 13:52:59 ton Exp $
+ * $Id: BLI_blenlib.h,v 1.23 2006/12/20 17:57:41 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -59,7 +59,7 @@
  * - The blenlib uses type defines from makesdna/, and functions from
  * standard libraries.
  * 
- * $Id: BLI_blenlib.h,v 1.19 2006/06/19 13:52:59 ton Exp $ 
+ * $Id: BLI_blenlib.h,v 1.23 2006/12/20 17:57:41 ton Exp $ 
 */
 
 #ifndef BLI_BLENLIB_H
@@ -84,6 +84,7 @@ struct rcti;
 struct EditVert;
 struct PackedFile;
 struct LinkNode;
+struct DynamicList;
 
 #ifdef __cplusplus
 extern "C" {
@@ -94,8 +95,8 @@ char *BLI_gethome(void);
 void BLI_make_file_string(const char *relabase, char *string,  const char *dir, const char *file);
 void BLI_make_exist(char *dir);
 void BLI_make_existing_file(char *name);
-void BLI_split_dirfile(char *string, char *dir, char *file);
-int BLI_testextensie(char *str, char *ext);
+void BLI_split_dirfile(const char *string, char *dir, char *file);
+int BLI_testextensie(const char *str, const char *ext);
 void addlisttolist(ListBase *list1, ListBase *list2);
 void BLI_insertlink(struct ListBase *listbase, void *vprevlink, void *vnewlink);
 void * BLI_findlink(struct ListBase *listbase, int number);
@@ -112,6 +113,17 @@ int BLI_countlist(struct ListBase *listbase);
 void BLI_freelinkN(ListBase *listbase, void *vlink);
 void BLI_splitdirstring(char *di,char *fi);
 
+struct DynamicList *BLI_dlist_from_listbase(struct ListBase *lb);
+struct ListBase *BLI_listbase_from_dlist(struct DynamicList *dlist, struct ListBase *lb);
+void * BLI_dlist_find_link(struct DynamicList *dlist, unsigned int index);
+unsigned int BLI_count_items(struct DynamicList *dlist);
+void BLI_dlist_free_item(struct DynamicList *dlist, unsigned int index);
+void BLI_dlist_rem_item(struct DynamicList *dlist, unsigned int index);
+void * BLI_dlist_add_item_index(struct DynamicList *dlist, void *item, unsigned int index);
+void BLI_dlist_destroy(struct DynamicList *dlist);
+void BLI_dlist_init(struct DynamicList *dlist);
+void BLI_dlist_reinit(struct DynamicList *dlist);
+
 	/**
 	 * dir can be any input, like from buttons, and this function
 	 * converts it to a regular full path.
@@ -181,7 +193,7 @@ char* BLI_strdupn(char *str, int len);
 	 *   the size of dst)
 	 * @retval Returns dst
 	 */
-char* BLI_strncpy(char *dst, char *src, int maxncpy);
+char* BLI_strncpy(char *dst, const char *src, int maxncpy);
 
 	/**
 	 * Compare two strings
@@ -337,6 +349,7 @@ void BLI_setErrorCallBack(void (*f)(char*));
  */
 void BLI_setInterruptCallBack(int (*f)(void));
 
+char *BLI_strcasestr(const char *s, const char *find);
 int BLI_strcasecmp(const char *s1, const char *s2);
 int BLI_strncasecmp(const char *s1, const char *s2, int n);
 void BLI_timestr(double time, char *str);
