@@ -1,5 +1,5 @@
 /**
- * $Id: storage.c,v 1.32 2006/03/19 13:28:01 elubie Exp $
+ * $Id: storage.c,v 1.33 2006/08/20 14:41:13 elubie Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -277,6 +277,14 @@ void BLI_builddir(char *dirname, char *relname)
 				BLI_addhead(dirbase,dlink);
 				newnum++;
 			}
+#else // WIN32
+			if (seen_ == 0) {	/* should only happen for root paths like "C:\" */
+				dlink = (struct dirlink *)malloc(sizeof(struct dirlink));
+				strcpy(buf+rellen,".");
+				dlink->name = BLI_strdup(buf);
+				BLI_addhead(dirbase,dlink);
+				newnum++;
+			}
 #endif			
 
 			if (files) files=(struct direntry *)realloc(files,(totnum+newnum) * sizeof(struct direntry));
