@@ -3,7 +3,7 @@
  * various string, file, list operations.
  *
  *
- * $Id: util.c,v 1.35 2006/02/11 15:02:23 elubie Exp $
+ * $Id: util.c,v 1.42 2006/12/20 17:57:42 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -47,6 +47,7 @@
 #include "DNA_listBase.h"
 #include "BLI_storage.h"
 #include "BLI_storage_types.h"
+#include "BLI_dynamiclist.h"
 
 #include "BLI_util.h"
 
@@ -403,6 +404,228 @@ void * BLI_findlink(ListBase *listbase, int number)
 	return (link);
 }
 
+/*=====================================================================================*/
+/* Methods for access array (realloc) */
+/*=====================================================================================*/
+
+/* remove item with index */
+static void rem_array_item(struct DynamicArray *da, unsigned int index)
+{
+	da->items[index]=NULL;
+	da->count--;
+	if(index==da->last_item_index){
+		while((!da->items[da->last_item_index]) && (da->last_item_index>0)){
+			da->last_item_index--;
+		}
+	}
+}
+
+/* add array (if needed, then realloc) */
+static void add_array_item(struct DynamicArray *da, void *item, unsigned int index)
+{
+	/* realloc of access array */
+	if(da->max_item_index < index){
+		unsigned int i, max = da->max_item_index;
+		void **nitems;
+
+		do {
+			da->max_item_index += PAGE_SIZE;	/* OS can allocate only PAGE_SIZE Bytes */
+		} while(da->max_item_index<=index);
+
+		nitems = (void**)MEM_mallocN(sizeof(void*)*(da->max_item_index+1), "dlist access array");
+		for(i=0;i<=max;i++)
+			nitems[i] = da->items[i];
+
+		/* set rest pointers to the NULL */
+		for(i=max+1; i<=da->max_item_index; i++)
+			nitems[i]=NULL;
+
+		MEM_freeN(da->items);		/* free old access array */
+		da->items = nitems;
+	}
+
+	da->items[index] = item;
+	da->count++;
+	if(index > da->last_item_index) da->last_item_index = index;
+}
+
+/* free access array */
+static void destroy_array(DynamicArray *da)
+{
+	da->count=0;
+	da->last_item_index=0;
+	da->max_item_index=0;
+	MEM_freeN(da->items);
+	da->items = NULL;
+}
+
+/* initialize dynamic array */
+static void init_array(DynamicArray *da)
+{
+	unsigned int i;
+
+	da->count=0;
+	da->last_item_index=0;
+	da->max_item_index = PAGE_SIZE-1;
+	da->items = (void*)MEM_mallocN(sizeof(void*)*(da->max_item_index+1), "dlist access array");
+	for(i=0; i<=da->max_item_index; i++) da->items[i]=NULL;
+}
+
+/* reinitialize dynamic array */
+static void reinit_array(DynamicArray *da)
+{
+	destroy_array(da);
+	init_array(da);
+}
+
+/*=====================================================================================*/
+/* Methods for two way dynamic list with access array */
+/*=====================================================================================*/
+
+/* create new two way dynamic list with access array from two way dynamic list
+ * it doesn't copy any items to new array or something like this It is strongly
+ * recomended to use BLI_dlist_ methods for adding/removing items from dynamic list
+ * unless you can end with inconsistence system !!! */
+DynamicList *BLI_dlist_from_listbase(ListBase *lb)
+{
+	DynamicList *dlist;
+	Link *item;
+	int i=0, count;
+	
+	if(!lb) return NULL;
+	
+	count = BLI_countlist(lb);
+
+	dlist = MEM_mallocN(sizeof(DynamicList), "temp dynamic list");
+	/* ListBase stuff */
+	dlist->lb.first = lb->first;
+	dlist->lb.last = lb->last;
+	/* access array stuff */
+	dlist->da.count=count;
+	dlist->da.max_item_index = count-1;
+	dlist->da.last_item_index = count -1;
+	dlist->da.items = (void*)MEM_mallocN(sizeof(void*)*count, "temp dlist access array");
+
+	item = (Link*)lb->first;
+	while(item){
+		dlist->da.items[i] = (void*)item;
+		item = item->next;
+		i++;
+	}
+
+	/* to prevent you of using original ListBase :-) */
+	lb->first = lb->last = NULL;
+
+	return dlist;
+}
+
+/* take out ListBase from DynamicList and destroy all temporary structures of DynamicList */
+ListBase *BLI_listbase_from_dlist(DynamicList *dlist, ListBase *lb)
+{
+	if(!dlist) return NULL;
+
+	if(!lb) lb = (ListBase*)MEM_mallocN(sizeof(ListBase), "ListBase");
+	
+	lb->first = dlist->lb.first;
+	lb->last = dlist->lb.last;
+
+	/* free all items of access array */
+	MEM_freeN(dlist->da.items);
+	/* free DynamicList*/
+	MEM_freeN(dlist);
+
+	return lb;
+}
+
+/* return pointer at item from th dynamic list with access array */
+void *BLI_dlist_find_link(DynamicList *dlist, unsigned int index)
+{
+	if(!dlist || !dlist->da.items) return NULL;
+
+	if((index <= dlist->da.last_item_index) && (index >= 0) && (dlist->da.count>0)){
+      		return dlist->da.items[index];
+	}
+	else {
+		return NULL;
+	}
+}
+
+/* return count of items in the dynamic list with access array */
+unsigned int BLI_count_items(DynamicList *dlist)
+{
+	if(!dlist) return 0;
+
+	return dlist->da.count;
+}
+
+/* free item from the dynamic list with access array */
+void BLI_dlist_free_item(DynamicList *dlist, unsigned int index)
+{
+	if(!dlist || !dlist->da.items) return;
+	
+	if((index <= dlist->da.last_item_index) && (dlist->da.items[index])){
+		BLI_freelinkN(&(dlist->lb), dlist->da.items[index]);
+		rem_array_item(&(dlist->da), index);
+	}
+}
+
+/* remove item from the dynamic list with access array */
+void BLI_dlist_rem_item(DynamicList *dlist, unsigned int index)
+{
+	if(!dlist || !dlist->da.items) return;
+	
+	if((index <= dlist->da.last_item_index) && (dlist->da.items[index])){
+		BLI_remlink(&(dlist->lb), dlist->da.items[index]);
+		rem_array_item(&(dlist->da), index);
+	}
+}
+
+/* add item to the dynamic list with access array (index) */
+void* BLI_dlist_add_item_index(DynamicList *dlist, void *item, unsigned int index)
+{
+	if(!dlist || !dlist->da.items) return NULL;
+
+	if((index <= dlist->da.max_item_index) && (dlist->da.items[index])) {
+		/* you can't place item at used index */
+		return NULL;
+	}
+	else {
+		add_array_item(&(dlist->da), item, index);
+		BLI_addtail(&(dlist->lb), item);
+		return item;
+	}
+}
+
+/* destroy dynamic list with access array */
+void BLI_dlist_destroy(DynamicList *dlist)
+{
+	if(!dlist) return;
+
+	BLI_freelistN(&(dlist->lb));
+	destroy_array(&(dlist->da));
+}
+
+/* initialize dynamic list with access array */
+void BLI_dlist_init(DynamicList *dlist)
+{
+	if(!dlist) return;
+
+	dlist->lb.first = NULL;
+	dlist->lb.last = NULL;
+
+	init_array(&(dlist->da));
+}
+
+/* reinitialize dynamic list with acces array */
+void BLI_dlist_reinit(DynamicList *dlist)
+{
+	if(!dlist) return;
+	
+	BLI_freelistN(&(dlist->lb));
+	reinit_array(&(dlist->da));
+}
+
+/*=====================================================================================*/
 
 char *BLI_strdupn(char *str, int len) {
 	char *n= MEM_mallocN(len+1, "strdup");
@@ -415,7 +638,7 @@ char *BLI_strdup(char *str) {
 	return BLI_strdupn(str, strlen(str));
 }
 
-char *BLI_strncpy(char *dst, char *src, int maxncpy) {
+char *BLI_strncpy(char *dst, const char *src, int maxncpy) {
 	int srclen= strlen(src);
 	int cpylen= (srclen>(maxncpy-1))?(maxncpy-1):srclen;
 	
@@ -449,8 +672,7 @@ void BLI_cleanup_dir(const char *relabase, char *dir)
 	
 #ifdef WIN32
 	if(dir[0]=='.') {	/* happens for example in FILE_MAIN */
-	   dir[0]= '\\';
-	   dir[1]= 0;
+	   get_default_root(dir);
 	   return;
 	}	
 
@@ -534,6 +756,9 @@ void BLI_makestringcode(const char *relfile, char *file)
 	/* if file is already relative, bail out */
 	if(file[0]=='/' && file[1]=='/') return;
 	
+	/* also bail out if relative path is not set */
+	if (relfile[0] == 0) return;
+
 	strcpy(temp, relfile);
 	
 #ifdef WIN32
@@ -592,11 +817,35 @@ void BLI_makestringcode(const char *relfile, char *file)
 
 int BLI_convertstringcode(char *path, const char *basepath, int framenum)
 {
-	int len, wasrelative;
+	int len, wasrelative, is_filepath;
 	char tmp[FILE_MAXDIR+FILE_MAXFILE];
 	char base[FILE_MAXDIR];
-	
+	char vol[3] = {'\0', '\0', '\0'};
+
+	BLI_strncpy(vol, path, 3);
+	wasrelative= (strncmp(vol, "//", 2)==0);
+
+#ifdef WIN32
+	/* we are checking here if we have an absolute path that is not in the current
+	   blend file as a lib main - we are basically checking for the case that a 
+	   UNIX root '/' is passed.
+	*/
+	if (!wasrelative && (vol[1] != ':' && (vol[0] == '\0' || vol[0] == '/' || vol[0] == '\\'))) {
+		char *p = path;
+		get_default_root(tmp);
+		// get rid of the slashes at the beginning of the path
+		while (*p == '\\' || *p == '/') {
+			p++;
+		}
+		strcat(tmp, p);
+	}
+	else {
+		strcpy(tmp, path);
+	}
+#else
 	strcpy(tmp, path);
+#endif
+
 	strcpy(base, basepath);
 	
 	/* push slashes into unix mode - strings entering this part are
@@ -606,9 +855,7 @@ int BLI_convertstringcode(char *path, const char *basepath, int framenum)
 	   of paths and solving some problems (and prevent potential future
 	   ones) -jesterKing. */
 	BLI_char_switch(tmp, '\\', '/');
-	BLI_char_switch(base, '\\', '/');
-	
-	wasrelative= (strncmp(tmp, "//", 2)==0);
+	BLI_char_switch(base, '\\', '/');	
 
 	if (tmp[0] == '/' && tmp[1] == '/') {
 		char *filepart= BLI_strdup(tmp+2); /* skip code */
@@ -630,7 +877,7 @@ int BLI_convertstringcode(char *path, const char *basepath, int framenum)
 	if(len && tmp[len-1]=='#') {
 		sprintf(tmp+len-1, "%04d", framenum);
 	}
-	
+
 	strcpy(path, tmp);
 #ifdef WIN32
 	/* skip first two chars, which in case of
@@ -645,15 +892,15 @@ int BLI_convertstringcode(char *path, const char *basepath, int framenum)
 	return wasrelative;
 }
 
-void BLI_splitdirstring(char *di,char *fi)
+void BLI_splitdirstring(char *di, char *fi)
 {
 	char *lslash= BLI_last_slash(di);
 
 	if (lslash) {
-		strcpy(fi, lslash+1);
+		BLI_strncpy(fi, lslash+1, FILE_MAXFILE);
 		*(lslash+1)=0;
 	} else {
-		strcpy(fi, di);
+		BLI_strncpy(fi, di, FILE_MAXFILE);
 		di[0]= 0;
 	}
 }
@@ -761,7 +1008,8 @@ void BLI_make_exist(char *dir) {
 		}
 		if (a >= 0) dir[a+1] = 0;
 		else {
-			strcpy(dir,"\\");
+			/* defaulting to first valid drive hoping it's not empty CD and DVD drives */
+			get_default_root(dir);
 			break;
 		}
 	}
@@ -825,7 +1073,29 @@ void BLI_make_file_string(const char *relabase, char *string,  const char *dir,
 
 		dir+=2; /* Skip over the relative reference */
 	}
-	
+#ifdef WIN32
+	else {
+		if (strlen(dir) >= 2 && dir[1] == ':' ) {
+			BLI_strncpy(string, dir, 3);
+			dir += 2;
+		}
+		else { /* no drive specified */
+			/* first option: get the drive from the relabase if it has one */
+			if (relabase && strlen(relabase) >= 2 && relabase[1] == ':' ) {
+				BLI_strncpy(string, relabase, 3);	
+				string[2] = '\\';
+				string[3] = '\0';
+			}
+			else { /* we're out of luck here, guessing the first valid drive, usually c:\ */
+				get_default_root(string);
+			}
+			
+			/* ignore leading slashes */
+			while (*dir == '/' || *dir == '\\') dir++;
+		}
+	}
+#endif
+
 	strcat(string, dir);
 
 	/* Make sure string ends in one (and only one) slash */	
@@ -847,7 +1117,7 @@ void BLI_make_file_string(const char *relabase, char *string,  const char *dir,
 	BLI_clean(string);
 }
 
-int BLI_testextensie(char *str, char *ext)
+int BLI_testextensie(const char *str, const char *ext)
 {
 	short a, b;
 	int retval;
@@ -868,56 +1138,64 @@ int BLI_testextensie(char *str, char *ext)
 
 
 
-void BLI_split_dirfile(char *string, char *dir, char *file)
+void BLI_split_dirfile(const char *string, char *dir, char *file)
 {
 	int a;
-	
+	int sl;
+	short is_relative = 0;
+
 	dir[0]= 0;
 	file[0]= 0;
 
 #ifdef WIN32
 	BLI_char_switch(string, '/', '\\'); /* make sure we have a valid path format */
-
-	if (strlen(string)) {
+	sl = strlen(string);
+	if (sl) {
 		int len;
 		if (string[0] == '/' || string[0] == '\\') { 
 			BLI_strncpy(dir, string, FILE_MAXDIR);
-		} else if (string[1] == ':' && string[2] == '\\') {
-            BLI_strncpy(dir, string, FILE_MAXDIR);
-        } else {
-            BLI_getwdN(dir);
-            strcat(dir,"/");
-            strcat(dir,string);
-            BLI_strncpy(string,dir,FILE_MAXDIR+FILE_MAXFILE);
-        }
+			if (sl > 1 && string[0] == '\\' && string[1] == '\\') is_relative = 1;
+		} else if (sl > 2 && string[1] == ':' && string[2] == '\\') {
+			BLI_strncpy(dir, string, FILE_MAXDIR);
+		} else {
+			BLI_getwdN(dir);
+			strcat(dir,"\\");
+			strcat(dir,string);
+			BLI_strncpy(string,dir,FILE_MAXDIR+FILE_MAXFILE);
+		}
 
-        BLI_make_exist(dir);
+		// BLI_exist doesn't recognize a slashed dirname as a dir
+		//  check if a trailing slash exists, and remove it. Do not do this
+		//  when we are already at root. -jesterKing
+		a = strlen(dir);
+		if(a>=4 && dir[a-1]=='\\') dir[a-1] = 0;
 
-        // BLI_exist doesn't recognize a slashed dirname as a dir
-        //  check if a trailing slash exists, and remove it. Do not do this
-        //  when we are already at root. -jesterKing
-        a = strlen(dir);
-        if(a>=4 && dir[a-1]=='\\') dir[a-1] = 0;
+		if (is_relative) {
+			printf("WARNING: BLI_split_dirfile needs absolute dir");
+		}
+		else {
+			BLI_make_exist(dir);
+		}
 
-        if (S_ISDIR(BLI_exist(dir))) {
+		if (S_ISDIR(BLI_exist(dir))) {
 
-	    /* copy from end of string into file, to ensure filename itself isn't truncated 
-	       if string is too long. (aphex) */
+			/* copy from end of string into file, to ensure filename itself isn't truncated 
+			if string is too long. (aphex) */
 
-	    len = FILE_MAXFILE - strlen(string);
+			len = FILE_MAXFILE - strlen(string);
 
-	    if (len < 0)
-		BLI_strncpy(file,string + abs(len),FILE_MAXFILE);
-	    else
-		BLI_strncpy(file,string,FILE_MAXFILE);
-	    
-            if (strrchr(string,'\\')){
-		BLI_strncpy(file,strrchr(string,'\\')+1,FILE_MAXFILE);
-	    }
+			if (len < 0)
+				BLI_strncpy(file,string + abs(len),FILE_MAXFILE);
+			else
+				BLI_strncpy(file,string,FILE_MAXFILE);
+		    
+			if (strrchr(string,'\\')){
+				BLI_strncpy(file,strrchr(string,'\\')+1,FILE_MAXFILE);
+			}
 
-            if (a = strlen(dir)) {
-                if (dir[a-1] != '\\') strcat(dir,"\\");
-            }
+			if (a = strlen(dir)) {
+				if (dir[a-1] != '\\') strcat(dir,"\\");
+			}
 		}
 		else {
 			a = strlen(dir) - 1;
@@ -925,10 +1203,11 @@ void BLI_split_dirfile(char *string, char *dir, char *file)
 			dir[a + 1] = 0;
 			BLI_strncpy(file, string + strlen(dir),FILE_MAXFILE);
 		}
-		
+
 	}
 	else {
-		strcpy(dir, "\\");
+		/* defaulting to first valid drive hoping it's not empty CD and DVD drives */
+		get_default_root(dir);
 		file[0]=0;
 	}
 #else
@@ -1103,6 +1382,28 @@ char* BLI_getbundle(void) {
 }
 #endif
 
+/* strcasestr not available in MSVC */
+char *BLI_strcasestr(const char *s, const char *find)
+{
+    register char c, sc;
+    register size_t len;
+	
+    if ((c = *find++) != 0) {
+		c= tolower(c);
+		len = strlen(find);
+		do {
+			do {
+				if ((sc = *s++) == 0)
+					return (NULL);
+				sc= tolower(sc);
+			} while (sc != c);
+		} while (BLI_strncasecmp(s, find, len) != 0);
+		s--;
+    }
+    return ((char *) s);
+}
+
+
 int BLI_strcasecmp(const char *s1, const char *s2) {
 	int i;
 
