@@ -1,5 +1,5 @@
 /*
- * $Id: BLO_readfile.h,v 1.10 2006/01/28 18:33:07 hos Exp $
+ * $Id: BLO_readfile.h,v 1.11 2006/10/05 15:24:15 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -218,9 +218,8 @@ BLO_blendhandle_close(
 
 char *BLO_gethome(void);
 int BLO_has_bfile_extension(char *str);
-
 void BLO_library_append(struct SpaceFile *sfile, char *dir, int idcode);
-void BLO_script_library_append(BlendHandle *bh, char *dir, char *name, int idcode);
+void BLO_script_library_append(BlendHandle *bh, char *dir, char *name, int idcode, short flag);
 
 BlendFileData* blo_read_blendafterruntime(int file, int actualsize, BlendReadError *error_r);
 
