@@ -1,5 +1,5 @@
 /**
- * $Id: readblenentry.c,v 1.11 2006/01/28 18:33:07 hos Exp $
+ * $Id: readblenentry.c,v 1.15 2006/11/25 13:07:28 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -54,6 +54,7 @@
 #include "BKE_utildefines.h" // for ENDB
 
 #include "BKE_main.h"
+#include "BKE_global.h"
 #include "BKE_library.h" // for free_main
 
 #include "BLO_readfile.h"
@@ -81,6 +82,7 @@ typedef struct {
 static IDType idtypes[]= {
 	{ ID_AC,		"Action",	IDTYPE_FLAGS_ISLINKABLE}, 
 	{ ID_AR,		"Armature", IDTYPE_FLAGS_ISLINKABLE}, 
+	{ ID_BR,		"Brush",	IDTYPE_FLAGS_ISLINKABLE}, 
 	{ ID_CA,		"Camera",	IDTYPE_FLAGS_ISLINKABLE}, 
 	{ ID_CU,		"Curve",	IDTYPE_FLAGS_ISLINKABLE}, 
 	{ ID_GR,		"Group",	IDTYPE_FLAGS_ISLINKABLE}, 
@@ -97,7 +99,6 @@ static IDType idtypes[]= {
 	{ ID_ME,		"Mesh",		IDTYPE_FLAGS_ISLINKABLE}, 
 	{ ID_NT,		"NodeTree",	IDTYPE_FLAGS_ISLINKABLE}, 
 	{ ID_OB,		"Object",	IDTYPE_FLAGS_ISLINKABLE}, 
-	{ ID_SAMPLE,	"Sample",	0}, 
 	{ ID_SCE,		"Scene",	IDTYPE_FLAGS_ISLINKABLE}, 
 	{ ID_SCR,		"Screen",	0}, 
 	{ ID_SEQ,		"Sequence",	0}, 
@@ -262,7 +263,7 @@ BlendFileData *BLO_read_from_file(char *file, BlendReadError *error_r)
 		bfd= blo_read_file_internal(fd, error_r);
 		if (bfd) {
 			bfd->type= BLENFILETYPE_BLEND;
-			strcpy(bfd->main->name, file);
+			strncpy(bfd->main->name, file, sizeof(bfd->main->name)-1);
 		}
 		blo_freefiledata(fd);			
 	}
@@ -292,15 +293,42 @@ BlendFileData *BLO_read_from_memfile(const char *filename, MemFile *memfile, Ble
 {
 	BlendFileData *bfd = NULL;
 	FileData *fd;
-		
+	ListBase mainlist;
+	
 	fd = blo_openblendermemfile(memfile, error_r);
 	if (fd) {
 		strcpy(fd->filename, filename);
+		
+		/* separate libraries from G.main */
+		blo_split_main(&mainlist, G.main);
+		/* add the library pointers in oldmap lookup */
+		blo_add_library_pointer_map(&mainlist, fd);
+		
+		/* makes lookup of existing images in G.main */
+		blo_make_image_pointer_map(fd);
+		
 		bfd= blo_read_file_internal(fd, error_r);
 		if (bfd) {
 			bfd->type= BLENFILETYPE_BLEND;
 			strcpy(bfd->main->name, "");
 		}
+		
+		/* ensures relinked images are not freed */
+		blo_end_image_pointer_map(fd);
+		
+		/* move libraries from G.main to new main */
+		if(bfd && mainlist.first!=mainlist.last) {
+			
+			/* Library structs themselves */
+			bfd->main->library= G.main->library;
+			G.main->library.first= G.main->library.last= NULL;
+			
+			/* add the Library mainlist to the new main */
+			BLI_remlink(&mainlist, G.main);
+			BLI_addhead(&mainlist, bfd->main);
+		}
+		blo_join_main(&mainlist);
+		
 		blo_freefiledata(fd);			
 	}
 
