@@ -1,5 +1,5 @@
 /**
- * $Id: BDR_editcurve.h,v 1.13 2006/01/03 02:35:44 campbellbarton Exp $
+ * $Id: BDR_editcurve.h,v 1.14 2006/11/06 18:20:56 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -78,6 +78,11 @@ void extrude_nurb(void);
 void makecyclicNurb(void);
 void selectconnected_nurb(void);
 void selectrow_nurb(void);
+void selectends_nurb(int selFirst);
+void select_more_nurb(void);
+void select_less_nurb(void);
+void select_next_nurb(void);
+void select_prev_nurb(void);
 void adduplicate_nurb(void);
 void delNurb(void);
 void nurb_set_smooth(short event);
