@@ -1,5 +1,5 @@
 /**
- * $Id: BDR_editface.h,v 1.14 2006/03/11 16:13:10 blendix Exp $
+ * $Id: BDR_editface.h,v 1.18 2006/12/21 13:47:25 blendix Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -33,13 +33,13 @@
 #ifndef BDR_EDITFACE_H
 #define BDR_EDITFACE_H
 
-struct TFace;
+struct MTFace;
 struct Mesh;
+struct MCol;
 
-struct TFace *get_active_tface(void);
+struct MTFace *get_active_tface(struct MCol **mcol);
 void calculate_uv_map(unsigned short mapmode);
 void default_uv(float uv[][2], float size);
-void default_tface(struct TFace *tface);
 void make_tfaces(struct Mesh *me);
 void reveal_tface(void);
 void hide_tface(void);
@@ -54,9 +54,11 @@ void face_borderselect(void);
 void uv_autocalc_tface(void);
 void set_faceselect(void);
 void set_texturepaint(void);
-void face_draw(void);
 void get_same_uv(void);  
 void seam_mark_clear_tface(short mode);
 
+void select_mface_from_tface(struct Mesh *me);
+void select_tface_from_mface(struct Mesh *me);
+
 #endif /* BDR_EDITFACE_H */
 
