@@ -1,5 +1,5 @@
 /**
- * $Id: BDR_editobject.h,v 1.26 2006/06/03 17:21:45 ton Exp $
+ * $Id: BDR_editobject.h,v 1.31 2006/11/14 18:13:34 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -38,9 +38,11 @@ struct Tex;
 struct Material;
 struct Base;
 struct HookModifierData;
+struct Scene;
 
 void add_object_draw(int type);
 void add_objectLamp(short type);
+void free_and_unlink_base_from_scene(struct Scene *scene, struct Base *base);
 void free_and_unlink_base(struct Base *base);
 void delete_obj(int ok);
 void make_track(void);
@@ -53,10 +55,15 @@ void set_slowparent(void);
 void make_vertex_parent(void);
 int test_parent_loop(struct Object *par, struct Object *ob);
 void make_parent(void);
+void make_proxy(void);
 
-void exit_editmode(int freedata);
+#define EM_WAITCURSOR	(1 << 0)
+#define EM_FREEDATA 	(1 << 1)
+#define EM_FREEUNDO 	(1 << 2)
+
+void exit_editmode(int flag);
 void check_editmode(int type);
-void enter_editmode(void);
+void enter_editmode(int wc);
 
 void docentre(int centremode);
 void docentre_new(void);
@@ -101,7 +108,7 @@ void select_select_keys(void);
 int vergbaseco(const void *a1, const void *a2);
 void auto_timeoffs(void);
 void texspace_edit(void);
-void flip_subdivison(struct Object *ob, int);
+void flip_subdivison(int);
 void mirrormenu(void);
 
 
