@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_editarmature.h,v 1.11 2004/01/04 03:39:05 hos Exp $
+ * $Id: BIF_editarmature.h,v 1.33 2007/02/10 19:07:47 joeedh Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -33,8 +33,10 @@
 #define BIF_EDITARMATURE_H
 
 struct Object;
+struct Base;
 struct Bone;
 struct bArmature;
+struct ListBase;
 
 typedef struct EditBone
 {
@@ -42,10 +44,9 @@ typedef struct EditBone
 	struct EditBone *parent;/*	Editbones have a one-way link  (i.e. children refer
 									to parents.  This is converted to a two-way link for
 									normal bones when leaving editmode.	*/
-	void	*temp;		/*	Used to store temporary data */
+	void	*temp;			/*	Used to store temporary data */
 
 	char	name[32];
-	char	oldname[32];
 	float	roll;			/*	Roll along axis.  We'll ultimately use the axis/angle method
 								for determining the transformation matrix of the bone.  The axis
 								is tail-head while roll provides the angle. Refer to Graphics
@@ -56,25 +57,17 @@ typedef struct EditBone
 							/*	All joints are considered to have zero rotation with respect to
 							their parents.	Therefore any rotations specified during the
 							animation are automatically relative to the bones' rest positions*/
-	short	sHead[2];
-	short	sTail[2];
 	int		flag;
 
 	int		parNr;		/* Used for retrieving values from the menu system */
-	/*	Storage for transformation data used by the posing system.
-		Maybe a better solution would be to make bones a blenderObject
-		variant?  Or perhaps to use a TOTALLY custom transformation scheme
-		for pose element, rather than trying to use the existing transObject
-		system?
-	*/
+	
 	float dist, weight;
-	float loc[3], dloc[3];
-	float size[3], dsize[3];
-	float rot[3], drot[3];
-	float quat[4], dquat[4];
-	float obmat[4][4];
-	short boneclass;
-
+	float xwidth, length, zwidth;	/* put them in order! transform uses this as scale */
+	float ease1, ease2;
+	float rad_head, rad_tail;
+	short layer, segments;
+	
+	float oldlength;				/* for envelope scaling */
 
 } EditBone;
 
@@ -83,49 +76,63 @@ void	adduplicate_armature(void);
 void	addvert_armature(void);
 void	add_primitiveArmature(int type);
 void	apply_rot_armature (struct Object *ob, float mat[3][3]);
+void 	docentre_armature (struct Object *ob, int centremode);
 
 void	clear_armature(struct Object *ob, char mode);
-void	clever_numbuts_armature (void);
-void	clever_numbuts_posearmature (void);
+
 void	delete_armature(void);
-void	deselectall_armature(void);
-void	deselectall_posearmature (int test);
-void	draw_armature(struct Object *ob);
-void	extrude_armature(void);
+void	deselectall_armature(int toggle, int doundo);
+void	deselectall_posearmature (struct Object *ob, int test, int doundo);
+int		draw_armature(struct Base *base, int dt);
+void	extrude_armature(int forked);
+void	subdivide_armature(void);
+
 void	free_editArmature(void);
-void	join_armature(void);
+
+int		join_armature(void);
 void	load_editArmature(void);
-char*	make_bone_menu(struct bArmature *arm);
+
 void	make_bone_parent(void);
+void    clear_bone_parent(void);
+struct Bone	*get_indexed_bone (struct Object *ob, int index);
+
 void	make_editArmature(void);
 void	make_trans_bones (char mode);
-void	mousepose_armature(void);
+
+int		do_pose_selectbuffer(struct Base *base, unsigned int *buffer, short hits);
+
 void	mouse_armature(void);
 void	remake_editArmature(void);
 void	selectconnected_armature(void);
 void	selectconnected_posearmature(void);
-void	select_bone_by_name (struct bArmature *arm, char *name, int select);
-void    unique_editbone_name (char* name);
-void    attach_bone_to_parent(EditBone *bone);
-void    attach_bone_to_parent_cb(void *bonev, void *arg2_unused);
+void	select_bone_parent(void);
+void    unique_editbone_name (struct ListBase *ebones, char* name);
 
-struct	Bone *get_first_selected_bone (void);
-void auto_align_armature(void);
-void create_vgroups_from_armature(Object *ob, Object *par);
+void	auto_align_armature(void);
+void	create_vgroups_from_armature(Object *ob, Object *par);
 
-void hide_selected_pose_bones(void);
-void hide_unselected_pose_bones(void);
-void show_all_pose_bones(void);
+void	hide_selected_pose_bones(void);
+void	hide_unselected_pose_bones(void);
+void	show_all_pose_bones(void);
 
-int bone_looper(Object *ob, struct Bone *bone, void *data,
+int		bone_looper(Object *ob, struct Bone *bone, void *data,
 				int (*bone_func)(Object *, struct Bone *, void *));
-int ik_chain_looper(Object *ob, struct Bone *bone, void *data,
-				   int (*bone_func)(Object *, struct Bone *, void *));
-int is_delay_deform(void);
 
-#define	BONESEL_TIP		0x08000000
-#define	BONESEL_ROOT	0x04000000
-#define BONESEL_BONE	(BONESEL_TIP|BONESEL_ROOT)
+void	undo_push_armature(char *name);
+void	armature_bone_rename(struct bArmature *arm, char *oldname, char *newname);
+void	armature_flip_names(void);
+EditBone *armature_bone_get_mirrored(EditBone *ebo);
+void	transform_armature_mirror_update(void);
+
+void	hide_selected_armature_bones(void);
+void	hide_unselected_armature_bones(void);
+void	show_all_armature_bones(void);
+
+#define	BONESEL_ROOT	0x10000000
+#define	BONESEL_TIP		0x20000000
+#define	BONESEL_BONE	0x40000000
+#define BONESEL_ANY		(BONESEL_TIP|BONESEL_ROOT|BONESEL_BONE)
+
 #define BONESEL_NOSEL	0x80000000	/* Indicates a negative number */
 
 #endif
