@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_interface.h,v 1.39 2006/01/28 18:33:13 hos Exp $
+ * $Id: BIF_interface.h,v 1.42 2006/12/07 14:17:36 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -163,7 +163,8 @@ struct ScrArea;
 #define BUT_COLORBAND (30<<9)
 #define BUT_NORMAL (31<<9)
 #define BUT_CURVE (32<<9)
-
+#define BUT_TOGDUAL (33<<9)
+#define ICONTOGN (34<<9)
 #define BUTTYPE	(63<<9)
 
 
@@ -305,8 +306,9 @@ extern void uiMatchPanel_view2d(struct ScrArea *sa);
 
 extern void uiDrawBlocksPanels(struct ScrArea *sa, int re_align);
 extern void uiNewPanelHeight(struct uiBlock *block, int sizey);
-void uiPanelPush(uiBlock *block);
-void uiPanelPop(uiBlock *block);
+extern void uiNewPanelTitle(struct uiBlock *block, char *str);
+extern void uiPanelPush(struct uiBlock *block);
+extern void uiPanelPop(struct uiBlock *block);
 extern uiBlock *uiFindOpenPanelBlockName(ListBase *lb, char *name);
 extern int uiAlignPanelStep(struct ScrArea *sa, float fac);
 extern void uiPanelControl(int);
