@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_resources.h,v 1.57 2006/07/03 09:49:12 ton Exp $
+ * $Id: BIF_resources.h,v 1.64 2007/01/07 19:32:58 theeth Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -58,7 +58,11 @@ typedef enum {
 	ICON_SPACE4,
 	ICON_TRIA_LEFT,
 	ICON_TRIA_UP,
-
+	ICON_BLANK3,
+	ICON_BLANK4,
+	ICON_BLANK5,
+	ICON_BLANK6,
+	
 	ICON_ORTHO,
 	ICON_PERSP,
 	ICON_CAMERA,
@@ -80,6 +84,10 @@ typedef enum {
 	ICON_SHORTDISPLAY,
 	ICON_TRIA_DOWN,
 	ICON_TRIA_RIGHT,
+	ICON_BLANK7,
+	ICON_BLANK8,
+	ICON_BLANK9,
+	ICON_BLANK10,
 
 	ICON_VIEW_AXIS_ALL,
 	ICON_VIEW_AXIS_NONE,
@@ -103,6 +111,10 @@ typedef enum {
 	ICON_FF,
 	ICON_REW,
 	ICON_PYTHON,
+	ICON_BLANK11,
+	ICON_BLANK12,
+	ICON_BLANK13,
+	ICON_BLANK14,
 
 	
 	ICON_DOTSUP,
@@ -126,6 +138,10 @@ typedef enum {
 	ICON_WPAINT_DEHLT,
 	ICON_WPAINT_HLT,
 	ICON_MARKER,
+	ICON_BLANK15,
+	ICON_BLANK16,
+	ICON_BLANK17,
+	ICON_BLANK18,
 	
 	ICON_X,
 	ICON_GO_LEFT,
@@ -148,6 +164,10 @@ typedef enum {
 	ICON_FACESEL_DEHLT,
 	ICON_FACESEL_HLT,
 	ICON_EDIT_DEHLT,
+	ICON_BLANK19,
+	ICON_BLANK20,
+	ICON_BLANK21,
+	ICON_BLANK22,
 	
 	ICON_HELP,
 	ICON_ERROR,
@@ -161,15 +181,19 @@ typedef enum {
 	ICON_UGLY_GREEN_RING,
 	ICON_GHOST,
 	ICON_SORTBYEXT,
-	ICON_BLANK33,
+	ICON_SCULPTMODE_HLT,
 	ICON_VERTEXSEL,
 	ICON_EDGESEL,
 	ICON_FACESEL,
-	ICON_BLANK26,
+	ICON_PLUS,
 	ICON_BPIBFOLDER_X,
 	ICON_BPIBFOLDERGREY,
 	ICON_MAGNIFY,
 	ICON_INFO2,
+	ICON_BLANK23,
+	ICON_BLANK24,
+	ICON_BLANK25,
+	ICON_BLANK26,
 	
 	ICON_RIGHTARROW,
 	ICON_DOWNARROW_HLT,
@@ -192,6 +216,10 @@ typedef enum {
 	ICON_KEY_DEHLT,
 	ICON_KEY_HLT,
 	ICON_GRID2,
+	ICON_BLANK27,
+	ICON_BLANK28,
+	ICON_BLANK29,
+	ICON_BLANK30,
 	
 	ICON_EYE,
 	ICON_LAMP,
@@ -214,6 +242,10 @@ typedef enum {
 	ICON_PIN_DEHLT,
 	ICON_PIN_HLT,
 	ICON_LITTLEGRID,
+	ICON_BLANK31,
+	ICON_BLANK32,
+	ICON_BLANK33,
+	ICON_BLANK34,
 	
 	ICON_FULLSCREEN,
 	ICON_SPLITSCREEN,
@@ -236,6 +268,10 @@ typedef enum {
 	ICON_EYEDROPPER,
 	ICON_WINDOW_WINDOW,
 	ICON_PANEL_CLOSE,
+	ICON_BLANK35,
+	ICON_BLANK36,
+	ICON_BLANK37,
+	ICON_BLANK38,
 	
 	ICON_BLENDER,
 	ICON_PACKAGE,
@@ -258,6 +294,10 @@ typedef enum {
 	ICON_CONSTRAINT,
 	ICON_CAMERA_DEHLT,
 	ICON_ARMATURE_DEHLT,
+	ICON_SNAP_GEAR,
+	ICON_SNAP_GEO,
+	ICON_BLANK41,
+	ICON_BLANK42,
 	
 	ICON_SMOOTHCURVE,
 	ICON_SPHERECURVE,
@@ -265,6 +305,7 @@ typedef enum {
 	ICON_SHARPCURVE,
 	ICON_LINCURVE,
 	ICON_NOCURVE,
+	ICON_RNDCURVE,
 	ICON_PROP_OFF,
 	ICON_PROP_ON,
 	ICON_PROP_CON,
@@ -272,7 +313,6 @@ typedef enum {
 	ICON_SYNTAX_OFF,
 	ICON_MONKEY,
 	ICON_HAIR,
-	ICON_PLUS,
 	ICON_VIEWMOVE,
 	ICON_HOME,
 	ICON_CLIPUV_DEHLT,
@@ -280,12 +320,16 @@ typedef enum {
 	ICON_SOME_WACKY_VERTS_AND_LINES,
 	ICON_A_WACKY_VERT_AND_SOME_LINES,
 	ICON_VPAINT_COL,
+	ICON_RESTRICT_SELECT_OFF,
+	ICON_RESTRICT_SELECT_ON,
+	ICON_BLANK45,
+	ICON_BLANK46,
 	
 	ICON_MAN_TRANS,
 	ICON_MAN_ROT,
 	ICON_MAN_SCALE,
 	ICON_MANIPUL,
-	ICON_BLANK65,
+	ICON_BLANK_47,
 	ICON_MODIFIER,
 	ICON_MOD_WAVE,
 	ICON_MOD_BUILD,
@@ -302,6 +346,10 @@ typedef enum {
 	ICON_ROTATECOLLECTION,
 	ICON_ROTATECENTER,
 	ICON_ROTACTIVE,
+	ICON_RESTRICT_VIEW_OFF,
+	ICON_RESTRICT_VIEW_ON,
+	ICON_RESTRICT_RENDER_OFF,
+	ICON_RESTRICT_RENDER_ON,
 
 	VICON_VIEW3D,
 	VICON_EDIT,
@@ -412,6 +460,7 @@ enum {
 	TH_CUSTOM,
 	
 	TH_BUT_TEXTFIELD_HI,
+	TH_ICONFILE,
 	
 	TH_THEMEUI,
 // common colors among spaces
@@ -475,9 +524,15 @@ enum {
 	TH_SEQ_TRANSITION,
 	TH_SEQ_META,
 	
+	TH_EDGE_SHARP,
 };
 /* XXX WARNING: previous is saved in file, so do not change order! */
 
+/* theme drawtypes */
+#define TH_MINIMAL  0
+#define TH_SHADED   1
+#define TH_ROUNDED  2
+#define TH_OLDSKOOL 3
 
 /* specific defines per space should have higher define values */
 
