@@ -3,7 +3,7 @@
  * 
  *	Leftovers here are actually editscreen.c thingies
  * 
- * $Id: BIF_screen.h,v 1.18 2006/01/28 18:33:13 hos Exp $
+ * $Id: BIF_screen.h,v 1.19 2006/07/31 19:23:41 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -125,12 +125,6 @@ int do_screenhandlers(struct bScreen *sc);
 int has_screenhandler(struct bScreen *sc, short eventcode);
 
 	/***/
-	
-/**
- * Draw @a text in the header of each info window
- * of the given screen.
- */
-void screen_draw_info_text(struct bScreen *sc, char *text);
 
 /* return all layers of all 3d windows in a screen */
 unsigned int screen_view3d_layers(void);
