@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_transform.h,v 1.38 2005/08/23 18:13:30 theeth Exp $
+ * $Id: BIF_transform.h,v 1.41 2006/12/19 22:25:07 theeth Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -52,6 +52,8 @@
 #define TFM_MIRROR			14
 #define TFM_BONESIZE		15
 #define TFM_BONE_ENVELOPE	16
+#define TFM_CURVE_SHRINKFATTEN		17
+#define TFM_BONE_ROLL		18
 
 /* TRANSFORM CONTEXTS */
 #define CTX_NONE			0
@@ -74,6 +76,7 @@ void BIF_setLocalAxisConstraint(char axis, char *text);
 void BIF_setLocalLockConstraint(char axis, char *text);
 void BIF_drawConstraint(void);
 void BIF_drawPropCircle(void);
+void BIF_drawSnap(void);
 void BIF_getPropCenter(float *center);
 
 void BIF_TransformSetUndo(char *str);
