@@ -1,4 +1,4 @@
-/* $Id: blendef.h,v 1.53 2006/06/29 13:46:39 ton Exp $
+/* $Id: blendef.h,v 1.61 2006/12/26 02:00:55 briggs Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -44,8 +44,7 @@
 /* max length material array, 16 because of bits in matfrom */
 #define MAXPICKBUF      10000
 #define MAXSEQ          32
-/*  in Image struct */
-#define MAXMIPMAP       10
+
 /* in buttons.c */
 #define MAX_EFFECT      20
 
@@ -89,6 +88,9 @@
 
 #define SELECT			1
 #define ACTIVE			2
+/*#ifdef WITH_VERSE*/
+#define VERSE			3
+/*#endif*/
 #define DESELECT		0
 #define NOT_YET			0
 
@@ -111,6 +113,7 @@
 #define ISPOIN3(a, b, c, d)		( (a->b) && (a->c) && (a->d) )
 #define ISPOIN4(a, b, c, d, e)	( (a->b) && (a->c) && (a->d) && (a->e) )
 
+#define BEZSELECTED(bezt)   (((bezt)->f1 & 1) || ((bezt)->f2 & 1) || ((bezt)->f3 & 1))
 
 /* psfont */
 #define FNT_PDRAW 1
@@ -310,13 +313,25 @@
 #define B_SIMAGESAVE		364
 #define B_SIMACLONEBROWSE	365
 #define B_SIMACLONEDELETE	366
-#define B_SIMABRUSHCHANGE	367
 #define B_SIMANOTHING		368
 #define B_SIMACURVES		369
 #define B_SIMARANGE			370
 #define B_SIMA_USE_ALPHA	371
 #define B_SIMA_SHOW_ALPHA	372
 #define B_SIMA_SHOW_ZBUF	373
+#define B_SIMABRUSHBROWSE	374
+#define B_SIMABRUSHDELETE	375
+#define B_SIMABRUSHLOCAL	376
+#define B_SIMABRUSHCHANGE	377
+#define B_SIMABTEXBROWSE	378
+#define B_SIMABTEXDELETE	379
+#define B_SIMARELOAD		380
+#define B_SIMANAME			381
+#define B_SIMAMULTI			382
+#define B_TRANS_IMAGE		383
+#define B_SIMA_REPACK		384
+#define B_SIMA_PLAY			385
+#define B_SIMA_RECORD		386
 
 /* BUTS: 400 */
 #define B_BUTSHOME		401
@@ -402,6 +417,10 @@
 #define B_KNIFE			0x80
 #define B_PERCENTSUBD	0x40
 #define B_MESH_X_MIRROR	0x100
+#define B_JOINTRIA_UV 0x200
+#define B_JOINTRIA_VCOL 0X400
+#define B_JOINTRIA_SHARP 0X800
+#define B_JOINTRIA_MAT 0X1000
 
 /* DISPLAYMODE */
 #define R_DISPLAYIMAGE	0
@@ -416,5 +435,6 @@
 #define VP_SPRAY	16
 #define VP_MIRROR_X	32
 #define VP_HARD		64
+#define VP_ONLYVGROUP 128
 
 #endif
