@@ -1,5 +1,5 @@
 /**
- * $Id: butspace.h,v 1.83 2006/06/30 19:29:56 ton Exp $
+ * $Id: butspace.h,v 1.101 2007/01/01 08:32:11 aligorith Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -41,6 +41,9 @@ struct ColorBand;
 struct uiBlock;
 struct rctf;
 struct CurveMap;
+struct ImageUser;
+struct RenderResult;
+struct Image;
 
 /* buts->scaflag */		
 #define BUTS_SENS_SEL		1
@@ -90,6 +93,7 @@ extern void get_constraint_ipo_context(void *ob_v, char *actname);
 extern void editing_panels(void);
 extern void do_common_editbuts(unsigned short event);
 extern void do_meshbuts(unsigned short event);
+extern void do_vgroupbuts(unsigned short event);
 extern void do_curvebuts(unsigned short event);
 extern void do_fontbuts(unsigned short event);
 extern void do_mballbuts(unsigned short event);
@@ -98,6 +102,7 @@ extern void do_fpaintbuts(unsigned short event);
 extern void do_cambuts(unsigned short event);
 extern void do_armbuts(unsigned short event);
 extern void do_uvcalculationbuts(unsigned short event);
+extern void weight_paint_buttons(struct uiBlock *);
 
 extern char *get_vertexgroup_menustr(struct Object *ob);	// used in object buttons
 
@@ -113,6 +118,10 @@ extern void radio_panels(void);
 extern void do_radiobuts(unsigned short event);
 extern void texture_panels(void);
 extern void do_texbuts(unsigned short event);
+void uiblock_image_panel(struct uiBlock *block, struct Image **ima_pp, struct ImageUser *iuser, 
+						 short redraw, short imagechanged);
+void uiblock_layer_pass_buttons(struct uiBlock *block, struct RenderResult *rr, 
+							   struct ImageUser *iuser, int event, int x, int y, int w);
 
 /* logic */
 extern void do_logic_buts(unsigned short event);
@@ -149,18 +158,11 @@ void curvemap_buttons(struct uiBlock *block, struct CurveMapping *cumap, char la
 /* *********************** */
 #define B_VIEWBUTS		1100
 
-#define B_LOADBGPIC		1001
-#define B_BLENDBGPIC		1002
-#define B_BGPICBROWSE		1003
-#define B_BGPICTEX		1004
-#define B_BGPICCLEAR		1005
-#define B_BGPICTEXCLEAR		1006
-
 #define B_OBJECTPANELROT 	1007
-#define B_OBJECTPANELMEDIAN 	1008
+#define B_OBJECTPANELMEDIAN 1008
 #define B_ARMATUREPANEL1 	1009
 #define B_ARMATUREPANEL2 	1010
-#define B_OBJECTPANELPARENT 	1011
+#define B_OBJECTPANELPARENT 1011
 #define B_OBJECTPANEL		1012
 #define B_ARMATUREPANEL3 	1013
 #define B_OBJECTPANELSCALE 	1014
@@ -203,19 +205,20 @@ void curvemap_buttons(struct uiBlock *block, struct CurveMapping *cumap, char la
 
 #define B_MAT_LAYERBROWSE	1220
 #define B_MAT_USENODES		1221
+#define B_MAT_VCOL_PAINT	1222
+#define B_MAT_VCOL_LIGHT	1223
 
 /* *********************** */
 #define B_TEXBUTS		1400
 
 #define B_TEXTYPE		1301
 #define B_DEFTEXVAR		1302
-#define B_LOADTEXIMA	1303
+
 #define B_NAMEIMA		1304
 #define B_TEXCHANNEL	1305
 #define B_TEXREDR_PRV	1306
-#define B_TEXIMABROWSE	1307
-#define B_IMAPTEST		1308
-#define B_RELOADIMA		1309
+#define B_IMAGECHANGED	1307
+
 #define B_LOADPLUGIN	1310
 #define B_NAMEPLUGIN	1311
 #define B_COLORBAND		1312
@@ -229,26 +232,29 @@ void curvemap_buttons(struct uiBlock *block, struct CurveMapping *cumap, char la
 #define B_LOADTEXIMA1	1320
 #define B_TEXPRV		1321
 
+
 #define B_PLUGBUT		1325
 /* B_PLUGBUT reserves 24 buttons at least! */
 
 #define B_ENV_MAKE		1350
 #define B_ENV_FREE		1351
-#define B_ENV_DELETE		1352
+#define B_ENV_DELETE	1352
 #define B_ENV_SAVE		1353
 #define B_ENV_OB		1354
 
-#define B_PACKIMA		1355
-#define B_TEXSETFRAMES		1356
+#define B_ENV_FREE_ALL	1357
 
-#define B_ENV_FREE_ALL		1357
-#define B_UNLINKIMA		1358
 
 /* **************** animbuts = object buttons ******* */
 #define B_ANIMBUTS		1500
 
 #define B_RECALCPATH	1401
 #define B_TRACKBUTS		1402
+#define B_DUPLI_FRAME	1403
+#define B_DUPLI_VERTS	1404
+#define B_DUPLI_FACES	1405
+#define B_DUPLI_GROUP	1406
+
 
 #define B_PRINTSPEED	1413
 #define B_PRINTLEN		1414
@@ -329,6 +335,7 @@ void curvemap_buttons(struct uiBlock *block, struct CurveMapping *cumap, char la
 #define B_SET_EDGE			1643
 #define B_SET_ZBLUR			1644
 #define B_ADD_RENDERLAYER	1645
+#define B_SET_PASS			1646
 
 /* *********************** */
 #define B_ARMATUREBUTS		1800
@@ -365,7 +372,7 @@ void curvemap_buttons(struct uiBlock *block, struct CurveMapping *cumap, char la
 #define B_NEXTKEY		2046
 #define B_LOCKKEY		2047
 
-#define B_MESHBUTS		2100
+#define B_MESHBUTS		2090
 
 #define B_FLIPNORM		2050
 #define B_SPIN			2051
@@ -376,34 +383,42 @@ void curvemap_buttons(struct uiBlock *block, struct CurveMapping *cumap, char la
 #define B_SPLIT			2056
 #define B_REMDOUB		2057
 #define B_SUBDIV		2058
-#define B_FRACSUBDIV		2059
+#define B_FRACSUBDIV	2059
 #define B_XSORT			2060
 #define B_HASH			2061
-#define B_DELSTICKY		2062
-#define B_DELVERTCOL		2063
-#define B_MAKE_TFACES		2064
-#define B_TOSPHERE		2065
-#define B_DEL_TFACES		2066
-#define B_NEWVGROUP		2067
-#define B_DELVGROUP		2068
-#define B_ASSIGNVGROUP		2069
-#define B_REMOVEVGROUP		2070
-#define B_SELVGROUP		2071	
-#define B_DESELVGROUP		2072
-#define B_DECIM_FACES		2073
-#define B_DECIM_CANCEL		2074
-#define B_DECIM_APPLY		2075
-#define B_AUTOVGROUP		2076
-#define B_SLOWERDRAW		2077
-#define B_FASTERDRAW		2078
-#define B_VERTEXNOISE		2079
-#define B_VERTEXSMOOTH		2080
-#define B_MAKESTICKY		2082
-#define B_MAKEVERTCOL		2083
-#define B_CHROMADEPTH		2084
-#define B_DRAWEDGES		2087
-#define B_DRAWCREASES		2088
-#define B_LINKEDVGROUP		2089
+#define B_MAKESTICKY	2062
+#define B_DELSTICKY		2063
+#define B_NEWMCOL		2064
+#define B_DELMCOL		2065
+#define B_TOSPHERE			2066
+#define B_DECIM_FACES		2067
+#define B_DECIM_CANCEL		2068
+#define B_DECIM_APPLY		2069
+/* B_SLOWERDRAW and B_FASTERDRAW removed */
+#define B_VERTEXNOISE		2072
+#define B_VERTEXSMOOTH		2073
+#define B_NEWTFACE			2074
+#define B_DELTFACE			2075
+#define B_CHROMADEPTH		2076
+#define B_DRAWEDGES			2077
+#define B_DRAWCREASES		2078
+#define B_SETTFACE			2079
+#define B_SETMCOL			2080
+#define B_JOINTRIA			2081
+
+/* *********************** */
+#define B_VGROUPBUTS		2100
+
+#define B_NEWVGROUP			2091
+#define B_DELVGROUP			2092
+#define B_ASSIGNVGROUP		2093
+#define B_REMOVEVGROUP		2094
+#define B_SELVGROUP			2095	
+#define B_DESELVGROUP		2096
+#define B_AUTOVGROUP		2097
+#define B_LINKEDVGROUP		2098
+
+
 
 /* *********************** */
 #define B_CURVEBUTS		2200
@@ -461,6 +476,8 @@ void curvemap_buttons(struct uiBlock *block, struct CurveMapping *cumap, char la
 
 #define B_ARM_RECALCDATA	2301
 #define B_ARM_STRIDE		2302
+#define B_ARM_CALCPATHS		2303
+#define B_ARM_CLEARPATHS	2304
 
 /* *********************** */
 #define B_CAMBUTS		2500
@@ -537,6 +554,17 @@ void curvemap_buttons(struct uiBlock *block, struct CurveMapping *cumap, char la
 
 #define B_CLR_WPAINT		2850
 
+#define B_BRUSHBROWSE	2851
+#define B_BRUSHDELETE	2852
+#define B_BRUSHLOCAL	2853
+#define B_BRUSHCHANGE	2854
+#define B_BTEXBROWSE	2855
+#define B_BTEXDELETE	2856
+#define B_BRUSHKEEPDATA 2857
+
+/* Sculptmode */
+#define B_SCULPT_TEXBROWSE      2860
+
 /* *********************** */
 #define B_RADIOBUTS		3000
 
@@ -610,6 +638,10 @@ enum {
 	B_CONSTRAINT_ADD_FOLLOWPATH,
 	B_CONSTRAINT_ADD_DISTANCELIMIT,
 	B_CONSTRAINT_ADD_STRETCHTO,
+	B_CONSTRAINT_ADD_LOCLIMIT,
+	B_CONSTRAINT_ADD_ROTLIMIT,
+	B_CONSTRAINT_ADD_SIZELIMIT,
+	B_CONSTRAINT_ADD_RIGIDBODYJOINT,
 	B_CONSTRAINT_INF
 };
 
@@ -623,14 +655,8 @@ enum {
 	B_UVAUTO_WINDOW,
 	B_UVAUTO_CUBE,
 	B_UVAUTO_CUBESIZE,
-	B_UVAUTO_STD1,
-	B_UVAUTO_STD2,
-	B_UVAUTO_STD4,
-	B_UVAUTO_STD8,
-	B_UVAUTO_BOUNDS1,
-	B_UVAUTO_BOUNDS2,
-	B_UVAUTO_BOUNDS4,
-	B_UVAUTO_BOUNDS8,
+	B_UVAUTO_RESET,
+	B_UVAUTO_BOUNDS,
 	B_UVAUTO_TOP,
 	B_UVAUTO_FACE,
 	B_UVAUTO_OBJECT,
