@@ -1,5 +1,5 @@
 /**
- * $Id: transform.h,v 1.25 2006/07/08 13:18:56 ton Exp $
+ * $Id: transform.h,v 1.36 2006/12/28 22:42:58 theeth Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -39,8 +39,12 @@
 
 struct TransInfo;
 struct TransData;
+struct TransSnap;
 struct NumInput;
 struct Object;
+struct View3D;
+struct ScrArea;
+
 
 typedef struct NumInput {
     short  idx;
@@ -59,6 +63,20 @@ typedef struct NumInput {
 		Negative	: number is negative
 */
 
+typedef struct TransSnap {
+	short	modePoint;
+	short	modeTarget;
+	int  	status;
+	float	snapPoint[3];
+	float	snapTarget[3];
+	float	dist; // Distance from snapPoint to snapTarget
+	double	last;
+	void  (*applySnap)(struct TransInfo *, float *);
+	void  (*calcSnap)(struct TransInfo *, float *);
+	void  (*targetSnap)(struct TransInfo *);
+	float  (*distance)(struct TransInfo *, float p1[3], float p2[3]); // Get the transform distance between two points (used by Closest snap)
+} TransSnap;
+
 typedef struct TransCon {
     char  text[50];      /* Description of the Constraint for header_print                            */
     float mtx[3][3];     /* Matrix of the Constraint space                                            */
@@ -128,6 +146,9 @@ typedef struct TransData {
 	void *tdmir;		 /* mirrored element pointer, in editmode mesh to EditVert */
     short  flag;         /* Various flags */
 	short  protectflag;	 /* If set, copy of Object or PoseChannel protection */
+/*#ifdef WITH_VERSE*/
+	void *verse;			/* pointer at verse data struct (VerseVert, etc.) */
+/*#endif*/
 } TransData;
 
 typedef struct TransInfo {
@@ -144,6 +165,7 @@ typedef struct TransInfo {
 	TransDataExtension *ext;	/* transformed data extension (array)   */
 	TransData2D *data2d;		/* transformed data for 2d (array)      */
     TransCon    con;            /* transformed constraint               */
+    TransSnap	tsnap;
     NumInput    num;            /* numerical input                      */
     char        redraw;         /* redraw flag                          */
 	float		propsize;		/* proportional circle radius           */
@@ -232,6 +254,24 @@ typedef struct TransInfo {
 #define	TD_USEQUAT			4
 #define TD_NOTCONNECTED		8
 #define TD_SINGLESIZE		16	/* used for scaling of MetaElem->rad */
+#ifdef WITH_VERSE
+#define TD_VERSE_OBJECT		32
+#define TD_VERSE_VERT		64
+#endif
+
+/* transsnap->status */
+#define SNAP_ON			1
+#define TARGET_INIT		2
+#define POINT_INIT		4
+
+/* transsnap->modePoint */
+#define SNAP_GRID			0
+#define SNAP_GEO			1
+
+/* transsnap->modeTarget */
+#define SNAP_CLOSEST		0
+#define SNAP_CENTER			1
+#define SNAP_MEDIAN			2
 
 void checkFirstTime(void);
 
@@ -267,6 +307,9 @@ int ShrinkFatten(TransInfo *t, short mval[2]);
 void initTilt(TransInfo *t);
 int Tilt(TransInfo *t, short mval[2]);
 
+void initCurveShrinkFatten(TransInfo *t);
+int CurveShrinkFatten(TransInfo *t, short mval[2]);
+
 void initTrackball(TransInfo *t);
 int Trackball(TransInfo *t, short mval[2]);
 
@@ -282,15 +325,18 @@ int BoneSize(TransInfo *t, short mval[2]);
 void initBoneEnvelope(TransInfo *t);
 int BoneEnvelope(TransInfo *t, short mval[2]);
 
+void initBoneRoll(TransInfo *t);
+int BoneRoll(TransInfo *t, short mval[2]);
+
 /*********************** transform_conversions.c ********** */
 struct ListBase;
 void flushTransUVs(TransInfo *t);
 int clipUVTransform(TransInfo *t, float *vec, int resize);
 
 /*********************** exported from transform_manipulator.c ********** */
-struct ScrArea;
 void draw_manipulator_ext(struct ScrArea *sa, int type, char axis, int col, float vec[3], float mat[][3]);
 int calc_manipulator_stats(struct ScrArea *sa);
+float get_drawsize(struct View3D *v3d);
 
 /*********************** TransData Creation and General Handling *********** */
 void createTransData(TransInfo *t);
@@ -321,6 +367,23 @@ void postSelectConstraint(TransInfo *t);
 
 void setNearestAxis(TransInfo *t);
 
+/*********************** Snapping ********************************/
+
+typedef enum {
+	NO_GEARS 	= 0,
+	BIG_GEARS	= 1,
+	SMALL_GEARS	= 2
+} GearsType;
+
+void snapGrid(TransInfo *t, float *val);
+void snapGridAction(TransInfo *t, float *val, GearsType action);
+
+void initSnapping(struct TransInfo *t);
+void applySnapping(TransInfo *t, float *vec);
+void resetSnapping(TransInfo *t);
+int  handleSnapping(TransInfo *t, int event);
+void drawSnapping(TransInfo *t);
+
 /*********************** Generics ********************************/
 
 void initTrans(TransInfo *t);
@@ -346,8 +409,6 @@ void calculateCenterCursor(TransInfo *t);
 
 void calculatePropRatio(TransInfo *t);
 
-void snapGrid(TransInfo *t, float *val);
-
 void getViewVector(float coord[3], float vec[3]);
 
 TransInfo * BIF_GetTransInfo(void);
