@@ -1,5 +1,5 @@
 /*  
- * $Id: DNA_action_types.h,v 1.28 2006/05/06 15:26:52 schlaile Exp $
+ * $Id: DNA_action_types.h,v 1.38 2007/01/09 22:27:26 aligorith Exp $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -49,11 +49,14 @@ typedef struct bPoseChannel {
 	short				flag;		/* dynamic, for detecting transform changes */
 	short				constflag;  /* for quick detecting which constraints affect this channel */
 	short				ikflag;		/* settings for IK bones */
-	short                           pad1;
-	int				pathlen;	/* for drawing paths, the amount of frames */
-	short				protectflag;/* protect channels from being transformed */
+	short               selectflag;	/* copy of bone flag, so you can work with library armatures */
+	short				protectflag; /* protect channels from being transformed */
 	short				pad2;
 	
+	int				    pathlen;	/* for drawing paths, the amount of frames */
+	int 				pathsf;		/* for drawing paths, the start frame number */
+	int					pathef;		/* for drawing paths, the end frame number */
+	
 	struct Bone			*bone;		/* set on read file or rebuild pose */
 	struct bPoseChannel *parent;	/* set on read file or rebuild pose */
 	struct bPoseChannel *child;		/* set on read file or rebuild pose, the 'ik' child, for b-bones */
@@ -82,8 +85,10 @@ typedef struct bPoseChannel {
 
 typedef struct bPose{
 	ListBase			chanbase;
-	int flag;
-	float stride_offset[3];
+	short flag, proxy_layer;	/* proxy layer: copy from armature, gets synced */
+	float ctime;				/* local action time of this pose */
+	float stride_offset[3];		/* applied to object */
+	float cyclic_offset[3];		/* result of match and cycles, applied in where_is_pose() */
 } bPose;
 
 typedef struct bActionChannel {
@@ -98,7 +103,7 @@ typedef struct bActionChannel {
 
 typedef struct bAction {
 	ID				id;
-	ListBase		chanbase;	/* Channels in this action */
+	ListBase		chanbase;	/* Action Channels in this action */
 } bAction;
 
 typedef struct SpaceAction {
@@ -111,8 +116,8 @@ typedef struct SpaceAction {
 
 	View2D v2d;	
 	bAction		*action;
-	int	flag;
-	short pin, actnr, lock, pad;
+	short flag, autosnap;	
+	short pin, actnr, lock, actwidth;
 	float timeslide;
 } SpaceAction;
 
@@ -120,10 +125,17 @@ typedef struct SpaceAction {
 #define	ACHAN_SELECTED	0x00000001
 #define ACHAN_HILIGHTED	0x00000002
 #define ACHAN_HIDDEN	0x00000004
+#define ACHAN_PROTECTED 0x00000008
 #define ACHAN_MOVED     0x80000000
 
 /* SpaceAction flag */
-#define SACTION_MOVING	1
+#define SACTION_MOVING		1	/* during transform */
+#define SACTION_SLIDERS		2	/* show sliders (if relevant) - limited to shape keys for now */
+
+/* SpaceAction AutoSnap Settings */
+#define SACTSNAP_OFF	0	/* no auto-snap */
+#define SACTSNAP_STEP	1	/* snap to 1.0 frame intervals */
+#define SACTSNAP_FRAME	2	/* snap to actual frames (nla-action time) */
 
 /* Pose->flag */
 #define POSE_RECALC		1
