@@ -3,7 +3,7 @@
  *
  * Curve stuff.
  *
- * $Id: DNA_curve_types.h,v 1.26 2006/05/02 14:35:47 sirdude Exp $ 
+ * $Id: DNA_curve_types.h,v 1.29 2007/02/14 13:52:29 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -89,7 +89,7 @@ typedef struct BevPoint {
 */
 typedef struct BezTriple {
 	float vec[3][3];
-	float alfa, weight, pad;
+	float alfa, weight, radius;	/* alfa: tilt in 3D View, weight: used for softbody goal weight, radius: for bevel tapering */
 	short h1, h2;
 	char f1, f2, f3, hide;
 } BezTriple;
@@ -97,17 +97,18 @@ typedef struct BezTriple {
 /* note; alfa location in struct is abused by Key system */
 typedef struct BPoint {
 	float vec[4];
-	float alfa, weight;
+	float alfa, weight;		/* alfa: tilt in 3D View, weight: used for softbody goal weight */
 	short f1, hide;
+	float radius, pad;		/* user-set radius per point for bevelling etc */
 } BPoint;
 
 typedef struct Nurb {
-	struct Nurb *next, *prev;
+	struct Nurb *next, *prev;	/* multiple nurbs per curve object are allowed */
 	short type;
-	short mat_nr;          /* index into material list */
+	short mat_nr;		/* index into material list */
 	short hide, flag;
-	short pntsu, pntsv;
-	short resolu, resolv;
+	short pntsu, pntsv;		/* number of points in the U or V directions */
+	short resolu, resolv;	/* tesselation resolution in the U or V directions */
 	short orderu, orderv;
 	short flagu, flagv;
 	
@@ -146,6 +147,7 @@ typedef struct Curve {
 	
 	ListBase bev;
 	
+	/* texture space, copied as one block in editobject.c */
 	float loc[3];
 	float size[3];
 	float rot[3];
@@ -228,6 +230,7 @@ typedef struct IpoCurve {
 #define CU_STRETCH		128
 #define CU_OFFS_PATHDIST	256
 #define CU_FAST			512 /* Font: no filling inside editmode */
+#define CU_RETOPO               1024
 
 /* spacemode */
 #define CU_LEFT			0
