@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_effect_types.h (mar-2001 nzc)
  *	
- * $Id: DNA_effect_types.h,v 1.13 2006/01/28 18:33:14 hos Exp $ 
+ * $Id: DNA_effect_types.h,v 1.14 2006/11/16 20:23:00 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -117,7 +117,7 @@ typedef struct PartEff {
 	float force[3];
 	float damp;
 	
-	float nabla, vectsize, defvec[3];
+	float nabla, vectsize, maxlen, pad, defvec[3];
 	
 	float mult[4], life[4];
 	short child[4], mat[4];
