@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_material_types.h (mar-2001 nzc)
  *
- * $Id: DNA_material_types.h,v 1.42 2006/06/26 17:50:48 ton Exp $ 
+ * $Id: DNA_material_types.h,v 1.45 2006/12/08 09:40:43 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -77,7 +77,9 @@ typedef struct Material {
 	short flarec, starc, linec, ringc;
 	float hasize, flaresize, subsize, flareboost;
 	float strand_sta, strand_end, strand_ease;
+	
 	float sbias;			/* shadow bias */
+	float shad_alpha, padf;	/* in use for irregular shadowbuffer */
 	
 	/* for buttons and render*/
 	char rgbsel, texact, pr_type, use_nodes;
@@ -168,8 +170,11 @@ typedef struct Material {
 #define MA_SHADBUF		0x2000000
 		/* note; we drop MA_TANGENT_STR later to become tangent_u */
 #define MA_TANGENT_V	0x4000000
+/* qdn: a bit clumsy this, tangents needed for normal maps separated from shading */
+#define MA_NORMAP_TANG	0x8000000
+#define MA_GROUP_NOLAY	0x10000000
 
-#define	MA_MODE_MASK	0x3ffffff	/* all valid mode bits */
+#define	MA_MODE_MASK	0x1fffffff	/* all valid mode bits */
 
 /* diff_shader */
 #define MA_DIFF_LAMBERT		0
