@@ -1,10 +1,12 @@
 /**
- * $Id: DNA_modifier_types.h,v 1.19 2006/06/14 13:25:01 ton Exp $ 
+ * $Id: DNA_modifier_types.h,v 1.30 2007/01/29 16:28:11 artificer Exp $ 
  */
 
 #ifndef DNA_MODIFIER_TYPES_H
 #define DNA_MODIFIER_TYPES_H
 
+#define MODSTACK_DEBUG 1
+
 /* WARNING ALERT! TYPEDEF VALUES ARE WRITTEN IN FILES! SO DO NOT CHANGE! */
 
 typedef enum ModifierType {
@@ -21,14 +23,12 @@ typedef enum ModifierType {
 	eModifierType_Softbody,
 	eModifierType_Boolean,
 	eModifierType_Array,
-
+	eModifierType_EdgeSplit,
+	eModifierType_Displace,
+	eModifierType_UVProject,
 	NUM_MODIFIER_TYPES
 } ModifierType;
 
-	/* These numerical values are explicitly chosen so that 
-	 * (mode&1) is true for realtime calc and (mode&2) is true
-	 * for render calc.
-	 */
 typedef enum ModifierMode {
 	eModifierMode_Realtime = (1<<0),
 	eModifierMode_Render = (1<<1),
@@ -74,8 +74,18 @@ typedef struct CurveModifierData {
 
 	struct Object *object;
 	char name[32];			/* optional vertexgroup name */
+	short defaxis;			/* axis along which curve deforms */
+	char pad[6];
 } CurveModifierData;
 
+/* CurveModifierData->defaxis */
+#define MOD_CURVE_POSX	1
+#define MOD_CURVE_POSY	2
+#define MOD_CURVE_POSZ	3
+#define MOD_CURVE_NEGX	4
+#define MOD_CURVE_NEGY	5
+#define MOD_CURVE_NEGZ	6
+
 typedef struct BuildModifierData {
 	ModifierData modifier;
 
@@ -147,6 +157,66 @@ typedef struct MirrorModifierData {
 /* MirrorModifierData->flag */
 #define MOD_MIR_CLIPPING	1
 
+typedef struct EdgeSplitModifierData {
+	ModifierData modifier;
+
+	float split_angle;    /* angle above which edges should be split */
+	int flags;
+} EdgeSplitModifierData;
+
+/* EdgeSplitModifierData->flags */
+#define MOD_EDGESPLIT_FROMANGLE   1<<1
+#define MOD_EDGESPLIT_FROMFLAG    1<<2
+
+typedef struct DisplaceModifierData {
+	ModifierData modifier;
+
+	struct Tex *texture;
+	float strength;
+	int direction;
+	char defgrp_name[32];
+	float midlevel;
+	int texmapping;
+	struct Object *map_object;
+	char uvlayer_name[32];
+	int uvlayer_tmp, pad;
+} DisplaceModifierData;
+
+/* DisplaceModifierData->direction */
+enum {
+	MOD_DISP_DIR_X,
+	MOD_DISP_DIR_Y,
+	MOD_DISP_DIR_Z,
+	MOD_DISP_DIR_NOR,
+	MOD_DISP_DIR_RGB_XYZ,
+};
+
+/* DisplaceModifierData->texmapping */
+enum {
+	MOD_DISP_MAP_LOCAL,
+	MOD_DISP_MAP_GLOBAL,
+	MOD_DISP_MAP_OBJECT,
+	MOD_DISP_MAP_UV,
+};
+
+typedef struct UVProjectModifierData {
+	ModifierData modifier;
+
+	/* the objects which do the projecting */
+	struct Object *projectors[10];
+	struct Image *image;      /* the image to project */
+	int flags;
+	int num_projectors;
+	float aspectx, aspecty;
+	char uvlayer_name[32];
+	int uvlayer_tmp, pad;
+} UVProjectModifierData;
+
+#define MOD_UVPROJECT_MAXPROJECTORS 10
+
+/* UVProjectModifierData->flags */
+#define MOD_UVPROJECT_OVERRIDEIMAGE 1<<0
+
 typedef struct DecimateModifierData {
 	ModifierData modifier;
 
@@ -157,6 +227,8 @@ typedef struct DecimateModifierData {
 typedef struct WaveModifierData {
 	ModifierData modifier;
 
+	struct Object *objectcenter;
+
 	short flag, pad;
 
 	float startx, starty, height, width;
@@ -171,6 +243,7 @@ typedef struct ArmatureModifierData {
 	short deformflag, pad1;		/* deformflag replaces armature->deformflag */
 	int pad2;
 	struct Object *object;
+	char defgrp_name[32];
 } ArmatureModifierData;
 
 typedef struct HookModifierData {
