@@ -1,7 +1,7 @@
 /**
  * blenkernel/DNA_userdef_types.h (mar-2001 nzc)
  *
- *	$Id: DNA_userdef_types.h,v 1.56 2006/06/08 19:05:58 schlaile Exp $
+ *	$Id: DNA_userdef_types.h,v 1.64 2007/01/03 05:50:16 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -35,6 +35,8 @@
 #ifndef DNA_USERDEF_TYPES_H
 #define DNA_USERDEF_TYPES_H
 
+#include "DNA_listBase.h"
+
 /* themes; defines in BIF_resource.h */
 
 // global, button colors
@@ -59,7 +61,8 @@ typedef struct ThemeUI {
 	char menu_text_hi[4];
 	
 	char but_drawtype;
-	char pad1[3];
+	char pad[3];
+	char iconfile[80];	// FILE_MAXFILE length
 
 } ThemeUI;
 
@@ -83,7 +86,7 @@ typedef struct ThemeSpace {
 	char active[4], group[4], group_active[4], transform[4];
 	char vertex[4], vertex_select[4];
 	char edge[4], edge_select[4];
-	char edge_seam[4], edge_facesel[4];
+	char edge_seam[4], edge_sharp[4], edge_facesel[4];
 	char face[4], face_select[4];	// solid faces
 	char face_dot[4];				// selected color
 	char normal[4];
@@ -91,13 +94,14 @@ typedef struct ThemeSpace {
 	char strip[4], strip_select[4];
 	
 	char vertex_size, facedot_size;
-	char bpad[2];
+	char bpad[2]; 
 
 	char syntaxl[4], syntaxn[4], syntaxb[4]; // syntax for textwindow and nodes
 	char syntaxv[4], syntaxc[4];
 	
 	char movie[4], image[4], scene[4], audio[4];		// for sequence editor
 	char effect[4], plugin[4], transition[4], meta[4];
+	char bpad1[4]; 
 	
 } ThemeSpace;
 
@@ -166,13 +170,15 @@ typedef struct UserDef {
 	struct SolidLight light[3];
 	short tw_hotspot, tw_flag, tw_handlesize, tw_size;
 	int textimeout, texcollectrate;
-        int memcachelimit;
-        short frameserverport;
-	short pad;
+	int memcachelimit;
+	short frameserverport;
+	short pad_rot_angle;	/*control the rotation step of the view when PAD2,PAD4,PAD6&PAD8 is use*/
 	short obcenter_dia;
-	short rvisize;		/* rotating view icon size */
-	short rvibright;	/* rotating view icon brightness */
-	short pad1;
+	short rvisize;			/* rotating view icon size */
+	short rvibright;		/* rotating view icon brightness */
+	char versemaster[160];
+	char verseuser[160];
+	short pad;
 } UserDef;
 
 extern UserDef U; /* from usiblender.c !!!! */
@@ -223,6 +229,7 @@ extern UserDef U; /* from usiblender.c !!!! */
 #define USER_HIDE_DOT			65536
 #define USER_SHOW_ROTVIEWICON	131072
 #define USER_SHOW_VIEWPORTNAME	262144
+#define USER_KEYINSERTNEED		524288
 
 /* transopts */
 
@@ -254,7 +261,6 @@ extern UserDef U; /* from usiblender.c !!!! */
 #define USER_DISABLE_SOUND		2
 #define USER_DISABLE_MIPMAP		4
 
-
 /* vrml flag */
 
 #define USER_VRML_LAYERS		1
@@ -265,4 +271,3 @@ extern UserDef U; /* from usiblender.c !!!! */
 
 
 #endif
-
