@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_view3d_types.h (mar-2001 nzc)
  *
- * $Id: DNA_view3d_types.h,v 1.33 2006/07/19 11:28:48 ton Exp $ 
+ * $Id: DNA_view3d_types.h,v 1.41 2007/01/07 19:32:59 theeth Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -34,6 +34,7 @@
 #ifndef DNA_VIEW3D_TYPES_H
 #define DNA_VIEW3D_TYPES_H
 
+struct ViewDepths;
 struct Object;
 struct Image;
 struct Tex;
@@ -41,6 +42,7 @@ struct SpaceLink;
 struct Base;
 struct BoundBox;
 struct RenderInfo;
+struct RetopoViewData;
 
 /* This is needed to not let VC choke on near and far... old
  * proprietary MS extensions... */
@@ -52,17 +54,16 @@ struct RenderInfo;
 #endif
 
 #include "DNA_listBase.h"
+#include "DNA_image_types.h"
 
 /* The near/far thing is a Win EXCEPTION. Thus, leave near/far in the
  * code, and patch for windows. */
 
 typedef struct BGpic {
     struct Image *ima;
-	struct ImBuf *ibuf;
-	struct Tex *tex;
+	struct ImageUser iuser;
     float xof, yof, size, zoom, blend;
     short xim, yim;
-	unsigned int *rect;
 } BGpic;
 
 typedef struct View3D {
@@ -91,10 +92,14 @@ typedef struct View3D {
 	short persp;
 	short view;
 
-	struct Object *camera;
+	struct Object *camera, *ob_centre;
 	struct BGpic *bgpic;
 	struct View3D *localvd;
 	struct RenderInfo *ri;
+	struct RetopoViewData *retopo_view_data;
+	struct ViewDepths *depths;
+	
+	char ob_centre_bone[32];		/* optional string for armature bone to define centre */
 	
 	/**
 	 * The drawing mode for the 3d display. Set to OB_WIRE, OB_SOLID,
@@ -104,7 +109,7 @@ typedef struct View3D {
 	int lay, layact;
 	short scenelock, around, camzoom;
 	
-	short pad1;
+	char pivot_last, pad1; /* pivot_last is for rotating around the last edited element */
 	
 	float lens, grid, gridview, pixsize, near, far;
 	float camdx, camdy;		/* camera view offsets, 1.0 = viewplane moves entire width/height */
@@ -130,7 +135,13 @@ typedef struct View3D {
 
 	short flag, flag2;
 	
-	short pad2[3];
+	short gridsubdiv;	/* Number of subdivisions in the grid between each highlighted grid line */
+	
+	short snap_target;
+	
+	short pad2;
+	
+	void *properties_storage;	/* Nkey panel stores stuff here, not in file */
 
 } View3D;
 
@@ -156,6 +167,13 @@ typedef struct View3D {
 /* View3d->flag2 (short) */
 #define V3D_OPP_DIRECTION_NAME	1
 #define V3D_FLYMODE				2
+#define V3D_TRANSFORM_SNAP		4
+
+/* View3d->snap_target */
+#define V3D_SNAP_TARGET_CLOSEST	0
+#define V3D_SNAP_TARGET_CENTER	1
+#define V3D_SNAP_TARGET_MEDIAN	2
+
 
 /* View3D->around */
 #define V3D_CENTRE		 0
