@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_world_types.h (mar-2001 nzc)
  *
- * $Id: DNA_world_types.h,v 1.19 2006/01/28 18:33:14 hos Exp $ 
+ * $Id: DNA_world_types.h,v 1.20 2006/11/29 17:01:07 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -102,7 +102,7 @@ typedef struct World {
 	/* ambient occlusion */
 	float aodist, aodistfac, aoenergy, aobias;
 	short aomode, aosamp, aomix, aocolor;
-	float *aosphere;
+	float *aosphere, *aotables;
 	
 	struct Ipo *ipo;
 	struct MTex *mtex[10];
