@@ -1,5 +1,5 @@
 /* 
- * $Id: BPY_interface.c,v 1.88 2006/07/08 20:40:41 khughes Exp $
+ * $Id: BPY_interface.c,v 1.94 2007/02/09 18:27:20 elubie Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -77,6 +77,32 @@
 /* for pydrivers (ipo drivers defined by one-line Python expressions) */
 PyObject *bpy_pydriver_Dict = NULL;
 
+
+/*
+ * set up a weakref list for Armatures
+ *    creates list in __main__ module dict 
+ */
+  
+int setup_armature_weakrefs()
+{
+	PyObject *maindict;
+	PyObject *main_module;
+	char *list_name = ARM_WEAKREF_LIST_NAME;
+
+	main_module = PyImport_AddModule( "__main__");
+	if(main_module){
+		maindict= PyModule_GetDict(main_module);
+		if (PyDict_SetItemString(maindict, 
+								 list_name, 
+								 PyList_New(0)) == -1){
+			printf("Oops - setup_armature_weakrefs()\n");
+			
+			return 0;
+		}
+	}
+	return 1;
+}
+
 /*Declares the modules and their initialization functions
 *These are TOP-LEVEL modules e.g. import `module` - there is no
 *support for packages here e.g. import `package.module` */
@@ -116,8 +142,6 @@ PyObject *blender_import( PyObject * self, PyObject * args );
 void BPY_Err_Handle( char *script_name );
 PyObject *traceback_getFilename( PyObject * tb );
 
-void BPY_free_screen_spacehandlers(struct bScreen *sc);
-
 /****************************************************************************
 * Description: This function will start the interpreter and load all modules
 * as well as search for a python installation.
@@ -308,11 +332,11 @@ that dir info is available.
 ****************************************************************************/
 void BPY_post_start_python( void )
 {
-	char dirpath[FILE_MAXDIR];
+	char dirpath[FILE_MAX];
 	char *sdir = NULL;
 
 	if(U.pythondir[0] != '\0' ) {
-		char modpath[FILE_MAXDIR];
+		char modpath[FILE_MAX];
 		int upyslen = strlen(U.pythondir);
 
 		/* check if user pydir ends with a slash and, if so, remove the slash
@@ -325,7 +349,7 @@ void BPY_post_start_python( void )
 				U.pythondir[upyslen - 1] = '\0';
 		}
 
-		BLI_strncpy(dirpath, U.pythondir, FILE_MAXDIR);
+		BLI_strncpy(dirpath, U.pythondir, FILE_MAX);
 		BLI_convertstringcode(dirpath, G.sce, 0);
 		syspath_append(dirpath);	/* append to module search path */
 
@@ -526,6 +550,11 @@ int BPY_txt_do_python_Text( struct Text *text )
 
 	py_dict = CreateGlobalDictionary(  );
 
+	if( !setup_armature_weakrefs()){
+		printf("Oops - weakref dict\n");
+		return 0;
+	}
+
 	script->py_globaldict = py_dict;
 
 	info = ( BPy_constant * ) PyConstant_New(  );
@@ -818,6 +847,12 @@ int BPY_menu_do_python( short menutype, int event )
 
 	fclose( fp );
 
+
+	if( !setup_armature_weakrefs()){
+		printf("Oops - weakref dict\n");
+		return 0;
+	}
+
 	/* run the string buffer */
 
 	py_res = PyRun_String( buffer, Py_file_input, py_dict, py_dict );
@@ -1047,9 +1082,12 @@ static float pydriver_error(IpoDriver *driver) {
 	}
 
 	driver->flag |= IPO_DRIVER_FLAG_INVALID; /* py expression failed */
-
-	fprintf(stderr, "\nError in Ipo Driver: Object %s\nThis is the failed Python expression:\n'%s'\n\n", driver->ob->id.name+2, driver->name);
-
+	
+	if (driver->ob)
+		fprintf(stderr, "\nError in Ipo Driver: Object %s\nThis is the failed Python expression:\n'%s'\n\n", driver->ob->id.name+2, driver->name);
+	else
+		fprintf(stderr, "\nError in Ipo Driver: No Object\nThis is the failed Python expression:\n'%s'\n\n", driver->name);
+	
 	PyErr_Print();
 
 	return 0.0f;
@@ -1121,6 +1159,12 @@ float BPY_pydriver_eval(IpoDriver *driver)
 		}
 	}
 
+/* sds*/
+	if( !setup_armature_weakrefs()){
+		fprintf( stderr, "Oops - weakref dict setup\n");
+		return result;
+	}
+
 	retval = PyRun_String(expr, Py_eval_input, bpy_pydriver_Dict,
 		bpy_pydriver_Dict);
 
@@ -1191,6 +1235,12 @@ int BPY_button_eval(char *expr, double *value)
 		}
 	}
 
+/* sds*/
+	if( !setup_armature_weakrefs()){
+		fprintf(stderr, "Oops - weakref dict\n");
+		return -1;
+	}
+
 	retval = PyRun_String(expr, Py_eval_input, bpy_pydriver_Dict,
 		bpy_pydriver_Dict);
 
@@ -1648,23 +1698,19 @@ int BPY_do_spacehandlers( ScrArea *sa, unsigned short event,
 void BPY_free_scriptlink( struct ScriptLink *slink )
 {
 	if( slink->totscript ) {
-		if( slink->flag )
+		if( slink->flag ) {
 			MEM_freeN( slink->flag );
-		if( slink->scripts )
+			slink->flag= NULL;
+		}
+		if( slink->scripts ) {
 			MEM_freeN( slink->scripts );
+			slink->scripts= NULL;
+		}
 	}
 
 	return;
 }
 
-void BPY_free_screen_spacehandlers(struct bScreen *sc)
-{
-	ScrArea *sa;
-
-	for (sa = sc->areabase.first; sa; sa = sa->next)
-		BPY_free_scriptlink(&sa->scriptlink);
-}
-
 static int CheckAllSpaceHandlers(Text *text)
 {
 	bScreen *screen;
