@@ -1,5 +1,5 @@
 /* 
- * $Id: BPY_menus.c,v 1.27 2005/04/16 05:25:42 ianwill Exp $
+ * $Id: BPY_menus.c,v 1.37 2007/02/09 18:27:20 elubie Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -35,38 +35,21 @@
  * from Blender menus.  To know more, please start with its header file.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "BPY_menus.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <Python.h>
-
 #ifndef WIN32
-#include <dirent.h>
+  #include <dirent.h>
 #else
-#include "BLI_winstuff.h"
-#include <io.h>
-#include <direct.h>
+  #include "BLI_winstuff.h"
 #endif
-
 #include "BKE_global.h"
 #include "BKE_utildefines.h"
 #include "BLI_blenlib.h"
 #include "MEM_guardedalloc.h"
-
-#include <DNA_userdef_types.h>	/* for U.pythondir */
-
-#include "BPY_extern.h"
-#include "BPY_menus.h"
+#include "DNA_userdef_types.h"	/* for U.pythondir */
 #include "api2_2x/EXPP_interface.h" /* for bpy_gethome() */
 
-#include <errno.h>
-
 #define BPYMENU_DATAFILE "Bpymenus"
 #define MAX_DIR_DEPTH 4 /* max depth for traversing scripts dirs */
 #define MAX_DIR_NUMBER 30 /* max number of dirs in scripts dirs trees */
@@ -112,8 +95,16 @@ static int bpymenu_group_atoi( char *str )
 		return PYMENU_MATERIALS;
 	else if( !strcmp( str, "UV" ) )
 		return PYMENU_UV;
+	else if( !strcmp( str, "Image" ) )
+		return PYMENU_IMAGE;
 	else if( !strcmp( str, "FaceSelect" ) )
 		return PYMENU_FACESELECT;
+	else if( !strcmp( str, "WeightPaint" ) )
+		return PYMENU_WEIGHTPAINT;
+	else if( !strcmp( str, "VertexPaint" ) )
+		return PYMENU_VERTEXPAINT;
+	else if( !strcmp( str, "UVCalculation" ) )
+		return PYMENU_UVCALCULATION;
 	/* "Misc" or an inexistent group name: use misc */
 	else
 		return PYMENU_MISC;
@@ -167,9 +158,21 @@ char *BPyMenu_group_itoa( short menugroup )
 	case PYMENU_UV:
 		return "UV";
 		break;
+	case PYMENU_IMAGE:
+		return "Image";
+		break;
 	case PYMENU_FACESELECT:
 		return "FaceSelect";
 		break;
+	case PYMENU_WEIGHTPAINT:
+		return "WeightPaint";
+		break;
+	case PYMENU_VERTEXPAINT:
+		return "VertexPaint";
+		break;
+	case PYMENU_UVCALCULATION:
+		return "UVCalculation";
+		break;
 	case PYMENU_MISC:
 		return "Misc";
 		break;
@@ -494,7 +497,7 @@ static int bpymenu_CreateFromFile( void )
 		}
 
 		if( parsing == 1 ) {	/* got menu group string */
-			group = bpymenu_group_atoi( w1 );
+			group = (short)bpymenu_group_atoi( w1 );
 			if( group < 0 && DEBUG ) {	/* invalid type */
 				fprintf(stderr,
 					"BPyMenus error parsing config file: wrong group: %s,\n\
@@ -503,7 +506,7 @@ will use 'Misc'.\n", w1 );
 		} else
 			continue;
 
-		while( 1 ) {
+		for(;;) {
 			tip = NULL;	/* optional tooltip */
 			fgets( line, 255, fp );
 			if( line[0] == '}' )
@@ -592,7 +595,7 @@ static void bpymenu_WriteDataFile( void )
 		pymenu = BPyMenuTable[i];
 		if( !pymenu )
 			continue;
-		fprintf( fp, "\n%s {\n", BPyMenu_group_itoa( i ) );
+		fprintf( fp, "\n%s {\n", BPyMenu_group_itoa( (short)i ) );
 		while( pymenu ) {
 			fprintf( fp, "'%s' %d %s %d", pymenu->name,
 				 pymenu->version, pymenu->filename,
@@ -629,7 +632,7 @@ void BPyMenu_PrintAllEntries( void )
 
 	for( i = 0; i < PYMENU_TOTAL; i++ ) {
 		pymenu = BPyMenuTable[i];
-		printf( "\n%s {\n", BPyMenu_group_itoa( i ) );
+		printf( "\n%s {\n", BPyMenu_group_itoa( (short)i ) );
 		while( pymenu ) {
 			printf( "'%s' %d %s %d", pymenu->name, pymenu->version,
 				pymenu->filename, pymenu->dir );
@@ -690,7 +693,7 @@ static int bpymenu_ParseFile(FILE *file, char *fname, int is_userdir)
 	int parser_state;
 
 	char script_name[100];
-	int script_version;
+	int script_version = 1;
 	int script_group;
 
 	BPyMenu *scriptMenu = NULL;
@@ -751,7 +754,7 @@ static int bpymenu_ParseFile(FILE *file, char *fname, int is_userdir)
 						}
 						
 						else { /* register script */
-							scriptMenu = bpymenu_AddEntry(script_group,
+							scriptMenu = bpymenu_AddEntry((short)script_group,
 								(short int)script_version, script_name, fname, is_userdir,NULL);
 							if (scriptMenu == NULL) {
 								if (DEBUG)
@@ -820,10 +823,10 @@ static int bpymenu_ParseDir(char *dirname, char *parentdir, int is_userdir )
 	struct dirent *de;
 	struct stat status;
 	char *file_extension;
-	char path[FILE_MAXDIR];
-	char subdir[FILE_MAXDIR];
+	char path[FILE_MAX];
+	char subdir[FILE_MAX];
 	char *s = NULL;
-
+	
 	dir = opendir(dirname);
 
 	if (dir != NULL) {
@@ -833,9 +836,9 @@ static int bpymenu_ParseDir(char *dirname, char *parentdir, int is_userdir )
 			if ((de->d_name[0] == '.') || !strncmp(de->d_name, "bpy", 3)) {
 				continue;
 			}
-
+			
 			BLI_make_file_string("/", path, dirname, de->d_name);
-
+			
 			if (stat(path, &status) != 0) {
 				if (DEBUG)
 					fprintf(stderr, "stat %s failed: %s\n", path, strerror(errno));
@@ -851,7 +854,15 @@ static int bpymenu_ParseDir(char *dirname, char *parentdir, int is_userdir )
 					if (file) {
 						s = de->d_name;
 						if (parentdir) {
-							BLI_make_file_string(NULL, subdir, parentdir, de->d_name);
+							/* Join parentdir and de->d_name */
+							short a = strlen(parentdir);
+							strcpy(subdir, parentdir);
+							strcpy(subdir + a+1, de->d_name);
+#ifdef WIN32
+							subdir[a] = '\\';
+#else
+							subdir[a] = '/';
+#endif
 							s = subdir;
 						}
 						bpymenu_ParseFile(file, s, is_userdir);
@@ -891,6 +902,7 @@ static int bpymenu_ParseDir(char *dirname, char *parentdir, int is_userdir )
 					closedir(dir);
 					return -1;
 				}
+				Dir_Depth--;
 			}
 
 		}
