@@ -1,5 +1,5 @@
 /* 
- * $Id: BPY_menus.h,v 1.18 2006/05/07 13:09:29 campbellbarton Exp $
+ * $Id: BPY_menus.h,v 1.20 2006/12/12 04:18:44 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -90,6 +90,7 @@ typedef enum {
 	PYMENU_SYSTEM,
 	PYMENU_THEMES,
 	PYMENU_UV,/* UV editing tools, to go in UV/Image editor space, 'UV' menu */
+	PYMENU_IMAGE,/* Image editing tools, to go in UV/Image editor space, 'Image' menu */
 	PYMENU_WIZARDS,/* complex 'app' scripts */
 
 	/* entries put after Wizards don't appear at the Scripts win->Scripts menu;
@@ -98,6 +99,7 @@ typedef enum {
 	PYMENU_FACESELECT,
 	PYMENU_WEIGHTPAINT,
 	PYMENU_VERTEXPAINT,
+	PYMENU_UVCALCULATION,
 	PYMENU_HELP,/*Main Help menu items - prob best to leave for 'official' ones*/
 	PYMENU_HELPSYSTEM,/* Resources, troubleshooting, system tools */
 	PYMENU_HELPWEBSITES,/* Help -> Websites submenu */
