@@ -1,5 +1,5 @@
 /* 
- * $Id: Armature.h,v 1.14 2006/01/16 16:34:08 ascotan Exp $
+ * $Id: Armature.h,v 1.16 2007/01/25 15:19:28 ascotan Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -56,10 +56,16 @@ typedef struct {
 	PyObject_HEAD 
 	struct bArmature * armature;
 	BPy_BonesDict *Bones;          //BPy_BonesDict
+	PyObject *weaklist;
 } BPy_Armature;
 
 //-------------------VISIBLE PROTOTYPES-------------------------
 PyObject *PyArmature_FromArmature(struct bArmature *armature);
 struct bArmature *PyArmature_AsArmature(BPy_Armature *py_armature);
+PyObject * Armature_RebuildEditbones(PyObject *pyarmature);
+PyObject *Armature_RebuildBones(PyObject *pyarmature);
+
+int Armature_CheckPyObject( PyObject * pyobj );
+struct bArmature  *Armature_FromPyObject( PyObject * py_obj );
 
 #endif				
