@@ -1,5 +1,5 @@
 /* 
- * $Id: BGL.c,v 1.20 2006/04/16 15:28:50 stiv Exp $
+ * $Id: BGL.c,v 1.21 2006/09/30 14:14:48 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -246,7 +246,7 @@ static PyObject *Buffer_item(PyObject *self, int i)
 			case GL_BYTE: return Py_BuildValue("b", buf->buf.asbyte[i]);
 			case GL_SHORT: return Py_BuildValue("h", buf->buf.asshort[i]);
 			case GL_INT: return Py_BuildValue("i", buf->buf.asint[i]);
-			case GL_FLOAT: return Py_BuildValue("f", buf->buf.asfloat[i]);
+			case GL_FLOAT: return PyFloat_FromDouble(buf->buf.asfloat[i]);
 			case GL_DOUBLE: return Py_BuildValue("d", buf->buf.asdouble[i]);
 		}
 	} else {
