@@ -1,5 +1,5 @@
 /*
- * $Id: BezTriple.c,v 1.23 2006/05/19 21:45:27 ascotan Exp $
+ * $Id: BezTriple.c,v 1.25 2006/08/13 10:13:19 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -326,6 +326,23 @@ static int BezTriple_setWeight( BPy_BezTriple * self, PyObject *value )
 	return 0;
 }
 
+static PyObject *BezTriple_getRadius( BPy_BezTriple * self )
+{
+	return PyFloat_FromDouble( self->beztriple->radius );
+}
+
+static int BezTriple_setRadius( BPy_BezTriple * self, PyObject *value )
+{
+	PyObject *num = PyNumber_Float( value );
+
+	if( !num )
+		return EXPP_ReturnIntError( PyExc_TypeError, "expected a float" );
+
+	self->beztriple->radius = (float)PyFloat_AsDouble( num );
+	Py_DECREF( num );
+	return 0;
+}
+
 static PyObject *BezTriple_getHide( BPy_BezTriple * self )
 {
 	return PyInt_FromLong( self->beztriple->hide == IPO_BEZ );
@@ -454,6 +471,10 @@ static PyGetSetDef BPy_BezTriple_getseters[] = {
 	 (getter)BezTriple_getWeight, (setter)BezTriple_setWeight,
 	 "point weight",
 	 NULL},
+	{"radius",
+	 (getter)BezTriple_getRadius, (setter)BezTriple_setRadius,
+	 "point radius",
+	 NULL},
 	{NULL,NULL,NULL,NULL,NULL}  /* Sentinel */
 };
 
@@ -466,10 +487,10 @@ static PyObject *BezTriple_repr( BPy_BezTriple * self )
 {
 	char str[512];
 	sprintf( str,
-		"[BezTriple [%.2f, %.2f] [%.2f, %.2f] [%.2f, %.2f]\n",
-		 self->beztriple->vec[0][0], self->beztriple->vec[0][1],
-		 self->beztriple->vec[1][0], self->beztriple->vec[1][1],
-		 self->beztriple->vec[2][0], self->beztriple->vec[2][1]);
+		"[BezTriple [%.6f, %.6f, %.6f] [%.6f, %.6f, %.6f] [%.6f, %.6f, %.6f]\n",
+		 self->beztriple->vec[0][0], self->beztriple->vec[0][1], self->beztriple->vec[0][2],
+		 self->beztriple->vec[1][0], self->beztriple->vec[1][1], self->beztriple->vec[1][2],
+		 self->beztriple->vec[2][0], self->beztriple->vec[2][1], self->beztriple->vec[2][2]);
 	return PyString_FromString( str );
 }
 
