@@ -1,4 +1,5 @@
 /* 
+ * $Id: BezTriple.h,v 1.12 2005/11/28 05:03:25 khughes Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -24,7 +25,7 @@
  *
  * This is a new part of Blender.
  *
- * Contributor(s): Jacques Guignot
+ * Contributor(s): Jacques Guignot, Stephen Swaney
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
 */
@@ -33,17 +34,32 @@
 #define EXPP_BEZTRIPLE_H
 
 #include <Python.h>
-#include <DNA_curve_types.h>
+#include "DNA_curve_types.h"
 
-/*****************************************************************************/
-/* Python C_BezTriple structure definition:                                     */
-/*****************************************************************************/
+extern PyTypeObject BezTriple_Type;
 
-typedef struct
-{
-  PyObject_HEAD BezTriple *beztriple;
-}
-C_BezTriple;
+/* type check macro */
+#define BPy_BezTriple_Check(o) \
+ ( (o)->ob_type == &BezTriple_Type)
 
+/****************************************************************************
+ Python BPy_BezTriple structure definition:              
+****************************************************************************/
 
-#endif /* EXPP_BEZTRIPLE_H */
+typedef struct {
+	PyObject_HEAD		/* required python macro */
+	BezTriple * beztriple;
+	int own_memory;   /* true == we own this memory and must delete. */
+} BPy_BezTriple;
+
+/*
+ *    prototypes
+ */
+
+PyObject *BezTriple_CreatePyObject( BezTriple * bzt );
+int BezTriple_CheckPyObject( PyObject * pyobj );
+BezTriple *BezTriple_FromPyObject( PyObject * pyobj );
+PyObject *newBezTriple( float *args );
+PyObject *BezTriple_Init( void );
+
+#endif				/* EXPP_BEZTRIPLE_H */
