@@ -1,5 +1,5 @@
 /* 
- * $Id: Blender.c,v 1.61 2005/06/13 19:15:01 stiv Exp $
+ * $Id: Blender.c,v 1.94 2007/01/15 07:54:08 joeedh Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -29,51 +29,74 @@
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
 */
+struct ID; /*keep me up here */
 
-#include <Python.h>
-#include <stdio.h>
-
-#ifndef O_BINARY
-# define O_BINARY 0
-#endif
+#include "Blender.h" /*This must come first */
 
 /* for open, close in Blender_Load */
 #include <fcntl.h>
-#ifndef WIN32
-#include <unistd.h>
-#else
-#include <io.h>
-#endif
-
-#include <BDR_editobject.h>	/* exit_editmode() */
-#include <BIF_usiblender.h>
-#include <BLI_blenlib.h>
-#include <BLO_writefile.h>
-#include <BKE_exotic.h>
-#include <BKE_global.h>
-#include <BKE_packedFile.h>
-#include <BKE_object.h>
-#include <BKE_text.h>
-#include <BPI_script.h>
-#include <BSE_headerbuttons.h>
-#include <DNA_ID.h>
-#include <DNA_object_types.h>
-#include <DNA_scene_types.h>
-#include <DNA_screen_types.h>	/* for SPACE_VIEW3D */
-#include <DNA_space_types.h>	/* for SPACE_VIEW3D */
-#include <DNA_scriptlink_types.h>
-#include <DNA_userdef_types.h>
-#include <BKE_ipo.h>
-#include <blendef.h>
-
+#include "BDR_editobject.h"	/* exit_editmode() */
+#include "BIF_usiblender.h"
+#include "BLI_blenlib.h"
+#include "BLO_writefile.h"
+#include "BKE_blender.h"
+#include "BKE_exotic.h"
+#include "BKE_global.h"
+#include "BKE_packedFile.h"
+#include "BKE_utildefines.h"
+#include "BKE_object.h"
+#include "BKE_scene.h"
+#include "BKE_text.h"
+#include "BKE_ipo.h"
+#include "BKE_library.h"
+#include "BKE_main.h"
+#include "BPI_script.h"
+#include "BSE_headerbuttons.h"
+#include "DNA_screen_types.h"	/* for SPACE_VIEW3D */
+#include "DNA_userdef_types.h"
+#include "DNA_packedFile_types.h"
 #include "EXPP_interface.h" /* for bpy_gethome() */
 #include "gen_utils.h"
 #include "modules.h"
 #include "constant.h"
 #include "../BPY_extern.h" /* BPY_txt_do_python_Text */
 #include "../BPY_menus.h"	/* to update menus */
+#include "Armature.h"
+#include "BezTriple.h"
+#include "Camera.h"
+#include "Constraint.h"
+#include "Curve.h"
+#include "CurNurb.h"
+#include "Draw.h"
+#include "Effect.h"
+#include "Ipo.h"
+#include "Ipocurve.h"
+#include "Key.h"
+#include "Lamp.h"
+#include "Lattice.h"
+#include "Mathutils.h"
+#include "Geometry.h"
+#include "Mesh.h"
+#include "Metaball.h"
+#include "Modifier.h"
+#include "NMesh.h"
+#include "Object.h"
+#include "Group.h"
+#include "Registry.h"
+#include "Scene.h"
+#include "Sound.h"
+#include "SurfNurb.h"
+#include "Sys.h"
+#include "Text.h"
+#include "Texture.h"
+#include "Window.h"
+#include "World.h"
+#include "Types.h"
+
+//for the removefakeuser hack
+#include "NLA.h" /*This must come first*/
+#include "BKE_action.h"
 
-extern PyObject *bpy_registryDict; /* defined in ../BPY_interface.c */
 
 /**********************************************************/
 /* Python API function prototypes for the Blender module.	*/
@@ -85,8 +108,12 @@ static PyObject *Blender_Quit( PyObject * self );
 static PyObject *Blender_Load( PyObject * self, PyObject * args );
 static PyObject *Blender_Save( PyObject * self, PyObject * args );
 static PyObject *Blender_Run( PyObject * self, PyObject * args );
+static PyObject *Blender_RemoveFakeuser(PyObject *self, PyObject *args);
 static PyObject *Blender_ShowHelp( PyObject * self, PyObject * args );
 static PyObject *Blender_UpdateMenus( PyObject * self);
+static PyObject *Blender_PackAll( PyObject * self);
+static PyObject *Blender_UnpackAll( PyObject * self, PyObject * args);
+static PyObject *Blender_CountPackedFiles( PyObject * self );
 
 extern PyObject *Text3d_Init( void ); /* missing in some include */
 
@@ -147,6 +174,10 @@ static char Blender_Run_doc[] =
 	"(script) - Run the given Python script.\n\
 (script) - the path to a file or the name of an available Blender Text.";
 
+static char Blender_RemoveFakeuser_doc[] =
+	"(datablock) - remove the fake user from a datablock. useful for deleting actions.\n\
+(datablock) - the datablock that has a fakeuser. currently only action object accepted.";
+
 static char Blender_ShowHelp_doc[] =
 "(script) - Show help for the given Python script.\n\
   This will try to open the 'Scripts Help Browser' script, so to have\n\
@@ -160,6 +191,17 @@ static char Blender_UpdateMenus_doc[] =
 	"() - Update the menus where scripts are registered.  Only needed for\n\
 scripts that save other new scripts in the default or user defined folders.";
 
+static char Blender_PackAll_doc[] =
+"() - Pack all files.\n\
+All files will packed into the blend file.";
+static char Blender_UnpackAll_doc[] =
+"(mode) - Unpack files.\n\
+All files will be unpacked using specified mode.\n\n\
+(mode) - the unpack mode.";
+
+static char Blender_CountPackedFiles_doc[] =
+"() - Returns the number of packed files.";
+
 /*****************************************************************************/
 /* Python method structure definition.		 */
 /*****************************************************************************/
@@ -171,7 +213,11 @@ static struct PyMethodDef Blender_methods[] = {
 	{"Load", Blender_Load, METH_VARARGS, Blender_Load_doc},
 	{"Save", Blender_Save, METH_VARARGS, Blender_Save_doc},
 	{"Run", Blender_Run, METH_VARARGS, Blender_Run_doc},
+	{"RemoveFakeuser", Blender_RemoveFakeuser, METH_VARARGS, Blender_RemoveFakeuser_doc},
 	{"ShowHelp", Blender_ShowHelp, METH_VARARGS, Blender_ShowHelp_doc},
+	{"CountPackedFiles", ( PyCFunction ) Blender_CountPackedFiles, METH_NOARGS, Blender_CountPackedFiles_doc},
+	{"PackAll", ( PyCFunction ) Blender_PackAll, METH_NOARGS, Blender_PackAll_doc},
+	{"UnpackAll", Blender_UnpackAll, METH_VARARGS, Blender_UnpackAll_doc},
 	{"UpdateMenus", ( PyCFunction ) Blender_UpdateMenus, METH_NOARGS,
 	 Blender_UpdateMenus_doc},
 	{NULL, NULL, 0, NULL}
@@ -188,29 +234,63 @@ PyObject *g_blenderdict;
 /*****************************************************************************/
 static PyObject *Blender_Set( PyObject * self, PyObject * args )
 {
-	char *name;
+	char *name, *dir = NULL;
 	PyObject *arg;
-	int framenum;
 
-	if( !PyArg_ParseTuple( args, "sO", &name, &arg ) ) {
-		/* TODO: Do we need to generate a nice error message here? */
-		return ( NULL );
-	}
+	if( !PyArg_ParseTuple( args, "sO", &name, &arg ) )
+		return EXPP_ReturnPyObjError( PyExc_ValueError, "expected 2 args, where the first is always a string" );
 
 	if( StringEqual( name, "curframe" ) ) {
-		if( !PyArg_Parse( arg, "i", &framenum ) ) {
-			/* TODO: Do we need to generate a nice error message here? */
-			return ( NULL );
-		}
+		if( !PyInt_Check( arg ) )
+			return EXPP_ReturnPyObjError( PyExc_ValueError,
+					"expected an integer" );
 
-		G.scene->r.cfra = framenum;
+		G.scene->r.cfra = (short)PyInt_AsLong( arg ) ;
 
-		update_for_newframe(  );
-	} else {
+		/*	update all objects, so python scripts can export all objects
+		 in a scene without worrying about the view layers */
+		scene_update_for_newframe(G.scene, (1<<20) - 1);
+		
+	} else if (StringEqual( name , "uscriptsdir" ) ) {
+		if ( !PyArg_Parse( arg , "s" , &dir ))
+			return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a string" );
+		BLI_strncpy(U.pythondir, dir, FILE_MAXDIR);
+	} else if (StringEqual( name , "yfexportdir" ) ) {
+		if ( !PyArg_Parse( arg , "s" , &dir ))
+			return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a string" );
+		BLI_strncpy(U.yfexportdir, dir, FILE_MAXDIR);		
+	} else if (StringEqual( name , "fontsdir" ) ) {
+		if ( !PyArg_Parse( arg , "s" , &dir ))
+			return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a string" );
+		BLI_strncpy(U.fontdir, dir, FILE_MAXDIR);
+	} else if (StringEqual( name , "texturesdir" ) ) {
+		if ( !PyArg_Parse( arg , "s" , &dir ))
+			return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a string" );
+		BLI_strncpy(U.textudir, dir, FILE_MAXDIR);
+	} else if (StringEqual( name , "texpluginsdir" ) ) {
+		if ( !PyArg_Parse( arg , "s" , &dir ))
+			return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a string" );
+		BLI_strncpy(U.plugtexdir, dir, FILE_MAXDIR);
+	} else if (StringEqual( name , "seqpluginsdir" ) ) {
+		if ( !PyArg_Parse( arg , "s" , &dir ))
+			return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a string" );
+		BLI_strncpy(U.plugseqdir, dir, FILE_MAXDIR);
+	} else if (StringEqual( name , "renderdir" ) ) {
+		if ( !PyArg_Parse( arg , "s" , &dir ))
+			return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a string" );
+		BLI_strncpy(U.renderdir, dir, FILE_MAXDIR);
+	} else if (StringEqual( name , "soundsdir" ) ) {
+		if ( !PyArg_Parse( arg , "s" , &dir ))
+			return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a string" );
+		BLI_strncpy(U.sounddir, dir, FILE_MAXDIR);
+	} else if (StringEqual( name , "tempdir" ) ) {
+		if ( !PyArg_Parse( arg , "s" , &dir ))
+			return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a string" );
+		BLI_strncpy(U.tempdir, dir, FILE_MAXDIR);
+	} else
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
-						"bad request identifier" ) );
-	}
-	return ( EXPP_incr_ret( Py_None ) );
+						"value given is not a blender setting" ) );
+	Py_RETURN_NONE;
 }
 
 /*****************************************************************************/
@@ -219,7 +299,7 @@ static PyObject *Blender_Set( PyObject * self, PyObject * args )
 /*****************************************************************************/
 static PyObject *Blender_Get( PyObject * self, PyObject * args )
 {
-	PyObject *ret = NULL, *dict = NULL;
+	PyObject *ret = NULL;
 	char *str = NULL;
 
 	if( !PyArg_ParseTuple( args, "s", &str ) )
@@ -230,6 +310,8 @@ static PyObject *Blender_Get( PyObject * self, PyObject * args )
 		ret = PyInt_FromLong( G.scene->r.cfra );
 	else if( StringEqual( str, "curtime" ) )
 		ret = PyFloat_FromDouble( frame_to_float( G.scene->r.cfra ) );
+	else if( StringEqual( str, "rt" ) )
+		ret = PyInt_FromLong( frame_to_float( G.rt ) );
 	else if( StringEqual( str, "staframe" ) )
 		ret = PyInt_FromLong( G.scene->r.sfra );
 	else if( StringEqual( str, "endframe" ) )
@@ -395,19 +477,35 @@ static PyObject *Blender_Get( PyObject * self, PyObject * args )
 				ret = PyString_FromString( tempdir );
 		}
 		if (!ret) ret = EXPP_incr_ret(Py_None);
-	}	
+	}
+	/* icondir */
+	else if( StringEqual( str, "icondir" ) ) {
+		
+		char icondirstr[FILE_MAXDIR];
+
+		#ifdef WIN32
+			BLI_make_file_string("/", icondirstr, BLI_gethome(), "icons");
+		#else
+			BLI_make_file_string("/", icondirstr, BLI_gethome(), ".blender/icons");
+		#endif
+
+		if( BLI_exists( icondirstr ) )
+			ret = PyString_FromString( icondirstr );
+		
+		if (!ret) ret = EXPP_incr_ret(Py_None);
+	}
 	/* According to the old file (opy_blender.c), the following if
 	   statement is a quick hack and needs some clean up. */
 	else if( StringEqual( str, "vrmloptions" ) ) {
 		ret = PyDict_New(  );
 
-		PyDict_SetItemString( dict, "twoside",
+		EXPP_dict_set_item_str( ret, "twoside",
 			PyInt_FromLong( U.vrmlflag & USER_VRML_TWOSIDED ) );
 
-		PyDict_SetItemString( dict, "layers",
+		EXPP_dict_set_item_str( ret, "layers",
 			PyInt_FromLong( U.vrmlflag & USER_VRML_LAYERS ) );
 
-		PyDict_SetItemString( dict, "autoscale",
+		EXPP_dict_set_item_str( ret, "autoscale",
 			PyInt_FromLong( U.vrmlflag & USER_VRML_AUTOSCALE ) );
 
 	} /* End 'quick hack' part. */
@@ -448,8 +546,7 @@ static PyObject *Blender_Quit( PyObject * self )
 
 	exit_usiblender(  );	/* renames last autosave to quit.blend */
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 /**
@@ -467,7 +564,7 @@ static PyObject *Blender_Load( PyObject * self, PyObject * args )
 
 	if( !PyArg_ParseTuple( args, "|si", &fname, &keep_oldfname ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
-					      "expected filename and optional int or nothing as arguments" );
+			"expected filename and optional int or nothing as arguments" );
 
 	if( fname ) {
 		if( strlen( fname ) > FILE_MAXDIR )	/* G.main->name's max length */
@@ -505,6 +602,7 @@ static PyObject *Blender_Load( PyObject * self, PyObject * args )
 		is_blend_file = 1;	/* no arg given means default: .B.blend */
 
 	if( is_blend_file ) {
+
 		int during_slink = during_scriptlink(  );
 
 		/* when loading a .blend file from a scriptlink, the scriptlink pointer
@@ -531,27 +629,37 @@ static PyObject *Blender_Load( PyObject * self, PyObject * args )
 	}
 
 	if( G.obedit )
-		exit_editmode( 1 );
+		exit_editmode(EM_FREEDATA);
 
-	/* for safety, any filename with .B.blend is considered the default one.
-	 * It doesn't seem necessary to compare file attributes (like st_ino and
-	 * st_dev, according to the glibc info pages) to find out if the given
-	 * filename, that may have been given with a twisted misgiving path, is the
-	 * default one for sure.  Taking any .B.blend file as the default is good
-	 * enough here.  Note: the default file requires extra clean-up done by
-	 * BIF_read_homefile: freeing the user theme data. */
-	if( !fname || ( strstr( fname, ".B.blend" ) && is_blend_file ) )
-		BIF_read_homefile(  );
-	else
-		BIF_read_file( fname );
+	if (G.background) { /* background mode */
+		if (is_blend_file)
+			BKE_read_file(fname, NULL);
+		else {
+			return EXPP_ReturnPyObjError(PyExc_AttributeError,
+				"only .blend files can be loaded from command line,\n\
+	other file types require interactive mode.");
+		}
+	}
+	else { /* interactive mode */
+		/* for safety, any filename with .B.blend is considered the default one.
+		 * It doesn't seem necessary to compare file attributes (like st_ino and
+		 * st_dev, according to the glibc info pages) to find out if the given
+		 * filename, that may have been given with a twisted misgiving path, is the
+		 * default one for sure.  Taking any .B.blend file as the default is good
+		 * enough here.  Note: the default file requires extra clean-up done by
+		 * BIF_read_homefile: freeing the user theme data. */
+		if( !fname || ( strstr( fname, ".B.blend" ) && is_blend_file ) )
+			BIF_read_homefile(  );
+		else
+			BIF_read_file( fname );
 
-	if( fname && keep_oldfname ) {
-		/*BLI_strncpy(G.main->name, name, FILE_MAXDIR); */
-		BLI_strncpy( G.sce, name, FILE_MAXDIR );
+		if( fname && keep_oldfname ) {
+			/*BLI_strncpy(G.main->name, name, FILE_MAXDIR); */
+			BLI_strncpy( G.sce, name, FILE_MAXDIR );
+		}
 	}
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 static PyObject *Blender_Save( PyObject * self, PyObject * args )
@@ -589,7 +697,7 @@ static PyObject *Blender_Save( PyObject * self, PyObject * args )
 
 	disable_where_script( 1 );	/* to avoid error popups in the write_* functions */
 
-	if( BLI_testextensie( fname, ".blend" ) ) {
+	if( BLI_testextensie( fname, ".blend" ) ) {		
 		if( G.fileflags & G_AUTOPACK )
 			packAll(  );
 		if( !BLO_write_file( fname, G.fileflags, &error ) ) {
@@ -613,8 +721,7 @@ static PyObject *Blender_Save( PyObject * self, PyObject * args )
 
 	disable_where_script( 0 );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 static PyObject *Blender_ShowHelp(PyObject *self, PyObject *args)
@@ -651,14 +758,17 @@ static PyObject *Blender_ShowHelp(PyObject *self, PyObject *args)
 		return EXPP_ReturnPyObjError(PyExc_MemoryError,
 			"can't create py dictionary!");
 
+	/* note: don't use EXPP_dict_set_item_str for 'script', which is an
+	 * argument to the function we're in and so shouldn't be decref'ed: */
 	PyDict_SetItemString(rkeyd, "script", script);
-	PyDict_SetItemString(bpy_registryDict, "__help_browser", rkeyd);
+
+	EXPP_dict_set_item_str(bpy_registryDict, "__help_browser", rkeyd);
 
 	arglist = Py_BuildValue("(s)", hspath);
 	Blender_Run(self, arglist);
 	Py_DECREF(arglist);
 
-	return EXPP_incr_ret(Py_None);
+	Py_RETURN_NONE;
 }
 
 static PyObject *Blender_Run(PyObject *self, PyObject *args)
@@ -721,7 +831,7 @@ static PyObject *Blender_Run(PyObject *self, PyObject *args)
 
 	if (!is_blender_text) free_libblock(&G.main->text, text);
 
-	return EXPP_incr_ret(Py_None);
+	Py_RETURN_NONE;
 }
 
 static PyObject * Blender_UpdateMenus( PyObject * self )
@@ -733,31 +843,124 @@ static PyObject * Blender_UpdateMenus( PyObject * self )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 			"invalid scripts dir");
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
+}
+
+static PyObject *Blender_RemoveFakeuser(PyObject *self, PyObject *args)
+{
+	ID *id;
+	BPy_Action *py_thing; //lousy coder antont did not know how to accept any bpy thing with ID..
+
+	if( !PyArg_ParseTuple( args, "O!", &Action_Type, &py_thing ) )
+		return EXPP_ReturnPyObjError( PyExc_AttributeError,
+				       "expected python action type" );
+	
+	id= (ID *)py_thing->action;
+	
+	if(id) {
+		if( id->flag & LIB_FAKEUSER) {
+					id->flag -= LIB_FAKEUSER;
+					id->us--;
+				}
+		else
+			return EXPP_ReturnPyObjError( PyExc_AttributeError,
+						      "given datablock has no fakeusers");
+	} else
+		return EXPP_ReturnPyObjError( PyExc_AttributeError,
+					      "given object does not have a Blender ID");
+
+	Py_RETURN_NONE;
+}
+
+/*****************************************************************************/
+/* Function:		Blender_PackAll		 */
+/* Python equivalent:	Blender.PackAll			 */
+/*****************************************************************************/
+static PyObject *Blender_PackAll( PyObject * self)
+{
+	packAll();
+	Py_RETURN_NONE;
+}
+
+/*****************************************************************************/
+/* Function:		Blender_UnpackAll		 */
+/* Python equivalent:	Blender.UnpackAll			 */
+/*****************************************************************************/
+static PyObject *Blender_UnpackAll( PyObject * self, PyObject *args)
+{
+	int mode;
+	PyArg_ParseTuple( args, "i", &mode );
+	unpackAll(mode);
+	Py_RETURN_NONE;
+}
+ 
+/*****************************************************************************/
+/* Function:		Blender_CountPackedFiles		 */
+/* Python equivalent:	Blender.CountPackedFiles			 */
+/*****************************************************************************/
+static PyObject *Blender_CountPackedFiles( PyObject * self )
+{
+	int nfiles = countPackedFiles();
+	return PyInt_FromLong( nfiles );
+}
+static PyObject *Blender_UnpackModesDict( void )
+{
+	PyObject *UnpackModes = PyConstant_New(  );
+	if( UnpackModes ) {
+		BPy_constant *d = ( BPy_constant * ) UnpackModes;
+		PyConstant_Insert( d, "EQUAL", PyInt_FromLong((long)PF_EQUAL) );
+		PyConstant_Insert( d, "DIFFERS",PyInt_FromLong((long)PF_DIFFERS) );
+		PyConstant_Insert( d, "NOFILE", PyInt_FromLong((long)PF_NOFILE) );
+		PyConstant_Insert( d, "WRITE_ORIGINAL", PyInt_FromLong((long)PF_WRITE_ORIGINAL) );
+		PyConstant_Insert( d, "WRITE_LOCAL", PyInt_FromLong((long)PF_WRITE_LOCAL) );
+		PyConstant_Insert( d, "USE_LOCAL", PyInt_FromLong((long)PF_USE_LOCAL) );
+		PyConstant_Insert( d, "USE_ORIGINAL", PyInt_FromLong((long)PF_USE_ORIGINAL) );
+		PyConstant_Insert( d, "KEEP", PyInt_FromLong((long)PF_KEEP) );
+		PyConstant_Insert( d, "NOOP", PyInt_FromLong((long)PF_NOOP) );
+		PyConstant_Insert( d, "ASK", PyInt_FromLong((long)PF_EQUAL) );
+	}
+	return UnpackModes;
 }
 
 /*****************************************************************************/
 /* Function:		initBlender		 */
 /*****************************************************************************/
-void M_Blender_Init( void )
+extern PyTypeObject IDArray_Type;
+extern PyTypeObject IDGroup_Type;
+extern PyTypeObject IDGroup_Iter_Type;
+
+void M_Blender_Init(void)
 {
 	PyObject *module;
-	PyObject *dict, *smode, *SpaceHandlers;
+	PyObject *dict, *smode, *SpaceHandlers, *UnpackModes;
+	
+	/* G.scene should only aver be NULL if blender is executed in 
+	background mode, not loading a blend file and executing a python script eg.
+	blender -P somescript.py -b
+	The if below solves the segfaults that happen when python runs and
+	G.scene is NULL */
+	if(G.background && G.main->scene.first==0) {
+		Scene *sce= add_scene("1");
+		//set_scene(sce); /* causes a crash */
+		G.scene= sce;
+	}
+	
+	module = Py_InitModule3("Blender", Blender_methods,
+		"The main Blender module");
 
-	g_blenderdict = NULL;
+	types_InitAll();	/* set all our pytypes to &PyType_Type */
 
-	module = Py_InitModule3( "Blender", Blender_methods,
-		"The main Blender module" );
-
-	types_InitAll(  );	/* set all our pytypes to &PyType_Type */
-
-	SpaceHandlers = M_constant_New();
+	// constants for packed files
+	UnpackModes = Blender_UnpackModesDict(  );
+	if( UnpackModes )
+		PyModule_AddObject( module, "UnpackModes", UnpackModes );
+ 
+	SpaceHandlers = PyConstant_New();
 	if (SpaceHandlers) {
 		BPy_constant *d = (BPy_constant *)SpaceHandlers;
 
-		constant_insert(d,"VIEW3D_EVENT",PyInt_FromLong(SPACEHANDLER_VIEW3D_EVENT));
-		constant_insert(d,"VIEW3D_DRAW", PyInt_FromLong(SPACEHANDLER_VIEW3D_DRAW));
+		PyConstant_Insert(d,"VIEW3D_EVENT",PyInt_FromLong(SPACEHANDLER_VIEW3D_EVENT));
+		PyConstant_Insert(d,"VIEW3D_DRAW", PyInt_FromLong(SPACEHANDLER_VIEW3D_DRAW));
 
 		PyModule_AddObject(module, "SpaceHandlers", SpaceHandlers);
 	}
@@ -767,45 +970,53 @@ void M_Blender_Init( void )
 	else
 		smode = PyString_FromString("interactive");
 
-	dict = PyModule_GetDict( module );
+	dict = PyModule_GetDict(module);
 	g_blenderdict = dict;
 
-	PyDict_SetItemString( dict, "bylink", EXPP_incr_ret_False() );
-	PyDict_SetItemString( dict, "link", EXPP_incr_ret ( Py_None ) );
-	PyDict_SetItemString( dict, "event", PyString_FromString( "" ) );
-	PyDict_SetItemString( dict, "mode", smode );
-
-	PyDict_SetItemString( dict, "Types", Types_Init(  ) );
-	PyDict_SetItemString( dict, "sys", sys_Init(  ) );
-	PyDict_SetItemString( dict, "Registry", Registry_Init(  ) );
-	PyDict_SetItemString( dict, "Scene", Scene_Init(  ) );
-	PyDict_SetItemString( dict, "Object", Object_Init(  ) );
-	PyDict_SetItemString( dict, "Material", Material_Init(  ) );
-	PyDict_SetItemString( dict, "Camera", Camera_Init(  ) );
-	PyDict_SetItemString( dict, "Lamp", Lamp_Init(  ) );
-	PyDict_SetItemString( dict, "Lattice", Lattice_Init(  ) );
-	PyDict_SetItemString( dict, "Curve", Curve_Init(  ) );
-	PyDict_SetItemString( dict, "Armature", Armature_Init(  ) );
-	PyDict_SetItemString( dict, "Ipo", Ipo_Init(  ) );
-	PyDict_SetItemString( dict, "IpoCurve", IpoCurve_Init(  ) );
-	PyDict_SetItemString( dict, "Metaball", Metaball_Init(  ) );
-	PyDict_SetItemString( dict, "Image", Image_Init(  ) );
-	PyDict_SetItemString( dict, "Window", Window_Init(  ) );
-	PyDict_SetItemString( dict, "Draw", Draw_Init(  ) );
-	PyDict_SetItemString( dict, "BGL", BGL_Init(  ) );
-	PyDict_SetItemString( dict, "Effect", Effect_Init(  ) );
-	PyDict_SetItemString( dict, "Text", Text_Init(  ) );
-	PyDict_SetItemString( dict, "Text3d", Text3d_Init(  ) );
-	PyDict_SetItemString( dict, "World", World_Init(  ) );
-	PyDict_SetItemString( dict, "Texture", Texture_Init(  ) );
-	PyDict_SetItemString( dict, "NMesh", NMesh_Init(  ) );
-	PyDict_SetItemString( dict, "Noise", Noise_Init(  ) );
-	PyDict_SetItemString( dict, "Mathutils", Mathutils_Init(  ) );
-	PyDict_SetItemString( dict, "Library", Library_Init(  ) );
-	PyDict_SetItemString( dict, "Sound", Sound_Init(  ) );
-	PyDict_SetItemString( dict, "CurNurb", CurNurb_Init(  ) );
-	PyDict_SetItemString( dict, "BezTriple", BezTriple_Init(  ) );
-
-	PyModule_AddIntConstant( module, "TRUE", 1 );
+	PyModule_AddIntConstant(module, "TRUE", 1);
 	PyModule_AddIntConstant( module, "FALSE", 0 );
+
+	EXPP_dict_set_item_str(dict, "bylink", EXPP_incr_ret_False());
+	PyDict_SetItemString(dict, "link", Py_None);
+	EXPP_dict_set_item_str(dict, "event", PyString_FromString(""));
+	EXPP_dict_set_item_str(dict, "mode", smode);
+
+	PyDict_SetItemString(dict, "Armature", Armature_Init());
+	PyDict_SetItemString(dict, "BezTriple", BezTriple_Init());
+	PyDict_SetItemString(dict, "BGL", BGL_Init());
+	PyDict_SetItemString(dict, "CurNurb", CurNurb_Init());
+	PyDict_SetItemString(dict, "Constraint", Constraint_Init());
+	PyDict_SetItemString(dict, "Curve", Curve_Init());
+	PyDict_SetItemString(dict, "Camera", Camera_Init());
+	PyDict_SetItemString(dict, "Draw", Draw_Init());
+	PyDict_SetItemString(dict, "Effect", Effect_Init());
+	PyDict_SetItemString(dict, "Ipo", Ipo_Init());
+	PyDict_SetItemString(dict, "IpoCurve", IpoCurve_Init());
+	PyDict_SetItemString(dict, "Image", Image_Init());
+	PyDict_SetItemString(dict, "Key", Key_Init());
+	PyDict_SetItemString(dict, "Lamp", Lamp_Init());
+	PyDict_SetItemString(dict, "Lattice", Lattice_Init());
+	PyDict_SetItemString(dict, "Library", Library_Init());
+	PyDict_SetItemString(dict, "Material", Material_Init());
+	PyDict_SetItemString(dict, "Mesh", Mesh_Init()); 
+	PyDict_SetItemString(dict, "Metaball", Metaball_Init());
+	PyDict_SetItemString(dict, "Mathutils", Mathutils_Init());
+	PyDict_SetItemString(dict, "Geometry", Geometry_Init());
+	PyDict_SetItemString(dict, "Modifier", Modifier_Init());
+	PyDict_SetItemString(dict, "NMesh", NMesh_Init());
+	PyDict_SetItemString(dict, "Noise", Noise_Init());
+	PyDict_SetItemString(dict, "Object", Object_Init());
+	PyDict_SetItemString(dict, "Group", Group_Init());
+	PyDict_SetItemString(dict, "Registry", Registry_Init());
+	PyDict_SetItemString(dict, "Scene", Scene_Init());
+	PyDict_SetItemString(dict, "Sound", Sound_Init());
+	PyDict_SetItemString(dict, "SurfNurb", SurfNurb_Init());
+	PyDict_SetItemString(dict, "sys", sys_Init());
+	PyDict_SetItemString(dict, "Types", Types_Init());
+	PyDict_SetItemString(dict, "Text", Text_Init());
+	PyDict_SetItemString(dict, "Text3d", Text3d_Init());
+	PyDict_SetItemString(dict, "Texture", Texture_Init());
+	PyDict_SetItemString(dict, "Window", Window_Init());
+	PyDict_SetItemString(dict, "World", World_Init());
+
 }
