@@ -1,5 +1,5 @@
 /*
- * $Id: Constraint.c,v 1.4 2006/06/14 05:16:39 khughes Exp $
+ * $Id: Constraint.c,v 1.9 2006/12/02 03:48:13 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -67,7 +67,7 @@ enum constraint_constants {
 	EXPP_CONSTR_MINZ = TRACK_nZ,
 
 	EXPP_CONSTR_TARGET = 100,
-	EXPP_CONSTR_TOLERANCE,
+	EXPP_CONSTR_STRETCH,
 	EXPP_CONSTR_ITERATIONS,
 	EXPP_CONSTR_BONE,
 	EXPP_CONSTR_CHAINLEN,
@@ -102,6 +102,55 @@ enum constraint_constants {
 	EXPP_CONSTR_STICKY,
 
 	EXPP_CONSTR_COPY,
+	EXPP_CONSTR_LIMIT,
+	
+	EXPP_CONSTR_LIMXMIN = LIMIT_XMIN,
+	EXPP_CONSTR_LIMXMAX = LIMIT_XMAX,
+	EXPP_CONSTR_LIMYMIN = LIMIT_YMIN,
+	EXPP_CONSTR_LIMYMAX = LIMIT_YMAX,
+	EXPP_CONSTR_LIMZMIN = LIMIT_ZMIN,
+	EXPP_CONSTR_LIMZMAX = LIMIT_ZMAX,
+	
+	EXPP_CONSTR_LIMXROT = LIMIT_XROT,
+	EXPP_CONSTR_LIMYROT = LIMIT_YROT,
+	EXPP_CONSTR_LIMZROT = LIMIT_ZROT,
+	
+	EXPP_CONSTR_XMIN,
+	EXPP_CONSTR_XMAX,
+	EXPP_CONSTR_YMIN,
+	EXPP_CONSTR_YMAX,
+	EXPP_CONSTR_ZMIN,
+	EXPP_CONSTR_ZMAX,
+	
+	EXPP_CONSTR_LIMLOCALBONE,
+	EXPP_CONSTR_LIMLOCALNOPAR,
+	
+	EXPP_CONSTR_RB_TYPE,
+	EXPP_CONSTR_RB_BALL,
+	EXPP_CONSTR_RB_HINGE,
+	EXPP_CONSTR_RB_GENERIC6DOF,
+	EXPP_CONSTR_RB_VEHICLE,
+	EXPP_CONSTR_RB_PIVX,
+	EXPP_CONSTR_RB_PIVY,
+	EXPP_CONSTR_RB_PIVZ,
+	EXPP_CONSTR_RB_AXX,
+	EXPP_CONSTR_RB_AXY,
+	EXPP_CONSTR_RB_AXZ,
+	EXPP_CONSTR_RB_MINLIMIT0,
+	EXPP_CONSTR_RB_MINLIMIT1,
+	EXPP_CONSTR_RB_MINLIMIT2,
+	EXPP_CONSTR_RB_MINLIMIT3,
+	EXPP_CONSTR_RB_MINLIMIT4,
+	EXPP_CONSTR_RB_MINLIMIT5,
+	EXPP_CONSTR_RB_MAXLIMIT0,
+	EXPP_CONSTR_RB_MAXLIMIT1,
+	EXPP_CONSTR_RB_MAXLIMIT2,
+	EXPP_CONSTR_RB_MAXLIMIT3,
+	EXPP_CONSTR_RB_MAXLIMIT4,
+	EXPP_CONSTR_RB_MAXLIMIT5,
+	EXPP_CONSTR_RB_EXTRAFZ,
+	EXPP_CONSTR_RB_FLAG,
+	
 };
 
 /*****************************************************************************/
@@ -159,6 +208,7 @@ static PyMappingMethods Constraint_as_mapping = {
 /*****************************************************************************/
 static void Constraint_dealloc( BPy_Constraint * self );
 static PyObject *Constraint_repr( BPy_Constraint * self );
+static int Constraint_compare( BPy_Constraint * a, BPy_Constraint * b );
 
 /*****************************************************************************/
 /* Python Constraint_Type structure definition:                              */
@@ -177,7 +227,7 @@ PyTypeObject Constraint_Type = {
 	NULL,                       /* printfunc tp_print; */
 	NULL,                       /* getattrfunc tp_getattr; */
 	NULL,                       /* setattrfunc tp_setattr; */
-	NULL,                       /* cmpfunc tp_compare; */
+	( cmpfunc ) Constraint_compare, /* cmpfunc tp_compare; */
 	( reprfunc ) Constraint_repr, /* reprfunc tp_repr; */
 
 	/* Method suites for standard classes */
@@ -367,8 +417,8 @@ static PyObject *kinematic_getter( BPy_Constraint * self, int type )
 		return Object_CreatePyObject( con->tar );
 	case EXPP_CONSTR_BONE:
 		return PyString_FromString( con->subtarget );
-	case EXPP_CONSTR_TOLERANCE:
-		return PyFloat_FromDouble( (double)con->tolerance );
+	case EXPP_CONSTR_STRETCH:
+		return PyBool_FromLong( (long)( con->flag & CONSTRAINT_IK_STRETCH ) ) ;
 	case EXPP_CONSTR_ITERATIONS:
 		return PyInt_FromLong( (long)con->iterations );
 	case EXPP_CONSTR_CHAINLEN:
@@ -409,8 +459,8 @@ static int kinematic_setter( BPy_Constraint *self, int type, PyObject *value )
 
 		return 0;
 		}
-	case EXPP_CONSTR_TOLERANCE:
-		return EXPP_setFloatClamped( value, &con->tolerance, 0.0001, 1.0 );
+	case EXPP_CONSTR_STRETCH:
+		return EXPP_setBitfield( value, &con->flag, CONSTRAINT_IK_STRETCH, 'h' );
 	case EXPP_CONSTR_ITERATIONS:
 		return EXPP_setIValueClamped( value, &con->iterations, 1, 10000, 'h' );
 	case EXPP_CONSTR_CHAINLEN:
@@ -740,7 +790,7 @@ static PyObject *floor_getter( BPy_Constraint * self, int type )
 	case EXPP_CONSTR_OFFSET:
 		return PyFloat_FromDouble( (double)con->offset );
 	case EXPP_CONSTR_STICKY:
-		return PyBool_FromLong( (long)( con->sticky & SELECT ) ) ;
+		return PyBool_FromLong( (long)( con->flag & MINMAX_STICKY ) ) ;
 	default:
 		return EXPP_ReturnPyObjError( PyExc_KeyError, "key not found" );
 	}
@@ -775,7 +825,7 @@ static int floor_setter( BPy_Constraint *self, int type, PyObject *value )
 	case EXPP_CONSTR_OFFSET:
 		return EXPP_setFloatClamped( value, &con->offset, -100.0, 100.0 );
 	case EXPP_CONSTR_STICKY:
-		return EXPP_setBitfield( value, &con->sticky, SELECT, 'h' );
+		return EXPP_setBitfield( value, &con->flag, MINMAX_STICKY, 'h' );
 	default:
 		return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
 	}
@@ -959,6 +1009,290 @@ static int sizelike_setter( BPy_Constraint *self, int type, PyObject *value )
 	}
 }
 
+static PyObject *loclimit_getter( BPy_Constraint * self, int type)
+{
+	bLocLimitConstraint *con = (bLocLimitConstraint *)(self->con->data);
+
+	switch( type ) {
+	case EXPP_CONSTR_LIMIT:
+		return PyInt_FromLong( (long)con->flag );
+	case EXPP_CONSTR_LIMLOCALBONE:
+		return PyBool_FromLong( (long)
+					( self->con->flag & CONSTRAINT_LOCAL ) ) ;
+	case EXPP_CONSTR_LIMLOCALNOPAR:
+		return PyBool_FromLong( (long)
+					( con->flag2 & LIMIT_NOPARENT ) ) ;
+	case EXPP_CONSTR_XMIN:
+		return PyFloat_FromDouble( (double)con->xmin );
+	case EXPP_CONSTR_XMAX:
+		return PyFloat_FromDouble( (double)con->xmax );
+	case EXPP_CONSTR_YMIN:
+		return PyFloat_FromDouble( (double)con->ymin );
+	case EXPP_CONSTR_YMAX:
+		return PyFloat_FromDouble( (double)con->ymax );
+	case EXPP_CONSTR_ZMIN:
+		return PyFloat_FromDouble( (double)con->zmin );
+	case EXPP_CONSTR_ZMAX:
+		return PyFloat_FromDouble( (double)con->zmax );
+	default:
+		return EXPP_ReturnPyObjError( PyExc_KeyError, "key not found" );	
+	}
+}
+
+static int loclimit_setter( BPy_Constraint *self, int type, PyObject *value )
+{
+	bLocLimitConstraint *con = (bLocLimitConstraint *)(self->con->data);	
+	
+	switch( type ) {
+	case EXPP_CONSTR_LIMIT:
+		return EXPP_setIValueRange( value, &con->flag, 0, 
+			LIMIT_XMIN | LIMIT_XMAX | LIMIT_YMIN | LIMIT_YMAX | LIMIT_ZMIN | LIMIT_ZMAX , 'i' );
+	case EXPP_CONSTR_LIMLOCALBONE:
+		return EXPP_setBitfield( value, &self->con->flag,
+				CONSTRAINT_LOCAL, 'h' );
+	case EXPP_CONSTR_LIMLOCALNOPAR:
+		return EXPP_setBitfield( value, &con->flag2,
+				LIMIT_NOPARENT, 'h' );
+	case EXPP_CONSTR_XMIN:
+		return EXPP_setFloatClamped( value, &con->xmin, -1000.0, 1000.0 );
+	case EXPP_CONSTR_XMAX:
+		return EXPP_setFloatClamped( value, &con->xmax, -1000.0, 1000.0 );
+	case EXPP_CONSTR_YMIN:
+		return EXPP_setFloatClamped( value, &con->ymin, -1000.0, 1000.0 );
+	case EXPP_CONSTR_YMAX:
+		return EXPP_setFloatClamped( value, &con->ymax, -1000.0, 1000.0 );
+	case EXPP_CONSTR_ZMIN:
+		return EXPP_setFloatClamped( value, &con->zmin, -1000.0, 1000.0 );
+	case EXPP_CONSTR_ZMAX:
+		return EXPP_setFloatClamped( value, &con->zmax, -1000.0, 1000.0 );
+	default:
+		return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
+	}
+}
+
+static PyObject *rotlimit_getter( BPy_Constraint * self, int type )
+{
+	bRotLimitConstraint *con = (bRotLimitConstraint *)(self->con->data);
+
+	switch( type ) {
+	case EXPP_CONSTR_LIMIT:
+		return PyInt_FromLong( (long)con->flag );
+	case EXPP_CONSTR_LIMLOCALBONE:
+		return PyBool_FromLong( (long)
+					(self->con->flag & CONSTRAINT_LOCAL ) ); 
+	case EXPP_CONSTR_XMIN:
+		return PyFloat_FromDouble( (double)con->xmin );
+	case EXPP_CONSTR_XMAX:
+		return PyFloat_FromDouble( (double)con->xmax );
+	case EXPP_CONSTR_YMIN:
+		return PyFloat_FromDouble( (double)con->ymin );
+	case EXPP_CONSTR_YMAX:
+		return PyFloat_FromDouble( (double)con->ymax );
+	case EXPP_CONSTR_ZMIN:
+		return PyFloat_FromDouble( (double)con->zmin );
+	case EXPP_CONSTR_ZMAX:
+		return PyFloat_FromDouble( (double)con->zmax );
+	default:
+		return EXPP_ReturnPyObjError( PyExc_KeyError, "key not found" );	
+	}
+}
+
+static int rotlimit_setter( BPy_Constraint *self, int type, PyObject *value )
+{
+	bRotLimitConstraint *con = (bRotLimitConstraint *)(self->con->data);	
+	
+	switch( type ) {
+	case EXPP_CONSTR_LIMIT:
+		return EXPP_setIValueRange( value, &con->flag, 0, 
+			LIMIT_XROT | LIMIT_YROT | LIMIT_ZROT, 'i' );
+	case EXPP_CONSTR_LIMLOCALBONE:
+		return EXPP_setBitfield( value, &self->con->flag,
+				CONSTRAINT_LOCAL, 'h' );
+	case EXPP_CONSTR_XMIN:
+		return EXPP_setFloatClamped( value, &con->xmin, -360.0, 360.0 );
+	case EXPP_CONSTR_XMAX:
+		return EXPP_setFloatClamped( value, &con->xmax, -360.0, 360.0 );
+	case EXPP_CONSTR_YMIN:
+		return EXPP_setFloatClamped( value, &con->ymin, -360.0, 360.0 );
+	case EXPP_CONSTR_YMAX:
+		return EXPP_setFloatClamped( value, &con->ymax, -360.0, 360.0 );
+	case EXPP_CONSTR_ZMIN:
+		return EXPP_setFloatClamped( value, &con->zmin, -360.0, 360.0 );
+	case EXPP_CONSTR_ZMAX:
+		return EXPP_setFloatClamped( value, &con->zmax, -360.0, 360.0 );
+	default:
+		return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
+	}
+}
+
+static PyObject *sizelimit_getter( BPy_Constraint * self, int type)
+{
+	bSizeLimitConstraint *con = (bSizeLimitConstraint *)(self->con->data);
+
+	switch( type ) {
+	case EXPP_CONSTR_LIMIT:
+		return PyInt_FromLong( (long)con->flag );
+	case EXPP_CONSTR_XMIN:
+		return PyFloat_FromDouble( (double)con->xmin );
+	case EXPP_CONSTR_XMAX:
+		return PyFloat_FromDouble( (double)con->xmax );
+	case EXPP_CONSTR_YMIN:
+		return PyFloat_FromDouble( (double)con->ymin );
+	case EXPP_CONSTR_YMAX:
+		return PyFloat_FromDouble( (double)con->ymax );
+	case EXPP_CONSTR_ZMIN:
+		return PyFloat_FromDouble( (double)con->zmin );
+	case EXPP_CONSTR_ZMAX:
+		return PyFloat_FromDouble( (double)con->zmax );
+	default:
+		return EXPP_ReturnPyObjError( PyExc_KeyError, "key not found" );	
+	}
+}
+
+static int sizelimit_setter( BPy_Constraint *self, int type, PyObject *value )
+{
+	bSizeLimitConstraint *con = (bSizeLimitConstraint *)(self->con->data);	
+	
+	switch( type ) {
+	case EXPP_CONSTR_LIMIT:
+		return EXPP_setIValueRange( value, &con->flag, 0, 
+			LIMIT_XMIN | LIMIT_XMAX | LIMIT_YMIN | LIMIT_YMAX | LIMIT_ZMIN | LIMIT_ZMAX, 'i' );
+	case EXPP_CONSTR_XMIN:
+		return EXPP_setFloatClamped( value, &con->xmin, -1000.0, 1000.0 );
+	case EXPP_CONSTR_XMAX:
+		return EXPP_setFloatClamped( value, &con->xmax, -1000.0, 1000.0 );
+	case EXPP_CONSTR_YMIN:
+		return EXPP_setFloatClamped( value, &con->ymin, -1000.0, 1000.0 );
+	case EXPP_CONSTR_YMAX:
+		return EXPP_setFloatClamped( value, &con->ymax, -1000.0, 1000.0 );
+	case EXPP_CONSTR_ZMIN:
+		return EXPP_setFloatClamped( value, &con->zmin, -1000.0, 1000.0 );
+	case EXPP_CONSTR_ZMAX:
+		return EXPP_setFloatClamped( value, &con->zmax, -1000.0, 1000.0 );
+	default:
+		return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
+	}
+}
+
+
+static PyObject *rigidbody_getter( BPy_Constraint * self, int type)
+{
+	bRigidBodyJointConstraint *con = (bRigidBodyJointConstraint *)(self->con->data);
+
+	switch( type ) {
+	case EXPP_CONSTR_TARGET:
+		return Object_CreatePyObject( con->tar );
+	case EXPP_CONSTR_RB_PIVX:
+		return PyFloat_FromDouble( (double)con->pivX );
+	case EXPP_CONSTR_RB_PIVY:
+		return PyFloat_FromDouble( (double)con->pivY );
+	case EXPP_CONSTR_RB_PIVZ:
+		return PyFloat_FromDouble( (double)con->pivZ );
+	case EXPP_CONSTR_RB_AXX:
+		return PyFloat_FromDouble( (double)con->axX );
+	case EXPP_CONSTR_RB_AXY:
+		return PyFloat_FromDouble( (double)con->axY );
+	case EXPP_CONSTR_RB_AXZ:
+		return PyFloat_FromDouble( (double)con->axZ );
+	case EXPP_CONSTR_RB_MINLIMIT0:
+		return PyFloat_FromDouble( (double)con->minLimit[0] );
+	case EXPP_CONSTR_RB_MINLIMIT1:
+		return PyFloat_FromDouble( (double)con->minLimit[1] );
+	case EXPP_CONSTR_RB_MINLIMIT2:
+		return PyFloat_FromDouble( (double)con->minLimit[2] );
+	case EXPP_CONSTR_RB_MINLIMIT3:
+		return PyFloat_FromDouble( (double)con->minLimit[3] );
+	case EXPP_CONSTR_RB_MINLIMIT4:
+		return PyFloat_FromDouble( (double)con->minLimit[4] );
+	case EXPP_CONSTR_RB_MINLIMIT5:
+		return PyFloat_FromDouble( (double)con->minLimit[5] );
+	case EXPP_CONSTR_RB_MAXLIMIT0:
+		return PyFloat_FromDouble( (double)con->maxLimit[0] );
+	case EXPP_CONSTR_RB_MAXLIMIT1:
+		return PyFloat_FromDouble( (double)con->maxLimit[1] );
+	case EXPP_CONSTR_RB_MAXLIMIT2:
+		return PyFloat_FromDouble( (double)con->maxLimit[2] );
+	case EXPP_CONSTR_RB_MAXLIMIT3:
+		return PyFloat_FromDouble( (double)con->maxLimit[3] );
+	case EXPP_CONSTR_RB_MAXLIMIT4:
+		return PyFloat_FromDouble( (double)con->maxLimit[4] );
+	case EXPP_CONSTR_RB_MAXLIMIT5:
+		return PyFloat_FromDouble( (double)con->maxLimit[5] );
+	case EXPP_CONSTR_RB_EXTRAFZ:
+		return PyFloat_FromDouble( (double)con->extraFz );		
+	case EXPP_CONSTR_LIMIT:
+		return PyInt_FromLong( (int)con->flag );
+		
+	case EXPP_CONSTR_RB_TYPE:
+		return PyInt_FromLong( (int)con->type );
+	default:
+		return EXPP_ReturnPyObjError( PyExc_KeyError, "key not found" );	
+	}
+}
+
+
+static int rigidbody_setter( BPy_Constraint *self, int type, PyObject *value )
+{
+	bRigidBodyJointConstraint *con = (bRigidBodyJointConstraint *)(self->con->data);	
+	
+	switch( type ) {
+	case EXPP_CONSTR_TARGET: {
+		Object *obj = (( BPy_Object * )value)->object;
+		if( !BPy_Object_Check( value ) )
+			return EXPP_ReturnIntError( PyExc_TypeError, 
+					"expected BPy object argument" );
+		con->tar = obj;
+		return 0;
+		}
+	case EXPP_CONSTR_RB_PIVX:
+		return EXPP_setFloatClamped( value, &con->pivX , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_PIVY:
+		return EXPP_setFloatClamped( value, &con->pivY , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_PIVZ:
+		return EXPP_setFloatClamped( value, &con->pivZ , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_AXX:
+		return EXPP_setFloatClamped( value, &con->axX , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_AXY:
+		return EXPP_setFloatClamped( value, &con->axY , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_AXZ:
+		return EXPP_setFloatClamped( value, &con->axZ , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_MINLIMIT0:
+		return EXPP_setFloatClamped( value, &con->minLimit[0] , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_MINLIMIT1:
+		return EXPP_setFloatClamped( value, &con->minLimit[1] , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_MINLIMIT2:
+		return EXPP_setFloatClamped( value, &con->minLimit[2] , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_MINLIMIT3:
+		return EXPP_setFloatClamped( value, &con->minLimit[3] , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_MINLIMIT4:
+		return EXPP_setFloatClamped( value, &con->minLimit[4] , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_MINLIMIT5:
+		return EXPP_setFloatClamped( value, &con->minLimit[5] , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_MAXLIMIT0:
+		return EXPP_setFloatClamped( value, &con->maxLimit[0] , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_MAXLIMIT1:
+		return EXPP_setFloatClamped( value, &con->maxLimit[1] , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_MAXLIMIT2:
+		return EXPP_setFloatClamped( value, &con->maxLimit[2] , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_MAXLIMIT3:
+		return EXPP_setFloatClamped( value, &con->maxLimit[3] , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_MAXLIMIT4:
+		return EXPP_setFloatClamped( value, &con->maxLimit[4] , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_MAXLIMIT5:
+		return EXPP_setFloatClamped( value, &con->maxLimit[5] , -1000.0, 1000.0 );
+	case EXPP_CONSTR_RB_EXTRAFZ:
+		return EXPP_setFloatClamped( value, &con->extraFz , -1000.0, 1000.0 );						
+	case EXPP_CONSTR_LIMIT:
+		return EXPP_setIValueRange( value, &con->flag, 0, 
+			LIMIT_XMIN | LIMIT_XMAX | LIMIT_YMIN | LIMIT_YMAX | LIMIT_ZMIN | LIMIT_ZMAX, 'i' );
+	case EXPP_CONSTR_RB_TYPE:
+		return EXPP_setIValueRange( value, &con->type, 0, 
+			EXPP_CONSTR_RB_BALL | EXPP_CONSTR_RB_HINGE | EXPP_CONSTR_RB_GENERIC6DOF | EXPP_CONSTR_RB_VEHICLE, 'i' );
+	default:
+		return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
+	}
+}
+
 /*
  * get data from a constraint
  */
@@ -999,11 +1333,15 @@ static PyObject *Constraint_getData( BPy_Constraint * self, PyObject * key )
 			return rotatelike_getter( self, setting );
 		case CONSTRAINT_TYPE_SIZELIKE:
 			return sizelike_getter( self, setting );
-
-		case CONSTRAINT_TYPE_CHILDOF:	/* Unimplemented */
 		case CONSTRAINT_TYPE_ROTLIMIT:
+			return rotlimit_getter( self, setting );
 		case CONSTRAINT_TYPE_LOCLIMIT:
+			return loclimit_getter( self, setting );
 		case CONSTRAINT_TYPE_SIZELIMIT:
+			return sizelimit_getter( self, setting );
+		case CONSTRAINT_TYPE_RIGIDBODYJOINT:
+			return rigidbody_getter( self, setting );
+		case CONSTRAINT_TYPE_CHILDOF:	/* Unimplemented */
 		case CONSTRAINT_TYPE_PYTHON:
 		default:
 			return EXPP_ReturnPyObjError( PyExc_KeyError,
@@ -1055,12 +1393,21 @@ static int Constraint_setData( BPy_Constraint * self, PyObject * key,
 	case CONSTRAINT_TYPE_SIZELIKE:
 		result = sizelike_setter( self, key_int, arg );
 		break;
+	case CONSTRAINT_TYPE_ROTLIMIT:
+		result = rotlimit_setter( self, key_int, arg );
+		break;
+	case CONSTRAINT_TYPE_LOCLIMIT:
+		result = loclimit_setter( self, key_int, arg );
+		break;
+	case CONSTRAINT_TYPE_SIZELIMIT:
+		result = sizelimit_setter( self, key_int, arg);
+		break;
+	case CONSTRAINT_TYPE_RIGIDBODYJOINT:
+		result = rigidbody_setter( self, key_int, arg);
+		break;
 	case CONSTRAINT_TYPE_NULL:
 		return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
 	case CONSTRAINT_TYPE_CHILDOF:	/* Unimplemented */
-	case CONSTRAINT_TYPE_ROTLIMIT:
-	case CONSTRAINT_TYPE_LOCLIMIT:
-	case CONSTRAINT_TYPE_SIZELIMIT:
 	case CONSTRAINT_TYPE_PYTHON:
 	default:
 		return EXPP_ReturnIntError( PyExc_RuntimeError,
@@ -1081,6 +1428,16 @@ static void Constraint_dealloc( BPy_Constraint * self )
 	PyObject_DEL( self );
 }
 
+
+/*****************************************************************************/
+/* Function:    Constraint_compare                                           */
+/* Description: This compares 2 constraint python types, == or != only.      */
+/*****************************************************************************/
+static int Constraint_compare( BPy_Constraint * a, BPy_Constraint * b )
+{
+	return ( a->con == b->con ) ? 0 : -1;
+}
+
 /*****************************************************************************/
 /* Function:    Constraint_repr                                              */
 /* Description: This is a callback function for the BPy_Constraint type. It  */
@@ -1277,7 +1634,7 @@ static PyObject *ConstraintSeq_append( BPy_ConstraintSeq *self, PyObject *args )
 		EXPP_ReturnPyObjError( PyExc_TypeError, "expected int argument" );
 
 	/* type 0 is CONSTRAINT_TYPE_NULL, should we be able to add one of these? */
-	if( type < CONSTRAINT_TYPE_NULL || type > CONSTRAINT_TYPE_MINMAX ) 
+	if( type < CONSTRAINT_TYPE_NULL || type > CONSTRAINT_TYPE_RIGIDBODYJOINT ) 
 		return EXPP_ReturnPyObjError( PyExc_ValueError,
 				"int argument out of range" );
 
@@ -1535,6 +1892,14 @@ static PyObject *M_Constraint_TypeDict( void )
 				PyInt_FromLong( CONSTRAINT_TYPE_STRETCHTO ) );
 		PyConstant_Insert( d, "FLOOR", 
 				PyInt_FromLong( CONSTRAINT_TYPE_MINMAX ) );
+		PyConstant_Insert( d, "LIMITLOC", 
+				PyInt_FromLong( CONSTRAINT_TYPE_LOCLIMIT ) );
+		PyConstant_Insert( d, "LIMITROT", 
+				PyInt_FromLong( CONSTRAINT_TYPE_ROTLIMIT ) );
+		PyConstant_Insert( d, "LIMITSIZE", 
+				PyInt_FromLong( CONSTRAINT_TYPE_SIZELIMIT ) );
+		PyConstant_Insert( d, "RIGIDBODYJOINT", 
+				PyInt_FromLong( CONSTRAINT_TYPE_RIGIDBODYJOINT ) );
 	}
 	return S;
 }
@@ -1617,8 +1982,8 @@ static PyObject *M_Constraint_SettingsDict( void )
 
 		PyConstant_Insert( d, "TARGET",
 				PyInt_FromLong( EXPP_CONSTR_TARGET ) );
-		PyConstant_Insert( d, "TOLERANCE", 
-				PyInt_FromLong( EXPP_CONSTR_TOLERANCE ) );
+		PyConstant_Insert( d, "STRETCH", 
+				PyInt_FromLong( EXPP_CONSTR_STRETCH ) );
 		PyConstant_Insert( d, "ITERATIONS", 
 				PyInt_FromLong( EXPP_CONSTR_ITERATIONS ) );
 		PyConstant_Insert( d, "BONE", 
@@ -1675,6 +2040,98 @@ static PyObject *M_Constraint_SettingsDict( void )
 
 		PyConstant_Insert( d, "COPY",
 				PyInt_FromLong( EXPP_CONSTR_COPY ) );
+		PyConstant_Insert( d, "LIMIT",
+				PyInt_FromLong( EXPP_CONSTR_LIMIT ) );
+		
+		PyConstant_Insert( d, "LIMIT_XMIN",
+				PyInt_FromLong( EXPP_CONSTR_LIMXMIN ) );
+		PyConstant_Insert( d, "LIMIT_XMAX",
+				PyInt_FromLong( EXPP_CONSTR_LIMXMAX ) );
+		PyConstant_Insert( d, "LIMIT_YMIN",
+				PyInt_FromLong( EXPP_CONSTR_LIMYMIN ) );
+		PyConstant_Insert( d, "LIMIT_YMAX",
+				PyInt_FromLong( EXPP_CONSTR_LIMYMAX ) );
+		PyConstant_Insert( d, "LIMIT_ZMIN",
+				PyInt_FromLong( EXPP_CONSTR_LIMZMIN ) );
+		PyConstant_Insert( d, "LIMIT_ZMAX",
+				PyInt_FromLong( EXPP_CONSTR_LIMZMAX ) );
+		
+		PyConstant_Insert( d, "LIMIT_XROT",
+				PyInt_FromLong( EXPP_CONSTR_LIMXROT ) );
+		PyConstant_Insert( d, "LIMIT_YROT",
+				PyInt_FromLong( EXPP_CONSTR_LIMYROT ) );
+		PyConstant_Insert( d, "LIMIT_ZROT",
+				PyInt_FromLong( EXPP_CONSTR_LIMZROT ) );
+		
+		PyConstant_Insert( d, "XMIN",
+				PyInt_FromLong( EXPP_CONSTR_XMIN ) );
+		PyConstant_Insert( d, "XMAX",
+				PyInt_FromLong( EXPP_CONSTR_XMAX ) );
+		PyConstant_Insert( d, "YMIN",
+				PyInt_FromLong( EXPP_CONSTR_YMIN ) );
+		PyConstant_Insert( d, "YMAX",
+				PyInt_FromLong( EXPP_CONSTR_YMAX ) );
+		PyConstant_Insert( d, "ZMIN",
+				PyInt_FromLong( EXPP_CONSTR_ZMIN ) );
+		PyConstant_Insert( d, "ZMAX",
+				PyInt_FromLong( EXPP_CONSTR_ZMAX ) );
+		
+		PyConstant_Insert( d, "LIMIT_LOCAL_BONE",
+				PyInt_FromLong( EXPP_CONSTR_LIMLOCALBONE ) );
+		PyConstant_Insert( d, "LIMIT_LOCAL_NOPARENT",
+				PyInt_FromLong( EXPP_CONSTR_LIMLOCALNOPAR ) );
+
+
+		PyConstant_Insert( d, "CONSTR_RB_TYPE",
+				PyInt_FromLong( EXPP_CONSTR_RB_TYPE ) );
+		PyConstant_Insert( d, "CONSTR_RB_BALL",
+				PyInt_FromLong( EXPP_CONSTR_RB_BALL ) );
+		PyConstant_Insert( d, "CONSTR_RB_HINGE",
+				PyInt_FromLong( EXPP_CONSTR_RB_HINGE ) );
+		PyConstant_Insert( d, "CONSTR_RB_GENERIC6DOF",
+				PyInt_FromLong( EXPP_CONSTR_RB_GENERIC6DOF ) );
+		PyConstant_Insert( d, "CONSTR_RB_VEHICLE",
+				PyInt_FromLong( EXPP_CONSTR_RB_VEHICLE ) );
+		PyConstant_Insert( d, "CONSTR_RB_PIVX",
+				PyInt_FromLong( EXPP_CONSTR_RB_PIVX ) );
+		PyConstant_Insert( d, "CONSTR_RB_PIVY",
+				PyInt_FromLong( EXPP_CONSTR_RB_PIVY ) );
+		PyConstant_Insert( d, "CONSTR_RB_PIVZ",
+				PyInt_FromLong( EXPP_CONSTR_RB_PIVZ ) );
+		PyConstant_Insert( d, "CONSTR_RB_AXX",
+				PyInt_FromLong( EXPP_CONSTR_RB_AXX ) );
+		PyConstant_Insert( d, "CONSTR_RB_AXY",
+				PyInt_FromLong( EXPP_CONSTR_RB_AXY ) );
+		PyConstant_Insert( d, "CONSTR_RB_AXZ",
+				PyInt_FromLong( EXPP_CONSTR_RB_AXZ ) );
+		PyConstant_Insert( d, "CONSTR_RB_MINLIMIT0",
+				PyInt_FromLong( EXPP_CONSTR_RB_MINLIMIT0 ) );
+		PyConstant_Insert( d, "CONSTR_RB_MINLIMIT1",
+				PyInt_FromLong( EXPP_CONSTR_RB_MINLIMIT1 ) );
+		PyConstant_Insert( d, "CONSTR_RB_MINLIMIT2",
+				PyInt_FromLong( EXPP_CONSTR_RB_MINLIMIT2 ) );
+		PyConstant_Insert( d, "CONSTR_RB_MINLIMIT3",
+				PyInt_FromLong( EXPP_CONSTR_RB_MINLIMIT3 ) );
+		PyConstant_Insert( d, "CONSTR_RB_MINLIMIT4",
+				PyInt_FromLong( EXPP_CONSTR_RB_MINLIMIT4 ) );
+		PyConstant_Insert( d, "CONSTR_RB_MINLIMIT5",
+				PyInt_FromLong( EXPP_CONSTR_RB_MINLIMIT5 ) );
+		PyConstant_Insert( d, "CONSTR_RB_MAXLIMIT0",
+				PyInt_FromLong( EXPP_CONSTR_RB_MAXLIMIT0 ) );
+		PyConstant_Insert( d, "CONSTR_RB_MAXLIMIT1",
+				PyInt_FromLong( EXPP_CONSTR_RB_MAXLIMIT1 ) );
+		PyConstant_Insert( d, "CONSTR_RB_MAXLIMIT2",
+				PyInt_FromLong( EXPP_CONSTR_RB_MAXLIMIT2 ) );
+		PyConstant_Insert( d, "CONSTR_RB_MAXLIMIT3",
+				PyInt_FromLong( EXPP_CONSTR_RB_MAXLIMIT3 ) );
+		PyConstant_Insert( d, "CONSTR_RB_MAXLIMIT4",
+				PyInt_FromLong( EXPP_CONSTR_RB_MAXLIMIT4 ) );
+		PyConstant_Insert( d, "CONSTR_RB_MAXLIMIT5",
+				PyInt_FromLong( EXPP_CONSTR_RB_MAXLIMIT5 ) );				
+		PyConstant_Insert( d, "CONSTR_RB_EXTRAFZ",
+				PyInt_FromLong( EXPP_CONSTR_RB_EXTRAFZ ) );
+		PyConstant_Insert( d, "CONSTR_RB_FLAG",
+				PyInt_FromLong( EXPP_CONSTR_RB_FLAG ) );
 	}
 	return S;
 }
