@@ -1,5 +1,5 @@
 /*
- * $Id: CurNurb.c,v 1.11 2005/06/13 19:15:01 stiv Exp $
+ * $Id: CurNurb.c,v 1.20 2007/01/12 22:40:29 khughes Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -29,14 +29,12 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
 
-#include "Python.h"
-#include "DNA_curve_types.h"
+#include "CurNurb.h" /*This must come first */
+
 #include "BKE_curve.h"
 #include "BDR_editcurve.h"	/* for convertspline */
 #include "MEM_guardedalloc.h"
-
 #include "gen_utils.h"
-#include "CurNurb.h"
 #include "BezTriple.h"
 
 
@@ -46,7 +44,6 @@
 
 
 static PyObject *M_CurNurb_New( PyObject * self, PyObject * args );
-PyObject *CurNurb_CreatePyObject( Nurb * blen_nurb );
 static PyObject *CurNurb_setMatIndex( BPy_CurNurb * self, PyObject * args );
 static PyObject *CurNurb_getMatIndex( BPy_CurNurb * self );
 static PyObject *CurNurb_getFlagU( BPy_CurNurb * self );
@@ -56,16 +53,17 @@ static PyObject *CurNurb_setFlagV( BPy_CurNurb * self, PyObject * args );
 static PyObject *CurNurb_getType( BPy_CurNurb * self );
 static PyObject *CurNurb_setType( BPy_CurNurb * self, PyObject * args );
 /* static PyObject* CurNurb_setXXX( BPy_CurNurb* self, PyObject* args ); */
-PyObject *CurNurb_getPoint( BPy_CurNurb * self, int index );
 static int CurNurb_setPoint( BPy_CurNurb * self, int index, PyObject * ob );
 static int CurNurb_length( PyInstanceObject * inst );
 static PyObject *CurNurb_getIter( BPy_CurNurb * self );
 static PyObject *CurNurb_iterNext( BPy_CurNurb * self );
 PyObject *CurNurb_append( BPy_CurNurb * self, PyObject * args );
-PyObject *CurNurb_pointAtIndex( Nurb * nurb, int index );
+
 static PyObject *CurNurb_isNurb( BPy_CurNurb * self );
 static PyObject *CurNurb_isCyclic( BPy_CurNurb * self );
 static PyObject *CurNurb_dump( BPy_CurNurb * self );
+static PyObject *CurNurb_switchDirection( BPy_CurNurb * self );
+static PyObject *CurNurb_recalc( BPy_CurNurb * self );
 
 char M_CurNurb_doc[] = "CurNurb";
 
@@ -134,6 +132,10 @@ static PyMethodDef BPy_CurNurb_methods[] = {
 	 "( ) - boolean function tests if this spline is cyclic (closed) or not (open)"},
 	{"dump", ( PyCFunction ) CurNurb_dump, METH_NOARGS,
 	 "( ) - dumps Nurb data)"},
+	{"switchDirection", ( PyCFunction ) CurNurb_switchDirection, METH_NOARGS,
+	 "( ) - swaps curve beginning and end)"},
+	{"recalc", ( PyCFunction ) CurNurb_recalc, METH_NOARGS,
+	 "( ) - recalc Nurb data)"},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -406,10 +408,10 @@ static PyObject *CurNurb_getType( BPy_CurNurb * self )
 static PyObject *CurNurb_setType( BPy_CurNurb * self, PyObject * args )
 {
 	Nurb *nurb = self->nurb;
-	int type;
+	short type;
 
 	/* parameter type checking */
-	if( !PyArg_ParseTuple( args, "i", &type ) )
+	if( !PyArg_ParseTuple( args, "h", &type ) )
 		return EXPP_ReturnPyObjError
 			( PyExc_TypeError, "expected integer argument" );
 
@@ -500,9 +502,11 @@ PyObject *CurNurb_appendPointToNurb( Nurb * nurb, PyObject * args )
 				 ( PyExc_MemoryError, "allocation failed" ) );
 
 		/* copy old points to new */
-		memmove( nurb->bezt, tmp, sizeof( BezTriple ) * npoints );
-		if( tmp )
+		if( tmp ) {
+			memmove( nurb->bezt, tmp, sizeof( BezTriple ) * npoints );
 			MEM_freeN( tmp );
+		}
+
 		nurb->pntsu++;
 		/* add new point to end of list */
 		memcpy( nurb->bezt + npoints,
@@ -536,15 +540,24 @@ PyObject *CurNurb_appendPointToNurb( Nurb * nurb, PyObject * args )
 				sizeof( BPoint ) );
 
 			for( i = 0; i < 4; ++i ) {
-				float tmpx =
-					( float ) PyFloat_AsDouble
-					( PySequence_GetItem( pyOb, i ) );
-				nurb->bp[npoints].vec[i] = tmpx;
+				PyObject *item = PySequence_GetItem( pyOb, i );
 
+				if (item == NULL)
+					return NULL;
+
+
+				nurb->bp[npoints].vec[i] = ( float ) PyFloat_AsDouble( item );
+				Py_DECREF( item );
 			}
 
 			if (size == 5) {
-				nurb->bp[npoints].alfa = (float)PyFloat_AsDouble( PySequence_GetItem( pyOb, 4 ) );
+				PyObject *item = PySequence_GetItem( pyOb, i );
+
+				if (item == NULL)
+					return NULL;
+
+				nurb->bp[npoints].alfa = ( float ) PyFloat_AsDouble( item );
+				Py_DECREF( item );
 			}
 			else {
 				nurb->bp[npoints].alfa = 0.0f;
@@ -552,15 +565,15 @@ PyObject *CurNurb_appendPointToNurb( Nurb * nurb, PyObject * args )
 
 			makeknots( nurb, 1, nurb->flagu >> 1 );
 
-		} else if( size == 3 ) {	/* 3 xyz coords */
-			printf( "\nNot Yet Implemented!\n" );
-
+		} else {
+			return EXPP_ReturnPyObjError( PyExc_TypeError,
+					"expected a sequence of 4 or 5 floats" );
 		}
 
 	} else {
 		/* bail with error */
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
-					"expected a sequence of 4 (or optionaly 5) floats\n" );
+					"expected a sequence of 4 or 5 floats" );
 
 	}
 
@@ -584,7 +597,7 @@ static PyObject *CurNurb_setMatIndex( BPy_CurNurb * self, PyObject * args )
 			   "expected integer argument" ) );
 
 	/* fixme:  some range checking would be nice! */
-	self->nurb->mat_nr = index;
+	self->nurb->mat_nr = (short)index;
 
 	Py_INCREF( Py_None );
 	return Py_None;
@@ -629,7 +642,8 @@ static PyObject *CurNurb_getFlagU( BPy_CurNurb * self )
  *
  *  set curve's flagu and recalculate the knots
  *
- *  Possible values: 0 - uniform, 1 - endpoints, 2 - bezier
+ *  Possible values: 0 - uniform, 2 - endpoints, 4 - bezier
+ *    bit 0 controls CU_CYCLIC
  */
 
 static PyObject *CurNurb_setFlagU( BPy_CurNurb * self, PyObject * args )
@@ -637,12 +651,15 @@ static PyObject *CurNurb_setFlagU( BPy_CurNurb * self, PyObject * args )
 	int flagu;
 
 	if( !PyArg_ParseTuple( args, "i", &( flagu ) ) )
-		return ( EXPP_ReturnPyObjError
-			 ( PyExc_AttributeError,
-			   "expected integer argument" ) );
+		return EXPP_ReturnPyObjError( PyExc_TypeError,
+				"expected integer argument in range [0,5]" );
+
+	if( flagu < 0 || flagu > 5 )
+		return EXPP_ReturnPyObjError( PyExc_AttributeError,
+				"expected integer argument in range [0,5]" );
 
 	if( self->nurb->flagu != flagu ) {
-		self->nurb->flagu = flagu;
+		self->nurb->flagu = (short)flagu;
 		makeknots( self->nurb, 1, self->nurb->flagu >> 1 );
 	}
 
@@ -680,12 +697,15 @@ static PyObject *CurNurb_setFlagV( BPy_CurNurb * self, PyObject * args )
 	int flagv;
 
 	if( !PyArg_ParseTuple( args, "i", &( flagv ) ) )
-		return ( EXPP_ReturnPyObjError
-			 ( PyExc_AttributeError,
-			   "expected integer argument" ) );
+		return EXPP_ReturnPyObjError( PyExc_TypeError,
+				"expected integer argument in range [0,5]" );
+
+	if( flagv < 0 || flagv > 5 )
+		return EXPP_ReturnPyObjError( PyExc_AttributeError,
+				"expected integer argument in range [0,5]" );
 
 	if( self->nurb->flagv != flagv ) {
-		self->nurb->flagv = flagv;
+		self->nurb->flagv = (short)flagv;
 		makeknots( self->nurb, 2, self->nurb->flagv >> 1 );
 	}
 
@@ -890,15 +910,23 @@ static int CurNurb_setPoint( BPy_CurNurb * self, int index, PyObject * pyOb )
 
 		/* copy x, y, z, w */
 		for( i = 0; i < 4; ++i ) {
-			float tmpx =
-				( float ) PyFloat_AsDouble
-				( PySequence_GetItem( pyOb, i ) );
-			nurb->bp[index].vec[i] = tmpx;
+			PyObject *item = PySequence_GetItem( pyOb, i );
 
+			if (item == NULL)
+				return -1;
+
+			nurb->bp[index].vec[i] = ( float ) PyFloat_AsDouble( item );
+			Py_DECREF( item );
 		}
 
 		if (size == 5) {	/* set tilt, if present */
-			nurb->bp[index].alfa = (float)PyFloat_AsDouble( PySequence_GetItem( pyOb, 4 ) );
+			PyObject *item = PySequence_GetItem( pyOb, i );
+
+			if (item == NULL)
+				return -1;
+
+			nurb->bp[index].alfa = ( float ) PyFloat_AsDouble( item );
+			Py_DECREF( item );
 		}
 		else {				/* if not, set default	*/
 			nurb->bp[index].alfa = 0.0f;
@@ -967,7 +995,6 @@ PyObject *CurNurb_Init( void )
 	return ( submodule );
 }
 
-
 /*
   dump nurb
 */
@@ -1010,8 +1037,7 @@ PyObject *CurNurb_dump( BPy_CurNurb * self )
 			}
 		
 			/* alfa, s[2] */
-			printf("\n alpha: %5.2f   s: %d %d ", 
-				   bp->alfa, bp->s[0], bp->s[1] );
+			printf("\n alpha: %5.2f", bp->alfa);
 			/* f1, hide */
 			printf(" f1 %d  hide %d", bp->f1, bp->hide );
 			printf("\n");
@@ -1039,3 +1065,25 @@ PyObject *CurNurb_dump( BPy_CurNurb * self )
 
 	Py_RETURN_NONE;
 }
+
+/*
+  recalc nurb
+*/
+
+static PyObject *CurNurb_recalc( BPy_CurNurb * self )
+{
+	calchandlesNurb ( self->nurb );
+	Py_RETURN_NONE;
+}
+
+PyObject *CurNurb_switchDirection( BPy_CurNurb * self ) {
+	Nurb *nurb = self->nurb;
+	if( ! self->nurb ){  /* bail on error */
+		printf("\n no Nurb in this CurNurb");
+		Py_RETURN_NONE;
+	}
+	
+	switchdirectionNurb( nurb );
+	
+	Py_RETURN_NONE;
+}
