@@ -1,5 +1,5 @@
 /* 
- * $Id: Curve.c,v 1.39 2006/04/16 15:28:50 stiv Exp $
+ * $Id: Curve.c,v 1.50 2007/01/16 04:08:12 stiv Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -31,13 +31,16 @@
 
 #include "Curve.h" /*This must come first*/
 
+#include "BLI_blenlib.h"
 #include "BKE_main.h"
 #include "BKE_displist.h"
 #include "BKE_global.h"
 #include "BKE_library.h"
 #include "BKE_curve.h"
+#include "BKE_material.h"
 #include "MEM_guardedalloc.h"	/* because we wil be mallocing memory */
 #include "CurNurb.h"
+#include "SurfNurb.h"
 #include "Material.h"
 #include "Object.h"
 #include "Key.h"
@@ -107,12 +110,14 @@ static PyObject *Curve_appendPoint( BPy_Curve * self, PyObject * args );
 static PyObject *Curve_appendNurb( BPy_Curve * self, PyObject * args );
 
 static PyObject *Curve_getMaterials( BPy_Curve * self );
+static PyObject *Curve_setMaterials( BPy_Curve * self, PyObject * args );
 
 static PyObject *Curve_getBevOb( BPy_Curve * self );
 static PyObject *Curve_setBevOb( BPy_Curve * self, PyObject * args );
 
 static PyObject *Curve_getTaperOb( BPy_Curve * self );
 static PyObject *Curve_setTaperOb( BPy_Curve * self, PyObject * args );
+static PyObject *Curve_copy( BPy_Curve * self );
 
 static PyObject *Curve_getIter( BPy_Curve * self );
 static PyObject *Curve_iterNext( BPy_Curve * self );
@@ -226,6 +231,8 @@ Sets a control point "},
 	 "() - returns Taper Object assigned to this Curve"},
 	{"setTaperOb", ( PyCFunction ) Curve_setTaperOb, METH_VARARGS,
 	 "() - assign a Taper Object to this Curve"},
+	{"__copy__", ( PyCFunction ) Curve_copy, METH_NOARGS,
+	 "() - make a copy of this curve data"},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -237,6 +244,7 @@ static void CurveDeAlloc( BPy_Curve * msh );
 /* static int CurvePrint (BPy_Curve *msh, FILE *fp, int flags); */
 static int CurveSetAttr( BPy_Curve * msh, char *name, PyObject * v );
 static PyObject *CurveGetAttr( BPy_Curve * msh, char *name );
+static int CurveCopmpare( BPy_Curve * a, BPy_Curve * b );
 static PyObject *CurveRepr( BPy_Curve * msh );
 
 static PySequenceMethods Curve_as_sequence = {
@@ -267,7 +275,7 @@ PyTypeObject Curve_Type = {
 	0,			/* tp_print */
 	( getattrfunc ) CurveGetAttr,	/* tp_getattr */
 	( setattrfunc ) CurveSetAttr,	/* tp_setattr */
-	0,			/* tp_compare */
+	( cmpfunc ) CurveCopmpare,		/* tp_compare */
 	( reprfunc ) CurveRepr,	/* tp_repr */
 	/* methods for standard classes */
 	0,			/* tp_as_number */
@@ -322,7 +330,6 @@ PyTypeObject Curve_Type = {
 /*****************************************************************************/
 static PyObject *M_Curve_New( PyObject * self, PyObject * args )
 {
-	char buf[24];
 	char *name = NULL;
 	BPy_Curve *pycurve;	/* for Curve Data object wrapper in Python */
 	Curve *blcurve = 0;	/* for actual Curve Data we create in Blender */
@@ -350,10 +357,8 @@ static PyObject *M_Curve_New( PyObject * self, PyObject * args )
 			   "couldn't create Curve Data object" ) );
 
 	pycurve->curve = blcurve;	/* link Python curve wrapper to Blender Curve */
-	if( name ) {
-		PyOS_snprintf( buf, sizeof( buf ), "%s", name );
-		rename_id( &blcurve->id, buf );
-	}
+	if( name )
+		rename_id( &blcurve->id, name );
 
 	return ( PyObject * ) pycurve;
 }
@@ -465,18 +470,15 @@ PyObject *Curve_getName( BPy_Curve * self )
 PyObject *Curve_setName( BPy_Curve * self, PyObject * args )
 {
 	char *name;
-	char buf[50];
 
 	if( !PyArg_ParseTuple( args, "s", &( name ) ) )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
 						"expected string argument" ) );
-	PyOS_snprintf( buf, sizeof( buf ), "%s", name );
-	rename_id( &self->curve->id, buf );	/* proper way in Blender */
+
+	rename_id( &self->curve->id, name );	/* proper way in Blender */
 
 	Curve_update( self );
-
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 static PyObject *Curve_getPathLen( BPy_Curve * self )
@@ -498,8 +500,7 @@ static PyObject *Curve_setPathLen( BPy_Curve * self, PyObject * args )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
 						"expected int argument" ) );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 
@@ -522,8 +523,7 @@ static PyObject *Curve_setTotcol( BPy_Curve * self, PyObject * args )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
 						"expected int argument" ) );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 
@@ -546,8 +546,7 @@ PyObject *Curve_setMode( BPy_Curve * self, PyObject * args )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
 						"expected int argument" ) );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 
@@ -733,48 +732,6 @@ PyObject *Curve_setExt2( BPy_Curve * self, PyObject * args )
 }
 
 
-/*
-static PyObject *Curve_setControlPoint(BPy_Curve *self, PyObject *args)
-{
-  Nurb*ptrnurb = self->curve->nurb.first;
-  int numcourbe,numpoint,i,j;
-  float x,y,z,w;
-  float bez[9];
-  if (!ptrnurb){ Py_INCREF(Py_None);return Py_None;}
-
-  if (ptrnurb->bp)
-    if (!PyArg_ParseTuple(args, "iiffff", &numcourbe,&numpoint,&x,&y,&z,&w))  
-      return (EXPP_ReturnPyObjError (PyExc_AttributeError,
-								"expected int int float float float float arguments"));
-  if (ptrnurb->bezt)
-    if (!PyArg_ParseTuple(args, "iifffffffff", &numcourbe,&numpoint,
-						bez,bez+1,bez+2,bez+3,bez+4,bez+5,bez+6,bez+7,bez+8))  
-      return (EXPP_ReturnPyObjError (PyExc_AttributeError,
-					"expected int int float float float float float float "
-					"float float float arguments"));
-
-  for(i = 0;i< numcourbe;i++)
-    ptrnurb=ptrnurb->next;
-  if (ptrnurb->bp)
-    {
-      ptrnurb->bp[numpoint].vec[0] = x;
-      ptrnurb->bp[numpoint].vec[1] = y;
-      ptrnurb->bp[numpoint].vec[2] = z;
-      ptrnurb->bp[numpoint].vec[3] = w;
-    }
-  if (ptrnurb->bezt)
-    {
-      for(i = 0;i<3;i++)
-	for(j = 0;j<3;j++)
-	  ptrnurb->bezt[numpoint].vec[i][j] = bez[i*3+j];
-    }
-	
-  Py_INCREF(Py_None);
-  return Py_None;
-}
-*/
-
-
 /*
  * Curve_setControlPoint
  * this function sets an EXISTING control point.
@@ -787,10 +744,8 @@ static PyObject *Curve_setControlPoint( BPy_Curve * self, PyObject * args )
 	Nurb *ptrnurb = self->curve->nurb.first;
 	int numcourbe = 0, numpoint = 0, i, j;
 
-	if( !ptrnurb ) {
-		Py_INCREF( Py_None );
-		return Py_None;
-	}
+	if( !ptrnurb )
+		Py_RETURN_NONE;
 
 	if( ptrnurb->bp )
 		if( !PyArg_ParseTuple
@@ -822,8 +777,7 @@ static PyObject *Curve_setControlPoint( BPy_Curve * self, PyObject * args )
 							  ( listargs,
 							    i * 3 + j ) );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 static PyObject *Curve_getControlPoint( BPy_Curve * self, PyObject * args )
@@ -905,8 +859,7 @@ static PyObject *Curve_setLoc( BPy_Curve * self, PyObject * args )
 		PyObject *xx = PyList_GetItem( listargs, i );
 		self->curve->loc[i] = (float)PyFloat_AsDouble( xx );
 	}
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 static PyObject *Curve_getRot( BPy_Curve * self )
@@ -935,9 +888,7 @@ static PyObject *Curve_setRot( BPy_Curve * self, PyObject * args )
 		PyObject *xx = PyList_GetItem( listargs, i );
 		self->curve->rot[i] = (float)PyFloat_AsDouble( xx );
 	}
-	Py_INCREF( Py_None );
-	return Py_None;
-
+	Py_RETURN_NONE;
 }
 static PyObject *Curve_getSize( BPy_Curve * self )
 {
@@ -964,8 +915,7 @@ static PyObject *Curve_setSize( BPy_Curve * self, PyObject * args )
 		PyObject *xx = PyList_GetItem( listargs, i );
 		self->curve->size[i] = (float)PyFloat_AsDouble( xx );
 	}
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 
@@ -1203,34 +1153,31 @@ static PyObject *Curve_appendPoint( BPy_Curve * self, PyObject * args )
 
 
 /****
-  appendNurb( new_point )
-  create a new nurb in the Curve and add the point param to it.
-  returns a refernce to the newly created nurb.
-*****/
+ *
+ * appendNurb( new_point )
+ *
+ * create a new nurb in the Curve and add the point param to it.
+ * returns a refernce to the newly created nurb.
+ *
+ *****/
 
 static PyObject *Curve_appendNurb( BPy_Curve * self, PyObject * args )
 {
-	Nurb *nurb_ptr = self->curve->nurb.first;
-	Nurb **pptr = ( Nurb ** ) & ( self->curve->nurb.first );
+	ListBase *nurb_ptr = &(self->curve->nurb);
+
 	Nurb *new_nurb;
 
-
-	/* walk to end of nurblist */
-	if( nurb_ptr ) {
-		while( nurb_ptr->next ) {
-			nurb_ptr = nurb_ptr->next;
-		}
-		pptr = &nurb_ptr->next;
-	}
-
 	/* malloc new nurb */
 	new_nurb = ( Nurb * ) MEM_callocN( sizeof( Nurb ), "appendNurb" );
+
 	if( !new_nurb )
 		return EXPP_ReturnPyObjError
 			( PyExc_MemoryError, "unable to malloc Nurb" );
 
 	if( CurNurb_appendPointToNurb( new_nurb, args ) ) {
-		*pptr = new_nurb;
+		// add nurb to curve
+		BLI_addtail( nurb_ptr, new_nurb);
+
 		new_nurb->resolu = self->curve->resolu;
 		new_nurb->resolv = self->curve->resolv;
 		new_nurb->hide = 0;
@@ -1275,6 +1222,15 @@ static PyObject *Curve_appendNurb( BPy_Curve * self, PyObject * args )
 
 PyObject *Curve_update( BPy_Curve * self )
 {
+	Nurb *nu = self->curve->nurb.first;
+
+	/* recalculate handles for each curve: calchandlesNurb() will make
+	 * sure curves are bezier first */
+	while( nu ) {
+		calchandlesNurb ( nu );
+		nu = nu->next;
+	}
+
 	Object_updateDag( (void*) self->curve );
 
 	Py_RETURN_NONE;
@@ -1287,9 +1243,51 @@ PyObject *Curve_update( BPy_Curve * self )
 
 static PyObject *Curve_getMaterials( BPy_Curve * self )
 {
-	return ( EXPP_PyList_fromMaterialList( self->curve->mat,
-					       self->curve->totcol, 1 ) );
+	return EXPP_PyList_fromMaterialList( self->curve->mat,
+			self->curve->totcol, 1 );
+}
 
+static PyObject *Curve_setMaterials( BPy_Curve *self, PyObject * value )
+{
+	Material **matlist;
+	int len;
+
+	if( !PySequence_Check( value ) ||
+			!EXPP_check_sequence_consistency( value, &Material_Type ) )
+		return EXPP_ReturnPyObjError( PyExc_TypeError,
+			"sequence should only contain materials or None)" );
+
+	len = PySequence_Size( value );
+	if( len > 16 )
+		return EXPP_ReturnPyObjError( PyExc_TypeError,
+			"list can't have more than 16 materials" );
+
+	/* free old material list (if it exists) and adjust user counts */
+	if( self->curve->mat ) {
+		Curve *cur = self->curve;
+		int i;
+		for( i = cur->totcol; i-- > 0; )
+			if( cur->mat[i] )
+           		cur->mat[i]->id.us--;
+		MEM_freeN( cur->mat );
+	}
+
+	/* build the new material list, increment user count, store it */
+
+	matlist = EXPP_newMaterialList_fromPyList( value );
+	EXPP_incr_mats_us( matlist, len );
+	self->curve->mat = matlist;
+	self->curve->totcol = (short)len;
+
+/**@ This is another ugly fix due to the weird material handling of blender.
+    * it makes sure that object material lists get updated (by their length)
+    * according to their data material lists, otherwise blender crashes.
+    * It just stupidly runs through all objects...BAD BAD BAD.
+    */
+
+	test_object_materials( ( ID * ) self->curve );
+
+	Py_RETURN_NONE;
 }
 
 /*****************************************************************************/
@@ -1325,7 +1323,11 @@ PyObject *Curve_setBevOb( BPy_Curve * self, PyObject * args )
 	} else {
 	/* Accept Object with type 'Curve' */
 		if( Object_CheckPyObject( ( PyObject * ) pybevobj ) && 
-			pybevobj->object->type == OB_CURVE) {
+				pybevobj->object->type == OB_CURVE) {
+			if(self->curve == pybevobj->object->data )
+				return EXPP_ReturnPyObjError( PyExc_ValueError,
+							"objects cannot bevel themselves" );
+
 			self->curve->bevobj = 
 				Object_FromPyObject( ( PyObject * ) pybevobj );
 		} else {
@@ -1373,7 +1375,10 @@ PyObject *Curve_setTaperOb( BPy_Curve * self, PyObject * args )
 	} else {
 	/* Accept Object with type 'Curve' */
 		if( Object_CheckPyObject( ( PyObject * ) pytaperobj ) && 
-			pytaperobj->object->type == OB_CURVE) {
+				pytaperobj->object->type == OB_CURVE) {
+			if(self->curve == pytaperobj->object->data )
+				return EXPP_ReturnPyObjError( PyExc_ValueError,
+							"objects cannot taper themselves" );
 			self->curve->taperobj = 
 				Object_FromPyObject( ( PyObject * ) pytaperobj );
 		} else {
@@ -1385,6 +1390,40 @@ PyObject *Curve_setTaperOb( BPy_Curve * self, PyObject * args )
 	return EXPP_incr_ret( Py_None );
 }
 
+/*****************************************************************************/
+/* Function:    Curve_copy                                                   */
+/* Description: Return a copy of this curve data.                            */
+/*****************************************************************************/
+
+PyObject *Curve_copy( BPy_Curve * self )
+{
+	BPy_Curve *pycurve;	/* for Curve Data object wrapper in Python */
+	Curve *blcurve = 0;	/* for actual Curve Data we create in Blender */
+
+	/* copies the data */
+	blcurve = copy_curve( self->curve );	/* first create the Curve Data in Blender */
+
+	if( blcurve == NULL )	/* bail out if add_curve() failed */
+		return ( EXPP_ReturnPyObjError
+			 ( PyExc_RuntimeError,
+			   "couldn't create Curve Data in Blender" ) );
+
+	/* return user count to zero because add_curve() inc'd it */
+	blcurve->id.us = 0;
+	
+	/* create python wrapper obj */
+	pycurve = ( BPy_Curve * ) PyObject_NEW( BPy_Curve, &Curve_Type );
+
+	if( pycurve == NULL )
+		return ( EXPP_ReturnPyObjError
+			 ( PyExc_MemoryError,
+			   "couldn't create Curve Data object" ) );
+
+	pycurve->curve = blcurve;	/* link Python curve wrapper to Blender Curve */
+	return ( PyObject * ) pycurve;
+}
+
+
 /*
  * Curve_getIter
  *
@@ -1412,24 +1451,22 @@ static PyObject *Curve_getIter( BPy_Curve * self )
 
 static PyObject *Curve_iterNext( BPy_Curve * self )
 {
-	PyObject *po;		/* return value */
 	Nurb *pnurb;
 
 	if( self->iter_pointer ) {
 		pnurb = self->iter_pointer;
 		self->iter_pointer = pnurb->next;	/* advance iterator */
-		po = CurNurb_CreatePyObject( pnurb );	/* make a bpy_nurb */
-
-		return ( PyObject * ) po;
+		if( (pnurb->type & 7) == CU_BEZIER || pnurb->pntsv <= 1 )
+			return CurNurb_CreatePyObject( pnurb ); /* make a bpy_curnurb */
+		else
+			return SurfNurb_CreatePyObject( pnurb ); /* make a bpy_surfnurb */
 	}
 
 	/* if iter_pointer was null, we are at end */
-	return ( EXPP_ReturnPyObjError
-		 ( PyExc_StopIteration, "iterator at end" ) );
+	return EXPP_ReturnPyObjError( PyExc_StopIteration,
+			"iterator at end" );
 }
 
-
-
 /* tp_sequence methods */
 
 /*
@@ -1460,7 +1497,6 @@ static int Curve_length( PyInstanceObject * inst )
 
 PyObject *Curve_getNurb( BPy_Curve * self, int n )
 {
-	PyObject *pyo;
 	Nurb *pNurb;
 	int i;
 
@@ -1481,8 +1517,12 @@ PyObject *Curve_getNurb( BPy_Curve * self, int n )
 		return ( EXPP_ReturnPyObjError( PyExc_IndexError,
 						"index out of range" ) );
 
-	pyo = CurNurb_CreatePyObject( pNurb );	/* make a bpy_curnurb */
-	return ( PyObject * ) pyo;
+	/* until there is a Surface BPyType, distinquish between a curve and a
+	 * surface based on whether it's a Bezier and the v size */
+	if( (pNurb->type & 7) == CU_BEZIER || pNurb->pntsv <= 1 )
+		return CurNurb_CreatePyObject( pNurb );	/* make a bpy_curnurb */
+	else
+		return SurfNurb_CreatePyObject( pNurb );	/* make a bpy_surfnurb */
 
 }
 
@@ -1540,6 +1580,8 @@ static PyObject *CurveGetAttr( BPy_Curve * self, char *name )
 		return Curve_getTaperOb( self );
 	else if( strcmp( name, "key" ) == 0 )
 		return Curve_getKey( self );
+	else if( strcmp( name, "materials" ) == 0 )
+		return Curve_getMaterials( self );
 #if 0
 	else if( strcmp( name, "numpts" ) == 0 )
 		return Curve_getNumPoints( self );
@@ -1598,6 +1640,8 @@ static int CurveSetAttr( BPy_Curve * self, char *name, PyObject * value )
 		error = Curve_setBevOb( self, valtuple );
 	else if( strcmp( name, "taperob" ) == 0 )
 		error = Curve_setTaperOb( self, valtuple );
+	else if( strcmp( name, "materials" ) == 0 )
+		error = Curve_setMaterials( self, value );
 
 	else {			/* Error */
 		Py_DECREF( valtuple );
@@ -1622,6 +1666,16 @@ static int CurveSetAttr( BPy_Curve * self, char *name, PyObject * value )
 }
 
 
+/*****************************************************************************/
+/* Function:    CurveCopmpare		                                         */
+/* Description: This compares 2 curve python types, == or != only.			 */
+/*****************************************************************************/
+static int CurveCopmpare( BPy_Curve * a, BPy_Curve * b )
+{
+	return ( a->curve == b->curve ) ? 0 : -1;
+}
+
+
 /*****************************************************************************/
 /* Function:    CurveRepr                                                    */
 /* Description: This is a callback function for the BPy_Curve type. It       */
