@@ -1,5 +1,5 @@
 /* 
- * $Id: Draw.c,v 1.52 2006/06/27 17:51:11 ianwill Exp $
+ * $Id: Draw.c,v 1.58 2007/01/06 04:10:51 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -68,6 +68,8 @@ current image frame, some images change frame if they are a sequence */
 #define EXPP_BUTTON_EVENTS_MIN 0
 #define EXPP_BUTTON_EVENTS_MAX 15382 /* 16384 - 1 - OFFSET */
 
+#define ButtonObject_Check(v) ((v)->ob_type == &Button_Type)
+
 /* pointer to main dictionary defined in Blender.c */
 extern PyObject *g_blenderdict;
 
@@ -78,6 +80,7 @@ int EXPP_disable_force_draw = 0;
 static void Button_dealloc( PyObject * self );
 static PyObject *Button_getattr( PyObject * self, char *name );
 static PyObject *Button_repr( PyObject * self );
+static PyObject *Button_richcmpr(PyObject *objectA, PyObject *objectB, int comparison_type);
 static int Button_setattr( PyObject * self, char *name, PyObject * v );
 
 static Button *newbutton( void );
@@ -102,15 +105,18 @@ static PyObject *Method_Toggle( PyObject * self, PyObject * args );
 static PyObject *Method_Slider( PyObject * self, PyObject * args );
 static PyObject *Method_Scrollbar( PyObject * self, PyObject * args );
 static PyObject *Method_ColorPicker( PyObject * self, PyObject * args );
+static PyObject *Method_Normal( PyObject * self, PyObject * args );
 static PyObject *Method_Number( PyObject * self, PyObject * args );
 static PyObject *Method_String( PyObject * self, PyObject * args );
 static PyObject *Method_GetStringWidth( PyObject * self, PyObject * args );
 static PyObject *Method_Text( PyObject * self, PyObject * args );
 static PyObject *Method_PupMenu( PyObject * self, PyObject * args );
-/* next three by Campbell: */
+/* next Five by Campbell: */
 static PyObject *Method_PupIntInput( PyObject * self, PyObject * args );
 static PyObject *Method_PupFloatInput( PyObject * self, PyObject * args );
 static PyObject *Method_PupStrInput( PyObject * self, PyObject * args );
+static PyObject *Method_BeginAlign( PyObject * self, PyObject * args  );
+static PyObject *Method_EndAlign( PyObject * self, PyObject * args  );
 /* next by Jonathan Merritt (lancelet): */
 static PyObject *Method_Image( PyObject * self, PyObject * args);
 /* CLEVER NUMBUT */
@@ -158,6 +164,12 @@ static char Method_Button_doc[] =
 [tooltip=] The button's tooltip\n\n\
 This function can be called as Button() or PushButton().";
 
+static char Method_BeginAlign_doc[] =
+	"Buttons after this function will draw aligned (button layout only)";
+
+static char Method_EndAlign_doc[] =
+	"Use after BeginAlign() to stop aligning the buttons (button layout only).";
+
 static char Method_Menu_doc[] =
 	"(name, event, x, y, width, height, default, [tooltip]) - Create a new Menu \
 button\n\n\
@@ -220,6 +232,15 @@ Color picker button\n\n\
 (initial) 3-Float tuple of the color (values between 0 and 1)\
 [tooltip=] The button's tooltip";
 
+static char Method_Normal_doc[] = 
+	"(event, x, y, width, height, initial, [tooltip]) - Create a new Button \
+Normal button (a sphere that you can roll to change the normal)\n\n\
+(event) The event number to pass to the button event function when the color changes\n\
+(x, y) The lower left coordinate of the button\n\
+(width, height) The button width and height - non square will gave odd results\n\
+(initial) 3-Float tuple of the normal vector (values between -1 and 1)\
+[tooltip=] The button's tooltip";
+
 static char Method_Number_doc[] =
 	"(name, event, x, y, width, height, initial, min, max, [tooltip]) - Create a \
 new Number button\n\n\
@@ -333,6 +354,7 @@ static struct PyMethodDef Draw_methods[] = {
 	MethodDef( Slider ),
 	MethodDef( Scrollbar ),
 	MethodDef( ColorPicker ),
+	MethodDef( Normal ),
 	MethodDef( Number ),
 	MethodDef( String ),
 	MethodDef( GetStringWidth ),
@@ -348,6 +370,8 @@ static struct PyMethodDef Draw_methods[] = {
 	MethodDef( Draw ),
 	MethodDef( Register ),
 	{"PushButton", Method_Button, METH_VARARGS, Method_Button_doc},
+	MethodDef( BeginAlign ),
+	MethodDef( EndAlign),
 	{NULL, NULL, 0, NULL}
 };
 
@@ -360,10 +384,72 @@ PyTypeObject Button_Type = {
 	( printfunc ) 0,	/*tp_print */
 	( getattrfunc ) Button_getattr,	/*tp_getattr */
 	( setattrfunc ) Button_setattr,	/*tp_setattr */
-	( cmpfunc ) 0,		/*tp_cmp */
+	NULL,		/*tp_cmp */
 	( reprfunc ) Button_repr,	/*tp_repr */
-	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
-	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+
+	/* Method suites for standard classes */
+
+	NULL,                       /* PyNumberMethods *tp_as_number; */
+	NULL,                       /* PySequenceMethods *tp_as_sequence; */
+	NULL,                       /* PyMappingMethods *tp_as_mapping; */
+
+	/* More standard operations (here for binary compatibility) */
+
+	NULL,                       /* hashfunc tp_hash; */
+	NULL,                       /* ternaryfunc tp_call; */
+	NULL,                       /* reprfunc tp_str; */
+	NULL,                       /* getattrofunc tp_getattro; */
+	NULL,                       /* setattrofunc tp_setattro; */
+
+	/* Functions to access object as input/output buffer */
+	NULL,                       /* PyBufferProcs *tp_as_buffer; */
+
+  /*** Flags to define presence of optional/expanded features ***/
+	Py_TPFLAGS_DEFAULT,         /* long tp_flags; */
+
+	NULL,                       /*  char *tp_doc;  Documentation string */
+  /*** Assigned meaning in release 2.0 ***/
+	/* call function for all accessible objects */
+	NULL,                       /* traverseproc tp_traverse; */
+
+	/* delete references to contained objects */
+	NULL,                       /* inquiry tp_clear; */
+
+  /***  Assigned meaning in release 2.1 ***/
+  /*** rich comparisons ***/
+	(richcmpfunc)Button_richcmpr,                       /* richcmpfunc tp_richcompare; */
+
+  /***  weak reference enabler ***/
+	0,                          /* long tp_weaklistoffset; */
+
+  /*** Added in release 2.2 ***/
+	/*   Iterators */
+	NULL,                       /* getiterfunc tp_iter; */
+	NULL,                       /* iternextfunc tp_iternext; */
+
+  /*** Attribute descriptor and subclassing stuff ***/
+	NULL,           /* struct PyMethodDef *tp_methods; */
+	NULL,                       /* struct PyMemberDef *tp_members; */
+	NULL,         /* struct PyGetSetDef *tp_getset; */
+	NULL,                       /* struct _typeobject *tp_base; */
+	NULL,                       /* PyObject *tp_dict; */
+	NULL,                       /* descrgetfunc tp_descr_get; */
+	NULL,                       /* descrsetfunc tp_descr_set; */
+	0,                          /* long tp_dictoffset; */
+	NULL,                       /* initproc tp_init; */
+	NULL,                       /* allocfunc tp_alloc; */
+	NULL,                       /* newfunc tp_new; */
+	/*  Low-level free-memory routine */
+	NULL,                       /* freefunc tp_free;  */
+	/* For PyObject_IS_GC */
+	NULL,                       /* inquiry tp_is_gc;  */
+	NULL,                       /* PyObject *tp_bases; */
+	/* method resolution order */
+	NULL,                       /* PyObject *tp_mro;  */
+	NULL,                       /* PyObject *tp_cache; */
+	NULL,                       /* PyObject *tp_subclasses; */
+	NULL,                       /* PyObject *tp_weaklist; */
+	NULL
 };
 
 static void Button_dealloc( PyObject * self )
@@ -384,11 +470,11 @@ static PyObject *Button_getattr( PyObject * self, char *name )
 	
 	if( strcmp( name, "val" ) == 0 ) {
 		if( but->type == BINT_TYPE )
-			return Py_BuildValue( "i", but->val.asint );
+			return PyInt_FromLong( but->val.asint );
 		else if( but->type == BFLOAT_TYPE )
-			return Py_BuildValue( "f", but->val.asfloat );
+			return PyFloat_FromDouble( but->val.asfloat );
 		else if( but->type == BSTRING_TYPE )
-			return Py_BuildValue( "s", but->val.asstr );
+			return PyString_FromString( but->val.asstr );
 		else if( but->type == BVECTOR_TYPE )
 			return Py_BuildValue( "fff", but->val.asvec[0], but->val.asvec[1], but->val.asvec[2] );
 	}
@@ -469,6 +555,16 @@ static PyObject *Button_repr( PyObject * self )
 	return PyObject_Repr( Button_getattr( self, "val" ) );
 }
 
+static PyObject *Button_richcmpr(PyObject *objectA, PyObject *objectB, int comparison_type)
+{
+	if (ButtonObject_Check(objectA))
+		objectA = Button_getattr( objectA, "val" );
+	if (ButtonObject_Check(objectB))
+		objectB = Button_getattr( objectB, "val" );
+	return PyObject_RichCompare(objectA, objectB, comparison_type);
+}
+
+
 static Button *newbutton( void )
 {
 	Button *but = NULL;
@@ -856,6 +952,28 @@ static PyObject *Method_Button( PyObject * self, PyObject * args )
 	return EXPP_incr_ret( Py_None );
 }
 
+
+
+static PyObject *Method_BeginAlign( PyObject * self, PyObject * args )
+{
+	uiBlock *block = Get_uiBlock(  );
+	
+	if (block)
+		uiBlockBeginAlign(block);
+	
+	return EXPP_incr_ret( Py_None );
+}
+
+static PyObject *Method_EndAlign( PyObject * self, PyObject * args )
+{
+	uiBlock *block = Get_uiBlock(  );
+	
+	if (block)
+		uiBlockEndAlign(block);
+	
+	return EXPP_incr_ret( Py_None );
+}
+
 static PyObject *Method_Menu( PyObject * self, PyObject * args )
 {
 	uiBlock *block;
@@ -1129,6 +1247,49 @@ static PyObject *Method_ColorPicker( PyObject * self, PyObject * args )
 }
 
 
+
+static PyObject *Method_Normal( PyObject * self, PyObject * args )
+{
+	char USAGE_ERROR[] = "expected a 3-float tuple of values between -1 and 1";
+	Button *but;
+	PyObject *inio;
+	uiBlock *block;
+	char *tip = NULL;
+	float nor[3];
+	int event;
+	short x, y, w, h;
+	
+	if( !PyArg_ParseTuple( args, "ihhhhO!|s", &event,
+			       &x, &y, &w, &h, &PyTuple_Type, &inio, &tip ) )
+ 		return EXPP_ReturnPyObjError( PyExc_TypeError,
+					      "expected five ints, one tuple and optionally a string as arguments." );
+ 
+ 	if (check_button_event(&event) == -1)
+		return EXPP_ReturnPyObjError( PyExc_ValueError,
+ 			"button event argument must be in the range [0, 16382]");
+ 
+	if ( !PyArg_ParseTuple( inio, "fff", nor, nor+1, nor+2 ) )
+		return EXPP_ReturnPyObjError( PyExc_ValueError, USAGE_ERROR);
+
+	if ( EXPP_check_sequence_consistency( inio, &PyFloat_Type ) != 1 )
+		return EXPP_ReturnPyObjError( PyExc_ValueError, USAGE_ERROR);
+ 
+	but = newbutton();
+ 
+	but->type = BVECTOR_TYPE;
+	but->val.asvec[0] = nor[0];
+	but->val.asvec[1] = nor[1];
+	but->val.asvec[2] = nor[2];
+	
+	block = Get_uiBlock(  );
+	if( block ) {
+		uiBut *ubut;
+		ubut = uiDefButF( block, BUT_NORMAL, event, "", x, y, w, h, but->val.asvec, 0.0f, 1.0f, 0, 0, tip);
+	}
+	
+ 	return ( PyObject * ) but;
+}
+
 static PyObject *Method_Number( PyObject * self, PyObject * args )
 {
 	uiBlock *block;
@@ -1391,7 +1552,7 @@ static PyObject *Method_PupStrInput( PyObject * self, PyObject * args )
 		return Py_None;
 	}
 
-	ret = Py_BuildValue( "s", tmp );
+	ret = PyString_FromString( tmp );
 
 	if( ret )
 		return ret;
@@ -1416,8 +1577,8 @@ static PyObject *Method_PupBlock( PyObject * self, PyObject * args )
 	if (len == 0)
 		return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a string and a non-empty sequence." );
 
-	if (len > 24) /* LIMIT DEFINED IN toolbox.c	*/
-		return EXPP_ReturnPyObjError( PyExc_ValueError, "sequence cannot have more than 24 elements" );
+	if (len > 120) /* LIMIT DEFINED IN toolbox.c	*/
+		return EXPP_ReturnPyObjError( PyExc_ValueError, "sequence cannot have more than 120 elements" );
 
 	for ( i=0 ; i<len ; i++ ) {
 		PyObject *pyMin = NULL, *pyMax = NULL;
@@ -1534,6 +1695,7 @@ static PyObject *Method_Image( PyObject * self, PyObject * args )
 	PyObject *pyObjImage;
 	BPy_Image *py_img;
 	Image *image;
+	ImBuf *ibuf;
 	float originX, originY;
 	float zoomX = 1.0, zoomY = 1.0;
 	int clipX = 0, clipY = 0, clipW = -1, clipH = -1;
@@ -1559,14 +1721,15 @@ static PyObject *Method_Image( PyObject * self, PyObject * args )
 	/* fetch a C Image pointer from the passed-in Python object */
 	py_img = ( BPy_Image * ) pyObjImage;
 	image = py_img->image;
-
-	/* load the image data if necessary */
-	if( !image->ibuf )      /* if no image data is available ... */
-		load_image( image, IB_rect, G.sce,  G.scene->r.cfra );	/* loading it */
-	if( !image->ibuf )      /* if failed to load the image */
+	ibuf = BKE_image_get_ibuf( image, NULL );
+		
+	if( !ibuf )      /* if failed to load the image */
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-			"couldn't load image data in Blender" );
-
+									  "couldn't load image data in Blender" );
+	if( !ibuf->rect )      /* no float yet */
+		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
+									  "Image has no byte rect" );
+	
 	/* Update the time tag of the image */
 	tag_image_time(image);
 
@@ -1575,12 +1738,12 @@ static PyObject *Method_Image( PyObject * self, PyObject * args )
 	 * the clipping is just checked against the bounds of the image.
 	 * if clipW or clipH are less than zero then they include as much of
 	 * the image as they can. */
-	clipX = EXPP_ClampInt( clipX, 0, image->ibuf->x );
-	clipY = EXPP_ClampInt( clipY, 0, image->ibuf->y );
-	if( ( clipW < 0 ) || ( clipW > ( image->ibuf->x - clipW ) ) )
-		clipW = image->ibuf->x - clipX;
-	if( ( clipH < 0 ) || ( clipH > ( image->ibuf->y - clipH ) ) )
-		clipH = image->ibuf->y - clipY;
+	clipX = EXPP_ClampInt( clipX, 0, ibuf->x );
+	clipY = EXPP_ClampInt( clipY, 0, ibuf->y );
+	if( ( clipW < 0 ) || ( clipW > ( ibuf->x - clipW ) ) )
+		clipW = ibuf->x - clipX;
+	if( ( clipH < 0 ) || ( clipH > ( ibuf->y - clipH ) ) )
+		clipH = ibuf->y - clipY;
 
 	/* -- we are "Go" to Draw! -- */
 
@@ -1626,13 +1789,13 @@ static PyObject *Method_Image( PyObject * self, PyObject * args )
 	/* set the width of the image (ROW_LENGTH), and the offset to the
 	 * clip origin within the image in x (SKIP_PIXELS) and 
 	 * y (SKIP_ROWS) */
-	glPixelStorei( GL_UNPACK_ROW_LENGTH,  image->ibuf->x );
+	glPixelStorei( GL_UNPACK_ROW_LENGTH,  ibuf->x );
 	glPixelStorei( GL_UNPACK_SKIP_PIXELS, clipX );
 	glPixelStorei( GL_UNPACK_SKIP_ROWS,   clipY );
 
 	/* draw the image */
 	glDrawPixels( clipW, clipH, GL_RGBA, GL_UNSIGNED_BYTE, 
-		image->ibuf->rect );
+		ibuf->rect );
 
 	/* restore the defaults for some parameters (we could also use a
 	 * glPushClientAttrib() and glPopClientAttrib() pair). */
