@@ -1,3 +1,35 @@
+/*
+ * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version. The Blender
+ * Foundation also sells licenses for use in proprietary software under
+ * the Blender License.  See http://www.blender.org/BL/ for information
+ * about this.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
+ * All rights reserved.
+ *
+ * This is a new part of Blender.
+ *
+ * Contributor(s): Campbell Barton
+ *
+ * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ */
+
+#include "MEM_guardedalloc.h"
+
 #include "Group.h" /* This must come first */
 
 #include "DNA_group_types.h"
@@ -16,6 +48,9 @@
 #include "Object.h"
 #include "gen_utils.h"
 
+/* checks for the group being removed */
+#define GROUP_DEL_CHECK_PY(bpy_group) if (!(bpy_group->group)) return ( EXPP_ReturnPyObjError( PyExc_RuntimeError, "Group has been removed" ) )
+#define GROUP_DEL_CHECK_INT(bpy_group) if (!(bpy_group->group)) return ( EXPP_ReturnIntError( PyExc_RuntimeError, "Group has been removed" ) )
 
 /*****************************************************************************/
 /* Python API function prototypes for the Blender module.		 */
@@ -24,57 +59,79 @@ static PyObject *M_Group_New( PyObject * self, PyObject * args );
 PyObject *M_Group_Get( PyObject * self, PyObject * args );
 PyObject *M_Group_Unlink( PyObject * self, PyObject * args );
 
-/*****************************************************************************/
-/* The following string definitions are used for documentation strings.	 */
-/* In Python these will be written to the console when doing a		 */
-/* Blender.Group.__doc__						 */
-/*****************************************************************************/
-char M_Group_doc[] = "The Blender Group module\n\n\
-This module provides access to **Group Data** in Blender.\n";
-
-char M_Group_New_doc[] =
-	"(name) Add a new empty group";
-
-char M_Group_Get_doc[] =
-	"(name) - return the group with the name 'name', returns None if not\
-	found.\n\
-	If 'name' is not specified, it returns a list of all groups.";
-	
-char M_Group_Unlink_doc[] =
-"(group) - Unlink (delete) this group from Blender.";
+/* internal */
+static PyObject *GroupObSeq_CreatePyObject( BPy_Group *self, GroupObject *iter );
 
 /*****************************************************************************/
 /* Python method structure definition for Blender.Object module:	 */
 /*****************************************************************************/
 struct PyMethodDef M_Group_methods[] = {
 	{"New", ( PyCFunction ) M_Group_New, METH_VARARGS,
-	 M_Group_New_doc},
+	 "(name) Add a new empty group"},
 	{"Get", ( PyCFunction ) M_Group_Get, METH_VARARGS,
-	 M_Group_Get_doc},
+"(name) - return the group with the name 'name',\
+returns None if notfound.\nIf 'name' is not specified, it returns a list of all groups."},
 	{"Unlink", ( PyCFunction ) M_Group_Unlink, METH_VARARGS,
-	 M_Group_Unlink_doc},
+	 "(group) - Unlink (delete) this group from Blender."},
 	{NULL, NULL, 0, NULL}
 };
 
 /*****************************************************************************/
 /* Python BPy_Group methods table:					   */
 /*****************************************************************************/
+static PyObject *BPy_Group_copy( BPy_Group * self );
+
 static PyMethodDef BPy_Group_methods[] = {
 	/* name, method, flags, doc */
+	{"__copy__", ( PyCFunction ) BPy_Group_copy, METH_VARARGS,
+	 "() - Return a copy of the group containing the same objects."},
 	{NULL, NULL, 0, NULL}
 };
 
 
+static PyObject *BPy_Group_copy( BPy_Group * self )
+{
+	BPy_Group *py_group;	/* for Group Data object wrapper in Python */
+	struct Group *bl_group;
+	GroupObject *group_ob, *group_ob_new; /* Group object, copied and added to the groups */
+	
+	GROUP_DEL_CHECK_PY(self);
+	
+	bl_group= add_group();
+	
+	if( bl_group )		/* now create the wrapper grp in Python */
+		py_group = ( BPy_Group * ) Group_CreatePyObject( bl_group );
+	else
+		return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
+						"couldn't create Group Data in Blender" ) );
+	
+	rename_id( &bl_group->id, self->group->id.name + 2 );
+	
+	bl_group->id.us = 1;
+	
+	/* Now add the objects to the group */
+	group_ob= self->group->gobject.first;
+	while(group_ob) {
+		/* save time by not using */
+		group_ob_new= MEM_callocN(sizeof(GroupObject), "groupobject");
+		group_ob_new->ob= group_ob->ob;
+		BLI_addtail( &bl_group->gobject, group_ob_new);
+		group_ob= group_ob->next;
+	}
+	
+	return ( PyObject * ) py_group;
+	
+}
+
+
 /************************************************************************
  *
  * Python BPy_Object attributes
  *
  ************************************************************************/
-static PyObject *M_Group_getObjects( BPy_Group * self )
+static PyObject *Group_getObjects( BPy_Group * self )
 {
-	BPy_MGroupObSeq *seq = PyObject_NEW( BPy_MGroupObSeq, &MGroupObSeq_Type);
-	seq->bpygroup = self; Py_INCREF(self);
-	return (PyObject *)seq;
+	return GroupObSeq_CreatePyObject(self, NULL);
 }
 
 
@@ -92,13 +149,15 @@ static void add_to_group_wraper(Group *group, Object *ob) {
 }
 
 /* only for internal use Blender.Group.Get("MyGroup").objects= []*/
-static int M_Group_setObjects( BPy_Group * self, PyObject * args )
+static int Group_setObjects( BPy_Group * self, PyObject * args )
 {
 	int i, list_size;
 	Group *group;
 	Object *blen_ob;
 	group= self->group;
 	
+	GROUP_DEL_CHECK_INT(self);
+	
 	if( PyList_Check( args ) ) {
 		if( EXPP_check_sequence_consistency( args, &Object_Type ) != 1)
 			return ( EXPP_ReturnIntError( PyExc_TypeError, 
@@ -111,11 +170,6 @@ static int M_Group_setObjects( BPy_Group * self, PyObject * args )
 			blen_ob= ((BPy_Object *)PyList_GET_ITEM( args, i ))->object;
 			add_to_group_wraper(group, blen_ob);
 		}
-	/*
-	} else if( args->ob_type == &MGroupObSeq_Type ) {
-	*/
-		/* todo, handle sequences here */
-	
 	} else if (PyIter_Check(args)) {
 		PyObject *iterator = PyObject_GetIter(args);
 		PyObject *item;
@@ -163,31 +217,24 @@ static int Group_compare( BPy_Group * a, BPy_Group * b );
 static int Group_setName( BPy_Group * self, PyObject * value )
 {
 	char *name = NULL;
-	char buf[21];
 	
-	if( !(self->group) )
-		return EXPP_ReturnIntError( PyExc_RuntimeError,
-					      "Blender Group was deleted!" );
+	GROUP_DEL_CHECK_INT(self);
 	
 	name = PyString_AsString ( value );
 	if( !name )
 		return EXPP_ReturnIntError( PyExc_TypeError,
 					      "expected string argument" );
 
-	PyOS_snprintf( buf, sizeof( buf ), "%s", name );
-
-	rename_id( &self->group->id, buf );
+	rename_id( &self->group->id, name );
 
 	return 0;
 }
 
 
-static PyObject *Group_getName( BPy_Group * self, PyObject * args )
+static PyObject *Group_getName( BPy_Group * self )
 {
 	PyObject *attr;
-	if( !(self->group) )
-		return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					      "Blender Group was deleted!" ) );
+	GROUP_DEL_CHECK_PY(self);
 	
 	attr = PyString_FromString( self->group->id.name + 2 );
 
@@ -200,9 +247,57 @@ static PyObject *Group_getName( BPy_Group * self, PyObject * args )
 
 static PyObject *Group_getUsers( BPy_Group * self )
 {
+	GROUP_DEL_CHECK_PY(self);
 	return PyInt_FromLong( self->group->id.us );
 }
 
+static PyObject *Group_getFakeUser( BPy_Group * self )
+{
+	GROUP_DEL_CHECK_PY(self);
+	if (self->group->id.flag & LIB_FAKEUSER)
+		Py_RETURN_TRUE;
+	else
+		Py_RETURN_FALSE;
+}
+
+static int Group_setFakeUser( BPy_Group * self, PyObject * value )
+{
+	GROUP_DEL_CHECK_INT(self);
+	return SetIdFakeUser(&self->group->id, value);
+}
+
+
+
+/*****************************************************************************/
+/* Python BPy_Group methods:                                                  */
+/*****************************************************************************/
+static int Group_setLayers( BPy_Group * self, PyObject * value )
+{
+	unsigned int laymask = 0;
+	
+	GROUP_DEL_CHECK_INT(self);
+	
+	if( !PyInt_CheckExact( value ) )
+		return EXPP_ReturnIntError( PyExc_TypeError,
+			"expected an integer (bitmask) as argument" );
+	
+	laymask = ( unsigned int )PyInt_AS_LONG( value );
+	
+	if( laymask <= 0 )
+		return EXPP_ReturnIntError( PyExc_ValueError,
+					      "layer value cannot be zero or below" );
+	
+	self->group->layer= laymask & ((1<<20) - 1);
+	
+	return 0;
+}
+
+static PyObject *Group_getLayers( BPy_Group * self )
+{
+	return PyInt_FromLong( self->group->layer );
+}
+
+
 /*****************************************************************************/
 /* Python attributes get/set structure:                                      */
 /*****************************************************************************/
@@ -215,8 +310,16 @@ static PyGetSetDef BPy_Group_getseters[] = {
 	 (getter)Group_getUsers, (setter)NULL,
 	 "Number of group users",
 	 NULL},
+	{"fakeUser",
+	 (getter)Group_getFakeUser, (setter)Group_setFakeUser,
+	 "Groups fake user state",
+	 NULL},
+	{"layers",
+	 (getter)Group_getLayers, (setter)Group_setLayers,
+	 "Number of group users",
+	 NULL},
 	{"objects",
-	 (getter)M_Group_getObjects, (setter)M_Group_setObjects,
+	 (getter)Group_getObjects, (setter)Group_setObjects,
 	 "objects in this group",
 	 NULL},
 	{NULL,NULL,NULL,NULL,NULL}  /* Sentinel */
@@ -340,8 +443,7 @@ PyObject *M_Group_New( PyObject * self, PyObject * args )
 		rename_id( &bl_group->id, buf );
 	}
 	
-	/* user count be incremented in Group_CreatePyObject */
-	bl_group->id.us = 0;
+	bl_group->id.us = 1;
 	
 	return ( PyObject * ) py_group;
 }
@@ -429,19 +531,17 @@ PyObject *M_Group_Unlink( PyObject * self, PyObject * args )
 						"expected a group" ) );
 	
 	pygrp= (BPy_Group *)pyob;
-	group= pygrp->group;
 	
-	if( !group )
-		return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					      "Blender Group was deleted!" ) );
+	GROUP_DEL_CHECK_PY(pygrp);
+	
+	group= pygrp->group;
 	
 	pygrp->group= NULL;
 	free_group(group);
 	unlink_group(group);
 	group->id.us= 0;
 	free_libblock( &G.main->group, group );
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 
@@ -453,14 +553,15 @@ PyObject *Group_Init( void )
 	PyObject *submodule;
 	if( PyType_Ready( &Group_Type ) < 0 )
 		return NULL;
-	if( PyType_Ready( &MGroupObSeq_Type ) < 0 )
+	if( PyType_Ready( &GroupObSeq_Type ) < 0 )
 		return NULL;
 	
 	submodule = Py_InitModule3( "Blender.Group", M_Group_methods,
-				 M_Group_doc );
+				 "The Blender Group module\n\n\
+This module provides access to **Group Data** in Blender.\n" );
 
-	//Add SUBMODULES to the module
-	//PyDict_SetItemString(dict, "Constraint", Constraint_Init()); //creates a *new* module
+	/*Add SUBMODULES to the module*/
+	/*PyDict_SetItemString(dict, "Constraint", Constraint_Init()); //creates a *new* module*/
 	return submodule;
 }
 
@@ -475,7 +576,7 @@ PyObject *Group_CreatePyObject( struct Group * grp )
 	BPy_Group *pygrp;
 
 	if( !grp )
-		return EXPP_incr_ret( Py_None );
+		Py_RETURN_NONE;
 
 	pygrp =
 		( BPy_Group * ) PyObject_NEW( BPy_Group, &Group_Type );
@@ -565,6 +666,9 @@ static int Group_compare( BPy_Group * a, BPy_Group * b )
 /*****************************************************************************/
 static PyObject *Group_repr( BPy_Group * self )
 {
+	if (!self->group)
+		return PyString_FromString( "[Group - Removed]" );
+	
 	return PyString_FromFormat( "[Group \"%s\"]",
 				    self->group->id.name + 2 );
 }
@@ -576,15 +680,37 @@ static PyObject *Group_repr( BPy_Group * self )
  *
  ************************************************************************/
 /*
- * create a thin MGroupOb object
+ * create a thin GroupOb object
  */
 
-static PyObject *MGroupObSeq_CreatePyObject( Group *group, int i )
+static PyObject *GroupObSeq_CreatePyObject( BPy_Group *self, GroupObject *iter )
 {
+	BPy_GroupObSeq *seq = PyObject_NEW( BPy_GroupObSeq, &GroupObSeq_Type);
+	seq->bpygroup = self; Py_INCREF(self);
+	seq->iter= iter;
+	return (PyObject *)seq;
+}
+
+
+static int GroupObSeq_len( BPy_GroupObSeq * self )
+{
+	GROUP_DEL_CHECK_INT(self->bpygroup);
+	return BLI_countlist( &( self->bpygroup->group->gobject ) );
+}
+
+/*
+ * retrive a single GroupOb from somewhere in the GroupObex list
+ */
+
+static PyObject *GroupObSeq_item( BPy_GroupObSeq * self, int i )
+{
+	Group *group= self->bpygroup->group;
 	int index=0;
 	PyObject *bpy_obj;
 	GroupObject *gob;
 	
+	GROUP_DEL_CHECK_PY(self->bpygroup);
+	
 	for (gob= group->gobject.first; gob && i!=index; gob= gob->next, index++) {}
 	
 	if (!(gob))
@@ -598,28 +724,14 @@ static PyObject *MGroupObSeq_CreatePyObject( Group *group, int i )
 				"PyObject_New() failed" );
 
 	return (PyObject *)bpy_obj;
+	
 }
 
-
-static int MGroupObSeq_len( BPy_MGroupObSeq * self )
-{
-	return BLI_countlist( &( self->bpygroup->group->gobject ) );
-}
-
-/*
- * retrive a single MGroupOb from somewhere in the GroupObex list
- */
-
-static PyObject *MGroupObSeq_item( BPy_MGroupObSeq * self, int i )
-{
-	return MGroupObSeq_CreatePyObject( self->bpygroup->group, i );
-}
-
-static PySequenceMethods MGroupObSeq_as_sequence = {
-	( inquiry ) MGroupObSeq_len,	/* sq_length */
+static PySequenceMethods GroupObSeq_as_sequence = {
+	( inquiry ) GroupObSeq_len,	/* sq_length */
 	( binaryfunc ) 0,	/* sq_concat */
 	( intargfunc ) 0,	/* sq_repeat */
-	( intargfunc ) MGroupObSeq_item,	/* sq_item */
+	( intargfunc ) GroupObSeq_item,	/* sq_item */
 	( intintargfunc ) 0,	/* sq_slice */
 	( intobjargproc ) 0,	/* sq_ass_item */
 	( intintobjargproc ) 0,	/* sq_ass_slice */
@@ -628,7 +740,7 @@ static PySequenceMethods MGroupObSeq_as_sequence = {
 
 /************************************************************************
  *
- * Python MGroupObSeq_Type iterator (iterates over GroupObjects)
+ * Python GroupObSeq_Type iterator (iterates over GroupObjects)
  *
  ************************************************************************/
 
@@ -636,22 +748,30 @@ static PySequenceMethods MGroupObSeq_as_sequence = {
  * Initialize the interator index
  */
 
-static PyObject *MGroupObSeq_getIter( BPy_MGroupObSeq * self )
+static PyObject *GroupObSeq_getIter( BPy_GroupObSeq * self )
 {
-	self->iter = self->bpygroup->group->gobject.first;
-	return EXPP_incr_ret ( (PyObject *) self );
+	GROUP_DEL_CHECK_PY(self->bpygroup);
+	
+	if (!self->iter) {
+		self->iter = self->bpygroup->group->gobject.first;
+		return EXPP_incr_ret ( (PyObject *) self );
+	} else {
+		return GroupObSeq_CreatePyObject(self->bpygroup, self->bpygroup->group->gobject.first);
+	}
 }
 
 /*
- * Return next MGroupOb.
+ * Return next GroupOb.
  */
 
-static PyObject *MGroupObSeq_nextIter( BPy_MGroupObSeq * self )
+static PyObject *GroupObSeq_nextIter( BPy_GroupObSeq * self )
 {
 	PyObject *object;
-	if( !(self->iter) ||  !(self->bpygroup->group) )
+	if( !(self->iter) ||  !(self->bpygroup->group) ) {
+		self->iter = NULL; /* so we can add objects again */
 		return EXPP_ReturnPyObjError( PyExc_StopIteration,
 				"iterator at end" );
+	}
 	
 	object= Object_CreatePyObject( self->iter->ob ); 
 	self->iter= self->iter->next;
@@ -659,35 +779,44 @@ static PyObject *MGroupObSeq_nextIter( BPy_MGroupObSeq * self )
 }
 
 
-static PyObject *MGroupObSeq_append( BPy_MGroupObSeq * self, PyObject *args )
+static PyObject *GroupObSeq_link( BPy_GroupObSeq * self, PyObject *args )
 {
 	PyObject *pyobj;
 	Object *blen_ob;
-	Base *base= NULL;
+	
+	GROUP_DEL_CHECK_PY(self->bpygroup);
+	
 	if( !PyArg_ParseTuple( args, "O!", &Object_Type, &pyobj ) )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 				"expected a python object as an argument" ) );
 	
-	blen_ob = ( ( BPy_Object * ) pyobj )->object;
+	/*
+	if (self->iter != NULL)
+		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
+					      "Cannot modify group objects while iterating" );
+	*/
 	
-	base= object_in_scene(blen_ob, G.scene);
+	blen_ob = ( ( BPy_Object * ) pyobj )->object;
 	
 	add_to_group_wraper(self->bpygroup->group, blen_ob); /* this checks so as not to add the object into the group twice*/
 	
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
+}
+
+static PyObject *GroupObSeq_append( BPy_GroupObSeq * self, PyObject *args )
+{
+	printf("WARNING: grp.objects.append() has been deprecated, use grp.objects.link() instead.\nBlender releases after 2.43 will not support .append()");
+	return GroupObSeq_link(self, args);
 }
 
 
-
-static PyObject *MGroupObSeq_remove( BPy_MGroupObSeq * self, PyObject *args )
+static PyObject *GroupObSeq_unlink( BPy_GroupObSeq * self, PyObject *args )
 {
 	PyObject *pyobj;
 	Object *blen_ob;
 	Base *base= NULL;
 	
-	if( !(self->bpygroup->group) )
-		return (EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					      "Blender Group was deleted!" ));
+	GROUP_DEL_CHECK_PY(self->bpygroup);
 	
 	if( !PyArg_ParseTuple( args, "O!", &Object_Type, &pyobj ) )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
@@ -706,44 +835,57 @@ static PyObject *MGroupObSeq_remove( BPy_MGroupObSeq * self, PyObject *args )
 		if (base)
 			base->flag &= ~OB_FROMGROUP;
 	}
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
+}
+
+static PyObject *GroupObSeq_remove( BPy_GroupObSeq * self, PyObject *args )
+{
+	printf("WARNING: grp.objects.remove() has been deprecated, use grp.objects.unlink() instead.\nBlender releases after 2.43 will not support .remove()");
+	return GroupObSeq_unlink(self, args);
 }
 
 
-static struct PyMethodDef BPy_MGroupObSeq_methods[] = {
-	{"append", (PyCFunction)MGroupObSeq_append, METH_VARARGS,
-		"add object to group"},
-	{"remove", (PyCFunction)MGroupObSeq_remove, METH_VARARGS,
-		"remove object from group"},
-	{NULL, NULL, 0, NULL}
+static struct PyMethodDef BPy_GroupObSeq_methods[] = {
+	{"link", (PyCFunction)GroupObSeq_link, METH_VARARGS,
+		"make the object a part of this group"},
+	{"unlink", (PyCFunction)GroupObSeq_unlink, METH_VARARGS,
+		"unlink an object from this group"},
+		
+	/* Deprecated! use the above functions */
+	{"append", (PyCFunction)GroupObSeq_append, METH_VARARGS,
+		"make the object a part of this group - deprecated, use link"},
+	{"remove", (PyCFunction)GroupObSeq_remove, METH_VARARGS,
+		"remove an object from this group - deprecated, use unlink"},
+		
+	{NULL, NULL, 0, NULL}	
 };
 
 /************************************************************************
  *
- * Python MGroupObSeq_Type standard operations
+ * Python GroupObSeq_Type standard operations
  *
  ************************************************************************/
 
-static void MGroupObSeq_dealloc( BPy_MGroupObSeq * self )
+static void GroupObSeq_dealloc( BPy_GroupObSeq * self )
 {
 	Py_DECREF(self->bpygroup);
 	PyObject_DEL( self );
 }
 
 /*****************************************************************************/
-/* Python MGroupObSeq_Type structure definition:                               */
+/* Python GroupObSeq_Type structure definition:                               */
 /*****************************************************************************/
-PyTypeObject MGroupObSeq_Type = {
+PyTypeObject GroupObSeq_Type = {
 	PyObject_HEAD_INIT( NULL )  /* required py macro */
 	0,                          /* ob_size */
 	/*  For printing, in format "<module>.<name>" */
-	"Blender MGroupObSeq",           /* char *tp_name; */
-	sizeof( BPy_MGroupObSeq ),       /* int tp_basicsize; */
+	"Blender GroupObSeq",           /* char *tp_name; */
+	sizeof( BPy_GroupObSeq ),       /* int tp_basicsize; */
 	0,                          /* tp_itemsize;  For allocation */
 
 	/* Methods to implement standard operations */
 
-	( destructor ) MGroupObSeq_dealloc,/* destructor tp_dealloc; */
+	( destructor ) GroupObSeq_dealloc,/* destructor tp_dealloc; */
 	NULL,                       /* printfunc tp_print; */
 	NULL,                       /* getattrfunc tp_getattr; */
 	NULL,                       /* setattrfunc tp_setattr; */
@@ -753,7 +895,7 @@ PyTypeObject MGroupObSeq_Type = {
 	/* Method suites for standard classes */
 
 	NULL,                       /* PyNumberMethods *tp_as_number; */
-	&MGroupObSeq_as_sequence,	    /* PySequenceMethods *tp_as_sequence; */
+	&GroupObSeq_as_sequence,	    /* PySequenceMethods *tp_as_sequence; */
 	NULL,                       /* PyMappingMethods *tp_as_mapping; */
 
 	/* More standard operations (here for binary compatibility) */
@@ -787,11 +929,11 @@ PyTypeObject MGroupObSeq_Type = {
 
   /*** Added in release 2.2 ***/
 	/*   Iterators */
-	( getiterfunc) MGroupObSeq_getIter, /* getiterfunc tp_iter; */
-	( iternextfunc ) MGroupObSeq_nextIter, /* iternextfunc tp_iternext; */
+	( getiterfunc) GroupObSeq_getIter, /* getiterfunc tp_iter; */
+	( iternextfunc ) GroupObSeq_nextIter, /* iternextfunc tp_iternext; */
 
   /*** Attribute descriptor and subclassing stuff ***/
-	BPy_MGroupObSeq_methods,       /* struct PyMethodDef *tp_methods; */
+	BPy_GroupObSeq_methods,       /* struct PyMethodDef *tp_methods; */
 	NULL,                       /* struct PyMemberDef *tp_members; */
 	NULL,                       /* struct PyGetSetDef *tp_getset; */
 	NULL,                       /* struct _typeobject *tp_base; */
