@@ -1,5 +1,5 @@
 /* 
- * $Id: Image.c,v 1.49 2006/07/18 21:02:15 khughes Exp $
+ * $Id: Image.c,v 1.62 2006/12/27 05:04:17 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -37,6 +37,7 @@
 #include "BKE_global.h"
 #include "BKE_library.h"
 #include "BKE_image.h"
+#include "BKE_idprop.h"
 #include "BIF_drawimage.h"
 #include "BLI_blenlib.h"
 #include "DNA_space_types.h"	/* FILE_MAXDIR = 160 */
@@ -47,6 +48,7 @@
 #include "DNA_packedFile_types.h"
 #include "BKE_icons.h"
 #include "IMB_imbuf.h"
+#include "IDProp.h"
 
 /* used so we can get G.scene->r.cfra for getting the
 current image frame, some images change frame if they are a sequence */
@@ -105,6 +107,7 @@ static PyObject *Image_getMinXY( BPy_Image * self );
 static PyObject *Image_save( BPy_Image * self );
 static PyObject *Image_unpack( BPy_Image * self, PyObject * args );
 static PyObject *Image_pack( BPy_Image * self );
+static PyObject *Image_makeCurrent( BPy_Image * self );
 
 
 /*****************************************************************************/
@@ -171,7 +174,9 @@ static PyMethodDef BPy_Image_methods[] = {
 	{"unpack", ( PyCFunction ) Image_unpack, METH_VARARGS,
 	 "(int) - Unpack image. Uses the values defined in Blender.UnpackModes."},
 	{"pack", ( PyCFunction ) Image_pack, METH_NOARGS,
-	 "() Pack the image"},
+	 "() - Pack the image"},
+	{"makeCurrent", ( PyCFunction ) Image_makeCurrent, METH_NOARGS,
+	 "() - Make this the currently displayed image"},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -227,13 +232,15 @@ static PyObject *M_Image_New( PyObject * self, PyObject * args)
 	if (width > 5000 || height > 5000 || width < 1 || height < 1)
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 					"Image width and height must be between 1 and 5000" ) );
-	image = new_image(width, height, name, 0);
+	image = BKE_add_image_size(width, height, name, 0);
 	if( !image )
 		return ( EXPP_ReturnPyObjError( PyExc_MemoryError,
 						"couldn't create PyObject Image_Type" ) );
 
-	/* reset usage count, since new_image() incremented it */
-	image->id.us--;
+	/* reset usage count, since BKE_add_image_size() incremented it */
+	/* image->id.us--; */
+	/* Strange, new images have a user count of one???, otherwise it messes up */
+	
 	return Image_CreatePyObject( image );
 }
 
@@ -324,6 +331,7 @@ static PyObject *M_Image_GetCurrent( PyObject * self )
 	return Image_CreatePyObject( G.sima->image );
 }
 
+
 /*****************************************************************************/
 /* Function:	M_Image_Load		 */
 /* Python equivalent:	Blender.Image.Load   */
@@ -346,14 +354,13 @@ static PyObject *M_Image_Load( PyObject * self, PyObject * args )
 		return ( EXPP_ReturnPyObjError( PyExc_MemoryError,
 						"couldn't create PyObject Image_Type" ) );
 
-	img_ptr = add_image( fname );
+	img_ptr = BKE_add_image_file( fname );
 	if( !img_ptr )
 		return ( EXPP_ReturnPyObjError( PyExc_IOError,
 						"couldn't load image" ) );
 
-	//reload the image buffers
-	free_image_buffers(img_ptr);
-	img_ptr->ibuf = IMB_loadiffname(img_ptr->name , 0);
+	/* force a load the image buffers*/
+	BKE_image_get_ibuf(img_ptr, NULL);
 
 	image->image = img_ptr;
 
@@ -371,32 +378,34 @@ static PyObject *M_Image_Load( PyObject * self, PyObject * args )
 static PyObject *Image_getPixelF( BPy_Image * self, PyObject * args )
 {
 
-	PyObject *attr;
-	Image *image = self->image;
+	PyObject *attr = PyList_New(4);
+	ImBuf *ibuf= BKE_image_get_ibuf(self->image, NULL);
 	char *pixel;		/* image data */
 	int index;		/* offset into image data */
 	int x = 0;
 	int y = 0;
 	int pixel_size = 4;	/* each pixel is 4 x 8-bits packed in unsigned int */
+	int i;
 
+	if (!attr)
+		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
+				      "couldn't allocate memory for color list" );
+	
 	if( !PyArg_ParseTuple( args, "ii", &x, &y ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected 2 integers" );
 
-	if( !image->ibuf || !image->ibuf->rect )	/* if no image data available */
-		load_image( image, IB_rect, G.sce,  G.scene->r.cfra );	/* loading it */
-
-	if( !image->ibuf || !image->ibuf->rect )	/* didn't work */
+	if( !ibuf || !ibuf->rect )	/* loading didn't work */
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "couldn't load image data in Blender" );
 
-	if( image->ibuf->type == 1 )	/* bitplane image */
+	if( ibuf->type == 1 )	/* bitplane image */
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "unsupported bitplane image format" );
 
-	if( x > ( image->ibuf->x - 1 )
-	    || y > ( image->ibuf->y - 1 )
-	    || x < image->ibuf->xorig || y < image->ibuf->yorig )
+	if( x > ( ibuf->x - 1 )
+	    || y > ( ibuf->y - 1 )
+	    || x < ibuf->xorig || y < ibuf->yorig )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "x or y is out of range" );
 
@@ -405,20 +414,13 @@ static PyObject *Image_getPixelF( BPy_Image * self, PyObject * args )
 	   so we calc ourselves
 	 */
 
-	index = ( x + y * image->ibuf->x ) * pixel_size;
+	index = ( x + y * ibuf->x ) * pixel_size;
 
-	pixel = ( char * ) image->ibuf->rect;
-	attr = Py_BuildValue( "[f,f,f,f]",
-			      ( ( float ) pixel[index] ) / 255.0,
-			      ( ( float ) pixel[index + 1] ) / 255.0,
-			      ( ( float ) pixel[index + 2] ) / 255.0,
-			      ( ( float ) pixel[index + 3] / 255.0 ) );
-
-	if( attr )		/* normal return */
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				      "couldn't get pixel colors" );
+	pixel = ( char * ) ibuf->rect;
+	for (i=0; i<4; i++) {
+		PyList_SetItem( attr, i, PyFloat_FromDouble( ( ( double ) pixel[index+i] ) / 255.0 ));
+	}
+	return attr;
 }
 
 
@@ -431,32 +433,34 @@ static PyObject *Image_getPixelF( BPy_Image * self, PyObject * args )
 
 static PyObject *Image_getPixelI( BPy_Image * self, PyObject * args )
 {
-	PyObject *attr;
-	Image *image = self->image;
+	PyObject *attr = PyList_New(4);
+	ImBuf *ibuf= BKE_image_get_ibuf(self->image, NULL);
 	char *pixel;		/* image data */
 	int index;		/* offset into image data */
 	int x = 0;
 	int y = 0;
 	int pixel_size = 4;	/* each pixel is 4 x 8-bits packed in unsigned int */
+	int i;
 
+	if (!attr)
+		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
+				      "couldn't allocate memory for color list" );
+	
 	if( !PyArg_ParseTuple( args, "ii", &x, &y ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected 2 integers" );
 
-	if( !image->ibuf || !image->ibuf->rect )	/* if no image data available */
-		load_image( image, IB_rect, G.sce,  G.scene->r.cfra );	/* loading it */
-
-	if( !image->ibuf || !image->ibuf->rect )	/* didn't work */
+	if( !ibuf || !ibuf->rect )	/* didn't work */
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "couldn't load image data in Blender" );
 
-	if( image->ibuf->type == 1 )	/* bitplane image */
+	if( ibuf->type == 1 )	/* bitplane image */
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "unsupported bitplane image format" );
 
-	if( x > ( image->ibuf->x - 1 )
-	    || y > ( image->ibuf->y - 1 )
-	    || x < image->ibuf->xorig || y < image->ibuf->yorig )
+	if( x > ( ibuf->x - 1 )
+	    || y > ( ibuf->y - 1 )
+	    || x < ibuf->xorig || y < ibuf->yorig )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "x or y is out of range" );
 
@@ -465,19 +469,13 @@ static PyObject *Image_getPixelI( BPy_Image * self, PyObject * args )
 	   so we calc ourselves
 	 */
 
-	index = ( x + y * image->ibuf->x ) * pixel_size;
-
-	pixel = ( char * ) image->ibuf->rect;
-	attr = Py_BuildValue( "[i,i,i,i]",
-			      pixel[index],
-			      pixel[index + 1],
-			      pixel[index + 2], pixel[index + 3] );
-
-	if( attr )		/* normal return */
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				      "couldn't get pixel colors" );
+	index = ( x + y * ibuf->x ) * pixel_size;
+	pixel = ( char * ) ibuf->rect;
+	
+	for (i=0; i<4; i++) {
+		PyList_SetItem( attr, i, PyInt_FromLong( pixel[index+i] ));
+	}
+	return attr;
 }
 
 
@@ -485,7 +483,7 @@ static PyObject *Image_getPixelI( BPy_Image * self, PyObject * args )
 
 static PyObject *Image_setPixelF( BPy_Image * self, PyObject * args )
 {
-	Image *image = self->image;
+	ImBuf *ibuf= BKE_image_get_ibuf(self->image, NULL);
 	char *pixel;		/* image data */
 	int index;		/* offset into image data */
 	int x = 0;
@@ -499,20 +497,17 @@ static PyObject *Image_setPixelF( BPy_Image * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected 2 integers and an array of 4 floats" );
 
-	if( !image->ibuf || !image->ibuf->rect )	/* if no image data available */
-		load_image( image, IB_rect, G.sce,  G.scene->r.cfra );	/* loading it */
-
-	if( !image->ibuf || !image->ibuf->rect )	/* didn't work */
+	if( !ibuf || !ibuf->rect )	/* didn't work */
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "couldn't load image data in Blender" );
 
-	if( image->ibuf->type == 1 )	/* bitplane image */
+	if( ibuf->type == 1 )	/* bitplane image */
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "unsupported bitplane image format" );
 
-	if( x > ( image->ibuf->x - 1 )
-	    || y > ( image->ibuf->y - 1 )
-	    || x < image->ibuf->xorig || y < image->ibuf->yorig )
+	if( x > ( ibuf->x - 1 )
+	    || y > ( ibuf->y - 1 )
+	    || x < ibuf->xorig || y < ibuf->yorig )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "x or y is out of ruange" );
 
@@ -528,9 +523,9 @@ static PyObject *Image_setPixelF( BPy_Image * self, PyObject * args )
 	   so we calc ourselves
 	 */
 
-	index = ( x + y * image->ibuf->x ) * pixel_size;
+	index = ( x + y * ibuf->x ) * pixel_size;
 
-	pixel = ( char * ) image->ibuf->rect;
+	pixel = ( char * ) ibuf->rect;
 
 	pixel[index] = ( char ) ( p[0] * 255.0 );
 	pixel[index + 1] = ( char ) ( p[1] * 255.0 );
@@ -545,7 +540,7 @@ static PyObject *Image_setPixelF( BPy_Image * self, PyObject * args )
 
 static PyObject *Image_setPixelI( BPy_Image * self, PyObject * args )
 {
-	Image *image = self->image;
+	ImBuf *ibuf= BKE_image_get_ibuf(self->image, NULL);
 	char *pixel;		/* image data */
 	int index;		/* offset into image data */
 	int x = 0;
@@ -559,20 +554,17 @@ static PyObject *Image_setPixelI( BPy_Image * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected 2 integers and an list of 4 ints" );
 
-	if( !image->ibuf || !image->ibuf->rect )	/* if no image data available */
-		load_image( image, IB_rect, G.sce,  G.scene->r.cfra );	/* loading it */
-
-	if( !image->ibuf || !image->ibuf->rect )	/* didn't work */
+	if( !ibuf || !ibuf->rect )	/* didn't work */
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "couldn't load image data in Blender" );
 
-	if( image->ibuf->type == 1 )	/* bitplane image */
+	if( ibuf->type == 1 )	/* bitplane image */
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "unsupported bitplane image format" );
 
-	if( x > ( image->ibuf->x - 1 )
-	    || y > ( image->ibuf->y - 1 )
-	    || x < image->ibuf->xorig || y < image->ibuf->yorig )
+	if( x > ( ibuf->x - 1 )
+	    || y > ( ibuf->y - 1 )
+	    || x < ibuf->xorig || y < ibuf->yorig )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "x or y is out of range" );
 
@@ -587,9 +579,9 @@ static PyObject *Image_setPixelI( BPy_Image * self, PyObject * args )
 	   so we calc ourselves
 	 */
 
-	index = ( x + y * image->ibuf->x ) * pixel_size;
+	index = ( x + y * ibuf->x ) * pixel_size;
 
-	pixel = ( char * ) image->ibuf->rect;
+	pixel = ( char * ) ibuf->rect;
 
 	pixel[index] = ( char ) p[0];
 	pixel[index + 1] = ( char ) p[1];
@@ -604,17 +596,14 @@ static PyObject *Image_setPixelI( BPy_Image * self, PyObject * args )
 
 static PyObject *Image_getMaxXY( BPy_Image * self )
 {
-	Image *image = self->image;
+	ImBuf *ibuf= BKE_image_get_ibuf(self->image, NULL);
 	PyObject *attr;
 
-	if( !image->ibuf || !image->ibuf->rect )	/* if no image data available */
-		load_image( image, IB_rect, G.sce,  G.scene->r.cfra );	/* loading it */
-
-	if( !image->ibuf || !image->ibuf->rect )	/* didn't work */
+	if( !ibuf || !ibuf->rect )	/* didn't work */
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "couldn't load image data in Blender" );
 
-	attr = Py_BuildValue( "[i,i]", image->ibuf->x, image->ibuf->y );
+	attr = Py_BuildValue( "[i,i]", ibuf->x, ibuf->y );
 
 	if( attr )
 		return attr;
@@ -628,18 +617,15 @@ static PyObject *Image_getMaxXY( BPy_Image * self )
 
 static PyObject *Image_getMinXY( BPy_Image * self )
 {
-	Image *image = self->image;
+	ImBuf *ibuf= BKE_image_get_ibuf(self->image, NULL);
 	PyObject *attr;
 
-	if( !image->ibuf || !image->ibuf->rect )	/* if no image data available */
-		load_image( image, IB_rect, G.sce,  G.scene->r.cfra );	/* loading it */
-
-	if( !image->ibuf || !image->ibuf->rect )	/* didn't work */
+	if( !ibuf || !ibuf->rect )	/* didn't work */
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "couldn't load image data in Blender" );
 
-	attr = Py_BuildValue( "[i,i]", image->ibuf->xorig,
-			      image->ibuf->yorig );
+	attr = Py_BuildValue( "[i,i]", ibuf->xorig,
+			      ibuf->yorig );
 
 	if( attr )
 		return attr;
@@ -691,13 +677,23 @@ static PyObject *Image_pack( BPy_Image * self )
 }
 
 
+static PyObject *Image_makeCurrent( BPy_Image * self )
+{
+	if (!G.sima)
+		Py_RETURN_FALSE;
+	
+	G.sima->image= self->image;
+	Py_RETURN_TRUE;
+}
+
+
 /* save image to file */
 
 static PyObject *Image_save( BPy_Image * self )
 {
-	if( !IMB_saveiff
-	    ( self->image->ibuf, self->image->name,
-	      self->image->ibuf->flags ) )
+	ImBuf *ibuf= BKE_image_get_ibuf(self->image, NULL);
+	
+	if(!ibuf || !IMB_saveiff( ibuf, self->image->name, ibuf->flags ) )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "could not save image" );
 
@@ -831,39 +827,33 @@ static PyObject *Image_getFilename( BPy_Image * self )
 
 static PyObject *Image_getSize( BPy_Image * self )
 {
-	PyObject *attr;
-	Image *image = self->image;
+	ImBuf *ibuf= BKE_image_get_ibuf(self->image, NULL);
+	PyObject *attr = PyList_New(2);
 	
-	
-	if( !image->ibuf ) /* if no image data available */
-		load_image( image, IB_rect, G.sce,  G.scene->r.cfra );	/* loading it */
-
-	if( !image->ibuf )	/* didn't work */
+	if( !ibuf )	/* didn't work */
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "couldn't load image data in Blender" );
-
-	attr = Py_BuildValue( "[hh]", image->ibuf->x, image->ibuf->y );
-
-	if( attr )
-		return attr;
-
-	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
+	if( !attr )
+		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 				      "couldn't get Image.size attribute" );
+	
+	PyList_SetItem( attr, 0, PyInt_FromLong(ibuf->x));
+	PyList_SetItem( attr, 1, PyInt_FromLong(ibuf->y));
+	return attr;	
+
+
 }
 
 static PyObject *Image_getDepth( BPy_Image * self )
 {
+	ImBuf *ibuf= BKE_image_get_ibuf(self->image, NULL);
 	PyObject *attr;
-	Image *image = self->image;
 
-	if( !image->ibuf )	/* if no image data available */
-		load_image( image, IB_rect, G.sce,  G.scene->r.cfra );	/* loading it */
-
-	if( !image->ibuf )	/* didn't work */
+	if( !ibuf )	/* didn't work */
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "couldn't load image data in Blender" );
 
-	attr = Py_BuildValue( "h", image->ibuf->depth );
+	attr = Py_BuildValue( "h", ibuf->depth );
 
 	if( attr )
 		return attr;
@@ -943,8 +933,8 @@ static PyObject *Image_reload( BPy_Image * self )
 {
 	Image *image = self->image;
 
-	free_image_buffers( image );	/* force read again */
-	image->ok = 1;
+	BKE_image_signal(image, NULL, IMA_SIGNAL_RELOAD);
+
 	Py_RETURN_NONE;
 }
 
@@ -964,29 +954,27 @@ static PyObject *Image_glLoad( BPy_Image * self )
 	unsigned int *bind = &image->bindcode;
 
 	if( *bind == 0 ) {
+		ImBuf *ibuf= BKE_image_get_ibuf(self->image, NULL);
 
-		if( !image->ibuf )	/* if no image data is available */
-			load_image( image, IB_rect, G.sce,  G.scene->r.cfra );	/* loading it */
-
-		if( !image->ibuf )	/* didn't work */
+		if( !ibuf )	/* didn't work */
 			return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 						      "couldn't load image data in Blender" );
 
 		glGenTextures( 1, ( GLuint * ) bind );
 		glBindTexture( GL_TEXTURE_2D, *bind );
 
-		gluBuild2DMipmaps( GL_TEXTURE_2D, GL_RGBA, image->ibuf->x,
-				   image->ibuf->y, GL_RGBA, GL_UNSIGNED_BYTE,
-				   image->ibuf->rect );
+		gluBuild2DMipmaps( GL_TEXTURE_2D, GL_RGBA, ibuf->x,
+				   ibuf->y, GL_RGBA, GL_UNSIGNED_BYTE,
+				   ibuf->rect );
 		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
 				 GL_LINEAR_MIPMAP_NEAREST );
 		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,
 				 GL_LINEAR );
 		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 
-		glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, image->ibuf->x,
-			      image->ibuf->y, 0, GL_RGBA, GL_UNSIGNED_BYTE,
-			      image->ibuf->rect );
+		glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, ibuf->x,
+			      ibuf->y, 0, GL_RGBA, GL_UNSIGNED_BYTE,
+			      ibuf->rect );
 
 		/* raise the nocollect flag, 
 		   image is not available for garbage collection 
@@ -1001,15 +989,12 @@ static PyObject *Image_glLoad( BPy_Image * self )
 static PyObject *Image_setName( BPy_Image * self, PyObject * args )
 {
 	char *name;
-	char buf[21];
 
 	if( !PyArg_ParseTuple( args, "s", &name ) )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"expected string argument" ) );
 
-	PyOS_snprintf( buf, sizeof( buf ), "%s", name );
-
-	rename_id( &self->image->id, buf );
+	rename_id( &self->image->id, name );
 
 	Py_RETURN_NONE;
 }
@@ -1019,17 +1004,17 @@ static PyObject *Image_setFilename( BPy_Image * self, PyObject * args )
 	char *name;
 	int namelen = 0;
 
-	/* max len is FILE_MAXDIR = 160 chars like done in DNA_image_types.h */
+	/* max len is FILE_MAXDIR == 160, FILE_MAXFILE == 80 chars like done in DNA_image_types.h */
 
 	if( !PyArg_ParseTuple( args, "s#", &name, &namelen ) )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"expected a string argument" ) );
 
-	if( namelen >= FILE_MAXDIR )
+	if( namelen >= FILE_MAXDIR + FILE_MAXFILE )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
-						"string argument is limited to 160 chars at most" ) );
+						"string argument is limited to 240 chars at most" ) );
 
-	PyOS_snprintf( self->image->name, FILE_MAXDIR * sizeof( char ), "%s",
+	PyOS_snprintf( self->image->name, (FILE_MAXDIR + FILE_MAXFILE) * sizeof( char ), "%s",
 		       name );
 
 	Py_RETURN_NONE;
@@ -1134,6 +1119,8 @@ static PyObject *Image_getAttr( BPy_Image * self, char *name )
 
 	if( strcmp( name, "name" ) == 0 )
 		attr = PyString_FromString( self->image->id.name + 2 );
+	else if ( strcmp( name, "properties" ) == 0 )
+		return BPy_Wrap_IDProperty( (ID*)self->image, IDP_GetProperties((ID*)self->image, 1), NULL );
 	else if( strcmp( name, "filename" ) == 0 )
 		attr = PyString_FromString( self->image->name );
 	else if( strcmp( name, "size" ) == 0 )
@@ -1151,21 +1138,37 @@ static PyObject *Image_getAttr( BPy_Image * self, char *name )
 	else if( strcmp( name, "speed" ) == 0 )
 		attr = PyInt_FromLong( self->image->animspeed );
 	else if( strcmp( name, "packed" ) == 0 ) {
-		if (self->image->packedfile)
-			attr = EXPP_incr_ret_True();
-		else
-			attr = EXPP_incr_ret_False();
-	
+		if (self->image->packedfile) attr = Py_True;
+		else attr = Py_False;
+		EXPP_incr_ret(attr);
+	} else if( strcmp( name, "has_data" ) == 0 ) {
+		if (self->image->ibufs.first) attr = Py_True;
+		else attr = Py_False;
+		EXPP_incr_ret(attr);
+	} else if( strcmp( name, "fields" ) == 0 ) {
+		if (self->image->flag & IMA_FIELDS) attr = Py_True;
+		else attr = Py_False;
+		EXPP_incr_ret(attr);
+	} else if( strcmp( name, "fields_odd" ) == 0 ) {
+		if (self->image->flag & IMA_STD_FIELD) attr = Py_True;
+		else attr = Py_False;
+		EXPP_incr_ret(attr);
+	} else if( strcmp( name, "antialias" ) == 0 ) {
+		if (self->image->flag & IMA_ANTIALI) attr = Py_True;
+		else attr = Py_False;
+		EXPP_incr_ret(attr);
+		
 	} else if( strcmp( name, "bindcode" ) == 0 )
 		attr = PyInt_FromLong( self->image->bindcode );
 	else if( strcmp( name, "users" ) == 0 )
 		attr = PyInt_FromLong( self->image->id.us );
 	else if( strcmp( name, "__members__" ) == 0 )
-		attr = Py_BuildValue( "[s,s,s,s,s,s,s,s,s,s,s]",
+		attr = Py_BuildValue( "[s,s,s,s,s,s,s,s,s,s,s,s,s,s,s,s]",
 				      "name", "filename", "size", "depth",
 				      "xrep", "yrep", "start", "end",
-				      "speed", "packed",
-				      "bindcode", "users" );
+				      "speed", "packed", "has_data"
+				      "fields", "odd", "antialias",
+					  "bindcode", "users" );
 
 	if( !attr )
 		return ( EXPP_ReturnPyObjError( PyExc_MemoryError,
@@ -1202,7 +1205,7 @@ static int Image_setAttr( BPy_Image * self, char *name, PyObject * value )
 
 	if( strcmp( name, "name" ) == 0 )
 		error = Image_setName( self, valtuple );
-	if( strcmp( name, "filename" ) == 0 )
+	else if( strcmp( name, "filename" ) == 0 )
 		error = Image_setFilename( self, valtuple );
 	else if( strcmp( name, "xrep" ) == 0 )
 		error = Image_setXRep( self, valtuple );
@@ -1214,7 +1217,38 @@ static int Image_setAttr( BPy_Image * self, char *name, PyObject * value )
 		error = Image_setEnd( self, valtuple );
 	else if( strcmp( name, "speed" ) == 0 )
 		error = Image_setSpeed( self, valtuple );
-	else {			/* Error: no such member in the Image object structure */
+	
+	else if( strcmp( name, "fields" ) == 0 ) {
+		int param = PyObject_IsTrue( value );
+		if( param == -1 )
+			return EXPP_ReturnIntError( PyExc_TypeError,
+				"expected true/false argument" );
+		
+		if (param) self->image->flag |= IMA_FIELDS;
+		else self->image->flag &= ~IMA_FIELDS;
+		Py_INCREF( Py_None );
+		error = Py_None;
+	} else if( strcmp( name, "fields_odd" ) == 0 ) {
+		int param = PyObject_IsTrue( value );
+		if( param == -1 )
+			return EXPP_ReturnIntError( PyExc_TypeError,
+				"expected true/false argument" );
+		
+		if (param) self->image->flag |= IMA_STD_FIELD;
+		else self->image->flag &= ~IMA_STD_FIELD;
+		Py_INCREF( Py_None );
+		error = Py_None;
+	} else if( strcmp( name, "antialias" ) == 0 ) {
+		int param = PyObject_IsTrue( value );
+		if( param == -1 )
+			return EXPP_ReturnIntError( PyExc_TypeError,
+				"expected true/false argument" );
+		
+		if (param) self->image->flag |= IMA_ANTIALI;
+		else self->image->flag &= ~IMA_ANTIALI;
+		Py_INCREF( Py_None );
+		error = Py_None;
+	} else {			/* Error: no such member in the Image object structure */
 		/*Py_DECREF( value ); borrowed ref, no need to decref */
 		Py_DECREF( valtuple );
 		return ( EXPP_ReturnIntError( PyExc_KeyError,
@@ -1240,8 +1274,7 @@ static int Image_setAttr( BPy_Image * self, char *name, PyObject * value )
 /*****************************************************************************/
 static int Image_compare( BPy_Image * a, BPy_Image * b )
 {
-	Image *pa = a->image, *pb = b->image;
-	return ( pa == pb ) ? 0 : -1;
+	return ( a->image == b->image ) ? 0 : -1;
 }
 
 /*****************************************************************************/
