@@ -1,5 +1,5 @@
 /*
- * $Id: Ipo.c,v 1.55 2006/07/02 21:37:06 khughes Exp $
+ * $Id: Ipo.c,v 1.59 2006/12/27 05:04:17 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -228,6 +228,7 @@ static PySequenceMethods Ipo_as_sequence = {
 /*****************************************************************************/
 static void Ipo_dealloc( BPy_Ipo * self );
 //static int IpoPrint (BPy_Ipo *self, FILE *fp, int flags);
+static int Ipo_compare( BPy_Ipo * a, BPy_Ipo * b );
 static PyObject *Ipo_repr( BPy_Ipo * self );
 static PyObject *Ipo_getIter( BPy_Ipo * self );
 static PyObject *Ipo_nextIter( BPy_Ipo * self );
@@ -365,7 +366,7 @@ PyTypeObject Ipo_Type = {
 	NULL,                       /* printfunc tp_print; */
 	NULL,                       /* getattrfunc tp_getattr; */
 	NULL,                       /* setattrfunc tp_setattr; */
-	NULL,                       /* cmpfunc tp_compare; */
+	( cmpfunc ) Ipo_compare,   /* cmpfunc tp_compare; */
 	( reprfunc ) Ipo_repr,     /* reprfunc tp_repr; */
 
 	/* Method suites for standard classes */
@@ -818,16 +819,13 @@ static PyObject *Ipo_getName( BPy_Ipo * self )
 static int Ipo_setName( BPy_Ipo * self, PyObject * args )
 {
 	char *name;
-	char buf[21];
 
 	name = PyString_AsString( args );
 	if( !name )
 		return EXPP_ReturnIntError( PyExc_TypeError,
 				"expected string argument" );
 
-	PyOS_snprintf( buf, sizeof( buf ), "%s", name );
-
-	rename_id( &self->ipo->id, buf );
+	rename_id( &self->ipo->id, name );
 
 	return 0;
 }
@@ -1302,6 +1300,15 @@ static void Ipo_dealloc( BPy_Ipo * self )
 	PyObject_DEL( self );
 }
 
+/*****************************************************************************/
+/* Function:    Ipo_compare			                                         */
+/* Description: This compares 2 ipo python types, == or != only.             */
+/*****************************************************************************/
+static int Ipo_compare( BPy_Ipo * a, BPy_Ipo * b )
+{
+	return ( a->ipo == b->ipo ) ? 0 : -1;
+}
+
 /*****************************************************************************/
 /* Function:    Ipo_repr                                                     */
 /* Description: This is a callback function for the BPy_Ipo type. It         */
@@ -1655,7 +1662,7 @@ static PyObject *Ipo_getNBezPoints( BPy_Ipo * self, PyObject * args )
 		--num;
 	}
 
-	if( num < 0 && !icu )
+	if( num < 0 || !icu )
 		return EXPP_ReturnPyObjError( PyExc_IndexError,
 				"index out of range" );
 
