@@ -1,5 +1,5 @@
 /*
- * $Id: Ipocurve.c,v 1.35 2006/04/21 20:27:52 khughes Exp $
+ * $Id: Ipocurve.c,v 1.39 2007/01/13 04:53:40 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -88,6 +88,11 @@ static PyObject *IpoCurve_getDriverObject( C_IpoCurve * self);
 static int IpoCurve_setDriverObject( C_IpoCurve * self, PyObject * args );
 static PyObject *IpoCurve_getDriverChannel( C_IpoCurve * self);
 static int IpoCurve_setDriverChannel( C_IpoCurve * self, PyObject * args );
+static PyObject *IpoCurve_getDriverExpression( C_IpoCurve * self);
+static PyObject *IpoCurve_getFlag( C_IpoCurve * self, void *type);
+static int IpoCurve_setFlag( C_IpoCurve * self, PyObject *value, void *type);
+
+static int IpoCurve_setDriverExpression( C_IpoCurve * self, PyObject * args );
 static PyObject *IpoCurve_getCurval( C_IpoCurve * self, PyObject * args );
 static int IpoCurve_setCurval( C_IpoCurve * self, PyObject * key, 
 		PyObject * value );
@@ -141,7 +146,7 @@ static PyGetSetDef C_IpoCurve_getseters[] = {
 	NULL},
 	{"driver",
 	 (getter)IpoCurve_getDriver, (setter)IpoCurve_setDriver,
-	 "The status of the driver 1-on, 0-off",
+	 "The status of the driver 1-object, 2-py expression, 0-off",
 	 NULL},
 	{"driverObject",
 	 (getter)IpoCurve_getDriverObject, (setter)IpoCurve_setDriverObject,
@@ -151,6 +156,10 @@ static PyGetSetDef C_IpoCurve_getseters[] = {
 	 (getter)IpoCurve_getDriverChannel, (setter)IpoCurve_setDriverChannel,
 	 "The channel on the driver object used to drive the IpoCurve",
 	 NULL},
+	{"driverExpression",
+	 (getter)IpoCurve_getDriverExpression, (setter)IpoCurve_setDriverExpression,
+	 "The python expression on the driver used to drive the IpoCurve",
+	 NULL},
 	{"interpolation",
 	 (getter)IpoCurve_newgetInterp, (setter)IpoCurve_newsetInterp,
 	 "The interpolation mode of the curve",
@@ -159,6 +168,12 @@ static PyGetSetDef C_IpoCurve_getseters[] = {
 	 (getter)IpoCurve_newgetExtend, (setter)IpoCurve_newsetExtend,
 	 "The extend mode of the curve",
 	 NULL},
+	
+	{"sel",
+	 (getter)IpoCurve_getFlag, (setter)IpoCurve_setFlag,
+	 "the selection state of the curve",
+	 (void *)IPO_SELECT},
+	 
 	 {NULL,NULL,NULL,NULL,NULL}
 };
 
@@ -175,6 +190,7 @@ static PyMappingMethods IpoCurve_as_mapping = {
 /*****************************************************************************/
 /* Python IpoCurve_Type callback function prototypes:                        */
 /*****************************************************************************/
+static int IpoCurve_compare( C_IpoCurve * a, C_IpoCurve * b );
 static PyObject *IpoCurve_repr( C_IpoCurve * self );
 static void IpoCurve_dealloc( C_IpoCurve * self );
 
@@ -192,7 +208,7 @@ PyTypeObject IpoCurve_Type = {
 	0,									/* tp_print */
 	( getattrfunc ) NULL,	            /* tp_getattr */
 	( setattrfunc ) NULL,	            /* tp_setattr */
-	0,									/* tp_compare */
+	( cmpfunc ) IpoCurve_compare,		/* tp_compare */
 	( reprfunc ) IpoCurve_repr,			/* tp_repr */
 	/* Method suites for standard classes */
 
@@ -498,7 +514,7 @@ static PyObject *IpoCurve_append( C_IpoCurve * self, PyObject * args )
 		BezTriple *newb = MEM_callocN( (icu->totvert+1)*sizeof(BezTriple),
 				"BPyBeztriple" );
 		if( icu->bezt ) {
-			memcpy( newb, icu->bezt, ( icu->totvert+1 )*sizeof( BezTriple ) );
+			memcpy( newb, icu->bezt, ( icu->totvert )*sizeof( BezTriple ) );
 			MEM_freeN( icu->bezt );
 		}
 		icu->bezt = newb;
@@ -646,6 +662,15 @@ static void IpoCurve_dealloc( C_IpoCurve * self )
 	PyObject_DEL( self );
 }
 
+/*****************************************************************************/
+/* Function:    IpoCurve_compare                                     		 */
+/* Description: This compares 2 python types, == or != only.			     */
+/*****************************************************************************/
+static int IpoCurve_compare( C_IpoCurve * a, C_IpoCurve * b )
+{
+	return ( a->ipocurve == b->ipocurve ) ? 0 : -1;
+}
+
 /*****************************************************************************/
 /* Function:    IpoCurve_repr                                                */
 /* Description: This is a callback function for the C_IpoCurve type. It      */
@@ -775,36 +800,61 @@ static PyObject *IpoCurve_getDriver( C_IpoCurve * self )
 {
 	if( !self->ipocurve->driver )
 		return PyInt_FromLong( 0 );	
-	else
-		return PyInt_FromLong( 1 );	
+	else {
+		if (self->ipocurve->driver->type == IPO_DRIVER_TYPE_NORMAL)
+			return PyInt_FromLong( 1 );
+		if (self->ipocurve->driver->type == IPO_DRIVER_TYPE_PYTHON)
+			return PyInt_FromLong( 2 );
+	}
+	return EXPP_ReturnPyObjError( PyExc_RuntimeError,
+			"unknown driver type, internal error" );
+	
 }
 
+/*
+	sets the driver to
+	0: disabled
+	1: enabled (object)
+	2: enabled (python expression)
+*/
 static int IpoCurve_setDriver( C_IpoCurve * self, PyObject * args )
 {
 	IpoCurve *ipo = self->ipocurve;
-
+	int type;
 	if( !PyInt_CheckExact( args ) )
 		return EXPP_ReturnIntError( PyExc_TypeError,
 				"expected int argument 0 or 1 " );
-
-	switch( PyInt_AS_LONG( args ) ) {
-	case 0:
+	
+	type = PyInt_AS_LONG( args );
+	
+	if (type < 0 || type > 2)
+		return EXPP_ReturnIntError( PyExc_ValueError,
+				"expected int argument 0, 1 or 2" );
+	
+	if (type==0) { /* disable driver */
 		if( ipo->driver ) {
 			MEM_freeN( ipo->driver );
 			ipo->driver = NULL;			
 		}
-		break;
-	case 1:
-		if( !ipo->driver ) {
+	} else {
+		if( !ipo->driver ) { /*add driver if its not there */
 			ipo->driver = MEM_callocN( sizeof(IpoDriver), "ipo driver" );
 			ipo->driver->blocktype = ID_OB;
 			ipo->driver->adrcode = OB_LOC_X;
 		}
-		break;
-	default:
-		return EXPP_ReturnIntError( PyExc_ValueError,
-				"expected int argument 0 or 1 " );
+		
+		if (type==1 && ipo->driver->type != IPO_DRIVER_TYPE_NORMAL) {
+			ipo->driver->type = IPO_DRIVER_TYPE_NORMAL;
+			ipo->driver->ob = NULL;
+			ipo->driver->flag &= ~IPO_DRIVER_FLAG_INVALID;
+			
+		} else if (type==2 && ipo->driver->type != IPO_DRIVER_TYPE_PYTHON) {
+			ipo->driver->type = IPO_DRIVER_TYPE_PYTHON;
+			/* we should probably set ipo->driver->ob, but theres no way to do it properly */
+			ipo->driver->ob = NULL;
+		}
 	}
+	
 	return 0;
 }
 
@@ -869,6 +919,42 @@ static int IpoCurve_setDriverChannel( C_IpoCurve * self, PyObject * args )
 	return EXPP_ReturnIntError( PyExc_ValueError, "invalid int argument" );
 }
 
+static PyObject *IpoCurve_getDriverExpression( C_IpoCurve * self )
+{
+	IpoCurve *ipo = self->ipocurve;
+	
+	if( ipo->driver && ipo->driver->type == IPO_DRIVER_TYPE_PYTHON )
+		return PyString_FromString( ipo->driver->name );
+
+	Py_RETURN_NONE;
+}
+
+static int IpoCurve_setDriverExpression( C_IpoCurve * self, PyObject * arg )
+{
+	IpoCurve *ipo = self->ipocurve;
+	char *exp; /* python expression */
+	
+	if( !ipo->driver )
+		return EXPP_ReturnIntError( PyExc_RuntimeError,
+			"This IpoCurve does not have an active driver" );
+
+	if (ipo->driver->type != IPO_DRIVER_TYPE_PYTHON)
+		return EXPP_ReturnIntError( PyExc_RuntimeError,
+			"This IpoCurve is not a python expression set the driver attribute to 2" );
+	
+	if(!PyString_Check(arg) )
+		return EXPP_ReturnIntError( PyExc_RuntimeError,
+					      "expected a string argument" );
+	
+	exp = PyString_AsString(arg);
+	if (strlen(exp)>127)
+		return EXPP_ReturnIntError( PyExc_ValueError,
+					      "string is too long, use 127 characters or less" );
+
+	strcpy(ipo->driver->name, exp);
+	return 0;
+}
+
 static PyObject *M_IpoCurve_ExtendDict( void )
 {
 	PyObject *EM = PyConstant_New(  );
@@ -969,6 +1055,27 @@ static int IpoCurve_newsetExtend( C_IpoCurve * self, PyObject * value )
 			IPO_HORIZ, IPO_CYCLX, 'h' );
 }
 
+static PyObject *IpoCurve_getFlag( C_IpoCurve * self, void *type )
+{
+	if (self->ipocurve->flag & (int)type)
+		Py_RETURN_TRUE;
+	else
+		Py_RETURN_FALSE;
+}
+
+static int IpoCurve_setFlag( C_IpoCurve * self, PyObject *value, void *type )
+{
+	int param = PyObject_IsTrue( value );
+	
+	if (param)
+		self->ipocurve->flag |= (int)type;
+	else
+		self->ipocurve->flag &= ~(int)type;
+	
+	return 0;
+}
+
+
 /* #####DEPRECATED###### */
 
 static PyObject *IpoCurve_addBezier( C_IpoCurve * self, PyObject * args )
