@@ -1,5 +1,5 @@
 /* 
- * $Id: Ipocurve.h,v 1.9 2005/04/24 19:03:38 stiv Exp $
+ * $Id: Ipocurve.h,v 1.12 2006/04/21 20:27:52 khughes Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -34,7 +34,7 @@
 #define EXPP_IPOCURVE_H
 
 #include <Python.h>
-#include <DNA_curve_types.h>	/* declaration of IpoCurve */
+#include "DNA_curve_types.h"	/* declaration of IpoCurve */
 
 /*****************************************************************************/
 /* Python C_IpoCurve structure definition:                                   */
@@ -42,9 +42,10 @@
 typedef struct {
 	PyObject_HEAD		/* required macro */
 	IpoCurve * ipocurve;
+	char wrapped;
 } C_IpoCurve;
 
-
+extern PyTypeObject IpoCurve_Type;
 
 PyObject *IpoCurve_Init( void );
 PyObject *IpoCurve_CreatePyObject( IpoCurve * ipo );
