@@ -1,5 +1,5 @@
 /* 
- * $Id: Lamp.c,v 1.44 2006/06/04 13:16:30 khughes Exp $
+ * $Id: Lamp.c,v 1.47 2006/12/27 05:04:17 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -653,10 +653,8 @@ static PyObject *M_Lamp_New( PyObject * self, PyObject * args,
 
 	if( strcmp( name_str, "LampData" ) == 0 )
 		return ( PyObject * ) py_lamp;
-	else {			/* user gave us a name for the lamp, use it */
-		PyOS_snprintf( buf, sizeof( buf ), "%s", name_str );
-		rename_id( &bl_lamp->id, buf );
-	}
+	else			/* user gave us a name for the lamp, use it */
+		rename_id( &bl_lamp->id, name_str );
 
 	return ( PyObject * ) py_lamp;
 }
@@ -1431,8 +1429,7 @@ static void Lamp_dealloc( BPy_Lamp * self )
 /*****************************************************************************/
 static int Lamp_compare( BPy_Lamp * a, BPy_Lamp * b )
 {
-	Lamp *pa = a->lamp, *pb = b->lamp;
-	return ( pa == pb ) ? 0 : -1;
+	return ( a->lamp == b->lamp ) ? 0 : -1;
 }
 
 /*****************************************************************************/
@@ -1449,10 +1446,8 @@ static PyObject *Lamp_getIpo( BPy_Lamp * self )
 {
 	struct Ipo *ipo = self->lamp->ipo;
 
-	if( !ipo ) {
-		Py_INCREF( Py_None );
-		return Py_None;
-	}
+	if( !ipo )
+		Py_RETURN_NONE;
 
 	return Ipo_CreatePyObject( ipo );
 }
@@ -1499,7 +1494,7 @@ static int Lamp_setIpo( BPy_Lamp * self, PyObject * value )
 	self->lamp->ipo = ipo;
 	if ( ipo ) {
 		id = &ipo->id;
-		id->us++;
+		id_us_plus(id);
 	}
 
 	return 0;
@@ -1548,7 +1543,7 @@ static PyObject *Lamp_insertIpoKey( BPy_Lamp * self, PyObject * args )
 	EXPP_allqueue(REDRAWACTION, 0);
 	EXPP_allqueue(REDRAWNLA, 0);
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 static PyObject *Lamp_getModesConst( void )
