@@ -1,5 +1,5 @@
 /* 
- * $Id: MTex.c,v 1.8 2005/11/29 05:08:29 ascotan Exp $
+ * $Id: MTex.c,v 1.10 2007/01/13 05:06:27 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -299,8 +299,7 @@ static PyObject *MTex_setTexMethod( BPy_MTex * self, PyObject * args )
 	if( self->mtex->tex )
 		self->mtex->tex->id.us++;
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 static void MTex_dealloc( BPy_MTex * self )
@@ -327,10 +326,8 @@ static PyObject *MTex_getTex( BPy_MTex *self, void *closure )
 {
 	if( self->mtex->tex )
 		return Texture_CreatePyObject( self->mtex->tex );
-	else {
-		Py_INCREF( Py_None );
-		return Py_None;
-	}
+	else
+		Py_RETURN_NONE;
 }
 
 static int MTex_setTex( BPy_MTex *self, PyObject *value, void *closure)
@@ -376,7 +373,8 @@ static int MTex_setTexCo( BPy_MTex *self, PyObject *value, void *closure)
 
 	if (texco != TEXCO_ORCO && texco != TEXCO_REFL && texco != TEXCO_NORM &&
 		texco != TEXCO_GLOB && texco != TEXCO_UV && texco != TEXCO_OBJECT &&
-		texco != TEXCO_WINDOW && texco != TEXCO_VIEW && texco != TEXCO_STICKY )
+		texco != TEXCO_STRESS && texco != TEXCO_TANGENT && texco != TEXCO_WINDOW &&
+		texco != TEXCO_VIEW && texco != TEXCO_STICKY )
 		return EXPP_ReturnIntError( PyExc_ValueError,
 			"Value must be a member of Texture.TexCo dictionary" );
 
@@ -389,10 +387,8 @@ static PyObject *MTex_getObject( BPy_MTex *self, void *closure )
 {
 	if( self->mtex->object )
 		return Object_CreatePyObject( self->mtex->object );
-	else {
-		Py_INCREF( Py_None );
-		return Py_None;
-	}
+	else
+		Py_RETURN_NONE;
 }
 
 static int MTex_setObject( BPy_MTex *self, PyObject *value, void *closure)
