@@ -1,5 +1,5 @@
 /*
- * $Id: Modifier.c,v 1.9 2006/06/14 04:41:30 khughes Exp $
+ * $Id: Modifier.c,v 1.17 2007/02/05 05:09:15 stiv Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -20,9 +20,6 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
- * All rights reserved.
- *
  * This is a new part of Blender.
  *
  * Contributor(s): Ken Hughes, Campbell Barton
@@ -30,6 +27,9 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
 
+/* TODO, accessing a modifier sequence of a deleted object will crash blender at the moment, not sure how to fix this. */
+
+
 #include "Modifier.h" /*This must come first*/
 
 #include "DNA_object_types.h"
@@ -48,8 +48,13 @@
 #include "mydevice.h"
 
 #include "Object.h"
+#include "Mathutils.h"
 #include "gen_utils.h"
 
+/* checks for the scene being removed */
+#define MODIFIER_DEL_CHECK_PY(bpy_modifier) if (!(bpy_modifier->md)) return ( EXPP_ReturnPyObjError( PyExc_RuntimeError, "Modifier has been removed" ) )
+#define MODIFIER_DEL_CHECK_INT(bpy_modifier) if (!(bpy_modifier->md)) return ( EXPP_ReturnIntError( PyExc_RuntimeError, "Modifier has been removed" ) )
+
 enum mod_constants {
 	/*Apply to all modifiers*/
 	EXPP_MOD_RENDER = 0,
@@ -63,6 +68,7 @@ enum mod_constants {
 	EXPP_MOD_LIMIT, /*ARRAY, MIRROR*/
 	EXPP_MOD_FLAG, /*MIRROR, WAVE*/
 	EXPP_MOD_COUNT, /*DECIMATOR, ARRAY*/
+	EXPP_MOD_LENGTH, /*BUILD, ARRAY*/
 	
 	/*SUBSURF SPESIFIC*/
 	EXPP_MOD_TYPES,
@@ -74,9 +80,15 @@ enum mod_constants {
 	/*ARMATURE SPESIFIC*/
 	EXPP_MOD_ENVELOPES,
 	
+	/*ARRAY SPESIFIC*/
+	EXPP_MOD_OBJECT_OFFSET,
+	EXPP_MOD_OBJECT_CURVE,
+	EXPP_MOD_OFFSET_VEC,
+	EXPP_MOD_SCALE_VEC,
+	EXPP_MOD_MERGE_DIST,
+	
 	/*BUILD SPESIFIC*/
 	EXPP_MOD_START,
-	EXPP_MOD_LENGTH,
 	EXPP_MOD_SEED,
 	EXPP_MOD_RANDOMIZE,
 
@@ -98,11 +110,16 @@ enum mod_constants {
 	EXPP_MOD_TIMEOFFS,
 	
 	/*BOOLEAN SPESIFIC*/
-	EXPP_MOD_OPERATION
+	EXPP_MOD_OPERATION,
+
+	/*EDGE SPLIT SPESIFIC */
+	EXPP_MOD_EDGESPLIT_ANGLE,
+	EXPP_MOD_EDGESPLIT_FROM_ANGLE,
+	EXPP_MOD_EDGESPLIT_FROM_SHARP
 
 	/* yet to be implemented */
 	/* EXPP_MOD_HOOK_,*/
-	/* EXPP_MOD_ARRAY_, */
+	/* , */
 };
 
 /*****************************************************************************/
@@ -247,10 +264,7 @@ PyTypeObject Modifier_Type = {
 
 static PyObject *Modifier_getName( BPy_Modifier * self )
 {
-	if (self->md==NULL)
-		return (EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"This modifier has been removed!" ));
-	
+	MODIFIER_DEL_CHECK_PY(self);
 	return PyString_FromString( self->md->name );
 }
 
@@ -264,9 +278,7 @@ static int Modifier_setName( BPy_Modifier * self, PyObject * attr )
 	if( !name )
 		return EXPP_ReturnIntError( PyExc_TypeError, "expected string arg" );
 
-	if (self->md==NULL)
-		return (EXPP_ReturnIntError( PyExc_RuntimeError,
-				"This modifier has been removed!" ));
+	MODIFIER_DEL_CHECK_INT(self);
 	
 	BLI_strncpy( self->md->name, name, sizeof( self->md->name ) );
 
@@ -279,9 +291,7 @@ static int Modifier_setName( BPy_Modifier * self, PyObject * attr )
 
 static PyObject *Modifier_getType( BPy_Modifier * self )
 {
-	if (self->md==NULL )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"This modifier has been removed!" );
+	MODIFIER_DEL_CHECK_PY(self);
 	
 	return PyInt_FromLong( self->md->type );
 }
@@ -354,14 +364,14 @@ static int armature_setter( BPy_Modifier *self, int type, PyObject *value )
 
 	switch( type ) {
 	case EXPP_MOD_OBJECT: {
-		Object *obj = (( BPy_Object * )value)->object;
-		if( !BPy_Object_Check( value ) || obj->type != OB_ARMATURE )
+		Object *ob = (( BPy_Object * )value)->object;
+		if( !BPy_Object_Check( value ) || ob->type != OB_ARMATURE )
 			return EXPP_ReturnIntError( PyExc_TypeError, 
 					"expected BPy armature object argument" );
-		if(obj == self->obj )
+		if(ob == self->object )
 			return EXPP_ReturnIntError( PyExc_TypeError,
 					"Cannot lattice deform an object with its self" );
-		md->object = obj;
+		md->object = ob;
 		return 0;
 		}
 	case EXPP_MOD_VERTGROUP:
@@ -393,14 +403,14 @@ static int lattice_setter( BPy_Modifier *self, int type, PyObject *value )
 
 	switch( type ) {
 	case EXPP_MOD_OBJECT: {
-		Object *obj = (( BPy_Object * )value)->object;
-		if( !BPy_Object_Check( value ) || obj->type != OB_LATTICE )
+		Object *ob = (( BPy_Object * )value)->object;
+		if( !BPy_Object_Check( value ) || ob->type != OB_LATTICE )
 			return EXPP_ReturnIntError( PyExc_TypeError, 
 					"expected BPy lattice object argument" );
-		if(obj == self->obj )
+		if(ob == self->object )
 			return EXPP_ReturnIntError( PyExc_TypeError,
 					"Cannot curve deform an object with its self" );
-		md->object = obj;
+		md->object = ob;
 		break;
 		}
 	case EXPP_MOD_VERTGROUP: {
@@ -437,14 +447,14 @@ static int curve_setter( BPy_Modifier *self, int type, PyObject *value )
 
 	switch( type ) {
 	case EXPP_MOD_OBJECT: {
-		Object *obj = (( BPy_Object * )value)->object;
-		if( !BPy_Object_Check( value ) || obj->type != OB_CURVE )
+		Object *ob = (( BPy_Object * )value)->object;
+		if( !BPy_Object_Check( value ) || ob->type != OB_CURVE )
 			return EXPP_ReturnIntError( PyExc_TypeError,
 					"expected BPy lattice object argument" );
-		if(obj == self->obj )
+		if(ob == self->object )
 			return EXPP_ReturnIntError( PyExc_TypeError,
 					"Cannot curve deform an object with its self" );
-		md->object = obj;
+		md->object = ob;
 		break;
 		}
 	case EXPP_MOD_VERTGROUP: {
@@ -613,6 +623,71 @@ static int wave_setter( BPy_Modifier *self, int type, PyObject *value )
 	}
 }
 
+static PyObject *array_getter( BPy_Modifier * self, int type )
+{
+	ArrayModifierData *md = (ArrayModifierData *)(self->md);
+
+	if( type == EXPP_MOD_OBJECT_OFFSET )
+		return Object_CreatePyObject( md->offset_ob );
+	else if( type == EXPP_MOD_OBJECT_CURVE )
+		return Object_CreatePyObject( md->curve_ob );	
+	else if( type == EXPP_MOD_COUNT )
+		return PyInt_FromLong( (long)md->count );
+	else if( type == EXPP_MOD_LENGTH )
+		return PyFloat_FromDouble( md->length );
+	else if( type == EXPP_MOD_MERGE_DIST )
+		return PyFloat_FromDouble( md->merge_dist );
+	else if( type == EXPP_MOD_MERGE_DIST )
+		return PyFloat_FromDouble( md->merge_dist );
+	else if( type == EXPP_MOD_OFFSET_VEC)
+		return newVectorObject( md->offset, 3, Py_NEW );
+	else if( type == EXPP_MOD_SCALE_VEC)
+		return newVectorObject( md->scale, 3, Py_NEW );
+	
+	return EXPP_ReturnPyObjError( PyExc_KeyError, "key not found" );
+}
+
+static int array_setter( BPy_Modifier *self, int type, PyObject *value )
+{
+	ArrayModifierData *md = (ArrayModifierData *)(self->md);
+	switch( type ) {
+	case EXPP_MOD_OBJECT_OFFSET: {
+		Object *ob = (( BPy_Object * )value)->object;
+		if( !BPy_Object_Check( value ) )
+			return EXPP_ReturnIntError( PyExc_TypeError, 
+					"expected BPy object argument" );
+		if(ob == self->object )
+			return EXPP_ReturnIntError( PyExc_TypeError,
+					"Cannot lattice deform an object with its self" );
+		md->offset_ob = ob;
+		return 0;
+	}
+	case EXPP_MOD_OBJECT_CURVE: {
+		Object *ob = (( BPy_Object * )value)->object;
+		if( !BPy_Object_Check( value ) || ob->type != OB_CURVE )
+			return EXPP_ReturnIntError( PyExc_TypeError, 
+					"expected BPy object argument" );
+		if(ob == self->object )
+			return EXPP_ReturnIntError( PyExc_TypeError,
+					"Cannot lattice deform an object with its self" );
+		md->curve_ob = ob;
+		return 0;
+	}
+	case EXPP_MOD_COUNT:
+		return EXPP_setIValueClamped( value, &md->count, 1, 1000, 'i' );
+	case EXPP_MOD_LENGTH:
+		return EXPP_setFloatClamped( value, &md->length, 0.0, 1000.0 );
+	case EXPP_MOD_MERGE_DIST:
+		return EXPP_setFloatClamped( value, &md->merge_dist, 0.0, 1000.0 );
+	case EXPP_MOD_OFFSET_VEC:
+		return EXPP_setVec3Clamped( value, md->offset, -10000.0, 10000.0 );
+	case EXPP_MOD_SCALE_VEC:
+		return EXPP_setVec3Clamped( value, md->scale, -10000.0, 10000.0 );
+	default:
+		return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
+	}
+}
+
 static PyObject *boolean_getter( BPy_Modifier * self, int type )
 {
 	BooleanModifierData *md = (BooleanModifierData *)(self->md);
@@ -630,14 +705,14 @@ static int boolean_setter( BPy_Modifier *self, int type, PyObject *value )
 	BooleanModifierData *md = (BooleanModifierData *)(self->md);
 
 	if( type == EXPP_MOD_OBJECT ) {
-		Object *obj = (( BPy_Object * )value)->object;
-		if( !BPy_Object_Check( value ) || obj->type != OB_MESH )
+		Object *ob = (( BPy_Object * )value)->object;
+		if( !BPy_Object_Check( value ) || ob->type != OB_MESH )
 			return EXPP_ReturnIntError( PyExc_TypeError,
 					"expected BPy mesh object argument" );
-		if(obj == self->obj )
+		if(ob == self->object )
 			return EXPP_ReturnIntError( PyExc_TypeError,
 					"Cannot boolean an object with its self" );
-		md->object = obj;
+		md->object = ob;
 		return 0;
 	} else if( type == EXPP_MOD_OPERATION )
 		return EXPP_setIValueRange( value, &md->operation, 
@@ -647,6 +722,48 @@ static int boolean_setter( BPy_Modifier *self, int type, PyObject *value )
 	return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
 }
 
+
+static PyObject *edgesplit_getter( BPy_Modifier * self, int type )
+{
+	EdgeSplitModifierData *md = (EdgeSplitModifierData *)(self->md);
+
+	switch( type ) {
+	case EXPP_MOD_EDGESPLIT_ANGLE:
+		return PyFloat_FromDouble( (double)md->split_angle );
+	case EXPP_MOD_EDGESPLIT_FROM_ANGLE:
+		return PyBool_FromLong( ( long ) 
+				( md->flags & MOD_EDGESPLIT_FROMANGLE ) ) ;
+	case EXPP_MOD_EDGESPLIT_FROM_SHARP:
+		return PyBool_FromLong( ( long ) 
+				( md->flags & MOD_EDGESPLIT_FROMFLAG ) ) ;
+	
+	default:
+		return EXPP_ReturnPyObjError( PyExc_KeyError, "key not found" );
+	}
+}
+
+static int edgesplit_setter( BPy_Modifier *self, int type, PyObject *value )
+{
+	EdgeSplitModifierData *md = (EdgeSplitModifierData *)(self->md);
+
+	switch( type ) {
+	case EXPP_MOD_EDGESPLIT_ANGLE:
+		return EXPP_setFloatClamped( value, &md->split_angle, 0.0, 180.0 );
+	case EXPP_MOD_EDGESPLIT_FROM_ANGLE:
+		return EXPP_setBitfield( value, &md->flags,
+				MOD_EDGESPLIT_FROMANGLE, 'h' );
+	case EXPP_MOD_EDGESPLIT_FROM_SHARP:
+		return EXPP_setBitfield( value, &md->flags,
+				MOD_EDGESPLIT_FROMFLAG, 'h' );
+	
+	default:
+		return EXPP_ReturnIntError( PyExc_KeyError, "key not found" );
+	}
+}
+
+
+
+
 /*
  * get data from a modifier
  */
@@ -659,9 +776,7 @@ static PyObject *Modifier_getData( BPy_Modifier * self, PyObject * key )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 				"expected an int arg as stored in Blender.Modifier.Settings" );
 
-	if (self->md==NULL )
-		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"This modifier has been removed!" );
+	MODIFIER_DEL_CHECK_PY(self);
 	
 	setting = PyInt_AsLong( key );
 	switch( setting ) {
@@ -693,9 +808,12 @@ static PyObject *Modifier_getData( BPy_Modifier * self, PyObject * key )
 				return wave_getter( self, setting );
 			case eModifierType_Boolean:
 				return boolean_getter( self, setting );
+			case eModifierType_Array:
+				return array_getter( self, setting );
+			case eModifierType_EdgeSplit:
+				return edgesplit_getter( self, setting );
 			case eModifierType_Hook:
 			case eModifierType_Softbody:
-			case eModifierType_Array:
 			case eModifierType_None:
 				Py_RETURN_NONE;
 		}
@@ -713,11 +831,26 @@ static int Modifier_setData( BPy_Modifier * self, PyObject * key,
 		return EXPP_ReturnIntError( PyExc_TypeError,
 				"expected an int arg as stored in Blender.Modifier.Settings" );
 	
-	if (self->md==NULL )
-		return EXPP_ReturnIntError( PyExc_RuntimeError,
-				"This modifier has been removed!" );
+	MODIFIER_DEL_CHECK_INT(self);
 	
 	key_int = PyInt_AsLong( key );
+	
+	/* Chach for standard modifier settings */
+	switch( key_int ) {
+	case EXPP_MOD_RENDER:
+		return EXPP_setBitfield( arg, &self->md->mode,
+				eModifierMode_Render, 'h' );
+	case EXPP_MOD_REALTIME:
+		return EXPP_setBitfield( arg, &self->md->mode,
+				eModifierMode_Realtime, 'h' );
+	case EXPP_MOD_EDITMODE:
+		return EXPP_setBitfield( arg, &self->md->mode,
+				eModifierMode_Editmode, 'h' );
+	case EXPP_MOD_ONCAGE:
+		return EXPP_setBitfield( arg, &self->md->mode,
+				eModifierMode_OnCage, 'h' );
+	}
+	
 	switch( self->md->type ) {
     	case eModifierType_Subsurf:
 			return subsurf_setter( self, key_int, arg );
@@ -731,15 +864,18 @@ static int Modifier_setData( BPy_Modifier * self, PyObject * key,
 			return build_setter( self, key_int, arg );
     	case eModifierType_Mirror:
 			return mirror_setter( self, key_int, arg );
+		case eModifierType_Array:
+			return array_setter( self, key_int, arg );
 		case eModifierType_Decimate:
 			return decimate_setter( self, key_int, arg );
 		case eModifierType_Wave:
 			return wave_setter( self, key_int, arg );
 		case eModifierType_Boolean:
 			return boolean_setter( self, key_int, arg );
+		case eModifierType_EdgeSplit:
+			return edgesplit_setter( self, key_int, arg );
 		case eModifierType_Hook:
 		case eModifierType_Softbody:
-		case eModifierType_Array:
 		case eModifierType_None:
 			return 0;
 	}
@@ -779,7 +915,7 @@ static PyObject *Modifier_repr( BPy_Modifier * self )
 /* Description: This function will create a new BPy_Modifier from an         */
 /*              existing Blender modifier structure.                         */
 /*****************************************************************************/
-PyObject *Modifier_CreatePyObject( Object *obj, ModifierData * md )
+PyObject *Modifier_CreatePyObject( Object *ob, ModifierData * md )
 {
 	BPy_Modifier *pymod;
 	pymod = ( BPy_Modifier * ) PyObject_NEW( BPy_Modifier, &Modifier_Type );
@@ -787,7 +923,7 @@ PyObject *Modifier_CreatePyObject( Object *obj, ModifierData * md )
 		return EXPP_ReturnPyObjError( PyExc_MemoryError,
 					      "couldn't create BPy_Modifier object" );
 	pymod->md = md;
-	pymod->obj = obj;
+	pymod->object = ob;
 	return ( PyObject * ) pymod;
 }
 
@@ -819,64 +955,69 @@ ModifierData *Modifier_FromPyObject( PyObject * pyobj )
  * Initialize the interator
  */
 
-static PyObject *Modifiers_getIter( BPy_Modifiers * self )
+static PyObject *ModSeq_getIter( BPy_ModSeq * self )
 {
-	self->iter = (ModifierData *)self->obj->modifiers.first;
-	return EXPP_incr_ret ( (PyObject *) self );
+	if (!self->iter) {
+		self->iter = (ModifierData *)self->object->modifiers.first;
+		return EXPP_incr_ret ( (PyObject *) self );
+	} else {
+		return ModSeq_CreatePyObject(self->object, (ModifierData *)self->object->modifiers.first);
+	}
 }
 
 /*
  * Get the next Modifier
  */
 
-static PyObject *Modifiers_nextIter( BPy_Modifiers * self )
+static PyObject *ModSeq_nextIter( BPy_ModSeq * self )
 {
-	ModifierData *this = self->iter;
-	if( this ) {
-		self->iter = this->next;
-		return Modifier_CreatePyObject( self->obj, this );
+	ModifierData *iter = self->iter;
+	if( iter ) {
+		self->iter = iter->next;
+		return Modifier_CreatePyObject( self->object, iter );
 	}
-
+	
+	self->iter= NULL; /* mark as not iterating */
 	return EXPP_ReturnPyObjError( PyExc_StopIteration,
 			"iterator at end" );
 }
 
 /* return the number of modifiers */
 
-static int Modifiers_length( BPy_Modifiers * self )
+static int ModSeq_length( BPy_ModSeq * self )
 {
-	return BLI_countlist( &self->obj->modifiers );
+	return BLI_countlist( &self->object->modifiers );
 }
 
 /* return a modifier */
 
-static PyObject *Modifiers_item( BPy_Modifiers * self, int i )
+static PyObject *ModSeq_item( BPy_ModSeq * self, int i )
 {
 	ModifierData *md = NULL;
 
 	/* if index is negative, start counting from the end of the list */
 	if( i < 0 )
-		i += Modifiers_length( self );
+		i += ModSeq_length( self );
 
 	/* skip through the list until we get the modifier or end of list */
 
-	for( md = self->obj->modifiers.first; i && md; --i ) md = md->next;
+	for( md = self->object->modifiers.first; i && md; --i ) md = md->next;
 
 	if( md )
-		return Modifier_CreatePyObject( self->obj, md );
+		return Modifier_CreatePyObject( self->object, md );
 	else
 		return EXPP_ReturnPyObjError( PyExc_IndexError,
 				"array index out of range" );
 }
 
 /*****************************************************************************/
-/* Python BPy_Modifiers sequence table:                                      */
+/* Python BPy_ModSeq sequence table:                                      */
 /*****************************************************************************/
-static PySequenceMethods Modifiers_as_sequence = {
-	( inquiry ) Modifiers_length,	/* sq_length */
+static PySequenceMethods ModSeq_as_sequence = {
+	( inquiry ) ModSeq_length,	/* sq_length */
 	( binaryfunc ) 0,	/* sq_concat */
 	( intargfunc ) 0,	/* sq_repeat */
-	( intargfunc ) Modifiers_item,	/* sq_item */
+	( intargfunc ) ModSeq_item,	/* sq_item */
 	( intintargfunc ) 0,	/* sq_slice */
 	( intobjargproc ) 0,	/* sq_ass_item */
 	( intintobjargproc ) 0,	/* sq_ass_slice */
@@ -889,7 +1030,7 @@ static PySequenceMethods Modifiers_as_sequence = {
  * helper function to check for a valid modifier argument
  */
 
-static ModifierData *locate_modifier( BPy_Modifiers *self, PyObject * args )
+static ModifierData *locate_modifier( BPy_ModSeq *self, PyObject * args )
 {
 	BPy_Modifier *pyobj;
 	ModifierData *md;
@@ -905,7 +1046,7 @@ static ModifierData *locate_modifier( BPy_Modifiers *self, PyObject * args )
 				"This modifier has been removed!" );
 
 	/* find the modifier in the object's list */
-	for( md = self->obj->modifiers.first; md; md = md->next )
+	for( md = self->object->modifiers.first; md; md = md->next )
 		if( md == pyobj->md )
 			return md;
 
@@ -916,7 +1057,7 @@ static ModifierData *locate_modifier( BPy_Modifiers *self, PyObject * args )
 
 /* create a new modifier at the end of the list */
 
-static PyObject *Modifiers_append( BPy_Modifiers *self, PyObject *args )
+static PyObject *ModSeq_append( BPy_ModSeq *self, PyObject *args )
 {
 	int type;
 
@@ -929,13 +1070,13 @@ static PyObject *Modifiers_append( BPy_Modifiers *self, PyObject *args )
 		return EXPP_ReturnPyObjError( PyExc_ValueError,
 				"int argument out of range, expected an int from Blender.Modifier.Type" );
 	
-	BLI_addtail( &self->obj->modifiers, modifier_new( type ) );
-	return Modifier_CreatePyObject( self->obj, self->obj->modifiers.last );
+	BLI_addtail( &self->object->modifiers, modifier_new( type ) );
+	return Modifier_CreatePyObject( self->object, self->object->modifiers.last );
 }
 
 /* remove an existing modifier */
 
-static PyObject *Modifiers_remove( BPy_Modifiers *self, PyObject *args )
+static PyObject *ModSeq_remove( BPy_ModSeq *self, PyObject *args )
 {
 	ModifierData *md = locate_modifier( self, args );
 	BPy_Modifier *py_obj;
@@ -945,7 +1086,7 @@ static PyObject *Modifiers_remove( BPy_Modifiers *self, PyObject *args )
 		return (PyObject *)NULL;
 
 	/* do the actual removal */
-	BLI_remlink( &self->obj->modifiers, md );
+	BLI_remlink( &self->object->modifiers, md );
 	modifier_free( md );
 
 	/* erase the link to the modifier */
@@ -957,7 +1098,7 @@ static PyObject *Modifiers_remove( BPy_Modifiers *self, PyObject *args )
 
 /* move the modifier up in the stack */
 
-static PyObject *Modifiers_moveUp( BPy_Modifiers * self, PyObject * args )
+static PyObject *ModSeq_moveUp( BPy_ModSeq * self, PyObject * args )
 {
 	ModifierData *md = locate_modifier( self, args );
 
@@ -965,7 +1106,7 @@ static PyObject *Modifiers_moveUp( BPy_Modifiers * self, PyObject * args )
 	if( !md )
 		return (PyObject *)NULL;
 	
-	if( mod_moveUp( self->obj, md ) )
+	if( mod_moveUp( self->object, md ) )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 				"cannot move above a modifier requiring original data" );
 
@@ -974,7 +1115,7 @@ static PyObject *Modifiers_moveUp( BPy_Modifiers * self, PyObject * args )
 
 /* move the modifier down in the stack */
 
-static PyObject *Modifiers_moveDown( BPy_Modifiers * self, PyObject *args )
+static PyObject *ModSeq_moveDown( BPy_ModSeq * self, PyObject *args )
 {
 	ModifierData *md = locate_modifier( self, args );
 
@@ -982,7 +1123,7 @@ static PyObject *Modifiers_moveDown( BPy_Modifiers * self, PyObject *args )
 	if( !md )
 		return (PyObject *)NULL;
 	
-	if( mod_moveDown( self->obj, md ) )
+	if( mod_moveDown( self->object, md ) )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 				"cannot move beyond a non-deforming modifier" );
 
@@ -991,45 +1132,45 @@ static PyObject *Modifiers_moveDown( BPy_Modifiers * self, PyObject *args )
 
 
 /*****************************************************************************/
-/* Function:    Modifiers_dealloc                                            */
+/* Function:    ModSeq_dealloc                                            */
 /* Description: This is a callback function for the BPy_Modifier type. It    */
 /*              destroys data when the object is deleted.                    */
 /*****************************************************************************/
-static void Modifiers_dealloc( BPy_Modifier * self )
+static void ModSeq_dealloc( BPy_Modifier * self )
 {
 	PyObject_DEL( self );
 }
 
 /*****************************************************************************/
-/* Python BPy_Modifiers methods table:                                       */
+/* Python BPy_ModSeq methods table:                                       */
 /*****************************************************************************/
-static PyMethodDef BPy_Modifiers_methods[] = {
+static PyMethodDef BPy_ModSeq_methods[] = {
 	/* name, method, flags, doc */
-	{"append", ( PyCFunction ) Modifiers_append, METH_VARARGS,
+	{"append", ( PyCFunction ) ModSeq_append, METH_VARARGS,
 	 "(type) - add a new modifier, where type is the type of modifier"},
-	{"remove", ( PyCFunction ) Modifiers_remove, METH_VARARGS,
+	{"remove", ( PyCFunction ) ModSeq_remove, METH_VARARGS,
 	 "(modifier) - remove an existing modifier, where modifier is a modifier from this object."},
-	{"moveUp", ( PyCFunction ) Modifiers_moveUp, METH_VARARGS,
+	{"moveUp", ( PyCFunction ) ModSeq_moveUp, METH_VARARGS,
 	 "(modifier) - Move a modifier up in stack"},
-	{"moveDown", ( PyCFunction ) Modifiers_moveDown, METH_VARARGS,
+	{"moveDown", ( PyCFunction ) ModSeq_moveDown, METH_VARARGS,
 	 "(modifier) - Move a modifier down in stack"},
 	{NULL, NULL, 0, NULL}
 };
 
 /*****************************************************************************/
-/* Python Modifiers_Type structure definition:                               */
+/* Python ModSeq_Type structure definition:                               */
 /*****************************************************************************/
-PyTypeObject Modifiers_Type = {
+PyTypeObject ModSeq_Type = {
 	PyObject_HEAD_INIT( NULL )  /* required py macro */
 	0,                          /* ob_size */
 	/*  For printing, in format "<module>.<name>" */
 	"Blender.Modifiers",        /* char *tp_name; */
-	sizeof( BPy_Modifiers ),       /* int tp_basicsize; */
+	sizeof( BPy_ModSeq ),       /* int tp_basicsize; */
 	0,                          /* tp_itemsize;  For allocation */
 
 	/* Methods to implement standard operations */
 
-	( destructor ) Modifiers_dealloc,/* destructor tp_dealloc; */
+	( destructor ) ModSeq_dealloc,/* destructor tp_dealloc; */
 	NULL,                       /* printfunc tp_print; */
 	NULL,                       /* getattrfunc tp_getattr; */
 	NULL,                       /* setattrfunc tp_setattr; */
@@ -1039,7 +1180,7 @@ PyTypeObject Modifiers_Type = {
 	/* Method suites for standard classes */
 
 	NULL,                       /* PyNumberMethods *tp_as_number; */
-	&Modifiers_as_sequence,        /* PySequenceMethods *tp_as_sequence; */
+	&ModSeq_as_sequence,        /* PySequenceMethods *tp_as_sequence; */
 	NULL,                       /* PyMappingMethods *tp_as_mapping; */
 
 	/* More standard operations (here for binary compatibility) */
@@ -1073,11 +1214,11 @@ PyTypeObject Modifiers_Type = {
 
   /*** Added in release 2.2 ***/
 	/*   Iterators */
-	( getiterfunc )Modifiers_getIter, /* getiterfunc tp_iter; */
-    ( iternextfunc )Modifiers_nextIter, /* iternextfunc tp_iternext; */
+	( getiterfunc )ModSeq_getIter, /* getiterfunc tp_iter; */
+    ( iternextfunc )ModSeq_nextIter, /* iternextfunc tp_iternext; */
 
   /*** Attribute descriptor and subclassing stuff ***/
-	BPy_Modifiers_methods,         /* struct PyMethodDef *tp_methods; */
+	BPy_ModSeq_methods,         /* struct PyMethodDef *tp_methods; */
 	NULL,                       /* struct PyMemberDef *tp_members; */
 	NULL,                       /* struct PyGetSetDef *tp_getset; */
 	NULL,                       /* struct _typeobject *tp_base; */
@@ -1102,18 +1243,19 @@ PyTypeObject Modifiers_Type = {
 };
 
 /*****************************************************************************/
-/* Function:    Modifiers_CreatePyObject                                     */
-/* Description: This function will create a new BPy_Modifiers from an        */
+/* Function:    ModSeq_CreatePyObject                                     */
+/* Description: This function will create a new BPy_ModSeq from an        */
 /*              existing  ListBase structure.                                */
 /*****************************************************************************/
-PyObject *ModSeq_CreatePyObject( Object *obj )
+PyObject *ModSeq_CreatePyObject( Object *ob, ModifierData *iter )
 {
-	BPy_Modifiers *pymod;
-	pymod = ( BPy_Modifiers * ) PyObject_NEW( BPy_Modifiers, &Modifiers_Type );
+	BPy_ModSeq *pymod;
+	pymod = ( BPy_ModSeq * ) PyObject_NEW( BPy_ModSeq, &ModSeq_Type );
 	if( !pymod )
 		return EXPP_ReturnPyObjError( PyExc_MemoryError,
-				"couldn't create BPy_Modifiers object" );
-	pymod->obj = obj;
+				"couldn't create BPy_ModSeq object" );
+	pymod->object = ob;
+	pymod->iter = iter;
 	return ( PyObject * ) pymod;
 }
 
@@ -1142,10 +1284,15 @@ static PyObject *M_Modifier_TypeDict( void )
 				PyInt_FromLong( eModifierType_Wave ) );
 		PyConstant_Insert( d, "BOOLEAN",
 				PyInt_FromLong( eModifierType_Boolean ) );
+		PyConstant_Insert( d, "ARRAY",
+				PyInt_FromLong( eModifierType_Array ) );
+		PyConstant_Insert( d, "EDGESPLIT",
+				PyInt_FromLong( eModifierType_EdgeSplit ) );
 	}
 	return S;
 }
 
+
 static PyObject *M_Modifier_SettingsDict( void )
 {
 	PyObject *S = PyConstant_New(  );
@@ -1161,7 +1308,7 @@ st='''
 	EXPP_MOD_RENDER = 0,
 	EXPP_MOD_REALTIME,
 	EXPP_MOD_EDITMODE,
-	........ and so on, copy from above
+	etc.. copy from above
 '''
 
 base= '''
@@ -1175,7 +1322,7 @@ for var in st.replace(',','').split('\n'):
 	var= var[0]
 	if (not var) or var.startswith('/'): continue
 	
-	var=var.split('_')[-1]
+	var='_'.join(var.split('_')[2:])
 	print base % (var, var),
 # END PYSCRIPT
 */
@@ -1199,6 +1346,8 @@ for var in st.replace(',','').split('\n'):
 				PyInt_FromLong( EXPP_MOD_FLAG ) );
 			PyConstant_Insert( d, "COUNT", 
 				PyInt_FromLong( EXPP_MOD_COUNT ) );
+			PyConstant_Insert( d, "LENGTH", 
+				PyInt_FromLong( EXPP_MOD_LENGTH ) );
 			PyConstant_Insert( d, "TYPES", 
 				PyInt_FromLong( EXPP_MOD_TYPES ) );
 			PyConstant_Insert( d, "LEVELS", 
@@ -1211,10 +1360,18 @@ for var in st.replace(',','').split('\n'):
 				PyInt_FromLong( EXPP_MOD_UV ) );
 			PyConstant_Insert( d, "ENVELOPES", 
 				PyInt_FromLong( EXPP_MOD_ENVELOPES ) );
+			PyConstant_Insert( d, "OBJECT_OFFSET", 
+				PyInt_FromLong( EXPP_MOD_OBJECT_OFFSET ) );
+			PyConstant_Insert( d, "OBJECT_CURVE", 
+				PyInt_FromLong( EXPP_MOD_OBJECT_CURVE ) );
+			PyConstant_Insert( d, "OFFSET_VEC", 
+				PyInt_FromLong( EXPP_MOD_OFFSET_VEC ) );
+			PyConstant_Insert( d, "SCALE_VEC", 
+				PyInt_FromLong( EXPP_MOD_SCALE_VEC ) );
+			PyConstant_Insert( d, "MERGE_DIST", 
+				PyInt_FromLong( EXPP_MOD_MERGE_DIST ) );
 			PyConstant_Insert( d, "START", 
 				PyInt_FromLong( EXPP_MOD_START ) );
-			PyConstant_Insert( d, "LENGTH", 
-				PyInt_FromLong( EXPP_MOD_LENGTH ) );
 			PyConstant_Insert( d, "SEED", 
 				PyInt_FromLong( EXPP_MOD_SEED ) );
 			PyConstant_Insert( d, "RANDOMIZE", 
@@ -1243,6 +1400,12 @@ for var in st.replace(',','').split('\n'):
 				PyInt_FromLong( EXPP_MOD_TIMEOFFS ) );
 			PyConstant_Insert( d, "OPERATION", 
 				PyInt_FromLong( EXPP_MOD_OPERATION ) );
+			PyConstant_Insert( d, "EDGESPLIT_ANGLE", 
+				PyInt_FromLong( EXPP_MOD_EDGESPLIT_ANGLE ) );
+			PyConstant_Insert( d, "EDGESPLIT_FROM_ANGLE", 
+				PyInt_FromLong( EXPP_MOD_EDGESPLIT_FROM_ANGLE ) );
+			PyConstant_Insert( d, "EDGESPLIT_FROM_SHARP", 
+				PyInt_FromLong( EXPP_MOD_EDGESPLIT_FROM_SHARP ) );
 			/*End Auto generated code*/
 	}
 	return S;
@@ -1257,15 +1420,20 @@ PyObject *Modifier_Init( void )
 	PyObject *TypeDict = M_Modifier_TypeDict( );
 	PyObject *SettingsDict = M_Modifier_SettingsDict( );
 
-	if( PyType_Ready( &Modifiers_Type ) < 0 ||
+	if( PyType_Ready( &ModSeq_Type ) < 0 ||
 			PyType_Ready( &Modifier_Type ) < 0 )
 		return NULL;
 
 	submodule = Py_InitModule3( "Blender.Modifier", NULL,
 			"Modifer module for accessing and creating object modifier data" );
 
-	if( TypeDict )
-		PyModule_AddObject( submodule, "Type", TypeDict );
+	if( TypeDict ) {
+		PyModule_AddObject( submodule, "Type", TypeDict ); /* deprecated */
+		/* since PyModule_AddObject() steals a reference, we need to
+		   incref TypeDict to use it again */
+		Py_INCREF( TypeDict);
+		PyModule_AddObject( submodule, "Types", TypeDict );
+	}
 	
 	if( SettingsDict )
 		PyModule_AddObject( submodule, "Settings", SettingsDict );
