@@ -1,5 +1,5 @@
 /* 
- * $Id: NLA.c,v 1.16 2006/07/22 14:22:03 campbellbarton Exp $
+ * $Id: NLA.c,v 1.20 2006/12/27 05:04:18 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -124,8 +124,10 @@ static PyMethodDef BPy_Action_methods[] = {
 static void Action_dealloc( BPy_Action * bone );
 static PyObject *Action_getAttr( BPy_Action * bone, char *name );
 static int Action_setAttr( BPy_Action * bone, char *name, PyObject * v );
+static int Action_compare( BPy_Action * a, BPy_Action * b );
 static PyObject *Action_repr( BPy_Action * bone );
 
+
 /*****************************************************************************/
 /* Python TypeAction structure definition:				 */
 /*****************************************************************************/
@@ -140,7 +142,7 @@ PyTypeObject Action_Type = {
 	0,			/* tp_print */
 	( getattrfunc ) Action_getAttr,	/* tp_getattr */
 	( setattrfunc ) Action_setAttr,	/* tp_setattr */
-	0,			/* tp_compare */
+	( cmpfunc ) Action_compare,		/* tp_compare */
 	( reprfunc ) Action_repr,	/* tp_repr */
 	0,			/* tp_as_number */
 	0,			/* tp_as_sequence */
@@ -258,7 +260,6 @@ static PyObject *Action_getName( BPy_Action * self )
 static PyObject *Action_setName( BPy_Action * self, PyObject * args )
 {
 	char *name;
-	char buf[21];
 
 	if( !self->action )
 		( EXPP_ReturnPyObjError( PyExc_RuntimeError,
@@ -269,8 +270,7 @@ static PyObject *Action_setName( BPy_Action * self, PyObject * args )
 						"expected string argument" ) );
 
 	/*change name*/
-	PyOS_snprintf( buf, sizeof( buf ), "%s", name );
-	rename_id( &self->action->id, buf);
+	rename_id( &self->action->id, name);
 
 	Py_RETURN_NONE;
 }
@@ -369,8 +369,7 @@ static PyObject *Action_verifyChannel( BPy_Action * self, PyObject * args )
 
 	chan = verify_action_channel(self->action, chanName);
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 
@@ -398,8 +397,7 @@ static PyObject *Action_removeChannel( BPy_Action * self, PyObject * args )
 	/*remove channel*/
 	BLI_freelinkN( &self->action->chanbase, chan );
 
-	Py_INCREF( Py_None );
-	return ( Py_None );
+	Py_RETURN_NONE;
 }
 
 static PyObject *Action_getAllChannelIpos( BPy_Action * self )
@@ -495,6 +493,13 @@ static int Action_setAttr( BPy_Action * self, char *name, PyObject * value )
 	return 0;		/* normal exit */
 }
 
+/*----------------------------------------------------------------------*/
+static int Action_compare( BPy_Action * a, BPy_Action * b )
+{
+	return ( a->action == b->action ) ? 0 : -1;
+}
+
+
 /*----------------------------------------------------------------------*/
 static PyObject *Action_repr( BPy_Action * self )
 {
@@ -907,6 +912,19 @@ static PyObject *ActionStrip_getStrideBone( BPy_ActionStrip * self )
 	return PyString_FromString( self->strip->stridechannel );
 }
 
+static PyObject *ActionStrip_getGroupTarget( BPy_ActionStrip * self )
+{
+	if( !self->strip )
+		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
+				"This strip has been removed!" );
+				
+	if (self->strip->object) {
+		return Object_CreatePyObject( self->strip->object );
+	} else {
+		Py_RETURN_NONE;
+	}
+}
+
 /*
  * set the stride bone name
  */
@@ -926,6 +944,22 @@ static int ActionStrip_setStrideBone( BPy_ActionStrip * self, PyObject * attr )
 	return 0;
 }
 
+static int ActionStrip_setGroupTarget( BPy_ActionStrip * self, PyObject * args )
+{
+	if( !self->strip )
+		return EXPP_ReturnIntError( PyExc_RuntimeError,
+				"This strip has been removed!" );
+
+	if( (PyObject *)args == Py_None )
+		self->strip->object = NULL;
+	else if( BPy_Object_Check( args ) )
+		self->strip->object = ((BPy_Object *)args)->object;
+	else
+		return EXPP_ReturnIntError( PyExc_TypeError,
+				"expected an object or None" );
+	return 0;
+}
+
 /*****************************************************************************/
 /* Python BPy_Constraint attributes get/set structure:                       */
 /*****************************************************************************/
@@ -969,6 +1003,9 @@ static PyGetSetDef BPy_ActionStrip_getseters[] = {
 	{"strideBone",
 	(getter)ActionStrip_getStrideBone, (setter)ActionStrip_setStrideBone,
   	 "Name of Bone used for stride", NULL},
+	{"groupTarget",
+	(getter)ActionStrip_getGroupTarget, (setter)ActionStrip_setGroupTarget,
+	 "Name of target armature within group", NULL},	
 	{NULL,NULL,NULL,NULL,NULL}  /* Sentinel */
 };
 
@@ -1277,10 +1314,11 @@ static PySequenceMethods ActionStrips_as_sequence = {
  * helper function to check for a valid action strip argument
  */
 
-static bActionStrip *locate_strip( BPy_ActionStrips *self, PyObject * args )
+static bActionStrip *locate_strip( BPy_ActionStrips *self, 
+		PyObject *args, BPy_ActionStrip **stripobj )
 {
-	BPy_ActionStrip *pyobj;
 	bActionStrip *strip = NULL;
+	BPy_ActionStrip *pyobj;
 
 	/* check that argument is a constraint */
 	if( !PyArg_ParseTuple( args, "O!", &ActionStrip_Type, &pyobj ) ) {
@@ -1295,6 +1333,10 @@ static bActionStrip *locate_strip( BPy_ActionStrips *self, PyObject * args )
 		return NULL;
 	}
 
+	/* if caller needs the object, return it */
+	if( stripobj )
+		*stripobj = pyobj;
+
 	/* find the action strip in the NLA */
 	for( strip = self->ob->nlastrips.first; strip; strip = strip->next )
 		if( strip == pyobj->strip )
@@ -1313,7 +1355,7 @@ static bActionStrip *locate_strip( BPy_ActionStrips *self, PyObject * args )
 static PyObject *ActionStrips_remove( BPy_ActionStrips *self, PyObject * args )
 {
 	BPy_ActionStrip *pyobj;
-	bActionStrip *strip = locate_strip( self, args );
+	bActionStrip *strip = locate_strip( self, args, &pyobj );
 
 	/* return exception if we can't find the strip */
 	if( !strip )
@@ -1334,7 +1376,7 @@ static PyObject *ActionStrips_remove( BPy_ActionStrips *self, PyObject * args )
 
 static PyObject *ActionStrips_moveUp( BPy_ActionStrips *self, PyObject * args )
 {
-	bActionStrip *strip = locate_strip( self, args );
+	bActionStrip *strip = locate_strip( self, args, NULL );
 
 	/* return exception if we can't find the strip */
 	if( !strip )
@@ -1355,7 +1397,7 @@ static PyObject *ActionStrips_moveUp( BPy_ActionStrips *self, PyObject * args )
 
 static PyObject *ActionStrips_moveDown( BPy_ActionStrips *self, PyObject * args )
 {
-	bActionStrip *strip = locate_strip( self, args );
+	bActionStrip *strip = locate_strip( self, args, NULL );
 
 	/* return exception if we can't find the strip */
 	if( !strip )
