@@ -1,5 +1,5 @@
 /*
- * $Id: NMesh.h,v 1.39 2006/07/06 21:51:54 blendix Exp $
+ * $Id: NMesh.h,v 1.40 2006/12/23 10:37:50 blendix Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -41,6 +41,7 @@
 #include <config.h>
 #endif
 
+#include "DNA_customdata_types.h"
 #include "DNA_object_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
@@ -101,6 +102,7 @@ typedef struct {
 	unsigned char transp;
 	Image *image;
 	char mat_nr, mf_flag /* was char smooth */;
+	int orig_index;
 
 } BPy_NMFace;			/* an NMesh face */
 
@@ -131,6 +133,11 @@ typedef struct {
 #define NMESH_HASVERTUV	(1<<1)
 #define NMESH_HASFACEUV	(1<<2)
 
+	/* stores original data that is not accesible through NMesh, but that we
+	   still want to preserve, indexed by orig_index in NMFace */
+	CustomData fdata; 
+	int totfdata;
+
 } BPy_NMesh;
 
 /* PROTOS */
