@@ -54,6 +54,8 @@
 #include "NLA.h"
 #include "gen_utils.h"
 
+#include "DNA_armature_types.h" /*used for pose bone select*/
+
 extern void chan_calc_mat(bPoseChannel *chan);
 
 //------------------------ERROR CODES---------------------------------
@@ -178,9 +180,9 @@ static PyObject *PoseBonesDict_GetItem(BPy_PoseBonesDict *self, PyObject* key)
 	PyObject *value = NULL;
 
 	value = PyDict_GetItem(self->bonesMap, key);
-	if(value == NULL){
-        return EXPP_incr_ret(Py_None);
-	}
+	if(value == NULL)
+        Py_RETURN_NONE;
+	
 	return EXPP_incr_ret(value);
 }
 //------------------TYPE_OBECT DEFINITION--------------------------
@@ -288,7 +290,7 @@ static PyObject *Pose_update(BPy_Pose *self)
 	if(daddy)
 		where_is_pose(daddy);
 
-	return EXPP_incr_ret(Py_None);
+	Py_RETURN_NONE;
 }
 //------------------------tp_methods
 //This contains a list of all methods the object contains
@@ -330,6 +332,12 @@ static void Pose_dealloc(BPy_Pose *self)
 	Pose_Type.tp_free(self);
 	return;
 }
+//------------------------tp_cmp
+//This compares 2 pose types
+static int Pose_compare(BPy_Pose *a, BPy_Pose *b )
+{
+	return ( a->pose== b->pose ) ? 0 : -1;
+}
 //------------------------tp_repr
 //This is the string representation of the object
 static PyObject *Pose_repr(BPy_Pose *self)
@@ -505,7 +513,7 @@ static PyObject *PoseBone_insertKey(BPy_PoseBone *self, PyObject *args)
 	//update the IPOs
 	remake_action_ipos (((BPy_Object*)parent_object)->object->action);
 
-	return EXPP_incr_ret(Py_None);
+	Py_RETURN_NONE;
 
 AttributeError:
 	return EXPP_objError(PyExc_AttributeError, "%s%s%s",
@@ -908,6 +916,61 @@ static int PoseBone_setTail(BPy_PoseBone *self, PyObject *value, void *closure)
 	return EXPP_intError(PyExc_AttributeError, "%s%s%s",
 		sPoseBoneError, ".tail: ", "not able to set this property");
 }
+//------------------------PoseBone.sel (getter)
+//Gets the pose bones selection
+static PyObject *PoseBone_getSelect(BPy_PoseBone *self, void *closure)
+{
+	if (self->posechannel->bone->flag & BONE_SELECTED)
+		Py_RETURN_TRUE;
+	else
+		Py_RETURN_FALSE;
+}
+//------------------------PoseBone.sel (setter)
+//Sets the pose bones selection
+static int PoseBone_setSelect(BPy_PoseBone *self, PyObject *value, void *closure)
+{
+	if (PyObject_IsTrue( value ))
+		self->posechannel->bone->flag |= BONE_SELECTED;
+	else
+		self->posechannel->bone->flag &= ~(BONE_SELECTED | BONE_ACTIVE);
+	return 0;
+}
+
+
+//------------------------PoseBone.parent (getter)
+//Gets the pose bones selection
+static PyObject *PoseBone_getParent(BPy_PoseBone *self, void *closure)
+{
+	if (self->posechannel->parent)
+		return PyPoseBone_FromPosechannel(self->posechannel->parent);
+	else
+        Py_RETURN_NONE;
+}
+
+//------------------------PoseBone.displayObject (getter)
+//Gets the pose bones selection
+static PyObject *PoseBone_getDisplayObject(BPy_PoseBone *self, void *closure)
+{
+	if (self->posechannel->custom)
+		return Object_CreatePyObject(self->posechannel->custom);
+	else
+        Py_RETURN_NONE;
+}
+
+//------------------------PoseBone.displayObject (setter)
+//Gets the pose bones selection
+static int PoseBone_setDisplayObject(BPy_PoseBone *self, PyObject *value, void *closure)
+{
+	if (value == Py_None) {
+		self->posechannel->custom = NULL;
+	} else if (BPy_Object_Check(value)) {
+		self->posechannel->custom = Object_FromPyObject(value);
+	} else {
+		return EXPP_ReturnIntError(PyExc_AttributeError, "can only assign a Blender Object or None");
+	}
+	return 0;
+}
+
 //------------------TYPE_OBECT IMPLEMENTATION---------------------------
 //------------------------tp_getset
 //This contains methods for attributes that require checking
@@ -928,12 +991,18 @@ static PyGetSetDef BPy_PoseBone_getset[] = {
 		"The pose bone's head positon", NULL},
 	{"tail", (getter)PoseBone_getTail, (setter)PoseBone_setTail, 
 		"The pose bone's tail positon", NULL},
+	{"sel", (getter)PoseBone_getSelect, (setter)PoseBone_setSelect, 
+		"The pose selection state", NULL},
     {"limitMin", (getter)PoseBone_getLimitMin, (setter)PoseBone_setLimitMin,
         "The pose bone dof min", NULL},
     {"limitMax", (getter)PoseBone_getLimitMax, (setter)PoseBone_setLimitMax,
         "The pose bone dof max", NULL},
 	{"constraints", (getter)PoseBone_getConstraints, (setter)NULL, 
 		"The list of contraints that pertain to this pose bone", NULL},
+	{"parent", (getter)PoseBone_getParent, (setter)NULL, 
+		"The bones parent (read only for posebones)", NULL},
+	{"displayObject", (getter)PoseBone_getDisplayObject, (setter)PoseBone_setDisplayObject, 
+		"The poseMode object to draw in place of this bone", NULL},
 	{NULL, NULL, NULL, NULL, NULL}
 };
 //------------------------tp_dealloc
@@ -1016,7 +1085,7 @@ PyObject *Pose_Init(void)
 	//Initializes TypeObject.ob_type
 	if (PyType_Ready(&Pose_Type) < 0 || PyType_Ready(&PoseBone_Type)  < 0 ||
 		PyType_Ready(&PoseBonesDict_Type) < 0) {
-		return EXPP_incr_ret(Py_None);
+		Py_RETURN_NONE;
 	}
 
 	//Register the module
@@ -1061,7 +1130,7 @@ PyObject *PyPose_FromPose(bPose *pose, char *name)
 
 		return (PyObject*)py_pose;
 	}else{
-		return EXPP_incr_ret(Py_None);
+		Py_RETURN_NONE;
 	}
 
 RuntimeError:
@@ -1081,7 +1150,7 @@ PyObject *PyPoseBone_FromPosechannel(bPoseChannel *pchan)
 		py_posechannel->posechannel = pchan;
 		return (PyObject*)py_posechannel;
 	}else{
-		return EXPP_incr_ret(Py_None);
+		Py_RETURN_NONE;
 	}
 
 RuntimeError:
