@@ -1,5 +1,5 @@
 /* 
- * $Id: Sys.c,v 1.26 2006/05/17 10:09:20 intrr Exp $
+ * $Id: Sys.c,v 1.28 2006/12/16 03:36:54 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -39,6 +39,15 @@
 #include "PIL_time.h"
 #include "gen_utils.h"
 
+#ifdef WIN32
+#define DIRSEP '\\'
+#define DIRSEP_STR "\\"
+#else
+#define DIRSEP '/'
+#define DIRSEP_STR "/"
+#endif
+
+
 /*****************************************************************************/
 /* Python API function prototypes for the sys module.                        */
 /*****************************************************************************/
@@ -142,35 +151,23 @@ static PyObject *g_sysmodule = NULL;	/* pointer to Blender.sys module */
 
 PyObject *sys_Init( void )
 {
-	PyObject *submodule, *dict, *sep;
+	PyObject *submodule, *dict;
 
 	submodule = Py_InitModule3( "Blender.sys", M_sys_methods, M_sys_doc );
 
 	g_sysmodule = submodule;
 
 	dict = PyModule_GetDict( submodule );
-
-#ifdef WIN32
-	sep = Py_BuildValue( "s", "\\" );
-#else
-	sep = Py_BuildValue( "s", "/" );
-#endif
-
-	if( sep ) {
-		Py_INCREF( sep );
-		EXPP_dict_set_item_str( dict, "dirsep", sep );
-		EXPP_dict_set_item_str( dict, "sep", sep );
-	}
+	
+	EXPP_dict_set_item_str( dict, "dirsep", PyString_FromString(DIRSEP_STR) );
+	EXPP_dict_set_item_str( dict, "sep", PyString_FromString(DIRSEP_STR) );
 
 	return submodule;
 }
 
 static PyObject *M_sys_basename( PyObject * self, PyObject * args )
 {
-	PyObject *c;
-
-	char *name, *p, basename[FILE_MAXFILE];
-	char sep;
+	char *name, *p, basename[FILE_MAXDIR + FILE_MAXFILE];
 	int n, len;
 
 	if( !PyArg_ParseTuple( args, "s", &name ) )
@@ -179,64 +176,51 @@ static PyObject *M_sys_basename( PyObject * self, PyObject * args )
 
 	len = strlen( name );
 
-	c = PyObject_GetAttrString( g_sysmodule, "dirsep" );
-	sep = PyString_AsString( c )[0];
-	Py_DECREF( c );
-
-	p = strrchr( name, sep );
+	p = strrchr( name, DIRSEP );
 
 	if( p ) {
 		n = name + len - p - 1;	/* - 1 because we don't want the sep */
 
-		if( n > FILE_MAXFILE )
+		if( n > FILE_MAXDIR + FILE_MAXFILE )
 			return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 						      "path too long" );
 
 		BLI_strncpy( basename, p + 1, n + 1 );
-		return Py_BuildValue( "s", basename );
+		return PyString_FromString( basename );
 	}
 
-	return Py_BuildValue( "s", name );
+	return PyString_FromString( name );
 }
 
 static PyObject *M_sys_dirname( PyObject * self, PyObject * args )
 {
-	PyObject *c;
-
-	char *name, *p, dirname[FILE_MAXDIR];
-	char sep;
+	char *name, *p, dirname[FILE_MAXDIR + FILE_MAXFILE];
 	int n;
 
 	if( !PyArg_ParseTuple( args, "s", &name ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected string argument" );
 
-	c = PyObject_GetAttrString( g_sysmodule, "dirsep" );
-	sep = PyString_AsString( c )[0];
-	Py_DECREF( c );
-
-	p = strrchr( name, sep );
+	p = strrchr( name, DIRSEP );
 
 	if( p ) {
 		n = p - name;
 
-		if( n > FILE_MAXDIR )
+		if( n > FILE_MAXDIR + FILE_MAXFILE )
 			return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 						      "path too long" );
 
 		BLI_strncpy( dirname, name, n + 1 );
-		return Py_BuildValue( "s", dirname );
+		return PyString_FromString( dirname );
 	}
 
-	return Py_BuildValue( "s", "." );
+	return PyString_FromString( "." );
 }
 
 static PyObject *M_sys_join( PyObject * self, PyObject * args )
 {
-	PyObject *c = NULL;
 	char *name = NULL, *path = NULL;
 	char filename[FILE_MAXDIR + FILE_MAXFILE];
-	char sep;
 	int pathlen = 0, namelen = 0;
 
 	if( !PyArg_ParseTuple( args, "ss", &path, &name ) )
@@ -250,28 +234,21 @@ static PyObject *M_sys_join( PyObject * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "filename is too long." );
 
-	c = PyObject_GetAttrString( g_sysmodule, "dirsep" );
-	sep = PyString_AsString( c )[0];
-	Py_DECREF( c );
-
 	BLI_strncpy( filename, path, pathlen );
 
-	if( filename[pathlen - 2] != sep ) {
-		filename[pathlen - 1] = sep;
+	if( filename[pathlen - 2] != DIRSEP ) {
+		filename[pathlen - 1] = DIRSEP;
 		pathlen += 1;
 	}
 
 	BLI_strncpy( filename + pathlen - 1, name, namelen );
 
-	return Py_BuildValue( "s", filename );
+	return PyString_FromString( filename );
 }
 
 static PyObject *M_sys_splitext( PyObject * self, PyObject * args )
 {
-	PyObject *c;
-
-	char *name, *dot, *p, path[FILE_MAXFILE], ext[FILE_MAXFILE];
-	char sep;
+	char *name, *dot, *p, path[FILE_MAXDIR + FILE_MAXFILE], ext[FILE_MAXDIR + FILE_MAXFILE];
 	int n, len;
 
 	if( !PyArg_ParseTuple( args, "s", &name ) )
@@ -279,17 +256,12 @@ static PyObject *M_sys_splitext( PyObject * self, PyObject * args )
 					      "expected string argument" );
 
 	len = strlen( name );
-
-	c = PyObject_GetAttrString( g_sysmodule, "dirsep" );
-	sep = PyString_AsString( c )[0];
-	Py_DECREF( c );
-
 	dot = strrchr( name, '.' );
 
 	if( !dot )
 		return Py_BuildValue( "ss", name, "" );
 
-	p = strrchr( name, sep );
+	p = strrchr( name, DIRSEP );
 
 	if( p ) {
 		if( p > dot )
@@ -301,7 +273,7 @@ static PyObject *M_sys_splitext( PyObject * self, PyObject * args )
 	/* loong extensions are supported -- foolish, but Python's os.path.splitext
 	 * supports them, so ... */
 	 
-	if( n >= FILE_MAXFILE || ( len - n ) >= FILE_MAXFILE )
+	if( n >= FILE_MAXDIR + FILE_MAXFILE || ( len - n ) >= FILE_MAXDIR + FILE_MAXFILE )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError, "path too long" );
 
 	BLI_strncpy( ext, dot, n + 1 );
@@ -317,9 +289,7 @@ static PyObject *M_sys_makename( PyObject * self, PyObject * args,
 	int strip = 0;
 	static char *kwlist[] = { "path", "ext", "strip", NULL };
 	char *dot = NULL, *p = NULL, basename[FILE_MAXDIR + FILE_MAXFILE];
-	char sep;
 	int n, len, lenext = 0;
-	PyObject *c;
 
 	if( !PyArg_ParseTupleAndKeywords( args, kw, "|ssi", kwlist,
 					  &path, &ext, &strip ) )
@@ -334,11 +304,7 @@ static PyObject *M_sys_makename( PyObject * self, PyObject * args,
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "path too long" );
 
-	c = PyObject_GetAttrString( g_sysmodule, "dirsep" );
-	sep = PyString_AsString( c )[0];
-	Py_DECREF( c );
-
-	p = strrchr( path, sep );
+	p = strrchr( path, DIRSEP );
 
 	if( p && strip ) {
 		n = path + len - p;
@@ -367,9 +333,8 @@ static PyObject *M_sys_makename( PyObject * self, PyObject * args,
 }
 
 static PyObject *M_sys_time( PyObject * self )
-{
-	double t = PIL_check_seconds_timer(  );
-	return Py_BuildValue( "d", t );
+{	
+	return PyFloat_FromDouble( PIL_check_seconds_timer(  ) );
 }
 
 static PyObject *M_sys_sleep( PyObject * self, PyObject * args )
@@ -403,8 +368,8 @@ static PyObject *M_sys_exists( PyObject * self, PyObject * args )
 	else if( S_ISDIR( mode ) )
 		i = 2;
 	/* i stays as -1 if path exists but is neither a regular file nor a dir */
-
-	return Py_BuildValue( "i", i );
+	
+	return PyInt_FromLong(i);
 }
 
 static PyObject *M_sys_expandpath( PyObject * self, PyObject * args )
