@@ -1,5 +1,5 @@
 /*  
- *  $Id: Texture.c,v 1.21 2006/01/28 18:33:14 hos Exp $
+ *  $Id: Texture.c,v 1.26 2007/01/13 05:06:27 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -33,17 +33,22 @@
 
 #include "BKE_global.h"
 #include "BKE_main.h"
+#include "BKE_idprop.h"
 #include "BKE_library.h"
-#include "BLI_blenlib.h"
 #include "BKE_texture.h"
 #include "BKE_utildefines.h"
+
+#include "BLI_blenlib.h"
+
 #include "DNA_object_types.h"
 #include "DNA_material_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_texture_types.h"
+
 #include "MTex.h"
 #include "Image.h"
 #include "Ipo.h"
+#include "IDProp.h"
 #include "constant.h"
 #include "blendef.h"
 #include "gen_utils.h"
@@ -203,17 +208,13 @@ static const EXPP_map_pair tex_flag_map[] = {
 	{NULL, 0}
 };
 
+/* NOTE: flags moved to image... */
 static const EXPP_map_pair tex_imageflag_map[] = {
 	{"InterPol", TEX_INTERPOL},
 	{"UseAlpha", TEX_USEALPHA},
 	{"MipMap", TEX_MIPMAP},
-	{"Fields", TEX_FIELDS},
 	{"Rot90", TEX_IMAROT},
 	{"CalcAlpha", TEX_CALCALPHA},
-	{"Cyclic", TEX_ANIMCYCLIC},
-	{"Movie", TEX_ANIM5},
-	{"StField", TEX_STD_FIELD},
-	{"Anti", TEX_ANTIALI},
 	{"NormalMap", TEX_NORMALMAP},
 	{NULL, 0}
 };
@@ -392,15 +393,18 @@ GETFUNC( getName );
 GETFUNC( getType );
 GETFUNC( getSType );
 GETFUNC( clearIpo );
+GETFUNC( getAnimMontage );
+GETFUNC( getAnimLength );
+SETFUNC( setAnimLength );
+SETFUNC( setAnimMontage );
 #endif
 
 GETFUNC( oldgetSType );
 GETFUNC( oldgetType );
 
+GETFUNC(getProperties);
 GETFUNC( clearIpo );
 GETFUNC( getAnimFrames );
-GETFUNC( getAnimLength );
-GETFUNC( getAnimMontage );
 GETFUNC( getAnimOffset );
 GETFUNC( getAnimStart );
 GETFUNC( getBrightness );
@@ -449,8 +453,6 @@ OLDSETFUNC( setSType );
 OLDSETFUNC( setType );
 
 SETFUNC( setAnimFrames );
-SETFUNC( setAnimLength );
-SETFUNC( setAnimMontage );
 SETFUNC( setAnimOffset );
 SETFUNC( setAnimStart );
 SETFUNC( setBrightness );
@@ -539,10 +541,14 @@ static PyMethodDef BPy_Texture_methods[] = {
 /* Python Texture_Type attributes get/set structure:                         */
 /*****************************************************************************/
 static PyGetSetDef BPy_Texture_getseters[] = {
+	{"properties",
+	 (getter)Texture_getProperties, NULL,
+	 "Get this texture's ID Properties"},
 	{"animFrames",
 	 (getter)Texture_getAnimFrames, (setter)Texture_setAnimFrames,
 	 "Number of frames of a movie to use",
 	 NULL},
+#if 0
 	{"animLength",
 	 (getter)Texture_getAnimLength, (setter)Texture_setAnimLength,
 	 "Number of frames of a movie to use (0 for all)",
@@ -551,6 +557,7 @@ static PyGetSetDef BPy_Texture_getseters[] = {
 	 (getter)Texture_getAnimMontage, (setter)Texture_setAnimMontage,
 	 "Montage mode, start frames and durations",
 	 NULL},
+#endif
 	{"animOffset",
 	 (getter)Texture_getAnimOffset, (setter)Texture_setAnimOffset,
 	 "Offsets the number of the first movie frame to use",
@@ -720,14 +727,16 @@ static PyGetSetDef BPy_Texture_getseters[] = {
 	 (getter)Texture_getImageFlags, (setter)Texture_setImageFlags,
 	 "Mipmaps enabled ('ImageFlags')",
 	 (void *)TEX_MIPMAP},
-	{"fields",
-	 (getter)Texture_getImageFlags, (setter)Texture_setImageFlags,
-	 "Use of image's fields enabled ('ImageFlags')",
-	 (void *)TEX_FIELDS},
 	{"rot90",
 	 (getter)Texture_getImageFlags, (setter)Texture_setImageFlags,
 	 "X/Y flip for rendering enabled ('ImageFlags')",
 	 (void *)TEX_IMAROT},
+#if 0
+	/* disabled, moved to image */
+	{"fields",
+	 (getter)Texture_getImageFlags, (setter)Texture_setImageFlags,
+	 "Use of image's fields enabled ('ImageFlags')",
+	 (void *)TEX_FIELDS},
 	{"cyclic",
 	 (getter)Texture_getImageFlags, (setter)Texture_setImageFlags,
 	 "Looping of animated frames enabled ('ImageFlags')",
@@ -744,6 +753,7 @@ static PyGetSetDef BPy_Texture_getseters[] = {
 	 (getter)Texture_getImageFlags, (setter)Texture_setImageFlags,
 	 "Standard field deinterlacing enabled ('ImageFlags')",
 	 (void *)TEX_STD_FIELD},
+#endif
 	{"normalMap",
 	 (getter)Texture_getImageFlags, (setter)Texture_setImageFlags,
 	 "Use of image RGB values for normal mapping enabled ('ImageFlags')",
@@ -1103,6 +1113,8 @@ static PyObject *M_Texture_TexCoDict( void )
 		PyConstant_Insert(d, "WIN", PyInt_FromLong(TEXCO_WINDOW));
 		PyConstant_Insert(d, "VIEW", PyInt_FromLong(TEXCO_VIEW));
 		PyConstant_Insert(d, "STICK", PyInt_FromLong(TEXCO_STICKY));
+		PyConstant_Insert(d, "STRESS", PyInt_FromLong(TEXCO_STRESS));
+		PyConstant_Insert(d, "TANGENT", PyInt_FromLong(TEXCO_TANGENT));
 	}
 	return TexCo;
 }
@@ -1165,12 +1177,8 @@ static PyObject *M_Texture_ImageFlagsDict( void )
 		PyConstant_Insert(d, "INTERPOL", PyInt_FromLong(TEX_INTERPOL));
 		PyConstant_Insert(d, "USEALPHA", PyInt_FromLong(TEX_USEALPHA));
 		PyConstant_Insert(d, "MIPMAP", PyInt_FromLong(TEX_MIPMAP));
-		PyConstant_Insert(d, "FIELDS", PyInt_FromLong(TEX_FIELDS));
 		PyConstant_Insert(d, "ROT90", PyInt_FromLong(TEX_IMAROT));
 		PyConstant_Insert(d, "CALCALPHA", PyInt_FromLong(TEX_CALCALPHA));
-		PyConstant_Insert(d, "STFIELD", PyInt_FromLong(TEX_STD_FIELD));
-		PyConstant_Insert(d, "MOVIE", PyInt_FromLong(TEX_ANIM5));
-		PyConstant_Insert(d, "CYCLIC", PyInt_FromLong(TEX_ANIMCYCLIC));
 		PyConstant_Insert(d, "NORMALMAP", PyInt_FromLong(TEX_NORMALMAP));
 	}
 	return ImageFlags;
@@ -1324,6 +1332,12 @@ int Texture_CheckPyObject( PyObject * pyobj )
 /* Python BPy_Texture methods:                                               */
 /*****************************************************************************/
 
+static PyObject *Texture_getProperties( BPy_Texture * self )
+{
+	/*sanity check, we set parent property type to Group here*/
+	return BPy_Wrap_IDProperty( (ID*)self->texture, IDP_GetProperties((ID*)self->texture, 1), NULL );
+}
+
 static PyObject *Texture_getExtend( BPy_Texture * self )
 {
 	PyObject *attr = NULL;
@@ -1407,11 +1421,13 @@ static PyObject *Texture_oldgetType( BPy_Texture * self )
 
 static int Texture_setAnimFrames( BPy_Texture * self, PyObject * value )
 {
-	return EXPP_setIValueClamped ( value, &self->texture->frames,
+	return EXPP_setIValueClamped ( value, &self->texture->iuser.frames,
 								EXPP_TEX_ANIMFRAME_MIN,
 								EXPP_TEX_ANIMFRAME_MAX, 'h' );
 }
 
+#if 0
+/* this was stupid to begin with! (ton) */
 static int Texture_setAnimLength( BPy_Texture * self, PyObject * value )
 {
 	return EXPP_setIValueClamped ( value, &self->texture->len,
@@ -1419,6 +1435,7 @@ static int Texture_setAnimLength( BPy_Texture * self, PyObject * value )
 								EXPP_TEX_ANIMLEN_MAX, 'h' );
 }
 
+/* this is too simple to keep supporting? disabled for time being (ton) */
 static int Texture_setAnimMontage( BPy_Texture * self, PyObject * value )
 {
 	int fradur[4][2];
@@ -1443,17 +1460,18 @@ static int Texture_setAnimMontage( BPy_Texture * self, PyObject * value )
 
 	return 0;
 }
+#endif
 
 static int Texture_setAnimOffset( BPy_Texture * self, PyObject * value )
 {
-	return EXPP_setIValueClamped ( value, &self->texture->offset,
+	return EXPP_setIValueClamped ( value, &self->texture->iuser.offset,
 								EXPP_TEX_ANIMOFFSET_MIN,
 								EXPP_TEX_ANIMOFFSET_MAX, 'h' );
 }
 
 static int Texture_setAnimStart( BPy_Texture * self, PyObject * value )
 {
-	return EXPP_setIValueClamped ( value, &self->texture->sfra,
+	return EXPP_setIValueClamped ( value, &self->texture->iuser.sfra,
 								EXPP_TEX_ANIMSTART_MIN,
 								EXPP_TEX_ANIMSTART_MAX, 'h' );
 }
@@ -1595,13 +1613,8 @@ static int Texture_setImageFlags( BPy_Texture * self, PyObject * value,
 		int bitmask = TEX_INTERPOL
 					| TEX_USEALPHA
 					| TEX_MIPMAP
-					| TEX_FIELDS
 					| TEX_IMAROT
 					| TEX_CALCALPHA
-					| TEX_ANIMCYCLIC
-					| TEX_ANIM5
-					| TEX_ANTIALI
-					| TEX_STD_FIELD
 					| TEX_NORMALMAP;
 
 		if( !PyInt_CheckExact ( value ) ) {
@@ -1616,13 +1629,6 @@ static int Texture_setImageFlags( BPy_Texture * self, PyObject * value,
 							"invalid bit(s) set in mask" );
 	}
 
-	/* "mipmap" and "fields" can't be set at the same time */
-
-	if( ( param & TEX_MIPMAP ) &&
-			( param & TEX_FIELDS ) )
-		return EXPP_ReturnIntError( PyExc_ValueError,
-				"image flags MIPMAP and FIELDS cannot be used together" );
-
 	/* everything is OK; save the new flag setting */
 
 	self->texture->imaflag = param;
@@ -1887,8 +1893,7 @@ static int Texture_setDistMetric( BPy_Texture * self, PyObject * value )
 		return EXPP_ReturnPyObjError( PyExc_ValueError,
 					      "invalid dist metric type" );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 #else
 	return EXPP_setIValueRange ( value, &self->texture->vn_distm,
 							TEX_DISTANCE,
@@ -1942,10 +1947,8 @@ static PyObject *Texture_getIpo( BPy_Texture * self )
 {
 	struct Ipo *ipo = self->texture->ipo;
 
-	if( !ipo ) {
-		Py_INCREF( Py_None );
-		return Py_None;
-	}
+	if( !ipo )
+		Py_RETURN_NONE;
 
 	return Ipo_CreatePyObject( ipo );
 }
@@ -1992,7 +1995,7 @@ static int Texture_setIpo( BPy_Texture * self, PyObject * value )
 	self->texture->ipo = ipo;
 	if ( ipo ) {
 		id = &ipo->id;
-		id->us++;
+		id_us_plus(id);
 	}
 
 	return 0;
@@ -2009,6 +2012,8 @@ static PyObject *Texture_getAnimFrames( BPy_Texture *self )
 	return attr;
 }
 
+#if 0
+/* disabled. this option was too stupid! (ton) */
 static PyObject *Texture_getAnimLength( BPy_Texture *self )
 {
 	PyObject *attr = PyInt_FromLong( self->texture->len );
@@ -2040,10 +2045,11 @@ static PyObject *Texture_getAnimMontage( BPy_Texture *self )
 
 	return attr;
 }
+#endif
 
 static PyObject *Texture_getAnimOffset( BPy_Texture *self )
 {
-	PyObject *attr = PyInt_FromLong( self->texture->offset );
+	PyObject *attr = PyInt_FromLong( self->texture->iuser.offset );
 
 	if( !attr )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
@@ -2054,7 +2060,7 @@ static PyObject *Texture_getAnimOffset( BPy_Texture *self )
 
 static PyObject *Texture_getAnimStart( BPy_Texture *self )
 {
-	PyObject *attr = PyInt_FromLong( self->texture->sfra );
+	PyObject *attr = PyInt_FromLong( self->texture->iuser.sfra );
 
 	if( !attr )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
@@ -2147,7 +2153,7 @@ static PyObject *Texture_getIntExtend( BPy_Texture * self )
 
 static PyObject *Texture_getFieldsPerImage( BPy_Texture *self )
 {
-	PyObject *attr = PyInt_FromLong( self->texture->fie_ima );
+	PyObject *attr = PyInt_FromLong( self->texture->iuser.fie_ima );
 
 	if( !attr )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
@@ -2547,8 +2553,7 @@ static PyObject *Texture_oldsetType( BPy_Texture * self, PyObject * args )
 		self->texture->env->object= OBACT;
 	}
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 static PyObject *Texture_oldsetExtend( BPy_Texture * self, PyObject * args )
@@ -2563,8 +2568,7 @@ static PyObject *Texture_oldsetExtend( BPy_Texture * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_ValueError,
 					      "invalid extend mode" );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 /*
@@ -2593,8 +2597,7 @@ static PyObject *Texture_oldsetNoiseBasis( BPy_Texture * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_ValueError,
 					      "invalid noise basis" );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 static PyObject *Texture_oldsetDistNoise( BPy_Texture * self, PyObject * args )
@@ -2611,8 +2614,7 @@ static PyObject *Texture_oldsetDistNoise( BPy_Texture * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_ValueError,
 					      "invalid noise basis" );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 static PyObject *Texture_oldsetSType( BPy_Texture * self, PyObject * args )
@@ -2641,8 +2643,7 @@ static PyObject *Texture_oldsetSType( BPy_Texture * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_ValueError,
 					      "invalid texture stype" );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 static PyObject *Texture_oldsetDistMetric( BPy_Texture * self, PyObject * args )
@@ -2660,8 +2661,7 @@ static PyObject *Texture_oldsetDistMetric( BPy_Texture * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_ValueError,
 					      "invalid dist metric type" );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 static PyObject *Texture_oldsetImageFlags( BPy_Texture * self, PyObject * args )
@@ -2692,7 +2692,6 @@ static PyObject *Texture_oldsetImageFlags( BPy_Texture * self, PyObject * args )
 
 	self->texture->imaflag = (short)flag;
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
