@@ -1,5 +1,5 @@
 /* 
- * $Id: Window.c,v 1.40 2006/04/16 15:28:50 stiv Exp $
+ * $Id: Window.c,v 1.47 2007/02/15 11:26:19 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -57,7 +57,7 @@
 #include "Mathutils.h"
 #include "constant.h"
 #include "gen_utils.h"
-
+#include "Armature.h"
 
 /* See Draw.c */
 extern int EXPP_disable_force_draw;
@@ -385,13 +385,7 @@ PyObject *M_Window_Redraw( PyObject * self, PyObject * args )
 						scrarea_queue_redraw(sa);
 				}
 				else {
-					if( sa->spacetype == SPACE_TEXT ) {
-						st = sa->spacedata.first;
-						if( st->text ) {
-							if( st->text->flags & TXT_FOLLOW )	/* follow cursor display */
-								pop_space_text( st );
-						}
-					}
+					/* do not call fancy hacks here like pop_space_text(st); (ton) */
 					scrarea_do_windraw( sa );
 					if( sa->headwin ) scrarea_do_headdraw( sa );
 				}
@@ -884,6 +878,36 @@ static PyObject *M_Window_GetPerspMatrix( PyObject * self )
 	return perspmat;
 }
 
+
+/* update_armature_weakrefs()
+ * helper function used in M_Window_EditMode.
+ * rebuilds list of Armature weakrefs in __main__
+ */
+
+static int update_armature_weakrefs()
+{
+	/* stuff for armature weak refs */
+	char *list_name = ARM_WEAKREF_LIST_NAME;
+	PyObject *maindict = NULL, *armlist = NULL;
+	PyObject *pyarmature = NULL;
+	int x;
+
+	maindict= PyModule_GetDict(PyImport_AddModule(	"__main__"));
+	armlist = PyDict_GetItemString(maindict, list_name);
+	if( !armlist){
+		printf("Oops - update_armature_weakrefs()\n");
+		return 0;
+	}
+
+	for (x = 0; x < PySequence_Size(armlist); x++){
+		pyarmature = PyWeakref_GetObject(PySequence_GetItem(	armlist, x));
+		if (pyarmature != Py_None)
+			Armature_RebuildEditbones(pyarmature);
+	}
+	return 1;
+}
+
+
 static PyObject *M_Window_EditMode( PyObject * self, PyObject * args )
 {
 	short status = -1;
@@ -898,15 +922,31 @@ static PyObject *M_Window_EditMode( PyObject * self, PyObject * args )
 
 	if( status >= 0 ) {
 		if( status ) {
-			if( !G.obedit )
-				enter_editmode(  );
-		} else if( G.obedit ) {
-			if( do_undo && U.undosteps != 0 ) {
-				if( undo_str_len > 63 )
-					undo_str[63] = '\0';	/* 64 is max */
-				undo_push_mesh( undo_str );	/* use better solution after 2.34 */
+			if( !G.obedit ){
+
+				//update armatures
+				if(! update_armature_weakrefs()){
+					return EXPP_ReturnPyObjError( 
+						PyExc_RuntimeError,
+						"internal error -  update_armature_weakrefs");
+				}
+
+				//enter editmode
+				enter_editmode(0);
 			}
-			exit_editmode( 1 );
+		} else if( G.obedit ) {
+			if( undo_str_len > 63 )
+				undo_str[63] = '\0';	/* 64 is max */
+			BIF_undo_push( undo_str ); /* This checks user undo settings */
+			exit_editmode( EM_FREEDATA );
+
+			//update armatures
+			if(! update_armature_weakrefs()){
+				return EXPP_ReturnPyObjError( 
+					PyExc_RuntimeError,
+					"internal error -  update_armature_weakrefs");
+			}
+
 		}
 	}
 
@@ -1374,6 +1414,7 @@ PyObject *Window_Init( void )
 		PyConstant_Insert( d, "NLA", PyInt_FromLong( SPACE_NLA ) );
 		PyConstant_Insert( d, "SCRIPT", PyInt_FromLong( SPACE_SCRIPT ) );
 		PyConstant_Insert( d, "TIME", PyInt_FromLong( SPACE_TIME ) );
+		PyConstant_Insert( d, "NODE", PyInt_FromLong( SPACE_NODE ) );
 
 		PyModule_AddObject( submodule, "Types", Types );
 	}
