@@ -1,5 +1,5 @@
 /* 
- * $Id: World.c,v 1.33 2006/04/16 15:28:50 stiv Exp $
+ * $Id: World.c,v 1.37 2006/12/27 05:04:20 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -95,6 +95,7 @@ static PyObject *World_getScriptLinks( BPy_World * self, PyObject * args );
 static PyObject *World_addScriptLink( BPy_World * self, PyObject * args );
 static PyObject *World_clearScriptLinks( BPy_World * self, PyObject * args );
 static PyObject *World_setCurrent( BPy_World * self );
+static PyObject *World_copy( BPy_World * self );
 
 
 /*****************************************************************************/
@@ -222,6 +223,8 @@ static PyMethodDef BPy_World_methods[] = {
 	 "please use setCurrent instead, this alias will be removed."},
 	{"insertIpoKey", ( PyCFunction ) World_insertIpoKey, METH_VARARGS,
 	 "( World IPO type ) - Inserts a key into the IPO"},
+	{"__copy__", ( PyCFunction ) World_copy, METH_NOARGS,
+	 "() - Makes a copy of this world."},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -379,10 +382,9 @@ static PyObject *M_World_Get( PyObject * self, PyObject * args )
 static PyObject *M_World_GetCurrent( PyObject * self )
 {
 	BPy_World *w = NULL;
-	if( !G.scene->world ) {
-		Py_INCREF( Py_None );
-		return Py_None;
-	}
+	if( !G.scene->world )
+		Py_RETURN_NONE;
+	
 	w = ( BPy_World * ) PyObject_NEW( BPy_World, &World_Type );
 	w->world = G.scene->world;
 	return ( PyObject * ) w;
@@ -433,8 +435,7 @@ static PyObject *World_setRange( BPy_World * self, PyObject * args )
 		return ( EXPP_ReturnPyObjError
 			 ( PyExc_TypeError, "expected a float argument" ) );
 	self->world->range = range;
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 
@@ -442,10 +443,8 @@ static PyObject *World_getIpo( BPy_World * self )
 {
 	struct Ipo *ipo = self->world->ipo;
 
-	if( !ipo ) {
-		Py_INCREF( Py_None );
-		return Py_None;
-	}
+	if( !ipo )
+		Py_RETURN_NONE;
 
 	return Ipo_CreatePyObject( ipo );
 }
@@ -477,12 +476,11 @@ static PyObject *World_setIpo( BPy_World * self, PyObject * args )
 			id->us--;
 	}
 
-	( ( ID * ) & ipo->id )->us++;
+	id_us_plus(&ipo->id);
 
 	self->world->ipo = ipo;
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 static PyObject *World_clearIpo( BPy_World * self )
@@ -527,15 +525,12 @@ static PyObject *World_getName( BPy_World * self )
 static PyObject *World_setName( BPy_World * self, PyObject * args )
 {
 	char *name = 0;
-	char buf[21];
 	if( !PyArg_ParseTuple( args, "s", &name ) )
 		return ( EXPP_ReturnPyObjError
 			 ( PyExc_TypeError, "expected string argument" ) );
-	snprintf( buf, sizeof( buf ), "%s", name );
-	rename_id( &self->world->id, buf );
+	rename_id( &self->world->id, name );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 
@@ -574,8 +569,7 @@ static PyObject *World_setSkytype( BPy_World * self, PyObject * args )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"expected int argument" ) );
 	self->world->skytype = (short)skytype;
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 
@@ -611,8 +605,7 @@ static PyObject *World_setMode( BPy_World * self, PyObject * args )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"expected int argument" ) );
 	self->world->mode = (short)mode;
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 
@@ -660,8 +653,7 @@ static PyObject *World_setMistype( BPy_World * self, PyObject * args )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"expected int argument" ) );
 	self->world->mistype = (short)mistype;
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 
@@ -690,8 +682,7 @@ static PyObject *World_setHor( BPy_World * self, PyObject * args )
 	self->world->horr = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
 	self->world->horg = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
 	self->world->horb = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 
@@ -717,8 +708,7 @@ static PyObject *World_setZen( BPy_World * self, PyObject * args )
 	self->world->zenr = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
 	self->world->zeng = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
 	self->world->zenb = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 
@@ -752,8 +742,7 @@ static PyObject *World_setAmb( BPy_World * self, PyObject * args )
 	self->world->ambr = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
 	self->world->ambg = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
 	self->world->ambb = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 
@@ -797,8 +786,7 @@ static PyObject *World_setStar( BPy_World * self, PyObject * args )
 		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 5 ) );
 	self->world->starcolnoise =
 		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 6 ) );
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 
@@ -839,8 +827,7 @@ static PyObject *World_setMist( BPy_World * self, PyObject * args )
 		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
 	self->world->misthi =
 		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 3 ) );
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 /* world.addScriptLink */
@@ -893,8 +880,34 @@ static PyObject *World_setCurrent( BPy_World * self )
 		G.scene->world->id.us--;
 	world->id.us++;
 	G.scene->world = world;
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
+}
+
+/* world.__copy__ */
+static PyObject *World_copy( BPy_World * self )
+{
+	BPy_World *pyworld;
+	World *blworld;
+
+	blworld = copy_world( self->world );
+
+	if( blworld ) {
+		/* return user count to zero because add_world() inc'd it */
+		blworld->id.us = 0;
+		/* create python wrapper obj */
+		pyworld =
+			( BPy_World * ) PyObject_NEW( BPy_World, &World_Type );
+	} else
+		return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
+						"couldn't create World Data in Blender" ) );
+
+	if( pyworld == NULL )
+		return ( EXPP_ReturnPyObjError( PyExc_MemoryError,
+						"couldn't create World Data object" ) );
+
+	pyworld->world = blworld;
+
+	return ( PyObject * ) pyworld;
 }
 
 
@@ -1003,8 +1016,7 @@ static int World_SetAttr( BPy_World * self, char *name, PyObject * value )
 
 static int World_Compare( BPy_World * a, BPy_World * b )
 {
-	World *pa = a->world, *pb = b->world;
-	return ( pa == pb ) ? 0 : -1;
+	return ( a->world == b->world ) ? 0 : -1;
 }
 
 /**
@@ -1150,5 +1162,5 @@ static PyObject *World_insertIpoKey( BPy_World * self, PyObject * args )
 	EXPP_allqueue(REDRAWACTION, 0);
 	EXPP_allqueue(REDRAWNLA, 0);
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
