@@ -6,6 +6,8 @@ The Blender Python API Reference
 
  An asterisk (*) means the module has been updated.
 
+ for a full list of changes since 2.42 see U{http://mediawiki.blender.org/index.php/Release_Notes/Notes243/Python_API}
+
  Top Module:
  -----------
 
@@ -13,41 +15,41 @@ The Blender Python API Reference
 
  Submodules:
  -----------
-  - L{Armature}
+  - L{Armature} (*)
      - L{NLA}
      - L{Action<NLA.Action>}
   - L{BezTriple} (*)
   - L{BGL}
-  - L{Camera}
-  - L{Curve}
-  - L{Draw}
+  - L{Camera} (*)
+  - L{Curve} (*)
+  - L{Draw} (*)
   - L{Effect}
   - L{Geometry} (*)
   - L{Group} (*)
-  - L{Image}
+  - L{Image} (*)
   - L{Ipo} (*)
     - L{IpoCurve} (*)
   - L{Key} (*)
   - L{Lamp}
-  - L{Lattice}
-  - L{Library}
+  - L{Lattice} (*)
+  - L{Library} (*)
   - L{Material} (*)
   - L{Mathutils} (*)
   - L{Mesh} (*)
     - L{MeshPrimitives} (*)
-  - L{Metaball}
-  - L{Modifier} (*)
+  - L{Metaball} (*)
   - L{NMesh} (*)
   - L{Noise}
   - L{Object} (*)
-     - L{Pose}
+     - L{Modifier} (*)
+     - L{Pose} (*)
      - L{Constraint} (*)
      - L{ActionStrips<NLA>} (*)
   - L{Registry}
-  - L{Scene}
+  - L{Scene} (*)
      - L{Radio}
-     - L{Render}
-  - L{Sound}
+     - L{Render} (*)
+  - L{Sound} (*)
   - L{Text}
   - L{Text3d}
      - L{Font}
@@ -55,7 +57,7 @@ The Blender Python API Reference
   - L{TimeLine}
   - L{Types}
   - L{Window}
-     - L{Theme}
+     - L{Theme} (*)
   - L{World}
   - L{sys<Sys>}
 
@@ -222,16 +224,16 @@ A note to newbie script writers:
  to get an idea of what can be done, you may be surprised.
 
 @author: The Blender Python Team
-@requires: Blender 2.42 or newer.
-@version: 2.42
+@requires: Blender 2.43 or newer.
+@version: 2.43
 @see: U{www.blender3d.org<http://www.blender3d.org>}: main site
 @see: U{www.blender.org<http://www.blender.org>}: documentation and forum
-@see: U{www.elysiun.com<http://www.elysiun.com>}: user forum
+@see: U{blenderartists.org<http://blenderartists.org>}: user forum
 @see: U{projects.blender.org<http://projects.blender.org>}
 @see: U{blender architecture<http://www.blender3d.org/cms/Blender_Architecture.336.0.html>}: blender architecture document
 @see: U{www.python.org<http://www.python.org>}
 @see: U{www.python.org/doc<http://www.python.org/doc>}
-@see: U{en.wikibooks.org/wiki/Blender_3D:_Blending_Into_Python>}: User contributed documentation, featuring a blender/python cookbook with many examples.
+@see: U{Blending into Python<en.wikibooks.org/wiki/Blender_3D:_Blending_Into_Python>}: User contributed documentation, featuring a blender/python cookbook with many examples.
 @note: this documentation was generated by epydoc, which can output html and
    pdf.  For pdf it requires a working LaTeX environment.
 
