@@ -8,32 +8,70 @@ Curve Data
 
 This module provides access to B{Curve Data} objects in Blender.
 
-A Blender Curve can consist of multiple curves. Try converting a Text object to a Curve to see an example of this.   Each curve is of
-type Bezier or Nurb.  The underlying curves can be accessed with
-the [] operator.  Operator [] returns an object of type CurNurb.
+A Blender Curve Data consists of multiple L{CurNurb}(s). Try converting a Text object to a Curve to see an example of this.   Each curve is of
+type Bezier or Nurb.  The underlying L{CurNurb}(s) can be accessed with
+the [] operator.  Operator [] returns an object of type L{CurNurb}.
+
+Note that L{CurNurb} can be used to acces a curve of any type (Poly, Bezier or Nurb)
 
 The Curve module also supports the Python iterator interface.  This means you
-can access the curves in a Curve and the control points in a CurNurb using a
+can access the L{CurNurb}(s) in a Curve and the control points in a L{CurNurb} using a
 Python B{for} statement.
 
 
 Add a Curve to a Scene Example::
   from Blender import Curve, Object, Scene
-  c = Curve.New()             # create new  curve data
-  cur = Scene.getCurrent()    # get current scene
-  ob = Object.New('Curve')    # make curve object
-  ob.link(c)                  # link curve data with this object
-  cur.link(ob)                # link object into scene
+  cu = Curve.New()             # create new  curve data
+  scn = Scene.GetCurrent()    # get current scene
+  ob = scn.objects.new(cu)     # make a new curve from the curve data
 
 Iterator Example::
-  ob = Object.GetSelected()[0]
-  curve = ob.getData()
-  for cur in curve:
-    print type( cur ), cur
-    for point in cur:
+  from Blender import Curve, Object, Scene
+  scn = Scene.GetCurrent()    # get current scene
+  ob = scn.objects.active
+  curvedata = ob.data
+  for curnurb in curvedata:
+    print type( curnurb ), curnurb
+    for point in curnurb:
       print type( point ), point
 
-
+Creating a Curve from a list of Vec triples Examples::
+  from Blender import *
+  def bezList2Curve(bezier_vecs):
+    '''
+    Take a list or vector triples and converts them into a bezier curve object
+    '''
+    
+    def bezFromVecs(vecs):
+    	'''
+    	Bezier triple from 3 vecs, shortcut functon
+    	'''
+    	bt= BezTriple.New(\
+    	vecs[0].x, vecs[0].y, vecs[0].z,\
+    	vecs[1].x, vecs[1].y, vecs[1].z,\
+    	vecs[2].x, vecs[2].y, vecs[2].z)
+    	
+    	bt.handleTypes= (BezTriple.HandleTypes.FREE, BezTriple.HandleTypes.FREE)
+    	
+    	return bt
+    
+    # Create the curve data with one point
+    cu= Curve.New()
+    cu.appendNurb(bezFromVecs(bezier_vecs[0])) # We must add with a point to start with
+    cu_nurb= cu[0] # Get the first curve just added in the CurveData
+    
+    
+    i= 1 # skip first vec triple because it was used to init the curve
+    while i<len(bezier_vecs):
+    	bt_vec_triple= bezier_vecs[i]
+    	bt= bezFromVecs(bt_vec_triple)
+    	cu_nurb.append(bt)
+    	i+=1
+    
+    # Add the Curve into the scene
+    scn= Scene.GetCurrent()
+    ob = scn.objects.new(cu)
+  	return ob
 """
 
 def New ( name):
@@ -60,24 +98,24 @@ class Curve:
   """
   The Curve Data object
   =====================
-  This object gives access to Curve-specific data in Blender.
+  This object gives access to Curve and Surface data linked from Blender Objects.
   
   @ivar name: The Curve Data name.
   @type name: string
-  @ivar pathlen: The Curve Data path length.
+  @ivar pathlen: The Curve Data path length, used to set the number of frames for an animation (not the physical length).
   @type pathlen: int
-  @ivar totcol: The Curve Data maximal number of linked materials.
+  @ivar totcol: The Curve Data maximal number of linked materials. Read-only.
   @type totcol: int
   @ivar flag: The Curve Data flag value; see L{getFlag()} for the semantics.
-  @ivar bevresol: The Curve Data bevel resolution.
-  @type bevresol: float
-  @ivar resolu: The Curve Data U-resolution.
-  @type resolu: float
-  @ivar resolv: The Curve Data V-resolution.
-  @type resolv: float
-  @ivar width: The Curve Data width.
+  @ivar bevresol: The Curve Data bevel resolution. [0 - 32]
+  @type bevresol: int
+  @ivar resolu: The Curve Data U-resolution (used for curve and surface resolution) [0 - 1024].
+  @type resolu: int
+  @ivar resolv: The Curve Data V-resolution (used for surface resolution) [0 - 1024].
+  @type resolv: int
+  @ivar width: The Curve Data width [0 - 2].
   @type width: float
-  @ivar ext1: The Curve Data extent 1(for bevels).
+  @ivar ext1: The Curve Data extent1 (for bevels).
   @type ext1: float
   @ivar ext2: The Curve Data extent2 (for bevels).
   @type ext2: float
@@ -91,6 +129,15 @@ class Curve:
   @type bevob: Blender L{Object<Object.Object>} or None
   @ivar key: The Key object associated with this Curve, if any.
   @type key: Blender L{Key<Key.Key>}
+  @ivar materials: The curves's materials.  Each curve can reference up to
+    16 materials.  Empty slots in the curve's list are represented by B{None}.
+    B{Note}: L{Object.colbits<Object.Object.colbits>} needs to be set correctly
+    for each object in order for these materials to be used instead of
+    the object's materials.
+    B{Note}: The list that's returned is I{not} linked to the original curve.
+    curve.materials.append(material) won't do anything.
+    Use curve.materials += [material] instead.
+  @type materials: list of L{Material}s
   """
 
   def getName():
@@ -109,7 +156,7 @@ class Curve:
 
   def getPathLen():
     """
-    Get this Curve's path length.
+    Get this Curve's path frame length, used for an animated path.
     @rtype: int
     @return: the path length.
     """
@@ -131,10 +178,15 @@ class Curve:
 
   def setTotcol(totcol):
     """
-    Set the number of materials linked to the Curve.
+    Set the number of materials linked to the Curve.  B{Note}: this method
+    will probably be deprecated in the future.
     @rtype: None
     @type totcol: int
     @param totcol: number of materials linked.
+    @warn: It is not advisable to use this method unless you know what you
+    are doing; it's possible to
+    corrupt a .blend file if you don't know what you're doing.  If you want
+    to change the number of materials, use the L{materials} attribute.
     """
 
   def getFlag():
@@ -180,7 +232,8 @@ class Curve:
 
   def setResolu(resolu):
     """
-    Set the Curve's U-resolution value.
+    Set the Curve's U-resolution value. [0 - 1024]
+    This is used for surfaces and curves.
     @rtype: None
     @type resolu: float
     @param resolu: The new Curve's U-resolution value.
@@ -194,7 +247,8 @@ class Curve:
 
   def setResolv(resolv):
     """
-    Set the Curve's V-resolution value.
+    Set the Curve's V-resolution value. [0 - 1024].
+    This is used for surfaces only.
     @rtype: None
     @type resolv: float
     @param resolv: The new Curve's V-resolution value.
@@ -414,6 +468,20 @@ class Curve:
       @raise AttributeError:  throws exception if curve_num is out of range.
       """
 
+  def switchDirection( ):
+      """
+      Reverse the direction of a curve.
+      @return: None
+
+      I{B{Example:}}
+      # This example switches the direction of all curves on the active object.
+      from Blender import *
+      scn = Scene.GetCurrent()
+      ob = scn.objects.active # must be a curve
+      data = ob.data
+      for cu in data: cu.switchDirection()
+      """
+
   def getNumCurves():
       """
       Get the number of curves in this Curve Data object.
@@ -436,17 +504,31 @@ class Curve:
       @rtype: L{Key<Key.Key>} object or None
       """
 
+  def recalc():
+      """
+      Recalculate control point handles after a curve has been changed.
+      @rtype: None
+      """
+
+  def __copy__ ():
+    """
+    Make a copy of this curve
+    @rtype: Curve
+    @return:  a copy of this curve
+    """
 
 class CurNurb:
     """
     The CurNurb Object
     ==================
-    This object provides access to the control points of the curves that make up a Blender Curve.
+    This object provides access to the control points of the curves that make up a Blender Curve ObData.
 
     The CurNurb supports the python iterator protocol which means you can use a python for statement to access the points in a curve.
 
     The CurNurb also supports the sequence protocol which means you can access the control points of a CurNurb using the [] operator.
 
+    Note that CurNurb is used for accesing poly, bezier and nurbs type curves.
+
     @ivar flagU: The CurNurb knot flag U.  See L{setFlagU} for description.
     @type flagU: int
     @ivar flagV: The CurNurb knot flag V.  See L{setFlagU} for description.
@@ -454,10 +536,10 @@ class CurNurb:
     @ivar type: The type of the curve (Poly: 0, Bezier: 1, NURBS: 4)
     @type type: int
     """
-	
+
     def __setitem__( n, point ):
       """
-	  Replace the Nth point in the curve. The type of the argument must match the type of the curve. List of 4 floats (optional 5th float is the tilt value in radians) for Nurbs or BezTriple for Bezier.
+      Replace the Nth point in the curve. The type of the argument must match the type of the curve. List of 4 floats (optional 5th float is the tilt value in radians) for Nurbs or BezTriple for Bezier.
       @rtype: None
       @return: None
       @type n: integer
@@ -468,7 +550,7 @@ class CurNurb:
 
     def __getitem__( n ):
       """
-	  Get the Nth element in the curve. For Bezier curves, that element is a BezTriple. For the rest (Poly and Nurbs), it is a list of 5 floats: x, y, z, weight, tilt (in radians). NOTE 1: This element element is independant on the curve, modifying it will not affect the curve. NOTE 2: Each successive call returns a new object.
+      Get the Nth element in the curve. For Bezier curves, that element is a BezTriple. For the rest (Poly and Nurbs), it is a list of 5 floats: x, y, z, weight, tilt (in radians). NOTE 1: This element is independent on the curve, modifying it will not affect the curve. NOTE 2: Each successive call returns a new object.
       @rtype: BezTriple (Bezier Curve) or List of 5 floats [x, y, z, w, t] for Poly or Nurbs
       @return: The Nth element in the curve
       @type n: integer
@@ -512,7 +594,7 @@ class CurNurb:
       Boolean method checks whether a CurNurb is cyclic (a closed curve) or not.
       @rtype: boolean
       @return: True or False
-	  """
+      """
 
     def getFlagU():
       """
@@ -566,3 +648,66 @@ class CurNurb:
       @rtype: None
       @return: None
       """
+
+class SurfNurb:
+  """
+  The SurfNurb Object
+  ===================
+  This object provides access to the control points of the surfaces that make
+  up a Blender Curve.
+
+  The SurfNurb supports the Python iterator and sequence protocols which
+  means you can use a python B{for} statement or [] operator to access the
+  points in a surface.  Points are accessed linearly; for a N-by-M UV surface,
+  the first N control points correspond to V=0, then second N to V=1, and so
+  on.
+
+  @ivar flagU: The knot flag U.  Changing the knot type automatically
+  recalculates the knots.  The flag can be one of three values:
+           - 0 : uniform knots
+           - 1 : endpoints knots
+           - 2 : bezier knots
+  @type flagU: int
+  @ivar flagV: The knot flag V.  See L{flagU} for description.
+  @type flagV: int
+  @ivar pointsU: The number of control points in the U direction (read only).
+  @type pointsU: int
+  @ivar pointsV: The number of control points in the V direction (read only).
+  @type pointsV: int
+  @ivar cyclicU: The cyclic setting for the U direction (True = cyclic).
+  @type cyclicU: boolean
+  @ivar cyclicV: The cyclic setting for the V direction (True = cyclic).
+  @type cyclicV: boolean
+  @ivar orderU: The order setting for the U direction.  Values are clamped
+  to the range [2:6] and not greater than the U dimension.
+  @type orderU: int
+  @ivar orderV: The order setting for the V direction.  Values are clamped
+  to the range [2:6] and not greater than the V dimension.
+  @type orderV: int
+  """
+
+  def __setitem__( n, point ):
+      """
+      Set the Nth control point in the surface. 
+      @rtype: None
+      @return: None
+      @type n: integer
+      @param n: the index of the point to replace
+      @type point: list of 4 floats (optional 5th float is the tilt value
+      in radians)
+      @param point: the point that will replace the one in the curve.  The
+      point is  list of 4 floats in x,y,z,w (optionally tilt in radians as
+      5th value) format.
+      """
+
+  def __getitem__( n ):
+      """
+      Get the Nth control point in the surface. 
+      @rtype: List of 5 floats [x, y, z, w, t] for Poly or Nurbs
+      @return: The Nth point in the curve
+      @type n: integer
+      @param n: the index of the point to return
+      @note: This returned value is independent on the curve; modifying it will not affect the curve. 
+      @note: Each successive call returns a new object.
+      """
+
