@@ -64,6 +64,9 @@ Example::
     - TANGENTSTR   - Uses direction of strands as normal for tangent-shading.
     - TRANSPSHADOW - Lets Material receive transparent shadows based on material color and alpha.
     - FULLOSA      - Force rendering of all OSA samples.
+    - TANGENT_V    - Use the tangent vector in V direction for shading
+    - NMAP_TS      - Tangent space normal mapping.
+    - GROUP_EXCLUSIVE	- Light from this group even if the lights are on a hidden Layer.
 
 @type Shaders: readonly dictionary
 @var Shaders: The available Material Shaders.
@@ -103,6 +106,8 @@ class Material:
   The Material object
   ===================
    This object gives access to Materials in Blender.
+  @ivar properties: Returns an L{IDGroup<IDProp.IDGroup>} reference to this 
+  material's ID Properties.
   @ivar B:  Diffuse color (L{rgbCol}) blue component.
   Value is clamped to the range [0.0,1.0].
   @type B:  float
@@ -121,6 +126,9 @@ class Material:
   @ivar alpha:  Alpha (translucency) component of the material.
   Value is clamped to the range [0.0,1.0].
   @type alpha:  float
+  @ivar shadAlpha:  Shadow Alpha for irregular shadow buffer.
+  Value is clamped to the range [0.0,1.0].
+  @type shadAlpha:  float
   @ivar amb:  Amount of global ambient color material receives.
   Value is clamped to the range [0.0,1.0].
   @type amb:  float
@@ -273,9 +281,14 @@ class Material:
   @type translucency:  float
   @ivar users:  Number of material users.
   @type users:  int
+  @ivar fakeUser: The fake user status.
+    enabling this will keep it in the blend even if there are no users.
+  @type fakeUser: bool
   @ivar zOffset:  Artificial offset in the Z buffer (for Ztransp option).
   Value is clamped to the range [0.0,10.0].
   @type zOffset:  float
+  @ivar lightGroup:  Limits lights that affect this material to a group.
+  @type lightGroup:  Group or None
   @warning: Most member variables assume values in some [Min, Max] interval.
    When trying to set them, the given parameter will be clamped to lie in
    that range: if val < Min, then val = Min, if val > Max, then val = Max.
@@ -980,3 +993,10 @@ class Material:
     @type event: string
     @param event: "FrameChanged" or "Redraw".
     """
+
+  def __copy__ ():
+    """
+    Make a copy of this material
+    @rtype: Material
+    @return:  a copy of this material
+    """
\ No newline at end of file
