@@ -17,32 +17,29 @@ face's attributes (the vertex color):
 
 Example::
 
-  from Blender import *
+ from Blender import *
 
-  editmode = Window.EditMode()    # are we in edit mode?  If so ...
-  if editmode: Window.EditMode(0) # leave edit mode before getting the mesh
+ editmode = Window.EditMode()    # are we in edit mode?  If so ...
+ if editmode: Window.EditMode(0) # leave edit mode before getting the mesh
 
-  # define vertices and faces for a pyramid
-  coords=[ [-1,-1,-1], [1,-1,-1], [1,1,-1], [-1,1,-1], [0,0,1] ]	
-  faces= [ [3,2,1,0], [0,1,4], [1,2,4], [2,3,4], [3,0,4] ]
+ # define vertices and faces for a pyramid
+ coords=[ [-1,-1,-1], [1,-1,-1], [1,1,-1], [-1,1,-1], [0,0,1] ]	
+ faces= [ [3,2,1,0], [0,1,4], [1,2,4], [2,3,4], [3,0,4] ]
 
-  me = Mesh.New('myMesh')          # create a new mesh
+ me = Mesh.New('myMesh')          # create a new mesh
 
-  me.verts.extend(coords)          # add vertices to mesh
-  me.faces.extend(faces)           # add faces to the mesh (also adds edges)
+ me.verts.extend(coords)          # add vertices to mesh
+ me.faces.extend(faces)           # add faces to the mesh (also adds edges)
 
-  me.vertexColors = 1              # enable vertex colors 
-  me.faces[1].col[0].r = 255       # make each vertex a different color
-  me.faces[1].col[1].g = 255
-  me.faces[1].col[2].b = 255
+ me.vertexColors = 1              # enable vertex colors 
+ me.faces[1].col[0].r = 255       # make each vertex a different color
+ me.faces[1].col[1].g = 255
+ me.faces[1].col[2].b = 255
 
-  ob = Object.New('Mesh','myObj')  # link mesh to an object
-  ob.link(me)
+ scn = Scene.GetCurrent()          # link object to current scene
+ ob = scn.objects.new(me, 'myObj')
 
-  sc = Scene.GetCurrent()          # link object to current scene
-  sc.link(ob)
-
-  if editmode: Window.EditMode(1)  # optional, just being nice
+ if editmode: Window.EditMode(1)  # optional, just being nice
 
 Vertices, edges and faces are added to a mesh using the .extend() methods.
 For best speed and efficiency, gather all vertices, edges or faces into a
@@ -60,13 +57,13 @@ done once.
     - AUTOSMOOTH - turn auto smoothing of faces "on".
     - note: SUBSURF and OPTIMAL have been removed, use Modifiers to apply subsurf.
 @var FaceFlags: The available *texture face* (uv face select mode) selection
-  flags.  Note: these refer to TexFace faces, available if mesh.faceUV()
+  flags.  Note: these refer to TexFace faces, available if mesh.faceUV
   returns true.
     - SELECT - selected.
     - HIDE - hidden.
     - ACTIVE - the active face, read only - Use L{mesh.activeFace<Mesh.Mesh.activeFace>} to set.
 @var FaceModes: The available *texture face* modes. Note: these are only
-  meaningful if mesh.faceUV() returns true, since in Blender this info is
+  meaningful if mesh.faceUV returns true, since in Blender this info is
   stored at the TexFace (TexFace button in Edit Mesh buttons) structure.
     - ALL - set all modes at once.
     - BILLBOARD - always orient after camera.
@@ -94,6 +91,7 @@ done once.
     - SEAM - edge is a seam for UV unwrapping
     - FGON - edge is part of a F-Gon.
     - LOOSE - Edge is not a part of a face (only set on leaving editmode)
+    - SHARP - Edge will be rendered sharp when used with the "Edge Split" modifier.
 @type AssignModes: readonly dictionary.
 @var AssignModes: The available vertex group assignment modes, used by 
   L{mesh.assignVertsToGroup()<Mesh.Mesh.assignVertsToGroup>}.
@@ -190,7 +188,7 @@ class MVert:
     vector will not effect the verts normal. The result is only visible
     when faces have the smooth option enabled.
     Example::
-      # This wont work.
+      # This won't work.
       for v in me.verts:
         v.no.x= 0
         v.no.y= 0
@@ -380,6 +378,10 @@ class MEdge:
   changing the select state of an edge changes the select state of the edge's
   vertices.
   @type sel: int
+  @ivar key: The edge's vert indices in an ordered tuple, which can be used
+  as a dictionary key. Read-only.
+  This is the same as (min(ed.v1.index, ed.v2.index), max(ed.v1.index, ed.v2.index))
+  @type key: tuple
   """
 
   def __iter__():
@@ -532,7 +534,8 @@ class MFace:
       Getting this attribute throw an exception if the mesh does not have 
       UV faces; use L{Mesh.faceUV} to test.  
       Assigning an image will automatically set the TEX attribute of the
-      L{mode} bitfield.
+      L{mode} bitfield.  Use "del f.image" or "f.image = None" to clear the
+      image assigned to the face.
   @type image: Image
   @ivar mode: The texture mode bitfield (see L{FaceModes}).
       Will throw an exception if the mesh does not have UV faces; use
@@ -561,7 +564,7 @@ class MFace:
       Setting this attribute will create UV faces if they do not exist.
       Getting this attribute throw an exception if the mesh does not have 
       UV faces; use L{Mesh.faceUV} to test.  
-  @type uv: list of vectors (WRAPPED DATA)
+  @type uv: tuple of vectors (WRAPPED DATA)
   @ivar uvSel: The face's UV coordinates selection state; a 1 indicates the
       vertex is selected.  Each vertex has its own UV coordinate select state
       (this is not the same as the vertex's edit mode selection state).
@@ -575,6 +578,24 @@ class MFace:
   @type cent: vector
   @ivar area: The area of the face. Read-only.
   @type area: float
+  @ivar edge_keys: A tuple, each item a key that can reference an edge by its
+  ordered indices. Read-only.  This is useful for building connectivity data.
+  Example::
+       from Blender import Mesh
+       me = Mesh.Get('Cube')
+       # a dictionary where the edge is the key, and a list of faces that use it are the value
+       edge_faces = dict([(ed.key, []) for ed in me.edges])
+
+       # Add the faces to the dict
+       for f in me.faces:
+           for key in f.edge_keys:
+               edge_faces[key].append(f) # add this face to the edge as a user
+
+       # Print the edges and the number of face users
+       for key, face_users in edge_faces.iteritems():
+           print 'Edge:', key, 'uses:', len(face_users),'faces'
+
+  @type edge_keys: tuple
   @note: there are regular faces and textured faces in Blender, both currently
     with their own selection and visibility states, due to a mix of old and new
     code.  To (un)select or (un)hide regular faces (visible in EditMode), use
@@ -613,7 +634,7 @@ class MFaceSeq:
     This object provides sequence and iterator access to the mesh's faces.
   """
 
-  def extend(vertseq):
+  def extend(vertseq,ignoreDups=True,indexList=True):
     """
     Add zero or more faces and edges to the mesh.  Faces which already exist
     in the mesh, or faces which contain the same vertex multiple times are
@@ -634,6 +655,22 @@ class MFaceSeq:
     @type vertseq: sequence(s) of MVerts
     @param vertseq: either two to four ints or MVerts, or sequence (list or
     tuple) of sequences each containing two to four ints or MVerts.
+    @type ignoreDups: boolean
+    @param ignoreDups: keyword parameter (default is False).  If supplied and
+    True, do not check the input list or mesh for duplicate faces.  This can
+    speed up scripts but can prossibly produce undesirable effects.  Only
+    use if you know what you're doing.
+    @type indexList: boolean
+    @param indexList: keyword parameter (default is False).  If supplied and
+    True, the method will return a list representing the new index for each
+    face in the input list.  If faces are removed as duplicates, None is
+    inserted in place of the index.
+    @warning: Faces using the first vertex at the 3rd or 4th location in the
+    face's vertex list will have their order rotated so that the zero index
+    on in the first or second location in the face. When creating face data
+    with UVs or vertex colors, you may need to work around this, either by
+    checking for zero indices yourself or by adding a dummy first vertex to
+    the mesh that can be removed when your script has finished.
     """
 
   def delete(deledges, faces):
@@ -658,15 +695,20 @@ class MFaceSeq:
     @rtype: list of ints
     """
 
+from IDProp import IDGroup, IDArray
 class Mesh:
   """
   The Mesh Data object
   ====================
     This object gives access to mesh data in Blender.
+
   @note: the verts, edges and faces attributes are implemented as sequences.
   The operator[] and len() are defined for these sequences.  You cannot
   assign to an item in the sequence, but you can assign to most of the
   attributes of individual items.
+  @ivar properties: Returns an L{IDGroup<IDProp.IDGroup>} reference to this 
+  mesh's ID Properties.
+  @type properties: L{IDGroup<IDProp.IDGroup>}
   @ivar edges: The mesh's edges.
   @type edges: sequence of MEdges
   @ivar faces: The mesh's faces.
@@ -679,12 +721,12 @@ class Mesh:
     B{Note}: L{Object.colbits<Object.Object.colbits>} needs to be set correctly
     for each object in order for these materials to be used instead of
     the object's materials.
-    B{Note}: Making the material list shorter does not change the faces material indices,
-    take care when using the faces material indices to reference a material in the materials list.
-    B{Note}: The the list thats returned is not linked to the original mesh.
-    mesh.materials.append(material) wont do anything.
+    B{Note}: Making the material list shorter does not change the face's material indices.
+    Take care when using the face's material indices to reference a material in this list.
+    B{Note}: The list that's returned is I{not} linked to the original mesh.
+    mesh.materials.append(material) won't do anything.
     Use mesh.materials += [material] instead.
-  @type materials: list of Materials
+  @type materials: list of L{Material}s
   @ivar degr: The max angle for auto smoothing in [1,80].  
   @type degr: int
   @ivar maxSmoothAngle: Same as L{degr}.  This attribute is only for
@@ -703,8 +745,11 @@ class Mesh:
   @type name: str
   @ivar subDivLevels: The [display, rendering] subdivision levels in [1, 6].
   @type subDivLevels: list of 2 ints
-  @ivar users: The number of Objects using (linked to) this mesh.
+  @ivar users: The number of Objects using (linked to) this mesh. (read only)
   @type users: int
+  @ivar fakeUser: The fake user status.
+    enabling this will keep it in the blend even if there are no users.
+  @type fakeUser: bool
 
   @ivar faceUV: The mesh contains UV-mapped textured faces.  Enabling faceUV
     does not initialize the face colors like the Blender UI does; this must
@@ -730,9 +775,35 @@ class Mesh:
   @ivar activeGroup: The mesh's active vertex group.  The mesh must be
     linked to an object (read the comment in L{addVertGroup} for more info).
   @type activeGroup: string or None
+  @ivar texMesh: The mesh's texMesh setting, used so coordinates from another
+    mesh can be used for rendering textures.
+  @type texMesh: Mesh or None
+  @ivar key: The L{Key<Key.Key>} object containing the keyframes for this mesh, if any.
+  @type key: Key or None
+
+  @ivar activeUVLayer: The mesh's active UV/Image layer. None if there is no UV/Image layers.
+  @type activeUVLayer: string
+  @ivar activeColorLayer: The mesh's active Vertex Color layer. None if there is no UV/Image layers.
+  @type activeColorLayer: string
+
+  @ivar multires: The mesh has multires data, set True to add multires data.
+    Will throw an exception if the mesh has shape keys; use L{key} to test.
+  @type multires: bool
+  @ivar multiresLevelCount: The mesh has multires data. (read only)
+  @type multiresLevelCount: int
+  @ivar multiresDrawLevel: The multires level to display in the 3dview in [1 - multiresLevelCount].
+  @type multiresDrawLevel: int
+  @ivar multiresEdgeLevel: The multires level edge display in the 3dview [1 - multiresLevelCount].
+  @type multiresEdgeLevel: int
+  @ivar multiresPinLevel: The multires pin level, used for applying modifiers [1 - multiresLevelCount].
+  @type multiresPinLevel: int
+  @ivar multiresRenderLevel: The multires level to render [1 - multiresLevelCount].
+  @type multiresRenderLevel: int
+  
+  
   """
 
-  def getFromObject(name,cage=0):
+  def getFromObject(object, cage=0, render=0):
     """
     Replace the mesh's existing data with the raw mesh data from a Blender
     Object.  This method supports all the geometry based objects (mesh, text,
@@ -744,10 +815,12 @@ class Mesh:
     be multiplied by the object's 4x4 transform matrix (see L{transform}).
     @note: The objects materials will not be copied into the existing mesh,
     however the face material indices will match the material list of the original data.
-    @type name: string
-    @param name: name of the Blender object which contains the geometry data.
+    @type object: blender object or string
+    @param object: The Blender object or its name, which contains the geometry data.
     @type cage: int
     @param cage: determines whether the original vertices or derived vertices
+    @type render: int
+    @param render: determines whether the render setting for modifiers will be used or not.
     (for objects with modifiers) are used.  The default is derived vertices.
     """
 
@@ -812,6 +885,8 @@ class Mesh:
     place for using a directed acyclic graph (DAG) for scene and object
     updating, this method may be only temporary and may be removed in future
     releases.
+    @warn: Since Blender 2.42 this function has changed; now it won't recalculate
+    vertex normals (seen when faces are smooth). See L{Mesh.calcNormals()}.
     """
 
   def findEdges(edges):
@@ -935,6 +1010,22 @@ class Mesh:
     associated with the mesh's object
     """
 
+  def getUVLayerNames():
+    """
+    Return a list of all UV layer names
+    @rtype: list of strings
+    @return: returns a list of strings representing all UV layers
+    associated with the mesh's object
+    """
+
+  def getColorLayerNames():
+    """
+    Return a list of all color layer names
+    @rtype: list of strings
+    @return: returns a list of strings representing all color layers
+    associated with the mesh's object
+    """
+
   def getVertexInfluences(index):
     """
     Get the bone influences for a specific vertex.
@@ -945,6 +1036,77 @@ class Mesh:
         and weight is a float value.
     """
 
+  def removeAllKeys():
+    """
+    Remove all mesh keys stored in this mesh.
+    @rtype: bool
+    @return: True if successful or False if the Mesh has no keys.
+    """
+
+  def insertKey(frame = None, type = 'relative'):
+    """
+    Insert a mesh key at the given frame. 
+    @type frame: int
+    @type type: string
+    @param frame: The Scene frame where the mesh key should be inserted.  If
+        None or the arg is not given, the current frame is used.
+    @param type: The mesh key type: 'relative' or 'absolute'.  This is only
+        relevant on meshes with no keys.
+    @warn: This and L{removeAllKeys} were included in this release only to
+        make accessing vertex keys possible, but may not be a proper solution
+        and may be substituted by something better later.  For example, it
+        seems that 'frame' should be kept in the range [1, 100]
+        (the curves can be manually tweaked in the Ipo Curve Editor window in
+        Blender itself later).
+    @warn: Will throw an error if the mesh has multires. use L{multires} to check.
+    """
+
+  def addUVLayer(name):
+    """
+    Adds a new UV/Image layer to this mesh, it will always be the last layer but not made active.
+    @type name: string
+    @param name: The name of the new UV layer, 31 characters max.
+    """
+
+  def addColorLayer(name):
+    """
+    Adds a new Vertex Color layer to this mesh, it will always be the last layer but not made active.
+    @type name: string
+    @param name: The name of the new Color layer, 31 characters max.
+    """
+
+  def removeUVLayer(name):
+    """
+    Removes the active UV/Image layer.
+    @type name: string
+    @param name: The name of the UV layer to remove.
+    """
+
+  def removeColorLayer(name):
+    """
+    Removes the active Vertex Color layer.
+    @type name: string
+    @param name: The name of the Color layer to remove.
+    """
+
+  def renameUVLayer(name, newname):
+    """
+    Removes the active UV/Image layer.
+    @type name: string
+    @param name: The UV layer to rename.
+    @type newname: string
+    @param newname: The new name of the UV layer, will be made unique.
+    """
+
+  def renameColorLayer(name, newname):
+    """
+    Removes the active Vertex Color layer.
+    @type name: string
+    @param name: The Color layer to rename.
+    @type newname: string
+    @param newname: The new name of the Color layer, will be made unique.
+    """
+
   def smooth():
     """
     Flattens angle of selected faces. Experimental mesh tool.
@@ -1014,3 +1176,10 @@ class Mesh:
     @param direction: specifies outward (0) or inward (1) normals.  Outward
     is the default.  Value must be in the range [0,1].
     """
+
+  def __copy__ ():
+    """
+    Make a copy of this mesh
+    @rtype: Mesh
+    @return:  a copy of this mesh
+    """
