@@ -6,55 +6,9 @@ The Blender.Armature.NLA submodule.
 NLA
 ===
 
-This module provides access to B{Action} objects in Blender.  Actions are a series of keyframes/Ipo curves
-that define the movement of a bone. Actions are linked to objects of type armature.
-
-Example::
-  import Blender
-  from Blender import *
-  from Blender.Armature import *
-  from Blender.Armature.Bone import *
-
-  armObj = Object.New('Armature', "Armature_obj")
-  armData = Armature.New()
-
-  bn1=Blender.Armature.Bone.New("bone1")
-  bn1.setHead(0.0,0.0,0.0)
-  bn1.setTail(2.0,1.0,0.0)
-
-  bn2=Blender.Armature.Bone.New("bone2")
-  bn2.setHead(3.0,2.0,1.0)
-  bn2.setTail(4.0,4.0,1.0)
-  bn2.setRoll(.5)
-  bn2.setParent(bn1)
-
-  armData.addBone(bn1)
-  armData.addBone(bn2)
-
-  armObj.link(armData)
-  scn = Blender.Scene.getCurrent()
-  scn.link(armObj)
-
-  armObj.makeDisplayList()
-  Blender.Window.RedrawAll()
-
-  action = Blender.Armature.NLA.NewAction()
-  action.setActive(armObj)
-  
-  bn2.setPose([ROT,LOC,SIZE])  
-  
-  context = scn.getRenderingContext()
-  context.currentFrame(2)
-
-  quat = Blender.Mathutils.Quaternion([1,2,3,4])
-  bn2.setQuat(quat)
-  bn2.setLoc([3,4,5])
-  
-  bn2.setPose([ROT,LOC,SIZE])
-
-  print action.name
-  action2 = Blender.Armature.NLA.CopyAction(action)
-  action2.name = "Copy"
+This module provides access to B{Action} objects in Blender.  Actions are a
+series of keyframes/Ipo curves that define the movement of a bone. 
+Actions are linked to objects of type armature.
 
 @type Flags: readonly dictionary
 @var Flags: Constant dict used by the L{ActionStrip.flag} attribute.
@@ -166,17 +120,17 @@ class ActionStrips:
 
   def __getitem__(index):
     """
-    This operator returns one of the constraints in the stack.
+    This operator returns one of the action strips in the stack.
     @type index: int
-    @return: an Constraint object
-    @rtype: Constraint
+    @return: an action strip object
+    @rtype: ActionStrip
     @raise KeyError: index was out of range
     """
 
   def __len__():
     """
     Returns the number of action strips for the object.
-    @return: number of Constraints
+    @return: number of action strips
     @rtype: int
     """
 
@@ -185,8 +139,8 @@ class ActionStrips:
     Appends a new action to the end of the action strip sequence.
     @type action: L{Action<NLA.Action>}
     @param action: the action to use in the action strip
-    @rtype: Constraint
-    @return: the new Constraint
+    @rtype: ActionStrip
+    @return: the new action strip
     """
 
   def remove(actionstrip):
@@ -240,6 +194,8 @@ class ActionStrip:
   @type strideLength: float 
   @ivar strideBone: Name of Bone used for stride
   @type strideBone: string 
+  @ivar groupTarget: Armature object within DupliGroup for local animation
+  @type groupTarget: object
   @ivar blendIn: Number of frames of motion blending.
   @type blendIn: float 
   @ivar blendOut: Number of frames of ease-out.
@@ -258,7 +214,7 @@ class ActionStrip:
   def resetStripSize():
     """
     Activates the functionality found in NLA Strip menu under "Reset  Strip
-    Size".  This method resets the Action Strip size to its creation values.
+    Size".  This method resets the action strip size to its creation values.
     @rtype: None
     """
 
