@@ -1,5 +1,5 @@
 /* 
- * $Id: gen_utils.c,v 1.42 2006/05/01 05:28:51 khughes Exp $
+ * $Id: gen_utils.c,v 1.48 2006/12/27 05:04:20 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -38,6 +38,9 @@
 #include "BIF_space.h"
 #include "BKE_global.h"
 #include "BKE_main.h"
+#include "BKE_library.h"
+
+#include "Mathutils.h"
 
 #include "constant.h"
 
@@ -156,6 +159,34 @@ ID *GetIdFromList( ListBase * list, char *name )
 	return id;
 }
 
+/*****************************************************************************/
+/* Description: This function sets the fake user status of the ID            */
+/* returns an int error, so from getsetattrs                                 */
+/*****************************************************************************/
+int SetIdFakeUser( ID * id, PyObject *value)
+{
+	int param;
+	param = PyObject_IsTrue( value );
+	if( param == -1 )
+		return EXPP_ReturnIntError( PyExc_TypeError,
+				"expected int argument in range [0,1]" );
+	
+	if (param) {
+		if (!(id->flag & LIB_FAKEUSER)) {
+			id->flag |= LIB_FAKEUSER;
+			id_us_plus(id);
+		}
+	} else {
+		if (id->flag & LIB_FAKEUSER) {
+			id->flag &= ~LIB_FAKEUSER;
+			id->us--;
+		}
+	}
+	return 0;
+}
+
+
+
 /*****************************************************************************/
 /* Description: These functions set an internal string with the given type   */
 /*		  and error_msg arguments.				     */
@@ -654,13 +685,13 @@ int EXPP_setFloatClamped( PyObject *value, float *param,
 int EXPP_setIValueClamped( PyObject *value, void *param,
 								int min, int max, char type )
 {
-	char errstr[128];
 	int number;
 
-	sprintf ( errstr, "expected int argument in [%d,%d]", min, max );
-
-	if( !PyInt_CheckExact ( value ) )
+	if( !PyInt_CheckExact ( value ) ) {
+		char errstr[128];
+		sprintf ( errstr, "expected int argument in [%d,%d]", min, max );
 		return EXPP_ReturnIntError( PyExc_TypeError, errstr );
+	}
 
 	number = PyInt_AS_LONG( value );
 
@@ -683,6 +714,26 @@ int EXPP_setIValueClamped( PyObject *value, void *param,
 	}
 }
 
+int EXPP_setVec3Clamped( PyObject *value, float *param,
+								float min, float max )
+{	
+	if( VectorObject_Check( value ) ) {
+		VectorObject *vect = (VectorObject *)value;
+		if( vect->size == 3 ) {
+			param[0] = EXPP_ClampFloat( vect->vec[0], min, max );
+			param[1] = EXPP_ClampFloat( vect->vec[1], min, max );
+			param[2] = EXPP_ClampFloat( vect->vec[2], min, max );
+			return 0;
+		}
+	}
+	
+	if (1) {
+		char errstr[128];
+		sprintf ( errstr, "expected vector argument in [%f,%f]", min, max );
+		return EXPP_ReturnIntError( PyExc_TypeError, errstr );
+	}
+}
+
 
 /*
  * Utility routines to range-check and store various datatypes.  The object 
@@ -788,6 +839,7 @@ int EXPP_setModuleConstant ( BPy_constant *constant, void *param, char type )
 		return 0;
 	case 'f':
 		*(float *)param = (float)PyFloat_AS_DOUBLE( item );
+		Py_DECREF(item); /* line above increfs */
 		return 0;
 	default:
 		return EXPP_ReturnIntError( PyExc_RuntimeError,
