@@ -1,5 +1,5 @@
 /*
- * $Id: matrix.c,v 1.36 2006/04/27 12:32:44 campbellbarton Exp $
+ * $Id: matrix.c,v 1.42 2006/10/07 11:49:53 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -47,6 +47,7 @@ char Matrix_scalePart_doc[] = "() - convert matrix to a 3D vector";
 char Matrix_Resize4x4_doc[] = "() - resize the matrix to a 4x4 square matrix";
 char Matrix_toEuler_doc[] = "() - convert matrix to a euler angle rotation";
 char Matrix_toQuat_doc[] = "() - convert matrix to a quaternion rotation";
+char Matrix_copy_doc[] = "() - return a copy of the matrix";
 /*-----------------------METHOD DEFINITIONS ----------------------*/
 struct PyMethodDef Matrix_methods[] = {
 	{"zero", (PyCFunction) Matrix_Zero, METH_NOARGS, Matrix_Zero_doc},
@@ -60,6 +61,8 @@ struct PyMethodDef Matrix_methods[] = {
 	{"resize4x4", (PyCFunction) Matrix_Resize4x4, METH_NOARGS, Matrix_Resize4x4_doc},
 	{"toEuler", (PyCFunction) Matrix_toEuler, METH_NOARGS, Matrix_toEuler_doc},
 	{"toQuat", (PyCFunction) Matrix_toQuat, METH_NOARGS, Matrix_toQuat_doc},
+	{"copy", (PyCFunction) Matrix_copy, METH_NOARGS, Matrix_copy_doc},
+	{"__copy__", (PyCFunction) Matrix_copy, METH_NOARGS, Matrix_copy_doc},
 	{NULL, NULL, 0, NULL}
 };
 /*-----------------------------METHODS----------------------------*/
@@ -223,12 +226,13 @@ PyObject *Matrix_Invert(MatrixObject * self)
 
 	if(self->rowSize != self->colSize){
 		return EXPP_ReturnPyObjError(PyExc_AttributeError,
-			"Matrix.invert: only square matrices are supported\n");
+			"Matrix.invert(ed): only square matrices are supported\n");
 	}
 
 	/*calculate the determinant*/
 	f = Matrix_Determinant(self);
-	det = (float)PyFloat_AS_DOUBLE(f);
+	det = (float)PyFloat_AS_DOUBLE(f); /*Increfs, so we need to decref*/
+	Py_DECREF(f);
 
 	if(det != 0) {
 		/*calculate the classical adjoint*/
@@ -261,6 +265,8 @@ PyObject *Matrix_Invert(MatrixObject * self)
 	}
 	return EXPP_incr_ret((PyObject*)self);
 }
+
+
 /*---------------------------Matrix.determinant() ----------------*/
 PyObject *Matrix_Determinant(MatrixObject * self)
 {
@@ -293,7 +299,7 @@ PyObject *Matrix_Transpose(MatrixObject * self)
 
 	if(self->rowSize != self->colSize){
 		return EXPP_ReturnPyObjError(PyExc_AttributeError,
-			"Matrix.transpose: only square matrices are supported\n");
+			"Matrix.transpose(d): only square matrices are supported\n");
 	}
 
 	if(self->rowSize == 2) {
@@ -308,6 +314,8 @@ PyObject *Matrix_Transpose(MatrixObject * self)
 
 	return EXPP_incr_ret((PyObject*)self);
 }
+
+
 /*---------------------------Matrix.zero() -----------------------*/
 PyObject *Matrix_Zero(MatrixObject * self)
 {
@@ -341,6 +349,13 @@ PyObject *Matrix_Identity(MatrixObject * self)
 
 	return EXPP_incr_ret((PyObject*)self);
 }
+
+/*---------------------------Matrix.inverted() ------------------*/
+PyObject *Matrix_copy(MatrixObject * self)
+{
+	return (PyObject*)(MatrixObject*)newMatrixObject((float (*))*self->matrix, self->rowSize, self->colSize, Py_NEW);
+}
+
 /*----------------------------dealloc()(internal) ----------------*/
 /*free the py_object*/
 static void Matrix_dealloc(MatrixObject * self)
@@ -680,7 +695,6 @@ static PyObject *Matrix_mul(PyObject * m1, PyObject * m2)
 	double dot = 0.0f;
 	MatrixObject *mat1 = NULL, *mat2 = NULL;
 	PyObject *f = NULL;
-	VectorObject *vec = NULL;
 	PointObject *pt = NULL;
 
 	mat1 = (MatrixObject*)m1;
@@ -696,6 +710,7 @@ static PyObject *Matrix_mul(PyObject * m1, PyObject * m2)
 			}
 
 			scalar = (float)PyFloat_AS_DOUBLE(f);
+			Py_DECREF(f);
 			for(x = 0; x < mat2->rowSize; x++) {
 				for(y = 0; y < mat2->colSize; y++) {
 					mat[((x * mat2->colSize) + y)] = scalar * mat2->matrix[x][y];
@@ -705,10 +720,12 @@ static PyObject *Matrix_mul(PyObject * m1, PyObject * m2)
 		}
 	}else{
 		if(mat2->coerced_object){
-			if(VectorObject_Check(mat2->coerced_object)){ /*MATRIX * VECTOR*/
+			/* MATRIX * VECTOR   operation is now being done by vector */
+			/*if(VectorObject_Check(mat2->coerced_object)){ 
 				vec = (VectorObject*)mat2->coerced_object;
 				return column_vector_multiplication(mat1, vec);
-			}else if(PointObject_Check(mat2->coerced_object)){ /*MATRIX * POINT*/
+			}else */
+			if(PointObject_Check(mat2->coerced_object)){ /*MATRIX * POINT*/
 				pt = (PointObject*)mat2->coerced_object;
 				return column_point_multiplication(mat1, pt);
 			}else if (PyFloat_Check(mat2->coerced_object) || 
@@ -720,6 +737,7 @@ static PyObject *Matrix_mul(PyObject * m1, PyObject * m2)
 				}
 
 				scalar = (float)PyFloat_AS_DOUBLE(f);
+				Py_DECREF(f);
 				for(x = 0; x < mat1->rowSize; x++) {
 					for(y = 0; y < mat1->colSize; y++) {
 						mat[((x * mat1->colSize) + y)] = scalar * mat1->matrix[x][y];
