@@ -1,5 +1,5 @@
 /* 
- * $Id: sceneRender.c,v 1.44 2006/07/04 00:08:40 khughes Exp $
+ * $Id: sceneRender.c,v 1.52 2007/01/30 03:02:58 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -42,6 +42,7 @@ struct View3D; /* keep me up here */
 
 #include "BIF_drawscene.h"
 #include "BIF_renderwin.h"
+#include "BIF_writeimage.h"
 
 #include "BLI_blenlib.h"
 
@@ -86,11 +87,7 @@ enum rend_constants {
 #define EXPP_RENDER_ATTR_QUALITY             11
 #define EXPP_RENDER_ATTR_GAUSS               13
 #define EXPP_RENDER_ATTR_BLURFAC             14
-#define EXPP_RENDER_ATTR_POSTADD             15
-#define EXPP_RENDER_ATTR_POSTGAMMA           16
-#define EXPP_RENDER_ATTR_POSTMUL             17
-#define EXPP_RENDER_ATTR_POSTHUE             18
-#define EXPP_RENDER_ATTR_POSTSAT             19
+
 #define EXPP_RENDER_ATTR_YF_EXPOSURE         20
 #define EXPP_RENDER_ATTR_YF_GAMMA            21
 #define EXPP_RENDER_ATTR_YF_GIDEPTH          22
@@ -108,7 +105,6 @@ enum rend_constants {
 #define EXPP_RENDER_ATTR_YF_GIMETHOD         34
 #define EXPP_RENDER_ATTR_YF_GIQUALITY        35
 
-extern void save_rendered_image_cb_real(char *name, int zbuf, int confirm);
 
 /* Render doc strings */
 static char M_Render_doc[] = "The Blender Render module";
@@ -172,7 +168,7 @@ static PyObject *M_Render_BitToggleInt( PyObject * args, int setting,
 		*structure &= ~setting;
 	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 
 }
 
@@ -197,7 +193,7 @@ static PyObject *M_Render_BitToggleShort( PyObject * args, short setting,
 		*structure &= ~setting;
 	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 
 }
 
@@ -222,9 +218,9 @@ static PyObject *M_Render_GetSetAttributeFloat( PyObject * args,
 
 		*structure = property;
 		EXPP_allqueue( REDRAWBUTSSCENE, 0 );
-		return EXPP_incr_ret( Py_None );
+		Py_RETURN_NONE;
 	} else
-		return Py_BuildValue( "f", *structure );
+		return PyFloat_FromDouble( *structure );
 }
 
 static PyObject *M_Render_GetSetAttributeShort( PyObject * args,
@@ -248,9 +244,9 @@ static PyObject *M_Render_GetSetAttributeShort( PyObject * args,
 
 		*structure = property;
 		EXPP_allqueue( REDRAWBUTSSCENE, 0 );
-		return EXPP_incr_ret( Py_None );
+		Py_RETURN_NONE;
 	} else
-		return Py_BuildValue( "h", *structure );
+		return PyInt_FromLong( (long int)*structure );
 }
 
 static PyObject *M_Render_GetSetAttributeInt( PyObject * args, int *structure,
@@ -272,10 +268,21 @@ static PyObject *M_Render_GetSetAttributeInt( PyObject * args, int *structure,
 		}
 
 		*structure = property;
+		
+		/* compare memory locations, not values */
+		if (&G.scene->r.cfra == structure) {
+			/* are we changing the current frame?
+			update all objects, so python scripts can export all objects
+			in a scene without worrying about the view layers */
+			scene_update_for_newframe(G.scene, (1<<20) - 1);
+		}
+		
+		/*I dont think this should be here, whatif the scene is not the current scene - campbell*/
 		EXPP_allqueue( REDRAWBUTSSCENE, 0 );
-		return EXPP_incr_ret( Py_None );
+		
+		Py_RETURN_NONE;
 	} else
-		return Py_BuildValue( "i", *structure );
+		return  PyInt_FromLong( *structure );
 }
 
 
@@ -306,7 +313,7 @@ static void M_Render_DoSizePreset( BPy_RenderData * self, short xsch,
 PyObject *M_Render_CloseRenderWindow( PyObject * self )
 {
 	BIF_close_render_display(  );
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *M_Render_SetRenderWinPos( PyObject * self, PyObject * args )
@@ -349,7 +356,7 @@ PyObject *M_Render_SetRenderWinPos( PyObject * self, PyObject * args )
 	}
 	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *M_Render_EnableDispView( PyObject * self )
@@ -357,7 +364,7 @@ PyObject *M_Render_EnableDispView( PyObject * self )
 	G.displaymode = R_DISPLAYIMAGE;
 	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *M_Render_EnableDispWin( PyObject * self )
@@ -365,18 +372,9 @@ PyObject *M_Render_EnableDispWin( PyObject * self )
 	G.displaymode = R_DISPLAYWIN;
 	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
-PyObject *M_Render_EnableEdgeShift( PyObject * self, PyObject * args )
-{
-	return M_Render_BitToggleInt( args, 1, &G.compat );
-}
-
-PyObject *M_Render_EnableEdgeAll( PyObject * self, PyObject * args )
-{
-	return M_Render_BitToggleInt( args, 1, &G.notonlysolid );
-}
 
 /***************************************************************************/
 /* BPy_RenderData Function Definitions                                     */
@@ -409,7 +407,7 @@ PyObject *RenderData_Render( BPy_RenderData * self )
 		G.scene->r.efra = (short)end_frame;
 	}
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 /* 
@@ -420,8 +418,8 @@ PyObject *RenderData_SaveRenderedImage ( BPy_RenderData * self, PyObject *args )
 	char dir[FILE_MAXDIR * 2], str[FILE_MAXFILE * 2];
 	char *name_str, filepath[FILE_MAXDIR+FILE_MAXFILE];
 	RenderResult *rr = NULL;
-	int zbuff = 0;
-
+	int zbuff;
+	
 	if( !PyArg_ParseTuple( args, "s|i", &name_str, &zbuff ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 				"expected a filename (string) and optional int" );
@@ -431,7 +429,7 @@ PyObject *RenderData_SaveRenderedImage ( BPy_RenderData * self, PyObject *args )
 		return EXPP_ReturnPyObjError( PyExc_ValueError,
 				"full filename too long" );
 
-	if (zbuff !=0	) zbuff = 1; /*required 1/0 */
+	if (zbuff !=0	) zbuff = 1; /*required 1/0 */ /* removed! (ton) */
 
 	BLI_strncpy( filepath, self->renderContext->pic, sizeof(filepath) );
 	strcat(filepath, name_str);
@@ -445,9 +443,9 @@ PyObject *RenderData_SaveRenderedImage ( BPy_RenderData * self, PyObject *args )
 			BLI_splitdirstring(dir, str);
 			strcpy(G.ima, dir);
 		}
-		save_rendered_image_cb_real(filepath, zbuff,0);
+		BIF_save_rendered_image(filepath);
 	}
-	return EXPP_incr_ret(Py_None);
+	Py_RETURN_NONE;
 }
 
 PyObject *RenderData_RenderAnim( BPy_RenderData * self )
@@ -473,7 +471,7 @@ PyObject *RenderData_RenderAnim( BPy_RenderData * self )
 		
 		RE_BlenderAnim(re, G.scene, G.scene->r.sfra, G.scene->r.efra);
 	}
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *RenderData_Play( BPy_RenderData * self )
@@ -518,17 +516,22 @@ PyObject *RenderData_Play( BPy_RenderData * self )
 			 pos[1], file );
 		system( str );
 	} else {
-		BKE_makepicstring( file, self->renderContext->sfra );
+		BKE_makepicstring( file, G.scene->r.pic, self->renderContext->sfra, G.scene->r.imtype);
 		if( BLI_exist( file ) ) {
 			calc_renderwin_rectangle(640, 480, G.winpos, pos, size);
+#ifdef WIN32
 			sprintf( str, "%s -a -p %d %d \"%s\"", bprogname,
 				 pos[0], pos[1], file );
+#else
+			sprintf( str, "\"%s\" -a -p %d %d \"%s\"", bprogname,
+				 pos[0], pos[1], file );
+#endif
 			system( str );
 		} else
 			sprintf( "Can't find image: %s", file );
 	}
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *RenderData_EnableBackbuf( BPy_RenderData * self, PyObject * args )
@@ -537,12 +540,6 @@ PyObject *RenderData_EnableBackbuf( BPy_RenderData * self, PyObject * args )
 					&self->renderContext->bufflag );
 }
 
-PyObject *RenderData_EnableThreads( BPy_RenderData * self, PyObject * args )
-{
-	return M_Render_BitToggleInt( args, R_THREADS,
-					&self->renderContext->mode );
-}
-
 PyObject *RenderData_EnableExtensions( BPy_RenderData * self, PyObject * args )
 {
 	return M_Render_BitToggleShort( args, R_EXTENSION,
@@ -598,7 +595,7 @@ PyObject *RenderData_SetEdgeColor( BPy_RenderData * self, PyObject * args )
 	self->renderContext->edgeG = green;
 	self->renderContext->edgeB = blue;
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *RenderData_GetEdgeColor( BPy_RenderData * self )
@@ -610,13 +607,6 @@ PyObject *RenderData_GetEdgeColor( BPy_RenderData * self )
 	return PyString_FromString( rgb );
 }
 
-PyObject *RenderData_EdgeAntiShift( BPy_RenderData * self, PyObject * args )
-{
-	return M_Render_GetSetAttributeShort( args,
-					      &self->renderContext->
-					      same_mat_redux, 0, 255 );
-}
-
 PyObject *RenderData_EnableOversampling( BPy_RenderData * self,
 					 PyObject * args )
 {
@@ -676,7 +666,7 @@ PyObject *RenderData_EnableSky( BPy_RenderData * self )
 	self->renderContext->alphamode = R_ADDSKY;
 	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *RenderData_EnablePremultiply( BPy_RenderData * self )
@@ -684,7 +674,7 @@ PyObject *RenderData_EnablePremultiply( BPy_RenderData * self )
 	self->renderContext->alphamode = R_ALPHAPREMUL;
 	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *RenderData_EnableKey( BPy_RenderData * self )
@@ -692,7 +682,7 @@ PyObject *RenderData_EnableKey( BPy_RenderData * self )
 	self->renderContext->alphamode = R_ALPHAKEY;
 	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *RenderData_EnableShadow( BPy_RenderData * self, PyObject * args )
@@ -962,7 +952,7 @@ PyObject *RenderData_EnableGrayscale( BPy_RenderData * self )
 	self->renderContext->planes = R_PLANESBW;
 	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *RenderData_EnableRGBColor( BPy_RenderData * self )
@@ -970,7 +960,7 @@ PyObject *RenderData_EnableRGBColor( BPy_RenderData * self )
 	self->renderContext->planes = R_PLANES24;
 	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *RenderData_EnableRGBAColor( BPy_RenderData * self )
@@ -978,7 +968,7 @@ PyObject *RenderData_EnableRGBAColor( BPy_RenderData * self )
 	self->renderContext->planes = R_PLANES32;
 	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *RenderData_SizePreset( BPy_RenderData * self, PyObject * args )
@@ -1050,14 +1040,7 @@ PyObject *RenderData_SizePreset( BPy_RenderData * self, PyObject * args )
 						"unknown constant - see modules dict for help" ) );
 
 	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
-	return EXPP_incr_ret( Py_None );
-}
-
-PyObject *RenderData_EnableUnifiedRenderer( BPy_RenderData * self,
-					    PyObject * args )
-{
-	return M_Render_BitToggleInt( args, R_UNIFIED,
-				      &self->renderContext->mode );
+	Py_RETURN_NONE;
 }
 
 PyObject *RenderData_SetYafrayGIQuality( BPy_RenderData * self,
@@ -1078,7 +1061,7 @@ PyObject *RenderData_SetYafrayGIQuality( BPy_RenderData * self,
 						"unknown constant - see modules dict for help" ) );
 
 	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *RenderData_SetYafrayGIMethod( BPy_RenderData * self,
@@ -1097,7 +1080,7 @@ PyObject *RenderData_SetYafrayGIMethod( BPy_RenderData * self,
 						"unknown constant - see modules dict for help" ) );
 
 	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *RenderData_YafrayGIPower( BPy_RenderData * self, PyObject * args )
@@ -1300,19 +1283,19 @@ PyObject *RenderData_YafrayProcessorCount( BPy_RenderData * self,
 PyObject *RenderData_EnableGameFrameStretch( BPy_RenderData * self )
 {
 	self->scene->framing.type = SCE_GAMEFRAMING_SCALE;
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *RenderData_EnableGameFrameExpose( BPy_RenderData * self )
 {
 	self->scene->framing.type = SCE_GAMEFRAMING_EXTEND;
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *RenderData_EnableGameFrameBars( BPy_RenderData * self )
 {
 	self->scene->framing.type = SCE_GAMEFRAMING_BARS;
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *RenderData_SetGameFrameColor( BPy_RenderData * self,
@@ -1340,7 +1323,7 @@ PyObject *RenderData_SetGameFrameColor( BPy_RenderData * self,
 	self->scene->framing.col[1] = green;
 	self->scene->framing.col[2] = blue;
 
-	return EXPP_incr_ret( Py_None );
+	Py_RETURN_NONE;
 }
 
 PyObject *RenderData_GetGameFrameColor( BPy_RenderData * self )
@@ -1352,51 +1335,6 @@ PyObject *RenderData_GetGameFrameColor( BPy_RenderData * self )
 	return PyString_FromString( rgb );
 }
 
-PyObject *RenderData_GammaLevel( BPy_RenderData * self, PyObject * args )
-{
-	if( self->renderContext->mode & R_UNIFIED ) {
-		return M_Render_GetSetAttributeFloat( args,
-						      &self->renderContext->
-						      gamma, 0.2f, 5.0f );
-	} else
-		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
-						"Unified Render must be enabled" ) );
-}
-
-PyObject *RenderData_PostProcessAdd( BPy_RenderData * self, PyObject * args )
-{
-	if( self->renderContext->mode & R_UNIFIED ) {
-		return M_Render_GetSetAttributeFloat( args,
-						      &self->renderContext->
-						      postadd, -1.0f, 1.0f );
-	} else
-		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
-						"Unified Render must be enabled" ) );
-}
-
-PyObject *RenderData_PostProcessMultiply( BPy_RenderData * self,
-					  PyObject * args )
-{
-	if( self->renderContext->mode & R_UNIFIED ) {
-		return M_Render_GetSetAttributeFloat( args,
-						      &self->renderContext->
-						      postmul, 0.01f, 4.0f );
-	} else
-		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
-						"Unified Render must be enabled" ) );
-}
-
-PyObject *RenderData_PostProcessGamma( BPy_RenderData * self, PyObject * args )
-{
-	if( self->renderContext->mode & R_UNIFIED ) {
-		return M_Render_GetSetAttributeFloat( args,
-						      &self->renderContext->
-						      postgamma, 0.2f, 2.0f );
-	} else
-		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
-						"Unified Render must be enabled" ) );
-}
-
 #ifdef __sgi
 PyObject *RenderData_SGIMaxsize( BPy_RenderData * self, PyObject * args )
 {
@@ -1655,7 +1593,7 @@ static int RenderData_setModeBit( BPy_RenderData* self, PyObject *value,
 
 #define MODE_MASK ( R_OSA | R_SHADOW | R_GAMMA | R_ENVMAP | R_EDGE | \
 	R_FIELDS | R_FIELDSTILL | R_RADIO | R_BORDER | R_PANORAMA | R_CROP | \
-	R_ODDFIELD | R_MBLUR | R_UNIFIED | R_RAYTRACE | R_THREADS )
+	R_ODDFIELD | R_MBLUR | R_RAYTRACE | R_THREADS )
 
 static PyObject *RenderData_getMode( BPy_RenderData *self )
 {
@@ -1902,11 +1840,9 @@ static int RenderData_setBackbufPath( BPy_RenderData *self, PyObject *value )
 	strcpy( self->renderContext->backbuf, name );
 	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
-	ima = add_image( name );
-	if( ima ) {
-		free_image_buffers( ima );
-		ima->ok = 1;
-	}
+	ima = BKE_add_image_file( name );
+	if( ima )
+		BKE_image_signal( ima, NULL, IMA_SIGNAL_RELOAD );
 
 	return 0;
 }
@@ -2026,6 +1962,28 @@ static int RenderData_setSet( BPy_RenderData *self, PyObject *value )
 	return 0;
 }
 
+
+PyObject *RenderData_getThreads( BPy_RenderData * self )
+{
+	return PyInt_FromLong( (long) self->renderContext->threads );
+}
+
+static int RenderData_setThreads( BPy_RenderData *self, PyObject *value )
+{
+	int threads;
+
+	if( !PyInt_CheckExact( value ) )
+		return EXPP_ReturnIntError( PyExc_TypeError, "Error, threads must be an int" );
+
+	threads = PyInt_AsLong( value );
+	if (threads<1) threads = 1;
+	else if (threads>8) threads = 8;
+	
+	self->renderContext->threads = (short)threads;
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
+	return 0;
+}
+
 /***************************************************************************/
 /* BPy_RenderData attribute def                                            */
 /***************************************************************************/
@@ -2084,34 +2042,53 @@ static PyGetSetDef BPy_RenderData_getseters[] = {
 	 (getter)RenderData_getModeBit, (setter)RenderData_setModeBit,
 	 "Motion blur enabled",
 	 (void *)R_MBLUR},
-	{"unified",
-	 (getter)RenderData_getModeBit, (setter)RenderData_setModeBit,
-	 "Unified Renderer enabled",
-	 (void *)R_UNIFIED},
 	{"rayTracing",
 	 (getter)RenderData_getModeBit, (setter)RenderData_setModeBit,
 	 "Ray tracing enabled",
 	 (void *)R_RAYTRACE},
 /* R_GAUSS unused */
 /* R_FBUF unused */
+/* R_THREADS unused */
 	{"threads",
-	 (getter)RenderData_getModeBit, (setter)RenderData_setModeBit,
-	 "Render in two threads enabled",
-	 (void *)R_THREADS},
+	 (getter)RenderData_getThreads, (setter)RenderData_setThreads,
+	 "Number of threads used to render",
+	 NULL},
 /* R_SPEED unused */
 	{"mode",
 	 (getter)RenderData_getMode, (setter)RenderData_setMode,
 	 "Mode bitfield",
 	 NULL},
 
-	{"extensions",
-     (getter)RenderData_getSceModeBits, (setter)RenderData_setSceModeBits,
-     "Add extensions to output (when rendering animations) enabled",
-     (void *)R_EXTENSION},
+	/* scene modes */
 	{"sequencer",
      (getter)RenderData_getSceModeBits, (setter)RenderData_setSceModeBits,
      "'Do Sequence' enabled",
      (void *)R_DOSEQ},
+	{"extensions",
+     (getter)RenderData_getSceModeBits, (setter)RenderData_setSceModeBits,
+     "Add extensions to output (when rendering animations) enabled",
+     (void *)R_EXTENSION},
+	{"compositor",
+     (getter)RenderData_getSceModeBits, (setter)RenderData_setSceModeBits,
+     "'Do Compositor' enabled.",
+     (void *)R_DOCOMP},
+	{"freeImages",
+     (getter)RenderData_getSceModeBits, (setter)RenderData_setSceModeBits,
+     "Free texture images after render.",
+     (void *)R_FREE_IMAGE},
+	{"singleLayer",
+     (getter)RenderData_getSceModeBits, (setter)RenderData_setSceModeBits,
+     "Only render the active layer.",
+     (void *)R_SINGLE_LAYER},
+	{"saveBuffers",
+     (getter)RenderData_getSceModeBits, (setter)RenderData_setSceModeBits,
+     "Save render buffers to disk while rendering, saves memory.",
+     (void *)R_EXR_TILE_FILE},
+	{"compositeFree",
+     (getter)RenderData_getSceModeBits, (setter)RenderData_setSceModeBits,
+     "Free nodes that are not used while composite.",
+     (void *)R_COMP_FREE},
+
 	{"sceneMode",
      (getter)RenderData_getSceMode, (setter)RenderData_setSceMode,
      "Scene mode bitfield",
@@ -2137,7 +2114,7 @@ static PyGetSetDef BPy_RenderData_getseters[] = {
 	 NULL},
 	{"edgeColor",
 	 (getter)RenderData_getEdgeColor, (setter)RenderData_setEdgeColor,
-	 "RGB color triplet for edges in Toon shading (unified renderer)",
+	 "RGB color triplet for edges in Toon shading",
 	 NULL},
 	{"OSALevel",
 	 (getter)RenderData_getOSALevel, (setter)RenderData_setOSALevel,
@@ -2275,9 +2252,6 @@ static PyMethodDef BPy_RenderData_methods[] = {
 	{"enableBackbuf", ( PyCFunction ) RenderData_EnableBackbuf,
 	 METH_VARARGS,
 	 "(bool) - enable/disable the backbuf image"},
-	{"enableThreads", ( PyCFunction ) RenderData_EnableThreads,
-	 METH_VARARGS,
-	 "(bool) - enable/disable threaded rendering"},
 	{"setFtypePath", ( PyCFunction ) RenderData_SetFtypePath, METH_VARARGS,
 	 "(string) - get/set the path to output the Ftype file"},
 	{"getFtypePath", ( PyCFunction ) RenderData_getFtypePath, METH_NOARGS,
@@ -2301,9 +2275,6 @@ static PyMethodDef BPy_RenderData_methods[] = {
 	 "(f,f,f) - set the edge color for toon shading - Red,Green,Blue expected."},
 	{"getEdgeColor", ( PyCFunction ) RenderData_GetEdgeColor, METH_NOARGS,
 	 "() - get the edge color for toon shading - Red,Green,Blue expected."},
-	{"edgeAntiShift", ( PyCFunction ) RenderData_EdgeAntiShift,
-	 METH_VARARGS,
-	 "(int) - with the unified renderer to reduce intensity on boundaries."},
 	{"enableOversampling", ( PyCFunction ) RenderData_EnableOversampling,
 	 METH_VARARGS,
 	 "(bool) - enable/disable oversampling (anit-aliasing)."},
@@ -2364,9 +2335,6 @@ static PyMethodDef BPy_RenderData_methods[] = {
 	 "(bool) - enable/disable small cut-out rendering"},
 	{"setBorder", ( PyCFunction ) RenderData_SetBorder, METH_VARARGS,
 	 "(f,f,f,f) - set the border for border rendering"},
-	{"enableGammaCorrection",
-	 ( PyCFunction ) RenderData_EnableGammaCorrection, METH_VARARGS,
-	 "(bool) - enable/disable gamma correction"},
 	{"gaussFilterSize", ( PyCFunction ) RenderData_GaussFilterSize,
 	 METH_VARARGS,
 	 "(float) - get/sets the Gauss filter size"},
@@ -2408,9 +2376,6 @@ static PyMethodDef BPy_RenderData_methods[] = {
 	 "() - images are saved with RGB and Alpha data (if supported)"},
 	{"sizePreset", ( PyCFunction ) RenderData_SizePreset, METH_VARARGS,
 	 "(enum) - get/set the render to one of a few preget/sets"},
-	{"enableUnifiedRenderer",
-	 ( PyCFunction ) RenderData_EnableUnifiedRenderer, METH_VARARGS,
-	 "(bool) - use the unified renderer"},
 	{"setYafrayGIQuality", ( PyCFunction ) RenderData_SetYafrayGIQuality,
 	 METH_VARARGS,
 	 "(enum) - get/set yafray global Illumination quality"},
@@ -2485,17 +2450,6 @@ static PyMethodDef BPy_RenderData_methods[] = {
 	{"getGameFrameColor", ( PyCFunction ) RenderData_GetGameFrameColor,
 	 METH_NOARGS,
 	 "() - get the red, green, blue component of the bars"},
-	{"gammaLevel", ( PyCFunction ) RenderData_GammaLevel, METH_VARARGS,
-	 "(float) - get/set the gamma value for blending oversampled images (1.0 = no correction"},
-	{"postProcessAdd", ( PyCFunction ) RenderData_PostProcessAdd,
-	 METH_VARARGS,
-	 "(float) - get/set post processing add"},
-	{"postProcessMultiply", ( PyCFunction ) RenderData_PostProcessMultiply,
-	 METH_VARARGS,
-	 "(float) - get/set post processing multiply"},
-	{"postProcessGamma", ( PyCFunction ) RenderData_PostProcessGamma,
-	 METH_VARARGS,
-	 "(float) - get/set post processing gamma"},
 	{"SGIMaxsize", ( PyCFunction ) RenderData_SGIMaxsize, METH_VARARGS,
 	 "(int) - get/set maximum size per frame to save in an SGI movie"},
 	{"enableSGICosmo", ( PyCFunction ) RenderData_EnableSGICosmo,
@@ -2606,11 +2560,6 @@ struct PyMethodDef M_Render_methods[] = {
 	{"SetRenderWinPos", ( PyCFunction ) M_Render_SetRenderWinPos,
 	 METH_VARARGS,
 	 "([string list]) - position the rendering window in around the edge of the screen"},
-	{"EnableEdgeShift", ( PyCFunction ) M_Render_EnableEdgeShift,
-	 METH_VARARGS,
-	 "(bool) - with the unified renderer the outlines are shifted a bit."},
-	{"EnableEdgeAll", ( PyCFunction ) M_Render_EnableEdgeAll, METH_VARARGS,
-	 "(bool) - also consider transparent faces for edge-rendering with the unified renderer"},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -2633,7 +2582,6 @@ static PyObject *M_Render_ModesDict( void )
 		PyConstant_Insert( d, "CROP", PyInt_FromLong( R_CROP ) );
 		PyConstant_Insert( d, "ODDFIELD", PyInt_FromLong( R_ODDFIELD ) );
 		PyConstant_Insert( d, "MBLUR", PyInt_FromLong( R_MBLUR ) );
-		PyConstant_Insert( d, "UNIFIED", PyInt_FromLong( R_UNIFIED ) );
 		PyConstant_Insert( d, "RAYTRACING", PyInt_FromLong( R_RAYTRACE ) );
 		PyConstant_Insert( d, "THREADS", PyInt_FromLong( R_THREADS ) );
 	}
