@@ -1,4 +1,4 @@
-/* $Id: vector.h,v 1.17 2006/04/16 15:28:50 stiv Exp $
+/* $Id: vector.h,v 1.21 2006/10/07 11:49:53 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -41,23 +41,10 @@ extern PyTypeObject vector_Type;
 
 typedef struct {
 	PyObject_VAR_HEAD 
-	struct{
-		float *py_data;		//python managed
-		float *blend_data;	//blender managed
-	}data;
-	float *vec;				//1D array of data (alias)
-	int size;
-	int wrapped;			//is wrapped data?
-	PyObject *coerced_object;
+	float *vec;				/*1D array of data (alias), wrapped status depends on wrapped status */
+	short size;				/* vec size 2,3 or 4 */
+	short wrapped;			/* is wrapped data? */
 } VectorObject;
-/*coerced_object is a pointer to the object that it was
-coerced from when a dummy vector needs to be created from
-the coerce() function for numeric protocol operations*/
-
-/*struct data contains a pointer to the actual data that the
-object uses. It can use either PyMem allocated data (which will
-be stored in py_data) or be a wrapper for data allocated through
-blender (stored in blend_data). This is an either/or struct not both*/
 
 //prototypes
 PyObject *Vector_Zero( VectorObject * self );
@@ -68,6 +55,7 @@ PyObject *Vector_Resize3D( VectorObject * self );
 PyObject *Vector_Resize4D( VectorObject * self );
 PyObject *Vector_toPoint( VectorObject * self );
 PyObject *Vector_ToTrackQuat( VectorObject * self, PyObject * args );
+PyObject *Vector_copy( VectorObject * self );
 PyObject *newVectorObject(float *vec, int size, int type);
 
 #endif				/* EXPP_vector_h */
