@@ -1,5 +1,5 @@
 /* 
- * $Id: windowTheme.c,v 1.12 2006/06/08 19:05:58 schlaile Exp $
+ * $Id: windowTheme.c,v 1.15 2007/01/03 05:50:16 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -33,10 +33,13 @@
 #include "windowTheme.h" /*This must come first*/
 
 #include "BLI_blenlib.h"
+#include "BIF_interface_icons.h"
+#include "BIF_resources.h"
 #include "MEM_guardedalloc.h"
 #include "charRGBA.h"
 #include "gen_utils.h"
 
+
 #define EXPP_THEME_VTX_SIZE_MIN 1
 #define EXPP_THEME_VTX_SIZE_MAX 10
 #define EXPP_THEME_FDOT_SIZE_MIN 1
@@ -44,7 +47,7 @@
 #define EXPP_THEME_DRAWTYPE_MIN 1
 #define EXPP_THEME_DRAWTYPE_MAX 4
 
-#define EXPP_THEME_NUMBEROFTHEMES 15
+#define EXPP_THEME_NUMBEROFTHEMES 16
 static const EXPP_map_pair themes_map[] = {
 	{"ui", -1},
 	{"buts", SPACE_BUTS},
@@ -61,6 +64,7 @@ static const EXPP_map_pair themes_map[] = {
 	{"text", SPACE_TEXT},
 	{"oops", SPACE_OOPS},
 	{"time", SPACE_TIME},
+	{"node", SPACE_NODE},
 	{NULL, 0}
 };
 
@@ -156,15 +160,18 @@ static PyObject *ThemeSpace_getAttr( BPy_ThemeSpace * self, char *name )
 		ELSEIF_TSP_RGBA( hilite )
 		ELSEIF_TSP_RGBA( grid )
 		ELSEIF_TSP_RGBA( wire )
-		ELSEIF_TSP_RGBA( lamp )
 		ELSEIF_TSP_RGBA( select )
+		ELSEIF_TSP_RGBA( lamp )
 		ELSEIF_TSP_RGBA( active )
+		ELSEIF_TSP_RGBA( group )
+		ELSEIF_TSP_RGBA( group_active )
 		ELSEIF_TSP_RGBA( transform )
 		ELSEIF_TSP_RGBA( vertex )
 		ELSEIF_TSP_RGBA( vertex_select )
 		ELSEIF_TSP_RGBA( edge )
 		ELSEIF_TSP_RGBA( edge_select )
 		ELSEIF_TSP_RGBA( edge_seam )
+		ELSEIF_TSP_RGBA( edge_sharp )
 		ELSEIF_TSP_RGBA( edge_facesel )
 		ELSEIF_TSP_RGBA( face )
 		ELSEIF_TSP_RGBA( face_select )
@@ -192,17 +199,19 @@ static PyObject *ThemeSpace_getAttr( BPy_ThemeSpace * self, char *name )
 		else if( !strcmp( name, "facedot_size" ) )
 		attrib = Py_BuildValue( "i", tsp->facedot_size );
 	else if( !strcmp( name, "__members__" ) )
-		attrib = Py_BuildValue("[sssssssssssssssssssssssssssssssssssssssssss]", "theme",
+		attrib = Py_BuildValue("[sssssssssssssssssssssssssssssssssssssssssssssss]", "theme",
 					"back", "text", "text_hi", "header",
 					"panel", "shade1", "shade2", "hilite",
-					"grid", "wire", "lamp", "select", "active",
+					"grid", "wire", "select", "lamp", "active",
+					"group", "group_active",
 					"transform", "vertex", "vertex_select",
-					"edge", "edge_select", "edge_seam",
+					"edge", "edge_select", "edge_seam", "edge_sharp",
 					"edge_facesel", "face", "face_select",
 					"face_dot", "normal", "bone_solid", "bone_pose",
 					"strip", "strip_select",
 					"syntaxl", "syntaxn", "syntaxb", "syntaxv", "syntaxc",
-					"movie", "image", "scene", "audio", "effect", "plugin", "transition", "meta", 
+					"movie", "image", "scene", "audio", "effect", "plugin",
+					"transition", "meta", 
 					"vertex_size", "facedot_size" );
 
 	if( attrib != Py_None )
@@ -230,15 +239,18 @@ static int ThemeSpace_setAttr( BPy_ThemeSpace * self, char *name,
 		ELSEIF_TSP_RGBA( hilite )
 		ELSEIF_TSP_RGBA( grid )
 		ELSEIF_TSP_RGBA( wire )
-		ELSEIF_TSP_RGBA( lamp )
 		ELSEIF_TSP_RGBA( select )
+		ELSEIF_TSP_RGBA( lamp )
 		ELSEIF_TSP_RGBA( active )
+		ELSEIF_TSP_RGBA( group )
+		ELSEIF_TSP_RGBA( group_active )
 		ELSEIF_TSP_RGBA( transform )
 		ELSEIF_TSP_RGBA( vertex )
 		ELSEIF_TSP_RGBA( vertex_select )
 		ELSEIF_TSP_RGBA( edge )
 		ELSEIF_TSP_RGBA( edge_select )
 		ELSEIF_TSP_RGBA( edge_seam )
+		ELSEIF_TSP_RGBA( edge_sharp )
 		ELSEIF_TSP_RGBA( edge_facesel )
 		ELSEIF_TSP_RGBA( face )
 		ELSEIF_TSP_RGBA( face_select )
@@ -398,14 +410,16 @@ static PyObject *ThemeUI_getAttr( BPy_ThemeUI * self, char *name )
 		ELSEIF_TUI_RGBA( menu_text_hi )
 		else if( !strcmp( name, "drawType" ) )
 		attrib = PyInt_FromLong( ( char ) tui->but_drawtype );
+		else if( !strcmp( name, "iconTheme" ) )
+		attrib = PyString_FromString( tui->iconfile );
 	else if( !strcmp( name, "__members__" ) )
-		attrib = Py_BuildValue( "[ssssssssssssssssss]", "theme",
+		attrib = Py_BuildValue( "[ssssssssssssssssssss]", "theme",
 					"outline", "neutral", "action",
 					"setting", "setting1", "setting2",
 					"num", "textfield", "textfield_hi", "popup", "text",
 					"text_hi", "menu_back", "menu_item",
 					"menu_hilite", "menu_text",
-					"menu_text_hi", "drawType" );
+					"menu_text_hi", "drawType", "iconTheme" );
 
 	if( attrib != Py_None )
 		return attrib;
@@ -449,6 +463,16 @@ static int ThemeUI_setAttr( BPy_ThemeUI * self, char *name, PyObject * value )
 						   EXPP_THEME_DRAWTYPE_MIN,
 						   EXPP_THEME_DRAWTYPE_MAX );
 		ret = 0;
+	} else if ( !strcmp( name, "iconTheme" ) ) {
+		if ( !PyString_Check(value) )
+			return EXPP_ReturnIntError( PyExc_TypeError,
+						    "expected string value" );
+		BLI_strncpy(tui->iconfile, PyString_AsString(value), 80);
+		
+		BIF_icons_free();
+		BIF_icons_init(BIFICONID_LAST+1);
+		
+		ret = 0;
 	} else
 		return EXPP_ReturnIntError( PyExc_AttributeError,
 					    "attribute not found" );
@@ -726,6 +750,9 @@ static PyObject *Theme_get( BPy_Theme * self, PyObject * args )
 	case SPACE_TIME:
 		tsp = &btheme->ttime;
 		break;
+	case SPACE_NODE:
+		tsp = &btheme->tnode;
+		break;
 	}
 
 	if( tui ) {
