@@ -36,11 +36,12 @@
     - faces
     - filtering and node-limit
     - apply to meshes
-    $Id: radpostprocess.c,v 1.7 2005/03/19 21:08:11 zuster Exp $
+    $Id: radpostprocess.c,v 1.19 2006/12/12 21:29:06 blendix Exp $
 
  *************************************** */
 
 #include <stdlib.h>
+#include <string.h>
 #include <math.h>
 
 #ifdef HAVE_CONFIG_H
@@ -51,6 +52,7 @@
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
+#include "BLI_ghash.h"
 #include "BIF_toolbox.h"  // notice()
 
 #include "DNA_material_types.h"
@@ -59,12 +61,13 @@
 #include "DNA_object_types.h"
 #include "DNA_radio_types.h"
 
-#include "BKE_utildefines.h"
+#include "BKE_customdata.h"
 #include "BKE_global.h"
 #include "BKE_main.h"
+#include "BKE_material.h"
 #include "BKE_mesh.h"
 #include "BKE_object.h"
-#include "BKE_material.h"
+#include "BKE_utildefines.h"
 
 #include "BIF_screen.h"		/* waitcursor */
 #include "BIF_editview.h"	/* deselectall */
@@ -171,7 +174,7 @@ Face *addface()
 
 }
 
-void makeface(float *v1, float *v2, float *v3, float *v4, RNode *rn)
+Face * makeface(float *v1, float *v2, float *v3, float *v4, RNode *rn)
 {
 	Face *face;
 	
@@ -182,140 +185,140 @@ void makeface(float *v1, float *v2, float *v3, float *v4, RNode *rn)
 	face->v4= v4;
 	face->col= rn->col;
 	face->matindex= rn->par->matindex;
-}
+	face->orig= rn->orig;
 
+	return face;
+}
 
 void anchorQuadface(RNode *rn, float *v1, float *v2, float *v3, float *v4, int flag)
 {
+	Face *face;
 
 	switch(flag) {
 		case 1:
-			makeface(rn->v1, v1, rn->v4, 0, rn);
-			makeface(v1, rn->v3, rn->v4, 0, rn);
-			makeface(v1, rn->v2, rn->v3, 0, rn);
+			face = makeface(rn->v1, v1, rn->v4, NULL, rn);
+			face = makeface(v1, rn->v3, rn->v4, NULL, rn);
+			face = makeface(v1, rn->v2, rn->v3, NULL, rn);
 			break;
 		case 2:
-			makeface(rn->v2, v2, rn->v1, 0, rn);
-			makeface(v2, rn->v4, rn->v1, 0, rn);
-			makeface(v2, rn->v3, rn->v4, 0, rn);
+			face = makeface(rn->v2, v2, rn->v1, NULL, rn);
+			face = makeface(v2, rn->v4, rn->v1, NULL, rn);
+			face = makeface(v2, rn->v3, rn->v4, NULL, rn);
 			break;
 		case 4:
-			makeface(rn->v3, v3, rn->v2, 0, rn);
-			makeface(v3, rn->v1, rn->v2, 0, rn);
-			makeface(v3, rn->v4, rn->v1, 0, rn);
+			face = makeface(rn->v3, v3, rn->v2, NULL, rn);
+			face = makeface(v3, rn->v1, rn->v2, NULL, rn);
+			face = makeface(v3, rn->v4, rn->v1, NULL, rn);
 			break;
 		case 8:
-			makeface(rn->v4, v4, rn->v3, 0, rn);
-			makeface(v4, rn->v2, rn->v3, 0, rn);
-			makeface(v4, rn->v1, rn->v2, 0, rn);
+			face = makeface(rn->v4, v4, rn->v3, NULL, rn);
+			face = makeface(v4, rn->v2, rn->v3, NULL, rn);
+			face = makeface(v4, rn->v1, rn->v2, NULL, rn);
 			break;
-			
 		case 3:
-			makeface(rn->v1, v1, rn->v4, 0, rn);
-			makeface(v1, v2, rn->v4, 0, rn);
-			makeface(v1, rn->v2, v2, 0, rn);
-			makeface(v2, rn->v3, rn->v4, 0, rn);
+			face = makeface(rn->v1, v1, rn->v4, NULL, rn);
+			face = makeface(v1, v2, rn->v4, NULL, rn);
+			face = makeface(v1, rn->v2, v2, NULL, rn);
+			face = makeface(v2, rn->v3, rn->v4, NULL, rn);
 			break;
 		case 6:
-			makeface(rn->v2, v2, rn->v1, 0, rn);
-			makeface(v2, v3, rn->v1, 0, rn);
-			makeface(v2, rn->v3, v3, 0, rn);
-			makeface(v3, rn->v4, rn->v1, 0, rn);
+			face = makeface(rn->v2, v2, rn->v1, NULL, rn);
+			face = makeface(v2, v3, rn->v1, NULL, rn);
+			face = makeface(v2, rn->v3, v3, NULL, rn);
+			face = makeface(v3, rn->v4, rn->v1, NULL, rn);
 			break;
 		case 12:
-			makeface(rn->v3, v3, rn->v2, 0, rn);
-			makeface(v3, v4, rn->v2, 0, rn);
-			makeface(v3, rn->v4, v4, 0, rn);
-			makeface(v4, rn->v1, rn->v2, 0, rn);
+			face = makeface(rn->v3, v3, rn->v2, NULL, rn);
+			face = makeface(v3, v4, rn->v2, NULL, rn);
+			face = makeface(v3, rn->v4, v4, NULL, rn);
+			face = makeface(v4, rn->v1, rn->v2, NULL, rn);
 			break;
 		case 9:
-			makeface(rn->v4, v4, rn->v3, 0, rn);
-			makeface(v4, v1, rn->v3, 0, rn);
-			makeface(v4, rn->v1, v1, 0, rn);
-			makeface(v1, rn->v2, rn->v3, 0, rn);
+			face = makeface(rn->v4, v4, rn->v3, NULL, rn);
+			face = makeface(v4, v1, rn->v3, NULL, rn);
+			face = makeface(v4, rn->v1, v1, NULL, rn);
+			face = makeface(v1, rn->v2, rn->v3, NULL, rn);
 			break;
-			
 		case 5:
-			makeface(rn->v1, v1, v3, rn->v4, rn);
-			makeface(v1, rn->v2, rn->v3, v3, rn);
+			face = makeface(rn->v1, v1, v3, rn->v4, rn);
+			face = makeface(v1, rn->v2, rn->v3, v3, rn);
 			break;
 		case 10:
-			makeface(rn->v2, v2, v4, rn->v1, rn);
-			makeface(v2, rn->v3, rn->v4, v4, rn);
+			face = makeface(rn->v1, rn->v2, v2, v4, rn);
+			face = makeface(v4, v2, rn->v3, rn->v4, rn);
 			break;
-			
 		case 7:
-			makeface(rn->v1, v1, v3, rn->v4, rn);
-			makeface(v1, v2, v3, 0, rn);
-			makeface(v1, rn->v2, v2, 0, rn);
-			makeface(v2, rn->v3, v3, 0, rn);
+			face = makeface(rn->v1, v1, v3, rn->v4, rn);
+			face = makeface(v1, v2, v3, NULL, rn);
+			face = makeface(v1, rn->v2, v2, NULL, rn);
+			face = makeface(v2, rn->v3, v3, NULL, rn);
 			break;
 		case 14:
-			makeface(rn->v2, v2, v4, rn->v1, rn);
-			makeface(v2, v3, v4, 0, rn);
-			makeface(v2, rn->v3, v3, 0, rn);
-			makeface(v3, rn->v4, v4, 0, rn);
+			face = makeface(rn->v2, v2, v4, rn->v1, rn);
+			face = makeface(v2, v3, v4, NULL, rn);
+			face = makeface(v2, rn->v3, v3, NULL, rn);
+			face = makeface(v3, rn->v4, v4, NULL, rn);
 			break;
 		case 13:
-			makeface(rn->v3, v3, v1, rn->v2, rn);
-			makeface(v3, v4, v1, 0, rn);
-			makeface(v3, rn->v4, v4, 0, rn);
-			makeface(v4, rn->v1, v1, 0, rn);
+			face = makeface(rn->v3, v3, v1, rn->v2, rn);
+			face = makeface(v3, v4, v1, NULL, rn);
+			face = makeface(v3, rn->v4, v4, NULL, rn);
+			face = makeface(v4, rn->v1, v1, NULL, rn);
 			break;
 		case 11:
-			makeface(rn->v4, v4, v2, rn->v3, rn);
-			makeface(v4, v1, v2, 0, rn);
-			makeface(v4, rn->v1, v1, 0, rn);
-			makeface(v1, rn->v2, v2, 0, rn);
+			face = makeface(rn->v4, v4, v2, rn->v3, rn);
+			face = makeface(v4, v1, v2, NULL, rn);
+			face = makeface(v4, rn->v1, v1, NULL, rn);
+			face = makeface(v1, rn->v2, v2, NULL, rn);
 			break;
-		
 		case 15:
-			makeface(v1, v2, v3, v4, rn);
-			makeface(v1, rn->v2, v2, 0, rn);
-			makeface(v2, rn->v3, v3, 0, rn);
-			makeface(v3, rn->v4, v4, 0, rn);
-			makeface(v4, rn->v1, v1, 0, rn);
+			face = makeface(v1, v2, v3, v4, rn);
+			face = makeface(v1, rn->v2, v2, NULL, rn);
+			face = makeface(v2, rn->v3, v3, NULL, rn);
+			face = makeface(v3, rn->v4, v4, NULL, rn);
+			face = makeface(v4, rn->v1, v1, NULL, rn);
 			break;
 	}
 }
 
 void anchorTriface(RNode *rn, float *v1, float *v2, float *v3, int flag)
 {
+	Face *face;
+
 	switch(flag) {
 		case 1:
-			makeface(rn->v1, v1, rn->v3, 0, rn);
-			makeface(v1, rn->v2, rn->v3, 0, rn);
+			face = makeface(rn->v1, v1, rn->v3, NULL, rn);
+			face = makeface(v1, rn->v2, rn->v3, NULL, rn);
 			break;
 		case 2:
-			makeface(rn->v2, v2, rn->v1, 0, rn);
-			makeface(v2, rn->v3, rn->v1, 0, rn);
+			face = makeface(rn->v2, v2, rn->v1, NULL, rn);
+			face = makeface(v2, rn->v3, rn->v1, NULL, rn);
 			break;
 		case 4:
-			makeface(rn->v3, v3, rn->v2, 0, rn);
-			makeface(v3, rn->v1, rn->v2, 0, rn);
+			face = makeface(rn->v3, v3, rn->v2, NULL, rn);
+			face = makeface(v3, rn->v1, rn->v2, NULL, rn);
 			break;
-			
 		case 3:
-			makeface(rn->v1, v2, rn->v3, 0, rn);
-			makeface(rn->v1, v1, v2, 0, rn);
-			makeface(v1, rn->v2, v2, 0, rn);
+			face = makeface(rn->v1, v2, rn->v3, NULL, rn);
+			face = makeface(rn->v1, v1, v2, NULL, rn);
+			face = makeface(v1, rn->v2, v2, NULL, rn);
 			break;
 		case 6:
-			makeface(rn->v2, v3, rn->v1, 0, rn);
-			makeface(rn->v2, v2, v3, 0, rn);
-			makeface(v2, rn->v3, v3, 0, rn);
+			face = makeface(rn->v2, v3, rn->v1, NULL, rn);
+			face = makeface(rn->v2, v2, v3, NULL, rn);
+			face = makeface(v2, rn->v3, v3, NULL, rn);
 			break;
 		case 5:
-			makeface(rn->v3, v1, rn->v2, 0, rn);
-			makeface(rn->v3, v3, v1, 0, rn);
-			makeface(v3, rn->v1, v1, 0, rn);
+			face = makeface(rn->v3, v1, rn->v2, NULL, rn);
+			face = makeface(rn->v3, v3, v1, NULL, rn);
+			face = makeface(v3, rn->v1, v1, NULL, rn);
 			break;
 			
 		case 7:
-			makeface(v1, v2, v3, 0, rn);
-			makeface(rn->v1, v1, v3, 0, rn);
-			makeface(rn->v2, v2, v1, 0, rn);
-			makeface(rn->v3, v3, v2, 0, rn);
+			face = makeface(v1, v2, v3, NULL, rn);
+			face = makeface(rn->v1, v1, v3, NULL, rn);
+			face = makeface(rn->v2, v2, v1, NULL, rn);
+			face = makeface(rn->v3, v3, v2, NULL, rn);
 			break;
 	}	
 }
@@ -345,7 +348,6 @@ float *findmiddlevertex(RNode *node, RNode *nb, float *v1, float *v2)
 			if(nb->v4==v1 || nb->v4==v2) test+=2;
 			if(test==1) return nb->v4;
 			else if(test==2) return nb->v3;
-			
 		}
 		else {
 			if(nb->v3==v1 || nb->v3==v2) test++;
@@ -662,9 +664,9 @@ void removeEqualNodes(short limit)
 					}
 					
 					if(ok) {
-						rn->up->totrad[0]= 0.5*(rn->totrad[0]+rn1->totrad[0]);
-						rn->up->totrad[1]= 0.5*(rn->totrad[1]+rn1->totrad[1]);
-						rn->up->totrad[2]= 0.5*(rn->totrad[2]+rn1->totrad[2]);
+						rn->up->totrad[0]= 0.5f*(rn->totrad[0]+rn1->totrad[0]);
+						rn->up->totrad[1]= 0.5f*(rn->totrad[1]+rn1->totrad[1]);
+						rn->up->totrad[2]= 0.5f*(rn->totrad[2]+rn1->totrad[2]);
 						rn1= rn->up;
 						deleteNodes(rn1);
 						if(rn1->down1) ;
@@ -683,7 +685,23 @@ void removeEqualNodes(short limit)
 	waitcursor(0);
 }
 
-#define BLSIZE	32000
+unsigned int rad_find_or_add_mvert(Mesh *me, MFace *mf, RNode *orignode, float *w, float *radco, GHash *hash)
+{
+	MVert *mvert = BLI_ghash_lookup(hash, radco);
+
+	if(!mvert) {
+		mvert = &me->mvert[me->totvert];
+		VECCOPY(mvert->co, radco);
+		me->totvert++;
+
+		BLI_ghash_insert(hash, radco, mvert);
+	}
+
+	InterpWeightsQ3Dfl(orignode->v1, orignode->v2, orignode->v3,
+		orignode->v4, mvert->co, w);
+
+	return (unsigned int)(mvert - me->mvert);
+}
 
 void rad_addmesh(void)
 {
@@ -691,168 +709,103 @@ void rad_addmesh(void)
 	Object *ob;
 	Mesh *me;
 	MVert *mvert;
-	MFace *mface;
+	MFace *mf;
+	RNode *node;
 	Material *ma=0;
-	unsigned int *md, *coldata, *cd;
-	float **fpp, **poindata;
-	float cent[3], min[3], max[3];
-	int a, vcount, vlnr, startf, endf;
+	GHash *verthash;
+	unsigned int *mcol;
+	float cent[3], min[3], max[3], w[4][4];
+	int a;
+
+	if(RG.totface==0)
+		return;
 	
-	if(RG.totface==0) return;
+	if(RG.totmat==MAXMAT)
+		notice("warning: cannot assign more than 16 materials to 1 mesh");
+
+	/* create the mesh */
+	ob= add_object(OB_MESH);
 	
-	if(RG.totmat==MAXMAT) notice("warning: cannot assign more than 16 materials to 1 mesh");
-	
-	/* make sure there's alpha in the color, to distinguish */
-	for(a=0; a<RG.totface; a++) {
+	me= ob->data;
+	me->totvert= totalRadVert();
+	me->totface= RG.totface;
+	me->flag= 0;
+
+	CustomData_add_layer(&me->vdata, CD_MVERT, CD_CALLOC, NULL, me->totvert);
+	CustomData_add_layer(&me->fdata, CD_MFACE, CD_CALLOC, NULL, me->totface);
+	CustomData_add_layer(&me->fdata, CD_MCOL, CD_CALLOC, NULL, me->totface);
+
+	CustomData_merge(RG.mfdata, &me->fdata, CD_MASK_MESH, CD_CALLOC, me->totface);
+	mesh_update_customdata_pointers(me);
+
+	/* create materials and set vertex color flag */
+	for(a=0; a<RG.totmat; a++) {
+		assign_material(ob, RG.matar[a], a+1);
+		ma= RG.matar[a];
+		if(ma) ma->mode |= MA_VERTEXCOL;
+	}
+
+	/* create vertices and faces in one go, adding vertices to the end of the
+	   mvert array if they were not added already */
+	me->totvert= 0;
+	verthash= BLI_ghash_new(BLI_ghashutil_ptrhash, BLI_ghashutil_ptrcmp);
+
+	mcol= (unsigned int*)me->mcol;
+	mf= me->mface;
+
+	for(a=0; a<me->totface; a++, mf++, mcol+=4) {
 		RAD_NEXTFACE(a);
-		*((unsigned int *)face->v1+3) |= 0x1000000; 
-		*((unsigned int *)face->v2+3) |= 0x1000000;
-		*((unsigned int *)face->v3+3) |= 0x1000000;
-		if(face->v4) {
-			*((unsigned int *)face->v4+3) |= 0x1000000;
-		}
+
+		/* the original node that this node is a subnode of */
+		node= RG.mfdatanodes[face->orig];
+
+		/* set mverts from the radio data, and compute interpolation weights */
+		mf->v1= rad_find_or_add_mvert(me, mf, node, w[0], face->v1, verthash);
+		mf->v2= rad_find_or_add_mvert(me, mf, node, w[1], face->v2, verthash);
+		mf->v3= rad_find_or_add_mvert(me, mf, node, w[2], face->v3, verthash);
+		if(face->v4)
+			mf->v4= rad_find_or_add_mvert(me, mf, node, w[3], face->v4, verthash);
+
+		/* copy face and interpolate data */
+		mf->mat_nr= face->matindex;
+
+		CustomData_copy_data(RG.mfdata, &me->fdata, face->orig, a, 1);
+		CustomData_interp(RG.mfdata, &me->fdata, &face->orig, NULL, (float*)w, 1, a);
+
+		/* load face vertex colors, with alpha added */
+		mcol[0]= *((unsigned int*)face->v1+3) | 0x1000000;
+		mcol[1]= *((unsigned int*)face->v2+3) | 0x1000000;
+		mcol[2]= *((unsigned int*)face->v3+3) | 0x1000000;
+		if(face->v4)
+			mcol[3]= *((unsigned int*)face->v4+3) | 0x1000000;
+
+		/* reorder face indices if needed to make face->v4 == 0 */
+		test_index_face(mf, &me->fdata, a, face->v4? 4: 3);
+	}
+
+	BLI_ghash_free(verthash, NULL, NULL);
+
+	/* boundbox and centre new */
+	INIT_MINMAX(min, max);
+
+	mvert= me->mvert;
+	for(a=0; a<me->totvert; a++, mvert++) {
+		DO_MINMAX(mvert->co, min, max);
+	}
+
+	cent[0]= (min[0]+max[0])/2.0f;
+	cent[1]= (min[1]+max[1])/2.0f;
+	cent[2]= (min[2]+max[2])/2.0f;
+
+	mvert= me->mvert;
+	for(a=0; a<me->totvert; a++, mvert++) {
+		VecSubf(mvert->co, mvert->co, cent);
 	}
 	
-	/* we write in blocks of BLSIZE vertices max */
-	coldata= MEM_mallocN(4*BLSIZE, "writefaces");
-	poindata= MEM_mallocN(sizeof(void *)*BLSIZE, "writefaces1");
-	
-	vlnr= 0; 
-	endf= 0;
-	while(endf<RG.totface) {
-		
-		cd= coldata;
-		fpp= poindata;
-		startf= endf;
-		vcount= 0;
-	
-		face= RG.facebase[(startf-1)>>10]+((startf-1) & 1023);
-		for(vlnr=startf; vlnr<RG.totface; vlnr++) {
-			RAD_NEXTFACE(vlnr);
+	VECCOPY(ob->loc, cent);
 
-			if( *((unsigned int *)face->v1+3) & 0x1000000 ) { /* is a color */
-				*cd= *((unsigned int *)face->v1+3);
-				*((unsigned int *)face->v1+3) = vcount;
-				*fpp= face->v1;
-				fpp++; vcount++; cd++;
-			}
-			if( *((unsigned int *)face->v2+3) & 0x1000000 ) {
-				*cd= *((unsigned int *)face->v2+3);
-				*((unsigned int *)face->v2+3) = vcount;
-				*fpp= face->v2;
-				fpp++; vcount++; cd++;
-			}
-			if( *((unsigned int *)face->v3+3) & 0x1000000 ) {
-				*cd= *((unsigned int *)face->v3+3);
-				*((unsigned int *)face->v3+3) = vcount;
-				*fpp= face->v3;
-				fpp++; vcount++; cd++;
-			}
-			if(face->v4 && ( *((unsigned int *)face->v4+3) & 0x1000000 ) ) {
-				*cd= *((unsigned int *)face->v4+3);
-				*((unsigned int *)face->v4+3) = vcount;
-				*fpp= face->v4;
-				fpp++; vcount++; cd++;
-			}
-
-			if(vcount>=BLSIZE-4) {
-				vlnr++;
-				break;
-			}
-		}
-
-		/* we now make the Mesh */
-		endf= vlnr;
-
-		ob= add_object(OB_MESH);
-		
-		me= ob->data;
-		me->totvert= vcount;
-		me->totface= endf-startf;
-		me->flag= 0;
-		me->mvert= MEM_callocN(me->totvert*sizeof(MVert), "mverts");
-		me->mcol= MEM_callocN(4*me->totface*sizeof(MCol), "mverts");
-		me->mface= MEM_callocN(me->totface*sizeof(MFace), "mface");
-
-
-		/* materials, and set VCOL flag */
-		for(a=0; a<RG.totmat; a++) {
-			assign_material(ob, RG.matar[a], a+1);
-			ma= RG.matar[a];
-			if(ma) ma->mode |= MA_VERTEXCOL;
-		}
-
-		/* verts */
-		mvert= me->mvert;
-		fpp= poindata;
-		for(a=0; a<me->totvert; a++, mvert++, fpp++) {
-			VECCOPY(mvert->co, *fpp);
-		}
-
-		/* faces and mcol */
-		mface= me->mface;
-		md= (unsigned int *)me->mcol;
-
-		face= RG.facebase[(startf-1)>>10]+((startf-1) & 1023);
-		for(a=startf; a<endf; a++, md+=4, mface++) {
-			RAD_NEXTFACE(a);
-			mface->v1= *((unsigned int *)face->v1+3);
-			mface->v2= *((unsigned int *)face->v2+3);
-			mface->v3= *((unsigned int *)face->v3+3);
-			if(face->v4) mface->v4= *((unsigned int *)face->v4+3);
-
-			mface->edcode= 3;
-			test_index_mface(mface, face->v4 ? 4 : 3);
-			mface->mat_nr= face->matindex;
-			
-			md[0]= coldata[mface->v1];
-			md[1]= coldata[mface->v2];
-			md[2]= coldata[mface->v3];
-			md[3]= coldata[mface->v4];
-		}
-		
-		/* boundbox and centrenew */
-		
-		INIT_MINMAX(min, max);
-
-		mvert= me->mvert;
-		for(a=0; a<me->totvert; a++, mvert++) {
-			DO_MINMAX(mvert->co, min, max);
-		}
-
-		cent[0]= (min[0]+max[0])/2.0;
-		cent[1]= (min[1]+max[1])/2.0;
-		cent[2]= (min[2]+max[2])/2.0;
-
-		mvert= me->mvert;
-		for(a=0; a<me->totvert; a++, mvert++) {
-			VecSubf(mvert->co, mvert->co, cent);
-		}
-		
-		VECCOPY(ob->loc, cent);
-		tex_space_mesh(me);
-		
-		/* restore colors */
-		face= RG.facebase[(startf-1)>>10]+((startf-1) & 1023);
-		for(a=startf; a<endf; a++) {
-			RAD_NEXTFACE(a);
-
-			cd= ((unsigned int *)face->v1+3);
-			if( *cd < 0x1000000 ) *cd= coldata[*cd];
-			cd= ((unsigned int *)face->v2+3);
-			if( *cd < 0x1000000 ) *cd= coldata[*cd];
-			cd= ((unsigned int *)face->v3+3);
-			if( *cd < 0x1000000 ) *cd= coldata[*cd];
-			if(face->v4) {
-				cd= ((unsigned int *)face->v4+3);
-				if( *cd < 0x1000000 ) *cd= coldata[*cd];
-			}
-		}
-	}
-		
-	MEM_freeN(coldata);
-	MEM_freeN(poindata);
-	
+	/* create edges */
+	make_edges(me, 0);
 }
 
 void rad_replacemesh(void)
@@ -863,7 +816,11 @@ void rad_replacemesh(void)
 	
 	rp= RG.patchbase.first;
 	while(rp) {
-		if( exist_object(rp->from)) rp->from->flag |= SELECT;
+		if( exist_object(rp->from)) {
+			if (rp->from->type == OB_MESH) {
+				rp->from->flag |= SELECT;
+			}
+		}
 		rp= rp->next;
 	}
 	
