@@ -37,7 +37,7 @@
 	- countglobaldata()
 	- makeGlobalElemArray()
 	
-   $Id: radpreprocess.c,v 1.17 2006/06/25 13:58:41 campbellbarton Exp $
+   $Id: radpreprocess.c,v 1.19 2006/11/20 04:27:57 blendix Exp $
 
   *************************************** */
 
@@ -57,14 +57,18 @@
 #include "DNA_scene_types.h"
 #include "DNA_view3d_types.h"
 
-#include "BKE_utildefines.h"
+#include "BKE_customdata.h"
 #include "BKE_global.h"
 #include "BKE_main.h"
 #include "BKE_material.h"
+#include "BKE_mesh.h"
 #include "BKE_object.h" /* during_script() */
+#include "BKE_utildefines.h"
 
 #include "BIF_toolbox.h"
 
+#include "BDR_editface.h"
+
 #include "radio.h"
 
 #ifdef HAVE_CONFIG_H
@@ -306,12 +310,13 @@ void rad_collect_meshes()
 	Mesh *me;
 	MVert *mvert;
 	MFace *mface;
+	MTFace *tf, *tface;
 	Material *ma = NULL, *noma= NULL;
 	RPatch *rp;
 	RNode *rn;
 	VeNoCo *vnc, **nodevert;
 	float *vd, *v1, *v2, *v3, *v4 = NULL;
-	int a, b, offs, index;
+	int a, b, offs, index, mfdatatot;
 	
 	if (G.vd==NULL) {
 		printf("Error, trying to collect radiosity meshes with no 3d view\n");
@@ -351,6 +356,8 @@ void rad_collect_meshes()
 
 	RG.min[0]= RG.min[1]= RG.min[2]= 1.0e20f;
 	RG.max[0]= RG.max[1]= RG.max[2]= -1.0e20f;
+
+	mfdatatot= 0;
 	
 	/* min-max and material array */
 	base= (G.scene->base.first);
@@ -394,6 +401,8 @@ void rad_collect_meshes()
 						RG.totmat++;
 					}
 				}
+
+				mfdatatot += me->totface;
 			}
 		}
 		base= base->next;
@@ -407,6 +416,10 @@ void rad_collect_meshes()
 	RG.size[2]= (RG.max[2]- RG.min[2]);
 	RG.maxsize= MAX3(RG.size[0],RG.size[1],RG.size[2]);
 
+	RG.mfdata= MEM_callocN(sizeof(CustomData), "radiomfdata");
+	RG.mfdatanodes= MEM_mallocN(sizeof(RNode*)*mfdatatot, "radiomfdatanodes");
+	RG.mfdatatot= mfdatatot;
+
 	/* make patches */
 
 	RG.totelem= 0;
@@ -421,107 +434,103 @@ void rad_collect_meshes()
 				ob= base->object;
 				me= ob->data;
 				mface= me->mface;
+				tface= me->mtface;
 				
 				index= -1;
 
+				CustomData_merge(&me->fdata, RG.mfdata, CD_MASK_DERIVEDMESH,
+					CD_DEFAULT, mfdatatot);
+				
 				for(a=0; a<me->totface; a++, mface++) {
-					TFace *tface = me->tface ? &((TFace*)me->tface)[a] : NULL;						
+					tf= tface? tface+a: NULL;
 					
-					if (tface == NULL || (tface->mode & TF_INVISIBLE)==0) {
+					if (tf && (tf->mode & TF_INVISIBLE))
+						continue;
 
-    					rp= callocPatch();
-    					BLI_addtail(&(RG.patchbase), rp);
-    					rp->from= ob;
-    					
-    					if(mface->v4) rp->type= 4;
-    					else rp->type= 3;
-    					
-    					rp->first= rn= callocNode();
-    					
-    					if(mface->flag & ME_SMOOTH) rp->f1= RAD_NO_SPLIT;
-    					
-    					/* temporal: we store the venoco in the node */
-    					rn->v1= (float *)(RG.verts+mface->v1+offs);
-    					v1= (RG.verts+mface->v1+offs)->v;
-    					rn->v2= (float *)(RG.verts+mface->v2+offs);
-    					v2= (RG.verts+mface->v2+offs)->v;
-    					rn->v3= (float *)(RG.verts+mface->v3+offs);
-    					v3= (RG.verts+mface->v3+offs)->v;
+					rp= callocPatch();
+					BLI_addtail(&(RG.patchbase), rp);
+					rp->from= ob;
+					
+					if(mface->v4) rp->type= 4;
+					else rp->type= 3;
+					
+					rp->first= rn= callocNode();
+					
+					if(mface->flag & ME_SMOOTH) rp->f1= RAD_NO_SPLIT;
+					
+					/* temporal: we store the venoco in the node */
+					rn->v1= (float *)(RG.verts+mface->v1+offs);
+					v1= (RG.verts+mface->v1+offs)->v;
+					rn->v2= (float *)(RG.verts+mface->v2+offs);
+					v2= (RG.verts+mface->v2+offs)->v;
+					rn->v3= (float *)(RG.verts+mface->v3+offs);
+					v3= (RG.verts+mface->v3+offs)->v;
 
-    					if(mface->v4) {
-    						rn->v4= (float *)(RG.verts+mface->v4+offs);
-    						v4= (RG.verts+mface->v4+offs)->v;
-    					}			
-    					rn->par= rp;
-    					rn->f= RAD_PATCH;	/* this node is a Patch */
-    					rn->type= rp->type;
+					if(mface->v4) {
+						rn->v4= (float *)(RG.verts+mface->v4+offs);
+						v4= (RG.verts+mface->v4+offs)->v;
+					}			
+					rn->par= rp;
+					rn->f= RAD_PATCH;	/* this node is a Patch */
+					rn->type= rp->type;
 
-    					if(rn->type==4) {
-    						rp->area= AreaQ3Dfl(v1, v2, v3, v4);
-    						CalcNormFloat4(v1, v2, v3, v4, rp->norm);
-    					}
-    					else {
-    						rp->area= AreaT3Dfl(v1, v2, v3);
-    						CalcNormFloat(v1, v2, v3, rp->norm);
-    					}
+					if(rn->type==4) {
+						rp->area= AreaQ3Dfl(v1, v2, v3, v4);
+						CalcNormFloat4(v1, v2, v3, v4, rp->norm);
+					}
+					else {
+						rp->area= AreaT3Dfl(v1, v2, v3);
+						CalcNormFloat(v1, v2, v3, rp->norm);
+					}
 
-    					if (tface) {
-    						memcpy(rn->uv, tface->uv, sizeof(float) * 4 * 2);
-    						rn->tface = tface;
-    					}
-    					else {
-    						rn->uv[0][0] = 0.0f;
-    						rn->uv[0][1] = 0.0f;
-    						rn->uv[1][0] = 1.0f;
-    						rn->uv[1][1] = 0.0f;
-    						rn->uv[2][0] = 1.0f;
-    						rn->uv[2][1] = 1.0f;
-    						rn->uv[3][0] = 0.0f;
-    						rn->uv[3][1] = 1.0f;
-    						rn->tface = NULL;
-    					}
+					rn->area= rp->area;
 
-    					rn->area= rp->area;
+					/* color and emit */
+					if(mface->mat_nr != index) {
+						index= mface->mat_nr;
+						ma= give_current_material(ob, index+1);
+						if(ma==0) ma= &defmaterial;
+					}
+					rp->ref[0]= ma->r;
+					rp->ref[1]= ma->g;
+					rp->ref[2]= ma->b;
 
-    					/* color and emit */
-    					if(mface->mat_nr != index) {
-    						index= mface->mat_nr;
-    						ma= give_current_material(ob, index+1);
-    						if(ma==0) ma= &defmaterial;
-    					}
-    					rp->ref[0]= ma->r;
-    					rp->ref[1]= ma->g;
-    					rp->ref[2]= ma->b;
+					if(ma->emit) RG.totlamp++;
 
-    					if(ma->emit) RG.totlamp++;
-    
-    					rp->emit[0]= rp->emit[1]= rp->emit[2]= ma->emit;
-    					rp->emit[0]*= rp->ref[0];
-    					rp->emit[1]*= rp->ref[1];
-    					rp->emit[2]*= rp->ref[2];
+					rp->emit[0]= rp->emit[1]= rp->emit[2]= ma->emit;
+					rp->emit[0]*= rp->ref[0];
+					rp->emit[1]*= rp->ref[1];
+					rp->emit[2]*= rp->ref[2];
 
 // uncommented, this is not satisfying, but i leave it in code for now (ton)						
 //						if(ma->translucency!=0.0) rn->f |= RAD_TWOSIDED;
 
-    					nodevert= (VeNoCo **)&(rn->v1);
-    					for(b=0; b<rp->type; b++) {
-    						rp->cent[0]+= (*nodevert)->v[0];
-    						rp->cent[1]+= (*nodevert)->v[1];
-    						rp->cent[2]+= (*nodevert)->v[2];
-    						nodevert++;
-    					}
-    					rp->cent[0]/= (float)rp->type;
-    					rp->cent[1]/= (float)rp->type;
-    					rp->cent[2]/= (float)rp->type;
-    					
-    					/* for reconstruction materials */
-    					rp->matindex= materialIndex(ma);
-    					if(rp->matindex==-1) rp->matindex= 1;
-    					
-    					RG.totelem++;
-    					RG.totpatch++;
-    				}
+					nodevert= (VeNoCo **)&(rn->v1);
+					for(b=0; b<rp->type; b++) {
+						rp->cent[0]+= (*nodevert)->v[0];
+						rp->cent[1]+= (*nodevert)->v[1];
+						rp->cent[2]+= (*nodevert)->v[2];
+						nodevert++;
+					}
+					rp->cent[0]/= (float)rp->type;
+					rp->cent[1]/= (float)rp->type;
+					rp->cent[2]/= (float)rp->type;
+					
+					/* for reconstruction materials */
+					rp->matindex= materialIndex(ma);
+					if(rp->matindex==-1) rp->matindex= 1;
+
+					/* these RNode's are stored now for later use in rad_addmesh
+					   they should not get deleted before that */
+					rn->orig= RG.totelem;
+					RG.mfdatanodes[RG.totelem]= rn;
+
+					CustomData_copy_data(&me->fdata, RG.mfdata, a, RG.totelem, 1);
+					
+					RG.totelem++;
+					RG.totpatch++;
 				}
+
 				offs+= me->totvert;
 			}
 		}
