@@ -1,5 +1,5 @@
 /**
- * $Id: RE_pipeline.h,v 1.18 2006/06/30 14:21:25 ton Exp $
+ * $Id: RE_pipeline.h,v 1.24 2006/12/21 10:41:43 ton Exp $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -59,20 +59,26 @@ typedef struct Render Render;
 typedef struct RenderPass {
 	struct RenderPass *next, *prev;
 	int passtype, channels;
+	char name[16];		/* amount defined in openexr_multi.h */
+	char chan_id[8];	/* amount defined in openexr_multi.h */
 	float *rect;
 } RenderPass;
 
 /* a renderlayer is a full image, but with all passes and samples */
 /* size of the rects is defined in RenderResult */
+/* after render, the Combined pass is in rectf, for renderlayers read from files it is a real pass */
 typedef struct RenderLayer {
 	struct RenderLayer *next, *prev;
 	
 	/* copy of RenderData */
 	char name[RE_MAXNAME];		
 	unsigned int lay;			
-	int layflag, passflag;		
+	int layflag, passflag, pass_xor;		
 	
-	float *rectf;		/* 4 float, standard rgba buffer */
+	struct Material *mat_override;
+	struct Group *light_override;
+	
+	float *rectf;		/* 4 float, standard rgba buffer (read not above!) */
 	float *acolrect;	/* 4 float, optional transparent buffer, needs storage for display updates */
 	
 	ListBase passes;
@@ -108,6 +114,9 @@ typedef struct RenderResult {
 	char exrfile[FILE_MAXDIR];
 	void *exrhandle;
 	
+	/* for render results in Image, verify validity for sequences */
+	int framenr;
+	
 } RenderResult;
 
 typedef struct RenderStats {
@@ -131,6 +140,7 @@ void RE_FreeRender (struct Render *re);
 void RE_FreeAllRender (void);
 
 /* get results and statistics */
+void RE_FreeRenderResult(struct RenderResult *rr);
 struct RenderResult *RE_GetResult(struct Render *re);
 void RE_GetResultImage(struct Render *re, struct RenderResult *rr);
 struct RenderStats *RE_GetStats(struct Render *re);
@@ -159,6 +169,9 @@ void RE_Database_Free (struct Render *re);
 /* project dbase again, when viewplane/perspective changed */
 void RE_DataBase_ApplyWindow(struct Render *re);
 
+/* override the scene setting for amount threads, commandline */
+void RE_set_max_threads(int threads);
+
 /* the main processor, assumes all was set OK! */
 void RE_TileProcessor(struct Render *re, int firsttile);
 
@@ -167,6 +180,8 @@ void RE_BlenderFrame(struct Render *re, struct Scene *scene, int frame);
 void RE_BlenderAnim(struct Render *re, struct Scene *scene, int sfra, int efra);
 
 void RE_ReadRenderResult(struct Scene *scene, struct Scene *scenode);
+void RE_WriteRenderResult(RenderResult *rr, char *filename, int compress);
+struct RenderResult *RE_MultilayerConvert(void *exrhandle, int rectx, int recty);
 
 /* ancient stars function... go away! */
 void RE_make_stars(struct Render *re, void (*initfunc)(void),
@@ -187,8 +202,14 @@ float RE_filter_value(int type, float x);
 /* vector blur zbuffer method */
 void RE_zbuf_accumulate_vecblur(struct NodeBlurData *nbd, int xsize, int ysize, float *newrect, float *imgrect, float *vecbufrect, float *zbufrect);
 
-/* shaded view options */
-void RE_Database_Shaded(struct Render *re, struct Scene *scene);
+/* shaded view or baking options */
+#define RE_BAKE_LIGHT	0
+#define RE_BAKE_ALL		1
+#define RE_BAKE_AO		2
+#define RE_BAKE_NORMALS	3
+#define RE_BAKE_TEXTURE	4
+void RE_Database_Baking(struct Render *re, struct Scene *scene, int type);
+
 void RE_DataBase_GetView(struct Render *re, float mat[][4]);
 
 #endif /* RE_PIPELINE_H */
