@@ -1,5 +1,5 @@
 /**
- * $Id: render_types.h,v 1.15 2006/07/13 10:26:35 ton Exp $
+ * $Id: render_types.h,v 1.26 2006/12/21 13:47:26 blendix Exp $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -37,19 +37,20 @@
 #include "DNA_object_types.h"
 #include "DNA_vec_types.h"
 
+#include "BLI_threads.h"
+
 #include "RE_pipeline.h"
 #include "RE_shader_ext.h"	/* TexResult, ShadeResult, ShadeInput */
 
 struct Object;
 struct MemArena;
 struct VertTableNode;
+struct VlakTableNode;
 struct Octree;
 struct GHash;
 
 #define TABLEINITSIZE 1024
 #define LAMPINITSIZE 256
-		/* hardcoded maximum now, for optimize tables */
-#define MAX_THREADS		2
 
 typedef struct SampleTables
 {
@@ -77,6 +78,7 @@ typedef struct RenderPart
 	short sample, nr;				/* sample can be used by zbuffers, nr is partnr */
 	short thread;					/* thread id */
 	
+	char *clipflag;					/* clipflags for part zbuffering */
 } RenderPart;
 
 typedef struct Octree {
@@ -108,6 +110,7 @@ struct Render
 	rcti disprect;			/* part within winx winy */
 	rctf viewplane;			/* mapped on winx winy */
 	float viewdx, viewdy;	/* size of 1 pixel */
+	float clipcrop;			/* 2 pixel boundary to prevent clip when filter used */
 	
 	/* final picture width and height (within disprect) */
 	int rectx, recty;
@@ -120,7 +123,6 @@ struct Render
 	
 	/* values for viewing */
 	float lens, ycor, viewfac;
-	float bluroffsx, bluroffsy;
 	float panophi, panosi, panoco, panodxp, panodxv;
 	
 	/* Matrices */
@@ -152,20 +154,23 @@ struct Render
 	
 	/* render database */
 	int totvlak, totvert, tothalo, totlamp;
-	ListBase lights;
+	ListBase lights;	/* GroupObject pointers */
+	ListBase lampren;	/* storage, for free */
 	
 	int vertnodeslen;
 	struct VertTableNode *vertnodes;
+	int vlaknodeslen;
+	struct VlakTableNode *vlaknodes;
 	int blohalen;
 	struct HaloRen **bloha;
-	int blovllen;
-	struct VlakRen **blovl;
 	ListBase objecttable;
-	
-	struct Image *backbuf;
+
+	struct Image *backbuf, *bakebuf;
 	
 	struct GHash *orco_hash;
 
+	ListBase customdata_names;
+
 	/* arena for allocating data for use during render, for
 		* example dynamic TFaces to go in the VlakRen structure.
 		*/
@@ -188,6 +193,8 @@ struct Render
 
 /* ------------------------------------------------------------------------- */
 
+struct ISBData;
+
 typedef struct ShadSampleBuf {
 	struct ShadSampleBuf *next, *prev;
 	long *zbuf;
@@ -195,6 +202,7 @@ typedef struct ShadSampleBuf {
 } ShadSampleBuf;
 
 typedef struct ShadBuf {
+	/* regular shadowbuffer */
 	short samp, shadhalostep, totbuf;
 	float persmat[4][4];
 	float viewmat[4][4];
@@ -204,6 +212,9 @@ typedef struct ShadBuf {
 	int co[3];
 	int size, bias;
 	ListBase buffers;
+	
+	/* irregular shadowbufer, result stored per thread */
+	struct ISBData *isb_result[BLENDER_MAX_THREADS];
 } ShadBuf;
 
 /* ------------------------------------------------------------------------- */
@@ -238,7 +249,7 @@ struct halosort {
 
 /* ------------------------------------------------------------------------- */
 struct Material;
-struct TFace;
+struct MTFace;
 
 typedef struct RadFace {
 	float unshot[3], totrad[3];
@@ -246,18 +257,16 @@ typedef struct RadFace {
 	int flag;
 } RadFace;
 
-typedef struct VlakRen
-{
-	struct VertRen *v1, *v2, *v3, *v4;
+typedef struct VlakRen {
+	struct VertRen *v1, *v2, *v3, *v4;	/* keep in order for ** addressing */
 	unsigned int lay;
 	float n[3];
 	struct Material *mat;
-	struct TFace *tface;
-	unsigned int *vcol;
 	char snproj, puno;
 	char flag, ec;
 	RadFace *radface;
 	Object *ob;
+	int index;
 } VlakRen;
 
 typedef struct HaloRen
@@ -283,8 +292,19 @@ struct MTex;
  * For each lamp in a scene, a LampRen is created. It determines the
  * properties of a lightsource.
  */
-typedef struct LampRen
-{
+
+typedef struct LampShadowSubSample {
+	int samplenr;
+	float shadfac[4];	/* rgba shadow */
+} LampShadowSubSample;
+
+typedef struct LampShadowSample {
+	LampShadowSubSample s[16];	/* MAX OSA */
+} LampShadowSample;
+
+typedef struct LampRen {
+	struct LampRen *next, *prev;
+	
 	float xs, ys, dist;
 	float co[3];
 	short type, mode;
@@ -304,8 +324,12 @@ typedef struct LampRen
 	short samp;
 	/** Softness factor for shadow */
 	float soft;
-	/** amount of subsample buffers */
+	/** amount of subsample buffers and type of filter for sampling */
 	short buffers, filtertype;
+	/** shadow buffer type (regular, irregular) */
+	short buftype;
+	/** autoclip */
+	short bufflag;
 	/** shadow plus halo: detail level */
 	short shadhalostep;
 	/** Near clip of the lamp */
@@ -316,7 +340,7 @@ typedef struct LampRen
 	float bias;
 	
 	short ray_samp, ray_sampy, ray_sampz, ray_samp_type, area_shape, ray_totsamp;
-	short xold1, yold1, xold2, yold2;	/* last jitter table for area lights */
+	short xold[BLENDER_MAX_THREADS], yold[BLENDER_MAX_THREADS];	/* last jitter table for area lights */
 	float area_size, area_sizey, area_sizez;
 	
 	struct ShadBuf *shb;
@@ -329,6 +353,9 @@ typedef struct LampRen
 	float mat[3][3];	/* 3x3 part from lampmat x viewmat */
 	float area[8][3], areasize;
 	
+	/* passes & node shader support: all shadow info for a pixel */
+	LampShadowSample *shadsamp;
+	
 	/* yafray: photonlight params */
 	int YF_numphotons, YF_numsearch;
 	short YF_phdepth, YF_useqmc, YF_bufsize;
@@ -337,7 +364,7 @@ typedef struct LampRen
 	short YF_glowtype;
 	
 	/* ray optim */
-	VlakRen *vlr_last[MAX_THREADS];
+	VlakRen *vlr_last[BLENDER_MAX_THREADS];
 	
 	struct MTex *mtex[MAX_MTEX];
 } LampRen;
@@ -351,6 +378,7 @@ typedef struct LampRen
 #define R_HALO			2
 #define R_SEC_FIELD		4
 #define R_LAMPHALO		8
+#define R_GLOB_NOPUNOFLIP	16
 
 /* vlakren->flag (vlak = face in dutch) char!!! */
 #define R_SMOOTH		1
