@@ -1,7 +1,7 @@
 /*
  * rendercore_ext.h
  *
- * $Id: rendercore.h,v 1.27 2006/06/13 14:51:17 ton Exp $
+ * $Id: rendercore.h,v 1.30 2006/12/05 16:42:59 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -46,22 +46,18 @@
 struct HaloRen;
 struct ShadeInput;
 struct ShadeResult;
+struct World;
+struct RenderPart;
+struct RenderLayer;
 
 /* ------------------------------------------------------------------------- */
 
-/* to make passing on variables to shadepixel() easier */
-typedef struct ShadePixelInfo {
-	int thread;
-	int layflag, passflag;
-	unsigned int lay;
-	ShadeResult shr;
-} ShadePixelInfo;
-
 typedef struct PixStr
 {
 	struct PixStr *next;
 	int facenr, z;
-	unsigned short mask, amount;
+	unsigned short mask;
+	short shadfac;
 } PixStr;
 
 typedef struct PixStrMain
@@ -77,25 +73,17 @@ typedef struct PixStrMain
 void	calc_view_vector(float *view, float x, float y);
 float   mistfactor(float zcor, float *co);	/* dist and height, return alpha */
 
+void	renderspothalo(struct ShadeInput *shi, float *col, float alpha);
 void	add_halo_flare(Render *re);
 
-void	shade_input_set_coords(ShadeInput *shi, float u, float v, int i1, int i2, int i3);
+void calc_renderco_zbuf(float *co, float *view, int z);
+void calc_renderco_ortho(float *co, float x, float y, int z);
 
-void	shade_color(struct ShadeInput *shi, ShadeResult *shr);
-void	shade_lamp_loop(struct ShadeInput *shi, ShadeResult *shr);
-
-float	fresnel_fac(float *view, float *vn, float fresnel, float fac);
-void	calc_R_ref(struct ShadeInput *shi);
-
-/* for nodes */
-void shade_material_loop(struct ShadeInput *shi, struct ShadeResult *shr);
+int count_mask(unsigned short mask);
 
 void zbufshade(void);
 void zbufshadeDA(void);	/* Delta Accum Pixel Struct */
 
-void *shadepixel(ShadePixelInfo *shpi, float x, float y, int z, volatile int facenr, int mask, float *rco);
-int count_mask(unsigned short mask);
-
 void zbufshade_tile(struct RenderPart *pa);
 void zbufshadeDA_tile(struct RenderPart *pa);
 
@@ -108,7 +96,7 @@ extern void ray_shadow(ShadeInput *, LampRen *, float *);
 extern void ray_trace(ShadeInput *, ShadeResult *);
 extern void ray_ao(ShadeInput *, float *);
 extern void init_jitter_plane(LampRen *lar);
-extern void init_ao_sphere(float *sphere, int tot, int iter);
+extern void init_ao_sphere(struct World *wrld);
 
 #endif /* RENDER_EXT_H */
 
