@@ -1,5 +1,5 @@
 /*
- * $Id: zbuf.h,v 1.12 2006/02/12 20:21:07 ton Exp $
+ * $Id: zbuf.h,v 1.16 2006/12/05 16:42:59 ton Exp $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -49,13 +49,14 @@ int testclip(float *v);
 void set_part_zbuf_clipflag(struct RenderPart *pa);
 void zbuffer_shadow(struct Render *re, struct LampRen *lar, int *rectz, int size, float jitx, float jity);
 void zbuffer_solid(struct RenderPart *pa, unsigned int layer, short layflag);
-void zbuffer_transp_shade(struct RenderPart *pa, struct RenderLayer *rl, float *pass);
+unsigned short *zbuffer_transp_shade(struct RenderPart *pa, struct RenderLayer *rl, float *pass);
 void convert_zbuf_to_distbuf(struct RenderPart *pa, struct RenderLayer *rl);
 
 typedef struct APixstr {
     unsigned short mask[4]; /* jitter mask */
-    int z[4];       /* distance    */
-    int p[4];       /* index       */
+    int z[4];			/* distance    */
+    int p[4];			/* index       */
+	short shadfac[4];	/* optimize storage for irregular shadow */
     struct APixstr *next;
 } APixstr;
 
@@ -76,11 +77,13 @@ typedef struct ZSpan {
 	float zmulx, zmuly, zofsx, zofsy;		/* transform from hoco to zbuf co */
 	
 	int *rectz, *arectz;					/* zbuffers, arectz is for transparant */
+	int *rectz1;							/* seconday z buffer for shadowbuffer (2nd closest z) */
 	int *rectp;								/* polygon index buffer */
 	APixstr *apixbuf, *curpstr;				/* apixbuf for transparent */
 	struct ListBase *apsmbase;
 	
 	int polygon_offset;						/* offset in Z */
+	float shad_alpha;						/* copy from material, used by irregular shadbuf */
 	int mask, apsmcounter;					/* in use by apixbuf */
 	
 	void (*zbuffunc)(struct ZSpan *, int, float *, float *, float *, float *);
@@ -88,10 +91,17 @@ typedef struct ZSpan {
 	
 } ZSpan;
 
-/* exported for evil edge render... */
+/* exported to shadbuf.c */
+void zbufclip4(struct ZSpan *zspan, int zvlnr, float *f1, float *f2, float *f3, float *f4, int c1, int c2, int c3, int c4);
+void zbuf_free_span(struct ZSpan *zspan);
+
+/* to rendercore.c */
+void zspan_scanconvert(struct ZSpan *zpan, void *handle, float *v1, float *v2, float *v3, void (*func)(void *, int, int, float, float) );
+
+/* exported to edge render... */
 void zbufclip(struct ZSpan *zspan, int zvlnr, float *f1, float *f2, float *f3, int c1, int c2, int c3);
-void zbuf_alloc_span(ZSpan *zspan, int rectx, int recty);
-void zbufclipwire(ZSpan *zspan, int zvlnr, struct VlakRen *vlr); 
+void zbuf_alloc_span(struct ZSpan *zspan, int rectx, int recty);
+void zbufclipwire(struct ZSpan *zspan, int zvlnr, struct VlakRen *vlr); 
 
 #endif
 
