@@ -1,5 +1,5 @@
 /* 
- * $Id: envmap.c,v 1.31 2006/05/11 10:27:12 ton Exp $
+ * $Id: envmap.c,v 1.39 2007/01/31 11:49:30 ton Exp $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -66,43 +66,36 @@
 
 /* ------------------------------------------------------------------------- */
 
-static void envmap_split_ima(EnvMap *env)
+static void envmap_split_ima(EnvMap *env, ImBuf *ibuf)
 {
-	ImBuf *ibuf;
-	Image *ima;
-/*  	extern rectcpy(); */
 	int dx, part;
 	
 	BKE_free_envmapdata(env);	
 	
-	dx= env->ima->ibuf->y;
+	dx= ibuf->y;
 	dx/= 2;
-	if(3*dx != env->ima->ibuf->x) {
+	if(3*dx != ibuf->x) {
 		printf("Incorrect envmap size\n");
 		env->ok= 0;
 		env->ima->ok= 0;
 	}
 	else {
 		for(part=0; part<6; part++) {
-			ibuf= IMB_allocImBuf(dx, dx, 24, IB_rect, 0);
-			ima= MEM_callocN(sizeof(Image), "image");
-			ima->ibuf= ibuf;
-			ima->ok= 1;
-			env->cube[part]= ima;
+			env->cube[part]= IMB_allocImBuf(dx, dx, 24, IB_rect, 0);
 		}
-		IMB_rectcpy(env->cube[0]->ibuf, env->ima->ibuf, 
+		IMB_rectcpy(env->cube[0], ibuf, 
 			0, 0, 0, 0, dx, dx);
-		IMB_rectcpy(env->cube[1]->ibuf, env->ima->ibuf, 
+		IMB_rectcpy(env->cube[1], ibuf, 
 			0, 0, dx, 0, dx, dx);
-		IMB_rectcpy(env->cube[2]->ibuf, env->ima->ibuf, 
+		IMB_rectcpy(env->cube[2], ibuf, 
 			0, 0, 2*dx, 0, dx, dx);
-		IMB_rectcpy(env->cube[3]->ibuf, env->ima->ibuf, 
+		IMB_rectcpy(env->cube[3], ibuf, 
 			0, 0, 0, dx, dx, dx);
-		IMB_rectcpy(env->cube[4]->ibuf, env->ima->ibuf, 
+		IMB_rectcpy(env->cube[4], ibuf, 
 			0, 0, dx, dx, dx, dx);
-		IMB_rectcpy(env->cube[5]->ibuf, env->ima->ibuf, 
+		IMB_rectcpy(env->cube[5], ibuf, 
 			0, 0, 2*dx, dx, dx, dx);
-		env->ok= 2;
+		env->ok= ENV_OSA;
 	}
 }
 
@@ -134,7 +127,7 @@ static Render *envmap_render_copy(Render *re, EnvMap *env)
 	envre->r.size= 100;
 	envre->r.yasp= envre->r.xasp= 1;
 	
-	RE_InitState(envre, &re->r, cuberes, cuberes, NULL);
+	RE_InitState(envre, &envre->r, cuberes, cuberes, NULL);
 	envre->scene= re->scene;	/* unsure about this... */
 
 	/* view stuff in env render */
@@ -161,8 +154,9 @@ static Render *envmap_render_copy(Render *re, EnvMap *env)
 	envre->vertnodes= re->vertnodes;
 	envre->blohalen= re->blohalen;
 	envre->bloha= re->bloha;
-	envre->blovllen= re->blovllen;
-	envre->blovl= re->blovl;
+	envre->vlaknodeslen= re->vlaknodeslen;
+	envre->vlaknodes= re->vlaknodes;
+	envre->customdata_names= re->customdata_names;
 	envre->oc= re->oc;
 	
 	return envre;
@@ -180,8 +174,9 @@ static void envmap_free_render_copy(Render *envre)
 	envre->vertnodes= NULL;
 	envre->blohalen= 0;
 	envre->bloha= NULL;
-	envre->blovllen= 0;
-	envre->blovl= NULL;
+	envre->vlaknodeslen= 0;
+	envre->vlaknodes= NULL;
+	envre->customdata_names.first= envre->customdata_names.last= NULL;
 	envre->oc.adrbranch= NULL;
 	envre->oc.adrnode= NULL;
 	
@@ -265,7 +260,7 @@ static void env_rotate_scene(Render *re, float mat[][4], int mode)
 	}
 		
 	for(a=0; a<re->totvlak; a++) {
-		if((a & 255)==0) vlr= re->blovl[a>>8];
+		if((a & 255)==0) vlr= re->vlaknodes[a>>8].vlak;
 		else vlr++;
 		
 		xn= vlr->n[0];
@@ -328,7 +323,7 @@ static void env_layerflags(Render *re, unsigned int notlay)
 	notlay= ~notlay;
 	
 	for(a=0; a<re->totvlak; a++) {
-		if((a & 255)==0) vlr= re->blovl[a>>8];
+		if((a & 255)==0) vlr= re->vlaknodes[a>>8].vlak;
 		else vlr++;
 		if((vlr->lay & notlay)==0)
 			vlr->flag &= ~R_VISIBLE;
@@ -341,7 +336,7 @@ static void env_hideobject(Render *re, Object *ob)
 	int a;
 	
 	for(a=0; a<re->totvlak; a++) {
-		if((a & 255)==0) vlr= re->blovl[a>>8];
+		if((a & 255)==0) vlr= re->vlaknodes[a>>8].vlak;
 		else vlr++;
 		if(vlr->ob == ob) vlr->flag &= ~R_VISIBLE;
 	}
@@ -371,7 +366,6 @@ static void render_envmap(Render *re, EnvMap *env)
 	/* only the cubemap and planar map is implemented */
 	Render *envre;
 	ImBuf *ibuf;
-	Image *ima;
 	float orthmat[4][4];
 	float oldviewinv[4][4], mat[4][4], tmat[4][4];
 	short part;
@@ -430,11 +424,7 @@ static void render_envmap(Render *re, EnvMap *env)
 			IMB_rect_from_float(ibuf);
 			ibuf->rect_float= NULL;
 				
-			ima= MEM_callocN(sizeof(Image), "image");
-			
-			ima->ibuf= ibuf;
-			ima->ok= 1;
-			env->cube[part]= ima;
+			env->cube[part]= ibuf;
 		}
 		
 		if(re->test_break()) break;
@@ -476,33 +466,47 @@ void make_envmaps(Render *re)
 		while(tex) {
 			if(tex->id.us && tex->type==TEX_ENVMAP) {
 				if(tex->env && tex->env->object) {
-					if(tex->env->object->lay & G.scene->lay) {
-						if(tex->env->stype!=ENV_LOAD) {
+					EnvMap *env= tex->env;
+					
+					if(env->object->lay & G.scene->lay) {
+						if(env->stype==ENV_LOAD) {
+							float orthmat[4][4], mat[4][4], tmat[4][4];
+							
+							/* precalc orthmat for object */
+							MTC_Mat4CpyMat4(orthmat, env->object->obmat);
+							MTC_Mat4Ortho(orthmat);
+							
+							/* need imat later for texture imat */
+							MTC_Mat4MulMat4(mat, orthmat, re->viewmat);
+							MTC_Mat4Invert(tmat, mat);
+							MTC_Mat3CpyMat4(env->obimat, tmat);
+						}
+						else {
 							
 							/* decide if to render an envmap (again) */
-							if(tex->env->depth >= depth) {
+							if(env->depth >= depth) {
 								
 								/* set 'recalc' to make sure it does an entire loop of recalcs */
 								
-								if(tex->env->ok) {
+								if(env->ok) {
 										/* free when OSA, and old one isn't OSA */
-									if((re->r.mode & R_OSA) && tex->env->ok==ENV_NORMAL) 
-										BKE_free_envmapdata(tex->env);
+									if((re->r.mode & R_OSA) && env->ok==ENV_NORMAL) 
+										BKE_free_envmapdata(env);
 										/* free when size larger */
-									else if(tex->env->lastsize < re->r.size) 
-										BKE_free_envmapdata(tex->env);
+									else if(env->lastsize < re->r.size) 
+										BKE_free_envmapdata(env);
 										/* free when env is in recalcmode */
-									else if(tex->env->recalc)
-										BKE_free_envmapdata(tex->env);
+									else if(env->recalc)
+										BKE_free_envmapdata(env);
 								}
 								
-								if(tex->env->ok==0 && depth==0) tex->env->recalc= 1;
+								if(env->ok==0 && depth==0) env->recalc= 1;
 								
-								if(tex->env->ok==0) {
+								if(env->ok==0) {
 									do_init= 1;
-									render_envmap(re, tex->env);
+									render_envmap(re, env);
 									
-									if(depth==tex->env->depth) tex->env->recalc= 0;
+									if(depth==env->depth) env->recalc= 0;
 								}
 							}
 						}
@@ -614,7 +618,7 @@ int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexRe
 	extern Render R;				/* only in this call */
 	/* texvec should be the already reflected normal */
 	EnvMap *env;
-	Image *ima;
+	ImBuf *ibuf;
 	float fac, vec[3], sco[3], dxts[3], dyts[3];
 	int face, face1;
 	
@@ -626,11 +630,13 @@ int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexRe
 	if(env->stype==ENV_LOAD) {
 		env->ima= tex->ima;
 		if(env->ima && env->ima->ok) {
-			// Now thread safe
-			BLI_lock_thread(LOCK_MALLOC);
-			if(env->ima->ibuf==NULL) ima_ibuf_is_nul(tex, tex->ima);
-			if(env->ima->ok && env->ok==0) envmap_split_ima(env);
-			BLI_unlock_thread(LOCK_MALLOC);
+			if(env->cube[0]==NULL) {
+				ImBuf *ibuf= BKE_image_get_ibuf(env->ima, NULL);
+				if(ibuf)
+					envmap_split_ima(env, ibuf);
+				else
+					env->ok= 0;
+			}
 		}
 	}
 
@@ -646,7 +652,7 @@ int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexRe
 	else MTC_Mat4Mul3Vecfl(R.viewinv, vec);
 	
 	face= envcube_isect(env, vec, sco);
-	ima= env->cube[face];
+	ibuf= env->cube[face];
 	
 	if(osatex) {
 		if(env->object) {
@@ -658,7 +664,7 @@ int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexRe
 			MTC_Mat4Mul3Vecfl(R.viewinv, dyt);
 		}
 		set_dxtdyt(dxts, dyts, dxt, dyt, face);
-		imagewraposa(tex, ima, sco, dxts, dyts, texres);
+		imagewraposa(tex, NULL, ibuf, sco, dxts, dyts, texres);
 		
 		/* edges? */
 		
@@ -672,9 +678,9 @@ int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexRe
 			VecSubf(vec, vec, dxt);
 			
 			if(face!=face1) {
-				ima= env->cube[face1];
+				ibuf= env->cube[face1];
 				set_dxtdyt(dxts, dyts, dxt, dyt, face1);
-				imagewraposa(tex, ima, sco, dxts, dyts, &texr1);
+				imagewraposa(tex, NULL, ibuf, sco, dxts, dyts, &texr1);
 			}
 			else texr1.tr= texr1.tg= texr1.tb= texr1.ta= 0.0;
 			
@@ -685,9 +691,9 @@ int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexRe
 			VecSubf(vec, vec, dyt);
 			
 			if(face!=face1) {
-				ima= env->cube[face1];
+				ibuf= env->cube[face1];
 				set_dxtdyt(dxts, dyts, dxt, dyt, face1);
-				imagewraposa(tex, ima, sco, dxts, dyts, &texr2);
+				imagewraposa(tex, NULL, ibuf, sco, dxts, dyts, &texr2);
 			}
 			else texr2.tr= texr2.tg= texr2.tb= texr2.ta= 0.0;
 			
@@ -703,7 +709,7 @@ int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexRe
 		}
 	}
 	else {
-		imagewrap(tex, ima, sco, texres);
+		imagewrap(tex, NULL, ibuf, sco, texres);
 	}
 	
 	return 1;
