@@ -1,5 +1,5 @@
 /**
- * $Id: initrender.c,v 1.80 2006/07/19 15:39:36 ton Exp $
+ * $Id: initrender.c,v 1.86 2006/12/28 11:14:08 ton Exp $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -74,7 +74,6 @@
 
 #include "rendercore.h"
 #include "pixelshading.h"
-#include "gammaCorrectionTables.h"
 #include "zbuf.h"
 
 /* Own includes */
@@ -98,7 +97,7 @@ static void init_render_jit(Render *re)
 }
 
 
-/* ****************** GAMMA, MASKS and LUTS **************** */
+/* ****************** MASKS and LUTS **************** */
 
 static float filt_quadratic(float x)
 {
@@ -264,7 +263,6 @@ void make_sample_tables(Render *re)
 
 	/* optimization tables, only once */
 	if(firsttime) {
-		makeGammaTables(2.0);	/* tables only used for adding colors */
 		firsttime= 0;
 	}
 	
@@ -278,12 +276,6 @@ void make_sample_tables(Render *re)
 		return;
 	}
 	
-	re->do_gamma= 0;
-	if(re->r.mode & R_GAMMA) {
-		if(re->r.alphamode!=R_ALPHAKEY)	/* alpha corrected gamma doesnt work for key alpha */
-			re->do_gamma= 1;
-	}
-	
 	st= re->samples= MEM_callocN(sizeof(SampleTables), "sample tables");
 	
 	for(a=0; a<9;a++) {
@@ -455,9 +447,7 @@ void RE_SetCamera(Render *re, Object *camera)
 	Camera *cam=NULL;
 	rctf viewplane;
 	float pixsize, clipsta, clipend;
-	float lens;
-	float xd, yd;
-	int blursample= 0;	/* make new call for that */
+	float lens, shiftx=0.0, shifty=0.0, winside;
 	
 	/* question mark */
 	re->ycor= ( (float)re->r.yasp)/( (float)re->r.xasp);
@@ -475,20 +465,22 @@ void RE_SetCamera(Render *re, Object *camera)
 			execute_ipo(&cam->id, cam->ipo);
 		}
 		lens= cam->lens;
+		shiftx=cam->shiftx;
+		shifty=cam->shifty;
 
 		clipsta= cam->clipsta;
 		clipend= cam->clipend;
 	}
 	else if(camera->type==OB_LAMP) {
-		/* fac= cos( PI*((float)(256- la->spsi))/512.0 ); */
-
-		/* phi= acos(fac); */
-		/* lens= 16.0*fac/sin(phi); */
-		lens= re->lens;
+		Lamp *la= camera->data;
+		float fac= cos( M_PI*la->spotsize/360.0 );
+		float phi= acos(fac);
+		
+		lens= 16.0*fac/sin(phi);
 		if(lens==0.0f)
 			lens= 35.0;
-		clipsta= 0.1;
-		clipend= 1000.0;
+		clipsta= la->clipsta;
+		clipend= la->clipend;
 	}
 	else {	/* envmap exception... */
 		lens= re->lens;
@@ -504,7 +496,7 @@ void RE_SetCamera(Render *re, Object *camera)
 	}
 
 	/* ortho only with camera available */
-	if(re->r.mode & R_ORTHO) {
+	if(cam && (re->r.mode & R_ORTHO)) {
 		if( (re->r.xasp*re->winx) >= (re->r.yasp*re->winy) ) {
 			re->viewfac= re->winx;
 		}
@@ -526,10 +518,11 @@ void RE_SetCamera(Render *re, Object *camera)
 	}
 	
 	/* viewplane fully centered, zbuffer fills in jittered between -.5 and +.5 */
-	viewplane.xmin= -0.5f*(float)re->winx; 
-	viewplane.ymin= -0.5f*re->ycor*(float)re->winy; 
-	viewplane.xmax=  0.5f*(float)re->winx; 
-	viewplane.ymax=  0.5f*re->ycor*(float)re->winy; 
+	winside= MAX2(re->winx, re->winy);
+	viewplane.xmin= -0.5f*(float)re->winx + shiftx*winside; 
+	viewplane.ymin= -0.5f*re->ycor*(float)re->winy + shifty*winside;
+	viewplane.xmax=  0.5f*(float)re->winx + shiftx*winside; 
+	viewplane.ymax=  0.5f*re->ycor*(float)re->winy + shifty*winside; 
 
 	if(re->flag & R_SEC_FIELD) {
 		if(re->r.mode & R_ODDFIELD) {
@@ -541,18 +534,12 @@ void RE_SetCamera(Render *re, Object *camera)
 			viewplane.ymax+= .5*re->ycor;
 		}
 	}
-
-	xd= yd= 0.0;
-	if(blursample != -1 && re->osa != 0 ) {
-		re->bluroffsx= xd= re->jit[blursample % re->osa][0];
-		re->bluroffsy= yd= re->ycor*re->jit[blursample % re->osa][1];
-	}
-	else re->bluroffsx=re->bluroffsy= 0.0f;
-
-	viewplane.xmin= pixsize*(viewplane.xmin+xd);
-	viewplane.xmax= pixsize*(viewplane.xmax+xd);
-	viewplane.ymin= pixsize*(viewplane.ymin+yd);
-	viewplane.ymax= pixsize*(viewplane.ymax+yd);
+	/* the window matrix is used for clipping, and not changed during OSA steps */
+	/* using an offset of +0.5 here would give clip errors on edges */
+	viewplane.xmin= pixsize*(viewplane.xmin);
+	viewplane.xmax= pixsize*(viewplane.xmax);
+	viewplane.ymin= pixsize*(viewplane.ymin);
+	viewplane.ymax= pixsize*(viewplane.ymax);
 	
 	re->viewdx= pixsize;
 	re->viewdy= re->ycor*pixsize;
@@ -562,7 +549,6 @@ void RE_SetCamera(Render *re, Object *camera)
 	else 
 		RE_SetWindow(re, &viewplane, clipsta, clipend);
 
-	//printmatrix4("win", re->winmat);
 }
 
 
