@@ -40,6 +40,7 @@
 
 #include "BKE_global.h"
 #include "BKE_image.h"
+#include "BKE_main.h"
 #include "BKE_node.h"
 #include "BKE_object.h"
 #include "BKE_scene.h"
@@ -63,9 +64,12 @@
 
 #include "BSE_sequence.h"  /* <----------------- bad!!! */
 
+#ifndef DISABLE_YAFRAY
 /* yafray: include for yafray export/render */
 #include "YafRay_Api.h"
 
+#endif /* disable yafray */
+
 /* internal */
 #include "render_types.h"
 #include "renderpipeline.h"
@@ -111,6 +115,9 @@ static struct ListBase RenderList= {NULL, NULL};
 /* hardcopy of current render, used while rendering for speed */
 Render R;
 
+/* commandline thread override */
+static int commandline_threads= 0;
+
 /* ********* alloc and free ******** */
 
 
@@ -156,42 +163,42 @@ static void stats_background(RenderStats *rs)
 	}	
 }
 
-static void free_render_result(RenderResult *res)
+void RE_FreeRenderResult(RenderResult *res)
 {
 	if(res==NULL) return;
 
 	while(res->layers.first) {
 		RenderLayer *rl= res->layers.first;
 		
-		if(rl->rectf) MEM_freeT(rl->rectf);
+		if(rl->rectf) MEM_freeN(rl->rectf);
 		/* acolrect is optionally allocated in shade_tile, only free here since it can be used for drawing */
-		if(rl->acolrect) MEM_freeT(rl->acolrect);
+		if(rl->acolrect) MEM_freeN(rl->acolrect);
 		
 		while(rl->passes.first) {
 			RenderPass *rpass= rl->passes.first;
-			if(rpass->rect) MEM_freeT(rpass->rect);
+			if(rpass->rect) MEM_freeN(rpass->rect);
 			BLI_remlink(&rl->passes, rpass);
-			MEM_freeT(rpass);
+			MEM_freeN(rpass);
 		}
 		BLI_remlink(&res->layers, rl);
-		MEM_freeT(rl);
+		MEM_freeN(rl);
 	}
 	
 	if(res->rect32)
-		MEM_freeT(res->rect32);
+		MEM_freeN(res->rect32);
 	if(res->rectz)
-		MEM_freeT(res->rectz);
+		MEM_freeN(res->rectz);
 	if(res->rectf)
-		MEM_freeT(res->rectf);
+		MEM_freeN(res->rectf);
 	
-	MEM_freeT(res);
+	MEM_freeN(res);
 }
 
 /* all layers except the active one get temporally pushed away */
 static void push_render_result(Render *re)
 {
 	/* officially pushed result should be NULL... error can happen with do_seq */
-	free_render_result(re->pushedresult);
+	RE_FreeRenderResult(re->pushedresult);
 	
 	re->pushedresult= re->result;
 	re->result= NULL;
@@ -230,73 +237,157 @@ static void pop_render_result(Render *re)
 			}
 		}
 		
-		free_render_result(re->pushedresult);
+		RE_FreeRenderResult(re->pushedresult);
 		re->pushedresult= NULL;
 	}
 }
 
-
+/* NOTE: OpenEXR only supports 32 chars for layer+pass names
+   In blender we now use max 10 chars for pass, max 20 for layer */
 static char *get_pass_name(int passtype, int channel)
 {
 	
 	if(passtype == SCE_PASS_COMBINED) {
+		if(channel==-1) return "Combined";
 		if(channel==0) return "Combined.R";
-		else if(channel==1) return "Combined.G";
-		else if(channel==2) return "Combined.B";
-		else return "Combined.A";
+		if(channel==1) return "Combined.G";
+		if(channel==2) return "Combined.B";
+		return "Combined.A";
+	}
+	if(passtype == SCE_PASS_Z) {
+		if(channel==-1) return "Depth";
+		return "Depth.Z";
 	}
-	if(passtype == SCE_PASS_Z)
-		return "Z";
 	if(passtype == SCE_PASS_VECTOR) {
+		if(channel==-1) return "Vector";
 		if(channel==0) return "Vector.X";
-		else if(channel==1) return "Vector.Y";
-		else if(channel==2) return "Vector.Z";
-		else return "Vector.W";
+		if(channel==1) return "Vector.Y";
+		if(channel==2) return "Vector.Z";
+		return "Vector.W";
 	}
 	if(passtype == SCE_PASS_NORMAL) {
+		if(channel==-1) return "Normal";
 		if(channel==0) return "Normal.X";
-		else if(channel==1) return "Normal.Y";
-		else return "Normal.Z";
+		if(channel==1) return "Normal.Y";
+		return "Normal.Z";
+	}
+	if(passtype == SCE_PASS_UV) {
+		if(channel==-1) return "UV";
+		if(channel==0) return "UV.U";
+		if(channel==1) return "UV.V";
+		return "UV.A";
 	}
 	if(passtype == SCE_PASS_RGBA) {
+		if(channel==-1) return "Color";
 		if(channel==0) return "Color.R";
-		else if(channel==1) return "Color.G";
-		else if(channel==2) return "Color.B";
-		else return "Color.A";
+		if(channel==1) return "Color.G";
+		if(channel==2) return "Color.B";
+		return "Color.A";
 	}
 	if(passtype == SCE_PASS_DIFFUSE) {
+		if(channel==-1) return "Diffuse";
 		if(channel==0) return "Diffuse.R";
-		else if(channel==1) return "Diffuse.G";
-		else return "Diffuse.B";
+		if(channel==1) return "Diffuse.G";
+		return "Diffuse.B";
 	}
 	if(passtype == SCE_PASS_SPEC) {
+		if(channel==-1) return "Spec";
 		if(channel==0) return "Spec.R";
-		else if(channel==1) return "Spec.G";
-		else return "Spec.B";
+		if(channel==1) return "Spec.G";
+		return "Spec.B";
 	}
 	if(passtype == SCE_PASS_SHADOW) {
+		if(channel==-1) return "Shadow";
 		if(channel==0) return "Shadow.R";
-		else if(channel==1) return "Shadow.G";
-		else return "Shadow.B";
+		if(channel==1) return "Shadow.G";
+		return "Shadow.B";
 	}
 	if(passtype == SCE_PASS_AO) {
+		if(channel==-1) return "AO";
 		if(channel==0) return "AO.R";
-		else if(channel==1) return "AO.G";
-		else return "AO.B";
+		if(channel==1) return "AO.G";
+		return "AO.B";
 	}
-	if(passtype == SCE_PASS_RAY) {
-		if(channel==0) return "Ray.R";
-		else if(channel==1) return "Ray.G";
-		else return "Ray.B";
+	if(passtype == SCE_PASS_REFLECT) {
+		if(channel==-1) return "Reflect";
+		if(channel==0) return "Reflect.R";
+		if(channel==1) return "Reflect.G";
+		return "Reflect.B";
+	}
+	if(passtype == SCE_PASS_REFRACT) {
+		if(channel==-1) return "Refract";
+		if(channel==0) return "Refract.R";
+		if(channel==1) return "Refract.G";
+		return "Refract.B";
+	}
+	if(passtype == SCE_PASS_RADIO) {
+		if(channel==-1) return "Radio";
+		if(channel==0) return "Radio.R";
+		if(channel==1) return "Radio.G";
+		return "Radio.B";
+	}
+	if(passtype == SCE_PASS_INDEXOB) {
+		if(channel==-1) return "IndexOB";
+		return "IndexOB.X";
 	}
 	return "Unknown";
 }
 
+static int passtype_from_name(char *str)
+{
+	
+	if(strcmp(str, "Combined")==0)
+		return SCE_PASS_COMBINED;
+
+	if(strcmp(str, "Depth")==0)
+		return SCE_PASS_Z;
+
+	if(strcmp(str, "Vector")==0)
+		return SCE_PASS_VECTOR;
+
+	if(strcmp(str, "Normal")==0)
+		return SCE_PASS_NORMAL;
+
+	if(strcmp(str, "UV")==0)
+		return SCE_PASS_UV;
+
+	if(strcmp(str, "Color")==0)
+		return SCE_PASS_RGBA;
+
+	if(strcmp(str, "Diffuse")==0)
+		return SCE_PASS_DIFFUSE;
+
+	if(strcmp(str, "Spec")==0)
+		return SCE_PASS_SPEC;
+
+	if(strcmp(str, "Shadow")==0)
+		return SCE_PASS_SHADOW;
+	
+	if(strcmp(str, "AO")==0)
+		return SCE_PASS_AO;
+
+	if(strcmp(str, "Reflect")==0)
+		return SCE_PASS_REFLECT;
+
+	if(strcmp(str, "Refract")==0)
+		return SCE_PASS_REFRACT;
+
+	if(strcmp(str, "Radio")==0)
+		return SCE_PASS_RADIO;
+
+	if(strcmp(str, "IndexOB")==0)
+		return SCE_PASS_INDEXOB;
+
+	return 0;
+}
+
+
+
 static void render_unique_exr_name(Render *re, char *str)
 {
-	char di[FILE_MAXDIR+FILE_MAXFILE], name[FILE_MAXFILE], fi[FILE_MAXFILE];
+	char di[FILE_MAX], name[FILE_MAXFILE], fi[FILE_MAXFILE];
 	
-	BLI_strncpy(di, G.sce, FILE_MAXDIR+FILE_MAXFILE);
+	BLI_strncpy(di, G.sce, FILE_MAX);
 	BLI_splitdirstring(di, fi);
 	sprintf(name, "%s_%s.exr", fi, re->scene->id.name+2);
 	if(G.background)
@@ -309,7 +400,7 @@ static void render_unique_exr_name(Render *re, char *str)
 static void render_layer_add_pass(RenderResult *rr, RenderLayer *rl, int channels, int passtype)
 {
 	char *typestr= get_pass_name(passtype, 0);
-	RenderPass *rpass= MEM_callocT(sizeof(RenderPass), typestr);
+	RenderPass *rpass= MEM_callocN(sizeof(RenderPass), typestr);
 	int rectsize= rr->rectx*rr->recty*channels;
 	
 	BLI_addtail(&rl->passes, rpass);
@@ -319,7 +410,7 @@ static void render_layer_add_pass(RenderResult *rr, RenderLayer *rl, int channel
 	if(rr->exrhandle) {
 		int a;
 		for(a=0; a<channels; a++)
-			IMB_exr_add_channel(rr->exrhandle, rl->name, get_pass_name(passtype, a));
+			IMB_exr_add_channel(rr->exrhandle, rl->name, get_pass_name(passtype, a), 0, 0, NULL);
 	}
 	else {
 		if(passtype==SCE_PASS_VECTOR) {
@@ -327,12 +418,12 @@ static void render_layer_add_pass(RenderResult *rr, RenderLayer *rl, int channel
 			int x;
 			
 			/* initialize to max speed */
-			rect= rpass->rect= MEM_mapallocT(sizeof(float)*rectsize, typestr);
+			rect= rpass->rect= MEM_mapallocN(sizeof(float)*rectsize, typestr);
 			for(x= rectsize-1; x>=0; x--)
 				rect[x]= PASS_VECTOR_MAX;
 		}
 		else
-			rpass->rect= MEM_mapallocT(sizeof(float)*rectsize, typestr);
+			rpass->rect= MEM_mapallocN(sizeof(float)*rectsize, typestr);
 	}
 }
 
@@ -376,7 +467,7 @@ static RenderResult *new_render_result(Render *re, rcti *partrct, int crop, int
 	if(rectx<=0 || recty<=0)
 		return NULL;
 	
-	rr= MEM_callocT(sizeof(RenderResult), "new render result");
+	rr= MEM_callocN(sizeof(RenderResult), "new render result");
 	rr->rectx= rectx;
 	rr->recty= recty;
 	rr->renrect.xmin= 0; rr->renrect.xmax= rectx-2*crop;
@@ -398,23 +489,28 @@ static RenderResult *new_render_result(Render *re, rcti *partrct, int crop, int
 		
 		if((re->r.scemode & R_SINGLE_LAYER) && nr!=re->r.actlay)
 			continue;
+		if(srl->layflag & SCE_LAY_DISABLE)
+			continue;
 		
-		rl= MEM_callocT(sizeof(RenderLayer), "new render layer");
+		rl= MEM_callocN(sizeof(RenderLayer), "new render layer");
 		BLI_addtail(&rr->layers, rl);
 		
 		strcpy(rl->name, srl->name);
 		rl->lay= srl->lay;
 		rl->layflag= srl->layflag;
 		rl->passflag= srl->passflag;
+		rl->pass_xor= srl->pass_xor;
+		rl->light_override= srl->light_override;
+		rl->mat_override= srl->mat_override;
 		
 		if(rr->exrhandle) {
-			IMB_exr_add_channel(rr->exrhandle, rl->name, "Combined.R");
-			IMB_exr_add_channel(rr->exrhandle, rl->name, "Combined.G");
-			IMB_exr_add_channel(rr->exrhandle, rl->name, "Combined.B");
-			IMB_exr_add_channel(rr->exrhandle, rl->name, "Combined.A");
+			IMB_exr_add_channel(rr->exrhandle, rl->name, "Combined.R", 0, 0, NULL);
+			IMB_exr_add_channel(rr->exrhandle, rl->name, "Combined.G", 0, 0, NULL);
+			IMB_exr_add_channel(rr->exrhandle, rl->name, "Combined.B", 0, 0, NULL);
+			IMB_exr_add_channel(rr->exrhandle, rl->name, "Combined.A", 0, 0, NULL);
 		}
 		else
-			rl->rectf= MEM_mapallocT(rectx*recty*sizeof(float)*4, "Combined rgba");
+			rl->rectf= MEM_mapallocN(rectx*recty*sizeof(float)*4, "Combined rgba");
 		
 		if(srl->passflag  & SCE_PASS_Z)
 			render_layer_add_pass(rr, rl, 1, SCE_PASS_Z);
@@ -422,26 +518,38 @@ static RenderResult *new_render_result(Render *re, rcti *partrct, int crop, int
 			render_layer_add_pass(rr, rl, 4, SCE_PASS_VECTOR);
 		if(srl->passflag  & SCE_PASS_NORMAL)
 			render_layer_add_pass(rr, rl, 3, SCE_PASS_NORMAL);
+		if(srl->passflag  & SCE_PASS_UV) 
+			render_layer_add_pass(rr, rl, 3, SCE_PASS_UV);
 		if(srl->passflag  & SCE_PASS_RGBA)
 			render_layer_add_pass(rr, rl, 4, SCE_PASS_RGBA);
 		if(srl->passflag  & SCE_PASS_DIFFUSE)
 			render_layer_add_pass(rr, rl, 3, SCE_PASS_DIFFUSE);
 		if(srl->passflag  & SCE_PASS_SPEC)
 			render_layer_add_pass(rr, rl, 3, SCE_PASS_SPEC);
-		if(srl->passflag  & SCE_PASS_SHADOW)
-			render_layer_add_pass(rr, rl, 3, SCE_PASS_SHADOW);
-		if(srl->passflag  & SCE_PASS_AO)
-			render_layer_add_pass(rr, rl, 3, SCE_PASS_AO);
-		if(srl->passflag  & SCE_PASS_RAY)
-			render_layer_add_pass(rr, rl, 3, SCE_PASS_RAY);
+		if(re->r.mode & R_SHADOW)
+			if(srl->passflag  & SCE_PASS_SHADOW)
+				render_layer_add_pass(rr, rl, 3, SCE_PASS_SHADOW);
+		if(re->r.mode & R_RAYTRACE) {
+			if(srl->passflag  & SCE_PASS_AO)
+				render_layer_add_pass(rr, rl, 3, SCE_PASS_AO);
+			if(srl->passflag  & SCE_PASS_REFLECT)
+				render_layer_add_pass(rr, rl, 3, SCE_PASS_REFLECT);
+			if(srl->passflag  & SCE_PASS_REFRACT)
+				render_layer_add_pass(rr, rl, 3, SCE_PASS_REFRACT);
+		}
+		if(re->r.mode & R_RADIO)
+			if(srl->passflag  & SCE_PASS_RADIO)
+				render_layer_add_pass(rr, rl, 3, SCE_PASS_RADIO);
+		if(srl->passflag  & SCE_PASS_INDEXOB)
+			render_layer_add_pass(rr, rl, 1, SCE_PASS_INDEXOB);
 		
 	}
 	/* previewrender and envmap don't do layers, so we make a default one */
 	if(rr->layers.first==NULL) {
-		rl= MEM_callocT(sizeof(RenderLayer), "new render layer");
+		rl= MEM_callocN(sizeof(RenderLayer), "new render layer");
 		BLI_addtail(&rr->layers, rl);
 		
-		rl->rectf= MEM_mapallocT(rectx*recty*sizeof(float)*4, "prev/env float rgba");
+		rl->rectf= MEM_mapallocN(rectx*recty*sizeof(float)*4, "prev/env float rgba");
 		
 		/* note, this has to be in sync with scene.c */
 		rl->lay= (1<<20) -1;
@@ -464,8 +572,9 @@ static int render_scene_needs_vector(Render *re)
 		SceneRenderLayer *srl;
 	
 		for(srl= re->scene->r.layers.first; srl; srl= srl->next)
-			if(srl->passflag & SCE_PASS_VECTOR)
-				return 1;
+			if(!(srl->layflag & SCE_LAY_DISABLE))
+				if(srl->passflag & SCE_PASS_VECTOR)
+					return 1;
 	}
 	return 0;
 }
@@ -531,7 +640,7 @@ static void save_render_result_tile(Render *re, RenderPart *pa)
 	RenderPass *rpassp;
 	int offs, partx, party;
 	
-	BLI_lock_thread(LOCK_CUSTOM1);
+	BLI_lock_thread(LOCK_IMAGE);
 	
 	for(rlp= rrpart->layers.first; rlp; rlp= rlp->next) {
 		
@@ -564,24 +673,162 @@ static void save_render_result_tile(Render *re, RenderPart *pa)
 	partx= rrpart->tilerect.xmin + rrpart->crop;
 	IMB_exrtile_write_channels(re->result->exrhandle, partx, party);
 
-	BLI_unlock_thread(LOCK_CUSTOM1);
+	BLI_unlock_thread(LOCK_IMAGE);
 
 }
 
+static void save_empty_result_tiles(Render *re)
+{
+	RenderPart *pa;
+	
+	IMB_exrtile_clear_channels(re->result->exrhandle);
+		
+	for(pa= re->parts.first; pa; pa= pa->next) {
+		if(pa->ready==0) {
+			int party= pa->disprect.ymin - re->disprect.ymin + pa->crop;
+			int partx= pa->disprect.xmin - re->disprect.xmin + pa->crop;
+			IMB_exrtile_write_channels(re->result->exrhandle, partx, party);
+		}
+	}
+}
+
+
+/* for passes read from files, these have names stored */
+static char *make_pass_name(RenderPass *rpass, int chan)
+{
+	static char name[16];
+	int len;
+	
+	BLI_strncpy(name, rpass->name, EXR_PASS_MAXNAME);
+	len= strlen(name);
+	name[len]= '.';
+	name[len+1]= rpass->chan_id[chan];
+	name[len+2]= 0;
+
+	return name;
+}
+
+/* filename already made absolute */
+/* called from within UI, saves both rendered result as a file-read result */
+void RE_WriteRenderResult(RenderResult *rr, char *filename, int compress)
+{
+	RenderLayer *rl;
+	RenderPass *rpass;
+	void *exrhandle= IMB_exr_get_handle();
+	
+	/* composite result */
+	if(rr->rectf) {
+		IMB_exr_add_channel(exrhandle, "Composite", "Combined.R", 4, 4*rr->rectx, rr->rectf);
+		IMB_exr_add_channel(exrhandle, "Composite", "Combined.G", 4, 4*rr->rectx, rr->rectf+1);
+		IMB_exr_add_channel(exrhandle, "Composite", "Combined.B", 4, 4*rr->rectx, rr->rectf+2);
+		IMB_exr_add_channel(exrhandle, "Composite", "Combined.A", 4, 4*rr->rectx, rr->rectf+3);
+	}
+	
+	/* add layers/passes and assign channels */
+	for(rl= rr->layers.first; rl; rl= rl->next) {
+		
+		/* combined */
+		if(rl->rectf) {
+			int a, xstride= 4;
+			for(a=0; a<xstride; a++)
+				IMB_exr_add_channel(exrhandle, rl->name, get_pass_name(SCE_PASS_COMBINED, a), 
+									xstride, xstride*rr->rectx, rl->rectf+a);
+		}
+		
+		/* passes are allocated in sync */
+		for(rpass= rl->passes.first; rpass; rpass= rpass->next) {
+			int a, xstride= rpass->channels;
+			for(a=0; a<xstride; a++) {
+				if(rpass->passtype)
+					IMB_exr_add_channel(exrhandle, rl->name, get_pass_name(rpass->passtype, a), 
+										xstride, xstride*rr->rectx, rpass->rect+a);
+				else
+					IMB_exr_add_channel(exrhandle, rl->name, make_pass_name(rpass, a), 
+										xstride, xstride*rr->rectx, rpass->rect+a);
+			}
+		}
+	}
+	
+	IMB_exr_begin_write(exrhandle, filename, rr->rectx, rr->recty, compress);
+	
+	IMB_exr_write_channels(exrhandle);
+	IMB_exr_close(exrhandle);
+}
+
+/* callbacks for RE_MultilayerConvert */
+static void *ml_addlayer_cb(void *base, char *str)
+{
+	RenderResult *rr= base;
+	RenderLayer *rl;
+	
+	rl= MEM_callocN(sizeof(RenderLayer), "new render layer");
+	BLI_addtail(&rr->layers, rl);
+	
+	BLI_strncpy(rl->name, str, EXR_LAY_MAXNAME);
+	return rl;
+}
+static void ml_addpass_cb(void *base, void *lay, char *str, float *rect, int totchan, char *chan_id)
+{
+	RenderLayer *rl= lay;	
+	RenderPass *rpass= MEM_callocN(sizeof(RenderPass), "loaded pass");
+	int a;
+	
+	BLI_addtail(&rl->passes, rpass);
+	rpass->channels= totchan;
+	
+	rpass->passtype= passtype_from_name(str);
+	if(rpass->passtype==0) printf("unknown pass %s\n", str);
+	rl->passflag |= rpass->passtype;
+	
+	BLI_strncpy(rpass->name, str, EXR_PASS_MAXNAME);
+	/* channel id chars */
+	for(a=0; a<totchan; a++)
+		rpass->chan_id[a]= chan_id[a];
+	
+	rpass->rect= rect;
+}
+
+/* from imbuf, if a handle was returned we convert this to render result */
+RenderResult *RE_MultilayerConvert(void *exrhandle, int rectx, int recty)
+{
+	RenderResult *rr= MEM_callocN(sizeof(RenderResult), "loaded render result");
+	
+	rr->rectx= rectx;
+	rr->recty= recty;
+	
+	IMB_exr_multilayer_convert(exrhandle, rr, ml_addlayer_cb, ml_addpass_cb);
+	
+	return rr;
+}
+
+/* called in end of render, to add names to passes... for UI only */
+static void renderresult_add_names(RenderResult *rr)
+{
+	RenderLayer *rl;
+	RenderPass *rpass;
+	
+	for(rl= rr->layers.first; rl; rl= rl->next)
+		for(rpass= rl->passes.first; rpass; rpass= rpass->next)
+			strcpy(rpass->name, get_pass_name(rpass->passtype, -1));
+}
+
+
+/* only for temp buffer files, makes exact copy of render result */
 static void read_render_result(Render *re)
 {
 	RenderLayer *rl;
 	RenderPass *rpass;
 	void *exrhandle= IMB_exr_get_handle();
 	int rectx, recty;
-	char str[FILE_MAXDIR+FILE_MAXFILE];
+	char str[FILE_MAX];
 	
-	free_render_result(re->result);
+	RE_FreeRenderResult(re->result);
 	re->result= new_render_result(re, &re->disprect, 0, RR_USEMEM);
 
 	render_unique_exr_name(re, str);
 	if(IMB_exr_begin_read(exrhandle, str, &rectx, &recty)==0) {
-		printf("cannot read render result\n");
+		IMB_exr_close(exrhandle);
+		printf("cannot read: %s\n", str);
 		return;
 	}
 	
@@ -609,6 +856,7 @@ static void read_render_result(Render *re)
 			
 		}
 		IMB_exr_read_channels(exrhandle);
+		renderresult_add_names(re->result);
 	}
 	
 	IMB_exr_close(exrhandle);
@@ -716,7 +964,7 @@ Render *RE_NewRender(const char *name)
 	if(re==NULL) {
 		
 		/* new render data struct */
-		re= MEM_callocT(sizeof(Render), "new render");
+		re= MEM_callocN(sizeof(Render), "new render");
 		BLI_addtail(&RenderList, re);
 		strncpy(re->name, name, RE_MAXNAME);
 	}
@@ -747,11 +995,11 @@ void RE_FreeRender(Render *re)
 	free_renderdata_tables(re);
 	free_sample_tables(re);
 	
-	free_render_result(re->result);
-	free_render_result(re->pushedresult);
+	RE_FreeRenderResult(re->result);
+	RE_FreeRenderResult(re->pushedresult);
 	
 	BLI_remlink(&RenderList, re);
-	MEM_freeT(re);
+	MEM_freeN(re);
 }
 
 /* exit blender */
@@ -805,10 +1053,16 @@ void RE_InitState(Render *re, RenderData *rd, int winx, int winy, rcti *disprect
 		make_sample_tables(re);	
 		
 		/* make empty render result, so display callbacks can initialize */
-		free_render_result(re->result);
+		RE_FreeRenderResult(re->result);
 		re->result= MEM_callocN(sizeof(RenderResult), "new render result");
 		re->result->rectx= re->rectx;
 		re->result->recty= re->recty;
+		
+		/* we clip faces with a minimum of 2 pixel boundary outside of image border. see zbuf.c */
+		re->clipcrop= 1.0f + 2.0f/(float)(re->winx>re->winy?re->winy:re->winx);
+		
+		if(commandline_threads>0 && commandline_threads<=BLENDER_MAX_THREADS)
+			re->r.threads= commandline_threads;
 	}
 }
 
@@ -819,7 +1073,7 @@ void RE_SetDispRect (struct Render *re, rcti *disprect)
 	re->recty= disprect->ymax-disprect->ymin;
 	
 	/* initialize render result */
-	free_render_result(re->result);
+	RE_FreeRenderResult(re->result);
 	re->result= new_render_result(re, &re->disprect, 0, RR_USEMEM);
 }
 
@@ -833,6 +1087,7 @@ void RE_SetWindow(Render *re, rctf *viewplane, float clipsta, float clipend)
 	re->r.mode &= ~R_ORTHO;
 
 	i_window(re->viewplane.xmin, re->viewplane.xmax, re->viewplane.ymin, re->viewplane.ymax, re->clipsta, re->clipend, re->winmat);
+	
 }
 
 void RE_SetOrtho(Render *re, rctf *viewplane, float clipsta, float clipend)
@@ -938,7 +1193,7 @@ static void render_tile_processor(Render *re, int firsttile)
 
 	/* hrmf... exception, this is used for preview render, re-entrant, so render result has to be re-used */
 	if(re->result==NULL || re->result->layers.first==NULL) {
-		if(re->result) free_render_result(re->result);
+		if(re->result) RE_FreeRenderResult(re->result);
 		re->result= new_render_result(re, &re->disprect, 0, RR_USEMEM);
 	}
 	
@@ -967,7 +1222,7 @@ static void render_tile_processor(Render *re, int firsttile)
 					re->i.partsdone++;
 					re->stats_draw(&re->i);
 				}
-				free_render_result(pa->result);
+				RE_FreeRenderResult(pa->result);
 				pa->result= NULL;
 			}		
 			if(re->test_break())
@@ -1097,10 +1352,10 @@ static void threaded_tile_processor(Render *re)
 	RenderPart *pa, *nextpa;
 	RenderResult *rr;
 	rctf viewplane= re->viewplane;
-	int maxthreads, rendering=1, counter= 1, drawtimer=0, hasdrawn, minx=0;
+	int rendering=1, counter= 1, drawtimer=0, hasdrawn, minx=0;
 	
 	/* first step; the entire render result, or prepare exr buffer saving */
-	free_render_result(re->result);
+	RE_FreeRenderResult(re->result);
 	rr= re->result= new_render_result(re, &re->disprect, 0, re->r.scemode & R_EXR_TILE_FILE);
 	
 	if(rr==NULL)
@@ -1112,7 +1367,7 @@ static void threaded_tile_processor(Render *re)
 	initparts(re);
 	
 	if(rr->exrhandle) {
-		char str[FILE_MAXDIR+FILE_MAXFILE];
+		char str[FILE_MAX];
 		
 		render_unique_exr_name(re, str);
 		
@@ -1120,10 +1375,7 @@ static void threaded_tile_processor(Render *re)
 		IMB_exrtile_begin_write(rr->exrhandle, str, rr->rectx, rr->recty, rr->rectx/re->xparts, rr->recty/re->yparts);
 	}
 	
-	if(re->r.mode & R_THREADS) maxthreads= 2;
-	else maxthreads= 1;
-	
-	BLI_init_threads(&threads, do_part_thread, maxthreads);
+	BLI_init_threads(&threads, do_part_thread, re->r.threads);
 	
 	/* assuming no new data gets added to dbase... */
 	R= *re;
@@ -1150,7 +1402,7 @@ static void threaded_tile_processor(Render *re)
 			nextpa= find_next_part(re, minx);
 		}
 		else if(re->r.mode & R_PANORAMA) {
-			if(nextpa==NULL && BLI_available_threads(&threads)==maxthreads)
+			if(nextpa==NULL && BLI_available_threads(&threads)==re->r.threads)
 				nextpa= find_next_pano_slice(re, &minx, &viewplane);
 			else {
 				PIL_sleep_ms(50);
@@ -1167,13 +1419,14 @@ static void threaded_tile_processor(Render *re)
 		hasdrawn= 0;
 		for(pa= re->parts.first; pa; pa= pa->next) {
 			if(pa->ready) {
+				
+				BLI_remove_thread(&threads, pa);
+				
 				if(pa->result) {
-					BLI_remove_thread(&threads, pa);
-
 					re->display_draw(pa->result, NULL);
 					print_part_stats(re, pa);
 					
-					free_render_result(pa->result);
+					RE_FreeRenderResult(pa->result);
 					pa->result= NULL;
 					re->i.partsdone++;
 					hasdrawn= 1;
@@ -1191,16 +1444,16 @@ static void threaded_tile_processor(Render *re)
 			drawtimer= 0;
 
 		/* on break, wait for all slots to get freed */
-		if( (g_break=re->test_break()) && BLI_available_threads(&threads)==maxthreads)
+		if( (g_break=re->test_break()) && BLI_available_threads(&threads)==re->r.threads)
 			rendering= 0;
 		
 	}
 	
 	if(rr->exrhandle) {
+		save_empty_result_tiles(re);
 		IMB_exr_close(rr->exrhandle);
 		rr->exrhandle= NULL;
-		if(!re->test_break())
-			read_render_result(re);
+		read_render_result(re);
 	}
 	
 	/* unset threadsafety */
@@ -1219,10 +1472,8 @@ void RE_TileProcessor(Render *re, int firsttile)
 
 	re->i.starttime= PIL_check_seconds_timer();
 
-	//if(re->r.mode & R_THREADS) 
 	//	threaded_tile_processor(re);
-	//else
-		render_tile_processor(re, firsttile);
+	render_tile_processor(re, firsttile);
 		
 	re->i.lastframetime= PIL_check_seconds_timer()- re->i.starttime;
 	re->stats_draw(&re->i);
@@ -1323,7 +1574,7 @@ static void do_render_blur_3d(Render *re)
 	}
 	
 	/* swap results */
-	free_render_result(re->result);
+	RE_FreeRenderResult(re->result);
 	re->result= rres;
 	
 	set_mblur_offs(0.0f);
@@ -1429,9 +1680,9 @@ static void do_render_fields_3d(Render *re)
 		else
 			merge_renderresult_fields(re->result, rr1, rr2);
 		
-		free_render_result(rr2);
+		RE_FreeRenderResult(rr2);
 	}
-	free_render_result(rr1);
+	RE_FreeRenderResult(rr1);
 	
 	re->i.curfield= 0;	/* stats */
 	
@@ -1443,7 +1694,7 @@ static void do_render_fields_3d(Render *re)
 static void load_backbuffer(Render *re)
 {
 	if(re->r.alphamode == R_ADDSKY) {
-		Image *bima;
+		ImBuf *ibuf;
 		char name[256];
 		
 		strcpy(name, re->r.backbuf);
@@ -1451,32 +1702,21 @@ static void load_backbuffer(Render *re)
 		
 		if(re->backbuf) {
 			re->backbuf->id.us--;
-			bima= re->backbuf;
-		}
-		else bima= NULL;
-		
-		re->backbuf= add_image(name);
-		
-		if(bima && bima->id.us<1) {
-			free_image_buffers(bima);
+			if(re->backbuf->id.us<1)
+				BKE_image_signal(re->backbuf, NULL, IMA_SIGNAL_RELOAD);
 		}
 		
-		if(re->backbuf && re->backbuf->ibuf==NULL) {
-			re->backbuf->ibuf= IMB_loadiffname(re->backbuf->name, IB_rect);
-			if(re->backbuf->ibuf==NULL) 
-				re->backbuf->ok= 0;
-			else {
-				re->backbuf->ok= 1;
-				
-				if (re->r.mode & R_FIELDS)
-					image_de_interlace(re->backbuf, re->r.mode & R_ODDFIELD);
-			}
-		}
-		if(re->backbuf==NULL || re->backbuf->ok==0) {
+		re->backbuf= BKE_add_image_file(name);
+		ibuf= BKE_image_get_ibuf(re->backbuf, NULL);
+		if(ibuf==NULL) {
 			// error() doesnt work with render window open
 			//error("No backbuf there!");
 			printf("Error: No backbuf %s\n", name);
 		}
+		else {
+			if (re->r.mode & R_FIELDS)
+				image_de_interlace(re->backbuf, re->r.mode & R_ODDFIELD);
+		}
 	}
 }
 
@@ -1523,7 +1763,7 @@ static void do_render_fields_blur_3d(Render *re)
 				rres= new_render_result(re, &re->disprect, 0, RR_USEMEM);
 				
 				merge_render_result(rres, re->result);
-				free_render_result(re->result);
+				RE_FreeRenderResult(re->result);
 				re->result= rres;
 				
 				/* weak... the display callback wants an active renderlayer pointer... */
@@ -1636,7 +1876,7 @@ static void do_render_composite_fields_blur_3d(Render *re)
 	bNodeTree *ntree= re->scene->nodetree;
 	
 	/* INIT seeding, compositor can use random texture */
-	BLI_srand(re->r.cfra);
+	BLI_srandom(re->r.cfra);
 	
 	if(composite_needs_render(re->scene)) {
 		/* save memory... free all cached images */
@@ -1650,7 +1890,7 @@ static void do_render_composite_fields_blur_3d(Render *re)
 		pop_render_result(re);
 	
 	if(!re->test_break() && ntree) {
-		ntreeCompositTagRender(ntree);
+		ntreeCompositTagRender(re->scene);
 		ntreeCompositTagAnimated(ntree);
 		
 		if(re->r.scemode & R_DOCOMP) {
@@ -1674,11 +1914,11 @@ static void do_render_composite_fields_blur_3d(Render *re)
 	re->display_draw(re->result, NULL);
 }
 
-
+#ifndef DISABLE_YAFRAY
 /* yafray: main yafray render/export call */
 static void yafrayRender(Render *re)
 {
-	free_render_result(re->result);
+	RE_FreeRenderResult(re->result);
 	re->result= new_render_result(re, &re->disprect, 0, RR_USEMEM);
 	
 	// need this too, for aspect/ortho/etc info
@@ -1718,7 +1958,7 @@ static void yafrayRender(Render *re)
 				rres= new_render_result(re, &re->disprect, 0, RR_USEMEM);
 				
 				merge_render_result(rres, re->result);
-				free_render_result(re->result);
+				RE_FreeRenderResult(re->result);
 				re->result= rres;
 				
 				re->display_init(re->result);
@@ -1733,16 +1973,21 @@ static void yafrayRender(Render *re)
 	RE_Database_Free(re);
 }
 
+#endif /* disable yafray */
+
 
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
 
-
 /* main loop: doing sequence + fields + blur + 3d render + compositing */
 static void do_render_all_options(Render *re)
 {
 	re->i.starttime= PIL_check_seconds_timer();
 
+	/* ensure no images are in memory from previous animated sequences */
+	BKE_image_all_free_anim_ibufs(re->r.cfra);
+	
 	if(re->r.scemode & R_DOSEQ) {
+		/* note: do_render_seq() frees rect32 when sequencer returns float images */
 		if(!re->test_break()) 
 			do_render_seq(re->result, re->r.cfra);
 		
@@ -1751,18 +1996,26 @@ static void do_render_all_options(Render *re)
 		
 	}
 	else {
+#ifndef DISABLE_YAFRAY
 		if(re->r.renderer==R_YAFRAY)
 			yafrayRender(re);
 		else
 			do_render_composite_fields_blur_3d(re);
+#else
+		do_render_composite_fields_blur_3d(re);
+#endif
 	}
 	
+	/* for UI only */
+	renderresult_add_names(re->result);
+	
 	re->i.lastframetime= PIL_check_seconds_timer()- re->i.starttime;
 	re->stats_draw(&re->i);
 }
 
 static int is_rendering_allowed(Render *re)
 {
+	SceneRenderLayer *srl;
 	
 	/* forbidden combinations */
 	if(re->r.mode & R_PANORAMA) {
@@ -1789,7 +2042,7 @@ static int is_rendering_allowed(Render *re)
 	}
 	
 	if(re->r.scemode & R_EXR_TILE_FILE) {
-		char str[FILE_MAXDIR+FILE_MAXFILE];
+		char str[FILE_MAX];
 		
 		render_unique_exr_name(re, str);
 		
@@ -1809,11 +2062,12 @@ static int is_rendering_allowed(Render *re)
 				re->error("No Nodetree in Scene");
 				return 0;
 			}
-		
+			
 			for(node= ntree->nodes.first; node; node= node->next)
 				if(node->type==CMP_NODE_COMPOSITE)
 					break;
 			
+			
 			if(node==NULL) {
 				re->error("No Render Output Node in Scene");
 				return 0;
@@ -1832,6 +2086,26 @@ static int is_rendering_allowed(Render *re)
 		}
 	}
 	
+	/* layer flag tests */
+	if(re->r.scemode & R_SINGLE_LAYER) {
+		srl= BLI_findlink(&re->scene->r.layers, re->r.actlay);
+		/* force layer to be enabled */
+		srl->layflag &= ~SCE_LAY_DISABLE;
+	}
+	
+	for(srl= re->scene->r.layers.first; srl; srl= srl->next)
+		if(!(srl->layflag & SCE_LAY_DISABLE))
+			break;
+	if(srl==NULL) {
+		re->error("All RenderLayers are disabled");
+		return 0;
+	}
+	
+	/* renderer */
+	if(!ELEM(re->r.renderer, R_INTERN, R_YAFRAY)) {
+		re->error("Unknown render engine set");
+		return 0;
+	}
 	return 1;
 }
 
@@ -1881,6 +2155,9 @@ static int render_initialize_from_scene(Render *re, Scene *scene)
 	
 	RE_InitState(re, &scene->r, winx, winy, &disprect);
 	
+	/* initstate makes new result, have to send changed tags around */
+	ntreeCompositTagRender(re->scene);
+	
 	re->scene= scene;
 	if(!is_rendering_allowed(re))
 		return 0;
@@ -1910,7 +2187,7 @@ void RE_BlenderFrame(Render *re, Scene *scene, int frame)
 
 static void do_write_image_or_movie(Render *re, Scene *scene, bMovieHandle *mh)
 {
-	char name[FILE_MAXDIR+FILE_MAXFILE];
+	char name[FILE_MAX];
 	RenderResult rres;
 	
 	RE_GetResultImage(re, &rres);
@@ -1920,50 +2197,58 @@ static void do_write_image_or_movie(Render *re, Scene *scene, bMovieHandle *mh)
 		int dofree = 0;
 		/* note; the way it gets 32 bits rects is weak... */
 		if(rres.rect32==NULL) {
-			rres.rect32= MEM_mapallocT(sizeof(int)*rres.rectx*rres.recty, "temp 32 bits rect");
+			rres.rect32= MEM_mapallocN(sizeof(int)*rres.rectx*rres.recty, "temp 32 bits rect");
 			dofree = 1;
 		}
 		RE_ResultGet32(re, rres.rect32);
 		mh->append_movie(scene->r.cfra, rres.rect32, rres.rectx, rres.recty);
 		if(dofree) {
-			MEM_freeT(rres.rect32);
+			MEM_freeN(rres.rect32);
 		}
 		printf("Append frame %d", scene->r.cfra);
-	} else {
-		ImBuf *ibuf= IMB_allocImBuf(rres.rectx, rres.recty, scene->r.planes, 0, 0);
-		int ok;
+	} 
+	else {
+		BKE_makepicstring(name, scene->r.pic, scene->r.cfra, scene->r.imtype);
 		
-		BKE_makepicstring(name, (scene->r.cfra));
-
-                /* if not exists, BKE_write_ibuf makes one */
-		ibuf->rect= rres.rect32;    
-		ibuf->rect_float= rres.rectf;
-		ibuf->zbuf_float= rres.rectz;
-		
-		/* float factor for random dither, imbuf takes care of it */
-		ibuf->dither= scene->r.dither_intensity;
-
-		ok= BKE_write_ibuf(ibuf, name, scene->r.imtype, scene->r.subimtype, scene->r.quality);
-		
-		if(ok==0) {
-			printf("Render error: cannot save %s\n", name);
-			G.afbreek=1;
-			return;
+		if(re->r.imtype==R_MULTILAYER) {
+			if(re->result) {
+				RE_WriteRenderResult(re->result, name, scene->r.quality);
+				printf("Saved: %s", name);
+			}
 		}
-		else printf("Saved: %s", name);
-		
-		/* optional preview images for exr */
-		if(ok && scene->r.imtype==R_OPENEXR && (scene->r.subimtype & R_PREVIEW_JPG)) {
-			if(BLI_testextensie(name, ".exr")) 
-				name[strlen(name)-4]= 0;
-			BKE_add_image_extension(name, R_JPEG90);
-			ibuf->depth= 24; 
-			BKE_write_ibuf(ibuf, name, R_JPEG90, scene->r.subimtype, scene->r.quality);
-			printf("\nSaved: %s", name);
+		else {
+			ImBuf *ibuf= IMB_allocImBuf(rres.rectx, rres.recty, scene->r.planes, 0, 0);
+			int ok;
+			
+					/* if not exists, BKE_write_ibuf makes one */
+			ibuf->rect= rres.rect32;    
+			ibuf->rect_float= rres.rectf;
+			ibuf->zbuf_float= rres.rectz;
+			
+			/* float factor for random dither, imbuf takes care of it */
+			ibuf->dither= scene->r.dither_intensity;
+
+			ok= BKE_write_ibuf(ibuf, name, scene->r.imtype, scene->r.subimtype, scene->r.quality);
+			
+			if(ok==0) {
+				printf("Render error: cannot save %s\n", name);
+				G.afbreek=1;
+			}
+			else printf("Saved: %s", name);
+			
+			/* optional preview images for exr */
+			if(ok && scene->r.imtype==R_OPENEXR && (scene->r.subimtype & R_PREVIEW_JPG)) {
+				if(BLI_testextensie(name, ".exr")) 
+					name[strlen(name)-4]= 0;
+				BKE_add_image_extension(name, R_JPEG90);
+				ibuf->depth= 24; 
+				BKE_write_ibuf(ibuf, name, R_JPEG90, scene->r.subimtype, scene->r.quality);
+				printf("\nSaved: %s", name);
+			}
+			
+					/* imbuf knows which rects are not part of ibuf */
+			IMB_freeImBuf(ibuf);
 		}
-		
-                /* imbuf knows which rects are not part of ibuf */
-		IMB_freeImBuf(ibuf);	
 	}
 	
 	BLI_timestr(re->i.lastframetime, name);
@@ -2030,6 +2315,7 @@ void RE_BlenderAnim(Render *re, Scene *scene, int sfra, int efra)
 
 /* note; repeated win/disprect calc... solve that nicer, also in compo */
 
+/* only temp file! */
 void RE_ReadRenderResult(Scene *scene, Scene *scenode)
 {
 	Render *re;
@@ -2063,3 +2349,11 @@ void RE_ReadRenderResult(Scene *scene, Scene *scenode)
 	
 	read_render_result(re);
 }
+
+void RE_set_max_threads(int threads)
+{
+	if(threads>0 && threads<=BLENDER_MAX_THREADS)
+		commandline_threads= threads;
+	else
+		printf("Error, threads has to be in range 1-%d\n", BLENDER_MAX_THREADS);
+}
