@@ -4,7 +4,7 @@
  * Functions to blend pixels with or without alpha, in various formats
  * nzc - June 2000
  *
- * $Id: pixelblending.c,v 1.16 2006/02/02 17:54:21 ton Exp $
+ * $Id: pixelblending.c,v 1.17 2006/12/05 16:43:00 ton Exp $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -280,134 +280,7 @@ void addalphaAddFloat(float *dest, float *source)
 }
 
 
-/* ------------------------------------------------------------------------- */
-
-/* ------------------------------------------------------------------------- */
-/* Colour buffer related:                                                    */
-/* This transforms the 4 inputvalues RE_COLBUFTYPE to a new value            */
-/* It expects the values R.r.postigamma, R.r.postmul and R.r.postadd.         */
-/* This is the standard transformation, more elaborate tools are for later.  */
-/* ------------------------------------------------------------------------- */
-void std_floatcol_to_charcol( float *buf, char *target)
-{
-	float col[3];
-	
-	float dither_value;
-	
-	dither_value = ((BLI_frand()-0.5)*R.r.dither_intensity)/256.0; 
-	
-	/* alpha */
-	if((buf[3]+dither_value)<=0.0) target[3]= 0;
-	else if((buf[3]+dither_value)>1.0) target[3]= 255;
-	else target[3]= 255.0*(buf[3]+dither_value);
-	
-	if(R.r.postgamma==1.0) {
-		/* r */
-		col[0]= R.r.postmul*buf[0] + R.r.postadd + dither_value;
-		/* g */
-		col[1]= R.r.postmul*buf[1] + R.r.postadd + dither_value;
-		/* b */
-		col[2]= R.r.postmul*buf[2] + R.r.postadd + dither_value;
-	}
-	else {
-		/* putting the postmul within the pow() gives an
-		* easier control for the user, values from 1.0-2.0
-		* are relevant then
-		*/
-		
-		/* r */
-		col[0]= pow(R.r.postmul*buf[0], R.r.postigamma) + R.r.postadd + dither_value;
-		/* g */
-		col[1]= pow( R.r.postmul*buf[1], R.r.postigamma) + R.r.postadd + dither_value;
-		/* b */
-		col[2]= pow(R.r.postmul*buf[2], R.r.postigamma) + R.r.postadd + dither_value;
-	}
-	
-	if(R.r.posthue!=0.0 || R.r.postsat!=1.0) {
-		float hsv[3];
-		
-		rgb_to_hsv(col[0], col[1], col[2], hsv, hsv+1, hsv+2);
-		hsv[0]+= R.r.posthue;
-		if(hsv[0]>1.0) hsv[0]-=1.0; else if(hsv[0]<0.0) hsv[0]+= 1.0;
-		hsv[1]*= R.r.postsat;
-		if(hsv[1]>1.0) hsv[1]= 1.0; else if(hsv[1]<0.0) hsv[1]= 0.0;
-		hsv_to_rgb(hsv[0], hsv[1], hsv[2], col, col+1, col+2);
-	}
-	
-	if(col[0]<=0.0) target[0]= 0;
-	else if(col[0]>1.0) target[0]= 255;
-	else target[0]= 255.0*col[0];
-	
-	if(col[1]<=0.0) target[1]= 0;
-	else if(col[1]>1.0) target[1]= 255;
-	else target[1]= 255.0*col[1];
-	
-	if(col[2]<=0.0) target[2]= 0;
-	else if(col[2]>1.0) target[2]= 255;
-	else target[2]= 255.0*col[2];
-}
-
-/* ----------------------------------------------------------------------------
-
-Colour buffer related:
-
-The colour buffer is a buffer of a single screen line. It contains        
-four fields of type RE_COLBUFTYPE per pixel.
-
-We can do several post-process steps. I would prefer to move them outside
-the render module later on, but it's ok to leave it here for now. For the
-time being, we have:
-- post-process function
-    Does some operations with the colours.
-- Multiply with some factor
-- Add constant offset
-- Apply extra gamma correction (seems weird...)
-- key-alpha correction
-    Key alpha means 'un-applying' the alpha. For fully covered pixels, this
-	operation has no effect.
-
-- XXX WARNING! Added the inverse render gamma here, so this cannot be used external
-	without setting Osa or Gamma flags off (ton)
-
----------------------------------------------------------------------------- */
-/* used external! */
-void transferColourBufferToOutput( float *buf, int y)
-{
-    /* Copy the contents of AColourBuffer3 to R.rectot + y * R.rectx */
-    int x = 0;
-//    char *target = (char*) (R.rectot + (y * R.rectx));
-	
-	/* Copy the first <R.rectx> pixels. We can do some more clipping on    */
-	/* the z buffer, I think.                                                 */
-	while (x < R.rectx) {
-		
-		
-		/* invert gamma corrected additions */
-		if(R.do_gamma) {
-			buf[0] = invGammaCorrect(buf[0]);
-			buf[1] = invGammaCorrect(buf[1]);
-			buf[2] = invGammaCorrect(buf[2]);
-		}			
-		
-//		std_floatcol_to_charcol(buf, target);
-		
-		/*
-		 Key-alpha mode:
-		 Need to un-apply alpha if alpha is non-full. For full alpha,
-		 the operation doesn't have effect. Do this after the post-
-		 processing, so we can still use the benefits of that.
-		 
-		 */
-		
-		if (R.r.alphamode == R_ALPHAKEY) {  
-//			applyKeyAlphaCharCol(target);
-		}				
-		
-//        target+=4;
-        buf+=4;
-        x++;
-    }
-}
+/* ---------------------------------------------------------------------------- */
 
 
 /* eof pixelblending.c */
