@@ -1,5 +1,5 @@
 /*
- * $Id: texture.c,v 1.68 2006/07/13 15:41:36 ton Exp $
+ * $Id: texture.c,v 1.83 2007/02/09 10:35:04 ton Exp $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -47,6 +47,7 @@
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"
 
+#include "BKE_image.h"
 #include "BKE_plugin_types.h"
 #include "BKE_utildefines.h"
 
@@ -59,10 +60,11 @@
 #include "BKE_key.h"
 #include "BKE_ipo.h"
 
+#include "envmap.h"
 #include "renderpipeline.h"
 #include "render_types.h"
 #include "rendercore.h"
-#include "envmap.h"
+#include "shading.h"
 #include "texture.h"
 
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
@@ -72,112 +74,26 @@ extern struct Render R;
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
 
 
-/* prototypes */
-static int calcimanr(int cfra, Tex *tex);
 
-/* ------------------------------------------------------------------------- */
 
-int calcimanr(int cfra, Tex *tex)
-{
-	int imanr, len, a, fra, dur;
-	
-	/* here (+fie_ima/2-1) makes sure that division happens correctly */
-	
-	if(tex->frames==0) return 1;
-	
-	cfra= cfra-tex->sfra+1;
-	
-	/* cyclic */
-	if(tex->len==0) len= (tex->fie_ima*tex->frames)/2;
-	else len= tex->len;
-	
-	if(tex->imaflag & TEX_ANIMCYCLIC) {
-		cfra= ( (cfra) % len );
-		if(cfra < 0) cfra+= len;
-		if(cfra==0) cfra= len;
-	}
-	
-	if(cfra<1) cfra= 1;
-	else if(cfra>len) cfra= len;
-	
-	/* convert current frame to current field */
-	cfra= 2*(cfra);
-	if(R.flag & R_SEC_FIELD) cfra++;
-	
-	/* transform to images space */
-	imanr= (cfra+tex->fie_ima-2)/tex->fie_ima;
-	if(imanr>tex->frames) imanr= tex->frames;
-	imanr+= tex->offset;
-	
-	if(tex->imaflag & TEX_ANIMCYCLIC) {
-		imanr= ( (imanr) % len );
-		while(imanr < 0) imanr+= len;
-		if(imanr==0) imanr= len;
-	}
-	
-	/* are there images that last longer? */
-	for(a=0; a<4; a++) {
-		if(tex->fradur[a][0]) {
-			
-			fra= tex->fradur[a][0];
-			dur= tex->fradur[a][1]-1;
-			
-			while(dur>0 && imanr>fra) {
-				imanr--;
-				dur--;
-			}
-		}
-	}
-	
-	return imanr;
-}
-
-/* note; this function is called in src/drawview.c for animated background image, option should move to kernel */
+/* note; this is called WITH RENDER IS NULL in src/drawview.c for animated 
+   background image, option should move to kernel */
 void init_render_texture(Render *re, Tex *tex)
 {
-	Image *ima;
-	int imanr;
-	unsigned short numlen;
-	char name[FILE_MAXDIR+FILE_MAXFILE], head[FILE_MAXDIR+FILE_MAXFILE], tail[FILE_MAXDIR+FILE_MAXFILE];
-
-	/* imap test */
-	if(tex->frames && tex->ima && tex->ima->name) {	/* frames */
-		strcpy(name, tex->ima->name);
-		
-		imanr= calcimanr(G.scene->r.cfra, tex);
-		
-		if(tex->imaflag & TEX_ANIM5) {
-			if(tex->ima->lastframe != imanr) {
-				if(tex->ima->ibuf) IMB_freeImBuf(tex->ima->ibuf);
-				tex->ima->ibuf= 0;
-				tex->ima->lastframe= imanr;
-			}
-		}
-		else {
-				/* for patch field-ima rendering */
-			tex->ima->lastframe= imanr;
-			
-			BLI_stringdec(name, head, tail, &numlen);
-			BLI_stringenc(name, head, tail, numlen, imanr);
+	int cfra= G.scene->r.cfra;
 	
-			ima= add_image(name);
-
-			if(ima) {
-				ima->flag |= IMA_FROMANIM;
-				
-				if(tex->ima) tex->ima->id.us--;
-				tex->ima= ima;
-				
-				ima->ok= 1;
-			}
-		}
+	if(re) cfra= re->r.cfra;
+	
+	/* imap test */
+	if(tex->ima && ELEM(tex->ima->source, IMA_SRC_MOVIE, IMA_SRC_SEQUENCE)) {
+		BKE_image_user_calc_imanr(&tex->iuser, cfra, re?re->flag & R_SEC_FIELD:0);
 	}
 	
 	if(tex->type==TEX_PLUGIN) {
 		if(tex->plugin && tex->plugin->doit) {
-				if(tex->plugin->cfra) {
- 					*(tex->plugin->cfra)= frame_to_float(G.scene->r.cfra); 
-				}
+			if(tex->plugin->cfra) {
+				*(tex->plugin->cfra)= frame_to_float(cfra); 
+			}
 		}
 	}
 	else if(tex->type==TEX_ENVMAP) {
@@ -190,7 +106,7 @@ void init_render_texture(Render *re, Tex *tex)
 				tex->extend= TEX_EXTEND;
 			
 			/* only free envmap when rendermode was set to render envmaps, for previewrender */
-			if(G.rendering) {
+			if(G.rendering && re) {
 				if (re->r.mode & R_ENVMAP)
 					if(tex->env->stype==ENV_ANIM) 
 						BKE_free_envmapdata(tex->env);
@@ -210,8 +126,6 @@ void init_render_textures(Render *re)
 		if(tex->id.us) init_render_texture(re, tex);
 		tex= tex->id.next;
 	}
-	
-	free_unused_animimages();
 }
 
 /* ------------------------------------------------------------------------- */
@@ -1001,14 +915,28 @@ static void do_2d_mapping(MTex *mtex, float *t, VlakRen *vlr, float *dxt, float
 		/* repeat */
 		if(tex->extend==TEX_REPEAT) {
 			if(tex->xrepeat>1) {
-				fx *= tex->xrepeat;
+				float origf= fx *= tex->xrepeat;
+				
 				if(fx>1.0) fx -= (int)(fx);
 				else if(fx<0.0) fx+= 1-(int)(fx);
+				
+				if(tex->flag & TEX_REPEAT_XMIR) {
+					int orig= (int)floor(origf);
+					if(orig & 1) 
+						fx= 1.0-fx;
+				}
 			}
 			if(tex->yrepeat>1) {
-				fy *= tex->yrepeat;
+				float origf= fy *= tex->yrepeat;
+				
 				if(fy>1.0) fy -= (int)(fy);
 				else if(fy<0.0) fy+= 1-(int)(fy);
+				
+				if(tex->flag & TEX_REPEAT_YMIR) {
+					int orig= (int)floor(origf);
+					if(orig & 1) 
+						fy= 1.0-fy;
+				}
 			}
 		}
 		/* crop */
@@ -1110,21 +1038,39 @@ static void do_2d_mapping(MTex *mtex, float *t, VlakRen *vlr, float *dxt, float
 		if(tex->extend==TEX_REPEAT) {
 			float max= 1.0f;
 			if(tex->xrepeat>1) {
+				float origf= fx *= tex->xrepeat;
+				
+				if(fx>1.0f) fx -= (int)(fx);
+				else if(fx<0.0f) fx+= 1-(int)(fx);
+				
+				if(tex->flag & TEX_REPEAT_XMIR) {
+					int orig= (int)floor(origf);
+					if(orig & 1) 
+						fx= 1.0f-fx;
+				}
+				
 				max= tex->xrepeat;
-				fx *= tex->xrepeat;
+				
 				dxt[0]*= tex->xrepeat;
 				dyt[0]*= tex->xrepeat;
-				if(fx>1.0) fx -= (int)(fx);
-				else if(fx<0.0) fx+= 1-(int)(fx);
 			}
 			if(tex->yrepeat>1) {
+				float origf= fy *= tex->yrepeat;
+				
+				if(fy>1.0f) fy -= (int)(fy);
+				else if(fy<0.0f) fy+= 1-(int)(fy);
+				
+				if(tex->flag & TEX_REPEAT_YMIR) {
+					int orig= (int)floor(origf);
+					if(orig & 1) 
+						fy= 1.0f-fy;
+				}
+				
 				if(max<tex->yrepeat)
 					max= tex->yrepeat;
-				fy *= tex->yrepeat;
+
 				dxt[1]*= tex->yrepeat;
 				dyt[1]*= tex->yrepeat;
-				if(fy>1.0) fy -= (int)(fy);
-				else if(fy<0.0) fy+= 1-(int)(fy);
 			}
 			if(max!=1.0f) {
 				dxt[2]*= max;
@@ -1156,6 +1102,7 @@ static void do_2d_mapping(MTex *mtex, float *t, VlakRen *vlr, float *dxt, float
 
 static int multitex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexResult *texres)
 {
+	float tmpvec[3];
 	int retval=0; /* return value, int:0, col:1, nor:2, everything:3 */
 
 	texres->talpha= 0;	/* is set when image texture returns alpha (considered premul) */
@@ -1163,7 +1110,7 @@ static int multitex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex,
 	switch(tex->type) {
 	
 	case 0:
-		texres->tin= 0.0;
+		texres->tin= 0.0f;
 		return 0;
 	case TEX_CLOUDS:
 		retval= clouds(tex, texvec, texres);
@@ -1187,8 +1134,8 @@ static int multitex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex,
 		retval= texnoise(tex, texres); 
 		break;
 	case TEX_IMAGE:
-		if(osatex) retval= imagewraposa(tex, tex->ima, texvec, dxt, dyt, texres); 
-		else retval= imagewrap(tex, tex->ima, texvec, texres); 
+		if(osatex) retval= imagewraposa(tex, tex->ima, NULL, texvec, dxt, dyt, texres); 
+		else retval= imagewrap(tex, tex->ima, NULL, texvec, texres); 
 		tag_image_time(tex->ima); /* tag image as having being used */
 		break;
 	case TEX_PLUGIN:
@@ -1200,35 +1147,44 @@ static int multitex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex,
 	case TEX_MUSGRAVE:
 		/* newnoise: musgrave types */
 		
-		/* ton: added this, for Blender convention reason. scaling texvec here is so-so ... */
-		VecMulf(texvec, 1.0/tex->noisesize);
+		/* ton: added this, for Blender convention reason. 
+		 * artificer: added the use of tmpvec to avoid scaling texvec
+		 */
+		VECCOPY(tmpvec, texvec);
+		VecMulf(tmpvec, 1.0/tex->noisesize);
 		
 		switch(tex->stype) {
 		case TEX_MFRACTAL:
 		case TEX_FBM:
-			retval= mg_mFractalOrfBmTex(tex, texvec, texres);
+			retval= mg_mFractalOrfBmTex(tex, tmpvec, texres);
 			break;
 		case TEX_RIDGEDMF:
 		case TEX_HYBRIDMF:
-			retval= mg_ridgedOrHybridMFTex(tex, texvec, texres);
+			retval= mg_ridgedOrHybridMFTex(tex, tmpvec, texres);
 			break;
 		case TEX_HTERRAIN:
-			retval= mg_HTerrainTex(tex, texvec, texres);
+			retval= mg_HTerrainTex(tex, tmpvec, texres);
 			break;
 		}
 		break;
 	/* newnoise: voronoi type */
 	case TEX_VORONOI:
-		/* ton: added this, for Blender convention reason. scaling texvec here is so-so ... */
-		VecMulf(texvec, 1.0/tex->noisesize);
+		/* ton: added this, for Blender convention reason.
+		 * artificer: added the use of tmpvec to avoid scaling texvec
+		 */
+		VECCOPY(tmpvec, texvec);
+		VecMulf(tmpvec, 1.0/tex->noisesize);
 		
-		retval= voronoiTex(tex, texvec, texres);
+		retval= voronoiTex(tex, tmpvec, texres);
 		break;
 	case TEX_DISTNOISE:
-		/* ton: added this, for Blender convention reason. scaling texvec here is so-so ... */
-		VecMulf(texvec, 1.0/tex->noisesize);
+		/* ton: added this, for Blender convention reason.
+		 * artificer: added the use of tmpvec to avoid scaling texvec
+		 */
+		VECCOPY(tmpvec, texvec);
+		VecMulf(tmpvec, 1.0/tex->noisesize);
 		
-		retval= mg_distNoiseTex(tex, texvec, texres);
+		retval= mg_distNoiseTex(tex, tmpvec, texres);
 		break;
 	}
 
@@ -1249,7 +1205,10 @@ static int multitex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex,
 int multitex_ext(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexResult *texres)
 {
 	
-	if(tex==NULL) return 0;
+	if(tex==NULL) {
+		memset(texres, 0, sizeof(TexResult));
+		return 0;
+	}
 	
 	/* Image requires 2d mapping conversion */
 	if(tex->type==TEX_IMAGE) {
@@ -1428,12 +1387,13 @@ void do_material_tex(ShadeInput *shi)
 {
 	MTex *mtex;
 	Tex *tex;
-	TexResult texres;
+	TexResult texres= {0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0, NULL};
 	float *co = NULL, *dx = NULL, *dy = NULL;
 	float fact, facm, factt, facmm, stencilTin=1.0;
 	float texvec[3], dxt[3], dyt[3], tempvec[3], norvec[3], warpvec[3], Tnor=1.0;
 	int tex_nr, rgbnor= 0, warpdone=0;
 
+	if (R.r.scemode & R_NO_TEX) return;
 	/* here: test flag if there's a tex (todo) */
 	
 	for(tex_nr=0; tex_nr<MAX_MTEX; tex_nr++) {
@@ -1488,7 +1448,21 @@ void do_material_tex(ShadeInput *shi)
 				co= shi->gl; dx= shi->dxco; dy= shi->dyco;
 			}
 			else if(mtex->texco==TEXCO_UV) {
-				co= shi->uv; dx= shi->dxuv; dy= shi->dyuv; 
+				ShadeInputUV *suv= &shi->uv[0];
+				int i;
+
+				if(mtex->uvname[0] != 0) {
+					for(i = 0; i < shi->totuv; i++) {
+						if(strcmp(shi->uv[i].name, mtex->uvname)==0) {
+							suv= &shi->uv[i];
+							break;
+						}
+					}
+				}
+
+				co= suv->uv;
+				dx= suv->dxuv;
+				dy= suv->dyuv; 
 			}
 			else if(mtex->texco==TEXCO_WINDOW) {
 				co= shi->winco; dx= shi->dxwin; dy= shi->dywin;
@@ -1568,6 +1542,17 @@ void do_material_tex(ShadeInput *shi)
 					dyt[0]= mtex->size[0]*dyt[0];
 					dyt[1]= mtex->size[1]*dyt[1];
 				}
+				
+				/* problem: repeat-mirror is not a 'repeat' but 'extend' in imagetexture.c */
+				if(tex->flag & TEX_REPEAT_XMIR) {
+					if(texvec[0]>1.0f) texvec[0] -= (int)(texvec[0]);
+					else if(texvec[0]<0.0f) texvec[0]+= 1-(int)(texvec[0]);
+				}
+				if(tex->flag & TEX_REPEAT_YMIR) {
+					if(texvec[1]>1.0f) texvec[1] -= (int)(texvec[1]);
+					else if(texvec[1]<0.0f) texvec[1]+= 1-(int)(texvec[1]);
+				}
+				
 			}
 			else {
 
@@ -1727,16 +1712,31 @@ void do_material_tex(ShadeInput *shi)
 					
 					/* we need to code blending modes for normals too once.. now 1 exception hardcoded */
 					
-					if(tex->type==TEX_IMAGE && (tex->imaflag & TEX_NORMALMAP)) {
-						fact= Tnor*tex->norfac;
-						if(fact>1.0) fact= 1.0; else if(fact<-1.0) fact= -1.0;
-						facm= 1.0- fact;
-						if(shi->mat->mode & MA_TANGENT_V) {
-							shi->tang[0]= facm*shi->tang[0] + fact*texres.nor[0];
-							shi->tang[1]= facm*shi->tang[1] + fact*texres.nor[1];
-							shi->tang[2]= facm*shi->tang[2] + fact*texres.nor[2];
+					if ((tex->type==TEX_IMAGE) && (tex->imaflag & TEX_NORMALMAP)) {
+						/* qdn: for normalmaps, to invert the normalmap vector,
+						   it is better to negate x & y instead of subtracting the vector as was done before */
+						tex->norfac = mtex->norfac;
+						if (mtex->maptoneg & MAP_NORM) {
+							texres.nor[0] = -texres.nor[0];
+							texres.nor[1] = -texres.nor[1];
+						}
+						fact = Tnor*tex->norfac;
+						if (fact>1.f) fact = 1.f;
+						facm = 1.f-fact;
+						if (shi->mat->mode & MA_NORMAP_TANG) {
+							/* qdn: tangent space */
+							float B[3], tv[3];
+							Crossf(B, shi->vn, shi->tang);	/* bitangent */
+							/* transform norvec from tangent space to object surface in camera space */
+							tv[0] = texres.nor[0]*shi->tang[0] + texres.nor[1]*B[0] + texres.nor[2]*shi->vn[0];
+							tv[1] = texres.nor[0]*shi->tang[1] + texres.nor[1]*B[1] + texres.nor[2]*shi->vn[1];
+							tv[2] = texres.nor[0]*shi->tang[2] + texres.nor[1]*B[2] + texres.nor[2]*shi->vn[2];
+							shi->vn[0]= facm*shi->vn[0] + fact*tv[0];
+							shi->vn[1]= facm*shi->vn[1] + fact*tv[1];
+							shi->vn[2]= facm*shi->vn[2] + fact*tv[2];
 						}
 						else {
+							/* qdn: worldspace */
 							shi->vn[0]= facm*shi->vn[0] + fact*texres.nor[0];
 							shi->vn[1]= facm*shi->vn[1] + fact*texres.nor[1];
 							shi->vn[2]= facm*shi->vn[2] + fact*texres.nor[2];
@@ -1750,7 +1750,6 @@ void do_material_tex(ShadeInput *shi)
 						}
 						else {
 							float nor[3], dot;
-						
 							/* prevent bump to become negative normal */
 							nor[0]= Tnor*tex->norfac*texres.nor[0];
 							nor[1]= Tnor*tex->norfac*texres.nor[1];
@@ -1901,9 +1900,11 @@ void do_material_tex(ShadeInput *shi)
 void do_halo_tex(HaloRen *har, float xn, float yn, float *colf)
 {
 	MTex *mtex;
-	TexResult texres;
+	TexResult texres= {0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0, NULL};
 	float texvec[3], dxt[3], dyt[3], fact, facm, dx;
 	int rgb, osatex;
+
+	if (R.r.scemode & R_NO_TEX) return;
 	
 	mtex= har->mat->mtex[0];
 	if(mtex->tex==NULL) return;
@@ -2027,11 +2028,12 @@ void do_halo_tex(HaloRen *har, float xn, float yn, float *colf)
 void do_sky_tex(float *rco, float *lo, float *dxyview, float *hor, float *zen, float *blend, int skyflag)
 {
 	MTex *mtex;
-	TexResult texres;
+	TexResult texres= {0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0, NULL};
 	float *co, fact, stencilTin=1.0;
 	float tempvec[3], texvec[3], dxt[3], dyt[3];
 	int tex_nr, rgb= 0, ok;
 	
+	if (R.r.scemode & R_NO_TEX) return;
 	/* todo: add flag to test if there's a tex */
 	texres.nor= NULL;
 	
@@ -2058,14 +2060,14 @@ void do_sky_tex(float *rco, float *lo, float *dxyview, float *hor, float *zen, f
 			/* Grab the mapping settings for this texture */
 			switch(mtex->texco) {
 			case TEXCO_ANGMAP:
-				
+				/* only works with texture being "real" */
 				fact= (1.0/M_PI)*acos(lo[2])/(sqrt(lo[0]*lo[0] + lo[1]*lo[1])); 
 				tempvec[0]= lo[0]*fact;
 				tempvec[1]= lo[1]*fact;
 				tempvec[2]= 0.0;
 				co= tempvec;
 				break;
-			
+				
 			case TEXCO_H_SPHEREMAP:
 			case TEXCO_H_TUBEMAP:
 				if(skyflag & WO_ZENUP) {
@@ -2203,11 +2205,12 @@ void do_lamp_tex(LampRen *la, float *lavec, ShadeInput *shi, float *colf)
 	Object *ob;
 	MTex *mtex;
 	Tex *tex;
-	TexResult texres;
+	TexResult texres= {0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0, NULL};
 	float *co = NULL, *dx = NULL, *dy = NULL, fact, stencilTin=1.0;
 	float texvec[3], dxt[3], dyt[3], tempvec[3];
 	int tex_nr, rgb= 0;
 	
+	if (R.r.scemode & R_NO_TEX) return;
 	tex_nr= 0;
 	
 	for(; tex_nr<MAX_MTEX; tex_nr++) {
@@ -2362,7 +2365,7 @@ void do_lamp_tex(LampRen *la, float *lavec, ShadeInput *shi, float *colf)
 
 /* ------------------------------------------------------------------------- */
 
-void externtex(MTex *mtex, float *vec, float *tin, float *tr, float *tg, float *tb, float *ta)
+int externtex(MTex *mtex, float *vec, float *tin, float *tr, float *tg, float *tb, float *ta)
 {
 	Tex *tex;
 	TexResult texr;
@@ -2370,7 +2373,7 @@ void externtex(MTex *mtex, float *vec, float *tin, float *tr, float *tg, float *
 	int rgb;
 	
 	tex= mtex->tex;
-	if(tex==NULL) return;
+	if(tex==NULL) return 0;
 	texr.nor= NULL;
 	
 	/* placement */
@@ -2404,20 +2407,21 @@ void externtex(MTex *mtex, float *vec, float *tin, float *tr, float *tg, float *
 	*tg= texr.tg;
 	*tb= texr.tb;
 	*ta= texr.ta;
+
+	return (rgb != 0);
 }
 
 
 /* ------------------------------------------------------------------------- */
 
-void render_realtime_texture(ShadeInput *shi)
+void render_realtime_texture(ShadeInput *shi, Image *ima)
 {
 	TexResult texr;
-	Image *ima;
 	static Tex tex1, tex2;	// threadsafe
 	static int firsttime= 1;
 	Tex *tex;
-	float texvec[2], dx[2], dy[2];
-	
+	float texvec[3], dx[2], dy[2];
+
 	if(firsttime) {
 		firsttime= 0;
 		default_tex(&tex1);
@@ -2428,27 +2432,23 @@ void render_realtime_texture(ShadeInput *shi)
 	
 	if(shi->ys & 1) tex= &tex1; else tex= &tex2;	// threadsafe
 	
-	ima = shi->vlr->tface->tpage;
-	if(ima) {
-		
-		texvec[0]= 0.5+0.5*shi->uv[0];
-		texvec[1]= 0.5+0.5*shi->uv[1];
-		if(shi->osatex) {
-			dx[0]= 0.5*shi->dxuv[0];
-			dx[1]= 0.5*shi->dxuv[1];
-			dy[0]= 0.5*shi->dyuv[0];
-			dy[1]= 0.5*shi->dyuv[1];
-		}
-		
-		texr.nor= NULL;
-		
-		if(shi->osatex) imagewraposa(tex, ima, texvec, dx, dy, &texr); 
-		else imagewrap(tex, ima, texvec, &texr); 
-		
-		shi->vcol[0]*= texr.tr;
-		shi->vcol[1]*= texr.tg;
-		shi->vcol[2]*= texr.tb;
+	texvec[0]= 0.5+0.5*shi->uv[0].uv[0];
+	texvec[1]= 0.5+0.5*shi->uv[0].uv[1];
+	if(shi->osatex) {
+		dx[0]= 0.5*shi->uv[0].dxuv[0];
+		dx[1]= 0.5*shi->uv[0].dxuv[1];
+		dy[0]= 0.5*shi->uv[0].dyuv[0];
+		dy[1]= 0.5*shi->uv[0].dyuv[1];
 	}
+	
+	texr.nor= NULL;
+	
+	if(shi->osatex) imagewraposa(tex, ima, NULL, texvec, dx, dy, &texr); 
+	else imagewrap(tex, ima, NULL, texvec, &texr); 
+	
+	shi->vcol[0]*= texr.tr;
+	shi->vcol[1]*= texr.tg;
+	shi->vcol[2]*= texr.tb;
 }
 
 /* eof */
