@@ -100,29 +100,8 @@ char texstr[20][12]= {"None"  , "Clouds" , "Wood", "Marble", "Magic"  , "Blend",
 void test_idbutton_cb(void *namev, void *arg2)
 {
 	char *name= namev;
-	test_idbutton(name+2);
 	
-	/* temporal! image rename for procedural Image has issues */
-	if(arg2 && GS(name)==ID_IM) {
-		/* when name changed */
-		if(strcmp(name+2, (char *)arg2)) {
-			ID *id= NULL;
-			if(strcmp((char *)arg2, "Render Result")==0)
-				id= find_id(name, name+2);
-			else if(strcmp((char *)arg2, "Compositor")==0)
-				id= find_id(name, name+2);
-			if(id) {
-				Image *ima= (Image *)id;
-				if(ima->ibuf) {
-					/* ibuf rect or rect_float is possibly not allocated but borrowed */
-					ImBuf *ibuf= IMB_dupImBuf(ima->ibuf);
-					IMB_freeImBuf(ima->ibuf);
-					ima->ibuf= ibuf;
-					printf("changed ima %s\n", name);
-				}
-			}
-		}
-	}
+	test_idbutton(name+2);
 }
 
 
@@ -148,7 +127,7 @@ void test_actionpoin_but(char *name, ID **idpp)
 	id= G.main->action.first;
 	while(id) {
 		if( strcmp(name, id->name+2)==0 ) {
-			id_lib_extern(id);	/* checks lib data, sets correct flag for saving then */
+			id_us_plus(id);
 			*idpp= id;
 			return;
 		}
@@ -403,6 +382,14 @@ static void curvemap_tools_dofunc(void *cumap_v, int event)
 			curvemap_sethandle(cuma, 0);
 			curvemapping_changed(cumap, 0);
 			break;
+		case 4: /* extend horiz */
+			cuma->flag &= ~CUMA_EXTEND_EXTRAPOLATE;
+			curvemapping_changed(cumap, 0);
+			break;
+		case 5: /* extend extrapolate */
+			cuma->flag |= CUMA_EXTEND_EXTRAPOLATE;
+			curvemapping_changed(cumap, 0);
+			break;
 	}
 	addqueue(curarea->win, REDRAW, 1);
 }
@@ -418,6 +405,8 @@ static uiBlock *curvemap_tools_func(void *cumap_v)
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Reset View",				0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 1, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Vector Handle",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 2, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Auto Handle",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 3, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Extend Horizontal",		0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 4, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Extend Extrapolated",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Reset Curve",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 0, "");
 	
 	uiBlockSetDirection(block, UI_RIGHT);
@@ -484,7 +473,7 @@ void curvemap_buttons(uiBlock *block, CurveMapping *cumap, char labeltype, short
 	bt= uiDefIconBlockBut(block, curvemap_clipping_func, cumap, event, icon, xco, yco, dx, 18, "Clipping Options");
 	
 	xco= (short)(rect->xmin+8.0f*dx);
-	bt= uiDefIconBut(block, BUT, redraw, ICON_X, xco, yco, dx, 18, NULL, 0.0, 0.0, 0.0, 0.0, "Delete points");
+	bt= uiDefIconBut(block, BUT, event, ICON_X, xco, yco, dx, 18, NULL, 0.0, 0.0, 0.0, 0.0, "Delete points");
 	uiButSetFunc(bt, curvemap_buttons_delete, cumap, NULL);
 	
 	uiBlockSetEmboss(block, UI_EMBOSS);
@@ -569,6 +558,9 @@ void do_butspace(unsigned short event)
 	else if(event<=B_MESHBUTS) {
 		do_meshbuts(event);
 	}
+	else if(event<=B_VGROUPBUTS) {
+		do_vgroupbuts(event);
+	}
 	else if(event<=B_CURVEBUTS) {
 		do_curvebuts(event);
 	}
