@@ -1,5 +1,5 @@
 /**
- * $Id: buttons_scene.c,v 1.141 2006/07/11 11:54:59 ton Exp $ 
+ * $Id: buttons_scene.c,v 1.165 2007/01/20 08:01:55 elubie Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -64,6 +64,7 @@
 #include "BIF_keyval.h"
 #include "BIF_mainqueue.h"
 #include "BIF_mywindow.h"
+#include "BIF_meshtools.h"
 #include "BIF_resources.h"
 #include "BIF_renderwin.h"
 #include "BIF_screen.h"
@@ -78,15 +79,22 @@
 /* -----includes for this file specific----- */
 
 #include "DNA_image_types.h"
+
 #include "BKE_writeavi.h"
 #include "BKE_writeffmpeg.h"
 #include "BKE_image.h"
+
+#include "BLI_threads.h"
+
+#include "BIF_editsound.h"
 #include "BIF_writeimage.h"
 #include "BIF_writeavicodec.h"
-#include "BIF_editsound.h"
+
 #include "BSE_seqaudio.h"
 #include "BSE_headerbuttons.h"
 
+#include "RE_pipeline.h"
+
 #include "butspace.h" // own module
 
 #ifdef WITH_QUICKTIME
@@ -118,7 +126,7 @@ void playback_anim(void);
 /* ************************ SOUND *************************** */
 static void load_new_sample(char *str)	/* called from fileselect */
 {
-	char name[FILE_MAXDIR+FILE_MAXFILE];
+	char name[FILE_MAX];
 	bSound *sound;
 	bSample *sample, *newsample;
 
@@ -158,7 +166,7 @@ static void load_new_sample(char *str)	/* called from fileselect */
 
 void do_soundbuts(unsigned short event)
 {
-	char name[FILE_MAXDIR+FILE_MAXFILE];
+	char name[FILE_MAX];
 	bSound *sound;
 	bSample *sample;
 	bSound* tempsound;
@@ -188,11 +196,7 @@ void do_soundbuts(unsigned short event)
 		break;
 
 	case B_SOUND_MENU_SAMPLE:
-		if (G.buts->menunr == -2) {
-			if (sound) {
-				activate_databrowse((ID *)sound->sample, ID_SAMPLE, 0, B_SOUND_MENU_SAMPLE, &G.buts->menunr, do_soundbuts);
-			}
-		} else if (G.buts->menunr > 0) {
+		if (G.buts->menunr > 0) {
 			sample = BLI_findlink(samples, G.buts->menunr - 1);
 			if (sample && sound) {
 				BLI_strncpy(sound->name, sample->name, sizeof(sound->name));
@@ -348,13 +352,32 @@ static void sound_panel_sequencer(void)
 	
 }
 
+static char *make_sample_menu(void)
+{
+	int len= BLI_countlist(samples);	/* BKE_sound.h */
+	
+	if(len) {
+		bSample *sample;
+		char *str;
+		int nr, a=0;
+		
+		str= MEM_callocN(32*len, "menu");
+		
+		for(nr=1, sample= samples->first; sample; sample= sample->id.next, nr++) {
+			a+= sprintf(str+a, "|%s %%x%d", sample->id.name+2, nr);
+		}
+		return str;
+	}
+	return NULL;
+}
+
 static void sound_panel_sound(bSound *sound)
 {
 	static int packdummy=0;
 	ID *id, *idfrom;
 	uiBlock *block;
 	bSample *sample;
-	char *strp, str[32], ch[256];
+	char *strp, ch[256];
 
 	block= uiNewBlock(&curarea->uiblocks, "sound_panel_sound", UI_EMBOSS, UI_HELV, curarea->win);
 	if(uiNewPanel(curarea, block, "Sound", "Sound", 0, 0, 318, 204)==0) return;
@@ -388,27 +411,26 @@ static void sound_panel_sound(bSound *sound)
 		}
 
 		/* sample browse buttons */
-
-		id= (ID *)sound->sample;
-		IDnames_to_pupstring(&strp, NULL, NULL, samples, id, &(G.buts->menunr));
-		if (strp[0]) uiDefButS(block, MENU, B_SOUND_MENU_SAMPLE, strp, 10,120,23,20, &(G.buts->menunr), 0, 0, 0, 0, "Select another loaded sample");
-		MEM_freeN(strp);
-		
-		uiDefBut(block, TEX, B_SOUND_NAME_SAMPLE, "",		35,120,225,20, sound->name, 0.0, 79.0, 0, 0, "The sample file used by this Sound");
-		
-		sprintf(str, "%d", sample->id.us);
-		uiDefBut(block, BUT, B_SOUND_UNLINK_SAMPLE, str,	260,120,25,20, 0, 0, 0, 0, 0, "The number of users");
+		uiBlockBeginAlign(block);
+		strp= make_sample_menu();
+		if (strp) {
+			uiDefButS(block, MENU, B_SOUND_MENU_SAMPLE, strp, 10,120,23,20, &(G.buts->menunr), 0, 0, 0, 0, "Select another loaded sample");
+			MEM_freeN(strp);
+		}
+		uiDefBut(block, TEX, B_SOUND_NAME_SAMPLE, "",		35,120,250,20, sound->name, 0.0, 79.0, 0, 0, "The sample file used by this Sound");
 		
 		if (sound->sample->packedfile) packdummy = 1;
 		else packdummy = 0;
 		
 		uiDefIconButBitI(block, TOG, 1, B_SOUND_UNPACK_SAMPLE, ICON_PACKAGE,
-			285, 120,25,24, &packdummy, 0, 0, 0, 0,"Pack/Unpack this sample");
+			285, 120,25,20, &packdummy, 0, 0, 0, 0,"Pack/Unpack this sample");
 		
+		uiBlockBeginAlign(block);
 		uiDefBut(block, BUT, B_SOUND_LOAD_SAMPLE, "Load sample", 10, 95,150,24, 0, 0, 0, 0, 0, "Load a different sample file");
 
 		uiDefBut(block, BUT, B_SOUND_PLAY_SAMPLE, "Play", 	160, 95, 150, 24, 0, 0.0, 0, 0, 0, "Playback sample using settings below");
 		
+		uiBlockBeginAlign(block);
 		uiDefButF(block, NUMSLI, B_SOUND_CHANGED, "Volume: ",
 			10,70,150,20, &sound->volume, 0.0, 1.0, 0, 0, "Game engine only: Set the volume of this sound");
 
@@ -416,6 +438,7 @@ static void sound_panel_sound(bSound *sound)
 			160,70,150,20, &sound->pitch, -12.0, 12.0, 0, 0, "Game engine only: Set the pitch of this sound");
 
 		/* looping */
+		uiBlockBeginAlign(block);
 		uiDefButBitI(block, TOG, SOUND_FLAGS_LOOP, B_SOUND_REDRAW, "Loop",
 			10, 50, 95, 20, &sound->flags, 0.0, 0.0, 0, 0, "Game engine only: Toggle between looping on/off");
 
@@ -427,6 +450,7 @@ static void sound_panel_sound(bSound *sound)
 	
 
 		/* 3D settings ------------------------------------------------------------------ */
+		uiBlockBeginAlign(block);
 
 		if (sound->sample->channels == 1) {
 			uiDefButBitI(block, TOG, SOUND_FLAGS_3D, B_SOUND_REDRAW, "3D Sound",
@@ -459,11 +483,10 @@ static void backbuf_pic(char *name)
 	strcpy(G.scene->r.backbuf, name);
 	allqueue(REDRAWBUTSSCENE, 0);
 
-	ima= add_image(name);
-	if(ima) {
-		free_image_buffers(ima);	/* force read again */
-		ima->ok= 1;
-	}
+	ima= BKE_add_image_file(name);
+	if(ima)
+		BKE_image_signal(ima, NULL, IMA_SIGNAL_RELOAD);
+
 	BIF_undo_push("Change background picture");
 }
 
@@ -476,20 +499,23 @@ static void ftype_pic(char *name)
 static void run_playanim(char *file) 
 {
 	extern char bprogname[];	/* usiblender.c */
-	char str[FILE_MAXDIR+FILE_MAXFILE];
+	char str[FILE_MAX];
 	int pos[2], size[2];
 
 	/* use current settings for defining position of window. it actually should test image size */
 	calc_renderwin_rectangle((G.scene->r.xsch*G.scene->r.size)/100, 
 							 (G.scene->r.ysch*G.scene->r.size)/100, G.winpos, pos, size);
-
+#ifdef WIN32
 	sprintf(str, "%s -a -p %d %d \"%s\"", bprogname, pos[0], pos[1], file);
+#else
+	sprintf(str, "\"%s\" -a -p %d %d \"%s\"", bprogname, pos[0], pos[1], file);
+#endif
 	system(str);
 }
 
 void playback_anim(void)
 {	
-	char file[FILE_MAXDIR+FILE_MAXFILE];
+	char file[FILE_MAX];
 
 	if(BKE_imtype_is_movie(G.scene->r.imtype)) {
 		switch (G.scene->r.imtype) {
@@ -513,7 +539,7 @@ void playback_anim(void)
 		else error("Can't find movie: %s", file);
 	}
 	else {
-		BKE_makepicstring(file, G.scene->r.sfra);
+		BKE_makepicstring(file, G.scene->r.pic, G.scene->r.sfra, G.scene->r.imtype);
 		if(BLI_exist(file)) {
 			run_playanim(file);
 		}
@@ -808,6 +834,14 @@ void do_render_panels(unsigned short event)
 		}
 		allqueue(REDRAWBUTSSCENE, 0);
 		allqueue(REDRAWNODE, 0);
+		break;
+	case B_SET_PASS:
+		if(G.scene->nodetree) {
+			ntreeCompositForceHidden(G.scene->nodetree);
+			allqueue(REDRAWNODE, 0);
+		}
+		allqueue(REDRAWBUTSSCENE, 0);
+		allqueue(REDRAWOOPS, 0);
 	}
 }
 
@@ -829,11 +863,11 @@ static uiBlock *edge_render_menu(void *arg_unused)
 	
 	uiBlockBeginAlign(block);
 	uiDefButF(block, NUMSLI, 0, "R ",   45, 50, 175,19,   &G.scene->r.edgeR, 0.0, 1.0, B_EDGECOLSLI, 0,
-		  "For unified renderer: Colour for edges in toon shading mode.");
+		  "Colour for edges in toon shading mode.");
 	uiDefButF(block, NUMSLI, 0, "G ",  	45, 30, 175,19,  &G.scene->r.edgeG, 0.0, 1.0, B_EDGECOLSLI, 0,
-		  "For unified renderer: Colour for edges in toon shading mode.");
+		  "Colour for edges in toon shading mode.");
 	uiDefButF(block, NUMSLI, 0, "B ",  	45, 10, 175,19,  &G.scene->r.edgeB, 0.0, 1.0, B_EDGECOLSLI, 0,
-		  "For unified renderer: Colour for edges in toon shading mode.");
+		  "Colour for edges in toon shading mode.");
 
 	
 	uiBlockSetDirection(block, UI_TOP);
@@ -841,30 +875,6 @@ static uiBlock *edge_render_menu(void *arg_unused)
 	return block;
 }
 
-#if 0
-/* NOTE: this is a block-menu, needs 0 events, otherwise the menu closes */
-static uiBlock *post_render_menu(void *arg_unused)
-{
-	uiBlock *block;
-	
-	block= uiNewBlock(&curarea->uiblocks, "post render", UI_EMBOSS, UI_HELV, curarea->win);
-		
-	/* use this for a fake extra empy space around the buttons */
-	uiDefBut(block, LABEL, 0, "",			-10, -10, 200, 120, NULL, 0, 0, 0, 0, "");
-	uiBlockBeginAlign(block);
-	uiDefButF(block, NUMSLI, 0, "Add:",		0,80,180,19, &G.scene->r.postadd, -1.0, 1.0, 0, 0, "");
-	uiDefButF(block, NUMSLI, 0, "Mul:",		0,60,180,19,  &G.scene->r.postmul, 0.01, 4.0, 0, 0, "");
-	uiDefButF(block, NUMSLI, 0, "Gamma:",	0,40,180,19,  &G.scene->r.postgamma, 0.1, 4.0, 0, 0, "");
-	uiDefButF(block, NUMSLI, 0, "Hue:",		0,20,180,19,  &G.scene->r.posthue, -0.5, 0.5, 0, 0, "");
-	uiDefButF(block, NUMSLI, 0, "Sat:",		0, 0,180,19,  &G.scene->r.postsat, 0.0, 4.0, 0, 0, "");
-
-	uiBlockSetDirection(block, UI_TOP);
-	
-	addqueue(curarea->win, UI_BUT_EVENT, B_FBUF_REDO);
-	
-	return block;
-}
-#endif
 
 /* NOTE: this is a block-menu, needs 0 events, otherwise the menu closes */
 static uiBlock *framing_render_menu(void *arg_unused)
@@ -1024,7 +1034,7 @@ static char *imagetype_pup(void)
 	char formatstring[1024];
 	char appendstring[1024];
 
-	strcpy(formatstring, "Save image as: %%t|%s %%x%d|%s %%x%d|%s %%x%d|%s %%x%d|%s %%x%d|%s %%x%d|%s %%x%d|%s %%x%d");
+	strcpy(formatstring, "Save image as: %%t|%s %%x%d|%s %%x%d|%s %%x%d|%s %%x%d|%s %%x%d|%s %%x%d|%s %%x%d");
 
 #ifdef __sgi
 	strcat(formatstring, "|%s %%x%d");	// add space for Movie
@@ -1071,7 +1081,6 @@ static char *imagetype_pup(void)
 			"Jpeg",           R_JPEG90,
 			"HamX",           R_HAMX,
 			"Iris",           R_IRIS,
-			"Iris + Zbuffer", R_IRIZ,
 			"Radiance HDR",   R_RADHDR,
 			"Cineon",		  R_CINEON,
 			"DPX",			  R_DPX
@@ -1097,7 +1106,6 @@ static char *imagetype_pup(void)
 			"Jpeg",           R_JPEG90,
 			"HamX",           R_HAMX,
 			"Iris",           R_IRIS,
-			"Iris + Zbuffer", R_IRIZ,
 			"Radiance HDR",   R_RADHDR,
 			"Cineon",		  R_CINEON,
 			"DPX",			  R_DPX
@@ -1111,6 +1119,8 @@ static char *imagetype_pup(void)
 	strcpy(formatstring, "|%s %%x%d");
 	sprintf(appendstring, formatstring, "OpenEXR", R_OPENEXR);
 	strcat(string, appendstring);
+	sprintf(appendstring, formatstring, "MultiLayer", R_MULTILAYER);
+	strcat(string, appendstring);
 #endif
 	
 	if (G.have_libtiff) {
@@ -1164,7 +1174,7 @@ static void render_panel_output(void)
 
 	if(G.scene->set) {
 		uiSetButLock(1, NULL);
-		uiDefIDPoinBut(block, test_scenepoin_but, ID_SCE, 0, "",	31, 120, 100, 20, &(G.scene->set), "Name of the Set");
+		uiDefIDPoinBut(block, test_scenepoin_but, ID_SCE, B_NOP, "",	31, 120, 100, 20, &(G.scene->set), "Name of the Set");
 		uiClearButLock();
 		uiDefIconBut(block, BUT, B_CLEARSET, ICON_X, 		132, 120, 20, 20, 0, 0, 0, 0, 0, "Remove Set link");
 	}
@@ -1172,7 +1182,7 @@ static void render_panel_output(void)
 
 	uiBlockSetCol(block, TH_BUT_SETTING1);
 	uiDefButBitS(block, TOG, R_BACKBUF, B_NOP,"Backbuf",	10, 94, 80, 20, &G.scene->r.bufflag, 0, 0, 0, 0, "Enable/Disable use of Backbuf image");	
-	uiDefButBitI(block, TOG, R_THREADS, B_NOP,"Threads",	10, 68, 80, 20, &G.scene->r.mode, 0, 0, 0, 0, "Enable/Disable render in two threads");	
+	uiDefButS(block, NUM, B_NOP, "Threads:",				10, 68, 100, 20, &G.scene->r.threads, 1, BLENDER_MAX_THREADS, 0, 0, "Amount of threads for render");	
 	uiBlockSetCol(block, TH_AUTO);
 		
 	uiBlockBeginAlign(block);
@@ -1197,7 +1207,46 @@ static void render_panel_output(void)
 	uiDefBlockBut(block, edge_render_menu, NULL, "Edge Settings", 170, 94, 140, 20, "Display Edge settings");
 	uiBlockEndAlign(block);
 	
-	uiDefButBitS(block, TOG, R_FREE_IMAGE, B_NOP, "Free Tex Images", 170, 68, 140, 20, &G.scene->r.scemode, 0.0, 0.0, 0, 0, "Frees all Images used by Textures after each render");
+	uiBlockBeginAlign(block);
+	uiDefButBitS(block, TOG, R_NO_TEX, B_NOP, "Disable Tex", 115, 68, 70, 20, &G.scene->r.scemode, 0.0, 0.0, 0, 0, "Disables Textures for render");
+	uiDefButBitS(block, TOG, R_FREE_IMAGE, B_NOP, "Free Tex Images", 205, 68, 100, 20, &G.scene->r.scemode, 0.0, 0.0, 0, 0, "Frees all Images used by Textures after each render");
+	uiBlockEndAlign(block);
+}
+
+static void do_bake_func(void *unused_v, void *unused_p)
+{
+	objects_bake_render(0);
+}
+
+static void render_panel_bake(void)
+{
+	uiBlock *block;
+	uiBut *but;
+	
+	block= uiNewBlock(&curarea->uiblocks, "render_panel_bake", UI_EMBOSS, UI_HELV, curarea->win);
+	uiNewPanelTabbed("Anim", "Render");
+	if(uiNewPanel(curarea, block, "Bake", "Render", 320, 0, 318, 204)==0) return;
+	
+	but= uiDefBut(block, BUT, B_NOP, "BAKE",	10, 150, 190,40, 0, 0, 0, 0, 0, "Start the bake render for selected Objects");
+	uiButSetFunc(but, do_bake_func, NULL, NULL);
+#if 0	
+	uiBlockBeginAlign(block);
+	uiDefButBitS(block, TOG, R_BAKE_OSA, B_DIFF, "OSA",		10,120,190,20, &G.scene->r.bake_flag, 0, 0, 0, 0, "Enables Oversampling (Anti-aliasing)");
+	uiDefButS(block, ROW,B_DIFF,"5",			10,100,50,20,&G.scene->r.bake_osa,2.0,5.0, 0, 0, "Sets oversample level to 5");
+	uiDefButS(block, ROW,B_DIFF,"8",			60,100,45,20,&G.scene->r.bake_osa,2.0,8.0, 0, 0, "Sets oversample level to 8");
+	uiDefButS(block, ROW,B_DIFF,"11",			105,100,45,20,&G.scene->r.bake_osa,2.0,11.0, 0, 0, "Sets oversample level to 11");
+	uiDefButS(block, ROW,B_DIFF,"16",			150,100,50,20,&G.scene->r.bake_osa,2.0,16.0, 0, 0, "Sets oversample level to 16");
+#endif	
+	uiBlockBeginAlign(block);
+	uiDefButS(block, ROW,B_DIFF,"Full Render",		210,170,120,20,&G.scene->r.bake_mode, 1.0, RE_BAKE_ALL, 0, 0, "");
+	uiDefButS(block, ROW,B_DIFF,"Ambient Occlusion",210,150,120,20,&G.scene->r.bake_mode, 1.0, RE_BAKE_AO, 0, 0, "");
+	uiDefButS(block, ROW,B_DIFF,"Normals",		210,130,120,20,&G.scene->r.bake_mode, 1.0, RE_BAKE_NORMALS, 0, 0, "");
+	uiDefButS(block, ROW,B_DIFF,"Textures",		210,110,120,20,&G.scene->r.bake_mode, 1.0, RE_BAKE_TEXTURE, 0, 0, "");
+	uiBlockEndAlign(block);
+	
+	uiDefButBitS(block, TOG, R_BAKE_CLEAR, B_DIFF, "Clear",		210,80,120,20,&G.scene->r.bake_flag, 0.0, 0, 0, 0, "Clear Images before baking");
+	
+	uiDefButS(block, NUM, B_DIFF,"Margin:",				210,50,120,20,&G.scene->r.bake_filter, 0.0, 32.0, 0, 0, "Amount of pixels to extend the baked result with, as post process filter");
 }
 
 static void render_panel_render(void)
@@ -1210,12 +1259,17 @@ static void render_panel_render(void)
 
 	uiBlockBeginAlign(block);
 	uiDefBut(block, BUT,B_DORENDER,"RENDER",	369, 164, 191,37, 0, 0, 0, 0, 0, "Start the rendering");
+#ifndef DISABLE_YAFRAY
 	/* yafray: on request, render engine menu is back again, and moved to Render panel */
 	uiDefButS(block, MENU, B_SWITCHRENDER, "Rendering Engine %t|Blender Internal %x0|YafRay %x1", 
 												369, 142, 191, 20, &G.scene->r.renderer, 0, 0, 0, 0, "Choose rendering engine");	
+#else
+	uiDefButS(block, MENU, B_SWITCHRENDER, "Rendering Engine %t|Blender Internal %x0", 
+												369, 142, 191, 20, &G.scene->r.renderer, 0, 0, 0, 0, "Choose rendering engine");	
+#endif /* disable yafray */
 
 	uiBlockBeginAlign(block);
-	uiDefButBitI(block, TOG, R_OSA, 0, "OSA",		369,109,122,20,&G.scene->r.mode, 0, 0, 0, 0, "Enables Oversampling (Anti-aliasing)");
+	uiDefButBitI(block, TOG, R_OSA, B_DIFF, "OSA",	369,109,122,20,&G.scene->r.mode, 0, 0, 0, 0, "Enables Oversampling (Anti-aliasing)");
 	uiDefButS(block, ROW,B_DIFF,"5",			369,88,29,20,&G.scene->r.osa,2.0,5.0, 0, 0, "Sets oversample level to 5");
 	uiDefButS(block, ROW,B_DIFF,"8",			400,88,29,20,&G.scene->r.osa,2.0,8.0, 0, 0, "Sets oversample level to 8 (Recommended)");
 	uiDefButS(block, ROW,B_DIFF,"11",			431,88,29,20,&G.scene->r.osa,2.0,11.0, 0, 0, "Sets oversample level to 11");
@@ -1265,8 +1319,7 @@ static void render_panel_render(void)
 	uiDefButS(block, MENU, B_DIFF,str,		565,34,60,20, &G.scene->r.filtertype, 0, 0, 0, 0, "Set sampling filter for antialiasing");
 	uiDefButF(block, NUM,B_DIFF,"",			627,34,60,20,&G.scene->r.gauss,0.5, 1.5, 10, 2, "Sets the filter size");
 	
-	uiDefButBitI(block, TOG, R_BORDER, REDRAWVIEWCAM, "Border",	565,13,60,20, &G.scene->r.mode, 0, 0, 0, 0, "Render a small cut-out of the image");
-	uiDefButBitI(block, TOG, R_GAMMA, B_REDR, "Gamma",	627,13,60,20, &G.scene->r.mode, 0, 0, 0, 0, "Enable gamma correction");
+	uiDefButBitI(block, TOG, R_BORDER, REDRAWVIEWCAM, "Border",	565,13,122,20, &G.scene->r.mode, 0, 0, 0, 0, "Render a small cut-out of the image");
 	uiBlockEndAlign(block);
 
 }
@@ -1294,7 +1347,7 @@ static void render_panel_anim(void)
 
 	uiBlockBeginAlign(block);
 	uiDefButI(block, NUM,REDRAWSEQ,"Sta:",692,10,94,24, &G.scene->r.sfra,1.0,MAXFRAMEF, 0, 0, "The start frame of the animation");
-	uiDefButI(block, NUM,REDRAWSEQ,"End:",789,10,95,24, &G.scene->r.efra,1.0,MAXFRAMEF, 0, 0, "The end  frame of the animation");
+	uiDefButI(block, NUM,REDRAWSEQ,"End:",789,10,95,24, &G.scene->r.efra,SFRA,MAXFRAMEF, 0, 0, "The end  frame of the animation");
 	uiBlockEndAlign(block);
 }
 
@@ -1351,110 +1404,111 @@ static void set_ffmpeg_preset(int preset)
 
 static void render_panel_ffmpeg_video(void)
 {
-       int yofs;
-       int xcol1;
-       int xcol2;
-       uiBlock *block;
-       block = uiNewBlock(&curarea->uiblocks, "render_panel_ffmpeg_video", 
-			  UI_EMBOSS, UI_HELV, curarea->win);
-	   uiNewPanelTabbed("Format", "Render");
-       if (uiNewPanel(curarea, block, "Video", "Render", 960, 0, 318, 204) 
-	   == 0) return;
-
-       if (ffmpeg_preset_sel != 0) {
-	       set_ffmpeg_preset(ffmpeg_preset_sel);
-	       ffmpeg_preset_sel = 0;
-	       allqueue(REDRAWBUTSSCENE, 0);
-       }
-
-       xcol1 = 872;
-       xcol2 = 1002;
-
-       yofs = 54;
-       uiDefBut(block, LABEL, 0, "Format", xcol1, yofs+88, 
-		110, 20, 0, 0, 0, 0, 0, "");
-       uiDefBut(block, LABEL, 0, "Preset", xcol2, yofs+88, 
-		110, 20, 0, 0, 0, 0, 0, "");
-       uiDefButI(block, MENU, B_DIFF, ffmpeg_format_pup(), 
-		 xcol1, yofs+66, 110, 20, &G.scene->r.ffcodecdata.type, 
-		 0,0,0,0, "output file format");
-       uiDefButI(block, NUM, B_DIFF, "Bitrate", 
-		 xcol1, yofs+44, 110, 20, 
-		 &G.scene->r.ffcodecdata.video_bitrate, 
-		 1, 14000, 0, 0, "Video bitrate(kb/s)");
-       uiDefButI(block, NUM, B_DIFF, "Min Rate", 
-		 xcol1, yofs+22, 110, 20, &G.scene->r.ffcodecdata.rc_min_rate, 
-		 0, 14000, 0, 0, "Rate control: min rate(kb/s)");
-       uiDefButI(block, NUM, B_DIFF, "Max Rate", 
-		 xcol1, yofs, 110, 20, &G.scene->r.ffcodecdata.rc_max_rate, 
-		 1, 14000, 0, 0, "Rate control: max rate(kb/s)");
-
-       uiDefButI(block, NUM, B_DIFF, "Mux Rate", 
-		 xcol1, yofs-22, 110, 20, 
-		 &G.scene->r.ffcodecdata.mux_rate, 
-		 0, 100000000, 0, 0, "Mux rate (bits/s(!))");
-
-
-       uiDefButI(block, MENU, B_REDR, ffmpeg_preset_pup(), 
-		 xcol2, yofs+66, 110, 20, &ffmpeg_preset_sel, 
-		 0,0,0,0, "Output file format preset selection");
-       uiDefButI(block, NUM, B_DIFF, "GOP Size", 
-		 xcol2, yofs+44, 110, 20, &G.scene->r.ffcodecdata.gop_size, 
-		 0, 100, 0, 0, "Distance between key frames");
-       uiDefButI(block, NUM, B_DIFF, "Buffersize", 
-		 xcol2, yofs+22, 110, 20,
-		 &G.scene->r.ffcodecdata.rc_buffer_size, 
-		 0, 2000, 0, 0, "Rate control: buffer size (kb)");
-       uiDefButI(block, NUM, B_DIFF, "Mux PSize", 
-		 xcol2, yofs, 110, 20, 
-		 &G.scene->r.ffcodecdata.mux_packet_size, 
-		 0, 16384, 0, 0, "Mux packet size (byte)");
-
-       uiDefButBitI(block, TOG, FFMPEG_AUTOSPLIT_OUTPUT, B_NOP,
-		    "Autosplit Output", 
-		    xcol2, yofs-22, 110, 20, 
-		    &G.scene->r.ffcodecdata.flags, 
-		    0, 1, 0,0, "Autosplit output at 2GB boundary.");
-
-
-       if (G.scene->r.ffcodecdata.type == FFMPEG_AVI 
-	   || G.scene->r.ffcodecdata.type == FFMPEG_MOV) {
-               uiDefBut(block, LABEL, 0, "Codec", 
-			xcol1, yofs-44, 110, 20, 0, 0, 0, 0, 0, "");
-               uiDefButI(block, MENU,B_NOP, ffmpeg_codec_pup(), 
-			 xcol1, yofs-66, 110, 20, 
-			 &G.scene->r.ffcodecdata.codec, 
-			 0,0,0,0, "FFMpeg codec to use");
-       }
-
-
+	uiBlock *block;
+	int yofs;
+	int xcol1;
+	int xcol2;
+	
+	block = uiNewBlock(&curarea->uiblocks, "render_panel_ffmpeg_video", 
+					   UI_EMBOSS, UI_HELV, curarea->win);
+	
+	uiNewPanelTabbed("Format", "Render");
+	if (uiNewPanel(curarea, block, "Video", "Render", 960, 0, 318, 204)== 0) 
+		return;
+	
+	if (ffmpeg_preset_sel != 0) {
+		set_ffmpeg_preset(ffmpeg_preset_sel);
+		ffmpeg_preset_sel = 0;
+		allqueue(REDRAWBUTSSCENE, 0);
+	}
+	
+	xcol1 = 872;
+	xcol2 = 1002;
+	
+	yofs = 54;
+	uiDefBut(block, LABEL, B_DIFF, "Format", xcol1, yofs+88, 
+			 110, 20, 0, 0, 0, 0, 0, "");
+	uiDefBut(block, LABEL, B_DIFF, "Preset", xcol2, yofs+88, 
+			 110, 20, 0, 0, 0, 0, 0, "");
+	uiDefButI(block, MENU, B_DIFF, ffmpeg_format_pup(), 
+			  xcol1, yofs+66, 110, 20, &G.scene->r.ffcodecdata.type, 
+			  0,0,0,0, "output file format");
+	uiDefButI(block, NUM, B_DIFF, "Bitrate", 
+			  xcol1, yofs+44, 110, 20, 
+			  &G.scene->r.ffcodecdata.video_bitrate, 
+			  1, 14000, 0, 0, "Video bitrate(kb/s)");
+	uiDefButI(block, NUM, B_DIFF, "Min Rate", 
+			  xcol1, yofs+22, 110, 20, &G.scene->r.ffcodecdata.rc_min_rate, 
+			  0, 14000, 0, 0, "Rate control: min rate(kb/s)");
+	uiDefButI(block, NUM, B_DIFF, "Max Rate", 
+			  xcol1, yofs, 110, 20, &G.scene->r.ffcodecdata.rc_max_rate, 
+			  1, 14000, 0, 0, "Rate control: max rate(kb/s)");
+	
+	uiDefButI(block, NUM, B_DIFF, "Mux Rate", 
+			  xcol1, yofs-22, 110, 20, 
+			  &G.scene->r.ffcodecdata.mux_rate, 
+			  0, 100000000, 0, 0, "Mux rate (bits/s(!))");
+	
+	
+	uiDefButI(block, MENU, B_REDR, ffmpeg_preset_pup(), 
+			  xcol2, yofs+66, 110, 20, &ffmpeg_preset_sel, 
+			  0,0,0,0, "Output file format preset selection");
+	uiDefButI(block, NUM, B_DIFF, "GOP Size", 
+			  xcol2, yofs+44, 110, 20, &G.scene->r.ffcodecdata.gop_size, 
+			  0, 100, 0, 0, "Distance between key frames");
+	uiDefButI(block, NUM, B_DIFF, "Buffersize", 
+			  xcol2, yofs+22, 110, 20,
+			  &G.scene->r.ffcodecdata.rc_buffer_size, 
+			  0, 2000, 0, 0, "Rate control: buffer size (kb)");
+	uiDefButI(block, NUM, B_DIFF, "Mux PSize", 
+			  xcol2, yofs, 110, 20, 
+			  &G.scene->r.ffcodecdata.mux_packet_size, 
+			  0, 16384, 0, 0, "Mux packet size (byte)");
+	
+	uiDefButBitI(block, TOG, FFMPEG_AUTOSPLIT_OUTPUT, B_NOP,
+				 "Autosplit Output", 
+				 xcol2, yofs-22, 110, 20, 
+				 &G.scene->r.ffcodecdata.flags, 
+				 0, 1, 0,0, "Autosplit output at 2GB boundary.");
+	
+	
+	if (ELEM(G.scene->r.ffcodecdata.type, FFMPEG_AVI, FFMPEG_MOV)) {
+		uiDefBut(block, LABEL, 0, "Codec", 
+				xcol1, yofs-44, 110, 20, 0, 0, 0, 0, 0, "");
+		uiDefButI(block, MENU,B_REDR, ffmpeg_codec_pup(), 
+				  xcol1, yofs-66, 110, 20, 
+				  &G.scene->r.ffcodecdata.codec, 
+				  0,0,0,0, "FFMpeg codec to use");
+	}
 }
+
 static void render_panel_ffmpeg_audio(void)
 {
-       int yofs;
-       int xcol;
-       uiBlock *block;
-       block = uiNewBlock(&curarea->uiblocks, "render_panel_ffmpeg_audio", UI_EMBOSS, UI_HELV, curarea->win);
-	   uiNewPanelTabbed("Format", "Render");
-       if (uiNewPanel(curarea, block, "Audio", "Render", 960, 0, 318, 204) 
-	   == 0) return;
-       yofs = 54;
-       xcol = 892;
-
-       uiDefButBitI(block, TOG, FFMPEG_MULTIPLEX_AUDIO, B_NOP,
-		    "Multiplex audio", xcol, yofs, 225, 20, 
-		    &G.scene->r.ffcodecdata.flags, 
-		    0, 1, 0,0, "Interleave audio with the output video");
-       uiDefBut(block, LABEL, 0, "Codec", 
-		xcol, yofs-22, 225, 20, 0, 0, 0, 0, 0, "");
-       uiDefButI(block, MENU,B_NOP, ffmpeg_audio_codec_pup(), 
-		 xcol, yofs-44, 225, 20, 
-		 &G.scene->r.ffcodecdata.audio_codec, 
-		 0,0,0,0, "FFMpeg codec to use");
-       uiDefButI(block, NUM, B_DIFF, "Bitrate", 
-		 xcol, yofs-66, 110, 20, 
-		 &G.scene->r.ffcodecdata.audio_bitrate, 
-		 32, 384, 0, 0, "Audio bitrate(kb/s)");
+	uiBlock *block;
+	int yofs;
+	int xcol;
+	
+	block = uiNewBlock(&curarea->uiblocks, "render_panel_ffmpeg_audio", UI_EMBOSS, UI_HELV, curarea->win);
+	uiNewPanelTabbed("Format", "Render");
+	if (uiNewPanel(curarea, block, "Audio", "Render", 960, 0, 318, 204) == 0) return;
+	
+	yofs = 54;
+	xcol = 892;
+	
+	uiDefButBitI(block, TOG, FFMPEG_MULTIPLEX_AUDIO, B_NOP,
+				 "Multiplex audio", xcol, yofs, 225, 20, 
+				 &G.scene->r.ffcodecdata.flags, 
+				 0, 1, 0,0, "Interleave audio with the output video");
+	uiDefBut(block, LABEL, 0, "Codec", 
+			 xcol, yofs-22, 225, 20, 0, 0, 0, 0, 0, "");
+	uiDefButI(block, MENU,B_NOP, ffmpeg_audio_codec_pup(), 
+			  xcol, yofs-44, 225, 20, 
+			  &G.scene->r.ffcodecdata.audio_codec, 
+			  0,0,0,0, "FFMpeg codec to use");
+	uiDefButI(block, NUM, B_DIFF, "Bitrate", 
+			  xcol, yofs-66, 110, 20, 
+			  &G.scene->r.ffcodecdata.audio_bitrate, 
+			  32, 384, 0, 0, "Audio bitrate(kb/s)");
 }
 #endif
 
@@ -1528,15 +1582,16 @@ static void render_panel_format(void)
 		}
 #ifdef WITH_OPENEXR
 	} 
-	else if (G.scene->r.imtype == R_OPENEXR ) {
-		if (G.scene->r.quality > 5) G.scene->r.quality = 0;
-		
-		uiBlockBeginAlign(block);
-		uiDefButBitS(block, TOG, R_OPENEXR_HALF, B_NOP,"Half",	892,yofs+44,60,20, &G.scene->r.subimtype, 0, 0, 0, 0, "Use 16 bits float 'Half' type");
-		uiDefButBitS(block, TOG, R_OPENEXR_ZBUF, B_NOP,"Zbuf",	952,yofs+44,60,20, &G.scene->r.subimtype, 0, 0, 0, 0, "Save the zbuffer as 32 bits unsigned int");
-		uiBlockEndAlign(block);
-		uiDefButBitS(block, TOG, R_PREVIEW_JPG, B_NOP,"Preview",1027,yofs+44,90,20, &G.scene->r.subimtype, 0, 0, 0, 0, "When animation render, save JPG preview images in same directory");
+	else if (ELEM(G.scene->r.imtype, R_OPENEXR, R_MULTILAYER)) {
+		if (G.scene->r.quality > 5) G.scene->r.quality = 2;
 		
+		if(G.scene->r.imtype==R_OPENEXR) {
+			uiBlockBeginAlign(block);
+			uiDefButBitS(block, TOG, R_OPENEXR_HALF, B_NOP,"Half",	892,yofs+44,60,20, &G.scene->r.subimtype, 0, 0, 0, 0, "Use 16 bits float 'Half' type");
+			uiDefButBitS(block, TOG, R_OPENEXR_ZBUF, B_NOP,"Zbuf",	952,yofs+44,60,20, &G.scene->r.subimtype, 0, 0, 0, 0, "Save the zbuffer as 32 bits unsigned int");
+			uiBlockEndAlign(block);
+			uiDefButBitS(block, TOG, R_PREVIEW_JPG, B_NOP,"Preview",1027,yofs+44,90,20, &G.scene->r.subimtype, 0, 0, 0, 0, "When animation render, save JPG preview images in same directory");
+		}		
 		uiDefButS(block, MENU,B_NOP, "Codec %t|None %x0|Pxr24 (lossy) %x1|ZIP (lossless) %x2|PIZ (lossless) %x3|RLE (lossless) %x4",  
 															892,yofs,112,20, &G.scene->r.quality, 0, 0, 0, 0, "Set codec settings for OpenEXR");
 		
@@ -1725,6 +1780,7 @@ static void rename_scene_layer_func(void *srl_v, void *unused_v)
 		}
 	}
 	allqueue(REDRAWBUTSSCENE, 0);
+	allqueue(REDRAWOOPS, 0);
 	allqueue(REDRAWNODE, 0);
 }
 
@@ -1738,7 +1794,10 @@ static char *scene_layer_menu(void)
 	strcpy(str, "ADD NEW %x32767");
 	a= strlen(str);
 	for(nr=0, srl= G.scene->r.layers.first; srl; srl= srl->next, nr++) {
-		a+= sprintf(str+a, "|%s %%x%d", srl->name, nr);
+		if(srl->layflag & SCE_LAY_DISABLE)
+			a+= sprintf(str+a, "|%s %%i%d %%x%d", srl->name, ICON_BLANK1, nr);
+		else 
+			a+= sprintf(str+a, "|%s %%i%d %%x%d", srl->name, ICON_CHECKBOX_HLT, nr);
 	}
 	
 	return str;
@@ -1791,67 +1850,67 @@ static void render_panel_layers(void)
 	
 	/* first, as reminder, the scene layers */
 	uiDefBut(block, LABEL, 0, "Scene:",				10,170,100,20, NULL, 0, 0, 0, 0, "");
+	draw_3d_layer_buttons(block, &G.scene->lay,		130, 170, 35, 30);
 	
-	draw_3d_layer_buttons(block, &G.scene->lay, 130, 170, 35, 30);
-	
-	/* layer menu, name, delete button */
+	/* layer disable, menu, name, delete button */
 	uiBlockBeginAlign(block);
+	uiDefIconButBitI(block, ICONTOGN, SCE_LAY_DISABLE, B_REDR, ICON_CHECKBOX_HLT-1,	10, 145, 20, 20, &srl->layflag, 0.0, 0.0, 0, 0, "Disable or enable this RenderLayer");
 	strp= scene_layer_menu();
-	uiDefButS(block, MENU, B_ADD_RENDERLAYER, strp, 10,130,23,20, &(G.scene->r.actlay), 0, 0, 0, 0, "Choose Active Render Layer");
+	uiDefButS(block, MENU, B_ADD_RENDERLAYER, strp, 30,145,23,20, &(G.scene->r.actlay), 0, 0, 0, 0, "Choose Active Render Layer");
 	MEM_freeN(strp);
 	
-	bt= uiDefBut(block, TEX, REDRAWNODE, "",  33,130,192,20, srl->name, 0.0, 31.0, 0, 0, "");
+	/* name max 20, exr format limit... */
+	bt= uiDefBut(block, TEX, REDRAWNODE, "",  53,145,172,20, srl->name, 0.0, 20.0, 0, 0, "");
 	uiButSetFunc(bt, rename_scene_layer_func, srl, NULL);
 	
-	uiDefButBitS(block, TOG, R_SINGLE_LAYER, B_NOP, "Single",	230,130,60,20, &G.scene->r.scemode, 0, 0, 0, 0, "Only render this layer");	
-	bt=uiDefIconBut(block, BUT, B_NOP, ICON_X,	285, 130, 25, 20, 0, 0, 0, 0, 0, "Deletes current Render Layer");
+	uiDefButBitS(block, TOG, R_SINGLE_LAYER, B_NOP, "Single",	230,145,60,20, &G.scene->r.scemode, 0, 0, 0, 0, "Only render this layer");	
+	bt=uiDefIconBut(block, BUT, B_NOP, ICON_X,	285, 145, 25, 20, 0, 0, 0, 0, 0, "Deletes current Render Layer");
 	uiButSetFunc(bt, delete_scene_layer_func, srl, (void *)(long)G.scene->r.actlay);
 	uiBlockEndAlign(block);
 
 	/* RenderLayer visible-layers */
-	uiDefBut(block, LABEL, 0, "Layer:",	10,95,100,20, NULL, 0, 0, 0, 0, "");
-	draw_3d_layer_buttons(block, &srl->lay,		130, 95, 35, 30);
+	uiDefBut(block, LABEL, 0, "Layer:",			10,110,100,20, NULL, 0, 0, 0, 0, "");
+	draw_3d_layer_buttons(block, &srl->lay,		130,110, 35, 30);
 	
 	uiBlockBeginAlign(block);
-	uiDefButBitS(block, TOG, SCE_LAY_SKY, B_NOP,"Sky",	10, 70, 40, 20, &srl->layflag, 0, 0, 0, 0, "Render Sky or backbuffer in this Layer");	
-	uiDefButBitS(block, TOG, SCE_LAY_SOLID, B_NOP,"Solid",	50, 70, 65, 20, &srl->layflag, 0, 0, 0, 0, "Render Solid faces in this Layer");	
-	uiDefButBitS(block, TOG, SCE_LAY_HALO, B_NOP,"Halo",	115, 70, 65, 20, &srl->layflag, 0, 0, 0, 0, "Render Halos in this Layer (on top of Solid)");	
-	uiDefButBitS(block, TOG, SCE_LAY_ZTRA, B_NOP,"Ztra",	180, 70, 65, 20, &srl->layflag, 0, 0, 0, 0, "Render Z-Transparent faces in this Layer (On top of Solid and Halos)");	
-	uiDefButBitS(block, TOG, SCE_LAY_EDGE, B_NOP,"Edge",	245, 70, 65, 20, &srl->layflag, 0, 0, 0, 0, "Render Edge-enhance in this Layer (only works for Solid faces)");	
+	uiDefButBitI(block, TOG, SCE_LAY_ALL_Z, B_NOP,"AllZ",	10, 85, 40, 20, &srl->layflag, 0, 0, 0, 0, "Fill in Z values for all not-rendered faces, for masking");	
+	uiBlockBeginAlign(block);
+	uiDefButBitI(block, TOG, SCE_LAY_SOLID, B_NOP,"Solid",	50, 85, 60, 20, &srl->layflag, 0, 0, 0, 0, "Render Solid faces in this Layer");	
+	uiDefButBitI(block, TOG, SCE_LAY_HALO, B_NOP,"Halo",	110, 85, 55, 20, &srl->layflag, 0, 0, 0, 0, "Render Halos in this Layer (on top of Solid)");	
+	uiDefButBitI(block, TOG, SCE_LAY_ZTRA, B_NOP,"Ztra",	165, 85, 55, 20, &srl->layflag, 0, 0, 0, 0, "Render Z-Transparent faces in this Layer (On top of Solid and Halos)");	
+	uiDefButBitI(block, TOG, SCE_LAY_SKY, B_NOP,"Sky",		220, 85, 40, 20, &srl->layflag, 0, 0, 0, 0, "Render Sky or backbuffer in this Layer");	
+	uiDefButBitI(block, TOG, SCE_LAY_EDGE, B_NOP,"Edge",	260, 85, 50, 20, &srl->layflag, 0, 0, 0, 0, "Render Edge-enhance in this Layer (only works for Solid faces)");	
+	
+	uiDefIDPoinBut(block, test_grouppoin_but, ID_GR, B_SET_PASS, "Light:",	10, 65, 150, 20, &(srl->light_override), "Name of Group to use as Lamps instead");
+	uiDefIDPoinBut(block, test_matpoin_but, ID_MA, B_SET_PASS, "Mat:",	160, 65, 150, 20, &(srl->mat_override), "Name of Material to use as Materials instead");
 	uiBlockEndAlign(block);
 
-	uiDefBut(block, LABEL, 0, "Passes:",					10,30,150,20, NULL, 0, 0, 0, 0, "");
-	
 	uiBlockBeginAlign(block);
-	uiDefButBitS(block, TOG, SCE_PASS_COMBINED, B_NOP,"Combined",	10, 10, 150, 20, &srl->passflag, 0, 0, 0, 0, "Deliver full combined RGBA buffer");	
-	uiDefButBitS(block, TOG, SCE_PASS_Z, B_NOP,"Z",			160, 10, 40, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Z values pass");	
-	uiDefButBitS(block, TOG, SCE_PASS_VECTOR, B_NOP,"Vec",	200, 10, 55, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Vector pass");	
-	uiDefButBitS(block, TOG, SCE_PASS_NORMAL, B_NOP,"Nor",	255, 10, 55, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Normal pass");	
-#if 0
-	/* bring back after release */
-	uiBlockBeginAlign(block);
-	uiDefButBitS(block, TOG, SCE_PASS_COMBINED, B_NOP,"Combined",	130, 30, 115, 20, &srl->passflag, 0, 0, 0, 0, "Deliver full combined RGBA buffer");	
-	uiDefButBitS(block, TOG, SCE_PASS_Z, B_NOP,"Z",			245, 30, 25, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Z values pass");	
-	uiDefButBitS(block, TOG, SCE_PASS_VECTOR, B_NOP,"Vec",	270, 30, 40, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Vector pass");	
+	uiDefButBitI(block, TOG, SCE_PASS_COMBINED, B_SET_PASS,"Combined",	10, 30, 80, 20, &srl->passflag, 0, 0, 0, 0, "Deliver full combined RGBA buffer");	
+	uiDefButBitI(block, TOG, SCE_PASS_Z, B_SET_PASS,"Z",			90, 30, 30, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Z values pass");	
+	uiDefButBitI(block, TOG, SCE_PASS_VECTOR, B_SET_PASS,"Vec",		120, 30, 40, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Speed Vector pass");	
+	uiDefButBitI(block, TOG, SCE_PASS_NORMAL, B_SET_PASS,"Nor",		160, 30, 40, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Normal pass");	
+	uiDefButBitI(block, TOG, SCE_PASS_UV, B_SET_PASS,"UV",			200, 30, 40, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Texture UV pass");	
+	uiDefButBitI(block, TOG, SCE_PASS_INDEXOB, B_SET_PASS,"IndexOb",240, 30, 70, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Object Index pass");	
 	
-	uiDefButBitS(block, TOG, SCE_PASS_RGBA, B_NOP,"Col",	10, 10, 45, 20, &srl->passflag, 0, 0, 0, 0, "Deliver shade-less Color pass");	
-	uiDefButBitS(block, TOG, SCE_PASS_DIFFUSE, B_NOP,"Diff",55, 10, 45, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Diffuse pass");	
-	uiDefButBitS(block, TOG, SCE_PASS_SPEC, B_NOP,"Spec",	100, 10, 45, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Specular pass");	
-	uiDefButBitS(block, TOG, SCE_PASS_SHADOW, B_NOP,"Shad",	145, 10, 45, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Shadow pass");	
-	uiDefButBitS(block, TOG, SCE_PASS_AO, B_NOP,"AO",		185, 10, 40, 20, &srl->passflag, 0, 0, 0, 0, "Deliver AO pass");	
-	uiDefButBitS(block, TOG, SCE_PASS_RAY, B_NOP,"Ray",	225, 10, 45, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Raytraced Mirror and Transparent pass");	
-	uiDefButBitS(block, TOG, SCE_PASS_NORMAL, B_NOP,"Nor",	270, 10, 40, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Normal pass");	
-#endif
+	uiDefButBitI(block, TOG, SCE_PASS_RGBA, B_SET_PASS,"Col",				10, 10, 35, 20, &srl->passflag, 0, 0, 0, 0, "Deliver shade-less Color pass");	
+	uiDefButBitI(block, TOG, SCE_PASS_DIFFUSE, B_SET_PASS,"Diff",			45, 10, 35, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Diffuse pass");	
+	uiDefButBitI(block, BUT_TOGDUAL, SCE_PASS_SPEC, B_SET_PASS,"Spec",		80, 10, 40, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Specular pass");	
+	uiDefButBitI(block, BUT_TOGDUAL, SCE_PASS_SHADOW, B_SET_PASS,"Shad",	120, 10, 40, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Shadow pass");	
+	uiDefButBitI(block, BUT_TOGDUAL, SCE_PASS_AO, B_SET_PASS,"AO",			160, 10, 30, 20, &srl->passflag, 0, 0, 0, 0, "Deliver AO pass");	
+	uiDefButBitI(block, BUT_TOGDUAL, SCE_PASS_REFLECT, B_SET_PASS,"Refl",	190, 10, 40, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Raytraced Reflection pass");	
+	uiDefButBitI(block, BUT_TOGDUAL, SCE_PASS_REFRACT, B_SET_PASS,"Refr",	230, 10, 40, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Raytraced Refraction pass");	
+	uiDefButBitI(block, BUT_TOGDUAL, SCE_PASS_RADIO, B_SET_PASS,"Rad",		270, 10, 40, 20, &srl->passflag, 0, 0, 0, 0, "Deliver Radiosity pass");	
 }	
 
 void render_panels()
 {
 
 	render_panel_output();
-//	render_panel_sfx();
 	render_panel_layers();
 	render_panel_render();
 	render_panel_anim();
+	render_panel_bake();
 
 	render_panel_format();
 #ifdef WITH_FFMPEG
@@ -1861,6 +1920,7 @@ void render_panels()
        }
 #endif
 
+#ifndef DISABLE_YAFRAY
 	/* yafray: GI & Global panel, only available when yafray enabled for rendering */
 	if (G.scene->r.renderer==R_YAFRAY) {
 		if (G.scene->r.YF_gamma==0.0) G.scene->r.YF_gamma=1.0;
@@ -1873,6 +1933,7 @@ void render_panels()
 		render_panel_yafrayGlobal();
 		render_panel_yafrayGI();
 	}
+#endif
 
 }
 
