@@ -1,5 +1,5 @@
 /**
- * $Id: drawaction.c,v 1.21 2005/05/25 16:37:24 ton Exp $
+ * $Id: drawaction.c,v 1.48 2007/02/11 04:02:17 aligorith Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -48,7 +48,9 @@
 #include "BLI_arithb.h"
 
 /* Types --------------------------------------------------------------- */
+#include "DNA_listBase.h"
 #include "DNA_action_types.h"
+#include "DNA_armature_types.h"
 #include "DNA_curve_types.h"
 #include "DNA_ipo_types.h"
 #include "DNA_object_types.h"
@@ -59,57 +61,37 @@
 #include "DNA_key_types.h"
 
 #include "BKE_action.h"
+#include "BKE_ipo.h"
 #include "BKE_global.h"
 
 /* Everything from source (BIF, BDR, BSE) ------------------------------ */ 
 
+#include "BIF_editaction.h"
+#include "BIF_editkey.h"
+#include "BIF_interface.h"
+#include "BIF_interface_icons.h"
 #include "BIF_gl.h"
 #include "BIF_glutil.h"
 #include "BIF_resources.h"
 #include "BIF_screen.h"
-#include "BIF_interface.h"
 #include "BIF_mywindow.h"
 #include "BIF_space.h"
 
-#include "BDR_editcurve.h"
-#include "BSE_view.h"
-#include "BSE_drawipo.h"
-#include "BSE_editaction.h"
-#include "BSE_editaction_types.h"
 #include "BDR_drawaction.h"
+#include "BDR_editcurve.h"
+
+#include "BSE_drawnla.h"
+#include "BSE_drawipo.h"
+#include "BSE_time.h"
+#include "BSE_view.h"
 
 /* 'old' stuff": defines and types, and own include -------------------- */
 
 #include "blendef.h"
 #include "mydevice.h"
 
-#include "BKE_ipo.h"
 
-/* local functions ----------------------------------------------------- */
-void drawactionspace(ScrArea *sa, void *spacedata);
-static void draw_channel_names(void);
-static void draw_channel_strips(SpaceAction *saction);
-int count_action_levels(bAction *act);
-static BezTriple **ipo_to_keylist(Ipo *ipo, int flags, int *totvert);
-static BezTriple **action_to_keylist(bAction *act, int flags, int *totvert);
-static BezTriple **ob_to_keylist(Object *ob, int flags, int *totvert);
-static BezTriple **icu_to_keylist(IpoCurve *icu, int flags, int *totvert);
-static void draw_keylist(gla2DDrawInfo *di, int totvert, BezTriple **blist, float ypos);
-void draw_icu_channel(gla2DDrawInfo *di, IpoCurve *icu, int flags, float ypos);
-static void draw_action_mesh_names(Key *key);
-
-/* missing local prototypes -------------------------------------------- */
-void meshactionbuts(SpaceAction *saction, Key *key);
-void do_actionbuts(unsigned short event);
-
-/* implementation ------------------------------------------------------ */
-
-extern void make_rvk_slider(uiBlock *block, Key *key, int i,
-							int x, int y, int w, int h); /* editkey.c */
-extern short showsliders; /* editaction .c */
-extern short ACTWIDTH;
-
-void meshactionbuts(SpaceAction *saction, Key *key)
+static void meshactionbuts(SpaceAction *saction, Object *ob, Key *key)
 {
 	int           i;
 	char          str[64];
@@ -146,24 +128,24 @@ void meshactionbuts(SpaceAction *saction, Key *key)
 
 	uiBlockSetEmboss(block, UI_EMBOSSN);
 
-	if (!showsliders) {
+	if (!(G.saction->flag & SACTION_SLIDERS)) {
 		ACTWIDTH = NAMEWIDTH;
-		but=uiDefIconButS(block, TOG, B_REDR, 
+		but=uiDefIconButBitS(block, TOG, SACTION_SLIDERS, B_REDR, 
 					  ICON_DISCLOSURE_TRI_RIGHT,
 					  NAMEWIDTH - XIC - 5, y + CHANNELHEIGHT,
 					  XIC,YIC-2,
-					  &(showsliders), 0, 0, 0, 0, 
+					  &(G.saction->flag), 0, 0, 0, 0, 
 					  "Show action window sliders");
 		// no hilite, the winmatrix is not correct later on...
 		uiButSetFlag(but, UI_NO_HILITE);
 
 	}
 	else {
-		but= uiDefIconButS(block, TOG, B_REDR, 
+		but= uiDefIconButBitS(block, TOG, SACTION_SLIDERS, B_REDR, 
 					  ICON_DISCLOSURE_TRI_DOWN,
 					  NAMEWIDTH - XIC - 5, y + CHANNELHEIGHT,
 					  XIC,YIC-2,
-					  &(showsliders), 0, 0, 0, 0, 
+					  &(G.saction->flag), 0, 0, 0, 0, 
 					  "Hide action window sliders");
 		// no hilite, the winmatrix is not correct later on...
 		uiButSetFlag(but, UI_NO_HILITE);
@@ -176,11 +158,13 @@ void meshactionbuts(SpaceAction *saction, Key *key)
 		glRects(NAMEWIDTH,  0,  NAMEWIDTH+SLIDERWIDTH,  curarea->winy);
 		uiBlockSetEmboss(block, UI_EMBOSS);
 		for (i=1 ; i < key->totkey ; ++ i) {
-			make_rvk_slider(block, key, i, 
-							x, y, SLIDERWIDTH-2, CHANNELHEIGHT-1);
+			make_rvk_slider(block, ob, i, 
+							x, y, SLIDERWIDTH-2, CHANNELHEIGHT-1, "Slider to control Shape Keys");
 
 			y-=CHANNELHEIGHT+CHANNELSKIP;
 			
+			/* see sliderval array in editkey.c */
+			if(i>=255) break;
 		}
 	}
 	uiDrawBlock(block);
@@ -222,6 +206,7 @@ void draw_cfra_action(void)
 }
 
 
+/* left hand */
 static void draw_action_channel_names(bAction	*act) 
 {
     bActionChannel *chan;
@@ -232,28 +217,48 @@ static void draw_action_channel_names(bAction	*act)
 	y= count_action_levels(act)*(CHANNELHEIGHT+CHANNELSKIP);
 
 	for (chan=act->chanbase.first; chan; chan=chan->next){
-		BIF_ThemeColorShade(TH_HEADER, 20);
-		glRectf(x,  y-CHANNELHEIGHT/2,  (float)NAMEWIDTH,  y+CHANNELHEIGHT/2);
-	
-		if (chan->flag & ACHAN_SELECTED)
-			BIF_ThemeColor(TH_TEXT_HI);
-		else
-			BIF_ThemeColor(TH_TEXT);
-		glRasterPos2f(x+8,  y-4);
-		BMF_DrawString(G.font, chan->name);
-		y-=CHANNELHEIGHT+CHANNELSKIP;
-	
-		/* Draw constraint channels */
-		for (conchan=chan->constraintChannels.first; 
-				conchan; conchan=conchan->next){
-			if (conchan->flag & CONSTRAINT_CHANNEL_SELECT)
+		if((chan->flag & ACHAN_HIDDEN)==0) {
+			glEnable(GL_BLEND);
+			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA) ;
+			
+			/* draw backing strip behind action channel name */
+			BIF_ThemeColorShade(TH_HEADER, 20);
+			glRectf(x,  y-CHANNELHEIGHT/2,  (float)NAMEWIDTH,  y+CHANNELHEIGHT/2);
+			
+			/* draw 'lock' indicating whether channel is protected */
+			if (chan->flag & ACHAN_PROTECTED) 
+				BIF_icon_draw(NAMEWIDTH-16, y-CHANNELHEIGHT/2, ICON_LOCKED);
+			else 
+				BIF_icon_draw(NAMEWIDTH-16, y-CHANNELHEIGHT/2, ICON_UNLOCKED);
+			
+			/* draw name of action channel */
+			if (chan->flag & ACHAN_SELECTED)
 				BIF_ThemeColor(TH_TEXT_HI);
 			else
 				BIF_ThemeColor(TH_TEXT);
-				
-			glRasterPos2f(x+32,  y-4);
-			BMF_DrawString(G.font, conchan->name);
+			glRasterPos2f(x+8,  y-4);
+			BMF_DrawString(G.font, chan->name);
 			y-=CHANNELHEIGHT+CHANNELSKIP;
+		
+			/* Draw constraint channels */
+			for (conchan=chan->constraintChannels.first; conchan; conchan=conchan->next) {
+				/* draw 'lock' to indicate if constraint channel is protected */
+				if (conchan->flag & CONSTRAINT_CHANNEL_PROTECTED) 
+					BIF_icon_draw(NAMEWIDTH-16, y-CHANNELHEIGHT/2, ICON_LOCKED);
+				else 
+					BIF_icon_draw(NAMEWIDTH-16, y-CHANNELHEIGHT/2, ICON_UNLOCKED);	
+				
+				/* draw name of constraint channel */
+				if (conchan->flag & CONSTRAINT_CHANNEL_SELECT)
+					BIF_ThemeColor(TH_TEXT_HI);
+				else
+					BIF_ThemeColor(TH_TEXT);
+				glRasterPos2f(x+32,  y-4);
+				BMF_DrawString(G.font, conchan->name);
+				y-=CHANNELHEIGHT+CHANNELSKIP;
+			}
+			
+			glDisable(GL_BLEND);
 		}
 	}
 }
@@ -302,6 +307,7 @@ static void draw_action_mesh_names(Key *key)
 	}
 }
 
+/* left hand part */
 static void draw_channel_names(void) 
 {
 	short ofsx, ofsy = 0; 
@@ -355,46 +361,37 @@ int count_action_levels(bAction *act)
 		return 0;
 
 	for (achan=act->chanbase.first; achan; achan=achan->next){
-		y+=1;
-		y+=BLI_countlist(&achan->constraintChannels);
+		if((achan->flag & ACHAN_HIDDEN)==0) {
+			y+=1;
+			y+=BLI_countlist(&achan->constraintChannels);
+		}
 	}
 
 	return y;
 }
-	/** Draw a nicely beveled button (in screen space) */
-void draw_bevel_but(int x, int y, int w, int h, int sel)
+
+/* sets or clears hidden flags */
+void check_action_context(SpaceAction *saction)
 {
-	int xmin= x, ymin= y;
-	int xmax= x+w-1, ymax= y+h-1;
-	int i;
-
-	glColor3ub(0,0,0);
-	glBegin(GL_LINE_LOOP);
-	glVertex2i(xmin, ymin);
-	glVertex2i(xmax, ymin);
-	glVertex2i(xmax, ymax);
-	glVertex2i(xmin, ymax);
-	glEnd();
-
-	glBegin(GL_LINE_LOOP);
-	if (sel) glColor3ub(0xD0, 0x7E, 0x06);
-	else glColor3ub(0x8C, 0x8C, 0x8C);
-	glVertex2i(xmax-1, ymin+1);
-	glVertex2i(xmax-1, ymax-1);
-	if (sel) glColor3ub(0xF4, 0xEE, 0x8E);
-	else glColor3ub(0xDF, 0xDF, 0xDF);
-	glVertex2i(xmin+1, ymax-1);
-	glVertex2i(xmin+1, ymin+1);
-	glEnd();
-
-	if (sel) glColor3ub(0xF1, 0xCA, 0x13);
-	else glColor3ub(0xAC, 0xAC, 0xAC);
-	glBegin(GL_LINES);
-	for (i=xmin+2; i<=xmax-2; i++) {
-		glVertex2f(i, ymin+2);
-		glVertex2f(i, ymax-1);
+	bActionChannel *achan;
+	
+	if(saction->action==NULL) return;
+	
+	for (achan=saction->action->chanbase.first; achan; achan=achan->next)
+		achan->flag &= ~ACHAN_HIDDEN;
+	
+	if (G.saction->pin==0 && OBACT) {
+		Object *ob= OBACT;
+		bPoseChannel *pchan;
+		bArmature *arm= ob->data;
+		
+		for (achan=saction->action->chanbase.first; achan; achan=achan->next) {
+			pchan= get_pose_channel(ob->pose, achan->name);
+			if(pchan)
+				if((pchan->bone->layer & arm->layer)==0)
+					achan->flag |= ACHAN_HIDDEN;
+		}
 	}
-	glEnd();
 }
 
 static void draw_channel_strips(SpaceAction *saction)
@@ -404,7 +401,8 @@ static void draw_channel_strips(SpaceAction *saction)
 	bAction	*act;
 	bActionChannel *chan;
 	bConstraintChannel *conchan;
-	float	y;
+	float y, sta, end;
+	int act_start, act_end, dummy;
 	char col1[3], col2[3];
 	
 	BIF_GetThemeColor3ubv(TH_SHADE2, col2);
@@ -414,53 +412,91 @@ static void draw_channel_strips(SpaceAction *saction)
 	if (!act)
 		return;
 
-	scr_rct.xmin= saction->area->winrct.xmin + ACTWIDTH;
+	scr_rct.xmin= saction->area->winrct.xmin + saction->v2d.mask.xmin;
 	scr_rct.ymin= saction->area->winrct.ymin + saction->v2d.mask.ymin;
 	scr_rct.xmax= saction->area->winrct.xmin + saction->v2d.hor.xmax;
 	scr_rct.ymax= saction->area->winrct.ymin + saction->v2d.mask.ymax; 
 	di= glaBegin2DDraw(&scr_rct, &G.v2d->cur);
 
-	y= count_action_levels(act)*(CHANNELHEIGHT+CHANNELSKIP);
-
-	for (chan=act->chanbase.first; chan; chan=chan->next){
-		int frame1_x, channel_y;
-
-		gla2DDrawTranslatePt(di, 1, y, &frame1_x, &channel_y);
-
-		glEnable(GL_BLEND);
-		if (chan->flag & ACHAN_SELECTED) glColor4ub(col1[0], col1[1], col1[2], 0x22);
-		else glColor4ub(col2[0], col2[1], col2[2], 0x22);
-		glRectf(0,  channel_y-CHANNELHEIGHT/2,  frame1_x,  channel_y+CHANNELHEIGHT/2);
-
-		if (chan->flag & ACHAN_SELECTED) glColor4ub(col1[0], col1[1], col1[2], 0x44);
-		else glColor4ub(col2[0], col2[1], col2[2], 0x44);
-		glRectf(frame1_x,  channel_y-CHANNELHEIGHT/2,  G.v2d->hor.xmax,  channel_y+CHANNELHEIGHT/2);
-		glDisable(GL_BLEND);
+	/* if in NLA there's a strip active, map the view */
+	if (G.saction->pin==0 && OBACT)
+		map_active_strip(di, OBACT, 0);
 	
-		draw_ipo_channel(di, chan->ipo, 0, y);
-
-		/*	Increment the step */
-		y-=CHANNELHEIGHT+CHANNELSKIP;
-
-
-		/* Draw constraint channels */
-		for (conchan=chan->constraintChannels.first; conchan; conchan=conchan->next){
-			gla2DDrawTranslatePt(di, 1, y, &frame1_x, &channel_y);
-			glEnable(GL_BLEND);
-			if (conchan->flag & ACHAN_SELECTED) glColor4ub(col1[0], col1[1], col1[2], 0x22);
+	/* start and end of action itself */
+	calc_action_range(act, &sta, &end, 0);
+	gla2DDrawTranslatePt(di, sta, 0.0f, &act_start, &dummy);
+	gla2DDrawTranslatePt(di, end, 0.0f, &act_end, &dummy);
+	
+	if (G.saction->pin==0 && OBACT)
+		map_active_strip(di, OBACT, 1);
+	
+	/* first backdrop strips */
+	y= count_action_levels(act)*(CHANNELHEIGHT+CHANNELSKIP);
+	glEnable(GL_BLEND);
+	for (chan=act->chanbase.first; chan; chan=chan->next){
+		if((chan->flag & ACHAN_HIDDEN)==0) {
+			int frame1_x, channel_y;
+			
+			gla2DDrawTranslatePt(di, G.v2d->cur.xmin, y, &frame1_x, &channel_y);
+			
+			if (chan->flag & ACHAN_SELECTED) glColor4ub(col1[0], col1[1], col1[2], 0x22);
 			else glColor4ub(col2[0], col2[1], col2[2], 0x22);
-			glRectf(0,  channel_y-CHANNELHEIGHT/2+4,  frame1_x,  channel_y+CHANNELHEIGHT/2-4);
+			glRectf(frame1_x,  channel_y-CHANNELHEIGHT/2,  G.v2d->hor.xmax,  channel_y+CHANNELHEIGHT/2);
 			
-			if (conchan->flag & ACHAN_SELECTED) glColor4ub(col1[0], col1[1], col1[2], 0x44);
-			else glColor4ub(col2[0], col2[1], col2[2], 0x44);
-			glRectf(frame1_x,  channel_y-CHANNELHEIGHT/2+4,  G.v2d->hor.xmax,  channel_y+CHANNELHEIGHT/2-4);
-			glDisable(GL_BLEND);
+			if (chan->flag & ACHAN_SELECTED) glColor4ub(col1[0], col1[1], col1[2], 0x22);
+			else glColor4ub(col2[0], col2[1], col2[2], 0x22);
+			glRectf(act_start,  channel_y-CHANNELHEIGHT/2,  act_end,  channel_y+CHANNELHEIGHT/2);
 			
-			draw_ipo_channel(di, conchan->ipo, 0, y);
+			/*	Increment the step */
 			y-=CHANNELHEIGHT+CHANNELSKIP;
+			
+			/* Draw constraint channels */
+			for (conchan=chan->constraintChannels.first; conchan; conchan=conchan->next){
+				gla2DDrawTranslatePt(di, 1, y, &frame1_x, &channel_y);
+				
+				if (conchan->flag & ACHAN_SELECTED) glColor4ub(col1[0], col1[1], col1[2], 0x22);
+				else glColor4ub(col2[0], col2[1], col2[2], 0x22);
+				glRectf(frame1_x,  channel_y-CHANNELHEIGHT/2+4,  G.v2d->hor.xmax,  channel_y+CHANNELHEIGHT/2-4);
+				
+				if (conchan->flag & ACHAN_SELECTED) glColor4ub(col1[0], col1[1], col1[2], 0x22);
+				else glColor4ub(col2[0], col2[1], col2[2], 0x22);
+				glRectf(act_start,  channel_y-CHANNELHEIGHT/2+4,  act_end,  channel_y+CHANNELHEIGHT/2-4);
+				
+				y-=CHANNELHEIGHT+CHANNELSKIP;
+			}
+		}
+	}		
+	glDisable(GL_BLEND);
+	
+	if (G.saction->pin==0 && OBACT)
+		map_active_strip(di, OBACT, 0);
+	
+	/* dot thingies */
+	y= count_action_levels(act)*(CHANNELHEIGHT+CHANNELSKIP);
+	for (chan=act->chanbase.first; chan; chan=chan->next){
+		if((chan->flag & ACHAN_HIDDEN)==0) {
+			
+			draw_ipo_channel(di, chan->ipo, y);
+			y-=CHANNELHEIGHT+CHANNELSKIP;
+
+			/* Draw constraint channels */
+			for (conchan=chan->constraintChannels.first; conchan; conchan=conchan->next){
+				draw_ipo_channel(di, conchan->ipo, y);
+				y-=CHANNELHEIGHT+CHANNELSKIP;
+			}
 		}
 	}
 
+	if(saction->flag & SACTION_MOVING) {
+		int frame1_x, channel_y;
+		gla2DDrawTranslatePt(di, saction->timeslide, 0, &frame1_x, &channel_y);
+		cpack(0x0);
+		glBegin(GL_LINES);
+		glVertex2f(frame1_x, G.v2d->mask.ymin - 100);
+		glVertex2f(frame1_x, G.v2d->mask.ymax);
+		glEnd();
+	}
+	
 	glaEnd2DDraw(di);
 }
 
@@ -515,7 +551,7 @@ static void draw_mesh_strips(SpaceAction *saction, Key *key)
 
 		/* draw the little squares
 		 */
-		draw_icu_channel(di, icu, 0, y); 
+		draw_icu_channel(di, icu, y); 
 	}
 
 	glaEnd2DDraw(di);
@@ -594,7 +630,7 @@ void drawactionspace(ScrArea *sa, void *spacedata)
 	 * they have scrolled off of the screen ... this
 	 * oughta fix it
 	 */
-		   
+	
 	if (key) {
 		if (G.v2d->cur.ymin < -CHANNELHEIGHT) 
 			G.v2d->cur.ymin = -CHANNELHEIGHT;
@@ -607,7 +643,7 @@ void drawactionspace(ScrArea *sa, void *spacedata)
 	 * is set to an appropriate value based on whether sliders
 	 * are showing of not
 	 */
-	if (key && showsliders) ACTWIDTH = NAMEWIDTH + SLIDERWIDTH;
+	if (key && (G.saction->flag & SACTION_SLIDERS)) ACTWIDTH = NAMEWIDTH + SLIDERWIDTH;
 	else ACTWIDTH = NAMEWIDTH;
 
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA) ;
@@ -644,6 +680,8 @@ void drawactionspace(ScrArea *sa, void *spacedata)
 	calc_ipogrid();	
 	draw_ipogrid();
 
+	check_action_context(G.saction);
+	
 	/* Draw channel strips */
 	draw_channel_strips(G.saction);
 
@@ -652,10 +690,8 @@ void drawactionspace(ScrArea *sa, void *spacedata)
 		 * then draw the key frames in the action window
 		 */
 		draw_mesh_strips(G.saction, key);
-		/*meshactionbuts(G.saction, key);*/
 	}
-
-
+	
 	/* Draw current frame */
 	glViewport(ofsx+G.v2d->mask.xmin,  
              ofsy+G.v2d->mask.ymin, 
@@ -667,6 +703,9 @@ void drawactionspace(ScrArea *sa, void *spacedata)
             ( ofsy+G.v2d->mask.ymax-1)-( ofsy+G.v2d->mask.ymin)+1);
 	myortho2(G.v2d->cur.xmin, G.v2d->cur.xmax,  G.v2d->cur.ymin, G.v2d->cur.ymax);
 	draw_cfra_action();
+	
+	/* Draw markers */
+	draw_markers_timespace();
 
 	/* Draw scroll */
 	mywinset(curarea->win);	// reset scissor too
@@ -675,16 +714,20 @@ void drawactionspace(ScrArea *sa, void *spacedata)
       if(G.v2d->scroll) drawscroll(0);
 	}
 
-	/* Draw channel names */
-	draw_channel_names();
+	if(G.v2d->mask.xmin!=0) {
+		/* Draw channel names */
+		draw_channel_names();
 
-	if ( key ) {
-		/* if there is a mesh with rvk's selected,
-		 * then draw the key frames in the action window
-		 */
-		meshactionbuts(G.saction, key);
+		if(sa->winx > 50 + NAMEWIDTH + SLIDERWIDTH) {
+			if ( key ) {
+				/* if there is a mesh with rvk's selected,
+				 * then draw the key frames in the action window
+				 */
+				meshactionbuts(G.saction, OBACT, key);
+			}
+		}
 	}
-
+	
 	mywinset(curarea->win);	// reset scissor too
 	myortho2(-0.375, curarea->winx-0.375, -0.375, curarea->winy-0.375);
 	draw_area_emboss(sa);
@@ -696,225 +739,234 @@ void drawactionspace(ScrArea *sa, void *spacedata)
 	curarea->win_swap= WIN_BACK_OK;
 }
 
-/* unused and blank
-void draw_channel_name(const char* name, short type, float ypos, int selected)
+static void draw_keylist(gla2DDrawInfo *di, ListBase *keys, ListBase *blocks, float ypos, int totcurve)
 {
-}
-*/
-
-static void draw_keylist(gla2DDrawInfo *di, int totvert, BezTriple **blist, float ypos)
-{
-	int v;
-
-	if (!blist)
-		return;
-
-	for (v = 0; v<totvert; v++){
-		if (v==0 || (blist[v]->vec[1][0] != blist[v-1]->vec[1][0])){
+	CfraElem *ce;
+	ActKeyBlock *ab;
+	
+	glEnable(GL_BLEND);
+	
+	/* draw keyblocks */
+	if (blocks) {
+		for (ab= blocks->first; ab; ab= ab->next) {
+			/* only draw keyblock if it appears in all curves sampled */
+			if (ab->totcurve == totcurve) {
+				int sc_xa, sc_ya;
+				int sc_xb, sc_yb;
+				
+				/* get co-ordinates of block */
+				gla2DDrawTranslatePt(di, ab->start, ypos, &sc_xa, &sc_ya);
+				gla2DDrawTranslatePt(di, ab->end, ypos, &sc_xb, &sc_yb);
+				
+				/* draw block */
+				if (ab->sel & 1)
+					BIF_ThemeColor4(TH_STRIP_SELECT);
+				else
+					BIF_ThemeColor4(TH_STRIP);
+				glRectf(sc_xa,  sc_ya-3,  sc_xb,  sc_yb+5);
+			}
+		}
+	}
+	
+	/* draw keys */
+	if (keys) {
+		for (ce= keys->first; ce; ce= ce->next) {
 			int sc_x, sc_y;
-			gla2DDrawTranslatePt(di, blist[v]->vec[1][0], ypos, &sc_x, &sc_y);
-			draw_bevel_but(sc_x-2, sc_y-5, 7, 13, (blist[v]->f2 & 1));
-		}
-	}			
-}
-
-void draw_object_channel(gla2DDrawInfo *di, Object *ob, int flags, float ypos)
-{
-	BezTriple **blist;
-	int totvert;
-
-	blist = ob_to_keylist(ob, flags, &totvert);
-	if (blist){
-		draw_keylist(di,totvert, blist, ypos);
-		MEM_freeN(blist);
+			
+			/* get co-ordinate to draw at */
+			gla2DDrawTranslatePt(di, ce->cfra, ypos, &sc_x, &sc_y);
+			
+			if(ce->sel & 1) BIF_icon_draw_aspect(sc_x-7, sc_y-6, ICON_SPACE2, 1.0f);
+			else BIF_icon_draw_aspect(sc_x-7, sc_y-6, ICON_SPACE3, 1.0f);
+		}	
 	}
+	
+	glDisable(GL_BLEND);
 }
 
-void draw_ipo_channel(gla2DDrawInfo *di, Ipo *ipo, int flags, float ypos)
+void draw_object_channel(gla2DDrawInfo *di, Object *ob, float ypos)
 {
-	BezTriple **blist;
-	int totvert;
+	ListBase keys = {0, 0};
+	ListBase blocks = {0, 0};
+	int totcurve;
 
-	blist = ipo_to_keylist(ipo, flags, &totvert);
-	if (blist){
-		draw_keylist(di,totvert, blist, ypos);
-		MEM_freeN(blist);
-	}
+	totcurve= ob_to_keylist(ob, &keys, &blocks);
+	draw_keylist(di, &keys, &blocks, ypos, totcurve);
+	
+	BLI_freelistN(&keys);
+	BLI_freelistN(&blocks);
 }
 
-void draw_icu_channel(gla2DDrawInfo *di, IpoCurve *icu, int flags, float ypos)
+void draw_ipo_channel(gla2DDrawInfo *di, Ipo *ipo, float ypos)
 {
-	/* draw the keys for an IpoCurve
+	ListBase keys = {0, 0};
+	ListBase blocks = {0, 0};
+	int totcurve;
+
+	totcurve= ipo_to_keylist(ipo, &keys, &blocks);
+	draw_keylist(di, &keys, &blocks, ypos, totcurve);
+	
+	BLI_freelistN(&keys);
+	BLI_freelistN(&blocks);
+}
+
+void draw_icu_channel(gla2DDrawInfo *di, IpoCurve *icu, float ypos)
+{
+	ListBase keys = {0, 0};
+	ListBase blocks = {0, 0};
+
+	icu_to_keylist(icu, &keys, &blocks);
+	draw_keylist(di, &keys, &blocks, ypos, 1);
+	
+	BLI_freelistN(&keys);
+	BLI_freelistN(&blocks);
+}
+
+void draw_action_channel(gla2DDrawInfo *di, bAction *act, float ypos)
+{
+	ListBase keys = {0, 0};
+
+	action_to_keylist(act, &keys, NULL);
+	draw_keylist(di, &keys, NULL, ypos, 0);
+	BLI_freelistN(&keys);
+}
+
+static void add_bezt_to_keyblockslist(BezTriple *bezt, BezTriple *prev, ListBase *blocks)
+{
+	/* The equivilant of add_to_cfra_elem except this version 
+	 * makes ActKeyBlocks - one of the two datatypes required
+	 * for action editor drawing.
 	 */
-	BezTriple **blist;
-	int totvert;
-
-	blist = icu_to_keylist(icu, flags, &totvert);
-	if (blist){
-		draw_keylist(di,totvert, blist, ypos);
-		MEM_freeN(blist);
+	ActKeyBlock *ab, *abn;
+	
+	/* check if block needed */
+	if (bezt->vec[1][1] != prev->vec[1][1])
+		return;
+	
+	/* try to find a keyblock that starts on the previous beztriple */
+	for (ab= blocks->first; ab; ab= ab->next) {
+		/* check if alter existing block or add new block */
+		if (ab->start == prev->vec[1][0]) {			
+			/* set selection status and 'touched' status */
+			if (BEZSELECTED(bezt)) ab->sel = SELECT;
+			ab->modified += 1;
+			
+			return;
+		}
+		else if (ab->start > prev->vec[1][0]) break;
 	}
+	
+	/* add new block */
+	abn= MEM_callocN(sizeof(ActKeyBlock), "add_bezt_to_keyblockslist");
+	if (ab) BLI_insertlinkbefore(blocks, ab, abn);
+	else BLI_addtail(blocks, abn);
+	
+	abn->start= prev->vec[1][0];
+	abn->end= bezt->vec[1][0];
+	abn->val= bezt->vec[1][1];
+	
+	if (BEZSELECTED(prev) || BEZSELECTED(bezt))
+		abn->sel = SELECT;
+	else
+		abn->sel = 0;
+	abn->modified += 1;
 }
 
-void draw_action_channel(gla2DDrawInfo *di, bAction *act, int flags, float ypos)
+int ob_to_keylist(Object *ob, ListBase *keys, ListBase *blocks)
 {
-	BezTriple **blist;
-	int totvert;
+	bConstraintChannel *conchan;
+	int totcurve = 0;
 
-	blist = action_to_keylist(act, flags, &totvert);
-	if (blist){
-		draw_keylist(di,totvert, blist, ypos);
-		MEM_freeN(blist);
-	}
-}
-
-static BezTriple **ob_to_keylist(Object *ob, int flags, int *totvert)
-{
-	IpoCurve *icu;
-	int v, count=0;
-
-	BezTriple **list = NULL;
-
-	if (ob){
-
-		/* Count Object Keys */
-		if (ob->ipo){
-			for (icu=ob->ipo->curve.first; icu; icu=icu->next){
-				count+=icu->totvert;
-			}
+	if (ob) {
+		/* Add object keyframes */
+		if (ob->ipo) {
+			totcurve += ipo_to_keylist(ob->ipo, keys, blocks);
 		}
 		
-		/* Count Constraint Keys */
-		/* Count object data keys */
-
-		/* Build the list */
-		if (count){
-			list = MEM_callocN(sizeof(BezTriple*)*count, "beztlist");
-			count=0;
-			
-			/* Add object keyframes */
-			for (icu=ob->ipo->curve.first; icu; icu=icu->next){
-				for (v=0; v<icu->totvert; v++){
-					list[count++]=&icu->bezt[v];
-				}
-			}
-			
-			/* Add constraint keyframes */
-			/* Add object data keyframes */
-
-			/* Sort */
-			qsort(list, count, sizeof(BezTriple*), bezt_compare);
+		/* Add constraint keyframes */
+		for (conchan=ob->constraintChannels.first; conchan; conchan=conchan->next){
+			if(conchan->ipo) {
+				totcurve += ipo_to_keylist(conchan->ipo, keys, blocks);
+			}				
 		}
+			
+		/* Add object data keyframes */
+		// 		TODO??
 	}
-	(*totvert)=count;
-	return list;
+	
+	return totcurve;
 }
 
-static BezTriple **icu_to_keylist(IpoCurve *icu, int flags, int *totvert)
+void icu_to_keylist(IpoCurve *icu, ListBase *keys, ListBase *blocks)
 {
-	/* compile a list of all bezier triples in an
-	 * IpoCurve.
-	 */
-	int v, count = 0;
-
-	BezTriple **list = NULL;
-
-	count=icu->totvert;
-
-	if (count){
-		list = MEM_callocN(sizeof(BezTriple*)*count, "beztlist");
-		count=0;
+	BezTriple *bezt, *prev;
+	ActKeyBlock *ab, *abn;
+	int v;
+	
+	if (icu && icu->totvert){
+		/* loop through beztriples, making ActKeys and ActKeyBlocks */
+		bezt= icu->bezt;
+		prev= NULL;
+		
+		for (v=0; v<icu->totvert; v++, bezt++) {
+			add_to_cfra_elem(keys, bezt);
+			if (blocks && v) add_bezt_to_keyblockslist(bezt, prev, blocks);
 			
-		for (v=0; v<icu->totvert; v++){
-			list[count++]=&icu->bezt[v];
-		}
-		qsort(list, count, sizeof(BezTriple*), bezt_compare);
-	}
-	(*totvert)=count;
-	return list;
-
-}
-
-static BezTriple **ipo_to_keylist(Ipo *ipo, int flags, int *totvert)
-{
-	IpoCurve *icu;
-	int v, count=0;
-
-	BezTriple **list = NULL;
-
-	if (ipo){
-		/* Count required keys */
-		for (icu=ipo->curve.first; icu; icu=icu->next){
-			count+=icu->totvert;
+			prev= bezt;
 		}
 		
-		/* Build the list */
-		if (count){
-			list = MEM_callocN(sizeof(BezTriple*)*count, "beztlist");
-			count=0;
-			
-			for (icu=ipo->curve.first; icu; icu=icu->next){
-				for (v=0; v<icu->totvert; v++){
-					list[count++]=&icu->bezt[v];
+		/* update the number of curves the blocks have appeared in */
+		if (blocks) {
+			for (ab= blocks->first; ab; ab= abn) {
+				abn= ab->next;
+				
+				if (ab->modified) {
+					ab->modified = 0;
+					ab->totcurve += 1;
 				}
-			}			
-			qsort(list, count, sizeof(BezTriple*), bezt_compare);
+			}
 		}
 	}
-	(*totvert)=count;
-	return list;
 }
 
-static BezTriple **action_to_keylist(bAction *act, int flags, int *totvert)
+int ipo_to_keylist(Ipo *ipo, ListBase *keys, ListBase *blocks)
 {
 	IpoCurve *icu;
+	int totcurve = 0;
+	
+	if (ipo) {
+		for (icu= ipo->curve.first; icu; icu= icu->next) {
+			icu_to_keylist(icu, keys, blocks);
+			totcurve++;
+		}
+	}
+	
+	return totcurve;
+}
+
+int action_to_keylist(bAction *act, ListBase *keys, ListBase *blocks)
+{
 	bActionChannel *achan;
 	bConstraintChannel *conchan;
-	int v, count=0;
+	int totcurve = 0;
 
-	BezTriple **list = NULL;
-
-	if (act){
-		/* Count required keys */
-		for (achan=act->chanbase.first; achan; achan=achan->next){
-			/* Count transformation keys */
-			if(achan->ipo) {
-				for (icu=achan->ipo->curve.first; icu; icu=icu->next)
-					count+=icu->totvert;
-				
-				/* Count constraint keys */
-				for (conchan=achan->constraintChannels.first; conchan; conchan=conchan->next)
-					for (icu=conchan->ipo->curve.first; icu; icu=icu->next)
-						count+=icu->totvert;
-				
+	if (act) {
+		/* loop through action channels */
+		for (achan= act->chanbase.first; achan; achan= achan->next) {
+			/* firstly, add keys from action channel's ipo block */
+			if (achan->ipo) {
+				totcurve+= ipo_to_keylist(achan->ipo, keys, blocks);
+			}
+			
+			/* then, add keys from constraint channels */
+			for (conchan= achan->constraintChannels.first; conchan; conchan= conchan->next) {
+				if (conchan->ipo) {
+					totcurve+= ipo_to_keylist(achan->ipo, keys, blocks);
+				}
 			}
 		}
-		
-		/* Build the list */
-		if (count){
-			list = MEM_callocN(sizeof(BezTriple*)*count, "beztlist");
-			count=0;
-			
-			for (achan=act->chanbase.first; achan; achan=achan->next){
-				/* Add transformation keys */
-				for (icu=achan->ipo->curve.first; icu; icu=icu->next){
-					for (v=0; v<icu->totvert; v++)
-						list[count++]=&icu->bezt[v];
-				}
-					
-					/* Add constraint keys */
-				for (conchan=achan->constraintChannels.first; conchan; conchan=conchan->next){
-					for (icu=conchan->ipo->curve.first; icu; icu=icu->next)
-						for (v=0; v<icu->totvert; v++)
-							list[count++]=&icu->bezt[v];
-				}
-							
-			}		
-			qsort(list, count, sizeof(BezTriple*), bezt_compare);
-			
-		}
 	}
-	(*totvert)=count;
-	return list;
+	
+	return totcurve;
 }
 
