@@ -1,5 +1,5 @@
 /**
- * $Id: drawarmature.c,v 1.36 2006/01/28 18:33:17 hos Exp $
+ * $Id: drawarmature.c,v 1.41 2007/01/30 21:43:01 aligorith Exp $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -61,8 +61,10 @@
 #include "BKE_global.h"
 #include "BKE_main.h"
 #include "BKE_object.h"
+#include "BKE_ipo.h"
 #include "BKE_utildefines.h"
 
+#include "BIF_editaction.h"
 #include "BIF_editarmature.h"
 #include "BIF_gl.h"
 #include "BIF_glutil.h"
@@ -75,6 +77,7 @@
 
 #include "BDR_editobject.h"
 #include "BDR_drawobject.h"
+#include "BDR_drawaction.h"
 
 #include "BSE_edit.h"
 #include "BSE_view.h"
@@ -1378,7 +1381,7 @@ static void draw_pose_channels(Base *base, int dt)
 					if(bone->parent && (bone->parent->flag & BONE_HIDDEN_P))
 						flag &= ~BONE_CONNECTED;
 					
-					if(pchan->custom)
+					if(pchan->custom && !(arm->flag & ARM_NO_CUSTOM))
 						draw_custom_bone(pchan->custom, OB_SOLID, arm->flag, flag, index, bone->length);
 					else if(arm->drawtype==ARM_LINE)
 						;	/* nothing in solid */
@@ -1465,7 +1468,7 @@ static void draw_pose_channels(Base *base, int dt)
 					if(pchan->flag & POSE_STRIDE)
 						constflag |= PCHAN_HAS_STRIDE;
 
-					if(pchan->custom) {
+					if(pchan->custom && !(arm->flag & ARM_NO_CUSTOM)) {
 						if(dt<OB_SOLID)
 							draw_custom_bone(pchan->custom, OB_WIRE, arm->flag, flag, index, bone->length);
 					}
@@ -1731,18 +1734,34 @@ static void draw_pose_paths(Object *ob)
 {
 	bArmature *arm= ob->data;
 	bPoseChannel *pchan;
+	bAction *act;
+	bActionChannel *achan;
+	CfraElem *ce;
+	ListBase ak;
 	float *fp;
 	int a;
+	int stepsize, sfra;
 	
 	if(G.vd->zbuf) glDisable(GL_DEPTH_TEST);
 	
 	glPushMatrix();
 	glLoadMatrixf(G.vd->viewmat);
 	
+	/* version patch here - cannot access frame info from file reading */
+	if (arm->pathsize == 0) arm->pathsize= 1;
+	stepsize = arm->pathsize;
+	
 	for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
 		if(pchan->bone->layer & arm->layer) {
 			if(pchan->path) {
+				/* version patch here - cannot access frame info from file reading */
+				if ((pchan->pathsf == 0) || (pchan->pathef == 0)) {
+					pchan->pathsf= SFRA;
+					pchan->pathef= EFRA;
+				}
+				sfra= pchan->pathsf;
 				
+				/* draw curve-line of path */
 				BIF_ThemeColorBlend(TH_WIRE, TH_BACK, 0.7);
 				glBegin(GL_LINE_STRIP);
 				for(a=0, fp= pchan->path; a<pchan->pathlen; a++, fp+=3)
@@ -1750,17 +1769,82 @@ static void draw_pose_paths(Object *ob)
 				glEnd();
 				
 				glPointSize(1.0);
-				BIF_ThemeColor(TH_WIRE);
+				
+				/* draw little black point at each frame
+				 * NOTE: this is not really visible/noticable
+				 */
 				glBegin(GL_POINTS);
-				for(a=0, fp= pchan->path; a<pchan->pathlen; a++, fp+=3)
+				for(a=0, fp= pchan->path; a<pchan->pathlen; a++, fp+=3) 
 					glVertex3fv(fp);
 				glEnd();
 				
+				/* Draw little white dots at each framestep value */
 				BIF_ThemeColor(TH_TEXT_HI);
 				glBegin(GL_POINTS);
-				for(a=0, fp= pchan->path; a<pchan->pathlen; a+=10, fp+=30)
+				for(a=0, fp= pchan->path; a<pchan->pathlen; a+=stepsize, fp+=(stepsize*3)) 
 					glVertex3fv(fp);
 				glEnd();
+				
+				/* Draw frame numbers at each framestep value */
+				if (arm->pathflag & ARM_PATH_FNUMS) {
+					for(a=0, fp= pchan->path; a<pchan->pathlen; a+=stepsize, fp+=(stepsize*3)) {
+						char str[32];
+						
+						/* only draw framenum if several consecutive highlighted points occur on same point */
+						if (a == 0) {
+							glRasterPos3fv(fp);
+							sprintf(str, "  %d\n", (a+sfra));
+							BMF_DrawString(G.font, str);
+						}
+						else if ((a > stepsize) && (a < pchan->pathlen-stepsize)) { 
+							if ((VecEqual(fp, fp-(stepsize*3))==0) || (VecEqual(fp, fp+(stepsize*3))==0)) {
+								glRasterPos3fv(fp);
+								sprintf(str, "  %d\n", (a+sfra));
+								BMF_DrawString(G.font, str);
+							}
+						}
+					}
+				}
+				
+				/* Keyframes - dots and numbers */
+				if (arm->pathflag & ARM_PATH_KFRAS) {
+					/* build list of all keyframes in active action for pchan */
+					ak.first = ak.last = NULL;	
+					act= ob_get_action(ob);
+					if (act) {
+						achan= get_action_channel(act, pchan->name);
+						if (achan) 
+							ipo_to_keylist(achan->ipo, &ak, NULL);
+					}
+					
+					/* Draw little yellow dots at each keyframe */
+					BIF_ThemeColor(TH_VERTEX_SELECT);
+					glBegin(GL_POINTS);
+					for(a=0, fp= pchan->path; a<pchan->pathlen; a++, fp+=3) {
+						for (ce= ak.first; ce; ce= ce->next) {
+							if (ce->cfra == (a+sfra))
+								glVertex3fv(fp);
+						}
+					}
+					glEnd();
+					
+					/* Draw frame numbers of keyframes  */
+					if (arm->pathflag & ARM_PATH_FNUMS) {
+						for(a=0, fp= pchan->path; a<pchan->pathlen; a++, fp+=3) {
+							for (ce= ak.first; ce; ce= ce->next) {
+								if (ce->cfra == (a+sfra)) {
+									char str[32];
+									
+									glRasterPos3fv(fp);
+									sprintf(str, "  %d\n", (a+sfra));
+									BMF_DrawString(G.font, str);
+								}
+							}
+						}
+					}
+					
+					BLI_freelistN(&ak);
+				}
 			}
 		}
 	}
@@ -1769,6 +1853,66 @@ static void draw_pose_paths(Object *ob)
 	glPopMatrix();
 }
 
+/* draw ghosts that occur within a frame range 
+ * 	note: object should be in posemode */
+static void draw_ghost_poses_range(Base *base)
+{
+	Object *ob= base->object;
+	bArmature *arm= ob->data;
+	bPose *posen, *poseo;
+	float start, end, stepsize, range, colfac;
+	int cfrao, flago, ipoflago;
+	
+	start = arm->ghostsf;
+	end = arm->ghostef;
+	if (end<=start)
+		return;
+	
+	stepsize= (float)(arm->ghostsize);
+	range= (float)(end - start);
+	
+	/* store values */
+	ob->flag &= ~OB_POSEMODE;
+	cfrao= CFRA;
+	flago= arm->flag;
+	arm->flag &= ~(ARM_DRAWNAMES|ARM_DRAWAXES);
+	ipoflago= ob->ipoflag; 
+	ob->ipoflag |= OB_DISABLE_PATH;
+	
+	/* copy the pose */
+	poseo= ob->pose;
+	copy_pose(&posen, ob->pose, 1);
+	ob->pose= posen;
+	armature_rebuild_pose(ob, ob->data);	/* child pointers for IK */
+	
+	glEnable(GL_BLEND);
+	if(G.vd->zbuf) glDisable(GL_DEPTH_TEST);
+	
+	/* draw from first frame of range to last */
+	for(CFRA= start; CFRA<end; CFRA+=stepsize) {
+		colfac = (end-CFRA)/range;
+		BIF_ThemeColorShadeAlpha(TH_WIRE, 0, -128-(int)(120.0f*sqrt(colfac)));
+		
+		do_all_pose_actions(ob);
+		where_is_pose(ob);
+		draw_pose_channels(base, OB_WIRE);
+	}
+	glDisable(GL_BLEND);
+	if(G.vd->zbuf) glEnable(GL_DEPTH_TEST);
+
+	free_pose_channels(posen);
+	MEM_freeN(posen);
+	
+	/* restore */
+	CFRA= cfrao;
+	ob->pose= poseo;
+	arm->flag= flago;
+	armature_rebuild_pose(ob, ob->data);
+	ob->flag |= OB_POSEMODE;
+	ob->ipoflag= ipoflago; 
+
+}
+
 /* object is supposed to be armature in posemode */
 static void draw_ghost_poses(Base *base)
 {
@@ -1898,23 +2042,28 @@ int draw_armature(Base *base, int dt)
 		/*	Draw Pose */
 		if(ob->pose && ob->pose->chanbase.first) {
 			/* drawing posemode selection indices or colors only in these cases */
-			if(G.f & G_PICKSEL) {
-				if(ob->flag & OB_POSEMODE) arm->flag |= ARM_POSEMODE;
-			}
-			else if(ob->flag & OB_POSEMODE) {
-	
-				if(arm->ghostep) {
-					draw_ghost_poses(base);
+			if(!(base->flag & OB_FROMDUPLI)) {
+				if(G.f & G_PICKSEL) {
+					if(ob->flag & OB_POSEMODE) 
+						arm->flag |= ARM_POSEMODE;
 				}
-				
-				if(ob==OBACT) 
-					arm->flag |= ARM_POSEMODE;
-				else if(G.f & G_WEIGHTPAINT)
-					arm->flag |= ARM_POSEMODE;
-				
-				draw_pose_paths(ob);
-			}	
-			
+				else if(ob->flag & OB_POSEMODE) {
+					if (arm->ghosttype == ARM_GHOST_RANGE){
+						draw_ghost_poses_range(base);
+					}
+					else {
+						if (arm->ghostep)
+							draw_ghost_poses(base);
+					}
+					
+					if(ob==OBACT) 
+						arm->flag |= ARM_POSEMODE;
+					else if(G.f & G_WEIGHTPAINT)
+						arm->flag |= ARM_POSEMODE;
+					
+					draw_pose_paths(ob);
+				}	
+			}			
 			draw_pose_channels(base, dt);
 			arm->flag &= ~ARM_POSEMODE; 
 			
