@@ -1,5 +1,5 @@
 /**
- * $Id: drawipo.c,v 1.84 2006/06/25 12:30:33 ton Exp $
+ * $Id: drawipo.c,v 1.94 2007/02/11 04:02:17 aligorith Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -79,12 +79,14 @@
 #include "BIF_toolbox.h"
 #include "BIF_glutil.h"
 #include "BIF_editseq.h"
+#include "BIF_editaction.h"
 
 #include "BSE_drawipo.h"
 #include "BSE_view.h"
 #include "BSE_editipo.h"
 #include "BSE_editipo_types.h"
 #include "BSE_editnla_types.h"
+#include "BSE_time.h"
 
 #include "BPY_extern.h"
 
@@ -106,8 +108,6 @@ static int ipomachtx, ipomachty;
 
 static int vertymin, vertymax, horxmin, horxmax;	/* globals to test LEFTMOUSE for scrollbar */
 
-extern short ACTWIDTH;		/* this is ugly! */
-
 static void scroll_prstr(float x, float y, float val, char dir, int disptype)
 {
 	int len, macht;
@@ -1092,7 +1092,7 @@ static void draw_ipovertices(int sel)
 	
 	if(G.f & G_PICKSEL) return;
 	
-	glPointSize(3.0);
+	glPointSize(BIF_GetThemeValuef(TH_VERTEX_SIZE));
 	
 	ei= G.sipo->editipo;
 	for(nr=0; nr<G.sipo->totipo; nr++, ei++) {
@@ -1117,14 +1117,17 @@ static void draw_ipovertices(int sel)
 			 * on TNT2 / Linux with NVidia's drivers
 			 * (at least up to ver. 4349) */		
 			
+			a= ei->icu->totvert;
+			bezt= ei->icu->bezt;
 			bglBegin(GL_POINTS);
 			
-			bezt= ei->icu->bezt;
-			a= ei->icu->totvert;
-			while(a--) {				
+			while(a--) {
 				
+				/* IPO_DISPBITS is used for displaying layer ipo types as well as modes */
 				if(ei->disptype==IPO_DISPBITS) {
+					/*if (G.v2d->cur.xmin < bezt->vec[1][0] < G.v2d->cur.xmax) {*/
 					ok= 0;
+					
 					if(ei->flag & IPO_EDIT) {
 						if( (bezt->f2 & 1) == sel ) ok= 1;
 					}
@@ -1143,28 +1146,33 @@ static void draw_ipovertices(int sel)
 							b++;
 						}
 					}
-				}
-				else {
-					
+					/*}*/
+				} else { /* normal non bit curves */
 					if(ei->flag & IPO_EDIT) {
 						if(ei->icu->ipo==IPO_BEZ) {
-							if( (bezt->f1 & 1) == sel )	
+							/* Draw the editmode hendels for a bezier curve */
+							if( (bezt->f1 & 1) == sel)/* && G.v2d->cur.xmin < bezt->vec[0][0] < G.v2d->cur.xmax)*/
 								bglVertex3fv(bezt->vec[0]);
-							if( (bezt->f3 & 1) == sel )	
+							
+							if( (bezt->f3 & 1) == sel)/* && G.v2d->cur.xmin < bezt->vec[2][0] < G.v2d->cur.xmax)*/
 								bglVertex3fv(bezt->vec[2]);
+							
 						}
-						if( (bezt->f2 & 1) == sel )	
+						
+						if( (bezt->f2 & 1) == sel) /* && G.v2d->cur.xmin < bezt->vec[1][0] < G.v2d->cur.xmax)*/
 							bglVertex3fv(bezt->vec[1]);
 						
 					}
 					else {
+						/* draw only if in bounds */
+						/*if (G.v2d->cur.xmin < bezt->vec[1][0] < G.v2d->cur.xmax)*/
 						bglVertex3fv(bezt->vec[1]);
+						
 					}
 				}
 				
 				bezt++;
 			}
-
 			bglEnd();
 		}
 	}
@@ -1455,13 +1463,39 @@ static void draw_ipocurves(int sel)
 	}
 }
 
+static int get_ipo_cfra_from_cfra(SpaceIpo * sipo, int cfra)
+{
+	if (sipo->blocktype==ID_SEQ) {
+		Sequence * seq = (Sequence*) sipo->from;
+
+		if (!seq) {
+			return cfra;
+		}
+
+		if ((seq->flag & SEQ_IPO_FRAME_LOCKED) != 0) {
+			return cfra;
+		} else {
+			float ctime= frame_to_float(cfra - seq->startdisp);
+			float div= (seq->enddisp - seq->startdisp)/100.0f;
+
+			if(div == 0.0) {
+				return 0;
+			} else {
+				return ctime / div; 
+			}
+		}
+	} else {
+		return cfra;
+	}
+}
+
 static void draw_cfra(SpaceIpo *sipo)
 {
 	View2D *v2d= &sipo->v2d;
 	Object *ob;
 	float vec[2];
 	
-	vec[0]= (G.scene->r.cfra);
+	vec[0] = get_ipo_cfra_from_cfra(sipo, G.scene->r.cfra);
 	vec[0]*= G.scene->r.framelen;
 	
 	vec[1]= v2d->cur.ymin;
@@ -1968,11 +2002,11 @@ static void ipo_panel_properties(short cntrl)	// IPO_HANDLER_PROPERTIES
 
 			if(driver->type == IPO_DRIVER_TYPE_PYTHON) {
 				uiDefBut(block, TEX, B_IPO_REDR, "",				35,240,275,20, driver->name, 0, 127, 0, 0, "Python Expression");
+				uiBlockEndAlign(block);
 	 			if(driver->flag & IPO_DRIVER_FLAG_INVALID) {
 					uiDefBut(block, LABEL, 0, "Error: invalid Python expression",
 							5,215,230,19, NULL, 0, 0, 0, 0, "");
 				}
-				uiBlockEndAlign(block);
 			}
 			else {
 				uiDefIDPoinBut(block, test_obpoin_but, ID_OB, B_IPO_DEPCHANGE, "OB:",	35, 240, 125, 20, &(driver->ob), "Driver Object");
@@ -2128,6 +2162,9 @@ void drawipospace(ScrArea *sa, void *spacedata)
 		draw_ipohandles(1);
 		draw_ipovertices(1);
 		
+		/* draw markers */
+		draw_markers_timespace();
+		
 		/* restore viewport */
 		mywinset(sa->win);
 		
@@ -2520,6 +2557,9 @@ int view2dmove(unsigned short event)
 			view2d_do_locks(curarea, V2D_LOCK_COPY|V2D_LOCK_REDRAW);
 			areawinset(sa->win);
 			
+			if(curarea->spacetype==SPACE_OOPS)
+				((SpaceOops *)curarea->spacedata.first)->storeflag |= SO_TREESTORE_REDRAW;
+			
 			scrarea_do_windraw(curarea);
 			screen_swapbuffers();
 				
@@ -2610,4 +2650,3 @@ EditIpo *select_proj_ipo(rctf *rectf, int event)
 	}
 	return 0;
 }
-
