@@ -1,5 +1,5 @@
 /**
- * $Id: drawmesh.c,v 1.48 2006/03/11 17:15:10 blendix Exp $
+ * $Id: drawmesh.c,v 1.60 2006/12/21 13:47:26 blendix Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,6 +43,7 @@
 #include "BLI_arithb.h"
 #include "BLI_edgehash.h"
 
+#include "IMB_imbuf.h"
 #include "IMB_imbuf_types.h"
 
 #include "DNA_image_types.h"
@@ -133,13 +134,12 @@ void set_mipmap(int mipmap)
 	}
 }
 
-
 /**
  * Returns the current setting for mipmapping.
  */
-int get_mipmap(void)
+static int get_mipmap(void)
 {
-	return fDoMipMap;
+	return fDoMipMap && (!(G.f & G_TEXTUREPAINT));
 }
 
 /**
@@ -178,11 +178,12 @@ void clear_realtime_image_cache()
 }
 
 /* REMEMBER!  Changes here must go into my_set_tpage() as well */
-int set_tpage(TFace *tface)
+int set_tpage(MTFace *tface)
 {	
 	static int alphamode= -1;
-	static TFace *lasttface= 0;
+	static MTFace *lasttface= 0;
 	Image *ima;
+	ImBuf *ibuf;
 	unsigned int *rect=NULL, *bind;
 	int tpx=0, tpy=0, tilemode, tileXRep,tileYRep;
 	
@@ -256,8 +257,7 @@ int set_tpage(TFace *tface)
 	tilemode= tface->mode & TF_TILES;
 	tileXRep = 0;
 	tileYRep = 0;
-	if (ima)
-	{
+	if (ima) {
 		tileXRep = ima->xrep;
 		tileYRep = ima->yrep;
 	}
@@ -265,18 +265,17 @@ int set_tpage(TFace *tface)
 
 	if(ima==fCurpage && fCurtile==tface->tile && tilemode==fCurmode && fCurtileXRep==tileXRep && fCurtileYRep == tileYRep) return ima!=0;
 
-	if(tilemode!=fCurmode || fCurtileXRep!=tileXRep || fCurtileYRep != tileYRep)
-	{
+	if(tilemode!=fCurmode || fCurtileXRep!=tileXRep || fCurtileYRep != tileYRep) {
 		glMatrixMode(GL_TEXTURE);
 		glLoadIdentity();
 		
-		if(tilemode && ima!=0)
+		if(tilemode && ima!=NULL)
 			glScalef(ima->xrep, ima->yrep, 1.0);
 
 		glMatrixMode(GL_MODELVIEW);
 	}
 
-	if(ima==0 || ima->ok==0) {
+	if(ima==NULL || ima->ok==0) {
 		glDisable(GL_TEXTURE_2D);
 		
 		fCurtile= tface->tile;
@@ -288,24 +287,22 @@ int set_tpage(TFace *tface)
 		return 0;
 	}
 
-	if(ima->ibuf==0) {
-		load_image(ima, IB_rect, G.sce, G.scene->r.cfra);
+	ibuf= BKE_image_get_ibuf(ima, NULL);
+	if(ibuf==NULL) {
 
-		if(ima->ibuf==0) {
-			ima->ok= 0;
-
-			fCurtile= tface->tile;
-			fCurpage= 0;
-			fCurmode= tilemode;
-			fCurtileXRep = tileXRep;
-			fCurtileYRep = tileYRep;
-			
-			glDisable(GL_TEXTURE_2D);
-			return 0;
-		}
+		fCurtile= tface->tile;
+		fCurpage= 0;
+		fCurmode= tilemode;
+		fCurtileXRep = tileXRep;
+		fCurtileYRep = tileYRep;
 		
+		glDisable(GL_TEXTURE_2D);
+		return 0;
 	}
 
+	if ((ibuf->rect==NULL) && ibuf->rect_float)
+		IMB_rect_from_float(ibuf);
+
 	if(ima->tpageflag & IMA_TWINANIM) fCurtile= ima->lastframe;
 	else fCurtile= tface->tile;
 
@@ -321,8 +318,8 @@ int set_tpage(TFace *tface)
 		
 		if(*bind==0) {
 			
-			fTexwindx= ima->ibuf->x/ima->xrep;
-			fTexwindy= ima->ibuf->y/ima->yrep;
+			fTexwindx= ibuf->x/ima->xrep;
+			fTexwindy= ibuf->y/ima->yrep;
 			
 			if(fCurtile>=ima->xrep*ima->yrep) fCurtile= ima->xrep*ima->yrep-1;
 	
@@ -335,16 +332,16 @@ int set_tpage(TFace *tface)
 			tpx= fTexwindx;
 			tpy= fTexwindy;
 
-			rect= ima->ibuf->rect + fTexwinsy*ima->ibuf->x + fTexwinsx;
+			rect= ibuf->rect + fTexwinsy*ibuf->x + fTexwinsx;
 		}
 	}
 	else {
 		bind= &ima->bindcode;
 		
 		if(*bind==0) {
-			tpx= ima->ibuf->x;
-			tpy= ima->ibuf->y;
-			rect= ima->ibuf->rect;
+			tpx= ibuf->x;
+			tpy= ibuf->y;
+			rect= ibuf->rect;
 		}
 	}
 
@@ -365,7 +362,7 @@ int set_tpage(TFace *tface)
 				
 			tilerect= MEM_mallocN(rectw*recth*sizeof(*tilerect), "tilerect");
 			for (y=0; y<recth; y++) {
-				unsigned int *rectrow= &rect[y*ima->ibuf->x];
+				unsigned int *rectrow= &rect[y*ibuf->x];
 				unsigned int *tilerectrow= &tilerect[y*rectw];
 					
 				memcpy(tilerectrow, rectrow, tpx*sizeof(*rectrow));
@@ -395,7 +392,7 @@ int set_tpage(TFace *tface)
 		}
 		glBindTexture( GL_TEXTURE_2D, *bind);
 
-		if (!fDoMipMap)
+		if (!get_mipmap())
 		{
 			glTexImage2D(GL_TEXTURE_2D, 0,  GL_RGBA,  rectw, recth, 0, GL_RGBA, GL_UNSIGNED_BYTE, rect);
 			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
@@ -403,10 +400,12 @@ int set_tpage(TFace *tface)
 		} else
 		{
 			int minfilter= fLinearMipMap?GL_LINEAR_MIPMAP_LINEAR:GL_LINEAR_MIPMAP_NEAREST;
-			
+
 			gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, rectw, recth, GL_RGBA, GL_UNSIGNED_BYTE, rect);
 			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, minfilter);
 			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+
+			ima->tpageflag |= IMA_MIPMAP_COMPLETE;
 		}
 
 		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
@@ -418,8 +417,6 @@ int set_tpage(TFace *tface)
 	}
 	else glBindTexture( GL_TEXTURE_2D, *bind);
 	
-	tag_image_time(ima);
-
 	glEnable(GL_TEXTURE_2D);
 
 	fCurpage= ima;
@@ -430,17 +427,55 @@ int set_tpage(TFace *tface)
 	return 1;
 }
 
+void update_realtime_image(Image *ima, int x, int y, int w, int h)
+{
+	ImBuf *ibuf= BKE_image_get_ibuf(ima, NULL);
+	
+	if (ima->repbind || get_mipmap() || !ima->bindcode || !ibuf ||
+		(!is_pow2(ibuf->x) || !is_pow2(ibuf->y)) ||
+		(w == 0) || (h == 0)) {
+		/* these special cases require full reload still */
+		free_realtime_image(ima);
+	}
+	else {
+		int row_length = glaGetOneInteger(GL_UNPACK_ROW_LENGTH);
+		int skip_pixels = glaGetOneInteger(GL_UNPACK_SKIP_PIXELS);
+		int skip_rows = glaGetOneInteger(GL_UNPACK_SKIP_ROWS);
+
+		if ((ibuf->rect==NULL) && ibuf->rect_float)
+			IMB_rect_from_float(ibuf);
+
+		glBindTexture(GL_TEXTURE_2D, ima->bindcode);
+
+		glPixelStorei(GL_UNPACK_ROW_LENGTH, ibuf->x);
+		glPixelStorei(GL_UNPACK_SKIP_PIXELS, x);
+		glPixelStorei(GL_UNPACK_SKIP_ROWS, y);
+
+		glTexSubImage2D(GL_TEXTURE_2D, 0, x, y, w, h, GL_RGBA,
+			GL_UNSIGNED_BYTE, ibuf->rect);
+
+		glPixelStorei(GL_UNPACK_ROW_LENGTH, row_length);
+		glPixelStorei(GL_UNPACK_SKIP_PIXELS, skip_pixels);
+		glPixelStorei(GL_UNPACK_SKIP_ROWS, skip_rows);
+
+		if(ima->tpageflag & IMA_MIPMAP_COMPLETE)
+			ima->tpageflag &= ~IMA_MIPMAP_COMPLETE;
+	}
+}
+
 void free_realtime_image(Image *ima)
 {
 	if(ima->bindcode) {
 		glDeleteTextures(1, (GLuint *)&ima->bindcode);
 		ima->bindcode= 0;
+		ima->tpageflag &= ~IMA_MIPMAP_COMPLETE;
 	}
 	if(ima->repbind) {
 		glDeleteTextures(ima->totbind, (GLuint *)ima->repbind);
 	
 		MEM_freeN(ima->repbind);
-		ima->repbind= 0;
+		ima->repbind= NULL;
+		ima->tpageflag &= ~IMA_MIPMAP_COMPLETE;
 	}
 }
 
@@ -448,21 +483,63 @@ void free_all_realtime_images(void)
 {
 	Image* ima;
 
-	ima= G.main->image.first;
-	while(ima) {
+	for(ima=G.main->image.first; ima; ima=ima->id.next)
 		free_realtime_image(ima);
-		ima= ima->id.next;
+}
+
+/* these two functions are called on entering and exiting texture paint mode,
+   temporary disabling/enabling mipmapping on all images for quick texture
+   updates with glTexSubImage2D. images that didn't change don't have to be
+   re-uploaded to OpenGL */
+void texpaint_disable_mipmap(void)
+{
+	Image* ima;
+	
+	if(!fDoMipMap)
+		return;
+
+	for(ima=G.main->image.first; ima; ima=ima->id.next) {
+		if(ima->bindcode) {
+			glBindTexture(GL_TEXTURE_2D, ima->bindcode);
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+		}
+	}
+}
+
+void texpaint_enable_mipmap(void)
+{
+	Image* ima;
+
+	if(!fDoMipMap)
+		return;
+
+	for(ima=G.main->image.first; ima; ima=ima->id.next) {
+		if(ima->bindcode) {
+			if(ima->tpageflag & IMA_MIPMAP_COMPLETE) {
+				int minfilter= fLinearMipMap?GL_LINEAR_MIPMAP_LINEAR:GL_LINEAR_MIPMAP_NEAREST;
+
+				glBindTexture(GL_TEXTURE_2D, ima->bindcode);
+				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, minfilter);
+				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+			}
+			else
+				free_realtime_image(ima);
+		}
 	}
 }
 
 void make_repbind(Image *ima)
 {
-	if(ima==0 || ima->ibuf==0) return;
+	ImBuf *ibuf= BKE_image_get_ibuf(ima, NULL);
+	
+	if(ibuf==NULL) return;
 
 	if(ima->repbind) {
 		glDeleteTextures(ima->totbind, (GLuint *)ima->repbind);
 		MEM_freeN(ima->repbind);
 		ima->repbind= 0;
+		ima->tpageflag &= ~IMA_MIPMAP_COMPLETE;
 	}
 	ima->totbind= ima->xrep*ima->yrep;
 	if(ima->totbind>1) {
@@ -523,7 +600,7 @@ EdgeHash *get_tface_mesh_marked_edge_info(Mesh *me)
 
 	for (i=0; i<me->totface; i++) {
 		MFace *mf = &me->mface[i];
-		TFace *tf = &me->tface[i];
+		MTFace *tf = &me->mtface[i];
 		
 		if (mf->v3) {
 			if (!(tf->flag&TF_HIDE)) {
@@ -623,8 +700,8 @@ static int draw_tfaces3D__drawFaceOpts(void *userData, int index)
 {
 	Mesh *me = (Mesh*)userData;
 
-	if (me->tface) {
-		TFace *tface = &me->tface[index];
+	if (me->mtface) {
+		MTFace *tface = &me->mtface[index];
 		if (!(tface->flag&TF_HIDE) && (tface->flag&TF_SELECT))
 			return 2; /* Don't set color */
 		else
@@ -766,26 +843,26 @@ static Material *give_current_material_or_def(Object *ob, int matnr)
 	return ma?ma:&defmaterial;
 }
 
-static int set_draw_settings_cached(int clearcache, int textured, TFace *texface, int lit, Object *litob, int litmatnr, int doublesided)
+static int set_draw_settings_cached(int clearcache, int textured, MTFace *texface, int lit, Object *litob, int litmatnr, int doublesided)
 {
 	static int c_textured;
 	static int c_lit;
 	static int c_doublesided;
-	static TFace *c_texface;
+	static MTFace *c_texface;
 	static Object *c_litob;
 	static int c_litmatnr;
 	static int c_badtex;
 
 	if (clearcache) {
 		c_textured= c_lit= c_doublesided= -1;
-		c_texface= (TFace*) -1;
+		c_texface= (MTFace*) -1;
 		c_litob= (Object*) -1;
 		c_litmatnr= -1;
 		c_badtex= 0;
 	}
 
 	if (texface) {
-		lit = lit && (texface->mode&TF_LIGHT);
+		lit = lit && (lit==-1 || texface->mode&TF_LIGHT);
 		textured = textured && (texface->mode&TF_TEX);
 		doublesided = texface->mode&TF_TWOSIDE;
 	} else {
@@ -844,32 +921,50 @@ static Object *g_draw_tface_mesh_ob = NULL;
 static int g_draw_tface_mesh_islight = 0;
 static int g_draw_tface_mesh_istex = 0;
 static unsigned char g_draw_tface_mesh_obcol[4];
-static int draw_tface__set_draw(TFace *tface, int matnr)
+
+static int draw_tface__set_draw(MTFace *tface, MCol *mcol, int matnr)
 {
 	if (tface && ((tface->flag&TF_HIDE) || (tface->mode&TF_INVISIBLE))) return 0;
 
-	if (set_draw_settings_cached(0, g_draw_tface_mesh_istex, tface, g_draw_tface_mesh_islight, g_draw_tface_mesh_ob, matnr, TF_TWOSIDE)) {
+	if (tface && set_draw_settings_cached(0, g_draw_tface_mesh_istex, tface, g_draw_tface_mesh_islight, g_draw_tface_mesh_ob, matnr, TF_TWOSIDE)) {
 		glColor3ub(0xFF, 0x00, 0xFF);
 		return 2; /* Don't set color */
 	} else if (tface && tface->mode&TF_OBCOL) {
 		glColor3ubv(g_draw_tface_mesh_obcol);
 		return 2; /* Don't set color */
-	} else if (!tface) {
-		Material *ma= give_current_material(g_draw_tface_mesh_ob, matnr+1);
-		if(ma) glColor3f(ma->r, ma->g, ma->b);
-		else glColor3f(0.5, 0.5, 0.5);
-		return 1; /* Set color from mcol if available */
+	} else if (!mcol) {
+		if (tface) glColor3f(1.0, 1.0, 1.0);
+		else {
+			Material *ma= give_current_material(g_draw_tface_mesh_ob, matnr+1);
+			if(ma) glColor3f(ma->r, ma->g, ma->b);
+			else glColor3f(1.0, 1.0, 1.0);
+		}
+		return 2; /* Don't set color */
 	} else {
-		return 1; /* Set color from tface */
+		return 1; /* Set color from mcol */
 	}
 }
+
 static int draw_tface_mapped__set_draw(void *userData, int index)
 {
 	Mesh *me = (Mesh*)userData;
-	TFace *tface = (me->tface)? &me->tface[index]: NULL;
+	MTFace *tface = (me->mtface)? &me->mtface[index]: NULL;
+	MCol *mcol = (me->mcol)? &me->mcol[index]: NULL;
 	int matnr = me->mface[index].mat_nr;
 
-	return draw_tface__set_draw(tface, matnr);
+	return draw_tface__set_draw(tface, mcol, matnr);
+}
+
+static int wpaint__setSolidDrawOptions(void *userData, int index, int *drawSmooth_r)
+{
+	MTFace *tface = (MTFace *)userData;
+	if (tface) {
+		tface+= index;
+		if ((tface->flag&TF_HIDE) || (tface->mode&TF_INVISIBLE)) 
+			return 0;
+	}
+	*drawSmooth_r = 1;
+	return 1;
 }
 
 void draw_tface_mesh(Object *ob, Mesh *me, int dt)
@@ -877,17 +972,22 @@ void draw_tface_mesh(Object *ob, Mesh *me, int dt)
 {
 	unsigned char obcol[4];
 	int a;
-	short islight, istex;
+	short istex, solidtex=0;
 	DerivedMesh *dm;
-	int dmNeedsFree;
 	
 	if(me==NULL) return;
 
-	dm = mesh_get_derived_final(ob, &dmNeedsFree);
+	dm = mesh_get_derived_final(ob, get_viewedit_datamask());
 
 	glShadeModel(GL_SMOOTH);
 
-	islight= set_gl_light(ob);
+	/* option to draw solid texture with default lights */
+	if(dt>OB_WIRE && G.vd->drawtype==OB_SOLID) {
+		solidtex= 1;
+		g_draw_tface_mesh_islight= -1;
+	}
+	else
+		g_draw_tface_mesh_islight= set_gl_light(ob);
 	
 	obcol[0]= CLAMPIS(ob->col[0]*255, 0, 255);
 	obcol[1]= CLAMPIS(ob->col[1]*255, 0, 255);
@@ -900,36 +1000,46 @@ void draw_tface_mesh(Object *ob, Mesh *me, int dt)
 	else glFrontFace(GL_CCW);
 	
 	glCullFace(GL_BACK); glEnable(GL_CULL_FACE);
-	if(G.vd->drawtype==OB_TEXTURE) istex= 1;
+	if(solidtex || G.vd->drawtype==OB_TEXTURE) istex= 1;
 	else istex= 0;
 
 	g_draw_tface_mesh_ob = ob;
-	g_draw_tface_mesh_islight = islight;
 	g_draw_tface_mesh_istex = istex;
 	memcpy(g_draw_tface_mesh_obcol, obcol, sizeof(obcol));
 	set_draw_settings_cached(1, 0, 0, 0, 0, 0, 0);
 
-	if(dt > OB_SOLID) {
-		TFace *tface= me->tface;
-		MFace *mface= me->mface;
+	if(dt > OB_SOLID || g_draw_tface_mesh_islight==-1) {
 		bProperty *prop = get_property(ob, "Text");
 		int editing= (G.f & (G_VERTEXPAINT+G_FACESELECT+G_TEXTUREPAINT+G_WEIGHTPAINT)) && (ob==((G.scene->basact) ? (G.scene->basact->object) : 0));
-		int start, totface;
 
-		if(ob==OBACT && (G.f & G_FACESELECT) && me && me->tface)
-			dm->drawMappedFacesTex(dm, draw_tface_mapped__set_draw, (void*)me);
+#ifdef WITH_VERSE
+		if(me->vnode) {
+			/* verse-blender doesn't support uv mapping of textures yet */
+			dm->drawFacesTex(dm, NULL);
+		}
+		else if(ob==OBACT && (G.f & G_FACESELECT) && me && me->mtface) {
+#else
+		if(ob==OBACT && (G.f & G_FACESELECT) && me && me->mtface) {
+#endif		
+			if(G.f & G_WEIGHTPAINT)
+				dm->drawMappedFaces(dm, wpaint__setSolidDrawOptions, me->mtface, 1);
+			else
+				dm->drawMappedFacesTex(dm, draw_tface_mapped__set_draw, (void*)me);
+		}
 		else
 			dm->drawFacesTex(dm, draw_tface__set_draw);
 
-		start = 0;
-		totface = me->totface;
-
-		if (!editing && prop && tface) {
-			int dmNeedsFree;
-			DerivedMesh *dm = mesh_get_derived_deform(ob, &dmNeedsFree);
+		/* drawing game engine text hack */
+		if (!editing && prop && me->tface) {
+			DerivedMesh *ddm = mesh_get_derived_deform(ob, CD_MASK_BAREMESH);
+			MFace *mface= me->mface;
+			MTFace *tface= me->mtface;
+			MCol *mcol= me->mcol;	/* why does mcol exist? */
+			int start= 0, totface= me->totface;
 
 			tface+= start;
-			for (a=start; a<totface; a++, tface++) {
+			mcol+= start*4;
+			for (a=start; a<totface; a++, tface++, mcol+=4) {
 				MFace *mf= &mface[a];
 				int mode= tface->mode;
 				int matnr= mf->mat_nr;
@@ -940,16 +1050,16 @@ void draw_tface_mesh(Object *ob, Mesh *me, int dt)
 					float v1[3], v2[3], v3[3], v4[3];
 					char string[MAX_PROPSTRING];
 					int characters, index;
-					Image *ima;
+					ImBuf *ibuf;
 					float curpos;
 
 					if (badtex)
 						continue;
 
-					dm->getVertCo(dm, mf->v1, v1);
-					dm->getVertCo(dm, mf->v2, v2);
-					dm->getVertCo(dm, mf->v3, v3);
-					if (mf->v4) dm->getVertCo(dm, mf->v4, v4);
+					ddm->getVertCo(ddm, mf->v1, v1);
+					ddm->getVertCo(ddm, mf->v2, v2);
+					ddm->getVertCo(ddm, mf->v3, v3);
+					if (mf->v4) ddm->getVertCo(ddm, mf->v4, v4);
 
 					// The BM_FONT handling code is duplicated in the gameengine
 					// Search for 'Frank van Beek' ;-)
@@ -958,8 +1068,8 @@ void draw_tface_mesh(Object *ob, Mesh *me, int dt)
 					set_property_valstr(prop, string);
 					characters = strlen(string);
 					
-					ima = tface->tpage;
-					if (ima == NULL) {
+					ibuf= BKE_image_get_ibuf(tface->tpage, NULL);
+					if (ibuf == NULL) {
 						characters = 0;
 					}
 
@@ -982,11 +1092,12 @@ void draw_tface_mesh(Object *ob, Mesh *me, int dt)
 						// space starts at offset 1
 						// character = character - ' ' + 1;
 						
-						matrixGlyph(ima->ibuf, character, & centerx, &centery, &sizex, &sizey, &transx, &transy, &movex, &movey, &advance);
+						matrixGlyph(ibuf, character, & centerx, &centery, &sizex, &sizey, &transx, &transy, &movex, &movey, &advance);
 						movex+= curpos;
 
 						if (tface->mode & TF_OBCOL) glColor3ubv(obcol);
-						else cp= (char *)&(tface->col[0]);
+						else if (mcol) cp= (char *)mcol;
+						else glColor3ub(255, 255, 255);
 
 						glTexCoord2f((tface->uv[0][0] - centerx) * sizex + transx, (tface->uv[0][1] - centery) * sizey + transy);
 						if (cp) glColor3ub(cp[3], cp[2], cp[1]);
@@ -1012,7 +1123,7 @@ void draw_tface_mesh(Object *ob, Mesh *me, int dt)
 				}
 			}
 
-			if (dmNeedsFree) dm->release(dm);
+			ddm->release(ddm);
 		}
 
 		/* switch off textures */
@@ -1021,7 +1132,7 @@ void draw_tface_mesh(Object *ob, Mesh *me, int dt)
 	glShadeModel(GL_FLAT);
 	glDisable(GL_CULL_FACE);
 	
-	if(ob==OBACT && (G.f & G_FACESELECT) && me && me->tface) {
+	if(ob==OBACT && (G.f & G_FACESELECT) && me && me->mtface) {
 		draw_tfaces3D(ob, me, dm);
 	}
 	
@@ -1041,7 +1152,7 @@ void draw_tface_mesh(Object *ob, Mesh *me, int dt)
 	
 	glFrontFace(GL_CCW);
 
-	if(dt > OB_SOLID && !(ob==OBACT && (G.f & G_FACESELECT) && me && me->tface)) {
+	if(dt > OB_SOLID && !(ob==OBACT && (G.f & G_FACESELECT) && me && me->mtface)) {
 		if(ob->flag & SELECT) {
 			BIF_ThemeColor((ob==OBACT)?TH_ACTIVE:TH_SELECT);
 		} else {
@@ -1049,6 +1160,8 @@ void draw_tface_mesh(Object *ob, Mesh *me, int dt)
 		}
 		dm->drawLooseEdges(dm);
 	}
+
+	dm->release(dm);
 }
 
 void init_realtime_GL(void)
