@@ -1,5 +1,5 @@
 /**
- * $Id: drawseq.c,v 1.66 2006/07/08 10:54:53 ton Exp $
+ * $Id: drawseq.c,v 1.71 2006/12/31 15:38:14 schlaile Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -116,6 +116,9 @@ static char *give_seqname(Sequence *seq)
 	else if(seq->type==SEQ_OVERDROP) return "Over Drop";
 	else if(seq->type==SEQ_WIPE) return "Wipe";
 	else if(seq->type==SEQ_GLOW) return "Glow";
+	else if(seq->type==SEQ_TRANSFORM) return "Transform";
+	else if(seq->type==SEQ_COLOR) return "Color";
+	else if(seq->type==SEQ_SPEED) return "Speed";
 	else if(seq->type==SEQ_PLUGIN) {
 		if(!(seq->flag & SEQ_EFFECT_NOT_LOADED) &&
 		   seq->plugin && seq->plugin->doit) return seq->plugin->pname;
@@ -139,7 +142,8 @@ static void get_seq_color3ubv(Sequence *seq, char *col)
 {
 	char blendcol[3];
 	float hsv[3], rgb[3];
-	
+	SolidColorVars *colvars = (SolidColorVars *)seq->effectdata;
+
 	switch(seq->type) {
 	case SEQ_IMAGE:
 		BIF_GetThemeColor3ubv(TH_SEQ_IMAGE, col);
@@ -178,6 +182,8 @@ static void get_seq_color3ubv(Sequence *seq, char *col)
 		break;
 		
 	/* effects */
+	case SEQ_TRANSFORM:
+	case SEQ_SPEED:
 	case SEQ_ADD:
 	case SEQ_SUB:
 	case SEQ_MUL:
@@ -198,12 +204,21 @@ static void get_seq_color3ubv(Sequence *seq, char *col)
 		if (seq->type == SEQ_ALPHAUNDER)	hsv[0]+= 0.20;
 		if (seq->type == SEQ_OVERDROP)	hsv[0]+= 0.24;
 		if (seq->type == SEQ_GLOW)		hsv[0]+= 0.28;
-		
+		if (seq->type == SEQ_TRANSFORM)		hsv[0]+= 0.36;
+
 		if(hsv[0]>1.0) hsv[0]-=1.0; else if(hsv[0]<0.0) hsv[0]+= 1.0;
 		hsv_to_rgb(hsv[0], hsv[1], hsv[2], rgb, rgb+1, rgb+2);
 		col[0] = (char)(rgb[0]*255); col[1] = (char)(rgb[1]*255); col[2] = (char)(rgb[2]*255); 
 		break;
-		
+	case SEQ_COLOR:
+		if (colvars->col) {
+			col[0]= (char)(colvars->col[0]*255);
+			col[1]= (char)(colvars->col[1]*255);
+			col[2]= (char)(colvars->col[2]*255);
+		} else {
+			col[0] = col[1] = col[2] = 128;
+		}
+		break;
 	case SEQ_PLUGIN:
 		BIF_GetThemeColor3ubv(TH_SEQ_PLUGIN, col);
 		break;
@@ -401,7 +416,8 @@ static void draw_seq_handle(Sequence *seq, SpaceSeq *sseq, short direction)
 	}
 	
 	/* draw! */
-	if(seq->type < SEQ_EFFECT) {
+	if(seq->type < SEQ_EFFECT || 
+	   get_sequence_effect_num_inputs(seq->type) == 0) {
 		glEnable( GL_BLEND );
 		
 		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -579,8 +595,10 @@ static void draw_seq_text(Sequence *seq, float x1, float x2, float y1, float y2)
 
 			if(seq->seq3!=seq->seq2 && seq->seq1!=seq->seq3)
 				sprintf(str, "%d | %s: %d>%d (use %d)%s", seq->len, give_seqname(seq), seq->seq1->machine, seq->seq2->machine, seq->seq3->machine, can_float ? "" : " No float, upgrade plugin!");
-			else
+			else if (seq->seq1 && seq->seq2)
 				sprintf(str, "%d | %s: %d>%d%s", seq->len, give_seqname(seq), seq->seq1->machine, seq->seq2->machine, can_float ? "" : " No float, upgrade plugin!");
+			else 
+				sprintf(str, "%d | %s", seq->len, give_seqname(seq));
 		}
 		else if (seq->type == SEQ_RAM_SOUND) {
 			sprintf(str, "%d | %s", seq->len, seq->strip->stripdata->name);
@@ -668,6 +686,7 @@ static void draw_seq_strip(Sequence *seq, ScrArea *sa, SpaceSeq *sseq)
 {
 	float x1, x2, y1, y2;
 	char col[3];
+	Sequence *last_seq = get_last_seq();
 
 	/* body */
 	if(seq->startstill) x1= seq->start;
@@ -700,6 +719,7 @@ static void draw_seq_strip(Sequence *seq, ScrArea *sa, SpaceSeq *sseq)
 			col[0]= 255; col[1]= col[2]= 40;
 		} else BIF_GetColorPtrBlendShade3ubv(col, col, col, 0.0, 120);
 	}
+	else if (seq == last_seq) BIF_GetColorPtrBlendShade3ubv(col, col, col, 0.0, 120);
 	else if (seq->flag & SELECT) BIF_GetColorPtrBlendShade3ubv(col, col, col, 0.0, -150);
 	else BIF_GetColorPtrBlendShade3ubv(col, col, col, 0.0, -60);
 
@@ -994,6 +1014,7 @@ void seq_viewmove(SpaceSeq *sseq)
 #define SEQ_BUT_PLUGIN	1
 #define SEQ_BUT_RELOAD	2
 #define SEQ_BUT_EFFECT	3
+#define SEQ_BUT_RELOAD_ALL 4
 
 void do_seqbuttons(short val)
 {
@@ -1001,20 +1022,25 @@ void do_seqbuttons(short val)
 
 	switch(val) {
 	case SEQ_BUT_PLUGIN:
-		new_stripdata(last_seq);
-		free_imbuf_effect_spec(CFRA);
+	case SEQ_BUT_EFFECT:
+		update_changed_seq_and_deps(last_seq, 0, 1);
 		break;
 
 	case SEQ_BUT_RELOAD:
+	case SEQ_BUT_RELOAD_ALL:
+		update_seq_ipo_rect(last_seq);
+		update_seq_icu_rects(last_seq);
+
 		free_imbuf_seq();	// frees all
-		break;
-	case SEQ_BUT_EFFECT:
-		new_stripdata(last_seq);
-		calc_sequence(last_seq);
+
 		break;
 	}
 
-	allqueue(REDRAWSEQ, 0);
+	if (val == SEQ_BUT_RELOAD_ALL) {
+		allqueue(REDRAWALL, 0);
+	} else {
+		allqueue(REDRAWSEQ, 0);
+	}
 }
 
 static void seq_panel_properties(short cntrl)	// SEQ_HANDLER_PROPERTIES
@@ -1049,7 +1075,7 @@ static void seq_panel_properties(short cntrl)	// SEQ_HANDLER_PROPERTIES
 			}
 		}
 		uiDefButBitS(block, TOG, SEQ_IPO_FRAME_LOCKED,
-			     SEQ_BUT_RELOAD, "IPO Frame locked",
+			     SEQ_BUT_RELOAD_ALL, "IPO Frame locked",
 			     10,-40,150,19, &last_seq->flag,
 			     0.0, 1.0, 0, 0,
 			     "Lock the IPO coordinates to the "
@@ -1106,7 +1132,7 @@ static void seq_panel_properties(short cntrl)	// SEQ_HANDLER_PROPERTIES
 		
 		uiBlockBeginAlign(block);
 		uiDefButBitS(block, TOG, SEQ_IPO_FRAME_LOCKED,
-			     SEQ_BUT_RELOAD, "IPO Frame locked",
+			     SEQ_BUT_RELOAD_ALL, "IPO Frame locked",
 			     10,90,150,19, &last_seq->flag, 
 			     0.0, 1.0, 0, 0, 
 			     "Lock the IPO coordinates to the "
@@ -1122,7 +1148,7 @@ static void seq_panel_properties(short cntrl)	// SEQ_HANDLER_PROPERTIES
 		uiDefBut(block, TEX, B_NOP, "Name: ", 10,120,150,19, last_seq->name+2, 0.0, 21.0, 100, 0, "");
 		
 		uiDefButBitS(block, TOG, SEQ_IPO_FRAME_LOCKED,
-			     SEQ_BUT_RELOAD, "IPO Frame locked",
+			     SEQ_BUT_RELOAD_ALL, "IPO Frame locked",
 			     10,90,150,19, &last_seq->flag, 
 			     0.0, 1.0, 0, 0, 
 			     "Lock the IPO coordinates to the "
@@ -1149,7 +1175,6 @@ static void seq_panel_properties(short cntrl)	// SEQ_HANDLER_PROPERTIES
 		else if(last_seq->type==SEQ_GLOW){
 			GlowVars *glow = (GlowVars *)last_seq->effectdata;
 
-			uiBlockBeginAlign(block);
 			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "Threshold:", 	10,70,150,19, &glow->fMini, 0.0, 1.0, 0, 0, "Trigger Intensity");
 			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "Clamp:",			10,50,150,19, &glow->fClamp, 0.0, 1.0, 0, 0, "Brightness limit of intensity");
 			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "Boost factor:", 	10,30,150,19, &glow->fBoost, 0.0, 10.0, 0, 0, "Brightness multiplier");
@@ -1157,6 +1182,56 @@ static void seq_panel_properties(short cntrl)	// SEQ_HANDLER_PROPERTIES
 			uiDefButI(block, NUM, B_NOP, "Quality:", 10,-5,150,19, &glow->dQuality, 1.0, 5.0, 0, 0, "Accuracy of the blur effect");
 			uiDefButI(block, TOG, B_NOP, "Only boost", 10,-25,150,19, &glow->bNoComp, 0.0, 0.0, 0, 0, "Show the glow buffer only");
 		}
+		else if(last_seq->type==SEQ_TRANSFORM){
+			TransformVars *transform = (TransformVars *)last_seq->effectdata;
+
+			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "xScale Start:", 	10,70,150,19, &transform->ScalexIni, 0.0, 10.0, 0, 0, "X Scale Start");
+			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "xScale End:", 	160,70,150,19, &transform->ScalexFin, 0.0, 10.0, 0, 0, "X Scale End");
+			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "yScale Start:",	10,50,150,19, &transform->ScaleyIni, 0.0, 10.0, 0, 0, "Y Scale Start");
+			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "yScale End:", 	160,50,150,19, &transform->ScaleyFin, 0.0, 10.0, 0, 0, "Y Scale End");
+			
+			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "x Start:", 	10,30,150,19, &transform->xIni, -1000.0, 1000.0, 0, 0, "X Position Start");
+			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "x End:", 	160,30,150,19, &transform->xFin, -1000.0, 1000.0, 0, 0, "X Position End");
+
+			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "y Start:", 	10,10,150,19, &transform->yIni, -1000.0, 1000.0, 0, 0, "Y Position Start");
+			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "y End:", 	160,10,150,19, &transform->yFin, -1000.0, 1000.0, 0, 0, "Y Position End");
+
+			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "rot Start:",10,-10,150,19, &transform->rotIni, 0.0, 360.0, 0, 0, "Rotation Start");
+			uiDefButF(block, NUM, SEQ_BUT_EFFECT, "rot End:",160,-10,150,19, &transform->rotFin, 0.0, 360.0, 0, 0, "Rotation End");
+		} else if(last_seq->type==SEQ_COLOR) {
+			SolidColorVars *colvars = (SolidColorVars *)last_seq->effectdata;
+			uiDefButF(block, COL, SEQ_BUT_RELOAD, "",10,90,150,19, colvars->col, 0, 0, 0, 0, "");
+		} else if(last_seq->type==SEQ_SPEED){
+			SpeedControlVars *sp = 
+				(SpeedControlVars *)last_seq->effectdata;
+
+			uiDefButF(block, NUM, SEQ_BUT_RELOAD, "Global Speed:", 	10,70,150,19, &sp->globalSpeed, 0.0, 100.0, 0, 0, "Global Speed");
+
+			uiDefButBitI(block, TOG, SEQ_SPEED_INTEGRATE,
+				     SEQ_BUT_RELOAD, 
+				     "IPO is velocity",
+				     10,50,150,19, &sp->flags, 
+				     0.0, 1.0, 0, 0, 
+				     "Interpret the IPO value as a "
+				     "velocity instead of a frame number");
+
+			uiDefButBitI(block, TOG, SEQ_SPEED_BLEND,
+				     SEQ_BUT_RELOAD, 
+				     "Enable frame blending",
+				     10,30,150,19, &sp->flags, 
+				     0.0, 1.0, 0, 0, 
+				     "Blend two frames into the "
+				     "target for a smoother result");
+
+			uiDefButBitI(block, TOG, SEQ_SPEED_COMPRESS_IPO_Y,
+				     SEQ_BUT_RELOAD, 
+				     "IPO value runs from [0..1]",
+				     10,10,150,19, &sp->flags, 
+				     0.0, 1.0, 0, 0, 
+				     "Scale IPO value to get the "
+				     "target frame number.");
+		}
+
 		uiBlockEndAlign(block);
 	}
 }
