@@ -1,5 +1,5 @@
 /**
- * $Id: drawtext.c,v 1.61 2006/07/13 17:13:31 elubie Exp $
+ * $Id: drawtext.c,v 1.67 2006/12/26 12:51:54 harkyman Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -64,6 +64,7 @@
 #include "BKE_main.h"
 
 #include "BIF_gl.h"
+#include "BIF_glutil.h"
 #include "BIF_keyval.h"
 #include "BIF_interface.h"
 #include "BIF_drawtext.h"
@@ -820,6 +821,7 @@ static void do_textscroll(SpaceText *st, int mode)
 
 	glDrawBuffer(GL_FRONT);
 	uiEmboss(st->txtbar.xmin, st->txtbar.ymin, st->txtbar.xmax, st->txtbar.ymax, st->flags & ST_SCROLL_SELECT);
+	bglFlush();
 	glDrawBuffer(GL_BACK);
 
 	getmouseco_areawin(mval);
@@ -857,6 +859,7 @@ static void do_textscroll(SpaceText *st, int mode)
 
 	glDrawBuffer(GL_FRONT);
 	uiEmboss(st->txtbar.xmin, st->txtbar.ymin, st->txtbar.xmax, st->txtbar.ymax, st->flags & ST_SCROLL_SELECT);
+	bglFlush();
 	glDrawBuffer(GL_BACK);
 }
 
@@ -973,7 +976,7 @@ void drawtextspace(ScrArea *sa, void *spacedata)
 	}
 	
 	if(st->showsyntax) {
-		if (!tmp->format) {
+		if (tmp && !tmp->format) {
 			get_format_string();
 		}
 	}
@@ -1079,7 +1082,10 @@ void txt_write_file(Text *text)
 	
 	/* Do we need to get a filename? */
 	if (text->flags & TXT_ISMEM) {
-		activate_fileselect(FILE_SPECIAL, "SAVE TEXT FILE", G.sce, save_mem_text);
+		if (text->name)
+			activate_fileselect(FILE_SPECIAL, "SAVE TEXT FILE", text->name, save_mem_text);
+		else
+			activate_fileselect(FILE_SPECIAL, "SAVE TEXT FILE", text->id.name+2, save_mem_text);
 		return;	
 	}
 	
@@ -1411,8 +1417,9 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 	int do_draw=0, p;
 
 	/* smartass code to prevent the CTRL/ALT events below from not working! */
-	if(!ispunct(ascii)) 
-		if (!isprint(ascii) || (G.qual & ~LR_SHIFTKEY)) ascii= 0;
+	if(G.qual & (LR_ALTKEY|LR_CTRLKEY))
+		if(!ispunct(ascii)) 
+			ascii= 0;
 
 	text= st->text;
 	
@@ -1447,7 +1454,9 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 				}
 			} 
 			else if (event==QKEY) {
+				if (G.qual & LR_CTRLKEY) {
 					if(okee("Quit Blender")) exit_usiblender();
+				}
 			}
 			else if (event==NKEY) {
 				if (G.qual & LR_ALTKEY) {
