@@ -45,6 +45,8 @@
 #include "DNA_scene_types.h"
 #include "DNA_space_types.h"
 #include "DNA_screen_types.h"
+#include "DNA_sound_types.h"
+#include "DNA_view2d_types.h"
 
 #include "BKE_ipo.h"
 #include "BKE_object.h"
@@ -162,7 +164,7 @@ static void draw_marker(TimeMarker *marker)
 	}
 }
 
-static void draw_markers_time( void )
+static void draw_markers_time(void)
 {
 	TimeMarker *marker;
 
@@ -179,6 +181,35 @@ static void draw_markers_time( void )
 	}
 }
 
+void draw_markers_timespace()
+{
+	TimeMarker *marker;
+	float yspace, ypixels;
+	
+	/* move ortho view to align with slider in bottom */
+	glTranslatef(0.0f, G.v2d->cur.ymin, 0.0f);
+	
+	/* bad hacks in drawing markers... inverse correct that as well */
+	yspace= G.v2d->cur.ymax - G.v2d->cur.ymin;
+	ypixels= G.v2d->mask.ymax - G.v2d->mask.ymin;
+	glTranslatef(0.0f, -11.0*yspace/ypixels, 0.0f);
+		
+	/* unselected markers are drawn at the first time */
+	for(marker= G.scene->markers.first; marker; marker= marker->next) {
+		if(!(marker->flag & SELECT)) draw_marker(marker);
+	}
+	
+	/* selected markers are drawn later ... selected markers have to cover unselected
+		* markers laying at the same position as selected markers */
+	for(marker= G.scene->markers.first; marker; marker= marker->next) {
+		if(marker->flag & SELECT) draw_marker(marker);
+	}
+
+	glTranslatef(0.0f, -G.v2d->cur.ymin, 0.0f);
+	glTranslatef(0.0f, 11.0*yspace/ypixels, 0.0f);
+
+}
+
 static void draw_sfra_efra()
 {
 	BIF_ThemeColorShade(TH_BACK, -25);
