@@ -1,5 +1,5 @@
 /**
- * $Id: edit.c,v 1.90 2006/07/06 17:14:03 ton Exp $
+ * $Id: edit.c,v 1.99 2007/01/22 00:48:53 nicholasbishop Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -82,6 +82,10 @@
 #include "BKE_object.h"
 #include "BKE_utildefines.h"
 
+#ifdef WITH_VERSE
+#include "BKE_verse.h"
+#endif
+
 #include "BIF_editmesh.h"
 #include "BIF_editview.h"
 #include "BIF_editarmature.h"
@@ -94,6 +98,10 @@
 #include "BIF_screen.h"
 #include "BIF_toolbox.h"
 
+#ifdef WITH_VERSE
+#include "BIF_verse.h"
+#endif
+
 #include "BSE_edit.h"
 #include "BSE_drawipo.h"
 #include "BSE_drawview.h"
@@ -247,7 +255,7 @@ int get_border(rcti *rect, short flag)
 						BMF_DrawString(G.fonts, str);
 					}
 					else if(G.vd->persp==2) {
-						rcti vb;
+						rctf vb;
 	
 						calc_viewborder(G.vd, &vb);
 	
@@ -511,12 +519,12 @@ static void count_object(Object *ob, int sel, int totob)
 			ModifierData *md = modifiers_findByType(ob, eModifierType_Subsurf);
 			int totvert, totface;
 			
+			subsurf= 1;
 			if (md) {
 				SubsurfModifierData *smd = (SubsurfModifierData*) md;
-				
-				subsurf= 1<<(2*smd->levels);
+				if(smd->modifier.mode & eModifierMode_Realtime)
+					subsurf= 1<<(2*smd->levels);
 			}
-			else subsurf= 1;
 			
 			totvert= subsurf*me->totvert*totob;
 			totface= subsurf*me->totface*totob;
@@ -604,7 +612,7 @@ void countall()
 				if(efa->f & SELECT) G.totfacesel++;
 			}
 			
-			/*add code to strip editselections*/
+			EM_validate_selections();
 		}
 		else if (G.obedit->type==OB_ARMATURE){
 			for (ebo=G.edbo.first;ebo;ebo=ebo->next){
@@ -723,7 +731,7 @@ void countall()
 			
 			if(base->flag & SELECT) G.totobjsel++;
 			
-			if(ob->parent && (ob->parent->transflag & OB_DUPLIVERTS)) {
+			if(ob->parent && (ob->parent->transflag & (OB_DUPLIVERTS|OB_DUPLIFACES))) {
 				int tot= count_duplilist(ob->parent);
 				G.totobj+=tot;
 				count_object(ob, base->flag & SELECT, tot);
@@ -759,6 +767,10 @@ static void special_transvert_update(void)
 		DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 		
 		if(G.obedit->type==OB_MESH) {
+#ifdef WITH_VERSE
+			if(G.editMesh->vnode)
+				sync_all_verseverts_with_editverts((VNode*)G.editMesh->vnode);
+#endif
 			recalc_editnormals();	// does face centers too
 		}
 		else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) {
@@ -773,10 +785,27 @@ static void special_transvert_update(void)
 		else if(G.obedit->type==OB_ARMATURE){
 			bArmature *arm= G.obedit->data;
 			EditBone *ebo;
+			TransVert *tv= transvmain;
+			int a=0;
+			
+			/* Ensure all bone tails are correctly adjusted */
+			for (ebo=G.edbo.first; ebo; ebo=ebo->next) {
+				/* adjust tip if both ends selected */
+				if ((ebo->flag & BONE_ROOTSEL) && (ebo->flag & BONE_TIPSEL)) {
+					if (tv) {
+						float diffvec[3];
+						
+						VecSubf(diffvec, tv->loc, tv->oldloc);
+						VecAddf(ebo->tail, ebo->tail, diffvec);
+						
+						a++;
+						if (a<tottrans) tv++;
+					}
+				}
+			}
 			
 			/* Ensure all bones are correctly adjusted */
-			for (ebo=G.edbo.first; ebo; ebo=ebo->next){
-				
+			for (ebo=G.edbo.first; ebo; ebo=ebo->next) {
 				if ((ebo->flag & BONE_CONNECTED) && ebo->parent){
 					/* If this bone has a parent tip that has been moved */
 					if (ebo->parent->flag & BONE_TIPSEL){
@@ -901,25 +930,30 @@ static void make_trans_verts(float *min, float *max, int mode)
 		
 		for (ebo=G.edbo.first;ebo;ebo=ebo->next){
 			if(ebo->layer & arm->layer) {
-				if (ebo->flag & BONE_TIPSEL){
-					VECCOPY (tv->oldloc, ebo->tail);
-					tv->loc= ebo->tail;
-					tv->nor= NULL;
-					tv->flag= 1;
-					tv++;
-					tottrans++;
-				}
-
-				/*  Only add the root if there is no connection */
-				if (ebo->flag & BONE_ROOTSEL){
-					if (!(ebo->parent && (ebo->flag & BONE_CONNECTED) && ebo->parent->flag & BONE_TIPSEL)){
+				short tipsel= (ebo->flag & BONE_TIPSEL);
+				short rootsel= (ebo->flag & BONE_ROOTSEL);
+				short rootok= (!(ebo->parent && (ebo->flag & BONE_CONNECTED) && ebo->parent->flag & BONE_TIPSEL));
+				
+				if ((tipsel && rootsel) || (rootsel)) {
+					/* Only add the root if there is no connection.
+					 * Don't add the tip, otherwise we get zero-length bones.
+					 */
+					if (rootok) {
 						VECCOPY (tv->oldloc, ebo->head);
 						tv->loc= ebo->head;
 						tv->nor= NULL;
 						tv->flag= 1;
 						tv++;
 						tottrans++;
-					}		
+					}	
+				}
+				else if (tipsel) {
+					VECCOPY (tv->oldloc, ebo->tail);
+					tv->loc= ebo->tail;
+					tv->nor= NULL;
+					tv->flag= 1;
+					tv++;
+					tottrans++;
 				}
 			}			
 		}
@@ -1077,10 +1111,10 @@ void snap_sel_to_grid()
 
 		}
 
+		special_transvert_update();
+		
 		MEM_freeN(transvmain);
 		transvmain= 0;
-		
-		special_transvert_update();
 
 		allqueue(REDRAWVIEW3D, 0);
 		return;
@@ -1091,7 +1125,35 @@ void snap_sel_to_grid()
 		if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
 			ob= base->object;
 			if(ob->flag & OB_POSEMODE) {
-				;	// todo
+				bPoseChannel *pchan;
+				bArmature *arm= ob->data;
+				
+				for (pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
+					if(pchan->bone->flag & BONE_SELECTED) {
+						if(pchan->bone->layer & arm->layer) {
+							if((pchan->bone->flag & BONE_CONNECTED)==0) { 
+								float vecN[3], nLoc[3]; 
+								
+								/* get nearest grid point to snap to */
+								VECCOPY(nLoc, pchan->pose_mat[3]);
+								vec[0]= gridf * (float)(floor(.5+ nLoc[0]/gridf));
+								vec[1]= gridf * (float)(floor(.5+ nLoc[1]/gridf));
+								vec[2]= gridf * (float)(floor(.5+ nLoc[2]/gridf));
+								
+								/* get bone-space location of grid point */
+								armature_loc_pose_to_bone(pchan, vec, vecN);
+								
+								/* adjust location */
+								VECCOPY(pchan->loc, vecN);
+							}
+							/* if the bone has a parent and is connected to the parent, 
+							 * don't do anything - will break chain unless we do auto-ik. 
+							 */
+						}
+					}
+				}
+				ob->pose->flag |= (POSE_LOCKED|POSE_DO_UNLOCK);
+				ob->recalc |= OB_RECALC_DATA;
 			}
 			else {
 				ob->recalc |= OB_RECALC_OB;
@@ -1114,6 +1176,9 @@ void snap_sel_to_grid()
 					ob->loc[1]+= vec[1];
 					ob->loc[2]+= vec[2];
 				}
+#ifdef WITH_VERSE
+				if(ob->vnode) b_verse_send_transformation(ob);
+#endif
 			}
 		}
 
@@ -1155,11 +1220,12 @@ void snap_sel_to_curs()
 			VECCOPY(tv->loc, vec);
 
 		}
+		
+		special_transvert_update();
+		
 		MEM_freeN(transvmain);
 		transvmain= 0;
 
-		special_transvert_update();
-
 		allqueue(REDRAWVIEW3D, 0);
 		return;
 	}
@@ -1180,11 +1246,18 @@ void snap_sel_to_curs()
 				for (pchan = ob->pose->chanbase.first; pchan; pchan=pchan->next) {
 					if(pchan->bone->flag & BONE_SELECTED) {
 						if(pchan->bone->layer & arm->layer) {
-							if(pchan->parent==NULL) {
-								/* this is wrong... lazy! */
-								VECCOPY(pchan->loc, cursp);
+							if((pchan->bone->flag & BONE_CONNECTED)==0) { 
+								float curspn[3];
+								
+								/* get location of cursor in bone-space */
+								armature_loc_pose_to_bone(pchan, cursp, curspn);
+								
+								/* calculate new position */
+								VECCOPY(pchan->loc, curspn);
 							}
-							/* else todo... */
+							/* if the bone has a parent and is connected to the parent, 
+							 * don't do anything - will break chain unless we do auto-ik. 
+							 */
 						}
 					}
 				}
@@ -1212,6 +1285,9 @@ void snap_sel_to_curs()
 					ob->loc[1]+= vec[1];
 					ob->loc[2]+= vec[2];
 				}
+#ifdef WITH_VERSE
+				if(ob->vnode) b_verse_send_transformation(ob);
+#endif
 			}
 		}
 
@@ -1441,10 +1517,29 @@ void snap_to_center()
 		base= (G.scene->base.first);
 		while(base) {
 			if(((base)->flag & SELECT) && ((base)->lay & G.vd->lay) ) {
-				VECCOPY(vec, base->object->obmat[3]);
-				VecAddf(centroid, centroid, vec);
-				DO_MINMAX(vec, min, max);
-				count++;
+				ob= base->object;
+				if(ob->flag & OB_POSEMODE) {
+					bPoseChannel *pchan;
+					bArmature *arm= ob->data;
+					
+					for (pchan = ob->pose->chanbase.first; pchan; pchan=pchan->next) {
+						if(pchan->bone->flag & BONE_SELECTED) {
+							if(pchan->bone->layer & arm->layer) {
+								VECCOPY(vec, pchan->pose_mat[3]);
+								VecAddf(centroid, centroid, vec);
+								DO_MINMAX(vec, min, max);
+								count++;
+							}
+						}
+					}
+				}
+				else {
+					/* not armature bones (i.e. objects) */
+					VECCOPY(vec, base->object->obmat[3]);
+					VecAddf(centroid, centroid, vec);
+					DO_MINMAX(vec, min, max);
+					count++;
+				}
 			}
 			base= base->next;
 		}
@@ -1482,11 +1577,12 @@ void snap_to_center()
 			Mat3MulVecfl(imat, vec);
 			VECCOPY(tv->loc, vec);
 		}
+		
+		special_transvert_update();
+		
 		MEM_freeN(transvmain);
 		transvmain= 0;
 
-		special_transvert_update();
-
 		allqueue(REDRAWVIEW3D, 0);
 		return;
 	}
@@ -1496,7 +1592,27 @@ void snap_to_center()
 		if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
 			ob= base->object;
 			if(ob->flag & OB_POSEMODE) {
-				; // todo
+				bPoseChannel *pchan;
+				bArmature *arm= ob->data;
+				
+				for (pchan = ob->pose->chanbase.first; pchan; pchan=pchan->next) {
+					if(pchan->bone->flag & BONE_SELECTED) {
+						if(pchan->bone->layer & arm->layer) {
+							if((pchan->bone->flag & BONE_CONNECTED)==0) { 
+								/* get location of cursor in bone-space */
+								armature_loc_pose_to_bone(pchan, snaploc, vec);
+								
+								/* calculate new position */
+								VECCOPY(pchan->loc, vec);
+							}
+							/* if the bone has a parent and is connected to the parent, 
+							 * don't do anything - will break chain unless we do auto-ik. 
+							 */
+						}
+					}
+				}
+				ob->pose->flag |= (POSE_LOCKED|POSE_DO_UNLOCK);
+				ob->recalc |= OB_RECALC_DATA;
 			}
 			else {
 				ob->recalc |= OB_RECALC_OB;
@@ -1519,6 +1635,9 @@ void snap_to_center()
 					ob->loc[1]+= vec[1];
 					ob->loc[2]+= vec[2];
 				}
+#ifdef WITH_VERSE
+				if(ob->vnode) b_verse_send_transformation(ob);
+#endif
 			}
 		}
 
@@ -1655,6 +1774,10 @@ void toggle_shading(void)
 		if(G.vd->drawtype==OB_SOLID || G.vd->drawtype==OB_SHADED) G.vd->drawtype= OB_WIRE;
 		else G.vd->drawtype= OB_SOLID;
 	}
+	
+	if(G.vd->drawtype != OB_WIRE && G.vd->depths) {
+		G.vd->depths->damaged= 1;
+	}
 }
 
 void minmax_verts(float *min, float *max)
