@@ -1,5 +1,5 @@
 /**
- * $Id: editconstraint.c,v 1.38 2006/06/15 04:13:25 hos Exp $
+ * $Id: editconstraint.c,v 1.45 2006/12/20 18:29:23 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -619,6 +619,12 @@ void add_constraint(int only_IK)
 		pchanact= get_active_posechannel(ob);
 		if(pchanact==NULL) return;
 	
+		/* check protection */
+		if(ob->proxy && (pchanact->bone->layer & arm->layer_protected)) {
+			error("Bone is Proxy protected");
+			return;
+		}
+		
 		/* find selected bone */
 		for(pchansel= ob->pose->chanbase.first; pchansel; pchansel= pchansel->next) {
 			if(pchansel!=pchanact)
@@ -653,21 +659,21 @@ void add_constraint(int only_IK)
 	else {
 		if(pchanact) {
 			if(pchansel)
-				nr= pupmenu("Add Constraint to Active Bone%t|Copy Location%x1|Copy Rotation%x2|Copy Scale%x8|Track To%x3|Floor%x4|Locked Track%x5|Stretch To%x7");
+				nr= pupmenu("Add Constraint to Active Bone%t|Copy Location%x1|Copy Rotation%x2|Copy Scale%x8|Limit Location%x13|Limit Rotation%x14|Limit Scale%x15|Track To%x3|Floor%x4|Locked Track%x5|Stretch To%x7|Action%x16");
 			else if(obsel && obsel->type==OB_CURVE)
-				nr= pupmenu("Add Constraint to Active Object%t|Copy Location%x1|Copy Rotation%x2|Copy Scale%x8|Track To%x3|Floor%x4|Locked Track%x5|Follow Path%x6|Stretch To%x7");
+				nr= pupmenu("Add Constraint to Active Object%t|Copy Location%x1|Copy Rotation%x2|Copy Scale%x8|Limit Location%x13|Limit Rotation%x14|Limit Scale%x15|Track To%x3|Floor%x4|Locked Track%x5|Follow Path%x6|Stretch To%x7|Action%x16");
 			else if(obsel)
-				nr= pupmenu("Add Constraint to Active Object%t|Copy Location%x1|Copy Rotation%x2|Copy Scale%x8|Track To%x3|Floor%x4|Locked Track%x5|Stretch To%x7");
+				nr= pupmenu("Add Constraint to Active Object%t|Copy Location%x1|Copy Rotation%x2|Copy Scale%x8|Limit Location%x13|Limit Rotation%x14|Limit Scale%x15|Track To%x3|Floor%x4|Locked Track%x5|Stretch To%x7|Action%x16");
 			else
-				nr= pupmenu("Add Constraint to New Empty Object%t|Copy Location%x1|Copy Rotation%x2|Copy Scale%x8|Track To%x3|Floor%x4|Locked Track%x5|Stretch To%x7");
+				nr= pupmenu("Add Constraint to New Empty Object%t|Copy Location%x1|Copy Rotation%x2|Copy Scale%x8|Limit Location%x13|Limit Rotation%x14|Limit Scale%x15|Track To%x3|Floor%x4|Locked Track%x5|Stretch To%x7");
 		}
 		else {
 			if(obsel && obsel->type==OB_CURVE)
-				nr= pupmenu("Add Constraint to Active Object%t|Copy Location%x1|Copy Rotation%x2|Copy Scale%x8|Track To%x3|Floor%x4|Locked Track%x5|Follow Path%x6");
+				nr= pupmenu("Add Constraint to Active Object%t|Copy Location%x1|Copy Rotation%x2|Copy Scale%x8|Limit Location%x13|Limit Rotation%x14|Limit Scale%x15|Track To%x3|Floor%x4|Locked Track%x5|Follow Path%x6");
 			else if(obsel)
-				nr= pupmenu("Add Constraint to Active Object%t|Copy Location%x1|Copy Rotation%x2|Copy Scale%x8|Track To%x3|Floor%x4|Locked Track%x5");
+				nr= pupmenu("Add Constraint to Active Object%t|Copy Location%x1|Copy Rotation%x2|Copy Scale%x8|Limit Location%x13|Limit Rotation%x14|Limit Scale%x15|Track To%x3|Floor%x4|Locked Track%x5");
 			else
-				nr= pupmenu("Add Constraint to New Empty Object%t|Copy Location%x1|Copy Rotation%x2|Copy Scale%x8|Track To%x3|Floor%x4|Locked Track%x5");
+				nr= pupmenu("Add Constraint to New Empty Object%t|Copy Location%x1|Copy Rotation%x2|Copy Scale%x8|Limit Location%x13|Limit Rotation%x14|Limit Scale%x15|Track To%x3|Floor%x4|Locked Track%x5");
 		}
 	}
 	
@@ -718,6 +724,10 @@ void add_constraint(int only_IK)
 		}
 		else if(nr==7) con = add_new_constraint(CONSTRAINT_TYPE_STRETCHTO);
 		else if(nr==8) con = add_new_constraint(CONSTRAINT_TYPE_SIZELIKE);
+		else if(nr==13) con = add_new_constraint(CONSTRAINT_TYPE_LOCLIMIT);
+		else if(nr==14) con = add_new_constraint(CONSTRAINT_TYPE_ROTLIMIT);
+		else if(nr==15) con = add_new_constraint(CONSTRAINT_TYPE_SIZELIMIT);
+		else if(nr==16) con = add_new_constraint(CONSTRAINT_TYPE_ACTION);
 		
 		if(con==NULL) return;	/* paranoia */
 		
@@ -739,7 +749,7 @@ void add_constraint(int only_IK)
 	else if(obsel) {
 		set_constraint_target(con, obsel, NULL);
 	}
-	else if(nr!=11) {	/* add new empty as target */
+	else if(ELEM4(nr, 11, 13, 14, 15)==0) {	/* add new empty as target */
 		Base *base= BASACT, *newbase;
 		Object *obt;
 		
@@ -814,3 +824,66 @@ void ob_clear_constraints(void)
 	
 }
 
+/* con already has the new name */
+void rename_constraint(Object *ob, bConstraint *con, char *oldname)
+{
+	bConstraint *tcon;
+	bConstraintChannel *conchan;
+	ListBase *conlist= NULL;
+	int from_object= 0;
+	char *channame="";
+	
+	/* get context by searching for con (primitive...) */
+	for(tcon= ob->constraints.first; tcon; tcon= tcon->next)
+		if(tcon==con)
+			break;
+	
+	if(tcon) {
+		conlist= &ob->constraints;
+		channame= "Object";
+		from_object= 1;
+	}
+	else if(ob->pose) {
+		bPoseChannel *pchan;
+		
+		for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
+			for(tcon= pchan->constraints.first; tcon; tcon= tcon->next) {
+				if(tcon==con)
+					break;
+			}
+			if(tcon)
+				break;
+		}
+		if(tcon) {
+			conlist= &pchan->constraints;
+			channame= pchan->name;
+		}
+	}
+	
+	if(conlist==NULL) {
+		printf("rename constraint failed\n");	/* should not happen in UI */
+		return;
+	}
+	
+	/* first make sure it's a unique name within context */
+	unique_constraint_name (con, conlist);
+
+	/* own channels */
+	if(from_object) {
+		for(conchan= ob->constraintChannels.first; conchan; conchan= conchan->next) {
+			if( strcmp(oldname, conchan->name)==0 )
+				BLI_strncpy(conchan->name, con->name, sizeof(conchan->name));
+		}
+	}
+	/* own action */
+	if(ob->action) {
+		bActionChannel *achan= get_action_channel(ob->action, channame);
+		if(achan) {
+			conchan= get_constraint_channel(&achan->constraintChannels, oldname);
+			if(conchan)
+				BLI_strncpy(conchan->name, con->name, sizeof(conchan->name));
+		}
+	}
+	
+}
+
