@@ -1,5 +1,5 @@
 /**
- * $Id: editcurve.c,v 1.49 2006/06/25 08:20:45 ton Exp $
+ * $Id: editcurve.c,v 1.56 2007/01/02 20:33:10 nicholasbishop Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -58,6 +58,7 @@
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
+#include "DNA_space_types.h"
 #include "DNA_view3d_types.h"
 #include "DNA_userdef_types.h"
 
@@ -81,6 +82,7 @@
 #include "BIF_mywindow.h"
 #include "BIF_interface.h"
 #include "BIF_transform.h"
+#include "BIF_retopo.h"
 
 #include "BSE_view.h"	/* For persp... */
 #include "BSE_edit.h"
@@ -92,12 +94,9 @@
 #include "mydevice.h"
 #include "blendef.h"
 
-
 #include "BDR_editcurve.h"
 /* still need to eradicate a few :( */
 #define callocstructN(x,y,name) (x*)MEM_callocN((y)* sizeof(x),name)
-/* only used sparingly: */
-#define BEZSELECTED(bezt)   (((bezt)->f1 & 1) || ((bezt)->f2 & 1) || ((bezt)->f3 & 1))
 
 
 ListBase editNurb;
@@ -168,8 +167,7 @@ void printknots()
 	Nurb *nu;
 	int a, num;
 
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		if(isNurbsel(nu) &&  (nu->type & 7)==CU_NURBS) {
 			if(nu->knotsu) {
 				num= KNOTSU(nu);
@@ -180,7 +178,6 @@ void printknots()
 				for(a=0;a<num;a++) printf("knotv %d: %f\n", a, nu->knotsv[a]);
 			}
 		}
-		nu= nu->next;
 	}
 }
 
@@ -257,16 +254,13 @@ void load_editNurb()
 		else {
 			freeNurblist(&(cu->nurb));
 			
-			nu= editNurb.first;
-			while(nu) {
+			for(nu= editNurb.first; nu; nu= nu->next) {
 				newnu= duplicateNurb(nu);
 				BLI_addtail(&(cu->nurb), newnu);
 				
 				if((nu->type & 7)==CU_NURBS) {
 					if(nu->pntsu < nu->orderu) nu->orderu= nu->pntsu;
 				}
-				
-				nu= nu->next;
 			}
 		}
 		
@@ -459,8 +453,7 @@ void setflagsNurb(short flag)
 	BPoint *bp;
 	int a;
 
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		if( (nu->type & 7)==CU_BEZIER) {
 			a= nu->pntsu;
 			bezt= nu->bezt;
@@ -477,7 +470,6 @@ void setflagsNurb(short flag)
 				bp++;
 			}
 		}
-		nu= nu->next;
 	}
 }
 
@@ -488,8 +480,7 @@ void rotateflagNurb(short flag, float *cent, float rotmat[][3])
 	BPoint *bp;
 	int a;
 
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		if((nu->type & 7)==CU_NURBS) {
 			bp= nu->bp;
 			a= nu->pntsu*nu->pntsv;
@@ -507,7 +498,6 @@ void rotateflagNurb(short flag, float *cent, float rotmat[][3])
 				bp++;
 			}
 		}
-		nu= nu->next;
 	}
 }
 
@@ -520,8 +510,7 @@ void translateflagNurb(short flag, float *vec)
 	BPoint *bp;
 	int a;
 
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		if( (nu->type & 7)==CU_BEZIER) {
 			a= nu->pntsu;
 			bezt= nu->bezt;
@@ -542,8 +531,6 @@ void translateflagNurb(short flag, float *vec)
 		}
 
 		test2DNurb(nu);
-
-		nu= nu->next;
 	}
 }
 
@@ -553,8 +540,7 @@ void weightflagNurb(short flag, float w, int mode)	/* mode==0: replace, mode==1:
 	BPoint *bp;
 	int a;
 
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		if((nu->type & 7)==CU_NURBS) {
 			a= nu->pntsu*nu->pntsv;
 			bp= nu->bp;
@@ -566,7 +552,6 @@ void weightflagNurb(short flag, float w, int mode)	/* mode==0: replace, mode==1:
 				bp++;
 			}
 		}
-		nu= nu->next;
 	}
 }
 
@@ -988,10 +973,8 @@ void switchdirectionNurb2(void)
 	if(G.obedit->lay & G.vd->lay);
 	else return;
 	
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		if( isNurbsel(nu) ) switchdirectionNurb(nu);
-		nu= nu->next;
 	}
 	
 	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
@@ -1056,8 +1039,7 @@ void deselectall_nurb()
 	BIF_undo_push("Deselect all");
 	
 	a= 0;
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		if((nu->type & 7)==CU_BEZIER) {
 			b= nu->pntsu;
 			bezt= nu->bezt;
@@ -1093,11 +1075,9 @@ void deselectall_nurb()
 			}
 		}
 		if(a) break;
-		nu= nu->next;
 	}
 
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		if((nu->type & 7)==1) {
 			b= nu->pntsu;
 			bezt= nu->bezt;
@@ -1128,7 +1108,6 @@ void deselectall_nurb()
 				bp++;
 			}
 		}
-		nu= nu->next;
 	}
 	countall();
 	allqueue(REDRAWVIEW3D, 0);
@@ -1145,8 +1124,7 @@ void hideNurb(int swap)
 
 	BIF_undo_push("Hide");
 
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		if((nu->type & 7)==CU_BEZIER) {
 			bezt= nu->bezt;
 			a= nu->pntsu;
@@ -1182,7 +1160,6 @@ void hideNurb(int swap)
 			}
 			if(sel==nu->pntsu*nu->pntsv) nu->hide= 1;
 		}
-		nu= nu->next;
 	}
 
 	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
@@ -1200,8 +1177,7 @@ void revealNurb()
 
 	if(G.obedit==0) return;
 
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		nu->hide= 0;
 		if((nu->type & 7)==CU_BEZIER) {
 			bezt= nu->bezt;
@@ -1227,7 +1203,6 @@ void revealNurb()
 				bp++;
 			}
 		}
-		nu= nu->next;
 	}
 
 	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
@@ -1246,8 +1221,7 @@ void selectswapNurb()
 
 	if(G.obedit==0) return;
 
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		if((nu->type & 7)==CU_BEZIER) {
 			bezt= nu->bezt;
 			a= nu->pntsu;
@@ -1274,7 +1248,6 @@ void selectswapNurb()
 				bp++;
 			}
 		}
-		nu= nu->next;
 	}
 
 	countall();
@@ -1301,8 +1274,7 @@ void subdivideNurb()
 
    // printf("*** subdivideNurb: entering subdivide\n");
 
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		amount= 0;
 		if((nu->type & 7)==CU_BEZIER) {
         /* 
@@ -1367,6 +1339,9 @@ void subdivideNurb()
 						if(a==0 && (nu->flagu & 1)) {VECCOPY(beztnew->vec[0], vec+6);}
 						else {VECCOPY(bezt->vec[0], vec+6);}
 						
+						beztn->radius = (prevbezt->radius + bezt->radius)/2.0f;
+						beztn->weight = (prevbezt->weight + bezt->weight)/2.0f;
+						
 						beztn++;
 					}
 
@@ -1651,7 +1626,6 @@ void subdivideNurb()
 			MEM_freeN(vsel);
 
 		} /* End of 'if((nu->type & 7)==CU_NURBS)'  */
-		nu= nu->next;
 	}
 
 
@@ -1731,8 +1705,7 @@ static void findselectedNurbvert(Nurb **nu, BezTriple **bezt, BPoint **bp)
 	*nu= 0;
 	*bezt= 0;
 	*bp= 0;
-	nu1= editNurb.first;
-	while(nu1) {
+	for(nu1= editNurb.first; nu1; nu1= nu1->next) {
 		if((nu1->type & 7)==CU_BEZIER) {
 			bezt1= nu1->bezt;
 			a= nu1->pntsu;
@@ -1779,7 +1752,6 @@ static void findselectedNurbvert(Nurb **nu, BezTriple **bezt, BPoint **bp)
 				bp1++;
 			}
 		}
-		nu1= nu1->next;
 	}
 }
 
@@ -1801,6 +1773,8 @@ int convertspline(short type, Nurb *nu)
 				VECCOPY(bezt->vec[1], bp->vec);
 				bezt->f1=bezt->f2=bezt->f3= bp->f1;
 				bezt->h1= bezt->h2= HD_VECT;
+				bezt->weight= bp->weight;
+				bezt->radius= bp->radius;
 				bp++;
 				bezt++;
 			}
@@ -1840,6 +1814,8 @@ int convertspline(short type, Nurb *nu)
 					bp->vec[3]= 1.0;
 					bp->f1= bezt->f2;
 					nr-= 2;
+					bp->radius= bezt->radius;
+					bp->weight= bezt->weight;
 					bp++;
 				}
 				else {
@@ -1849,6 +1825,8 @@ int convertspline(short type, Nurb *nu)
 						if(c==0) bp->f1= bezt->f1;
 						else if(c==1) bp->f1= bezt->f2;
 						else bp->f1= bezt->f3;
+						bp->radius= bezt->radius;
+						bp->weight= bezt->weight;
 						bp++;
 					}
 				}
@@ -1898,6 +1876,8 @@ int convertspline(short type, Nurb *nu)
 					bp++;
 					VECCOPY(bezt->vec[2], bp->vec);
 					bezt->f3= bp->f1;
+					bezt->radius= bp->radius;
+					bezt->weight= bp->weight;
 					bp++;
 					bezt++;
 				}
@@ -1923,13 +1903,11 @@ void setsplinetype(short type)
 		return;
 	}
 	
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		if(isNurbsel(nu)) {
 			if (convertspline(type, nu))
 				error("no conversion possible");
 		}
-		nu= nu->next;
 	}
 	BIF_undo_push("Set spline type");
 	
@@ -1996,8 +1974,7 @@ void make_selection_list_nurb()
 	float dist, headdist, taildist;
 	int a;
 	
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		if( isNurbsel(nu) ) {
 			
 			nus = (NurbSort*)MEM_callocN(sizeof(NurbSort), "sort");
@@ -2014,7 +1991,6 @@ void make_selection_list_nurb()
 			
 			
 		}
-		nu= nu->next;
 	}
 
 	/* just add the first one */
@@ -2264,8 +2240,7 @@ void addsegment_nurb()
 	}
 	
 	/* find both nurbs and points, nu1 will be put behind nu2 */
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		if((nu->flagu & 1)==0) {    /* not cyclic */
 			if( (nu->type & 7)==CU_BEZIER ) {
 				bezt= nu->bezt;
@@ -2320,7 +2295,6 @@ void addsegment_nurb()
 				else break;
 			}
 		}
-		nu= nu->next;
 	}
 
 	if((nu1 && nu2) && (nu1!=nu2)) {
@@ -2564,14 +2538,12 @@ static void spin_nurb(float *dvec, short mode)
 	}
 
 	if(ok) {
-		nu= editNurb.first;
-		while(nu) {
+		for(nu= editNurb.first; nu; nu= nu->next) {
 			if(isNurbsel(nu)) {
 				nu->orderv= 4;
 				nu->flagv |= 1;
 				makeknots(nu, 2, nu->flagv>>1);
 			}
-			nu= nu->next;
 		}
 	}
 	
@@ -2699,6 +2671,8 @@ void addvert_Nurb(int mode)
 		}
 	}
 
+	retopo_do_all();
+
 	test2DNurb(nu);
 	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 	countall();
@@ -2727,10 +2701,8 @@ void extrude_nurb()
 	if(G.obedit && G.obedit->type==OB_SURF) {
 
 		/* first test: curve? */
-		nu= editNurb.first;
-		while(nu) {
+		for(nu= editNurb.first; nu; nu= nu->next) {
 			if(nu->pntsv==1 && isNurbsel_count(nu)==1 ) break;
-			nu= nu->next;
 		}
 		if(nu) {
 			addvert_Nurb('e');
@@ -2761,15 +2733,14 @@ void makecyclicNurb()
 	float *fp;
 	int a, b, cyclmode=0;
 
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		if( nu->pntsu>1 || nu->pntsv>1) {
 			if( (nu->type & 7)==0 ) {
 				a= nu->pntsu;
 				bp= nu->bp;
 				while(a--) {
 					if( bp->f1 & 1 ) {
-						if(nu->flagu & 1) nu->flagu--;
+						if(nu->flagu & CU_CYCLIC) nu->flagu--;
 						else nu->flagu++;
 						break;
 					}
@@ -2781,7 +2752,7 @@ void makecyclicNurb()
 				bezt= nu->bezt;
 				while(a--) {
 					if( BEZSELECTED(bezt) ) {
-						if(nu->flagu & 1) nu->flagu--;
+						if(nu->flagu & CU_CYCLIC) nu->flagu--;
 						else nu->flagu++;
 						break;
 					}
@@ -2794,7 +2765,7 @@ void makecyclicNurb()
 				bp= nu->bp;
 				while(a--) {
 					if( bp->f1 & 1 ) {
-						if(nu->flagu & 1) nu->flagu--;
+						if(nu->flagu & CU_CYCLIC) nu->flagu--;
 						else {
 							nu->flagu++;
 							nu->flagu &= ~2;	/* endpoint flag, fixme */
@@ -2823,7 +2794,7 @@ void makecyclicNurb()
 	
 					if( bp->f1 & 1) {
 						if(cyclmode==1 && nu->pntsu>1) {
-							if(nu->flagu & 1) nu->flagu--;
+							if(nu->flagu & CU_CYCLIC) nu->flagu--;
 							else {
 								nu->flagu++;
 								fp= MEM_mallocN(sizeof(float)*KNOTSU(nu), "makecyclicN");
@@ -2855,7 +2826,6 @@ void makecyclicNurb()
 	
 			}
 		}
-		nu= nu->next;
 	}
 	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 	BIF_undo_push("Cyclic");
@@ -2923,8 +2893,7 @@ void selectrow_nurb()
 	if(lastselbp==NULL) return;
 
 	/* find the correct nurb and toggle with u of v */
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		bp= nu->bp;
 		for(v=0; v<nu->pntsv; v++) {
 			for(u=0; u<nu->pntsu; u++, bp++) {
@@ -2959,12 +2928,411 @@ void selectrow_nurb()
 			allqueue(REDRAWVIEW3D, 0);
 			return;
 		}
-		nu= nu->next;
 	}
 	BIF_undo_push("Select Row");
 	
 }
 
+void selectends_nurb(int selFirst)
+{
+	Nurb *nu;
+	BPoint *bp;
+	BezTriple *bezt;
+	int a;
+
+	if(G.obedit==0) return;
+
+	for(nu= editNurb.first; nu; nu= nu->next) {
+		if((nu->type & 7)==CU_BEZIER) {
+			a= nu->pntsu;
+			
+			/* which point? */
+			if (selFirst==0) /* select last */ 
+				bezt= (nu->bezt + (a-1));
+			else /* select first */
+				bezt= nu->bezt;
+			
+			if (bezt->hide == 0) {
+				/* check if anything is selected */
+				if (bezt->f1 & 1 || bezt->f2 & 1 || bezt->f3 & 1) {
+					/* deselct all handles */
+					bezt->f1 &= ~1;
+					bezt->f2 &= ~1;
+					bezt->f3 &= ~1;
+				} 
+				else {
+					/* just select all handles */
+					bezt->f1 |= 1;
+					bezt->f2 |= 1;
+					bezt->f3 |= 1;
+				}
+			}
+		}
+		else {
+			/* ummm... doesn't really make sense, but... */
+			a= nu->pntsu*nu->pntsv;
+			
+			/* which point? */
+			if (selFirst==0) /* select last */
+				bp= (nu->bp + (a-1));
+			else /* select first */
+				bp= nu->bp;
+			
+			if (bp->hide == 0) {
+				if (bp->f1 & 1) /* deselect */
+					bp->f1 &= ~1;
+				else /* select */
+					bp->f1 |= 1;
+			}
+		}
+	}
+	
+	countall();
+	allqueue(REDRAWVIEW3D, 0);
+	BIF_undo_push("Select/Deselect End");
+}
+
+void select_next_nurb()
+{
+	Nurb *nu;
+	BezTriple *bezt;
+	int *selectFlags; /* array of ints defining selection status */
+	int totCount=0, totSel=0, totChange=0;
+	int i=0;
+
+	if(G.obedit==0) return;
+	
+	for(nu= editNurb.first; nu; nu= nu->next) {
+		/* check what type of curve/nurb it is */
+		if((nu->type & 7)==CU_BEZIER) {			
+			totCount= nu->pntsu;
+			selectFlags= MEM_callocN(sizeof(int)*totCount, "selectlist");
+			
+			/* find out which beztriples are selected */
+			for (i=0; i<totCount; i++) {
+				bezt= (nu->bezt + i);
+				selectFlags[i]= BEZSELECTED(bezt);
+				if (selectFlags[i]) totSel++; 
+			}
+			
+			/* check if anything is selected at all */
+			if (totSel==0) {
+				MEM_freeN(selectFlags);
+				continue;
+			}
+				
+			/* find out which ones deserve an extra flag */
+			for (i=0; i<totCount; i++) {
+				if (selectFlags[i] == 1) {
+					if (((i+1) < totCount) && (selectFlags[i+1] == 0)) {
+						selectFlags[i+1]= 2;
+						totChange++;
+					}
+				}
+			}
+			
+			/* set select flags based on select flag */
+			for (i=0; i<totCount; i++) {
+				if (selectFlags[i] == 2) {
+					bezt = (nu->bezt + i);
+					
+					if (bezt->hide == 0) {
+						bezt->f1 |= 1;
+						bezt->f2 |= 1;
+						bezt->f3 |= 1;
+					}
+				}
+			}
+			
+			/* free tempolary array */
+			MEM_freeN(selectFlags);
+		}
+	}
+	
+	countall();
+	allqueue(REDRAWVIEW3D, 0);
+	BIF_undo_push("Select Next");
+}
+
+void select_prev_nurb()
+{
+	Nurb *nu;
+	BezTriple *bezt;
+	int *selectFlags; /* array of ints defining selection status */
+	int totCount=0, totSel=0, totChange=0;
+	int i=0;
+
+	if(G.obedit==0) return;
+	
+	for(nu= editNurb.first; nu; nu= nu->next) {
+		/* check what type of curve/nurb it is */
+		if((nu->type & 7)==CU_BEZIER) {			
+			totCount= nu->pntsu;
+			selectFlags= MEM_callocN(sizeof(int)*totCount, "selectlist");
+			
+			/* find out which beztriples are selected */
+			for (i=0; i<totCount; i++) {
+				bezt= (nu->bezt + i);
+				selectFlags[i]= BEZSELECTED(bezt);
+				if (selectFlags[i]) totSel++; 
+			}
+			
+			/* check if anything is selected at all */
+			if (totSel==0) {
+				MEM_freeN(selectFlags);
+				continue;
+			}
+				
+			/* find out which ones deserve an extra flag */
+			for (i= totCount-1; i>=0; i--) {
+				if (selectFlags[i] == 1) {
+					if (((i-1) >= 0) && (selectFlags[i-1] == 0)) {
+						selectFlags[i-1]= 2;
+						totChange++;
+					}
+				}
+			}
+			
+			/* set select flags based on select flag */
+			for (i=0; i<totCount; i++) {
+				if (selectFlags[i] == 2) {
+					bezt = (nu->bezt + i);
+					
+					if (bezt->hide == 0) {
+						bezt->f1 |= 1;
+						bezt->f2 |= 1;
+						bezt->f3 |= 1;
+					}
+				}
+			}
+			
+			/* free tempolary array */
+			MEM_freeN(selectFlags);
+		}
+	}
+	
+	countall();
+	allqueue(REDRAWVIEW3D, 0);
+	BIF_undo_push("Select Previous");
+}
+
+void select_more_nurb()
+{
+	Nurb *nu;
+	BezTriple *bezt;
+	BPoint *bp;
+	int *selectFlags; /* array of ints defining selection status */
+	int totCount=0, totSel=0, totChange=0, totU=0, totV=0;
+	int i=0, n=0, x=0;
+
+	if(G.obedit==0) return;
+	
+	for(nu= editNurb.first; nu; nu= nu->next) {
+		/* check what type of curve/nurb it is */
+		if((nu->type & 7)==CU_BEZIER) {			
+			totCount= nu->pntsu;
+			selectFlags= MEM_callocN(sizeof(int)*totCount, "selectlist");
+			
+			/* find out which beztriples are selected */
+			for (i=0; i<totCount; i++) {
+				bezt= (nu->bezt + i);
+				selectFlags[i]= BEZSELECTED(bezt);
+				if (selectFlags[i]) totSel++; 
+			}
+			
+			/* check if anything is selected at all */
+			if (totSel==0) {
+				MEM_freeN(selectFlags);
+				continue;
+			}
+				
+			/* find out which ones deserve an extra flag */
+			for (i=0; i<totCount; i++) {
+				if (selectFlags[i] == 1) {
+					if ((totCount > (i+1)) && (selectFlags[i+1] == 0)) {
+						selectFlags[i+1] = 2;
+						totChange++;
+					}
+				}
+				else {
+					if ((totCount > (i+1)) && (selectFlags[i+1] == 1)) {
+						selectFlags[i] = 2;
+						totChange++;
+					}
+				}
+			}
+			
+			/* set select flags based on select flag */
+			for (i=0; i<totCount; i++) {
+				if (selectFlags[i] == 2) {
+					bezt = (nu->bezt + i);
+					
+					if (bezt->hide == 0) {
+						bezt->f1 |= 1;
+						bezt->f2 |= 1;
+						bezt->f3 |= 1;
+					}
+				}
+			}
+			
+			/* free tempolary array */
+			MEM_freeN(selectFlags);
+		}
+		else {
+			totCount= nu->pntsu*nu->pntsv;
+			totU= nu->pntsu;
+			totV= nu->pntsv;
+			selectFlags= MEM_callocN(sizeof(int)*totCount, "selectlist");
+			
+			/* find out which bpoints are selected */
+			for (i=0; i<totCount; i++) {
+				bp = (nu->bp + i);
+				selectFlags[i]= (bp->f1 & 1);
+				if (selectFlags[i]) totSel++;
+			}
+			
+			/* check if anything selected */
+			if (totSel==0) continue;
+				
+			/* find out which ones deserve an extra flag */
+			/* FIXME: why the heck does this go wrong? */
+			for (i=0; i<totV; i++) {
+				/* search column */
+				for (n=0; n<totU; n++) {
+					x = ((i * totU) + n);
+					
+					if (selectFlags[i] == 1) {
+						if ((totU > (n+1)) && (selectFlags[x+1] == 0))
+							selectFlags[x+1] = 2;
+					}
+					else {
+						if ((totU > (n+1)) && (selectFlags[x+1] == 1))
+							selectFlags[i] = 2;
+					}
+				}
+				
+				/* search row */
+				/*
+				if (selectFlags[(i*totU)] == 1) {
+					if ((totV > (i+1)) && (selectFlags[((i+1)*totU)] == 0))
+						selectFlags[((i+1)*totU)] = 2;
+				}
+				else {
+					if ((totV > (i+1)) && (selectFlags[((i+1)*totU)] == 1))
+						selectFlags[(i*totU)] = 2;
+				} */
+			}
+			
+			/* set select flags based on select flag */
+			for (i=0; i<totCount; i++) {
+				if (selectFlags[i] == 2) {
+					bp = (nu->bp + i);
+					if (bp->hide == 0) bp->f1 |= 1;
+				}
+			}
+			
+			/* free tempolary array */
+			MEM_freeN(selectFlags);
+		}
+	}
+	
+	countall();
+	allqueue(REDRAWVIEW3D, 0);
+	BIF_undo_push("Select More");
+}
+
+void select_less_nurb()
+{
+	Nurb *nu;
+	/* BPoint *bp; */
+	BezTriple *bezt;
+	int *selectFlags; /* array of ints*/
+	int totCount=0, totSel=0, totChange=0, i;
+
+	if(G.obedit==0) return;
+
+	for(nu= editNurb.first; nu; nu= nu->next) {
+		/* check what type of curve/nurb it is */
+		if((nu->type & 7)==CU_BEZIER) {			
+			totCount= nu->pntsu;
+			totSel= 0;
+			selectFlags= MEM_callocN(sizeof(int)*totCount, "selectlist");
+			
+			/* find out which beztriples are selected */
+			for (i=0; i<totCount; i++) {
+				bezt= (nu->bezt + i);
+				selectFlags[i]= BEZSELECTED(bezt);
+				if (selectFlags[i]) totSel++; 
+			}
+			
+			/* determine best course of action */
+			if (totSel<=1) {
+				/* not enough to select-less of */
+				MEM_freeN(selectFlags);
+				continue;
+			}
+			else if (totSel==totCount) {
+				/* deselect first and last points */
+				selectFlags[0]= 2;
+				selectFlags[totCount-1]= 2;
+				totChange= 2;
+			}
+			else {
+				/* find out which ones deserve an extra flag */
+				for (i=0; i<totCount; i++) {
+					if (selectFlags[i] == 1) {
+						if ((i < (totCount-1)) && (selectFlags[i+1] == 0)) {
+							selectFlags[i] = 2;
+							totChange++;
+						}
+						else if ((i != 0) && (selectFlags[i-1] == 0)) {
+							selectFlags[i] = 2;
+							totChange++;
+						}
+					}
+				}
+				
+				/* second pass - for the ends. done after as it may affect results */
+				for (i=0; i<totCount; i++) {
+					if (selectFlags[i] == 1) {
+						if ((i==0) && (i != (totCount-1)) && (selectFlags[i+1] == 1)) {
+							selectFlags[i] = 2;
+							totChange++;
+						}
+						else if ((i==(totCount-1)) && (i!=0) && (selectFlags[i-1] == 1)) {
+							selectFlags[i] = 2;
+							totChange++;
+						}
+					}
+				}
+			}
+			
+			/* set select flags based on select flag */
+			for (i=0; i<totCount; i++) {
+				if (selectFlags[i] == 2) {
+					bezt = (nu->bezt + i);
+					
+					if (bezt->hide == 0) {
+						bezt->f1 &= ~1;
+						bezt->f2 &= ~1;
+						bezt->f3 &= ~1;
+					}
+				}
+			}
+			
+			/* free tempolary array */
+			MEM_freeN(selectFlags);
+		}
+		else {
+			/* TODO: figure out method of nurbs surfaces */
+		}
+	}
+	
+	countall();
+	allqueue(REDRAWVIEW3D, 0);
+	BIF_undo_push("Select Less");
+}
+
 void adduplicate_nurb()
 {
 
@@ -3260,13 +3628,11 @@ void nurb_set_smooth(short event)
 	
 	if(G.obedit->type != OB_CURVE) return;
 	
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		if(isNurbsel(nu)) {
 			if(event==1) nu->flag |= CU_SMOOTH;
 			else if(event==0) nu->flag &= ~CU_SMOOTH;
 		}
-		nu= nu->next;
 	}
 	
 	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
@@ -3346,8 +3712,8 @@ int join_curve(int type)
 	
 	DAG_scene_sort(G.scene);	// because we removed object(s), call before editmode!
 	
-	enter_editmode();
-	exit_editmode(1);
+	enter_editmode(EM_WAITCURSOR);
+	exit_editmode(EM_FREEDATA|EM_WAITCURSOR);
 	
 	allqueue(REDRAWVIEW3D, 0);
 	allqueue(REDRAWBUTSEDIT, 0);
@@ -3415,6 +3781,7 @@ Nurb *addNurbprim(int type, int stype, int newname)
 			bezt= nu->bezt;
 			bezt->h1= bezt->h2= HD_ALIGN;
 			bezt->f1= bezt->f2= bezt->f3= 1;
+			bezt->radius = 1.0;
 
 			for(a=0;a<3;a++) {
 				VECCOPY(bezt->vec[a], cent);
@@ -3429,6 +3796,7 @@ Nurb *addNurbprim(int type, int stype, int newname)
 			bezt++;
 			bezt->h1= bezt->h2= HD_ALIGN;
 			bezt->f1= bezt->f2= bezt->f3= 1;
+			bezt->radius = bezt->weight = 1.0;
 
 			for(a=0;a<3;a++) {
 				VECCOPY(bezt->vec[a], cent);
@@ -3449,6 +3817,7 @@ Nurb *addNurbprim(int type, int stype, int newname)
 				VECCOPY(bp->vec, cent);
 				bp->vec[3]= 1.0;
 				bp->f1= 1;
+				bp->radius = bp->weight = 1.0;
 			}
 
 			bp= nu->bp;
@@ -3472,7 +3841,7 @@ Nurb *addNurbprim(int type, int stype, int newname)
 
 		}
 		break;
-	case 6:	/* 5 point pad */
+	case 6:	/* 5 point path */
 		nu->pntsu= 5;
 		nu->pntsv= 1;
 		nu->orderu= 5;
@@ -3485,6 +3854,7 @@ Nurb *addNurbprim(int type, int stype, int newname)
 			VECCOPY(bp->vec, cent);
 			bp->vec[3]= 1.0;
 			bp->f1= 1;
+			bp->radius = bp->weight = 1.0;
 		}
 
 		bp= nu->bp;
@@ -3523,7 +3893,8 @@ Nurb *addNurbprim(int type, int stype, int newname)
 			bezt->f1= bezt->f2= bezt->f3= 1;
 			bezt->vec[1][0]+= -G.vd->grid;
 			for(a=0;a<3;a++) Mat3MulVecfl(imat,bezt->vec[a]);
-
+			bezt->radius = bezt->weight = 1.0;
+			
 			bezt++;
 			for(a=0;a<3;a++) {
 				VECCOPY(bezt->vec[a], cent);
@@ -3532,6 +3903,7 @@ Nurb *addNurbprim(int type, int stype, int newname)
 			bezt->f1= bezt->f2= bezt->f3= 1;
 			bezt->vec[1][1]+= G.vd->grid;
 			for(a=0;a<3;a++) Mat3MulVecfl(imat,bezt->vec[a]);
+			bezt->radius = bezt->weight = 1.0;
 
 			bezt++;
 			for(a=0;a<3;a++) {
@@ -3541,6 +3913,7 @@ Nurb *addNurbprim(int type, int stype, int newname)
 			bezt->f1= bezt->f2= bezt->f3= 1;
 			bezt->vec[1][0]+= G.vd->grid;
 			for(a=0;a<3;a++) Mat3MulVecfl(imat,bezt->vec[a]);
+			bezt->radius = bezt->weight = 1.0;
 
 			bezt++;
 			for(a=0;a<3;a++) {
@@ -3550,6 +3923,7 @@ Nurb *addNurbprim(int type, int stype, int newname)
 			bezt->f1= bezt->f2= bezt->f3= 1;
 			bezt->vec[1][1]+= -G.vd->grid;
 			for(a=0;a<3;a++) Mat3MulVecfl(imat,bezt->vec[a]);
+			bezt->radius = bezt->weight = 1.0;
 
 			calchandlesNurb(nu);
 		}
@@ -3576,6 +3950,8 @@ Nurb *addNurbprim(int type, int stype, int newname)
 				if(a & 1) bp->vec[3]= 0.25*sqrt(2.0);
 				else bp->vec[3]= 1.0;
 				Mat3MulVecfl(imat,bp->vec);
+				bp->radius = bp->weight = 1.0;
+				
 				bp++;
 			}
 
@@ -3875,8 +4251,7 @@ void clear_tilt()
 
 	if(okee("Clear tilt")==0) return;
 
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		if( nu->bezt ) {
 			bezt= nu->bezt;
 			a= nu->pntsu;
@@ -3893,7 +4268,6 @@ void clear_tilt()
 				bp++;
 			}
 		}
-		nu= nu->next;
 	}
 
 	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
@@ -3941,11 +4315,9 @@ static void undoCurve_to_editCurve(void *lbv)
 	freeNurblist(&editNurb);
 
 	/* copy  */
-	nu= lb->first;
-	while(nu) {
+	for(nu= lb->first; nu; nu= nu->next) {
 		newnu= duplicateNurb(nu);
 		BLI_addtail(&editNurb, newnu);
-		nu= nu->next;
 	}
 	/* restore */
 	for(nr=0, lastnu= editNurb.first; lastnu; lastnu = lastnu->next, nr++) if(nr==lastnunr) break;
@@ -3960,11 +4332,9 @@ static void *editCurve_to_undoCurve(void)
 	lb= MEM_callocN(sizeof(ListBase), "listbase undo");
 	
 	/* copy  */
-	nu= editNurb.first;
-	while(nu) {
+	for(nu= editNurb.first; nu; nu= nu->next) {
 		newnu= duplicateNurb(nu);
 		BLI_addtail(lb, newnu);
-		nu= nu->next;
 	}
 	return lb;
 }
