@@ -1,5 +1,5 @@
 /**
- * $Id: editdeform.c,v 1.26 2005/12/19 11:55:31 ton Exp $
+ * $Id: editdeform.c,v 1.37 2007/01/04 22:16:49 nicholasbishop Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -37,6 +37,7 @@
 #include "MEM_guardedalloc.h"
 
 #include "DNA_curve_types.h"
+#include "DNA_lattice_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
@@ -44,12 +45,15 @@
 #include "BLI_blenlib.h"
 #include "BLI_editVert.h"
 
+#include "BKE_customdata.h"
 #include "BKE_DerivedMesh.h"
 #include "BKE_depsgraph.h"
-#include "BKE_global.h"
 #include "BKE_deform.h"
 #include "BKE_displist.h"
+#include "BKE_global.h"
+#include "BKE_lattice.h"
 #include "BKE_mesh.h"
+#include "BKE_utildefines.h"
 
 #include "BIF_editdeform.h"
 #include "BIF_editmesh.h"
@@ -57,33 +61,34 @@
 
 #include "BSE_edit.h"
 
+#include "editmesh.h"
+#include "multires.h"
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-/* ----------------- function prototypes ---------------- */
-void remove_vert_def_nr (Object *, int , int );
-void add_vert_defnr (Object *, int , int vertnum, float , int );
-
-
+/* only in editmode */
 void sel_verts_defgroup (int select)
 {
-	EditMesh *em = G.editMesh;
-	EditVert		*eve;
-	Object			*ob;
-	int				i;
+	EditVert *eve;
+	Object *ob;
+	int i;
+	MDeformVert *dvert;
 
-	ob=G.obedit;
+	ob= G.obedit;
 
 	if (!ob)
 		return;
 
 	switch (ob->type){
 	case OB_MESH:
-		for (eve=em->verts.first; eve; eve=eve->next){
-			if (eve->totweight){
-				for (i=0; i<eve->totweight; i++){
-					if (eve->dw[i].def_nr == (ob->actdef-1)){
+		for (eve=G.editMesh->verts.first; eve; eve=eve->next){
+			dvert= CustomData_em_get(&G.editMesh->vdata, eve->data, CD_MDEFORMVERT);
+
+			if (dvert && dvert->totweight){
+				for (i=0; i<dvert->totweight; i++){
+					if (dvert->dw[i].def_nr == (ob->actdef-1)){
 						if (select) eve->f |= SELECT;
 						else eve->f &= ~SELECT;
 						
@@ -92,14 +97,53 @@ void sel_verts_defgroup (int select)
 				}
 			}
 		}
+		/* this has to be called, because this function operates on vertices only */
+		if(select) EM_select_flush();	// vertices to edges/faces
+		else EM_deselect_flush();
+		
 		break;
+	case OB_LATTICE:
+		if(editLatt->dvert) {
+			BPoint *bp;
+			int a, tot;
+			
+			dvert= editLatt->dvert;
+
+			tot= editLatt->pntsu*editLatt->pntsv*editLatt->pntsw;
+			for(a=0, bp= editLatt->def; a<tot; a++, bp++, dvert++) {
+				for (i=0; i<dvert->totweight; i++){
+					if (dvert->dw[i].def_nr == (ob->actdef-1)) {
+						if(select) bp->f1 |= SELECT;
+						else bp->f1 &= ~SELECT;
+						
+						break;
+					}
+				}
+			}
+		}	
+		break;
+		
 	default:
 		break;
 	}
+	
 	countall();
-	/* this has to be called, because this function operates on vertices only */
-	if(select) EM_select_flush();	// vertices to edges/faces
-	else EM_deselect_flush();
+	
+}
+
+/* check if deform vertex has defgroup index */
+MDeformWeight *get_defweight (MDeformVert *dv, int defgroup)
+{
+	int i;
+	
+	if (!dv || defgroup<0)
+		return NULL;
+	
+	for (i=0; i<dv->totweight; i++){
+		if (dv->dw[i].def_nr == defgroup)
+			return dv->dw+i;
+	}
+	return NULL;
 }
 
 /* Ensures that mv has a deform weight entry for
@@ -108,16 +152,15 @@ void sel_verts_defgroup (int select)
 MDeformWeight *verify_defweight (MDeformVert *dv, int defgroup)
 {
 	MDeformWeight *newdw;
-	int	i;
 
+	/* do this check always, this function is used to check for it */
 	if (!dv || defgroup<0)
 		return NULL;
-
-	for (i=0; i<dv->totweight; i++){
-		if (dv->dw[i].def_nr == defgroup)
-			return dv->dw+i;
-	}
-
+	
+	newdw = get_defweight (dv, defgroup);
+	if (newdw)
+		return newdw;
+	
 	newdw = MEM_callocN (sizeof(MDeformWeight)*(dv->totweight+1), "deformWeight");
 	if (dv->dw){
 		memcpy (newdw, dv->dw, sizeof(MDeformWeight)*dv->totweight);
@@ -125,7 +168,7 @@ MDeformWeight *verify_defweight (MDeformVert *dv, int defgroup)
 	}
 	dv->dw=newdw;
 	
-	dv->dw[dv->totweight].weight=0;
+	dv->dw[dv->totweight].weight=0.0f;
 	dv->dw[dv->totweight].def_nr=defgroup;
 	/* Group index */
 	
@@ -160,12 +203,9 @@ bDeformGroup *add_defgroup_name (Object *ob, char *name)
 
 void del_defgroup (Object *ob)
 {
-	EditMesh *em = G.editMesh;
 	bDeformGroup	*defgroup;
-	EditVert		*eve;
 	int				i;
 
-
 	if (!ob)
 		return;
 
@@ -180,14 +220,35 @@ void del_defgroup (Object *ob)
 	remove_verts_defgroup(1);
 
 	/* Make sure that any verts with higher indices are adjusted accordingly */
-	for (eve=em->verts.first; eve; eve=eve->next){
-		for (i=0; i<eve->totweight; i++){
-			if (eve->dw[i].def_nr > (ob->actdef-1))
-				eve->dw[i].def_nr--;
+	if(ob->type==OB_MESH) {
+		EditMesh *em = G.editMesh;
+		EditVert *eve;
+		MDeformVert *dvert;
+		
+		for (eve=em->verts.first; eve; eve=eve->next){
+			dvert= CustomData_em_get(&G.editMesh->vdata, eve->data, CD_MDEFORMVERT);
+
+			if (dvert)
+				for (i=0; i<dvert->totweight; i++)
+					if (dvert->dw[i].def_nr > (ob->actdef-1))
+						dvert->dw[i].def_nr--;
+		}
+	}
+	else {
+		BPoint *bp;
+		MDeformVert *dvert= editLatt->dvert;
+		int a, tot;
+		
+		tot= editLatt->pntsu*editLatt->pntsv*editLatt->pntsw;
+		for(a=0, bp= editLatt->def; a<tot; a++, bp++, dvert++) {
+			for (i=0; i<dvert->totweight; i++){
+				if (dvert->dw[i].def_nr > (ob->actdef-1))
+					dvert->dw[i].def_nr--;
+			}
 		}
 	}
 
-	/* Update the active material index if necessary */
+	/* Update the active deform index if necessary */
 	if (ob->actdef==BLI_countlist(&ob->defbase))
 		ob->actdef--;
 	
@@ -197,24 +258,28 @@ void del_defgroup (Object *ob)
 	/* remove all dverts */
 	if(ob->actdef==0) {
 		Mesh *me= ob->data;
-		free_dverts(me->dvert, me->totvert);
+		CustomData_free_layer_active(&me->vdata, CD_MDEFORMVERT, me->totvert);
 		me->dvert= NULL;
 	}
 }
 
-void create_dverts(Mesh *me)
+void create_dverts(ID *id)
 {
-	/* create deform verts for the mesh
+	/* create deform verts
 	 */
-	int i;
 
-	me->dvert= MEM_mallocN(sizeof(MDeformVert)*me->totvert, "deformVert");
-	for (i=0; i < me->totvert; ++i) {
-		me->dvert[i].totweight = 0;
-		me->dvert[i].dw        = NULL;
+	if( GS(id->name)==ID_ME) {
+		Mesh *me= (Mesh *)id;
+		me->dvert= CustomData_add_layer(&me->vdata, CD_MDEFORMVERT, CD_CALLOC, NULL, me->totvert);
+	}
+	else if( GS(id->name)==ID_LT) {
+		Lattice *lt= (Lattice *)id;
+		lt->dvert= MEM_callocN(sizeof(MDeformVert)*lt->pntsu*lt->pntsv*lt->pntsw, "lattice deformVert");
 	}
 }
 
+/* for mesh in object mode
+   lattice can be in editmode */
 void remove_vert_def_nr (Object *ob, int def_nr, int vertnum)
 {
 	/* This routine removes the vertex from the deform
@@ -228,18 +293,29 @@ void remove_vert_def_nr (Object *ob, int def_nr, int vertnum)
 	 */
 
 	MDeformWeight *newdw;
-	MDeformVert *dvert;
+	MDeformVert *dvert= NULL;
 	int i;
 
-	/* if this mesh has no deform mesh abort
-	 */
-	if (!((Mesh*)ob->data)->dvert) return;
-
-	/* get the deform mesh cooresponding to the
+	/* get the deform vertices corresponding to the
 	 * vertnum
 	 */
-	dvert = ((Mesh*)ob->data)->dvert + vertnum;
-
+	if(ob->type==OB_MESH) {
+		if( ((Mesh*)ob->data)->dvert )
+			dvert = ((Mesh*)ob->data)->dvert + vertnum;
+	}
+	else if(ob->type==OB_LATTICE) {
+		Lattice *lt= ob->data;
+		
+		if(ob==G.obedit)
+			lt= editLatt;
+		
+		if(lt->dvert)
+			dvert = lt->dvert + vertnum;
+	}
+	
+	if(dvert==NULL)
+		return;
+	
 	/* for all of the deform weights in the
 	 * deform vert
 	 */
@@ -273,28 +349,43 @@ void remove_vert_def_nr (Object *ob, int def_nr, int vertnum)
 			else {
 				MEM_freeN (dvert->dw);
 				dvert->dw = NULL;
+				break;
 			}
 		}
 	}
 
 }
 
+/* for Mesh in Object mode */
+/* allows editmode for Lattice */
 void add_vert_defnr (Object *ob, int def_nr, int vertnum, 
                            float weight, int assignmode)
 {
 	/* add the vert to the deform group with the
 	 * specified number
 	 */
-
-	MDeformVert *dv;
+	MDeformVert *dv= NULL;
 	MDeformWeight *newdw;
 	int	i;
 
-	/* get the vert
-	 */
-	if(((Mesh*)ob->data)->dvert)
-		dv = ((Mesh*)ob->data)->dvert + vertnum;
-	else 
+	/* get the vert */
+	if(ob->type==OB_MESH) {
+		if(((Mesh*)ob->data)->dvert)
+			dv = ((Mesh*)ob->data)->dvert + vertnum;
+	}
+	else if(ob->type==OB_LATTICE) {
+		Lattice *lt;
+		
+		if(ob==G.obedit)
+			lt= editLatt;
+		else
+			lt= ob->data;
+			
+		if(lt->dvert)
+			dv = lt->dvert + vertnum;
+	}
+	
+	if(dv==NULL)
 		return;
 	
 	/* Lets first check to see if this vert is
@@ -363,6 +454,7 @@ void add_vert_defnr (Object *ob, int def_nr, int vertnum,
 	}
 }
 
+/* called while not in editmode */
 void add_vert_to_defgroup (Object *ob, bDeformGroup *dg, int vertnum, 
                            float weight, int assignmode)
 {
@@ -377,11 +469,16 @@ void add_vert_to_defgroup (Object *ob, bDeformGroup *dg, int vertnum,
 	def_nr = get_defgroup_num(ob, dg);
 	if (def_nr < 0) return;
 
-	/* if this mesh has no deform verts then
+	/* if there's no deform verts then
 	 * create some
 	 */
-	if (!((Mesh*)ob->data)->dvert) {
-		create_dverts((Mesh*)ob->data);
+	if(ob->type==OB_MESH) {
+		if (!((Mesh*)ob->data)->dvert)
+			create_dverts(ob->data);
+	}
+	else if(ob->type==OB_LATTICE) {
+		if (!((Lattice*)ob->data)->dvert)
+			create_dverts(ob->data);
 	}
 
 	/* call another function to do the work
@@ -389,19 +486,20 @@ void add_vert_to_defgroup (Object *ob, bDeformGroup *dg, int vertnum,
 	add_vert_defnr (ob, def_nr, vertnum, weight, assignmode);
 }
 
-
-void assign_verts_defgroup (void)
 /* Only available in editmode */
+void assign_verts_defgroup (void)
 {
-	EditMesh *em = G.editMesh;
+	extern float editbutvweight;	/* buttons.c */
 	Object *ob;
 	EditVert *eve;
-	bDeformGroup	*dg, *eg;
-	extern float editbutvweight;	/* buttons.c */
-	int	i, done;
+	bDeformGroup *dg, *eg;
 	MDeformWeight *newdw;
+	MDeformVert *dvert;
+	int	i, done;
+	
+	if(multires_level1_test()) return;
 
-	ob=G.obedit;
+	ob= G.obedit;
 
 	if (!ob)
 		return;
@@ -414,40 +512,60 @@ void assign_verts_defgroup (void)
 
 	switch (ob->type){
 	case OB_MESH:
+		if (!CustomData_has_layer(&G.editMesh->vdata, CD_MDEFORMVERT))
+			EM_add_data_layer(&G.editMesh->vdata, CD_MDEFORMVERT);
+
 		/* Go through the list of editverts and assign them */
-		for (eve=em->verts.first; eve; eve=eve->next){
-			if (eve->f & 1){
+		for (eve=G.editMesh->verts.first; eve; eve=eve->next){
+			dvert= CustomData_em_get(&G.editMesh->vdata, eve->data, CD_MDEFORMVERT);
+
+			if (dvert && (eve->f & 1)){
 				done=0;
 				/* See if this vert already has a reference to this group */
 				/*		If so: Change its weight */
 				done=0;
-				for (i=0; i<eve->totweight; i++){
-					eg = BLI_findlink (&ob->defbase, eve->dw[i].def_nr);
+				for (i=0; i<dvert->totweight; i++){
+					eg = BLI_findlink (&ob->defbase, dvert->dw[i].def_nr);
 					/* Find the actual group */
 					if (eg==dg){
-						eve->dw[i].weight=editbutvweight;
+						dvert->dw[i].weight=editbutvweight;
 						done=1;
 						break;
 					}
 			 	}
 				/*		If not: Add the group and set its weight */
 				if (!done){
-					newdw = MEM_callocN (sizeof(MDeformWeight)*(eve->totweight+1), "deformWeight");
-					if (eve->dw){
-						memcpy (newdw, eve->dw, sizeof(MDeformWeight)*eve->totweight);
-						MEM_freeN (eve->dw);
+					newdw = MEM_callocN (sizeof(MDeformWeight)*(dvert->totweight+1), "deformWeight");
+					if (dvert->dw){
+						memcpy (newdw, dvert->dw, sizeof(MDeformWeight)*dvert->totweight);
+						MEM_freeN (dvert->dw);
 					}
-					eve->dw=newdw;
+					dvert->dw=newdw;
 
-					eve->dw[eve->totweight].weight=editbutvweight;
-					eve->dw[eve->totweight].def_nr=ob->actdef-1;
+					dvert->dw[dvert->totweight].weight= editbutvweight;
+					dvert->dw[dvert->totweight].def_nr= ob->actdef-1;
 
-					eve->totweight++;
+					dvert->totweight++;
 
 				}
 			}
 		}
 		break;
+	case OB_LATTICE:
+		{
+			BPoint *bp;
+			int a, tot;
+			
+			if(editLatt->dvert==NULL)
+				create_dverts(&editLatt->id);
+			
+			tot= editLatt->pntsu*editLatt->pntsv*editLatt->pntsw;
+			for(a=0, bp= editLatt->def; a<tot; a++, bp++) {
+				if(bp->f1 & SELECT)
+					add_vert_defnr (ob, ob->actdef-1, a, editbutvweight, WEIGHT_REPLACE);
+			}
+		}	
+		break;
 	default:
 		printf ("Assigning deformation groups to unknown object type\n");
 		break;
@@ -455,6 +573,7 @@ void assign_verts_defgroup (void)
 
 }
 
+/* mesh object mode, lattice can be in editmode */
 void remove_vert_defgroup (Object *ob, bDeformGroup	*dg, int vertnum)
 {
 	/* This routine removes the vertex from the specified
@@ -468,10 +587,6 @@ void remove_vert_defgroup (Object *ob, bDeformGroup	*dg, int vertnum)
 	if (!ob)
 		return;
 
-	/* if this isn't a mesh abort
-	 */
-	if (ob->type != OB_MESH) return;
-
 	/* get the deform number that cooresponds
 	 * to this deform group, and abort if it
 	 * can not be found.
@@ -484,17 +599,20 @@ void remove_vert_defgroup (Object *ob, bDeformGroup	*dg, int vertnum)
 	remove_vert_def_nr (ob, def_nr, vertnum);
 }
 
-void remove_verts_defgroup (int allverts)
 /* Only available in editmode */
+/* removes from active defgroup, if allverts==0 only selected vertices */
+void remove_verts_defgroup (int allverts)
 {
-	EditMesh *em = G.editMesh;
 	Object *ob;
 	EditVert *eve;
+	MDeformVert *dvert;
 	MDeformWeight *newdw;
-	bDeformGroup	*dg, *eg;
+	bDeformGroup *dg, *eg;
 	int	i;
+	
+	if(multires_level1_test()) return;
 
-	ob=G.obedit;
+	ob= G.obedit;
 
 	if (!ob)
 		return;
@@ -507,68 +625,61 @@ void remove_verts_defgroup (int allverts)
 
 	switch (ob->type){
 	case OB_MESH:
-		for (eve=em->verts.first; eve; eve=eve->next){
-			if (eve->dw && ((eve->f & 1) || allverts)){
-				for (i=0; i<eve->totweight; i++){
+		for (eve=G.editMesh->verts.first; eve; eve=eve->next){
+			dvert= CustomData_em_get(&G.editMesh->vdata, eve->data, CD_MDEFORMVERT);
+
+			if (dvert && dvert->dw && ((eve->f & 1) || allverts)){
+				for (i=0; i<dvert->totweight; i++){
 					/* Find group */
-					eg = BLI_findlink (&ob->defbase, eve->dw[i].def_nr);
+					eg = BLI_findlink (&ob->defbase, dvert->dw[i].def_nr);
 					if (eg == dg){
-						eve->totweight--;
-						if (eve->totweight){
-							newdw = MEM_mallocN (sizeof(MDeformWeight)*(eve->totweight), "deformWeight");
+						dvert->totweight--;
+						if (dvert->totweight){
+							newdw = MEM_mallocN (sizeof(MDeformWeight)*(dvert->totweight), "deformWeight");
 							
-							if (eve->dw){
-								memcpy (newdw, eve->dw, sizeof(MDeformWeight)*i);
-								memcpy (newdw+i, eve->dw+i+1, sizeof(MDeformWeight)*(eve->totweight-i));
-								MEM_freeN (eve->dw);
+							if (dvert->dw){
+								memcpy (newdw, dvert->dw, sizeof(MDeformWeight)*i);
+								memcpy (newdw+i, dvert->dw+i+1, sizeof(MDeformWeight)*(dvert->totweight-i));
+								MEM_freeN (dvert->dw);
 							}
-							eve->dw=newdw;
+							dvert->dw=newdw;
 						}
 						else{
-							MEM_freeN (eve->dw);
-							eve->dw=NULL;
+							MEM_freeN (dvert->dw);
+							dvert->dw=NULL;
+							break;
 						}
 					}
 				}
 			}
 		}
 		break;
+	case OB_LATTICE:
+		
+		if(editLatt->dvert) {
+			BPoint *bp;
+			int a, tot= editLatt->pntsu*editLatt->pntsv*editLatt->pntsw;
+				
+			for(a=0, bp= editLatt->def; a<tot; a++, bp++) {
+				if(allverts || (bp->f1 & SELECT))
+					remove_vert_defgroup (ob, dg, a);
+			}
+		}
+		break;
+		
 	default:
 		printf ("Removing deformation groups from unknown object type\n");
 		break;
 	}
 }
 
-void verify_defgroups (Object *ob)
-{
-	/* Ensure the defbase & the dverts match */
-	switch (ob->type){
-	case OB_MESH:
-
-		/* I'm pretty sure this means "If there are no
-		 * deform groups defined, yet there are deform
-		 * vertices, then delete the deform vertices
-		 */
-		if (!ob->defbase.first){
-			if (((Mesh*)ob->data)->dvert){
-				free_dverts(((Mesh*)ob->data)->dvert, 
-							((Mesh*)ob->data)->totvert);
-				((Mesh*)ob->data)->dvert=NULL;
-			}
-		}
-		break;
-	default:
-		break;
-	}
-}
-
 void unique_vertexgroup_name (bDeformGroup *dg, Object *ob)
 {
-	char		tempname[64];
-	int			number;
-	char		*dot;
-	int exists = 0;
 	bDeformGroup *curdef;
+	int number;
+	int exists = 0;
+	char tempname[64];
+	char *dot;
 	
 	if (!ob)
 		return;
@@ -614,7 +725,8 @@ void vertexgroup_select_by_name(Object *ob, char *name)
 	bDeformGroup *curdef;
 	int actdef= 1;
 	
-	if(ob==NULL || ob->type!=OB_MESH) return;
+	if(ob==NULL) return;
+	
 	for (curdef = ob->defbase.first; curdef; curdef=curdef->next, actdef++){
 		if (!strcmp(curdef->name, name)) {
 			ob->actdef= actdef;
