@@ -1,5 +1,5 @@
 /**
- * $Id: editmesh.c,v 1.177 2006/05/16 03:40:36 briggs Exp $
+ * $Id: editmesh.c,v 1.201 2007/02/09 13:02:10 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,6 +43,7 @@
 
 #include "PIL_time.h"
 
+#include "DNA_customdata_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
@@ -64,6 +65,7 @@
 
 #include "BKE_DerivedMesh.h"
 #include "BKE_depsgraph.h"
+#include "BKE_customdata.h"
 #include "BKE_global.h"
 #include "BKE_key.h"
 #include "BKE_library.h"
@@ -74,16 +76,25 @@
 #include "BKE_texture.h"
 #include "BKE_utildefines.h"
 
+#ifdef WITH_VERSE
+#include "BKE_verse.h"
+#endif
+
 #include "BIF_editkey.h"
 #include "BIF_editmesh.h"
 #include "BIF_editmode_undo.h"
 #include "BIF_interface.h"
 #include "BIF_meshtools.h"
 #include "BIF_mywindow.h"
+#include "BIF_retopo.h"
 #include "BIF_space.h"
 #include "BIF_screen.h"
 #include "BIF_toolbox.h"
 
+#ifdef WITH_VERSE
+#include "BIF_verse.h"
+#endif
+
 #include "BSE_view.h"
 #include "BSE_edit.h"
 #include "BSE_trans_types.h"
@@ -93,6 +104,9 @@
 #include "BDR_editface.h"
 #include "BDR_vpaint.h"
 
+#include "LBM_fluidsim.h"
+
+#include "multires.h"
 #include "mydevice.h"
 #include "blendef.h"
 
@@ -123,7 +137,7 @@ static void *(*callocvert)(size_t, size_t) = calloc;
 static void *(*callocedge)(size_t, size_t) = calloc;
 static void *(*callocface)(size_t, size_t) = calloc;
 
-EditVert *addvertlist(float *vec)
+EditVert *addvertlist(float *vec, EditVert *example)
 {
 	EditMesh *em = G.editMesh;
 	EditVert *eve;
@@ -141,16 +155,35 @@ EditVert *addvertlist(float *vec)
 	 * have a pre-editmode vertex order
 	 */
 	eve->keyindex = -1;
+#ifdef WITH_VERSE
+	createVerseVert(eve);
+#endif
+
+	if(example)
+		CustomData_em_copy_data(&em->vdata, &em->vdata, example->data, &eve->data);
+	else
+		CustomData_em_set_default(&em->vdata, &eve->data);
+
 	return eve;
 }
 
 void free_editvert (EditVert *eve)
 {
-	if(eve->dw) MEM_freeN(eve->dw);
-	EM_remove_selection(eve, EDITVERT);
-	if(eve->fast==0){ 
-		free(eve);
+#ifdef WITH_VERSE
+	if(eve->vvert) {
+		/* it prevents from removing all verse vertexes
+		 * during entering edit mode ... messy solution */
+		if(G.editMesh->vnode)
+			b_verse_send_vertex_delete(eve);
+		else
+			((VerseVert*)eve->vvert)->vertex = NULL;
 	}
+#endif
+
+	EM_remove_selection(eve, EDITVERT);
+	CustomData_em_free_block(&G.editMesh->vdata, &eve->data);
+	if(eve->fast==0)
+		free(eve);
 }
 
 
@@ -262,6 +295,7 @@ EditEdge *addedgelist(EditVert *v1, EditVert *v2, EditEdge *example)
 		   rule is to do this with addedgelist call, before addfacelist */
 		if(example) {
 			eed->crease= example->crease;
+			eed->sharp = example->sharp;
 			eed->seam = example->seam;
 			eed->h |= (example->h & EM_FGON);
 		}
@@ -288,10 +322,20 @@ void free_editedge(EditEdge *eed)
 
 void free_editface(EditFace *efa)
 {
-	EM_remove_selection(efa, EDITFACE);
-	if(efa->fast==0){ 
-		free(efa);
+#ifdef WITH_VERSE
+	if(efa->vface) {
+		/* it prevents from removing all verse faces
+		 * during entering edit mode ... messy solution */
+		if(G.editMesh->vnode)
+			b_verse_send_face_delete(efa);
+		else
+			((VerseFace*)efa->vface)->face = NULL;
 	}
+#endif
+	EM_remove_selection(efa, EDITFACE);
+	CustomData_em_free_block(&G.editMesh->fdata, &efa->data);
+	if(efa->fast==0)
+		free(efa);
 }
 
 void free_vertlist(ListBase *edve) 
@@ -341,6 +385,9 @@ EditFace *addfacelist(EditVert *v1, EditVert *v2, EditVert *v3, EditVert *v4, Ed
 	EditFace *efa;
 	EditEdge *e1, *e2=0, *e3=0, *e4=0;
 
+	/* added sanity check... seems to happen for some tools, or for enter editmode for corrupted meshes */
+	if(v1==v4 || v2==v4 || v3==v4) v4= NULL;
+	
 	/* add face to list and do the edges */
 	if(exampleEdges) {
 		e1= addedgelist(v1, v2, exampleEdges->e1);
@@ -373,17 +420,14 @@ EditFace *addfacelist(EditVert *v1, EditVert *v2, EditVert *v3, EditVert *v4, Ed
 
 	if(example) {
 		efa->mat_nr= example->mat_nr;
-		efa->tf= example->tf;
-		efa->tf.flag &= ~TF_ACTIVE;
 		efa->flag= example->flag;
+		CustomData_em_copy_data(&em->fdata, &em->fdata, example->data, &efa->data);
 	}
 	else {
 		if (G.obedit && G.obedit->actcol)
 			efa->mat_nr= G.obedit->actcol-1;
-		default_uv(efa->tf.uv, 1.0);
 
-		/* Initialize colors */
-		efa->tf.col[0]= efa->tf.col[1]= efa->tf.col[2]= efa->tf.col[3]= vpaint_get_current_col();
+		CustomData_em_set_default(&em->fdata, &efa->data);
 	}
 
 	BLI_addtail(&em->faces, efa);
@@ -397,6 +441,10 @@ EditFace *addfacelist(EditVert *v1, EditVert *v2, EditVert *v3, EditVert *v4, Ed
 		CalcCent3f(efa->cent, efa->v1->co, efa->v2->co, efa->v3->co);
 	}
 
+#ifdef WITH_VERSE
+	createVerseFace(efa);
+#endif
+
 	return efa;
 }
 
@@ -471,7 +519,6 @@ static void *calloc_fastface(size_t size, size_t nr)
 */
 static void init_editmesh_fastmalloc(EditMesh *em, int totvert, int totedge, int totface)
 {
-	
 	if(totvert) em->allverts= MEM_callocN(totvert*sizeof(EditVert), "allverts");
 	else em->allverts= NULL;
 	em->curvert= em->allverts;
@@ -500,23 +547,45 @@ static void end_editmesh_fastmalloc(void)
 
 void free_editMesh(EditMesh *em)
 {
+#ifdef WITH_VERSE
+	struct VNode *vnode=NULL;
+#endif
 	if(em==NULL) return;
-	
+
+#ifdef WITH_VERSE
+	if(em->vnode) {
+		vnode = (VNode*)em->vnode;
+		em->vnode = NULL;
+	}
+#endif
+
 	if(em->verts.first) free_vertlist(&em->verts);
 	if(em->edges.first) free_edgelist(&em->edges);
 	if(em->faces.first) free_facelist(&em->faces);
 	if(em->selected.first) BLI_freelistN(&(em->selected));
+
+	CustomData_free(&em->vdata, 0);
+	CustomData_free(&em->fdata, 0);
+
 	if(em->derivedFinal) {
 		if (em->derivedFinal!=em->derivedCage) {
+			em->derivedFinal->needsFree= 1;
 			em->derivedFinal->release(em->derivedFinal);
 		}
 		em->derivedFinal= NULL;
 	}
 	if(em->derivedCage) {
+		em->derivedCage->needsFree= 1;
 		em->derivedCage->release(em->derivedCage);
 		em->derivedCage= NULL;
 	}
 
+#ifdef WITH_VERSE
+	if(vnode) {
+		em->vnode = (void*)vnode;
+	}
+#endif
+
 	/* DEBUG: hashtabs are slowest part of enter/exit editmode. here a testprint */
 #if 0
 	if(em->hashedgetab) {
@@ -550,6 +619,9 @@ void free_editMesh(EditMesh *em)
 	mesh_octree_table(NULL, NULL, 'e');
 	
 	G.totvert= G.totface= 0;
+
+	if(em->retopo_paint_data) retopo_free_paint_data(em->retopo_paint_data);
+	em->retopo_paint_data= NULL;
 }
 
 /* on G.editMesh */
@@ -716,8 +788,8 @@ static void edge_drawflags(void)
 void make_editMesh()
 {
 	Mesh *me= G.obedit->data;
+	EditMesh *em= G.editMesh;
 	MFace *mface;
-	TFace *tface;
 	MVert *mvert;
 	MSelect *mselect;
 	KeyBlock *actkey;
@@ -727,8 +799,15 @@ void make_editMesh()
 	EditSelection *ese;
 	int tot, a, eekadoodle= 0;
 
+#ifdef WITH_VERSE
+	if(me->vnode){
+		create_edit_mesh_from_geom_node(me->vnode);
+		return;
+	}
+#endif
+
 	/* because of reload */
-	free_editMesh(G.editMesh);
+	free_editMesh(em);
 	
 	G.totvert= tot= me->totvert;
 	G.totedge= me->totedge;
@@ -739,10 +818,8 @@ void make_editMesh()
 		return;
 	}
 	
-	waitcursor(1);
-
 	/* initialize fastmalloc for editmesh */
-	init_editmesh_fastmalloc(G.editMesh, me->totvert, me->totedge, me->totface);
+	init_editmesh_fastmalloc(em, me->totvert, me->totedge, me->totface);
 
 	actkey = ob_get_keyblock(G.obedit);
 	if(actkey) {
@@ -752,11 +829,12 @@ void make_editMesh()
 	}
 
 	/* make editverts */
+	CustomData_copy(&me->vdata, &em->vdata, CD_MASK_EDITMESH, CD_CALLOC, 0);
 	mvert= me->mvert;
 
 	evlist= (EditVert **)MEM_mallocN(tot*sizeof(void *),"evlist");
 	for(a=0; a<tot; a++, mvert++) {
-		eve= addvertlist(mvert->co);
+		eve= addvertlist(mvert->co, NULL);
 		evlist[a]= eve;
 		
 		// face select sets selection in next loop
@@ -774,19 +852,11 @@ void make_editMesh()
 		 */
 		eve->keyindex = a;
 
-		if (me->dvert){
-			eve->totweight = me->dvert[a].totweight;
-			if (me->dvert[a].dw){
-				eve->dw = MEM_callocN (sizeof(MDeformWeight) * me->dvert[a].totweight, "deformWeight");
-				memcpy (eve->dw, me->dvert[a].dw, sizeof(MDeformWeight) * me->dvert[a].totweight);
-			}
-		}
-
+		CustomData_to_em_block(&me->vdata, &em->vdata, a, &eve->data);
 	}
-	
+
 	if(actkey && actkey->totelem!=me->totvert);
 	else {
-		unsigned int *mcol;
 		MEdge *medge= me->medge;
 		
 		/* make edges */
@@ -797,6 +867,7 @@ void make_editMesh()
 				eed->crease= ((float)medge->crease)/255.0;
 				
 				if(medge->flag & ME_SEAM) eed->seam= 1;
+				if(medge->flag & ME_SHARP) eed->sharp = 1;
 				if(medge->flag & SELECT) eed->f |= SELECT;
 				if(medge->flag & ME_FGON) eed->h= EM_FGON;	// 2 different defines!
 				if(medge->flag & ME_HIDE) eed->h |= 1;
@@ -805,11 +876,13 @@ void make_editMesh()
 			}
 		}
 		
+		CustomData_copy(&me->fdata, &em->fdata, CD_MASK_EDITMESH, CD_CALLOC, 0);
+		if (G.f & G_FACESELECT)
+			select_mface_from_tface(me);
+
 		/* make faces */
 		mface= me->mface;
-		tface= me->tface;
-		mcol= (unsigned int *)me->mcol;
-		
+
 		for(a=0; a<me->totface; a++, mface++) {
 			eve1= evlist[mface->v1];
 			eve2= evlist[mface->v2];
@@ -820,34 +893,20 @@ void make_editMesh()
 			efa= addfacelist(eve1, eve2, eve3, eve4, NULL, NULL);
 
 			if(efa) {
-			
-				if(mcol) memcpy(efa->tf.col, mcol, 4*sizeof(int));
+				CustomData_to_em_block(&me->fdata, &em->fdata, a, &efa->data);
 
-				if(me->tface) {
-					efa->tf= *tface;
-				}
-			
 				efa->mat_nr= mface->mat_nr;
 				efa->flag= mface->flag & ~ME_HIDE;
 				
-				if((G.f & G_FACESELECT)==0) {
-					/* select face flag, if no edges we flush down */
-					if(mface->flag & ME_FACE_SEL) {
-						efa->f |= SELECT;
-					}
-					if(mface->flag & ME_HIDE) efa->h= 1;
+				/* select and hide face flag */
+				if(mface->flag & ME_FACE_SEL) {
+					efa->f |= SELECT;
 				}
-				else if (tface) {
-					if( tface->flag & TF_HIDE) 
-						efa->h= 1;
-					else if( tface->flag & TF_SELECT) {
-						EM_select_face(efa, 1);
-					}
-				}
-			}
+				if(mface->flag & ME_HIDE) efa->h= 1;
 
-			if(me->tface) tface++;
-			if(mcol) mcol+=4;
+				if((G.f & G_FACESELECT) && (efa->f & SELECT))
+					EM_select_face(efa, 1); /* flush down */
+			}
 		}
 	}
 	
@@ -871,7 +930,7 @@ void make_editMesh()
 				if(ese->type == EDITVERT) ese->data = EM_get_vert_for_index(mselect->index); else
 				if(ese->type == EDITEDGE) ese->data = EM_get_edge_for_index(mselect->index); else
 				if(ese->type == EDITFACE) ese->data = EM_get_face_for_index(mselect->index);
-				BLI_addtail(&(G.editMesh->selected),ese);
+				BLI_addtail(&(em->selected),ese);
 			}
 		}
 		EM_free_index_arrays();
@@ -885,7 +944,6 @@ void make_editMesh()
 	
 	countall();
 	
-	waitcursor(0);
 }
 
 /* makes Mesh out of editmesh */
@@ -896,7 +954,6 @@ void load_editMesh(void)
 	MVert *mvert, *oldverts;
 	MEdge *medge;
 	MFace *mface;
-	MSticky *ms;
 	MSelect *mselect;
 	EditVert *eve;
 	EditFace *efa;
@@ -904,9 +961,15 @@ void load_editMesh(void)
 	EditSelection *ese;
 	float *fp, *newkey, *oldkey, nor[3];
 	int i, a, ototvert, totedge=0;
-	MDeformVert *dvert;
 
-	waitcursor(1);
+#ifdef WITH_VERSE
+	if(em->vnode) {
+		struct VNode *vnode = (VNode*)em->vnode;
+		((VGeomData*)vnode->data)->editmesh = NULL;
+		em->vnode = NULL;
+	}
+#endif
+
 	countall();
 
 	/* this one also tests of edges are not in faces: */
@@ -933,32 +996,33 @@ void load_editMesh(void)
 	/* new Face block */
 	if(G.totface==0) mface= NULL;
 	else mface= MEM_callocN(G.totface*sizeof(MFace), "loadeditMesh face");
-	
-	/* are we adding dverts? */
-	if (G.totvert==0) dvert= NULL;
-	else if(G.obedit->defbase.first==NULL) dvert= NULL;
-	else dvert = MEM_callocN(G.totvert*sizeof(MDeformVert), "loadeditMesh3");
-
-	if (me->dvert) free_dverts(me->dvert, me->totvert);
-	me->dvert=dvert;
 
 	/* lets save the old verts just in case we are actually working on
 	 * a key ... we now do processing of the keys at the end */
-	oldverts = me->mvert;
+	oldverts= me->mvert;
 	ototvert= me->totvert;
 
-	/* put new data in Mesh */
-	me->mvert= mvert;
-	me->totvert= G.totvert;
+	/* don't free this yet */
+	CustomData_set_layer(&me->vdata, CD_MVERT, NULL);
 
-	if(me->medge) MEM_freeN(me->medge);
-	me->medge= medge;
+	/* free custom data */
+	CustomData_free(&me->vdata, me->totvert);
+	CustomData_free(&me->edata, me->totedge);
+	CustomData_free(&me->fdata, me->totface);
+
+	/* add new custom data */
+	me->totvert= G.totvert;
 	me->totedge= totedge;
-	
-	if(me->mface) MEM_freeN(me->mface);
-	me->mface= mface;
 	me->totface= G.totface;
-		
+
+	CustomData_copy(&em->vdata, &me->vdata, CD_MASK_MESH, CD_CALLOC, me->totvert);
+	CustomData_copy(&em->fdata, &me->fdata, CD_MASK_MESH, CD_CALLOC, me->totface);
+
+	CustomData_add_layer(&me->vdata, CD_MVERT, CD_ASSIGN, mvert, me->totvert);
+	CustomData_add_layer(&me->edata, CD_MEDGE, CD_ASSIGN, medge, me->totedge);
+	CustomData_add_layer(&me->fdata, CD_MFACE, CD_ASSIGN, mface, me->totface);
+	mesh_update_customdata_pointers(me);
+
 	/* the vertices, use ->tmp.l as counter */
 	eve= em->verts.first;
 	a= 0;
@@ -972,16 +1036,9 @@ void load_editMesh(void)
 		VecMulf(nor, 32767.0);
 		VECCOPY(mvert->no, nor);
 
-		/* note: it used to remove me->dvert when it was not in use, cancelled that... annoying when you have a fresh vgroup */
-		if (dvert){
-			dvert->totweight=eve->totweight;
-			if (eve->dw){
-				dvert->dw = MEM_callocN (sizeof(MDeformWeight)*eve->totweight,
-										 "deformWeight");
-				memcpy (dvert->dw, eve->dw, 
-						sizeof(MDeformWeight)*eve->totweight);
-			}
-		}
+		/* note: it used to remove me->dvert when it was not in use, cancelled
+		   that... annoying when you have a fresh vgroup */
+		CustomData_from_em_block(&em->vdata, &me->vdata, eve->data, a);
 
 		eve->tmp.l = a++;  /* counter */
 			
@@ -989,10 +1046,15 @@ void load_editMesh(void)
 		if(eve->f1==1) mvert->flag |= ME_SPHERETEST;
 		mvert->flag |= (eve->f & SELECT);
 		if (eve->h) mvert->flag |= ME_HIDE;			
-			
+
+#ifdef WITH_VERSE
+		if(eve->vvert) {
+			((VerseVert*)eve->vvert)->vertex = NULL;
+			eve->vvert = NULL;
+		}
+#endif			
 		eve= eve->next;
 		mvert++;
-		if(dvert) dvert++;
 	}
 
 	/* the edges */
@@ -1005,6 +1067,7 @@ void load_editMesh(void)
 		medge->flag= (eed->f & SELECT) | ME_EDGERENDER;
 		if(eed->f2<2) medge->flag |= ME_EDGEDRAW;
 		if(eed->f2==0) medge->flag |= ME_LOOSEEDGE;
+		if(eed->sharp) medge->flag |= ME_SHARP;
 		if(eed->seam) medge->flag |= ME_SEAM;
 		if(eed->h & EM_FGON) medge->flag |= ME_FGON;	// different defines yes
 		if(eed->h & 1) medge->flag |= ME_HIDE;
@@ -1028,7 +1091,7 @@ void load_editMesh(void)
 		mface->v2= (unsigned int) efa->v2->tmp.l;
 		mface->v3= (unsigned int) efa->v3->tmp.l;
 		if (efa->v4) mface->v4 = (unsigned int) efa->v4->tmp.l;
-			
+
 		mface->mat_nr= efa->mat_nr;
 		
 		mface->flag= efa->flag;
@@ -1066,65 +1129,27 @@ void load_editMesh(void)
 			efa->e4->f2= 2;
 		}
 
+		CustomData_from_em_block(&em->fdata, &me->fdata, efa->data, i);
 
 		/* no index '0' at location 3 or 4 */
-		test_index_face(mface, NULL, &efa->tf, efa->v4?4:3);
-		
+		test_index_face(mface, &me->fdata, i, efa->v4?4:3);
+
+#ifdef WITH_VERSE
+		if(efa->vface) {
+			((VerseFace*)efa->vface)->face = NULL;
+			efa->vface = NULL;
+		}
+#endif		
 		efa->tmp.l = a++;
 		i++;
 		efa= efa->next;
 	}
-	
-	/* tface block */
-	if( me->tface && me->totface ) {
-		TFace *tfn, *tf;
-			
-		tf=tfn= MEM_callocN(sizeof(TFace)*me->totface, "tface");
-		efa= em->faces.first;
-		while(efa) {
-				
-			*tf= efa->tf;
-				
-			if(G.f & G_FACESELECT) {
-				if( efa->h) tf->flag |= TF_HIDE;
-				else tf->flag &= ~TF_HIDE;
-				if( efa->f & SELECT)  tf->flag |= TF_SELECT;
-				else tf->flag &= ~TF_SELECT;
-			}
-				
-			tf++;
-			efa= efa->next;
-		}
 
-		if(me->tface) MEM_freeN(me->tface);
-		me->tface= tfn;
-	}
-	else if(me->tface) {
-		MEM_freeN(me->tface);
-		me->tface= NULL;
-	}
-		
-	/* mcol: same as tface... */
-	if(me->mcol && me->totface) {
-		unsigned int *mcn, *mc;
+	/* sync hide and select flags with faceselect mode */
+	if (G.f & G_FACESELECT)
+		select_tface_from_mface(me);
 
-		mc=mcn= MEM_mallocN(4*sizeof(int)*me->totface, "mcol");
-		efa= em->faces.first;
-		while(efa) {
-			memcpy(mc, efa->tf.col, 4*sizeof(int));
-				
-			mc+=4;
-			efa= efa->next;
-		}
-		if(me->mcol) MEM_freeN(me->mcol);
-			me->mcol= (MCol *)mcn;
-	}
-	else if(me->mcol) {
-		MEM_freeN(me->mcol);
-		me->mcol= 0;
-	}
-
-	/* patch hook indices */
+	/* patch hook indices and vertex parents */
 	{
 		Object *ob;
 		ModifierData *md;
@@ -1132,6 +1157,31 @@ void load_editMesh(void)
 		int i,j;
 
 		for (ob=G.main->object.first; ob; ob=ob->id.next) {
+			if (ob->parent==G.obedit && ELEM(ob->partype, PARVERT1,PARVERT3)) {
+				
+				/* duplicate code from below, make it function later...? */
+				if (!vertMap) {
+					vertMap = MEM_callocN(sizeof(*vertMap)*ototvert, "vertMap");
+					
+					for (eve=em->verts.first; eve; eve=eve->next) {
+						if (eve->keyindex!=-1)
+							vertMap[eve->keyindex] = eve;
+					}
+				}
+				if(ob->par1 < ototvert) {
+					eve = vertMap[ob->par1];
+					if(eve) ob->par1= eve->tmp.l;
+				}
+				if(ob->par2 < ototvert) {
+					eve = vertMap[ob->par2];
+					if(eve) ob->par2= eve->tmp.l;
+				}
+				if(ob->par3 < ototvert) {
+					eve = vertMap[ob->par3];
+					if(eve) ob->par3= eve->tmp.l;
+				}
+				
+			}
 			if (ob->data==me) {
 				for (md=ob->modifiers.first; md; md=md->next) {
 					if (md->type==eModifierType_Hook) {
@@ -1265,20 +1315,8 @@ void load_editMesh(void)
 			}
 		}
 	}
-	
-	/* sticky */
-	if(me->msticky) {
-		if (ototvert<me->totvert) {
-			ms= MEM_callocN(me->totvert*sizeof(MSticky), "msticky");
-			memcpy(ms, me->msticky, ototvert*sizeof(MSticky));
-			MEM_freeN(me->msticky);
-			me->msticky= ms;
-			error("Sticky was too small");
-		}
-	}
 
 	mesh_calc_normals(me->mvert, me->totvert, me->mface, me->totface, NULL);
-	waitcursor(0);
 }
 
 void remake_editMesh(void)
@@ -1298,7 +1336,7 @@ void separatemenu(void)
 
 	if(G.editMesh->verts.first==NULL) return;
 	   
-	event = pupmenu("Separate %t|Selected%x1|All Loose Parts%x2");
+	event = pupmenu("Separate %t|Selected%x1|All Loose Parts%x2|By Material%x3");
 	
 	if (event==0) return;
 	waitcursor(1);
@@ -1310,10 +1348,45 @@ void separatemenu(void)
 	case 2:	    	    	    
 		separate_mesh_loose();	    	    
 		break;
+	case 3:
+		separate_material();
+		break;
 	}
 	waitcursor(0);
 }
 
+void separate_material(void)
+{
+	EditMesh *em = G.editMesh;
+	unsigned char curr_mat;
+	Mesh *me;
+	
+	if(multires_test()) return;
+	
+	me= get_mesh(G.obedit);
+	if(me->key) {
+		error("Can't separate with vertex keys");
+		return;
+	}
+
+	if(G.obedit && em) {
+		if(G.obedit->type == OB_MESH) {
+			for (curr_mat = 1; curr_mat < G.obedit->totcol; ++curr_mat) {
+				/* clear selection, we're going to use that to select material group */
+				EM_clear_flag_all(SELECT);
+				/* select the material */
+				editmesh_select_by_material(curr_mat);
+				/* and now separate */
+				separate_mesh();
+			}
+		}
+	}
+	
+	countall();
+	allqueue(REDRAWVIEW3D, 0);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
+
+}
 
 void separate_mesh(void)
 {
@@ -1326,8 +1399,12 @@ void separate_mesh(void)
 	Mesh *me, *men;
 	Base *base, *oldbase;
 	ListBase edve, eded, edvl;
+#ifdef WITH_VERSE
+	struct VNode *vnode = NULL;
+#endif
 	
-	TEST_EDITMESH	
+	TEST_EDITMESH
+	if(multires_test()) return;
 
 	waitcursor(1);
 	
@@ -1359,9 +1436,21 @@ void separate_mesh(void)
 		base= base->next;
 	}
 	
+#ifdef WITH_VERSE
+	if(G.editMesh->vnode) {
+		vnode = G.editMesh->vnode;
+		G.editMesh->vnode = NULL;
+	}
+#endif
 	/* no test for split, split doesn't split when a loose part is selected */
 	/* SPLIT: first make duplicate */
 	adduplicateflag(SELECT);
+
+#ifdef WITH_VERSE
+	if(vnode) {
+		G.editMesh->vnode = vnode;
+	}
+#endif
 	/* SPLIT: old faces have 3x flag 128 set, delete these ones */
 	delfaceflag(128);
 	
@@ -1376,7 +1465,14 @@ void separate_mesh(void)
 		if((eve->f & SELECT)==0) {
 			BLI_remlink(&em->verts, eve);
 			BLI_addtail(&edve, eve);
+#ifdef WITH_VERSE
+			if(eve->vvert) {
+				((VerseVert*)eve->vvert)->vertex = NULL;
+				eve->vvert = NULL;
+			}
+#endif
 		}
+		
 		eve= v1;
 	}
 	eed= em->edges.first;
@@ -1394,6 +1490,12 @@ void separate_mesh(void)
 		if((efa->f & SELECT)==0) {
 			BLI_remlink(&em->faces, efa);
 			BLI_addtail(&edvl, efa);
+#ifdef WITH_VERSE
+			if(efa->vface) {
+				((VerseFace*)efa->vface)->face = NULL;
+				efa->vface = NULL;
+			}
+#endif
 		}
 		efa= vl1;
 	}
@@ -1401,10 +1503,29 @@ void separate_mesh(void)
 	oldob= G.obedit;
 	oldbase= BASACT;
 	
-	adduplicate(1, 0); /* notrans and a linked duplicate*/
+#ifdef WITH_VERSE
+	if(G.obedit->vnode) {
+		vnode = G.obedit->vnode;
+		G.obedit->vnode = NULL;
+	}
+#endif
+	adduplicate(1, 0); /* notrans and a linked duplicate */
+	
+#ifdef WITH_VERSE
+	if(vnode) {
+		G.obedit->vnode = vnode;
+	}
+#endif
 	
 	G.obedit= BASACT->object;	/* basact was set in adduplicate()  */
-
+	
+	/* blender crashes in derivedmesh drawing if I don't do this... but why? 
+		Anyhoo, this function is horrible anyway (ton) */
+	if(G.obedit->fluidsimSettings) {
+		fluidsimSettingsFree(G.obedit->fluidsimSettings);
+		G.obedit->fluidsimSettings= NULL;
+	}
+	
 	men= copy_mesh(me);
 	set_mesh(G.obedit, men);
 	/* because new mesh is a copy: reduce user count */
@@ -1420,6 +1541,8 @@ void separate_mesh(void)
 	emcopy.alledges= NULL;
 	emcopy.allfaces= NULL;
 	emcopy.derivedFinal= emcopy.derivedCage= NULL;
+	memset(&emcopy.vdata, 0, sizeof(emcopy.vdata));
+	memset(&emcopy.fdata, 0, sizeof(emcopy.fdata));
 	free_editMesh(&emcopy);
 	
 	em->verts= edve;
@@ -1428,7 +1551,7 @@ void separate_mesh(void)
 	
 	/* hashedges are freed now, make new! */
 	editMesh_set_hash();
-	
+
 	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);	
 	G.obedit= oldob;
 	BASACT= oldbase;
@@ -1449,14 +1572,30 @@ void separate_mesh_loose(void)
 	EditVert *eve, *v1;
 	EditEdge *eed, *e1;
 	EditFace *efa, *vl1;
-	Object *oldob;
+	Object *oldob=NULL;
 	Mesh *me, *men;
 	Base *base, *oldbase;
 	ListBase edve, eded, edvl;
 	int vertsep=0;	
 	short done=0, check=1;
-		
+#ifdef WITH_VERSE
+	struct VNode *vnode = NULL;
+#endif
+			
+	me= get_mesh(G.obedit);
+#ifdef WITH_VERSE
+	if(me->vnode) {
+		error("Can't separate a mesh shared at verse server");
+		return;
+	}
+#endif
+	if(me->key) {
+		error("Can't separate a mesh with vertex keys");
+		return;
+	}
+	
 	TEST_EDITMESH
+	if(multires_test()) return;
 	waitcursor(1);	
 	
 	/* we are going to abuse the system as follows:
@@ -1467,19 +1606,11 @@ void separate_mesh_loose(void)
 	 * 5. freelist and get back old verts, edges, facs
 	 */
 			
-	
-			
 	while(!done){		
 		vertsep=check=1;
 		
 		countall();
 		
-		me= get_mesh(G.obedit);
-		if(me->key) {
-			error("Can't separate a mesh with vertex keys");
-			return;
-		}		
-		
 		/* make only obedit selected */
 		base= FIRSTBASE;
 		while(base) {
@@ -1541,6 +1672,11 @@ void separate_mesh_loose(void)
 				if((eve->f & SELECT)==0) {
 					BLI_remlink(&em->verts, eve);
 					BLI_addtail(&edve, eve);
+#ifdef WITH_VERSE
+					if(eve->vvert) {
+						b_verse_send_vertex_delete(eve);
+					}
+#endif
 				}
 				eve= v1;
 			}
@@ -1559,6 +1695,11 @@ void separate_mesh_loose(void)
 				if( (efa->f & SELECT)==0 ) {
 					BLI_remlink(&em->faces, efa);
 					BLI_addtail(&edvl, efa);
+#ifdef WITH_VERSE
+					if(efa->vface) {
+						b_verse_send_face_delete(efa);
+					}
+#endif
 				}
 				efa= vl1;
 			}
@@ -1566,10 +1707,21 @@ void separate_mesh_loose(void)
 			oldob= G.obedit;
 			oldbase= BASACT;
 			
-			adduplicate(1, 0); /* notrans and 0 for linked duplicate */
+#ifdef WITH_VERSE
+			if(G.obedit->vnode) {
+				vnode = G.obedit->vnode;
+				G.obedit->vnode = NULL;
+			}
+#endif
+			adduplicate(1, 0); /* notrans and a linked duplicate*/
+#ifdef WITH_VERSE
+			if(vnode) {
+				G.obedit->vnode = vnode;
+			}
+#endif
 			
 			G.obedit= BASACT->object;	/* basact was set in adduplicate()  */
-		
+
 			men= copy_mesh(me);
 			set_mesh(G.obedit, men);
 			/* because new mesh is a copy: reduce user count */
@@ -1585,6 +1737,8 @@ void separate_mesh_loose(void)
 			emcopy.alledges= NULL;
 			emcopy.allfaces= NULL;
 			emcopy.derivedFinal= emcopy.derivedCage= NULL;
+			memset(&emcopy.vdata, 0, sizeof(emcopy.vdata));
+			memset(&emcopy.fdata, 0, sizeof(emcopy.fdata));
 			free_editMesh(&emcopy);
 			
 			em->verts= edve;
@@ -1625,15 +1779,13 @@ typedef struct EditVertC
 	float no[3];
 	float co[3];
 	unsigned char f, h;
-	short totweight;
-	struct MDeformWeight *dw;
-	int keyindex; 
+	int keyindex;
 } EditVertC;
 
 typedef struct EditEdgeC
 {
 	int v1, v2;
-	unsigned char f, h, seam, pad;
+	unsigned char f, h, seam, sharp, pad;
 	short crease, fgoni;
 } EditEdgeC;
 
@@ -1649,35 +1801,45 @@ typedef struct EditSelectionC{
 	int index;
 }EditSelectionC;
 
+typedef struct EM_MultiresUndo {
+	int users;
+	Multires *mr;
+} EM_MultiresUndo;
 
 typedef struct UndoMesh {
 	EditVertC *verts;
 	EditEdgeC *edges;
 	EditFaceC *faces;
 	EditSelectionC *selected;
-	TFace *tfaces;
-	int totvert, totedge, totface,totsel;
+	int totvert, totedge, totface, totsel;
 	short selectmode;
+	RetopoPaintData *retopo_paint_data;
+	char retopo_mode;
+	CustomData vdata, fdata;
+	EM_MultiresUndo *mru;
 } UndoMesh;
 
-
 /* for callbacks */
 
 static void free_undoMesh(void *umv)
 {
 	UndoMesh *um= umv;
-	EditVertC *evec;
-	int a;
-	
-	for(a=0, evec= um->verts; a<um->totvert; a++, evec++) {
-		if(evec->dw) MEM_freeN(evec->dw);
-	}
 	
 	if(um->verts) MEM_freeN(um->verts);
 	if(um->edges) MEM_freeN(um->edges);
 	if(um->faces) MEM_freeN(um->faces);
-	if(um->tfaces) MEM_freeN(um->tfaces);
 	if(um->selected) MEM_freeN(um->selected);
+	if(um->retopo_paint_data) retopo_free_paint_data(um->retopo_paint_data);
+	CustomData_free(&um->vdata, um->totvert);
+	CustomData_free(&um->fdata, um->totface);
+	if(um->mru) {
+		--um->mru->users;
+		if(um->mru->users==0) {
+			multires_free(um->mru->mr);
+			um->mru->mr= NULL;
+			MEM_freeN(um->mru);
+		}
+	}
 	MEM_freeN(um);
 }
 
@@ -1685,7 +1847,6 @@ static void *editMesh_to_undoMesh(void)
 {
 	EditMesh *em= G.editMesh;
 	UndoMesh *um;
-	Mesh *me= G.obedit->data;
 	EditVert *eve;
 	EditEdge *eed;
 	EditFace *efa;
@@ -1694,7 +1855,6 @@ static void *editMesh_to_undoMesh(void)
 	EditEdgeC *eedc=NULL;
 	EditFaceC *efac=NULL;
 	EditSelectionC *esec=NULL;
-	TFace *tface= NULL;
 	int a;
 	
 	um= MEM_callocN(sizeof(UndoMesh), "undomesh");
@@ -1711,11 +1871,9 @@ static void *editMesh_to_undoMesh(void)
 	if(um->totedge) eedc= um->edges= MEM_callocN(um->totedge*sizeof(EditEdgeC), "alledgesC");
 	if(um->totface) efac= um->faces= MEM_callocN(um->totface*sizeof(EditFaceC), "allfacesC");
 	if(um->totsel) esec= um->selected= MEM_callocN(um->totsel*sizeof(EditSelectionC), "allselections");
-	
-	if(me->tface || me->mcol) tface= um->tfaces= MEM_mallocN(um->totface*sizeof(TFace), "all tfacesC");
 
-		//printf("copy editmesh %d\n", um->totvert*sizeof(EditVert) + um->totedge*sizeof(EditEdge) + um->totface*sizeof(EditFace));
-		//printf("copy undomesh %d\n", um->totvert*sizeof(EditVertC) + um->totedge*sizeof(EditEdgeC) + um->totface*sizeof(EditFaceC));
+	if(um->totvert) CustomData_copy(&em->vdata, &um->vdata, CD_MASK_EDITMESH, CD_CALLOC, um->totvert);
+	if(um->totface) CustomData_copy(&em->fdata, &um->fdata, CD_MASK_EDITMESH, CD_CALLOC, um->totface);
 	
 	/* now copy vertices */
 	a = 0;
@@ -1726,9 +1884,9 @@ static void *editMesh_to_undoMesh(void)
 		evec->f= eve->f;
 		evec->h= eve->h;
 		evec->keyindex= eve->keyindex;
-		evec->totweight= eve->totweight;
-		evec->dw= MEM_dupallocN(eve->dw);
 		eve->tmp.l = a; /*store index*/
+
+		CustomData_from_em_block(&em->vdata, &um->vdata, eve->data, a);
 	}
 	
 	/* copy edges */
@@ -1739,6 +1897,7 @@ static void *editMesh_to_undoMesh(void)
 		eedc->f= eed->f;
 		eedc->h= eed->h;
 		eedc->seam= eed->seam;
+		eedc->sharp= eed->sharp;
 		eedc->crease= (short)(eed->crease*255.0);
 		eedc->fgoni= eed->fgoni;
 		eed->tmp.l = a; /*store index*/
@@ -1758,12 +1917,10 @@ static void *editMesh_to_undoMesh(void)
 		efac->f= efa->f;
 		efac->h= efa->h;
 		efac->fgonf= efa->fgonf;
-		
-		if(tface) {
-			*tface= efa->tf;
-			tface++;
-		}
+
 		efa->tmp.l = a; /*store index*/
+
+		CustomData_from_em_block(&em->fdata, &um->fdata, efa->data, a);
 	}
 	
 	a = 0;
@@ -1773,6 +1930,28 @@ static void *editMesh_to_undoMesh(void)
 		else if(ese->type == EDITEDGE) a = esec->index = ((EditEdge*)ese->data)->tmp.l; 
 		else if(ese->type == EDITFACE) a = esec->index = ((EditFace*)ese->data)->tmp.l;
 	}
+
+	um->retopo_paint_data= retopo_paint_data_copy(em->retopo_paint_data);
+	um->retopo_mode= G.scene->toolsettings->retopo_mode;
+	
+	{
+		Multires *mr= get_mesh(G.obedit)->mr;
+		UndoMesh *prev= undo_editmode_get_prev(G.obedit);
+		
+		um->mru= NULL;
+		
+		if(mr) {
+			if(prev && prev->mru && prev->mru->mr && prev->mru->mr->current == mr->current) {
+				um->mru= prev->mru;
+				++um->mru->users;
+			}
+			else {
+				um->mru= MEM_callocN(sizeof(EM_MultiresUndo), "EM_MultiresUndo");
+				um->mru->users= 1;
+				um->mru->mr= multires_copy(mr);
+			}
+		}
+	}
 	
 	return um;
 }
@@ -1789,32 +1968,48 @@ static void undoMesh_to_editMesh(void *umv)
 	EditEdgeC *eedc;
 	EditFaceC *efac;
 	EditSelectionC *esec;
-	TFace *tface;
 	int a=0;
-	
+
+#ifdef WITH_VERSE
+	struct VNode *vnode = G.editMesh->vnode;
+	if(vnode) {
+		/* send delete command to all verse vertexes and verse face ...
+		 * verse mesh will be recreated from new edit mesh */
+		destroy_versemesh(vnode);
+	}
+#endif	
 	G.scene->selectmode = um->selectmode;
 	
 	free_editMesh(G.editMesh);
 	
 	/* malloc blocks */
 	memset(em, 0, sizeof(EditMesh));
-
-	
 		
 	init_editmesh_fastmalloc(em, um->totvert, um->totedge, um->totface);
 
+#ifdef WITH_VERSE
+	G.editMesh->vnode = vnode;
+#endif
+
+	CustomData_free(&em->vdata, 0);
+	CustomData_free(&em->fdata, 0);
+
+	CustomData_copy(&um->vdata, &em->vdata, CD_MASK_EDITMESH, CD_CALLOC, 0);
+	CustomData_copy(&um->fdata, &em->fdata, CD_MASK_EDITMESH, CD_CALLOC, 0);
+
 	/* now copy vertices */
+
 	if(um->totvert) evar= MEM_mallocN(um->totvert*sizeof(EditVert *), "vertex ar");
 	for(a=0, evec= um->verts; a<um->totvert; a++, evec++) {
-		eve= addvertlist(evec->co);
+		eve= addvertlist(evec->co, NULL);
 		evar[a]= eve;
 
 		VECCOPY(eve->no, evec->no);
 		eve->f= evec->f;
 		eve->h= evec->h;
-		eve->totweight= evec->totweight;
 		eve->keyindex= evec->keyindex;
-		eve->dw= MEM_dupallocN(evec->dw);
+
+		CustomData_to_em_block(&um->vdata, &em->vdata, a, &eve->data);
 	}
 
 	/* copy edges */
@@ -1824,12 +2019,12 @@ static void undoMesh_to_editMesh(void *umv)
 		eed->f= eedc->f;
 		eed->h= eedc->h;
 		eed->seam= eedc->seam;
+		eed->sharp= eedc->sharp;
 		eed->fgoni= eedc->fgoni;
 		eed->crease= ((float)eedc->crease)/255.0;
 	}
 	
 	/* copy faces */
-	tface= um->tfaces;
 	for(a=0, efac= um->faces; a<um->totface; a++, efac++) {
 		if(efac->v4 != -1)
 			efa= addfacelist(evar[efac->v1], evar[efac->v2], evar[efac->v3], evar[efac->v4], NULL, NULL);
@@ -1842,10 +2037,7 @@ static void undoMesh_to_editMesh(void *umv)
 		efa->h= efac->h;
 		efa->fgonf= efac->fgonf;
 		
-		if(tface) {
-			efa->tf= *tface;
-			tface++;
-		}
+		CustomData_to_em_block(&um->fdata, &em->fdata, a, &efa->data);
 	}
 	
 	end_editmesh_fastmalloc();
@@ -1867,6 +2059,22 @@ static void undoMesh_to_editMesh(void *umv)
 		}
 		EM_free_index_arrays();
 	}
+
+	retopo_free_paint();
+	em->retopo_paint_data= retopo_paint_data_copy(um->retopo_paint_data);
+	G.scene->toolsettings->retopo_mode= um->retopo_mode;
+	if(G.scene->toolsettings->retopo_mode) {
+		if(G.vd->depths) G.vd->depths->damaged= 1;
+		retopo_queue_updates(G.vd);
+		retopo_paint_view_update(G.vd);
+	}
+	
+	{
+		Mesh *me= get_mesh(G.obedit);
+		multires_free(me->mr);
+		me->mr= NULL;
+		if(um->mru && um->mru->mr) me->mr= multires_copy(um->mru->mr);
+	}
 }
 
 
