@@ -46,6 +46,7 @@
 #include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
+#include "DNA_space_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_view3d_types.h"
 
@@ -60,22 +61,33 @@
 #include "BKE_object.h"
 #include "BKE_utildefines.h"
 
+#ifdef WITH_VERSE
+#include "BKE_verse.h"
+#endif
+
+
 #include "BIF_editmesh.h"
 #include "BIF_graphics.h"
 #include "BIF_interface.h"
 #include "BIF_mywindow.h"
+#include "BIF_retopo.h"
 #include "BIF_screen.h"
 #include "BIF_space.h"
 #include "BIF_toolbox.h"
 #include "BIF_transform.h"
 
+#ifdef WITH_VERSE
+#include "BIF_verse.h"
+#endif
+
 #include "BDR_editobject.h" 
 
 #include "BSE_view.h"
 #include "BSE_edit.h"
 
-#include "mydevice.h"
 #include "blendef.h"
+#include "multires.h"
+#include "mydevice.h"
 
 #include "editmesh.h"
 
@@ -143,6 +155,7 @@ void add_click_mesh(void)
 	int done= 0;
 	
 	TEST_EDITMESH
+	if(multires_test()) return;
 	
 	INIT_MINMAX(min, max);
 	
@@ -218,7 +231,7 @@ void add_click_mesh(void)
 		float mat[3][3],imat[3][3];
 		float *curs= give_cursor();
 		
-		eve= addvertlist(0);
+		eve= addvertlist(0, NULL);
 
 		Mat3CpyMat4(mat, G.obedit->obmat);
 		Mat3Inv(imat, mat);
@@ -231,8 +244,16 @@ void add_click_mesh(void)
 		eve->f= SELECT;
 	}
 	
+	retopo_do_all();
+	
 	countall();
 
+#ifdef WITH_VERSE
+	if(G.editMesh->vnode) {
+		sync_all_verseverts_with_editverts((VNode*)G.editMesh->vnode);
+	}
+#endif
+
 	BIF_undo_push("Add vertex/edge/face");
 	allqueue(REDRAWVIEW3D, 0);
 	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);	
@@ -242,19 +263,16 @@ void add_click_mesh(void)
 }
 
 /* selected faces get hidden edges */
-static void make_fgon(void)
+static void make_fgon(int make)
 {
 	EditMesh *em = G.editMesh;
 	EditFace *efa;
 	EditEdge *eed;
 	EditVert *eve;
 	float *nor=NULL;	// reference
-	int done=0, ret;
+	int done=0;
 	
-	ret= pupmenu("FGon %t|Make|Clear");
-	if(ret<1) return;
-	
-	if(ret==2) {
+	if(!make) {
 		for(efa= em->faces.first; efa; efa= efa->next) {
 			if(efa->f & SELECT) {
 				efa->fgonf= 0;
@@ -561,7 +579,69 @@ static void fix_new_face(EditFace *eface)
 		eface->flag &= ~ME_SMOOTH;
 
 	/* flip face, when too much "face normals" in neighbourhood is different */
-	if(count > 0) flipface(eface);
+	if(count > 0) {
+		flipface(eface);
+#ifdef WITH_VERSE
+		if(eface->vface) {
+			struct VNode *vnode;
+			struct VLayer *vlayer;
+			vnode = (VNode*)((Mesh*)G.obedit->data)->vnode;
+			vlayer = find_verse_layer_type((VGeomData*)vnode->data, POLYGON_LAYER);
+			add_item_to_send_queue(&(vlayer->queue), (void*)eface->vface, VERSE_FACE);
+		}
+#endif
+	}
+}
+
+void addfaces_from_edgenet()
+{
+	EditVert *eve1, *eve2, *eve3, *eve4;
+	EditMesh *em= G.editMesh;
+	
+	for(eve1= em->verts.first; eve1; eve1= eve1->next) {
+		for(eve2= em->verts.first; (eve1->f & 1) && eve2; eve2= eve2->next) {
+			if(findedgelist(eve1,eve2)) {
+				for(eve3= em->verts.first; (eve2->f & 1) && eve3; eve3= eve3->next) {
+					if((eve2!=eve3 && (eve3->f & 1) && findedgelist(eve1,eve3))) {
+						EditEdge *sh_edge= NULL;
+						EditVert *sh_vert= NULL;
+						
+						sh_edge= findedgelist(eve2,eve3);
+						
+						if(sh_edge) { /* Add a triangle */
+							if(!exist_face_overlaps(eve1,eve2,eve3,NULL))
+								fix_new_face(addfacelist(eve1,eve2,eve3,NULL,NULL,NULL));
+						}
+						else { /* Check for a shared vertex */
+							for(eve4= em->verts.first; eve4; eve4= eve4->next) {
+								if(eve4!=eve1 && eve4!=eve2 && eve4!=eve3 && (eve4->f & 1) &&
+								   !findedgelist(eve1,eve4) && findedgelist(eve2,eve4) &&
+								   findedgelist(eve3,eve4)) {
+									sh_vert= eve4;
+									break;
+								}
+							}
+							
+							if(sh_vert) {
+								if(sh_vert) {
+									if(!exist_face_overlaps(eve1,eve2,eve4,eve3))
+										fix_new_face(addfacelist(eve1,eve2,eve4,eve3,NULL,NULL));
+								}
+							}
+						}
+					}
+				}
+			}
+		}
+	}
+
+	countall();
+
+	EM_select_flush();
+	
+	BIF_undo_push("Add faces");
+	allqueue(REDRAWVIEW3D, 0);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 }
 
 void addedgeface_mesh(void)
@@ -573,6 +653,7 @@ void addedgeface_mesh(void)
 	short amount=0;
 
 	if( (G.vd->lay & G.obedit->lay)==0 ) return;
+	if(multires_test()) return;
 
 	/* how many selected ? */
 	if(G.scene->selectmode & SCE_SELECT_EDGE) {
@@ -599,7 +680,10 @@ void addedgeface_mesh(void)
 		return;
 	}
 	else if(amount > 4) {
-		make_fgon();
+		int ret= pupmenu("Make Faces %t|Auto|Make FGon|Clear FGon");
+		if(ret==1) addfaces_from_edgenet();
+		else if(ret==2) make_fgon(1);
+		else if(ret==3) make_fgon(0);
 		return;
 	}
 	else if(amount<2) {
@@ -665,7 +749,8 @@ void addedgeface_mesh(void)
 		EM_select_face(efa, 1);
 
 		fix_new_face(efa);
-
+		
+		recalc_editnormals();
 		BIF_undo_push("Add face");
 	}
 	
@@ -679,6 +764,7 @@ void adduplicate_mesh(void)
 {
 
 	TEST_EDITMESH
+	if(multires_test()) return;
 
 	waitcursor(1);
 
@@ -763,7 +849,7 @@ void make_prim(int type, float imat[3][3], short tot, short seg,
 			vec[1]= cent[1]- dia;
 			vec[2]= cent[2];
 			Mat3MulVecfl(imat,vec);
-			eve= addvertlist(vec);
+			eve= addvertlist(vec, NULL);
 			eve->f= 1+2+4;
 			if (a) {
 				addedgelist(eve->prev, eve, NULL);
@@ -795,7 +881,7 @@ void make_prim(int type, float imat[3][3], short tot, short seg,
 			vec[0]= dia*sin(phi);
 			vec[1]= 0.0;
 			vec[2]= dia*cos(phi);
-			eve= addvertlist(vec);
+			eve= addvertlist(vec, NULL);
 			eve->f= 1+2+4;
 			if(a==0) v1= eve;
 			else addedgelist(eve->prev, eve, NULL);
@@ -842,7 +928,7 @@ void make_prim(int type, float imat[3][3], short tot, short seg,
 				vec[0]= dia*icovert[a][0];
 				vec[1]= dia*icovert[a][1];
 				vec[2]= dia*icovert[a][2];
-				eva[a]= addvertlist(vec);
+				eva[a]= addvertlist(vec, NULL);
 				eva[a]->f= 1+2;
 			}
 			for(a=0;a<20;a++) {
@@ -888,9 +974,9 @@ void make_prim(int type, float imat[3][3], short tot, short seg,
 			for (i=0; i<monkeynv; i++) {
 				float v[3];
 				v[0]= (monkeyv[i][0]+127)/128.0, v[1]= monkeyv[i][1]/128.0, v[2]= monkeyv[i][2]/128.0;
-				tv[i]= addvertlist(v);
+				tv[i]= addvertlist(v, NULL);
 				tv[i]->f |= SELECT;
-				tv[monkeynv+i]= (fabs(v[0]= -v[0])<0.001)?tv[i]:addvertlist(v);
+				tv[monkeynv+i]= (fabs(v[0]= -v[0])<0.001)?tv[i]:addvertlist(v, NULL);
 				tv[monkeynv+i]->f |= SELECT;
 			}
 			for (i=0; i<monkeynf; i++) {
@@ -923,7 +1009,7 @@ void make_prim(int type, float imat[3][3], short tot, short seg,
 				vec[2]= cent[2]+d;
 				
 				Mat3MulVecfl(imat, vec);
-				eve= addvertlist(vec);
+				eve= addvertlist(vec, NULL);
 				eve->f= SELECT;
 				if(a==0) {
 					if(b==0) v1= eve;
@@ -938,12 +1024,12 @@ void make_prim(int type, float imat[3][3], short tot, short seg,
 			VECCOPY(vec,cent);
 			vec[2]-= -d;
 			Mat3MulVecfl(imat,vec);
-			vdown= addvertlist(vec);
+			vdown= addvertlist(vec, NULL);
 			if(ext || type==7) {
 				VECCOPY(vec,cent);
 				vec[2]-= d;
 				Mat3MulVecfl(imat,vec);
-				vtop= addvertlist(vec);
+				vtop= addvertlist(vec, NULL);
 			}
 		} else {
 			vdown= v1;
