@@ -57,17 +57,22 @@ editmesh_lib: generic (no UI, no menus) operations/evaluators for editmesh data
 #include "BLI_arithb.h"
 #include "BLI_editVert.h"
 
+#include "BKE_customdata.h"
 #include "BKE_global.h"
 #include "BKE_mesh.h"
 #include "BKE_utildefines.h"
 
 #include "BIF_editmesh.h"
 
+#ifdef WITH_VERSE
+#include "BIF_verse.h"
+#endif
+
 #include "BSE_edit.h"
 
 #include "editmesh.h"
 
-/* ********* Selection ************ */
+/* ********* Selection History ************ */
 static int EM_check_selection(void *data)
 {
 	EditSelection *ese;
@@ -101,6 +106,20 @@ void EM_store_selection(void *data, int type)
 	}
 }
 
+void EM_validate_selections(void)
+{
+	EditSelection *ese, *nextese;
+	EditMesh *em = G.editMesh;
+	ese = em->selected.first;
+	while(ese){
+		nextese = ese->next;
+		if(ese->type == EDITVERT && !(((EditVert*)ese->data)->f & SELECT)) BLI_freelinkN(&(em->selected), ese);
+		else if(ese->type == EDITEDGE && !(((EditEdge*)ese->data)->f & SELECT)) BLI_freelinkN(&(em->selected), ese);
+		else if(ese->type == EDITFACE && !(((EditFace*)ese->data)->f & SELECT)) BLI_freelinkN(&(em->selected), ese);
+		ese = nextese;
+	}
+}
+
 static void EM_strip_selections(void)
 {
 	EditSelection *ese, *nextese;
@@ -690,6 +709,112 @@ void EM_hide_reset(void)
 		
 }
 
+void EM_data_interp_from_verts(EditVert *v1, EditVert *v2, EditVert *eve, float fac)
+{
+	EditMesh *em= G.editMesh;
+	void *src[2];
+	float w[2];
+
+	if (v1->data && v2->data) {
+		src[0]= v1->data;
+		src[1]= v2->data;
+		w[0] = 1.0f-fac;
+		w[1] = fac;
+
+		CustomData_em_interp(&em->vdata, src, w, NULL, 2, eve->data);
+	}
+}
+
+void EM_data_interp_from_faces(EditFace *efa1, EditFace *efa2, EditFace *efan, int i1, int i2, int i3, int i4)
+{
+	EditMesh *em= G.editMesh;
+	float w[2][4][4];
+	void *src[2];
+	int count = (efa2)? 2: 1;
+
+	if (efa1->data) {
+		/* set weights for copying from corners directly to other corners */
+		memset(w, 0, sizeof(w));
+
+		w[i1/4][0][i1%4]= 1.0f;
+		w[i2/4][1][i2%4]= 1.0f;
+		w[i3/4][2][i3%4]= 1.0f;
+		if (i4 != -1)
+			w[i4/4][3][i4%4]= 1.0f;
+
+		src[0]= efa1->data;
+		src[1]= (efa2)? efa2->data: NULL;
+
+		CustomData_em_interp(&em->fdata, src, NULL, (float*)w, count, efan->data);
+	}
+}
+
+EditFace *EM_face_from_faces(EditFace *efa1, EditFace *efa2, int i1, int i2, int i3, int i4)
+{
+	EditFace *efan;
+	EditVert **v[2];
+	
+	v[0]= &efa1->v1;
+	v[1]= (efa2)? &efa2->v1: NULL;
+
+	efan= addfacelist(v[i1/4][i1%4], v[i2/4][i2%4], v[i3/4][i3%4],
+		(i4 == -1)? 0: v[i4/4][i4%4], efa1, NULL);
+
+	EM_data_interp_from_faces(efa1, efa2, efan, i1, i2, i3, i4);
+	
+	return efan;
+}
+
+static void update_data_blocks(CustomData *olddata, CustomData *data)
+{
+	EditMesh *em= G.editMesh;
+	EditFace *efa;
+	EditVert *eve;
+	void *block;
+
+	if (data == &G.editMesh->vdata) {
+		for(eve= em->verts.first; eve; eve= eve->next) {
+			block = NULL;
+			CustomData_em_set_default(data, &block);
+			CustomData_em_copy_data(olddata, data, eve->data, &block);
+			CustomData_em_free_block(olddata, &eve->data);
+			eve->data= block;
+		}
+	}
+	else if (data == &G.editMesh->fdata) {
+		for(efa= em->faces.first; efa; efa= efa->next) {
+			block = NULL;
+			CustomData_em_set_default(data, &block);
+			CustomData_em_copy_data(olddata, data, efa->data, &block);
+			CustomData_em_free_block(olddata, &efa->data);
+			efa->data= block;
+		}
+	}
+}
+
+void EM_add_data_layer(CustomData *data, int type)
+{
+	CustomData olddata;
+
+	olddata= *data;
+	olddata.layers= (olddata.layers)? MEM_dupallocN(olddata.layers): NULL;
+	CustomData_add_layer(data, type, CD_CALLOC, NULL, 0);
+
+	update_data_blocks(&olddata, data);
+	if (olddata.layers) MEM_freeN(olddata.layers);
+}
+
+void EM_free_data_layer(CustomData *data, int type)
+{
+	CustomData olddata;
+
+	olddata= *data;
+	olddata.layers= (olddata.layers)? MEM_dupallocN(olddata.layers): NULL;
+	CustomData_free_layer_active(data, type, 0);
+
+	update_data_blocks(&olddata, data);
+	if (olddata.layers) MEM_freeN(olddata.layers);
+}
 
 /* ********  EXTRUDE ********* */
 
@@ -786,15 +911,16 @@ short extrudeflag_face_indiv(short flag, float *nor)
 	/* step 2: make new faces from faces */
 	for(efa= em->faces.last; efa; efa= efa->prev) {
 		if(efa->f & SELECT) {
-			v1= addvertlist(efa->v1->co);
-			v2= addvertlist(efa->v2->co);
-			v3= addvertlist(efa->v3->co);
+			v1= addvertlist(efa->v1->co, efa->v1);
+			v2= addvertlist(efa->v2->co, efa->v2);
+			v3= addvertlist(efa->v3->co, efa->v3);
+			
 			v1->f1= v2->f1= v3->f1= 1;
 			VECCOPY(v1->no, efa->n);
 			VECCOPY(v2->no, efa->n);
 			VECCOPY(v3->no, efa->n);
 			if(efa->v4) {
-				v4= addvertlist(efa->v4->co); 
+				v4= addvertlist(efa->v4->co, efa->v4); 
 				v4->f1= 1;
 				VECCOPY(v4->no, efa->n);
 			}
@@ -866,11 +992,11 @@ short extrudeflag_edges_indiv(short flag, float *nor)
 	/* make the faces */
 	for(eed= em->edges.first; eed; eed= eed->next) {
 		if(eed->f & flag) {
-			if(eed->v1->tmp.v == NULL) 
-				eed->v1->tmp.v = addvertlist(eed->v1->co);
-			if(eed->v2->tmp.v == NULL) 
-				eed->v2->tmp.v = addvertlist(eed->v2->co);
-			
+			if(eed->v1->tmp.v == NULL)
+				eed->v1->tmp.v = addvertlist(eed->v1->co, eed->v1);
+			if(eed->v2->tmp.v == NULL)
+				eed->v2->tmp.v = addvertlist(eed->v2->co, eed->v2);
+
 			if(eed->dir==1) 
 				addfacelist(eed->v1, eed->v2, 
 							eed->v2->tmp.v, eed->v1->tmp.v, 
@@ -914,7 +1040,7 @@ short extrudeflag_verts_indiv(short flag, float *nor)
 	/* make the edges */
 	for(eve= em->verts.first; eve; eve= eve->next) {
 		if(eve->f & flag) {
-			eve->tmp.v = addvertlist(eve->co);
+			eve->tmp.v = addvertlist(eve->co, eve);
 			addedgelist(eve, eve->tmp.v, NULL);
 		}
 		else eve->tmp.v = NULL;
@@ -1048,10 +1174,10 @@ static short extrudeflag_edge(short flag, float *nor)
 		if(eed->f & SELECT) {
 			if(eed->f2<2) {
 				if(eed->v1->tmp.v == NULL)
-					eed->v1->tmp.v = addvertlist(eed->v1->co);
+					eed->v1->tmp.v = addvertlist(eed->v1->co, eed->v1);
 				if(eed->v2->tmp.v == NULL)
-					eed->v2->tmp.v = addvertlist(eed->v2->co);
-				
+					eed->v2->tmp.v = addvertlist(eed->v2->co, eed->v2);
+
 				/* if del_old, the preferred normal direction is exact 
 				 * opposite as for keep old faces
 				 */
@@ -1070,14 +1196,14 @@ static short extrudeflag_edge(short flag, float *nor)
 	/* step 3: make new faces from faces */
 	for(efa= em->faces.last; efa; efa= efa->prev) {
 		if(efa->f & SELECT) {
-			if (efa->v1->tmp.v == NULL) 
-				efa->v1->tmp.v = addvertlist(efa->v1->co);
+			if (efa->v1->tmp.v == NULL)
+				efa->v1->tmp.v = addvertlist(efa->v1->co, efa->v1);
 			if (efa->v2->tmp.v ==NULL)
-				efa->v2->tmp.v = addvertlist(efa->v2->co);
-			if (efa->v3->tmp.v ==NULL) 
-				efa->v3->tmp.v = addvertlist(efa->v3->co);
-			if (efa->v4 && (efa->v4->tmp.v == NULL)) 
-				efa->v4->tmp.v = addvertlist(efa->v4->co);
+				efa->v2->tmp.v = addvertlist(efa->v2->co, efa->v2);
+			if (efa->v3->tmp.v ==NULL)
+				efa->v3->tmp.v = addvertlist(efa->v3->co, efa->v3);
+			if (efa->v4 && (efa->v4->tmp.v == NULL))
+				efa->v4->tmp.v = addvertlist(efa->v4->co, efa->v4);
 			
 			if(del_old==0) {	// keep old faces means flipping normal
 				if(efa->v4)
@@ -1307,7 +1433,7 @@ short extrudeflag_vert(short flag, float *nor)
 		eve->f &= ~128;  /* clear, for later test for loose verts */
 		if(eve->f & flag) {
 			sel= 1;
-			v1= addvertlist(0);
+			v1= addvertlist(0, NULL);
 			
 			VECCOPY(v1->co, eve->co);
 			v1->f= eve->f;
@@ -1363,8 +1489,8 @@ short extrudeflag_vert(short flag, float *nor)
 			if(eed->tmp.f) {
 				efa = eed->tmp.f;
 				efa2->mat_nr= efa->mat_nr;
-				efa2->tf= efa->tf;
 				efa2->flag= efa->flag;
+				CustomData_em_copy_data(&em->fdata, &em->fdata, &efa->data, &efa2->data);
 			}
 			
 			/* Needs smarter adaption of existing creases.
@@ -1503,7 +1629,8 @@ void translateflag(short flag, float *vec)
 /* helper call for below */
 static EditVert *adduplicate_vertex(EditVert *eve, int flag)
 {
-	EditVert *v1= addvertlist(eve->co);
+	/* FIXME: copy deformation weight from eve ok here? */
+	EditVert *v1= addvertlist(eve->co, eve);
 	
 	v1->f= eve->f;
 	eve->f-= flag;
@@ -1511,15 +1638,6 @@ static EditVert *adduplicate_vertex(EditVert *eve, int flag)
 	
 	eve->tmp.v = v1;
 	
-	/* FIXME: Copy deformation weight ? */
-	v1->totweight = eve->totweight;
-	if (eve->totweight){
-		v1->dw = MEM_mallocN (eve->totweight * sizeof(MDeformWeight), "deformWeight");
-		memcpy (v1->dw, eve->dw, eve->totweight * sizeof(MDeformWeight));
-	}
-	else
-		v1->dw=NULL;
-	
 	return v1;
 }
 
@@ -1682,18 +1800,15 @@ void flipface(EditFace *efa)
 		SWAP(EditVert *, efa->v2, efa->v4);
 		SWAP(EditEdge *, efa->e1, efa->e4);
 		SWAP(EditEdge *, efa->e2, efa->e3);
-		SWAP(unsigned int, efa->tf.col[1], efa->tf.col[3]);
-		SWAP(float, efa->tf.uv[1][0], efa->tf.uv[3][0]);
-		SWAP(float, efa->tf.uv[1][1], efa->tf.uv[3][1]);
+		EM_data_interp_from_faces(efa, NULL, efa, 0, 3, 2, 1);
 	}
 	else {
 		SWAP(EditVert *, efa->v2, efa->v3);
 		SWAP(EditEdge *, efa->e1, efa->e3);
-		SWAP(unsigned int, efa->tf.col[1], efa->tf.col[2]);
 		efa->e2->dir= 1-efa->e2->dir;
-		SWAP(float, efa->tf.uv[1][0], efa->tf.uv[2][0]);
-		SWAP(float, efa->tf.uv[1][1], efa->tf.uv[2][1]);
+		EM_data_interp_from_faces(efa, NULL, efa, 0, 2, 1, 3);
 	}
+
 	if(efa->v4) CalcNormFloat4(efa->v1->co, efa->v2->co, efa->v3->co, efa->v4->co, efa->n);
 	else CalcNormFloat(efa->v1->co, efa->v2->co, efa->v3->co, efa->n);
 }
@@ -1711,6 +1826,10 @@ void flip_editnormals(void)
 		}
 		efa= efa->next;
 	}
+#ifdef WITH_VERSE
+	if(G.editMesh->vnode)
+		sync_all_versefaces_with_editfaces((VNode*)G.editMesh->vnode);
+#endif
 }
 
 /* does face centers too */
