@@ -1,5 +1,5 @@
 /**
- * $Id: editnla.c,v 1.62 2006/06/15 04:13:25 hos Exp $
+ * $Id: editnla.c,v 1.72 2007/01/13 02:40:28 aligorith Exp $
  *
  * ***** BEGIN GPL BLOCK *****
  *
@@ -54,6 +54,7 @@
 #include "DNA_userdef_types.h"
 
 #include "BKE_action.h"
+#include "BKE_blender.h"
 #include "BKE_depsgraph.h"
 #include "BKE_group.h"
 #include "BKE_global.h"
@@ -70,6 +71,7 @@
 #include "BIF_editview.h"
 #include "BIF_toolbox.h"
 #include "BIF_editnla.h"
+#include "BIF_editaction.h"
 
 #include "BSE_editipo.h"
 #include "BSE_editnla_types.h"
@@ -80,6 +82,7 @@
 #include "BSE_filesel.h"
 #include "BDR_editobject.h"
 #include "BSE_drawnla.h"
+#include "BSE_time.h"
 
 #include "blendef.h"
 #include "mydevice.h"
@@ -96,7 +99,6 @@ static Base *get_nearest_nlachannel_ob_key (float *index, short *sel);
 static bAction *get_nearest_nlachannel_ac_key (float *index, short *sel);
 static Base *get_nearest_nlastrip (bActionStrip **rstrip, short *sel);
 static void mouse_nlachannels(short mval[2]);
-static void convert_nla(short mval[2]);
 
 /* ******************** SPACE: NLA ********************** */
 
@@ -106,6 +108,9 @@ void shift_nlastrips_up(void) {
 	bActionStrip *strip, *prevstrip;
 
 	for (base=G.scene->base.first; base; base=base->next) {
+		if (base->object->nlaflag & OB_NLA_COLLAPSED)
+			continue;
+		
 		for (strip = base->object->nlastrips.first; 
 			 strip; strip=strip->next){
 			if (strip->flag & ACTSTRIP_SELECT) {
@@ -143,6 +148,9 @@ void shift_nlastrips_down(void) {
 	bActionStrip *strip, *nextstrip;
 
 	for (base=G.scene->base.first; base; base=base->next) {
+		if (base->object->nlaflag & OB_NLA_COLLAPSED)
+			continue;
+		
 		for (strip = base->object->nlastrips.last; 
 			 strip; strip=strip->prev){
 			if (strip->flag & ACTSTRIP_SELECT) {
@@ -207,6 +215,9 @@ void reset_action_strips(int val)
 	bActionStrip *strip;
 	
 	for (base=G.scene->base.first; base; base=base->next) {
+		if (base->object->nlaflag & OB_NLA_COLLAPSED)
+			continue;
+		
 		for (strip = base->object->nlastrips.last; strip; strip=strip->prev) {
 			if (strip->flag & ACTSTRIP_SELECT) {
 				if(val==2) {
@@ -228,16 +239,36 @@ void reset_action_strips(int val)
 	allqueue (REDRAWNLA, 0);
 }
 
-void snap_action_strips(void)
+void snap_action_strips(int snap_mode)
 {
 	Base *base;
 	bActionStrip *strip;
 	
 	for (base=G.scene->base.first; base; base=base->next) {
+		if (base->object->nlaflag & OB_NLA_COLLAPSED)
+			continue;
+		
 		for (strip = base->object->nlastrips.last; strip; strip=strip->prev) {
 			if (strip->flag & ACTSTRIP_SELECT) {
-				strip->start= floor(strip->start+0.5);
-				strip->end= floor(strip->end+0.5);
+				if (snap_mode==1) {
+					/* nearest frame */
+					strip->start= floor(strip->start+0.5);
+					strip->end= floor(strip->end+0.5);
+				}
+				else if (snap_mode==2) {
+					/* current frame */
+					float diff;
+					if (CFRA < strip->start) {
+						diff = (strip->start - CFRA);
+						strip->start -= diff;
+						strip->end -= diff;
+					}
+					else {
+						diff = (CFRA - strip->start);
+						strip->start += diff;
+						strip->end += diff;
+					}
+				}
 			}
 		}
 	}
@@ -276,70 +307,29 @@ static void set_active_strip(Object *ob, bActionStrip *act)
 	}	
 }
 
-static void convert_nla(short mval[2])
+void convert_nla(void)
 {
-	bActionStrip *strip, *nstrip;
-	Base *base;
-	float x,y;
-	float	ymax, ymin;
-	int sel=0;
-	short event;
+	bActionStrip *strip;
+	Object *ob= OBACT;
 	char str[128];
+	short event;
 	
-	/* Find out what strip we're over */
-	ymax = count_nla_levels() * (NLACHANNELSKIP+NLACHANNELHEIGHT);
-	ymax+= NLACHANNELHEIGHT/2;
-	
-	areamouseco_to_ipoco(G.v2d, mval, &x, &y);
-	
-	for (base=G.scene->base.first; base; base=base->next){
-		if (nla_filter(base)) {
-			
-			/* Area that encloses object name (or ipo) */
-			ymin=ymax-(NLACHANNELHEIGHT+NLACHANNELSKIP);
-			ymax=ymin;
-			
-			/* Check action ipo */
-			if (base->object->action) {
-				ymin=ymax-(NLACHANNELSKIP+NLACHANNELHEIGHT);
-				if (y>=ymin && y<=ymax)
-					break;
-				ymax=ymin;
-			}			
-				
-			/* Check nlastrips */
-			for (strip=base->object->nlastrips.first; strip; strip=strip->next){
-				ymin=ymax-(NLACHANNELSKIP+NLACHANNELHEIGHT);
-				if (y>=ymin && y<=ymax){
-					sel = 1;
-					break;
-				}
-				ymax=ymin;
-			}
-			if (sel)
-				break;
-		}
+	if ((ob==NULL)||(ob->action==NULL)) {
+		error("Need active Object to convert Action to NLA Strip");
+		return;
 	}
 	
-	if (base==0 || base->object->action==NULL)
-		return;
-	
-	sprintf(str, "Convert Action%%t|%s to NLA Strip%%x1", base->object->action->id.name+2);
+	sprintf(str, "Convert Action%%t|%s to NLA Strip%%x1", ob->action->id.name+2);
 	event = pupmenu(str);
 	
-	switch (event){
-	case 1:
-		if (base->object->action) {
+	if (event==1) {
+		if (ob->action) {
 			deselect_nlachannel_keys(0);
-			nstrip = convert_action_to_strip(base->object); //creates a new NLA strip from the action in given object
-			set_active_strip(base->object, nstrip);
+			strip = convert_action_to_strip(ob); //creates a new NLA strip from the action in given object
+			set_active_strip(ob, strip);
 			BIF_undo_push("Convert NLA");
 			allqueue (REDRAWNLA, 0);
 		}
-
-		break;
-	default:
-		break;
 	}
 }
 
@@ -392,6 +382,58 @@ static void add_nla_block(short event)
 	BIF_undo_push("Add NLA strip");
 }
 
+static void add_nla_block_by_name(char name[32], Object *ob, short hold, short add, float repeat)
+{
+	bAction *act=NULL;
+	bActionStrip *strip;
+	int		cur;
+
+	if (name){
+		for (cur = 1, act=G.main->action.first; act; act=act->id.next, cur++){
+			if (strcmp(name,act->id.name)==0) {
+				break;
+			}
+		}
+	}
+	
+	/* Bail out if no action was chosen */
+	if (!act){
+		return;
+	}
+	
+	/* Initialize the new action block */
+	strip = MEM_callocN(sizeof(bActionStrip), "bActionStrip");
+	
+	deselect_nlachannel_keys(0);
+	
+	/* Link the action to the strip */
+	strip->act = act;
+	calc_action_range(strip->act, &strip->actstart, &strip->actend, 1);
+	strip->start = G.scene->r.cfra;		/* could be mval[0] another time... */
+	strip->end = strip->start + (strip->actend-strip->actstart);
+		/* simple prevention of zero strips */
+	if(strip->start>strip->end-2) 
+		strip->end= strip->start+100;
+	
+	strip->flag = ACTSTRIP_SELECT|ACTSTRIP_LOCK_ACTION; //|ACTSTRIP_USEMATCH;
+	
+	if (hold==1)
+		strip->flag = strip->flag|ACTSTRIP_HOLDLASTFRAME;
+		
+	if (add==1)
+		strip->mode = ACTSTRIPMODE_ADD;
+	
+	find_stridechannel(ob, strip);
+	
+	set_active_strip(ob, strip);
+	
+	strip->repeat = repeat;
+	
+	act->id.us++;
+	
+	BLI_addtail(&ob->nlastrips, strip);
+}
+
 static void add_nla_databrowse_callback(unsigned short val)
 {
 	/* val is not used, databrowse needs it to optional pass an event */
@@ -405,7 +447,7 @@ static void add_nla_databrowse_callback(unsigned short val)
 }
 
 /* Adds strip to to active Object */
-static void add_nlablock(void)
+void add_nlablock(void)
 {
 	Object *ob= OBACT;
 	short event;
@@ -437,6 +479,48 @@ static void add_nlablock(void)
 	}
 }
 
+/* Creates a new action, and makes a new actionstrip of that */
+void add_empty_nlablock(void)
+{
+	Object *ob= OBACT;
+	bAction *act= NULL;
+	bActionStrip *strip;
+	
+	/* check for active object first - will add strip to active object */
+	if (ob == NULL) 
+		return;
+		
+	/* make new action */
+	if ((ob->type == OB_ARMATURE) && (ob->flag & OB_POSEMODE))
+		act= add_empty_action(ID_AR);
+	else
+		act= add_empty_action(ID_OB);
+		
+	/* make a new strip for it */
+	add_nla_block_by_name(act->id.name, ob, 0, 1, 1.0f);
+	strip= ob->nlastrips.last; 
+	
+	/* change some settings of the strip - try to avoid bad scaling */
+	if ((EFRA-CFRA) < 100) {
+		strip->flag &= ~ACTSTRIP_LOCK_ACTION;
+		strip->actstart = CFRA;
+		strip->actend = CFRA + 100;
+		
+		strip->start = CFRA;
+		strip->end = CFRA + 100;
+	}
+	else {
+		strip->flag &= ~ACTSTRIP_LOCK_ACTION;
+		strip->actstart = CFRA;
+		strip->actend = EFRA;
+		
+		strip->start = CFRA;
+		strip->end = EFRA;
+	}
+	
+	BIF_undo_push("Add NLA strip");
+}
+
 /* Adds strip to to active Object */
 static void relink_active_strip(void)
 {
@@ -448,6 +532,7 @@ static void relink_active_strip(void)
 	char *str;
 	
 	if(ob==NULL) return;
+	if(ob->nlaflag & OB_NLA_COLLAPSED) return;
 	
 	for (strip = ob->nlastrips.first; strip; strip=strip->next)
 		if(strip->flag & ACTSTRIP_ACTIVE)
@@ -511,22 +596,25 @@ static void mouse_nlachannels(short mval[2])
 			}
 			click--;
 			
-			/* See if this is an action */
-			if (ob->action){
-				if (click==0) {
-					actclick= 1;
-					break;
+			/* see if any strips under object */
+			if ((ob->nlaflag & OB_NLA_COLLAPSED)==0) {
+				/* See if this is an action */
+				if (ob->action){
+					if (click==0) {
+						actclick= 1;
+						break;
+					}
+					click--;
 				}
-				click--;
-			}
 
-			/* See if this is an nla strip */
-			if(ob->nlastrips.first) {
-				for (strip = ob->nlastrips.first; strip; strip=strip->next){
-					if (click==0) break;
-					click--;				
+				/* See if this is an nla strip */
+				if(ob->nlastrips.first) {
+					for (strip = ob->nlastrips.first; strip; strip=strip->next){
+						if (click==0) break;
+						click--;				
+					}
+					if (strip && click==0) break;
 				}
-				if (strip && click==0) break;
 			}
 		}
 	}
@@ -552,9 +640,15 @@ static void mouse_nlachannels(short mval[2])
 	else if(strip) /* set action */
 		set_active_strip(ob, strip);
 
-	/* override option for NLA */
-	if(obclick && mval[0]<25)
+	/* icon toggles beside strip */
+	if (obclick && mval[0]<20) {
+		/* collapse option for NLA object strip */
+		ob->nlaflag ^= OB_NLA_COLLAPSED;
+	}
+	else if(obclick && mval[0]<36) {
+		/* override option for NLA */
 		ob->nlaflag ^= OB_NLA_OVERRIDE;
+	}
 	
 	ob->ctime= -1234567.0f;	// eveil! 
 	DAG_object_flush_update(G.scene, ob, OB_RECALC_OB|OB_RECALC_DATA);
@@ -576,8 +670,7 @@ void deselect_nlachannel_keys (int test)
 	
 	/* Determine if this is selection or deselection */
 	if (test){
-		for (base=G.scene->base.first; base && sel; base=base->next){
-			
+		for (base=G.scene->base.first; base && sel; base=base->next){		
 			/* Test object ipos */
 			if (is_ipo_key_selected(base->object->ipo)){
 				sel = 0;
@@ -594,6 +687,10 @@ void deselect_nlachannel_keys (int test)
 				}
 			}
 			
+			/* check if collapsed */
+			if (base->object->nlaflag & OB_NLA_COLLAPSED)
+				continue;
+			
 			/* Test action ipos */
 			if (sel){
 				if (base->object->action){
@@ -632,8 +729,7 @@ void deselect_nlachannel_keys (int test)
 	
 	
 	/* Set the flags */
-	for (base=G.scene->base.first; base; base=base->next){
-		
+	for (base=G.scene->base.first; base; base=base->next){		
 		/* Set the object ipos */
 		set_ipo_key_selection(base->object->ipo, sel);
 		
@@ -642,6 +738,10 @@ void deselect_nlachannel_keys (int test)
 			set_ipo_key_selection(conchan->ipo, sel);			
 		}
 
+		/* check if collapsed */
+		if (base->object->nlaflag & OB_NLA_COLLAPSED)
+			continue;
+		
 		/* Set the action ipos */
 		if (base->object->action){
 			for (chan=base->object->action->chanbase.first; chan; chan=chan->next){
@@ -697,7 +797,6 @@ void transform_nlachannel_keys(int mode, int dummy)
 
 	/* Ensure that partial selections result in beztriple selections */
 	for (base=G.scene->base.first; base; base=base->next){
-
 		/* Check object ipos */
 		i= fullselect_ipo_keys(base->object->ipo);
 		if(i) base->flag |= BA_HAS_RECALC_OB;
@@ -707,6 +806,10 @@ void transform_nlachannel_keys(int mode, int dummy)
 		for(conchan=base->object->constraintChannels.first; conchan; conchan=conchan->next)
 			tvtot+=fullselect_ipo_keys(conchan->ipo);			
 		
+		/* skip actions and nlastrips if object is collapsed */
+		if (base->object->nlaflag & OB_NLA_COLLAPSED)
+			continue;
+		
 		/* Check action ipos */
 		if (base->object->action){
 			/* exclude if strip is selected too */
@@ -718,13 +821,17 @@ void transform_nlachannel_keys(int mode, int dummy)
 			if(strip==NULL) {
 				
 				for (chan=base->object->action->chanbase.first; chan; chan=chan->next){
-					i= fullselect_ipo_keys(chan->ipo);
-					if(i) base->flag |= BA_HAS_RECALC_OB|BA_HAS_RECALC_DATA;
-					tvtot+=i;
+					if (EDITABLE_ACHAN(chan)) {
+						i= fullselect_ipo_keys(chan->ipo);
+						if(i) base->flag |= BA_HAS_RECALC_OB|BA_HAS_RECALC_DATA;
+						tvtot+=i;
+					}
 					
 					/* Check action constraint ipos */
-					for (conchan=chan->constraintChannels.first; conchan; conchan=conchan->next)
-						tvtot+=fullselect_ipo_keys(conchan->ipo);
+					for (conchan=chan->constraintChannels.first; conchan; conchan=conchan->next) {
+						if (EDITABLE_CONCHAN(conchan))
+							tvtot+=fullselect_ipo_keys(conchan->ipo);
+					}
 				}
 			}		
 		}
@@ -754,6 +861,10 @@ void transform_nlachannel_keys(int mode, int dummy)
 		for (conchan=base->object->constraintChannels.first; conchan; conchan=conchan->next)
 			tvtot=add_trans_ipo_keys(conchan->ipo, tv, tvtot);
 
+		/* skip actions and nlastrips if object collapsed */
+		if (base->object->nlaflag & OB_NLA_COLLAPSED)
+			continue;
+			
 		/* Manipulate action ipos */
 		if (base->object->action){
 			/* exclude if strip is selected too */
@@ -762,14 +873,18 @@ void transform_nlachannel_keys(int mode, int dummy)
 					if(strip->act==base->object->action)
 						break;
 			}
+			
+			/* can include - no selected strip is action */
 			if(strip==NULL) {
-				
 				for (chan=base->object->action->chanbase.first; chan; chan=chan->next){
-					tvtot=add_trans_ipo_keys(chan->ipo, tv, tvtot);
+					if (EDITABLE_ACHAN(chan))
+						tvtot=add_trans_ipo_keys(chan->ipo, tv, tvtot);
 
 					/* Manipulate action constraint ipos */
-					for (conchan=chan->constraintChannels.first; conchan; conchan=conchan->next)
-						tvtot=add_trans_ipo_keys(conchan->ipo, tv, tvtot);
+					for (conchan=chan->constraintChannels.first; conchan; conchan=conchan->next) {
+						if (EDITABLE_CONCHAN(conchan))
+							tvtot=add_trans_ipo_keys(conchan->ipo, tv, tvtot);
+					}
 				}
 			}
 		}
@@ -952,12 +1067,8 @@ void delete_nlachannel_keys(void)
 	bActionChannel *chan;
 	bConstraintChannel *conchan;
 	bActionStrip *strip, *nextstrip;
-	
-	if (!okee("Erase selected strips and/or keys"))
-		return;
-	
+		
 	for (base = G.scene->base.first; base; base=base->next){
-
 		/* Delete object ipos */
 		delete_ipo_keys(base->object->ipo);
 		
@@ -965,6 +1076,10 @@ void delete_nlachannel_keys(void)
 		for(conchan=base->object->constraintChannels.first; conchan; conchan=conchan->next)
 			delete_ipo_keys(conchan->ipo);
 
+		/* skip actions and nlastrips if object collapsed */
+		if (base->object->nlaflag & OB_NLA_COLLAPSED)
+			continue;
+			
 		/* Delete NLA strips */
 		for (strip = base->object->nlastrips.first; strip; strip=nextstrip){
 			nextstrip=strip->next;
@@ -978,10 +1093,14 @@ void delete_nlachannel_keys(void)
 		/* Delete action ipos */
 		if (base->object->action){
 			for (chan=base->object->action->chanbase.first; chan; chan=chan->next){
-				delete_ipo_keys(chan->ipo);
+				if (EDITABLE_ACHAN(chan))
+					delete_ipo_keys(chan->ipo);
+					
 				/* Delete action constraint keys */
-				for(conchan=chan->constraintChannels.first; conchan; conchan=conchan->next)
-					delete_ipo_keys(conchan->ipo);
+				for(conchan=chan->constraintChannels.first; conchan; conchan=conchan->next){
+					if (EDITABLE_CONCHAN(conchan))
+						delete_ipo_keys(conchan->ipo);
+				}
 			}
 		}
 	}
@@ -1004,14 +1123,18 @@ void duplicate_nlachannel_keys(void)
 	bActionStrip *strip, *laststrip;
 	
 	/* Find selected items */
-	for (base = G.scene->base.first; base; base=base->next){
+	for (base = G.scene->base.first; base; base=base->next){	
 		/* Duplicate object keys */
 		duplicate_ipo_keys(base->object->ipo);
 		
 		/* Duplicate object constraint keys */
 		for(conchan=base->object->constraintChannels.first; conchan; conchan=conchan->next)
 			duplicate_ipo_keys(conchan->ipo);
-
+		
+		/* skip actions and nlastrips if object collapsed */
+		if (base->object->nlaflag & OB_NLA_COLLAPSED)
+			continue;
+			
 		/* Duplicate nla strips */
 		laststrip = base->object->nlastrips.last;
 		for (strip=base->object->nlastrips.first; strip; strip=strip->next){
@@ -1034,10 +1157,14 @@ void duplicate_nlachannel_keys(void)
 		/* Duplicate actionchannel keys */
 		if (base->object->action){
 			for (chan=base->object->action->chanbase.first; chan; chan=chan->next){
-				duplicate_ipo_keys(chan->ipo);
+				if (EDITABLE_ACHAN(chan))
+					duplicate_ipo_keys(chan->ipo);
+					
 				/* Duplicate action constraint keys */
-				for(conchan=chan->constraintChannels.first; conchan; conchan=conchan->next)
-					duplicate_ipo_keys(conchan->ipo);
+				for(conchan=chan->constraintChannels.first; conchan; conchan=conchan->next) {
+					if (EDITABLE_CONCHAN(conchan))
+						duplicate_ipo_keys(conchan->ipo);
+				}
 			}
 		}
 	}
@@ -1058,11 +1185,11 @@ void borderselect_nla(void)
 	bConstraintChannel *conchan;
 	
 	if ( (val = get_border (&rect, 3)) ){
-    if (val == LEFTMOUSE)
-      selectmode = SELECT_ADD;
-    else
-      selectmode = SELECT_SUBTRACT;
-
+	    if (val == LEFTMOUSE)
+			selectmode = SELECT_ADD;
+	    else
+			selectmode = SELECT_SUBTRACT;
+	  
 		mval[0]= rect.xmin;
 		mval[1]= rect.ymin+2;
 		areamouseco_to_ipoco(G.v2d, mval, &rectf.xmin, &rectf.ymin);
@@ -1093,7 +1220,11 @@ void borderselect_nla(void)
 				}
 				
 				ymax=ymin;
-
+				
+				/* skip actions and nlastrips if object collapsed */
+				if (base->object->nlaflag & OB_NLA_COLLAPSED)
+					continue;
+				
 				/* Check action ipos */
 				if (base->object->action){
 					bActionChannel *chan;
@@ -1135,9 +1266,11 @@ void borderselect_nla(void)
 			}
 		}	
 		BIF_undo_push("Border select NLA");
-		allqueue(REDRAWNLA, 0);
-		allqueue(REDRAWACTION, 0);
+		allqueue(REDRAWTIME, 0);
 		allqueue(REDRAWIPO, 0);
+		allqueue(REDRAWACTION, 0);
+		allqueue(REDRAWNLA, 0);
+		allqueue(REDRAWSOUND, 0);
 	}
 }
 
@@ -1149,6 +1282,7 @@ static void mouse_nla(int selectmode)
 	bActionChannel *chan;
 	bActionStrip *rstrip;
 	bConstraintChannel *conchan;
+	TimeMarker *marker;
 	float	selx;
 	short	mval[2];
 	short sel, isdone=0;
@@ -1157,6 +1291,7 @@ static void mouse_nla(int selectmode)
 	
 	/* Try object ipo or ob-constraint ipo selection */
 	base= get_nearest_nlachannel_ob_key(&selx, &sel);
+	marker=find_nearest_marker(1);
 	if (base) {
 		isdone= 1;
 		
@@ -1171,6 +1306,31 @@ static void mouse_nla(int selectmode)
 		for (conchan=base->object->constraintChannels.first; conchan; conchan=conchan->next)
 			select_ipo_key(conchan->ipo, selx, selectmode);
 	}
+	else if (marker) {
+		/* marker */		
+		if (selectmode == SELECT_REPLACE) {			
+			deselect_markers(0, 0);
+			marker->flag |= SELECT;
+		}
+		else if (selectmode == SELECT_INVERT) {
+			if (marker->flag & SELECT)
+				marker->flag &= ~SELECT;
+			else
+				marker->flag |= SELECT;
+		}
+		else if (selectmode == SELECT_ADD) 
+			marker->flag |= SELECT;
+		else if (selectmode == SELECT_SUBTRACT)
+			marker->flag &= ~SELECT;
+		
+		std_rmouse_transform(transform_markers);
+		
+		allqueue(REDRAWTIME, 0);
+		allqueue(REDRAWIPO, 0);
+		allqueue(REDRAWACTION, 0);
+		allqueue(REDRAWNLA, 0);
+		allqueue(REDRAWSOUND, 0);
+	}
 	else {
 		/* Try action ipo selection */
 		act= get_nearest_nlachannel_ac_key(&selx, &sel);
@@ -1250,6 +1410,10 @@ static Base *get_nearest_nlastrip (bActionStrip **rstrip, short *sel)
 			
 			/* Skip object ipos */
 			ymax-=(NLACHANNELHEIGHT+NLACHANNELSKIP);
+			
+			/* check if skip strips if collapsed */
+			if (base->object->nlaflag & OB_NLA_COLLAPSED)
+				continue;
 
 			/* Skip action ipos */
 			if (base->object->action)
@@ -1383,6 +1547,10 @@ static Base *get_nearest_nlachannel_ob_key (float *index, short *sel)
 
 			ymax=ymin;
 			
+			/* Skip actions and nlastrips if object is collapsed */
+			if (base->object->nlaflag & OB_NLA_COLLAPSED)
+				continue;
+			
 			/* Skip action ipos */
 			if (base->object->action){
 				ymax-=(NLACHANNELHEIGHT+NLACHANNELSKIP);
@@ -1433,6 +1601,11 @@ static bAction *get_nearest_nlachannel_ac_key (float *index, short *sel)
 			
 			/* Skip object ipo and ob-constraint ipo */
 			ymin=ymax-(NLACHANNELHEIGHT+NLACHANNELSKIP);
+			
+			/* skip this object if it is collapsed */
+			if (base->object->nlaflag & OB_NLA_COLLAPSED)
+				continue;
+			
 			ymax=ymin;
 			
 			/* Handle action ipos */
@@ -1553,10 +1726,12 @@ static Object *get_object_from_active_strip(void) {
 	bActionStrip *strip;
 	
 	for (base=G.scene->base.first; base; base=base->next) {
-		for (strip = base->object->nlastrips.first; 
-			 strip; strip=strip->next){
-			if (strip->flag & ACTSTRIP_SELECT) {
-				return base->object;
+		if ((base->object->nlaflag & OB_NLA_COLLAPSED)==0) {
+			for (strip = base->object->nlastrips.first; 
+				 strip; strip=strip->next){
+				if (strip->flag & ACTSTRIP_SELECT) {
+					return base->object;
+				}
 			}
 		}
 	}
@@ -1605,21 +1780,53 @@ void winqreadnlaspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 				break;
 				
 			case EQUALKEY:
-			case PAGEUPKEY:
 				shift_nlastrips_up();
 				break;
+			
+			case PAGEUPKEY:
+				if (G.qual & LR_CTRLKEY)
+					shift_nlastrips_up();
+				else {
+					nextprev_marker(1);
+					allqueue(REDRAWTIME, 0);
+					allqueue(REDRAWIPO, 0);
+					allqueue(REDRAWACTION, 0);
+					allqueue(REDRAWNLA, 0);
+					allqueue(REDRAWSOUND, 0);
+				}				
+				break;
 				
 			case MINUSKEY:
-			case PAGEDOWNKEY:
 				shift_nlastrips_down();
 				break;
 				
+			case PAGEDOWNKEY:
+				if (G.qual & LR_CTRLKEY)
+					shift_nlastrips_down();
+				else {
+					nextprev_marker(-1);
+					allqueue(REDRAWTIME, 0);
+					allqueue(REDRAWIPO, 0);
+					allqueue(REDRAWACTION, 0);
+					allqueue(REDRAWNLA, 0);
+					allqueue(REDRAWSOUND, 0);
+				}
+				break;
+				
 			case AKEY:
 				if (G.qual & LR_SHIFTKEY){
 					add_nlablock();
 					allqueue (REDRAWNLA, 0);
 					allqueue (REDRAWVIEW3D, 0);
 				}
+				else if (G.qual & LR_CTRLKEY) {
+					deselect_markers(1, 0);
+					allqueue(REDRAWTIME, 0);
+					allqueue(REDRAWIPO, 0);
+					allqueue(REDRAWACTION, 0);
+					allqueue(REDRAWNLA, 0);
+					allqueue(REDRAWSOUND, 0);
+				}
 				else{
 					if (mval[0]>=NLAWIDTH)
 						deselect_nlachannel_keys(1);
@@ -1634,40 +1841,74 @@ void winqreadnlaspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 				break;
 				
 			case BKEY:
-			  if (G.qual & LR_SHIFTKEY){
-			    bake_all_to_action();
-			    allqueue (REDRAWNLA, 0);
-			    allqueue (REDRAWVIEW3D, 0);
-			    BIF_undo_push("Bake All To Action");
-			    ob = get_object_from_active_strip();
-			    //build_match_caches(ob);
-			  }
-			  else
-			    borderselect_nla();
-			  break;
+				if (G.qual & LR_SHIFTKEY){
+					bake_all_to_action();
+					allqueue (REDRAWNLA, 0);
+				    allqueue (REDRAWVIEW3D, 0);
+				    BIF_undo_push("Bake All To Action");
+				    ob = get_object_from_active_strip();
+				    //build_match_caches(ob);
+				}
+				else if (G.qual & LR_CTRLKEY) 
+					borderselect_markers();
+				else
+					borderselect_nla();
+				break;
 				
 			case CKEY:
-				convert_nla(mval);
+				if(G.qual==LR_CTRLKEY) {
+					if(okee("Copy Modifiers"))
+						copy_action_modifiers();
+				}
+				else convert_nla();
 				break;
 				
 			case DKEY:
-				if (G.qual & LR_SHIFTKEY && mval[0]>=NLAWIDTH){
+				if (G.qual == (LR_CTRLKEY|LR_SHIFTKEY) && mval[0]>=NLAWIDTH) {
+					duplicate_marker();
+				}
+				else if (G.qual & LR_SHIFTKEY && mval[0]>=NLAWIDTH){
 					duplicate_nlachannel_keys();
 					update_for_newframe_muted();
 				}
+				
 				break;
 				
 			case GKEY:
-				if (mval[0]>=NLAWIDTH)
-					transform_nlachannel_keys ('g', 0);
-				update_for_newframe_muted();
+				if (mval[0]>=NLAWIDTH) {
+					if (G.qual & LR_CTRLKEY) {
+						transform_markers('g', 0);
+					}
+					else {
+						transform_nlachannel_keys ('g', 0);
+						update_for_newframe_muted();
+					}
+				}
 				break;
-				
+			
+			case MKEY:
+				/* marker operations */
+				if (G.qual == 0)
+					add_marker(CFRA);
+				else if (G.qual == LR_CTRLKEY)
+					rename_marker();
+				else 
+					break;
+				allqueue(REDRAWTIME, 0);
+				allqueue(REDRAWIPO, 0);
+				allqueue(REDRAWACTION, 0);
+				allqueue(REDRAWNLA, 0);
+				allqueue(REDRAWSOUND, 0);
+				break;				
+			
 			case NKEY:
 				if(G.qual==0) {
 					toggle_blockhandler(curarea, NLA_HANDLER_PROPERTIES, UI_PNL_TO_MOUSE);
 					scrarea_queue_winredraw(curarea);
 				}
+				else if (G.qual & LR_SHIFTKEY) {
+					add_empty_nlablock();
+				}
 				break;
 			case LKEY:
 				relink_active_strip();
@@ -1682,8 +1923,9 @@ void winqreadnlaspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 						reset_action_strips(2);
 				}
 				else if(G.qual & LR_SHIFTKEY) {
-					if(okee("Snap Strips to Frame"))
-						snap_action_strips();
+					val= pupmenu("Snap To%t|Nearest Frame%x1|Current Frame%x2");
+					if (val==1 || val==2)
+						snap_action_strips(val);
 				}
 				else {
 					if (mval[0]>=NLAWIDTH)
@@ -1694,10 +1936,22 @@ void winqreadnlaspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 				
 			case DELKEY:
 			case XKEY:
-				if (mval[0]>=NLAWIDTH)
-					delete_nlachannel_keys ();
-				
-				update_for_newframe_muted();
+				if (mval[0]>=NLAWIDTH) {
+					val= pupmenu("Erase selected%t|Strips and/or Keys%x1|Markers%x2");
+					if (val==1) {
+						delete_nlachannel_keys();
+						update_for_newframe_muted();
+					}
+					else if (val==2) {
+						remove_marker();
+						
+						allqueue(REDRAWTIME, 0);
+						allqueue(REDRAWIPO, 0);
+						allqueue(REDRAWACTION, 0);
+						allqueue(REDRAWNLA, 0);
+						allqueue(REDRAWSOUND, 0);
+					}				
+				}
 				break;
 				
 				/* LEFTMOUSE and RIGHTMOUSE event codes can be swapped above,
@@ -1760,60 +2014,6 @@ void winqreadnlaspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 	if(doredraw) scrarea_queue_winredraw(curarea);
 }
 
-static void add_nla_block_by_name(char name[32], Object *ob, short hold, short add, float repeat)
-{
-	bAction *act=NULL;
-	bActionStrip *strip;
-	int		cur;
-
-	if (name){
-		for (cur = 1, act=G.main->action.first; act; act=act->id.next, cur++){
-			if (strcmp(name,act->id.name)==0) {
-				break;
-			}
-		}
-	}
-	
-	/* Bail out if no action was chosen */
-	if (!act){
-		return;
-	}
-	
-	/* Initialize the new action block */
-	strip = MEM_callocN(sizeof(bActionStrip), "bActionStrip");
-	
-	deselect_nlachannel_keys(0);
-	
-	/* Link the action to the strip */
-	strip->act = act;
-	calc_action_range(strip->act, &strip->actstart, &strip->actend, 1);
-	strip->start = G.scene->r.cfra;		/* could be mval[0] another time... */
-	strip->end = strip->start + (strip->actend-strip->actstart);
-		/* simple prevention of zero strips */
-	if(strip->start>strip->end-2) 
-		strip->end= strip->start+100;
-	
-	strip->flag = ACTSTRIP_SELECT|ACTSTRIP_LOCK_ACTION; //|ACTSTRIP_USEMATCH;
-	
-	if (hold==1)
-		strip->flag = strip->flag|ACTSTRIP_HOLDLASTFRAME;
-		
-	if (add==1)
-		strip->mode = ACTSTRIPMODE_ADD;
-	
-	find_stridechannel(ob, strip);
-	
-	set_active_strip(ob, strip);
-	
-	strip->repeat = repeat;
-	
-	act->id.us++;
-	
-	BLI_addtail(&ob->nlastrips, strip);
-
-	BIF_undo_push("Add NLA strip");
-}
-
 void bake_all_to_action(void)
 {
 	Object		*ob;
@@ -1844,7 +2044,41 @@ void bake_all_to_action(void)
 				repeat=1.0;
 				printf("about to add nla block...\n");
 				add_nla_block_by_name(newAction->id.name, ob, hold, add, repeat);
+				BIF_undo_push("Add NLA strip");
 			}
 		}
 	}
 }
+
+void copy_action_modifiers(void)
+{
+	bActionStrip *strip, *actstrip;
+	Object *ob= OBACT;
+	
+	if(ob==NULL)
+		return;
+	
+	/* active strip */
+	for (actstrip=ob->nlastrips.first; actstrip; actstrip=actstrip->next)
+		if(actstrip->flag & ACTSTRIP_ACTIVE)
+			break;
+	if(actstrip==NULL)
+		return;
+	
+	/* copy to selected items */
+	for (strip=ob->nlastrips.first; strip; strip=strip->next){
+		if (strip->flag & ACTSTRIP_SELECT) {
+			if(strip!=actstrip) {
+				if (strip->modifiers.first)
+					BLI_freelistN(&strip->modifiers);
+				if (actstrip->modifiers.first)
+					duplicatelist (&strip->modifiers, &actstrip->modifiers);
+			}
+		}
+	}
+	
+	BIF_undo_push("Copy Action Modifiers");
+	allqueue(REDRAWNLA, 0);
+	DAG_scene_flush_update(G.scene, screen_view3d_layers());
+}
+
