@@ -35,6 +35,7 @@
 #include "MEM_guardedalloc.h"
 
 #include "DNA_action_types.h"
+#include "DNA_color_types.h"
 #include "DNA_image_types.h"
 #include "DNA_ipo_types.h"
 #include "DNA_object_types.h"
@@ -45,6 +46,7 @@
 #include "DNA_scene_types.h"
 #include "DNA_userdef_types.h"
 
+#include "BKE_colortools.h"
 #include "BKE_global.h"
 #include "BKE_image.h"
 #include "BKE_library.h"
@@ -54,6 +56,7 @@
 #include "BKE_scene.h"
 #include "BKE_utildefines.h"
 
+#include "BIF_cursors.h"
 #include "BIF_editview.h"
 #include "BIF_gl.h"
 #include "BIF_graphics.h"
@@ -78,11 +81,13 @@
 #include "BDR_editobject.h"
 
 #include "RE_pipeline.h"
+#include "IMB_imbuf_types.h"
 
 #include "blendef.h"
 #include "butspace.h"
 #include "PIL_time.h"
 #include "mydevice.h"
+#include "winlay.h"
 
 
 /* currently called from BIF_preview_changed */
@@ -149,7 +154,9 @@ static void snode_handle_recalc(SpaceNode *snode)
 			snode->nodetree->timecursor= set_timecursor;
 			G.afbreek= 0;
 			snode->nodetree->test_break= blender_test_break;
-
+			
+			BIF_store_spare();
+			
 			ntreeCompositExecTree(snode->nodetree, &G.scene->r, 1);	/* 1 is do_previews */
 			
 			snode->nodetree->timecursor= NULL;
@@ -186,16 +193,17 @@ static void load_node_image(char *str)	/* called from fileselect */
 	bNode *node= nodeGetActive(snode->edittree);
 	Image *ima= NULL;
 	
-	ima= add_image(str);
+	ima= BKE_add_image_file(str);
 	if(ima) {
 		if(node->id)
 			node->id->us--;
 		
 		node->id= &ima->id;
-		ima->id.us++;
+		id_us_plus(node->id);
 
-		free_image_buffers(ima);	/* force read again */
-		ima->ok= 1;
+		BLI_strncpy(node->name, node->id->name+2, 21);
+				   
+		BKE_image_signal(ima, node->storage, IMA_SIGNAL_RELOAD);
 		
 		NodeTagChanged(snode->edittree, node);
 		snode_handle_recalc(snode);
@@ -363,6 +371,8 @@ void node_composit_default(Scene *sce)
 	nodeAddLink(sce->nodetree, in, fromsock, out, tosock);
 	
 	ntreeSolveOrder(sce->nodetree);	/* needed for pointers */
+	
+	ntreeCompositForceHidden(sce->nodetree);
 }
 
 /* Here we set the active tree(s), even called for each redraw now, so keep it fast :) */
@@ -388,6 +398,11 @@ void snode_set_context(SpaceNode *snode)
 	else if(snode->treetype==NTREE_COMPOSIT) {
 		snode->from= NULL;
 		snode->id= &G.scene->id;
+		
+		/* bit clumsy but reliable way to see if we draw first time */
+		if(snode->nodetree==NULL)
+			ntreeCompositForceHidden(G.scene->nodetree);
+		
 		snode->nodetree= G.scene->nodetree;
 	}
 	
@@ -403,6 +418,28 @@ void snode_set_context(SpaceNode *snode)
 		snode->edittree= snode->nodetree;
 }
 
+/* on activate image viewer, check if we show it */
+static void node_active_image(Image *ima)
+{
+	ScrArea *sa;
+	SpaceImage *sima= NULL;
+	
+	/* find an imagewindow showing render result */
+	for(sa=G.curscreen->areabase.first; sa; sa= sa->next) {
+		if(sa->spacetype==SPACE_IMAGE) {
+			sima= sa->spacedata.first;
+			if(sima->image && sima->image->source!=IMA_SRC_VIEWER)
+				break;
+		}
+	}
+	if(sa && sima) {
+		sima->image= ima;
+		scrarea_queue_winredraw(sa);
+		scrarea_queue_headredraw(sa);
+	}
+}
+
+
 static void node_set_active(SpaceNode *snode, bNode *node)
 {
 	
@@ -424,12 +461,12 @@ static void node_set_active(SpaceNode *snode, bNode *node)
 		}
 		else if(snode->treetype==NTREE_COMPOSIT) {
 			/* make active viewer, currently only 1 supported... */
-			if(node->type==CMP_NODE_VIEWER) {
+			if( ELEM(node->type, CMP_NODE_VIEWER, CMP_NODE_SPLITVIEWER)) {
 				bNode *tnode;
 				int was_output= node->flag & NODE_DO_OUTPUT;
 
 				for(tnode= snode->edittree->nodes.first; tnode; tnode= tnode->next)
-					if(tnode->type==CMP_NODE_VIEWER)
+					if( ELEM(tnode->type, CMP_NODE_VIEWER, CMP_NODE_SPLITVIEWER))
 						tnode->flag &= ~NODE_DO_OUTPUT;
 				
 				node->flag |= NODE_DO_OUTPUT;
@@ -446,18 +483,17 @@ static void node_set_active(SpaceNode *snode, bNode *node)
 					snode_handle_recalc(snode);
 				}
 				
-				/* add node doesnt link this yet... */
-				if(node->id==NULL) {
-					node->id= find_id("IM", "Viewer Node");
-					if(node->id==NULL) {
-						Image *ima= alloc_libblock(&G.main->image, ID_IM, "Viewer Node");
-						strcpy(ima->name, "Viewer Node");
-						ima->ok= 1;
-						ima->xrep= ima->yrep= 1;
-						node->id= &ima->id;
-					}
-					else 
-						node->id->us++;
+				/* addnode() doesnt link this yet... */
+				node->id= (ID *)BKE_image_verify_viewer(IMA_TYPE_COMPOSITE, "Viewer Node");
+			}
+			else if(node->type==CMP_NODE_IMAGE) {
+				if(node->id)
+					node_active_image((Image *)node->id);
+			}
+			else if(node->type==CMP_NODE_R_LAYERS) {
+				if(node->id==NULL || node->id==(ID *)G.scene) {
+					G.scene->r.actlay= node->custom1;
+					allqueue(REDRAWBUTSSCENE, 0);
 				}
 			}
 		}
@@ -526,7 +562,6 @@ void node_ungroup(SpaceNode *snode)
 	else {
 		if(nodeGroupUnGroup(snode->edittree, gnode)) {
 			
-			ntreeSolveOrder(snode->edittree);
 			BIF_undo_push("Deselect all nodes");
 			allqueue(REDRAWNODE, 0);
 		}
@@ -697,6 +732,9 @@ void snode_home(ScrArea *sa, SpaceNode *snode)
 		}
 	}
 	snode->v2d.tot= snode->v2d.cur;
+	
+	snode->xof = snode->yof = 0.0;
+	
 	test_view2d(G.v2d, sa->winx, sa->winy);
 	
 }
@@ -727,6 +765,67 @@ void snode_zoom_in(ScrArea *sa)
 	test_view2d(G.v2d, sa->winx, sa->winy);
 }
 
+static void snode_bg_viewmove(SpaceNode *snode)
+{
+	ScrArea *sa;
+	Image *ima;
+	ImBuf *ibuf;
+	Window *win;
+	short mval[2], mvalo[2];
+	short rectx, recty, xmin, xmax, ymin, ymax, pad;
+	int oldcursor;
+	
+	ima= BKE_image_verify_viewer(IMA_TYPE_COMPOSITE, "Viewer Node");
+	ibuf= BKE_image_get_ibuf(ima, NULL);
+	
+	sa = snode->area;
+	
+	if(ibuf) {
+		rectx = ibuf->x;
+		recty = ibuf->y;
+	} else {
+		rectx = recty = 1;
+	}
+	
+	pad = 10;
+	xmin = -(sa->winx/2) - rectx/2 + pad;
+	xmax = sa->winx/2 + rectx/2 - pad;
+	ymin = -(sa->winy/2) - recty/2 + pad;
+	ymax = sa->winy/2 + recty/2 - pad;
+	
+	getmouseco_sc(mvalo);
+	
+	/* store the old cursor to temporarily change it */
+	oldcursor=get_cursor();
+	win=winlay_get_active_window();
+	
+	SetBlenderCursor(BC_NSEW_SCROLLCURSOR);
+	
+	while(get_mbut()&(L_MOUSE|M_MOUSE)) {
+		
+		getmouseco_sc(mval);
+		
+		if(mvalo[0]!=mval[0] || mvalo[1]!=mval[1]) {
+			
+			snode->xof -= (mvalo[0]-mval[0]);
+			snode->yof -= (mvalo[1]-mval[1]);
+			
+			/* prevent dragging image outside of the window and losing it! */
+			CLAMP(snode->xof, xmin, xmax);
+			CLAMP(snode->yof, ymin, ymax);
+			
+			mvalo[0]= mval[0];
+			mvalo[1]= mval[1];
+			
+			scrarea_do_windraw(curarea);
+			screen_swapbuffers();
+		}
+		else BIF_wait_for_statechange();
+	}
+	
+	window_set_cursor(win, oldcursor);
+}
+
 /* checks mouse position, and returns found node/socket */
 /* type is SOCK_IN and/or SOCK_OUT */
 static int find_indicated_socket(SpaceNode *snode, bNode **nodep, bNodeSocket **sockp, int in_out)
@@ -748,7 +847,7 @@ static int find_indicated_socket(SpaceNode *snode, bNode **nodep, bNodeSocket **
 	for(node= snode->edittree->nodes.first; node; node= node->next) {
 		if(in_out & SOCK_IN) {
 			for(sock= node->inputs.first; sock; sock= sock->next) {
-				if(!(sock->flag & SOCK_HIDDEN)) {
+				if(!(sock->flag & (SOCK_HIDDEN|SOCK_UNAVAIL))) {
 					if(BLI_in_rctf(&rect, sock->locx, sock->locy)) {
 						if(node == visible_node(snode, &rect)) {
 							*nodep= node;
@@ -761,7 +860,7 @@ static int find_indicated_socket(SpaceNode *snode, bNode **nodep, bNodeSocket **
 		}
 		if(in_out & SOCK_OUT) {
 			for(sock= node->outputs.first; sock; sock= sock->next) {
-				if(!(sock->flag & SOCK_HIDDEN)) {
+				if(!(sock->flag & (SOCK_HIDDEN|SOCK_UNAVAIL))) {
 					if(BLI_in_rctf(&rect, sock->locx, sock->locy)) {
 						if(node == visible_node(snode, &rect)) {
 							*nodep= node;
@@ -1305,6 +1404,58 @@ void node_border_select(SpaceNode *snode)
 
 /* ****************** Add *********************** */
 
+void snode_autoconnect(SpaceNode *snode, bNode *node_to, int flag)
+{
+	bNodeSocket *sock, *sockfrom[8];
+	bNode *node, *nodefrom[8];
+	int totsock= 0, socktype=0;
+
+	if(node_to->inputs.first==NULL)
+		return;
+	
+	/* no inputs for node allowed (code it) */
+
+	/* connect first 1 socket type now */
+	for(sock= node_to->inputs.first; sock; sock= sock->next)
+		if(socktype<sock->type)
+			socktype= sock->type;
+
+	
+	/* find potential sockets, max 8 should work */
+	for(node= snode->edittree->nodes.first; node; node= node->next) {
+		if((node->flag & flag) && node!=node_to) {
+			for(sock= node->outputs.first; sock; sock= sock->next) {
+				if(!(sock->flag & (SOCK_HIDDEN|SOCK_UNAVAIL))) {
+					sockfrom[totsock]= sock;
+					nodefrom[totsock]= node;
+					totsock++;
+					if(totsock>7)
+						break;
+				}
+			}
+		}
+		if(totsock>7)
+			break;
+	}
+
+	/* now just get matching socket types and create links */
+	for(sock= node_to->inputs.first; sock; sock= sock->next) {
+		int a;
+		
+		for(a=0; a<totsock; a++) {
+			if(sockfrom[a]) {
+				if(sock->type==sockfrom[a]->type && sock->type==socktype) {
+					nodeAddLink(snode->edittree, nodefrom[a], sockfrom[a], node_to, sock);
+					sockfrom[a]= NULL;
+					break;
+				}
+			}
+		}
+	}
+	
+	ntreeSolveOrder(snode->edittree);
+}
+
 /* can be called from menus too, but they should do own undopush and redraws */
 bNode *node_add_node(SpaceNode *snode, int type, float locx, float locy)
 {
@@ -1343,6 +1494,11 @@ bNode *node_add_node(SpaceNode *snode, int type, float locx, float locy)
 		
 		if(node->id)
 			id_us_plus(node->id);
+		
+		if(snode->nodetree->type==NTREE_COMPOSIT)
+			ntreeCompositForceHidden(snode->edittree);
+		
+		NodeTagChanged(snode->edittree, node);
 	}
 	return node;
 }
@@ -1359,6 +1515,7 @@ void node_adduplicate(SpaceNode *snode)
 	transform_nodes(snode->edittree, 'g', "Duplicate");
 }
 
+#if 0
 static void node_insert_convertor(SpaceNode *snode, bNodeLink *link)
 {
 	bNode *newnode= NULL;
@@ -1392,13 +1549,14 @@ static void node_insert_convertor(SpaceNode *snode, bNodeLink *link)
 	}
 }
 
+#endif
 
 /* loop that adds a nodelink, called by function below  */
 /* in_out = starting socket */
 static int node_add_link_drag(SpaceNode *snode, bNode *node, bNodeSocket *sock, int in_out)
 {
 	bNode *tnode;
-	bNodeSocket *tsock;
+	bNodeSocket *tsock= NULL;
 	bNodeLink *link= NULL;
 	short mval[2], mvalo[2], firsttime=1;	/* firsttime reconnects a link broken by caller */
 	
@@ -1471,7 +1629,7 @@ static int node_add_link_drag(SpaceNode *snode, bNode *node, bNodeSocket *sock,
 		
 		/* we might need to remove a link */
 		if(in_out==SOCK_OUT) {
-			if(nodeCountSocketLinks(snode->edittree, link->tosock) > tsock->limit) {
+			if(tsock && nodeCountSocketLinks(snode->edittree, link->tosock) > tsock->limit) {
 				
 				for(tlink= snode->edittree->links.first; tlink; tlink= tlink->next) {
 					if(link!=tlink && tlink->tosock==link->tosock)
@@ -1492,15 +1650,6 @@ static int node_add_link_drag(SpaceNode *snode, bNode *node, bNodeSocket *sock,
 				}					
 			}
 		}
-		
-		/* and last trick: insert a convertor when types dont match */
-		if(snode->treetype==NTREE_SHADER) {
-			if(link->tosock->type!=link->fromsock->type) {
-				node_insert_convertor(snode, link);
-				/* so nice do it twice! well, the sort-order can only handle 1 added link at a time */
-				ntreeSolveOrder(snode->edittree);
-			}
-		}
 	}
 	
 	ntreeSolveOrder(snode->edittree);
@@ -1570,8 +1719,12 @@ void node_delete(SpaceNode *snode)
 	
 	for(node= snode->edittree->nodes.first; node; node= next) {
 		next= node->next;
-		if(node->flag & SELECT)
+		if(node->flag & SELECT) {
+			/* check id user here, nodeFreeNode is called for free dbase too */
+			if(node->id)
+				node->id->us--;
 			nodeFreeNode(snode->edittree, node);
+		}
 	}
 	
 	snode_verify_groups(snode);
@@ -1604,7 +1757,57 @@ void node_hide(SpaceNode *snode)
 	BIF_undo_push("Hide nodes");
 	allqueue(REDRAWNODE, 1);
 }
+
+void node_insert_key(SpaceNode *snode)
+{
+	bNode *node= editnode_get_active(snode->edittree);
+
+	if(node->type==CMP_NODE_TIME) {
+		if(node->custom1<node->custom2) {
+
+			CurveMapping *cumap= node->storage;
+			float fval, curval;
+		
+			curval= (float)(CFRA - node->custom1)/(float)(node->custom2-node->custom1);
+			fval= curvemapping_evaluateF(cumap, 0, curval);
 			
+			if(fbutton(&fval, 0.0f, 1.0f, 10, 10, "Insert Value")) {
+				curvemap_insert(cumap->cm, curval, fval);
+
+				BIF_undo_push("Insert key in Time node");
+				allqueue(REDRAWNODE, 1);
+			}
+		}
+	}
+}
+
+void node_select_linked(SpaceNode *snode, int out)
+{
+	bNodeLink *link;
+	bNode *node;
+	
+	/* NODE_TEST is the free flag */
+	for(node= snode->edittree->nodes.first; node; node= node->next)
+		node->flag &= ~NODE_TEST;
+
+	for(link= snode->edittree->links.first; link; link= link->next) {
+		if(out) {
+			if(link->fromnode->flag & NODE_SELECT)
+				link->tonode->flag |= NODE_TEST;
+		}
+		else {
+			if(link->tonode->flag & NODE_SELECT)
+				link->fromnode->flag |= NODE_TEST;
+		}
+	}
+	
+	for(node= snode->edittree->nodes.first; node; node= node->next)
+		if(node->flag & NODE_TEST)
+			node->flag |= NODE_SELECT;
+	
+	BIF_undo_push("Select Linked nodes");
+	allqueue(REDRAWNODE, 1);
+}
 
 static void node_border_link_delete(SpaceNode *snode)
 {
@@ -1697,15 +1900,52 @@ void node_read_renderlayers(SpaceNode *snode)
 			if(id==NULL) id= (ID *)G.scene;
 			if(id->flag & LIB_DOIT) {
 				RE_ReadRenderResult(G.scene, (Scene *)id);
+				ntreeCompositTagRender((Scene *)id);
 				id->flag &= ~LIB_DOIT;
 			}
 		}
 	}
 	
-	ntreeCompositTagRender(snode->edittree);
 	snode_handle_recalc(snode);
 }
 
+/* gets active viewer user */
+struct ImageUser *ntree_get_active_iuser(bNodeTree *ntree)
+{
+	bNode *node;
+	
+	if(ntree)
+		for(node= ntree->nodes.first; node; node= node->next)
+			if( ELEM(node->type, CMP_NODE_VIEWER, CMP_NODE_SPLITVIEWER)) 
+				if(node->flag & NODE_DO_OUTPUT)
+					return node->storage;
+	return NULL;
+}
+
+void imagepaint_composite_tags(bNodeTree *ntree, Image *image, ImageUser *iuser)
+{
+	bNode *node;
+	
+	if(ntree==NULL)
+		return;
+	
+	/* search for renderresults */
+	if(image->type==IMA_TYPE_R_RESULT) {
+		for(node= ntree->nodes.first; node; node= node->next) {
+			if(node->type==CMP_NODE_R_LAYERS && node->id==NULL) {
+				/* imageuser comes from ImageWin, so indexes are offset 1 */
+				if(node->custom1==iuser->layer-1)
+					NodeTagChanged(ntree, node);
+			}
+		}
+	}
+	else {
+		for(node= ntree->nodes.first; node; node= node->next) {
+			if(node->id== &image->id)
+				NodeTagChanged(ntree, node);
+		}
+	}
+}
 
 /* ********************** */
 
@@ -1834,10 +2074,16 @@ void winqreadnodespace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 			break;
 			
 		case RIGHTMOUSE: 
-			node_mouse_select(snode, event);
+			if(!node_mouse_select(snode, event)) 
+				toolbox_n();
 
 			break;
 		case MIDDLEMOUSE:
+			if (G.qual==LR_SHIFTKEY) {
+				snode_bg_viewmove(snode);
+			} else {
+				view2dmove(event);
+			}
 		case WHEELUPMOUSE:
 		case WHEELDOWNMOUSE:
 			view2dmove(event);	/* in drawipo.c */
@@ -1901,6 +2147,9 @@ void winqreadnodespace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 				else node_adduplicate(snode);
 			}
 			break;
+		case EKEY:
+			snode_handle_recalc(snode);
+			break;
 		case GKEY:
 			if(fromlib) fromlib= -1;
 			else {
@@ -1922,6 +2171,12 @@ void winqreadnodespace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 		case HKEY:
 			node_hide(snode);
 			break;
+		case IKEY:
+			node_insert_key(snode);
+			break;
+		case LKEY:
+			node_select_linked(snode, G.qual==LR_SHIFTKEY);
+			break;
 		case RKEY:
 			if(okee("Read saved Render Layers"))
 				node_read_renderlayers(snode);
