@@ -1,5 +1,5 @@
 /**
- * $Id: editscreen.c,v 1.135 2006/07/06 12:50:48 ton Exp $
+ * $Id: editscreen.c,v 1.150 2007/02/09 09:38:30 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -44,6 +44,8 @@
 #include <stdio.h>
 #include <math.h>
 
+#include "GHOST_Types.h"
+
 #include "MEM_guardedalloc.h"
 
 #include "PIL_time.h"
@@ -74,6 +76,10 @@
 #include "BKE_blender.h"
 #include "BKE_screen.h"
 
+#ifdef WITH_VERSE
+#include "BKE_verse.h"
+#endif
+
 #include "BIF_cursors.h"
 #include "BIF_drawscene.h"
 #include "BIF_editsound.h"
@@ -86,6 +92,7 @@
 #include "BIF_mywindow.h"
 #include "BIF_previewrender.h"
 #include "BIF_renderwin.h"
+#include "BIF_retopo.h"
 #include "BIF_screen.h"
 #include "BIF_space.h"
 #include "BIF_toets.h"
@@ -167,9 +174,10 @@ static int choose_cursor(ScrArea *sa)
 {
 	if (sa->spacetype==SPACE_VIEW3D) {
 		if(G.obedit) return CURSOR_EDIT;
-		else if(G.f & G_VERTEXPAINT) return CURSOR_VPAINT;
-		else if(G.f & G_WEIGHTPAINT) return CURSOR_VPAINT;
+		else if(G.f & (G_VERTEXPAINT|G_WEIGHTPAINT|G_TEXTUREPAINT))
+				return CURSOR_VPAINT;
 		else if(G.f & G_FACESELECT) return CURSOR_FACESEL;
+		else if(G.f & G_SCULPTMODE) return CURSOR_EDIT;
 		else return CURSOR_STD;
 	}
 	else if (sa->spacetype==SPACE_TEXT) {
@@ -473,8 +481,7 @@ static void headmenu(ScrArea *sa)
 static void addqueue_ext(short win, unsigned short event, short val, char ascii)
 {
 	if (win<4 || !areawinar[win]) {
-		if(win==0) // other win ids are for mainwin & renderwin
-			printf("bad call to addqueue: %d (%d, %d)\n", win, event, val);
+		/* other win ids are for mainwin & renderwin */
 	} 
 	else {
 		BWinEvent evt;
@@ -712,7 +719,7 @@ void splash(void *data, int datasize, char *string)
 			BMF_DrawString(font, string);
 		}
 
-		glFlush();
+		bglFlush();
 		glDrawBuffer(GL_BACK);
 		
 		IMB_freeImBuf(bbuf);
@@ -1116,6 +1123,10 @@ int do_screenhandlers(bScreen *sc)
 					done= 1;
 					break;
 				case SCREEN_HANDLER_VERSE:
+#ifdef WITH_VERSE
+					b_verse_update();
+#endif
+
 					done= 1;
 					break;
 			}
@@ -1353,7 +1364,7 @@ void screenmain(void)
 		else if (event==QKEY) {
 			if((G.obedit && G.obedit->type==OB_FONT && g_activearea->spacetype==SPACE_VIEW3D)||g_activearea->spacetype==SPACE_TEXT||g_activearea->spacetype==SPACE_SCRIPT);
 			else {
-				if(val) {
+				if(val && (G.qual & LR_CTRLKEY)) {
 					if(okee("Quit Blender")) exit_usiblender();
 				}
 				towin= 0;
@@ -1825,12 +1836,14 @@ static void del_area(ScrArea *sa)
 	uiFreeBlocks(&sa->uiblocks);
 	uiFreePanels(&sa->panels);
 	
-	if(sa==curarea) curarea= 0;
-	if(sa==g_activearea) g_activearea= 0;
+	BPY_free_scriptlink(&sa->scriptlink);
+	
+	if(sa==curarea) curarea= NULL;
+	if(sa==g_activearea) g_activearea= NULL;
 }
 
-/* sa2 to sa1 */
-static void copy_areadata(ScrArea *sa1, ScrArea *sa2)
+/* sa2 to sa1, we swap spaces for fullscreen to keep all allocated data */
+static void copy_areadata(ScrArea *sa1, ScrArea *sa2, int swap_space)
 {
 	Panel *pa1, *pa2, *patab;
 	ScriptLink *slink1 = &sa1->scriptlink, *slink2 = &sa2->scriptlink;
@@ -1839,9 +1852,17 @@ static void copy_areadata(ScrArea *sa1, ScrArea *sa2)
 	sa1->spacetype= sa2->spacetype;
 	Mat4CpyMat4(sa1->winmat, sa2->winmat);
 
-	freespacelist(sa1);
-	duplicatespacelist(sa1, &sa1->spacedata, &sa2->spacedata);
-
+	if(swap_space) {
+		SWAP(ListBase, sa1->spacedata, sa2->spacedata);
+		/* exception: ensure preview is reset */
+		if(sa1->spacetype==SPACE_VIEW3D)
+			BIF_view3d_previewrender_free(sa1->spacedata.first);
+	}
+	else {
+		freespacelist(sa1);
+		duplicatespacelist(sa1, &sa1->spacedata, &sa2->spacedata);
+	}
+	
 	BLI_freelistN(&sa1->panels);
 	duplicatelist(&sa1->panels, &sa2->panels);
 
@@ -1941,12 +1962,15 @@ static void testareas(void)
 			}
 
 			if (!rcti_eq(&oldwr, &sa->winrct)) {
+				SpaceLink *sl= sa->spacedata.first;
+				
 				mywinposition(sa->win, sa->winrct.xmin, sa->winrct.xmax, sa->winrct.ymin, sa->winrct.ymax);
 				addqueue(sa->win, CHANGED, 1);
 				
 				/* exception handling... probably we need generic event */
-				if(sa->spacetype==SPACE_VIEW3D)
-					BIF_view3d_previewrender_free(sa->spacedata.first);
+				for(; sl; sl= sl->next)
+					if(sl->spacetype==SPACE_VIEW3D)
+						BIF_view3d_previewrender_free((View3D *)sl);
 			}
 		}
 	}
@@ -2077,6 +2101,11 @@ short get_mbut(void)
 	winlay_process_events(0);
 	return window_get_mbut(mainwin);
 }
+const GHOST_TabletData* get_tablet_data()
+{
+	winlay_process_events(0);
+	return window_get_tablet_data(mainwin);
+}
 
 void add_to_mainqueue(Window *win, void *user_data, short evt, short val, char ascii)
 {
@@ -2155,7 +2184,9 @@ static bScreen *addscreen(char *name)		/* use setprefsize() if you want somethin
 		glClear(GL_COLOR_BUFFER_BIT);
 		window_swap_buffers(mainwin);
 		
-		warp_pointer(sc->sizex/2,  sc->sizey/2);
+		/* this is unneeded and with large monitors can be a
+		 * pain so commenting out */
+		/* warp_pointer(sc->sizex/2,  sc->sizey/2); */
 		
 		mainqenter(REDRAW, 1);
 	}
@@ -2309,7 +2340,7 @@ void area_fullscreen(void)	/* with curarea */
 				headertype = curarea->headertype;
 			}
 
-			copy_areadata(old, curarea);
+			copy_areadata(old, curarea, 1);	/*  1 = swap spacelist */
 			old->headertype = headertype;
 
 			old->full= 0;
@@ -2341,7 +2372,7 @@ void area_fullscreen(void)	/* with curarea */
 		G.curscreen= oldscreen;	/* needed because of setscreen */
 		
 		/* copy area */
-		copy_areadata(newa, curarea);
+		copy_areadata(newa, curarea, 1);	/* 1 = swap spacelist */
 		
 		curarea->full= oldscreen;
 		newa->full= oldscreen;
@@ -2350,6 +2381,9 @@ void area_fullscreen(void)	/* with curarea */
 		setscreen(sc);
 		wich_cursor(newa);
 	}
+	
+	if(curarea->full)
+		retopo_force_update();
 
 	/* there's also events in queue for this, but we call fullscreen for render output
 	now, and that doesn't go back to queue. Bad code, but doesn't hurt... (ton) */
@@ -2359,6 +2393,9 @@ void area_fullscreen(void)	/* with curarea */
 	}
 	/* bad code #2: setscreen() ends with first area active. fullscreen render assumes this too */
 	curarea= sc->areabase.first;
+	
+	if(!curarea->full)
+		retopo_force_update();
 }
 
 static void area_autoplayscreen(void)
@@ -2393,7 +2430,7 @@ static void area_autoplayscreen(void)
 			G.curscreen= oldscreen;	/* because of setscreen */
 			
 			/* copy area settings */
-			copy_areadata(newa, curarea);
+			copy_areadata(newa, curarea, 1);	/* swap spacedata */
 			newa->headertype= 0;
 			
 			curarea->full= oldscreen;
@@ -2447,7 +2484,9 @@ static void copy_screen(bScreen *to, bScreen *from)
 		sa->spacedata.first= sa->spacedata.last= NULL;
 		sa->uiblocks.first= sa->uiblocks.last= NULL;
 		sa->panels.first= sa->panels.last= NULL;
-		copy_areadata(sa, saf);
+		sa->scriptlink.totscript= 0;
+		
+		copy_areadata(sa, saf, 0);
 		
 		sa= sa->next;
 		saf= saf->next;
@@ -2750,7 +2789,7 @@ static void joinarea_interactive(ScrArea *area, ScrEdge *onedge)
 		scrarea_draw_shape_dark(scr,'d');
 	else if(scr==down)
 		scrarea_draw_shape_dark(scr,'u');
-	glFlush();
+	bglFlush();
 
 	/* "never ending loop" of interactive selection */
 	while(!ok) {
@@ -2793,7 +2832,7 @@ static void joinarea_interactive(ScrArea *area, ScrEdge *onedge)
 		/* cancel joining of joining */
 		if(val && (event==ESCKEY || event==RIGHTMOUSE)) ok= -1;
 
-		glFlush();
+		bglFlush();
 	}
 
 	glReadBuffer(GL_BACK);
@@ -2916,7 +2955,7 @@ static void splitarea(ScrArea *sa, char dir, float fac)
 		
 		/* new areas: top */
 		newa= screen_addarea(sc, sv1, sa->v2, sa->v3, sv2, sa->headertype, sa->spacetype);
-		copy_areadata(newa, sa);
+		copy_areadata(newa, sa, 0);
 
 		/* area below */
 		sa->v2= sv1;
@@ -2937,7 +2976,7 @@ static void splitarea(ScrArea *sa, char dir, float fac)
 		
 		/* new areas: left */
 		newa= screen_addarea(sc, sa->v1, sa->v2, sv2, sv1, sa->headertype, sa->spacetype);
-		copy_areadata(newa, sa);
+		copy_areadata(newa, sa, 0);
 
 		/* area right */
 		sa->v1= sv1;		
@@ -2984,7 +3023,11 @@ static void splitarea_interactive(ScrArea *area, ScrEdge *onedge)
 	
 	if(sa->win==0) return;
 	if(sa->full) return;
-
+	if(myswinopen_allowed()==0) {
+		error("Max amount of subwindows reached");
+		return;
+	}
+	
 	dir= scredge_is_horizontal(onedge)?'v':'h';
 	
 	mywinset(G.curscreen->mainwin);
@@ -3074,12 +3117,12 @@ static void splitarea_interactive(ScrArea *area, ScrEdge *onedge)
 		if(val && (event==ESCKEY || event==RIGHTMOUSE)) {
 			ok= -1;
 		}
-		glFlush();
+		bglFlush();
 	}
 
 	if (!first) {
 		scrarea_draw_splitpoint(sa, dir, fac);
-		glFlush();
+		bglFlush();
 	}
 	glReadBuffer(GL_BACK);
 	glDrawBuffer(GL_BACK);
@@ -3390,7 +3433,7 @@ static void moveareas(ScrEdge *edge)
 			delta= (dir=='h')?(mval[1]-mvalo[1]):(mval[0]-mvalo[0]);
 			delta= CLAMPIS(delta, -smaller, bigger);
 			draw_front_xor_dirdist_line(dir, edge_position+delta, edge_start, edge_end);
-			glFlush();
+			bglFlush();
 		} 
 		else if (event==LEFTMOUSE) {
 			doit= 1;
@@ -3405,7 +3448,7 @@ static void moveareas(ScrEdge *edge)
 		
 	}
 	draw_front_xor_dirdist_line(dir, edge_position+delta, edge_start, edge_end);
-	glFlush();
+	bglFlush();
 	glReadBuffer(GL_BACK);
 	glDrawBuffer(GL_BACK);
 
@@ -3548,22 +3591,22 @@ void draw_area_emboss(ScrArea *sa)
 	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
 
 	/* right  */
-	glColor4ub(0,0,0, 80);
+	glColor4ub(0,0,0, 50);
 	sdrawline(sa->winx-1, 0, sa->winx-1, sa->winy-1);
 	
 	/* bottom  */
 	if(sa->headertype!=HEADERDOWN) {
-		glColor4ub(0,0,0, 128);
+		glColor4ub(0,0,0, 80);
 		sdrawline(0, 0, sa->winx-1, 0);
 	}
 	
 	/* top  */
 	if(sa->headertype!=HEADERTOP) {
-		glColor4ub(255,255,255, 96);
+		glColor4ub(255,255,255, 60);
 		sdrawline(0, sa->winy-1, sa->winx-1, sa->winy-1);
 	}
 	/* left  */
-	glColor4ub(255,255,255, 80);
+	glColor4ub(255,255,255, 50);
 	sdrawline(0, 0, 0, sa->winy);
 
 	glDisable( GL_BLEND );
@@ -3611,7 +3654,7 @@ void drawscredge_area(ScrArea *sa)
 unsigned int screen_view3d_layers(void)
 {
 	ScrArea *sa;
-	int layer= 0;
+	int layer= G.scene->lay;	/* as minimum this */
 	
 	for(sa= G.curscreen->areabase.first; sa; sa= sa->next) {
 		if(sa->spacetype==SPACE_VIEW3D)
@@ -3638,47 +3681,6 @@ void initscreen(void)
 	default_twosplit();
 }
 
-/***/
-
-void screen_draw_info_text(bScreen *sc, char *text) {
-	Window *oldactwin= winlay_get_active_window();
-	ScrArea *sa;
-	
-		/*
-		 * Because this is called from within rendering
-		 * internals it is possible our window is not
-		 * active.
-		 */
-	window_make_active(mainwin);
-	
-	for (sa= sc->areabase.first; sa; sa= sa->next) {
-		if (sa->spacetype==SPACE_INFO) {
-			int x= sa->headbutlen - 28;
-			int y= 6;
-			
-			areawinset(sa->headwin);
-			glDrawBuffer(GL_FRONT);
-
-			cpack(0xA08060);
-			glRecti(x-11,  y-6,  x+55,  y+13);
-			
-			cpack(0x909090);
-			glRecti(x+55,  y-6,  x+1280,  y+14);
-			
-			cpack(0x0);
-			glRasterPos2i(x,  y);
-			BMF_DrawString(G.fonts, text);
-			
-			glFlush();
-			glDrawBuffer(GL_BACK);
-			
-			sa->head_swap= WIN_FRONT_OK;
-		}
-	}
-	
-	if (oldactwin && oldactwin!=mainwin) window_make_active(oldactwin);
-}
-
 static int curcursor;
 
 int get_cursor(void) {
