@@ -1,5 +1,5 @@
 /**
- * $Id: editsound.c,v 1.27 2006/06/14 12:42:23 ton Exp $
+ * $Id: editsound.c,v 1.31 2006/11/30 05:29:05 aligorith Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -55,6 +55,7 @@
 
 #include "DNA_object_types.h"
 #include "DNA_screen_types.h"
+#include "DNA_space_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_sound_types.h"
 #include "DNA_packedFile_types.h"
@@ -72,9 +73,13 @@
 #include "BIF_interface.h"
 #include "BIF_editsound.h"
 #include "BIF_mywindow.h"
+#include "BIF_toolbox.h"
 
 #include "BSE_drawipo.h"
 #include "BSE_headerbuttons.h"
+#include "BSE_time.h"
+
+#include "BDR_editobject.h"
 
 #include "blendef.h"
 
@@ -162,14 +167,28 @@ void winqreadsoundspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 			view2dmove(event);	/* in drawipo.c */
 			break;
 		case RIGHTMOUSE:
-			nr= pupmenu("Time value%t|Frames %x1|Seconds%x2");
-			if (nr>0) {
-				if(nr==1) ssound->flag |= SND_DRAWFRAMES;
-				else ssound->flag &= ~SND_DRAWFRAMES;
-				doredraw= 1;
-			}
+			{
+				TimeMarker *marker;
+				
+				getmouseco_areawin(mval);
+				areamouseco_to_ipoco(G.v2d, mval, &dx, &dy);
 
+				marker = find_nearest_marker(0);
+				if (marker) {
+					if ((G.qual & LR_SHIFTKEY)==0)
+						deselect_markers(0, 0);
+						
+					if (marker->flag & SELECT)
+						marker->flag &= ~SELECT;
+					else
+						marker->flag |= SELECT;
+				}
+				
+				force_draw(0);
+				std_rmouse_transform(transform_markers);
+			}
 			break;
+			
 		case PADPLUSKEY:
 			dx= (float)(0.1154*(G.v2d->cur.xmax-G.v2d->cur.xmin));
 			G.v2d->cur.xmin+= dx;
@@ -189,6 +208,79 @@ void winqreadsoundspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 		case HOMEKEY:
 			do_sound_buttons(B_SOUNDHOME);
 			break;
+			
+		case PAGEUPKEY: /* cfra to next marker */
+			nextprev_marker(1);
+			break;
+		case PAGEDOWNKEY: /* cfra to prev marker */
+			nextprev_marker(-1);
+			break;
+		
+		case AKEY: /* select/deselect all  */
+			deselect_markers(1, 0);
+			
+			allqueue(REDRAWTIME, 0);
+			allqueue(REDRAWIPO, 0);
+			allqueue(REDRAWACTION, 0);
+			allqueue(REDRAWNLA, 0);
+			allqueue(REDRAWSOUND, 0);
+			break;
+			
+		case BKEY: /* borderselect markers */
+			borderselect_markers();
+			break;
+		
+		case DKEY: /* duplicate selected marker(s) */
+			if (G.qual & LR_SHIFTKEY) {
+				duplicate_marker();
+				
+				allqueue(REDRAWTIME, 0);
+				allqueue(REDRAWIPO, 0);
+				allqueue(REDRAWACTION, 0);
+				allqueue(REDRAWNLA, 0);
+				allqueue(REDRAWSOUND, 0);
+			}
+			break;
+			
+		case GKEY:
+			transform_markers('g', 0);
+			break;
+			
+		case MKEY: /* add marker or rename first selected */
+			if (G.qual & LR_CTRLKEY)
+				rename_marker();
+			else
+				add_marker(CFRA);
+			
+			allqueue(REDRAWTIME, 0);
+			allqueue(REDRAWIPO, 0);
+			allqueue(REDRAWACTION, 0);
+			allqueue(REDRAWNLA, 0);
+			allqueue(REDRAWSOUND, 0);
+			break;		
+		
+		case TKEY: /* toggle time display */
+			nr= pupmenu("Time value%t|Frames %x1|Seconds%x2");
+			if (nr>0) {
+				if(nr==1) ssound->flag |= SND_DRAWFRAMES;
+				else ssound->flag &= ~SND_DRAWFRAMES;
+				doredraw= 1;
+			}
+
+			break;
+			
+		case DELKEY: /* delete selected markers */
+		case XKEY:
+			if (okee("Erase selected")) {
+				remove_marker();
+				
+				allqueue(REDRAWTIME, 0);
+				allqueue(REDRAWIPO, 0);
+				allqueue(REDRAWACTION, 0);
+				allqueue(REDRAWNLA, 0);
+				allqueue(REDRAWSOUND, 0);
+			}
+			break;
 		}
 	}
 
@@ -200,7 +292,7 @@ void winqreadsoundspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 
 void sound_initialize_sounds(void)
 {
-	bSound* sound;
+	bSound *sound;
 
 	if(ghSoundScene) {
 
@@ -211,8 +303,7 @@ void sound_initialize_sounds(void)
 	
 	/* initialize sample blocks (doesnt call audio system, needs to be done once after load */
 	sound = G.main->sound.first;
-	while (sound)
-	{
+	while (sound) {
 		sound_sample_is_null(sound);
 		sound = (bSound *) sound->id.next;
 	}
@@ -220,9 +311,9 @@ void sound_initialize_sounds(void)
 
 
 
-bSound* sound_make_copy(bSound* originalsound)
+bSound *sound_make_copy(bSound *originalsound)
 {
-	bSound* sound = NULL;
+	bSound *sound = NULL;
 	char name[160];
 	int len;
 	
@@ -272,7 +363,7 @@ bSound* sound_make_copy(bSound* originalsound)
 
 
 
-void sound_initialize_sample(bSound* sound)
+void sound_initialize_sample(bSound *sound)
 {
 	if(ghSoundScene==NULL) sound_init_audio();
 
@@ -281,7 +372,7 @@ void sound_initialize_sample(bSound* sound)
 }
 
 
-void sound_read_wav_data(bSound* sound, PackedFile* pf)
+void sound_read_wav_data(bSound *sound, PackedFile *pf)
 {
 	int i, temp;
 	short shortbuf, *temps;
@@ -298,36 +389,31 @@ void sound_read_wav_data(bSound* sound, PackedFile* pf)
 	rewindPackedFile(pf);
 
 	/* check to see if it is a file in "RIFF WAVE fmt" format */
-	if (readPackedFile(pf, buffer, 16) != 16)
-	{
+	if (readPackedFile(pf, buffer, 16) != 16) {
 		if (G.f & G_DEBUG) printf("File too short\n");
 		return;
 	}
 
-	if(!(memcmp(buffer, "RIFF", 4) && memcmp(&(buffer[8]), "WAVEfmt ", 8)))
-	{
+	if(!(memcmp(buffer, "RIFF", 4) && memcmp(&(buffer[8]), "WAVEfmt ", 8))) {
 		readPackedFile(pf, &i, 4);// start of data
 		if(G.order==B_ENDIAN)
 			SWITCH_INT(i);
 
 		/* read the sampleformat */
 		readPackedFile(pf, &shortbuf, 2);
-		if(G.order==B_ENDIAN)
-		{
+		if(G.order==B_ENDIAN) {
 			SWITCH_SHORT(shortbuf);
 		}
 
 		/* read the number of channels */
 		readPackedFile(pf, &shortbuf, 2);
 
-		if(G.order==B_ENDIAN)
-		{
+		if(G.order==B_ENDIAN) {
 			SWITCH_SHORT(shortbuf);
 		}
 
 		/* check the number of channels */
-		if(shortbuf != 1 && shortbuf != 2)
-		{
+		if(shortbuf != 1 && shortbuf != 2) {
 			if (G.f & G_DEBUG) printf("Unsupported number of channels\n");
 			return;
 		}
@@ -355,8 +441,7 @@ void sound_read_wav_data(bSound* sound, PackedFile* pf)
 
 		readPackedFile(pf, &shortbuf, 2);
 		readPackedFile(pf, &shortbuf, 2);
-		if(G.order==B_ENDIAN)
-		{
+		if(G.order==B_ENDIAN) {
 			SWITCH_SHORT(shortbuf);
 		}
 		bits = shortbuf;
@@ -364,8 +449,7 @@ void sound_read_wav_data(bSound* sound, PackedFile* pf)
 		seekPackedFile(pf, i-16, SEEK_CUR);
 		readPackedFile(pf, buffer, 4);
 		/* check if we have a 'data' chunk */
-		while(memcmp(buffer, "data", 4)!=0)
-		{
+		while(memcmp(buffer, "data", 4)!=0) {
 			if (readPackedFile(pf, &i, 4) != 4)
 				break;
 			
@@ -378,13 +462,11 @@ void sound_read_wav_data(bSound* sound, PackedFile* pf)
 		}
 		
 		/* guess not */
-		if (memcmp(buffer, "data", 4) !=0)
-		{
+		if (memcmp(buffer, "data", 4) !=0) {
 			if (G.f & G_DEBUG) printf("No data found\n");
 		}
 		/* or maybe we do! */
-		else
-		{
+		else {
 			readPackedFile(pf, &longbuf, 4); 
 			if(G.order==B_ENDIAN) SWITCH_INT(longbuf);
 			
@@ -398,21 +480,17 @@ void sound_read_wav_data(bSound* sound, PackedFile* pf)
 			len = longbuf /*/ 4.0*/; /* for some strange reason the sample length is off by a factor of 4... */
 			/* intrr's comment: Funny eh, how one 16-bit stereo sample is 4 bytes? :-) */
 			
-			if(data)
-			{
+			if(data) {
 				readPackedFile(pf, data, len);
 				/* data is only used to draw! */
-				if (bits == 8)
-				{
+				if (bits == 8) {
 					temps = (short *) data;
 					tempc = (char *) data;
 					for (i = len - 1; i >= 0; i--)
 						temps[i] = tempc[i] << 8;
 				}
-				else
-				{
-					if(G.order==B_ENDIAN)
-					{
+				else {
+					if(G.order==B_ENDIAN) {
 						swab(data, data, len);
 					}
 				}
@@ -427,8 +505,7 @@ void sound_read_wav_data(bSound* sound, PackedFile* pf)
 			}
 		}
 	}
-	else
-	{
+	else {
 		sound->sample->type = SAMPLE_INVALID;
 		if (G.f & G_DEBUG) printf("Unsupported sound format: %s\n", sound->name);
 	}
@@ -437,7 +514,7 @@ void sound_read_wav_data(bSound* sound, PackedFile* pf)
 
 
 /* ugly, but it works (for now) */
-static int sound_get_filetype_from_header(bSound* sound, PackedFile* pf)
+static int sound_get_filetype_from_header(bSound *sound, PackedFile *pf)
 {
 	int filetype = SAMPLE_INVALID;
 	int i;
@@ -446,22 +523,19 @@ static int sound_get_filetype_from_header(bSound* sound, PackedFile* pf)
 	
 	rewindPackedFile(pf);
 	
-	if (readPackedFile(pf, buffer, 16) != 16)
-	{
+	if (readPackedFile(pf, buffer, 16) != 16) {
 		if (G.f & G_DEBUG) printf("File too short\n");
 		return filetype;
 	}
 	
-	if(!(memcmp(buffer, "RIFF", 4) && memcmp(&(buffer[8]), "WAVEfmt ", 8)))
-	{
+	if(!(memcmp(buffer, "RIFF", 4) && memcmp(&(buffer[8]), "WAVEfmt ", 8))) {
 		readPackedFile(pf, &i, 4);
 		if(G.order==B_ENDIAN)
 			SWITCH_INT(i);
 		
 		/* read the sampleformat */
 		readPackedFile(pf, &shortbuf, 2);
-		if(G.order==B_ENDIAN)
-		{
+		if(G.order==B_ENDIAN) {
 			char s_i, *p_i;
 			p_i= (char *)&(shortbuf);
 			s_i= p_i[0];
@@ -469,8 +543,7 @@ static int sound_get_filetype_from_header(bSound* sound, PackedFile* pf)
 			p_i[1]= s_i;
 		}
 		
-		if (shortbuf == SND_WAVE_FORMAT_PCM)
-		{
+		if (shortbuf == SND_WAVE_FORMAT_PCM) {
 			filetype = SAMPLE_WAV;
 		}
 		else
@@ -499,17 +572,14 @@ static int sound_get_filetype_from_header(bSound* sound, PackedFile* pf)
 #ifdef USE_FMOD
 		}
 	}
-	else if (!memcmp(buffer, "OggS", 4))
-	{
+	else if (!memcmp(buffer, "OggS", 4)) {
 		filetype = SAMPLE_OGG_VORBIS;
 	}
-	else if ((!memcmp(buffer, "ID3", 3)) || (!memcmp(buffer, "", 2)))
-	{
+	else if ((!memcmp(buffer, "ID3", 3)) || (!memcmp(buffer, "", 2))) {
 		filetype = SAMPLE_MP3;
 	}
 #endif
-	else
-	{
+	else {
 		filetype = SAMPLE_INVALID;
 		if (G.f & G_DEBUG) printf("Unsupported sound format: %s\n", sound->name);
 	}
@@ -519,7 +589,7 @@ static int sound_get_filetype_from_header(bSound* sound, PackedFile* pf)
 
 
 
-static int check_filetype(bSound* sound, PackedFile* pf)
+static int check_filetype(bSound *sound, PackedFile *pf)
 {
 //	char* pdest;
 	sound->sample->type = SAMPLE_INVALID;
@@ -549,8 +619,7 @@ static int check_filetype(bSound* sound, PackedFile* pf)
 	sound->sample->type = sound_get_filetype_from_header(sound, pf);
 
 	/* get some info from the sample */
-	switch (sound->sample->type)
-	{
+	switch (sound->sample->type) {
 	case SAMPLE_WAV:
 		{
 			sound_read_wav_data(sound, pf);
@@ -575,24 +644,22 @@ static int check_filetype(bSound* sound, PackedFile* pf)
 
 
 
-int sound_load_sample(bSound* sound)
+int sound_load_sample(bSound *sound)
 {
 	int result = FALSE;
-	PackedFile* pf;
+	PackedFile *pf;
 	int freePF = FALSE;
 	int buffer = -1;
 
 	if(ghSoundScene==NULL) sound_init_audio();
 
 	/* check the sample (valid?) */
-	if (sound->sample->type == SAMPLE_UNKNOWN || sound->snd_sound == NULL)
-	{
+	if (sound->sample->type == SAMPLE_UNKNOWN || sound->snd_sound == NULL) {
 		/* find... */
 		pf = sound_find_packedfile(sound);
 
 		/* ...or create a (temp)packedfile */
-		if (pf == NULL)
-		{
+		if (pf == NULL) {
 			pf = newPackedFile(sound->name);
 			
 			/* if autopack is off, free the pf afterwards */
@@ -601,14 +668,12 @@ int sound_load_sample(bSound* sound)
 		}
 		
 		/* if we have a valid pf... */
-		if (pf)
-		{
+		if (pf) {
 			/* check the content of the pf */
 			check_filetype(sound, pf);
 
 			/* check if the sampletype is supported */
-			if (sound->sample->type != SAMPLE_INVALID && sound->sample->type != SAMPLE_UNKNOWN)
-			{
+			if (sound->sample->type != SAMPLE_INVALID && sound->sample->type != SAMPLE_UNKNOWN) {
 				/* register the sample at the audiodevice */
 				buffer = SND_AddSample(ghSoundScene, sound->sample->name, pf->data, pf->size);
 
@@ -627,33 +692,28 @@ int sound_load_sample(bSound* sound)
 					result = TRUE;
 			}
 			/* if not, free the pf */
-			else
-			{
+			else {
 				freePF = TRUE;
 			}
 			
 			/* if you want it freed, make it so */
-			if (freePF)
-			{
+			if (freePF) {
 				freePackedFile(pf);
 				pf = NULL;
 			} 
 			/* or else connect the pf to the sound and sample */
-//			else
-//			{
+//			else {
 				sound->newpackedfile = pf;
 				sound->sample->packedfile = pf;
 //			}
 		}
-		else 
-		{
+		else  {
 			if (G.f & G_DEBUG) printf("%s: File not found!\n", sound->name);
 			sound->sample->type = SAMPLE_INVALID;
 		}
 	}
 	/* if the sample ain't invalid, we're ready to go! */
-	else if (sound->sample->type != SAMPLE_INVALID)
-	{
+	else if (sound->sample->type != SAMPLE_INVALID) {
 		result = TRUE;
 	}
 
@@ -662,7 +722,7 @@ int sound_load_sample(bSound* sound)
 
 
 
-bSound* sound_new_sound(char* name)
+bSound *sound_new_sound(char *name)
 {
 	bSound *sound = NULL;
 	int len, file;
@@ -678,8 +738,7 @@ bSound* sound_new_sound(char* name)
 	/* check if the sample on disk can be opened */
 	file = open(str, O_BINARY|O_RDONLY);
 	
-	if (file != -1)
-	{
+	if (file != -1) {
 		close(file);
 
 
@@ -698,8 +757,7 @@ bSound* sound_new_sound(char* name)
 		/* load the sample & check if this blender supports the sound format */
 //		sound_load_sample(sound);
 
-		if (sound->sample->type == SAMPLE_INVALID)
-		{
+		if (sound->sample->type == SAMPLE_INVALID) {
 			free_libblock(&G.main->sound, sound);
 			sound = NULL;
 		} 
@@ -724,13 +782,8 @@ int sound_set_sample(bSound *sound, bSample *sample)
 	
 	if(ghSoundScene==NULL) sound_init_audio();
 	
-	/* decrease the usernumber for this sample */
-	if (sound->sample)
-		sound->sample->id.us--;
-
 	/* delete the soundobject */
-	if (sound->snd_sound)
-	{
+	if (sound->snd_sound) {
 		SND_RemoveSound(ghSoundScene, sound->snd_sound);
 		sound->snd_sound = NULL;
 	}
@@ -739,22 +792,17 @@ int sound_set_sample(bSound *sound, bSample *sample)
 	sound->sample = sample;
 	sound->newpackedfile = NULL;
 	
-	/* increase the usercount */
-	if (sound->sample)
-	{
-		sound->sample->id.us++;
+	if (sound->sample) {
 
 		/* and set the right pf */
 		sound->newpackedfile = sample->packedfile;
 
 		/* if the sampletype is unknown initialize it */
-		if (sound->sample->type == SAMPLE_UNKNOWN)
-		{
+		if (sound->sample->type == SAMPLE_UNKNOWN) {
 			sound_initialize_sample(sound);
 			
 			/* load the sample & check if this blender supports the sound format */
-			if (!sound_load_sample(sound))
-			{
+			if (!sound_load_sample(sound)) {
 				result = FALSE;
 			}
 		}
@@ -765,14 +813,13 @@ int sound_set_sample(bSound *sound, bSample *sample)
 
 
 
-bSample *sound_new_sample(bSound * sound)
+bSample *sound_new_sample(bSound *sound)
 {
 	bSample *sample = NULL;
 	int len;
 	char *name;
 	
-	if (sound != NULL)
-	{
+	if (sound != NULL) {
 		name = sound->name;	
 		len = strlen(name);
 		/* do some name magic */
@@ -780,7 +827,10 @@ bSample *sound_new_sample(bSound * sound)
 			len--;
 		
 		/* allocate the memory for the sample */
-		sample = alloc_libblock(samples, ID_SAMPLE, name + len);
+		sample = MEM_callocN(sizeof(bSample), "sample");
+		BLI_strncpy(sample->id.name+2, name+len, 20);
+		BLI_addtail(samples, sample);	/* samples is ugly global */
+		
 		sample->data = &sample->fakedata[0];
 		sample->type = SAMPLE_UNKNOWN;
 		
@@ -807,9 +857,9 @@ bSample *sound_new_sample(bSound * sound)
 
 
 /* find a sample that might already be loaded */
-bSample* sound_find_sample(bSound* sound)
+bSample *sound_find_sample(bSound *sound)
 {
-	bSample* sample;
+	bSample *sample;
 	char name[FILE_MAXDIR + FILE_MAXFILE];
 	char samplename[FILE_MAXDIR + FILE_MAXFILE];
 	
@@ -819,14 +869,11 @@ bSample* sound_find_sample(bSound* sound)
 	
 	/* search through the list of loaded samples */
 	sample = samples->first;
-	
-	while (sample)
-	{
+	while (sample) {
 		strcpy(samplename, sample->name);
 		BLI_convertstringcode(samplename, G.sce, G.scene->r.cfra);
 		
-		if (strcmp(name, samplename) == 0)
-		{
+		if (strcmp(name, samplename) == 0)	{
 			break;
 		}
 		sample = sample->id.next;
@@ -837,16 +884,15 @@ bSample* sound_find_sample(bSound* sound)
 
 
 
-int sound_sample_is_null(bSound* sound)
+int sound_sample_is_null(bSound *sound)
 {
 	int result = FALSE;
-	bSample* sample;
+	bSample *sample;
 	
 	if(ghSoundScene==NULL) sound_init_audio();
 	
 	/* find the right sample or else create one */
-	if (sound->sample == NULL)
-	{
+	if (sound->sample == NULL) {
 		/* find... */
 		sample = sound_find_sample(sound);
 
@@ -887,7 +933,7 @@ void sound_end_all_sounds(void)
 
 
 
-void sound_play_sound(bSound* sound)
+void sound_play_sound(bSound *sound)
 {
 #if GAMEBLENDER == 1
 	if(ghSoundScene==NULL) sound_init_audio();
