@@ -1,5 +1,5 @@
 /**
- * $Id: editview.c,v 1.107 2006/07/19 11:28:48 ton Exp $
+ * $Id: editview.c,v 1.130 2007/01/25 21:43:49 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -52,11 +52,14 @@
 #include "DNA_lattice_types.h"
 #include "DNA_meta_types.h"
 #include "DNA_mesh_types.h"
+#include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
+#include "DNA_space_types.h"
 #include "DNA_view3d_types.h"
 #include "DNA_userdef_types.h"
+#include "DNA_ipo_types.h" /* for fly mode recording */
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
@@ -90,6 +93,7 @@
 #include "BIF_toolbox.h"
 
 #include "BDR_editobject.h"	/* For headerprint */
+#include "BDR_sculptmode.h"
 #include "BDR_vpaint.h"
 #include "BDR_editface.h"
 #include "BDR_drawobject.h"
@@ -160,7 +164,7 @@ void EM_backbuf_checkAndSelectFaces(EditMesh *em, int select)
 
 void EM_backbuf_checkAndSelectTFaces(Mesh *me, int select)
 {
-	TFace *tface = me->tface;
+	MTFace *tface = me->mtface;
 	int a;
 
 	if (tface) {
@@ -189,6 +193,22 @@ void arrows_move_cursor(unsigned short event)
 	}
 }
 
+/* simple API for object selection, rather than just using the flag
+ * this takes into account the 'restrict selection in 3d view' flag.
+ * deselect works always, the restriction just prevents selection */
+void select_base_v3d(Base *base, short mode)
+{
+	if (base) {
+		if (mode==BA_SELECT) {
+			if (!(base->object->restrictflag & OB_RESTRICT_SELECT))
+				if (mode==BA_SELECT) base->flag |= SELECT;
+		}
+		else if (mode==BA_DESELECT) {
+			base->flag &= ~SELECT;
+		}
+	}
+}
+
 /* *********************** GESTURE AND LASSO ******************* */
 
 /* helper also for borderselect */
@@ -336,8 +356,8 @@ static void do_lasso_select_objects(short mcords[][2], short moves, short select
 			project_short(base->object->obmat[3], &base->sx);
 			if(lasso_inside(mcords, moves, base->sx, base->sy)) {
 				
-				if(select) base->flag |= SELECT;
-				else base->flag &= ~SELECT;
+				if(select) select_base_v3d(base, BA_SELECT);
+				else select_base_v3d(base, BA_DESELECT);
 				base->object->flag= base->flag;
 			}
 			if(base->object->flag & OB_POSEMODE) {
@@ -536,7 +556,7 @@ static void do_lasso_select_facemode(short mcords[][2], short moves, short selec
 	rcti rect;
 	
 	me= get_mesh(OBACT);
-	if(me==NULL || me->tface==NULL) return;
+	if(me==NULL || me->mtface==NULL) return;
 	if(me->totface==0) return;
 	
 	em_vertoffs= me->totface+1;	/* max index array */
@@ -766,7 +786,7 @@ int gesture(void)
 				if(i) {
 					if(lasso) draw_lasso_select(mcords, i, 0);
 					else sdrawXORline(mcords[i-1][0], mcords[i-1][1], mcords[i][0], mcords[i][1]);
-					glFlush();
+					bglFlush();
 				}
 				i++;
 			}
@@ -900,8 +920,10 @@ void deselectall(void)	/* is toggle */
 	base= FIRSTBASE;
 	while(base) {
 		if(base->lay & G.vd->lay) {
-			if(a) base->flag &= ~SELECT;
-			else base->flag |= SELECT;
+			if(a) 
+				select_base_v3d(base, BA_DESELECT);
+			else 
+				select_base_v3d(base, BA_SELECT);
 			base->object->flag= base->flag;
 		}
 		base= base->next;
@@ -922,8 +944,10 @@ void selectswap(void)
 
 	for(base= FIRSTBASE; base; base= base->next) {
 		if(base->lay & G.vd->lay) {
-			if TESTBASE(base) base->flag &= ~SELECT;
-			else base->flag |= SELECT;
+			if TESTBASE(base)
+				select_base_v3d(base, BA_DESELECT);
+			else
+				select_base_v3d(base, BA_SELECT);
 			base->object->flag= base->flag;
 		}
 	}
@@ -945,7 +969,7 @@ void selectall_type(short obtype)
 	base= FIRSTBASE;
 	while(base) {
 		if((base->lay & G.vd->lay) && (base->object->type == obtype)) {
-			base->flag |= SELECT;
+			select_base_v3d(base, BA_SELECT);
 			base->object->flag= base->flag;
 		}
 		base= base->next;
@@ -966,7 +990,7 @@ void selectall_layer(unsigned int layernum)
 	base= FIRSTBASE;
 	while(base) {
 		if (base->lay == (1<< (layernum -1))) {
-			base->flag |= SELECT;
+			select_base_v3d(base, BA_SELECT);
 			base->object->flag= base->flag;
 		}
 		base= base->next;
@@ -988,7 +1012,7 @@ static void deselectall_except(Base *b)   /* deselect all except b */
 	while(base) {
 		if (base->flag & SELECT) {
 			if(b!=base) {
-				base->flag &= ~SELECT;
+				select_base_v3d(base, BA_DESELECT);
 				base->object->flag= base->flag;
 			}
 		}
@@ -1049,6 +1073,25 @@ void set_active_base(Base *base)
 {
 	Base *tbase;
 	
+	/* activating a non-mesh, should end a couple of modes... */
+	if(base) {
+		if(base->object->type!=OB_MESH) {
+			if(G.f & G_SCULPTMODE)
+				set_sculptmode(); /* toggle */
+			if(G.f & G_WEIGHTPAINT)
+				set_wpaint();	/* toggle */
+			if(G.f & G_VERTEXPAINT)
+				set_vpaint();	/* toggle */
+			if(G.f & G_TEXTUREPAINT) 
+				set_texturepaint(); /* Switch off tex paint */
+		}
+		/* always end this */
+		if(G.f & G_FACESELECT) {
+			set_faceselect();	/* toggle */
+		}
+	}
+	
+	/* sets scene->basact */
 	BASACT= base;
 	
 	if(base) {
@@ -1098,16 +1141,19 @@ static void select_all_from_groups(Base *basact)
 		if(object_in_group(basact->object, group)) {
 			for(go= group->gobject.first; go; go= go->next) {
 				if(deselect) go->ob->flag &= ~SELECT;
-				else go->ob->flag |= SELECT;
+				else {
+					if (!(go->ob->restrictflag & OB_RESTRICT_SELECT))
+						go->ob->flag |= SELECT;
+				}
 			}
 		}
 	}
 	/* sync bases */
 	for(basact= G.scene->base.first; basact; basact= basact->next) {
 		if(basact->object->flag & SELECT)
-			basact->flag |= SELECT;
+			select_base_v3d(basact, BA_SELECT);
 		else
-			basact->flag &= ~SELECT;
+			select_base_v3d(basact, BA_DESELECT);
 	}
 }
 
@@ -1235,7 +1281,9 @@ void mouse_select(void)
 	/* This block uses the control key to make the object selected by its centre point rather then its contents */
 	if(G.obedit==0 && (G.qual & LR_CTRLKEY)) {
 		
-		if(G.qual & LR_ALTKEY) basact= mouse_select_menu(NULL, 0, mval);
+		/* note; shift+alt goes to group-flush-selecting */
+		if(G.qual == (LR_ALTKEY|LR_CTRLKEY)) 
+			basact= mouse_select_menu(NULL, 0, mval);
 		else {
 			base= startbase;
 			while(base) {
@@ -1269,7 +1317,8 @@ void mouse_select(void)
 			
 			for(a=0; a<hits; a++) if(buffer[4*a+3] & 0xFFFF0000) has_bones= 1;
 
-			if(has_bones==0 && (G.qual & LR_ALTKEY)) 
+			/* note; shift+alt goes to group-flush-selecting */
+			if(has_bones==0 && (G.qual == LR_ALTKEY)) 
 				basact= mouse_select_menu(buffer, hits, mval);
 			else {
 				static short lastmval[2]={-100, -100};
@@ -1324,6 +1373,14 @@ void mouse_select(void)
 					
 					base= startbase;
 					while(base) {
+						/* skip objects with select restriction, to prevent prematurely ending this loop
+						 * with an un-selectable choice */
+						if (base->object->restrictflag & OB_RESTRICT_SELECT) {
+							base=base->next;
+							if(base==NULL) base= FIRSTBASE;
+							if(base==startbase) break;
+						}
+					
 						if(base->lay & G.vd->lay) {
 							for(a=0; a<hits; a++) {
 								if(has_bones) {
@@ -1374,15 +1431,17 @@ void mouse_select(void)
 		if(G.obedit) {
 			/* only do select */
 			deselectall_except(basact);
-			basact->flag |= SELECT;
+			select_base_v3d(basact, BA_SELECT);
 		}
-		else {
+		/* also prevent making it active on mouse selection */
+		else if (!(basact->object->restrictflag & OB_RESTRICT_SELECT)) {
+
 			oldbasact= BASACT;
 			BASACT= basact;
 			
 			if((G.qual & LR_SHIFTKEY)==0) {
 				deselectall_except(basact);
-				basact->flag |= SELECT;
+				select_base_v3d(basact, BA_SELECT);
 			}
 			else if(G.qual==(LR_SHIFTKEY|LR_ALTKEY)) {
 				select_all_from_groups(basact);
@@ -1390,9 +1449,9 @@ void mouse_select(void)
 			else {
 				if(basact->flag & SELECT) {
 					if(basact==oldbasact)
-						basact->flag &= ~SELECT;
+						select_base_v3d(basact, BA_DESELECT);
 				}
-				else basact->flag |= SELECT;
+				else select_base_v3d(basact, BA_SELECT);
 			}
 
 			/* copy */
@@ -1414,19 +1473,6 @@ void mouse_select(void)
 				}
 			}
 			
-			/* selecting a non-mesh, should end a couple of modes... */
-			if(basact->object->type!=OB_MESH) {
-				if(G.f & G_WEIGHTPAINT) {
-					set_wpaint();	/* toggle */
-				}
-				if(G.f & G_VERTEXPAINT) {
-					set_vpaint();	/* toggle */
-				}
-				if(G.f & G_FACESELECT) {
-					set_faceselect();	/* toggle */
-				}
-			}
-			
 			allqueue(REDRAWBUTSLOGIC, 0);
 			allqueue(REDRAWDATASELECT, 0);
 			allqueue(REDRAWBUTSOBJECT, 0);
@@ -1760,9 +1806,9 @@ void borderselect(void)
 						}
 						else {
 							if (selecting)
-								base->flag |= SELECT;
+								select_base_v3d(base, BA_SELECT);
 							else
-								base->flag &= ~SELECT;
+								select_base_v3d(base, BA_DESELECT);
 
 							base->object->flag= base->flag;
 						}
@@ -1777,7 +1823,7 @@ void borderselect(void)
 			}
 		}
 		/* frontbuffer flush */
-		glFlush();
+		bglFlush();
 
 		MEM_freeN(vbuffer);
 		
@@ -1975,19 +2021,17 @@ void set_render_border(void)
 {
 	rcti rect;
 	short val;
-
-	if(G.vd->persp!=2) return;
 	
 	val= get_border(&rect, 3);
 	if(val) {
-		rcti vb;
+		rctf vb;
 
 		calc_viewborder(G.vd, &vb);
 
-		G.scene->r.border.xmin= (float) (rect.xmin-vb.xmin)/(vb.xmax-vb.xmin);
-		G.scene->r.border.ymin= (float) (rect.ymin-vb.ymin)/(vb.ymax-vb.ymin);
-		G.scene->r.border.xmax= (float) (rect.xmax-vb.xmin)/(vb.xmax-vb.xmin);
-		G.scene->r.border.ymax= (float) (rect.ymax-vb.ymin)/(vb.ymax-vb.ymin);
+		G.scene->r.border.xmin= ((float)rect.xmin-vb.xmin)/(vb.xmax-vb.xmin);
+		G.scene->r.border.ymin= ((float)rect.ymin-vb.ymin)/(vb.ymax-vb.ymin);
+		G.scene->r.border.xmax= ((float)rect.xmax-vb.xmin)/(vb.xmax-vb.xmin);
+		G.scene->r.border.ymax= ((float)rect.ymax-vb.ymin)/(vb.ymax-vb.ymin);
 		
 		CLAMP(G.scene->r.border.xmin, 0.0, 1.0);
 		CLAMP(G.scene->r.border.ymin, 0.0, 1.0);
@@ -2001,6 +2045,40 @@ void set_render_border(void)
 	}
 }
 
+void view3d_border_zoom(void)
+{
+	/* Zooms in on a border drawn by the user */
+	rcti rect;
+	short val;
+	float dvec[3], vb[2], xscale, yscale, scale;
+	
+	/* doesn't work fine for perspective */
+	if(G.vd->persp==1)
+		return;
+	
+	val = get_border(&rect, 3); //box select input
+	if(val)
+	{
+		/* find the current window width and height */
+		vb[0] = G.vd->area->winx;
+		vb[1] = G.vd->area->winy;
+		
+		/* convert the drawn rectangle into 3d space */
+		initgrabz(-G.vd->ofs[0], -G.vd->ofs[1], -G.vd->ofs[2]);
+		window_to_3d(dvec, (rect.xmin+rect.xmax-vb[0])/2, (rect.ymin+rect.ymax-vb[1])/2);
+		
+		/* center the view to the center of the rectangle */
+		VecSubf(G.vd->ofs, G.vd->ofs, dvec);
+		
+		/* work out the ratios, so that everything selected fits when we zoom */
+		xscale = ((rect.xmax-rect.xmin)/vb[0]);
+		yscale = ((rect.ymax-rect.ymin)/vb[1]);
+		scale = (xscale >= yscale)?xscale:yscale;
+		
+		/* zoom in as required, or as far as we can go */
+		G.vd->dist = ((G.vd->dist*scale) >= 0.001*G.vd->grid)? G.vd->dist*scale:0.001*G.vd->grid;
+	}
+}
 
 void fly(void)
 {
@@ -2011,6 +2089,9 @@ void fly(void)
 	float speed=0.0, /* the speed the view is moving per redraw */
 	mat[3][3], /* 3x3 copy of the view matrix so we can move allong the view axis */
 	dvec[3]={0,0,0}, /* this is the direction thast added to the view offset per redraw */
+	dvec_old[3]={0,0,0}, /* old for some lag */
+	dvec_tmp[3]={0,0,0}, /* old for some lag */
+	dvec_lag=0.0, /* old for some lag */
 	
 	/* Camera Uprighting variables */
 	roll, /* similar to the angle between the camera's up and the Z-up, but its very rough so just roll*/
@@ -2028,7 +2109,8 @@ void fly(void)
 	double time_current, time_lastdraw;
 	
 	short val, /* used for toets to see if a buttons pressed */
-	cent[2], /* view center */
+	cent_orig[2], /* view center */
+	cent[2], /* view center modified */
 	mval[2], /* mouse location */
 	action=0, /* while zero stay in fly mode and wait for action, also used to see if we accepted or canceled 1:ok 2:Cancel */
 	xmargin, ymargin; /* x and y margin are define the safe area where the mouses movement wont rotate the view */
@@ -2037,15 +2119,52 @@ void fly(void)
 	unsigned char
 	apply_rotation= 1, /* if the user presses shift they can look about without movinf the direction there looking*/
 	axis= 2, /* Axis index to move allong by default Z to move allong the view */
-	persp_backup; /* remember if were ortho or not, only used for restoring the view if it was a ortho view */
+	persp_backup, /* remember if were ortho or not, only used for restoring the view if it was a ortho view */
+	pan_view=0; /* if true, pan the view instead of rotating */
+	
+	/* relative view axis locking - xlock, zlock
+	0; disabled
+	1; enabled but not checking because mouse hasnt moved outside the margin since locking was checked an not needed
+	   when the mouse moves, locking is set to 2 so checks are done.
+	2; mouse moved and checking needed, if no view altering is donem its changed back to 1 */
 	short xlock=0, zlock=0;
+	float xlock_momentum=0.0f, zlock_momentum=0.0f; /* nicer dynamics */
+	
+	/* for recording */
+	int playing_anim = has_screenhandler(G.curscreen, SCREEN_HANDLER_ANIM);
+	int cfra = -1; /*so the first frame always has a key added */
+	char *actname="";
+	
 	
 	if(curarea->spacetype!=SPACE_VIEW3D) return;
-	if(G.vd->persp==2 && G.vd->camera->id.lib) return;
+		
+	if(G.vd->persp==2 && G.vd->camera->id.lib) {
+		error("Cannot fly a camera from an external library");
+		return;
+	}
+	
+	if(G.vd->ob_centre) {
+		error("Cannot fly when the view is locked to an object");
+		return;
+	}
+	
+	
+	/* detect weather to start with Z locking */
+	upvec[0]=1; upvec[1]=0; upvec[2]=0;
+	Mat3CpyMat4(mat, G.vd->viewinv);
+	Mat3MulVecfl(mat, upvec);
+	if (fabs(upvec[2]) < 0.1)
+		zlock = 1;
+	upvec[0]=0; upvec[1]=0; upvec[2]=0;
 	
 	persp_backup= G.vd->persp;
 	dist_backup= G.vd->dist;
 	if (G.vd->persp==2) { /* Camera */
+		if(G.vd->camera->constraints.first) {
+			error("Cannot fly an object with constraints");
+			return;
+		}
+		
 		/* store the origoinal camera loc and rot */
 		VECCOPY(ofs_backup, G.vd->camera->loc);
 		VECCOPY(rot_backup, G.vd->camera->rot);
@@ -2056,6 +2175,11 @@ void fly(void)
 		
 		G.vd->dist=0.0;
 		G.vd->viewbut=0;
+		
+		/* used for recording */
+		if(G.vd->camera->ipoflag & OB_ACTION_OB)
+			actname= "Object";
+		
 	} else {
 		/* perspective or ortho */
 		if (G.vd->persp==0)
@@ -2065,7 +2189,6 @@ void fly(void)
 		G.vd->dist= 0.0;
 		
 		upvec[2]=dist_backup; /*x and y are 0*/
-		Mat3CpyMat4(mat, G.vd->viewinv);
 		Mat3MulVecfl(mat, upvec);
 		VecSubf(G.vd->ofs, G.vd->ofs, upvec);
 		/*Done with correcting for the dist*/
@@ -2081,10 +2204,10 @@ void fly(void)
 	xmargin= (short)((float)(curarea->winx)/20.0);
 	ymargin= (short)((float)(curarea->winy)/20.0);
 	
-	cent[0]= curarea->winrct.xmin+(curarea->winx)/2;
-	cent[1]= curarea->winrct.ymin+(curarea->winy)/2;
+	cent_orig[0]= curarea->winrct.xmin+(curarea->winx)/2;
+	cent_orig[1]= curarea->winrct.ymin+(curarea->winy)/2;
 	
-	warp_pointer(cent[0], cent[1]);
+	warp_pointer(cent_orig[0], cent_orig[1]);
 
 	/* we have to rely on events to give proper mousecoords after a warp_pointer */
 	mval[0]= cent[0]=  (curarea->winx)/2;
@@ -2096,7 +2219,7 @@ void fly(void)
 	
 	time_lastdraw= PIL_check_seconds_timer();
 	
-	G.vd->flag2 |= V3D_FLYMODE;
+	G.vd->flag2 |= V3D_FLYMODE; /* so we draw the corner margins */
 	scrarea_do_windraw(curarea);
 	screen_swapbuffers();
 	
@@ -2119,6 +2242,13 @@ void fly(void)
 					if (speed>0) speed=0;
 					else speed-= G.vd->grid;
 				
+				} else if (toets==MIDDLEMOUSE) {
+					/* make it so the camera direction dosent follow the view
+					good for flying backwards! - Only when MMB is held */
+					
+					/*apply_rotation=0;*/
+					pan_view= 1;
+					
 				/* impliment WASD keys */
 				} else if(toets==WKEY) {
 					if (speed<0) speed=-speed; /* flip speed rather then stopping, game like motion */
@@ -2144,11 +2274,26 @@ void fly(void)
 				
 				/* axis locking */
 				} else if(toets==XKEY) {
-					xlock= !xlock;
+					if (xlock) xlock=0;
+					else {
+						xlock = 2;
+						xlock_momentum = 0.0;
+					}
 				} else if(toets==ZKEY) {
-					zlock= !zlock;
+					if (zlock) zlock=0;
+					else {
+						zlock = 2;
+						zlock_momentum = 0.0;
+					}
 				}
 				
+			} else {
+				/* mouse buttons lifted */
+				if (toets==MIDDLEMOUSE && pan_view) {
+					/*apply_rotation=1;*/
+					warp_pointer(cent_orig[0], cent_orig[1]);
+					pan_view= 0;
+				}
 			}
 		}
 		if(action != 0) break;
@@ -2177,24 +2322,7 @@ void fly(void)
 		}
 		
 		/* Should we redraw? */
-		if(speed!=0.0 || moffset[0] || moffset[1] || zlock || xlock ) {
-			
-			/* Set the view direction usine the current axis */
-			if ((G.qual & LR_ALTKEY) && speed!=0.0 && (moffset[0]||moffset[1]))
-				/* make it so the camera direction dosent follow the view
-				good for flying backwards! - Only when Alt is held */
-			
-				/*(Above IF) We need to make sure we have some mouse offset
-				and are moving before we ignore the rotation code, otherwise the view spins out */
-				apply_rotation=0;
-			else {
-				/* Normal operation */
-				apply_rotation=1;
-				/* define dvec, view direction vector */
-				dvec[0]= dvec[1]= dvec[2]= 0;
-				/* move along the current axis */
-				dvec[axis]= 1.0f;
-			}
+		if(speed!=0.0 || moffset[0] || moffset[1] || zlock || xlock || dvec[0] || dvec[1] || dvec[2] ) {
 			
 			time_current= PIL_check_seconds_timer();
 			time_redraw= (float)(time_current-time_lastdraw);
@@ -2208,101 +2336,164 @@ void fly(void)
 				speed= speed * (1-time_redraw_clamped);
 			
 			Mat3CpyMat4(mat, G.vd->viewinv);
-			if (apply_rotation) {
-				Normalise(dvec);
-				Mat3MulVecfl(mat, dvec);
-			}
-			/* rotate about the X axis- look up/down */
-			if (moffset[1]) {
-				upvec[0]=1;
-				upvec[1]=0;
-				upvec[2]=0;
-				Mat3MulVecfl(mat, upvec);
-				VecRotToQuat( upvec, (float)moffset[1]*-time_redraw*20, tmp_quat); /* Rotate about the relative up vec */
-				QuatMul(G.vd->viewquat, G.vd->viewquat, tmp_quat);
-			}
 			
-			/* rotate about the Y axis- look left/right */
-			if (moffset[0]) {
-				upvec[0]=0;
-				upvec[1]=1;
-				upvec[2]=0;
-				Mat3MulVecfl(mat, upvec);
-				VecRotToQuat( upvec, (float)moffset[0]*time_redraw*20, tmp_quat); /* Rotate about the relative up vec */
-				QuatMul(G.vd->viewquat, G.vd->viewquat, tmp_quat);
-			}
-			
-			if (zlock) {
-				upvec[0]=1;
-				upvec[1]=0;
-				upvec[2]=0;
-				Mat3MulVecfl(mat, upvec);
-
-				/*make sure we have some z rolling*/
-				if (fabs(upvec[2]) > 0.00001) {
-					roll= upvec[2]*180;
-					upvec[0]=0; /*rotate the view about this axis*/
-					upvec[1]=0;
-					upvec[2]=1;
-					
-					/*clamp teh max roll speed - hides the fact that the roll calculation
-					only realy detects if we're rolling, but not by how much */
-					if (roll>4) roll= 4;
-					else if (roll<-4) roll= -4;
-					
-					Mat3MulVecfl(mat, upvec);
-					VecRotToQuat( upvec, roll*time_redraw_clamped*0.1, tmp_quat); /* Rotate about the relative up vec */
-					QuatMul(G.vd->viewquat, G.vd->viewquat, tmp_quat);
-				}
-			}
-			
-			if (xlock && moffset[1]==0) { /*only apply xcorrect when mouse isnt applying x rot*/
-				upvec[0]=0;
-				upvec[1]=0;
-				upvec[2]=1;
-				Mat3MulVecfl(mat, upvec);
-				/*make sure we have some z rolling*/
-				if (fabs(upvec[2]) > 0.00001) {
-					roll= upvec[2]*-360;
-					
-					upvec[0]=1; /*rotate the view about this axis*/
+			if (pan_view) {
+				/* pan only */
+				dvec_tmp[0]= -moffset[0];
+				dvec_tmp[1]= -moffset[1];
+				dvec_tmp[2]= 0;
+				/* z axis can stay teh same, just keep costing */
+				
+				Mat3MulVecfl(mat, dvec_tmp);
+				VecMulf(dvec_tmp, time_redraw*200.0 * G.vd->grid);
+				
+			} else {
+				/* rotate about the X axis- look up/down */
+				if (moffset[1]) {
+					upvec[0]=1;
 					upvec[1]=0;
 					upvec[2]=0;
-					
-					/*clamp teh max roll speed - hides the fact that the roll calculation
-					only realy detects if we're rolling, but not by how much */
-					if (roll>4) roll= 4;
-					else if (roll<-4) roll= -4;
-					
 					Mat3MulVecfl(mat, upvec);
-					
-					VecRotToQuat( upvec, roll*time_redraw_clamped*0.1, tmp_quat); /* Rotate about the relative up vec */
+					VecRotToQuat( upvec, (float)moffset[1]*-time_redraw*20, tmp_quat); /* Rotate about the relative up vec */
 					QuatMul(G.vd->viewquat, G.vd->viewquat, tmp_quat);
+					
+					if (xlock) xlock = 2; /*check for rotation*/
+					if (zlock) zlock = 2;
+					xlock_momentum= 0.0;
+				}
+				
+				/* rotate about the Y axis- look left/right */
+				if (moffset[0]) {
+					
+					if (zlock) {
+						upvec[0]=0;
+						upvec[1]=0;
+						upvec[2]=1;
+					} else {
+						upvec[0]=0;
+						upvec[1]=1;
+						upvec[2]=0;
+						Mat3MulVecfl(mat, upvec);
+					}
+						
+					VecRotToQuat( upvec, (float)moffset[0]*time_redraw*20, tmp_quat); /* Rotate about the relative up vec */
+					QuatMul(G.vd->viewquat, G.vd->viewquat, tmp_quat);
+					
+					if (xlock) xlock = 2;/*check for rotation*/
+					if (zlock) zlock = 2;
+				}
+				
+				if (zlock==2) {
+					upvec[0]=1;
+					upvec[1]=0;
+					upvec[2]=0;
+					Mat3MulVecfl(mat, upvec);
+
+					/*make sure we have some z rolling*/
+					if (fabs(upvec[2]) > 0.00001) {
+						roll= upvec[2]*5;
+						upvec[0]=0; /*rotate the view about this axis*/
+						upvec[1]=0;
+						upvec[2]=1;
+						
+						Mat3MulVecfl(mat, upvec);
+						VecRotToQuat( upvec, roll*time_redraw_clamped*zlock_momentum*0.1, tmp_quat); /* Rotate about the relative up vec */
+						QuatMul(G.vd->viewquat, G.vd->viewquat, tmp_quat);
+						
+						zlock_momentum += 0.05;
+					} else {
+						zlock=1; /* dont check until the view rotates again */
+						zlock_momentum= 0.0;
+					}
+				}
+				
+				if (xlock==2 && moffset[1]==0) { /*only apply xcorrect when mouse isnt applying x rot*/
+					upvec[0]=0;
+					upvec[1]=0;
+					upvec[2]=1;
+					Mat3MulVecfl(mat, upvec);
+					/*make sure we have some z rolling*/
+					if (fabs(upvec[2]) > 0.00001) {
+						roll= upvec[2]*-5;
+						
+						upvec[0]=1; /*rotate the view about this axis*/
+						upvec[1]=0;
+						upvec[2]=0;
+						
+						Mat3MulVecfl(mat, upvec);
+						
+						VecRotToQuat( upvec, roll*time_redraw_clamped*xlock_momentum*0.1, tmp_quat); /* Rotate about the relative up vec */
+						QuatMul(G.vd->viewquat, G.vd->viewquat, tmp_quat);
+						
+						xlock_momentum += 0.05;
+					} else {
+						xlock=1; /* see above */
+						xlock_momentum= 0.0;
+					}
+				}
+
+
+				if (apply_rotation) {
+					/* Normal operation */
+					/* define dvec, view direction vector */
+					dvec_tmp[0]= dvec_tmp[1]= dvec_tmp[2]= 0;
+					/* move along the current axis */
+					dvec_tmp[axis]= 1.0f;
+					
+					Mat3MulVecfl(mat, dvec_tmp);
+					
+					VecMulf(dvec_tmp, speed*time_redraw*0.5);
 				}
 			}
-
-
-			if (apply_rotation)
-				VecMulf(dvec, speed*time_redraw*0.5);
-
+			
+			/* impose a directional lag */
+			dvec_lag = 1.0/(1+(time_redraw*5));
+			dvec[0] = dvec_tmp[0]*(1-dvec_lag) + dvec_old[0]*dvec_lag;
+			dvec[1] = dvec_tmp[1]*(1-dvec_lag) + dvec_old[1]*dvec_lag;
+			dvec[2] = dvec_tmp[2]*(1-dvec_lag) + dvec_old[2]*dvec_lag;
+			
 			VecAddf(G.vd->ofs, G.vd->ofs, dvec);
-			headerprint("FlyKeys  Speed:(+/- | Wheel),  MouseLook:Alt,  Upright Axis:X/Z,  Slow:Shift,  Direction:WASDRF,  Ok:LMB,  Cancel:RMB");
-
+			if (zlock && xlock)
+				headerprint("FlyKeys  Speed:(+/- | Wheel),  Upright Axis:X  on/Z on,   Slow:Shift,  Direction:WASDRF,  Ok:LMB,  Pan:MMB,  Cancel:RMB");
+			else if (zlock) 
+				headerprint("FlyKeys  Speed:(+/- | Wheel),  Upright Axis:X off/Z on,   Slow:Shift,  Direction:WASDRF,  Ok:LMB,  Pan:MMB,  Cancel:RMB");
+			else if (xlock)
+				headerprint("FlyKeys  Speed:(+/- | Wheel),  Upright Axis:X  on/Z off,  Slow:Shift,  Direction:WASDRF,  Ok:LMB,  Pan:MMB,  Cancel:RMB");
+			else
+				headerprint("FlyKeys  Speed:(+/- | Wheel),  Upright Axis:X off/Z off,  Slow:Shift,  Direction:WASDRF,  Ok:LMB,  Pan:MMB,  Cancel:RMB");
+			
+			do_screenhandlers(G.curscreen); /* advance the next frame */
+			
 			/* we are in camera view so apply the view ofs and quat to the view matrix and set the camera to teh view */
 			if (G.vd->persp==2) {
 				G.vd->persp= 1; /*set this so setviewmatrixview3d uses the ofs and quat instead of the camera */
 				setviewmatrixview3d();
 				setcameratoview3d();
 				G.vd->persp= 2;
-				DAG_object_flush_update(G.scene, G.vd->camera, OB_RECALC_OB);
+				
+				/* record the motion */
+				if (G.flags & G_RECORDKEYS && (!playing_anim || cfra != G.scene->r.cfra)) {
+					cfra = G.scene->r.cfra;
+					
+					if (xlock || zlock || moffset[0] || moffset[1]) {
+						insertkey(&G.vd->camera->id, ID_OB, actname, NULL, OB_ROT_X);
+						insertkey(&G.vd->camera->id, ID_OB, actname, NULL, OB_ROT_Y);
+						insertkey(&G.vd->camera->id, ID_OB, actname, NULL, OB_ROT_Z);
+					}
+					if (speed) {
+						insertkey(&G.vd->camera->id, ID_OB, actname, NULL, OB_LOC_X);
+						insertkey(&G.vd->camera->id, ID_OB, actname, NULL, OB_LOC_Y);
+						insertkey(&G.vd->camera->id, ID_OB, actname, NULL, OB_LOC_Z);
+					}
+				}
 			}
-
 			scrarea_do_windraw(curarea);
 			screen_swapbuffers();
 		} else 
 			/*were not redrawing but we need to update the time else the view will jump */
 			time_lastdraw= PIL_check_seconds_timer();
 		/* end drawing */
+		VECCOPY(dvec_old, dvec);
 	}
 	
 	G.vd->dist= dist_backup;
@@ -2320,7 +2511,22 @@ void fly(void)
 			VECCOPY(G.vd->ofs, ofs_backup);
 			G.vd->persp= persp_backup;
 		}
-	} else if (persp_backup!=2) { /* not camera */
+	}
+	else if (persp_backup==2) {	/* camera */
+		float mat3[3][3];
+		Mat3CpyMat4(mat3, G.vd->camera->obmat);
+		Mat3ToCompatibleEul(mat3, G.vd->camera->rot, rot_backup);
+		
+		DAG_object_flush_update(G.scene, G.vd->camera, OB_RECALC_OB);
+		
+		if (G.flags & G_RECORDKEYS) {
+			allqueue(REDRAWIPO, 0);
+			allspace(REMAKEIPO, 0);
+			allqueue(REDRAWNLA, 0);
+			allqueue(REDRAWTIME, 0);
+		}
+	}
+	else { /* not camera */
 		/* Apply the fly mode view */
 		/*restore the dist*/
 		upvec[0]= upvec[1]= 0;
@@ -2336,7 +2542,6 @@ void fly(void)
 	BIF_view3d_previewrender_signal(curarea, PR_DBASE|PR_DISPRECT); /* not working at the moment not sure why */
 }
 
-
 void view3d_edit_clipping(View3D *v3d)
 {
 	
@@ -2369,6 +2574,11 @@ void view3d_edit_clipping(View3D *v3d)
 		glGetIntegerv(GL_VIEWPORT, viewport);
 		glGetDoublev(GL_MODELVIEW_MATRIX, mvmatrix);
 		glGetDoublev(GL_PROJECTION_MATRIX, projmatrix);
+
+		/* near zero floating point values can give issues with gluUnProject
+		   in side view on some implementations */
+		if(fabs(mvmatrix[0]) < 1e-6) mvmatrix[0]= 0.0;
+		if(fabs(mvmatrix[5]) < 1e-6) mvmatrix[5]= 0.0;
 		
 		/* Set up viewport so that gluUnProject will give correct values */
 		viewport[0] = 0;
