@@ -1,5 +1,5 @@
 /**
- * $Id: filesel.c,v 1.76 2006/06/16 12:17:35 ton Exp $
+ * $Id: filesel.c,v 1.92 2007/02/09 11:47:51 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -64,6 +64,7 @@
 #include "DNA_ipo_types.h"
 #include "DNA_material_types.h"
 #include "DNA_mesh_types.h"
+#include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
 #include "DNA_texture_types.h"
 #include "DNA_space_types.h"
@@ -84,14 +85,16 @@
 #include "BKE_material.h"
 #include "BKE_utildefines.h"
 
+#include "BIF_editview.h"
 #include "BIF_gl.h"
 #include "BIF_interface.h"
-#include "BIF_toolbox.h"
+#include "BIF_language.h"
 #include "BIF_mywindow.h"
-#include "BIF_editview.h"
+#include "BIF_resources.h"
 #include "BIF_space.h"
 #include "BIF_screen.h"
-#include "BIF_resources.h"
+#include "BIF_toolbox.h"
+#include "BIF_usiblender.h"
 
 #include "BLO_readfile.h"
 
@@ -108,6 +111,10 @@
 
 #include "BIF_fsmenu.h"  /* include ourselves */
 
+#if defined WITH_ICONV
+	#include "iconv.h"
+#endif
+
 #if defined WIN32 || defined __BeOS
 int fnmatch(const char *pattern, const char *string, int flags)
 {
@@ -133,6 +140,18 @@ int fnmatch(const char *pattern, const char *string, int flags)
 
 #define STARTSWITH(x, y) (strncmp(x, y, sizeof(x) - 1) == 0)
 
+/* button events */
+#define B_FS_FILENAME	1
+#define B_FS_DIRNAME	2
+#define B_FS_DIR_MENU	3
+#define B_FS_PARDIR	4
+#define B_FS_LOAD	5
+#define B_FS_CANCEL	6
+#define B_FS_LIBNAME	7
+
+/* max length of library group name within filesel */
+#define GROUP_MAX 32
+
 static int is_a_library(SpaceFile *sfile, char *dir, char *group);
 static void do_library_append(SpaceFile *sfile);
 static void library_to_filelist(SpaceFile *sfile);
@@ -142,13 +161,13 @@ static int groupname_to_code(char *group);
 
 extern void countall(void);
 
-/* local globals */
+/* very bad local globals */
 
 static rcti scrollrct, textrct, bar;
 static int filebuty1, filebuty2, page_ofs, collumwidth, selecting=0;
 static int filetoname= 0;
 static float pixels_to_ofs;
-static char otherdir[FILE_MAXDIR+FILE_MAXFILE];
+static char otherdir[FILE_MAX];
 static ScrArea *otherarea;
 
 /* FSMENU HANDLING */
@@ -425,13 +444,13 @@ static int compare_extension(const void *a1, const void *a2) {
 }
 
 /* **************************************** */
-
-void clear_global_filesel_vars()
+static int filesel_has_func(SpaceFile *sfile)
 {
-	selecting= 0;
+	if(sfile->returnfunc || sfile->returnfunc_event || sfile->returnfunc_args)
+		return 1;
+	return 0;
 }
 
-
 void filesel_statistics(SpaceFile *sfile, int *totfile, int *selfile, float *totlen, float *sellen)
 {
 	double len;
@@ -481,7 +500,7 @@ void test_flags_file(SpaceFile *sfile)
 				file->flags |= BLENDERFILE;
 				
 				if(sfile->type==FILE_LOADLIB) {
-					char name[FILE_MAXDIR+FILE_MAXFILE];
+					char name[FILE_MAX];
 					BLI_strncpy(name, sfile->dir, sizeof(name));
 					strcat(name, file->relname);
 					
@@ -598,7 +617,7 @@ void sort_filelist(SpaceFile *sfile)
 void read_dir(SpaceFile *sfile)
 {
 	int num, len;
-	char wdir[FILE_MAXDIR+FILE_MAXFILE];
+	char wdir[FILE_MAX];
 
 	/* sfile->act is used for example in databrowse: double names of library objects */
 	sfile->act= -1;
@@ -659,7 +678,7 @@ void freefilelist(SpaceFile *sfile)
 
 static void split_sfile(SpaceFile *sfile, char *s1)
 {
-	char string[FILE_MAXDIR+FILE_MAXFILE], dir[FILE_MAXDIR+FILE_MAXFILE], file[FILE_MAXDIR+FILE_MAXFILE];
+	char string[FILE_MAX], dir[FILE_MAX], file[FILE_MAX];
 
 	BLI_strncpy(string, s1, sizeof(string));
 
@@ -683,7 +702,7 @@ void parent(SpaceFile *sfile)
 	char *dir;
 	
 	/* if databrowse: no parent */
-	if(sfile->type==FILE_MAIN && sfile->returnfunc) return;
+	if(sfile->type==FILE_MAIN && filesel_has_func(sfile)) return;
 
 	dir= sfile->dir;
 	
@@ -705,7 +724,9 @@ void parent(SpaceFile *sfile)
 	if( (a = strlen(dir)) ) {
 		if (dir[a-1] != '\\') strcat(dir,"\\");
 	}
-	else if(sfile->type!=FILE_MAIN) strcpy(dir,"\\");
+	else if(sfile->type!=FILE_MAIN) { 
+		get_default_root(dir);
+	}
 #else
 	if( (a = strlen(dir)) ) {				/* remove all '/' at the end */
 		while(dir[a-1] == '/') {
@@ -875,6 +896,31 @@ static void linerect(int id, int x, int y)
 
 }
 
+#ifdef WITH_ICONV
+static void string_to_utf8(char *original, char *utf_8) 
+{
+	size_t inbytesleft=strlen(original);
+	size_t outbytesleft=512;
+	size_t rv=0;
+	iconv_t cd;
+	
+	cd=iconv_open("UTF-8", "gb2312");
+	if (cd == (iconv_t)(-1)) {
+		printf("iconv_open Error");
+		*utf_8='\0';
+		return ;
+	}
+	rv=iconv(cd, &original, &inbytesleft, &utf_8, &outbytesleft);
+	if (rv == (size_t) -1) {
+		printf("iconv Error\n");
+		return ;
+	}
+	*utf_8 = '\0';
+	iconv_close(cd);
+}
+#endif
+
+
 static void print_line(SpaceFile *sfile, struct direntry *files, int x, int y)
 {
 	int boxcol=0;
@@ -922,7 +968,16 @@ static void print_line(SpaceFile *sfile, struct direntry *files, int x, int y)
 	s = files->string;
 	if(s) {
 		glRasterPos2i(x,  y);
+#ifdef WITH_ICONV
+		{
+			char utf_8[512];
+			string_to_utf8(files->relname, utf_8);
+			BIF_RasterPos((float)x, (float)y);	/* texture fonts */
+			BIF_DrawString(G.font, utf_8, (U.transopts & USER_TR_MENUS));
+		}
+#else
 		BMF_DrawString(G.font, files->relname);
+#endif
 		
 		x += sfile->maxnamelen + 100;
 
@@ -1076,6 +1131,21 @@ static void draw_filetext(SpaceFile *sfile)
 	uiEmboss(textrct.xmin, textrct.ymin, textrct.xmax, textrct.ymax, 1);
 }
 
+static char *library_string(void)
+{
+	Library *lib;
+	char *str;
+	int nr=0, tot= BLI_countlist(&G.main->library);
+	
+	if(tot==0) return NULL;
+	str= MEM_callocN(tot*(FILE_MAX), "filesel lib menu");
+	
+	for(tot=0, lib= G.main->library.first; lib; lib= lib->id.next, nr++) {
+		tot+= sprintf(str+tot, "%s %%x%d|", lib->name+2, nr);
+	}
+	return str;
+}
+
 void drawfilespace(ScrArea *sa, void *spacedata)
 {
 	SpaceFile *sfile;
@@ -1084,7 +1154,7 @@ void drawfilespace(ScrArea *sa, void *spacedata)
 	int act, loadbutton;
 	short mval[2];
 	char name[20];
-	char *menu;
+	char *menu, *strp= NULL;
 
 	myortho2(-0.375, sa->winx-0.375, -0.375, sa->winy-0.375);
 
@@ -1106,11 +1176,16 @@ void drawfilespace(ScrArea *sa, void *spacedata)
 	}
 	else calc_file_rcts(sfile);
 
+	/* check if we load library, extra button */
+	if(sfile->type==FILE_LOADLIB)
+		strp= library_string();
+	
 	/* HEADER */
 	sprintf(name, "win %d", sa->win);
 	block= uiNewBlock(&sa->uiblocks, name, UI_EMBOSS, UI_HELV, sa->win);
 	
-	uiSetButLock( sfile->type==FILE_MAIN && sfile->returnfunc, NULL);
+	/* browse 1 datablock */
+	uiSetButLock( sfile->type==FILE_MAIN && filesel_has_func(sfile), NULL);
 
 	/* space available for load/save buttons? */
 	loadbutton= MAX2(80, 20+BMF_GetStringWidth(G.font, sfile->title));
@@ -1120,28 +1195,33 @@ void drawfilespace(ScrArea *sa, void *spacedata)
 	else loadbutton= 0;
 
 	uiBlockBeginAlign(block);
-	uiDefBut(block, TEX,2,"",	textrct.xmin, filebuty2, textrct.xmax-textrct.xmin-loadbutton, 21, sfile->dir, 0.0, (float)FILE_MAXFILE-1, 0, 0, "Directory, enter a directory and press enter to create it"); /* Directory input */
+	uiDefBut(block, TEX, B_FS_DIRNAME,"",	textrct.xmin + (strp?20:0), filebuty2, textrct.xmax-textrct.xmin-loadbutton - (strp?20:0), 21, sfile->dir, 0.0, (float)FILE_MAXFILE-1, 0, 0, "Directory, enter a directory and press enter to create it"); /* Directory input */
 	if(loadbutton) {
 		uiSetCurFont(block, UI_HELV);
-		uiDefBut(block, BUT,	    5, sfile->title,	textrct.xmax-loadbutton, filebuty2, loadbutton, 21, sfile->dir, 0.0, (float)FILE_MAXFILE-1, 0, 0, "");
+		uiDefBut(block, BUT, B_FS_LOAD, sfile->title,	textrct.xmax-loadbutton, filebuty2, loadbutton, 21, sfile->dir, 0.0, (float)FILE_MAXFILE-1, 0, 0, "");
 	}
 	uiBlockEndAlign(block);
 	
 	uiBlockBeginAlign(block);
-	uiDefBut(block, TEX,1,"",	textrct.xmin, filebuty1, textrct.xmax-textrct.xmin-loadbutton, 21, sfile->file, 0.0, (float)FILE_MAXFILE-1, 0, 0, "File, increment version number with (+/-)"); /* File input */
+	uiDefBut(block, TEX, B_FS_FILENAME,"",	textrct.xmin, filebuty1, textrct.xmax-textrct.xmin-loadbutton, 21, sfile->file, 0.0, (float)FILE_MAXFILE-1, 0, 0, "File, increment version number with (+/-)"); /* File input */
 	if(loadbutton) {
 		uiSetCurFont(block, UI_HELV);
-		uiDefBut(block, BUT,	    6, "Cancel",	textrct.xmax-loadbutton, filebuty1, loadbutton, 21, sfile->file, 0.0, (float)FILE_MAXFILE-1, 0, 0, "");
+		uiDefBut(block, BUT, B_FS_CANCEL, "Cancel",	textrct.xmax-loadbutton, filebuty1, loadbutton, 21, sfile->file, 0.0, (float)FILE_MAXFILE-1, 0, 0, "");
 	}
 	uiBlockEndAlign(block);
 	
 	menu= fsmenu_build_menu();
-	if(menu[0])	// happens when no .Bfs is there, and first time browse
-		uiDefButS(block, MENU,	3, menu, scrollrct.xmin, filebuty1, scrollrct.xmax-scrollrct.xmin, 21, &sfile->menu, 0, 0, 0, 0, "");
+	if(menu[0])	/* happens when no .Bfs is there, and first time browse */
+		uiDefButS(block, MENU, B_FS_DIR_MENU, menu, scrollrct.xmin, filebuty1, scrollrct.xmax-scrollrct.xmin, 21, &sfile->menu, 0, 0, 0, 0, "");
 	MEM_freeN(menu);
 
-	uiDefBut(block, BUT,		4, "P", scrollrct.xmin, filebuty2, scrollrct.xmax-scrollrct.xmin, 21, 0, 0, 0, 0, 0, "Move to the parent directory (PKEY)");
-
+	uiBlockBeginAlign(block);
+	uiDefBut(block, BUT, B_FS_PARDIR, "P", scrollrct.xmin, filebuty2, scrollrct.xmax-scrollrct.xmin, 21, 0, 0, 0, 0, 0, "Move to the parent directory (PKEY)");
+	if(strp) {
+		uiDefIconTextButS(block, MENU, B_FS_LIBNAME, ICON_LIBRARY_DEHLT, strp, scrollrct.xmin+20, filebuty2, scrollrct.xmax-scrollrct.xmin, 21, &sfile->menu, 0, 0, 0, 0, "");
+		MEM_freeN(strp);
+	}
+			 
 	uiDrawBlock(block);
 
 	draw_filescroll(sfile);
@@ -1156,6 +1236,7 @@ void drawfilespace(ScrArea *sa, void *spacedata)
 	sa->win_swap= WIN_BACK_OK;
 }
 
+
 static void do_filescroll(SpaceFile *sfile)
 {
 	short mval[2], oldy, yo;
@@ -1223,10 +1304,15 @@ static void do_filescrollwheel(SpaceFile *sfile, int move)
 	}
 }
 
-void activate_fileselect(int type, char *title, char *file, void (*func)(char *))
+/* the complete call; pulldown menu, and three callback types */
+static void activate_fileselect_(int type, char *title, char *file, short *menup, char *pupmenu,
+										 void (*func)(char *),
+										 void (*func_event)(unsigned short),
+										 void (*func_args)(char *, void *arg1, void *arg2),
+										 void *arg1, void *arg2)
 {
 	SpaceFile *sfile;
-	char group[24], name[FILE_MAXDIR+FILE_MAXFILE], temp[FILE_MAXDIR+FILE_MAXFILE];
+	char group[GROUP_MAX], name[FILE_MAX], temp[FILE_MAX];
 	
 	if(curarea==0) return;
 	if(curarea->win==0) return;
@@ -1241,13 +1327,24 @@ void activate_fileselect(int type, char *title, char *file, void (*func)(char *)
 	BLI_strncpy(name, file, sizeof(name));
 	
 	sfile= curarea->spacedata.first;
-	/* sfile wants a (*)(short), but get (*)(char*) */
+
 	sfile->returnfunc= func;
+	sfile->returnfunc_event= func_event;
+	sfile->returnfunc_args= func_args;
+	sfile->arg1= arg1;
+	sfile->arg2= arg2;
+	
 	sfile->type= type;
 	sfile->ofs= 0;
+	
+	if(sfile->pupmenu)
+		MEM_freeN(sfile->pupmenu);
+	sfile->pupmenu= pupmenu;
+	sfile->menup= menup;
+	
 	/* sfile->act is used for databrowse: double names of library objects */
 	sfile->act= -1;
-	
+
 	if(BLI_convertstringcode(name, G.sce, G.scene->r.cfra)) sfile->flag |= FILE_STRINGCODE;
 	else sfile->flag &= ~FILE_STRINGCODE;
 
@@ -1295,12 +1392,28 @@ void activate_fileselect(int type, char *title, char *file, void (*func)(char *)
 	filetoname= 1;
 }
 
+void activate_fileselect(int type, char *title, char *file, void (*func)(char *))
+{
+	activate_fileselect_(type, title, file, NULL, NULL, func, NULL, NULL, NULL, NULL);
+}
+
+void activate_fileselect_menu(int type, char *title, char *file, char *pupmenu, short *menup, void (*func)(char *))
+{
+	activate_fileselect_(type, title, file, menup, pupmenu, func, NULL, NULL, NULL, NULL);
+}
+
+void activate_fileselect_args(int type, char *title, char *file, void (*func)(char *, void *, void *), void *arg1, void *arg2)
+{
+	activate_fileselect_(type, title, file, NULL, NULL, NULL, NULL, func, arg1, arg2);
+}
+
+
 void activate_imageselect(int type, char *title, char *file, void (*func)(char *))
 {
 	SpaceImaSel *simasel;
-	char dir[FILE_MAXDIR+FILE_MAXFILE], name[FILE_MAXDIR+FILE_MAXFILE];
+	char dir[FILE_MAX], name[FILE_MAX];
 	
-	if(curarea==0) return;
+	if(curarea==NULL) return;
 	if(curarea->win==0) return;
 	
 	newspace(curarea, SPACE_IMASEL);
@@ -1343,19 +1456,45 @@ void activate_databrowse(ID *id, int idcode, int fromcode, int retval, short *me
 	if(id) BLI_strncpy(str, id->name, sizeof(str));
 	else return;
 	
-	activate_fileselect(FILE_MAIN, "SELECT DATABLOCK", str, (void (*) (char*))func);
+	activate_fileselect_(FILE_MAIN, "SELECT DATABLOCK", str, menup, NULL, NULL, func, NULL, NULL, NULL);
 	
 	sfile= curarea->spacedata.first;
 	sfile->retval= retval;
 	sfile->ipotype= fromcode;
-	sfile->menup= menup;
+}
+
+void activate_databrowse_args(struct ID *id, int idcode, int fromcode, short *menup, void (*func)(char *, void *, void *), void *arg1, void *arg2)
+{
+	ListBase *lb;
+	SpaceFile *sfile;
+	char str[32];
+	
+	if(id==NULL) {
+		lb= wich_libbase(G.main, idcode);
+		id= lb->first;
+	}
+	
+	if(id) BLI_strncpy(str, id->name, sizeof(str));
+	else return;
+	
+	activate_fileselect_(FILE_MAIN, "SELECT DATABLOCK", str, menup, NULL, NULL, NULL, func, arg1, arg2);
+	
+	sfile= curarea->spacedata.first;
+	sfile->ipotype= fromcode;
 }
 
 void filesel_prevspace()
 {
-	SpaceFile *sfile;
+	SpaceFile *sfile= curarea->spacedata.first;
 	
-	sfile= curarea->spacedata.first;
+	/* cleanup */
+	if(sfile->spacetype==SPACE_FILE) {
+		if(sfile->pupmenu) {
+			MEM_freeN(sfile->pupmenu);
+			sfile->pupmenu= NULL;
+		}
+	}
+
 	if(sfile->next) {
 	
 		BLI_remlink(&curarea->spacedata, sfile);
@@ -1450,21 +1589,28 @@ void free_filesel_spec(char *dir)
 	}
 }
 
-
+/* NOTE: this is called for file read, after the execfunc no UI memory is valid! */
 static void filesel_execute(SpaceFile *sfile)
 {
 	struct direntry *files;
-	char name[FILE_MAXDIR+FILE_MAXFILE];
+	char name[FILE_MAX];
 	int a;
 	
 	filesel_prevspace();
 
 	if(sfile->type==FILE_LOADLIB) {
+		if(sfile->flag & FILE_STRINGCODE) {
+			if (!G.relbase_valid) {
+				okee("You have to save the .blend file before using relative paths! Using absolute path instead.");
+				sfile->flag &= ~FILE_STRINGCODE;
+			}
+		}
+
 		do_library_append(sfile);
 		BIF_undo_push("Append from file");
 		allqueue(REDRAWALL, 1);
 	}
-	else if(sfile->returnfunc) {
+	else if(filesel_has_func(sfile)) {
 		fsmenu_insert_entry(sfile->dir, 1);
 	
 		if(sfile->type==FILE_MAIN) { /* DATABROWSE */
@@ -1509,7 +1655,10 @@ static void filesel_execute(SpaceFile *sfile)
 					}
 				}
 			}
-			sfile->returnfunc((char*) (long)sfile->retval);
+			if(sfile->returnfunc_event)
+				sfile->returnfunc_event(sfile->retval);
+			else if(sfile->returnfunc_args)
+				sfile->returnfunc_args(NULL, sfile->arg1, sfile->arg2);
 		}
 		else {
 			if(strncmp(sfile->title, "Save", 4)==0) free_filesel_spec(sfile->dir);
@@ -1518,18 +1667,24 @@ static void filesel_execute(SpaceFile *sfile)
 			BLI_strncpy(name, sfile->dir, sizeof(name));
 			strcat(name, sfile->file);
 			
-			if(sfile->flag & FILE_STRINGCODE) BLI_makestringcode(G.sce, name);
-
-			sfile->returnfunc(name);
+			if(sfile->flag & FILE_STRINGCODE) {
+				/* still weak, but we don't want saving .blends to make relative paths */
+				if(sfile->returnfunc!=BIF_write_file)
+					BLI_makestringcode(G.sce, name);
+			}
+			if(sfile->returnfunc)
+				sfile->returnfunc(name);
+			else if(sfile->returnfunc_args)
+				sfile->returnfunc_args(name, sfile->arg1, sfile->arg2);
 		}
 	}
 }
 
 static void do_filesel_buttons(short event, SpaceFile *sfile)
 {
-	char butname[FILE_MAXDIR+FILE_MAXFILE];
+	char butname[FILE_MAX];
 	
-	if (event == 1) {
+	if (event == B_FS_FILENAME) {
 		if (strchr(sfile->file, '*') || strchr(sfile->file, '?') || strchr(sfile->file, '[')) {
 			int i, match = FALSE;
 			
@@ -1544,11 +1699,13 @@ static void do_filesel_buttons(short event, SpaceFile *sfile)
 			scrarea_queue_winredraw(curarea);
 		}
 	}
-	else if(event== 2) {
+	else if(event== B_FS_DIRNAME) {
 		/* reuse the butname variable */
 		BLI_cleanup_dir(G.sce, sfile->dir);
 
 		BLI_make_file_string(G.sce, butname, sfile->dir, "");
+		BLI_strncpy(sfile->dir, butname, sizeof(sfile->dir));
+
 		/* strip the trailing slash if its a real dir */
 		if (strlen(butname)!=1)
 			butname[strlen(butname)-1]=0;
@@ -1565,7 +1722,7 @@ static void do_filesel_buttons(short event, SpaceFile *sfile)
 		sfile->ofs= 0;
 		scrarea_queue_winredraw(curarea);
 	}
-	else if(event== 3) {
+	else if(event== B_FS_DIR_MENU) {
 		char *selected= fsmenu_get_entry(sfile->menu-1);
 		
 		/* which string */
@@ -1581,11 +1738,26 @@ static void do_filesel_buttons(short event, SpaceFile *sfile)
 		sfile->act= -1;
 		
 	}
-	else if(event== 4) parent(sfile);
-	else if(event== 5) {
-		if(sfile->type) filesel_execute(sfile);
+	else if(event== B_FS_PARDIR) 
+		parent(sfile);
+	else if(event== B_FS_LOAD) {
+		if(sfile->type) 
+			filesel_execute(sfile);
+	}
+	else if(event== B_FS_CANCEL) 
+		filesel_prevspace();
+	else if(event== B_FS_LIBNAME) {
+		Library *lib= BLI_findlink(&G.main->library, sfile->menu);
+		if(lib) {
+			BLI_strncpy(sfile->dir, lib->filename, sizeof(sfile->dir));
+			BLI_make_exist(sfile->dir);
+			BLI_cleanup_dir(G.sce, sfile->dir);
+			freefilelist(sfile);
+			sfile->ofs= 0;
+			scrarea_queue_winredraw(curarea);
+			sfile->act= -1;
+		}
 	}
-	else if(event== 6) filesel_prevspace();
 	
 }
 
@@ -1650,19 +1822,20 @@ static void replace_image(ID *oldblock, ID *newblock) {
 	}
 
 	for (me= G.main->mesh.first; me; me= me->id.next) {
-		TFace *tfaces= me->tface;
+		int i, a;
+		MTFace *tface;
 
-		if (tfaces) {
-			int i;
+		for(i=0; i<me->fdata.totlayer; i++) {
+			if(me->fdata.layers[i].type == CD_MTFACE) {
+				tface= (MTFace*)me->fdata.layers[i].data;
 
-			for (i=0; i<me->totface; i++) {
-				TFace *tf= &tfaces[i];
-
-				if (tf->tpage == oldima) {
-						/* not change_id_link, tpage's aren't owners :(
-						 * see hack below.
-						 */
-					tf->tpage= newima;
+				for (a=0; a<me->totface; a++, tface++) {
+					if (tface->tpage == oldima) {
+							/* not change_id_link, tpage's aren't owners :(
+							 * see hack below.
+							 */
+						tface->tpage= newima;
+					}
 				}
 			}
 		}
@@ -1754,7 +1927,7 @@ static void fs_fake_users(SpaceFile *sfile)
 	int a;
 	
 	/* only for F4 DATABROWSE */
-	if(sfile->returnfunc) return;
+	if(filesel_has_func(sfile)) return;
 	
 	for(a=0; a<sfile->totfile; a++) {
 		if(sfile->filelist[a].flags & ACTIVE) {
@@ -1797,7 +1970,7 @@ void winqreadfilespace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 	SpaceFile *sfile;
 	int act, do_draw= 0, i, test, ret = 0;
 	short qual, mval[2];
-	char str[FILE_MAXDIR+FILE_MAXFILE+12];
+	char str[FILE_MAX+12];
 	
 	sfile= curarea->spacedata.first;
 	if(sfile==0) return;
@@ -2045,8 +2218,11 @@ void winqreadfilespace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 		case PKEY:
 			if(G.qual & LR_SHIFTKEY) {
 				extern char bprogname[];	/* usiblender.c */
-			
+#ifdef WIN32			
 				sprintf(str, "%s -a \"%s%s\"", bprogname, sfile->dir, sfile->file);
+#else
+				sprintf(str, "\"%s\" -a \"%s%s\"", bprogname, sfile->dir, sfile->file);
+#endif
 				system(str);
 			}
 			else 
@@ -2139,10 +2315,10 @@ void winqreadfilespace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 
 static int groupname_to_code(char *group)
 {
-	char buf[32];
+	char buf[GROUP_MAX];
 	char *lslash;
 	
-	BLI_strncpy(buf, group, 31);
+	BLI_strncpy(buf, group, GROUP_MAX);
 	lslash= BLI_last_slash(buf);
 	if (lslash)
 		lslash[0]= '\0';
@@ -2162,7 +2338,7 @@ static int is_a_library(SpaceFile *sfile, char *dir, char *group)
 	len= strlen(dir);
 	if(len<7) return 0;
 	if( dir[len-1] != '/' && dir[len-1] != '\\') return 0;
-		
+	
 	group[0]= 0;
 	dir[len-1]= 0;
 
@@ -2172,89 +2348,27 @@ static int is_a_library(SpaceFile *sfile, char *dir, char *group)
 	if(fd==0) return 0;
 	*fd= 0;
 	if(BLO_has_bfile_extension(fd+1)) {
-		*fd= '/';
+		/* the last part of the dir is a .blend file, no group follows */
+		*fd= '/'; /* put back the removed slash separating the dir and the .blend file name */
 	}
-	else {
-		strcpy(group, fd+1);
-			
+	else {		
+		char *gp = fd+1; // in case we have a .blend file, gp points to the group
+
 		/* Find the last slash */
 		fd= (strrchr(dir, '/')>strrchr(dir, '\\'))?strrchr(dir, '/'):strrchr(dir, '\\');
 		if (!fd || !BLO_has_bfile_extension(fd+1)) return 0;
+
+		/* now we know that we are in a blend file and it is safe to 
+		   assume that gp actually points to a group */
+		BLI_strncpy(group, gp, GROUP_MAX);
 	}
 	return 1;
 }
 
-/* orange hack... :) */
-static void do_sync_pose(Library *lib)
-{
-	Object *ob, *obt;
-	Base *base;
-	bArmature *arm;
-	
-	/* find the armature and the pose from library */
-	for(ob= G.main->object.first; ob; ob= ob->id.next)
-		if(ob->type==OB_ARMATURE && ob->id.lib==lib)
-			break;
-	
-	if(ob==NULL || ob->pose==NULL) {
-		error("No pose appended");
-		return;
-	}
-	
-	arm= ob->data;
-
-	/* for all visible objects in this scene */
-	for(base= G.scene->base.first; base; base= base->next) {
-		if((base->flag & SELECT)) {
-			obt= base->object;
-			if(obt->type==OB_ARMATURE && obt->pose && ob!=obt) {
-				char str[128];
-				
-				sprintf(str, "Replace Object %s", obt->id.name);
-				if(okee(str)) {
-					bPoseChannel *chan;
-					bArmature *oldarm= obt->data;
-					
-					/* link armature */
-					oldarm->id.us--;
-					obt->data= arm;
-					arm->id.us++;
-					
-					/* link pose */
-					free_pose_channels(obt->pose);
-					MEM_freeN(obt->pose);
-					copy_pose(&obt->pose, ob->pose, 1);
-					
-					/* relink */
-					ob->id.newid= &obt->id;
-					for (chan = obt->pose->chanbase.first; chan; chan=chan->next){
-						relink_constraints(&chan->constraints);
-					}
-					
-					obt->pose->flag |= POSE_RECALC;
-					obt->recalc |= OB_RECALC_DATA;
-				}
-			}
-		}
-	}
-
-	/* prevent saving in file, unlink from scene */
-	for(base= G.scene->base.first; base; base= base->next) {
-		if(base->object==ob)
-			break;
-	}
-	
-	if(base) {
-		free_and_unlink_base(base);
-	}
-
-	DAG_scene_sort(G.scene);	// for accidentally appended other objects
-}
-
 static void do_library_append(SpaceFile *sfile)
 {
 	Library *lib;
-	char dir[FILE_MAXDIR+FILE_MAXFILE], group[32];
+	char dir[FILE_MAX], group[GROUP_MAX];
 	
 	if ( is_a_library(sfile, dir, group)==0 ) {
 		error("Not a library");
@@ -2287,8 +2401,6 @@ static void do_library_append(SpaceFile *sfile)
 		}
 		
 		if(lib) {
-			if(sfile->flag & FILE_SYNCPOSE)
-				do_sync_pose(lib);
 			if((sfile->flag & FILE_LINK)==0) 
 				all_local(lib);
 		}
@@ -2303,8 +2415,8 @@ static void library_to_filelist(SpaceFile *sfile)
 {
 	LinkNode *l, *names;
 	int ok, i, nnames, idcode;
-	char filename[FILE_MAXDIR+FILE_MAXFILE];
-	char dir[FILE_MAXDIR+FILE_MAXFILE], group[24];
+	char filename[FILE_MAX];
+	char dir[FILE_MAX], group[GROUP_MAX];
 	
 	/* name test */
 	ok= is_a_library(sfile, dir, group);
@@ -2379,7 +2491,7 @@ static void filesel_select_objects(SpaceFile *sfile)
 	int a;
 	
 	/* only when F4 DATABROWSE */
-	if(sfile->returnfunc) return;
+	if(filesel_has_func(sfile)) return;
 	
 	if( strcmp(sfile->dir, "Object/")==0 ) {
 		for(a=0; a<sfile->totfile; a++) {
@@ -2421,7 +2533,7 @@ static void active_file_object(SpaceFile *sfile)
 	Object *ob;
 	
 	/* only when F4 DATABROWSE */
-	if(sfile->returnfunc) return;
+	if(filesel_has_func(sfile)) return;
 	
 	if( strcmp(sfile->dir, "Object/")==0 ) {
 		if(sfile->act >= 0) {
@@ -2464,7 +2576,7 @@ void main_to_filelist(SpaceFile *sfile)
 	if( sfile->dir[0]==0) {
 		
 		/* make directories */
-		sfile->totfile= 23;
+		sfile->totfile= 24;
 		sfile->filelist= (struct direntry *)malloc(sfile->totfile * sizeof(struct direntry));
 		
 		for(a=0; a<sfile->totfile; a++) {
@@ -2495,6 +2607,7 @@ void main_to_filelist(SpaceFile *sfile)
 		sfile->filelist[20].relname= BLI_strdup("Armature");
 		sfile->filelist[21].relname= BLI_strdup("Action");
 		sfile->filelist[22].relname= BLI_strdup("NodeTree");
+		sfile->filelist[23].relname= BLI_strdup("Brush");
 		
 		qsort(sfile->filelist, sfile->totfile, sizeof(struct direntry), compare_name);
 	}
@@ -2509,7 +2622,7 @@ void main_to_filelist(SpaceFile *sfile)
 		id= lb->first;
 		sfile->totfile= 0;
 		while(id) {
-			if(sfile->returnfunc && idcode==ID_IP) {
+			if(filesel_has_func(sfile) && idcode==ID_IP) {
 				if(sfile->ipotype== ((Ipo *)id)->blocktype) sfile->totfile++;
 			}
 			else if (hide==0 || id->name[2] != '.')
@@ -2518,12 +2631,12 @@ void main_to_filelist(SpaceFile *sfile)
 			id= id->next;
 		}
 		
-		if(sfile->returnfunc==0) sfile->totfile+= 2;
+		if(!filesel_has_func(sfile)) sfile->totfile+= 2;
 		sfile->filelist= (struct direntry *)malloc(sfile->totfile * sizeof(struct direntry));
 		
 		files= sfile->filelist;
 		
-		if(sfile->returnfunc==0) {
+		if(!filesel_has_func(sfile)) {
 			memset( &(sfile->filelist[0]), 0 , sizeof(struct direntry));
 			sfile->filelist[0].relname= BLI_strdup(".");
 			sfile->filelist[0].type |= S_IFDIR;
@@ -2540,7 +2653,7 @@ void main_to_filelist(SpaceFile *sfile)
 		while(id) {
 			
 			ok= 0;
-			if(sfile->returnfunc && idcode==ID_IP) {
+			if(filesel_has_func(sfile) && idcode==ID_IP) {
 				if(sfile->ipotype== ((Ipo *)id)->blocktype) ok= 1;
 			}
 			else ok= 1;
@@ -2552,14 +2665,14 @@ void main_to_filelist(SpaceFile *sfile)
 					if(id->lib==NULL)
 						files->relname= BLI_strdup(id->name+2);
 					else {
-						char tmp[FILE_MAXDIR+FILE_MAXFILE], fi[FILE_MAXFILE];
-						BLI_strncpy(tmp, id->lib->name, FILE_MAXDIR+FILE_MAXFILE);
+						char tmp[FILE_MAX], fi[FILE_MAXFILE];
+						BLI_strncpy(tmp, id->lib->name, FILE_MAX);
 						BLI_splitdirstring(tmp, fi);
 						files->relname= MEM_mallocN(FILE_MAXFILE+32, "filename for lib");
 						sprintf(files->relname, "%s / %s", fi, id->name+2);
 					}
 					
-					if(sfile->returnfunc==0) { /* F4 DATA BROWSE */
+					if(!filesel_has_func(sfile)) { /* F4 DATA BROWSE */
 						if(idcode==ID_OB) {
 							if( ((Object *)id)->flag & SELECT) files->flags |= ACTIVE;
 						}
@@ -2604,7 +2717,7 @@ void main_to_filelist(SpaceFile *sfile)
 			if( strcmp(sfile->file, sfile->filelist[a].relname)==0) {
 				sfile->ofs= a-( sfile->collums*(curarea->winy-FILESELHEAD-10)/(2*FILESEL_DY));
 				filetoname= 0;
-				if(sfile->returnfunc) sfile->filelist[a].flags |= ACTIVE;
+				if(filesel_has_func(sfile)) sfile->filelist[a].flags |= ACTIVE;
 			}
 		}
 	}
@@ -2614,9 +2727,9 @@ void main_to_filelist(SpaceFile *sfile)
 void clever_numbuts_filesel()
 {
 	SpaceFile *sfile;
-	char orgname[FILE_MAXDIR+FILE_MAXFILE+12];
-	char filename[FILE_MAXDIR+FILE_MAXFILE+12];
-	char newname[FILE_MAXDIR+FILE_MAXFILE+12];
+	char orgname[FILE_MAX+12];
+	char filename[FILE_MAX+12];
+	char newname[FILE_MAX+12];
 	int test;
 	int len;
 	
