@@ -1,5 +1,5 @@
 /**
- * $Id: ghostwinlay.c,v 1.44 2006/06/18 11:38:34 ton Exp $
+ * $Id: ghostwinlay.c,v 1.45 2006/08/03 12:25:32 broken Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -714,6 +714,11 @@ void window_get_position(Window *win, int *posx_r, int *posy_r) {
 	*posy_r= win->position[1];
 }
 
+const GHOST_TabletData* window_get_tablet_data(Window *win)
+{
+	return GHOST_GetTabletData(win->ghostwin);
+}
+
 void window_get_size(Window *win, int *width_r, int *height_r) {
 	*width_r= win->size[0];
 	*height_r= win->size[1];
