@@ -1,7 +1,7 @@
 
 
 /**
- * $Id: glutil.c,v 1.23 2006/06/17 17:07:49 ton Exp $
+ * $Id: glutil.c,v 1.29 2007/01/17 03:57:01 nicholasbishop Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -32,7 +32,9 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
 
+#include <stdio.h>
 #include <math.h>
+#include <string.h>
 
 #include "MEM_guardedalloc.h"
 
@@ -46,6 +48,7 @@
 
 #include "BIF_gl.h"
 #include "BIF_glutil.h"
+#include "BIF_mywindow.h"
 
 	/* Invert line handling */
 	
@@ -135,6 +138,20 @@ void sdrawXORline4(int nr, int x0, int y0, int x1, int y1)
 	set_inverted_drawing(0);
 }
 
+void fdrawXORellipse(float xofs, float yofs, float hw, float hh)
+{
+	if(hw==0) return;
+
+	set_inverted_drawing(1);
+
+	glPushMatrix();
+	glTranslatef(xofs, yofs, 0.0);
+	glScalef(1,hh/hw,1);
+	glutil_draw_lined_arc(0.0, M_PI*2.0, hw, 20);
+	glPopMatrix();
+
+	set_inverted_drawing(0);
+}
 void fdrawXORcirc(float xofs, float yofs, float rad)
 {
 	set_inverted_drawing(1);
@@ -221,7 +238,6 @@ static int get_cached_work_texture(int *w_r, int *h_r)
 
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 
 		tbuf= MEM_callocN(tex_w*tex_h*4, "tbuf");
 		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, tex_w, tex_h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tbuf);
@@ -247,6 +263,7 @@ void glaDrawPixelsTex(float x, float y, int img_w, int img_h, int format, void *
 	int nsubparts_x= (img_w+(tex_w-1))/tex_w;
 	int nsubparts_y= (img_h+(tex_h-1))/tex_h;
 
+	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 	glPixelStorei(GL_UNPACK_ROW_LENGTH, img_w);
 	glBindTexture(GL_TEXTURE_2D, texid);
 
@@ -283,6 +300,7 @@ void glaDrawPixelsTex(float x, float y, int img_w, int img_h, int format, void *
 
 	glBindTexture(GL_TEXTURE_2D, ltexid);
 	glPixelStorei(GL_UNPACK_ROW_LENGTH, lrowlength);
+	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 }
 
 #define FTOCHAR(val) val<=0.0f?0: (val>=1.0f?255: (char)(255.0f*val))
@@ -295,8 +313,7 @@ void glaDrawPixelsSafe_to32(float fx, float fy, int img_w, int img_h, int row_w,
 	/* copy imgw-imgh to a temporal 32 bits rect */
 	if(img_w<1 || img_h<1) return;
 	
-	/* happens during threaded render... */
-	rc= rect32= MEM_mallocT(img_w*img_h*sizeof(int), "temp 32 bits");
+	rc= rect32= MEM_mallocN(img_w*img_h*sizeof(int), "temp 32 bits");
 	
 	for(y=0; y<img_h; y++) {
 		rf= rectf;
@@ -310,8 +327,8 @@ void glaDrawPixelsSafe_to32(float fx, float fy, int img_w, int img_h, int row_w,
 	}
 	
 	glaDrawPixelsSafe(fx, fy, img_w, img_h, img_w, GL_RGBA, GL_UNSIGNED_BYTE, rect32);
-	
-	MEM_freeT(rect32);
+
+	MEM_freeN(rect32);
 }
 
 void glaDrawPixelsSafe(float x, float y, int img_w, int img_h, int row_w, int format, int type, void *rect)
@@ -579,6 +596,29 @@ void bglEnd(void)
 	
 }
 
+/* Uses current OpenGL state to get view matrices for gluProject/gluUnProject */
+void bgl_get_mats(bglMats *mats)
+{
+	const double badvalue= 1.0e-6;
+
+	glGetDoublev(GL_MODELVIEW_MATRIX, mats->modelview);
+	glGetDoublev(GL_PROJECTION_MATRIX, mats->projection);
+	glGetIntegerv(GL_VIEWPORT, (GLint *)mats->viewport);
+	
+	/* Very strange code here - it seems that certain bad values in the
+	   modelview matrix can cause gluUnProject to give bad results. */
+	if(mats->modelview[0] < badvalue &&
+	   mats->modelview[0] > -badvalue)
+		mats->modelview[0]= 0;
+	if(mats->modelview[5] < badvalue &&
+	   mats->modelview[5] > -badvalue)
+		mats->modelview[5]= 0;
+	
+	/* Set up viewport so that gluUnProject will give correct values */
+	mats->viewport[0] = 0;
+	mats->viewport[1] = 0;
+}
+
 /* *************** glPolygonOffset hack ************* */
 
 // both temporal, so here for now (ton)
@@ -621,6 +661,24 @@ void bglPolygonOffset(float dist)
 	}
 }
 
+static int is_a_really_crappy_intel_card(void)
+{
+	static int well_is_it= -1;
 
+		/* Do you understand the implication? Do you? */
+	if (well_is_it==-1)
+		well_is_it= (strcmp((char*) glGetString(GL_VENDOR), "Intel Inc.") == 0);
+
+	return well_is_it;
+}
+
+void bglFlush(void) 
+{
+	glFlush();
+#ifdef __APPLE__
+	if(is_a_really_crappy_intel_card())
+		myswapbuffers(); //hack to get mac intel graphics to show frontbuffer
+#endif
+}
 
 
