@@ -4,7 +4,7 @@
  * Functions to draw the "Buttons Window" window header
  * and handle user events sent to it.
  * 
- * $Id: header_buttonswin.c,v 1.42 2006/07/22 17:11:36 ton Exp $
+ * $Id: header_buttonswin.c,v 1.44 2006/11/06 01:08:24 nicholasbishop Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -44,6 +44,7 @@
 
 #include "DNA_ID.h"
 #include "DNA_armature_types.h"
+#include "DNA_brush_types.h"
 #include "DNA_lamp_types.h"
 #include "DNA_material_types.h"
 #include "DNA_node_types.h"
@@ -321,6 +322,20 @@ void buttons_active_id(ID **id, ID **idfrom)
 					if(mtex) *id= (ID *)mtex->tex;
 				}
 			}
+			else if(G.buts->texfrom==3) {
+				if(G.f & G_SCULPTMODE) {
+					if(G.scene->sculptdata.texact != -1) {
+						mtex= G.scene->sculptdata.mtex[G.scene->sculptdata.texact];
+						if(mtex) *id= (ID*)mtex->tex;
+					}
+				} else {
+					Brush *brush= G.scene->toolsettings->imapaint.brush;
+					if (brush) {
+						mtex= brush->mtex[brush->texact];
+						if(mtex) *id= (ID*)mtex->tex;
+					}
+				}
+			}
 		}
 	}
 	else if(G.buts->mainb==CONTEXT_OBJECT || G.buts->mainb==CONTEXT_LOGIC) {
