@@ -4,7 +4,7 @@
  * Functions to draw the "UV/Image Editor" window header
  * and handle user events sent to it.
  * 
- * $Id: header_image.c,v 1.60 2006/07/03 11:21:38 ton Exp $
+ * $Id: header_image.c,v 1.70 2006/12/23 11:22:20 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -42,189 +42,67 @@
 #include <config.h>
 #endif
 
+#include "DNA_brush_types.h"
 #include "DNA_ID.h"
 #include "DNA_image_types.h"
-#include "DNA_mesh_types.h"
-#include "DNA_object_types.h"
-#include "DNA_packedFile_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_space_types.h"
+#include "DNA_texture_types.h"
 #include "DNA_userdef_types.h"
 
+#include "BLI_blenlib.h"
+
 #include "BDR_drawmesh.h"
 #include "BDR_unwrapper.h"
+
+#include "BKE_brush.h"
+#include "BKE_colortools.h"
 #include "BKE_global.h"
 #include "BKE_image.h"
-#include "BKE_library.h"
 #include "BKE_main.h"
-#include "BKE_packedFile.h"
 #include "BKE_utildefines.h"
-#include "BKE_depsgraph.h"
 
-#include "BLI_blenlib.h"
+#include "BIF_butspace.h"
 #include "BIF_drawimage.h"
 #include "BIF_editsima.h"
 #include "BIF_interface.h"
-#include "BIF_previewrender.h"
 #include "BIF_resources.h"
 #include "BIF_screen.h"
 #include "BIF_space.h"
-#include "BIF_toets.h"
-#include "BIF_toolbox.h"
 #include "BIF_transform.h"
-#include "BIF_writeimage.h"
+#include "BIF_toolbox.h"
 
+#include "BSE_drawview.h"
 #include "BSE_filesel.h"
 #include "BSE_headerbuttons.h"
+#include "BSE_trans_types.h"
 
 #include "BPY_extern.h"
 #include "BPY_menus.h"
 
-#include "IMB_imbuf.h"
 #include "IMB_imbuf_types.h"
-#include "BSE_trans_types.h"
+
+#include "RE_pipeline.h"
 
 #include "blendef.h"
+#include "butspace.h"
 #include "mydevice.h"
 
-static void load_space_image(char *str)	/* called from fileselect */
-{
-	Image *ima=0;
- 
-	if(G.obedit) {
-		error("Can't perfom this in editmode");
-		return;
-	}
-
-	ima= add_image(str);
-	if(ima) {
-
-		G.sima->image= ima;
-
-		free_image_buffers(ima);	/* force read again */
-		ima->ok= 1;
-		image_changed(G.sima, 0);
-
-	}
-	BIF_undo_push("Load image UV");
-	allqueue(REDRAWIMAGE, 0);
-}
-
-static void image_replace(Image *old, Image *new)
-{
-	TFace *tface;
-	Mesh *me;
-	int a, rep=0;
-
-	new->tpageflag= old->tpageflag;
-	new->twsta= old->twsta;
-	new->twend= old->twend;
-	new->xrep= old->xrep;
-	new->yrep= old->yrep;
- 
-	me= G.main->mesh.first;
-	while(me) {
-
-		if(me->tface) {
-			tface= me->tface;
-			a= me->totface;
-			while(a--) {
-				if(tface->tpage==old) {
-					tface->tpage= new;
-					rep++;
-				}
-				tface++;
-			}
-		}
-		me= me->id.next;
- 
-	}
-	if(rep) {
-		if(new->id.us==0) new->id.us= 1;
-	}
-	else error("Nothing replaced");
-}
-
-static void replace_space_image(char *str)		/* called from fileselect */
-{
-	Image *ima=0;
-
-	if(G.obedit) {
-		error("Can't perfom this in editmode");
-		return;
-	}
-
-	ima= add_image(str);
-	if(ima) {
- 
-		if(G.sima->image && G.sima->image != ima) {
-			image_replace(G.sima->image, ima);
-		}
- 
-		G.sima->image= ima;
-
-		free_image_buffers(ima);	/* force read again */
-		ima->ok= 1;
-		/* replace also assigns: */
-		image_changed(G.sima, 0);
-
-	}
-	BIF_undo_push("Replace image UV");
-	allqueue(REDRAWIMAGE, 0);
-}
-
-static void save_paint(char *name)
-{
-	Image *ima = G.sima->image;
-	int len;
-	char str[FILE_MAXDIR+FILE_MAXFILE];
-
-	if (ima  && ima->ibuf) {
-		BLI_strncpy(str, name, sizeof(str));
-
-		BLI_convertstringcode(str, G.sce, G.scene->r.cfra);
-		
-		if(G.scene->r.scemode & R_EXTENSION) 
-			BKE_add_image_extension(str, G.scene->r.imtype);
-
-		if (saveover(str)) {
-			
-			/* enforce user setting for RGB or RGBA, but skip BW */
-			if(G.scene->r.planes==32)
-				ima->ibuf->depth= 32;
-			else if(G.scene->r.planes==24)
-				ima->ibuf->depth= 24;
-			
-			waitcursor(1);
-			if (BKE_write_ibuf(ima->ibuf, str, G.scene->r.imtype, G.scene->r.subimtype, G.scene->r.quality)) {
-				BLI_strncpy(ima->name, name, sizeof(ima->name));
-				ima->ibuf->userflags &= ~IB_BITMAPDIRTY;
-				allqueue(REDRAWHEADERS, 0);
-				allqueue(REDRAWBUTSSHADING, 0);
-			} else {
-				error("Couldn't write image: %s", str);
-			}
-			
-			/* name image as how we saved it */
-			len= strlen(str);
-			while (len > 0 && str[len - 1] != '/' && str[len - 1] != '\\') len--;
-			rename_id(&ima->id, str+len);
-
-			waitcursor(0);
-		}
-	}
-}
-
 void do_image_buttons(unsigned short event)
 {
-	Image *ima;
+	ToolSettings *settings= G.scene->toolsettings;
 	ID *id, *idtest;
 	int nr;
-	char name[256];
 
 	if(curarea->win==0) return;
 
+	if(event<=100) {
+		if(event<=50) do_global_buttons2(event);
+		else do_global_buttons(event);
+		return;
+	}
+	
 	switch(event) {
 	case B_SIMAGEHOME:
 		image_home();
@@ -241,21 +119,15 @@ void do_image_buttons(unsigned short event)
 		nr= 1;
 		id= (ID *)G.sima->image;
 
-		idtest= G.main->image.first;
-		while(idtest) {
-			if(nr==G.sima->imanr) {
-				break;
-			}
-			nr++;
-			idtest= idtest->next;
-		}
-		if(idtest==0) { /* no new */
+		idtest= BLI_findlink(&G.main->image, G.sima->imanr-1);
+		if(idtest==NULL) { /* no new */
 			return;
 		}
 	
 		if(idtest!=id) {
 			G.sima->image= (Image *)idtest;
 			if(idtest->us==0) idtest->us= 1;
+			BKE_image_signal(G.sima->image, &G.sima->iuser, IMA_SIGNAL_USER_NEW_IMAGE);
 			allqueue(REDRAWIMAGE, 0);
 		}
 		/* also when image is the same: assign! 0==no tileflag: */
@@ -263,102 +135,44 @@ void do_image_buttons(unsigned short event)
 		BIF_undo_push("Assign image UV");
 
 		break;
-	case B_SIMAGELOAD:
-		
-		if(G.sima->image) strcpy(name, G.sima->image->name);
-		else strcpy(name, U.textudir);
-		
-		if(G.qual==LR_CTRLKEY)
-			activate_imageselect(FILE_SPECIAL, "SELECT IMAGE", name, load_space_image);
-		else
-			activate_fileselect(FILE_SPECIAL, "SELECT IMAGE", name, load_space_image);
-		break;
-		
-	case B_SIMAGEREPLACE:
-		
-		if(G.sima->image) strcpy(name, G.sima->image->name);
-		else strcpy(name, U.textudir);
-		
-		if(G.qual==LR_CTRLKEY)
-			activate_imageselect(FILE_SPECIAL, "REPLACE IMAGE", name, replace_space_image);
-		else
-			activate_fileselect(FILE_SPECIAL, "REPLACE IMAGE", name, replace_space_image);
-		break;
 		
 	case B_SIMAGEDRAW:
-		
 		if(G.f & G_FACESELECT) {
 			make_repbind(G.sima->image);
 			image_changed(G.sima, 1);
 		}
+		/* XXX might be another event needed for this? */
+		if(G.sima->image)
+			if(ELEM(G.sima->image->source, IMA_SRC_MOVIE, IMA_SRC_SEQUENCE))
+				if(G.sima->iuser.flag & IMA_ANIM_ALWAYS)
+					BKE_image_user_calc_imanr(&G.sima->iuser, G.scene->r.cfra, 0);
 		allqueue(REDRAWVIEW3D, 0);
 		allqueue(REDRAWIMAGE, 0);
 		break;
-
+		
 	case B_SIMAGEDRAW1:
 		image_changed(G.sima, 2);		/* 2: only tileflag */
 		allqueue(REDRAWVIEW3D, 0);
 		allqueue(REDRAWIMAGE, 0);
 		break;
 		
-	case B_TWINANIM:
-		ima = G.sima->image;
-		if (ima) {
-			if(ima->flag & IMA_TWINANIM) {
-				nr= ima->xrep*ima->yrep;
-				if(ima->twsta>=nr) ima->twsta= 1;
-				if(ima->twend>=nr) ima->twend= nr-1;
-				if(ima->twsta>ima->twend) ima->twsta= 1;
-				allqueue(REDRAWIMAGE, 0);
-			}
-		}
-		break;
-
 	case B_SIMAGEPAINTTOOL:
-		// check for packed file here
+		if(G.sima->flag & SI_DRAWTOOL)
+			/* add new brush if none exists */
+			brush_check_exists(&G.scene->toolsettings->imapaint.brush);
 		allqueue(REDRAWIMAGE, 0);
 		allqueue(REDRAWVIEW3D, 0);
 		break;
+
 	case B_SIMAPACKIMA:
-		ima = G.sima->image;
-		if (ima) {
-			if (ima->packedfile) {
-				if (G.fileflags & G_AUTOPACK) {
-					if (okee("Disable AutoPack ?")) {
-						G.fileflags &= ~G_AUTOPACK;
-					}
-				}
-				
-				if ((G.fileflags & G_AUTOPACK) == 0) {
-					unpackImage(ima, PF_ASK);
-				}
-			} else {
-				if (ima->ibuf && (ima->ibuf->userflags & IB_BITMAPDIRTY)) {
-					error("Can't pack painted image. Save image first.");
-				} else {
-					ima->packedfile = newPackedFile(ima->name);
-				}
-			}
-			allqueue(REDRAWBUTSSHADING, 0);
-			allqueue(REDRAWHEADERS, 0);
-		}
+		pack_image_sima();
 		break;
-	case B_SIMAGESAVE:
-		ima = G.sima->image;
-		if (ima) {
-			strcpy(name, ima->name);
-			if (ima->ibuf) {
-				char str[64];
-				save_image_filesel_str(str);
-				
-				/* so it shows extension in file window */
-				if(G.scene->r.scemode & R_EXTENSION) 
-					BKE_add_image_extension(name, G.scene->r.imtype);
-				
-				activate_fileselect(FILE_SPECIAL, str, name, save_paint);
-			}
-		}
+		
+	case B_SIMA_REPACK:
+		BKE_image_memorypack(G.sima->image);
+		allqueue(REDRAWIMAGE, 0);
 		break;
+		
 	case B_SIMA_USE_ALPHA:
 		G.sima->flag &= ~(SI_SHOW_ALPHA|SI_SHOW_ZBUF);
 		scrarea_queue_winredraw(curarea);
@@ -374,6 +188,157 @@ void do_image_buttons(unsigned short event)
 		scrarea_queue_winredraw(curarea);
 		scrarea_queue_headredraw(curarea);
 		break;
+	case B_SIMARELOAD:
+		reload_image_sima();
+		break;
+	case B_SIMAGELOAD:
+		open_image_sima(0);
+		break;
+	case B_SIMANAME:
+		if(G.sima->image) {
+			Image *ima;
+			char str[FILE_MAXDIR+FILE_MAXFILE];
+			
+			/* name in ima has been changed by button! */
+			BLI_strncpy(str, G.sima->image->name, sizeof(str));
+			ima= BKE_add_image_file(str);
+			if(ima) {
+				
+				G.sima->image= ima;
+				
+				BKE_image_signal(ima, &G.sima->iuser, IMA_SIGNAL_RELOAD);
+				image_changed(G.sima, 0);
+				
+			}
+			BIF_undo_push("Load image");
+			allqueue(REDRAWIMAGE, 0);
+		}
+		break;
+	case B_SIMAMULTI:
+		if(G.sima && G.sima->image) {
+			BKE_image_multilayer_index(G.sima->image->rr, &G.sima->iuser);
+			allqueue(REDRAWIMAGE, 0);
+		}
+		break;
+	case B_TRANS_IMAGE:
+		image_editvertex_buts(NULL);
+		break;
+		
+	case B_TWINANIM:
+	{
+		Image *ima;
+		int nr;
+		
+		ima = G.sima->image;
+		if (ima) {
+			if(ima->flag & IMA_TWINANIM) {
+				nr= ima->xrep*ima->yrep;
+				if(ima->twsta>=nr) ima->twsta= 1;
+				if(ima->twend>=nr) ima->twend= nr-1;
+				if(ima->twsta>ima->twend) ima->twsta= 1;
+				allqueue(REDRAWIMAGE, 0);
+			}
+		}
+		break;
+	}	
+	case B_SIMACLONEBROWSE:
+		if (settings->imapaint.brush)
+			if (brush_clone_image_set_nr(settings->imapaint.brush, G.sima->menunr))
+				allqueue(REDRAWIMAGE, 0);
+		break;
+		
+	case B_SIMACLONEDELETE:
+		if (settings->imapaint.brush)
+			if (brush_clone_image_delete(settings->imapaint.brush))
+				allqueue(REDRAWIMAGE, 0);
+		break;
+		
+	case B_SIMABRUSHCHANGE:
+		allqueue(REDRAWIMAGE, 0);
+		allqueue(REDRAWBUTSEDIT, 0);
+		break;
+		
+	case B_SIMACURVES:
+		curvemapping_do_ibuf(G.sima->cumap, imagewindow_get_ibuf(G.sima));
+		allqueue(REDRAWIMAGE, 0);
+		break;
+		
+	case B_SIMARANGE:
+		curvemapping_set_black_white(G.sima->cumap, NULL, NULL);
+		curvemapping_do_ibuf(G.sima->cumap, imagewindow_get_ibuf(G.sima));
+		allqueue(REDRAWIMAGE, 0);
+		break;
+		
+	case B_SIMABRUSHBROWSE:
+		if(G.sima->menunr==-2) {
+			activate_databrowse((ID*)settings->imapaint.brush, ID_BR, 0, B_SIMABRUSHBROWSE, &G.sima->menunr, do_global_buttons);
+			break;
+		}
+		else if(G.sima->menunr < 0) break;
+		
+		if(brush_set_nr(&settings->imapaint.brush, G.sima->menunr)) {
+			BIF_undo_push("Browse Brush");
+			allqueue(REDRAWBUTSEDIT, 0);
+			allqueue(REDRAWIMAGE, 0);
+		}
+		break;
+	case B_SIMABRUSHDELETE:
+		if(brush_delete(&settings->imapaint.brush)) {
+			BIF_undo_push("Unlink Brush");
+			allqueue(REDRAWIMAGE, 0);
+			allqueue(REDRAWBUTSEDIT, 0);
+		}
+		break;
+	case B_KEEPDATA:
+		brush_toggle_fake_user(settings->imapaint.brush);
+		allqueue(REDRAWIMAGE, 0);
+		allqueue(REDRAWBUTSEDIT, 0);
+		break;
+	case B_SIMABRUSHLOCAL:
+		if(settings->imapaint.brush && settings->imapaint.brush->id.lib) {
+			if(okee("Make local")) {
+				make_local_brush(settings->imapaint.brush);
+				allqueue(REDRAWIMAGE, 0);
+				allqueue(REDRAWBUTSEDIT, 0);
+			}
+		}
+		break;
+	case B_SIMABTEXBROWSE:
+		if(settings->imapaint.brush) {
+			Brush *brush= settings->imapaint.brush;
+			
+			if(G.sima->menunr==-2) {
+				MTex *mtex= brush->mtex[brush->texact];
+				ID *id= (ID*)((mtex)? mtex->tex: NULL);
+				activate_databrowse(id, ID_TE, 0, B_SIMABTEXBROWSE, &G.sima->menunr, do_global_buttons);
+				break;
+			}
+			else if(G.sima->menunr < 0) break;
+			
+			if(brush_texture_set_nr(brush, G.sima->menunr)) {
+				BIF_undo_push("Browse Brush Texture");
+				allqueue(REDRAWBUTSSHADING, 0);
+				allqueue(REDRAWBUTSEDIT, 0);
+				allqueue(REDRAWIMAGE, 0);
+			}
+		}
+		break;
+	case B_SIMABTEXDELETE:
+		if(settings->imapaint.brush) {
+			if (brush_texture_delete(settings->imapaint.brush)) {
+				BIF_undo_push("Unlink Brush Texture");
+				allqueue(REDRAWBUTSSHADING, 0);
+				allqueue(REDRAWBUTSEDIT, 0);
+				allqueue(REDRAWIMAGE, 0);
+			}
+		}
+		break;
+	case B_SIMA_PLAY:
+		play_anim(0);
+		break;
+	case B_SIMA_RECORD:
+		imagespace_composite_flipbook(curarea);
+		break;
 	}
 }
 
@@ -484,7 +449,11 @@ static void do_image_viewmenu(void *arg, int event)
 	case 12: /* composite preview */
 		toggle_blockhandler(curarea, IMAGE_HANDLER_PREVIEW, 0);
 		scrarea_queue_winredraw(curarea);
-
+		break;
+	case 13: /* Realtime Panel... */
+		add_blockhandler(curarea, IMAGE_HANDLER_GAME_PROPERTIES, UI_PNL_UNSTOW);
+		break;
+		
 	}
 	allqueue(REDRAWVIEW3D, 0);
 }
@@ -499,6 +468,7 @@ static uiBlock *image_viewmenu(void *arg_unused)
 	uiBlockSetButmFunc(block, do_image_viewmenu, NULL);
 
 	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Properties...",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Real-time Properties...",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 13, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Paint Tool...|C",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 8, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Curves Tool...",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 11, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Composite Preview...|Shift P",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 12, "");
@@ -644,12 +614,12 @@ static void do_image_image_rtmappingmenu(void *arg, int event)
 {
 	switch(event) {
 	case 0: /* UV Co-ordinates */
-		G.sima->image->flag = BCLR(G.sima->image->flag, 4);
+		G.sima->image->flag &= ~IMA_REFLECT;
 		break;
 	case 1: /* Reflection */
-		G.sima->image->flag = BSET(G.sima->image->flag, 4);
+		G.sima->image->flag |= IMA_REFLECT;
 		break;
-		}
+	}
 
  	allqueue(REDRAWVIEW3D, 0);
 }
@@ -677,160 +647,83 @@ static uiBlock *image_image_rtmappingmenu(void *arg_unused)
 
 static void do_image_imagemenu(void *arg, int event)
 {
-	Image *ima;
-	char name[256];
-
+	/* events >=20 are registered bpython scripts */
+	if (event >= 20) BPY_menu_do_python(PYMENU_IMAGE, event - 20);
+	
 	switch(event)
 	{
-	case 0: /* Open */
-		if(G.sima->image) strcpy(name, G.sima->image->name);
-		else strcpy(name, U.textudir);
-		if(G.qual==LR_CTRLKEY)
-			activate_imageselect(FILE_SPECIAL, "Open Image", name, load_space_image);
-		else
-			activate_fileselect(FILE_SPECIAL, "Open Image", name, load_space_image);
+	case 0:
+		open_image_sima((G.qual==LR_CTRLKEY));
 		break;
-	case 1: /* Replace */
-		if(G.sima->image) strcpy(name, G.sima->image->name);
-		else strcpy(name, U.textudir);
-		
-		if(G.qual==LR_CTRLKEY)
-			activate_imageselect(FILE_SPECIAL, "Replace Image", name, replace_space_image);
-		else
-			activate_fileselect(FILE_SPECIAL, "Replace Image", name, replace_space_image);
+	case 1:
+		replace_image_sima((G.qual==LR_CTRLKEY));
 		break;
-	case 2: /* Pack Image */
-		ima = G.sima->image;
-		if (ima) {
-			if (ima->packedfile) {
-				error("Image is already packed.");
-			} else {
-				if (ima->ibuf && (ima->ibuf->userflags & IB_BITMAPDIRTY)) {
-					error("Can't pack painted image. Save the painted image first.");
-				} else {
-					ima->packedfile = newPackedFile(ima->name);
-				}
-			}
-		}
-		BIF_undo_push("Pack image");
-		allqueue(REDRAWBUTSSHADING, 0);
-		allqueue(REDRAWHEADERS, 0);
-		break;
-	case 3: /* Unpack Image */
-		ima = G.sima->image;
-		if (ima) {
-			if (ima->packedfile) {
-				if (G.fileflags & G_AUTOPACK) {
-					if (okee("Disable AutoPack?")) {
-						G.fileflags &= ~G_AUTOPACK;
-					}
-				}
-				
-				if ((G.fileflags & G_AUTOPACK) == 0) {
-					unpackImage(ima, PF_ASK);
-				}
-			} else {
-				error("There are no packed images to unpack");
-			}
-		}
-		BIF_undo_push("Unpack image");
-		allqueue(REDRAWBUTSSHADING, 0);
-		allqueue(REDRAWHEADERS, 0);
+	case 2:
+		pack_image_sima();
 		break;
 	case 4: /* Texture Painting */
 		if(G.sima->flag & SI_DRAWTOOL) G.sima->flag &= ~SI_DRAWTOOL;
 		else G.sima->flag |= SI_DRAWTOOL;
 		break;
-	case 5: /* Save Painted Image */
-		ima = G.sima->image;
-		if (ima) {
-			strcpy(name, ima->name);
-			if (ima->ibuf) {
-				char str[64];
-				save_image_filesel_str(str);
-				
-				/* so it shows an extension in filewindow */
-				if(G.scene->r.scemode & R_EXTENSION) 
-					BKE_add_image_extension(name, G.scene->r.imtype);
-				
-				activate_fileselect(FILE_SPECIAL, str, name, save_paint);
-			}
-		}
+	case 5:
+		save_as_image_sima();
 		break;
-	case 6: /* Reload Image */
-		ima = G.sima->image;
-		if (ima) {
-			if (ima->packedfile) {
-				PackedFile *pf;
-				pf = newPackedFile(ima->name);
-				if (pf) {
-					freePackedFile(ima->packedfile);
-					ima->packedfile = pf;
-				}
-				else
-					error("Image not available. Keeping packed image.");
-			}
-			if (ima->preview) {
-				free_image_preview(ima);
-			}
-			free_image_buffers(ima);	/* force read again */
-			ima->ok= 1;
-			image_changed(G.sima, 0);
-		}
+	case 6:
+		reload_image_sima();
+		break;
+	case 7:
+		new_image_sima();
+		break;
+	case 8:
+		save_image_sima();
+		break;
+	case 9:
+		save_image_sequence_sima();
+		break;
+	case 10:
+		BKE_image_memorypack(G.sima->image);
 		allqueue(REDRAWIMAGE, 0);
-		allqueue(REDRAWVIEW3D, 0);
-		BIF_preview_changed(ID_TE);
 		break;
-	case 7: /* New Image */
-		{
-			static int width= 256, height= 256;
-			static short uvtestgrid=0;
-			char name[256];
-
-			strcpy(name, "Image");
-
-			add_numbut(0, TEX, "Name:", 0, 255, name, NULL);
-			add_numbut(1, NUM|INT, "Width:", 1, 5000, &width, NULL);
-			add_numbut(2, NUM|INT, "Height:", 1, 5000, &height, NULL);
-			add_numbut(3, TOG|SHO, "UV Test Grid", 0, 0, &uvtestgrid, NULL);
-			if (!do_clever_numbuts("New Image", 4, REDRAW))
-				return;
-
-			G.sima->image= new_image(width, height, name, uvtestgrid);
-			image_changed(G.sima, 0);
-
-			allqueue(REDRAWIMAGE, 0);
-			allqueue(REDRAWVIEW3D, 0);
-
-			break;
-		}
 	}
 }
 
 static uiBlock *image_imagemenu(void *arg_unused)
 {
+	ImBuf *ibuf= BKE_image_get_ibuf(G.sima->image, &G.sima->iuser);
 	uiBlock *block;
 	short yco= 0, menuwidth=150;
+	BPyMenu *pym;
+	int i = 0;
 
 	block= uiNewBlock(&curarea->uiblocks, "image_imagemenu", UI_EMBOSSP, UI_HELV, curarea->headwin);
 	uiBlockSetButmFunc(block, do_image_imagemenu, NULL);
 
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "New...", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Open...", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 0, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "New...|Alt N", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Open...|Alt O", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 0, "");
 	
 	if (G.sima->image) {
-		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Save As...", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
 		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Replace...", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 1, "");
-		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Reload", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 6, "");
+		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Reload|Alt R", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 6, "");
+		uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
 
+		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Save|Alt S", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 8, "");
+		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Save As...", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
+		if(G.sima->image->source==IMA_SRC_SEQUENCE)
+			uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Save Changed Images", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 9, "");
 		uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
 		
 		if (G.sima->image->packedfile) {
-			uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Unpack Image...", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 3, "");
+			uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Unpack Image...", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 2, "");
 		} else {
 			uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Pack Image", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 2, "");
 		}
-			
+		
+		/* only for dirty && specific image types */
+		if(ibuf && (ibuf->userflags & IB_BITMAPDIRTY))
+			if( ELEM(G.sima->image->source, IMA_SRC_FILE, IMA_SRC_GENERATED))
+				if(G.sima->image->type!=IMA_TYPE_MULTILAYER)
+					uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Pack Image as PNG", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 10, "");
+		
 		uiDefBut(block, SEPR, 0, "",        0, yco-=7, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 
 		if(G.sima->flag & SI_DRAWTOOL) {
@@ -845,6 +738,14 @@ static uiBlock *image_imagemenu(void *arg_unused)
 		// uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Realtime Texture Animation|",		0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
 	}
 	
+	/* note that we acount for the N previous entries with i+20: */
+	for (pym = BPyMenuTable[PYMENU_IMAGE]; pym; pym = pym->next, i++) {
+
+		uiDefIconTextBut(block, BUTM, 1, ICON_PYTHON, pym->name, 0, yco-=20, menuwidth, 19, 
+				 NULL, 0.0, 0.0, 1, i+20, 
+				 pym->tooltip?pym->tooltip:pym->filename);
+	}
+	
 	if(curarea->headertype==HEADERTOP) {
 		uiBlockSetDirection(block, UI_DOWN);
 	}
@@ -1023,13 +924,41 @@ static uiBlock *image_uvs_weldalignmenu(void *arg_unused)
 	return block;
 }
 
+static void do_image_uvs_scriptsmenu(void *arg, int event)
+{
+	BPY_menu_do_python(PYMENU_UV, event);
+
+	allqueue(REDRAWIMAGE, 0);
+}
+
+static uiBlock *image_uvs_scriptsmenu (void *args_unused)
+{
+	uiBlock *block;
+	BPyMenu *pym;
+	int i= 0;
+	short yco = 20, menuwidth = 120;
+	
+	block= uiNewBlock(&curarea->uiblocks, "image_uvs_scriptsmenu", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetButmFunc(block, do_image_uvs_scriptsmenu, NULL);
+	
+	/* note that we acount for the N previous entries with i+20: */
+	for (pym = BPyMenuTable[PYMENU_UV]; pym; pym = pym->next, i++) {
+		
+		uiDefIconTextBut(block, BUTM, 1, ICON_PYTHON, pym->name, 0, yco-=20, menuwidth, 19, 
+						 NULL, 0.0, 0.0, 1, i, 
+						 pym->tooltip?pym->tooltip:pym->filename);
+	}
+	
+	uiBlockSetDirection(block, UI_RIGHT);
+	uiTextBoundsBlock(block, 60);
+	
+	return block;
+}
+
 static void do_image_uvsmenu(void *arg, int event)
 {
-	/* events >=20 are registered bpython scripts */
-	if (event >= 20) BPY_menu_do_python(PYMENU_UV, event - 20);
 
-	else switch(event)
-	{
+	switch(event) {
 	case 1: /* UVs Constrained Rectangular */
 		if(G.sima->flag & SI_BE_SQUARE) G.sima->flag &= ~SI_BE_SQUARE;
 		else G.sima->flag |= SI_BE_SQUARE;
@@ -1076,8 +1005,6 @@ static uiBlock *image_uvsmenu(void *arg_unused)
 {
 	uiBlock *block;
 	short yco= 0, menuwidth=120;
-	BPyMenu *pym;
-	int i = 0;
 
 	block= uiNewBlock(&curarea->uiblocks, "image_uvsmenu", UI_EMBOSSP, UI_HELV, curarea->headwin);
 	uiBlockSetButmFunc(block, do_image_uvsmenu, NULL);
@@ -1122,20 +1049,14 @@ static uiBlock *image_uvsmenu(void *arg_unused)
 
 	uiDefIconTextBlockBut(block, image_uvs_propfalloffmenu, NULL, ICON_RIGHTARROW_THIN, "Proportional Falloff", 0, yco-=20, 120, 19, "");
 
-	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
-	
-	/* note that we acount for the 3 previous entries with i+3: */
-	for (pym = BPyMenuTable[PYMENU_UV]; pym; pym = pym->next, i++) {
-
-		uiDefIconTextBut(block, BUTM, 1, ICON_PYTHON, pym->name, 0, yco-=20, menuwidth, 19, 
-				 NULL, 0.0, 0.0, 1, i+20, 
-				 pym->tooltip?pym->tooltip:pym->filename);
-	}
-
 	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
 
 	uiDefIconTextBlockBut(block, image_uvs_showhidemenu, NULL, ICON_RIGHTARROW_THIN, "Show/Hide Faces", 0, yco-=20, menuwidth, 19, "");
 
+	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
+	
+	uiDefIconTextBlockBut(block, image_uvs_scriptsmenu, NULL, ICON_RIGHTARROW_THIN, "Scripts", 0, yco-=20, 120, 19, "");
+	
 	if(curarea->headertype==HEADERTOP) {
 		uiBlockSetDirection(block, UI_DOWN);
 	}
@@ -1151,9 +1072,11 @@ static uiBlock *image_uvsmenu(void *arg_unused)
 
 void image_buttons(void)
 {
+	Image *ima;
+	ImBuf *ibuf;
 	uiBlock *block;
 	short xco, xmax;
-	char naam[256];
+	char naam[256], *menuname;
 	/* This should not be a static var */
 	static int headerbuttons_packdummy;
 
@@ -1166,6 +1089,8 @@ void image_buttons(void)
 	else uiBlockSetCol(block, TH_HEADERDESEL);
 
 	what_image(G.sima);
+	ima= G.sima->image;
+	ibuf= BKE_image_get_ibuf(ima, &G.sima->iuser);
 
 	curarea->butspacetype= SPACE_IMAGE;
 
@@ -1187,6 +1112,7 @@ void image_buttons(void)
 	xco+=XIC;
 
 	if((curarea->flag & HEADER_NO_PULLDOWN)==0) {
+		
 		/* pull down menus */
 		uiBlockSetEmboss(block, UI_EMBOSSP);
 	
@@ -1194,49 +1120,84 @@ void image_buttons(void)
 		uiDefPulldownBut(block, image_viewmenu, NULL, "View", xco, -2, xmax-3, 24, "");
 		xco+= xmax;
 		
-		xmax= GetButStringLength("Select");
-		uiDefPulldownBut(block, image_selectmenu, NULL, "Select", xco, -2, xmax-3, 24, "");
-        
+		if((G.f & G_FACESELECT) && !(ima && (G.sima->flag & SI_DRAWTOOL))) {
+			xmax= GetButStringLength("Select");
+			uiDefPulldownBut(block, image_selectmenu, NULL, "Select", xco, -2, xmax-3, 24, "");
+			xco+= xmax;
+		}
+		
+		if (ibuf && (ibuf->userflags & IB_BITMAPDIRTY))
+			menuname= "Image*";
+		else
+			menuname= "Image";
+		xmax= GetButStringLength(menuname);
+		uiDefPulldownBut(block, image_imagemenu, NULL, menuname, xco, -2, xmax-3, 24, "");
 		xco+= xmax;
 		
-		xmax= GetButStringLength("Image");
-		uiDefPulldownBut(block, image_imagemenu, NULL, "Image", xco, -2, xmax-3, 24, "");
-		xco+= xmax;
-		
-		xmax= GetButStringLength("UVs");
-		uiDefPulldownBut(block, image_uvsmenu, NULL, "UVs", xco, -2, xmax-3, 24, "");
-		xco+= xmax;
+		if((G.f & G_FACESELECT) && !(ima && (G.sima->flag & SI_DRAWTOOL))) {
+			xmax= GetButStringLength("UVs");
+			uiDefPulldownBut(block, image_uvsmenu, NULL, "UVs", xco, -2, xmax-3, 24, "");
+			xco+= xmax;
+		}
 	}
 	
 	/* other buttons: */
 	uiBlockSetEmboss(block, UI_EMBOSS);
 
-	xco= std_libbuttons(block, xco, 0, 0, NULL, B_SIMABROWSE, ID_IM, 0, (ID *)G.sima->image, 0, &(G.sima->imanr), 0, 0, B_IMAGEDELETE, 0, 0);
+	xco= std_libbuttons(block, xco, 0, 0, NULL, B_SIMABROWSE, ID_IM, 0, (ID *)ima, 0, &(G.sima->imanr), 0, 0, B_IMAGEDELETE, 0, 0);
 
-	if (G.sima->image) {
+	if (ima) {
+		RenderResult *rr= BKE_image_get_renderresult(ima);
+		
 		xco+= 8;
 	
-		if (G.sima->image->packedfile) {
-			headerbuttons_packdummy = 1;
+		if(rr) {
+			uiBlockBeginAlign(block);
+			uiblock_layer_pass_buttons(block, rr, &G.sima->iuser, B_REDR, xco, 0, 160);
+			uiBlockEndAlign(block);
+			xco+= 166;
 		}
-		uiDefIconButBitI(block, TOG, 1, B_SIMAPACKIMA, ICON_PACKAGE,	xco,0,XIC,YIC, &headerbuttons_packdummy, 0, 0, 0, 0, "Pack/Unpack this image");
-		xco+= XIC+8;
+		if( !ELEM3(ima->source, IMA_SRC_SEQUENCE, IMA_SRC_MOVIE, IMA_SRC_VIEWER) && ima->ok) {
 
+			if (ima->packedfile) {
+				headerbuttons_packdummy = 1;
+			}
+			if (ima->packedfile && ibuf && (ibuf->userflags & IB_BITMAPDIRTY))
+				uiDefIconButBitI(block, TOG, 1, B_SIMA_REPACK, ICON_UGLYPACKAGE,	xco,0,XIC,YIC, &headerbuttons_packdummy, 0, 0, 0, 0, "Re-Pack this image as PNG");
+			else
+				uiDefIconButBitI(block, TOG, 1, B_SIMAPACKIMA, ICON_PACKAGE,	xco,0,XIC,YIC, &headerbuttons_packdummy, 0, 0, 0, 0, "Pack/Unpack this image");
+				
+			xco+= XIC+8;
+		}
+		
 		uiDefIconButBitI(block, TOG, SI_DRAWTOOL, B_SIMAGEPAINTTOOL, ICON_TPAINT_HLT, xco,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "Enables painting textures on the image with left mouse button");
 		xco+= XIC+8;
 
 		uiBlockBeginAlign(block);
-		uiDefIconButBitI(block, TOG, SI_USE_ALPHA, B_SIMA_USE_ALPHA, ICON_TRANSP_HLT, xco,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "Draws image with alpha");
-		xco+= XIC;
-		uiDefIconButBitI(block, TOG, SI_SHOW_ALPHA, B_SIMA_SHOW_ALPHA, ICON_DOT, xco,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "Draws only alpha");
-		xco+= XIC;
-		if(G.sima->image->ibuf) {
-			if(G.sima->image->ibuf->zbuf || G.sima->image->ibuf->zbuf_float) {
+		if(ibuf==NULL || ibuf->channels==4) {
+			uiDefIconButBitI(block, TOG, SI_USE_ALPHA, B_SIMA_USE_ALPHA, ICON_TRANSP_HLT, xco,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "Draws image with alpha");
+			xco+= XIC;
+			uiDefIconButBitI(block, TOG, SI_SHOW_ALPHA, B_SIMA_SHOW_ALPHA, ICON_DOT, xco,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "Draws only alpha");
+			xco+= XIC;
+		}
+		if(ibuf) {
+			if(ibuf->zbuf || ibuf->zbuf_float || (ibuf->channels==1)) {
 				uiDefIconButBitI(block, TOG, SI_SHOW_ZBUF, B_SIMA_SHOW_ZBUF, ICON_SOLID, xco,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "Draws zbuffer values");
 				xco+= XIC;
 			}
 			else G.sima->flag &= ~SI_SHOW_ZBUF;	/* no confusing display for non-zbuf images */
 		}		
+		xco+= 8;
+		
+		uiBlockBeginAlign(block);
+		if(ima->type==IMA_TYPE_COMPOSITE) {
+			uiDefIconBut(block, BUT, B_SIMA_RECORD, ICON_REC,  xco, 0, XIC, YIC, 0, 0, 0, 0, 0, "Record Composite");
+			xco+= XIC;
+		}
+		if((ima->type==IMA_TYPE_COMPOSITE) || ELEM(ima->source, IMA_SRC_MOVIE, IMA_SRC_SEQUENCE)) {
+			uiDefIconBut(block, BUT, B_SIMA_PLAY, ICON_PLAY, xco, 0, XIC, YIC, 0, 0, 0, 0, 0, "Play");
+			xco+= XIC;
+		}
 		uiBlockEndAlign(block);
 		xco+= 8;
 	}
