@@ -4,7 +4,7 @@
  * Functions to draw the "User Preferences" window header
  * and handle user events sent to it.
  * 
- * $Id: header_info.c,v 1.90 2006/07/11 09:07:18 ton Exp $
+ * $Id: header_info.c,v 1.104 2007/01/12 01:13:27 broken Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -68,6 +68,7 @@
 #include "BIF_interface.h"
 #include "BIF_language.h"
 #include "BIF_mainqueue.h"
+#include "BIF_meshtools.h"
 #include "BIF_previewrender.h"
 #include "BIF_renderwin.h"
 #include "BIF_resources.h"
@@ -79,6 +80,10 @@
 #include "BIF_writeimage.h"
 #include "BIF_drawscene.h"
 
+#ifdef WITH_VERSE
+#include "BIF_verse.h"
+#endif
+
 #include "BKE_blender.h"
 #include "BKE_depsgraph.h"
 #include "BKE_exotic.h"
@@ -91,6 +96,10 @@
 #include "BKE_scene.h"
 #include "BKE_world.h"
 
+#ifdef WITH_VERSE
+#include "BKE_verse.h"
+#endif
+
 #include "BLI_arithb.h"
 #include "BLI_blenlib.h"
 #include "BLO_writefile.h"
@@ -99,6 +108,7 @@
 #include "BSE_filesel.h"
 #include "BSE_headerbuttons.h"
 #include "BSE_sequence.h"
+#include "BSE_edit.h"
 
 #include "IMB_imbuf_types.h"
 
@@ -311,7 +321,7 @@ Scene *copy_scene(Scene *sce, int level)
 	obase= sce->base.first;
 	base= scen->base.first;
 	while(base) {
-		base->object->id.us++;
+		id_us_plus(&base->object->id);
 		if(obase==sce->basact) scen->basact= base;
 
 		obase= obase->next;
@@ -331,7 +341,7 @@ Scene *copy_scene(Scene *sce, int level)
 	/* level 2 */
 	if(level>=2) {
 		if(scen->world) {
-			scen->world->id.us--;
+			id_us_plus(&scen->world->id);
 			scen->world= copy_world(scen->world);
 		}
 		single_obdata_users(0);
@@ -470,6 +480,7 @@ void do_info_buttons(unsigned short event)
 			else sce= copy_scene(G.scene, nr-2);
 
 			set_scene(sce);
+			countall();
 		}
 		BIF_preview_changed(ID_TE);
 
@@ -482,7 +493,7 @@ void do_info_buttons(unsigned short event)
 		if(okee("Delete current scene")) {
 			
 			/* exit modes... could become single call once */
-			exit_editmode(1);
+			exit_editmode(EM_FREEDATA|EM_WAITCURSOR);
 			if(G.f & G_VERTEXPAINT) set_vpaint(); /* Switch off vertex paint */
 			if(G.f & G_TEXTUREPAINT) set_texturepaint(); /* Switch off tex paint */
 			if(G.f & G_WEIGHTPAINT) set_wpaint();		/* Switch off weight paint */
@@ -506,6 +517,7 @@ void do_info_buttons(unsigned short event)
 			}
 			free_libblock(&G.main->scene, G.scene);
 			set_scene(sce);
+			countall();
 		}
 
 		break;
@@ -517,15 +529,16 @@ static void check_packAll()
 	// first check for dirty images
 	Image *ima;
 
-	ima = G.main->image.first;
-	while (ima) {
-		if (ima->ibuf && (ima->ibuf->userflags &= IB_BITMAPDIRTY)) {
-			break;
-		}
-		ima= ima->id.next;
+	for(ima = G.main->image.first; ima; ima= ima->id.next) {
+		if (ima->ibufs.first) { /* XXX FIX */
+			ImBuf *ibuf= BKE_image_get_ibuf(ima, NULL);
+
+			if (ibuf && (ibuf->userflags &= IB_BITMAPDIRTY))
+				break;
 		}
+	}
 	
-	if (ima == 0 || okee("Some images are painted on. These changes will be lost. Continue ?")) {
+	if (ima == NULL || okee("Some images are painted on. These changes will be lost. Continue ?")) {
 		packAll();
 		G.fileflags |= G_AUTOPACK;
 	}
@@ -720,6 +733,87 @@ static uiBlock *info_file_exportmenu(void *arg_unused)
 	return block;
 }
 
+#ifdef WITH_VERSE
+
+extern ListBase session_list;
+
+static void do_verse_filemenu(void *arg, int event)
+{
+	char address[64];		/* lenght of domain name is 63 characters or less */
+	VerseSession *session = NULL;
+	ScrArea *sa;
+	
+	if(curarea->spacetype==SPACE_INFO) {
+		sa= closest_bigger_area();
+		areawinset(sa->win);
+	}
+
+	switch(event) {
+		case 0:
+			waitcursor(1);
+			printf("Connecting to localhost!\n");
+			b_verse_connect("localhost");
+			waitcursor(0);
+			break;
+		case 1:
+			address[0] = '\0';
+			if(sbutton(address, 0, 63, "Server:")) {
+				waitcursor(1);
+				printf("Connecting to %s\n", address);
+				b_verse_connect(address);
+				waitcursor(0);
+			}
+			break;
+		case 2:
+			session = session_menu();
+			if(session) {
+				printf("Disconnecting session: %s!\n", session->address);
+				end_verse_session(session);
+			}
+			break;
+		case 3:
+			printf("Disconnecting all sessions!\n");
+			end_all_verse_sessions();
+			break;
+        case 4:
+            printf("sending get to master server\n");
+            b_verse_ms_get();
+            break;
+	}
+}
+
+static uiBlock *verse_filemenu(void *unusedargs)
+{
+	uiBlock *block;
+	short yco = 20, menuwidth = 120;
+
+	block= uiNewBlock(&curarea->uiblocks, "verse_filemenu", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetButmFunc(block, do_verse_filemenu, NULL);
+		
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Connect to localhost", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 0, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Connect ...", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 1, "");
+	if(session_list.first != NULL) {
+		if(session_list.first != session_list.last) {
+			uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Disconnect ...",
+					0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 2, "");
+			uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Disconnect all",
+					0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 3, "");
+		}
+		else {
+			uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Disconnect",
+					0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 3, "");
+		}
+		
+	}
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Get Servers", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 4, "");
+
+	uiBlockSetDirection(block, UI_RIGHT);
+	uiTextBoundsBlock(block, 60);
+
+	return block;
+}
+#endif
+
 static void do_info_filemenu(void *arg, int event)
 {
 	ScrArea *sa;
@@ -759,7 +853,7 @@ static void do_info_filemenu(void *arg, int event)
 		}
 		break;
 	case 6: /* save image */
-		BIF_save_rendered_image_fs(0);
+		BIF_save_rendered_image_fs();
 		break;
 	case 22: /* save runtime */
 		activate_fileselect(FILE_SPECIAL, "Save Runtime", "", write_runtime_check);
@@ -871,6 +965,9 @@ static uiBlock *info_filemenu(void *arg_unused)
 	
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "New|Ctrl X",				0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 0, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Open...|F1",				0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 1, "");
+#ifdef WITH_VERSE
+	uiDefIconTextBlockBut(block, verse_filemenu, NULL, ICON_RIGHTARROW_THIN, "Verse", 0, yco-=20, menuwidth, 19, "");
+#endif
 	uiDefIconTextBlockBut(block, info_openrecentmenu, NULL, ICON_RIGHTARROW_THIN, "Open Recent",0, yco-=20, 120, 19, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Recover Last Session",				0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 15, "");
 
@@ -903,7 +1000,7 @@ static uiBlock *info_filemenu(void *arg_unused)
 
 	uiDefBut(block, SEPR, 0, "",					0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Append...|Shift F1",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 3, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Append or Link|Shift F1",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 3, "");
 	uiDefIconTextBlockBut(block, info_file_importmenu, NULL, ICON_RIGHTARROW_THIN, "Import", 0, yco-=20, menuwidth, 19, "");
 	uiDefIconTextBlockBut(block, info_file_exportmenu, NULL, ICON_RIGHTARROW_THIN, "Export", 0, yco-=20, menuwidth, 19, "");
 	
@@ -915,7 +1012,7 @@ static uiBlock *info_filemenu(void *arg_unused)
 
 	uiDefBut(block, SEPR, 0, "",					0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Quit Blender| Q",				0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 13, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Quit Blender|Ctrl Q",				0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 13, "");
 
 	uiBlockSetDirection(block, UI_DOWN);
 	uiTextBoundsBlock(block, 80);
@@ -1343,6 +1440,7 @@ static void do_info_gamemenu(void *arg, int event)
 	case G_FILE_AUTOPLAY:
 	case G_FILE_GAME_TO_IPO:
 	case G_FILE_GAME_MAT:
+	case G_FILE_SHOW_PHYSICS:
 		G.fileflags ^= event;
 		break;
 	default:
@@ -1397,6 +1495,13 @@ static uiBlock *info_gamemenu(void *arg_unused)
 		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Show Framerate and Profile",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_FRAMERATE, "");
 	}
 
+
+	if(G.fileflags & G_FILE_SHOW_PHYSICS) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Show Physics Visualization",		 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_PHYSICS, "");
+	} else {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Show Physics Visualization",		 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_PHYSICS, "");
+	}
+	
 	if(G.fileflags & G_FILE_SHOW_DEBUG_PROPS) {
 		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Show Debug Properties",		 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_DEBUG_PROPS, "");
 	} else {
@@ -1535,6 +1640,33 @@ static uiBlock *info_timelinemenu(void *arg_unused)
 
 /************************** RENDER *****************************/
 
+void do_info_render_bakemenu(void *arg, int event)
+{
+
+	objects_bake_render(event);
+	
+	allqueue(REDRAWINFO, 0);
+}
+
+static uiBlock *info_render_bakemenu(void *arg_unused)
+{
+	uiBlock *block;
+	short yco= 0;
+	
+	block= uiNewBlock(&curarea->uiblocks, "render_bakemenu", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetButmFunc(block, do_info_render_bakemenu, NULL);
+	
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Full Render|Ctrl Alt B, 1",				0, yco-=20, 160, 19, NULL, 0.0, 0.0, 1, 1, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Ambient Occlusion|Ctrl Alt B, 2",				0, yco-=20, 160, 19, NULL, 0.0, 0.0, 1, 2, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Normals|Ctrl Alt B, 3",				0, yco-=20, 160, 19, NULL, 0.0, 0.0, 1, 3, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Texture Only|Ctrl Alt B, 4",				0, yco-=20, 160, 19, NULL, 0.0, 0.0, 1, 4, "");
+
+	uiBlockSetDirection(block, UI_RIGHT);
+	uiTextBoundsBlock(block, 50);
+		
+	return block;
+}
+
 static void do_info_rendermenu(void *arg, int event)
 {
 
@@ -1593,6 +1725,10 @@ static uiBlock *info_rendermenu(void *arg_unused)
 	
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Render Current Frame|F12",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 0, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Render Animation|Ctrl F12",		0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 1, "");
+	
+	uiDefBut(block, SEPR, 0, "",				0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
+	
+	uiDefIconTextBlockBut(block, info_render_bakemenu, NULL, ICON_RIGHTARROW_THIN, "Bake Render Meshes", 0, yco-=20, 120, 19, "");
 
 	uiDefBut(block, SEPR, 0, "",				0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 
