@@ -4,7 +4,7 @@
  * Functions to draw the "Ipo Curve Editor" window header
  * and handle user events sent to it.
  * 
- * $Id: header_ipo.c,v 1.50 2006/06/05 19:04:19 schlaile Exp $
+ * $Id: header_ipo.c,v 1.60 2007/01/11 12:13:54 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -79,6 +79,7 @@
 #include "BSE_edit.h"
 #include "BSE_editipo.h"
 #include "BSE_headerbuttons.h"
+#include "BSE_time.h"
 
 #include "BIF_editaction.h"
 #include "BIF_interface.h"
@@ -110,7 +111,7 @@ void spaceipo_assign_ipo(SpaceIpo *si, Ipo *ipo)
 		bActionChannel *achan;
 		
 		if(ob->action) {
-			achan= get_action_channel(ob->action, si->actname);
+			achan= verify_action_channel(ob->action, si->actname);
 		
 			if(achan) {
 				/* constraint exception */
@@ -316,6 +317,35 @@ static uiBlock *ipo_editmenu_snapmenu(void *arg_unused)
 	return block;
 }
 
+static void do_ipo_editmenu_mirrormenu(void *arg, int event)
+{
+	switch(event) {
+		case 1: /* mirror over current frame */
+		case 2: /* mirror over frame 0 */
+		case 3: /* mirror over horizontal axis */
+			ipo_mirror(event);
+			break;
+	}
+	allqueue(REDRAWVIEW3D, 0);
+}
+
+static uiBlock *ipo_editmenu_mirrormenu(void *arg_unused)
+{
+	uiBlock *block;
+	short yco = 20, menuwidth = 120;
+
+	block= uiNewBlock(&curarea->uiblocks, "ipo_editmenu_mirrormenu", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetButmFunc(block, do_ipo_editmenu_mirrormenu, NULL);
+	
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Over Current Frame|Shift M, 1",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 1, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Over Vertical Axis|Shift M, 2",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 2, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Over Horizontal Axis|Shift M, 3",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 3, "");	
+	
+	uiBlockSetDirection(block, UI_RIGHT);
+	uiTextBoundsBlock(block, 60);
+	return block;
+} 
+
 static void do_ipo_editmenu_joinmenu(void *arg, int event)
 {
 	switch(event) {
@@ -335,8 +365,8 @@ static uiBlock *ipo_editmenu_joinmenu(void *arg_unused)
 	block= uiNewBlock(&curarea->uiblocks, "ipo_editmenu_joinmenu", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
 	uiBlockSetButmFunc(block, do_ipo_editmenu_joinmenu, NULL);
 	
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "All Selected|J, 1",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 1, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Selected Doubles|J, 2",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 2, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "All Selected|Ctrl J, 1",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 1, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Selected Doubles|Ctrl J, 2",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 2, "");
 
 	uiBlockSetDirection(block, UI_RIGHT);
 	uiTextBoundsBlock(block, 60);
@@ -526,7 +556,7 @@ static void do_ipo_editmenu(void *arg, int event)
 	switch(event)
 	{
 	case 0:
-		del_ipo();
+		del_ipo(1);
 		break;
 	case 1:
 		add_duplicate_editipo();
@@ -550,6 +580,12 @@ static void do_ipo_editmenu(void *arg, int event)
 	case 7:
 		sethandles_ipo(HD_AUTO_ANIM);
 		break;
+	case 8: /* clean ipo */
+		clean_ipo();
+		break;
+	case 9: /* smooth ipo */
+		smooth_ipo();
+		break;
 	}
 }
 
@@ -573,6 +609,8 @@ static uiBlock *ipo_editmenu(void *arg_unused)
 	
 	uiDefIconTextBlockBut(block, ipo_editmenu_snapmenu, NULL, ICON_RIGHTARROW_THIN, "Snap", 0, yco-=20, 120, 19, "");	
 
+	uiDefIconTextBlockBut(block, ipo_editmenu_mirrormenu, NULL, ICON_RIGHTARROW_THIN, "Mirror", 0, yco-=20, 120, 19, "");	
+	
 	uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
 
 	/*Look to see if any ipos are being edited, so there can be a check next to the menu option*/
@@ -603,6 +641,8 @@ static uiBlock *ipo_editmenu(void *arg_unused)
 
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Duplicate|Shift D", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 1, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Record Mouse Movement|R", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 2, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Clean IPO Curves|O", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 8, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Smooth IPO Curves|Shift O", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 9, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Delete|X", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 0, "");
 	uiDefIconTextBlockBut(block, ipo_editmenu_joinmenu, NULL, ICON_RIGHTARROW_THIN, "Join", 0, yco-=20, 120, 19, "");	
 
@@ -669,7 +709,11 @@ static void do_ipo_viewmenu(void *arg, int event)
 		G.v2d->flag ^= V2D_VIEWLOCK;
 		if(G.v2d->flag & V2D_VIEWLOCK)
 			view2d_do_locks(curarea, 0);
-		break;			
+		break;	
+	case 10: /* center view to current frame */
+		center_currframe();
+		scrarea_queue_winredraw(curarea);
+		break;
 	}
 }
 
@@ -739,6 +783,17 @@ static void do_ipo_selectmenu(void *arg, int event)
 	case 1:
 		swap_selectall_editipo();
 		break;
+	case 2:
+		borderselect_markers();
+		break;
+	case 3:
+		deselect_markers(1, 0);
+		allqueue(REDRAWTIME, 0);
+		allqueue(REDRAWIPO, 0);
+		allqueue(REDRAWACTION, 0);
+		allqueue(REDRAWNLA, 0);
+		allqueue(REDRAWSOUND, 0);
+		break;
 	}
 }
 
@@ -751,8 +806,12 @@ static uiBlock *ipo_selectmenu(void *arg_unused)
 	uiBlockSetButmFunc(block, do_ipo_selectmenu, NULL);
 
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Border Select|B", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 0, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Border Select Markers|Ctrl B", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 2, "");
+	
 	uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
+	
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Select/Deselect All|A", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 1, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Select/Deselect All Markers|Ctrl A", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 3, "");
 
 	if(curarea->headertype==HEADERTOP) {
 		uiBlockSetDirection(block, UI_DOWN);
@@ -767,6 +826,69 @@ static uiBlock *ipo_selectmenu(void *arg_unused)
 	return block;
 }
 
+static void do_ipo_markermenu(void *arg, int event)
+{	
+	switch(event)
+	{
+		case 1:
+			add_marker(CFRA);
+			break;
+		case 2:
+			duplicate_marker();
+			break;
+		case 3:
+			remove_marker();
+			break;
+		case 4:
+			rename_marker();
+			break;
+		case 5:
+			transform_markers('g', 0);
+			break;
+	}
+	
+	allqueue(REDRAWTIME, 0);
+	allqueue(REDRAWIPO, 0);
+	allqueue(REDRAWACTION, 0);
+	allqueue(REDRAWNLA, 0);
+	allqueue(REDRAWSOUND, 0);
+}
+
+static uiBlock *ipo_markermenu(void *arg_unused)
+{
+	uiBlock *block;
+	short yco= 0, menuwidth=120;
+
+	block= uiNewBlock(&curarea->uiblocks, "ipo_markermenu", 
+					  UI_EMBOSSP, UI_HELV, curarea->headwin);
+	uiBlockSetButmFunc(block, do_ipo_markermenu, NULL);
+
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Add Marker|M", 0, yco-=20, 
+					 menuwidth, 19, NULL, 0.0, 0.0, 1, 1, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Duplicate Marker|Ctrl Shift D", 0, yco-=20, 
+					 menuwidth, 19, NULL, 0.0, 0.0, 1, 2, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Delete Marker|X", 0, yco-=20,
+					 menuwidth, 19, NULL, 0.0, 0.0, 1, 3, "");
+					 
+	uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
+
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "(Re)Name Marker|Ctrl M", 0, yco-=20,
+					 menuwidth, 19, NULL, 0.0, 0.0, 1, 4, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Grab/Move Marker|Ctrl G", 0, yco-=20,
+					 menuwidth, 19, NULL, 0.0, 0.0, 1, 5, "");
+	
+	if(curarea->headertype==HEADERTOP) {
+		uiBlockSetDirection(block, UI_DOWN);
+	}
+	else {
+		uiBlockSetDirection(block, UI_TOP);
+		uiBlockFlipOrder(block);
+	}
+
+	uiTextBoundsBlock(block, 50);
+
+	return block;
+}
 
 static char *ipo_modeselect_pup(void)
 {
@@ -1067,7 +1189,11 @@ void ipo_buttons(void)
 		xmax= GetButStringLength("Select");
 		uiDefPulldownBut(block,ipo_selectmenu, NULL, "Select", xco, -2, xmax-3, 24, "");
 		xco+=xmax;
-	
+		
+		xmax= GetButStringLength("Marker");
+		uiDefPulldownBut(block,ipo_markermenu, NULL, "Marker", xco, -2, xmax-3, 24, "");
+		xco+=xmax;
+		
 		if (G.sipo->showkey) {
 			xmax= GetButStringLength("Key");
 			uiDefPulldownBut(block,ipo_editmenu, NULL, "Key", xco, -2, xmax-3, 24, "");
@@ -1080,7 +1206,6 @@ void ipo_buttons(void)
 			xmax= GetButStringLength("Curve");
 			uiDefPulldownBut(block,ipo_editmenu, NULL, "Curve", xco, -2, xmax-3, 24, "");
 		}
-			
 		xco+=xmax;
 	}
 
