@@ -163,6 +163,12 @@ static void do_node_selectmenu(void *arg, int event)
 		case 2: /* select/deselect all */
 			node_deselectall(snode, 1);
 			break;
+		case 3:	/* select linked in */
+			node_select_linked(snode, 0);
+			break;
+		case 4:	/* select linked out */
+			node_select_linked(snode, 1);
+			break;
 	}
 	allqueue(REDRAWNODE, 0);
 }
@@ -181,6 +187,8 @@ static uiBlock *node_selectmenu(void *arg_unused)
 	uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 	
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Select/Deselect All|A", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 2, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Select Linked From|L", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 3, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Select Linked To|Shift L", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 4, "");
 	
 	if(curarea->headertype==HEADERTOP) {
 		uiBlockSetDirection(block, UI_DOWN);
@@ -198,13 +206,23 @@ static uiBlock *node_selectmenu(void *arg_unused)
 void do_node_addmenu(void *arg, int event)
 {
 	SpaceNode *snode= curarea->spacedata.first;
+	bNode *node;
 	float locx, locy;
 	short mval[2];
 	
+	/* store selection in temp test flag */
+	for(node= snode->edittree->nodes.first; node; node= node->next) {
+		if(node->flag & NODE_SELECT) node->flag |= NODE_TEST;
+		else node->flag &= ~NODE_TEST;
+	}
+	
 	getmouseco_areawin(mval);
 	areamouseco_to_ipoco(G.v2d, mval, &locx, &locy);
-	node_add_node(snode, event, locx, locy);
+	node= node_add_node(snode, event, locx, locy);
 	
+	/* uses test flag */
+	snode_autoconnect(snode, node, NODE_TEST);
+		
 	addqueue(curarea->win, UI_BUT_EVENT, B_NODE_TREE_EXEC);
 	
 	BIF_undo_push("Add Node");
@@ -356,6 +374,36 @@ static uiBlock *node_add_convertermenu(void *arg_unused)
 	
 	return block;
 }
+static uiBlock *node_add_mattemenu(void *arg_unused)
+{
+	SpaceNode *snode= curarea->spacedata.first;
+	uiBlock *block;
+	
+	block= uiNewBlock(&curarea->uiblocks, "node_add_mattemenu", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetButmFunc(block, do_node_addmenu, NULL);
+	
+	node_make_addmenu(snode, NODE_CLASS_MATTE, block);
+	
+	uiBlockSetDirection(block, UI_RIGHT);
+	uiTextBoundsBlock(block, 60);
+	
+	return block;
+}
+static uiBlock *node_add_distortmenu(void *arg_unused)
+{
+	SpaceNode *snode= curarea->spacedata.first;
+	uiBlock *block;
+	
+	block= uiNewBlock(&curarea->uiblocks, "node_add_distortmenu", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetButmFunc(block, do_node_addmenu, NULL);
+	
+	node_make_addmenu(snode, NODE_CLASS_DISTORT, block);
+	
+	uiBlockSetDirection(block, UI_RIGHT);
+	uiTextBoundsBlock(block, 60);
+	
+	return block;
+}
 static uiBlock *node_add_groupmenu(void *arg_unused)
 {
 	SpaceNode *snode= curarea->spacedata.first;
@@ -397,7 +445,10 @@ static uiBlock *node_addmenu(void *arg_unused)
 		uiDefIconTextBlockBut(block, node_add_vectormenu, NULL, ICON_RIGHTARROW_THIN, "Vector", 0, yco-=20, 120, 19, "");
 		uiDefIconTextBlockBut(block, node_add_filtermenu, NULL, ICON_RIGHTARROW_THIN, "Filter", 0, yco-=20, 120, 19, "");
 		uiDefIconTextBlockBut(block, node_add_convertermenu, NULL, ICON_RIGHTARROW_THIN, "Convertor", 0, yco-=20, 120, 19, "");
+		uiDefIconTextBlockBut(block, node_add_mattemenu, NULL, ICON_RIGHTARROW_THIN, "Matte", 0, yco-=20, 120, 19, "");
+		uiDefIconTextBlockBut(block, node_add_distortmenu, NULL, ICON_RIGHTARROW_THIN, "Distort", 0, yco-=20, 120, 19, "");
 		uiDefIconTextBlockBut(block, node_add_groupmenu, NULL, ICON_RIGHTARROW_THIN, "Group", 0, yco-=20, 120, 19, "");
+
 	} else
 		uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
 	
@@ -455,6 +506,9 @@ static void do_node_nodemenu(void *arg, int event)
 		case 9: /* show cyclic */
 			ntreeSolveOrder(snode->edittree);
 			break;
+		case 10: /* execute */
+			addqueue(curarea->win, UI_BUT_EVENT, B_NODE_TREE_EXEC);
+			break;
 	}
 	
 	if(fromlib==-1) error("Can't edit Library Data");
@@ -490,8 +544,10 @@ static uiBlock *node_nodemenu(void *arg_unused)
 	
 	uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 	
-	if(snode->treetype==NTREE_COMPOSIT)
+	if(snode->treetype==NTREE_COMPOSIT) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Execute Composite|E", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 10, "");
 		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Read Saved Render Results|R", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 8, "");
+	}
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Show Cyclic Dependencies|C", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 9, "");
 	
 	if(curarea->headertype==HEADERTOP) {
@@ -602,6 +658,8 @@ void node_buttons(ScrArea *sa)
 		xco+= 80;
 		uiDefButBitS(block, TOG, R_COMP_FREE, B_NOP, "Free Unused", xco+5,0,80,19, &G.scene->r.scemode, 0.0f, 0.0f, 0, 0, "Free Nodes that are not used while composite");
 		xco+= 80;
+		uiDefButBitS(block, TOG, SNODE_BACKDRAW, B_NOP, "Backdrop", xco+5,0,80,19, &snode->flag, 0.0f, 0.0f, 0, 0, "Use active Viewer Node output as backdrop");
+		xco+= 80;
 	}
 	
 	/* always as last  */
