@@ -1,5 +1,5 @@
 /**
- * $Id: header_oops.c,v 1.25 2006/01/28 18:33:18 hos Exp $
+ * $Id: header_oops.c,v 1.29 2006/11/20 05:12:57 broken Exp $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -135,6 +135,7 @@ void do_oops_buttons(short event)
 
 static void do_oops_viewmenu(void *arg, int event)
 {
+	SpaceOops *soops= curarea->spacedata.first;
 
 	switch(event) {
 	case 0: /* Shuffle Selected Blocks */
@@ -153,14 +154,11 @@ static void do_oops_viewmenu(void *arg, int event)
 		/* using event B_FULL */
 		break;
 	case 5: /* show outliner */
-		{
-			SpaceOops *soops= curarea->spacedata.first;
-			if(soops->type==SO_OOPS || soops->type==SO_DEPSGRAPH) soops->type= SO_OUTLINER;
-			else soops->type= SO_OOPS;
-			init_v2d_oops(curarea, soops);
-			test_view2d(G.v2d, curarea->winx, curarea->winy);
-			scrarea_queue_winredraw(curarea);
-		}
+		if(soops->type==SO_OOPS || soops->type==SO_DEPSGRAPH) soops->type= SO_OUTLINER;
+		else soops->type= SO_OOPS;
+		init_v2d_oops(curarea, soops);
+		test_view2d(G.v2d, curarea->winx, curarea->winy);
+		scrarea_queue_winredraw(curarea);
 		break;
 	case 6:
 		outliner_toggle_visible(curarea);
@@ -193,6 +191,10 @@ static void do_oops_viewmenu(void *arg, int event)
 		}
 		break;
 #endif
+	case 12:
+		if (soops->flag & SO_HIDE_RESTRICTCOLS) soops->flag &= ~SO_HIDE_RESTRICTCOLS;
+		else soops->flag |= SO_HIDE_RESTRICTCOLS;
+		break;
 	}
 }			
 
@@ -229,13 +231,20 @@ static uiBlock *oops_viewmenu(void *arg_unused)
 	else {
 		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Show Oops Schematic", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 5, "");
 		
-		uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
+		uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");  
 		
+		if (soops->flag & SO_HIDE_RESTRICTCOLS)
+			uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Show Restriction Columns", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 12, "");
+		else
+			uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Show Restriction Columns", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 12, "");
+
+		uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
+
 		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Expand One Level|NumPad +", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 9, "");
 		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Collapse One Level|NumPad -", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 10, "");
-		
-		uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");  
 
+		uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");  
+			
 		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Show/Hide All", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 6, "");
 		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Show Hierarchy|Home", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 7, "");
 		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Show Active|NumPad .", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 8, "");
@@ -348,7 +357,67 @@ static uiBlock *oops_blockmenu(void *arg_unused)
 	return block;
 }
 
+static void do_oops_searchmenu(void *arg, int event)
+{	
+	int search_flags = OL_FIND, again = 0;
+	
+	switch(event)
+	{
+	case 0: /* plain new find */	
+		search_flags = OL_FIND;
+		break;
+	case 1: /* case sensitive */
+		search_flags = OL_FIND_CASE;
+		break;
+	case 2: /* full search */
+		search_flags = OL_FIND_COMPLETE;
+		break;
+	case 3: /* full case sensitive */
+		search_flags = OL_FIND_COMPLETE_CASE;
+		break;
+	case 4: /* again */
+		again = 1;
+		break;
+	default: /* nothing valid */
+		return;
+	}
+	
+	/* run search */
+	outliner_find_panel(curarea, again, search_flags);
+}
 
+static uiBlock *oops_searchmenu(void *arg_unused)
+{
+	uiBlock *block;
+	short yco= 0, menuwidth=120;
+
+	block= uiNewBlock(&curarea->uiblocks, "oops_searchmenu", UI_EMBOSSP, UI_HELV, curarea->headwin);
+	uiBlockSetButmFunc(block, do_oops_searchmenu, NULL);
+
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Find|F", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 0, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Find (Case Sensitive)|Ctrl F", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 1, "");
+	
+	uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");  
+	
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Find Complete|Alt F", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 2, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Find Complete (Case Sensitive)|Ctrl Alt F", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 3, "");
+	
+	uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");  
+	
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Find Again|Shift F", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 4, "");
+
+	if(curarea->headertype==HEADERTOP) {
+		uiBlockSetDirection(block, UI_DOWN);
+	}
+	else {
+		uiBlockSetDirection(block, UI_TOP);
+		uiBlockFlipOrder(block);
+	}
+
+	uiTextBoundsBlock(block, 50);
+
+	return block;
+}
 
 void oops_buttons(void)
 {	
@@ -405,6 +474,11 @@ void oops_buttons(void)
 			xco+= xmax;
 			
 		}
+		else {
+			xmax= GetButStringLength("Search");
+			uiDefPulldownBut(block, oops_searchmenu, NULL, "Search", xco, -2, xmax-3, 24, "");
+			xco+= xmax;
+		}
 	}
 
 	uiBlockSetEmboss(block, UI_EMBOSS);
@@ -477,7 +551,18 @@ void oops_buttons(void)
 	} 
 #endif
 	else {
-		uiDefButS(block, MENU, B_REDR, "Outliner Display%t|All Scenes %x0|Current Scene %x1|Visible Layers %x2|Groups %x6|Same Types %x5|Selected %x3|Active %x4",	 xco, 0, 100, 20,  &soops->outlinevis, 0, 0, 0, 0, "");
+		if(G.main->library.first) 
+#ifdef WITH_VERSE
+			uiDefButS(block, MENU, B_REDR, "Outliner Display%t|Verse Servers %x9|Verse Sessions %x8|Libraries %x7|All Scenes %x0|Current Scene %x1|Visible Layers %x2|Groups %x6|Same Types %x5|Selected %x3|Active %x4",	 xco, 0, 100, 20,  &soops->outlinevis, 0, 0, 0, 0, "");
+#else
+			uiDefButS(block, MENU, B_REDR, "Outliner Display%t|Libraries %x7|All Scenes %x0|Current Scene %x1|Visible Layers %x2|Groups %x6|Same Types %x5|Selected %x3|Active %x4",	 xco, 0, 100, 20,  &soops->outlinevis, 0, 0, 0, 0, "");
+#endif /* WITH_VERSE */
+		else
+#ifdef WITH_VERSE
+			uiDefButS(block, MENU, B_REDR, "Outliner Display%t|Verse Servers %x9|Verse Sessions %x8|All Scenes %x0|Current Scene %x1|Visible Layers %x2|Groups %x6|Same Types %x5|Selected %x3|Active %x4",	 xco, 0, 100, 20,  &soops->outlinevis, 0, 0, 0, 0, "");
+#else
+			uiDefButS(block, MENU, B_REDR, "Outliner Display%t|All Scenes %x0|Current Scene %x1|Visible Layers %x2|Groups %x6|Same Types %x5|Selected %x3|Active %x4",	 xco, 0, 100, 20,  &soops->outlinevis, 0, 0, 0, 0, "");
+#endif /* WITH_VERSE */
 	}
 	
 	/* always do as last */
