@@ -1,6 +1,6 @@
 
 /**
- * $Id: interface_draw.c,v 1.58 2006/06/29 09:44:08 ton Exp $
+ * $Id: interface_draw.c,v 1.66 2006/12/26 10:18:38 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -160,10 +160,9 @@ void uiEmboss(float x1, float y1, float x2, float y2, int sel)
 /* icons have been standardized... and this call draws in untransformed coordinates */
 #define ICON_HEIGHT		16.0f
 
-static void ui_draw_icon(uiBut *but, BIFIconID icon)
+static void ui_draw_icon(uiBut *but, BIFIconID icon, int blend)
 {
 	float xs=0, ys=0, aspect, height;
-	int blend= 0;
 
 	/* this icon doesn't need draw... */
 	if(icon==ICON_BLANK1) return;
@@ -184,7 +183,10 @@ static void ui_draw_icon(uiBut *but, BIFIconID icon)
 		height= ICON_HEIGHT;
 	
 	if(but->flag & UI_ICON_LEFT) {
-		if (but->type==BUTM) {
+		if (but->type==BUT_TOGDUAL && but->drawstr[0]) {
+			xs= but->x1-1.0;
+		}
+		else if (but->type==BUTM ) {
 			xs= but->x1+1.0;
 		}
 		else if ((but->type==ICONROW) || (but->type==ICONTEXTROW)) {
@@ -207,7 +209,7 @@ static void ui_draw_icon(uiBut *but, BIFIconID icon)
 	glEnable(GL_BLEND);
 
 	/* calculate blend color */
-	if ELEM4(but->type, ICONTOG, TOG, ROW, TOGN) {
+	if ELEM3(but->type, TOG, ROW, TOGN) {
 		if(but->flag & UI_SELECT);
 		else if(but->flag & UI_ACTIVE);
 		else blend= -60;
@@ -1455,19 +1457,17 @@ static void ui_draw_text_icon(uiBut *but)
 	
 	/* check for button text label */
 	if (but->type == ICONTEXTROW) {
-		ui_draw_icon(but, (BIFIconID) (but->icon+but->iconadd));
+		ui_draw_icon(but, (BIFIconID) (but->icon+but->iconadd), 0);
 	}
 	else {
 
 		/* text button selection and cursor */
 		if(but->pos != -1) {
 		
-			if (SELWIDTH > 0) {
+			if ((but->selend - but->selsta) > 0) {
 				/* text button selection */
 				selsta_tmp = but->selsta + strlen(but->str);
 				selend_tmp = but->selend + strlen(but->str);
-				if (but->ofs >= strlen(but->str))
-					selsta_tmp += (but->ofs - strlen(but->str));
 					
 				if(but->drawstr[0]!=0) {
 					ch= but->drawstr[selsta_tmp];
@@ -1507,6 +1507,17 @@ static void ui_draw_text_icon(uiBut *but)
 				}
 			}
 		}
+		
+		if(but->type==BUT_TOGDUAL) {
+			int dualset= 0;
+			if(but->pointype==SHO)
+				dualset= BTST( *(((short *)but->poin)+1), but->bitnr);
+			else if(but->pointype==INT)
+				dualset= BTST( *(((int *)but->poin)+1), but->bitnr);
+			
+			ui_draw_icon(but, ICON_DOT, dualset?0:-100);
+		}
+		
 		if(but->drawstr[0]!=0) {
 			int transopts;
 			int tog3= 0;
@@ -1519,7 +1530,7 @@ static void ui_draw_text_icon(uiBut *but)
 			and offset the text label to accomodate it */
 			
 			if ( (but->flag & UI_HAS_ICON) && (but->flag & UI_ICON_LEFT) ) {
-				ui_draw_icon(but, but->icon);
+				ui_draw_icon(but, but->icon, 0);
 
 				if(but->flag & UI_TEXT_LEFT) x= but->x1 + but->aspect*BIF_icon_get_width(but->icon)+5.0;
 				else x= (but->x1+but->x2-but->strwidth+1)/2.0;
@@ -1565,7 +1576,7 @@ static void ui_draw_text_icon(uiBut *but)
 			if(but->type==LABEL && but->min!=0.0) BIF_ThemeColor(TH_BUT_TEXT_HI);
 		
 			ui_rasterpos_safe(x, (but->y1+but->y2- 9.0)/2.0, but->aspect);
-			if(but->type==TEX || but->type==IDPOIN) transopts= 0;	// no translation, of course!
+			if(but->type==IDPOIN) transopts= 0;	// no translation, of course!
 			else transopts= (U.transopts & USER_TR_BUTTONS);
 			BIF_DrawString(but->font, but->drawstr+but->ofs, transopts);
 
@@ -1579,7 +1590,7 @@ static void ui_draw_text_icon(uiBut *but)
 		}
 		/* if there's no text label, then check to see if there's an icon only and draw it */
 		else if( but->flag & UI_HAS_ICON ) {
-			ui_draw_icon(but, (BIFIconID) (but->icon+but->iconadd));
+			ui_draw_icon(but, (BIFIconID) (but->icon+but->iconadd), 0);
 		}
 	}
 }
@@ -2179,6 +2190,15 @@ static void ui_draw_but_CURVE(uiBut *but)
 	glVertex2f(but->x1 + zoomx*(-offsx), but->y2);
 	glEnd();
 	
+	/* cfra option */
+	if(cumap->flag & CUMA_DRAW_CFRA) {
+		glColor3ub(0x60, 0xc0, 0x40);
+		glBegin(GL_LINES);
+		glVertex2f(but->x1 + zoomx*(cumap->black[0]-offsx), but->y1);
+		glVertex2f(but->x1 + zoomx*(cumap->black[0]-offsx), but->y2);
+		glEnd();
+	}
+	
 	/* the curve */
 	BIF_ThemeColor(TH_TEXT);
 	glBegin(GL_LINE_STRIP);
@@ -2187,13 +2207,27 @@ static void ui_draw_but_CURVE(uiBut *but)
 		curvemapping_changed(cumap, 0);	/* 0 = no remove doubles */
 	cmp= cuma->table;
 	
-	glVertex2f(but->x1, but->y1 + zoomy*(cmp[0].y-offsy));	/* first point */
+	/* first point */
+	if((cuma->flag & CUMA_EXTEND_EXTRAPOLATE)==0)
+		glVertex2f(but->x1, but->y1 + zoomy*(cmp[0].y-offsy));
+	else {
+		fx= but->x1 + zoomx*(cmp[0].x-offsx + cuma->ext_in[0]);
+		fy= but->y1 + zoomy*(cmp[0].y-offsy + cuma->ext_in[1]);
+		glVertex2f(fx, fy);
+	}
 	for(a=0; a<=CM_TABLE; a++) {
 		fx= but->x1 + zoomx*(cmp[a].x-offsx);
 		fy= but->y1 + zoomy*(cmp[a].y-offsy);
 		glVertex2f(fx, fy);
 	}
-	glVertex2f(but->x2, but->y1 + zoomy*(cmp[CM_TABLE].y-offsy));	/* last point */
+	/* last point */
+	if((cuma->flag & CUMA_EXTEND_EXTRAPOLATE)==0)
+		glVertex2f(but->x2, but->y1 + zoomy*(cmp[CM_TABLE].y-offsy));	
+	else {
+		fx= but->x1 + zoomx*(cmp[CM_TABLE].x-offsx - cuma->ext_out[0]);
+		fy= but->y1 + zoomy*(cmp[CM_TABLE].y-offsy - cuma->ext_out[1]);
+		glVertex2f(fx, fy);
+	}
 	glEnd();
 
 	/* the points, use aspect to make them visible on edges */
@@ -2261,19 +2295,22 @@ void ui_set_embossfunc(uiBut *but, int drawtype)
 	else {
 		int theme= BIF_GetThemeValue(TH_BUT_DRAWTYPE);
 		
-		// and the themes
-		if(theme==1) {
+		switch(theme) {
+		
+		case TH_ROUNDED:
+			but->embossfunc= ui_draw_round;
+			break;
+		case TH_OLDSKOOL:
+			but->embossfunc= ui_draw_oldskool;
+			break;
+		case TH_MINIMAL:
+			but->embossfunc= ui_draw_minimal;
+			break;
+		case TH_SHADED:
+		default:
 			but->embossfunc= ui_draw_default;
 			but->sliderfunc= ui_default_slider;
-		}
-		else if(theme==2) {
-			but->embossfunc= ui_draw_round;
-		}
-		else if(theme==3) {
-			but->embossfunc= ui_draw_oldskool;
-		}
-		else {
-			but->embossfunc= ui_draw_minimal;
+			break;
 		}
 	}
 	
@@ -2330,7 +2367,7 @@ void ui_draw_but(uiBut *but)
 
 	case LINK:
 	case INLINK:
-		ui_draw_icon(but, but->icon);
+		ui_draw_icon(but, but->icon, 0);
 		break;
 		
 	case ROUNDBOX:
