@@ -1,5 +1,5 @@
 /**
- * $Id: poseobject.c,v 1.37 2006/06/15 04:13:26 hos Exp $
+ * $Id: poseobject.c,v 1.55 2007/01/30 01:04:39 aligorith Exp $
  *
  * ***** BEGIN GPL LICENSE BLOCK *****
  *
@@ -47,6 +47,7 @@
 
 #include "BKE_action.h"
 #include "BKE_armature.h"
+#include "BKE_blender.h"
 #include "BKE_constraint.h"
 #include "BKE_deform.h"
 #include "BKE_depsgraph.h"
@@ -113,7 +114,7 @@ void enter_posemode(void)
 		return;
 	}
 
-	if (G.obedit) exit_editmode(1);
+	if (G.obedit) exit_editmode(EM_FREEDATA|EM_WAITCURSOR);
 	G.f &= ~(G_VERTEXPAINT | G_FACESELECT | G_TEXTUREPAINT | G_WEIGHTPAINT);
 }
 
@@ -170,6 +171,33 @@ bPoseChannel *get_active_posechannel (Object *ob)
 	return NULL;
 }
 
+/* if a selected or active bone is protected, throw error and return 1 */
+/* only_selected==1 : the active bone is allowed to be protected */
+static int pose_has_protected_selected(Object *ob, int only_selected)
+{
+	
+	/* check protection */
+	if(ob->proxy) {
+		bPoseChannel *pchan;
+		bArmature *arm= ob->data;
+		
+		for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
+			if(pchan->bone && (pchan->bone->layer & arm->layer)) {
+				if(pchan->bone->layer & arm->layer_protected) {
+					if(only_selected && (pchan->bone->flag & BONE_ACTIVE));
+					else if(pchan->bone->flag & (BONE_ACTIVE|BONE_SELECTED)) 
+					   break;
+				}
+			}
+		}
+		if(pchan) {
+			error("Cannot change Proxy protected bones");
+			return 1;
+		}
+	}
+	return 0;
+}
+
 /* only for real IK, not for auto-IK */
 int pose_channel_in_IK_chain(Object *ob, bPoseChannel *pchan)
 {
@@ -201,12 +229,26 @@ void pose_calculate_path(Object *ob)
 	Base *base;
 	float *fp;
 	int cfra;
+	int sfra, efra;
 	
 	if(ob==NULL || ob->pose==NULL)
 		return;
 	arm= ob->data;
 	
-	if(EFRA<=SFRA) return;
+	/* version patch for older files here (do_versions patch too complicated) */
+	if ((arm->pathsf == 0) || (arm->pathef == 0)) {
+		arm->pathsf = SFRA;
+		arm->pathef = EFRA;
+	}
+	if (arm->pathsize == 0) {
+		arm->pathsize = 1;
+	}
+	
+	/* set frame values */
+	cfra= CFRA;
+	sfra = arm->pathsf;
+	efra = arm->pathef;
+	if (efra<=sfra) return;
 	
 	DAG_object_update_flags(G.scene, ob, screen_view3d_layers());
 	
@@ -214,7 +256,9 @@ void pose_calculate_path(Object *ob)
 	for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
 		if(pchan->bone && (pchan->bone->flag & BONE_SELECTED)) {
 			if(arm->layer & pchan->bone->layer) {
-				pchan->pathlen= EFRA-SFRA;
+				pchan->pathlen= efra-sfra+1;
+				pchan->pathsf= sfra;
+				pchan->pathef= efra+1;
 				if(pchan->path)
 					MEM_freeN(pchan->path);
 				pchan->path= MEM_callocN(3*pchan->pathlen*sizeof(float), "pchan path");
@@ -222,9 +266,7 @@ void pose_calculate_path(Object *ob)
 		}
 	}
 	
-	cfra= CFRA;
-	for(CFRA=SFRA; CFRA<EFRA; CFRA++) {
-		
+	for(CFRA=sfra; CFRA<=efra; CFRA++) {
 		/* do all updates */
 		for(base= FIRSTBASE; base; base= base->next) {
 			if(base->object->recalc) {
@@ -238,8 +280,15 @@ void pose_calculate_path(Object *ob)
 			if(pchan->bone && (pchan->bone->flag & BONE_SELECTED)) {
 				if(arm->layer & pchan->bone->layer) {
 					if(pchan->path) {
-						fp= pchan->path+3*(CFRA-SFRA);
-						VECCOPY(fp, pchan->pose_tail);
+						fp= pchan->path+3*(CFRA-sfra);
+						
+						if (arm->pathflag & ARM_PATH_HEADS) { 
+							VECCOPY(fp, pchan->pose_head);
+						}
+						else {
+							VECCOPY(fp, pchan->pose_tail);
+						}
+						
 						Mat4MulVecfl(ob->obmat, fp);
 					}
 				}
@@ -249,6 +298,7 @@ void pose_calculate_path(Object *ob)
 	
 	CFRA= cfra;
 	allqueue(REDRAWVIEW3D, 0);	/* recalc tags are still there */
+	allqueue(REDRAWBUTSEDIT, 0);
 }
 
 
@@ -295,7 +345,8 @@ void pose_select_constraint_target(void)
 					if(ob==target) {
 						if(subtarget) {
 							bPoseChannel *pchanc= get_pose_channel(ob->pose, subtarget);
-							pchanc->bone->flag |= BONE_SELECTED|BONE_TIPSEL|BONE_ROOTSEL;
+							if(pchanc)
+								pchanc->bone->flag |= BONE_SELECTED|BONE_TIPSEL|BONE_ROOTSEL;
 						}
 					}
 				}
@@ -321,7 +372,7 @@ void pose_special_editmenu(void)
 	if(!ob && !ob->pose) return;
 	if(ob==G.obedit || (ob->flag & OB_POSEMODE)==0) return;
 	
-	nr= pupmenu("Specials%t|Select Constraint Target%x1|Flip Left-Right Names%x2|Calculate Paths%x3|Clear All Paths%x4");
+	nr= pupmenu("Specials%t|Select Constraint Target%x1|Flip Left-Right Names%x2|Calculate Paths%x3|Clear All Paths%x4|Clear User Transform %x5");
 	if(nr==1) {
 		pose_select_constraint_target();
 	}
@@ -334,6 +385,11 @@ void pose_special_editmenu(void)
 	else if(nr==4) {
 		pose_clear_paths(ob);
 	}
+	else if(nr==5) {
+		rest_pose(ob->pose);
+		DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+		BIF_undo_push("Clear User Transform Pose");
+	}
 }
 
 void pose_add_IK(void)
@@ -360,6 +416,9 @@ void pose_clear_IK(void)
 	if(!ob && !ob->pose) return;
 	if(ob==G.obedit || (ob->flag & OB_POSEMODE)==0) return;
 	
+	if(pose_has_protected_selected(ob, 0))
+		return;
+	
 	if(okee("Remove IK constraint(s)")==0) return;
 
 	for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
@@ -398,6 +457,9 @@ void pose_clear_constraints(void)
 	if(!ob && !ob->pose) return;
 	if(ob==G.obedit || (ob->flag & OB_POSEMODE)==0) return;
 	
+	if(pose_has_protected_selected(ob, 0))
+		return;
+	
 	if(okee("Remove Constraints")==0) return;
 	
 	/* find active */
@@ -427,6 +489,7 @@ void pose_copy_menu(void)
 	bArmature *arm= ob->data;
 	bPoseChannel *pchan, *pchanact;
 	short nr;
+	int i=0;
 	
 	/* paranoia checks */
 	if(!ob && !ob->pose) return;
@@ -438,31 +501,122 @@ void pose_copy_menu(void)
 	}
 	
 	if(pchan==NULL) return;
+	
+	if(pose_has_protected_selected(ob, 1))
+		return;
+
 	pchanact= pchan;
 	
-	nr= pupmenu("Copy Pose Attributes %t|Location%x1|Rotation%x2|Scale%x3|Constraints");
+	i= BLI_countlist(&(pchanact->constraints)); /* if there are 24 or less, allow for the user to select constraints */
+	if (i<25)
+		nr= pupmenu("Copy Pose Attributes %t|Local Location%x1|Local Rotation%x2|Local Size%x3|%l|Visual Location %x9|Visual Rotation%x10|Visual Size%x11|%l|Constraints (All)%x4|Constraints...%x5|%l|Transform Locks%x6|IK Limits%x7|Bone Shape%x8");
+	else
+		nr= pupmenu("Copy Pose Attributes %t|Local Location%x1|Local Rotation%x2|Local Size%x3|%l|Visual Location %x9|Visual Rotation%x10|Visual Size%x11|%l|Constraints (All)%x4|%l|Transform Locks%x6|IK Limits%x7|Bone Shape%x8");
 	
-	for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
-		if(arm->layer & pchan->bone->layer) {
-			if(pchan->bone->flag & BONE_SELECTED) {
-				if(pchan!=pchanact) {
-					if(nr==1) {
+	if(nr==-1) return;
+	if(nr!=5)  {
+		for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
+			if(	(arm->layer & pchan->bone->layer) &&
+				(pchan->bone->flag & BONE_SELECTED) &&
+				(pchan!=pchanact)
+			) {
+				switch (nr) {
+					case 1:
 						VECCOPY(pchan->loc, pchanact->loc);
-					}
-					else if(nr==2) {
+						break;
+					case 2:
 						QUATCOPY(pchan->quat, pchanact->quat);
-					}
-					else if(nr==3) {
+						break;
+					case 3:
 						VECCOPY(pchan->size, pchanact->size);
-					}
-					else if(nr==4) {
+						break;
+					case 4:
+					{
 						free_constraints(&pchan->constraints);
 						copy_constraints(&pchan->constraints, &pchanact->constraints);
 						pchan->constflag = pchanact->constflag;
 					}
+						break;
+					case 6:
+						pchan->protectflag = pchanact->protectflag;
+						break;
+					case 7:
+					{
+						pchan->ikflag = pchanact->ikflag;
+						VECCOPY(pchan->limitmin, pchanact->limitmin);
+						VECCOPY(pchan->limitmax, pchanact->limitmax);
+						VECCOPY(pchan->stiffness, pchanact->stiffness);
+						pchan->ikstretch= pchanact->ikstretch;
+					}
+						break;
+					case 8:
+						pchan->custom = pchanact->custom;
+						break;
+					case 9:
+						armature_loc_pose_to_bone(pchan, pchanact->pose_mat[3], pchan->loc);
+						break;
+					case 10:
+					{
+						float delta_mat[4][4], quat[4];
+						
+						armature_mat_pose_to_bone(pchan, pchanact->pose_mat, delta_mat);
+						Mat4ToQuat(delta_mat, quat);
+						QUATCOPY(pchan->quat, quat);
+					}
+						break;
+					case 11:
+					{
+						float delta_mat[4][4], size[4];
+						
+						armature_mat_pose_to_bone(pchan, pchanact->pose_mat, delta_mat);
+						Mat4ToSize(delta_mat, size);
+						VECCOPY(pchan->size, size);
+					}
 				}
 			}
 		}
+	} else { /* constraints, optional */
+		bConstraint *con, *con_back;
+		int const_toggle[24];
+		ListBase const_copy={0, 0};
+		
+		duplicatelist (&const_copy, &(pchanact->constraints));
+		
+		/* build the puplist of constraints */
+		for (con = pchanact->constraints.first, i=0; con; con=con->next, i++){
+			const_toggle[i]= 1;
+			add_numbut(i, TOG|INT, con->name, 0, 0, &(const_toggle[i]), "");
+		}
+		
+		if (!do_clever_numbuts("Select Constraints", i, REDRAW)) {
+			BLI_freelistN(&const_copy);
+			return;
+		}
+		
+		/* now build a new listbase from the options selected */
+		for (i=0, con=const_copy.first; con; i++) {
+			if (!const_toggle[i]) {
+				con_back= con->next;
+				BLI_freelinkN(&const_copy, con);
+				con= con_back;
+			} else {
+				con= con->next;
+			}
+		}
+		
+		/* Copy the temo listbase to the selected posebones */
+		for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
+			if(	(arm->layer & pchan->bone->layer) &&
+				(pchan->bone->flag & BONE_SELECTED) &&
+				(pchan!=pchanact)
+			) {
+				free_constraints(&pchan->constraints);
+				copy_constraints(&pchan->constraints, &const_copy);
+				pchan->constflag = pchanact->constflag;
+			}
+		}
+		BLI_freelistN(&const_copy);
+		update_pose_constraint_flags(ob->pose); /* we could work out the flags but its simpler to do this */
 	}
 	
 	DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);	// and all its relations
@@ -640,11 +794,10 @@ void pose_adds_vgroups(Object *meshobj)
 	bPoseChannel *pchan;
 	Bone *bone;
 	bDeformGroup *dg, *curdef;
-	int DMneedsFree;
 	
 	if(poseobj==NULL || (poseobj->flag & OB_POSEMODE)==0) return;
 	
-	dm = mesh_get_derived_final(meshobj, &DMneedsFree);
+	dm = mesh_get_derived_final(meshobj, CD_MASK_BAREMESH);
 	
 	map.meshobj= meshobj;
 	
@@ -696,7 +849,7 @@ void pose_adds_vgroups(Object *meshobj)
 		}
 	}
 	
-	if (DMneedsFree) dm->release(dm);
+	dm->release(dm);
 
 	allqueue(REDRAWVIEW3D, 0);
 	allqueue(REDRAWBUTSEDIT, 0);
@@ -719,6 +872,9 @@ void pose_flip_names(void)
 	if(!ob && !ob->pose) return;
 	if(ob==G.obedit || (ob->flag & OB_POSEMODE)==0) return;
 	
+	if(pose_has_protected_selected(ob, 0))
+		return;
+	
 	for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
 		if(arm->layer & pchan->bone->layer) {
 			if(pchan->bone->flag & (BONE_ACTIVE|BONE_SELECTED)) {
@@ -798,9 +954,25 @@ void pose_movetolayer(void)
 	if(ob==NULL) return;
 	arm= ob->data;
 	
-	if(ob->flag & OB_POSEMODE) {
+	if(G.qual & LR_SHIFTKEY) {
+		lay= arm->layer;
+		if( movetolayer_short_buts(&lay, "Armature Layers")==0 ) return;
+		if(lay==0) return;
+		arm->layer= lay;
+		if(ob->pose)
+			ob->pose->proxy_layer= lay;
+		
+		allqueue(REDRAWVIEW3D, 0);
+		allqueue(REDRAWACTION, 0);
+		allqueue(REDRAWBUTSEDIT, 0);
+		
+	}
+	else if(ob->flag & OB_POSEMODE) {
 		bPoseChannel *pchan;
 		
+		if(pose_has_protected_selected(ob, 0))
+			return;
+		
 		for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
 			if(arm->layer & pchan->bone->layer) {
 				if(pchan->bone->flag & BONE_SELECTED)
@@ -809,7 +981,7 @@ void pose_movetolayer(void)
 		}
 		if(lay==0) return;
 		
-		if( movetolayer_short_buts(&lay)==0 ) return;
+		if( movetolayer_short_buts(&lay, "Bone Layers")==0 ) return;
 		if(lay==0) return;
 
 		for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
@@ -819,6 +991,7 @@ void pose_movetolayer(void)
 			}
 		}
 		
+		BIF_undo_push("Move Bone layer");
 		allqueue(REDRAWVIEW3D, 0);
 		allqueue(REDRAWACTION, 0);
 		allqueue(REDRAWBUTSEDIT, 0);
