@@ -1,5 +1,5 @@
 /* 
- * $Id: previewrender.c,v 1.85 2006/07/06 10:10:41 ton Exp $
+ * $Id: previewrender.c,v 1.88 2007/01/26 15:34:32 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -206,6 +206,8 @@ void BIF_preview_changed(short id_code)
 					addafterqueue(sa->win, RENDERPREVIEW, 1);
 				}
 			}
+			else if (sbuts->ri) 
+				sbuts->ri->curtile= 0;	/* ensure changes always result in re-render when context is restored */
 		}
 		else if(sa->spacetype==SPACE_NODE) {
 			SpaceNode *snode= sa->spacedata.first;
@@ -269,6 +271,8 @@ static Scene *preview_prepare_scene(RenderInfo *ri, int id_type, ID *id, int pr_
 		/* set world always back, is used now */
 		sce->world= pr_main->world.first;
 		
+		sce->r.cfra= G.scene->r.cfra;
+		
 		if(id_type==ID_MA) {
 			Material *mat= (Material *)id;
 			
@@ -382,7 +386,7 @@ static void previewrender_progress(RenderResult *rr, volatile rcti *renrect)
 	
 	glDrawBuffer(GL_FRONT);
 	glaDrawPixelsSafe_to32(ofsx, ofsy, rr->rectx, rr->recty, rr->rectx, rl->rectf);
-	glFlush();
+	bglFlush();
 	glDrawBuffer(GL_BACK);
 }
 
@@ -626,7 +630,7 @@ static void view3d_previewrender_progress(RenderResult *rr, volatile rcti *renre
 	glDrawBuffer(GL_FRONT);
 	glaDefine2DArea(&curarea->winrct);
 	glaDrawPixelsSafe_to32(ofsx, ofsy, rr->rectx, rr->recty, rr->rectx, rl->rectf);
-	glFlush();
+	bglFlush();
 	glDrawBuffer(GL_BACK);
 
 }
