@@ -1,5 +1,5 @@
 /**
- * $Id: resources.c,v 1.60 2006/07/03 09:49:12 ton Exp $
+ * $Id: resources.c,v 1.65 2007/01/03 05:50:17 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -133,7 +133,10 @@ char *BIF_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colorid)
 			
 			case TH_BUT_DRAWTYPE:
 				cp= &btheme->tui.but_drawtype; break;
-			
+
+			case TH_ICONFILE:
+				cp= btheme->tui.iconfile; break;
+				
 			case TH_REDALERT:
 				cp= alert; break;
 			case TH_CUSTOM:
@@ -247,6 +250,8 @@ char *BIF_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colorid)
 				cp= ts->edge_select; break;
 			case TH_EDGE_SEAM:
 				cp= ts->edge_seam; break;
+			case TH_EDGE_SHARP:
+				cp= ts->edge_sharp; break;
 			case TH_EDGE_FACESEL:
 				cp= ts->edge_facesel; break;
 			case TH_FACE:
@@ -358,7 +363,9 @@ void BIF_InitTheme(void)
 	SETCOL(btheme->tui.menu_hilite, 0x7F,0x7F,0x7F, 255);
 	SETCOL(btheme->tui.menu_text, 	0, 0, 0, 255);
 	SETCOL(btheme->tui.menu_text_hi, 255, 255, 255, 255);
-	btheme->tui.but_drawtype= 1;
+	btheme->tui.but_drawtype= TH_SHADED;
+	
+	BLI_strncpy(btheme->tui.iconfile, "", sizeof(btheme->tui.iconfile));
 	
 	/* space view3d */
 	SETCOL(btheme->tv3d.back, 	115, 115, 115, 255);
@@ -416,6 +423,7 @@ void BIF_InitTheme(void)
 	SETCOL(btheme->tipo.vertex, 0xff, 0x70, 0xff, 255);
 	SETCOL(btheme->tipo.vertex_select, 0xff, 0xff, 0x70, 255);
 	SETCOL(btheme->tipo.hilite, 0x60, 0xc0, 0x40, 255); 
+	btheme->tipo.vertex_size= 3;
 
 	/* space file */
 	/* to have something initialized */
@@ -547,6 +555,8 @@ char *BIF_ThemeColorsPup(int spacetype)
 		str += sprintf(str, "Menu Text Highlight %%x%d|", TH_MENU_TEXT_HI);
 		str += sprintf(str, "%%l|");
 		str += sprintf(str, "Drawtype %%x%d|", TH_BUT_DRAWTYPE);
+		str += sprintf(str, "%%l|");
+		str += sprintf(str, "Icon File %%x%d|", TH_ICONFILE);
 	}
 	else {
 		// first defaults for each space
@@ -573,6 +583,7 @@ char *BIF_ThemeColorsPup(int spacetype)
 			str += sprintf(str, "Vertex Size %%x%d|", TH_VERTEX_SIZE);
 			str += sprintf(str, "Edge Selected %%x%d|", TH_EDGE_SELECT);
 			str += sprintf(str, "Edge Seam %%x%d|", TH_EDGE_SEAM);
+			str += sprintf(str, "Edge Sharp %%x%d|", TH_EDGE_SHARP);
 			str += sprintf(str, "Edge UV Face Select %%x%d|", TH_EDGE_FACESEL);
 			str += sprintf(str, "Face (transp) %%x%d|", TH_FACE);
 			str += sprintf(str, "Face Selected (transp) %%x%d|", TH_FACE_SELECT);
@@ -590,6 +601,7 @@ char *BIF_ThemeColorsPup(int spacetype)
 			str += sprintf(str, "Ipo Channels %%x%d|", TH_SHADE2);
 			str += sprintf(str, "Vertex %%x%d|", TH_VERTEX);
 			str += sprintf(str, "Vertex Selected %%x%d|", TH_VERTEX_SELECT);
+			str += sprintf(str, "Vertex Size %%x%d|", TH_VERTEX_SIZE);
 			break;
 		case SPACE_FILE:
 			str += sprintf(str, "Selected file %%x%d", TH_HILITE);
@@ -612,6 +624,8 @@ char *BIF_ThemeColorsPup(int spacetype)
 			str += sprintf(str, "View Sliders %%x%d|", TH_SHADE1);
 			str += sprintf(str, "Channels %%x%d|", TH_SHADE2);
 			str += sprintf(str, "Channels Selected %%x%d|", TH_HILITE);
+			str += sprintf(str, "Long Key %%x%d|", TH_STRIP);
+			str += sprintf(str, "Long Key selected %%x%d|", TH_STRIP_SELECT);
 			break;
 		case SPACE_IMAGE:
 			str += sprintf(str, "%%l|");
