@@ -1,5 +1,5 @@
 /**
- * $Id: transform.c,v 1.140 2006/07/08 14:24:57 khughes Exp $
+ * $Id: transform.c,v 1.157 2007/02/07 23:28:33 theeth Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -64,6 +64,7 @@
 
 #include "BIF_editview.h"		/* arrows_move_cursor	*/
 #include "BIF_gl.h"
+#include "BIF_glutil.h"
 #include "BIF_mywindow.h"
 #include "BIF_resources.h"
 #include "BIF_screen.h"
@@ -79,8 +80,6 @@
 #include "BKE_utildefines.h"
 #include "BKE_bad_level_calls.h"/* popmenu and error	*/
 
-#include "BDR_editobject.h"		/* compatible_eul		*/
-
 #include "BSE_view.h"
 
 #include "BLI_arithb.h"
@@ -99,6 +98,14 @@
 TransInfo Trans = {TFM_INIT, 0};	// enforce init on first usage
 
 /******************************** Helper functions ************************************/
+
+/* GLOBAL Wrapper Fonctions */
+
+void BIF_drawSnap()
+{
+	drawSnapping(&Trans);
+}
+
 /* ************************** Dashed help line **************************** */
 
 
@@ -135,7 +142,7 @@ static void helpline(TransInfo *t, float *vec)
 		setlinestyle(0);
 		
 		persp(PERSP_VIEW);
-		glFlush(); // flush display for frontbuffer
+		bglFlush(); // flush display for frontbuffer
 		glDrawBuffer(GL_BACK);
 	}
 }
@@ -162,26 +169,29 @@ float InputScaleRatio(TransInfo *t, short mval[2]) {
 }
 
 float InputHorizontalRatio(TransInfo *t, short mval[2]) {
-	int y, pad;
+	float x, pad;
 
 	pad = curarea->winx / 10;
 
 	if (t->flag & T_SHIFT_MOD) {
 		/* deal with Shift key by adding motion / 10 to motion before shift press */
-		y = t->shiftmval[0] + (mval[0] - t->shiftmval[0]) / 10;
+		x = t->shiftmval[0] + (float)(mval[0] - t->shiftmval[0]) / 10.0f;
 	}
 	else {
-		y = mval[0];
+		x = mval[0];
 	}
-	return (float)(y - pad) / (float)(curarea->winx - 2 * pad);
+	return (x - pad) / (curarea->winx - 2 * pad);
 }
 
 float InputHorizontalAbsolute(TransInfo *t, short mval[2]) {
 	float vec[3];
 	if(t->flag & T_SHIFT_MOD) {
-		short dx = t->shiftmval[0] + (mval[0] - t->shiftmval[0]) / 10 - t->imval[0];
-		short dy = t->shiftmval[1] + (mval[1] - t->shiftmval[1]) / 10 - t->imval[1];
-		convertViewVec(t, t->vec, dx, dy);
+		float dvec[3];
+		/* calculate the main translation and the precise one separate */
+		convertViewVec(t, dvec, (short)(mval[0] - t->shiftmval[0]), (short)(mval[1] - t->shiftmval[1]));
+		VecMulf(dvec, 0.1f);
+		convertViewVec(t, t->vec, (short)(t->shiftmval[0] - t->imval[0]), (short)(t->shiftmval[1] - t->imval[1]));
+		VecAddf(t->vec, t->vec, dvec);
 	}
 	else {
 		convertViewVec(t, t->vec, (short)(mval[0] - t->imval[0]), (short)(mval[1] - t->imval[1]));
@@ -190,12 +200,30 @@ float InputHorizontalAbsolute(TransInfo *t, short mval[2]) {
 	return Inpf(t->viewinv[0], vec) * 2.0f;
 }
 
+float InputVerticalRatio(TransInfo *t, short mval[2]) {
+	float y, pad;
+
+	pad = curarea->winy / 10;
+
+	if (t->flag & T_SHIFT_MOD) {
+		/* deal with Shift key by adding motion / 10 to motion before shift press */
+		y = t->shiftmval[1] + (float)(mval[1] - t->shiftmval[1]) / 10.0f;
+	}
+	else {
+		y = mval[0];
+	}
+	return (y - pad) / (curarea->winy - 2 * pad);
+}
+
 float InputVerticalAbsolute(TransInfo *t, short mval[2]) {
 	float vec[3];
 	if(t->flag & T_SHIFT_MOD) {
-		short dx = t->shiftmval[0] + (mval[0] - t->shiftmval[0]) / 10 - t->imval[0];
-		short dy = t->shiftmval[1] + (mval[1] - t->shiftmval[1]) / 10 - t->imval[1];
-		convertViewVec(t, t->vec, dx, dy);
+		float dvec[3];
+		/* calculate the main translation and the precise one separate */
+		convertViewVec(t, dvec, (short)(mval[0] - t->shiftmval[0]), (short)(mval[1] - t->shiftmval[1]));
+		VecMulf(dvec, 0.1f);
+		convertViewVec(t, t->vec, (short)(t->shiftmval[0] - t->imval[0]), (short)(t->shiftmval[1] - t->imval[1]));
+		VecAddf(t->vec, t->vec, dvec);
 	}
 	else {
 		convertViewVec(t, t->vec, (short)(mval[0] - t->imval[0]), (short)(mval[1] - t->imval[1]));
@@ -700,6 +728,7 @@ static void transformEvent(unsigned short event, short val) {
 			break;
 		}
 		Trans.redraw |= handleNumInput(&(Trans.num), event);
+		Trans.redraw |= handleSnapping(&Trans, event);
 		arrows_move_cursor(event);
 	}
 	else {
@@ -748,6 +777,8 @@ void initTransform(int mode, int context) {
 
 	createTransData(&Trans);			// make TransData structs from selection
 
+	initSnapping(&Trans); // Initialize snapping data AFTER mode flags
+
 	if (Trans.total == 0) {
 		postTrans(&Trans);
 		return;
@@ -785,6 +816,9 @@ void initTransform(int mode, int context) {
 	case TFM_TILT:
 		initTilt(&Trans);
 		break;
+	case TFM_CURVE_SHRINKFATTEN:
+		initCurveShrinkFatten(&Trans);
+		break;
 	case TFM_TRACKBALL:
 		initTrackball(&Trans);
 		break;
@@ -806,6 +840,9 @@ void initTransform(int mode, int context) {
 	case TFM_BONE_ENVELOPE:
 		initBoneEnvelope(&Trans);
 		break;
+	case TFM_BONE_ROLL:
+		initBoneRoll(&Trans);
+		break;
 	}
 }
 
@@ -903,6 +940,8 @@ void initManipulator(int mode)
 	if (Trans.total == 0)
 		return;
 
+	initSnapping(&Trans); // Initialize snapping data AFTER mode flags
+
 	/* EVIL! posemode code can switch translation to rotate when 1 bone is selected. will be removed (ton) */
 	/* EVIL2: we gave as argument also texture space context bit... was cleared */
 	mode= Trans.mode;
@@ -1023,9 +1062,6 @@ void ManipulatorTransform()
 	if(Trans.state == TRANS_CANCEL) {
 		restoreTransObjects(&Trans);
 	}
-	else {
-		BIF_undo_push(transform_to_undostr(&Trans));
-	}
 	
 	/* free data, reset vars */
 	postTrans(&Trans);
@@ -1035,6 +1071,11 @@ void ManipulatorTransform()
 	
 	/* send events out for redraws */
 	viewRedrawPost(&Trans);
+
+	if(Trans.state != TRANS_CANCEL) {
+		BIF_undo_push(transform_to_undostr(&Trans));
+	}
+	
 }
 
 /* ************************** TRANSFORMATIONS **************************** */
@@ -1165,9 +1206,8 @@ int Warp(TransInfo *t, short mval[2])
 	Mat4MulVecfl(t->viewmat, cursor);
 	VecSubf(cursor, cursor, t->viewmat[3]);
 
-	// amount of degrees for warp, 450 = allow to create 360 degree warp
-	circumfac= 450.0f*(mval[1] - t->imval[1]) / (float)(curarea->winy);
-	circumfac+= 90.0f;
+	// amount of degrees for warp
+	circumfac= 360.0f * InputHorizontalRatio(t, mval);
 
 	snapGrid(t, &circumfac);
 	applyNumInput(&t->num, &circumfac);
@@ -1606,7 +1646,7 @@ int ToSphere(TransInfo *t, short mval[2])
 	float vec[3];
 	float ratio, radius;
 	int i;
-	char str[50];
+	char str[64];
 	TransData *td = t->data;
 
 	ratio = InputHorizontalRatio(t, mval);
@@ -1692,6 +1732,54 @@ static void ElementRotation(TransInfo *t, TransData *td, float mat[3][3]) {
 			QuatMul(td->ext->quat, quat, td->ext->iquat);
 		}
 	}
+	/**
+	 * HACK WARNING
+	 * 
+	 * This is some VERY ugly special case to deal with pose mode.
+	 * 
+	 * The problem is that mtx and smtx include each bone orientation.
+	 * 
+	 * That is needed to rotate each bone properly, HOWEVER, to calculate
+	 * the translation component, we only need the actual armature object's
+	 * matrix (and inverse). That is not all though. Once the proper translation
+	 * has been computed, it has to be converted back into the bone's space.
+	 */
+	else if (t->flag & T_POSE) {
+		float pmtx[3][3], imtx[3][3];
+
+		// Extract and invert armature object matrix		
+		Mat3CpyMat4(pmtx, t->poseobj->obmat);
+		Mat3Inv(imtx, pmtx);
+		
+		VecSubf(vec, td->center, t->center);
+		
+		Mat3MulVecfl(pmtx, vec);	// To Global space
+		Mat3MulVecfl(mat, vec);		// Applying rotation
+		Mat3MulVecfl(imtx, vec);	// To Local space
+
+		VecAddf(vec, vec, t->center);
+		/* vec now is the location where the object has to be */
+		
+		VecSubf(vec, vec, td->center); // Translation needed from the initial location
+		
+		Mat3MulVecfl(pmtx, vec);	// To Global space
+		Mat3MulVecfl(td->smtx, vec);// To Pose space
+
+		protectedTransBits(td->protectflag, vec);
+
+		VecAddf(td->loc, td->iloc, vec);
+		
+		/* rotation */
+		if ((t->flag & T_V3D_ALIGN)==0) { // align mode doesn't rotate objects itself
+			Mat3MulSerie(fmat, td->mtx, mat, td->smtx, 0, 0, 0, 0, 0);
+
+			Mat3ToQuat(fmat, quat);	// Actual transform
+			
+			QuatMul(td->ext->quat, quat, td->ext->iquat);
+			/* this function works on end result */
+			protectedQuaternionBits(td->protectflag, td->ext->quat, td->ext->iquat);
+		}
+	}
 	else {
 		/* translation */
 		
@@ -1713,66 +1801,68 @@ static void ElementRotation(TransInfo *t, TransData *td, float mat[3][3]) {
 		else VecAddf(td->loc, td->iloc, vec);
 
 		/* rotation */
+		if ((t->flag & T_V3D_ALIGN)==0) { // align mode doesn't rotate objects itself
 		
-		if(td->flag & TD_USEQUAT) {
-			Mat3MulSerie(fmat, td->mtx, mat, td->smtx, 0, 0, 0, 0, 0);
-			Mat3ToQuat(fmat, quat);	// Actual transform
-			
-			QuatMul(td->ext->quat, quat, td->ext->iquat);
-			/* this function works on end result */
-			protectedQuaternionBits(td->protectflag, td->ext->quat, td->ext->iquat);
-		}
-		else if ((t->flag & T_V3D_ALIGN)==0) {	// align mode doesn't rotate objects itself
-			float obmat[3][3];
-			
-			/* are there ipo keys? */
-			if(td->tdi) {
-				TransDataIpokey *tdi= td->tdi;
-				float rot[3];
+			if(td->flag & TD_USEQUAT) {
+				Mat3MulSerie(fmat, td->mtx, mat, td->smtx, 0, 0, 0, 0, 0);
+				Mat3ToQuat(fmat, quat);	// Actual transform
 				
-				/* calculate the total rotatation in eulers */
-				VecAddf(eul, td->ext->irot, td->ext->drot);
-				EulToMat3(eul, obmat);
-				/* mat = transform, obmat = object rotation */
-				Mat3MulMat3(fmat, mat, obmat);
-				Mat3ToEul(fmat, eul);
-				compatible_eul(eul, td->ext->irot);
-				
-				/* correct back for delta rot */
-				if(tdi->flag & TOB_IPODROT) {
-					VecSubf(rot, eul, td->ext->irot);
-				}
-				else {
-					VecSubf(rot, eul, td->ext->drot);
-				}
-				
-				VecMulf(rot, (float)(9.0/M_PI_2));
-				VecSubf(rot, rot, tdi->oldrot);
-				
-				protectedRotateBits(td->protectflag, rot, tdi->oldrot);
-				
-				add_tdi_poin(tdi->rotx, tdi->oldrot, rot[0]);
-				add_tdi_poin(tdi->roty, tdi->oldrot+1, rot[1]);
-				add_tdi_poin(tdi->rotz, tdi->oldrot+2, rot[2]);
+				QuatMul(td->ext->quat, quat, td->ext->iquat);
+				/* this function works on end result */
+				protectedQuaternionBits(td->protectflag, td->ext->quat, td->ext->iquat);
 			}
 			else {
-				Mat3MulMat3(totmat, mat, td->mtx);
-				Mat3MulMat3(smat, td->smtx, totmat);
+				float obmat[3][3];
 				
-				/* calculate the total rotatation in eulers */
-				VecAddf(eul, td->ext->irot, td->ext->drot); /* we have to correct for delta rot */
-				EulToMat3(eul, obmat);
-				/* mat = transform, obmat = object rotation */
-				Mat3MulMat3(fmat, smat, obmat);
-				Mat3ToEul(fmat, eul);
-				compatible_eul(eul, td->ext->irot);
-				
-				/* correct back for delta rot */
-				VecSubf(eul, eul, td->ext->drot);
-				
-				/* and apply */
-				protectedRotateBits(td->protectflag, eul, td->ext->irot);
-				VECCOPY(td->ext->rot, eul);
+				/* are there ipo keys? */
+				if(td->tdi) {
+					TransDataIpokey *tdi= td->tdi;
+					float rot[3];
+					
+					/* calculate the total rotatation in eulers */
+					VecAddf(eul, td->ext->irot, td->ext->drot);
+					EulToMat3(eul, obmat);
+					/* mat = transform, obmat = object rotation */
+					Mat3MulMat3(fmat, mat, obmat);
+					
+					Mat3ToCompatibleEul(fmat, eul, td->ext->irot);
+					
+					/* correct back for delta rot */
+					if(tdi->flag & TOB_IPODROT) {
+						VecSubf(rot, eul, td->ext->irot);
+					}
+					else {
+						VecSubf(rot, eul, td->ext->drot);
+					}
+					
+					VecMulf(rot, (float)(9.0/M_PI_2));
+					VecSubf(rot, rot, tdi->oldrot);
+					
+					protectedRotateBits(td->protectflag, rot, tdi->oldrot);
+					
+					add_tdi_poin(tdi->rotx, tdi->oldrot, rot[0]);
+					add_tdi_poin(tdi->roty, tdi->oldrot+1, rot[1]);
+					add_tdi_poin(tdi->rotz, tdi->oldrot+2, rot[2]);
+				}
+				else {
+					Mat3MulMat3(totmat, mat, td->mtx);
+					Mat3MulMat3(smat, td->smtx, totmat);
+					
+					/* calculate the total rotatation in eulers */
+					VecAddf(eul, td->ext->irot, td->ext->drot); /* we have to correct for delta rot */
+					EulToMat3(eul, obmat);
+					/* mat = transform, obmat = object rotation */
+					Mat3MulMat3(fmat, smat, obmat);
+					
+					Mat3ToCompatibleEul(fmat, eul, td->ext->irot);
+					
+					/* correct back for delta rot */
+					VecSubf(eul, eul, td->ext->drot);
+					
+					/* and apply */
+					protectedRotateBits(td->protectflag, eul, td->ext->irot);
+					VECCOPY(td->ext->rot, eul);
+				}
 			}
 		}
 	}
@@ -1790,7 +1880,7 @@ static void applyRotation(TransInfo *t, float angle, float axis[3])
 	}
 
 	VecRotToMat3(axis, angle, mat);
-
+	
 	for(i = 0 ; i < t->total; i++, td++) {
 
 		if (td->flag & TD_NOACTION)
@@ -1827,8 +1917,7 @@ static void applyRotation(TransInfo *t, float angle, float axis[3])
 
 int Rotation(TransInfo *t, short mval[2]) 
 {
-	TransData *td = t->data;
-	char str[50];
+	char str[64];
 
 	float final;
 
@@ -1880,6 +1969,8 @@ int Rotation(TransInfo *t, short mval[2])
 	if (t->con.applyRot) {
 		t->con.applyRot(t, NULL, axis);
 	}
+	
+	applySnapping(t, &final);
 
 	if (hasNumInput(&t->num)) {
 		char c[20];
@@ -1890,13 +1981,20 @@ int Rotation(TransInfo *t, short mval[2])
 
 		sprintf(str, "Rot: %s %s", &c[0], t->proptext);
 
+		/* Clamp between -180 and 180 */
+		while (final >= 180.0)
+			final -= 360.0;
+		
+		while (final <= -180.0)
+			final += 360.0;
+
 		final *= (float)(M_PI / 180.0);
 	}
 	else {
 		sprintf(str, "Rot: %.2f%s %s", 180.0*final/M_PI, t->con.text, t->proptext);
 	}
 
-	VecRotToMat3(axis, final * td->factor, mat);
+	VecRotToMat3(axis, final, mat);
 
 	t->val = final;				// used in manipulator
 	Mat3CpyMat3(t->mat, mat);	// used in manipulator
@@ -1963,7 +2061,7 @@ static void applyTrackball(TransInfo *t, float axis1[3], float axis2[3], float a
 
 int Trackball(TransInfo *t, short mval[2]) 
 {
-	char str[80];
+	char str[128];
 	float axis1[3], axis2[3];
 	float mat[3][3], totmat[3][3], smat[3][3];
 	float phi[2];
@@ -2148,6 +2246,7 @@ int Translation(TransInfo *t, short mval[2])
 
 	if (t->con.mode & CON_APPLY) {
 		float pvec[3] = {0.0f, 0.0f, 0.0f};
+		applySnapping(t, t->vec);
 		t->con.applyVec(t, NULL, t->vec, tvec, pvec);
 		VECCOPY(t->vec, tvec);
 		headerTranslation(t, pvec, str);
@@ -2155,9 +2254,10 @@ int Translation(TransInfo *t, short mval[2])
 	else {
 		snapGrid(t, t->vec);
 		applyNumInput(&t->num, t->vec);
+		applySnapping(t, t->vec);
 		headerTranslation(t, t->vec, str);
 	}
-
+	
 	applyTranslation(t, t->vec);
 
 	/* evil hack - redo translation if cliiping needeed */
@@ -2167,9 +2267,11 @@ int Translation(TransInfo *t, short mval[2])
 	recalcData(t);
 
 	headerprint(str);
-
+	
 	viewRedrawForce(t);
 
+	drawSnapping(t);
+
 	return 1;
 }
 
@@ -2198,7 +2300,7 @@ int ShrinkFatten(TransInfo *t, short mval[2])
 	float vec[3];
 	float distance;
 	int i;
-	char str[50];
+	char str[64];
 	TransData *td = t->data;
 
 	distance = -InputVerticalAbsolute(t, mval);
@@ -2328,6 +2430,85 @@ int Tilt(TransInfo *t, short mval[2])
 	return 1;
 }
 
+
+/* ******************** Curve Shrink/Fatten *************** */
+
+int CurveShrinkFatten(TransInfo *t, short mval[2]) 
+{
+	TransData *td = t->data;
+	float ratio;
+	int i;
+	char str[50];
+	
+	if(t->flag & T_SHIFT_MOD) {
+		/* calculate ratio for shiftkey pos, and for total, and blend these for precision */
+		float dx= (float)(t->center2d[0] - t->shiftmval[0]);
+		float dy= (float)(t->center2d[1] - t->shiftmval[1]);
+		ratio = (float)sqrt( dx*dx + dy*dy)/t->fac;
+		
+		dx= (float)(t->center2d[0] - mval[0]);
+		dy= (float)(t->center2d[1] - mval[1]);
+		ratio+= 0.1f*(float)(sqrt( dx*dx + dy*dy)/t->fac -ratio);
+		
+	}
+	else {
+		float dx= (float)(t->center2d[0] - mval[0]);
+		float dy= (float)(t->center2d[1] - mval[1]);
+		ratio = (float)sqrt( dx*dx + dy*dy)/t->fac;
+	}
+	
+	snapGrid(t, &ratio);
+	
+	applyNumInput(&t->num, &ratio);
+	
+	/* header print for NumInput */
+	if (hasNumInput(&t->num)) {
+		char c[20];
+		
+		outputNumInput(&(t->num), c);
+		sprintf(str, "Shrink/Fatten: %s", c);
+	}
+	else {
+		sprintf(str, "Shrink/Fatten: %3f", ratio);
+	}
+	
+	for(i = 0 ; i < t->total; i++, td++) {
+		if (td->flag & TD_NOACTION)
+			break;
+		
+		if(td->val) {
+			//*td->val= ratio;
+			*td->val= td->ival*ratio;
+			if (*td->val <= 0.0f) *td->val = 0.0001f;
+		}
+	}
+	
+	recalcData(t);
+	
+	headerprint(str);
+	
+	viewRedrawForce(t);
+	
+	if(!(t->flag & T_USES_MANIPULATOR)) helpline (t, t->center);
+	
+	return 1;
+}
+
+void initCurveShrinkFatten(TransInfo *t)
+{
+	t->idx_max = 0;
+	t->num.idx_max = 0;
+	t->snap[0] = 0.0f;
+	t->snap[1] = 0.1f;
+	t->snap[2] = t->snap[1] * 0.1f;
+	t->transform = CurveShrinkFatten;
+	t->fac = (float)sqrt( (
+		   ((float)(t->center2d[1] - t->imval[1]))*((float)(t->center2d[1] - t->imval[1]))
+		   +
+		   ((float)(t->center2d[0] - t->imval[0]))*((float)(t->center2d[0] - t->imval[0]))
+		   ) );
+}
+
 /* ************************** PUSH/PULL *************************** */
 
 void initPushPull(TransInfo *t) 
@@ -2347,7 +2528,7 @@ int PushPull(TransInfo *t, short mval[2])
 	float vec[3], axis[3];
 	float distance;
 	int i;
-	char str[50];
+	char str[128];
 	TransData *td = t->data;
 
 	distance = InputVerticalAbsolute(t, mval);
@@ -2498,6 +2679,284 @@ int Crease(TransInfo *t, short mval[2])
 	return 1;
 }
 
+/* ******************** EditBone (B-bone) width scaling *************** */
+
+void initBoneSize(TransInfo *t)
+{
+	t->idx_max = 0;
+	t->num.idx_max = 0;
+	t->snap[0] = 0.0f;
+	t->snap[1] = 0.1f;
+	t->snap[2] = t->snap[1] * 0.1f;
+	t->transform = BoneSize;
+	t->fac = (float)sqrt( (
+					   ((float)(t->center2d[1] - t->imval[1]))*((float)(t->center2d[1] - t->imval[1]))
+					   +
+					   ((float)(t->center2d[0] - t->imval[0]))*((float)(t->center2d[0] - t->imval[0]))
+					   ) );
+	
+	if(t->fac==0.0f) t->fac= 1.0f;	// prevent Inf
+}
+
+static void ElementBoneSize(TransInfo *t, TransData *td, float mat[3][3]) 
+{
+	float tmat[3][3], smat[3][3], oldy;
+	float sizemat[3][3];
+	
+	Mat3MulMat3(smat, mat, td->mtx);
+	Mat3MulMat3(tmat, td->smtx, smat);
+	
+	if (t->con.applySize) {
+		t->con.applySize(t, td, tmat);
+	}
+	
+	/* we've tucked the scale in loc */
+	oldy= td->iloc[1];
+	SizeToMat3(td->iloc, sizemat);
+	Mat3MulMat3(tmat, tmat, sizemat);
+	Mat3ToSize(tmat, td->loc);
+	td->loc[1]= oldy;
+}
+
+int BoneSize(TransInfo *t, short mval[2]) 
+{
+	TransData *td = t->data;
+	float size[3], mat[3][3];
+	float ratio;
+	int i;
+	char str[50];
+	
+	/* for manipulator, center handle, the scaling can't be done relative to center */
+	if( (t->flag & T_USES_MANIPULATOR) && t->con.mode==0) {
+		ratio = 1.0f - ((t->imval[0] - mval[0]) + (t->imval[1] - mval[1]))/100.0f;
+	}
+	else {
+		
+		if(t->flag & T_SHIFT_MOD) {
+			/* calculate ratio for shiftkey pos, and for total, and blend these for precision */
+			float dx= (float)(t->center2d[0] - t->shiftmval[0]);
+			float dy= (float)(t->center2d[1] - t->shiftmval[1]);
+			ratio = (float)sqrt( dx*dx + dy*dy)/t->fac;
+			
+			dx= (float)(t->center2d[0] - mval[0]);
+			dy= (float)(t->center2d[1] - mval[1]);
+			ratio+= 0.1f*(float)(sqrt( dx*dx + dy*dy)/t->fac -ratio);
+			
+		}
+		else {
+			float dx= (float)(t->center2d[0] - mval[0]);
+			float dy= (float)(t->center2d[1] - mval[1]);
+			ratio = (float)sqrt( dx*dx + dy*dy)/t->fac;
+		}
+		
+		/* flip scale, but not for manipulator center handle */
+		if	((t->center2d[0] - mval[0]) * (t->center2d[0] - t->imval[0]) + 
+			 (t->center2d[1] - mval[1]) * (t->center2d[1] - t->imval[1]) < 0)
+			ratio *= -1.0f;
+	}
+	
+	size[0] = size[1] = size[2] = ratio;
+	
+	snapGrid(t, size);
+	
+	if (hasNumInput(&t->num)) {
+		applyNumInput(&t->num, size);
+		constraintNumInput(t, size);
+	}
+	
+	SizeToMat3(size, mat);
+	
+	if (t->con.applySize) {
+		t->con.applySize(t, NULL, mat);
+	}
+	
+	Mat3CpyMat3(t->mat, mat);	// used in manipulator
+	
+	headerResize(t, size, str);
+	
+	for(i = 0 ; i < t->total; i++, td++) {
+		if (td->flag & TD_NOACTION)
+			break;
+		
+		ElementBoneSize(t, td, mat);
+	}
+	
+	recalcData(t);
+	
+	headerprint(str);
+	
+	viewRedrawForce(t);
+	
+	if(!(t->flag & T_USES_MANIPULATOR)) helpline (t, t->center);
+	
+	return 1;
+}
+
+
+/* ******************** EditBone envelope *************** */
+
+void initBoneEnvelope(TransInfo *t)
+{
+	t->idx_max = 0;
+	t->num.idx_max = 0;
+	t->snap[0] = 0.0f;
+	t->snap[1] = 0.1f;
+	t->snap[2] = t->snap[1] * 0.1f;
+	t->transform = BoneEnvelope;
+	t->fac = (float)sqrt( (
+						   ((float)(t->center2d[1] - t->imval[1]))*((float)(t->center2d[1] - t->imval[1]))
+						   +
+						   ((float)(t->center2d[0] - t->imval[0]))*((float)(t->center2d[0] - t->imval[0]))
+						   ) );
+	
+	if(t->fac==0.0f) t->fac= 1.0f;	// prevent Inf
+}
+
+int BoneEnvelope(TransInfo *t, short mval[2]) 
+{
+	TransData *td = t->data;
+	float ratio;
+	int i;
+	char str[50];
+	
+	if(t->flag & T_SHIFT_MOD) {
+		/* calculate ratio for shiftkey pos, and for total, and blend these for precision */
+		float dx= (float)(t->center2d[0] - t->shiftmval[0]);
+		float dy= (float)(t->center2d[1] - t->shiftmval[1]);
+		ratio = (float)sqrt( dx*dx + dy*dy)/t->fac;
+		
+		dx= (float)(t->center2d[0] - mval[0]);
+		dy= (float)(t->center2d[1] - mval[1]);
+		ratio+= 0.1f*(float)(sqrt( dx*dx + dy*dy)/t->fac -ratio);
+		
+	}
+	else {
+		float dx= (float)(t->center2d[0] - mval[0]);
+		float dy= (float)(t->center2d[1] - mval[1]);
+		ratio = (float)sqrt( dx*dx + dy*dy)/t->fac;
+	}
+	
+	snapGrid(t, &ratio);
+	
+	applyNumInput(&t->num, &ratio);
+	
+	/* header print for NumInput */
+	if (hasNumInput(&t->num)) {
+		char c[20];
+		
+		outputNumInput(&(t->num), c);
+		sprintf(str, "Envelope: %s", c);
+	}
+	else {
+		sprintf(str, "Envelope: %3f", ratio);
+	}
+	
+	for(i = 0 ; i < t->total; i++, td++) {
+		if (td->flag & TD_NOACTION)
+			break;
+		
+		if(td->val) *td->val= td->ival*ratio;
+	}
+	
+	recalcData(t);
+	
+	headerprint(str);
+	
+	force_draw(0);
+	
+	if(!(t->flag & T_USES_MANIPULATOR)) helpline (t, t->center);
+	
+	return 1;
+}
+
+
+/* ******************** EditBone roll *************** */
+
+void initBoneRoll(TransInfo *t)
+{
+	t->idx_max = 0;
+	t->num.idx_max = 0;
+	t->snap[0] = 0.0f;
+	t->snap[1] = (float)((5.0/180)*M_PI);
+	t->snap[2] = t->snap[1] * 0.2f;
+	
+	t->fac = 0.0f;
+	
+	t->transform = BoneRoll;
+}
+
+int BoneRoll(TransInfo *t, short mval[2]) 
+{
+	TransData *td = t->data;
+	int i;
+	char str[50];
+
+	float final;
+
+	int dx2 = t->center2d[0] - mval[0];
+	int dy2 = t->center2d[1] - mval[1];
+	double B = sqrt(dx2*dx2+dy2*dy2);
+
+	int dx1 = t->center2d[0] - t->imval[0];
+	int dy1 = t->center2d[1] - t->imval[1];
+	double A = sqrt(dx1*dx1+dy1*dy1);
+
+	int dx3 = mval[0] - t->imval[0];
+	int dy3 = mval[1] - t->imval[1];
+		/* use doubles here, to make sure a "1.0" (no rotation) doesnt become 9.999999e-01, which gives 0.02 for acos */
+	double deler= ((double)((dx1*dx1+dy1*dy1)+(dx2*dx2+dy2*dy2)-(dx3*dx3+dy3*dy3) ))
+		/ (2.0 * (A*B?A*B:1.0));
+	/* (A*B?A*B:1.0f) this takes care of potential divide by zero errors */
+
+	float dphi;
+	
+	dphi = saacos((float)deler);
+	if( (dx1*dy2-dx2*dy1)>0.0 ) dphi= -dphi;
+
+	if(G.qual & LR_SHIFTKEY) t->fac += dphi/30.0f;
+	else t->fac += dphi;
+
+	final = t->fac;
+
+	snapGrid(t, &final);
+
+	t->imval[0] = mval[0];
+	t->imval[1] = mval[1];
+
+	if (hasNumInput(&t->num)) {
+		char c[20];
+
+		applyNumInput(&t->num, &final);
+
+		outputNumInput(&(t->num), c);
+
+		sprintf(str, "Roll: %s", &c[0]);
+
+		final *= (float)(M_PI / 180.0);
+	}
+	else {
+		sprintf(str, "Roll: %.2f", 180.0*final/M_PI);
+	}
+	
+	/* set roll values */
+	for (i = 0; i < t->total; i++, td++) {  
+		if (td->flag & TD_NOACTION)
+			break;
+		
+		*(td->val) = td->ival - final;
+	}
+		
+	recalcData(t);
+
+	headerprint(str);
+
+	viewRedrawForce(t);
+
+	if(!(t->flag & T_USES_MANIPULATOR)) helpline (t, t->center);
+
+	return 1;
+}
+
 /* ************************** MIRROR *************************** */
 
 void Mirror(short mode) 
@@ -2597,196 +3056,6 @@ void Mirror(short mode)
 	viewRedrawPost(&Trans);
 }
 
-/* ******************** EditBone (B-bone) width scaling *************** */
-
-static void ElementBoneSize(TransInfo *t, TransData *td, float mat[3][3]) 
-{
-	float tmat[3][3], smat[3][3], oldy;
-	float sizemat[3][3];
-	
-	Mat3MulMat3(smat, mat, td->mtx);
-	Mat3MulMat3(tmat, td->smtx, smat);
-	
-	if (t->con.applySize) {
-		t->con.applySize(t, td, tmat);
-	}
-	
-	/* we've tucked the scale in loc */
-	oldy= td->iloc[1];
-	SizeToMat3(td->iloc, sizemat);
-	Mat3MulMat3(tmat, tmat, sizemat);
-	Mat3ToSize(tmat, td->loc);
-	td->loc[1]= oldy;
-}
-
-
-int BoneSize(TransInfo *t, short mval[2]) 
-{
-	TransData *td = t->data;
-	float size[3], mat[3][3];
-	float ratio;
-	int i;
-	char str[50];
-	
-	/* for manipulator, center handle, the scaling can't be done relative to center */
-	if( (t->flag & T_USES_MANIPULATOR) && t->con.mode==0) {
-		ratio = 1.0f - ((t->imval[0] - mval[0]) + (t->imval[1] - mval[1]))/100.0f;
-	}
-	else {
-		
-		if(t->flag & T_SHIFT_MOD) {
-			/* calculate ratio for shiftkey pos, and for total, and blend these for precision */
-			float dx= (float)(t->center2d[0] - t->shiftmval[0]);
-			float dy= (float)(t->center2d[1] - t->shiftmval[1]);
-			ratio = (float)sqrt( dx*dx + dy*dy)/t->fac;
-			
-			dx= (float)(t->center2d[0] - mval[0]);
-			dy= (float)(t->center2d[1] - mval[1]);
-			ratio+= 0.1f*(float)(sqrt( dx*dx + dy*dy)/t->fac -ratio);
-			
-		}
-		else {
-			float dx= (float)(t->center2d[0] - mval[0]);
-			float dy= (float)(t->center2d[1] - mval[1]);
-			ratio = (float)sqrt( dx*dx + dy*dy)/t->fac;
-		}
-		
-		/* flip scale, but not for manipulator center handle */
-		if	((t->center2d[0] - mval[0]) * (t->center2d[0] - t->imval[0]) + 
-			 (t->center2d[1] - mval[1]) * (t->center2d[1] - t->imval[1]) < 0)
-			ratio *= -1.0f;
-	}
-	
-	size[0] = size[1] = size[2] = ratio;
-	
-	snapGrid(t, size);
-	
-	if (hasNumInput(&t->num)) {
-		applyNumInput(&t->num, size);
-		constraintNumInput(t, size);
-	}
-	
-	SizeToMat3(size, mat);
-	
-	if (t->con.applySize) {
-		t->con.applySize(t, NULL, mat);
-	}
-	
-	Mat3CpyMat3(t->mat, mat);	// used in manipulator
-	
-	headerResize(t, size, str);
-	
-	for(i = 0 ; i < t->total; i++, td++) {
-		if (td->flag & TD_NOACTION)
-			break;
-		
-		ElementBoneSize(t, td, mat);
-	}
-	
-	recalcData(t);
-	
-	headerprint(str);
-	
-	viewRedrawForce(t);
-	
-	if(!(t->flag & T_USES_MANIPULATOR)) helpline (t, t->center);
-	
-	return 1;
-}
-
-void initBoneSize(TransInfo *t)
-{
-	t->idx_max = 0;
-	t->num.idx_max = 0;
-	t->snap[0] = 0.0f;
-	t->snap[1] = 0.1f;
-	t->snap[2] = t->snap[1] * 0.1f;
-	t->transform = BoneSize;
-	t->fac = (float)sqrt( (
-					   ((float)(t->center2d[1] - t->imval[1]))*((float)(t->center2d[1] - t->imval[1]))
-					   +
-					   ((float)(t->center2d[0] - t->imval[0]))*((float)(t->center2d[0] - t->imval[0]))
-					   ) );
-	
-	if(t->fac==0.0f) t->fac= 1.0f;	// prevent Inf
-}
-
-/* ******************** EditBone envelope *************** */
-
-int BoneEnvelope(TransInfo *t, short mval[2]) 
-{
-	TransData *td = t->data;
-	float ratio;
-	int i;
-	char str[50];
-	
-	if(t->flag & T_SHIFT_MOD) {
-		/* calculate ratio for shiftkey pos, and for total, and blend these for precision */
-		float dx= (float)(t->center2d[0] - t->shiftmval[0]);
-		float dy= (float)(t->center2d[1] - t->shiftmval[1]);
-		ratio = (float)sqrt( dx*dx + dy*dy)/t->fac;
-		
-		dx= (float)(t->center2d[0] - mval[0]);
-		dy= (float)(t->center2d[1] - mval[1]);
-		ratio+= 0.1f*(float)(sqrt( dx*dx + dy*dy)/t->fac -ratio);
-		
-	}
-	else {
-		float dx= (float)(t->center2d[0] - mval[0]);
-		float dy= (float)(t->center2d[1] - mval[1]);
-		ratio = (float)sqrt( dx*dx + dy*dy)/t->fac;
-	}
-	
-	snapGrid(t, &ratio);
-	
-	applyNumInput(&t->num, &ratio);
-	
-	/* header print for NumInput */
-	if (hasNumInput(&t->num)) {
-		char c[20];
-		
-		outputNumInput(&(t->num), c);
-		sprintf(str, "Envelope: %s", c);
-	}
-	else {
-		sprintf(str, "Envelope: %3f", ratio);
-	}
-	
-	for(i = 0 ; i < t->total; i++, td++) {
-		if (td->flag & TD_NOACTION)
-			break;
-		
-		if(td->val) *td->val= td->ival*ratio;
-	}
-	
-	recalcData(t);
-	
-	headerprint(str);
-	
-	force_draw(0);
-	
-	if(!(t->flag & T_USES_MANIPULATOR)) helpline (t, t->center);
-	
-	return 1;
-}
-
-void initBoneEnvelope(TransInfo *t)
-{
-	t->idx_max = 0;
-	t->num.idx_max = 0;
-	t->snap[0] = 0.0f;
-	t->snap[1] = 0.1f;
-	t->snap[2] = t->snap[1] * 0.1f;
-	t->transform = BoneEnvelope;
-	t->fac = (float)sqrt( (
-						   ((float)(t->center2d[1] - t->imval[1]))*((float)(t->center2d[1] - t->imval[1]))
-						   +
-						   ((float)(t->center2d[0] - t->imval[0]))*((float)(t->center2d[0] - t->imval[0]))
-						   ) );
-	
-	if(t->fac==0.0f) t->fac= 1.0f;	// prevent Inf
-}
-
 /* ************************************ */
 
 void BIF_TransformSetUndo(char *str)
