@@ -1,5 +1,5 @@
 /**
- * $Id: transform_constraints.c,v 1.68 2006/06/04 18:38:58 theeth Exp $
+ * $Id: transform_constraints.c,v 1.72 2006/12/19 22:25:07 theeth Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -70,6 +70,7 @@
 #include "BIF_resources.h"
 #include "BIF_mywindow.h"
 #include "BIF_gl.h"
+#include "BIF_glutil.h"
 
 #include "BKE_global.h"
 #include "BKE_utildefines.h"
@@ -240,24 +241,29 @@ static void applyAxisConstraintVec(TransInfo *t, TransData *td, float in[3], flo
 	VECCOPY(out, in);
 	if (!td && t->con.mode & CON_APPLY) {
 		Mat3MulVecfl(t->con.pmtx, out);
-		if (getConstraintSpaceDimension(t) == 2) {
-			if (out[0] != 0.0f || out[1] != 0.0f || out[2] != 0.0f) {
-				planeProjection(t, in, out);
+		
+		// With snap, a projection is alright, no need to correct for view alignment
+		if ((t->tsnap.status & SNAP_ON) == 0)
+			{
+			if (getConstraintSpaceDimension(t) == 2) {
+				if (out[0] != 0.0f || out[1] != 0.0f || out[2] != 0.0f) {
+					planeProjection(t, in, out);
+				}
 			}
-		}
-		else if (getConstraintSpaceDimension(t) == 1) {
-			float c[3];
-
-			if (t->con.mode & CON_AXIS0) {
-				VECCOPY(c, t->con.mtx[0]);
+			else if (getConstraintSpaceDimension(t) == 1) {
+				float c[3];
+	
+				if (t->con.mode & CON_AXIS0) {
+					VECCOPY(c, t->con.mtx[0]);
+				}
+				else if (t->con.mode & CON_AXIS1) {
+					VECCOPY(c, t->con.mtx[1]);
+				}
+				else if (t->con.mode & CON_AXIS2) {
+					VECCOPY(c, t->con.mtx[2]);
+				}
+				axisProjection(t, c, in, out);
 			}
-			else if (t->con.mode & CON_AXIS1) {
-				VECCOPY(c, t->con.mtx[1]);
-			}
-			else if (t->con.mode & CON_AXIS2) {
-				VECCOPY(c, t->con.mtx[2]);
-			}
-			axisProjection(t, c, in, out);
 		}
 		postConstraintChecks(t, out, pvec);
 	}
@@ -697,7 +703,9 @@ void BIF_drawPropCircle()
 		mygetmatrix(tmat);
 		Mat4Invert(imat, tmat);
 
- 		drawcircball(GL_LINE_LOOP, t->center, t->propsize, imat);
+		set_inverted_drawing(1);
+		drawcircball(GL_LINE_LOOP, t->center, t->propsize, imat);
+		set_inverted_drawing(0);
 		
 		/* if editmode we restore */
 		if(G.obedit) myloadmatrix(G.vd->viewmat);
@@ -714,7 +722,6 @@ void BIF_getPropCenter(float *center)
 	else
 		center[0] = center[1] = center[2] = 0.0f;
 }
-
 static void drawObjectConstraint(TransInfo *t) {
 	int i;
 	TransData * td = t->data;
