@@ -1,5 +1,5 @@
 /**
- * $Id: transform_conversions.c,v 1.78 2006/06/24 14:16:36 blendix Exp $
+ * $Id: transform_conversions.c,v 1.91 2006/12/07 09:31:51 aligorith Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -268,6 +268,11 @@ static void createTransTexspace(TransInfo *t)
 	ID *id;
 	
 	ob= OBACT;
+	
+	if (ob==NULL) { // Should logically happen, but still...
+		t->total = 0;
+		return;
+	}
 
 	id= ob->data;
 	if(id==NULL || !ELEM3( GS(id->name), ID_ME, ID_CU, ID_MB )) {
@@ -612,12 +617,13 @@ static void set_pose_transflags(TransInfo *t, Object *ob)
 	
 	/* make sure no bone can be transformed when a parent is transformed */
 	/* since pchans are depsgraph sorted, the parents are in beginning of list */
-	for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
-		bone= pchan->bone;
-		if(bone->flag & BONE_TRANSFORM)
-			bone_children_clear_transflag(&bone->childbase);
-	}
-	
+	if(t->mode!=TFM_BONESIZE) {
+		for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
+			bone= pchan->bone;
+			if(bone->flag & BONE_TRANSFORM)
+				bone_children_clear_transflag(&bone->childbase);
+		}
+	}	
 	/* now count, and check if we have autoIK or have to switch from translate to rotate */
 	for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
 		bone= pchan->bone;
@@ -665,6 +671,10 @@ static void pose_grab_with_ik_add(bPoseChannel *pchan)
 	bKinematicConstraint *data;
 	bConstraint *con;
 	
+	if (pchan == NULL) { // Sanity check
+		return;
+	}
+	
 	/* rule: not if there's already an IK on this channel */
 	for(con= pchan->constraints.first; con; con= con->next)
 		if(con->type==CONSTRAINT_TYPE_KINEMATIC)
@@ -716,11 +726,13 @@ static void pose_grab_with_ik_children(bPose *pose, Bone *bone)
 /* main call which adds temporal IK chains */
 static void pose_grab_with_ik(Object *ob)
 {
-	bArmature *arm= ob->data;
+	bArmature *arm;
 	bPoseChannel *pchan, *pchansel= NULL;
 	
 	if(ob==NULL || ob->pose==NULL || (ob->flag & OB_POSEMODE)==0)
 		return;
+		
+	arm = ob->data;
 	
 	/* rule: only one Bone */
 	for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
@@ -824,6 +836,10 @@ static void createTransArmatureVerts(TransInfo *t)
 				if (ebo->flag & BONE_SELECTED)
 					t->total++;
 			}
+			else if (t->mode==TFM_BONE_ROLL) {
+				if (ebo->flag & BONE_SELECTED)
+					t->total++;
+			}
 			else {
 				if (ebo->flag & BONE_TIPSEL)
 					t->total++;
@@ -910,6 +926,21 @@ static void createTransArmatureVerts(TransInfo *t)
 					td++;
 				}
 			}
+			else if (t->mode==TFM_BONE_ROLL) {
+				if (ebo->flag & BONE_SELECTED) {
+					td->loc= NULL;
+					td->val= &(ebo->roll);
+					td->ival= ebo->roll;
+					
+					VECCOPY (td->center, ebo->head);
+					td->flag= TD_SELECTED;
+
+					td->ext = NULL;
+					td->tdi = NULL;
+					
+					td++;
+				}
+			}
 			else {
 				if (ebo->flag & BONE_TIPSEL){
 					VECCOPY (td->iloc, ebo->tail);
@@ -1139,8 +1170,14 @@ static void createTransCurveVerts(TransInfo *t)
 						else td->flag= 0;
 						td->ext = NULL;
 						td->tdi = NULL;
-						td->val = &(bezt->alfa);
-						td->ival = bezt->alfa;
+						
+						if (t->mode==TFM_CURVE_SHRINKFATTEN) {
+							td->val = &(bezt->radius);
+							td->ival = bezt->radius;
+						} else {
+							td->val = &(bezt->alfa);
+							td->ival = bezt->alfa;
+						}
 
 						Mat3CpyMat3(td->smtx, smtx);
 						Mat3CpyMat3(td->mtx, mtx);
@@ -1188,8 +1225,14 @@ static void createTransCurveVerts(TransInfo *t)
 						else td->flag= 0;
 						td->ext = NULL;
 						td->tdi = NULL;
-						td->val = &(bp->alfa);
-						td->ival = bp->alfa;
+						
+						if (t->mode==TFM_CURVE_SHRINKFATTEN) {
+							td->val = &(bp->radius);
+							td->ival = bp->radius;
+						} else {
+							td->val = &(bp->alfa);
+							td->ival = bp->alfa;
+						}
 
 						Mat3CpyMat3(td->smtx, smtx);
 						Mat3CpyMat3(td->mtx, mtx);
@@ -1272,6 +1315,7 @@ static void createTransLatticeVerts(TransInfo *t)
 /* proportional distance based on connectivity  */
 #define E_VEC(a)	(vectors + (3 * (a)->tmp.l))
 #define E_NEAR(a)	(nears[((a)->tmp.l)])
+#define THRESHOLD	0.0001f
 static void editmesh_set_connectivity_distance(int total, float *vectors, EditVert **nears)
 {
 	EditMesh *em = G.editMesh;
@@ -1327,12 +1371,14 @@ static void editmesh_set_connectivity_distance(int total, float *vectors, EditVe
 						if (v2->f2 != 2) {
 							VecSubf(nvec, v2->co, E_NEAR(v1)->co);
 							lenn = VecLength(nvec);
-							if (lenn - len1 > 0.00001f && len2 - lenn > 0.00001f) {
+							/* 1 < n < 2 */
+							if (lenn - len1 > THRESHOLD && len2 - lenn > THRESHOLD) {
 								VECCOPY(vec2, nvec);
 								E_NEAR(v2) = E_NEAR(v1);
 								done = 1;
 							}
-							else if (len2 - len1 > 0.00001f && len1 - lenn > 0.00001f) {
+							/* n < 1 < 2 */
+							else if (len2 - len1 > THRESHOLD && len1 - lenn > THRESHOLD) {
 								VECCOPY(vec2, vec1);
 								E_NEAR(v2) = E_NEAR(v1);
 								done = 1;
@@ -1342,12 +1388,14 @@ static void editmesh_set_connectivity_distance(int total, float *vectors, EditVe
 						if (v1->f2 != 2) {
 							VecSubf(nvec, v1->co, E_NEAR(v2)->co);
 							lenn = VecLength(nvec);
-							if (lenn - len2 > 0.00001f && len1 - lenn > 0.00001f) {
+							/* 2 < n < 1 */
+							if (lenn - len2 > THRESHOLD && len1 - lenn > THRESHOLD) {
 								VECCOPY(vec1, nvec);
 								E_NEAR(v1) = E_NEAR(v2);
 								done = 1;
 							}
-							else if (len1 - len2 > 0.00001f && len2 - lenn > 0.00001f) {
+							/* n < 2 < 1 */
+							else if (len1 - len2 > THRESHOLD && len2 - lenn > THRESHOLD) {
 								VECCOPY(vec1, vec2);
 								E_NEAR(v1) = E_NEAR(v2);
 								done = 1;
@@ -1357,7 +1405,8 @@ static void editmesh_set_connectivity_distance(int total, float *vectors, EditVe
 					else {
 						v2->f2 = 1;
 						VecSubf(vec2, v2->co, E_NEAR(v1)->co);
-						if (VecLength(vec1) - VecLength(vec2) > 0.00001f) {
+						/* 2 < 1 */
+						if (VecLength(vec1) - VecLength(vec2) > THRESHOLD) {
 							VECCOPY(vec2, vec1);
 						}
 						E_NEAR(v2) = E_NEAR(v1);
@@ -1367,7 +1416,8 @@ static void editmesh_set_connectivity_distance(int total, float *vectors, EditVe
 				else if (v2->f2) {
 					v1->f2 = 1;
 					VecSubf(vec1, v1->co, E_NEAR(v2)->co);
-					if (VecLength(vec2) - VecLength(vec1) > 0.00001f) {
+					/* 2 < 1 */
+					if (VecLength(vec2) - VecLength(vec1) > THRESHOLD) {
 						VECCOPY(vec1, vec2);
 					}
 					E_NEAR(v1) = E_NEAR(v2);
@@ -1416,6 +1466,15 @@ static void VertsToTransData(TransData *td, EditVert *eve)
 	td->tdi = NULL;
 	td->val = NULL;
 	td->tdmir= NULL;
+
+#ifdef WITH_VERSE
+	if(eve->vvert) {
+		td->verse = (void*)eve->vvert;
+		td->flag |= TD_VERSE_VERT;
+	}
+	else
+		td->flag &= ~TD_VERSE_VERT;
+#endif
 }
 
 /* *********************** CrazySpace correction. Now without doing subsurf optimal ****************** */
@@ -1451,7 +1510,6 @@ static float *get_crazy_mapped_editverts(void)
 	DerivedMesh *dm;
 	ModifierData *md;
 	float *vertexcos;
-	int needsFree;
 	int i;
 	
 	for( i = 0, md=G.obedit->modifiers.first; md; ++i, md=md->next ) {
@@ -1463,16 +1521,16 @@ static float *get_crazy_mapped_editverts(void)
 		/* this call disables subsurf and enables the underlying modifier to deform, apparently */
 		modifiers_setOnCage(G.obedit, md);
 		/* make it all over */
-		makeDispListMesh(G.obedit);
+		makeDerivedMesh(G.obedit, CD_MASK_BAREMESH);
 	}
 	
 	/* now get the cage */
-	dm= editmesh_get_derived_cage(&needsFree);
+	dm= editmesh_get_derived_cage(CD_MASK_BAREMESH);
 
 	vertexcos= MEM_mallocN(3*sizeof(float)*G.totvert, "vertexcos map");
 	dm->foreachMappedVert(dm, make_vertexcos__mapFunc, vertexcos);
 	
-	if (needsFree) dm->release(dm);
+	dm->release(dm);
 	
 	if(md) {
 		/* set back the flag, no new cage needs to be built, transform does it */
@@ -1754,8 +1812,8 @@ static void createTransUVs(TransInfo *t)
 	TransData *td = NULL;
 	TransData2D *td2d = NULL;
 	Mesh *me;
-	TFace *tf;
 	MFace *mf;
+	MTFace *tf;
 	int a, count=0, countsel=0;
 	int propmode = t->flag & T_PROP_EDIT;
 	
@@ -1763,7 +1821,7 @@ static void createTransUVs(TransInfo *t)
 	me= get_mesh(OBACT);
 
 	/* count */
-	tf= me->tface;
+	tf= me->mtface;
 	mf= me->mface;
 	for(a=me->totface; a>0; a--, tf++, mf++) {
 		if(mf->v3 && tf->flag & TF_SELECT) {
@@ -1790,7 +1848,7 @@ static void createTransUVs(TransInfo *t)
 
 	td= t->data;
 	td2d= t->data2d;
-	tf= me->tface;
+	tf= me->mtface;
 	mf= me->mface;
 	for(a=me->totface; a>0; a--, tf++, mf++) {
 		if(mf->v3 && tf->flag & TF_SELECT) {
@@ -1834,7 +1892,8 @@ void flushTransUVs(TransInfo *t)
 		}
 	}
 
-	if((G.sima->flag & SI_BE_SQUARE) && (t->state != TRANS_CANCEL))
+	/* always call this, also for cancel (it transforms non-selected vertices...) */
+	if((G.sima->flag & SI_BE_SQUARE))
 		be_square_tface_uv(me);
 
 	/* this is overkill if G.sima->lock is not set, but still needed */
@@ -2039,6 +2098,14 @@ static void ObjectToTransData(TransData *td, Object *ob)
 		Mat3One(td->smtx);
 		Mat3One(td->mtx);
 	}
+#ifdef WITH_VERSE
+	if(ob->vnode) {
+		td->verse = (void*)ob;
+		td->flag |= TD_VERSE_OBJECT;
+	}
+	else
+		td->flag &= ~TD_VERSE_OBJECT;
+#endif
 }
 
 
@@ -2082,8 +2149,8 @@ static void set_trans_object_base_flags(TransInfo *t)
 			ob->recalc |= OB_RECALC_OB;
 		}
 	}
-	/* all recalc flags get flushed */
-	DAG_scene_flush_update(G.scene, screen_view3d_layers());
+	/* all recalc flags get flushed to all layers, so a layer flip later on works fine */
+	DAG_scene_flush_update(G.scene, -1);
 	
 	/* and we store them temporal in base (only used for transform code) */
 	/* this because after doing updates, the object->recalc is cleared */
@@ -2169,15 +2236,31 @@ void special_aftertrans_update(TransInfo *t)
 
 							if (achan->ipo && !strcmp (achan->name, pchan->name)){
 								for (icu = achan->ipo->curve.first; icu; icu=icu->next){
-									insertkey(&ob->id, ID_PO, pchan->name, NULL, icu->adrcode);
+									if (U.uiflag & USER_KEYINSERTNEED)
+										insertkey_smarter(&ob->id, ID_PO, pchan->name, NULL, icu->adrcode);
+									else
+										insertkey(&ob->id, ID_PO, pchan->name, NULL, icu->adrcode);
 								}
 
 								break;
 							}
 						}
 					}
-					else{
+					else if (U.uiflag & USER_KEYINSERTNEED) {
+						insertkey_smarter(&ob->id, ID_PO, pchan->name, NULL, AC_LOC_X);
+						insertkey_smarter(&ob->id, ID_PO, pchan->name, NULL, AC_LOC_Y);
+						insertkey_smarter(&ob->id, ID_PO, pchan->name, NULL, AC_LOC_Z);
 
+						insertkey_smarter(&ob->id, ID_PO, pchan->name, NULL, AC_QUAT_W);
+						insertkey_smarter(&ob->id, ID_PO, pchan->name, NULL, AC_QUAT_X);
+						insertkey_smarter(&ob->id, ID_PO, pchan->name, NULL, AC_QUAT_Y);
+						insertkey_smarter(&ob->id, ID_PO, pchan->name, NULL, AC_QUAT_Z);
+
+						insertkey_smarter(&ob->id, ID_PO, pchan->name, NULL, AC_SIZE_X);
+						insertkey_smarter(&ob->id, ID_PO, pchan->name, NULL, AC_SIZE_Y);
+						insertkey_smarter(&ob->id, ID_PO, pchan->name, NULL, AC_SIZE_Z);
+					}
+					else {
 						insertkey(&ob->id, ID_PO, pchan->name, NULL, AC_SIZE_X);
 						insertkey(&ob->id, ID_PO, pchan->name, NULL, AC_SIZE_Y);
 						insertkey(&ob->id, ID_PO, pchan->name, NULL, AC_SIZE_Z);
@@ -2238,13 +2321,32 @@ void special_aftertrans_update(TransInfo *t)
 						icu= base->object->ipo->curve.first;
 						while(icu) {
 							icu->flag &= ~IPO_SELECT;
-							insertkey(id, ID_OB, actname, NULL, icu->adrcode);
+							if (U.uiflag & USER_KEYINSERTNEED)
+								insertkey_smarter(id, ID_OB, actname, NULL, icu->adrcode);
+							else
+								insertkey(id, ID_OB, actname, NULL, icu->adrcode);
 							icu= icu->next;
 						}
 					}
 				}
+				else if (U.uiflag & USER_KEYINSERTNEED) {
+					if (t->mode==TFM_RESIZE) {
+						insertkey_smarter(&base->object->id, ID_OB, actname, NULL, OB_SIZE_X);
+						insertkey_smarter(&base->object->id, ID_OB, actname, NULL, OB_SIZE_Y);
+						insertkey_smarter(&base->object->id, ID_OB, actname, NULL, OB_SIZE_Z);
+					}
+					else if (t->mode==TFM_ROTATION) {
+						insertkey_smarter(&base->object->id, ID_OB, actname, NULL, OB_ROT_X);
+						insertkey_smarter(&base->object->id, ID_OB, actname, NULL, OB_ROT_Y);
+						insertkey_smarter(&base->object->id, ID_OB, actname, NULL, OB_ROT_Z);
+					}
+					else if (t->mode==TFM_TRANSLATION) {
+						insertkey_smarter(&base->object->id, ID_OB, actname, NULL, OB_LOC_X);
+						insertkey_smarter(&base->object->id, ID_OB, actname, NULL, OB_LOC_Y);
+						insertkey_smarter(&base->object->id, ID_OB, actname, NULL, OB_LOC_Z);
+					}
+				}
 				else {
-
 					insertkey(&base->object->id, ID_OB, actname, NULL, OB_ROT_X);
 					insertkey(&base->object->id, ID_OB, actname, NULL, OB_ROT_Y);
 					insertkey(&base->object->id, ID_OB, actname, NULL, OB_ROT_Z);
@@ -2353,7 +2455,12 @@ static void createTransObject(TransInfo *t)
 					ipoflag= ob->ipoflag;
 					ob->ipoflag &= ~OB_OFFS_OB;
 					
-					pushdata(ob->loc, 7*3*4); // tsk! tsk!
+					/*
+					 * This is really EVIL code that pushes down Object values
+					 * (loc, dloc, orig, size, dsize, rot, drot)
+					 * */
+					 
+					pushdata((void*)ob->loc, 7 * 3 * sizeof(float)); // tsk! tsk!
 					
 					for(ik= elems.first; ik; ik= ik->next) {
 						
