@@ -1,5 +1,5 @@
 /**
- * $Id: transform_generics.c,v 1.49 2005/06/05 13:50:21 theeth Exp $
+ * $Id: transform_generics.c,v 1.101 2007/02/05 18:00:03 theeth Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -30,73 +30,67 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
 
-#include <stdlib.h>
 #include <string.h>
 #include <math.h>
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#ifndef WIN32
-#include <unistd.h>
-#else
-#include <io.h>
-#endif
-
 #include "MEM_guardedalloc.h"
 
 #include "DNA_action_types.h"
 #include "DNA_armature_types.h"
-#include "DNA_camera_types.h"
+#include "DNA_constraint_types.h"
 #include "DNA_curve_types.h"
-#include "DNA_effect_types.h"
-#include "DNA_ika_types.h"
-#include "DNA_image_types.h"
-#include "DNA_ipo_types.h"
-#include "DNA_key_types.h"
-#include "DNA_lamp_types.h"
 #include "DNA_lattice_types.h"
 #include "DNA_mesh_types.h"
-#include "DNA_meshdata_types.h"
-#include "DNA_meta_types.h"
+#include "DNA_modifier_types.h"
 #include "DNA_object_types.h"
-#include "DNA_scene_types.h"
+#include "DNA_object_force.h"
 #include "DNA_screen_types.h"
-#include "DNA_texture_types.h"
-#include "DNA_view3d_types.h"
-#include "DNA_world_types.h"
+#include "DNA_space_types.h"
+#include "DNA_scene_types.h"
 #include "DNA_userdef_types.h"
-#include "DNA_property_types.h"
-#include "DNA_vfont_types.h"
-#include "DNA_constraint_types.h"
+#include "DNA_view3d_types.h"
 
 #include "BIF_screen.h"
-#include "BIF_space.h"
-#include "BIF_editview.h"
 #include "BIF_resources.h"
 #include "BIF_mywindow.h"
 #include "BIF_gl.h"
-#include "BIF_editlattice.h"
 #include "BIF_editarmature.h"
 #include "BIF_editmesh.h"
+#include "BIF_editsima.h"
+#include "BIF_meshtools.h"
+#include "BIF_retopo.h"
+
+#ifdef WITH_VERSE
+#include "BIF_verse.h"
+#endif
 
 #include "BKE_action.h"
 #include "BKE_anim.h"
 #include "BKE_armature.h"
 #include "BKE_curve.h"
+#include "BKE_depsgraph.h"
 #include "BKE_displist.h"
+#include "BKE_depsgraph.h"
 #include "BKE_global.h"
+#include "BKE_group.h"
 #include "BKE_ipo.h"
 #include "BKE_lattice.h"
+#include "BKE_mesh.h"
+#include "BKE_modifier.h"
 #include "BKE_object.h"
 #include "BKE_utildefines.h"
 
-#include "BSE_edit.h"
+#ifdef WITH_VERSE
+#include "BKE_verse.h"
+#endif
+
 #include "BSE_view.h"
+#include "BDR_unwrapper.h"
 
 #include "BLI_arithb.h"
+#include "BLI_blenlib.h"
 #include "BLI_editVert.h"
+#include "BLI_rand.h"
 
 #include "blendef.h"
 
@@ -107,14 +101,16 @@
 extern ListBase editNurb;
 extern ListBase editelems;
 
-/* GLOBAL VARIABLE THAT SHOULD MOVED TO SCREEN MEMBER OR SOMETHING  */
-extern TransInfo Trans;
+extern TransInfo Trans;	/* From transform.c */
 
 /* ************************** Functions *************************** */
 
 
-void getViewVector(float coord[3], float vec[3]) {
-	if (G.vd->persp)
+void getViewVector(float coord[3], float vec[3])
+{
+	TransInfo *t = BIF_GetTransInfo();
+
+	if (t->persp)
 	{
 		float p1[4], p2[4];
 
@@ -122,210 +118,262 @@ void getViewVector(float coord[3], float vec[3]) {
 		p1[3] = 1.0f;
 		VECCOPY(p2, p1);
 		p2[3] = 1.0f;
-		Mat4MulVec4fl(G.vd->viewmat, p2);
+		Mat4MulVec4fl(t->viewmat, p2);
 
 		p2[0] = 2.0f * p2[0];
 		p2[1] = 2.0f * p2[1];
 		p2[2] = 2.0f * p2[2];
 
-		Mat4MulVec4fl(G.vd->viewinv, p2);
+		Mat4MulVec4fl(t->viewinv, p2);
 
 		VecSubf(vec, p1, p2);
 	}
 	else {
-		VECCOPY(vec, G.vd->viewinv[2]);
+		VECCOPY(vec, t->viewinv[2]);
 	}
 	Normalise(vec);
 }
 
 /* ************************** GENERICS **************************** */
 
-static int pose_flags_reset_done(Object *ob) {
-	/* Clear the constraint done status for every pose channe;
-	* that has been flagged as needing constant updating
-	*/
-	bPoseChannel *chan;
-	int numreset = 0;
-	
-	if (ob->pose) {
-		for (chan = ob->pose->chanbase.first; chan; chan=chan->next){
-			if (chan->flag & PCHAN_TRANS_UPDATE) {
-				chan->flag &= ~PCHAN_DONE;
-				numreset++;
+static void clipMirrorModifier(TransInfo *t, Object *ob)
+{
+	ModifierData *md= ob->modifiers.first;
+	float tolerance[3];
+	int axis = 0;
+
+	for (; md; md=md->next) {
+		if (md->type==eModifierType_Mirror) {
+			MirrorModifierData *mmd = (MirrorModifierData*) md;	
+		
+			if(mmd->flag & MOD_MIR_CLIPPING) {
+				switch(mmd->axis){
+					case 0:
+						axis |= 1;
+						tolerance[0] = mmd->tolerance;
+						break;
+					case 1:
+						axis |= 2;
+						tolerance[1] = mmd->tolerance;
+						break;
+					case 2:
+						axis |= 4;
+						tolerance[2] = mmd->tolerance;
+				}
+			}
+		}
+	}
+	if (axis) {
+		TransData *td = t->data;
+		int i;
+		
+		for(i = 0 ; i < t->total; i++, td++) {
+			if (td->flag & TD_NOACTION)
+				break;
+			if (td->loc==NULL)
+				break;
+			
+			if(axis & 1) {
+				if(fabs(td->iloc[0])<=tolerance[0] || td->loc[0]*td->iloc[0]<0.0f) td->loc[0]= 0.0f;
+			}
+			
+			if(axis & 2) {
+				if(fabs(td->iloc[1])<=tolerance[1] || td->loc[1]*td->iloc[1]<0.0f) td->loc[1]= 0.0f;
+			}
+			if(axis & 4) {
+				if(fabs(td->iloc[2])<=tolerance[2] || td->loc[2]*td->iloc[2]<0.0f) td->loc[2]= 0.0f;
 			}
-			
 		}
 	}
-	return numreset;
 }
 
+/* assumes G.obedit set to mesh object */
+static void editmesh_apply_to_mirror(TransInfo *t)
+{
+	TransData *td = t->data;
+	EditVert *eve;
+	int i;
+	
+	for(i = 0 ; i < t->total; i++, td++) {
+		if (td->flag & TD_NOACTION)
+			break;
+		if (td->loc==NULL)
+			break;
+		
+		eve= td->tdmir;
+		if(eve) {
+			eve->co[0]= -td->loc[0];
+			eve->co[1]= td->loc[1];
+			eve->co[2]= td->loc[2];
+		}		
+	}		
+}
 
-/* called for objects updating while transform acts, once per redraw */
+/* called for updating while transform acts, once per redraw */
 void recalcData(TransInfo *t)
 {
 	Base *base;
-	
-	if(G.obpose) {
-		TransData *td= t->data;
-		bPoseChannel	*chan;
-		int	i;
+#ifdef WITH_VERSE
+	struct TransData *td;
+#endif
 		
-		if (!G.obpose->pose) G.obpose->pose= MEM_callocN(sizeof(bPose), "pose");
-		
-		/*	Make channels for the transforming bones (in posemode) */
-		for (i=0; i<t->total; i++, td++) {
-			chan = MEM_callocN (sizeof (bPoseChannel), "transPoseChannel");
-			
-			if (t->mode == TFM_ROTATION || t->mode == TFM_TRACKBALL) {
-				chan->flag |= POSE_ROT;
-				memcpy (chan->quat, td->ext->quat, sizeof (chan->quat));
-			}
-			if (t->mode == TFM_TRANSLATION) {
-				chan->flag |= POSE_LOC;
-				memcpy (chan->loc, td->loc, sizeof (chan->loc));
-			}
-			if (t->mode == TFM_RESIZE) {
-				chan->flag |= POSE_SIZE;
-				memcpy (chan->size, td->ext->size, sizeof (chan->size));
-			}
-			
-			strcpy (chan->name, ((Bone*) td->ext->bone)->name);
-			
-			set_pose_channel (G.obpose->pose, chan);
-
-		}
-		
-		clear_pose_constraint_status(G.obpose);
-		
-		if (!is_delay_deform()) make_displists_by_armature(G.obpose);
-		
-	}
-	else if (G.obedit) {
+	if (G.obedit) {
 		if (G.obedit->type == OB_MESH) {
+			retopo_do_all();
+
+			/* mirror modifier clipping? */
+			if(t->state != TRANS_CANCEL)
+				clipMirrorModifier(t, G.obedit);
+			
+			if(G.scene->toolsettings->editbutflag & B_MESH_X_MIRROR)
+				editmesh_apply_to_mirror(t);
+			
+			DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);  /* sets recalc flags */
+			
 			recalc_editnormals();
-			makeDispList(G.obedit);
 		}
 		else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) {
 			Nurb *nu= editNurb.first;
+			DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);  /* sets recalc flags */
+			
 			while(nu) {
 				test2DNurb(nu);
 				testhandlesNurb(nu); /* test for bezier too */
 				nu= nu->next;
 			}
-			makeDispList(G.obedit);
 
-			makeBevelList(G.obedit); // might be needed for deform
-			calc_curvepath(G.obedit);
-			
-			// deform, bevel, taper
-			base= FIRSTBASE;
-			while(base) {
-				if(base->lay & G.vd->lay) {
-					if(base->object->parent==G.obedit && base->object->partype==PARSKEL)
-						makeDispList(base->object);
-					else if(base->object->type==OB_CURVE) {
-						Curve *cu= base->object->data;
-						if(G.obedit==cu->bevobj || G.obedit==cu->taperobj)
-							makeDispList(base->object);
-					}
-				}
-				base= base->next;
-			}
+			retopo_do_all();
 		}
-		else if(G.obedit->type==OB_ARMATURE){
+		else if(G.obedit->type==OB_ARMATURE){   /* no recalc flag, does pose */
+			bArmature *arm= G.obedit->data;
 			EditBone *ebo;
 			
 			/* Ensure all bones are correctly adjusted */
 			for (ebo=G.edbo.first; ebo; ebo=ebo->next){
 				
-				if ((ebo->flag & BONE_IK_TOPARENT) && ebo->parent){
+				if ((ebo->flag & BONE_CONNECTED) && ebo->parent){
 					/* If this bone has a parent tip that has been moved */
 					if (ebo->parent->flag & BONE_TIPSEL){
 						VECCOPY (ebo->head, ebo->parent->tail);
+						if(t->mode==TFM_BONE_ENVELOPE) ebo->rad_head= ebo->parent->rad_tail;
 					}
 					/* If this bone has a parent tip that has NOT been moved */
 					else{
 						VECCOPY (ebo->parent->tail, ebo->head);
+						if(t->mode==TFM_BONE_ENVELOPE) ebo->parent->rad_tail= ebo->rad_head;
 					}
 				}
+				
+				/* on extrude bones, oldlength==0.0f, so we scale radius of points */
+				ebo->length= VecLenf(ebo->head, ebo->tail);
+				if(ebo->oldlength==0.0f) {
+					ebo->rad_head= 0.25f*ebo->length;
+					ebo->rad_tail= 0.10f*ebo->length;
+					ebo->dist= 0.25f*ebo->length;
+					if(ebo->parent) {
+						if(ebo->rad_head > ebo->parent->rad_tail)
+							ebo->rad_head= ebo->parent->rad_tail;
+					}
+				}
+				else if(t->mode!=TFM_BONE_ENVELOPE) {
+					/* if bones change length, lets do that for the deform distance as well */
+					ebo->dist*= ebo->length/ebo->oldlength;
+					ebo->rad_head*= ebo->length/ebo->oldlength;
+					ebo->rad_tail*= ebo->length/ebo->oldlength;
+					ebo->oldlength= ebo->length;
+				}
 			}
+			if(arm->flag & ARM_MIRROR_EDIT) 
+				transform_armature_mirror_update();
+			
 		}
 		else if(G.obedit->type==OB_LATTICE) {
+			DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);  /* sets recalc flags */
 			
 			if(editLatt->flag & LT_OUTSIDE) outside_lattice(editLatt);
+		}
+		else {
+			DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);  /* sets recalc flags */
+		}
+	}
+	else if( (t->flag & T_POSE) && t->poseobj) {
+		Object *ob= t->poseobj;
+		bArmature *arm= ob->data;
+		
+		/* old optimize trick... this enforces to bypass the depgraph */
+		if (!(arm->flag & ARM_DELAYDEFORM)) {
+			DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);  /* sets recalc flags */
 			
-			base= FIRSTBASE;
-			while(base) {
-				if(base->lay & G.vd->lay) {
-					if(base->object->parent==G.obedit) {
-						makeDispList(base->object);
-					}
+			/* bah, softbody exception... recalcdata doesnt reset */
+			for(base= FIRSTBASE; base; base= base->next) {
+				if(base->object->recalc & OB_RECALC_DATA)
+					if(modifiers_isSoftbodyEnabled(base->object)) {
+						base->object->softflag |= OB_SB_REDO;
 				}
-				base= base->next;
 			}
 		}
-		else if (G.obedit->type == OB_MBALL) {
-	 		makeDispList(G.obedit);	
-  		}   	
+		else
+			where_is_pose(ob);
+	}
+	else if(t->spacetype==SPACE_IMAGE) {
+		flushTransUVs(t);
+		if (G.sima->flag & SI_LIVE_UNWRAP)
+			unwrap_lscm_live_re_solve();
 	}
 	else {
-		
-		base= FIRSTBASE;
-		while(base) {
+		for(base= FIRSTBASE; base; base= base->next) {
+			Object *ob= base->object;
+			
+			/* this flag is from depgraph, was stored in nitialize phase, handled in drawview.c */
+			if(base->flag & BA_HAS_RECALC_OB)
+				ob->recalc |= OB_RECALC_OB;
+			if(base->flag & BA_HAS_RECALC_DATA)
+				ob->recalc |= OB_RECALC_DATA;
+			
+			/* thanks to ob->ctime usage, ipos are not called in where_is_object,
+			   unless we edit ipokeys */
 			if(base->flag & BA_DO_IPO) {
-				if(base->object->ipo) {
+				if(ob->ipo) {
 					IpoCurve *icu;
 					
-					base->object->ctime= -1234567.0;
+					ob->ctime= -1234567.0;
 					
-					icu= base->object->ipo->curve.first;
+					icu= ob->ipo->curve.first;
 					while(icu) {
 						calchandles_ipocurve(icu);
 						icu= icu->next;
 					}
 				}				
 			}
-			if(base->object->partype & PARSLOW) {
-				base->object->partype -= PARSLOW;
-				where_is_object(base->object);
-				base->object->partype |= PARSLOW;
-			}
-			else if(base->flag & BA_WHERE_UPDATE) {
-				where_is_object(base->object);
+			
+			/* softbody exception */
+			if(modifiers_isSoftbodyEnabled(ob)) {
+				if(ob->recalc & OB_RECALC_DATA)
+					ob->softflag |= OB_SB_REDO;
 			}
 			
-			base= base->next;
+			/* proxy exception */
+			if(ob->proxy)
+				ob->proxy->recalc |= ob->recalc;
+			if(ob->proxy_group)
+				group_tag_recalc(ob->proxy_group->dup_group);
 		} 
-		
-		base= FIRSTBASE;
-		while(base) {
-			
-			if(base->flag & BA_DISP_UPDATE) makeDispList(base->object);
-			
-			base= base->next;
-		}
 	}
-	
-	/* ugly stuff for posemode, copied from old system */
-	base= FIRSTBASE;
-	while(base) {
-		
-		if (pose_flags_reset_done(base->object)) {
-			if (!is_delay_deform())
-				make_displists_by_armature(base->object);
-		}
-		
-		base= base->next;
-	}
-	
-	if (G.obpose && G.obpose->type == OB_ARMATURE)
-		clear_pose_constraint_status(G.obpose);
-	
-	if (!is_delay_deform()) make_displists_by_armature(G.obpose);
 
+#ifdef WITH_VERSE
+	for (td = t->data; td < t->data + t->total; td++) {
+		if(td->flag & TD_VERSE_VERT) {
+			if(td->verse)
+				send_versevert_pos((VerseVert*)td->verse);
+		}
+		else if(td->flag & TD_VERSE_OBJECT)
+			if(td->verse) b_verse_send_transformation((Object*)td->verse);
+	}
+#endif
 	
 	/* update shaded drawmode while transform */
-	if(G.vd->drawtype == OB_SHADED) reshadeall_displist();
+	if(t->spacetype==SPACE_VIEW3D && G.vd->drawtype == OB_SHADED)
+		reshadeall_displist();
 	
 }
 
@@ -353,9 +401,9 @@ void initTransModeFlags(TransInfo *t, int mode)
 		t->flag |= T_NO_CONSTRAINT;
 		break;
 	case TFM_SHEAR:
-		t->flag |= T_NO_CONSTRAINT;
-		break;
 	case TFM_CREASE:
+	case TFM_BONE_ENVELOPE:
+	case TFM_CURVE_SHRINKFATTEN:
 		t->flag |= T_NO_CONSTRAINT;
 		break;
 	}
@@ -395,9 +443,9 @@ void drawLine(float *center, float *dir, char axis, short options)
 
 void initTrans (TransInfo *t)
 {
-
+	
 	/* moving: is shown in drawobject() (transform color) */
-	if(G.obedit || G.obpose) G.moving= G_TRANSFORM_EDIT;
+	if(G.obedit || (t->flag & T_POSE) ) G.moving= G_TRANSFORM_EDIT;
 	else G.moving= G_TRANSFORM_OBJ;
 
 	t->data = NULL;
@@ -436,29 +484,46 @@ void initTrans (TransInfo *t)
 	
 	Mat3One(t->mat);
 	
-	Mat4CpyMat4(t->viewmat, G.vd->viewmat);
-	Mat4CpyMat4(t->viewinv, G.vd->viewinv);
-	Mat4CpyMat4(t->persinv, G.vd->persinv);
+	t->spacetype = curarea->spacetype;
+	if(t->spacetype==SPACE_VIEW3D) {
+		if(G.vd->flag & V3D_ALIGN) t->flag |= T_V3D_ALIGN;
+		t->around = G.vd->around;
+	}
+	else
+		t->around = V3D_CENTRE;
+
+	setTransformViewMatrices(t);
 }
 
 /* Here I would suggest only TransInfo related issues, like free data & reset vars. Not redraws */
 void postTrans (TransInfo *t) 
 {
+	TransData *td;
+
 	G.moving = 0; // Set moving flag off (display as usual)
+#ifdef WITH_VERSE
+
+	for (td = t->data; td < t->data + t->total; td++) {
+		if(td->flag & TD_VERSE_VERT) {
+			if(td->verse) send_versevert_pos((VerseVert*)td->verse);
+		}
+		else if(td->flag & TD_VERSE_OBJECT) {
+			if(td->verse) {
+				struct VNode *vnode;
+				vnode = (VNode*)((Object*)td->verse)->vnode;
+				((VObjectData*)vnode->data)->flag |= POS_SEND_READY;
+				((VObjectData*)vnode->data)->flag |= ROT_SEND_READY;
+				((VObjectData*)vnode->data)->flag |= SCALE_SEND_READY;
+				b_verse_send_transformation((Object*)td->verse);
+			}
+		}
+	}
+#endif
 
 	stopConstraint(t);
-	/* Not needed anymore but will keep there in case it will be
-	t->con.drawExtra = NULL;
-	t->con.applyVec	= NULL;
-	t->con.applySize= NULL;
-	t->con.applyRot	= NULL;
-	t->con.mode		= 0;
-	*/
-
 	
 	/* postTrans can be called when nothing is selected, so data is NULL already */
 	if (t->data) {
-		TransData *td;
 		int a;
 
 		/* since ipokeys are optional on objects, we mallocced them per trans-data */
@@ -469,50 +534,15 @@ void postTrans (TransInfo *t)
 	}
 
 	if (t->ext) MEM_freeN(t->ext);
-	
-}
-
-static void apply_grid3(float *val, int max_index, float fac1, float fac2, float fac3)
-{
-	/* fac1 is for 'nothing', fac2 for CTRL, fac3 for SHIFT */
-	int invert = U.flag & USER_AUTOGRABGRID;
-	int ctrl;
-	int i;
-
-	for (i=0; i<=max_index; i++) {
-
-		if(invert) {
-			if(G.qual & LR_CTRLKEY) ctrl= 0;
-			else ctrl= 1;
-		}
-		else ctrl= (G.qual & LR_CTRLKEY);
-
-		if(ctrl && (G.qual & LR_SHIFTKEY)) {
-			if(fac3!= 0.0) {
-				for (i=0; i<=max_index; i++) {
-					val[i]= fac3*(float)floor(val[i]/fac3 +.5);
-				}
-			}
-		}
-		else if(ctrl) {
-			if(fac2!= 0.0) {
-				for (i=0; i<=max_index; i++) {
-					val[i]= fac2*(float)floor(val[i]/fac2 +.5);
-				}
-			}
-		}
-		else {
-			if(fac1!= 0.0) {
-				for (i=0; i<=max_index; i++) {
-					val[i]= fac1*(float)floor(val[i]/fac1 +.5);
-				}
-			}
-		}
+	if (t->data2d) {
+		MEM_freeN(t->data2d);
+		t->data2d= NULL;
 	}
-}
 
-void snapGrid(TransInfo *t, float *val) {
-	apply_grid3(val, t->idx_max, t->snap[0], t->snap[1], t->snap[2]);
+	if(t->spacetype==SPACE_IMAGE) {
+		if (G.sima->flag & SI_LIVE_UNWRAP)
+			unwrap_lscm_live_end(t->state == TRANS_CANCEL);
+	}
 }
 
 void applyTransObjects(TransInfo *t)
@@ -585,9 +615,43 @@ void restoreTransObjects(TransInfo *t)
 	
 	for (td = t->data; td < t->data + t->total; td++) {
 		restoreElement(td);
+#ifdef WITH_VERSE
+		/* position of vertexes and object transformation matrix is sent
+		 * extra, becuase blender uses synchronous sending of vertexes
+		 * position as well object trans. matrix and it isn't possible to
+		 * send it in recalcData sometimes */
+		if(td->flag & TD_VERSE_VERT) {
+			if(td->verse) {
+				((VerseVert*)td->verse)->flag |= VERT_POS_OBSOLETE;
+			}
+		}
+		else if(td->flag & TD_VERSE_OBJECT)
+			if(td->verse) {
+				struct VNode *vnode;
+				vnode = (VNode*)((Object*)td->verse)->vnode;
+				((VObjectData*)vnode->data)->flag |= POS_SEND_READY;
+				((VObjectData*)vnode->data)->flag |= ROT_SEND_READY;
+				((VObjectData*)vnode->data)->flag |= SCALE_SEND_READY;
+			}
+#endif
 	}	
 	recalcData(t);
-} 
+}
+
+void calculateCenter2D(TransInfo *t)
+{
+	if (t->flag & (T_EDIT|T_POSE)) {
+		Object *ob= G.obedit?G.obedit:t->poseobj;
+		float vec[3];
+		
+		VECCOPY(vec, t->center);
+		Mat4MulVecfl(ob->obmat, vec);
+		projectIntView(t, vec, t->center2d);
+	}
+	else {
+		projectIntView(t, t->center, t->center2d);
+	}
+}
 
 void calculateCenterCursor(TransInfo *t)
 {
@@ -596,23 +660,18 @@ void calculateCenterCursor(TransInfo *t)
 	cursor = give_cursor();
 	VECCOPY(t->center, cursor);
 
-	if(t->flag & (T_EDIT|T_POSE)) {
-		Object *ob= G.obedit?G.obedit:G.obpose;
+	/* If edit or pose mode, move cursor in local space */
+	if (t->flag & (T_EDIT|T_POSE)) {
+		Object *ob = G.obedit?G.obedit:t->poseobj;
 		float mat[3][3], imat[3][3];
-		float vec[3];
 		
 		VecSubf(t->center, t->center, ob->obmat[3]);
 		Mat3CpyMat4(mat, ob->obmat);
 		Mat3Inv(imat, mat);
 		Mat3MulVecfl(imat, t->center);
-		
-		VECCOPY(vec, t->center);
-		Mat4MulVecfl(ob->obmat, vec);
-		project_int(vec, t->center2d);
-	}
-	else {
-		project_int(t->center, t->center2d);
 	}
+	
+	calculateCenter2D(t);
 }
 
 void calculateCenterMedian(TransInfo *t)
@@ -634,17 +693,7 @@ void calculateCenterMedian(TransInfo *t)
 	VecMulf(partial, 1.0f / i);
 	VECCOPY(t->center, partial);
 
-	if (t->flag & (T_EDIT|T_POSE)) {
-		Object *ob= G.obedit?G.obedit:G.obpose;
-		float vec[3];
-		
-		VECCOPY(vec, t->center);
-		Mat4MulVecfl(ob->obmat, vec);
-		project_int(vec, t->center2d);
-	}
-	else {
-		project_int(t->center, t->center2d);
-	}
+	calculateCenter2D(t);
 }
 
 void calculateCenterBound(TransInfo *t)
@@ -673,22 +722,12 @@ void calculateCenterBound(TransInfo *t)
 	VecAddf(t->center, min, max);
 	VecMulf(t->center, 0.5);
 
-	if (t->flag & (T_EDIT|T_POSE)) {
-		Object *ob= G.obedit?G.obedit:G.obpose;
-		float vec[3];
-		
-		VECCOPY(vec, t->center);
-		Mat4MulVecfl(ob->obmat, vec);
-		project_int(vec, t->center2d);
-	}
-	else {
-		project_int(t->center, t->center2d);
-	}
+	calculateCenter2D(t);
 }
 
 void calculateCenter(TransInfo *t) 
 {
-	switch(G.vd->around) {
+	switch(t->around) {
 	case V3D_CENTRE:
 		calculateCenterBound(t);
 		break;
@@ -704,21 +743,28 @@ void calculateCenter(TransInfo *t)
 		break;
 	case V3D_ACTIVE:
 		/* set median, and if if if... do object center */
+		
+		/* EDIT MODE ACTIVE EDITMODE ELEMENT */
+		if (G.obedit && G.obedit->type == OB_MESH && G.editMesh->selected.last) {
+			EM_editselection_center(t->center, G.editMesh->selected.last);
+			calculateCenter2D(t);
+			break;
+		} /* END EDIT MODE ACTIVE ELEMENT */
+		
 		calculateCenterMedian(t);
 		if((t->flag & (T_EDIT|T_POSE))==0) {
 			Object *ob= OBACT;
 			if(ob) {
 				VECCOPY(t->center, ob->obmat[3]);
-				project_int(t->center, t->center2d);
+				projectIntView(t, t->center, t->center2d);
 			}
 		}
-		
 	}
 
 	/* setting constraint center */
 	VECCOPY(t->con.center, t->center);
 	if(t->flag & (T_EDIT|T_POSE)) {
-		Object *ob= G.obedit?G.obedit:G.obpose;
+		Object *ob= G.obedit?G.obedit:t->poseobj;
 		Mat4MulVecfl(ob->obmat, t->con.center);
 	}
 
@@ -727,7 +773,7 @@ void calculateCenter(TransInfo *t)
 		if( G.vd->camera==OBACT && G.vd->persp>1) {
 			float axis[3];
 			/* persinv is nasty, use viewinv instead, always right */
-			VECCOPY(axis, G.vd->viewinv[2]);
+			VECCOPY(axis, t->viewinv[2]);
 			Normalise(axis);
 
 			/* 6.0 = 6 grid units */
@@ -735,13 +781,18 @@ void calculateCenter(TransInfo *t)
 			axis[1]= t->center[1]- 6.0f*axis[1];
 			axis[2]= t->center[2]- 6.0f*axis[2];
 			
-			project_int(axis, t->center2d);
+			projectIntView(t, axis, t->center2d);
 			
 			/* rotate only needs correct 2d center, grab needs initgrabz() value */
-			if(t->mode==TFM_TRANSLATION) VECCOPY(t->center, axis);
+			if(t->mode==TFM_TRANSLATION) {
+				VECCOPY(t->center, axis);
+				VECCOPY(t->con.center, t->center);
+			}
 		}
 	}	
-	initgrabz(t->center[0], t->center[1], t->center[2]);
+
+	if(t->spacetype==SPACE_VIEW3D)
+		initgrabz(t->center[0], t->center[1], t->center[2]);
 }
 
 void calculatePropRatio(TransInfo *t)
@@ -792,6 +843,10 @@ void calculatePropRatio(TransInfo *t)
 				case PROP_SPHERE:
 					td->factor = (float)sqrt(2*dist - dist * dist);
 					break;
+				case PROP_RANDOM:
+					BLI_srand( BLI_rand() ); /* random seed */
+					td->factor = BLI_frand()*dist;
+					break;
 				default:
 					td->factor = 1;
 				}
@@ -816,6 +871,9 @@ void calculatePropRatio(TransInfo *t)
 		case PROP_SPHERE:
 			strcpy(t->proptext, "(Sphere)");
 			break;
+		case PROP_RANDOM:
+			strcpy(t->proptext, "(Random)");
+			break;
 		default:
 			strcpy(t->proptext, "");
 		}
