@@ -400,9 +400,11 @@ int calc_manipulator_stats(ScrArea *sa)
 	else if(ob && (ob->flag & OB_POSEMODE)) {
 		bArmature *arm= ob->data;
 		bPoseChannel *pchan;
+		int mode;
 		
 		if((ob->lay & G.vd->lay)==0) return 0;
 		
+		mode= Trans.mode;
 		Trans.mode= TFM_ROTATION;	// mislead counting bones... bah
 		
 		/* count total, we use same method as transform will do */
@@ -422,6 +424,8 @@ int calc_manipulator_stats(ScrArea *sa)
 			Mat4MulVecfl(ob->obmat, G.scene->twmin);
 			Mat4MulVecfl(ob->obmat, G.scene->twmax);
 		}
+		/* restore, mode can be TFM_INIT */
+		Trans.mode= mode;
 	}
 	else if(G.f & (G_FACESELECT + G_VERTEXPAINT + G_TEXTUREPAINT +G_WEIGHTPAINT)) {
 		;
@@ -485,12 +489,11 @@ int calc_manipulator_stats(ScrArea *sa)
 			}
 			/* no break we define 'normal' as 'local' in Object mode */
 		case V3D_MANIP_LOCAL:
-			if(totsel==1 || v3d->around==V3D_LOCAL || G.obedit || (ob->flag & OB_POSEMODE)) {
-				strcpy(t->spacename, "local");
-				Mat4CpyMat4(v3d->twmat, ob->obmat);
-				Mat4Ortho(v3d->twmat);
-			}
+			strcpy(t->spacename, "local");
+			Mat4CpyMat4(v3d->twmat, ob->obmat);
+			Mat4Ortho(v3d->twmat);
 			break;
+			
 		case V3D_MANIP_VIEW:
 			{
 				float mat[3][3];
@@ -1325,9 +1328,9 @@ static void draw_manipulator_rotate_cyl(float mat[][4], int moving, int drawflag
 
 /* ********************************************* */
 
-static float get_manipulator_drawsize(ScrArea *sa)
+float get_drawsize(View3D *v3d)
 {
-	View3D *v3d= sa->spacedata.first;
+	ScrArea *sa = v3d->area;
 	float size, vec[3], len1, len2;
 	
 	/* size calculus, depending ortho/persp settings, like initgrabz() */
@@ -1338,7 +1341,7 @@ static float get_manipulator_drawsize(ScrArea *sa)
 	VECCOPY(vec, v3d->persinv[1]);
 	len2= Normalise(vec);
 	
-	size*= (0.01f*(float)U.tw_size)*(len1>len2?len1:len2);
+	size*= 0.01f*(len1>len2?len1:len2);
 
 	/* correct for window size to make widgets appear fixed size */
 	if(sa->winx > sa->winy) size*= 1000.0f/(float)sa->winx;
@@ -1347,6 +1350,16 @@ static float get_manipulator_drawsize(ScrArea *sa)
 	return size;
 }
 
+static float get_manipulator_drawsize(ScrArea *sa)
+{
+	View3D *v3d= sa->spacedata.first;
+	float size = get_drawsize(v3d);
+	
+	size*= (float)U.tw_size;
+
+	return size;
+}
+
 /* exported to transform_constraints.c */
 /* mat, vec = default orientation and location */
 /* type = transform type */
