@@ -1,5 +1,5 @@
 /**
- * $Id: usiblender.c,v 1.80 2005/06/10 13:12:59 jesterking Exp $
+ * $Id: usiblender.c,v 1.140 2007/01/19 20:33:00 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -37,21 +37,23 @@
 #include <stdio.h>
 #include <string.h>
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifdef WIN32
+#include <windows.h> /* need to include windows.h so _WIN32_IE is defined  */
+#ifndef _WIN32_IE
+#define _WIN32_IE 0x0400 /* minimal requirements for SHGetSpecialFolderPath on MINGW MSVC has this defined already */
+#endif
+#include <shlobj.h> /* for SHGetSpecialFolderPath, has to be done before BLI_winstuff because 'near' is disabled through BLI_windstuff */
 #include "BLI_winstuff.h"
 #include <process.h> /* getpid */
 #else
 #include <unistd.h> /* getpid */
 #endif
 #include "MEM_guardedalloc.h"
+#include "MEM_CacheLimiterC-Api.h"
 
 #include "BMF_Api.h"
-#ifdef INTERNATIONAL
 #include "BIF_language.h"
+#ifdef INTERNATIONAL
 #include "FTF_Api.h"
 #endif
 
@@ -66,6 +68,8 @@
 #include "DNA_space_types.h"
 #include "DNA_userdef_types.h"
 #include "DNA_sound_types.h"
+#include "DNA_scene_types.h"
+#include "DNA_screen_types.h"
 
 #include "BKE_blender.h"
 #include "BKE_curve.h"
@@ -78,6 +82,12 @@
 #include "BKE_packedFile.h"
 #include "BKE_utildefines.h"
 
+#ifdef WITH_VERSE
+#include "BKE_verse.h"
+#endif
+
+#include "BLI_vfontdata.h"
+
 #include "BIF_fsmenu.h"
 #include "BIF_gl.h"
 #include "BIF_interface.h"
@@ -89,6 +99,8 @@
 #include "BIF_editmesh.h"
 #include "BIF_editmode_undo.h"
 #include "BIF_editsound.h"
+#include "BIF_poseobject.h"
+#include "BIF_previewrender.h"
 #include "BIF_renderwin.h"
 #include "BIF_resources.h"
 #include "BIF_screen.h"
@@ -96,12 +108,17 @@
 #include "BIF_toolbox.h"
 #include "BIF_cursors.h"
 
+#ifdef WITH_VERSE
+#include "BIF_verse.h"
+#endif
+
+
 #include "BSE_drawview.h"
-#include "BSE_headerbuttons.h"
-#include "BSE_editipo.h"
-#include "BSE_editaction.h"
-#include "BSE_filesel.h"
 #include "BSE_edit.h"
+#include "BSE_editipo.h"
+#include "BSE_filesel.h"
+#include "BSE_headerbuttons.h"
+#include "BSE_node.h"
 
 #include "BLO_readfile.h"
 #include "BLO_writefile.h"
@@ -109,14 +126,16 @@
 #include "BDR_drawobject.h"
 #include "BDR_editobject.h"
 #include "BDR_editcurve.h"
+#include "BDR_imagepaint.h"
 #include "BDR_vpaint.h"
 
 #include "BPY_extern.h"
 
 #include "blendef.h"
 
+#include "RE_pipeline.h"		/* RE_ free stuff */
+
 #include "radio.h"
-#include "render.h"		// RE_ free stuff
 #include "datatoc.h"
 
 #include "SYS_System.h"
@@ -167,6 +186,8 @@ static void init_userdef_file(void)
 		U.tw_size= 20;			// percentage of window size
 		U.tw_handlesize= 16;	// percentage of widget radius
 	}
+	if(U.pad_rot_angle==0)
+		U.pad_rot_angle= 15;
 	
 	if (G.main->versionfile <= 191) {
 		strcpy(U.plugtexdir, U.textudir);
@@ -188,6 +209,7 @@ static void init_userdef_file(void)
 		bTheme *btheme;
 		
 		U.uiflag |= USER_GLOBALUNDO;
+		if (U.undosteps==0) U.undosteps=32;
 		
 		for(btheme= U.themes.first; btheme; btheme= btheme->next) {
 			/* check for alpha==0 is safe, then color was never set */
@@ -225,32 +247,188 @@ static void init_userdef_file(void)
 			}
 		}
 	}
+	if (G.main->versionfile <= 237) {
+		bTheme *btheme;
+		/* bone colors */
+		for(btheme= U.themes.first; btheme; btheme= btheme->next) {
+			/* check for alpha==0 is safe, then color was never set */
+			if(btheme->tv3d.bone_solid[3]==0) {
+				SETCOL(btheme->tv3d.bone_solid, 200, 200, 200, 255);
+				SETCOL(btheme->tv3d.bone_pose, 80, 200, 255, 80);
+			}
+		}
+	}
+	if (G.main->versionfile <= 238) {
+		bTheme *btheme;
+		/* bone colors */
+		for(btheme= U.themes.first; btheme; btheme= btheme->next) {
+			/* check for alpha==0 is safe, then color was never set */
+			if(btheme->tnla.strip[3]==0) {
+				SETCOL(btheme->tnla.strip_select, 	0xff, 0xff, 0xaa, 255);
+				SETCOL(btheme->tnla.strip, 0xe4, 0x9c, 0xc6, 255);
+			}
+		}
+	}
+	if (G.main->versionfile <= 239) {
+		bTheme *btheme;
+		
+		for(btheme= U.themes.first; btheme; btheme= btheme->next) {
+			/* Lamp theme, check for alpha==0 is safe, then color was never set */
+			if(btheme->tv3d.lamp[3]==0) {
+				SETCOL(btheme->tv3d.lamp, 	0, 0, 0, 40);
+/* TEMPORAL, remove me! (ton) */				
+				U.uiflag |= USER_PLAINMENUS;
+			}
+			
+			/* check for text field selection highlight, set it to text editor highlight by default */
+			if(btheme->tui.textfield_hi[3]==0) {
+				SETCOL(btheme->tui.textfield_hi, 	
+					btheme->text.shade2[0], 
+					btheme->text.shade2[1], 
+					btheme->text.shade2[2],
+					255);
+			}
+		}
+		if(U.obcenter_dia==0) U.obcenter_dia= 6;
+	}
+	if (G.main->versionfile <= 241) {
+		bTheme *btheme;
+		for(btheme= U.themes.first; btheme; btheme= btheme->next) {
+			/* Node editor theme, check for alpha==0 is safe, then color was never set */
+			if(btheme->tnode.syntaxn[3]==0) {
+				/* re-uses syntax color storage */
+				btheme->tnode= btheme->tv3d;
+				SETCOL(btheme->tnode.edge_select, 255, 255, 255, 255);
+				SETCOL(btheme->tnode.syntaxl, 150, 150, 150, 255);	/* TH_NODE, backdrop */
+				SETCOL(btheme->tnode.syntaxn, 129, 131, 144, 255);	/* in/output */
+				SETCOL(btheme->tnode.syntaxb, 127,127,127, 255);	/* operator */
+				SETCOL(btheme->tnode.syntaxv, 142, 138, 145, 255);	/* generator */
+				SETCOL(btheme->tnode.syntaxc, 120, 145, 120, 255);	/* group */
+			}
+			/* Group theme colours */
+			if(btheme->tv3d.group[3]==0) {
+				SETCOL(btheme->tv3d.group, 0x10, 0x40, 0x10, 255);
+				SETCOL(btheme->tv3d.group_active, 0x66, 0xFF, 0x66, 255);
+			}
+			/* Sequence editor theme*/
+			if(btheme->tseq.movie[3]==0) {
+				SETCOL(btheme->tseq.movie, 	81, 105, 135, 255);
+				SETCOL(btheme->tseq.image, 	109, 88, 129, 255);
+				SETCOL(btheme->tseq.scene, 	78, 152, 62, 255);
+				SETCOL(btheme->tseq.audio, 	46, 143, 143, 255);
+				SETCOL(btheme->tseq.effect, 	169, 84, 124, 255);
+				SETCOL(btheme->tseq.plugin, 	126, 126, 80, 255);
+				SETCOL(btheme->tseq.transition, 162, 95, 111, 255);
+				SETCOL(btheme->tseq.meta, 	109, 145, 131, 255);
+			}
+			if(!(btheme->tui.iconfile)) {
+				BLI_strncpy(btheme->tui.iconfile, "", sizeof(btheme->tui.iconfile));
+			}
+		}
+		
+		/* set defaults for 3D View rotating axis indicator */ 
+		/* since size can't be set to 0, this indicates it's not saved in .B.blend */
+		if (U.rvisize == 0) {
+			U.rvisize = 15;
+			U.rvibright = 8;
+			U.uiflag |= USER_SHOW_ROTVIEWICON;
+		}
+		
+	}
+	if (G.main->versionfile <= 242) {
+		bTheme *btheme;
+		
+		for(btheme= U.themes.first; btheme; btheme= btheme->next) {
+			/* long keyframe colour */
+			/* check for alpha==0 is safe, then color was never set */
+			if(btheme->tact.strip[3]==0) {
+				SETCOL(btheme->tv3d.edge_sharp, 255, 32, 32, 255);
+				SETCOL(btheme->tact.strip_select, 	0xff, 0xff, 0xaa, 204);
+				SETCOL(btheme->tact.strip, 0xe4, 0x9c, 0xc6, 204);
+			}
+			
+			/* IPO-Editor - Vertex Size*/
+			if(btheme->tipo.vertex_size == 0) {
+				btheme->tipo.vertex_size= 3;
+			}
+		}
+	}
 	
-	if (U.undosteps==0) U.undosteps=32;
+	/* GL Texture Garbage Collection (variable abused above!) */
+	if (U.textimeout == 0) {
+		U.texcollectrate = 60;
+		U.textimeout = 120;
+	}
+	if (U.memcachelimit <= 0) {
+		U.memcachelimit = 32;
+	}
+	if (U.frameserverport == 0) {
+		U.frameserverport = 8080;
+	}
+
+	MEM_CacheLimiter_set_maximum(U.memcachelimit * 1024 * 1024);
 	
 	reset_autosave();
 	
 #ifdef INTERNATIONAL
 	read_languagefile();
+#endif
 	
 	refresh_interface_font();
-#endif // INTERNATIONAL
+
+#ifdef WITH_VERSE
+	if(strlen(U.versemaster)<1) {
+			strcpy(U.versemaster, "master.uni-verse.org");
+	}
+	if(strlen(U.verseuser)<1) {
+			char *name = verse_client_name();
+			strcpy(U.verseuser, name);
+			MEM_freeN(name);
+	}
+#endif
 
 }
 
+#ifdef WITH_VERSE
+extern ListBase session_list;
+#endif
+
 void BIF_read_file(char *name)
 {
 	extern short winqueue_break; /* editscreen.c */
+	int retval;
+#ifdef WITH_VERSE
+	struct VerseSession *session;
+	struct VNode *vnode;
 
-	//here?
-	//sound_end_all_sounds();
+	session = session_list.first;
+	while(session) {
+		vnode = session->nodes.lb.first;
+		while(vnode) {
+			switch(vnode->type) {
+				case V_NT_OBJECT:
+					unsubscribe_from_obj_node(vnode);
+					break;
+				case V_NT_GEOMETRY:
+					unsubscribe_from_geom_node(vnode);
+					break;
+				case V_NT_BITMAP:
+					unsubscribe_from_bitmap_node(vnode);
+					break;
+			}
+			vnode = vnode->next;
+		}
+		session = session->next;
+	}
+#endif
 
-	// first try to read exotic file formats...
-	if (BKE_read_exotic(name) == 0) { /* throws first error box */
-		/* we didn't succeed, now try to read Blender file
-		   calls readfile, calls toolbox, throws one more,
-		   on failure calls the stream, and that is stubbed.... */
-		int retval= BKE_read_file(name, NULL);
+	/* first try to read exotic file formats... */
+	/* it throws error box when file doesnt exist and returns -1 */
+	retval= BKE_read_exotic(name);
+	
+	if (retval== 0) {
+		/* we didn't succeed, now try to read Blender file */
+		retval= BKE_read_file(name, NULL);
 
 		mainwindow_set_filename_to_title(G.main->name);
 		countall();
@@ -260,12 +438,35 @@ void BIF_read_file(char *name)
 
 		if(retval==2) init_userdef_file();	// in case a userdef is read from regular .blend
 		
+		G.relbase_valid = 1;
+
 		undo_editmode_clear();
 		BKE_reset_undo();
 		BKE_write_undo("original");	/* save current state */
+
 		refresh_interface_font();
 	}
-	else BIF_undo_push("Import file");
+	else if(retval==1)
+		BIF_undo_push("Import file");
+}
+
+static void outliner_242_patch(void)
+{
+	ScrArea *sa;
+	
+	for(sa= G.curscreen->areabase.first; sa; sa= sa->next) {
+		SpaceLink *sl= sa->spacedata.first;
+		for(; sl; sl= sl->next) {
+			if(sl->spacetype==SPACE_OOPS) {
+				SpaceOops *soops= (SpaceOops *)sl;
+				if(soops->type!=SO_OUTLINER) {
+					soops->type= SO_OUTLINER;
+					init_v2d_oops(sa, soops);
+				}
+			}
+		}
+	}
+	G.fileflags |= G_FILE_GAME_MAT;
 }
 
 /* only here settings for fullscreen */
@@ -274,16 +475,21 @@ int BIF_read_homefile(void)
 	char tstr[FILE_MAXDIR+FILE_MAXFILE], scestr[FILE_MAXDIR];
 	char *home= BLI_gethome();
 	int success;
+	struct TmpFont *tf;
 	
 	BLI_clean(home);
 
-#if 0
-//#ifdef _WIN32	// FULLSCREEN
-	static int screenmode = -1;
-	
-	screenmode = U.uiflag & USER_FLIPFULLSCREEN;
-#endif
-	
+	tf= G.ttfdata.first;
+	while(tf)
+	{
+		freePackedFile(tf->pf);
+		tf->pf = NULL;
+		tf->vfont = NULL;
+		tf= tf->next;
+	}
+	BLI_freelistN(&G.ttfdata);
+		
+	G.relbase_valid = 0;
 	BLI_make_file_string(G.sce, tstr, home, ".B.blend");
 	strcpy(scestr, G.sce);	/* temporal store */
 	
@@ -294,29 +500,13 @@ int BIF_read_homefile(void)
 		success = BKE_read_file(tstr, NULL);
 	} else {
 		success = BKE_read_file_from_memory(datatoc_B_blend, datatoc_B_blend_size, NULL);
+		/* outliner patch for 2.42 .b.blend */
+		outliner_242_patch();
 	}
+
 	BLI_clean(scestr);
 	strcpy(G.sce, scestr);
-	
-#if 0
-//#ifdef _WIN32	// FULLSCREEN
-	/* choose window startmode */
-	switch (G.windowstate){
-		case G_WINDOWSTATE_USERDEF: /* use the usersetting */
-			break;
-		case G_WINDOWSTATE_FULLSCREEN: /* force fullscreen */
-			U.uiflag |= USER_FLIPFULLSCREEN;
-			break;
-		case G_WINDOWSTATE_BORDER: /* force with borders */
-			U.uiflag &= ~USER_FLIPFULLSCREEN;
-	}
-	
-	if(screenmode != (U.uiflag & USER_FLIPFULLSCREEN)) {
-		mainwindow_toggle_fullscreen ((U.uiflag & USER_FLIPFULLSCREEN));
-		screenmode = (U.uiflag & USER_FLIPFULLSCREEN);
-	}
-#endif
-	
+
 	space_set_commmandline_options();
 	
 	init_userdef_file();
@@ -339,41 +529,69 @@ static void get_autosave_location(char buf[FILE_MAXDIR+FILE_MAXFILE])
 
 void BIF_read_autosavefile(void)
 {
-	char tstr[FILE_MAXDIR+FILE_MAXFILE], scestr[FILE_MAXDIR];
+	char tstr[FILE_MAX], scestr[FILE_MAX];
 	int save_over;
 
-	strcpy(scestr, G.sce);	/* temporal store */
+	BLI_strncpy(scestr, G.sce, FILE_MAX);	/* temporal store */
 	
 	get_autosave_location(tstr);
 
 	save_over = G.save_over;
 	BKE_read_file(tstr, NULL);
 	G.save_over = save_over;
-	strcpy(G.sce, scestr);
+	BLI_strncpy(G.sce, scestr, FILE_MAX);
 }
 
-/***/
+/* free strings of open recent files */
+static void free_openrecent(void)
+{
+	struct RecentFile *recent;
+
+	for(recent = G.recent_files.first; recent; recent=recent->next)
+		MEM_freeN(recent->filename);
+
+	BLI_freelistN(&(G.recent_files));
+}
 
 static void readBlog(void)
 {
-	char name[FILE_MAXDIR+FILE_MAXFILE], filename[FILE_MAXFILE];
+	char name[FILE_MAX], filename[FILE_MAX];
 	LinkNode *l, *lines;
+	struct RecentFile *recent;
+	char *line;
+	int num;
 
 	BLI_make_file_string("/", name, BLI_gethome(), ".Blog");
 	lines= BLI_read_file_as_lines(name);
 
-	if (lines && !BLI_streq(lines->link, "")) {
-		strcpy(G.sce, lines->link);
-	} else {
-		BLI_make_file_string("/", G.sce, BLI_gethome(), "untitled.blend");
+	G.recent_files.first = G.recent_files.last = NULL;
+
+	/* read list of recent opend files from .Blog to memory */
+	for (l= lines, num= 0; l && (num<10); l= l->next, num++) {
+		line = l->link;
+		if (!BLI_streq(line, "")) {
+			if (num==0) 
+				strcpy(G.sce, line);
+			
+			recent = (RecentFile*)MEM_mallocN(sizeof(RecentFile),"RecentFile");
+			BLI_addtail(&(G.recent_files), recent);
+			recent->filename = (char*)MEM_mallocN(sizeof(char)*(strlen(line)+1), "name of file");
+			recent->filename[0] = '\0';
+			
+			strcpy(recent->filename, line);
+		}
 	}
 
+	if(G.sce[0] == 0)
+		BLI_make_file_string("/", G.sce, BLI_gethome(), "untitled.blend");
+	
 	BLI_free_file_lines(lines);
 
 #ifdef WIN32
 	/* Add the drive names to the listing */
 	{
 		__int64 tmp;
+		char folder[MAX_PATH];
 		char tmps[4];
 		int i;
 			
@@ -389,7 +607,15 @@ static void readBlog(void)
 				fsmenu_insert_entry(tmps, 0);
 			}
 		}
-		
+
+		/* Adding Desktop and My Documents */
+		fsmenu_append_seperator();
+
+		SHGetSpecialFolderPath(0, folder, CSIDL_PERSONAL, 0);
+		fsmenu_insert_entry(folder, 0);
+		SHGetSpecialFolderPath(0, folder, CSIDL_DESKTOPDIRECTORY, 0);
+		fsmenu_insert_entry(folder, 0);
+
 		fsmenu_append_seperator();
 	}
 #endif
@@ -408,25 +634,54 @@ static void readBlog(void)
 	fsmenu_append_seperator();
 	
 	/* add last saved file */
-	BLI_split_dirfile(G.sce, name, filename);
+	BLI_split_dirfile(G.sce, name, filename); /* G.sce shouldn't be relative */
 	
 	fsmenu_insert_entry(name, 0);
 	
 	BLI_free_file_lines(lines);
 }
 
-
 static void writeBlog(void)
 {
+	struct RecentFile *recent, *next_recent;
 	char name[FILE_MAXDIR+FILE_MAXFILE];
 	FILE *fp;
+	int i;
 
 	BLI_make_file_string("/", name, BLI_gethome(), ".Blog");
 
-	fp= fopen(name, "w");
-	if (fp) {
-		fprintf(fp, G.sce);
-		fclose(fp);
+	recent = G.recent_files.first;
+	/* refresh .Blog of recent opened files, when current file was changed */
+	if(!(recent) || (strcmp(recent->filename, G.sce)!=0)) {
+		fp= fopen(name, "w");
+		if (fp) {
+			/* add current file to the beginning of list */
+			recent = (RecentFile*)MEM_mallocN(sizeof(RecentFile),"RecentFile");
+			recent->filename = (char*)MEM_mallocN(sizeof(char)*(strlen(G.sce)+1), "name of file");
+			recent->filename[0] = '\0';
+			strcpy(recent->filename, G.sce);
+			BLI_addhead(&(G.recent_files), recent);
+			/* write current file to .Blog */
+			fprintf(fp, "%s\n", recent->filename);
+			recent = recent->next;
+			i=1;
+			/* write rest of recent opened files to .Blog */
+			while((i<10) && (recent)){
+				/* this prevents to have duplicities in list */
+				if (strcmp(recent->filename, G.sce)!=0) {
+					fprintf(fp, "%s\n", recent->filename);
+					recent = recent->next;
+				}
+				else {
+					next_recent = recent->next;
+					MEM_freeN(recent->filename);
+					BLI_freelinkN(&(G.recent_files), recent);
+					recent = next_recent;
+				}
+				i++;
+			}
+			fclose(fp);
+		}
 	}
 }
 
@@ -458,6 +713,7 @@ static void do_history(char *name)
 void BIF_write_file(char *target)
 {
 	Library *li;
+	int writeflags;
 	char di[FILE_MAXDIR];
 	char *err;
 	
@@ -484,19 +740,25 @@ void BIF_write_file(char *target)
 			return; 
 	}
 	
-	waitcursor(1);
-	
 	if(G.obedit) {
 		exit_editmode(0);	/* 0 = no free data */
 	}
 	if (G.fileflags & G_AUTOPACK) {
 		packAll();
 	}
+	
+	waitcursor(1);	// exit_editmode sets cursor too
 
 	do_history(di);
-		
-	if (BLO_write_file(di, G.fileflags, &err)) {
+	
+	/* we use the UserDef to define compression flag */
+	writeflags= G.fileflags & ~G_FILE_COMPRESS;
+	if(U.flag & USER_FILECOMPRESS)
+		writeflags |= G_FILE_COMPRESS;
+	
+	if (BLO_write_file(di, writeflags, &err)) {
 		strcpy(G.sce, di);
+		G.relbase_valid = 1;
 		strcpy(G.main->name, di);	/* is guaranteed current file */
 
 		mainwindow_set_filename_to_title(G.main->name);
@@ -565,16 +827,16 @@ static void initbuttons(void)
 				BMF_GetFont(BMF_kHelvetica12), 
 				BMF_GetFont(BMF_kHelvetica10), 
 				BMF_GetFont(BMF_kHelveticaBold8));
-	
-	BIF_resources_init();
 
-	glClearColor(.7, .7, .6, 0.0);
+	glClearColor(.7f, .7f, .6f, 0.0);
 	
 	G.font= BMF_GetFont(BMF_kHelvetica12);
 	G.fonts= BMF_GetFont(BMF_kHelvetica10);
 	G.fontss= BMF_GetFont(BMF_kHelveticaBold8);
 
 	clear_matcopybuf();
+	
+	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
 }
 
 
@@ -593,13 +855,16 @@ void BIF_init(void)
 	initbuttons();
 	InitCursorData();
 	sound_init_listener();
+	init_node_butfuncs();
 	
-	init_draw_rects();	/* drawobject.c */
+	BIF_preview_init_dbase();
 	BIF_read_homefile();
+
+	BIF_resources_init();	/* after homefile, to dynamically load an icon file based on theme settings */
+	
 	init_gl_stuff();	/* drawview.c, after homefile */
 	readBlog();
-	strcpy(G.lib, G.sce);
-
+	BLI_strncpy(G.lib, G.sce, FILE_MAX);
 }
 
 /***/
@@ -609,6 +874,21 @@ extern ListBase editelems;
 
 void exit_usiblender(void)
 {
+	struct TmpFont *tf;
+	tf= G.ttfdata.first;
+	while(tf)
+	{
+		freePackedFile(tf->pf);
+		tf->pf= NULL;
+		tf->vfont= NULL;
+		tf= tf->next;
+	}
+	BLI_freelistN(&G.ttfdata);
+#ifdef WITH_VERSE
+	end_all_verse_sessions();
+#endif
+	free_openrecent();
+
 	freeAllRad();
 	BKE_freecubetable();
 
@@ -627,32 +907,39 @@ void exit_usiblender(void)
 	free_editArmature();
 	free_posebuf();
 
-	free_blender();	/* blender.c, does entire library */
+	/* before free_blender so py's gc happens while library still exists */
+	/* needed at least for a rare sigsegv that can happen in pydrivers */
+	BPY_end_python();
+
+	fastshade_free_render();	/* shaded view */
+	free_blender();				/* blender.c, does entire library */
 	free_matcopybuf();
 	free_ipocopybuf();
-	freefastshade();
 	free_vertexpaint();
+	free_imagepaint();
 	
 	/* editnurb can remain to exist outside editmode */
 	freeNurblist(&editNurb);
 
 	fsmenu_free();
+
 #ifdef INTERNATIONAL
 	free_languagemenu();
-#endif	
+#endif
 	
-	RE_free_render_data();
+	RE_FreeAllRender();
 	
 	free_txt_data();
 
 	sound_exit_audio();
 	if(G.listener) MEM_freeN(G.listener);
 
+
+	libtiff_exit();
+
 #ifdef WITH_QUICKTIME
 	quicktime_exit();
 #endif
-		
-	BPY_end_python();
 
 	if (!G.background) {
 		BIF_resources_free();
@@ -665,6 +952,9 @@ void exit_usiblender(void)
 	FTF_End();
 #endif
 
+	if (copybuf) MEM_freeN(copybuf);
+	if (copybufinfo) MEM_freeN(copybufinfo);
+
 	/* undo free stuff */
 	undo_editmode_clear();
 	
@@ -672,7 +962,8 @@ void exit_usiblender(void)
 	BKE_reset_undo(); 
 	
 	BLI_freelistN(&U.themes);
-	
+	BIF_preview_free_dbase();
+		
 	if(totblock!=0) {
 		printf("Error Totblock: %d\n",totblock);
 		MEM_printmemlist();
