@@ -1,5 +1,5 @@
 /**
- * $Id: view.c,v 1.69 2006/07/19 11:28:48 ton Exp $
+ * $Id: view.c,v 1.78 2007/02/07 11:25:56 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -61,6 +61,7 @@
 #include "DNA_userdef_types.h"
 #include "DNA_view3d_types.h"
 
+#include "BKE_action.h"
 #include "BKE_anim.h"
 #include "BKE_global.h"
 #include "BKE_main.h"
@@ -71,6 +72,7 @@
 #include "BIF_space.h"
 #include "BIF_mywindow.h"
 #include "BIF_previewrender.h"
+#include "BIF_retopo.h"
 #include "BIF_screen.h"
 #include "BIF_toolbox.h"
 
@@ -80,6 +82,7 @@
 
 #include "BDR_drawobject.h"	/* For draw_object */
 #include "BDR_editface.h"	/* For minmax_tface */
+#include "BDR_sculptmode.h"
 
 #include "mydevice.h"
 #include "blendef.h"
@@ -360,7 +363,8 @@ int boundbox_clip(float obmat[][4], BoundBox *bb)
 	float vec[4], min, max;
 	int a, flag= -1, fl;
 	
-	if(bb==0) return 1;
+	if(bb==NULL) return 1;
+	if(bb->flag & OB_BB_DISABLED) return 1;
 	
 	Mat4MulMat4(mat, obmat, G.vd->persmat);
 
@@ -559,7 +563,18 @@ void viewmove(int mode)
 
 	/* cumultime(0); */
 
-	if (ob && (U.uiflag & USER_ORBIT_SELECTION)) {
+	if(!G.obedit && (G.f & G_SCULPTMODE) && ob && G.vd->pivot_last) {
+		use_sel= 1;
+		VecCopyf(ofs, G.vd->ofs);
+
+		VecCopyf(obofs,&sculpt_session()->pivot.x);
+		Mat4MulVecfl(ob->obmat, obofs);
+		obofs[0]= -obofs[0];
+		obofs[1]= -obofs[1];
+		obofs[2]= -obofs[2];
+	}
+	//else if (G.obedit==NULL && ob && !(ob->flag & OB_POSEMODE) && U.uiflag & USER_ORBIT_SELECTION) {
+	else if (ob && (U.uiflag & USER_ORBIT_SELECTION)) {
 		use_sel = 1;
 		VECCOPY(ofs, G.vd->ofs);
 
@@ -588,18 +603,7 @@ void viewmove(int mode)
 					if(G.vd->view!=0) scrarea_queue_headredraw(curarea);	/*for button */
 					G.vd->view= 0;
 				}
-				if(G.vd->persp==2 && mode!=1) {
-					float upvec[3]={0,0,0}; /* the view vector */
-					float mat[3][3]; /* view matrix 3x3 to rotate the upvec */
-					where_is_object(G.vd->camera);
-					VecCopyf(G.vd->ofs, G.vd->camera->obmat[3]);
-					VecMulf(G.vd->ofs, -1.0f); /*flip the vector*/
-					
-					upvec[2]= G.vd->dist;
-					Mat3CpyMat4(mat, G.vd->viewinv);
-					Mat3MulVecfl(mat, upvec);
-					VecAddf(G.vd->ofs, G.vd->ofs, upvec);
-					
+				if(G.vd->persp==2 && mode!=1 && G.vd->camera) {
 					G.vd->persp= 1;
 					scrarea_do_windraw(curarea);
 					scrarea_queue_headredraw(curarea);
@@ -761,6 +765,12 @@ void viewmove(int mode)
 			if(G.f & G_PLAYANIM) inner_play_anim_loop(0, 0);
 			if(G.f & G_SIMULATION) break;
 
+			/* If in retopo paint mode, update lines */
+			if(retopo_mesh_paint_check() && G.vd->retopo_view_data) {
+				G.vd->retopo_view_data->queue_matrix_update= 1;
+				retopo_paint_view_update(G.vd);
+			}
+
 			scrarea_do_windraw(curarea);
 			screen_swapbuffers();
 		}
@@ -777,6 +787,10 @@ void viewmove(int mode)
 		if( !(get_mbut() & (L_MOUSE|M_MOUSE))) break;
 	}
 
+	if(G.vd->depths) G.vd->depths->damaged= 1;
+	retopo_queue_updates(G.vd);
+	allqueue(REDRAWVIEW3D, 0);
+
 	if(preview3d_event) 
 		BIF_view3d_previewrender_signal(curarea, PR_DBASE|PR_DISPRECT);
 	else
@@ -835,6 +849,7 @@ int get_view3d_viewplane(int winxi, int winyi, rctf *viewplane, float *clipsta,
 		orth= 1;
 	}
 	else {
+		/* fac for zoom, also used for camdx */
 		if(G.vd->persp==2) {
 			fac= (1.41421+( (float)G.vd->camzoom )/50.0);
 			fac*= fac;
@@ -869,8 +884,8 @@ int get_view3d_viewplane(int winxi, int winyi, rctf *viewplane, float *clipsta,
 		}
 		/* cam view offset */
 		if(cam) {
-			float dx= G.vd->camdx*(x2-x1);
-			float dy= G.vd->camdy*(y2-y1);
+			float dx= 0.5*fac*G.vd->camdx*(x2-x1);
+			float dy= 0.5*fac*G.vd->camdy*(y2-y1);
 			x1+= dx;
 			x2+= dx;
 			y1+= dy;
@@ -964,7 +979,21 @@ void setviewmatrixview3d()
 		
 		QuatToMat4(G.vd->viewquat, G.vd->viewmat);
 		if(G.vd->persp==1) G.vd->viewmat[3][2]-= G.vd->dist;
-		i_translate(G.vd->ofs[0], G.vd->ofs[1], G.vd->ofs[2], G.vd->viewmat);
+		if(G.vd->ob_centre) {
+			Object *ob= G.vd->ob_centre;
+			float vec[3];
+			
+			VECCOPY(vec, ob->obmat[3]);
+			if(ob->type==OB_ARMATURE && G.vd->ob_centre_bone[0]) {
+				bPoseChannel *pchan= get_pose_channel(ob->pose, G.vd->ob_centre_bone);
+				if(pchan) {
+					VECCOPY(vec, pchan->pose_mat[3]);
+					Mat4MulVecfl(ob->obmat, vec);
+				}
+			}
+			i_translate(-vec[0], -vec[1], -vec[2], G.vd->viewmat);
+		}
+		else i_translate(G.vd->ofs[0], G.vd->ofs[1], G.vd->ofs[2], G.vd->viewmat);
 	}
 }
 
@@ -1040,31 +1069,35 @@ short  view3d_opengl_select(unsigned int *buffer, unsigned int bufsize, short x1
 		G.vd->xray= TRUE;	// otherwise it postpones drawing
 		for(base= G.scene->base.first; base; base= base->next) {
 			if(base->lay & G.vd->lay) {
-				base->selcol= code;
-				glLoadName(code);
-				draw_object(base, DRAW_PICKING|DRAW_CONSTCOLOR);
 				
-				/* we draw group-duplicators for selection too */
-				if((base->object->transflag & OB_DUPLI) && base->object->dup_group) {
-					ListBase *lb;
-					DupliObject *dob;
-					Base tbase;
+				if (base->object->restrictflag & OB_RESTRICT_SELECT)
+					base->selcol= 0;
+				else {
+					base->selcol= code;
+					glLoadName(code);
+					draw_object(base, DRAW_PICKING|DRAW_CONSTCOLOR);
 					
-					tbase.flag= OB_FROMDUPLI;
-					lb= object_duplilist(G.scene, base->object);
-					
-					for(dob= lb->first; dob; dob= dob->next) {
-						tbase.object= dob->ob;
-						Mat4CpyMat4(dob->ob->obmat, dob->mat);
+					/* we draw group-duplicators for selection too */
+					if((base->object->transflag & OB_DUPLI) && base->object->dup_group) {
+						ListBase *lb;
+						DupliObject *dob;
+						Base tbase;
 						
-						draw_object(&tbase, DRAW_PICKING|DRAW_CONSTCOLOR);
+						tbase.flag= OB_FROMDUPLI;
+						lb= object_duplilist(G.scene, base->object);
 						
-						Mat4CpyMat4(dob->ob->obmat, dob->omat);
+						for(dob= lb->first; dob; dob= dob->next) {
+							tbase.object= dob->ob;
+							Mat4CpyMat4(dob->ob->obmat, dob->mat);
+							
+							draw_object(&tbase, DRAW_PICKING|DRAW_CONSTCOLOR);
+							
+							Mat4CpyMat4(dob->ob->obmat, dob->omat);
+						}
+						free_object_duplilist(lb);
 					}
-					free_object_duplilist(lb);
-				}
-				
-				code++;
+					code++;
+				}				
 			}
 		}
 		G.vd->xray= FALSE;	// restore
@@ -1410,14 +1443,11 @@ void view3d_home(int centre)
 		max[0]= max[1]= max[2]= -1.0e10;
 	}
 	
-	base= FIRSTBASE;
-	if(base==0) return;
-	while(base) {
+	for(base= FIRSTBASE; base; base= base->next) {
 		if(base->lay & G.vd->lay) {
 			onedone= 1;
 			minmax_object(base->object, min, max);
 		}
-		base= base->next;
 	}
 	if(!onedone) return;
 	
