@@ -1,5 +1,5 @@
 /*
- * $Id: creator.c,v 1.61 2006/07/12 20:01:48 djcapelis Exp $
+ * $Id: creator.c,v 1.68 2007/01/28 05:45:58 scourage Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -187,6 +187,7 @@ static void print_help(void)
 	printf ("    (formats that can be compiled into blender, not available on all systems)\n");
 	printf ("    \tHDR TIFF EXR MPEG AVICODEC QUICKTIME CINEON DPX\n");
 	printf ("    -x <bool>\tSet option to add the file extension to the end of the file.\n");
+	printf ("    -t <threads>\tUse amount of <threads> for rendering\n");
 	printf ("\nAnimation options:\n");
 	printf ("  -a <file(s)>\tPlayback <file(s)>\n");
 	printf ("    -p <sx> <sy>\tOpen with lower left corner at <sx>, <sy>\n");
@@ -233,6 +234,24 @@ int main(int argc, char **argv)
 #else
 	int audio = 0;
 #endif
+
+#ifndef YESIAMSTUPID
+	/* temporary: prevent people to make/use 64 bits versions without them knowing it might be
+		risky. I don't know for sure yet if we get problems, but I rather not get the burden of
+		having to fix all faulty saved 64 bits files (ton) */
+	if(sizeof(void *)==8) {
+		printf("64 bits compiles will give incorrectly saved .blend files. Do not use it. For testing purposes please remove this line from creator.c\n"); 
+		exit(0);
+	}
+#else
+	if(sizeof(void *)==8) {
+		printf("64 bits compiles will give incorrectly saved .blend files. Do not use it.\n\n"
+       "*** If you continue to run this executable, you really are quite stupid ***\n\n"); 
+
+	}
+
+#endif
+
 	setCallbacks();
 #ifdef __APPLE__
 		/* patch to ignore argument finder gives us (pid?) */
@@ -353,11 +372,9 @@ int main(int argc, char **argv)
 			case 'h':			
 				print_help();
 				exit(0);
-			
 			case 'v':
 				print_version();
 				exit(0);
-
 			default:
 				break;
 			}
@@ -399,6 +416,7 @@ int main(int argc, char **argv)
 				case 'd':
 					G.f |= G_DEBUG;		/* std output printf's */ 
 					printf ("Blender V %d.%02d\n", G.version/100, G.version%100);
+					MEM_set_memory_debug();
 #ifdef NAN_BUILDINFO
 					printf("Build: %s %s %s %s\n", build_date, build_time, build_platform, build_type);
 
@@ -650,6 +668,12 @@ int main(int argc, char **argv)
 				}
 				break;
 				
+			case 't':
+				a++;
+				if(G.background) {
+					RE_set_max_threads(atoi(argv[a]));
+				}
+				break;
 			case 'x': /* extension */
 				a++;
 				if (a < argc) {
