@@ -1,5 +1,5 @@
 /**
- * $Id: KX_BlenderGL.cpp,v 1.9 2006/02/13 05:45:31 erwin Exp $
+ * $Id: KX_BlenderGL.cpp,v 1.11 2006/12/20 18:39:29 ton Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -74,6 +74,7 @@
 
 #include "BKE_global.h"
 #include "BKE_bmfont.h"
+#include "BKE_image.h"
 
 #include "BIF_gl.h"
 extern "C" {
@@ -110,8 +111,8 @@ void BL_SwapBuffers()
 	myswapbuffers();
 }
 
-void BL_RenderText(int mode,const char* textstr,int textlen,struct TFace* tface,
-				   float v1[3],float v2[3],float v3[3],float v4[3])
+void BL_RenderText(int mode,const char* textstr,int textlen,struct MTFace* tface,
+                   unsigned int *col,float v1[3],float v2[3],float v3[3],float v4[3])
 {
 	Image* ima;
 
@@ -131,6 +132,8 @@ void BL_RenderText(int mode,const char* textstr,int textlen,struct TFace* tface,
 				characters = 0;
 			}
 
+			if(!col) glColor3f(1.0f, 1.0f, 1.0f);
+
 			glPushMatrix();
 			for (index = 0; index < characters; index++) {
 				// lets calculate offset stuff
@@ -139,31 +142,31 @@ void BL_RenderText(int mode,const char* textstr,int textlen,struct TFace* tface,
 				// space starts at offset 1
 				// character = character - ' ' + 1;
 				
-				matrixGlyph(ima->ibuf, character, & centerx, &centery, &sizex, &sizey, &transx, &transy, &movex, &movey, &advance);
-				
+				matrixGlyph((ImBuf *)ima->ibufs.first, character, & centerx, &centery, &sizex, &sizey, &transx, &transy, &movex, &movey, &advance);
+
 				glBegin(GL_POLYGON);
 				// printf(" %c %f %f %f %f\n", character, tface->uv[0][0], tface->uv[0][1], );
 				// glTexCoord2f((tface->uv[0][0] - centerx) * sizex + transx, (tface->uv[0][1] - centery) * sizey + transy);
 				glTexCoord2f((tface->uv[0][0] - centerx) * sizex + transx, (tface->uv[0][1] - centery) * sizey + transy);
 
-				spack(tface->col[0]);
+				if(col) spack(col[0]);
 				// glVertex3fv(v1);
 				glVertex3f(sizex * v1[0] + movex, sizey * v1[1] + movey, v1[2]);
 				
 				glTexCoord2f((tface->uv[1][0] - centerx) * sizex + transx, (tface->uv[1][1] - centery) * sizey + transy);
-				spack(tface->col[1]);
+				if(col) spack(col[1]);
 				// glVertex3fv(v2);
 				glVertex3f(sizex * v2[0] + movex, sizey * v2[1] + movey, v2[2]);
 	
 				glTexCoord2f((tface->uv[2][0] - centerx) * sizex + transx, (tface->uv[2][1] - centery) * sizey + transy);
-				spack(tface->col[2]);
+				if(col) spack(col[2]);
 				// glVertex3fv(v3);
 				glVertex3f(sizex * v3[0] + movex, sizey * v3[1] + movey, v3[2]);
 	
 				if(v4) {
 					// glTexCoord2f((tface->uv[3][0] - centerx) * sizex + transx, 1.0 - (1.0 - tface->uv[3][1]) * sizey - transy);
 					glTexCoord2f((tface->uv[3][0] - centerx) * sizex + transx, (tface->uv[3][1] - centery) * sizey + transy);
-					spack(tface->col[3]);
+					if(col) spack(col[3]);
 					// glVertex3fv(v4);
 					glVertex3f(sizex * v4[0] + movex, sizey * v4[1] + movey, v4[2]);
 				}
