@@ -1,5 +1,5 @@
 /**
- * $Id: BL_BlenderDataConversion.cpp,v 1.41 2006/04/17 01:33:10 erwin Exp $
+ * $Id: BL_BlenderDataConversion.cpp,v 1.58 2007/02/05 08:07:55 snailrose Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -136,6 +136,9 @@
 #include "BKE_mesh.h"
 #include "MT_Point3.h"
 
+extern "C" {
+	#include "BKE_customdata.h"
+}
 
 #include "BKE_material.h" /* give_current_material */
 /* end of blender include block */
@@ -167,6 +170,18 @@
 #include "BL_ArmatureObject.h"
 #include "BL_DeformableGameObject.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+#include "BSE_headerbuttons.h"
+void update_for_newframe();
+//void scene_update_for_newframe(struct Scene *sce, unsigned int lay);
+//#include "BKE_ipo.h"
+//void do_all_data_ipos(void);
+#ifdef __cplusplus
+}
+#endif
+
 static int default_face_mode = TF_DYNAMIC;
 
 static unsigned int KX_rgbaint2uint_new(unsigned int icol)
@@ -227,7 +242,6 @@ static void GetRGB(short type,
 	MFace* mface,
 	MCol* mmcol,
 	Material *mat,
-	TFace *tface,
 	unsigned int &c0, 
 	unsigned int &c1, 
 	unsigned int &c2, 
@@ -236,7 +250,7 @@ static void GetRGB(short type,
 	unsigned int colour = 0xFFFFFFFFL;
 	switch(type)
 	{
-		case 0:	// vertex colors?
+		case 0:	// vertex colors
 		{
 			if(mmcol) {
 				c0 = KX_Mcol2uint_new(mmcol[0]);
@@ -244,7 +258,6 @@ static void GetRGB(short type,
 				c2 = KX_Mcol2uint_new(mmcol[2]);
 				if (mface->v4)
 					c3 = KX_Mcol2uint_new(mmcol[3]);
-				mmcol += 4;
 			}else // backup white
 			{
 				c0 = KX_rgbaint2uint_new(colour);
@@ -276,26 +289,7 @@ static void GetRGB(short type,
 				c3 = KX_rgbaint2uint_new(colour);
 		} break;
 		
-		case 2: // vertex colors
-		{
-			if(tface ) {
-				c0 = KX_rgbaint2uint_new(tface->col[0]);
-				c1 = KX_rgbaint2uint_new(tface->col[1]);
-				c2 = KX_rgbaint2uint_new(tface->col[2]);
-			
-				if (mface->v4) {
-					c3 = KX_rgbaint2uint_new(tface->col[3]);
-				}
-			}else {
-				c0 = KX_rgbaint2uint_new(colour);
-				c1 = KX_rgbaint2uint_new(colour);
-				c2 = KX_rgbaint2uint_new(colour);	
-				if (mface->v4)
-					c3 = KX_rgbaint2uint_new( colour );
-			}
-		} break;
-		
-		case 3: // white
+		default: // white
 		{
 			c0 = KX_rgbaint2uint_new(colour);
 			c1 = KX_rgbaint2uint_new(colour);
@@ -306,21 +300,33 @@ static void GetRGB(short type,
 	}
 }
 
+typedef struct MTF_localLayer
+{
+	MTFace *face;
+	char *name;
+}MTF_localLayer;
+
 // ------------------------------------
-BL_Material* ConvertMaterial(  Mesh* mesh, Material *mat, TFace* tface,  MFace* mface, MCol* mmcol, int lightlayer, Object* blenderobj )
+BL_Material* ConvertMaterial(
+	Mesh* mesh, 
+	Material *mat, 
+	MTFace* tface,  
+	MFace* mface, 
+	MCol* mmcol, 
+	int lightlayer, 
+	Object* blenderobj,
+	MTF_localLayer *layers)
 {
 	//this needs some type of manager
 	BL_Material *material = new BL_Material();
 
 	int numchan =	-1;
 	bool validmat	= (mat!=0);
-	bool validface	= (mesh->tface && tface);
+	bool validface	= (mesh->mtface && tface);
 	
 	short type = 0;
 	if( validmat )
 		type = 1; // material color 
-	else if(mesh->tface && tface)
-		type = 2; // vertex colors
 	
 	material->IdMode = DEFAULT_BLENDER;
 
@@ -328,8 +334,8 @@ BL_Material* ConvertMaterial(  Mesh* mesh, Material *mat, TFace* tface,  MFace*
 	if(validmat) {
 
 		// use vertex colors by explicitly setting
-		if(mat->mode &MA_VERTEXCOLP) 
-			type = 2;
+		if(mat->mode &MA_VERTEXCOLP)
+			type = 0;
 
 		// use lighting?
 		material->ras_mode |= ( mat->mode & MA_SHLESS )?0:USE_LIGHT;
@@ -355,6 +361,7 @@ BL_Material* ConvertMaterial(  Mesh* mesh, Material *mat, TFace* tface,  MFace*
 		// foreach MTex
 		for(int i=0; i<numchan; i++) {
 			// use face tex
+
 			if(i==0 && facetex ) {
 				Image*tmp = (Image*)(tface->tpage);
 				if(tmp) {
@@ -366,7 +373,20 @@ BL_Material* ConvertMaterial(  Mesh* mesh, Material *mat, TFace* tface,  MFace*
 					if(material->img[i]->flag & IMA_REFLECT)
 						material->mapping[i].mapping |= USEREFL;
 					else
+					{
+						mttmp = getImageFromMaterial( mat, i );
+						if(mttmp && mttmp->texco &TEXCO_UV)
+						{
+							STR_String uvName = mttmp->uvname;
+
+							if (!uvName.IsEmpty())
+								material->mapping[i].uvCoName = mttmp->uvname;
+							else
+								material->mapping[i].uvCoName = "";
+						}
 						material->mapping[i].mapping |= USEUV;
+					}
+
 					if(material->ras_mode & USE_LIGHT)
 						material->ras_mode &= ~USE_LIGHT;
 					if(tface->mode & TF_LIGHT)
@@ -412,21 +432,19 @@ BL_Material* ConvertMaterial(  Mesh* mesh, Material *mat, TFace* tface,  MFace*
 					else if(mttmp->tex->type == TEX_ENVMAP) {
 						if( mttmp->tex->env->stype == ENV_LOAD ) {
 					
-							material->mtexname[i]= mttmp->tex->id.name;
-							material->cubemap[i] = mttmp->tex->env;
-							if(material->cubemap[i]->ima) {
-								material->texname[i] = material->cubemap[i]->ima->id.name;
+							material->mtexname[i]     = mttmp->tex->id.name;
+							EnvMap *env = mttmp->tex->env;
+							env->ima = mttmp->tex->ima;
+							material->cubemap[i] = env;
+
+							if (material->cubemap[i])
+							{
+								if (!material->cubemap[i]->cube[0])
+									BL_Texture::SplitEnvMap(material->cubemap[i]);
+
+								material->texname[i]= material->cubemap[i]->ima->id.name;
 								material->mapping[i].mapping |= USEENV;
 							}
-							else {
-								// in the player, we need to split it our self
-								material->cubemap[i]->ima = mttmp->tex->ima;
-								if(material->cubemap[i]->ima) {
-									material->texname[i] = material->cubemap[i]->ima->id.name;
-									material->mapping[i].mapping |= USEENV;
-								}
-								//
-							}
 						}
 					}
 					material->flag[i] |= (mat->ipo!=0)?HASIPO:0;
@@ -444,7 +462,15 @@ BL_Material* ConvertMaterial(  Mesh* mesh, Material *mat, TFace* tface,  MFace*
 					else if(mttmp->texco &(TEXCO_ORCO|TEXCO_GLOB))
 						material->mapping[i].mapping |= USEORCO;
 					else if(mttmp->texco &TEXCO_UV)
+					{
+						STR_String uvName = mttmp->uvname;
+
+						if (!uvName.IsEmpty())
+							material->mapping[i].uvCoName = mttmp->uvname;
+						else
+							material->mapping[i].uvCoName = "";
 						material->mapping[i].mapping |= USEUV;
+					}
 					else if(mttmp->texco &TEXCO_NORM)
 						material->mapping[i].mapping |= USENORM;
 					else if(mttmp->texco &TEXCO_TANGENT)
@@ -561,6 +587,7 @@ BL_Material* ConvertMaterial(  Mesh* mesh, Material *mat, TFace* tface,  MFace*
 		material->ref			= 0.8f;
 	}
 	MT_Point2 uv[4];
+	MT_Point2 uv2[4];
 
 	if( validface ) {
 
@@ -592,10 +619,78 @@ BL_Material* ConvertMaterial(  Mesh* mesh, Material *mat, TFace* tface,  MFace*
 		material->transp	= TF_SOLID;
 		material->tile		= 0;
 	}
+
+
+
+	// get uv sets
+	if(validmat) 
+	{
+		bool isFirstSet = true;
+
+		// only two sets implemented, but any of the eight 
+		// sets can make up the two layers
+		for (int vind = 0; vind<material->num_enabled; vind++)
+		{
+			BL_Mapping &map = material->mapping[vind];
+			if (map.uvCoName.IsEmpty())
+				isFirstSet = false;
+			else
+			{
+				MT_Point2 uvSet[4];
+				for (int lay=0; lay<MAX_MTFACE; lay++)
+				{
+					MTF_localLayer& layer = layers[lay];
+					if (layer.face == 0) break;
+
+
+					bool processed = false;
+					if (strcmp(map.uvCoName.ReadPtr(), layer.name)==0)
+					{
+						uvSet[0]	= MT_Point2(layer.face->uv[0]);
+						uvSet[1]	= MT_Point2(layer.face->uv[1]);
+						uvSet[2]	= MT_Point2(layer.face->uv[2]);
+
+						if (mface->v4) 
+							uvSet[3]	= MT_Point2(layer.face->uv[3]);
+
+						processed = true;
+					}
+
+					if (!processed) continue;
+
+					if (isFirstSet)
+					{
+						uv[0] = uvSet[0]; uv[1] = uvSet[1];
+						uv[2] = uvSet[2]; uv[3] = uvSet[3];
+						isFirstSet = false;
+					}
+					else
+					{
+						uv2[0] = uvSet[0]; uv2[1] = uvSet[1];
+						uv2[2] = uvSet[2]; uv2[3] = uvSet[3];
+						map.mapping |= USECUSTOMUV;
+					}
+				}
+			}
+		}
+	}
+
 	unsigned int rgb[4];
-	GetRGB(type,mface,mmcol,mat,tface,rgb[0],rgb[1],rgb[2], rgb[3]);
+	GetRGB(type,mface,mmcol,mat,rgb[0],rgb[1],rgb[2], rgb[3]);
+
+	// swap the material color, so MCol on TF_BMFONT works
+	if (validmat && type==1 && (tface && tface->mode & TF_BMFONT))
+	{
+		rgb[0] = KX_rgbaint2uint_new(rgb[0]);
+		rgb[1] = KX_rgbaint2uint_new(rgb[1]);
+		rgb[2] = KX_rgbaint2uint_new(rgb[2]);
+		rgb[3] = KX_rgbaint2uint_new(rgb[3]);
+	}
+
 	material->SetConversionRGB(rgb);
 	material->SetConversionUV(uv);
+	material->SetConversionUV2(uv2);
+
 
 	material->ras_mode |= (mface->v4==0)?TRIANGLE:0;
 	if(validmat)
@@ -632,7 +727,7 @@ static void BL_ComputeTriTangentSpace(const MT_Vector3 &v1, const MT_Vector3 &v2
 static MT_Vector4*  BL_ComputeMeshTangentSpace(Mesh* mesh)
 {
 	MFace* mface = static_cast<MFace*>(mesh->mface);
-	TFace* tface = static_cast<TFace*>(mesh->tface);
+	MTFace* tface = static_cast<MTFace*>(mesh->mtface);
 
 	MT_Vector3 *tan1 = new MT_Vector3[mesh->totvert];
 	MT_Vector3 *tan2 = new MT_Vector3[mesh->totvert];
@@ -696,10 +791,11 @@ RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools*
 	int lightlayer = blenderobj->lay;
 	
 	MFace* mface = static_cast<MFace*>(mesh->mface);
-	TFace* tface = static_cast<TFace*>(mesh->tface);
+	MTFace* tface = static_cast<MTFace*>(mesh->mtface);
 	MCol* mmcol = mesh->mcol;
 	MT_assert(mface || mesh->totface == 0);
-	
+
+
 	// Determine if we need to make a skinned mesh
 	if (mesh->dvert){
 		meshobj = new BL_SkinMeshObject(lightlayer);
@@ -712,10 +808,32 @@ RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools*
 	if (tface)
 		tangent = BL_ComputeMeshTangentSpace(mesh);
 	
+
+	// Extract avaiable layers
+	MTF_localLayer *layers =  new MTF_localLayer[MAX_MTFACE];
+	for (int lay=0; lay<MAX_MTFACE; lay++)
+	{
+		layers[lay].face = 0;
+		layers[lay].name = "";
+	}
+
+
+	int validLayers = 0;
+	for (int i=0; i<mesh->fdata.totlayer; i++)
+	{
+		if (mesh->fdata.layers[i].type == CD_MTFACE)
+		{
+			assert(validLayers <= 8);
+
+			layers[validLayers].face = (MTFace*)mesh->fdata.layers[i].data;;
+			layers[validLayers].name = mesh->fdata.layers[i].name;
+			validLayers++;
+		}
+	}
+
+
 	meshobj->SetName(mesh->id.name);
-	
 	meshobj->m_xyz_index_to_vertex_index_mapping.resize(mesh->totvert);
-	
 	for (int f=0;f<mesh->totface;f++,mface++)
 	{
 		
@@ -725,6 +843,7 @@ RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools*
 		if (mface->v3)
 		{
 			MT_Point2 uv0(0.0,0.0),uv1(0.0,0.0),uv2(0.0,0.0),uv3(0.0,0.0);
+			MT_Point2 uv20(0.0,0.0),uv21(0.0,0.0),uv22(0.0,0.0),uv23(0.0,0.0);
 			// rgb3 is set from the adjoint face in a square
 			unsigned int rgb0,rgb1,rgb2,rgb3 = 0;
 			MT_Vector3	no0(mesh->mvert[mface->v1].no[0], mesh->mvert[mface->v1].no[1], mesh->mvert[mface->v1].no[2]),
@@ -772,7 +891,7 @@ RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools*
 					else 
 						ma = give_current_material(blenderobj, 1);
 
-					BL_Material *bl_mat = ConvertMaterial(mesh, ma, tface, mface, mmcol, lightlayer, blenderobj);
+					BL_Material *bl_mat = ConvertMaterial(mesh, ma, tface, mface, mmcol, lightlayer, blenderobj, layers);
 					// set the index were dealing with
 					bl_mat->material_index =  (int)mface->mat_nr;
 
@@ -784,16 +903,17 @@ RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools*
 					
 					unsigned int rgb[4];
 					bl_mat->GetConversionRGB(rgb);
-					rgb0 = rgb[0];
-					rgb1 = rgb[1];
-					rgb2 = rgb[2];
-					rgb3 = rgb[3];
+					rgb0 = rgb[0]; rgb1 = rgb[1];
+					rgb2 = rgb[2]; rgb3 = rgb[3];
 					MT_Point2 uv[4];
 					bl_mat->GetConversionUV(uv);
-					uv0 = uv[0];
-					uv1 = uv[1];
-					uv2 = uv[2];
-					uv3 = uv[3];
+					uv0 = uv[0]; uv1 = uv[1];
+					uv2 = uv[2]; uv3 = uv[3];
+
+					bl_mat->GetConversionUV2(uv);
+					uv20 = uv[0]; uv21 = uv[1];
+					uv22 = uv[2]; uv23 = uv[3];
+
 					if(tangent){
 						tan0 = tangent[mface->v1];
 						tan1 = tangent[mface->v2];
@@ -808,10 +928,10 @@ RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools*
 				{
 					ma = give_current_material(blenderobj, 1);
 
-					Image* bima = ((mesh->tface && tface) ? (Image*) tface->tpage : NULL);
+					Image* bima = ((mesh->mtface && tface) ? (Image*) tface->tpage : NULL);
 		
 					STR_String imastr = 
-						((mesh->tface && tface) ? 
+						((mesh->mtface && tface) ? 
 						(bima? (bima)->id.name : "" ) : "" );
 			
 					char transp=0;
@@ -825,7 +945,7 @@ RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools*
 				
 					}
 
-					if (mesh->tface && tface)
+					if (mesh->mtface && tface)
 					{
 						// Use texface colors if available
 						//TF_DYNAMIC means the polygon is a collision face
@@ -839,76 +959,64 @@ RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools*
 						uv0 = MT_Point2(tface->uv[0]);
 						uv1 = MT_Point2(tface->uv[1]);
 						uv2 = MT_Point2(tface->uv[2]);
-						rgb0 = KX_rgbaint2uint_new(tface->col[0]);
-						rgb1 = KX_rgbaint2uint_new(tface->col[1]);
-						rgb2 = KX_rgbaint2uint_new(tface->col[2]);
 		
 						if (mface->v4)
-						{
 							uv3 = MT_Point2(tface->uv[3]);
-							rgb3 = KX_rgbaint2uint_new(tface->col[3]);
-						} 
 					} 
 					else
 					{
-						//
-						if (mmcol)
-						{
-							// Use vertex colours
-							rgb0 = KX_Mcol2uint_new(mmcol[0]);
-							rgb1 = KX_Mcol2uint_new(mmcol[1]);
-							rgb2 = KX_Mcol2uint_new(mmcol[2]);
-							
-							
-							if (mface->v4)
-							{
-								rgb3 = KX_Mcol2uint_new(mmcol[3]);
-								
-							}
+						// no texfaces, set COLLSION true and everything else FALSE
 						
-							mmcol += 4;
-						}
-						else
-						{
-							// If there are no vertex colors OR texfaces,
-							// Initialize face to white and set COLLSION true and everything else FALSE
-							unsigned int colour = 0xFFFFFFFFL;
-							mode = default_face_mode;	
-							transp = TF_SOLID;
-							tile = 0;
-							if (ma)
-							{
-								// If we have a material, take the default colour from the material.
-								union
-								{
-									unsigned char cp[4];
-									unsigned int integer;
-								} col_converter;
-								
-								col_converter.cp[3] = (unsigned char) (ma->r*255.0);
-								col_converter.cp[2] = (unsigned char) (ma->g*255.0);
-								col_converter.cp[1] = (unsigned char) (ma->b*255.0);
-								col_converter.cp[0] = (unsigned char) (ma->alpha*255.0);
-								
-								colour = col_converter.integer;
-							}
-							
-							rgb0 = KX_rgbaint2uint_new(colour);
-							rgb1 = KX_rgbaint2uint_new(colour);
-							rgb2 = KX_rgbaint2uint_new(colour);	
-							
-							if (mface->v4)
-								rgb3 = KX_rgbaint2uint_new(colour);
-						}
+						mode = default_face_mode;	
+						transp = TF_SOLID;
+						tile = 0;
 					}
+
+					if (mmcol)
+					{
+						// Use vertex colours
+						rgb0 = KX_Mcol2uint_new(mmcol[0]);
+						rgb1 = KX_Mcol2uint_new(mmcol[1]);
+						rgb2 = KX_Mcol2uint_new(mmcol[2]);
 						
+						if (mface->v4)
+							rgb3 = KX_Mcol2uint_new(mmcol[3]);
+					}
+					else {
+						// no vertex colors: take from material if we have one,
+						// otherwise set to white
+						unsigned int colour = 0xFFFFFFFFL;
+
+						if (ma)
+						{
+							union
+							{
+								unsigned char cp[4];
+								unsigned int integer;
+							} col_converter;
+							
+							col_converter.cp[3] = (unsigned char) (ma->r*255.0);
+							col_converter.cp[2] = (unsigned char) (ma->g*255.0);
+							col_converter.cp[1] = (unsigned char) (ma->b*255.0);
+							col_converter.cp[0] = (unsigned char) (ma->alpha*255.0);
+							
+							colour = col_converter.integer;
+						}
+	
+						rgb0 = KX_rgbaint2uint_new(colour);
+						rgb1 = KX_rgbaint2uint_new(colour);
+						rgb2 = KX_rgbaint2uint_new(colour);	
+						
+						if (mface->v4)
+							rgb3 = KX_rgbaint2uint_new(colour);
+					}
 					
 					bool istriangle = (mface->v4==0);
 					bool zsort = ma?(ma->mode & MA_ZTRA) != 0:false;
 					
 					polymat = new KX_PolygonMaterial(imastr, ma,
 						tile, tilexrep, tileyrep, 
-						mode, transp, zsort, lightlayer, istriangle, blenderobj, tface);
+						mode, transp, zsort, lightlayer, istriangle, blenderobj, tface, (unsigned int*)mmcol);
 		
 					if (ma)
 					{
@@ -946,19 +1054,19 @@ RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools*
 					d3=((BL_SkinMeshObject*)meshobj)->FindOrAddDeform(vtxarray, mface->v3, &mesh->dvert[mface->v3], polymat);
 					if (nverts==4)
 						d4=((BL_SkinMeshObject*)meshobj)->FindOrAddDeform(vtxarray, mface->v4, &mesh->dvert[mface->v4], polymat);
-					poly->SetVertex(0,((BL_SkinMeshObject*)meshobj)->FindOrAddVertex(vtxarray,pt0,uv0,tan0,rgb0,no0,d1,flat, polymat));
-					poly->SetVertex(1,((BL_SkinMeshObject*)meshobj)->FindOrAddVertex(vtxarray,pt1,uv1,tan1,rgb1,no1,d2,flat, polymat));
-					poly->SetVertex(2,((BL_SkinMeshObject*)meshobj)->FindOrAddVertex(vtxarray,pt2,uv2,tan2,rgb2,no2,d3,flat, polymat));
+					poly->SetVertex(0,((BL_SkinMeshObject*)meshobj)->FindOrAddVertex(vtxarray,pt0,uv0,uv20,tan0,rgb0,no0,d1,flat, polymat));
+					poly->SetVertex(1,((BL_SkinMeshObject*)meshobj)->FindOrAddVertex(vtxarray,pt1,uv1,uv21,tan1,rgb1,no1,d2,flat, polymat));
+					poly->SetVertex(2,((BL_SkinMeshObject*)meshobj)->FindOrAddVertex(vtxarray,pt2,uv2,uv22,tan2,rgb2,no2,d3,flat, polymat));
 					if (nverts==4)
-						poly->SetVertex(3,((BL_SkinMeshObject*)meshobj)->FindOrAddVertex(vtxarray,pt3,uv3,tan3,rgb3,no3,d4, flat,polymat));
+						poly->SetVertex(3,((BL_SkinMeshObject*)meshobj)->FindOrAddVertex(vtxarray,pt3,uv3,uv23,tan3,rgb3,no3,d4, flat,polymat));
 				}
 				else
 				{
-					poly->SetVertex(0,meshobj->FindOrAddVertex(vtxarray,pt0,uv0,tan0,rgb0,no0,polymat,mface->v1));
-					poly->SetVertex(1,meshobj->FindOrAddVertex(vtxarray,pt1,uv1,tan1,rgb1,no1,polymat,mface->v2));
-					poly->SetVertex(2,meshobj->FindOrAddVertex(vtxarray,pt2,uv2,tan2,rgb2,no2,polymat,mface->v3));
+					poly->SetVertex(0,meshobj->FindOrAddVertex(vtxarray,pt0,uv0,uv20,tan0,rgb0,no0,polymat,mface->v1));
+					poly->SetVertex(1,meshobj->FindOrAddVertex(vtxarray,pt1,uv1,uv21,tan1,rgb1,no1,polymat,mface->v2));
+					poly->SetVertex(2,meshobj->FindOrAddVertex(vtxarray,pt2,uv2,uv22,tan2,rgb2,no2,polymat,mface->v3));
 					if (nverts==4)
-						poly->SetVertex(3,meshobj->FindOrAddVertex(vtxarray,pt3,uv3,tan3,rgb3,no3,polymat,mface->v4));
+						poly->SetVertex(3,meshobj->FindOrAddVertex(vtxarray,pt3,uv3,uv23,tan3,rgb3,no3,polymat,mface->v4));
 				}
 				meshobj->AddPolygon(poly);
 				if (poly->IsCollider())
@@ -985,12 +1093,19 @@ RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools*
 		}
 		if (tface) 
 			tface++;
+		if (mmcol)
+			mmcol+=4;
+
+		for (int lay=0; lay<MAX_MTFACE; lay++)
+		{
+			MTF_localLayer &layer = layers[lay];
+			if (layer.face == 0) break;
+
+			layer.face++;
+		}
 	}
 	meshobj->UpdateMaterialList();
 
-
-
-	// -----------------------------------
 	// pre calculate texture generation
 	for(RAS_MaterialBucket::Set::iterator mit = meshobj->GetFirstMaterial();
 		mit != meshobj->GetLastMaterial(); ++ mit) {
@@ -1000,6 +1115,8 @@ RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools*
 	if(tangent)
 		delete [] tangent;
 
+	if (layers)
+		delete []layers;
 
 	return meshobj;
 }
@@ -1244,7 +1361,8 @@ void BL_CreatePhysicsObjectNew(KX_GameObject* gameobj,
 						 KX_Scene* kxscene,
 						 int activeLayerBitInfo,
 						 e_PhysicsEngine	physics_engine,
-						 KX_BlenderSceneConverter *converter
+						 KX_BlenderSceneConverter *converter,
+						 bool processCompoundChildren
 						 )
 					
 {
@@ -1252,6 +1370,25 @@ void BL_CreatePhysicsObjectNew(KX_GameObject* gameobj,
 	//int userigidbody = SYS_GetCommandLineInt(syshandle,"norigidbody",0);
 	//bool bRigidBody = (userigidbody == 0);
 
+	// get Root Parent of blenderobject
+	struct Object* parent= blenderobject->parent;
+	while(parent && parent->parent) {
+		parent= parent->parent;
+	}
+
+	bool isCompoundChild = false;
+
+	if (parent && (parent->gameflag & OB_DYNAMIC)) {
+		
+		if ((parent->gameflag & OB_CHILD) != 0)
+		{
+			isCompoundChild = true;
+		} 
+	}
+	if (processCompoundChildren != isCompoundChild)
+		return;
+
+
 	PHY_ShapeProps* shapeprops =
 			CreateShapePropsFromBlenderObject(blenderobject, 
 			kxscene);
@@ -1261,6 +1398,10 @@ void BL_CreatePhysicsObjectNew(KX_GameObject* gameobj,
 		CreateMaterialFromBlenderObject(blenderobject, kxscene);
 					
 	KX_ObjectProperties objprop;
+
+	objprop.m_isCompoundChild = isCompoundChild;
+	objprop.m_hasCompoundChildren = (blenderobject->gameflag & OB_CHILD) != 0;
+
 	if ((objprop.m_isactor = (blenderobject->gameflag & OB_ACTOR)!=0))
 	{
 		objprop.m_dyna = (blenderobject->gameflag & OB_DYNAMIC) != 0;
@@ -1280,7 +1421,6 @@ void BL_CreatePhysicsObjectNew(KX_GameObject* gameobj,
 	objprop.m_dynamic_parent=NULL;
 	objprop.m_isdeformable = ((blenderobject->gameflag2 & 2)) != 0;
 	objprop.m_boundclass = objprop.m_dyna?KX_BOUNDSPHERE:KX_BOUNDMESH;
-	
 	KX_BoxBounds bb;
 	my_get_local_bounds(blenderobject,objprop.m_boundobject.box.m_center,bb.m_extends);
 	if (blenderobject->gameflag & OB_BOUNDS)
@@ -1333,19 +1473,17 @@ void BL_CreatePhysicsObjectNew(KX_GameObject* gameobj,
 		}
 	}
 
-	// get Root Parent of blenderobject
-	struct Object* parent= blenderobject->parent;
-	while(parent && parent->parent) {
-		parent= parent->parent;
-	}
-
+	
 	if (parent && (parent->gameflag & OB_DYNAMIC)) {
 		
 		KX_GameObject *parentgameobject = converter->FindGameObject(parent);
 		objprop.m_dynamic_parent = parentgameobject;
-
+		//cannot be dynamic:
+		objprop.m_dyna = false;
+		shapeprops->m_mass = 0.f;
 	}
 
+	
 	objprop.m_concave = (blenderobject->boundtype & 4) != 0;
 	
 	switch (physics_engine)
@@ -1553,7 +1691,89 @@ static struct Scene *GetSceneForName(struct Main *maggie, const STR_String& scen
 	return (Scene*) maggie->scene.first;
 }
 
+#include "DNA_constraint_types.h"
+#include "BIF_editconstraint.h"
 
+bPoseChannel *get_active_posechannel2 (Object *ob)
+{
+	bArmature *arm= (bArmature*)ob->data;
+	bPoseChannel *pchan;
+	
+	/* find active */
+	for(pchan= (bPoseChannel *)ob->pose->chanbase.first; pchan; pchan= pchan->next) {
+		if(pchan->bone && (pchan->bone->flag & BONE_ACTIVE) && (pchan->bone->layer & arm->layer))
+			return pchan;
+	}
+	
+	return NULL;
+}
+
+ListBase *get_active_constraints2(Object *ob)
+{
+	if (!ob)
+		return NULL;
+
+	if (ob->flag & OB_POSEMODE) {
+		bPoseChannel *pchan;
+
+		pchan = get_active_posechannel2(ob);
+		if (pchan)
+			return &pchan->constraints;
+	}
+	else 
+		return &ob->constraints;
+
+	return NULL;
+}
+
+
+void RBJconstraints(Object *ob)//not used
+{
+	ListBase *conlist;
+	bConstraint *curcon;
+
+	conlist = get_active_constraints2(ob);
+
+	if (conlist) {
+		for (curcon = (bConstraint *)conlist->first; curcon; curcon=(bConstraint *)curcon->next) {
+
+			printf("%i\n",curcon->type);
+		}
+
+
+	}
+}
+
+#include "PHY_IPhysicsEnvironment.h"
+#include "KX_IPhysicsController.h"
+#include "PHY_DynamicTypes.h"
+
+KX_IPhysicsController* getPhId(CListValue* sumolist,STR_String busc){//not used
+
+    for (int j=0;j<sumolist->GetCount();j++)
+	{
+	    KX_GameObject* gameobje = (KX_GameObject*) sumolist->GetValue(j);
+	    if (gameobje->GetName()==busc)
+            return gameobje->GetPhysicsController();
+	}
+
+	return 0;
+
+}
+
+KX_GameObject* getGameOb(STR_String busc,CListValue* sumolist){
+
+    for (int j=0;j<sumolist->GetCount();j++)
+	{
+	    KX_GameObject* gameobje = (KX_GameObject*) sumolist->GetValue(j);
+	    if (gameobje->GetName()==busc)
+            return gameobje;
+	}
+	
+	return 0;
+
+}
+#include "BLI_arithb.h"
 // convert blender objects into ketsji gameobjects
 void BL_ConvertBlenderObjects(struct Main* maggie,
 							  const STR_String& scenename,
@@ -1577,6 +1797,7 @@ void BL_ConvertBlenderObjects(struct Main* maggie,
 	RAS_FrameSettings::RAS_FrameType frame_type;
 	int aspect_width;
 	int aspect_height;
+	vector<MT_Vector3> inivel,iniang;
 	
 	if (alwaysUseExpandFraming) {
 		frame_type = RAS_FrameSettings::e_frame_extend;
@@ -1652,9 +1873,20 @@ void BL_ConvertBlenderObjects(struct Main* maggie,
 										rendertools, 
 										converter,
 										blenderscene);
-											
-		if (gameobj)
+										
+		bool isInActiveLayer = (blenderobject->lay & activeLayerBitInfo) !=0;
+		bool addobj=true;
+		
+		if (converter->addInitFromFrame)
+			if (!isInActiveLayer)
+				addobj=false;
+										
+		if (gameobj&&addobj)
 		{
+			MT_Point3 posPrev;			
+			MT_Matrix3x3 angor;			
+			if (converter->addInitFromFrame) blenderscene->r.cfra=blenderscene->r.sfra;
+			
 			MT_Point3 pos = MT_Point3(
 				blenderobject->loc[0]+blenderobject->dloc[0],
 				blenderobject->loc[1]+blenderobject->dloc[1],
@@ -1670,7 +1902,26 @@ void BL_ConvertBlenderObjects(struct Main* maggie,
 				blenderobject->size[1],
 				blenderobject->size[2]
 			);
-			
+			if (converter->addInitFromFrame){//rcruiz
+				float eulxyzPrev[3];
+				blenderscene->r.cfra=blenderscene->r.sfra-1;
+				update_for_newframe();
+				MT_Vector3 tmp=pos-MT_Point3(blenderobject->loc[0]+blenderobject->dloc[0],
+											blenderobject->loc[1]+blenderobject->dloc[1],
+											blenderobject->loc[2]+blenderobject->dloc[2]
+									);
+				eulxyzPrev[0]=blenderobject->rot[0];
+				eulxyzPrev[1]=blenderobject->rot[1];
+				eulxyzPrev[2]=blenderobject->rot[2];
+				tmp.scale((float)blenderscene->r.frs_sec,(float)blenderscene->r.frs_sec,(float)blenderscene->r.frs_sec);
+				inivel.push_back(tmp);
+				tmp=eulxyz-eulxyzPrev;
+				tmp.scale((float)blenderscene->r.frs_sec,(float)blenderscene->r.frs_sec,(float)blenderscene->r.frs_sec);
+				iniang.push_back(tmp);
+				blenderscene->r.cfra=blenderscene->r.sfra;
+				update_for_newframe();
+			}		
+						
 			gameobj->NodeSetLocalPosition(pos);
 			gameobj->NodeSetLocalOrientation(MT_Matrix3x3(eulxyz));
 			gameobj->NodeSetLocalScale(scale);
@@ -1681,8 +1932,6 @@ void BL_ConvertBlenderObjects(struct Main* maggie,
 			Material *mat = give_current_material(blenderobject, 1);
 			if(mat) BL_ConvertMaterialIpos(mat, gameobj, converter);	
 	
-			bool isInActiveLayer = (blenderobject->lay & activeLayerBitInfo) !=0;
-	
 			sumolist->Add(gameobj->AddRef());
 			
 			BL_ConvertProperties(blenderobject,gameobj,timemgr,kxscene,isInActiveLayer);
@@ -1694,7 +1943,7 @@ void BL_ConvertBlenderObjects(struct Main* maggie,
 			templist->Add(gameobj->AddRef());
 			
 			// update children/parent hierarchy
-			if (blenderobject->parent != 0)
+			if ((blenderobject->parent != 0)&&(!converter->addInitFromFrame))
 			{
 				// blender has an additional 'parentinverse' offset in each object
 				SG_Node* parentinversenode = new SG_Node(NULL,NULL,SG_Callbacks());
@@ -1725,12 +1974,15 @@ void BL_ConvertBlenderObjects(struct Main* maggie,
 				logicmgr->RegisterGameMeshName(gameobj->GetMesh(i)->GetName(), blenderobject);
 	
 			converter->RegisterGameObject(gameobj, blenderobject);	
-			
 			// this was put in rapidly, needs to be looked at more closely
 			// only draw/use objects in active 'blender' layers
 	
 			logicbrick_conversionlist->Add(gameobj->AddRef());
 			
+			if (converter->addInitFromFrame){
+				posPrev=gameobj->NodeGetWorldPosition();
+				angor=gameobj->NodeGetWorldOrientation();
+			}
 			if (isInActiveLayer)
 			{
 				objectlist->Add(gameobj->AddRef());
@@ -1740,7 +1992,11 @@ void BL_ConvertBlenderObjects(struct Main* maggie,
 				gameobj->Bucketize();
 				
 			}
-			
+			if (converter->addInitFromFrame){
+				gameobj->NodeSetLocalPosition(posPrev);
+				gameobj->NodeSetLocalOrientation(angor);
+			}
+						
 		}
 			
 		base = base->next;
@@ -1833,6 +2089,8 @@ void BL_ConvertBlenderObjects(struct Main* maggie,
 		}
 	}
 	
+	bool processCompoundChildren = false;
+
 	// create physics information
 	for (i=0;i<sumolist->GetCount();i++)
 	{
@@ -1840,20 +2098,116 @@ void BL_ConvertBlenderObjects(struct Main* maggie,
 		struct Object* blenderobject = converter->FindBlenderObject(gameobj);
 		int nummeshes = gameobj->GetMeshCount();
 		RAS_MeshObject* meshobj = 0;
-
 		if (nummeshes > 0)
 		{
 			meshobj = gameobj->GetMesh(0);
 		}
+		BL_CreatePhysicsObjectNew(gameobj,blenderobject,meshobj,kxscene,activeLayerBitInfo,physics_engine,converter,processCompoundChildren);
+	}
 
-		BL_CreatePhysicsObjectNew(gameobj,blenderobject,meshobj,kxscene,activeLayerBitInfo,physics_engine,converter);
-
+	processCompoundChildren = true;
+	// create physics information
+	for (i=0;i<sumolist->GetCount();i++)
+	{
+		KX_GameObject* gameobj = (KX_GameObject*) sumolist->GetValue(i);
+		struct Object* blenderobject = converter->FindBlenderObject(gameobj);
+		int nummeshes = gameobj->GetMeshCount();
+		RAS_MeshObject* meshobj = 0;
+		if (nummeshes > 0)
+		{
+			meshobj = gameobj->GetMesh(0);
+		}
+		BL_CreatePhysicsObjectNew(gameobj,blenderobject,meshobj,kxscene,activeLayerBitInfo,physics_engine,converter,processCompoundChildren);
 	}
 	
+	
+	//set ini linearVel and int angularVel //rcruiz
+	if (converter->addInitFromFrame)
+		for (i=0;i<sumolist->GetCount();i++)
+		{
+			KX_GameObject* gameobj = (KX_GameObject*) sumolist->GetValue(i);
+			if (gameobj->IsDynamic()){
+				gameobj->setLinearVelocity(inivel[i],false);
+				gameobj->setAngularVelocity(iniang[i],false);
+			}
+		
+		
+		}	
+
+		// create physics joints
+	for (i=0;i<sumolist->GetCount();i++)
+	{
+		KX_GameObject* gameobj = (KX_GameObject*) sumolist->GetValue(i);
+		struct Object* blenderobject = converter->FindBlenderObject(gameobj);
+		int nummeshes = gameobj->GetMeshCount();
+		RAS_MeshObject* meshobj = 0;
+        ListBase *conlist;
+        bConstraint *curcon;
+        conlist = get_active_constraints2(blenderobject);
+        if (conlist) {
+            for (curcon = (bConstraint *)conlist->first; curcon; curcon=(bConstraint *)curcon->next) {
+                if (curcon->type==CONSTRAINT_TYPE_RIGIDBODYJOINT){
+                    bRigidBodyJointConstraint *dat=(bRigidBodyJointConstraint *)curcon->data;
+                    //if (dat->tar)
+                        if (!dat->child){
+							PHY_IPhysicsController* physctr2 = 0;
+							if (dat->tar)
+							{
+								KX_GameObject *gotar=getGameOb(dat->tar->id.name,sumolist);
+								if (gotar && gotar->GetPhysicsController())
+									physctr2 = (PHY_IPhysicsController*) gotar->GetPhysicsController()->GetUserData();
+							}
+
+							if (gameobj->GetPhysicsController())
+							{
+								float radsPerDeg = 6.283185307179586232f / 360.f;
+
+								PHY_IPhysicsController* physctrl = (PHY_IPhysicsController*) gameobj->GetPhysicsController()->GetUserData();
+								//we need to pass a full constraint frame, not just axis
+	                            
+								//localConstraintFrameBasis
+								MT_Matrix3x3 localCFrame(MT_Vector3(radsPerDeg*dat->axX,radsPerDeg*dat->axY,radsPerDeg*dat->axZ));
+								MT_Vector3 axis0 = localCFrame.getColumn(0);
+								MT_Vector3 axis1 = localCFrame.getColumn(1);
+								MT_Vector3 axis2 = localCFrame.getColumn(2);
+								
+								int constraintId = kxscene->GetPhysicsEnvironment()->createConstraint(physctrl,physctr2,(PHY_ConstraintType)dat->type,(float)dat->pivX,(float)dat->pivY,(float)dat->pivZ,
+								(float)axis0.x(),(float)axis0.y(),(float)axis0.z(),
+								(float)axis1.x(),(float)axis1.y(),(float)axis1.z(),
+								(float)axis2.x(),(float)axis2.y(),(float)axis2.z()
+											);
+								if (constraintId)
+								{
+									//if it is a generic 6DOF constraint, set all the limits accordingly
+									if (dat->type == PHY_GENERIC_6DOF_CONSTRAINT)
+									{
+										int dof;
+										int dofbit=1;
+										for (dof=0;dof<6;dof++)
+										{
+											if (dat->flag & dofbit)
+											{
+												kxscene->GetPhysicsEnvironment()->setConstraintParam(constraintId,dof,dat->minLimit[dof],dat->maxLimit[dof]);
+											} else
+											{
+												//minLimit > maxLimit means free(disabled limit) for this degree of freedom
+												kxscene->GetPhysicsEnvironment()->setConstraintParam(constraintId,dof,1,-1);
+											}
+											dofbit<<=1;
+										}
+									}
+								}
+							}
+                        }
+                }
+            }
+        }
+
+	}
+
 	templist->Release();
 	sumolist->Release();	
 
-
 	int executePriority=0; /* incremented by converter routines */
 	
 	// convert global sound stuff
