@@ -1,5 +1,5 @@
 /**
- * $Id: KXH_ketsji_hooks.cpp,v 1.7 2005/08/28 10:55:07 intrr Exp $
+ * $Id: KXH_ketsji_hooks.cpp,v 1.8 2006/12/06 20:13:14 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -589,7 +589,7 @@ initialize_gameengine(ketsji_engine_data* k, struct BlendFileData * active_file)
 		// Inside, the rasterizer is initialised! This
 		// requires gl calls...
 		k->rasterizer->Init();
-		k->kx_engine->StartEngine();
+		k->kx_engine->StartEngine( true );
 
 		// that's it:
 		k->kx_initialized = true;
