@@ -1,5 +1,5 @@
 /**
- * $Id: KX_PhysicsEngineEnums.h,v 1.6 2004/03/22 22:01:52 jesterking Exp $
+ * $Id: KX_PhysicsEngineEnums.h,v 1.7 2005/07/16 21:47:52 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -39,7 +39,8 @@ enum	e_PhysicsEngine
 		UseEnji     =  1,
 		UseSumo     =  2,
 		UseDynamo   =  3,
-		UseODE      =  4
+		UseODE      =  4,
+		UseBullet	=	5,
 };
 
 #endif //__KX_PHYSICSENGINEENUMS
