@@ -1,5 +1,5 @@
 /**
- * $Id: NG_NetworkScene.cpp,v 1.4 2004/03/22 22:02:05 jesterking Exp $
+ * $Id: NG_NetworkScene.cpp,v 1.6 2005/03/25 10:33:38 kester Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -31,7 +31,7 @@
  * NetworkSceneManagement generic implementation
  */
 #include <stdio.h>
-#include <assert.h>
+#include <MT_assert.h>
 #include <algorithm>
 
 #include "NG_NetworkScene.h"
@@ -56,7 +56,7 @@ NG_NetworkScene::~NG_NetworkScene()
 /**
  * progress one frame, handle all network traffic
  */
-void NG_NetworkScene::proceed(double curtime, double deltatime)
+void NG_NetworkScene::proceed(double curtime)
 {
 	if (!m_networkdevice) return;
 	if (!m_networkdevice->IsOnline()) return;
@@ -199,7 +199,7 @@ vector<NG_NetworkMessage*> NG_NetworkScene::FindMessages(
 		// possibly it's there, but maybe not (false hit)
 		if (to.IsEmpty()) {
 			// take all messages, and check other fields
-			assert("objectnames that are empty are not valid, so make it a hobby project :)\n");
+			MT_assert(!"objectnames that are empty are not valid, so make it a hobby project :)\n");
 		} else {
 			//todo: find intersection of messages (that are in other 2 maps)
 			vector<NG_NetworkMessage*>** tolistptr = m_messagesByDestinationName[to];
