@@ -1,5 +1,5 @@
 /**
- * $Id: SM_Object.cpp,v 1.11 2004/05/13 13:07:38 kester Exp $
+ * $Id: SM_Object.cpp,v 1.20 2005/03/25 10:33:39 kester Exp $
  * Copyright (C) 2001 NaN Technologies B.V.
  * The basic physics object.
  *
@@ -42,6 +42,8 @@
 #pragma warning( disable : 4786 )     
 #endif
 
+#include "MT_assert.h"
+
 #include "SM_Object.h"
 #include "SM_Scene.h"
 #include "SM_FhObject.h"
@@ -49,11 +51,96 @@
 
 #include "MT_MinMax.h"
 
+MT_Scalar SM_Object::ImpulseThreshold = -1.0;
+
+struct Contact
+{
+	SM_Object *obj1;
+	SM_Object *obj2;
+	MT_Vector3 normal;
+	MT_Point3 pos;
+	
+	// Sort objects by height
+	bool operator()(const Contact *a, const Contact *b)
+	{
+		return a->pos[2] < b->pos[2];
+	}
+	
+	Contact(SM_Object *o1, SM_Object *o2, const MT_Vector3 nor, const MT_Point3 p)
+		: obj1(o1),
+		  obj2(o2),
+		  normal(nor),
+		  pos(p)
+	{
+	}
+	
+	Contact()
+	{
+	}
+	
+	void resolve()
+	{
+		if (obj1->m_static || obj2->m_static)
+		{
+			if (obj1->isDynamic())
+			{
+				if (obj1->m_static && obj2->m_static)
+				{
+					if (obj1->m_static < obj2->m_static)
+					{
+						obj2->m_error -= normal;
+						obj2->m_static = obj1->m_static + 1;
+					}
+					else
+					{
+						obj1->m_error += normal;
+						obj1->m_static = obj2->m_static + 1;
+					}
+				}
+				else
+				{
+					if (obj1->m_static)
+					{
+						obj2->m_error -= normal;
+						obj2->m_static = obj1->m_static + 1;
+					}
+					else
+					{
+						obj1->m_error += normal;
+						obj1->m_static = obj2->m_static + 1;
+					}
+				}
+			}
+			else
+			{
+				obj2->m_error -= normal;
+				obj2->m_static = 1;
+			}
+		}
+		else
+		{
+			// This distinction between dynamic and non-dynamic objects should not be 
+			// necessary. Non-dynamic objects are assumed to have infinite mass.
+			if (obj1->isDynamic()) {
+				MT_Vector3 error = normal * 0.5f;
+				obj1->m_error += error;
+				obj2->m_error -= error;
+			}
+			else {
+				// Same again but now obj1 is non-dynamic
+				obj2->m_error -= normal;
+				obj2->m_static = obj1->m_static + 1;
+			}
+		}
+	
+	}
+		
+	
+	typedef std::set<Contact*, Contact> Set;
+};
+
+static Contact::Set contacts;
 
-// Tweak parameters
-static const MT_Scalar ImpulseThreshold = -10.;
-static const MT_Scalar FixThreshold = 0.01;
-static const MT_Scalar FixVelocity = 0.01;
 SM_Object::SM_Object(
 	DT_ShapeHandle shape, 
 	const SM_MaterialProps *materialProps,
@@ -62,20 +149,16 @@ SM_Object::SM_Object(
 	
 	m_dynamicParent(dynamicParent),
 	m_client_object(0),
-	
+	m_physicsClientObject(0),
 	m_shape(shape),
 	m_materialProps(materialProps),
 	m_materialPropsBackup(0),
 	m_shapeProps(shapeProps),
 	m_shapePropsBackup(0),
-	m_object(DT_CreateObject(this, shape)),
 	m_margin(0.0),
 	m_scaling(1.0, 1.0, 1.0),
 	m_reaction_impulse(0.0, 0.0, 0.0),
 	m_reaction_force(0.0, 0.0, 0.0),
-	m_kinematic(false),
-	m_prev_kinematic(false),
-	m_is_rigid_body(false),
 	m_lin_mom(0.0, 0.0, 0.0),
 	m_ang_mom(0.0, 0.0, 0.0),
 	m_force(0.0, 0.0, 0.0),
@@ -85,8 +168,13 @@ SM_Object::SM_Object(
 	m_combined_ang_vel (0.0, 0.0, 0.0),
 	m_fh_object(0),
 	m_inv_mass(0.0),
-	m_inv_inertia(0., 0., 0.)
+	m_inv_inertia(0., 0., 0.),
+	m_kinematic(false),
+	m_prev_kinematic(false),
+	m_is_rigid_body(false),
+	m_static(0)
 {
+	m_object = DT_CreateObject(this, shape);
 	m_xform.setIdentity();
 	m_xform.getValue(m_ogl_matrix);
 	if (shapeProps)
@@ -104,13 +192,14 @@ SM_Object::SM_Object(
 	m_suspended = false;
 }
 
+
 	void 
 SM_Object::
 integrateForces(
 	MT_Scalar timeStep
 ){
 	if (!m_suspended) {
-		m_prev_state = *this;
+		m_prev_state = getNextFrame();
 		m_prev_state.setLinearVelocity(actualLinVelocity());
 		m_prev_state.setAngularVelocity(actualAngVelocity());
 		if (isDynamic()) {
@@ -121,8 +210,8 @@ integrateForces(
 			m_lin_mom *= pow(m_shapeProps->m_lin_drag, timeStep);
 			m_ang_mom *= pow(m_shapeProps->m_ang_drag, timeStep);
 			// Set velocities according momentum
-			m_lin_vel = m_lin_mom * m_inv_mass;
-			m_ang_vel = m_inv_inertia_tensor * m_ang_mom;
+			getNextFrame().setLinearVelocity(m_lin_mom * m_inv_mass);
+			getNextFrame().setAngularVelocity(m_inv_inertia_tensor * m_ang_mom);
 		}
 	}	
 
@@ -148,19 +237,14 @@ integrateMomentum(
 //#define BACKWARD
 #ifdef  MIDPOINT
 // Midpoint rule
-		m_pos += (m_prev_state.getLinearVelocity() + actualLinVelocity()) * (timeStep * 0.5);
-		m_orn += (m_prev_state.getAngularVelocity() * m_prev_state.getOrientation() + actualAngVelocity() * m_orn) * (timeStep * 0.25);
+		getNextFrame().integrateMidpoint(timeStep, m_prev_state, actualLinVelocity(), actualAngVelocity());
 #elif defined BACKWARD
 // Backward Euler
-		m_pos += actualLinVelocity() * timeStep;
-		m_orn += actualAngVelocity() * m_orn * (timeStep * 0.5);
+		getNextFrame().integrateBackward(timeStep, actualLinVelocity(), actualAngVelocity());
 #else 
 // Forward Euler
-
-		m_pos += m_prev_state.getLinearVelocity() * timeStep;
-		m_orn += m_prev_state.getAngularVelocity() * m_orn * (timeStep * 0.5);
+		getNextFrame().integrateForward(timeStep, m_prev_state);
 #endif
-		m_orn.normalize(); // I might not be necessary to do this every call
 
 		calcXform();
 		notifyClient();		
@@ -196,11 +280,11 @@ void SM_Object::dynamicCollision(const MT_Point3 &local2,
 	/**
 	 * if rel_vel_normal > 0, the objects are moving apart! 
 	 */
-	if (rel_vel_normal < 0.) {
+	if (rel_vel_normal < -MT_EPSILON) {
 		/**
 		 * if rel_vel_normal < ImpulseThreshold, scale the restitution down.
 		 * This should improve the simulation where the object is stacked.
-		 */
+		 */	
 		restitution *= MT_min(MT_Scalar(1.0), rel_vel_normal/ImpulseThreshold);
 				
 		MT_Scalar impulse = -(1.0 + restitution) * rel_vel_normal;
@@ -214,7 +298,7 @@ void SM_Object::dynamicCollision(const MT_Point3 &local2,
 			 * Apply impulse at the collision point.
 			 * Take rotational inertia into account.
 			 */
-			applyImpulse(local2 + m_pos, impulse * normal);
+			applyImpulse(local2 + getNextFrame().getPosition(), impulse * normal);
 		} else {
 			/**
 			 * Apply impulse through object centre. (no rotation.)
@@ -251,7 +335,7 @@ void SM_Object::dynamicCollision(const MT_Point3 &local2,
 			 * local coordinates.
 			 */
 
-			MT_Matrix3x3 lcs(m_orn);
+			MT_Matrix3x3 lcs(getNextFrame().getOrientation());
 			
 			/**
 			 * We cannot use m_xform.getBasis() for the matrix, since 
@@ -303,7 +387,7 @@ void SM_Object::dynamicCollision(const MT_Point3 &local2,
 
 			// I guess the GEN_max is not necessary, so let's check it
 
-			assert(impulse >= 0.0);
+			MT_assert(impulse >= 0.0);
 
 			/**
 			 * Here's the trick. We compute the impulse to make the
@@ -325,7 +409,7 @@ void SM_Object::dynamicCollision(const MT_Point3 &local2,
 					(invMass + lateral.dot(temp.cross(local2)));
 
 				MT_Scalar friction = MT_min(impulse_lateral, max_friction);
-				applyImpulse(local2 + m_pos, -lateral * friction);
+				applyImpulse(local2 + getNextFrame().getPosition(), -lateral * friction);
 			}
 			else {
 				MT_Scalar impulse_lateral = rel_vel_lateral / invMass;
@@ -343,52 +427,71 @@ void SM_Object::dynamicCollision(const MT_Point3 &local2,
 	}
 }
 
+static void AddCallback(SM_Scene *scene, SM_Object *obj1, SM_Object *obj2)
+{
+	// If we have callbacks on either of the client objects, do a collision test
+	// and add a callback if they intersect.
+	DT_Vector3 v;
+	if ((obj1->getClientObject() && obj1->getClientObject()->hasCollisionCallback()) || 
+	    (obj2->getClientObject() && obj2->getClientObject()->hasCollisionCallback()) &&
+	     DT_GetIntersect(obj1->getObjectHandle(), obj2->getObjectHandle(), v))
+		scene->notifyCollision(obj1, obj2);
+}
+
 DT_Bool SM_Object::boing(
 	void *client_data,  
 	void *object1,
 	void *object2,
 	const DT_CollData *coll_data
 ){
-	//if (!coll_data)
-	//	return DT_CONTINUE;
-
 	SM_Scene  *scene = (SM_Scene *)client_data; 
 	SM_Object *obj1  = (SM_Object *)object1;  
 	SM_Object *obj2  = (SM_Object *)object2;  
 	
-	scene->addPair(obj1, obj2); // Record this collision for client callbacks
+	// at this point it is unknown whether we are really intersecting (broad phase)
+	
+	DT_Vector3 p1, p2;
+	if (!obj2->isDynamic()) {
+		std::swap(obj1, obj2);
+	}
 	
 	// If one of the objects is a ghost then ignore it for the dynamics
 	if (obj1->isGhost() || obj2->isGhost()) {
+		AddCallback(scene, obj1, obj2);
 		return DT_CONTINUE;
 	}
 
 	// Objects do not collide with parent objects
 	if (obj1->getDynamicParent() == obj2 || obj2->getDynamicParent() == obj1) {
+		AddCallback(scene, obj1, obj2);
 		return DT_CONTINUE;
 	}
 	
 	if (!obj2->isDynamic()) {
-		std::swap(obj1, obj2);
-	}
-
-	if (!obj2->isDynamic()) {
+		AddCallback(scene, obj1, obj2);
 		return DT_CONTINUE;
 	}
 
 	// Get collision data from SOLID
-	DT_Vector3 p1, p2;
 	if (!DT_GetPenDepth(obj1->getObjectHandle(), obj2->getObjectHandle(), p1, p2))
 		return DT_CONTINUE;
+	
 	MT_Point3 local1(p1), local2(p2);
 	MT_Vector3 normal(local2 - local1);
 	MT_Scalar dist = normal.length();
 	
 	if (dist < MT_EPSILON)
 		return DT_CONTINUE;
+		
+	// Now we are definitely intersecting.
+
+	// Set callbacks for game engine.
+	if ((obj1->getClientObject() && obj1->getClientObject()->hasCollisionCallback()) || 
+	    (obj2->getClientObject() && obj2->getClientObject()->hasCollisionCallback()))
+		scene->notifyCollision(obj1, obj2);
 	
-	local1 -= obj1->m_pos;
-	local2 -= obj2->m_pos;
+	local1 -= obj1->getNextFrame().getPosition();
+	local2 -= obj2->getNextFrame().getPosition();
 	
 	// Calculate collision parameters
 	MT_Vector3 rel_vel        = obj1->getVelocity(local1) - obj2->getVelocity(local2);
@@ -410,7 +513,11 @@ DT_Bool SM_Object::boing(
 		obj1->dynamicCollision(local1, normal, dist, rel_vel, restitution, friction_factor, invMass);
 		
 	if (obj2->isDynamic())
+	{
 		obj2->dynamicCollision(local2, -normal, dist, -rel_vel, restitution, friction_factor, invMass);
+		if (!obj1->isDynamic() || obj1->m_static)
+			obj2->m_static = obj1->m_static + 1;
+	}
 	
 	return DT_CONTINUE;
 }
@@ -421,7 +528,6 @@ DT_Bool SM_Object::fix(
 	void *object2,
 	const DT_CollData *coll_data
 ){
-	SM_Scene  *scene = (SM_Scene *)client_data; 
 	SM_Object *obj1  = (SM_Object *)object1;  
 	SM_Object *obj2  = (SM_Object *)object2;  
 	
@@ -450,62 +556,48 @@ DT_Bool SM_Object::fix(
 	// Get collision data from SOLID
 	MT_Vector3 normal(local2 - local1);
 	
-	if (normal.dot(normal) < MT_EPSILON)
+	MT_Scalar dist = normal.dot(normal);
+	if (dist < MT_EPSILON || dist > obj2->m_shapeProps->m_radius*obj2->m_shapeProps->m_radius)
 		return DT_CONTINUE;
 		
-	// This distinction between dynamic and non-dynamic objects should not be 
-	// necessary. Non-dynamic objects are assumed to have infinite mass.
-	if (obj1->isDynamic()) {
-		MT_Vector3 error = normal * 0.5f;
-		obj1->m_error += error;
-		obj2->m_error -= error;
-		// Remove the velocity component in the normal direction
-		// Calculate collision parameters
-		/*MT_Vector3 rel_vel = obj1->getLinearVelocity() - obj2->getLinearVelocity();
-		if (normal.length() > FixThreshold && rel_vel.length() < FixVelocity) {
-			normal.normalize();
-			MT_Scalar  rel_vel_normal = 0.49*(normal.dot(rel_vel));
-
-			obj1->addLinearVelocity(-rel_vel_normal*normal);
-			obj2->addLinearVelocity(rel_vel_normal*normal);
-		}*/
-	}
-	else {
-		// Same again but now obj1 is non-dynamic
-		obj2->m_error -= normal;
-		/*MT_Vector3 rel_vel = obj2->getLinearVelocity();
-		if (normal.length() > FixThreshold && rel_vel.length() < FixVelocity) {
-			// Calculate collision parameters
-			normal.normalize();
-			MT_Scalar  rel_vel_normal = -0.99*(normal.dot(rel_vel));
-
-			obj2->addLinearVelocity(rel_vel_normal*normal);
-		}*/ 
+		
+	if ((obj1->m_static || !obj1->isDynamic()) && obj1->m_static < obj2->m_static)
+	{
+		obj2->m_static = obj1->m_static + 1;
+	} else if (obj2->m_static && obj2->m_static < obj1->m_static)
+	{
+		obj1->m_static = obj2->m_static + 1;
 	}
 	
+	contacts.insert(new Contact(obj1, obj2, normal, MT_Point3(local1 + 0.5*(local2 - local1))));
+	
+	
 	return DT_CONTINUE;
 }
 
 void SM_Object::relax(void)
-{ 
+{
+	for (Contact::Set::iterator csit = contacts.begin() ; csit != contacts.end(); ++csit)
+	{
+		(*csit)->resolve();
+		delete (*csit);
+	}
+		
+	contacts.clear();
 	if (m_error.fuzzyZero())
 		return;
 	//std::cout << "SM_Object::relax: { " << m_error << " }" << std::endl;
 	
-	m_pos += m_error; 
+	getNextFrame().setPosition(getNextFrame().getPosition() + m_error); 
 	m_error.setValue(0., 0., 0.); 
-/*	m_pos.getValue(pos);
-	DT_SetPosition(m_object, pos); 
-	m_xform.setOrigin(m_pos); 
-	m_xform.getValue(m_ogl_matrix); */
-	calcXform();
-	notifyClient();
+	//calcXform();
+	//notifyClient();
 }
 	
 SM_Object::SM_Object() :
 	m_dynamicParent(0),
 	m_client_object(0),
-	
+	m_physicsClientObject(0),
 	m_shape(0),
 	m_materialProps(0),
 	m_materialPropsBackup(0),
@@ -516,9 +608,6 @@ SM_Object::SM_Object() :
 	m_scaling(1.0, 1.0, 1.0),
 	m_reaction_impulse(0.0, 0.0, 0.0),
 	m_reaction_force(0.0, 0.0, 0.0),
-	m_kinematic(false),
-	m_prev_kinematic(false),
-	m_is_rigid_body(false),
 	m_lin_mom(0.0, 0.0, 0.0),
 	m_ang_mom(0.0, 0.0, 0.0),
 	m_force(0.0, 0.0, 0.0),
@@ -526,7 +615,10 @@ SM_Object::SM_Object() :
 	m_error(0.0, 0.0, 0.0),
 	m_combined_lin_vel (0.0, 0.0, 0.0),
 	m_combined_ang_vel (0.0, 0.0, 0.0),
-	m_fh_object(0) 
+	m_fh_object(0),
+	m_kinematic(false),
+	m_prev_kinematic(false),
+	m_is_rigid_body(false)
 {
 	// warning no initialization of variables done by moto.
 }
@@ -645,15 +737,15 @@ calcXform() {
 	printf("                 m_scaling = { %-0.5f, %-0.5f, %-0.5f }\n",
 		m_scaling[0], m_scaling[1], m_scaling[2]);
 #endif
-	m_xform.setOrigin(m_pos);
-	m_xform.setBasis(MT_Matrix3x3(m_orn, m_scaling));
+	m_xform.setOrigin(getNextFrame().getPosition());
+	m_xform.setBasis(MT_Matrix3x3(getNextFrame().getOrientation(), m_scaling));
 	m_xform.getValue(m_ogl_matrix);
 	
 	/* Blender has been known to crash here.
 	   This usually means SM_Object *this has been deleted more than once. */
 	DT_SetMatrixd(m_object, m_ogl_matrix);
 	if (m_fh_object) {
-		m_fh_object->setPosition(m_pos);
+		m_fh_object->setPosition(getNextFrame().getPosition());
 		m_fh_object->calcXform();
 	}
 	updateInvInertiaTensor();
@@ -769,17 +861,19 @@ setPosition(
 	const MT_Point3& pos
 ){
 	m_kinematic = true;
-	m_pos = pos;
+	getNextFrame().setPosition(pos);
+	endFrame();
 }
-
+	
 	void 
 SM_Object::
 setOrientation(
 	const MT_Quaternion& orn
 ){
-	assert(!orn.fuzzyZero());
+	MT_assert(!orn.fuzzyZero());
 	m_kinematic = true;
-	m_orn = orn;
+	getNextFrame().setOrientation(orn);
+	endFrame();
 }
 
 	void 
@@ -816,10 +910,7 @@ SM_Object::
 addLinearVelocity(
 	const MT_Vector3& lin_vel
 ){
-	m_lin_vel += lin_vel;
-	if (m_shapeProps) {
-		m_lin_mom = m_lin_vel * m_shapeProps->m_mass;
-	}
+	setLinearVelocity(getNextFrame().getLinearVelocity() + lin_vel);
 }
 
 	void 
@@ -827,9 +918,9 @@ SM_Object::
 setLinearVelocity(
 	const MT_Vector3& lin_vel
 ){
-	m_lin_vel = lin_vel;
+	getNextFrame().setLinearVelocity(lin_vel);
 	if (m_shapeProps) {
-		m_lin_mom = m_lin_vel * m_shapeProps->m_mass;
+		m_lin_mom = getNextFrame().getLinearVelocity() * m_shapeProps->m_mass;
 	}
 }
 
@@ -858,9 +949,9 @@ SM_Object::
 setAngularVelocity(
 	const MT_Vector3& ang_vel
 ) {
-	m_ang_vel = ang_vel;
+	getNextFrame().setAngularVelocity(ang_vel);
 	if (m_shapeProps) {
-		m_ang_mom = m_ang_vel * m_shapeProps->m_inertia;
+		m_ang_mom = getNextFrame().getAngularVelocity() * m_shapeProps->m_inertia;
 	}
 }
 
@@ -869,10 +960,7 @@ SM_Object::
 addAngularVelocity(
 	const MT_Vector3& ang_vel
 ) {
-	m_ang_vel += ang_vel;
-	if (m_shapeProps) {
-		m_ang_mom = m_ang_vel * m_shapeProps->m_inertia;
-	}
+	setAngularVelocity(getNextFrame().getAngularVelocity() + ang_vel);
 }
 
 
@@ -905,18 +993,18 @@ resolveCombinedVelocities(
 	if (isDynamic()) {		
 
 #if 1
-		m_lin_vel += lin_vel;
-		m_ang_vel += ang_vel;
+		getNextFrame().setLinearVelocity(getNextFrame().getLinearVelocity() + lin_vel);
+		getNextFrame().setAngularVelocity(getNextFrame().getAngularVelocity() + ang_vel);
 #else
 
 		//compute the component of the physics velocity in the 
 		// direction of the set velocity and set it to zero.
 		MT_Vector3 lin_vel_norm = lin_vel.normalized();
 
-		m_lin_vel -= (m_lin_vel.dot(lin_vel_norm) * lin_vel_norm);
+		setLinearVelocity(getNextFrame().getLinearVelocity() - (getNextFrame().getLinearVelocity().dot(lin_vel_norm) * lin_vel_norm));
 #endif
-		m_lin_mom = m_lin_vel * m_shapeProps->m_mass;
-		m_ang_mom = m_ang_vel * m_shapeProps->m_inertia;
+		m_lin_mom = getNextFrame().getLinearVelocity() * m_shapeProps->m_mass;
+		m_ang_mom = getNextFrame().getAngularVelocity() * m_shapeProps->m_inertia;
 		clearCombinedVelocities();
 
 	}
@@ -979,7 +1067,7 @@ applyImpulse(
 	const MT_Point3& attach, const MT_Vector3& impulse
 ) {
 	applyCenterImpulse(impulse);                          // Change in linear momentum
-	applyAngularImpulse((attach - m_pos).cross(impulse)); // Change in angular momentump
+	applyAngularImpulse((attach - getNextFrame().getPosition()).cross(impulse)); // Change in angular momentump
 }
 
 	void 
@@ -990,7 +1078,7 @@ applyCenterImpulse(
 	if (m_shapeProps) {
 		m_lin_mom          += impulse;
 		m_reaction_impulse += impulse;
-		m_lin_vel           = m_lin_mom * m_inv_mass;
+		getNextFrame().setLinearVelocity(m_lin_mom * m_inv_mass);
 
 		// The linear velocity is immedialtely updated since otherwise
 		// simultaneous collisions will get a double impulse. 
@@ -1004,7 +1092,7 @@ applyAngularImpulse(
 ) {
 	if (m_shapeProps) {
 		m_ang_mom += impulse;
-		m_ang_vel = m_inv_inertia_tensor * m_ang_mom;
+		getNextFrame().setAngularVelocity( m_inv_inertia_tensor * m_ang_mom);
 	}
 }
 
@@ -1108,7 +1196,7 @@ const
 SM_Object::
 actualLinVelocity(
 ) const {
-	return m_combined_lin_vel + m_lin_vel;
+	return m_combined_lin_vel + getNextFrame().getLinearVelocity();
 };
 
 const 
@@ -1116,10 +1204,98 @@ const
 SM_Object::
 actualAngVelocity(
 ) const {
-	return m_combined_ang_vel + m_ang_vel;
-};
+	return m_combined_ang_vel + getNextFrame().getAngularVelocity();
+}
 
 
+SM_MotionState&
+SM_Object::
+getCurrentFrame()
+{
+	return m_frames[1];
+}
+
+SM_MotionState&
+SM_Object::
+getPreviousFrame()
+{
+	return m_frames[0];
+}
+
+SM_MotionState &
+SM_Object::
+getNextFrame()
+{
+	return m_frames[2];
+}
+
+const SM_MotionState &
+SM_Object::
+getCurrentFrame() const
+{
+	return m_frames[1];
+}
+
+const SM_MotionState &
+SM_Object::
+getPreviousFrame() const
+{
+	return m_frames[0];
+}
+
+const SM_MotionState &
+SM_Object::
+getNextFrame() const
+{
+	return m_frames[2];
+}
 
 
+const MT_Point3&     
+SM_Object::
+getPosition()        const
+{
+	return m_frames[1].getPosition();
+}
 
+const MT_Quaternion& 
+SM_Object::
+getOrientation()     const
+{
+	return m_frames[1].getOrientation();
+}
+
+const MT_Vector3&    
+SM_Object::
+getLinearVelocity()  const
+{
+	return m_frames[1].getLinearVelocity();
+}
+
+const MT_Vector3&    
+SM_Object::
+getAngularVelocity() const
+{
+	return m_frames[1].getAngularVelocity();
+}
+
+void
+SM_Object::
+interpolate(MT_Scalar timeStep)
+{
+	if (!actualLinVelocity().fuzzyZero() || !actualAngVelocity().fuzzyZero()) 
+	{
+		getCurrentFrame().setTime(timeStep);
+		getCurrentFrame().lerp(getPreviousFrame(), getNextFrame());
+		notifyClient();
+	}
+}
+
+void
+SM_Object::
+endFrame()
+{
+	getPreviousFrame() = getNextFrame();
+	getCurrentFrame() = getNextFrame();
+	m_static = 0;
+}
