@@ -1,5 +1,5 @@
 /**
- * $Id: RAS_VAOpenGLRasterizer.cpp,v 1.14 2006/04/02 21:04:20 erwin Exp $
+ * $Id: RAS_VAOpenGLRasterizer.cpp,v 1.16 2007/02/01 02:10:38 snailrose Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -325,9 +325,19 @@ void RAS_VAOpenGLRasterizer::TexCoordPtr(const RAS_TexVert *tv, int enabled)
 				break;
 			case RAS_TEXTANGENT:
 				glTexCoordPointer(4, GL_FLOAT, sizeof(RAS_TexVert),tv->getTangent());
+				break;
+			case RAS_TEXCO_UV2:
+				glTexCoordPointer(2, GL_FLOAT, sizeof(RAS_TexVert),tv->getUV2());
+				break;
 			}
 		}
 	}
+
+#ifdef GL_ARB_vertex_program
+	if(m_useTang && bgl::RAS_EXT_support._ARB_vertex_program)
+		bgl::blVertexAttrib4fvARB(1/*tangent*/, tv->getTangent());
+#endif
+
 #endif
 }
 
