@@ -21,7 +21,7 @@ def validate_arguments(args, bc):
             'BF_PTHREADS', 'BF_PTHREADS_INC', 'BF_PTHREADS_LIB', 'BF_PTHREADS_LIBPATH',
             'WITH_BF_FMOD',
             'WITH_BF_OPENEXR', 'BF_OPENEXR', 'BF_OPENEXR_INC', 'BF_OPENEXR_LIB', 'BF_OPENEXR_LIBPATH',
-            'WITH_BF_FFMPEG', 'BF_FFMPEG_LIB', 
+            'WITH_BF_FFMPEG', 'BF_FFMPEG_LIB', 'BF_FFMPEG',  'BF_FFMPEG_INC',
             'WITH_BF_JPEG', 'BF_JPEG', 'BF_JPEG_INC', 'BF_JPEG_LIB', 'BF_JPEG_LIBPATH',
             'WITH_BF_PNG', 'BF_PNG', 'BF_PNG_INC', 'BF_PNG_LIB', 'BF_PNG_LIBPATH',
             'BF_TIFF', 'BF_TIFF_INC',
@@ -31,7 +31,8 @@ def validate_arguments(args, bc):
             'WITH_BF_ICONV', 'BF_ICONV', 'BF_ICONV_INC', 'BF_ICONV_LIB', 'BF_ICONV_LIBPATH',
             'WITH_BF_ODE', 'BF_ODE', 'BF_ODE_INC', 'BF_ODE_LIB',
             'WITH_BF_GAMEENGINE', 'WITH_BF_BULLET', 'BF_BULLET', 'BF_BULLET_INC', 'BF_BULLET_LIB',
-            'BF_SOLID', 'BF_SOLID_INC',
+            'BF_SOLID', 'BF_SOLID_INC', 'BF_WINTAB', 'BF_WINTAB_INC',
+            'WITH_BF_YAFRAY',
             'BF_FREETYPE', 'BF_FREETYPE_INC', 'BF_FREETYPE_LIB', 'BF_FREETYPE_LIBPATH',
             'WITH_BF_QUICKTIME', 'BF_QUICKTIME', 'BF_QUICKTIME_INC', 'BF_QUICKTIME_LIB', 'BF_QUICKTIME_LIBPATH',
             'WITH_BF_STATICOPENGL', 'BF_OPENGL', 'BF_OPENGL_INC', 'BF_OPENGL_LIB', 'BF_OPENGL_LIBPATH', 'BF_OPENGL_LIB_STATIC', 'BF_OPENGL_LINKFLAGS',
@@ -41,12 +42,15 @@ def validate_arguments(args, bc):
             'CFLAGS', 'CCFLAGS', 'CPPFLAGS', 
             'REL_CFLAGS', 'REL_CCFLAGS',
             'C_WARN', 'CC_WARN', 'LLIBS', 'PLATFORM_LINKFLAGS',
-            'BF_PROFILE_FLAGS' ]
+            'BF_PROFILE_FLAGS', 'LCGDIR', 'WITH_BF_VERSE', 
+            'BF_VERSE_INCLUDE',
+            'VERSE_BUILD_BINARY', 'VERSE_BUILD_DIR', 'VERSE_REGEN_PROTO']
+
     arg_list = ['BF_DEBUG', 'BF_QUIET', 'BF_CROSS', 'BF_UPDATE',
             'BF_INSTALLDIR', 'BF_TOOLSET', 'BF_BINNAME',
-            'BF_BUILDDIR', 'BF_FANCY', 'BF_QUICK', 'BF_PROFILE', 'BF_DEBUG', 'BF_DEBUG_FLAGS',
-            'BF_PRIORITYLIST', 'BF_BUILDINFO','CC', 'CXX', "BF_QUICKDEBUG", "BF_LISTDEBUG"
-           ]
+            'BF_BUILDDIR', 'BF_FANCY', 'BF_QUICK', 'BF_PROFILE',
+            'BF_DEBUG_FLAGS', 'BF_BSC',
+            'BF_PRIORITYLIST', 'BF_BUILDINFO','CC', 'CXX', "BF_QUICKDEBUG", "BF_LISTDEBUG", 'LCGDIR']
 
     all_list = opts_list + arg_list
     okdict = {}
@@ -69,7 +73,7 @@ def print_targets(targs, bc):
 def validate_targets(targs, bc):
     valid_list = ['.', 'blender', 'blenderstatic', 'blenderplayer', 'webplugin',
             'blendernogame', 'blenderstaticnogame', 'release',
-            'everything', 'clean', 'install-bin', 'install']
+            'everything', 'clean', 'install-bin', 'install', 'nsis']
     oklist = []
     for t in targs:
         if t in valid_list:
@@ -110,6 +114,15 @@ def SetupBufferedOutput( env ):
 def read_opts(cfg, args):
     localopts = Options.Options(cfg, args)
     localopts.AddOptions(
+        ('VERSE_BUILD_BINARY', 'Build a release or debug binary.', 'release'),
+        ('VERSE_BUILD_DIR', 'Target directory for intermediate files.', "${BF_BUILDDIR}/extern/verse"),
+        ('VERSE_REGEN_PROTO', 'Whether to regenerate the protocol files', 'yes'),
+        (BoolOption('WITH_BF_VERSE', 'Use VERSE if true', 'false')),
+        ('BF_VERSE_INCLUDE', 'verse include dir', '/usr/include'),
+        ('LCGDIR', 'location of cvs lib dir'),
+        ('VERSE_BUILD_BINARY', 'Build a release or debug binary.', 'release'),
+        ('VERSE_BUILD_DIR', 'Target directory for intermediate files.', "${BF_BUILDDIR}/extern/verse"),
+        ('VERSE_REGEN_PROTO', 'Whether to regenerate the protocol files', 'yes'),
         ('BF_DEBUG_LIBS', 'list of libraries to build with debug symbols'),
         ('BF_PYTHON', 'base path for python', ''),
         ('BF_PYTHON_VERSION', 'Python version to use', ''),
@@ -118,7 +131,8 @@ def read_opts(cfg, args):
         ('BF_PYTHON_LIB', 'Python library', ''),
         ('BF_PYTHON_LIBPATH', 'Library path', ''),
         ('BF_PYTHON_LINKFLAGS', 'Python link flags', ''),
-
+        (BoolOption('BF_NO_ELBEEM', 'Disable Fluid Sim', 'false')),
+        (BoolOption('WITH_BF_YAFRAY', 'Enable Yafray', 'true')),
         (BoolOption('WITH_BF_OPENAL', 'Use OpenAL if true', '')),
         ('BF_OPENAL', 'base path for OpenAL', ''),
         ('BF_OPENAL_INC', 'include path for python headers', ''),
@@ -146,8 +160,13 @@ def read_opts(cfg, args):
         ('BF_OPENEXR_LIBPATH', 'OPENEXR library path', ''),
 
         (BoolOption('WITH_BF_FFMPEG', 'Use FFMPEG if true', 'false')),
+        ('BF_FFMPEG', 'FFMPEG base path', ''),
         ('BF_FFMPEG_LIB', 'FFMPEG library', ''),
 
+        ('BF_FFMPEG_INC', 'FFMPEG includes', ''),
+        ('BF_FFMPEG_LIBPATH', 'FFMPEG library path', ''),
+        
+
         (BoolOption('WITH_BF_JPEG', 'Use JPEG if true', 'true')),
         ('BF_JPEG', 'JPEG base path', ''),
         ('BF_JPEG_INC', 'JPEG include path', ''),
@@ -196,6 +215,8 @@ def read_opts(cfg, args):
         
         ('BF_SOLID', 'Solid base dir', '#/extern/solid'),
         ('BF_SOLID_INC', 'Solid include path', ''),
+        ('BF_WINTAB', 'WinTab base dir', ''),
+        ('BF_WINTAB_INC', 'WinTab include dir', ''),
 ##
 ##WITH_BF_NSPR = 'true'
 ##BF_NSPR = $(LCGDIR)/nspr
@@ -257,6 +278,8 @@ def read_opts(cfg, args):
 
         (BoolOption('BF_DEBUG', 'Add debug flags if true', 'false')),
         ('BF_DEBUG_FLAGS', 'Debug flags', ''),
+
+        (BoolOption('BF_BSC', 'Create .bsc files (msvc only)', 'true')),
         
         ('BF_BUILDDIR', 'Build dir', ''),
         ('BF_INSTALLDIR', 'Installation dir', ''),
